// Code generated by gen/main.go; DO NOT EDIT.
package golocales

// kv is a key-value pair used in sorted slices for map-free lookups.
type kv struct{ k, v string }

// localeEntry holds a locale name and its compact config as indices into interned tables.
type localeEntry struct {
	name                                           string
	monthsAbbr, monthsNarrow, monthsWide           uint16
	daysAbbr, daysNarrow, daysShort, daysWide      uint16
	decimal, group, minus, percent, perMille, plus uint16
	percentPattern                                 uint16
	dateFull, dateLong, dateMedium, dateShort      uint16
	timeFull, timeLong, timeMedium, timeShort      uint16
	stdCurrencyPattern, acctCurrencyPattern        uint16
	currencyOverrides                              uint16
	tzNames                                        uint16
}

type months [12]string
type days [7]string

var _months = [...]months{
	{"Qun", "Nah", "Cig", "Agd", "Cax", "Qas", "Qad", "Leq", "Way", "Dit", "Xim", "Kax"},
	{"", "", "", "", "", "", "", "", "", "", "", ""},
	{"Qunxa Garablu", "Kudo", "Ciggilta Kudo", "Agda Baxis", "Caxah Alsa", "Qasa Dirri", "Qado Dirri", "Liiqen", "Waysu", "Diteli", "Ximoli", "Kaxxa Garablu"},
	{"Qunxa Garablu", "Kudo", "Ciggilta Kudo", "Agda Baxis", "Caxah Alsa", "Qasa Dirri", "Qado Dirri", "Leqeeni", "Waysu", "Diteli", "Ximoli", "Kaxxa Garablu"},
	{"Ажь", "Жəаб", "Хəажә", "Мш", "Лаҵ", "Рашә", "Ԥхынгә", "Нанҳә", "Цəыб", "Жьҭ", "Абҵ", "Ԥхынҷ"},
	{"Жь", "Жə", "Хə", "М", "Л", "Р", "Гә", "Н", "Цə", "Ҭ", "Б", "Ҷ"},
	{"Ажьырныҳəа", "Жəабран", "Хəажəкыра", "Мшаԥы", "Лаҵара", "Рашəара", "Ԥхынгəы", "Нанҳəа", "Цəыббра", "Жьҭаара", "Абҵара", "Ԥхынҷкәын"},
	{"Jan.", "Feb.", "Mrt.", "Apr.", "Mei", "Jun.", "Jul.", "Aug.", "Sep.", "Okt.", "Nov.", "Des."},
	{"Januarie", "Februarie", "Maart", "April", "Mei", "Junie", "Julie", "Augustus", "September", "Oktober", "November", "Desember"},
	{"nùm", "kɨz", "tɨd", "taa", "see", "nzu", "dum", "fɔe", "dzu", "lɔm", "kaa", "fwo"},
	{"ndzɔ̀ŋɔ̀nùm", "ndzɔ̀ŋɔ̀kƗ̀zùʔ", "ndzɔ̀ŋɔ̀tƗ̀dʉ̀ghà", "ndzɔ̀ŋɔ̀tǎafʉ̄ghā", "ndzɔ̀ŋèsèe", "ndzɔ̀ŋɔ̀nzùghò", "ndzɔ̀ŋɔ̀dùmlo", "ndzɔ̀ŋɔ̀kwîfɔ̀e", "ndzɔ̀ŋɔ̀tƗ̀fʉ̀ghàdzughù", "ndzɔ̀ŋɔ̀ghǔuwelɔ̀m", "ndzɔ̀ŋɔ̀chwaʔàkaa wo", "ndzɔ̀ŋèfwòo"},
	{"Ɔp", "Ɔg", "Ɔb", "O", "K", "A", "Ku", "A", "S", "O", "N", "D"},
	{"Ɔpɛpɔn", "Ɔgyefoɔ", "Ɔbɛnem", "Oforisuo", "Kɔtɔnimma", "Ayɛwohomumu", "Kutawonsa", "Ɔsanaa", "Ɛbɔ", "Ahinime", "Obubuo", "Ɔpɛnimma"},
	{"ጃን", "ፌብ", "ማርች", "ኤፕሪ", "ሜይ", "ጁን", "ጁላይ", "ኦገስ", "ሴፕቴ", "ኦክቶ", "ኖቬም", "ዲሴም"},
	{"ጃንዋሪ", "ፌብሩዋሪ", "ማርች", "ኤፕሪል", "ሜይ", "ጁን", "ጁላይ", "ኦገስት", "ሴፕቴምበር", "ኦክቶበር", "ኖቬምበር", "ዲሴምበር"},
	{"chi.", "feb.", "mar.", "abr.", "may.", "chn.", "chl.", "ago.", "set.", "oct.", "nov.", "avi."},
	{"de chinero", "de febrero", "de marzo", "d’abril", "de mayo", "de chunyo", "de chuliol", "d’agosto", "de setiembre", "d’octubre", "de noviembre", "d’aviento"},
	{"M01", "M02", "M03", "M04", "M05", "M06", "M07", "M08", "M09", "M10", "M11", "M12"},
	{"كانون التاني", "شباط", "أذار", "نيسان", "أيار", "حزيران", "تموز", "آب", "أيلول", "تشرين الأول", "تشرين التاني", "كانون الأول"},
	{"يناير", "فبراير", "مارس", "أبريل", "مايو", "يونيو", "يوليو", "أغسطس", "سبتمبر", "أكتوبر", "نوفمبر", "ديسمبر"},
	{"جانفي", "فيفري", "مارس", "أفريل", "ماي", "جوان", "جويلية", "أوت", "سبتمبر", "أكتوبر", "نوفمبر", "ديسمبر"},
	{"كانون الثاني", "شباط", "آذار", "نيسان", "أيار", "حزيران", "تموز", "آب", "أيلول", "تشرين\u00a0الأول", "تشرين الثاني", "كانون الأول"},
	{"كانون الثاني", "شباط", "آذار", "نيسان", "أيار", "حزيران", "تموز", "آب", "أيلول", "تشرين الأول", "تشرين الثاني", "كانون الأول"},
	{"يناير", "فبراير", "مارس", "أبريل", "ماي", "يونيو", "يوليوز", "غشت", "شتنبر", "أكتوبر", "نونبر", "دجنبر"},
	{"يناير", "فبراير", "مارس", "إبريل", "مايو", "يونيو", "يوليو", "أغشت", "شتمبر", "أكتوبر", "نوفمبر", "دجمبر"},
	{"জানু", "ফেব্ৰু", "মাৰ্চ", "এপ্ৰিল", "মে’", "জুন", "জুলাই", "আগ", "ছেপ্তে", "অক্টো", "নৱে", "ডিচে"},
	{"জানুৱাৰী", "ফেব্ৰুৱাৰী", "মাৰ্চ", "এপ্ৰিল", "মে’", "জুন", "জুলাই", "আগষ্ট", "ছেপ্তেম্বৰ", "অক্টোবৰ", "নৱেম্বৰ", "ডিচেম্বৰ"},
	{"Jan", "Feb", "Mac", "Apr", "Mei", "Jun", "Jul", "Ago", "Sep", "Okt", "Nov", "Dec"},
	{"Januari", "Februari", "Machi", "Aprili", "Mei", "Juni", "Julai", "Agosti", "Septemba", "Oktoba", "Novemba", "Desemba"},
	{"xin", "feb", "mar", "abr", "may", "xun", "xnt", "ago", "set", "och", "pay", "avi"},
	{"de xineru", "de febreru", "de marzu", "d’abril", "de mayu", "de xunu", "de xunetu", "d’agostu", "de setiembre", "d’ochobre", "de payares", "d’avientu"},
	{"yan", "fev", "mar", "apr", "may", "iyn", "iyl", "avq", "sen", "okt", "noy", "dek"},
	{"yanvar", "fevral", "mart", "aprel", "may", "iyun", "iyul", "avqust", "sentyabr", "oktyabr", "noyabr", "dekabr"},
	{"јан", "фев", "мар", "апр", "май", "ијн", "ијл", "авг", "сен", "окт", "ној", "дек"},
	{"јанвар", "феврал", "март", "апрел", "май", "ијун", "ијул", "август", "сентјабр", "октјабр", "нојабр", "декабр"},
	{"ғин.", "фев.", "мар.", "апр.", "май", "июн.", "июл.", "авг.", "сент.", "окт.", "нояб.", "дек."},
	{"Ғ", "Ф", "М", "А", "М", "И", "И", "А", "С", "О", "Н", "Д"},
	{"ғинуар", "февраль", "март", "апрель", "май", "июнь", "июль", "август", "сентябрь", "октябрь", "ноябрь", "декабрь"},
	{"جن", "پر", "مار", "اپر", "مئیی", "جون", "جۆل", "اگست", "ستم", "اکت", "نئوم", "دسم"},
	{"جنوری", "پروری", "مارچ", "اپرێل", "مئیی", "جون", "جۆلایی", "اگست", "ستمبر", "اکتوبر", "نئومبر", "دسمبر"},
	{"Jan", "Par", "Már", "Apr", "Mai", "Jun", "Jól", "Aga", "Sat", "Akt", "Naw", "Das"},
	{"Janwari", "Parwari", "Márch", "Aprél", "Mai", "Jun", "Jólái", "Agast", "Satambar", "Aktubar", "Nawambar", "Dasambar"},
	{"kɔn", "mac", "mat", "mto", "mpu", "hil", "nje", "hik", "dip", "bio", "may", "liɓ"},
	{"Kɔndɔŋ", "Màcɛ̂l", "Màtùmb", "Màtop", "M̀puyɛ", "Hìlòndɛ̀", "Njèbà", "Hìkaŋ", "Dìpɔ̀s", "Bìòôm", "Màyɛsèp", "Lìbuy li ńyèe"},
	{"сту", "лют", "сак", "кра", "мая", "чэр", "ліп", "жні", "вер", "кас", "ліс", "сне"},
	{"студзеня", "лютага", "сакавіка", "красавіка", "мая", "чэрвеня", "ліпеня", "жніўня", "верасня", "кастрычніка", "лістапада", "снежня"},
	{"сту", "лют", "сак", "кра", "тра", "чэр", "ліп", "жні", "вер", "кас", "ліс", "сьн"},
	{"студзеня", "лютага", "сакавіка", "красавіка", "траўня", "чэрвеня", "ліпеня", "жніўня", "верасьня", "кастрычніка", "лістапада", "сьнежня"},
	{"Jan", "Feb", "Mac", "Epr", "Mei", "Jun", "Jul", "Oga", "Sep", "Okt", "Nov", "Dis"},
	{"Januari", "Februari", "Machi", "Epreo", "Mei", "Juni", "Julai", "Ogasti", "Septemba", "Oktoba", "Novemba", "Disemba"},
	{"Jan", "Pèb", "Mar", "Apr", "Méi", "Jun", "Jul", "Ags", "Sèp", "Okt", "Nop", "Dés"},
	{"Januari", "Pèbruari", "Maret", "April", "Méi", "Juni", "Juli", "Agustus", "Sèptèmber", "Oktober", "Nopèmber", "Désèmber"},
	{"Hut", "Vil", "Dat", "Tai", "Han", "Sit", "Sab", "Nan", "Tis", "Kum", "Kmj", "Kmb"},
	{"pa mwedzi gwa hutala", "pa mwedzi gwa wuvili", "pa mwedzi gwa wudatu", "pa mwedzi gwa wutai", "pa mwedzi gwa wuhanu", "pa mwedzi gwa sita", "pa mwedzi gwa saba", "pa mwedzi gwa nane", "pa mwedzi gwa tisa", "pa mwedzi gwa kumi", "pa mwedzi gwa kumi na moja", "pa mwedzi gwa kumi na mbili"},
	{"яну", "фев", "март", "апр", "май", "юни", "юли", "авг", "сеп", "окт", "ное", "дек"},
	{"януари", "февруари", "март", "април", "май", "юни", "юли", "август", "септември", "октомври", "ноември", "декември"},
	{"जनवरी", "फरवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सितम्बर", "अक्टूबर", "नवम्बर", "दिसंबर"},
	{"जनवरी", "फरवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सितम्बर", "अक्टूबर", "नवंबर", "दिसंबर"},
	{"kaw", "kpa", "ci", "ɖʊ", "ɖu5", "ɖu6", "la", "kǝu", "fʊm", "cim", "pom", "bʊn"},
	{"ɩjikawǝrka kaŋɔrɔ", "ɩjikpaka kaŋɔrɔ", "arɛ́cika kaŋɔrɔ", "njɩbɔ nɖʊka kaŋɔrɔ", "acafʊnɖuka kaŋɔrɔ", "anɔɔɖuka kaŋɔrɔ", "alàlaka kaŋɔrɔ", "ɩjikǝuka kaŋɔrɔ", "abofʊmka kaŋɔrɔ", "ɩjicimka kaŋɔrɔ", "acapomka kaŋɔrɔ", "anɔɔbʊnka kaŋɔrɔ"},
	{"zan", "feb", "mar", "awi", "mɛ", "zuw", "zul", "uti", "sɛt", "ɔku", "now", "des"},
	{"zanwuye", "feburuye", "marisi", "awirili", "mɛ", "zuwɛn", "zuluye", "uti", "sɛtanburu", "ɔkutɔburu", "nowanburu", "desanburu"},
	{"জানু", "ফেব", "মার্চ", "এপ্রি", "মে", "জুন", "জুল", "আগ", "সেপ", "অক্টো", "নভে", "ডিসে"},
	{"জানুয়ারি", "ফেব্রুয়ারি", "মার্চ", "এপ্রিল", "মে", "জুন", "জুলাই", "আগস্ট", "সেপ্টেম্বর", "অক্টোবর", "নভেম্বর", "ডিসেম্বর"},
	{"জানু", "ফেব", "মার্চ", "এপ্রি", "মে", "জুন", "জুল", "আগ", "সেপ্টেঃ", "অক্টোঃ", "নভেঃ", "ডিসেঃ"},
	{"ཟླ་༡", "ཟླ་༢", "ཟླ་༣", "ཟླ་༤", "ཟླ་༥", "ཟླ་༦", "ཟླ་༧", "ཟླ་༨", "ཟླ་༩", "ཟླ་༡༠", "ཟླ་༡༡", "ཟླ་༡༢"},
	{"ཟླ་བ་དང་པོ", "ཟླ་བ་གཉིས་པ", "ཟླ་བ་གསུམ་པ", "ཟླ་བ་བཞི་པ", "ཟླ་བ་ལྔ་པ", "ཟླ་བ་དྲུག་པ", "ཟླ་བ་བདུན་པ", "ཟླ་བ་བརྒྱད་པ", "ཟླ་བ་དགུ་པ", "ཟླ་བ་བཅུ་པ", "ཟླ་བ་བཅུ་གཅིག་པ", "ཟླ་བ་བཅུ་གཉིས་པ"},
	{"ژانویه", "فوریه", "مارس", "آوریل", "مهٔ", "ژوئن", "ژوئیه", "اوت", "سپتامبر", "اکتبر", "نوامبر", "دسامبر"},
	{"Gen.", "Cʼhwe.", "Meur.", "Ebr.", "Mae", "Mezh.", "Goue.", "Eost", "Gwen.", "Here", "Du", "Kzu."},
	{"Genver", "Cʼhwevrer", "Meurzh", "Ebrel", "Mae", "Mezheven", "Gouere", "Eost", "Gwengolo", "Here", "Du", "Kerzu"},
	{"जान", "फेब", "मार्च", "एप्रि", "मे", "जुन", "जुल", "आग", "सेप", "अक्ट’", "नवे", "डिसे"},
	{"जानुवारी", "फेब्रूवारी", "मार्च", "एप्रिल", "मे", "जुन", "जुलाई", "आगष्ट", "सेप्थेम्बर", "अक्ट’बर", "नवेम्बर", "डिसेम्बर"},
	{"jan", "feb", "mar", "apr", "maj", "jun", "jul", "aug", "sep", "okt", "nov", "dec"},
	{"jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "okt", "nov", "dec"},
	{"januar", "februar", "mart", "april", "maj", "juni", "juli", "august", "septembar", "oktobar", "novembar", "decembar"},
	{"јан", "феб", "мар", "апр", "мај", "јун", "јул", "ауг", "сеп", "окт", "нов", "дец"},
	{"јануар", "фебруар", "март", "април", "мај", "јуни", "јули", "аугуст", "септембар", "октобар", "новембар", "децембар"},
	{"нэгэдүгээр һара", "хоёрдугаар һара", "гурбадугаар һара", "дүрбэдүгээр һара", "табадугаар һара", "зургадугаар һара", "долодугаар һара", "наймадугаар һара", "юһэдүгээр һара", "арбадугаар һара", "арбан нэгэдүгээр һара", "арбан хоёрдугаар һара"},
	{"ልደት", "ካብኽ", "ክብላ", "ፋጅኺ", "ክቢቅ", "ም/ት", "ኰር", "ማርያ", "ያኸኒ", "መተሉ", "ም/ም", "ተሕሳ"},
	{"ልደትሪ", "ካብኽብቲ", "ክብላ", "ፋጅኺሪ", "ክቢቅሪ", "ምኪኤል ትጟኒሪ", "ኰርኩ", "ማርያም ትሪ", "ያኸኒ መሳቅለሪ", "መተሉ", "ምኪኤል መሽወሪ", "ተሕሳስሪ"},
	{"de gen.", "de febr.", "de març", "d’abr.", "de maig", "de juny", "de jul.", "d’ag.", "de set.", "d’oct.", "de nov.", "de des."},
	{"de gener", "de febrer", "de març", "d’abril", "de maig", "de juny", "de juliol", "d’agost", "de setembre", "d’octubre", "de novembre", "de desembre"},
	{"Cháykáhday Haˀimay", "Tsahkápbiˀ", "Wánit", "Háshnihtiˀtiˀ", "Háshnih Haˀimay", "Háshnihtsiˀ", "Násˀahˀatsus", "Dahósikah nish", "Híisikah nish", "Nípbaatiˀtiˀ", "Nípbaa Haˀimay", "Cháykáhdaytiˀtiˀ"},
	{"Dyon", "Baa", "Atat", "Anas", "Atyo", "Achi", "Atar", "Awur", "Shad", "Shak", "Naba", "Nata"},
	{"Pen Dyon", "Pen Baʼa", "Pen Atat", "Pen Anas", "Pen Atyon", "Pen Achirim", "Pen Atariba", "Pen Awurr", "Pen Shadon", "Pen Shakur", "Pen Kur Naba", "Pen Kur Natat"},
	{"𑄎𑄚𑄪", "𑄜𑄬𑄛𑄴", "𑄟𑄢𑄴𑄌𑄧", "𑄃𑄬𑄛𑄳𑄢𑄨𑄣𑄴", "𑄟𑄬", "𑄎𑄪𑄚𑄴", "𑄎𑄪𑄣𑄭", "𑄃𑄉𑄧𑄌𑄴𑄑𑄴", "𑄥𑄬𑄛𑄴𑄑𑄬𑄟𑄴𑄝𑄧𑄢𑄴", "𑄃𑄧𑄇𑄴𑄑𑄮𑄝𑄧𑄢𑄴", "𑄚𑄧𑄞𑄬𑄟𑄴𑄝𑄧𑄢𑄴", "𑄓𑄨𑄥𑄬𑄟𑄴𑄝𑄢𑄴"},
	{"𑄎𑄚𑄪𑄠𑄢𑄨", "𑄜𑄬𑄛𑄴𑄝𑄳𑄢𑄪𑄠𑄢𑄨", "𑄟𑄢𑄴𑄌𑄧", "𑄃𑄬𑄛𑄳𑄢𑄨𑄣𑄴", "𑄟𑄬", "𑄎𑄪𑄚𑄴", "𑄎𑄪𑄣𑄭", "𑄃𑄉𑄧𑄌𑄴𑄑𑄴", "𑄥𑄬𑄛𑄴𑄑𑄬𑄟𑄴𑄝𑄧𑄢𑄴", "𑄃𑄧𑄇𑄴𑄑𑄬𑄝𑄧𑄢𑄴", "𑄚𑄧𑄞𑄬𑄟𑄴𑄝𑄧𑄢𑄴", "𑄓𑄨𑄥𑄬𑄟𑄴𑄝𑄧𑄢𑄴"},
	{"янв", "фев", "мар", "апр", "май", "июн", "июл", "авг", "сен", "окт", "ноя", "дек"},
	{"январь", "февраль", "март", "апрель", "май", "июнь", "июль", "август", "сентябрь", "октябрь", "ноябрь", "декабрь"},
	{"Ene", "Peb", "Mar", "Abr", "May", "Hun", "Hul", "Ago", "Sep", "Okt", "Nob", "Dis"},
	{"Enero", "Pebrero", "Marso", "Abril", "Mayo", "Hunyo", "Hulyo", "Agosto", "Septiyembre", "Oktubre", "Nobyembre", "Disyembre"},
	{"KBZ", "KBR", "KST", "KKN", "KTN", "KMK", "KMS", "KMN", "KMW", "KKM", "KNK", "KNB"},
	{"Okwokubanza", "Okwakabiri", "Okwakashatu", "Okwakana", "Okwakataana", "Okwamukaaga", "Okwamushanju", "Okwamunaana", "Okwamwenda", "Okwaikumi", "Okwaikumi na kumwe", "Okwaikumi na ibiri"},
	{"ᎤᏃ", "ᎧᎦ", "ᎠᏅ", "ᎧᏬ", "ᎠᏂ", "ᏕᎭ", "ᎫᏰ", "ᎦᎶ", "ᏚᎵ", "ᏚᏂ", "ᏅᏓ", "ᎥᏍ"},
	{"ᎤᏃᎸᏔᏅ", "ᎧᎦᎵ", "ᎠᏅᏱ", "ᎧᏬᏂ", "ᎠᏂᏍᎬᏘ", "ᏕᎭᎷᏱ", "ᎫᏰᏉᏂ", "ᎦᎶᏂ", "ᏚᎵᏍᏗ", "ᏚᏂᏅᏗ", "ᏅᏓᏕᏆ", "ᎥᏍᎩᏱ"},
	{"Hashiʼ Ammoʼnaʼ", "Hashiʼ Atokloʼ", "Hashiʼ Atochchíʼnaʼ", "Iiplal", "Mih", "Choon", "Choola", "Akaas", "Siptimpaʼ", "Aaktopaʼ", "Nofimpaʼ", "Tiisimpaʼ"},
	{"کانوونی دووەم", "شوبات", "ئازار", "نیسان", "ئایار", "حوزەیران", "تەمووز", "ئاب", "ئەیلوول", "تشرینی یەکەم", "تشرینی دووەم", "کانونی یەکەم"},
	{"ghj.", "fer.", "mar.", "apr.", "mag.", "ghju.", "lug.", "aos.", "sit.", "ott.", "nuv.", "dic."},
	{"di ghjennaghju", "di ferraghju", "di marzu", "d’aprile", "di maghju", "di ghjugnu", "di lugliu", "d’aostu", "di sittembre", "d’ottobre", "di nuvembre", "di dicembre"},
	{"ⲓⲁⲛⲟⲩⲁⲣⲓ", "ⲫⲉⲃⲣⲟⲩⲁⲣⲓ", "ⲙⲁⲣⲧⲓ", "ⲁⲡⲣⲓⲗⲓ", "ⲙⲁⲓⲟ", "ⲓⲟⲩⲛⲓⲟⲩ", "ⲓⲟⲩⲗⲓⲟⲩ", "ⲁⲩⲅⲟⲩⲥⲑⲟⲩ", "ⲥⲉⲡⲧⲉⲙⲃⲣⲓ", "ⲟⲕⲧⲱⲃⲣⲓ", "ⲛⲟⲉⲙⲃⲣⲓ", "ⲇⲉⲕⲉⲙⲃⲣⲓ"},
	{"Ⲓ", "Ⲫ", "Ⲙ", "Ⲁ", "Ⲙ", "Ⲓ", "Ⲓ", "Ⲁ", "Ⲥ", "Ⲟ", "Ⲛ", "Ⲇ"},
	{"led", "úno", "bře", "dub", "kvě", "čvn", "čvc", "srp", "zář", "říj", "lis", "pro"},
	{"ledna", "února", "března", "dubna", "května", "června", "července", "srpna", "září", "října", "listopadu", "prosince"},
	{"ᐅᒉᒥᑮᓯᑳᐏᐲᓯᒼ", "ᐸᐚᐦᒐᑭᓇᓰᐢ", "ᒥᑭᓯᐏᐲᓯᒼ", "ᓂᐢᑭᐲᓯᒼ", "ᐊᓃᑭᐲᓯᒼ", "ᐚᐏᐲᓯᒼ", "ᐹᐢᑲᐦᐋᐏᐲᓯᒼ", "ᐅᐸᐦᐅᐏᐲᓯᒼ", "ᓄᒌᑐᐏᐲᓯᒼ", "ᐱᓈᐢᑯᐏᐲᓯᒼ", "ᐋᕽᐘᑎᓄᐏᐲᓯᒼ", "ᒪᑯᓭᑮᓭᑳᐏᐲᓯᒼ"},
	{"і҆аⷩ҇", "феⷡ҇", "маⷬ҇", "а҆пⷬ҇", "маꙵ", "і҆ꙋⷩ҇", "і҆ꙋⷧ҇", "а҆́ѵⷢ҇", "сеⷫ҇", "ѻ҆кⷮ", "ноеⷨ", "деⷦ҇"},
	{"і҆аннꙋа́рїа", "феврꙋа́рїа", "ма́рта", "а҆прі́ллїа", "ма́їа", "і҆ꙋ́нїа", "і҆ꙋ́лїа", "а҆́ѵгꙋста", "септе́мврїа", "ѻ҆ктѡ́врїа", "ное́мврїа", "деке́мврїа"},
	{"кӑр.", "нар.", "пуш", "ака", "ҫу", "ҫӗрт.", "утӑ", "ҫур.", "авӑн", "юпа", "чӳк", "раш."},
	{"кӑрлач", "нарӑс", "пуш", "ака", "ҫу", "ҫӗртме", "утӑ", "ҫурла", "авӑн", "юпа", "чӳк", "раштав"},
	{"Ion", "Chwef", "Maw", "Ebr", "Mai", "Meh", "Gorff", "Awst", "Medi", "Hyd", "Tach", "Rhag"},
	{"Ionawr", "Chwefror", "Mawrth", "Ebrill", "Mai", "Mehefin", "Gorffennaf", "Awst", "Medi", "Hydref", "Tachwedd", "Rhagfyr"},
	{"jan.", "feb.", "mar.", "apr.", "maj", "jun.", "jul.", "aug.", "sep.", "okt.", "nov.", "dec."},
	{"januar", "februar", "marts", "april", "maj", "juni", "juli", "august", "september", "oktober", "november", "december"},
	{"Imb", "Kaw", "Kad", "Kan", "Kas", "Kar", "Mfu", "Wun", "Ike", "Iku", "Imw", "Iwi"},
	{"Mori ghwa imbiri", "Mori ghwa kawi", "Mori ghwa kadadu", "Mori ghwa kana", "Mori ghwa kasanu", "Mori ghwa karandadu", "Mori ghwa mfungade", "Mori ghwa wunyanya", "Mori ghwa ikenda", "Mori ghwa ikumi", "Mori ghwa ikumi na imweri", "Mori ghwa ikumi na iwi"},
	{"Jan.", "Feb.", "März", "Apr.", "Mai", "Juni", "Juli", "Aug.", "Sept.", "Okt.", "Nov.", "Dez."},
	{"Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"},
	{"Jän.", "Feb.", "März", "Apr.", "Mai", "Juni", "Juli", "Aug.", "Sep.", "Okt.", "Nov.", "Dez."},
	{"Jänner", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"},
	{"Žan", "Fee", "Mar", "Awi", "Me", "Žuw", "Žuy", "Ut", "Sek", "Okt", "Noo", "Dee"},
	{"Žanwiye", "Feewiriye", "Marsi", "Awiril", "Me", "Žuweŋ", "Žuyye", "Ut", "Sektanbur", "Oktoobur", "Noowanbur", "Deesanbur"},
	{"जन.", "फर.", "मार्च", "अप्रैल", "मेई", "जून", "जुलाई", "अग.", "सित.", "अक्तू.", "नव.", "दिस."},
	{"जनवरी", "फरवरी", "मार्च", "अप्रैल", "मेई", "जून", "जुलाई", "अगस्त", "सितंबर", "अक्तूबर", "नवंबर", "दिसंबर"},
	{"jan.", "feb.", "měr.", "apr.", "maj.", "jun.", "jul.", "awg.", "sep.", "okt.", "now.", "dec."},
	{"januara", "februara", "měrca", "apryla", "maja", "junija", "julija", "awgusta", "septembra", "oktobra", "nowembra", "decembra"},
	{"di", "ŋgɔn", "sɔŋ", "diɓ", "emi", "esɔ", "mad", "diŋ", "nyɛt", "may", "tin", "elá"},
	{"dimɔ́di", "ŋgɔndɛ", "sɔŋɛ", "diɓáɓá", "emiasele", "esɔpɛsɔpɛ", "madiɓɛ́díɓɛ́", "diŋgindi", "nyɛtɛki", "mayésɛ́", "tiníní", "eláŋgɛ́"},
	{"Sa", "Fe", "Ma", "Ab", "Me", "Su", "Sú", "Ut", "Se", "Ok", "No", "De"},
	{"Sanvie", "Fébirie", "Mars", "Aburil", "Mee", "Sueŋ", "Súuyee", "Ut", "Settembar", "Oktobar", "Novembar", "Disambar"},
	{"༡", "༢", "༣", "༤", "༥", "༦", "༧", "༨", "༩", "༡༠", "༡༡", "12"},
	{"༡", "༢", "༣", "4", "༥", "༦", "༧", "༨", "9", "༡༠", "༡༡", "༡༢"},
	{"ཟླ་དངཔ་", "ཟླ་གཉིས་པ་", "ཟླ་གསུམ་པ་", "ཟླ་བཞི་པ་", "ཟླ་ལྔ་པ་", "ཟླ་དྲུག་པ", "ཟླ་བདུན་པ་", "ཟླ་བརྒྱད་པ་", "ཟླ་དགུ་པ་", "ཟླ་བཅུ་པ་", "ཟླ་བཅུ་གཅིག་པ་", "ཟླ་བཅུ་གཉིས་པ་"},
	{"Mbe", "Kai", "Kat", "Kan", "Gat", "Gan", "Mug", "Knn", "Ken", "Iku", "Imw", "Igi"},
	{"Mweri wa mbere", "Mweri wa kaĩri", "Mweri wa kathatũ", "Mweri wa kana", "Mweri wa gatano", "Mweri wa gatantatũ", "Mweri wa mũgwanja", "Mweri wa kanana", "Mweri wa kenda", "Mweri wa ikũmi", "Mweri wa ikũmi na ũmwe", "Mweri wa ikũmi na Kaĩrĩ"},
	{"dzv", "dzd", "ted", "afɔ", "dam", "mas", "sia", "dea", "any", "kel", "ade", "dzm"},
	{"dzove", "dzodze", "tedoxe", "afɔfĩe", "dame", "masa", "siamlɔm", "deasiamime", "anyɔnyɔ", "kele", "adeɛmekpɔxe", "dzome"},
	{"Ιαν", "Φεβ", "Μαρ", "Απρ", "Μαΐ", "Ιουν", "Ιουλ", "Αυγ", "Σεπ", "Οκτ", "Νοε", "Δεκ"},
	{"Ιανουαρίου", "Φεβρουαρίου", "Μαρτίου", "Απριλίου", "Μαΐου", "Ιουνίου", "Ιουλίου", "Αυγούστου", "Σεπτεμβρίου", "Οκτωβρίου", "Νοεμβρίου", "Δεκεμβρίου"},
	{"Ιαν", "Φεβ", "Μαρ", "Απρ", "Μαΐ", "Ιουν", "Ιουλ", "Αὐγ", "Σεπ", "Ὀκτ", "Νοε", "Δεκ"},
	{"Ιανουαρίου", "Φεβρουαρίου", "Μαρτίου", "Απριλίου", "Μαΐου", "Ιουνίου", "Ιουλίου", "Αὐγούστου", "Σεπτεμβρίου", "Ὀκτωβρίου", "Νοεμβρίου", "Δεκεμβρίου"},
	{"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"},
	{"J", "F", "M", "A", "M", "J", "J", "A", "S", "O", "N", "D"},
	{"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"},
	{"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"},
	{"Jan", "Feb", "Mar", "Apr", "May", "June", "July", "Aug", "Sept", "Oct", "Nov", "Dec"},
	{"𐐖𐐰𐑌", "𐐙𐐯𐐺", "𐐣𐐪𐑉", "𐐁𐐹𐑉", "𐐣𐐩", "𐐖𐐭𐑌", "𐐖𐐭𐑊", "𐐂𐑀", "𐐝𐐯𐐹", "𐐉𐐿𐐻", "𐐤𐐬𐑂", "𐐔𐐨𐑅"},
	{"𐐖𐐰𐑌𐐷𐐭𐐯𐑉𐐨", "𐐙𐐯𐐺𐑉𐐭𐐯𐑉𐐨", "𐐣𐐪𐑉𐐽", "𐐁𐐹𐑉𐐮𐑊", "𐐣𐐩", "𐐖𐐭𐑌", "𐐖𐐭𐑊𐐴", "𐐂𐑀𐐲𐑅𐐻", "𐐝𐐯𐐹𐐻𐐯𐑋𐐺𐐲𐑉", "𐐉𐐿𐐻𐐬𐐺𐐲𐑉", "𐐤𐐬𐑂𐐯𐑋𐐺𐐲𐑉", "𐐔𐐨𐑅𐐯𐑋𐐺𐐲𐑉"},
	{"·𐑡𐑨", "·𐑓𐑧", "·𐑥𐑸", "·𐑱𐑐", "·𐑥𐑱", "·𐑡𐑵", "·𐑡𐑫", "·𐑪𐑜", "·𐑕𐑧", "·𐑷𐑒", "·𐑯𐑴", "·𐑛𐑭"},
	{"𐑡", "𐑓", "𐑥", "𐑱", "𐑥", "𐑡", "𐑡", "𐑷", "𐑕", "𐑪", "𐑯", "𐑛"},
	{"·𐑡𐑨𐑙𐑘𐑭𐑢𐑺𐑰", "·𐑓𐑧𐑚𐑘𐑵𐑢𐑺𐑰", "·𐑥𐑸𐑗", "·𐑱𐑐𐑮𐑭𐑤", "·𐑥𐑱", "·𐑡𐑵𐑯", "·𐑡𐑫𐑤𐑲", "·𐑪𐑜𐑭𐑕𐑑", "·𐑕𐑧𐑐𐑑𐑧𐑥𐑚𐑸", "·𐑷𐑒𐑑𐑴𐑚𐑸", "·𐑯𐑴𐑝𐑧𐑥𐑚𐑸", "·𐑛𐑭𐑕𐑧𐑥𐑚𐑸"},
	{"Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Aŭg", "Sep", "Okt", "Nov", "Dec"},
	{"Januaro", "Februaro", "Marto", "Aprilo", "Majo", "Junio", "Julio", "Aŭgusto", "Septembro", "Oktobro", "Novembro", "Decembro"},
	{"ene", "feb", "mar", "abr", "may", "jun", "jul", "ago", "sept", "oct", "nov", "dic"},
	{"enero", "febrero", "marzo", "abril", "mayo", "junio", "julio", "agosto", "septiembre", "octubre", "noviembre", "diciembre"},
	{"ene", "feb", "mar", "abr", "may", "jun", "jul", "ago", "sep", "oct", "nov", "dic"},
	{"ene.", "feb.", "mar.", "abr.", "may.", "jun.", "jul.", "ago.", "set.", "oct.", "nov.", "dic."},
	{"enero", "febrero", "marzo", "abril", "mayo", "junio", "julio", "agosto", "setiembre", "octubre", "noviembre", "diciembre"},
	{"ene.", "feb.", "mar.", "abr.", "may.", "jun.", "jul.", "ago.", "sept.", "oct.", "nov.", "dic."},
	{"jaan", "veebr", "märts", "apr", "mai", "juuni", "juuli", "aug", "sept", "okt", "nov", "dets"},
	{"jaanuar", "veebruar", "märts", "aprill", "mai", "juuni", "juuli", "august", "september", "oktoober", "november", "detsember"},
	{"urt.", "ots.", "mar.", "api.", "mai.", "eka.", "uzt.", "abu.", "ira.", "urr.", "aza.", "abe."},
	{"urtarrila", "otsaila", "martxoa", "apirila", "maiatza", "ekaina", "uztaila", "abuztua", "iraila", "urria", "azaroa", "abendua"},
	{"ngo", "ngb", "ngl", "ngn", "ngt", "ngs", "ngz", "ngm", "nge", "nga", "ngad", "ngab"},
	{"ngɔn osú", "ngɔn bɛ̌", "ngɔn lála", "ngɔn nyina", "ngɔn tána", "ngɔn saməna", "ngɔn zamgbála", "ngɔn mwom", "ngɔn ebulú", "ngɔn awóm", "ngɔn awóm ai dziá", "ngɔn awóm ai bɛ̌"},
	{"ژانویه", "فوریه", "مارس", "آوریل", "مه", "ژوئن", "ژوئیه", "اوت", "سپتامبر", "اکتبر", "نوامبر", "دسامبر"},
	{"ژانویهٔ", "فوریهٔ", "مارس", "آوریل", "مهٔ", "ژوئن", "ژوئیهٔ", "اوت", "سپتامبر", "اکتبر", "نوامبر", "دسامبر"},
	{"جنو", "فبروری", "مارچ", "اپریل", "می", "جون", "جول", "اگست", "سپتمبر", "اکتوبر", "نومبر", "دسم"},
	{"جنوری", "فبروری", "مارچ", "اپریل", "می", "جون", "جولای", "اگست", "سپتمبر", "اکتوبر", "نومبر", "دسمبر"},
	{"sii", "col", "mbo", "see", "duu", "kor", "mor", "juk", "slt", "yar", "jol", "bow"},
	{"siilo", "colte", "mbooy", "seeɗto", "duujal", "korse", "morso", "juko", "siilto", "yarkomaa", "jolal", "bowte"},
	{"𞤅𞤭𞥅𞤤𞤮", "𞤕𞤮𞤤𞤼𞤮", "𞤐𞤦𞤮𞥅𞤴𞤮", "𞤅𞤫𞥅𞤼𞤮", "𞤁𞤵𞥅𞤶𞤮", "𞤑𞤮𞤪𞤧𞤮", "𞤃𞤮𞤪𞤧𞤮", "𞤔𞤵𞤳𞤮", "𞤅𞤭𞤤𞤼𞤮", "𞤒𞤢𞤪𞤳𞤮", "𞤔𞤮𞤤𞤮", "𞤄𞤮𞤱𞤼𞤮"},
	{"tammi", "helmi", "maalis", "huhti", "touko", "kesä", "heinä", "elo", "syys", "loka", "marras", "joulu"},
	{"tammikuuta", "helmikuuta", "maaliskuuta", "huhtikuuta", "toukokuuta", "kesäkuuta", "heinäkuuta", "elokuuta", "syyskuuta", "lokakuuta", "marraskuuta", "joulukuuta"},
	{"Ene", "Peb", "Mar", "Abr", "May", "Hun", "Hul", "Ago", "Set", "Okt", "Nob", "Dis"},
	{"Enero", "Pebrero", "Marso", "Abril", "Mayo", "Hunyo", "Hulyo", "Agosto", "Setyembre", "Oktubre", "Nobyembre", "Disyembre"},
	{"jan.", "feb.", "mar.", "apr.", "mai", "jun.", "jul.", "aug.", "sep.", "okt.", "nov.", "des."},
	{"januar", "februar", "mars", "apríl", "mai", "juni", "juli", "august", "september", "oktober", "november", "desember"},
	{"janv.", "févr.", "mars", "avr.", "mai", "juin", "juil.", "août", "sept.", "oct.", "nov.", "déc."},
	{"janvier", "février", "mars", "avril", "mai", "juin", "juillet", "août", "septembre", "octobre", "novembre", "décembre"},
	{"janv.", "févr.", "mars", "avr.", "mai", "juin", "juill.", "août", "sept.", "oct.", "nov.", "déc."},
	{"jan.", "fév.", "mar.", "avr.", "mai", "jui.", "juil.", "août", "sept.", "oct.", "nov.", "déc."},
	{"Jan", "Feb", "Mar", "Apr", "Mei", "Jün", "Jül", "Aug", "Sep", "Okt", "Nof", "Det"},
	{"Janewoore", "Febrewoore", "Maarts", "April", "Mei", "Jüüne", "Jüüle", "August", "September", "Oktuuber", "Nofember", "Detsember"},
	{"Zen", "Fev", "Mar", "Avr", "Mai", "Jug", "Lui", "Avo", "Set", "Otu", "Nov", "Dic"},
	{"Zenâr", "Fevrâr", "Març", "Avrîl", "Mai", "Jugn", "Lui", "Avost", "Setembar", "Otubar", "Novembar", "Dicembar"},
	{"Jan", "Feb", "Mrt", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Des"},
	{"Jannewaris", "Febrewaris", "Maart", "April", "Maaie", "Juny", "July", "Augustus", "Septimber", "Oktober", "Novimber", "Desimber"},
	{"Ean", "Feabh", "Márta", "Aib", "Beal", "Meith", "Iúil", "Lún", "MFómh", "DFómh", "Samh", "Noll"},
	{"Eanáir", "Feabhra", "Márta", "Aibreán", "Bealtaine", "Meitheamh", "Iúil", "Lúnasa", "Meán Fómhair", "Deireadh Fómhair", "Samhain", "Nollaig"},
	{"Aha", "Ofl", "Ots", "Abe", "Agb", "Otu", "Maa", "Man", "Gbo", "Ant", "Ale", "Afu"},
	{"Aharabata", "Oflɔ", "Otsokrikri", "Abɛibe", "Agbiɛnaa", "Otukwajaŋ", "Maawɛ", "Manyawale", "Gbo", "Antɔŋ", "Alemle", "Afuabe"},
	{"Faoi", "Gearr", "Màrt", "Gibl", "Cèit", "Ògmh", "Iuch", "Lùna", "Sult", "Dàmh", "Samh", "Dùbh"},
	{"dhen Fhaoilleach", "dhen Ghearran", "dhen Mhàrt", "dhen Ghiblean", "dhen Chèitean", "dhen Ògmhios", "dhen Iuchar", "dhen Lùnastal", "dhen t-Sultain", "dhen Dàmhair", "dhen t-Samhain", "dhen Dùbhlachd"},
	{"ጠሐረ", "ከተተ", "መገበ", "አኀዘ", "ግንባት", "ሠንየ", "ሐመለ", "ነሐሰ", "ከረመ", "ጠቀመ", "ኀደረ", "ኀሠሠ"},
	{"xan.", "feb.", "mar.", "abr.", "maio", "xuño", "xul.", "ago.", "set.", "out.", "nov.", "dec."},
	{"x.", "f.", "m.", "a.", "m.", "x.", "x.", "a.", "s.", "o.", "n.", "d."},
	{"xaneiro", "febreiro", "marzo", "abril", "maio", "xuño", "xullo", "agosto", "setembro", "outubro", "novembro", "decembro"},
	{"Jasyteĩ", "Jasykõi", "Jasyapy", "Jasyrundy", "Jasypo", "Jasypoteĩ", "Jasypokõi", "Jasypoapy", "Jasyporundy", "Jasypa", "Jasypateĩ", "Jasypakõi"},
	{"Jan", "Feb", "Mär", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"},
	{"Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "Auguscht", "Septämber", "Oktoober", "Novämber", "Dezämber"},
	{"જાન્યુ", "ફેબ્રુ", "માર્ચ", "એપ્રિલ", "મે", "જૂન", "જુલાઈ", "ઑગસ્ટ", "સપ્ટે", "ઑક્ટો", "નવે", "ડિસે"},
	{"જાન્યુઆરી", "ફેબ્રુઆરી", "માર્ચ", "એપ્રિલ", "મે", "જૂન", "જુલાઈ", "ઑગસ્ટ", "સપ્ટેમ્બર", "ઑક્ટોબર", "નવેમ્બર", "ડિસેમ્બર"},
	{"Can", "Feb", "Mac", "Apr", "Mei", "Jun", "Cul", "Agt", "Sep", "Okt", "Nob", "Dis"},
	{"Chanuari", "Feburari", "Machi", "Apiriri", "Mei", "Juni", "Chulai", "Agosti", "Septemba", "Okitoba", "Nobemba", "Disemba"},
	{"J-guer", "T-arree", "Mayrnt", "Avrril", "Boaldyn", "M-souree", "J-souree", "Luanistyn", "M-fouyir", "J-fouyir", "M-Houney", "M-Nollick"},
	{"Jerrey-geuree", "Toshiaght-arree", "Mayrnt", "Averil", "Boaldyn", "Mean-souree", "Jerrey-souree", "Luanistyn", "Mean-fouyir", "Jerrey-fouyir", "Mee Houney", "Mee ny Nollick"},
	{"Jan", "Fab", "Mar", "Afi", "May", "Yun", "Yul", "Agu", "Sat", "Okt", "Nuw", "Dis"},
	{"Janairu", "Faburairu", "Maris", "Afirilu", "Mayu", "Yuni", "Yuli", "Agusta", "Satumba", "Oktoba", "Nuwamba", "Disamba"},
	{"جَن", "ڢَب", "مَر", "أَڢْر", "مَي", "يُون", "يُول", "أَغُ", "سَت", "أُكْت", "نُو", "دِس"},
	{"جَنَيْرُ", "ڢَبْرَيْرُ", "مَرِسْ", "أَڢْرِلُ", "مَيُ", "يُونِ", "يُولِ", "أَغُسْتَ", "سَتُمْبَ", "أُكْتوُبَ", "نُوَمْبَ", "دِسَمْبَ"},
	{"Ian.", "Pep.", "Mal.", "ʻAp.", "Mei", "Iun.", "Iul.", "ʻAu.", "Kep.", "ʻOk.", "Now.", "Kek."},
	{"Ianuali", "Pepeluali", "Malaki", "ʻApelila", "Mei", "Iune", "Iulai", "ʻAukake", "Kepakemapa", "ʻOkakopa", "Nowemapa", "Kekemapa"},
	{"ינו׳", "פבר׳", "מרץ", "אפר׳", "מאי", "יוני", "יולי", "אוג׳", "ספט׳", "אוק׳", "נוב׳", "דצמ׳"},
	{"ינואר", "פברואר", "מרץ", "אפריל", "מאי", "יוני", "יולי", "אוגוסט", "ספטמבר", "אוקטובר", "נובמבר", "דצמבר"},
	{"जन॰", "फ़र॰", "मार्च", "अप्रैल", "मई", "जून", "जुल॰", "अग॰", "सित॰", "अक्टू॰", "नव॰", "दिस॰"},
	{"जनवरी", "फ़रवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सितंबर", "अक्टूबर", "नवंबर", "दिसंबर"},
	{"𞄆𞄬", "𞄛𞄨𞄱𞄄𞄤𞄲𞄨", "𞄒𞄫𞄰𞄒𞄪𞄱", "𞄤𞄨𞄱", "𞄀𞄪𞄴", "𞄛𞄤𞄱𞄞𞄤𞄦", "𞄔𞄩𞄴𞄆𞄨𞄰", "𞄕𞄩𞄲𞄔𞄄𞄰𞄤", "𞄛𞄤𞄱𞄒𞄤𞄰", "𞄪𞄱𞄀𞄤𞄴", "𞄚𞄦𞄲𞄤𞄚𞄄𞄰𞄫", "𞄒𞄩𞄱𞄔𞄬𞄴"},
	{"sij", "velj", "ožu", "tra", "svi", "lip", "srp", "kol", "ruj", "lis", "stu", "pro"},
	{"siječnja", "veljače", "ožujka", "travnja", "svibnja", "lipnja", "srpnja", "kolovoza", "rujna", "listopada", "studenoga", "prosinca"},
	{"jan.", "feb.", "měr.", "apr.", "mej.", "jun.", "jul.", "awg.", "sep.", "okt.", "now.", "dec."},
	{"januara", "februara", "měrca", "apryla", "meje", "junija", "julija", "awgusta", "septembra", "oktobra", "nowembra", "decembra"},
	{"jan.", "febr.", "márc.", "ápr.", "máj.", "jún.", "júl.", "aug.", "szept.", "okt.", "nov.", "dec."},
	{"január", "február", "március", "április", "május", "június", "július", "augusztus", "szeptember", "október", "november", "december"},
	{"հնվ", "փտվ", "մրտ", "ապր", "մյս", "հնս", "հլս", "օգս", "սեպ", "հոկ", "նոյ", "դեկ"},
	{"հունվարի", "փետրվարի", "մարտի", "ապրիլի", "մայիսի", "հունիսի", "հուլիսի", "օգոստոսի", "սեպտեմբերի", "հոկտեմբերի", "նոյեմբերի", "դեկտեմբերի"},
	{"jan", "feb", "mar", "apr", "mai", "jun", "jul", "aug", "sep", "oct", "nov", "dec"},
	{"j", "f", "m", "a", "m", "j", "j", "a", "s", "o", "n", "d"},
	{"januario", "februario", "martio", "april", "maio", "junio", "julio", "augusto", "septembre", "octobre", "novembre", "decembre"},
	{"Jan", "Feb", "Mar", "Apr", "Mei", "Jun", "Jul", "Agu", "Sep", "Okt", "Nov", "Des"},
	{"Januari", "Februari", "Maret", "April", "Mei", "Juni", "Juli", "Agustus", "September", "Oktober", "November", "Desember"},
	{"jan.", "febr.", "mar.", "apr.", "may", "jun.", "julí", "aug.", "sept.", "oct.", "nov.", "dec."},
	{"januar", "februar", "marte", "april", "may", "junio", "julí", "august", "septembre", "octobre", "novembre", "decembre"},
	{"Jen", "Feb", "Maa", "Epr", "Mee", "Juu", "Jul", "Ọgọ", "Sep", "Ọkt", "Nov", "Dis"},
	{"Jenụwarị", "Febrụwarị", "Maachị", "Epreel", "Mee", "Jun", "Julaị", "Ọgọọst", "Septemba", "Ọktoba", "Novemba", "Disemba"},
	{"ꋍꆪ", "ꑍꆪ", "ꌕꆪ", "ꇖꆪ", "ꉬꆪ", "ꃘꆪ", "ꏃꆪ", "ꉆꆪ", "ꈬꆪ", "ꊰꆪ", "ꊯꊪꆪ", "ꊰꑋꆪ"},
	{"jan.", "feb.", "mar.", "apr.", "maí", "jún.", "júl.", "ágú.", "sep.", "okt.", "nóv.", "des."},
	{"janúar", "febrúar", "mars", "apríl", "maí", "júní", "júlí", "ágúst", "september", "október", "nóvember", "desember"},
	{"gen", "feb", "mar", "apr", "mag", "giu", "lug", "ago", "set", "ott", "nov", "dic"},
	{"gennaio", "febbraio", "marzo", "aprile", "maggio", "giugno", "luglio", "agosto", "settembre", "ottobre", "novembre", "dicembre"},
	{"ᔭᓐᓄᐊᓕ", "ᕕᕝᕗᐊᓕ", "ᒫᑦᓯ", "ᐊᐃᑉᐳᓗ", "ᒪᐃ", "ᔫᓂ", "ᔪᓚᐃ", "ᐊᐅᒡᒍᓯ", "ᓯᑎᐱᕆ", "ᐆᑦᑑᕝᕙ", "ᓄᕕᐱᕆ", "ᑎᓯᐱᕆ"},
	{"1月", "2月", "3月", "4月", "5月", "6月", "7月", "8月", "9月", "10月", "11月", "12月"},
	{"Nduŋmbi Saŋ", "Pɛsaŋ Pɛ́pá", "Pɛsaŋ Pɛ́tát", "Pɛsaŋ Pɛ́nɛ́kwa", "Pɛsaŋ Pataa", "Pɛsaŋ Pɛ́nɛ́ntúkú", "Pɛsaŋ Saambá", "Pɛsaŋ Pɛ́nɛ́fɔm", "Pɛsaŋ Pɛ́nɛ́pfúꞋú", "Pɛsaŋ Nɛgɛ́m", "Pɛsaŋ Ntsɔ̌pmɔ́", "Pɛsaŋ Ntsɔ̌ppá"},
	{"Jan", "Feb", "Mac", "Apr", "Mei", "Jun", "Jul", "Ago", "Sep", "Okt", "Nov", "Des"},
	{"Januari", "Februari", "Machi", "Aprilyi", "Mei", "Junyi", "Julyai", "Agusti", "Septemba", "Oktoba", "Novemba", "Desemba"},
	{"Jan", "Feb", "Mar", "Apr", "Mei", "Jun", "Jul", "Agt", "Sep", "Okt", "Nov", "Des"},
	{"იან", "თებ", "მარ", "აპრ", "მაი", "ივნ", "ივლ", "აგვ", "სექ", "ოქტ", "ნოე", "დეკ"},
	{"იანვარი", "თებერვალი", "მარტი", "აპრილი", "მაისი", "ივნისი", "ივლისი", "აგვისტო", "სექტემბერი", "ოქტომბერი", "ნოემბერი", "დეკემბერი"},
	{"Я", "Ф", "М", "А", "М", "И", "И", "А", "С", "О", "Н", "Д"},
	{"Yen", "Fur", "Meɣ", "Yeb", "May", "Yun", "Yul", "Ɣuc", "Cte", "Tub", "Nun", "Duǧ"},
	{"Yennayer", "Fuṛar", "Meɣres", "Yebrir", "Mayyu", "Yunyu", "Yulyu", "Ɣuct", "Ctembeṛ", "Tubeṛ", "Nunembeṛ", "Duǧembeṛ"},
	{"A̱yr", "A̱hw", "A̱ta", "A̱na", "A̱pf", "A̱ki", "A̱ty", "A̱ni", "A̱ku", "Swa", "Sby", "Sbh"},
	{"Hywan A̱yrnig", "Hywan A̱hwa", "Hywan A̱tat", "Hywan A̱naai", "Hywan A̱pfwon", "Hywan A̱kitat", "Hywan A̱tyirin", "Hywan A̱ninai", "Hywan A̱kumviriyin", "Hywan Swak", "Hywan Swak B’a̱yrnig", "Hywan Swak B’a̱hwa"},
	{"Mbe", "Kel", "Ktũ", "Kan", "Ktn", "Tha", "Moo", "Nya", "Knd", "Ĩku", "Ĩkm", "Ĩkl"},
	{"Mwai wa mbee", "Mwai wa kelĩ", "Mwai wa katatũ", "Mwai wa kana", "Mwai wa katano", "Mwai wa thanthatũ", "Mwai wa muonza", "Mwai wa nyaanya", "Mwai wa kenda", "Mwai wa ĩkumi", "Mwai wa ĩkumi na ĩmwe", "Mwai wa ĩkumi na ilĩ"},
	{"Juw", "Swi", "Tsa", "Nya", "Tsw", "Ata", "Ana", "Ari", "Aku", "Swa", "Man", "Mas"},
	{"Zwat Juwung", "Zwat Swiyang", "Zwat Tsat", "Zwat Nyai", "Zwat Tswon", "Zwat Ataah", "Zwat Anatat", "Zwat Arinai", "Zwat Akubunyung", "Zwat Swag", "Zwat Mangjuwang", "Zwat Swag-Ma-Suyang"},
	{"Mwedi Ntandi", "Mwedi wa Pili", "Mwedi wa Tatu", "Mwedi wa Nchechi", "Mwedi wa Nnyano", "Mwedi wa Nnyano na Umo", "Mwedi wa Nnyano na Mivili", "Mwedi wa Nnyano na Mitatu", "Mwedi wa Nnyano na Nchechi", "Mwedi wa Nnyano na Nnyano", "Mwedi wa Nnyano na Nnyano na U", "Mwedi wa Nnyano na Nnyano na M"},
	{"Jan", "Feb", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Otu", "Nuv", "Diz"},
	{"Janeru", "Febreru", "Marsu", "Abril", "Maiu", "Junhu", "Julhu", "Agostu", "Setenbru", "Otubru", "Nuvenbru", "Dizenbru"},
	{"Xbʼeen Po", "Xkabʼ Po", "Rox Po", "Xkaa Po", "Roʼ Po", "Xwaq Po", "Xwuuq Po", "Xwaqxaq Po", "Xbʼelee Po", "Xlajee Po", "Xjunlaj Po", "Xkabʼlaj Po"},
	{"1Ky.", "2Ky.", "3Ky.", "4Ky.", "5Ky.", "6Ky.", "7Ky.", "8Ky.", "9Ky.", "10Ky.", "11Ky.", "12Ky."},
	{"1-Kysã", "2-Kysã", "3-Kysã", "4-Kysã", "5-Kysã", "6-Kysã", "7-Kysã", "8-Kysã", "9-Kysã", "10-Kysã", "11-Kysã", "12-Kysã"},
	{"JEN", "WKR", "WGT", "WKN", "WTN", "WTD", "WMJ", "WNN", "WKD", "WIK", "WMW", "DIT"},
	{"Njenuarĩ", "Mwere wa kerĩ", "Mwere wa gatatũ", "Mwere wa kana", "Mwere wa gatano", "Mwere wa gatandatũ", "Mwere wa mũgwanja", "Mwere wa kanana", "Mwere wa kenda", "Mwere wa ikũmi", "Mwere wa ikũmi na ũmwe", "Ndithemba"},
	{"қаң.", "ақп.", "нау.", "сәу.", "мам.", "мау.", "шіл.", "там.", "қыр.", "қаз.", "қар.", "жел."},
	{"қаңтар", "ақпан", "наурыз", "сәуір", "мамыр", "маусым", "шілде", "тамыз", "қыркүйек", "қазан", "қараша", "желтоқсан"},
	{"قاڭتار", "اقپان", "ناۋرىز", "ءساۋىر", "مامىر", "ماۋسىم", "شىلدە", "تامىز", "قىركۇيەك", "قازان", "قاراشا", "جەلتوقسان"},
	{"pamba", "wanja", "mbiyɔ mɛndoŋgɔ", "Nyɔlɔmbɔŋgɔ", "Mɔnɔ ŋgbanja", "Nyaŋgwɛ ŋgbanja", "kuŋgwɛ", "fɛ", "njapi", "nyukul", "M11", "ɓulɓusɛ"},
	{"jan", "febr", "mar", "apr", "maj", "jun", "jul", "aug", "sept", "okt", "nov", "dec"},
	{"januaarip", "februaarip", "marsip", "apriilip", "maajip", "juunip", "juulip", "aggustip", "septembarip", "oktobarip", "novembarip", "decembarip"},
	{"Mul", "Ngat", "Taa", "Iwo", "Mam", "Paa", "Nge", "Roo", "Bur", "Epe", "Kpt", "Kpa"},
	{"Mulgul", "Ng’atyaato", "Kiptaamo", "Iwootkuut", "Mamuut", "Paagi", "Ng’eiyeet", "Rooptui", "Bureet", "Epeeso", "Kipsuunde ne taai", "Kipsuunde nebo aeng’"},
	{"មករា", "កុម្ភៈ", "មីនា", "មេសា", "ឧសភា", "មិថុនា", "កក្កដា", "សីហា", "កញ្ញា", "តុលា", "វិច្ឆិកា", "ធ្នូ"},
	{"ಜನ", "ಫೆಬ್ರ", "ಮಾರ್ಚ್", "ಏಪ್ರಿ", "ಮೇ", "ಜೂನ್", "ಜುಲೈ", "ಆಗ", "ಸೆಪ್ಟೆಂ", "ಅಕ್ಟೋ", "ನವೆಂ", "ಡಿಸೆಂ"},
	{"ಜನವರಿ", "ಫೆಬ್ರವರಿ", "ಮಾರ್ಚ್", "ಏಪ್ರಿಲ್", "ಮೇ", "ಜೂನ್", "ಜುಲೈ", "ಆಗಸ್ಟ್", "ಸೆಪ್ಟೆಂಬರ್", "ಅಕ್ಟೋಬರ್", "ನವೆಂಬರ್", "ಡಿಸೆಂಬರ್"},
	{"1월", "2월", "3월", "4월", "5월", "6월", "7월", "8월", "9월", "10월", "11월", "12월"},
	{"जाने", "फेब्रु", "मार्च", "एप्री", "मे", "जून", "जुल", "ऑग", "सप्टें", "ऑक्टो", "नोव्हें", "डिसें"},
	{"जा", "फे", "मा", "ए", "मे", "जू", "जु", "ऑ", "स", "ऑ", "नो", "डि"},
	{"जानेवारी", "फेब्रुवारी", "मार्च", "एप्रील", "मे", "जून", "जुलय", "ऑगस्ट", "सप्टेंबर", "ऑक्टोबर", "नोव्हेंबर", "डिसेंबर"},
	{"Jan", "Feb", "Mar", "Abr", "Mai", "Jun", "Jul", "Ago", "Set", "Otu", "Nov", "Dez"},
	{"Janer", "Febrer", "Mars", "Abril", "Mai", "Jun", "Julai", "Agost", "Setembr", "Otubr", "Novembr", "Dezembr"},
	{"جنؤری", "فرؤری", "مارٕچ", "اپریل", "مئی", "جوٗن", "جُلَے", "اگست", "ستمبر", "اکتوٗبر", "نومبر", "دسمبر"},
	{"جنؤری", "فرؤری", "مارٕچ", "اپریل", "مئی", "جوٗن", "جُلَے", "اگست", "ستمبر", "اکتوٗبر", "نومبر", "دَسَمبَر"},
	{"जनवरी", "फ़रवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सतुंबर", "अक्तूबर", "नवूमबर", "दसमबर"},
	{"जनवरी", "फ़रवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सतमबर", "अक्तूबर", "नवमबर", "दसमबर"},
	{"Januali", "Febluali", "Machi", "Aplili", "Mei", "Juni", "Julai", "Agosti", "Septemba", "Oktoba", "Novemba", "Desemba"},
	{"ŋ1", "ŋ2", "ŋ3", "ŋ4", "ŋ5", "ŋ6", "ŋ7", "ŋ8", "ŋ9", "ŋ10", "ŋ11", "ŋ12"},
	{"ŋwíí a ntɔ́ntɔ", "ŋwíí akǝ bɛ́ɛ", "ŋwíí akǝ ráá", "ŋwíí akǝ nin", "ŋwíí akǝ táan", "ŋwíí akǝ táafɔk", "ŋwíí akǝ táabɛɛ", "ŋwíí akǝ táaraa", "ŋwíí akǝ táanin", "ŋwíí akǝ ntɛk", "ŋwíí akǝ ntɛk di bɔ́k", "ŋwíí akǝ ntɛk di bɛ́ɛ"},
	{"Jan", "Fäb", "Mäz", "Apr", "Mai", "Jun", "Jul", "Ouj", "Säp", "Okt", "Nov", "Dez"},
	{"Jannewa", "Fäbrowa", "Määz", "Aprell", "Mai", "Juuni", "Juuli", "Oujoß", "Septämber", "Oktohber", "Novämber", "Dezämber"},
	{"rbn", "sbt", "adr", "nsn", "gln", "hzr", "trm", "tbx", "îln", "cot", "mjd", "brf"},
	{"rêbendan", "sibat", "adar", "nîsan", "gulan", "hezîran", "tîrmeh", "tebax", "îlon", "cotmeh", "mijdar", "berfanbar"},
	{"Gen", "Hwe", "Meu", "Ebr", "Me", "Met", "Gor", "Est", "Gwn", "Hed", "Du", "Kev"},
	{"mis Genver", "mis Hwevrer", "mis Meurth", "mis Ebrel", "mis Me", "mis Metheven", "mis Gortheren", "mis Est", "mis Gwynngala", "mis Hedra", "mis Du", "mis Kevardhu"},
	{"pusu", "maha", "pagu", "hire", "bese", "jaṭṭa", "aasaḍi", "srabĩ", "bado", "dasara", "divi", "pande"},
	{"pusu lenju", "maha lenju", "pagu lenju", "hire lenju", "bese lenju", "jaṭṭa lenju", "aasaḍi lenju", "srabĩ lenju", "bado lenju", "dasara lenju", "divi lenju", "pande lenju"},
	{"पुसु", "माहा", "पागु", "हिर्रे", "बेसे", "जाट्टा", "आसाड़ी", "स्राबाँ", "बाॅदो", "दासारा", "दिवी", "पान्डे"},
	{"पुसु लेञ्जु", "माहाका लेञ्जु", "पागुणी लेञ्जु", "हिरे लेञ्जु", "बेसे लेञ्जु", "जाटा लेञ्जु", "आसाड़ी लेञ्जु", "स्राबाँ लेञ्जु", "बोदो लेञ्जु", "दसारा लेञ्जु", "दिवी लेञ्जु", "पान्डे लेञ्जु"},
	{"ପୁସୁ", "ମାହା", "ପାଗୁ", "ହିରେ", "ବେସେ", "ଜାଟା", "ଆସାଡ଼ି", "ସ୍ରାବାଁ", "ବଦ", "ଦାସାରା", "ଦିୱି", "ପାଣ୍ଡେ"},
	{"ପୁସୁ ଲେଞ୍ଜୁ", "ମାହାକା ଲେଞ୍ଜୁ", "ପାଗୁଣି ଲେଞ୍ଜୁ", "ହିରେ ଲେଞ୍ଜୁ", "ବେସେ ଲେଞ୍ଜୁ", "ଜାଟା ଲେଞ୍ଜୁ", "ଆସାଡ଼ି ଲେଞ୍ଜୁ", "ସ୍ରାବାଁ ଲେଞ୍ଜୁ", "ବଦ ଲେଞ୍ଜୁ", "ଦାସାରା ଲେଞ୍ଜୁ", "ଦିୱିଡ଼ି ଲେଞ୍ଜୁ", "ପାଣ୍ଡେ ଲେଞ୍ଜୁ"},
	{"మాగ", "గుండు", "హిరెఇ", "బెసెకి", "లండి", "రాత", "బాన్దపాణా", "బార్సి", "అస్ర", "దివెడి", "పాండు", "పుసు"},
	{"янв.", "фев.", "мар.", "апр.", "май", "июн.", "июл.", "авг.", "сен.", "окт.", "ноя.", "дек."},
	{"Ian", "Feb", "Mar", "Apr", "Mai", "Iun", "Iul", "Aug", "Sep", "Oct", "Nov", "Dec"},
	{"Ianuarii", "Februarii", "Martii", "Aprilis", "Maii", "Iunii", "Iulii", "Augusti", "Septembris", "Octobris", "Novembris", "Decembris"},
	{"Fúngatɨ", "Naanɨ", "Keenda", "Ikúmi", "Inyambala", "Idwaata", "Mʉʉnchɨ", "Vɨɨrɨ", "Saatʉ", "Inyi", "Saano", "Sasatʉ"},
	{"Kʉfúngatɨ", "Kʉnaanɨ", "Kʉkeenda", "Kwiikumi", "Kwiinyambála", "Kwiidwaata", "Kʉmʉʉnchɨ", "Kʉvɨɨrɨ", "Kʉsaatʉ", "Kwiinyi", "Kʉsaano", "Kʉsasatʉ"},
	{"Jan.", "Feb.", "Mäe.", "Abr.", "Mee", "Juni", "Juli", "Aug.", "Sep.", "Okt.", "Nov.", "Dez."},
	{"Januar", "Februar", "Mäerz", "Abrëll", "Mee", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"},
	{"Jan", "Feb", "Mar", "Apu", "Maa", "Juu", "Jul", "Agu", "Seb", "Oki", "Nov", "Des"},
	{"Janwaliyo", "Febwaliyo", "Marisi", "Apuli", "Maayi", "Juuni", "Julaayi", "Agusito", "Sebuttemba", "Okitobba", "Novemba", "Desemba"},
	{"de zen.", "de fre.", "de mar.", "d’arv.", "de maz.", "de zug.", "de lug.", "d’ago.", "de set.", "d’ott.", "de nov.", "de dex."},
	{"de zenâ", "de frevâ", "de marso", "d’arvî", "de mazzo", "de zugno", "de luggio", "d’agosto", "de settembre", "d’ottobre", "de novembre", "de dexembre"},
	{"Wiótheȟika Wí", "Thiyóȟeyuŋka Wí", "Ištáwičhayazaŋ Wí", "Pȟežítȟo Wí", "Čhaŋwápetȟo Wí", "Wípazukȟa-wašté Wí", "Čhaŋpȟásapa Wí", "Wasútȟuŋ Wí", "Čhaŋwápeǧi Wí", "Čhaŋwápe-kasná Wí", "Waníyetu Wí", "Tȟahékapšuŋ Wí"},
	{"J", "F", "M", "A", "M", "J", "M", "A", "S", "O", "N", "D"},
	{"de jená", "de forá", "de merz", "d’aurí", "de ma", "de jügn", "de messé", "d’aost", "de setëmber", "d’otober", "de novëmber", "de dezëmber"},
	{"sginer", "fevrer", "marz", "avril", "masg", "sgiugn", "luj", "avost", "setember", "otover", "november", "dicember"},
	{"yan", "fbl", "msi", "apl", "mai", "yun", "yul", "agt", "stb", "ɔtb", "nvb", "dsb"},
	{"sánzá ya yambo", "sánzá ya míbalé", "sánzá ya mísáto", "sánzá ya mínei", "sánzá ya mítáno", "sánzá ya motóbá", "sánzá ya nsambo", "sánzá ya mwambe", "sánzá ya libwa", "sánzá ya zómi", "sánzá ya zómi na mɔ̌kɔ́", "sánzá ya zómi na míbalé"},
	{"ມ.ກ.", "ກ.ພ.", "ມ.ນ.", "ມ.ສ.", "ພ.ພ.", "ມິ.ຖ.", "ກ.ລ.", "ສ.ຫ.", "ກ.ຍ.", "ຕ.ລ.", "ພ.ຈ.", "ທ.ວ."},
	{"ມັງກອນ", "ກຸມພາ", "ມີນາ", "ເມສາ", "ພຶດສະພາ", "ມິຖຸນາ", "ກໍລະກົດ", "ສິງຫາ", "ກັນຍາ", "ຕຸລາ", "ພະຈິກ", "ທັນວາ"},
	{"جانڤیە", "فئڤریە", "مارس", "آڤریل", "مئی", "جوٙأن", "جوٙلا", "آگوست", "سئپتامر", "ئوکتوڤر", "نوڤامر", "دئسامر"},
	{"saus.", "vas.", "kov.", "bal.", "geg.", "birž.", "liep.", "rugp.", "rugs.", "spal.", "lapkr.", "gruod."},
	{"sausio", "vasario", "kovo", "balandžio", "gegužės", "birželio", "liepos", "rugpjūčio", "rugsėjo", "spalio", "lapkričio", "gruodžio"},
	{"janvars", "februars", "marts", "apreļs", "majs", "juņs", "juļs", "augusts", "septembris", "oktobris", "novembris", "decembris"},
	{"Cio", "Lui", "Lus", "Muu", "Lum", "Luf", "Kab", "Lush", "Lut", "Lun", "Kas", "Cis"},
	{"Ciongo", "Lùishi", "Lusòlo", "Mùuyà", "Lumùngùlù", "Lufuimi", "Kabàlàshìpù", "Lùshìkà", "Lutongolo", "Lungùdi", "Kaswèkèsè", "Ciswà"},
	{"DAC", "DAR", "DAD", "DAN", "DAH", "DAU", "DAO", "DAB", "DOC", "DAP", "DGI", "DAG"},
	{"Dwe mar Achiel", "Dwe mar Ariyo", "Dwe mar Adek", "Dwe mar Ang’wen", "Dwe mar Abich", "Dwe mar Auchiel", "Dwe mar Abiriyo", "Dwe mar Aboro", "Dwe mar Ochiko", "Dwe mar Apar", "Dwe mar gi achiel", "Dwe mar Apar gi ariyo"},
	{"Jan", "Feb", "Mar", "Apr", "Mei", "Jun", "Jul", "Ago", "Sep", "Okt", "Nov", "Des"},
	{"janv.", "febr.", "marts", "apr.", "maijs", "jūn.", "jūl.", "aug.", "sept.", "okt.", "nov.", "dec."},
	{"janvāris", "februāris", "marts", "aprīlis", "maijs", "jūnijs", "jūlijs", "augusts", "septembris", "oktobris", "novembris", "decembris"},
	{"जन॰", "फ़र॰", "मार्च", "अप्रैल", "मई", "जून", "जुल॰", "अग॰", "सित॰", "अक्तू॰", "नव॰", "दिस॰"},
	{"जनवरी", "फरवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सितंबर", "अक्तूबर", "नवंबर", "दिसंबर"},
	{"Dal", "Ará", "Ɔɛn", "Doy", "Lép", "Rok", "Sás", "Bɔ́r", "Kús", "Gís", "Shʉ́", "Ntʉ́"},
	{"Oladalʉ́", "Arát", "Ɔɛnɨ́ɔɨŋɔk", "Olodoyíóríê inkókúâ", "Oloilépūnyīē inkókúâ", "Kújúɔrɔk", "Mórusásin", "Ɔlɔ́ɨ́bɔ́rárɛ", "Kúshîn", "Olgísan", "Pʉshʉ́ka", "Ntʉ́ŋʉ́s"},
	{"JAN", "FEB", "MAC", "ĨPU", "MĨĨ", "NJU", "NJR", "AGA", "SPT", "OKT", "NOV", "DEC"},
	{"Januarĩ", "Feburuarĩ", "Machi", "Ĩpurũ", "Mĩĩ", "Njuni", "Njuraĩ", "Agasti", "Septemba", "Oktũba", "Novemba", "Dicemba"},
	{"zan", "fev", "mar", "avr", "me", "zin", "zil", "out", "sep", "okt", "nov", "des"},
	{"zanvie", "fevriye", "mars", "avril", "me", "zin", "zilye", "out", "septam", "oktob", "novam", "desam"},
	{"Jan", "Feb", "Mar", "Apr", "Mey", "Jon", "Jol", "Aog", "Sep", "Okt", "Nov", "Des"},
	{"Janoary", "Febroary", "Martsa", "Aprily", "Mey", "Jona", "Jolay", "Aogositra", "Septambra", "Oktobra", "Novambra", "Desambra"},
	{"Kwa", "Una", "Rar", "Che", "Tha", "Moc", "Sab", "Nan", "Tis", "Kum", "Moj", "Yel"},
	{"Mweri wo kwanza", "Mweri wo unayeli", "Mweri wo uneraru", "Mweri wo unecheshe", "Mweri wo unethanu", "Mweri wo thanu na mocha", "Mweri wo saba", "Mweri wo nane", "Mweri wo tisa", "Mweri wo kumi", "Mweri wo kumi na moja", "Mweri wo kumi na yel’li"},
	{"mbegtug", "imeg àbùbì", "imeg mbəŋchubi", "iməg ngwə̀t", "iməg fog", "iməg ichiibɔd", "iməg àdùmbə̀ŋ", "iməg ichika", "iməg kud", "iməg tèsiʼe", "iməg zò", "iməg krizmed"},
	{"iməg mbegtug", "imeg àbùbì", "imeg mbəŋchubi", "iməg ngwə̀t", "iməg fog", "iməg ichiibɔd", "iməg àdùmbə̀ŋ", "iməg ichika", "iməg kud", "iməg tèsiʼe", "iməg zò", "iməg krizmed"},
	{"Hān", "Pēp", "Māe", "Āpe", "Mei", "Hun", "Hūr", "Āku", "Hep", "Oke", "Noe", "Tīh"},
	{"Hānuere", "Pēpuere", "Māehe", "Āpereira", "Mei", "Hune", "Hūrae", "Ākuhata", "Hepetema", "Oketopa", "Noema", "Tīhema"},
	{"јан.", "фев.", "мар.", "апр.", "мај", "јун.", "јул.", "авг.", "сеп.", "окт.", "ное.", "дек."},
	{"јануари", "февруари", "март", "април", "мај", "јуни", "јули", "август", "септември", "октомври", "ноември", "декември"},
	{"ജനു", "ഫെബ്രു", "മാർ", "ഏപ്രി", "മേയ്", "ജൂൺ", "ജൂലൈ", "ഓഗ", "സെപ്റ്റം", "ഒക്ടോ", "നവം", "ഡിസം"},
	{"ജനുവരി", "ഫെബ്രുവരി", "മാർച്ച്", "ഏപ്രിൽ", "മേയ്", "ജൂൺ", "ജൂലൈ", "ഓഗസ്റ്റ്", "സെപ്റ്റംബർ", "ഒക്\u200cടോബർ", "നവംബർ", "ഡിസംബർ"},
	{"1-р сар", "2-р сар", "3-р сар", "4-р сар", "5-р сар", "6-р сар", "7-р сар", "8-р сар", "9-р сар", "10-р сар", "11-р сар", "12-р сар"},
	{"нэгдүгээр сар", "хоёрдугаар сар", "гуравдугаар сар", "дөрөвдүгээр сар", "тавдугаар сар", "зургаадугаар сар", "долоодугаар сар", "наймдугаар сар", "есдүгээр сар", "аравдугаар сар", "арван нэгдүгээр сар", "арван хоёрдугаар сар"},
	{"1\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "2\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "3᠊ᠷ ᠰᠠᠷ\u180eᠠ", "4\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "5\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "6\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "7\u202f᠊ᠷ ᠰᠠᠷ\u180eᠠ", "8᠊ᠷ ᠰᠠᠷ\u180eᠠ", "9 ᠊ᠷ ᠰᠠᠷ\u180eᠠ", "10 ᠊ᠷ ᠰᠠᠷ\u180eᠠ", "11 ᠊ᠷ ᠰᠠᠷ\u180eᠠ", "12 ᠊ᠷ ᠰᠠᠷ\u180eᠠ"},
	{"ᠨᠢᠭᠡᠳᠥᠭᠡᠷ ᠰᠠᠷ\u180eᠠ", "ᠬᠣᠶᠠᠳᠣᠭᠠᠷ ᠰᠠᠷ\u202fᠠ", "ᠭᠣᠷᠪᠡᠳᠣᠭᠠᠷ ᠰᠠᠷ\u202fᠠ", "ᠳᠥᠷᠪᠡᠳᠥᠭᠡᠷ ᠰᠠᠷ\u180eᠠ", "ᠲᠠᠪᠣᠳᠣᠭᠠᠷ ᠰᠠᠷ\u202fᠠ", "ᠵᠢᠷᠭᠣᠭᠠᠳᠣᠭᠠᠷ ᠰᠠᠷ\u180eᠠ", "ᠲᠣᠯᠣᠭᠠᠳᠣᠭᠠᠷ ᠰᠠᠷ\u180eᠠ", "ᠨᠠᠢᠮᠠᠳᠥᠭᠠᠷ ᠰᠠᠷ\u180eᠠ", "ᠶᠢᠰᠥᠳᠥᠭᠡᠷ ᠰᠠᠷ\u180eᠠ", "ᠠᠷᠪᠠᠳᠣᠭᠠᠷ ᠰᠠᠷ\u180eᠠ", "ᠠᠷᠪᠠᠨ ᠨᠢᠭᠡᠳᠥᠭᠡᠷ ᠰᠠᠷ\u180eᠠ", "ᠠᠷᠪᠠᠨ ᠬᠣᠶᠠᠳᠣᠭᠠᠷ ᠰᠠᠷ\u180eᠠ"},
	{"জন", "ফেব্রুৱারি", "মার্চ", "এপ্রিল", "মে", "জুন", "জুলাই", "ওগ", "সেপ্টেম্বর", "ওক্টোবর", "নভেম্বর", "ডিসেম্বর"},
	{"জা", "ফে", "মার", "এপ", "মে", "জুন", "জুল", "আ", "সে", "ওক", "নব", "ডি"},
	{"জনুৱারী", "ফেব্রুৱারি", "মার্চ", "এপ্রিল", "মে", "জুন", "জুলাই", "\u200cওগষ্ট", "সেপ্টেম্বর", "ওক্টোবর", "নভেম্বর", "ডিসেম্বর"},
	{"जाने", "फेब्रु", "मार्च", "एप्रि", "मे", "जून", "जुलै", "ऑग", "सप्टें", "ऑक्टो", "नोव्हें", "डिसें"},
	{"जानेवारी", "फेब्रुवारी", "मार्च", "एप्रिल", "मे", "जून", "जुलै", "ऑगस्ट", "सप्टेंबर", "ऑक्टोबर", "नोव्हेंबर", "डिसेंबर"},
	{"Jan", "Feb", "Mac", "Apr", "Mei", "Jun", "Jul", "Ogo", "Sep", "Okt", "Nov", "Dis"},
	{"Januari", "Februari", "Mac", "April", "Mei", "Jun", "Julai", "Ogos", "September", "Oktober", "November", "Disember"},
	{"جانواري", "فيبواري", "مچ", "اڤريل", "مي", "جون", "جولاي", "ݢوس", "سيڤتيمبر", "اوکتوبر", "نوۏيمبر", "ديسيمبر"},
	{"Jan", "Fra", "Mar", "Apr", "Mej", "Ġun", "Lul", "Aww", "Set", "Ott", "Nov", "Diċ"},
	{"J", "F", "M", "A", "M", "Ġ", "L", "A", "S", "O", "N", "D"},
	{"Jannar", "Frar", "Marzu", "April", "Mejju", "Ġunju", "Lulju", "Awwissu", "Settembru", "Ottubru", "Novembru", "Diċembru"},
	{"FLO", "CLA", "CKI", "FMF", "MAD", "MBI", "MLI", "MAM", "FDE", "FMU", "FGW", "FYU"},
	{"Fĩi Loo", "Cokcwaklaŋne", "Cokcwaklii", "Fĩi Marfoo", "Madǝǝuutǝbijaŋ", "Mamǝŋgwãafahbii", "Mamǝŋgwãalii", "Madǝmbii", "Fĩi Dǝɓlii", "Fĩi Mundaŋ", "Fĩi Gwahlle", "Fĩi Yuru"},
	{"Rvfo Cuse", "Hotvle Hvse", "Tasahcuce", "Tasahce Rakko", "Ke Hvse", "Kvco Hvse", "Hiyuce", "Hiyo Rakko", "Otowoskuce", "Otowoskv Rakko", "Ehole", "Rvfo Rakko"},
	{"ဇန်", "ဖေ", "မတ်", "ဧ", "မေ", "ဇွန်", "ဇူ", "ဩ", "စက်", "အောက်", "နို", "ဒီ"},
	{"ဇန်နဝါရီ", "ဖေဖော်ဝါရီ", "မတ်", "ဧပြီ", "မေ", "ဇွန်", "ဇူလိုင်", "ဩဂုတ်", "စက်တင်ဘာ", "အောက်တိုဘာ", "နိုဝင်ဘာ", "ဒီဇင်ဘာ"},
	{"якшамков", "даволков", "эйзюрков", "чадыков", "панжиков", "аштемков", "медьков", "умарьков", "таштамков", "ожоков", "сундерьков", "ацамков"},
	{"ǃKhanni", "ǃKhanǀgôab", "ǀKhuuǁkhâb", "ǃHôaǂkhaib", "ǃKhaitsâb", "Gamaǀaeb", "ǂKhoesaob", "Aoǁkhuumûǁkhâb", "Taraǀkhuumûǁkhâb", "ǂNûǁnâiseb", "ǀHooǂgaeb", "Hôasoreǁkhâb"},
	{"jan.", "feb.", "mars", "apr.", "mai", "juni", "juli", "aug.", "sep.", "okt.", "nov.", "des."},
	{"januar", "februar", "mars", "april", "mai", "juni", "juli", "august", "september", "oktober", "november", "desember"},
	{"Zib", "Nhlo", "Mbi", "Mab", "Nkw", "Nhla", "Ntu", "Ncw", "Mpan", "Mfu", "Lwe", "Mpal"},
	{"Zibandlela", "Nhlolanja", "Mbimbitho", "Mabasa", "Nkwenkwezi", "Nhlangula", "Ntulikazi", "Ncwabakazi", "Mpandula", "Mfumfu", "Lwezi", "Mpalakazi"},
	{"Jan.", "Feb.", "März", "Apr.", "Mai", "Juni", "Juli", "Aug.", "Sep.", "Okt.", "Nov.", "Dez."},
	{"Januaar", "Februaar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktover", "November", "Dezember"},
	{"जन", "फेब", "मार्च", "अप्र", "मे", "जुन", "जुल", "अग", "सेप", "अक्टो", "नोभे", "डिसे"},
	{"जनवरी", "फेब्रुअरी", "मार्च", "अप्रिल", "मे", "जुन", "जुलाई", "अगस्ट", "सेप्टेम्बर", "अक्टोबर", "नोभेम्बर", "डिसेम्बर"},
	{"jan", "feb", "mrt", "apr", "mei", "jun", "jul", "aug", "sep", "okt", "nov", "dec"},
	{"januari", "februari", "maart", "april", "mei", "juni", "juli", "augustus", "september", "oktober", "november", "december"},
	{"ng1", "ng2", "ng3", "ng4", "ng5", "ng6", "ng7", "ng8", "ng9", "ng10", "ng11", "kris"},
	{"ngwɛn matáhra", "ngwɛn ńmba", "ngwɛn ńlal", "ngwɛn ńna", "ngwɛn ńtan", "ngwɛn ńtuó", "ngwɛn hɛmbuɛrí", "ngwɛn lɔmbi", "ngwɛn rɛbvuâ", "ngwɛn wum", "ngwɛn wum navǔr", "krísimin"},
	{"saŋ tsetsɛ̀ɛ lùm", "saŋ kàg ngwóŋ", "saŋ lepyè shúm", "saŋ cÿó", "saŋ tsɛ̀ɛ cÿó", "saŋ njÿoláʼ", "saŋ tyɛ̀b tyɛ̀b mbʉ̀ŋ", "saŋ mbʉ̀ŋ", "saŋ ngwɔ̀ʼ mbÿɛ", "saŋ tàŋa tsetsáʼ", "saŋ mejwoŋó", "saŋ lùm"},
	{"ߓߌ߲ߠ", "ߞߏ߲ߞ", "ߕߙߊ", "ߞߏ߲ߘ", "ߘߓߊ߬ߕ", "ߥߊ߬ߛ", "ߞߊ߬ߙ", "ߘߓߊ߬ߓ", "ߕߎߟߊߝߌ߲", "ߞߏ߲ߓ", "ߣߍߣ", "ߞߏߟ"},
	{"ߓߌ߲ߠߊߥߎߟߋ߲", "ߞߏ߲ߞߏߜߍ", "ߕߙߊߓߊ", "ߞߏ߲ߞߏߘߌ߬ߓߌ", "ߘߓߊ߬ߕߊ", "ߥߊ߬ߛߌ߬ߥߙߊ", "ߞߊ߬ߙߌߝߐ߭", "ߘߓߊ߬ߓߌߟߊ", "ߕߎߟߊߝߌ߲", "ߞߏ߲ߓߌߕߌ߮", "ߣߍߣߍߓߊ", "ߞߏߟߌ߲ߞߏߟߌ߲"},
	{"Jan", "Feb", "Mat", "Apr", "Mey", "Jun", "Jul", "Arh", "Sep", "Okt", "Usi", "Dis"},
	{"Janabari", "uFeberbari", "uMatjhi", "u-Apreli", "Meyi", "Juni", "Julayi", "Arhostosi", "Septemba", "Oktoba", "Usinyikhaba", "Disemba"},
	{"Jan", "Feb", "Mat", "Apo", "Mei", "Jun", "Jul", "Ago", "Sep", "Okt", "Nof", "Dis"},
	{"Janeware", "Febereware", "Matšhe", "Aporele", "Mei", "June", "Julae", "Agosetose", "Setemere", "Oktobore", "Nofemere", "Disemere"},
	{"Tiop", "Pɛt", "Duɔ̱ɔ̱", "Guak", "Duä", "Kor", "Pay", "Thoo", "Tɛɛ", "Laa", "Kur", "Tid"},
	{"Tiop thar pɛt", "Pɛt", "Duɔ̱ɔ̱ŋ", "Guak", "Duät", "Kornyoot", "Pay yie̱tni", "Tho̱o̱r", "Tɛɛr", "Laath", "Kur", "Tio̱p in di̱i̱t"},
	{"Jan", "Feb", "Mal", "Epu", "Mei", "Jun", "Jul", "Oga", "Sep", "Oku", "Nov", "Dis"},
	{"Januwale", "Febuluwale", "Malichi", "Epulo", "Mei", "Juni", "Julai", "Ogasiti", "Seputemba", "Okutoba", "Novemba", "Disemba"},
	{"gen.", "feb.", "març", "abr.", "mai", "junh", "jul.", "ago.", "set.", "oct.", "nov.", "dec."},
	{"de genièr", "de febrièr", "de març", "d’abril", "de mai", "de junh", "de julhet", "d’agost", "de setembre", "d’octòbre", "de novembre", "de decembre"},
	{"gèr", "her", "mar", "abr", "mai", "jun", "jur", "ago", "set", "oct", "nov", "dec"},
	{"G", "H", "M", "A", "M", "J", "J", "A", "S", "O", "N", "D"},
	{"gèr", "hereuèr", "març", "abriu", "mai", "junh", "juriòl", "agost", "seteme", "octobre", "noveme", "deseme"},
	{"Ama", "Gur", "Bitootessa", "Elb", "Cam", "Wax", "Ado", "Hag", "Ful", "Onk", "Sadaasa", "Mud"},
	{"Amajjii", "Guraandhala", "Bitootessa", "Eebila", "Caamsaa", "Waxabajjii", "Adoolessa", "Hagayya", "Fulbaana", "Onkoloolessa", "Sadaasa", "Mudde"},
	{"ଜାନୁଆରୀ", "ଫେବୃଆରୀ", "ମାର୍ଚ୍ଚ", "ଅପ୍ରେଲ", "ମଇ", "ଜୁନ", "ଜୁଲାଇ", "ଅଗଷ୍ଟ", "ସେପ୍ଟେମ୍ବର", "ଅକ୍ଟୋବର", "ନଭେମ୍ବର", "ଡିସେମ୍ବର"},
	{"янв.", "фев.", "мар.", "апр.", "майы", "июны", "июлы", "авг.", "сен.", "окт.", "ноя.", "дек."},
	{"январы", "февралы", "мартъийы", "апрелы", "майы", "июны", "июлы", "августы", "сентябры", "октябры", "ноябры", "декабры"},
	{"𐓄𐓘𐓡𐓛͘𐓧𐓟", "𐓵𐓪͘𐓬𐓘", "𐓵𐓘𐓜𐓣", "𐓰𐓪𐓬𐓘", "𐓮𐓘𐓰𐓘", "𐓯𐓘𐓬𐓟", "𐓄𐓟𐓵𐓪͘𐓬𐓘", "𐒼𐓣𐓟𐓰𐓪𐓬𐓘", "𐒿𐓟𐓜𐓛𐓲𐓟𐓷𐓣͘𐓤𐓟", "𐒿𐓟𐓜𐓛", "𐒰𐓧𐓣 𐓏𐓣͘𐓸𐓲𐓣", "𐒰𐓧𐓣 𐓍𐓪͘𐓬𐓘"},
	{"𐓀𐓣͘𐓪͘𐓬𐓘 𐓄𐓘𐓡𐓛͘𐓧𐓟", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓏𐓟𐓵𐓪͘𐓬𐓘", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓏𐓟𐓵𐓘𐓜𐓣", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓏𐓟𐓰𐓪𐓬𐓘", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓏𐓟𐓮𐓘𐓰𐓘", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓏𐓟𐓯𐓘𐓬𐓟", "𐓀𐓣͘𐓪͘𐓬𐓘 𐓄𐓟𐓵𐓪͘𐓬𐓘", "𐓀𐓣͘𐓪͘𐓬𐓘 𐒼𐓣𐓟𐓰𐓪𐓬𐓘", "𐓀𐓣͘𐓪͘𐓬𐓘 𐒿𐓟𐓜𐓛𐓲𐓟𐓷𐓣͘𐓤𐓟", "𐓀𐓣͘𐓪͘𐓬𐓘 𐒿𐓟𐓜𐓛", "𐓀𐓣͘𐓪͘𐓬𐓘 𐒰𐓧𐓣 𐓏𐓣͘𐓸𐓲𐓣", "𐓀𐓣͘𐓪͘𐓬𐓘 𐒰𐓧𐓣 𐓍𐓪͘𐓬𐓘"},
	{"ਜਨ", "ਫ਼ਰ", "ਮਾਰਚ", "ਅਪ੍ਰੈ", "ਮਈ", "ਜੂਨ", "ਜੁਲਾ", "ਅਗ", "ਸਤੰ", "ਅਕਤੂ", "ਨਵੰ", "ਦਸੰ"},
	{"ਜਨਵਰੀ", "ਫ਼ਰਵਰੀ", "ਮਾਰਚ", "ਅਪ੍ਰੈਲ", "ਮਈ", "ਜੂਨ", "ਜੁਲਾਈ", "ਅਗਸਤ", "ਸਤੰਬਰ", "ਅਕਤੂਬਰ", "ਨਵੰਬਰ", "ਦਸੰਬਰ"},
	{"جنوری", "فروری", "مارچ", "اپریل", "مئ", "جون", "جولائی", "اگست", "ستمبر", "اکتوبر", "نومبر", "دسمبر"},
	{"Yan", "Feb", "Mar", "Apr", "Mei", "Yün", "Yül", "Oug", "Sèp", "Òkt", "Nov", "Des"},
	{"Yanüari", "Febrüari", "Mart", "Aprel", "Mei", "Yüni", "Yüli", "Ougùstùs", "Sèptèmber", "Òktober", "Novèmber", "Desèmber"},
	{"Jén", "Fẹ́b", "Mach", "Épr", "Mee", "Jun", "Jul", "Ọgọ", "Sẹp", "Ọkt", "Nọv", "Dis"},
	{"Jénúári", "Fẹ́búári", "Mach", "Éprel", "Mee", "Jun", "Julai", "Ọgọst", "Sẹptẹ́mba", "Ọktóba", "Nọvẹ́mba", "Disẹ́mba"},
	{"Januare", "Febuare", "Mas", "Eprel", "Mei", "Jun", "Julae", "Ogus", "Septemba", "Oktoba", "Novemba", "Disemba"},
	{"sty", "lut", "mar", "kwi", "maj", "cze", "lip", "sie", "wrz", "paź", "lis", "gru"},
	{"s", "l", "m", "k", "m", "c", "l", "s", "w", "p", "l", "g"},
	{"stycznia", "lutego", "marca", "kwietnia", "maja", "czerwca", "lipca", "sierpnia", "września", "października", "listopada", "grudnia"},
	{"gené", "fërvé", "mars", "avril", "magg", "giugn", "lugn", "agost", "stèmber", "utuber", "novèmber", "dzèmber"},
	{"rag", "was", "pūl", "sak", "zal", "sīm", "līp", "dag", "sil", "spa", "lap", "sal"},
	{"rags", "wassarins", "pūlis", "sakkis", "zallaws", "sīmenis", "līpa", "daggis", "sillins", "spallins", "lapkrūtis", "sallaws"},
	{"ج", "ف", "م", "ا", "م", "ج", "ج", "ا", "س", "ا", "ن", "د"},
	{"جنوري", "فبروري", "مارچ", "اپریل", "مۍ", "جون", "جولای", "اګست", "سېپتمبر", "اکتوبر", "نومبر", "دسمبر"},
	{"jan.", "fev.", "mar.", "abr.", "mai.", "jun.", "jul.", "ago.", "set.", "out.", "nov.", "dez."},
	{"janeiro", "fevereiro", "março", "abril", "maio", "junho", "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"},
	{"Ene", "Feb", "Mar", "Abr", "May", "Jun", "Jul", "Ago", "Set", "Oct", "Nov", "Dic"},
	{"Enero", "Febrero", "Marzo", "Abril", "Mayo", "Junio", "Julio", "Agosto", "Setiembre", "Octubre", "Noviembre", "Diciembre"},
	{"yen", "feb", "mar", "yeb", "may", "yun", "yul", "ɣuc", "cut", "kṭu", "nuw", "duj"},
	{"yennayer", "febrayer", "mars", "yebril", "mayyu", "yunyu", "yulyu", "ɣuccet", "cutembir", "kṭuber", "nuwembir", "dujembir"},
	{"schan.", "favr.", "mars", "avr.", "matg", "zercl.", "fan.", "avust", "sett.", "oct.", "nov.", "dec."},
	{"da schaner", "da favrer", "da mars", "d’avrigl", "da matg", "da zercladur", "da fanadur", "d’avust", "da settember", "d’october", "da november", "da december"},
	{"Mut.", "Gas.", "Wer.", "Mat.", "Gic.", "Kam.", "Nya.", "Kan.", "Nze.", "Ukw.", "Ugu.", "Uku."},
	{"Nzero", "Ruhuhuma", "Ntwarante", "Ndamukiza", "Rusama", "Ruheshi", "Mukakaro", "Nyandagaro", "Nyakanga", "Gitugutu", "Munyonyo", "Kigarama"},
	{"ian.", "feb.", "mar.", "apr.", "mai", "iun.", "iul.", "aug.", "sept.", "oct.", "nov.", "dec."},
	{"ianuarie", "februarie", "martie", "aprilie", "mai", "iunie", "iulie", "august", "septembrie", "octombrie", "noiembrie", "decembrie"},
	{"M1", "M2", "M3", "M4", "M5", "M6", "M7", "M8", "M9", "M10", "M11", "M12"},
	{"Mweri wa kwanza", "Mweri wa kaili", "Mweri wa katatu", "Mweri wa kaana", "Mweri wa tanu", "Mweri wa sita", "Mweri wa saba", "Mweri wa nane", "Mweri wa tisa", "Mweri wa ikumi", "Mweri wa ikumi na moja", "Mweri wa ikumi na mbili"},
	{"янв.", "февр.", "мар.", "апр.", "мая", "июн.", "июл.", "авг.", "сент.", "окт.", "нояб.", "дек."},
	{"января", "февраля", "марта", "апреля", "мая", "июня", "июля", "августа", "сентября", "октября", "ноября", "декабря"},
	{"Mutarama", "Gashyantare", "Werurwe", "Mata", "Gicurasi", "Kamena", "Nyakanga", "Kanama", "Nzeri", "Ukwakira", "Ugushyingo", "Ukuboza"},
	{"जनवरी:", "फरवरी:", "मार्च:", "अप्रैल:", "मई", "जून:", "जुलाई:", "अगस्त:", "सितंबर:", "अक्तूबर:", "नवंबर:", "दिसंबर:"},
	{"ज", "फ", "मा", "अ", "म", "जू", "जु", "अ", "सि", "अ", "न", "दि"},
	{"जनवरीमासः", "फरवरीमासः", "मार्चमासः", "अप्रैलमासः", "मईमासः", "जूनमासः", "जुलाईमासः", "अगस्तमासः", "सितंबरमासः", "अक्तूबरमासः", "नवंबरमासः", "दिसंबरमासः"},
	{"Тохс", "Олун", "Клн", "Мсу", "Ыам", "Бэс", "Отй", "Атр", "Блҕ", "Алт", "Сэт", "Ахс"},
	{"Тохсунньу", "Олунньу", "Кулун тутар", "Муус устар", "Ыам ыйын", "Бэс ыйын", "От ыйын", "Атырдьых ыйын", "Балаҕан ыйын", "Алтынньы", "Сэтинньи", "ахсынньы"},
	{"Obo", "Waa", "Oku", "Ong", "Ime", "Ile", "Sap", "Isi", "Saa", "Tom", "Tob", "Tow"},
	{"Lapa le obo", "Lapa le waare", "Lapa le okuni", "Lapa le ong’wan", "Lapa le imet", "Lapa le ile", "Lapa le sapa", "Lapa le isiet", "Lapa le saal", "Lapa le tomon", "Lapa le tomon obo", "Lapa le tomon waare"},
	{"ᱡᱟᱱ", "ᱯᱷᱟ", "ᱢᱟᱨ", "ᱟᱯᱨ", "ᱢᱮ", "ᱡᱩᱱ", "ᱡᱩᱞ", "ᱟᱜᱟ", "ᱥᱮᱯ", "ᱚᱠᱴ", "ᱱᱟᱣ", "ᱫᱤᱥ"},
	{"ᱡᱟᱱᱣᱟᱨᱤ", "ᱯᱷᱟᱨᱣᱟᱨᱤ", "ᱢᱟᱨᱪ", "ᱟᱯᱨᱮᱞ", "ᱢᱮ", "ᱡᱩᱱ", "ᱡᱩᱞᱟᱭ", "ᱟᱜᱟᱥᱛ", "ᱥᱮᱯᱴᱮᱢᱵᱟᱨ", "ᱚᱠᱴᱚᱵᱟᱨ", "ᱱᱟᱣᱟᱢᱵᱟᱨ", "ᱫᱤᱥᱟᱢᱵᱟᱨ"},
	{"Mup", "Mwi", "Msh", "Mun", "Mag", "Muj", "Msp", "Mpg", "Mye", "Mok", "Mus", "Muh"},
	{"Mupalangulwa", "Mwitope", "Mushende", "Munyi", "Mushende Magali", "Mujimbi", "Mushipepo", "Mupuguto", "Munyense", "Mokhu", "Musongandembwe", "Muhaano"},
	{"ghe", "fre", "mar", "abr", "maj", "làm", "trì", "aus", "cab", "stG", "stA", "nad"},
	{"ghennàrgiu", "freàrgiu", "martzu", "abrile", "maju", "làmpadas", "trìulas", "austu", "cabudanni", "santugaine", "santandria", "nadale"},
	{"jin", "fri", "mar", "apr", "maj", "giu", "gnt", "agu", "sit", "utt", "nuv", "dic"},
	{"jinnaru", "frivaru", "marzu", "aprili", "maju", "giugnu", "giugnettu", "agustu", "sittèmmiru", "uttùviru", "nuvèmmiru", "dicèmmiru"},
	{"جنوري", "فيبروري", "مارچ", "اپريل", "مئي", "جون", "جولاءِ", "آگسٽ", "سيپٽمبر", "آڪٽوبر", "نومبر", "ڊسمبر"},
	{"जन", "फर", "मार्च", "अप्रै", "मई", "जून", "जु", "अग", "सप्टे", "ऑक्टो", "नवं", "डिसं"},
	{"ज", "फ़", "मा", "अ", "मा", "जू", "जु", "अग", "स", "ऑ", "न", "डि"},
	{"जनवरी", "फेबरवरी", "मार्चु", "अप्रेल", "मई", "जून", "जुलाई", "आगस्ट", "सप्टेंबर", "आक्टोबर", "नवंबर", "डिसंबर"},
	{"ođđj", "guov", "njuk", "cuo", "mies", "geas", "suoi", "borg", "čakč", "golg", "skáb", "juov"},
	{"ođđajagemánnu", "guovvamánnu", "njukčamánnu", "cuoŋománnu", "miessemánnu", "geassemánnu", "suoidnemánnu", "borgemánnu", "čakčamánnu", "golggotmánnu", "skábmamánnu", "juovlamánnu"},
	{"ođđj", "guov", "njuk", "cuoŋ", "mies", "geas", "suoi", "borg", "čakč", "golg", "skáb", "juov"},
	{"Jan", "Fev", "Mar", "Abr", "Mai", "Jun", "Jul", "Aug", "Set", "Otu", "Nov", "Dec"},
	{"Janeiro", "Fevreiro", "Marco", "Abril", "Maio", "Junho", "Julho", "Augusto", "Setembro", "Otubro", "Novembro", "Decembro"},
	{"Nye", "Ful", "Mbä", "Ngu", "Bêl", "Fön", "Len", "Kük", "Mvu", "Ngb", "Nab", "Kak"},
	{"Nyenye", "Fulundïgi", "Mbängü", "Ngubùe", "Bêläwü", "Föndo", "Lengua", "Kükürü", "Mvuka", "Ngberere", "Nabändüru", "Kakauka"},
	{"ⵉⵏⵏ", "ⴱⵕⴰ", "ⵎⴰⵕ", "ⵉⴱⵔ", "ⵎⴰⵢ", "ⵢⵓⵏ", "ⵢⵓⵍ", "ⵖⵓⵛ", "ⵛⵓⵜ", "ⴽⵜⵓ", "ⵏⵓⵡ", "ⴷⵓⵊ"},
	{"ⵉⵏⵏⴰⵢⵔ", "ⴱⵕⴰⵢⵕ", "ⵎⴰⵕⵚ", "ⵉⴱⵔⵉⵔ", "ⵎⴰⵢⵢⵓ", "ⵢⵓⵏⵢⵓ", "ⵢⵓⵍⵢⵓⵣ", "ⵖⵓⵛⵜ", "ⵛⵓⵜⴰⵏⴱⵉⵔ", "ⴽⵜⵓⴱⵔ", "ⵏⵓⵡⴰⵏⴱⵉⵔ", "ⴷⵓⵊⴰⵏⴱⵉⵔ"},
	{"inn", "bṛa", "maṛ", "ibr", "may", "yun", "yul", "ɣuc", "cut", "ktu", "nuw", "duj"},
	{"innayr", "bṛayṛ", "maṛṣ", "ibrir", "mayyu", "yunyu", "yulyuz", "ɣuct", "cutanbir", "ktubr", "nuwanbir", "dujanbir"},
	{"ၸၼ်ႇ", "ၾႅပ်ႇ", "မၢတ်ႉၶ်ျ", "ဢေႇ", "မေႇ", "ၸုၼ်ႇ", "ၸူႇ", "ဢေႃး", "သႅပ်ႇ", "ဢွၵ်ႇ", "ၼူဝ်ႇ", "တီႇ"},
	{"ၸ.", "ၾ.", "မ.", "ဢ.", "မ.", "ၸ.", "ၸ.", "ဢ.", "သ.", "ဢ.", "ၼ.", "တ."},
	{"ၸၼ်ႇဝႃႇရီႇ", "ၾႅပ်ႇဝႃႇရီႇ", "မၢတ်ႉၶ်ျ", "ဢေႇပရႄႇ", "မေႇ", "ၸုၼ်ႇ", "ၸူႇလၢႆႇ", "ဢေႃးၵၢတ်ႉ", "သႅပ်ႇထႅမ်ႇပႃႇ", "ဢွၵ်ႇထူဝ်ႇပႃႇ", "ၼူဝ်ႇဝႅမ်ႇပႃႇ", "တီႇသႅမ်ႇပႃႇ"},
	{"දුරුතු", "නවම්", "මැදින්", "බක්", "වෙසක්", "පොසොන්", "ඇසළ", "නිකිණි", "බිනර", "වප්", "ඉල්", "උඳුවප්"},
	{"දු", "න", "මැ", "බ", "වෙ", "පො", "ඇ", "නි", "බි", "ව", "ඉ", "උ"},
	{"jan", "feb", "mar", "apr", "máj", "jún", "júl", "aug", "sep", "okt", "nov", "dec"},
	{"januára", "februára", "marca", "apríla", "mája", "júna", "júla", "augusta", "septembra", "októbra", "novembra", "decembra"},
	{"جنوری", "فروری", "مارچ", "اپریل", "مئی", "جون", "جولائی", "اگست", "ستمبر", "اکتوبر", "نومبر", "دسمبر"},
	{"jan.", "feb.", "mar.", "apr.", "maj", "jun.", "jul.", "avg.", "sep.", "okt.", "nov.", "dec."},
	{"januar", "februar", "marec", "april", "maj", "junij", "julij", "avgust", "september", "oktober", "november", "december"},
	{"uđiv", "kuovâ", "njuhčâ", "cuáŋui", "vyesi", "kesi", "syeini", "porge", "čohčâ", "roovvâd", "skammâ", "juovlâ"},
	{"uđđâivemáánu", "kuovâmáánu", "njuhčâmáánu", "cuáŋuimáánu", "vyesimáánu", "kesimáánu", "syeinimáánu", "porgemáánu", "čohčâmáánu", "roovvâdmáánu", "skammâmáánu", "juovlâmáánu"},
	{"Ndi", "Kuk", "Kur", "Kub", "Chv", "Chk", "Chg", "Nya", "Gun", "Gum", "Mbu", "Zvi"},
	{"Ndira", "Kukadzi", "Kurume", "Kubvumbi", "Chivabvu", "Chikumi", "Chikunguru", "Nyamavhuvhu", "Gunyana", "Gumiguru", "Mbudzi", "Zvita"},
	{"Jan", "Feb", "Mar", "Abr", "May", "Jun", "Lul", "Ogs", "Seb", "Okt", "Nof", "Dis"},
	{"Janaayo", "Febraayo", "Maarso", "Abriil", "Maayo", "Juun", "Luulyo", "Agosto", "Sebtembar", "Oktoobar", "Noofeembar", "Diseembar"},
	{"jan", "shk", "mar", "pri", "maj", "qer", "korr", "gush", "sht", "tet", "nën", "dhj"},
	{"janar", "shkurt", "mars", "prill", "maj", "qershor", "korrik", "gusht", "shtator", "tetor", "nëntor", "dhjetor"},
	{"јан", "феб", "мар", "апр", "мај", "јун", "јул", "авг", "сеп", "окт", "нов", "дец"},
	{"јануар", "фебруар", "март", "април", "мај", "јун", "јул", "август", "септембар", "октобар", "новембар", "децембар"},
	{"јан", "феб", "март", "апр", "мај", "јун", "јул", "авг", "септ", "окт", "нов", "дец"},
	{"jan", "feb", "mar", "apr", "maj", "jun", "jul", "avg", "sep", "okt", "nov", "dec"},
	{"januar", "februar", "mart", "april", "maj", "jun", "jul", "avgust", "septembar", "oktobar", "novembar", "decembar"},
	{"jan", "feb", "mart", "apr", "maj", "jun", "jul", "avg", "sept", "okt", "nov", "dec"},
	{"Bhi", "Van", "Vol", "Mab", "Nkh", "Nhl", "Kho", "Ngc", "Nyo", "Mph", "Lwe", "Ngo"},
	{"Bhimbidvwane", "iNdlovana", "iNdlovu-lenkhulu", "Mabasa", "iNkhwekhweti", "iNhlaba", "Kholwane", "iNgci", "iNyoni", "iMphala", "Lweti", "iNgongoni"},
	{"Phe", "Kol", "Hlb", "Mme", "Mot", "Jan", "Upu", "Pha", "Leo", "Mph", "Pun", "Tsh"},
	{"Pherekgong", "Hlakola", "Hlakubele", "Mmesa", "Motsheanong", "Phupjane", "Phupu", "Phato", "Lwetse", "Mphalane", "Pudungwana", "Tshitwe"},
	{"Jan", "Péb", "Mar", "Apr", "Méi", "Jun", "Jul", "Ags", "Sép", "Okt", "Nop", "Dés"},
	{"Januari", "Pébruari", "Maret", "April", "Méi", "Juni", "Juli", "Agustus", "Séptémber", "Oktober", "Nopémber", "Désémber"},
	{"jan.", "feb.", "mars", "apr.", "maj", "juni", "juli", "aug.", "sep.", "okt.", "nov.", "dec."},
	{"januari", "februari", "mars", "april", "maj", "juni", "juli", "augusti", "september", "oktober", "november", "december"},
	{"ܟܢܘܢ ܒ", "ܫܒܛ", "ܐܕܪ", "ܢܝܣܢ", "ܐܝܪ", "ܚܙܝܪܢ", "ܬܡܘܙ", "ܐܒ", "ܐܝܠܘܠ", "ܬܫܪܝܢ ܐ", "ܬܫܪܝܢ ܒ", "ܟܢܘܢ ܐ"},
	{"ܟ", "ܫ", "ܐ", "ܢ", "ܐ", "ܚ", "ܬ", "ܐ", "ܐ", "ܬ", "ܬ", "ܟ"},
	{"ܟܢܘܢ ܐܚܪܝܐ", "ܫܒܛ", "ܐܕܪ", "ܢܝܣܢ", "ܐܝܪ", "ܚܙܝܪܢ", "ܬܡܘܙ", "ܐܒ", "ܐܝܠܘܠ", "ܬܫܪܝܢ ܩܕܡܝܐ", "ܬܫܪܝܢ ܐܚܪܝܐ", "ܟܢܘܢ ܩܕܡܝܐ"},
	{"sty", "lut", "mar", "kwi", "moj", "czy", "lip", "siy", "wrz", "paź", "lis", "gru"},
	{"stycznia", "lutego", "marca", "kwietnia", "moja", "czyrwca", "lipca", "siyrpnia", "września", "października", "listopada", "grudnia"},
	{"ஜன.", "பிப்.", "மார்.", "ஏப்.", "மே", "ஜூன்", "ஜூலை", "ஆக.", "செப்.", "அக்.", "நவ.", "டிச."},
	{"ஜனவரி", "பிப்ரவரி", "மார்ச்", "ஏப்ரல்", "மே", "ஜூன்", "ஜூலை", "ஆகஸ்ட்", "செப்டம்பர்", "அக்டோபர்", "நவம்பர்", "டிசம்பர்"},
	{"జన", "ఫిబ్ర", "మార్చి", "ఏప్రి", "మే", "జూన్", "జులై", "ఆగ", "సెప్టెం", "అక్టో", "నవం", "డిసెం"},
	{"జనవరి", "ఫిబ్రవరి", "మార్చి", "ఏప్రిల్", "మే", "జూన్", "జులై", "ఆగస్టు", "సెప్టెంబర్", "అక్టోబర్", "నవంబర్", "డిసెంబర్"},
	{"Rar", "Muk", "Kwa", "Dun", "Mar", "Mod", "Jol", "Ped", "Sok", "Tib", "Lab", "Poo"},
	{"Orara", "Omuk", "Okwamg’", "Odung’el", "Omaruk", "Omodok’king’ol", "Ojola", "Opedel", "Osokosokoma", "Otibar", "Olabor", "Opoo"},
	{"Янв", "Фев", "Мар", "Апр", "Май", "Июн", "Июл", "Авг", "Сен", "Окт", "Ноя", "Дек"},
	{"Январ", "Феврал", "Март", "Апрел", "Май", "Июн", "Июл", "Август", "Сентябр", "Октябр", "Ноябр", "Декабр"},
	{"ม.ค.", "ก.พ.", "มี.ค.", "เม.ย.", "พ.ค.", "มิ.ย.", "ก.ค.", "ส.ค.", "ก.ย.", "ต.ค.", "พ.ย.", "ธ.ค."},
	{"มกราคม", "กุมภาพันธ์", "มีนาคม", "เมษายน", "พฤษภาคม", "มิถุนายน", "กรกฎาคม", "สิงหาคม", "กันยายน", "ตุลาคม", "พฤศจิกายน", "ธันวาคม"},
	{"ጥሪ", "ለካ", "መጋ", "ሚያ", "ግን", "ሰነ", "ሓም", "ነሓ", "መስ", "ጥቅ", "ሕዳ", "ታሕ"},
	{"ጥሪ", "ለካቲት", "መጋቢት", "ሚያዝያ", "ጉንበት", "ሰነ", "ሓምለ", "ነሓሰ", "መስከረም", "ጥቅምቲ", "ሕዳር", "ታሕሳስ"},
	{"ጃንዩ", "ፌብሩ", "ማርች", "ኤፕረ", "ሜይ", "ጁን", "ጁላይ", "ኦገስ", "ሴፕቴ", "ኦክተ", "ኖቬም", "ዲሴም"},
	{"ጃንዩወሪ", "ፌብሩወሪ", "ማርች", "ኤፕረል", "ሜይ", "ጁን", "ጁላይ", "ኦገስት", "ሴፕቴምበር", "ኦክተውበር", "ኖቬምበር", "ዲሴምበር"},
	{"ýan", "few", "mart", "apr", "maý", "iýun", "iýul", "awg", "sen", "okt", "noý", "dek"},
	{"ýanwar", "fewral", "mart", "aprel", "maý", "iýun", "iýul", "awgust", "sentýabr", "oktýabr", "noýabr", "dekabr"},
	{"Fer", "Tlh", "Mop", "Mor", "Mot", "See", "Phu", "Pha", "Lwe", "Dip", "Ngw", "Sed"},
	{"Ferikgong", "Tlhakole", "Mopitlo", "Moranang", "Motsheganang", "Seetebosigo", "Phukwi", "Phatwe", "Lwetse", "Diphalane", "Ngwanatsele", "Sedimonthole"},
	{"Sān", "Fēp", "Maʻa", "ʻEpe", "Mē", "Sun", "Siu", "ʻAok", "Sēp", "ʻOka", "Nōv", "Tīs"},
	{"Sānuali", "Fēpueli", "Maʻasi", "ʻEpeleli", "Mē", "Sune", "Siulai", "ʻAokosi", "Sēpitema", "ʻOkatopa", "Nōvema", "Tīsema"},
	{"tenpo mun #1", "tenpo mun #2", "tenpo mun #3", "tenpo mun #4", "tenpo mun #5", "tenpo mun #6", "tenpo mun #7", "tenpo mun #8", "tenpo mun #9", "tenpo mun #10", "tenpo mun #11", "tenpo mun #12"},
	{"Jan", "Feb", "Mas", "Epr", "Me", "Jun", "Jul", "Oga", "Sep", "Okt", "Nov", "Des"},
	{"Janueri", "Februeri", "Mas", "Epril", "Me", "Jun", "Julai", "Ogas", "Septemba", "Oktoba", "Novemba", "Desemba"},
	{"Oca", "Şub", "Mar", "Nis", "May", "Haz", "Tem", "Ağu", "Eyl", "Eki", "Kas", "Ara"},
	{"Ocak", "Şubat", "Mart", "Nisan", "Mayıs", "Haziran", "Temmuz", "Ağustos", "Eylül", "Ekim", "Kasım", "Aralık"},
	{"Kii", "Dhi", "Tri", "Spi", "Rii", "Mti", "Emi", "Mai", "Mni", "Mxi", "Mxk", "Mxd"},
	{"Kingal idas", "Dha idas", "Tru idas", "Spat idas", "Rima idas", "Mataru idas", "Empitu idas", "Maspat idas", "Mngari idas", "Maxal idas", "Maxal kingal idas", "Maxal dha idas"},
	{"Sun", "Yan", "Kul", "Dzi", "Mud", "Kho", "Maw", "Mha", "Ndz", "Nhl", "Huk", "N’w"},
	{"Sunguti", "Nyenyenyani", "Nyenyankulu", "Dzivamisoko", "Mudyaxihi", "Khotavuxika", "Mawuwani", "Mhawuri", "Ndzhati", "Nhlangula", "Hukuri", "N’wendzamhala"},
	{"гыйн.", "фев.", "мар.", "апр.", "май", "июнь", "июль", "авг.", "сент.", "окт.", "нояб.", "дек."},
	{"гыйнвар", "февраль", "март", "апрель", "май", "июнь", "июль", "август", "сентябрь", "октябрь", "ноябрь", "декабрь"},
	{"Янв.", "Февр.", "Мар.", "Апр.", "Май", "Июн.", "Июл.", "Авг.", "Сент.", "Окт.", "Нояб.", "Дек."},
	{"Январь", "Февраль", "Март", "Апрель", "Май", "Июнь", "Июль", "Август", "Сентябрь", "Октябрь", "Ноябрь", "Декабрь"},
	{"Yen", "Yeb", "Mar", "Ibr", "May", "Yun", "Yul", "Ɣuc", "Cut", "Kṭu", "Nwa", "Duj"},
	{"Yennayer", "Yebrayer", "Mars", "Ibrir", "Mayyu", "Yunyu", "Yulyuz", "Ɣuct", "Cutanbir", "Kṭuber", "Nwanbir", "Dujanbir"},
	{"يانۋار", "فېۋرال", "مارت", "ئاپرېل", "ماي", "ئىيۇن", "ئىيۇل", "ئاۋغۇست", "سېنتەبىر", "ئۆكتەبىر", "نويابىر", "دېكابىر"},
	{"січ.", "лют.", "бер.", "квіт.", "трав.", "черв.", "лип.", "серп.", "вер.", "жовт.", "лист.", "груд."},
	{"с", "л", "б", "к", "т", "ч", "л", "с", "в", "ж", "л", "г"},
	{"січня", "лютого", "березня", "квітня", "травня", "червня", "липня", "серпня", "вересня", "жовтня", "листопада", "грудня"},
	{"yan", "fev", "mar", "apr", "may", "iyn", "iyl", "avg", "sen", "okt", "noy", "dek"},
	{"yanvar", "fevral", "mart", "aprel", "may", "iyun", "iyul", "avgust", "sentabr", "oktabr", "noyabr", "dekabr"},
	{"جنو", "فبر", "مار", "اپر", "می", "جون", "جول", "اگس", "سپت", "اکت", "نوم", "دسم"},
	{"январ", "феврал", "март", "апрел", "май", "июн", "июл", "август", "сентябр", "октябр", "ноябр", "декабр"},
	{"ꖨꖕꔞ", "ꕒꕡ", "ꕾꖺ", "ꖢꖕ", "ꖑꕱ", "ꖱꘋ", "ꖱꕞ", "ꗛꔕ", "ꕢꕌ", "ꕭꖃ", "ꔞꘋ", "ꖨꖕꗏ"},
	{"ꖨꖕ ꕪꕴ ꔞꔀꕮꕊ", "ꕒꕡꖝꖕ", "ꕾꖺ", "ꖢꖕ", "ꖑꕱ", "ꖱꘋ", "ꖱꕞꔤ", "ꗛꔕ", "ꕢꕌ", "ꕭꖃ", "ꔞꘋꕔꕿ ꕸꖃꗏ", "ꖨꖕ ꕪꕴ ꗏꖺꕮꕊ"},
	{"Pha", "Luh", "Ṱhf", "Lam", "Shu", "Lwi", "Lwa", "Ṱha", "Khu", "Tsh", "Ḽar", "Nye"},
	{"Phando", "Luhuhi", "Ṱhafamuhwe", "Lambamai", "Shundunthule", "Fulwi", "Fulwana", "Ṱhangule", "Khubvumedzi", "Tshimedzi", "Ḽara", "Nyendavhusiku"},
	{"jen", "feb", "mar", "apr", "maj", "jug", "luj", "ago", "set", "oto", "nov", "dez"},
	{"jenaro", "febraro", "marso", "aprile", "majo", "jugno", "lujo", "agosto", "setenbre", "otobre", "novenbre", "dezenbre"},
	{"thg 1", "thg 2", "thg 3", "thg 4", "thg 5", "thg 6", "thg 7", "thg 8", "thg 9", "thg 10", "thg 11", "thg 12"},
	{"tháng 1", "tháng 2", "tháng 3", "tháng 4", "tháng 5", "tháng 6", "tháng 7", "tháng 8", "tháng 9", "tháng 10", "tháng 11", "tháng 12"},
	{"janeiru", "fevereiru", "marsu", "abril", "maiu", "junyu", "julyu", "agostu", "setembru", "outubru", "novembru", "dezembru"},
	{"yan", "feb", "mäz", "prl", "may", "yun", "yul", "gst", "set", "ton", "nov", "dek"},
	{"yanul", "febul", "mäzul", "prilul", "mayul", "yunul", "yulul", "gustul", "setul", "tobul", "novul", "dekul"},
	{"Jen", "Hor", "Mär", "Abr", "Mei", "Brá", "Hei", "Öig", "Her", "Wím", "Win", "Chr"},
	{"Jenner", "Hornig", "Märze", "Abrille", "Meije", "Bráčet", "Heiwet", "Öigšte", "Herbštmánet", "Wímánet", "Wintermánet", "Chrištmánet"},
	{"Sam", "Few", "Mar", "Awr", "Mee", "Suw", "Sul", "Ut", "Sàt", "Okt", "Now", "Des"},
	{"Samwiyee", "Fewriyee", "Mars", "Awril", "Mee", "Suwe", "Sulet", "Ut", "Sàttumbar", "Oktoobar", "Nowàmbar", "Desàmbar"},
	{"Jan", "Feb", "Mat", "Epr", "Mey", "Jun", "Jul", "Aga", "Sept", "Okt", "Nov", "Dis"},
	{"Janyuwari", "Februwari", "Matshi", "Epreli", "Meyi", "Juni", "Julayi", "Agasti", "Septemba", "Okthobha", "Novemba", "Disemba"},
	{"जनवरी", "फ़रवरी", "मार्च", "अप्रैल", "मई", "जून", "जुलाई", "अगस्त", "सितंबर", "अक्तूबर", "नवंबर", "दिसंबर"},
	{"o.1", "o.2", "o.3", "o.4", "o.5", "o.6", "o.7", "o.8", "o.9", "o.10", "o.11", "o.12"},
	{"pikítíkítie, oólí ú kutúan", "siɛyɛ́, oóli ú kándíɛ", "ɔnsúmbɔl, oóli ú kátátúɛ", "mesiŋ, oóli ú kénie", "ensil, oóli ú kátánuɛ", "ɔsɔn", "efute", "pisuyú", "imɛŋ i puɔs", "imɛŋ i putúk,oóli ú kátíɛ", "makandikɛ", "pilɔndɔ́"},
	{"יאַנואַר", "פֿעברואַר", "מערץ", "אַפּריל", "מיי", "יוני", "יולי", "אויגוסט", "סעפּטעמבער", "אקטאבער", "נאוועמבער", "דעצעמבער"},
	{"Oṣù Ṣẹ́rẹ́", "Oṣù Èrèlè", "Oṣù Ẹrẹ̀nà", "Oṣù Ìgbé", "Oṣù Ẹ̀bibi", "Oṣù Òkúdu", "Oṣù Agẹmọ", "Oṣù Ògún", "Oṣù Owewe", "Oṣù Ọ̀wàrà", "Oṣù Bélú", "Oṣù Ọ̀pẹ̀"},
	{"Oshù Shɛ́rɛ́", "Oshù Èrèlè", "Oshù Ɛrɛ̀nà", "Oshù Ìgbé", "Oshù Ɛ̀bibi", "Oshù Òkúdu", "Oshù Agɛmɔ", "Oshù Ògún", "Oshù Owewe", "Oshù Ɔ̀wàrà", "Oshù Bélú", "Oshù Ɔ̀pɛ̀"},
	{"ye", "mk", "ms", "id", "pu", "py", "pm", "ps", "pi", "yp", "yy", "ym"},
	{"yepé", "mukũi", "musapíri", "irũdí", "pú", "pú-yepé", "pú-mukũi", "pú-musapíri", "pú-irũdí", "yepé-putimaã", "yepé-yepé", "yepé-mukũi"},
	{"一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"},
	{"ndwenit", "ndwenngeih", "ndwensam", "ndwenseiq", "ndwenngux", "ndwenloeg", "ndwencaet", "ndwenbet", "ndwengouj", "ndwencib", "ndwencib’it", "ndwencibngeih"},
	{"Jan", "Feb", "Mas", "Eph", "Mey", "Jun", "Jul", "Aga", "Sep", "Okt", "Nov", "Dis"},
	{"J", "F", "M", "E", "M", "J", "J", "A", "S", "O", "N", "D"},
	{"Januwari", "Februwari", "Mashi", "Ephreli", "Meyi", "Juni", "Julayi", "Agasti", "Septhemba", "Okthoba", "Novemba", "Disemba"},
}

var _days = [...]days{
	{"Aca", "Etl", "Tal", "Arb", "Kam", "Gum", "Sab"},
	{"", "", "", "", "", "", ""},
	{"Acaada", "Etleeni", "Talaata", "Arbaqa", "Kamiisi", "Gumqata", "Sabti"},
	{"Ам", "Ашә", "Аҩ", "Ах", "Аԥ", "Ахә", "Ас"},
	{"Амҽыша", "Ашәахьа", "Аҩаша", "Ахаша", "Аԥшьаша", "Ахәаша", "Асабша"},
	{"So.", "Ma.", "Di.", "Wo.", "Do.", "Vr.", "Sa."},
	{"Sondag", "Maandag", "Dinsdag", "Woensdag", "Donderdag", "Vrydag", "Saterdag"},
	{"nts", "kpa", "ghɔ", "tɔm", "ume", "ghɨ", "dzk"},
	{"tsuʔntsɨ", "tsuʔukpà", "tsuʔughɔe", "tsuʔutɔ̀mlò", "tsuʔumè", "tsuʔughɨ̂m", "tsuʔndzɨkɔʔɔ"},
	{"Kwa", "Dwo", "Ben", "Wuk", "Yaw", "Fia", "Mem"},
	{"Sun", "Dwoada", "Benada", "Wukuada", "Yawoada", "Fiada", "Memeneda"},
	{"እሑድ", "ሰኞ", "ማክሰ", "ረቡዕ", "ሐሙስ", "ዓርብ", "ቅዳሜ"},
	{"እ", "ሰ", "ማ", "ረ", "ሐ", "ዓ", "ቅ"},
	{"እሑድ", "ሰኞ", "ማክሰኞ", "ረቡዕ", "ሐሙስ", "ዓርብ", "ቅዳሜ"},
	{"dom", "lun", "mar", "mie", "chu", "vie", "sab"},
	{"dominche", "luns", "martz", "miercres", "chueves", "viernes", "sabado"},
	{"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"},
	{"الأحد", "التنين", "التلاتا", "الأربعا", "الخميس", "الجمعة", "السبت"},
	{"أحد", "إثنين", "ثلاثاء", "أربعاء", "خميس", "جمعة", "سبت"},
	{"الأحد", "الاثنين", "الثلاثاء", "الأربعاء", "الخميس", "الجمعة", "السبت"},
	{"দেও", "সোম", "মঙ্গল", "বুধ", "বৃহ", "শুক্ৰ", "শনি"},
	{"দেওবাৰ", "সোমবাৰ", "মঙ্গলবাৰ", "বুধবাৰ", "বৃহস্পতিবাৰ", "শুক্ৰবাৰ", "শনিবাৰ"},
	{"Jpi", "Jtt", "Jnn", "Jtn", "Alh", "Ijm", "Jmo"},
	{"Jumapili", "Jumatatu", "Jumanne", "Jumatano", "Alhamisi", "Ijumaa", "Jumamosi"},
	{"dom", "llu", "mar", "mié", "xue", "vie", "sáb"},
	{"do", "ll", "ma", "mi", "xu", "vi", "sá"},
	{"domingu", "llunes", "martes", "miércoles", "xueves", "vienres", "sábadu"},
	{"B.", "B.e.", "Ç.a.", "Ç.", "C.a.", "C.", "Ş."},
	{"B.", "B.E.", "Ç.A.", "Ç.", "C.A.", "C.", "Ş."},
	{"bazar", "bazar ertəsi", "çərşənbə axşamı", "çərşənbə", "cümə axşamı", "cümə", "şənbə"},
	{"Б.", "Б.Е.", "Ч.А.", "Ч.", "Ҹ.А.", "Ҹ.", "Ш."},
	{"базар", "базар ертәси", "чәршәнбә ахшамы", "чәршәнбә", "ҹүмә ахшамы", "ҹүмә", "шәнбә"},
	{"йәк.", "дүш.", "шиш.", "шар.", "кес.", "йом.", "шәм."},
	{"Й", "Д", "Ш", "Ш", "К", "Й", "Ш"},
	{"йш", "дш", "шш", "шр", "кс", "йм", "шб"},
	{"йәкшәмбе", "дүшәмбе", "шишәмбе", "шаршамбы", "кесаҙна", "йома", "шәмбе"},
	{"یک", "دو", "سئے", "چار", "پنچ", "جمه", "شم"},
	{"یکشمبه", "دوشمبه", "سئیشمبه", "چارشمبه", "پنچشمبه", "جمه", "شمبه"},
	{"Yak", "Do", "Say", "Chá", "Pan", "Jom", "Sha"},
	{"Y", "D", "S", "Ch", "P", "J", "Sh"},
	{"Yakshambeh", "Doshambeh", "Sayshambeh", "Chárshambeh", "Panchshambeh", "Jomah", "Shambeh"},
	{"nɔy", "nja", "uum", "ŋge", "mbɔ", "kɔɔ", "jon"},
	{"ŋgwà nɔ̂y", "ŋgwà njaŋgumba", "ŋgwà ûm", "ŋgwà ŋgê", "ŋgwà mbɔk", "ŋgwà kɔɔ", "ŋgwà jôn"},
	{"нд", "пн", "аў", "ср", "чц", "пт", "сб"},
	{"нядзеля", "панядзелак", "аўторак", "серада", "чацвер", "пятніца", "субота"},
	{"няд", "пан", "аўт", "сер", "чац", "пят", "суб"},
	{"нядзеля", "панядзелак", "аўторак", "серада", "чацьвер", "пятніца", "субота"},
	{"Pa Mulungu", "Palichimo", "Palichibuli", "Palichitatu", "Palichine", "Palichisano", "Pachibelushi"},
	{"Min", "Sen", "Sel", "Reb", "Kem", "Jum", "Sap"},
	{"Minggu", "Senèn", "Selasa", "Rebo", "Kemis", "Juma’at", "Saptu"},
	{"Mul", "Vil", "Hiv", "Hid", "Hit", "Hih", "Lem"},
	{"pa mulungu", "pa shahuviluha", "pa hivili", "pa hidatu", "pa hitayi", "pa hihanu", "pa shahulembela"},
	{"нд", "пн", "вт", "ср", "чт", "пт", "сб"},
	{"неделя", "понеделник", "вторник", "сряда", "четвъртък", "петък", "събота"},
	{"ऐतवार", "सोमवार", "मंगलवार", "बुधवार", "बृहस्पतवार", "शुक्रवार", "शनिवार"},
	{"रबीबार", "सोमबार", "मंगलबार", "बुधबार", "बृहस्पतिबार", "शुक्रबार", "सनीचर"},
	{"alah", "aɖɩt", "atal", "alar", "alam", "arɩs", "asib"},
	{"alh", "aɖt", "atl", "alr", "alm", "ars", "asb"},
	{"alahaɖɩ", "aɖɩtɛnɛɛ", "atalaata", "alaarba", "alaamɩshɩ", "arɩsǝma", "asiibi"},
	{"kar", "ntɛ", "tar", "ara", "ala", "jum", "sib"},
	{"kari", "ntɛnɛ", "tarata", "araba", "alamisa", "juma", "sibiri"},
	{"রবি", "সোম", "মঙ্গল", "বুধ", "বৃহস্পতি", "শুক্র", "শনি"},
	{"রঃ", "সোঃ", "মঃ", "বুঃ", "বৃঃ", "শুঃ", "শনি"},
	{"রবিবার", "সোমবার", "মঙ্গলবার", "বুধবার", "বৃহস্পতিবার", "শুক্রবার", "শনিবার"},
	{"রঃ", "সোঃ", "মঃ", "বুঃ", "বৃঃ", "শুঃ", "শঃ"},
	{"ཉི་མ་", "ཟླ་བ་", "མིག་དམར་", "ལྷག་པ་", "ཕུར་བུ་", "པ་སངས་", "སྤེན་པ་"},
	{"གཟའ་ཉི་མ་", "གཟའ་ཟླ་བ་", "གཟའ་མིག་དམར་", "གཟའ་ལྷག་པ་", "གཟའ་ཕུར་བུ་", "གཟའ་པ་སངས་", "གཟའ་སྤེན་པ་"},
	{"یه شمبه", "دوشمبه", "سه شمبه", "چار شمبه", "پنجشمبه", "جومه", "شمبه"},
	{"Sul", "Lun", "Meu.", "Mer.", "Yaou", "Gwe.", "Sad."},
	{"Sul", "Lun", "Meurzh", "Mercʼher", "Yaou", "Gwener", "Sadorn"},
	{"रबि", "सम", "मंगल", "बुध", "बिस्थि", "सुखुर", "सनि"},
	{"रबिबार", "समबार", "मंगलबार", "बुधबार", "बिस्थिबार", "सुखुरबार", "सनिबार"},
	{"ned", "pon", "uto", "sri", "čet", "pet", "sub"},
	{"N", "P", "U", "S", "Č", "P", "S"},
	{"nedjelja", "ponedjeljak", "utorak", "srijeda", "četvrtak", "petak", "subota"},
	{"нед", "пон", "уто", "сри", "чет", "пет", "суб"},
	{"недјеља", "понедјељак", "уторак", "сриједа", "четвртак", "петак", "субота"},
	{"Ни", "Да", "Ми", "Һа", "Пү", "Ба", "Би"},
	{"нима", "дабаа", "мигмар", "һарба", "пүрбэ", "баасан", "бимба"},
	{"ሰ/ቅ", "ሰኑ", "ሰሊጝ", "ለጓ", "ኣምድ", "ኣርብ", "ሰ/ሽ"},
	{"ሰንበር ቅዳዅ", "ሰኑ", "ሰሊጝ", "ለጓ ወሪ ለብዋ", "ኣምድ", "ኣርብ", "ሰንበር ሽጓዅ"},
	{"dg.", "dl.", "dt.", "dc.", "dj.", "dv.", "ds."},
	{"diumenge", "dilluns", "dimarts", "dimecres", "dijous", "divendres", "dissabte"},
	{"Inikuˀ", "Wísts’i hayashuh", "Bít hayashuh", "Dahó hayashuh", "Hiwí hayashuh", "Dissik’an hayashuh", "Inikuˀtiˀtiˀ"},
	{"Yok", "Tung", "Gitung", "Tsan", "Nas", "Nat", "Chir"},
	{"Wai Yoka Bawai", "Wai Tunga", "Toki Gitung", "Tsam Kasuwa", "Wai Na Nas", "Wai Na Tiyon", "Wai Na Chirim"},
	{"𑄢𑄧𑄝𑄨", "𑄥𑄧𑄟𑄴", "𑄟𑄧𑄁𑄉𑄧𑄣𑄴", "𑄝𑄪𑄖𑄴", "𑄝𑄳𑄢𑄨𑄥𑄪𑄛𑄴", "𑄥𑄪𑄇𑄴𑄇𑄮𑄢𑄴", "𑄥𑄧𑄚𑄨"},
	{"𑄢𑄧𑄝𑄨𑄝𑄢𑄴", "𑄥𑄧𑄟𑄴𑄝𑄢𑄴", "𑄟𑄧𑄁𑄉𑄧𑄣𑄴𑄝𑄢𑄴", "𑄝𑄪𑄖𑄴𑄝𑄢𑄴", "𑄝𑄳𑄢𑄨𑄥𑄪𑄛𑄴𑄝𑄢𑄴", "𑄥𑄪𑄇𑄴𑄇𑄮𑄢𑄴𑄝𑄢𑄴", "𑄥𑄧𑄚𑄨𑄝𑄢𑄴"},
	{"кӀи", "ор", "ши", "кха", "еа", "пӀе", "шуо"},
	{"кӀира", "оршот", "шинара", "кхаара", "еара", "пӀераска", "шуот"},
	{"Dom", "Lun", "Mar", "Miy", "Huw", "Biy", "Sab"},
	{"Domingo", "Lunes", "Martes", "Miyerkules", "Huwebes", "Biyernes", "Sabado"},
	{"SAN", "ORK", "OKB", "OKS", "OKN", "OKT", "OMK"},
	{"Sande", "Orwokubanza", "Orwakabiri", "Orwakashatu", "Orwakana", "Orwakataano", "Orwamukaaga"},
	{"ᏆᏍᎬ", "ᏉᏅᎯ", "ᏔᎵᏁ", "ᏦᎢᏁ", "ᏅᎩᏁ", "ᏧᎾᎩ", "ᏈᏕᎾ"},
	{"ᏍᎬ", "ᏅᎯ", "ᏔᎵ", "ᏦᎢ", "ᏅᎩ", "ᏧᎾ", "ᏕᎾ"},
	{"ᎤᎾᏙᏓᏆᏍᎬ", "ᎤᎾᏙᏓᏉᏅᎯ", "ᏔᎵᏁᎢᎦ", "ᏦᎢᏁᎢᎦ", "ᏅᎩᏁᎢᎦ", "ᏧᎾᎩᎶᏍᏗ", "ᎤᎾᏙᏓᏈᏕᎾ"},
	{"Nittak Holloʼ", "Mantiʼ", "Chostiʼ", "Winstiʼ", "Soistiʼ", "Nannalhchifaʼ Nittak", "Nittak Holloʼ Nakfish"},
	{"١ش", "٢ش", "٣ش", "٤ش", "٥ش", "ھ", "ش"},
	{"یەکشەممە", "دووشەممە", "سێشەممە", "چوارشەممە", "پێنجشەممە", "ھەینی", "شەممە"},
	{"dum.", "lun.", "mar.", "mer.", "ghj.", "ven.", "sab."},
	{"du", "lu", "ma", "me", "gh", "ve", "sa"},
	{"dumenica", "luni", "marti", "mercuri", "ghjovi", "venneri", "sabbatu"},
	{"ⲡⲓⲁ̅", "ⲡⲓⲃ̅", "ⲡⲓⲅ̅", "ⲡⲓⲇ̅", "ⲡⲓⲉ̅", "ⲡⲓⲋ̅", "ⲡⲓⲍ̅"},
	{"ⲡⲓⲟⲩⲁⲓ", "ⲡⲓⲥ̀ⲛⲁⲩ", "ⲡⲓϣⲟⲙⲧ", "ⲡⲓϥ̀ⲧⲟⲩ", "ⲡⲓⲧ̀ⲓⲟⲩ", "ⲡⲓⲥⲟⲟⲩ", "ⲡⲓϣⲁϣϥ"},
	{"ne", "po", "út", "st", "čt", "pá", "so"},
	{"neděle", "pondělí", "úterý", "středa", "čtvrtek", "pátek", "sobota"},
	{"ᐊᔭᒥᐦᐁᑮᓯᑳᐤ", "ᐴᓂ\u00a0ᐊᔭᒥᐦᐁᑮᓯᑳᐤ", "ᓃᓱᑮᓯᑳᐤ", "ᐋᐱᐦᑕᐘᐣ", "ᐴᓂᐋᐱᐦᑕᐘᐣ", "ᑫᑳᐨ ᒫᑎᓇᐍᑮᓯᑳᐤ", "ᒫᑎᓇᐍᑮᓯᑳᐤ"},
	{"ᐊ", "ᐴ", "ᓃ", "ᐋ", "ᐴ", "ᑫ", "ᒫ"},
	{"ндⷧ҇ѧ", "пнⷣе", "втоⷬ҇", "срⷣе", "чеⷦ҇", "пѧⷦ҇", "сꙋⷠ҇"},
	{"недѣ́лѧ", "понедѣ́льникъ", "вто́рникъ", "среда̀", "четверто́къ", "пѧто́къ", "сꙋббѡ́та"},
	{"вр", "тн", "ыт", "юн", "кҫ", "эр", "шм"},
	{"вырсарни кун", "тунти кун", "ытлари кун", "юн кун", "кӗҫнерни кун", "эрне кун", "шӑмат кун"},
	{"Sul", "Llun", "Maw", "Mer", "Iau", "Gwen", "Sad"},
	{"Su", "Ll", "Ma", "Me", "Ia", "Gw", "Sa"},
	{"Dydd Sul", "Dydd Llun", "Dydd Mawrth", "Dydd Mercher", "Dydd Iau", "Dydd Gwener", "Dydd Sadwrn"},
	{"søn.", "man.", "tirs.", "ons.", "tors.", "fre.", "lør."},
	{"sø.", "ma.", "ti.", "on.", "to.", "fr.", "lø."},
	{"søndag", "mandag", "tirsdag", "onsdag", "torsdag", "fredag", "lørdag"},
	{"Jum", "Jim", "Kaw", "Kad", "Kan", "Kas", "Ngu"},
	{"Ituku ja jumwa", "Kuramuka jimweri", "Kuramuka kawi", "Kuramuka kadadu", "Kuramuka kana", "Kuramuka kasanu", "Kifula nguwo"},
	{"So.", "Mo.", "Di.", "Mi.", "Do.", "Fr.", "Sa."},
	{"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"},
	{"So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"},
	{"Alh", "Ati", "Ata", "Ala", "Alm", "Alz", "Asi"},
	{"Alhadi", "Atinni", "Atalaata", "Alarba", "Alhamisi", "Alzuma", "Asibti"},
	{"ऐत", "सोम", "मंगल", "बुध", "बीर", "शुक्र", "शनि"},
	{"ऐ.", "सो.", "म.", "बु.", "बी.", "शु.", "श."},
	{"ऐतबार", "सोमबार", "मंगलबार", "बुधबार", "बीरबार", "शुक्रबार", "शनिबार"},
	{"nje", "pón", "wał", "srj", "stw", "pět", "sob"},
	{"nj", "pó", "wa", "sr", "st", "pě", "so"},
	{"njeźela", "pónjeźele", "wałtora", "srjoda", "stwórtk", "pětk", "sobota"},
	{"ét", "mɔ́s", "kwa", "muk", "ŋgi", "ɗón", "esa"},
	{"éti", "mɔ́sú", "kwasú", "mukɔ́sú", "ŋgisú", "ɗónɛsú", "esaɓasú"},
	{"Dim", "Ten", "Tal", "Ala", "Ara", "Arj", "Sib"},
	{"Dimas", "Teneŋ", "Talata", "Alarbay", "Aramisay", "Arjuma", "Sibiti"},
	{"ཟླ་", "མིར་", "ལྷག་", "ཕུར་", "སངས་", "སྤེན་", "ཉི་"},
	{"གཟའ་ཟླ་བ་", "གཟའ་མིག་དམར་", "གཟའ་ལྷག་པ་", "གཟའ་ཕུར་བུ་", "གཟའ་པ་སངས་", "གཟའ་སྤེན་པ་", "གཟའ་ཉི་མ་"},
	{"Kma", "Tat", "Ine", "Tan", "Arm", "Maa", "NMM"},
	{"Kiumia", "Njumatatu", "Njumaine", "Njumatano", "Aramithi", "Njumaa", "NJumamothii"},
	{"kɔs", "dzo", "bla", "kuɖ", "yaw", "fiɖ", "mem"},
	{"kɔsiɖa", "dzoɖa", "blaɖa", "kuɖa", "yawoɖa", "fiɖa", "memleɖa"},
	{"Κυρ", "Δευ", "Τρί", "Τετ", "Πέμ", "Παρ", "Σάβ"},
	{"Κυ", "Δε", "Τρ", "Τε", "Πέ", "Πα", "Σά"},
	{"Κυριακή", "Δευτέρα", "Τρίτη", "Τετάρτη", "Πέμπτη", "Παρασκευή", "Σάββατο"},
	{"S", "M", "T", "W", "T", "F", "S"},
	{"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"},
	{"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"},
	{"Su.", "M.", "Tu.", "W.", "Th.", "F.", "Sa."},
	{"𐐝𐐲𐑌", "𐐣𐐲𐑌", "𐐓𐐭𐑆", "𐐎𐐯𐑌", "𐐛𐐲𐑉", "𐐙𐑉𐐴", "𐐝𐐰𐐻"},
	{"𐐝𐐲𐑌𐐼𐐩", "𐐣𐐲𐑌𐐼𐐩", "𐐓𐐭𐑆𐐼𐐩", "𐐎𐐯𐑌𐑆𐐼𐐩", "𐐛𐐲𐑉𐑆𐐼𐐩", "𐐙𐑉𐐴𐐼𐐩", "𐐝𐐰𐐻𐐲𐑉𐐼𐐩"},
	{"·𐑕𐑭", "·𐑥𐑭", "·𐑑𐑵", "·𐑢𐑧", "·𐑔𐑻", "·𐑓𐑮", "·𐑕𐑨"},
	{"·𐑕𐑭𐑙𐑛𐑱", "·𐑥𐑭𐑙𐑛𐑱", "·𐑑𐑵𐑟𐑛𐑱", "·𐑢𐑧𐑙𐑟𐑛𐑱", "·𐑔𐑻𐑟𐑛𐑱", "·𐑓𐑮𐑲𐑛𐑱", "·𐑕𐑨𐑛𐑻𐑛𐑱"},
	{"di", "lu", "ma", "me", "ĵa", "ve", "sa"},
	{"dimanĉo", "lundo", "mardo", "merkredo", "ĵaŭdo", "vendredo", "sabato"},
	{"dom", "lun", "mar", "mié", "jue", "vie", "sáb"},
	{"DO", "LU", "MA", "MI", "JU", "VI", "SA"},
	{"domingo", "lunes", "martes", "miércoles", "jueves", "viernes", "sábado"},
	{"do", "lu", "ma", "mi", "ju", "vi", "sá"},
	{"D", "L", "M", "m", "J", "V", "S"},
	{"do", "lu", "ma", "mi", "ju", "vi", "sa"},
	{"Do", "Lu", "Ma", "Mi", "Ju", "Vi", "Sa"},
	{"P", "E", "T", "K", "N", "R", "L"},
	{"pühapäev", "esmaspäev", "teisipäev", "kolmapäev", "neljapäev", "reede", "laupäev"},
	{"ig.", "al.", "ar.", "az.", "og.", "or.", "lr."},
	{"igandea", "astelehena", "asteartea", "asteazkena", "osteguna", "ostirala", "larunbata"},
	{"sɔ́n", "mɔ́n", "smb", "sml", "smn", "fúl", "sér"},
	{"sɔ́ndɔ", "mɔ́ndi", "sɔ́ndɔ məlú mə́bɛ̌", "sɔ́ndɔ məlú mə́lɛ́", "sɔ́ndɔ məlú mə́nyi", "fúladé", "séradé"},
	{"۱ش", "۲ش", "۳ش", "۴ش", "۵ش", "ج", "ش"},
	{"یکشنبه", "دوشنبه", "سه\u200cشنبه", "چهارشنبه", "پنجشنبه", "جمعه", "شنبه"},
	{"dew", "aaɓ", "maw", "nje", "naa", "mwd", "hbi"},
	{"dewo", "aaɓnde", "mawbaare", "njeslaare", "naasaande", "mawnde", "hoore-biir"},
	{"𞤈𞤫𞤬", "𞤀𞥄𞤩𞤵", "𞤃𞤢𞤦", "𞤔𞤫𞤧", "𞤐𞤢𞥄𞤧", "𞤃𞤢𞤣", "𞤖𞤮𞤪"},
	{"𞤈𞤫𞤬𞤦𞤭𞤪𞥆𞤫", "𞤀𞥄𞤩𞤵𞤲𞥋𞤣𞤫", "𞤃𞤢𞤱𞤦𞤢𞥄𞤪𞤫", "𞤐𞤶𞤫𞤧𞤤𞤢𞥄𞤪𞤫", "𞤐𞤢𞥄𞤧𞤢𞥄𞤲𞤣𞤫", "𞤃𞤢𞤱𞤲𞤣𞤫", "𞤖𞤮𞤪𞤦𞤭𞤪𞥆𞤫"},
	{"su", "ma", "ti", "ke", "to", "pe", "la"},
	{"sunnuntaina", "maanantaina", "tiistaina", "keskiviikkona", "torstaina", "perjantaina", "lauantaina"},
	{"Lin", "Lun", "Mar", "Miy", "Huw", "Biy", "Sab"},
	{"Linggo", "Lunes", "Martes", "Miyerkules", "Huwebes", "Biyernes", "Sabado"},
	{"sun.", "mán.", "týs.", "mik.", "hós.", "frí.", "ley."},
	{"su.", "má.", "tý.", "mi.", "hó.", "fr.", "le."},
	{"sunnudagur", "mánadagur", "týsdagur", "mikudagur", "hósdagur", "fríggjadagur", "leygardagur"},
	{"dim.", "lun.", "mar.", "mer.", "jeu.", "ven.", "sam."},
	{"di", "lu", "ma", "me", "je", "ve", "sa"},
	{"dimanche", "lundi", "mardi", "mercredi", "jeudi", "vendredi", "samedi"},
	{"Sön", "Mun", "Tei", "Wed", "Tür", "Fre", "San"},
	{"Sö", "Mu", "Te", "We", "Tü", "Fr", "Sa"},
	{"Söndai", "Mundai", "Teisdai", "Weedensdai", "Tüürsdai", "Freidai", "Saninj"},
	{"dom", "lun", "mar", "mie", "joi", "vin", "sab"},
	{"domenie", "lunis", "martars", "miercus", "joibe", "vinars", "sabide"},
	{"si", "mo", "ti", "wo", "to", "fr", "so"},
	{"snein", "moandei", "tiisdei", "woansdei", "tongersdei", "freed", "sneon"},
	{"Domh", "Luan", "Máirt", "Céad", "Déar", "Aoine", "Sath"},
	{"Do", "Lu", "Má", "Cé", "Dé", "Ao", "Sa"},
	{"Dé Domhnaigh", "Dé Luain", "Dé Máirt", "Dé Céadaoin", "Déardaoin", "Dé hAoine", "Dé Sathairn"},
	{"Hɔg", "Ju", "Juf", "Shɔ", "Soo", "Soh", "Hɔɔ"},
	{"Hɔgbaa", "Ju", "Jufɔ", "Shɔ", "Soo", "Sohaa", "Hɔɔ"},
	{"DiD", "DiL", "DiM", "DiC", "Dia", "Dih", "DiS"},
	{"Dò", "Lu", "Mà", "Ci", "Da", "hA", "Sa"},
	{"DiDòmhnaich", "DiLuain", "DiMàirt", "DiCiadain", "DiarDaoin", "DihAoine", "DiSathairne"},
	{"እኁድ", "ሰኑይ", "ሠሉስ", "ራብዕ", "ሐሙስ", "ዓርበ", "ቀዳሚት"},
	{"dom.", "luns", "mar.", "mér.", "xov.", "ven.", "sáb."},
	{"d.", "l.", "m.", "m.", "x.", "v.", "s."},
	{"do.", "lu.", "ma.", "mé.", "xo.", "ve.", "sá."},
	{"domingo", "luns", "martes", "mércores", "xoves", "venres", "sábado"},
	{"Arateĩ", "Arakõi", "Araapy", "Ararundy", "Arapo", "Arapoteĩ", "Arapokõi"},
	{"Su.", "Mä.", "Zi.", "Mi.", "Du.", "Fr.", "Sa."},
	{"Sunntig", "Määntig", "Ziischtig", "Mittwuch", "Dunschtig", "Friitig", "Samschtig"},
	{"રવિ", "સોમ", "મંગળ", "બુધ", "ગુરુ", "શુક્ર", "શનિ"},
	{"ર", "સો", "મં", "બુ", "ગુ", "શુ", "શ"},
	{"રવિવાર", "સોમવાર", "મંગળવાર", "બુધવાર", "ગુરુવાર", "શુક્રવાર", "શનિવાર"},
	{"Cpr", "Ctt", "Cmn", "Cmt", "Ars", "Icm", "Est"},
	{"Chumapiri", "Chumatato", "Chumaine", "Chumatano", "Aramisi", "Ichuma", "Esabato"},
	{"Jed", "Jel", "Jem", "Jerc", "Jerd", "Jeh", "Jes"},
	{"Jedoonee", "Jelhein", "Jemayrt", "Jercean", "Jerdein", "Jeheiney", "Jesarn"},
	{"Lah", "Lit", "Tal", "Lar", "Alh", "Jum", "Asa"},
	{"Lh", "Li", "Ta", "Lr", "Al", "Ju", "As"},
	{"Lahadi", "Litinin", "Talata", "Laraba", "Alhamis", "Jummaʼa", "Asabar"},
	{"لَح", "لِت", "تَل", "لَر", "أَلْح", "جُم", "أَسَ"},
	{"لَحَدِ", "لِتِنِنْ", "تَلَتَ", "لَرَبَ", "أَلْحَمِسْ", "جُمَعَ", "أَسَبَرْ"},
	{"LP", "P1", "P2", "P3", "P4", "P5", "P6"},
	{"Lāpule", "Poʻakahi", "Poʻalua", "Poʻakolu", "Poʻahā", "Poʻalima", "Poʻaono"},
	{"יום א׳", "יום ב׳", "יום ג׳", "יום ד׳", "יום ה׳", "יום ו׳", "שבת"},
	{"א׳", "ב׳", "ג׳", "ד׳", "ה׳", "ו׳", "ש׳"},
	{"יום ראשון", "יום שני", "יום שלישי", "יום רביעי", "יום חמישי", "יום שישי", "יום שבת"},
	{"रवि", "सोम", "मंगल", "बुध", "गुरु", "शुक्र", "शनि"},
	{"र", "सो", "मं", "बु", "गु", "शु", "श"},
	{"रविवार", "सोमवार", "मंगलवार", "बुधवार", "गुरुवार", "शुक्रवार", "शनिवार"},
	{"Ravi", "Som", "Mangal", "Budh", "Guru", "Shukra", "Shani"},
	{"Ra", "So", "Ma", "Bu", "Gu", "Sh", "Sha"},
	{"Ra", "So", "Ma", "Bu", "Gu", "Shu", "Sha"},
	{"Raviwaar", "Somwaar", "Mangalwaar", "Budhwaar", "Guruwaar", "Shukrawaar", "Shaniwaar"},
	{"nje", "pón", "wut", "srj", "štw", "pja", "sob"},
	{"nj", "pó", "wu", "sr", "št", "pj", "so"},
	{"njedźela", "póndźela", "wutora", "srjeda", "štwórtk", "pjatk", "sobota"},
	{"V", "H", "K", "Sze", "Cs", "P", "Szo"},
	{"vasárnap", "hétfő", "kedd", "szerda", "csütörtök", "péntek", "szombat"},
	{"կիր", "երկ", "երք", "չրք", "հնգ", "ուր", "շբթ"},
	{"կր", "եկ", "եք", "չք", "հգ", "ու", "շբ"},
	{"կիրակի", "երկուշաբթի", "երեքշաբթի", "չորեքշաբթի", "հինգշաբթի", "ուրբաթ", "շաբաթ"},
	{"dom", "lun", "mar", "mer", "jov", "ven", "sab"},
	{"do", "lu", "ma", "me", "jo", "ve", "sa"},
	{"dominica", "lunedi", "martedi", "mercuridi", "jovedi", "venerdi", "sabbato"},
	{"Min", "Sen", "Sel", "Rab", "Kam", "Jum", "Sab"},
	{"Minggu", "Senin", "Selasa", "Rabu", "Kamis", "Jumat", "Sabtu"},
	{"sol.", "lun.", "mar.", "mer.", "jov.", "ven.", "sat."},
	{"So", "Lu", "Ma", "Me", "Jo", "Ve", "Sa"},
	{"soledí", "lunedí", "mardí", "mercurdí", "jovedí", "venerdí", "saturdí"},
	{"Sọn", "Mọn", "Tiu", "Wen", "Tọọ", "Fraị", "Sat"},
	{"Sọndee", "Mọnde", "Tiuzdee", "Wenezdee", "Tọọzdee", "Fraịdee", "Satọdee"},
	{"ꑬꆏ", "ꆏꋍ", "ꆏꑍ", "ꆏꌕ", "ꆏꇖ", "ꆏꉬ", "ꆏꃘ"},
	{"ꑬꆏꑍ", "ꆏꊂꋍ", "ꆏꊂꑍ", "ꆏꊂꌕ", "ꆏꊂꇖ", "ꆏꊂꉬ", "ꆏꊂꃘ"},
	{"sun.", "mán.", "þri.", "mið.", "fim.", "fös.", "lau."},
	{"su.", "má.", "þr.", "mi.", "fi.", "fö.", "la."},
	{"sunnudagur", "mánudagur", "þriðjudagur", "miðvikudagur", "fimmtudagur", "föstudagur", "laugardagur"},
	{"dom", "lun", "mar", "mer", "gio", "ven", "sab"},
	{"domenica", "lunedì", "martedì", "mercoledì", "giovedì", "venerdì", "sabato"},
	{"ᓈᑦᑏᖑᔭᖅ", "ᓇᒡᒐᔾᔭᐅ", "ᓇᒡᒐᔾᔭᐅᓕᖅᑭ", "ᐱᖓᑦᓯᖅ", "ᓯᑕᒻᒥᖅ", "ᑕᓪᓕᒻᒥᐅᑦ", "ᓈᑦᓰᖑᔭᓛᕐᓂᐊᖅ"},
	{"日", "月", "火", "水", "木", "金", "土"},
	{"日曜日", "月曜日", "火曜日", "水曜日", "木曜日", "金曜日", "土曜日"},
	{"Sɔ́ndi", "Mɔ́ndi", "Ápta Mɔ́ndi", "Wɛ́nɛsɛdɛ", "Tɔ́sɛdɛ", "Fɛlâyɛdɛ", "Sásidɛ"},
	{"Jpi", "Jtt", "Jnn", "Jtn", "Alh", "Iju", "Jmo"},
	{"Jumapilyi", "Jumatatuu", "Jumanne", "Jumatanu", "Alhamisi", "Ijumaa", "Jumamosi"},
	{"Ahad", "Sen", "Sel", "Rab", "Kam", "Jum", "Sab"},
	{"Ahad", "Senin", "Selasa", "Rabu", "Kamis", "Jumat", "Sabtu"},
	{"კვი", "ორშ", "სამ", "ოთხ", "ხუთ", "პარ", "შაბ"},
	{"კვ", "ორ", "სმ", "ოთ", "ხთ", "პრ", "შბ"},
	{"კვირა", "ორშაბათი", "სამშაბათი", "ოთხშაბათი", "ხუთშაბათი", "პარასკევი", "შაბათი"},
	{"Екш", "Дүй", "Сий", "Сәр", "Пий", "Жум", "Шем"},
	{"Е", "Д", "С", "С", "П", "Ж", "Ш"},
	{"Ек", "Дү", "Си", "Сә", "Пи", "Жу", "Ше"},
	{"екшемби", "дүйшемби", "сийшемби", "сәршемби", "пийшемби", "жума", "шемби"},
	{"Acer", "Arim", "Aram", "Ahad", "Amhad", "Sem", "Sed"},
	{"C", "R", "R", "H", "M", "S", "S"},
	{"Lad", "Lin", "Tal", "Lar", "Lam", "Jum", "Asa"},
	{"Ladi", "Lintani", "Talata", "Larba", "Lamit", "Juma", "Asabar"},
	{"Wky", "Wkw", "Wkl", "Wtũ", "Wkn", "Wtn", "Wth"},
	{"Wa kyumwa", "Wa kwambĩlĩlya", "Wa kelĩ", "Wa katatũ", "Wa kana", "Wa katano", "Wa thanthatũ"},
	{"Lad", "Tan", "Tal", "Lar", "Lam", "Jum", "Asa"},
	{"Ladi", "Tanii", "Talata", "Larba", "Lamit", "Juma", "Asabat"},
	{"Ll2", "Ll3", "Ll4", "Ll5", "Ll6", "Ll7", "Ll1"},
	{"Liduva lyapili", "Liduva lyatatu", "Liduva lyanchechi", "Liduva lyannyano", "Liduva lyannyano na linji", "Liduva lyannyano na mavili", "Liduva litandi"},
	{"dum", "sig", "ter", "kua", "kin", "ses", "sab"},
	{"du", "si", "te", "ku", "ki", "se", "sa"},
	{"dumingu", "sigunda-fera", "tersa-fera", "kuarta-fera", "kinta-fera", "sesta-fera", "sábadu"},
	{"Doʼkutan", "LuʼKutan", "MarʼKutan", "MerʼKutan", "JueʼKutan", "VierʼKutan", "SabʼKutan"},
	{"num.", "pir.", "rég.", "tẽg.", "vẽn.", "pén.", "sav."},
	{"N.", "1kh.", "2kh.", "3kh.", "4kh.", "5kh.", "S."},
	{"numĩggu", "pir-kurã-há", "régre-kurã-há", "tẽgtũ-kurã-há", "vẽnhkãgra-kurã-há", "pénkar-kurã-há", "savnu"},
	{"Alh", "Ati", "Ata", "Ala", "Alm", "Alj", "Ass"},
	{"Alhadi", "Atini", "Atalata", "Alarba", "Alhamiisa", "Aljuma", "Assabdu"},
	{"KMA", "NTT", "NMN", "NMT", "ART", "NMA", "NMM"},
	{"Kiumia", "Njumatatũ", "Njumaine", "Njumatana", "Aramithi", "Njumaa", "Njumamothi"},
	{"жс", "дс", "сс", "ср", "бс", "жм", "сб"},
	{"жексенбі", "дүйсенбі", "сейсенбі", "сәрсенбі", "бейсенбі", "жұма", "сенбі"},
	{"جەك", "دۇي", "سەي", "سار", "بەي", "جۇم", "سەن"},
	{"جە", "دۇ", "سە", "سا", "بە", "جۇ", "سن"},
	{"جەكسەنبى", "دۇيسەنبى", "سەيسەنبى", "سارسەنبى", "بەيسەنبى", "جۇما", "سەنبى"},
	{"sɔndi", "lundi", "mardi", "mɛrkɛrɛdi", "yedi", "vaŋdɛrɛdi", "mɔnɔ sɔndi"},
	{"sap", "ata", "mar", "pin", "sis", "tal", "arf"},
	{"sapaat", "ataasinngorneq", "marlunngorneq", "pingasunngorneq", "sisamanngorneq", "tallimanngorneq", "arfininngorneq"},
	{"Kts", "Kot", "Koo", "Kos", "Koa", "Kom", "Kol"},
	{"Kotisap", "Kotaai", "Koaeng’", "Kosomok", "Koang’wan", "Komuut", "Kolo"},
	{"អាទិត្យ", "ចន្ទ", "អង្គារ", "ពុធ", "ព្រហ", "សុក្រ", "សៅរ៍"},
	{"អា", "ច", "អ", "ពុ", "ព្រ", "សុ", "ស"},
	{"អាទិត្យ", "ច័ន្ទ", "អង្គារ", "ពុធ", "ព្រហស្បតិ៍", "សុក្រ", "សៅរ៍"},
	{"ಭಾನು", "ಸೋಮ", "ಮಂಗಳ", "ಬುಧ", "ಗುರು", "ಶುಕ್ರ", "ಶನಿ"},
	{"ಭಾನುವಾರ", "ಸೋಮವಾರ", "ಮಂಗಳವಾರ", "ಬುಧವಾರ", "ಗುರುವಾರ", "ಶುಕ್ರವಾರ", "ಶನಿವಾರ"},
	{"일", "월", "화", "수", "목", "금", "토"},
	{"일요일", "월요일", "화요일", "수요일", "목요일", "금요일", "토요일"},
	{"आ", "सो", "मं", "बु", "बि", "शु", "शे"},
	{"आ", "सोम", "मंगळार", "बुधवार", "बिरे", "शुक्रार", "शेनवार"},
	{"आयतार", "सोमार", "मंगळार", "बुधवार", "बिरेस्तार", "शुक्रार", "शेनवार"},
	{"Ait", "Som", "Mon", "Bud", "Bre", "Suk", "Son"},
	{"Ai", "Sm", "Mg", "Bu", "Br", "Su", "Sn"},
	{"Aitar", "Somar", "Mongllar", "Budhvar", "Birestar", "Sukrar", "Sonvar"},
	{"آتھوار", "ژٔندٕروار", "بۆموار", "بودوار", "برؠسوار", "جُمہ", "بٹوار"},
	{"اَتھوار", "ژٔندرٕروار", "بۆموار", "بودوار", "برؠسوار", "جُمہ", "بٹوار"},
	{"आथवार", "चंदिरवार", "बुवार", "बोदवार", "ब्रेसवार", "जुमा", "बटवार"},
	{"आथवार", "च़ंदिरवार", "बोमवार", "बोदवार", "ब्रेसवार", "जुमा", "बटवार"},
	{"Jpi", "Jtt", "Jmn", "Jtn", "Alh", "Iju", "Jmo"},
	{"Jumaapii", "Jumaatatu", "Jumaane", "Jumaatano", "Alhamisi", "Ijumaa", "Jumaamosi"},
	{"sɔ́n", "lǝn", "maa", "mɛk", "jǝǝ", "júm", "sam"},
	{"sɔ́ndǝ", "lǝndí", "maadí", "mɛkrɛdí", "jǝǝdí", "júmbá", "samdí"},
	{"Su.", "Mo.", "Di.", "Me.", "Du.", "Fr.", "Sa."},
	{"Su", "Mo", "Di", "Me", "Du", "Fr", "Sa"},
	{"Sunndaach", "Mohndaach", "Dinnsdaach", "Metwoch", "Dunnersdaach", "Friidaach", "Samsdaach"},
	{"yşm", "dşm", "sşm", "çşm", "pşm", "înî", "şem"},
	{"yş", "dş", "sş", "çş", "pş", "în", "şm"},
	{"yekşem", "duşem", "sêşem", "çarşem", "pêncşem", "înî", "şemî"},
	{"Sul", "Lun", "Mth", "Mhr", "Yow", "Gwe", "Sad"},
	{"dy Sul", "dy Lun", "dy Meurth", "dy Merher", "dy Yow", "dy Gwener", "dy Sadorn"},
	{"aadi", "smba", "manga", "puda", "laki", "sukru", "sani"},
	{"aa", "s", "ma", "pu", "laki", "su", "sa"},
	{"aadi vara", "smbara", "mangaḍa", "pudara", "laki vara", "sukru vara", "sani vara"},
	{"आदि", "साॅम्मा", "मान्गा", "पूदा", "लाक्की", "सुकुरु", "सान्नि"},
	{"आ", "साॅ", "मा", "पू", "ला", "सु", "सा"},
	{"आदि वारा", "साॅम्वारा", "मंगाड़ा", "पुद्दारा", "लाक्कि वारा", "सुकुरु वारा", "सान्नि वारा"},
	{"ଆଦି", "ସମ୍ବା", "ମାଙ୍ଗା", "ପୁଦା", "ଲାକି", "ସୁକ୍ରୁ", "ସାନି"},
	{"ଆ", "ସ", "ମା", "ପୁ", "ଲା", "ସୁ", "ସାନି"},
	{"ଆଦି ୱାରା", "ସମ୍ବାରା", "ମାଙ୍ଗାଡ଼ା", "ପୁଦାରା", "ଲାକି ୱାରା", "ସୁକ୍ରୁ ୱାରା", "ସାନି ୱାରା"},
	{"వారమి", "నమారా", "మాంగాడా", "వుదారా", "లాకివరా", "నుక్ వరా", "సానివరా"},
	{"жек.", "дүй.", "шейш.", "шарш.", "бейш.", "жума", "ишм."},
	{"жш.", "дш.", "шш.", "шр.", "бш.", "жм.", "иш."},
	{"жекшемби", "дүйшөмбү", "шейшемби", "шаршемби", "бейшемби", "жума", "ишемби"},
	{"Dom", "Lun", "Mar", "Mer", "Iov", "Ven", "Sab"},
	{"Dominica", "dies Lunae", "dies Martis", "dies Mercurii", "dies Iovis", "dies Veneris", "dies Sabbati"},
	{"Píili", "Táatu", "Íne", "Táano", "Alh", "Ijm", "Móosi"},
	{"Jumapíiri", "Jumatátu", "Jumaíne", "Jumatáano", "Alamíisi", "Ijumáa", "Jumamóosi"},
	{"Son.", "Méi.", "Dën.", "Mët.", "Don.", "Fre.", "Sam."},
	{"So.", "Mé.", "Dë.", "Më.", "Do.", "Fr.", "Sa."},
	{"Sonndeg", "Méindeg", "Dënschdeg", "Mëttwoch", "Donneschdeg", "Freideg", "Samschdeg"},
	{"Sab", "Bal", "Lw2", "Lw3", "Lw4", "Lw5", "Lw6"},
	{"Sabbiiti", "Balaza", "Lwakubiri", "Lwakusatu", "Lwakuna", "Lwakutaano", "Lwamukaaga"},
	{"dom.", "lun.", "mät.", "mäc.", "zeu.", "ven.", "sab."},
	{"domenega", "lunesdì", "mätesdì", "mäcordì", "zeuggia", "venardì", "sabbo"},
	{"A", "W", "N", "Y", "T", "Z", "O"},
	{"Aŋpétuwakȟaŋ", "Aŋpétuwaŋži", "Aŋpétunuŋpa", "Aŋpétuyamni", "Aŋpétutopa", "Aŋpétuzaptaŋ", "Owáŋgyužažapi"},
	{"dom", "lön", "mert", "merc", "jöb", "vën", "sab"},
	{"domënia", "lönesc", "mertesc", "mercui", "jöbia", "vëndres", "sabeda"},
	{"domenega", "lundì", "mardì", "mercoldì", "sgiovedì", "venerdì", "sabet"},
	{"eye", "ybo", "mbl", "mst", "min", "mtn", "mps"},
	{"eyenga", "mokɔlɔ mwa yambo", "mokɔlɔ mwa míbalé", "mokɔlɔ mwa mísáto", "mokɔlɔ ya mínéi", "mokɔlɔ ya mítáno", "mpɔ́sɔ"},
	{"ອາທິດ", "ຈັນ", "ອັງຄານ", "ພຸດ", "ພະຫັດ", "ສຸກ", "ເສົາ"},
	{"ອາ.", "ຈ.", "ອ.", "ພ.", "ພຫ.", "ສຸ.", "ສ."},
	{"ວັນອາທິດ", "ວັນຈັນ", "ວັນອັງຄານ", "ວັນພຸດ", "ວັນພະຫັດ", "ວັນສຸກ", "ວັນເສົາ"},
	{"sk", "pr", "an", "tr", "kt", "pn", "št"},
	{"Sk", "Pr", "An", "Tr", "Kt", "Pn", "Št"},
	{"sekmadienis", "pirmadienis", "antradienis", "trečiadienis", "ketvirtadienis", "penktadienis", "šeštadienis"},
	{"svātdīne", "pyrmūdīne", "ūtardīne", "trešdīne", "catūrtdīne", "pīktdīne", "sastdīne"},
	{"Lum", "Nko", "Ndy", "Ndg", "Njw", "Ngv", "Lub"},
	{"Lumingu", "Nkodya", "Ndàayà", "Ndangù", "Njòwa", "Ngòvya", "Lubingu"},
	{"JMP", "WUT", "TAR", "TAD", "TAN", "TAB", "NGS"},
	{"Jumapil", "Wuok Tich", "Tich Ariyo", "Tich Adek", "Tich Ang’wen", "Tich Abich", "Ngeso"},
	{"J2", "J3", "J4", "J5", "Al", "Ij", "J1"},
	{"Jumapiri", "Jumatatu", "Jumanne", "Jumatano", "Murwa wa Kanne", "Murwa wa Katano", "Jumamosi"},
	{"svētd.", "pirmd.", "otrd.", "trešd.", "ceturtd.", "piektd.", "sestd."},
	{"Sv", "Pr", "Ot", "Tr", "Ce", "Pk", "Se"},
	{"svētdiena", "pirmdiena", "otrdiena", "trešdiena", "ceturtdiena", "piektdiena", "sestdiena"},
	{"रवि दिन", "सोम दिन", "मंगल दिन", "बुध दिन", "बृहस्पति दिन", "शुक्र दिन", "शनि दिन"},
	{"Jumapílí", "Jumatátu", "Jumane", "Jumatánɔ", "Alaámisi", "Jumáa", "Jumamósi"},
	{"KIU", "MRA", "WAI", "WET", "WEN", "WTN", "JUM"},
	{"Kiumia", "Muramuko", "Wairi", "Wethatu", "Wena", "Wetano", "Jumamosi"},
	{"dim", "lin", "mar", "mer", "ze", "van", "sam"},
	{"dimans", "lindi", "mardi", "merkredi", "zedi", "vandredi", "samdi"},
	{"Alah", "Alats", "Tal", "Alar", "Alak", "Zom", "Asab"},
	{"Alahady", "Alatsinainy", "Talata", "Alarobia", "Alakamisy", "Zoma", "Asabotsy"},
	{"Sab", "Jtt", "Jnn", "Jtn", "Ara", "Iju", "Jmo"},
	{"Sabato", "Jumatatu", "Jumanne", "Jumatano", "Arahamisi", "Ijumaa", "Jumamosi"},
	{"A1", "A2", "A3", "A4", "A5", "A6", "A7"},
	{"Aneg 1", "Aneg 2", "Aneg 3", "Aneg 4", "Aneg 5", "Aneg 6", "Aneg 7"},
	{"Rāt", "Man", "Tūr", "Wen", "Tāi", "Par", "Rāh"},
	{"Rāt", "Man", "Tū", "Wen", "Tāi", "Par", "Rāh"},
	{"Rātapu", "Mane", "Tūrei", "Wenerei", "Tāite", "Paraire", "Rāhoroi"},
	{"нед.", "пон.", "вто.", "сре.", "чет.", "пет.", "саб."},
	{"недела", "понеделник", "вторник", "среда", "четврток", "петок", "сабота"},
	{"ഞായർ", "തിങ്കൾ", "ചൊവ്വ", "ബുധൻ", "വ്യാഴം", "വെള്ളി", "ശനി"},
	{"ഞ", "തി", "ചൊ", "ബു", "വ്യാ", "വെ", "ശ"},
	{"ഞാ", "തി", "ചൊ", "ബു", "വ്യാ", "വെ", "ശ"},
	{"ഞായറാഴ്\u200cച", "തിങ്കളാഴ്\u200cച", "ചൊവ്വാഴ്ച", "ബുധനാഴ്\u200cച", "വ്യാഴാഴ്\u200cച", "വെള്ളിയാഴ്\u200cച", "ശനിയാഴ്\u200cച"},
	{"Ня", "Да", "Мя", "Лх", "Пү", "Ба", "Бя"},
	{"ням", "даваа", "мягмар", "лхагва", "пүрэв", "баасан", "бямба"},
	{"ᠨᠢ", "ᠲᠠ", "ᠮᠢᠭ", "ᡀᠠ", "ᠫᠥᠷ", "ᠪᠠ", "ᠪᠢᠮ"},
	{"ᠨᠢ", "ᠳᠠ", "ᠮᠢᠭ", "ᡀᠠ", "ᠫᠥᠷ", "ᠪᠠ", "ᠪᠢ"},
	{"ᠨᠢᠮ\u180eᠠ", "ᠳᠠᠸᠠ", "ᠮᠢᠠᠠᠮᠠᠷ", "ᡀᠠᠭᠪᠠ", "ᠫᠦᠷᠪᠦ", "ᠪᠠᠰᠠᠩ", "ᠪᠢᠮᠪᠠ"},
	{"নোং", "নিং", "লৈবা", "য়ুম", "শগো", "ইরা", "থাং"},
	{"নোংমাইজিং", "নিংথৌকাবা", "লৈবাকপোকপা", "য়ুমশকৈশা", "শগোলশেন", "ইরাই", "থাংজ"},
	{"रवि", "सोम", "मंगळ", "बुध", "गुरु", "शुक्र", "शनि"},
	{"रविवार", "सोमवार", "मंगळवार", "बुधवार", "गुरुवार", "शुक्रवार", "शनिवार"},
	{"Ahd", "Isn", "Sel", "Rab", "Kha", "Jum", "Sab"},
	{"Ah", "Is", "Se", "Ra", "Kh", "Ju", "Sa"},
	{"Ahad", "Isnin", "Selasa", "Rabu", "Khamis", "Jumaat", "Sabtu"},
	{"احد", "اثنين", "ثلاث", "رابو", "خميس", "جمعة", "سبتو"},
	{"Ħad", "Tne", "Tli", "Erb", "Ħam", "Ġim", "Sib"},
	{"Ħd", "T", "Tl", "Er", "Ħm", "Ġm", "Sb"},
	{"Il-Ħadd", "It-Tnejn", "It-Tlieta", "L-Erbgħa", "Il-Ħamis", "Il-Ġimgħa", "Is-Sibt"},
	{"Cya", "Cla", "Czi", "Cko", "Cka", "Cga", "Cze"},
	{"Com’yakke", "Comlaaɗii", "Comzyiiɗii", "Comkolle", "Comkaldǝɓlii", "Comgaisuu", "Comzyeɓsuu"},
	{"Nettvʼcako", "Enhvteceskv", "Enhvteceskv Enhvyvtke", "Ennvrkvpv", "Ennvrkvpv Enhvyvtke", "Nak Okkoskv Nettv", "Nettv Cakʼcuse"},
	{"နွေ", "လာ", "ဂါ", "ဟူး", "တေး", "ကြာ", "နေ"},
	{"တနင်္ဂနွေ", "တနင်္လာ", "အင်္ဂါ", "ဗုဒ္ဓဟူး", "ကြာသပတေး", "သောကြာ", "စနေ"},
	{"тар", "атя", "вас", "кун", "кал", "сюк", "шля"},
	{"таргочистэ", "атяньчистэ", "вастаньчистэ", "куншкачистэ", "калоньчистэ", "сюконьчистэ", "шлямочистэ"},
	{"Son", "Ma", "De", "Wu", "Do", "Fr", "Sat"},
	{"Sontaxtsees", "Mantaxtsees", "Denstaxtsees", "Wunstaxtsees", "Dondertaxtsees", "Fraitaxtsees", "Satertaxtsees"},
	{"søn.", "man.", "tir.", "ons.", "tor.", "fre.", "lør."},
	{"Son", "Mvu", "Sib", "Sit", "Sin", "Sih", "Mgq"},
	{"Sonto", "Mvulo", "Sibili", "Sithathu", "Sine", "Sihlanu", "Mgqibelo"},
	{"Sü.", "Ma.", "Di.", "Mi.", "Du.", "Fr.", "Sa."},
	{"Sünndag", "Maandag", "Dingsdag", "Middeweken", "Dunnersdag", "Freedag", "Sünnavend"},
	{"आइत", "सोम", "मङ्गल", "बुध", "बिहि", "शुक्र", "शनि"},
	{"आइतबार", "सोमबार", "मङ्गलबार", "बुधबार", "बिहिबार", "शुक्रबार", "शनिबार"},
	{"zo", "ma", "di", "wo", "do", "vr", "za"},
	{"zondag", "maandag", "dinsdag", "woensdag", "donderdag", "vrijdag", "zaterdag"},
	{"sɔ́n", "mɔ́n", "smb", "sml", "smn", "mbs", "sas"},
	{"sɔ́ndɔ", "mɔ́ndɔ", "sɔ́ndɔ mafú mába", "sɔ́ndɔ mafú málal", "sɔ́ndɔ mafú mána", "mabágá má sukul", "sásadi"},
	{"sø.", "må.", "ty.", "on.", "to.", "fr.", "la."},
	{"søndag", "måndag", "tysdag", "onsdag", "torsdag", "fredag", "laurdag"},
	{"lyɛʼɛ́ sẅíŋtè", "mvfò lyɛ̌ʼ", "mbɔ́ɔntè mvfò lyɛ̌ʼ", "tsètsɛ̀ɛ lyɛ̌ʼ", "mbɔ́ɔntè tsetsɛ̀ɛ lyɛ̌ʼ", "mvfò màga lyɛ̌ʼ", "màga lyɛ̌ʼ"},
	{"ߞߊ߯ߙ", "ߞߐ߬ߓ", "ߞߐ߬ߟߏ߲", "ߞߎߣ", "ߓߌߟ", "ߛߌ߬ߣ", "ߞߍ߲ߘ"},
	{"ߞߊ߯", "ߞߐ߬", "ߞߐ߬ߟߏ߲", "ߞߎ", "ߓߌ", "ߛߌ߬", "ߞߍ߲"},
	{"ߞߊ߯ߙߌߟߏ߲", "ߞߐ߬ߓߊ߬ߟߏ߲", "ߞߐ߬ߟߏ߲", "ߞߎߣߎ߲ߟߏ߲", "ߓߌߟߏ߲", "ߛߌ߬ߣߌ߲߬ߟߏ߲", "ߞߍ߲ߘߍߟߏ߲"},
	{"Son", "Mvu", "Bil", "Tha", "Ne", "Hla", "Gqi"},
	{"uSonto", "uMvulo", "uLesibili", "Lesithathu", "uLesine", "ngoLesihlanu", "umGqibelo"},
	{"Lam", "Mos", "Bed", "Rar", "Ne", "Hla", "Mok"},
	{"Lamorena", "Mošupologo", "Labobedi", "Laboraro", "Labone", "Labohlano", "Mokibelo"},
	{"Cäŋ", "Jiec", "Rɛw", "Diɔ̱k", "Ŋuaan", "Dhieec", "Bäkɛl"},
	{"Cäŋ kuɔth", "Jiec la̱t", "Rɛw lätni", "Diɔ̱k lätni", "Ŋuaan lätni", "Dhieec lätni", "Bäkɛl lätni"},
	{"Mul", "Lem", "Wir", "Tat", "Nai", "San", "Wer"},
	{"Lamulungu", "Lolemba", "Lachiwiri", "Lachitatu", "Lachinayi", "Lachisanu", "Loweruka"},
	{"dg", "dl", "dm", "dc", "dj", "dv", "ds"},
	{"dimenge", "diluns", "dimars", "dimècres", "dijòus", "divendres", "dissabte"},
	{"dim", "del", "dma", "dmè", "dij", "diu", "dis"},
	{"di", "de", "da", "dm", "dj", "du", "ds"},
	{"dimenge", "deluns", "dimars", "dimèrcles", "dijaus", "diuendres", "dissabte"},
	{"Dil", "Wix", "Kib", "Rob", "Kam", "Jim", "San"},
	{"Dilbata", "Wiixata", "Kibxata", "Roobii", "Kamisa", "Jimaata", "Sanbata"},
	{"ରବି", "ସୋମ", "ମଙ୍ଗଳ", "ବୁଧ", "ଗୁରୁ", "ଶୁକ୍ର", "ଶନି"},
	{"ରବିବାର", "ସୋମବାର", "ମଙ୍ଗଳବାର", "ବୁଧବାର", "ଗୁରୁବାର", "ଶୁକ୍ରବାର", "ଶନିବାର"},
	{"хцб", "крс", "дцг", "ӕрт", "цпр", "мрб", "сбт"},
	{"хуыцаубон", "къуырисӕр", "дыццӕг", "ӕртыццӕг", "цыппӕрӕм", "майрӕмбон", "сабат"},
	{"𐓏", "𐓄", "𐓍", "𐒴", "𐓈", "𐓊", "𐓸"},
	{"𐒹𐓘͘𐓬𐓘 𐓏𐓘𐓤𐓘͘𐓰𐓘𐓤𐓣", "𐒹𐓘͘𐓬𐓘 𐓄𐓘𐓡𐓛͘𐓧𐓣", "𐒹𐓘͘𐓬𐓘 𐓏𐓟𐓵𐓪͘𐓬𐓘", "𐒹𐓘͘𐓬𐓘 𐓏𐓟𐓵𐓘𐓜𐓣", "𐒹𐓘͘𐓬𐓘 𐓏𐓟𐓰𐓪𐓬𐓘", "𐒹𐓘͘𐓬𐓘 𐓈𐓘 𐓵𐓘𐓲𐓘 𐓻𐓣͘", "𐒹𐓘͘𐓬𐓘 𐓂𐓤𐓘𐓸𐓟 𐓣͘𐓤𐓟"},
	{"ਐਤ", "ਸੋਮ", "ਮੰਗਲ", "ਬੁੱਧ", "ਵੀਰ", "ਸ਼ੁੱਕਰ", "ਸ਼ਨੀ"},
	{"ਐਤ", "ਸੋਮ", "ਮੰਗ", "ਬੁੱਧ", "ਵੀਰ", "ਸ਼ੁੱਕ", "ਸ਼ਨੀ"},
	{"ਐਤਵਾਰ", "ਸੋਮਵਾਰ", "ਮੰਗਲਵਾਰ", "ਬੁੱਧਵਾਰ", "ਵੀਰਵਾਰ", "ਸ਼ੁੱਕਰਵਾਰ", "ਸ਼ਨੀਵਾਰ"},
	{"اتوار", "پیر", "منگل", "بُدھ", "جمعرات", "جمعہ", "ہفتہ"},
	{"djadumingu", "djaluna", "djamars", "djarason", "djaweps", "djabièrnè", "djasabra"},
	{"Sọ́n", "Mọ́n", "Tiú", "Wẹ́n", "Tọ́z", "Fraí", "Sát"},
	{"Sọ́ndè", "Mọ́ndè", "Tiúzdè", "Wẹ́nẹ́zdè", "Tọ́zdè", "Fraídè", "Sátọdè"},
	{"Sande", "Mande", "Tiusde", "Wenesde", "Tosde", "Fraede", "Satade"},
	{"niedz.", "pon.", "wt.", "śr.", "czw.", "pt.", "sob."},
	{"n", "p", "w", "ś", "c", "p", "s"},
	{"nie", "pon", "wto", "śro", "czw", "pią", "sob"},
	{"niedziela", "poniedziałek", "wtorek", "środa", "czwartek", "piątek", "sobota"},
	{"dumìnica", "lun-es", "màrtes", "merco", "giòbia", "vënner", "saba"},
	{"nad", "pan", "wis", "pus", "ket", "pēn", "sab"},
	{"nadīli", "panadīli", "wisasīdis", "pussisawaiti", "ketwirtiks", "pēntniks", "sabattika"},
	{"يونۍ", "دونۍ", "درېنۍ", "څلرنۍ", "پينځنۍ", "جمعه", "اونۍ"},
	{"dom.", "seg.", "ter.", "qua.", "qui.", "sex.", "sáb."},
	{"domingo", "segunda-feira", "terça-feira", "quarta-feira", "quinta-feira", "sexta-feira", "sábado"},
	{"domingo", "segunda", "terça", "quarta", "quinta", "sexta", "sábado"},
	{"Dom", "Lun", "Mar", "Mié", "Jue", "Vie", "Sab"},
	{"Domingo", "Lunes", "Martes", "Miércoles", "Jueves", "Viernes", "Sábado"},
	{"lḥe", "let", "ttl", "lar", "lex", "jje", "sse"},
	{"lḥ", "le", "tt", "la", "lx", "jj", "ss"},
	{"lḥed", "letnayen", "ttlat", "larbeɛ", "lexmis", "jjemɛa", "ssebt"},
	{"du", "gli", "ma", "me", "gie", "ve", "so"},
	{"dumengia", "glindesdi", "mardi", "mesemna", "gievgia", "venderdi", "sonda"},
	{"cu.", "mbe.", "kab.", "gtu.", "kan.", "gnu.", "gnd."},
	{"Ku w’indwi", "Ku wa mbere", "Ku wa kabiri", "Ku wa gatatu", "Ku wa kane", "Ku wa gatanu", "Ku wa gatandatu"},
	{"dum.", "lun.", "mar.", "mie.", "joi", "vin.", "sâm."},
	{"du.", "lu.", "ma.", "mi.", "joi", "vi.", "sâ."},
	{"duminică", "luni", "marți", "miercuri", "joi", "vineri", "sâmbătă"},
	{"Dum", "Lun", "Mar", "Mie", "Joi", "Vin", "Sâm"},
	{"Du", "Lu", "Ma", "Mi", "Jo", "Vi", "Sâ"},
	{"Ijp", "Ijt", "Ijn", "Ijtn", "Alh", "Iju", "Ijm"},
	{"Ijumapili", "Ijumatatu", "Ijumanne", "Ijumatano", "Alhamisi", "Ijumaa", "Ijumamosi"},
	{"вс", "пн", "вт", "ср", "чт", "пт", "сб"},
	{"воскресенье", "понедельник", "вторник", "среда", "четверг", "пятница", "суббота"},
	{"Cyu.", "Mbe.", "Kab.", "Gtu.", "Kan.", "Gnu.", "Gnd."},
	{"Ku cyumweru", "Kuwa mbere", "Kuwa kabiri", "Kuwa gatatu", "Kuwa kane", "Kuwa gatanu", "Kuwa gatandatu"},
	{"रविवासरः", "सोमवासरः", "मंगलवासरः", "बुधवासरः", "गुरुवासर:", "शुक्रवासरः", "शनिवासरः"},
	{"бс", "бн", "оп", "сэ", "чп", "бэ", "сб"},
	{"баскыһыанньа", "бэнидиэнньик", "оптуорунньук", "сэрэдэ", "чэппиэр", "Бээтиҥсэ", "субуота"},
	{"Are", "Kun", "Ong", "Ine", "Ile", "Sap", "Kwe"},
	{"Mderot ee are", "Mderot ee kuni", "Mderot ee ong’wan", "Mderot ee inet", "Mderot ee ile", "Mderot ee sapa", "Mderot ee kwe"},
	{"ᱥᱤᱸ", "ᱚᱛ", "ᱵᱟ", "ᱥᱟᱹ", "ᱥᱟᱹᱨ", "ᱡᱟᱹ", "ᱧᱩ"},
	{"ᱥᱤᱸᱜᱮ", "ᱚᱛᱮ", "ᱵᱟᱞᱮ", "ᱥᱟᱹᱜᱩᱱ", "ᱥᱟᱹᱨᱫᱤ", "ᱡᱟᱹᱨᱩᱢ", "ᱧᱩᱦᱩᱢ"},
	{"Mul", "Jtt", "Jnn", "Jtn", "Alh", "Iju", "Jmo"},
	{"Mulungu", "Jumatatu", "Jumanne", "Jumatano", "Alahamisi", "Ijumaa", "Jumamosi"},
	{"dom", "lun", "mar", "mèr", "giò", "che", "sàb"},
	{"domìniga", "lunis", "martis", "mèrcuris", "giòbia", "chenàbura", "sàbadu"},
	{"dum", "lun", "mar", "mer", "jov", "ven", "sab"},
	{"du", "lu", "ma", "me", "jo", "ve", "sa"},
	{"dumìnica", "lunnidìa", "martidìa", "mercuridìa", "jovidìa", "venniridìa", "sàbbatu"},
	{"آچر", "سو", "اڱ", "اربع", "خم", "جم", "ڇن"},
	{"آچر", "سومر", "اڱارو", "اربع", "خميس", "جمعو", "ڇنڇر"},
	{"आर्त", "सू", "मंग", "ॿुध", "विस", "जुम", "छंछ"},
	{"आ", "सू", "मं", "ॿु", "वि", "जु", "छं"},
	{"आर्तवार", "सूमर", "मंगलु", "ॿुधर", "विस्पत", "जुमो", "छंछर"},
	{"sotn", "vuos", "maŋ", "gask", "duor", "bear", "láv"},
	{"sotnabeaivi", "vuossárga", "maŋŋebárga", "gaskavahkku", "duorasdat", "bearjadat", "lávvardat"},
	{"so", "má", "di", "ga", "du", "be", "lá"},
	{"", "M", "D", "", "", "", ""},
	{"sotnabeaivi", "mánnodat", "disdat", "gaskavahkku", "duorastat", "bearjadat", "lávvordat"},
	{"Dim", "Pos", "Pir", "Tat", "Nai", "Sha", "Sab"},
	{"Dimingu", "Chiposi", "Chipiri", "Chitatu", "Chinai", "Chishanu", "Sabudu"},
	{"Alhadi", "Atinni", "Atalaata", "Alarba", "Alhamiisa", "Alzuma", "Asibti"},
	{"Bk1", "Bk2", "Bk3", "Bk4", "Bk5", "Lâp", "Lây"},
	{"Bikua-ôko", "Bïkua-ûse", "Bïkua-ptâ", "Bïkua-usïö", "Bïkua-okü", "Lâpôsö", "Lâyenga"},
	{"ⴰⵙⴰ", "ⴰⵢⵏ", "ⴰⵙⵉ", "ⴰⴽⵕ", "ⴰⴽⵡ", "ⴰⵙⵉⵎ", "ⴰⵙⵉⴹ"},
	{"ⴰⵙⴰⵎⴰⵙ", "ⴰⵢⵏⴰⵙ", "ⴰⵙⵉⵏⴰⵙ", "ⴰⴽⵕⴰⵙ", "ⴰⴽⵡⴰⵙ", "ⵙⵉⵎⵡⴰⵙ", "ⴰⵙⵉⴹⵢⴰⵙ"},
	{"asa", "ayn", "asi", "akṛ", "akw", "asim", "asiḍ"},
	{"asamas", "aynas", "asinas", "akṛas", "akwas", "asimwas", "asiḍyas"},
	{"တိတ်ႉ", "ၸၼ်", "ၵၢၼ်း", "ပုတ်ႉ", "ၽတ်း", "သုၵ်း", "သဝ်"},
	{"တိ.", "ၸ.", "ၵ.", "ပု.", "ၽ.", "သု.", "သ."},
	{"ဝၼ်းဢႃးတိတ်ႉ", "ဝၼ်းၸၼ်", "ဝၼ်းဢင်းၵၢၼ်း", "ဝၼ်းပုတ်ႉ", "ဝၼ်းၽတ်း", "ဝၼ်းသုၵ်း", "ဝၼ်းသဝ်"},
	{"ඉරිදා", "සඳුදා", "අඟහ", "බදාදා", "බ්\u200dරහස්", "සිකු", "සෙන"},
	{"ඉරි", "සඳු", "අඟ", "බදා", "බ්\u200dරහ", "සිකු", "සෙන"},
	{"ඉරිදා", "සඳුදා", "අඟහරුවාදා", "බදාදා", "බ්\u200dරහස්පතින්දා", "සිකුරාදා", "සෙනසුරාදා"},
	{"Sam", "San", "Mak", "Row", "Ham", "Arb", "Qid"},
	{"Sambata", "Sanyo", "Maakisanyo", "Roowe", "Hamuse", "Arbe", "Qidaame"},
	{"ne", "po", "ut", "st", "št", "pi", "so"},
	{"nedeľa", "pondelok", "utorok", "streda", "štvrtok", "piatok", "sobota"},
	{"اتوار", "سوموار", "منگل", "ٻدھ", "خمیس", "جمعہ", "چھݨ چھݨ"},
	{"ned.", "pon.", "tor.", "sre.", "čet.", "pet.", "sob."},
	{"nedelja", "ponedeljek", "torek", "sreda", "četrtek", "petek", "sobota"},
	{"pas", "vuo", "maj", "kos", "tuo", "vás", "láv"},
	{"p", "V", "M", "K", "T", "V", "L"},
	{"pa", "vu", "ma", "ko", "tu", "vá", "lá"},
	{"pasepeeivi", "vuossaargâ", "majebaargâ", "koskoho", "tuorâstuv", "vástuppeeivi", "lávurduv"},
	{"Svo", "Muv", "Chp", "Cht", "Chn", "Chs", "Mug"},
	{"Sv", "Mu", "Cp", "Ct", "Cn", "Cs", "Mg"},
	{"Svondo", "Muvhuro", "Chipiri", "Chitatu", "China", "Chishanu", "Mugovera"},
	{"Axd", "Isn", "Tldo", "Arbc", "Khms", "Jmc", "Sbti"},
	{"Axad", "Isniin", "Talaado", "Arbaco", "Khamiis", "Jimco", "Sabti"},
	{"die", "hën", "mar", "mër", "enj", "pre", "sht"},
	{"e diel", "e hënë", "e martë", "e mërkurë", "e enjte", "e premte", "e shtunë"},
	{"нед", "пон", "уто", "сре", "чет", "пет", "суб"},
	{"не", "по", "ут", "ср", "че", "пе", "су"},
	{"недеља", "понедељак", "уторак", "среда", "четвртак", "петак", "субота"},
	{"недјеља", "понедељак", "уторак", "сриједа", "четвртак", "петак", "субота"},
	{"ned", "pon", "uto", "sre", "čet", "pet", "sub"},
	{"ne", "po", "ut", "sr", "če", "pe", "su"},
	{"nedelja", "ponedeljak", "utorak", "sreda", "četvrtak", "petak", "subota"},
	{"nedjelja", "ponedeljak", "utorak", "srijeda", "četvrtak", "petak", "subota"},
	{"Son", "Mso", "Bil", "Tsa", "Ne", "Hla", "Mgc"},
	{"Lisontfo", "uMsombuluko", "Lesibili", "Lesitsatfu", "Lesine", "Lesihlanu", "uMgcibelo"},
	{"Nab", "San", "Sal", "Rab", "Cam", "Jum", "Qun"},
	{"Naba Sambat", "Sani", "Salus", "Rabuq", "Camus", "Jumqata", "Qunxa Sambat"},
	{"Son", "Mma", "Bed", "Rar", "Ne", "Hla", "Moq"},
	{"Sontaha", "Mantaha", "Labobedi", "Laboraro", "Labone", "Labohlano", "Moqebelo"},
	{"Mng", "Sen", "Sal", "Reb", "Kem", "Jum", "Sap"},
	{"Minggu", "Senén", "Salasa", "Rebo", "Kemis", "Jumaah", "Saptu"},
	{"sön", "mån", "tis", "ons", "tors", "fre", "lör"},
	{"sö", "må", "ti", "on", "to", "fr", "lö"},
	{"söndag", "måndag", "tisdag", "onsdag", "torsdag", "fredag", "lördag"},
	{"ܚܕ", "ܬܪܝܢ", "ܬܠܬ", "ܐܪܒܥ", "ܚܡܫ", "ܥܪܘ", "ܫܒܬܐ"},
	{"ܚܕܒܫܒܐ", "ܬܪܝܢܒܫܒܐ", "ܬܠܬܒܫܒܐ", "ܐܪܒܥܒܫܒܐ", "ܚܡܫܒܫܒܐ", "ܥܪܘܒܬܐ", "ܫܒܬܐ"},
	{"niy", "pyń", "wto", "str", "szt", "piō", "sob"},
	{"nd", "pń", "wt", "st", "sz", "pt", "sb"},
	{"niydziela", "pyńdziałek", "wtorek", "strzoda", "sztwortek", "piōntek", "sobota"},
	{"ஞாயி.", "திங்.", "செவ்.", "புத.", "வியா.", "வெள்.", "சனி"},
	{"ஞா", "தி", "செ", "பு", "வி", "வெ", "ச"},
	{"ஞாயிறு", "திங்கள்", "செவ்வாய்", "புதன்", "வியாழன்", "வெள்ளி", "சனி"},
	{"ఆది", "సోమ", "మంగళ", "బుధ", "గురు", "శుక్ర", "శని"},
	{"ఆది", "సోమ", "మం", "బుధ", "గురు", "శుక్ర", "శని"},
	{"ఆదివారం", "సోమవారం", "మంగళవారం", "బుధవారం", "గురువారం", "శుక్రవారం", "శనివారం"},
	{"Jum", "Bar", "Aar", "Uni", "Ung", "Kan", "Sab"},
	{"Nakaejuma", "Nakaebarasa", "Nakaare", "Nakauni", "Nakaung’on", "Nakakany", "Nakasabiti"},
	{"Яшб", "Дшб", "Сшб", "Чшб", "Пшб", "Ҷмъ", "Шнб"},
	{"Якшанбе", "Душанбе", "Сешанбе", "Чоршанбе", "Панҷшанбе", "Ҷумъа", "Шанбе"},
	{"อาทิตย์", "จันทร์", "อังคาร", "พุธ", "พฤหัส", "ศุกร์", "เสาร์"},
	{"อา.", "จ.", "อ.", "พ.", "พฤ.", "ศ.", "ส."},
	{"วันอาทิตย์", "วันจันทร์", "วันอังคาร", "วันพุธ", "วันพฤหัสบดี", "วันศุกร์", "วันเสาร์"},
	{"ሰን", "ሰኑ", "ሰሉ", "ረቡ", "ሓሙ", "ዓር", "ቀዳ"},
	{"ሰንበት", "ሰኑይ", "ሰሉስ", "ረቡዕ", "ሓሙስ", "ዓርቢ", "ቀዳም"},
	{"ሰ/ዓ", "ሰኖ", "ታላሸ", "ኣረር", "ከሚሽ", "ጅምዓ", "ሰ/ን"},
	{"ሰንበት ዓባይ", "ሰኖ", "ታላሸኖ", "ኣረርባዓ", "ከሚሽ", "ጅምዓት", "ሰንበት ንኢሽ"},
	{"ýek", "duş", "siş", "çar", "pen", "ann", "şen"},
	{"ýb", "db", "sb", "çb", "pb", "an", "şb"},
	{"ýekşenbe", "duşenbe", "sişenbe", "çarşenbe", "penşenbe", "anna", "şenbe"},
	{"Tsh", "Mos", "Labb", "Labr", "Labn", "Labt", "Mat"},
	{"Tshipi", "Mosupologo", "Labobedi", "Laboraro", "Labone", "Labotlhano", "Matlhatso"},
	{"Sāp", "Mōn", "Tūs", "Pul", "Tuʻa", "Fal", "Tok"},
	{"Sāpate", "Mōnite", "Tūsite", "Pulelulu", "Tuʻapulelulu", "Falaite", "Tokonaki"},
	{"7", "1", "2", "3", "4", "5", "6"},
	{"suno esun #7", "suno esun #1", "suno esun #2", "suno esun #3", "suno esun #4", "suno esun #5", "suno esun #6"},
	{"San", "Man", "Tun", "Tri", "Fon", "Fra", "Sar"},
	{"Sande", "Mande", "Tunde", "Trinde", "Fonde", "Fraide", "Sarere"},
	{"Paz", "Pzt", "Sal", "Çar", "Per", "Cum", "Cmt"},
	{"Pa", "Pt", "Sa", "Ça", "Pe", "Cu", "Ct"},
	{"Pazar", "Pazartesi", "Salı", "Çarşamba", "Perşembe", "Cuma", "Cumartesi"},
	{"Emp", "Kin", "Dha", "Tru", "Spa", "Rim", "Mat"},
	{"Jiyax sngayan", "tgKingal jiyax iyax sngayan", "tgDha jiyax iyax sngayan", "tgTru jiyax iyax sngayan", "tgSpac jiyax iyax sngayan", "tgRima jiyax iyax sngayan", "tgMataru jiyax iyax sngayan"},
	{"ایکشیمے", "دُوشیمے", "گھن آنگا", "چارشیمے", "پَئ شیمے", "شُوگار", "لَو آنگا"},
	{"Son", "Mus", "Bir", "Har", "Ne", "Tlh", "Mug"},
	{"Sonta", "Musumbhunuku", "Ravumbirhi", "Ravunharhu", "Ravumune", "Ravuntlhanu", "Mugqivela"},
	{"якш.", "дүш.", "сиш.", "чәр.", "пәнҗ.", "җом.", "шим."},
	{"якшәмбе", "дүшәмбе", "сишәмбе", "чәршәмбе", "пәнҗешәмбе", "җомга", "шимбә"},
	{"УХ", "ПН", "ВТ", "СР", "ЧТ", "ПТ", "СБ"},
	{"У", "П", "В", "С", "Ч", "П", "С"},
	{"Улуг-хүн", "Понедельник", "Вторник", "Среда", "Четверг", "Пятница", "Суббота"},
	{"Asa", "Ayn", "Asn", "Akr", "Akw", "Asm", "Asḍ"},
	{"Asamas", "Aynas", "Asinas", "Akras", "Akwas", "Asimwas", "Asiḍyas"},
	{"يە", "دۈ", "سە", "چا", "پە", "جۈ", "شە"},
	{"ي", "د", "س", "چ", "پ", "ج", "ش"},
	{"يەكشەنبە", "دۈشەنبە", "سەيشەنبە", "چارشەنبە", "پەيشەنبە", "جۈمە", "شەنبە"},
	{"неділя", "понеділок", "вівторок", "середа", "четвер", "пʼятниця", "субота"},
	{"اتوار", "پیر", "منگل", "بدھ", "جمعرات", "جمعہ", "ہفتہ"},
	{"Yak", "Dush", "Sesh", "Chor", "Pay", "Jum", "Shan"},
	{"Ya", "Du", "Se", "Ch", "Pa", "Ju", "Sh"},
	{"yakshanba", "dushanba", "seshanba", "chorshanba", "payshanba", "juma", "shanba"},
	{"ی.", "د.", "س.", "چ.", "پ.", "ج.", "ش."},
	{"якш", "душ", "сеш", "чор", "пай", "жум", "шан"},
	{"як", "ду", "се", "чо", "па", "жу", "ша"},
	{"якшанба", "душанба", "сешанба", "чоршанба", "пайшанба", "жума", "шанба"},
	{"ꕞꕌꔵ", "ꗳꗡꘉ", "ꕚꕞꕚ", "ꕉꕞꕒ", "ꕉꔤꕆꕢ", "ꕉꔤꕀꕮ", "ꔻꔬꔳ"},
	{"lahadi", "tɛɛnɛɛ", "talata", "alaba", "aimisa", "aijima", "siɓiti"},
	{"Swo", "Mus", "Vhi", "Rar", "Ṋa", "Ṱan", "Mug"},
	{"Swondaha", "Musumbuluwo", "Ḽavhuvhili", "Ḽavhuraru", "Ḽavhuṋa", "Ḽavhuṱanu", "Mugivhela"},
	{"dom", "lun", "mar", "mer", "zob", "vèn", "sab"},
	{"doménega", "luni", "marti", "mèrcore", "zoba", "vènare", "sabo"},
	{"CN", "Thứ 2", "Thứ 3", "Thứ 4", "Thứ 5", "Thứ 6", "Thứ 7"},
	{"CN", "T2", "T3", "T4", "T5", "T6", "T7"},
	{"Chủ Nhật", "Thứ Hai", "Thứ Ba", "Thứ Tư", "Thứ Năm", "Thứ Sáu", "Thứ Bảy"},
	{"ettiminku", "nihiku noolempwa", "namaanli", "namararu", "namaxexe", "namathanu", "esaabadu"},
	{"su.", "mu.", "tu.", "ve.", "dö.", "fr.", "zä."},
	{"sudel", "mudel", "tudel", "vedel", "dödel", "fridel", "zädel"},
	{"Sun", "Män", "Ziš", "Mit", "Fró", "Fri", "Sam"},
	{"Sunntag", "Mäntag", "Zištag", "Mittwuč", "Fróntag", "Fritag", "Samštag"},
	{"ወጋ", "ሳይኖ", "ማቆሳኛ", "አሩዋ", "ሃሙሳ", "አርባ", "ቄራ"},
	{"Dib", "Alt", "Tal", "Àla", "Alx", "Àjj", "Ase"},
	{"Dibéer", "Altine", "Talaata", "Àlarba", "Alxamis", "Àjjuma", "Aseer"},
	{"Caw", "Mvu", "Lwesb", "Tha", "Sin", "Hla", "Mgq"},
	{"C", "Mv", "Sb", "Tht", "Sin", "Hl", "Mg"},
	{"Cawe", "Mvulo", "Lwesibini", "Lwesithathu", "Lwesine", "Lwesihlanu", "Mgqibelo"},
	{"तोआर", "सोआर", "मंगल", "बुध", "वीर", "शुक्कर", "शनि"},
	{"त", "सो", "मं", "बु", "वी", "शु", "श"},
	{"तोआर", "सोआर", "मंगलवार", "बुधवार", "वीरवार", "शुक्करवार", "शनिच्चरवार"},
	{"Sabi", "Bala", "Kubi", "Kusa", "Kuna", "Kuta", "Muka"},
	{"Sabiiti", "Balaza", "Owokubili", "Owokusatu", "Olokuna", "Olokutaanu", "Olomukaaga"},
	{"sd", "md", "mw", "et", "kl", "fl", "ss"},
	{"sɔ́ndiɛ", "móndie", "muányáŋmóndie", "metúkpíápɛ", "kúpélimetúkpiapɛ", "feléte", "séselé"},
	{"זונטיק", "מאָנטיק", "דינסטיק", "מיטוואך", "דאנערשטיק", "פֿרײַטיק", "שבת"},
	{"Àìkú", "Ajé", "Ìsẹ́gun", "Ọjọ́rú", "Ọjọ́bọ", "Ẹtì", "Àbámẹ́ta"},
	{"Ọjọ́ Àìkú", "Ọjọ́ Ajé", "Ọjọ́ Ìsẹ́gun", "Ọjọ́rú", "Ọjọ́bọ", "Ọjọ́ Ẹtì", "Ọjọ́ Àbámẹ́ta"},
	{"Àìkú", "Ajé", "Ìsɛ́gun", "Ɔjɔ́rú", "Ɔjɔ́bɔ", "Ɛtì", "Àbámɛ́ta"},
	{"Ɔjɔ́ Àìkú", "Ɔjɔ́ Ajé", "Ɔjɔ́ Ìsɛ́gun", "Ɔjɔ́rú", "Ɔjɔ́bɔ", "Ɔjɔ́ Ɛtì", "Ɔjɔ́ Àbámɛ́ta"},
	{"mit", "mur", "mmk", "mms", "sup", "yuk", "sau"},
	{"mituú", "murakipí", "murakí-mukũi", "murakí-musapíri", "supapá", "yukuakú", "saurú"},
	{"週日", "週一", "週二", "週三", "週四", "週五", "週六"},
	{"日", "一", "二", "三", "四", "五", "六"},
	{"星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六"},
	{"周日", "周一", "周二", "周三", "周四", "周五", "周六"},
	{"ngoenzsinghgiz", "singhgizit", "singhgizngeih", "singhgizsam", "singhgizseiq", "singhgizhaj", "singhgizroek"},
	{"ⴰⵙⴰⵎⴰⵙ", "ⴰⵢⵏⴰⵙ", "ⴰⵙⵉⵏⴰⵙ", "ⴰⴽⵕⴰⵙ", "ⴰⴽⵡⴰⵙ", "ⴰⵙⵉⵎⵡⴰⵙ", "ⴰⵙⵉⴹⵢⴰⵙ"},
	{"Son", "Mso", "Bil", "Tha", "Sin", "Hla", "Mgq"},
	{"ISonto", "UMsombuluko", "ULwesibili", "ULwesithathu", "ULwesine", "ULwesihlanu", "UMgqibelo"},
}

var _strings = [...]string{
	"",
	".",
	",",
	"-",
	"%",
	"‰",
	"+",
	"#,##0%",
	"EEEE, MMMM dd, y",
	"dd MMMM y",
	"dd-MMM-y",
	"dd/MM/yy",
	"h:mm:ss\u202fa zzzz",
	"h:mm:ss\u202fa z",
	"h:mm:ss\u202fa",
	"h:mm\u202fa",
	"¤#,##0.00",
	"\u00a0",
	"#,##0\u00a0%",
	"EEEE, d MMMM y 'ш'.",
	"d MMMM y 'ш'.",
	"d MMM y 'ш'.",
	"dd.MM.yy",
	"HH:mm:ss zzzz",
	"HH:mm:ss z",
	"HH:mm:ss",
	"HH:mm",
	"#,##0.00\u00a0¤",
	"EEEE dd MMMM y",
	"dd MMM y",
	"yy-MM-dd",
	"¤#,##0.00;(¤#,##0.00)",
	"EEEE d MMMM y",
	"d MMMM y",
	"d MMM y",
	"d MMM, y",
	"d/M/yy",
	"#,##0.00¤",
	"EEE, d, MMMM, y",
	"d, MMMM, y",
	"d, MMM, y",
	"h:mm:ss a zzzz",
	"h:mm:ss a z",
	"h:mm:ss a",
	"h:mm a",
	"EEEE, d MMMM 'de' y",
	"d MMMM 'de' y",
	"H:mm:ss zzzz",
	"H:mm:ss z",
	"H:mm:ss",
	"H:mm",
	"y MMMM d, EEEE",
	"y MMMM d",
	"y MMM d",
	"¤\u00a0#,##0.00",
	"\u200e-",
	"\u200e%\u200e",
	"\u200e+",
	"EEEE، d MMMM y",
	"dd\u200f/MM\u200f/y",
	"d\u200f/M\u200f/yy",
	"\u200f#,##0.00\u00a0¤;\u200f-#,##0.00\u00a0¤",
	"\u061c#,##0.00¤;(\u061c#,##0.00¤)",
	"٪",
	"#,##,##0%",
	"EEEE, d MMMM, y",
	"d MMMM, y",
	"dd-MM-y",
	"d-M-yy",
	"a h.mm.ss zzzz",
	"a h.mm.ss z",
	"a h.mm.ss",
	"a h.mm",
	"¤\u00a0#,##,##0.00",
	"EEEE, d MMMM y",
	"d MMMM y, EEEE",
	"d MMMM, EEEE, y 'й'.",
	"d MMMM, y 'й'.",
	"d MMM, y 'й'.",
	"hh:mm:ss a zzzz",
	"hh:mm:ss a zzz",
	"hh:mm:ss a",
	"hh:mm a",
	"dd,MM,y",
	"hh:mm:ss\u202fa zzzz",
	"hh:mm:ss\u202fa zzz",
	"hh:mm:ss\u202fa",
	"hh:mm\u202fa",
	"EEEE, d MMMM y\u202f'г'.",
	"d MMMM y\u202f'г'.",
	"d MMM y\u202f'г'.",
	"d.MM.yy",
	"HH:mm:ss, zzzz",
	"d.MM.y",
	"HH.mm.ss zzzz",
	"HH.mm.ss z",
	"HH.mm.ss",
	"HH.mm",
	"d.MM.y\u202f'г'.",
	"d.MM.yy\u202f'г'.",
	"H:mm:ss 'ч'. zzzz",
	"H:mm:ss 'ч'. z",
	"#,##0.00\u00a0¤;(#,##0.00\u00a0¤)",
	"٫",
	"،",
	"%\u00a0#,#0;%\u00a0-#,#0",
	"EEEE, MMMM d/y",
	"MMM d/y",
	"M/d/yy",
	"¤\u00a0#,##0.00;¤\u00a0-#,##0.00",
	"#,##,##0.00¤",
	"#,##,##0.00¤;(#,##,##0.00¤)",
	"¤#,##,##0.00",
	"¤#,##,##0.00;(¤#,##,##0.00)",
	"y MMMMའི་ཚེས་d, EEEE",
	"སྤྱི་ལོ་y MMMMའི་ཚེས་d",
	"y ལོའི་MMMཚེས་d",
	"\u200e−",
	"\u200e¤\u00a0#,##0.00",
	"\u200e¤\u00a0#,##0.00;\u200e(¤\u00a0#,##0.00)",
	"dd-MM-yy",
	"a h:mm:ss zzzz",
	"a h:mm:ss z",
	"a h:mm:ss",
	"a नि h:mm",
	"EEEE, d. MMMM y.",
	"d. MMMM y.",
	"d. MMM y.",
	"d. M. yy.",
	"EEEE, dd. MMMM y.",
	"dd. MMMM y.",
	"dd.MM.y.",
	"d.M.yy.",
	"EEEE፡ dd MMMM ግርጋ y G",
	"EEEE, d MMMM 'del' y",
	"d MMMM 'del' y",
	"H:mm:ss (zzzz)",
	"EEEE, MMMM d, y",
	"MMMM d, y",
	"MMM d, y",
	"EEEE, y MMMM dd",
	"yy/MM/dd",
	"dی MMMMی y",
	"EEEE d MMMM 'di' 'u' y",
	"d MMMM 'di' 'u' y",
	"#,##0.00\u00a0¤;(#,##0.00)\u00a0¤",
	"EEEE d. MMMM y",
	"d. MMMM y",
	"d. M. y",
	"H:mm:ss, zzzz",
	"EEEE, d MMMM 'л'. y.",
	"yy.MM.dd",
	"y, MMMM, d, EEEE",
	"y, MMMM, d",
	"y, MMM, d",
	"HH:mm:ss (zzzz)",
	"HH:mm:ss (z)",
	"EEEE 'den' d. MMMM y",
	"d. MMM y",
	"EEEE, d. MMMM y",
	"dd.MM.y",
	"'",
	"¤\u00a0#,##0.00;¤-#,##0.00",
	"EEEE, d, MMMM y",
	"d, MMMM y",
	"d, MMM y",
	"d.M.y",
	"d.M.yy",
	"#,##,##0\u00a0%",
	"EEEE, སྤྱི་ལོ་y MMMM ཚེས་dd",
	"སྤྱི་ལོ་y MMMM ཚེས་ dd",
	"སྤྱི་ལོ་y ཟླ་MMM ཚེས་dd",
	"ཆུ་ཚོད་ h སྐར་མ་ mm:ss a zzzz",
	"ཆུ་ཚོད་ h སྐར་མ་ mm:ss a z",
	"ཆུ་ཚོད་h:mm:ss a",
	"ཆུ་ཚོད་ h སྐར་མ་ mm a",
	"EEEE, MMMM d 'lia' y",
	"MMMM d 'lia' y",
	"MMM d 'lia', y",
	"a 'ga' h:mm:ss zzzz",
	"a 'ga' h:mm:ss z",
	"a 'ga' h:mm:ss",
	"a 'ga' h:mm",
	"EEEE, dd MMMM y",
	"H.mm.ss zzzz",
	"H.mm.ss z",
	"H.mm.ss",
	"H.mm",
	"\u202f",
	"¤\u00a0#,##0.00;(¤\u00a0#,##0.00)",
	"0/00",
	"0%",
	"¤\u00a00.00",
	"dd MMM,y",
	"EEEE, 'la' d-'a' 'de' MMMM y",
	"y-MMMM-dd",
	"y-MMM-dd",
	"#,##0.00\u202f¤",
	"#,##0.00\u202f¤;(#,##0.00\u202f¤)",
	"EEEE, d 'de' MMMM 'de' y",
	"d 'de' MMMM 'de' y",
	"d MMM 'de' y",
	"¤#,##0.00;¤-#,##0.00",
	"d/MM/y",
	"d/MM/yy",
	"EEEE dd 'de' MMMM 'de' y",
	"dd 'de' MMMM 'de' y",
	"MM/dd/y",
	"MM/dd/yy",
	"−",
	"%\u00a0#,##0",
	"y('e')'ko' MMMM'ren' d('a'), EEEE",
	"y('e')'ko' MMMM'ren' d('a')",
	"y('e')'ko' MMM d('a')",
	"yy/M/d",
	"H:mm:ss (z)",
	"¤\u00a0#,##0.00;\u200e(¤\u00a0#,##0.00)",
	"⹁",
	"EEEE d MMMM⹁ y",
	"d MMMM⹁ y",
	"d MMM⹁ y",
	"cccc d. MMMM y",
	"H 'h' mm 'min' ss 's' zzzz",
	"HH 'h' mm 'min' ss 's' zzzz",
	"HH 'h' mm 'min' ss 's' z",
	"HH 'h' mm 'min' ss 's'",
	"HH 'h' mm",
	"HH.mm:ss 'h' zzzz",
	"EEEE d 'di' MMMM 'dal' y",
	"d 'di' MMMM 'dal' y",
	"dd/MM/y",
	"¤\u00a0#,##0.00;¤\u00a0#,##0.00-",
	"EEEE, d'mh' MMMM y",
	"d'mh' MMMM y",
	"d'mh' MMM y",
	"ወ",
	"EEEE፥ dd MMMM መዓልት y G",
	"d 'de' MMM 'de' y",
	"hh:mm:ss a z",
	"MMM dd, y",
	"EEEE d MMMM, y",
	"EEEE, d בMMMM y",
	"d בMMMM y",
	"d בMMM y",
	"\u200f#,##0.00\u00a0\u200f¤;\u200f-#,##0.00\u00a0\u200f¤",
	"dd MMM, y",
	"dd. MM. yy.",
	"H:mm 'hodź'.",
	"y. MMMM d., EEEE",
	"y. MMMM d.",
	"y. MMM d.",
	"yy. MM. dd.",
	"y թ. MMMM d, EEEE",
	"dd MMMM, y թ.",
	"dd MMM, y թ.",
	"EEEE 'le' d 'de' MMMM y",
	"d 'de' MMMM y",
	"y年M月d日EEEE",
	"y年M月d日",
	"y/MM/dd",
	"H時mm分ss秒 zzzz",
	"EEEE, dd MMMM, y",
	"d MMM. y",
	"EEEE, d 'di' MMMM 'di' y",
	"d 'di' MMMM 'di' y",
	"EEEE, d 'ne' MMMM, y",
	"d 'ne' MMMM, y",
	"d 'ne' MMM, y",
	"y\u202f'ж'. d MMMM, EEEE",
	"y\u202f'ж'. d MMMM",
	"y\u202f'ж'. dd MMM",
	"y، d- MMMM، EEEE",
	"y، d- MMMM",
	"y، d- MMM",
	"dd/MM yy",
	"#,##0.00¤;(#,##0.00¤)",
	"y년 MMMM d일 EEEE",
	"y년 MMMM d일",
	"y. M. d.",
	"yy. M. d.",
	"a h시 m분 s초 zzzz",
	"a h시 m분 s초 z",
	"a h:mm",
	"zzzz HH:mm:ss",
	"z HH:mm:ss",
	"d-MMM-y",
	"a\u202fh:mm:ss\u202fzzzz",
	"a\u202fh:mm:ss z",
	"a\u202fh:mm:ss",
	"a\u202fh:mm",
	"EEEE, 'dä' d. MMMM y",
	"d. MMM. y",
	"d. M. yy",
	"%#,##0",
	"EEEE, d'ê' MMMM'a' y'an'",
	"d'ê' MMMM'a' y'an'",
	"y-'ж'., d-MMMM, EEEE",
	"y-'ж'., d-MMMM",
	"y-'ж'., d-MMM",
	"EEEE, 'die' d MMMM y G",
	"'die' d MMMM y G",
	"'die' d MMM y G",
	"d M yy G",
	"EEEE d MMMM 'do' y",
	"d MMMM 'do' y",
	"d MMM 'do' y",
	"EEEE, d MMMM 'dl' y",
	"H ໂມງ m ນາທີ ss ວິນາທີ zzzz",
	"H ໂມງ m ນາທີ ss ວິນາທີ z",
	"y 'm'. MMMM d 'd'., EEEE",
	"y 'm'. MMMM d 'd'.",
	"y-MM-dd",
	"¤#,##0.00;¤-\u00a0#,##0.00",
	"EEEE, y. 'gada' d. MMMM",
	"y. 'gada' d. MMMM",
	"y. 'gada' d. MMM",
	"d.M.yy\u202f'г'.",
	"zzzz h:mm:ss a",
	"z h:mm:ss a",
	"y\u202f'оны' MMMM'ын' d, EEEE 'гараг'",
	"y\u202f'оны' MMMM'ын' d",
	"y\u202f'оны' MMM'ын' d",
	"y ᠣᠨ \u180e\u180e\u180eᠤ MMMM\u180e\u180eᠢᠢᠨd. EEEE ᠋ᠭᠠᠷᠠᠭ",
	"y ᠋ᠣᠨ ᠤMMMM\u180e\u180e ᠤᠩ d",
	"y.MM.dd",
	"MMMM d, y, EEEE",
	"h.mm.ss a zzzz",
	"h.mm.ss a z",
	"h.mm.ss a",
	"h.mm. a",
	"EEEE, d 'ta'’ MMMM y",
	"d 'ta'’ MMMM y",
	"y MMMM d EEEE",
	"#,##0.00\u00a0¤;-#,##0.00\u00a0¤",
	"EEEE, 'de' d. MMMM y",
	"'Klock' H.mm:ss (zzzz)",
	"'Klock' H.mm:ss (z)",
	"'Klock' H.mm:ss",
	"'Kl'. H.mm",
	"EEEE , 'lyɛ'̌ʼ d 'na' MMMM, y",
	"'lyɛ'̌ʼ d 'na' MMMM, y",
	"zzzz h:mm:ss\u202fa",
	"z h:mm:ss\u202fa",
	"#,##0\u202f%",
	"EEEE d MMMM 'de' y",
	"H'h'mm",
	"EEEE, d MMMM, y\u202f'аз'",
	"d MMMM, y\u202f'аз'",
	"dd MMM y\u202f'аз'",
	"EEEE, y 'mettas' d. MMMM",
	"y 'mettas' d. MMMM",
	"dd.MM 'st'. y",
	"EEEE د y د MMMM d",
	"EEEE, 'ils' d MMMM y",
	"y\u202f'сыл' MMMM d 'күнэ', EEEE",
	"y, MMMM d",
	"y, MMM d",
	"EEEE d 'de' MMMM 'de' 'su' y",
	"d 'de' MMMM 'de' 'su' y",
	"d 'de' MMM y",
	"y MMMM d - EEEE",
	"cccc, MMMM d. y",
	"MMMM d. y",
	"MMM d. y",
	"h:mm:ss\u202fa, zzzz",
	"h:mm:ss\u202fa, z",
	"d. M. y.",
	"EEEE، d ܒMMMM y",
	"d ܒMMMM y",
	"d ܒMMM y",
	"d-MM-yy",
	"d, MMMM y, EEEE",
	"EEEEที่ d MMMM G y",
	"d MMMM G y",
	"H นาฬิกา mm นาที ss วินาที zzzz",
	"H นาฬิกา mm นาที ss วินาที z",
	"EEEE፣ d MMMM y",
	"EEEE፡ dd MMMM ዮም y G",
	"d MMMM y EEEE",
	"'sike' #y 'la' MMM 'la' 'suno' #d",
	"'tenpo' 'sike' #y 'la' 'tenpo' MMMM 'la' 'tenpo' 'suno' #d",
	"zzzz 'la' HH:mm:ss",
	"z 'la' HH:mm:ss",
	"¤#,#0.00",
	"EEE, dd MMMM y",
	"EEEE، d MMMM، y",
	"d MMMM، y",
	"d MMM، y",
	"d MMMM, y\u202f'ел', EEEE",
	"d MMMM, y\u202f'ел'",
	"d MMM, y\u202f'ел'",
	"y'ч' MMMM d, EEEE",
	"y'ч' MMMM d",
	"y'ч' MMM d",
	"y d-MMMM، EEEE",
	"d-MMMM، y",
	"d-MMM، y",
	"EEEE, d MMMM y\u202f'р'.",
	"d MMMM y\u202f'р'.",
	"d MMM y\u202f'р'.",
	"EEEE, d-MMMM, y",
	"d-MMMM, y",
	"d-MMM, y",
	"y نچی ییل d نچی MMMM EEEE کونی",
	"d نچی MMMM y",
	"y MMMM'a' 'd'. d'id'",
	"y MMM. d",
	"EEEE፥ dd MMMM ጋላሳ y G",
	"EEEE, d MMM, y",
	"EEEE, dטן MMMM y",
	"dטן MMMM y",
	"dטן MMM y",
	"EEEE, d MMM y",
	"d MM y",
	"H:m:s",
	"H:m",
	"y年M月d日 EEEE",
	"ah:mm:ss [zzzz]",
	"ah:mm:ss [z]",
	"ah:mm:ss",
	"ah:mm",
	"HH:mm:ss [zzzz]",
	"HH:mm:ss [z]",
	"Bh:mm:ss [zzzz]",
	"Bh:mm:ss [z]",
	"Bh:mm:ss",
	"Bh:mm",
	"zzzz ah:mm:ss",
	"z ah:mm:ss",
}

// Sorted slices for binary search lookups (no map allocation at init).

var _currencyDefaults = [...]kv{
	{"AFN", "؋"},
	{"AMD", "֏"},
	{"AOA", "Kz"},
	{"ARS", "$"},
	{"AUD", "A$"},
	{"AZN", "₼"},
	{"BAM", "KM"},
	{"BBD", "$"},
	{"BDT", "৳"},
	{"BMD", "$"},
	{"BND", "$"},
	{"BOB", "Bs"},
	{"BRL", "R$"},
	{"BSD", "$"},
	{"BWP", "P"},
	{"BZD", "$"},
	{"CAD", "CA$"},
	{"CLP", "$"},
	{"CNY", "CN¥"},
	{"COP", "$"},
	{"CRC", "₡"},
	{"CUC", "$"},
	{"CUP", "$"},
	{"CZK", "Kč"},
	{"DKK", "kr"},
	{"DOP", "$"},
	{"EGP", "E£"},
	{"ESP", "₧"},
	{"EUR", "€"},
	{"FJD", "$"},
	{"FKP", "£"},
	{"GBP", "£"},
	{"GEL", "₾"},
	{"GHS", "GH₵"},
	{"GIP", "£"},
	{"GNF", "FG"},
	{"GTQ", "Q"},
	{"GYD", "$"},
	{"HKD", "HK$"},
	{"HNL", "L"},
	{"HRK", "kn"},
	{"HUF", "Ft"},
	{"IDR", "Rp"},
	{"ILS", "₪"},
	{"INR", "₹"},
	{"ISK", "kr"},
	{"JMD", "$"},
	{"JPY", "JP¥"},
	{"KGS", "⃀"},
	{"KHR", "៛"},
	{"KMF", "CF"},
	{"KPW", "₩"},
	{"KRW", "₩"},
	{"KYD", "$"},
	{"KZT", "₸"},
	{"LAK", "₭"},
	{"LBP", "L£"},
	{"LKR", "Rs"},
	{"LRD", "$"},
	{"LTL", "Lt"},
	{"LVL", "Ls"},
	{"MGA", "Ar"},
	{"MMK", "K"},
	{"MNT", "₮"},
	{"MUR", "Rs"},
	{"MXN", "MX$"},
	{"MYR", "RM"},
	{"NAD", "$"},
	{"NGN", "₦"},
	{"NIO", "C$"},
	{"NOK", "kr"},
	{"NPR", "Rs"},
	{"NZD", "NZ$"},
	{"PHP", "₱"},
	{"PKR", "Rs"},
	{"PLN", "zł"},
	{"PYG", "₲"},
	{"RON", "lei"},
	{"RUB", "₽"},
	{"RWF", "RF"},
	{"SAR", "\u20c1"},
	{"SBD", "$"},
	{"SEK", "kr"},
	{"SGD", "$"},
	{"SHP", "£"},
	{"SRD", "$"},
	{"SSP", "£"},
	{"STN", "Db"},
	{"SYP", "£"},
	{"THB", "฿"},
	{"TOP", "T$"},
	{"TRY", "₺"},
	{"TTD", "$"},
	{"TWD", "NT$"},
	{"UAH", "₴"},
	{"USD", "US$"},
	{"UYU", "$"},
	{"VEF", "Bs"},
	{"VND", "₫"},
	{"XAF", "FCFA"},
	{"XCD", "EC$"},
	{"XCG", "Cg."},
	{"XOF", "F\u202fCFA"},
	{"XPF", "CFPF"},
	{"XXX", "¤"},
	{"ZAR", "R"},
	{"ZMW", "ZK"},
}

var _currencyOverrides = [...][]kv{
	nil,
	{{"ETB", "Br"}},
	{{"DJF", "Fdj"}, {"ETB", "Br"}},
	{{"ERN", "Nfk"}, {"ETB", "Br"}},
	{{"GEL", "ლ"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"RON", "L"}, {"USD", "$"}},
	{{"BYN", "р."}, {"CAD", "CAD"}, {"MXN", "MXN"}, {"PHP", "PHP"}, {"RON", "leu"}, {"USD", "USD"}},
	{{"IQD", "Irak dinaa"}, {"XOF", "AAS"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"ETB", "ብር"}, {"PHP", "PHP"}},
	{{"EUR", "r"}, {"JPY", "¥"}},
	{{"JOD", "د.أ.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DJF", "Fdj"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"ERN", "Nfk"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "SDG"}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "UK£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SOS", "S"}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"AED", "د.إ.\u200f"}, {"ARS", "AR$"}, {"AUD", "AU$"}, {"BBD", "BB$"}, {"BHD", "د.ب.\u200f"}, {"BMD", "BM$"}, {"BND", "BN$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CLP", "CL$"}, {"COP", "CO$"}, {"CUP", "CU$"}, {"DOP", "DO$"}, {"DZD", "د.ج.\u200f"}, {"EGP", "ج.م.\u200f"}, {"FJD", "FJ$"}, {"GBP", "GB£"}, {"GYD", "GY$"}, {"IQD", "د.ع.\u200f"}, {"IRR", "ر.إ."}, {"JMD", "JM$"}, {"JOD", "د.أ.\u200f"}, {"KWD", "د.ك.\u200f"}, {"KYD", "KY$"}, {"LBP", "ل.ل.\u200f"}, {"LRD", "$LR"}, {"LYD", "د.ل.\u200f"}, {"MAD", "د.م.\u200f"}, {"MRU", "أ.م."}, {"OMR", "ر.ع.\u200f"}, {"PHP", "PHP"}, {"QAR", "ر.ق.\u200f"}, {"SAR", "ر.س.\u200f"}, {"SBD", "SB$"}, {"SDD", "د.س.\u200f"}, {"SDG", "ج.س."}, {"SRD", "SR$"}, {"SYP", "ل.س.\u200f"}, {"TND", "د.ت.\u200f"}, {"TTD", "TT$"}, {"UYU", "UY$"}, {"YER", "ر.ي.\u200f"}},
	{{"BYN", "р."}, {"PHP", "PHP"}},
	{{"TZS", "TSh"}},
	{{"DKK", "DKK"}, {"HRK", "HRK"}, {"ISK", "ISK"}, {"JPY", "¥"}, {"NOK", "NOK"}, {"PHP", "PHP"}, {"PLN", "PLN"}, {"SEK", "SEK"}, {"USD", "$"}, {"XXX", "XXX"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"RON", "ley"}, {"SYP", "S£"}},
	{{"RUR", "р."}, {"USD", "$"}, {"XOF", "F CFA"}},
	{{"IRR", "ریال"}, {"JPY", "¥"}, {"USD", "$"}, {"XXX", "XXX"}},
	{{"AFN", "AWGA"}, {"ALL", "ALBL"}, {"AMD", "ARMD"}, {"ANG", "NLAG"}, {"AOA", "ANGK"}, {"ARS", "ARJP"}, {"AWG", "ARBF"}, {"AZN", "AZRM"}, {"BAM", "BHBM"}, {"BBD", "BRBD"}, {"BDT", "BGDT"}, {"BGN", "BLGL"}, {"BIF", "BRDF"}, {"BMD", "BRMD"}, {"BND", "BRND"}, {"BOB", "BLWB"}, {"BSD", "BHMD"}, {"BTN", "BTNN"}, {"BWP", "BTSP"}, {"BYN", "BLRR"}, {"BZD", "BLZD"}, {"CAD", "KN$"}, {"CDF", "KNGF"}, {"CHF", "SWZF"}, {"CNH", "CNYÁ"}, {"CNY", "CHNY"}, {"COP", "KLBP"}, {"CRC", "KRK"}, {"CUC", "KBKP"}, {"CUP", "CBAP"}, {"CVE", "KWRE"}, {"CZK", "CHKK"}, {"DJF", "DJBF"}, {"DKK", "DNMK"}, {"DOP", "DOMP"}, {"DZD", "ALJD"}, {"EGP", "MSRP"}, {"ERN", "ERTN"}, {"ETB", "ETPB"}, {"GBP", "BRTP"}, {"GEL", "JRJL"}, {"GHS", "GNAS"}, {"GIP", "GBRP"}, {"GMD", "GMBD"}, {"GNF", "GWNF"}, {"GTQ", "GTMK"}, {"HKD", "HGKD"}, {"HNL", "HNDL"}, {"HRK", "KRSK"}, {"HTG", "HTNG"}, {"HUF", "HGRF"}, {"IDR", "ENDR"}, {"INR", "HNDR"}, {"IRR", "ERNR"}, {"ISK", "ISLK"}, {"JMD", "JMKD"}, {"JPY", "¥"}, {"KES", "KINS"}, {"KGS", "KGSS"}, {"KHR", "KMBR"}, {"KMF", "KMRF"}, {"KPW", "SHKW"}, {"KRW", "ZBKW"}, {"KYD", "KMID"}, {"KZT", "KZKT"}, {"LAK", "LTKK"}, {"LBP", "LBNP"}, {"LKR", "SRLR"}, {"LRD", "LBRD"}, {"LSL", "LSTL"}, {"LYD", "LBYD"}, {"MAD", "MRKD"}, {"MDL", "MLDL"}, {"MGA", "MLGA"}, {"MKD", "MKDD"}, {"MMK", "MNMK"}, {"MNT", "MNGT"}, {"MOP", "MKNP"}, {"MRU", "MRTU"}, {"MUR", "MURR"}, {"MVR", "MLDR"}, {"MWK", "MLWK"}, {"MXN", "MKS$"}, {"MYR", "MLRG"}, {"MZN", "MZBM"}, {"NAD", "NMBD"}, {"NGN", "NJRN"}, {"NIO", "NKGC"}, {"NOK", "NRWK"}, {"NPR", "NPLR"}, {"PAB", "PNMB"}, {"PEN", "PRSL"}, {"PHP", "PLPP"}, {"PKR", "PKRS"}, {"PLN", "PLNZ"}, {"PYG", "PRGG"}, {"RON", "RMNL"}, {"RSD", "SRBD"}, {"RUB", "RUSR"}, {"RWF", "RWDF"}, {"SCR", "SCLR"}, {"SDG", "SDNP"}, {"SEK", "SWDK"}, {"SGD", "SGPD"}, {"SHP", "SHLP"}, {"SLE", "SLNL"}, {"SLL", "SRLL"}, {"SOS", "SÓMS"}, {"SRD", "SRND"}, {"SSP", "ZRSP"}, {"STN", "STPD"}, {"SYP", "SURP"}, {"SZL", "SWZL"}, {"THB", "TÁIB"}, {"TND", "TNSD"}, {"TRY", "TRKL"}, {"TTD", "TDTD"}, {"TWD", "NTWD"}, {"TZS", "TNZS"}, {"UAH", "YKNH"}, {"UGX", "YUGS"}, {"USD", "$"}, {"UYU", "YRGP"}, {"UZS", "OZBS"}, {"VES", "WNZB"}, {"VND", "WTND"}, {"XAF", "DARF"}, {"XCD", "RKB$"}, {"XOF", "RACF"}, {"XXX", "XXX"}, {"ZAR", "ZAPR"}, {"ZMW", "ZMBK"}},
	{{"BBD", "Bds$"}, {"BMD", "BD$"}, {"BRL", "BRL"}, {"BSD", "B$"}, {"BYN", "Br"}, {"BZD", "BZ$"}, {"CAD", "CAD"}, {"CUC", "CUC$"}, {"CUP", "$MN"}, {"DOP", "RD$"}, {"FJD", "FJ$"}, {"FKP", "FK£"}, {"GYD", "G$"}, {"ISK", "Íkr"}, {"JMD", "J$"}, {"JPY", "¥"}, {"KYD", "CI$"}, {"LRD", "L$"}, {"NAD", "N$"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"SBD", "SI$"}, {"SGD", "S$"}, {"TTD", "TT$"}, {"USD", "$"}, {"UYU", "$U"}},
	{{"BBD", "Bds$"}, {"BMD", "BD$"}, {"BRL", "BRL"}, {"BSD", "B$"}, {"BYN", "Br"}, {"BZD", "BZ$"}, {"CAD", "CAD"}, {"CUC", "CUC$"}, {"CUP", "$MN"}, {"DOP", "RD$"}, {"FJD", "FJ$"}, {"FKP", "FK£"}, {"GYD", "G$"}, {"ISK", "Íkr"}, {"JMD", "J$"}, {"JPY", "¥"}, {"KYD", "CI$"}, {"LRD", "L$"}, {"NAD", "N$"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"SBD", "SI$"}, {"SGD", "S$"}, {"TTD", "TT$"}, {"USD", "$"}, {"UYU", "$U"}, {"XOF", "CFA"}},
	{{"ZMW", "K"}},
	{{"AFN", "Af"}, {"AMD", "AMD"}, {"ARS", "ARS"}, {"AUD", "AUD"}, {"AZN", "AZN"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "лв."}, {"BMD", "BMD"}, {"BND", "BND"}, {"BRL", "BRL"}, {"BSD", "BSD"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CLP", "CLP"}, {"CNY", "CNY"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUP", "CUP"}, {"DOP", "DOP"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GBP"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GYD", "GYD"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JMD", "JMD"}, {"JPY", "JPY"}, {"KHR", "KHR"}, {"KRW", "KRW"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LRD", "LRD"}, {"MNT", "MNT"}, {"MXN", "MXN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PYG", "PYG"}, {"RON", "RON"}, {"SBD", "SBD"}, {"SGD", "SGD"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"UAH", "UAH"}, {"USD", "щ.д."}, {"UYU", "UYU"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"IRR", "ریال"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"CNY", "¥"}},
	{{"AUD", "$A"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "$CA"}, {"CNY", "CNY"}, {"EGP", "£ E"}, {"GBP", "£ RU"}, {"HKD", "$ HK"}, {"ILS", "ILS"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"LBP", "£L"}, {"NZD", "$ ZN"}, {"PHP", "PHP"}, {"RUR", "р."}, {"TOP", "$ T"}, {"TWD", "TWD"}, {"USD", "$ SU"}, {"VND", "VND"}, {"XCD", "XCD"}, {"XXX", "XXX"}},
	{{"AED", "ए.इ.दि"}, {"ALL", "अल"}, {"AMD", "ए.एम.दि"}, {"ANG", "ए.एन.जि"}, {"AOA", "ए.अ.ए"}, {"ARS", "ए.आर.एस"}, {"AWG", "ए.दब्ल्यु.जि"}, {"AZN", "ए.जेत.एन"}, {"BAM", "बि.ए.एम"}, {"BBD", "बि.बि.दि"}, {"BGN", "बि.जि.एन"}, {"BHD", "बि.ऐत्स.दि"}, {"BIF", "बि.आइ.एफ"}, {"BMD", "बि.एम.डि"}, {"BOB", "बि.अ.बि"}, {"BSD", "बि.एस.दि"}, {"BWP", "बि.दब्ल्यु.पि"}, {"BYN", "बि.वाई.एन"}, {"BZD", "बि.जेद.डि"}, {"CAD", "सि.ए $"}, {"CDF", "सि.दि.एफ"}, {"CHF", "सि.ऐत्स.एफ"}, {"CLP", "सि.एल.पि"}, {"CNH", "सि.एन.ऐत्स"}, {"CNY", "सिएन¥"}, {"COP", "सि.अ.पि"}, {"CRC", "सि.आर.सि"}, {"CUC", "सि.इउ.सि"}, {"CUP", "सि.इउ.पि"}, {"CVE", "सि.भि.इ"}, {"CZK", "सि.जेद.के"}, {"DJF", "दि.जे.एफ"}, {"DKK", "दि.के.के"}, {"DOP", "डि.अ.पि"}, {"DZD", "दि.जेत.दि"}, {"EGP", "ई.जि.पि"}, {"ERN", "इ.आर.एन"}, {"ETB", "इ.ति.बि"}, {"FKP", "एफ.के.पि"}, {"GEL", "जि.इ.एल"}, {"GHS", "जि.ऐत्स.एस"}, {"GIP", "जि.आइ.पि"}, {"GMD", "जि.एम.दि"}, {"GNF", "जि.एन.एफ"}, {"GTQ", "जि.ति.किउ"}, {"GYD", "जि.वाई.दि"}, {"HKD", "ऐत्स.के$"}, {"HNL", "ऐत्स.एन.एल"}, {"HRK", "ऐत्स.आर.के"}, {"HTG", "ऐत्स.ति.जि"}, {"HUF", "ऐत्स.इउ.एफ"}, {"IQD", "आइ.किउ.दि"}, {"ISK", "आई.एस.के"}, {"JMD", "जे.एम.दि"}, {"JOD", "जे.अ.दि"}, {"KES", "के.इ.एस"}, {"KGS", "के.जि.एस"}, {"KMF", "के.एम.एफ"}, {"KPW", "के.पि.दब्ल्यु"}, {"KWD", "के.दब्ल्यु.दि"}, {"KYD", "के.वाई.दि"}, {"KZT", "के.जेत.ति"}, {"LBP", "एल.बि.पि"}, {"LRD", "एल.आर.दि"}, {"LSL", "एल.एस.एल"}, {"LYD", "एल.वाई.दि"}, {"MAD", "एम.ए.दि"}, {"MDL", "एम.डि.एल"}, {"MGA", "एम.जि.ए"}, {"MKD", "एम.के.दि"}, {"MNT", "एम.एन.ति"}, {"MOP", "एम.अ.पि"}, {"MRU", "एम.आर.इउ"}, {"MUR", "एम.इउ.आर"}, {"MWK", "एम.दब्ल्यु.के"}, {"MXN", "एम.एक्स $"}, {"MZN", "एम.जेत.एन"}, {"NAD", "एन.ए.दि"}, {"NGN", "एन.जि.एन"}, {"NIO", "एन.आई.अ"}, {"NOK", "एन.अ.के"}, {"OMR", "अ.एम.आर"}, {"PAB", "पि.ए.बि"}, {"PEN", "पि.इ.एन"}, {"PLN", "पि.एल.एन"}, {"PYG", "पि.आई.जि"}, {"QAR", "किउ.ए.आर"}, {"RON", "आर.अ.एन"}, {"RSD", "आर.एस.दि"}, {"RUB", "रूब"}, {"RWF", "आर.एफ"}, {"SAR", "एस.ए.आर"}, {"SCR", "एस.सि.आर"}, {"SDG", "एस.दि.जि"}, {"SEK", "एस.इ.के"}, {"SHP", "एस.ऐत्स.पि"}, {"SLE", "एस.एल.इ"}, {"SLL", "एस.एल.एल"}, {"SOS", "एस.अ.एस"}, {"SRD", "एस.आर.डि"}, {"SSP", "एस.एस.पि"}, {"STN", "एस.ति.एन"}, {"SYP", "एस.वाई.पि"}, {"SZL", "एस.जेत.एल"}, {"TJS", "ति.जे.एस"}, {"TMT", "ति.एम.ति"}, {"TND", "ति.एन.दि"}, {"TRY", "ति.आर.वाई"}, {"TTD", "ति.ति.डि"}, {"TWD", "एन.ति$"}, {"TZS", "ति.जेत.एस"}, {"UAH", "इउ.ए.ऐत्स"}, {"UGX", "इउ.जि.एक्स"}, {"USD", "$"}, {"UYU", "इउ.वाई.इउ"}, {"UZS", "इउ.जेत.एस"}, {"VES", "भि.इ.एस"}, {"XCD", "इ.सि $"}, {"YER", "वाई.इ.आर"}, {"ZAR", "जेत.ए.आर"}, {"ZMW", "जेत.के"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"JPY", "¥"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RSD", "din."}, {"USD", "USD"}, {"XCD", "XCD"}, {"XPF", "XPF"}},
	{{"BAM", "КМ"}, {"CZK", "Кч"}, {"JPY", "¥"}, {"PLN", "зл"}, {"RSD", "дин."}, {"TRY", "Тл"}},
	{{"ERN", "Nfk"}},
	{{"AUD", "AU$"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"JPY", "¥"}, {"MXN", "MXN"}, {"PHP", "PHP"}, {"USD", "USD"}, {"VEF", "Bs F"}, {"XCD", "XCD"}, {"XCG", "XCG"}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"FRF", "F"}, {"JPY", "¥"}, {"MXN", "MXN"}, {"PHP", "PHP"}, {"USD", "USD"}, {"VEF", "Bs F"}, {"XCD", "XCD"}, {"XCG", "XCG"}, {"XXX", "XXX"}},
	{{"USD", "$"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"STD", "Db"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"RON", "лей"}},
	{{"USD", "US $"}},
	{{"UGX", "USh"}},
	{{"IQD", "د.ع.\u200f"}},
	{{"EUR", "EUR"}, {"GBP", "£GB"}, {"INR", "INR"}, {"USD", "$US"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CSK", "Kčs"}, {"ILS", "ILS"}, {"INR", "INR"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"VND", "VND"}, {"XEU", "ECU"}, {"XXX", "XXX"}},
	{{"BYN", "р."}, {"USD", "$"}},
	{{"BDT", "TK"}, {"BWP", "BWP"}, {"BYN", "р."}, {"KRW", "KRW"}, {"PHP", "PHP"}, {"XXX", "XXX"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}},
	{{"AUD", "AU$"}, {"BYN", "Br."}, {"DKK", "kr."}, {"ISK", "kr."}, {"NOK", "kr."}, {"PHP", "PHP"}, {"RON", "L"}, {"SEK", "kr."}},
	{{"KES", "Ksh"}},
	{{"ATS", "öS"}, {"AUD", "AU$"}, {"BGM", "BGK"}, {"BGO", "BGJ"}, {"BYN", "р."}, {"CUC", "Cub$"}, {"DEM", "DM"}, {"FKP", "Fl£"}, {"GHS", "₵"}, {"GNF", "F.G."}, {"JPY", "¥"}, {"KMF", "FC"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "F.Rw"}, {"SYP", "SYP"}, {"USD", "$"}, {"XXX", "XXX"}, {"ZMW", "K"}},
	{{"ATS", "öS"}, {"AUD", "AU$"}, {"BGM", "BGK"}, {"BGO", "BGJ"}, {"BYN", "р."}, {"CUC", "Cub$"}, {"DEM", "DM"}, {"EUR", "EUR"}, {"FKP", "Fl£"}, {"GHS", "₵"}, {"GNF", "F.G."}, {"JPY", "¥"}, {"KMF", "FC"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "F.Rw"}, {"SYP", "SYP"}, {"USD", "$"}, {"XXX", "XXX"}, {"ZMW", "K"}},
	{{"ATS", "öS"}, {"AUD", "AU$"}, {"BGM", "BGK"}, {"BGO", "BGJ"}, {"BYN", "р."}, {"CUC", "Cub$"}, {"DEM", "DM"}, {"FKP", "Fl£"}, {"GHS", "₵"}, {"GNF", "F.G."}, {"JPY", "¥"}, {"KMF", "FC"}, {"LUF", "F"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "F.Rw"}, {"SYP", "SYP"}, {"USD", "$"}, {"XXX", "XXX"}, {"ZMW", "K"}},
	{{"JPY", "¥"}, {"USD", "$"}},
	{{"AUD", "AUD"}, {"JPY", "¥"}, {"USD", "$"}},
	{{"MVR", "ރ."}},
	{{"AUD", "AU$"}, {"BTN", "Nu."}, {"ILS", "ILS"}, {"KRW", "KR₩"}, {"THB", "TH฿"}, {"XAF", "XAF"}},
	{{"AUD", "AU$"}, {"BYN", "р."}},
	{{"BYN", "р."}, {"GRD", "Δρχ"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"JPY", "¥"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"USD", "$"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"XCD", "$"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AUD", "$"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "Tk"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BRL", "BRL"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"CNY", "CNY"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "₱"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "£"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"EUR", "EUR"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GBP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HKD", "HKD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"ILS", "ILS"}, {"INR", "INR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "Kr"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"JPY", "JPY"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KRW", "KRW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MXN", "MXN"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"NZD", "NZD"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PHP", "PHP"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "Gs"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "Rs"}, {"SDG", "SDG"}, {"SEK", "Kr"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"USD", "USD"}, {"UYU", "$U"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VND", "VND"}, {"VUV", "VUV"}, {"WST", "WST"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}, {"XPF", "CFP"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "FBu"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"JPY", "¥"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"USD", "$"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CAD", "$"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"XCG", "Cg"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AUD", "$"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"NZD", "$"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "kr."}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "Nfk"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"GBP", "GB£"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GB£"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "D"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"USD", "$"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "Ksh"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP$"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GB£"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "Rf"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "K"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GB£"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "Le"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GB£"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "NAf."}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "E"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TSh"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "USh"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VT"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WS$"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}, {"ZWG", "ZWG"}},
	{{"AED", "AED"}, {"AFN", "AFN"}, {"ALL", "ALL"}, {"AMD", "AMD"}, {"ANG", "ANG"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AWG", "AWG"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BGN", "BGN"}, {"BHD", "BHD"}, {"BIF", "BIF"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BSD", "BSD"}, {"BTN", "BTN"}, {"BWP", "BWP"}, {"BYN", "BYN"}, {"BZD", "BZD"}, {"CDF", "CDF"}, {"CHF", "CHF"}, {"CLP", "CLP"}, {"CNH", "CNH"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CVE", "CVE"}, {"CZK", "CZK"}, {"DJF", "DJF"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"DZD", "DZD"}, {"EGP", "EGP"}, {"ERN", "ERN"}, {"ETB", "ETB"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GHS", "GHS"}, {"GIP", "GIP"}, {"GMD", "GMD"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HTG", "HTG"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"IQD", "IQD"}, {"IRR", "IRR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JOD", "JOD"}, {"KES", "KES"}, {"KGS", "KGS"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KWD", "KWD"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LSL", "LSL"}, {"LYD", "LYD"}, {"MAD", "MAD"}, {"MDL", "MDL"}, {"MGA", "MGA"}, {"MKD", "MKD"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MOP", "MOP"}, {"MRU", "MRU"}, {"MUR", "MUR"}, {"MVR", "MVR"}, {"MWK", "MWK"}, {"MYR", "MYR"}, {"MZN", "MZN"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"OMR", "OMR"}, {"PAB", "PAB"}, {"PEN", "PEN"}, {"PGK", "PGK"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"QAR", "QAR"}, {"RON", "RON"}, {"RSD", "RSD"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SAR", "SAR"}, {"SBD", "SBD"}, {"SCR", "SCR"}, {"SDG", "SDG"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SLE", "SLE"}, {"SLL", "SLL"}, {"SOS", "SOS"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"SZL", "SZL"}, {"THB", "THB"}, {"TJS", "TJS"}, {"TMT", "TMT"}, {"TND", "TND"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TZS", "TZS"}, {"UAH", "UAH"}, {"UGX", "UGX"}, {"UYU", "UYU"}, {"UZS", "UZS"}, {"VES", "VES"}, {"VUV", "VUV"}, {"WST", "WST"}, {"YER", "YER"}, {"ZAR", "ZAR"}, {"ZMW", "K"}, {"ZWG", "ZWG"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"GHS", "₵"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"LKR", "₨"}, {"MUR", "₨"}, {"MXN", "MXN"}, {"NPR", "₨"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PKR", "₨"}, {"TRY", "TRY"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XCG", "XCG"}, {"XOF", "XOF"}, {"XPF", "XPF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EGP", "EGP"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"TWD", "TWD"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"DOP", "RD$"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "$"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EGP", "EGP"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"TWD", "TWD"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "p."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MRO", "MRU"}, {"MRU", "UM"}, {"MXN", "$"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PAB", "B/."}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PEN", "S/"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EGP", "EGP"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"RON", "L"}, {"TWD", "TWD"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PYG", "Gs."}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "$"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"UYW", "UP"}, {"VEF", "BsF"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"FKP", "FK£"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SSP", "SD£"}, {"SYP", "S£"}, {"THB", "THB"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VEF", "Bs."}, {"VES", "Bs.S"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}},
	{{"AUD", "AU$"}, {"EEK", "kr"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "Br"}, {"CAD", "$CA"}, {"CNY", "¥CN"}, {"HKD", "$HK"}, {"IRR", "ریال"}, {"JPY", "¥"}, {"MXN", "$MX"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"USD", "$"}, {"XCD", "$EC"}, {"XOF", "فرانک\u202fCFA"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"NGN", "𞤐𞤐𞤘"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"GNF", "GNF"}, {"NGN", "𞤐𞤐𞤘"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"GMD", "D"}, {"GNF", "GNF"}, {"NGN", "𞤐𞤐𞤘"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"GNF", "GNF"}, {"MRU", "UM"}, {"NGN", "𞤐𞤐𞤘"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"GNF", "GNF"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"AFA", "𞤀𞤊𞤀"}, {"BYN", "р."}, {"GNF", "GNF"}, {"NGN", "𞤐𞤐𞤘"}, {"PGK", "𞤑𞤆𞤘"}, {"PHP", "𞤆𞤆𞤖"}, {"SLE", "Le"}, {"XAF", "𞤊𞤅𞤊𞤀"}, {"XOF", "𞤅𞤊𞤀"}},
	{{"GMD", "D"}},
	{{"MRU", "UM"}},
	{{"SLE", "Le"}},
	{{"AOA", "AOA"}, {"ARS", "ARS"}, {"AUD", "AUD"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BRL", "BRL"}, {"BSD", "BSD"}, {"BWP", "BWP"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CLP", "CLP"}, {"CNY", "CNY"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CZK", "CZK"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"EGP", "EGP"}, {"ESP", "ESP"}, {"FIM", "mk"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GEL", "GEL"}, {"GIP", "GIP"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HKD", "HKD"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"ILS", "ILS"}, {"INR", "INR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JPY", "¥"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KRW", "KRW"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"LTL", "LTL"}, {"LVL", "LVL"}, {"MGA", "MGA"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MUR", "MUR"}, {"MXN", "MXN"}, {"MYR", "MYR"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"RON", "RON"}, {"RWF", "RWF"}, {"SBD", "SBD"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STD"}, {"SYP", "SYP"}, {"THB", "THB"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"UAH", "UAH"}, {"USD", "$"}, {"UYU", "UYU"}, {"VEF", "VEF"}, {"VND", "VND"}, {"XCD", "XCD"}, {"XPF", "XPF"}, {"XXX", "XXX"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}},
	{{"BYN", "р."}, {"JPY", "¥"}, {"USD", "$"}},
	{{"BYN", "р."}, {"DKK", "kr."}, {"PHP", "PHP"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BIF", "FBu"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "ARS"}, {"AUD", "$\u00a0AU"}, {"BEF", "FB"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BYN", "Br"}, {"BZD", "BZD"}, {"CAD", "$"}, {"CLP", "CLP"}, {"COP", "COP"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"FRF", "F"}, {"GEL", "GEL"}, {"GIP", "GIP"}, {"HKD", "$\u00a0HK"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ILS", "ILS"}, {"INR", "INR"}, {"ITL", "₤IT"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"LBP", "LBP"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "MXN"}, {"NAD", "NAD"}, {"NZD", "$\u00a0NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "SBD"}, {"SGD", "$\u00a0SG"}, {"SRD", "SRD"}, {"TOP", "$T"}, {"TRY", "TL"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"USD", "$\u00a0US"}, {"UYU", "UYU"}, {"VND", "VND"}, {"WST", "WST"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XOF", "XOF"}, {"XPF", "XPF"}, {"XXX", "XXX"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CDF", "FC"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"DJF", "Fdj"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"DZD", "DA"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"HTG", "G"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "FRF"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"LUF", "F"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MRU", "UM"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SCR", "SR"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"SYP", "LS"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TND", "DT"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"RUR", "р."}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"VUV", "VT"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"ILS", "ILS"}, {"INR", "INR"}, {"KRW", "KRW"}, {"PHP", "PHP"}, {"VND", "VND"}, {"XCG", "XCG"}},
	{{"AUD", "AU$"}, {"CAD", "C$"}, {"FJD", "FJ$"}, {"SBD", "SI$"}, {"XPF", "XPF"}, {"XXX", "XXX"}},
	{{"JPY", "¥"}, {"RUR", "р."}, {"USD", "$"}, {"XXX", "XXX"}},
	{{"CAD", "KA$"}, {"CDF", "KDF"}, {"CHF", "SZF"}, {"COP", "KOP"}, {"CRC", "KRK"}, {"CUC", "KUK"}, {"CUP", "KUP"}, {"CZK", "TSK"}, {"GTQ", "GTK"}, {"KMF", "KF"}, {"MXN", "MZ$"}, {"NIO", "K$"}, {"SCR", "SSR"}, {"UGX", "UGS"}, {"USD", "$"}, {"XCD", "KB$"}, {"XOF", "SFA"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"RON", "leu"}, {"RUR", "р."}, {"USD", "$"}, {"XXX", "XXX"}},
	{{"BYN", "Br"}, {"KMF", "FC"}, {"MXN", "$MX"}, {"PHP", "PHP"}, {"RUB", "руб"}, {"USD", "$"}, {"XCD", "XCD"}, {"XCG", "ƒ"}},
	{{"ATS", "öS"}, {"JPY", "¥"}, {"USD", "$"}},
	{{"BYN", "р."}, {"MUR", "રૂ."}, {"PHP", "PHP"}},
	{{"BYN", "р"}, {"CNY", "\u200eCN¥\u200e"}, {"ILP", "ל״י"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "р."}, {"GEL", "ლ"}, {"PHP", "PHP"}, {"RON", "लेई"}, {"USD", "$"}},
	{{"USD", "𞅎"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RUR", "р."}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}, {"XPF", "XPF"}, {"XXX", "XXX"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"NLG", "ƒ"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"INR", "Rs"}, {"PHP", "PHP"}, {"XXX", "XXX"}},
	{{"CHF", "F.Sv."}, {"RUB", "Rub."}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"CAD", "CAD"}, {"DKK", "kr."}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"INR", "INR"}, {"ISK", "kr."}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NOK", "kr."}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"SEK", "kr."}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCG", "XCG"}},
	{{"BRL", "BRL"}, {"BYN", "Br"}, {"EGP", "£E"}, {"HKD", "HKD"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NOK", "NKr"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCG", "Cf"}},
	{{"BYN", "р."}, {"CNY", "元"}, {"JPY", "￥"}, {"PHP", "PHP"}, {"RON", "レイ"}, {"USD", "$"}, {"XCG", "Cg"}, {"XXX", "XXX"}},
	{{"AUD", "AUD"}, {"BYN", "р."}, {"CNY", "CNY"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"VND", "VND"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BEF", "FB"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BSD", "$BS"}, {"BYN", "р."}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"CNY", "CNY"}, {"COP", "$CO"}, {"CYP", "£CY"}, {"DZD", "DA"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"FRF", "F"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"IEP", "£IE"}, {"ILP", "£IL"}, {"ITL", "₤IT"}, {"JPY", "JPY"}, {"LBP", "£LB"}, {"LSL", "lLS"}, {"MAF", "fMA"}, {"MGF", "Fmg"}, {"MTP", "£MT"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NIO", "$C"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RHD", "$RH"}, {"RON", "L"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$"}, {"UYU", "$UY"}, {"WST", "WS$"}, {"XCD", "XCD"}, {"XDR", "DTS"}, {"XPF", "FCFP"}, {"XXX", "XXX"}, {"ZMW", "Kw"}},
	{{"AUD", "AU$"}, {"CVE", "\u200b"}},
	{{"AUD", "AU$"}, {"BYN", "p."}, {"FJD", "FJC"}, {"PHP", "PHP"}, {"PTE", "Vẽj."}, {"RON", "L"}, {"SYP", "S£"}, {"XOF", "CFA"}, {"ZMK", "SMK"}},
	{{"BYN", "р."}, {"LSL", "ЛСЛ"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"DKK", "kr."}, {"NOK", "Nkr"}, {"SEK", "Skr"}},
	{{"BYN", "р."}, {"LSL", "ឡូទី"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"RON", "ಲೀ"}, {"USD", "$"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"PHP", "PHP"}, {"RON", "L"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"RON", "रॉन"}},
	{{"LRD", "LRD"}},
	{{"AUD", "AUD"}, {"BMD", "BD$"}, {"BRL", "BRL"}, {"BSD", "B$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CAD", "CAD"}, {"DOP", "RD$"}, {"EGP", "LE"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"HRK", "Kn"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JMD", "J$"}, {"KGS", "сом"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TTD", "TT$"}, {"TWD", "TWD"}, {"USD", "USD"}, {"XCD", "XCD"}},
	{{"ATS", "öS"}, {"AUD", "AU$"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"CDF", "FC"}, {"CHF", "Fr."}, {"XPF", "F CFP"}},
	{{"BYN", "р."}, {"GEL", "ລາຣີ"}, {"PHP", "PHP"}},
	{{"AUD", "AUD"}, {"BDT", "BDT"}, {"BRL", "BRL"}, {"BYN", "Br"}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KHR", "KHR"}, {"KRW", "KRW"}, {"LAK", "LAK"}, {"MNT", "MNT"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PLN", "zl"}, {"PYG", "Gs"}, {"RUB", "rb"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}, {"XPF", "XPF"}},
	{{"CDF", "FC"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"GHS", "GHS"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"KES", "Ksh"}, {"TZS", "TSh"}},
	{{"MZN", "MTn"}},
	{{"JPY", "¥"}, {"NZD", "$"}},
	{{"AUD", "AUD"}, {"BYN", "р."}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MKD", "ден."}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"VND", "VND"}},
	{{"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"SEK", "кр"}, {"USD", "$"}},
	{{"GEL", "[₾]"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "р."}, {"CAD", "CAD"}, {"MXN", "MXN"}, {"PHP", "PHP"}, {"USD", "USD"}},
	{{"ANG", "NAf"}, {"AWG", "Afl"}, {"BBD", "Bds$"}, {"BSD", "B$"}, {"BYN", "р."}, {"HTG", "G"}, {"PAB", "B/."}, {"TTD", "TT$"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XPF", "XPF"}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"USD", "$"}},
	{{"BYN", "р."}, {"NPR", "नेरू"}, {"PHP", "PHP"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CAD", "C$"}, {"FJD", "FJ$"}, {"GEL", "ლ"}, {"PHP", "PHP"}, {"RUR", "р."}, {"SBD", "SI$"}, {"XCG", "Cg"}, {"XPF", "XPF"}, {"XXX", "XXX"}, {"ZWG", "ZiG"}},
	{{"AUD", "AU$"}, {"AWG", "Afl."}, {"BYN", "р."}, {"CAD", "C$"}, {"FJD", "FJ$"}, {"GEL", "ლ"}, {"PHP", "PHP"}, {"RUR", "р."}, {"SBD", "SI$"}, {"XCG", "Cg"}, {"XPF", "XPF"}, {"XXX", "XXX"}, {"ZWG", "ZiG"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CAD", "C$"}, {"FJD", "FJ$"}, {"GEL", "ლ"}, {"PHP", "PHP"}, {"RUR", "р."}, {"SBD", "SI$"}, {"USD", "$"}, {"XCG", "Cg"}, {"XPF", "XPF"}, {"XXX", "XXX"}, {"ZWG", "ZiG"}},
	{{"ANG", "NAf."}, {"AUD", "AU$"}, {"BYN", "р."}, {"CAD", "C$"}, {"FJD", "FJ$"}, {"GEL", "ლ"}, {"PHP", "PHP"}, {"RUR", "р."}, {"SBD", "SI$"}, {"XCG", "Cg"}, {"XPF", "XPF"}, {"XXX", "XXX"}, {"ZWG", "ZiG"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "$"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XPF", "XPF"}, {"XXX", "XXX"}},
	{{"AED", "ߊߐߘ"}, {"AFN", "ߊߝߊ"}, {"ALL", "ߊߟߟ"}, {"AMD", "ߊߡߘ"}, {"ANG", "ߊ߲ߕߝ"}, {"AOA", "ߊ߲ߜ߭ߎ"}, {"AON", "ߊߜ߭ߞ"}, {"AOR", "ߊߜ߭ߝ"}, {"ARS", "ߊߖ߭ߔ"}, {"AUD", "$ߐ"}, {"AWG", "ߊߥߖ߭"}, {"AZN", "ߊߖߣ"}, {"BAM", "ߓߍߡ"}, {"BBD", "ߓߓߘ"}, {"BDT", "ߓߘߕ"}, {"BGN", "ߓߜ߭ߟ"}, {"BHD", "ߓߤߘ"}, {"BIF", "ߓߙߝ"}, {"BMD", "ߓߡߘ"}, {"BND", "ߓߣߘ"}, {"BOB", "ߓߏߓ"}, {"BSD", "ߓߛߘ"}, {"BTN", "ߓߕߒ"}, {"BWP", "ߓߥߔ"}, {"BYN", "ߓߌߙ"}, {"CDF", "ߞߝ"}, {"CHF", "ߛߥߝ"}, {"CLP", "ߛߟߔ"}, {"CNH", "ߛߦߤ"}, {"CNY", "ߛߣߦ"}, {"COP", "ߞߐߔ"}, {"CRC", "ߞߙߞ"}, {"CUC", "ߞߎ߳ߞ"}, {"CUP", "ߞߎ߳ߔ"}, {"CVE", "ߍߛߞ"}, {"CZK", "ߗߞߞ"}, {"DJF", "ߖߓߝ"}, {"DKK", "ߘߞߞ"}, {"DOP", "ߘߏߔ"}, {"DZD", "ߊߟߘ"}, {"EGP", "ߡߛߔ"}, {"ERN", "ߋߙߝ"}, {"ETB", "ߋߗߓ"}, {"FJD", "$ ߝߖ"}, {"FKP", "ߝߞߔ"}, {"GEL", "ߖ߭ߐߟ"}, {"GHS", "ߜ߭ߛߘ"}, {"GIP", "ߖ߭ߓߔ£"}, {"GMD", "ߜ߭ߓߘ"}, {"GNF", "߿"}, {"GNS", "ߖߛ߾"}, {"GQE", "ߕߖߋ"}, {"GTQ", "ߜ߭ߕߞ"}, {"GWE", "ߖߓߍ"}, {"GWP", "ߖߓߔ"}, {"GYD", "ߜ߭ߎ߳ߘ"}, {"HKD", "ߤߞߘ"}, {"HNL", "ߤߘߟ"}, {"HRK", "ߞߛߞ"}, {"HTG", "ߤߕߜ߭"}, {"HUF", "ߤߜ߭ߝ"}, {"IDR", "ߍ߲ߘߙ"}, {"IQD", "ߌߞߘ"}, {"IRR", "ߌߙߙ"}, {"ISK", "ߌߛߞ"}, {"JMD", "ߖߡߘ"}, {"JOD", "ߖߐߘ"}, {"JPY", "ߗ߭ߔߦ"}, {"KES", "ߞߋߛ"}, {"KGS", "ߞߜ߭ߛ"}, {"KHR", "ߞߓߙ"}, {"KMF", "ߞߡߝ"}, {"KPW", "ߞߞߥ"}, {"KWD", "ߞߥߘ"}, {"KYD", "ߞߌߘ"}, {"KZT", "ߞߗ߭ߕ"}, {"LAK", "ߟߊߞ"}, {"LBP", "ߟߓ£"}, {"LKR", "ߟߞߙ"}, {"LRD", "ߟ߾"}, {"LSL", "ߟߛߟ"}, {"LYD", "ߟߓߘ"}, {"MAD", "ߡߘߤ"}, {"MAF", "ߡߙߝ"}, {"MDL", "ߡߘߟ"}, {"MGA", "ߡߘߙ"}, {"MGF", "ߡߘߝ"}, {"MKD", "ߡߞߘ"}, {"MLF", "ߡߝ"}, {"MMK", "ߡߡߞ"}, {"MNT", "ߡߣߕ"}, {"MOP", "ߡߏߔ"}, {"MRO", "ߡߙߏ"}, {"MRU", "ߡߎߜ߭"}, {"MUR", "ߡߙߔ"}, {"MVR", "ߡߝ߭ߙ"}, {"MWK", "ߡߟߞ"}, {"MXN", "ߡߞ$"}, {"MYR", "ߡߦߙ"}, {"MZE", "ߡߖ߭ߋ"}, {"MZM", "ߡߖߡ"}, {"MZN", "ߡߖ߭ߡ"}, {"NAD", "ߣߡߘ"}, {"NGN", "ߖߣ"}, {"NIO", "ߣߌߏ"}, {"NOK", "ߣߐߞ"}, {"NPR", "ߣߔߙ"}, {"NZD", "$ߣߖ߭"}, {"OMR", "ߏߡߙ"}, {"PAB", "ߔߊߓ"}, {"PEN", "ߔߋߛ"}, {"PGK", "ߔߖߞ"}, {"PKR", "ߔߞߙ"}, {"PLN", "ߔߟߗ߭"}, {"PYG", "ߔߙߜ߭"}, {"QAR", "ߞߊߙ"}, {"RHD", "ߖ߭ߓߘ"}, {"RON", "ߙߎߟ"}, {"RSD", "ߙߛߘ"}, {"RUB", "ߌߎߙ"}, {"RWF", "ߙߥߝ"}, {"SAR", "ߛߊߙ"}, {"SBD", "ߛߓߘ"}, {"SCR", "ߛߛߥ"}, {"SDD", "ߛߘߘ"}, {"SDG", "ߛߘߜ߭"}, {"SDP", "ߛߘߔ"}, {"SEK", "ߛߥߞ"}, {"SGD", "ߛߜ߭ߘ"}, {"SHP", "ߛߤߔ"}, {"SLE", "ߛߙߔ"}, {"SLL", "ߛߟߟ"}, {"SOS", "ߛߡߛ"}, {"SRD", "ߛߎߘ"}, {"SSP", "ߛߛߔ"}, {"STD", "ߛߕߘ"}, {"STN", "ߛߔߘ"}, {"SYP", "ߛߌߔ"}, {"SZL", "ߛߖ߭ߟ"}, {"THB", "ߕߘߓ"}, {"TJS", "ߕߖߛ"}, {"TMT", "ߕߡߕ"}, {"TND", "ߕߣߘ"}, {"TOP", "ߕߏߔ"}, {"TRY", "ߕߎߟ"}, {"TTD", "ߕߕߘ"}, {"TWD", "ߕߥߘ"}, {"TZS", "ߕߖ߭ߛ"}, {"UAH", "ߌߞߤ"}, {"UGS", "ߎߜ߭ߥ"}, {"UGX", "ߎߜ߭ߛ"}, {"USD", "ߊߞߘ$"}, {"UYU", "ߎ߳ߔ"}, {"UZS", "ߎߗ߭ߛ"}, {"VES", "ߝ߭ߋߓ"}, {"VUV", "ߝ߭ߕߝ߭"}, {"WST", "$ߥߛ"}, {"XAF", "ߝߛߝ"}, {"XCD", "ߛߟߘ"}, {"XOF", "߾"}, {"XPF", "߾ ߝߛߝ"}, {"YER", "ߦߡߙ"}, {"ZAR", "ߥߝߙ"}, {"ZMK", "ߖ߭ߓߞ"}, {"ZMW", "ߖ߭ߓߥ"}, {"ZRN", "ߖ߭ߙ"}, {"ZRZ", "ߖ߭ߙߖ߭"}, {"ZWD", "ߖ߭ߥߘ"}, {"ZWL", "ߖ߭ߥߟ"}, {"ZWR", "ߖ߭ߥߙ"}},
	{{"GBP", "GB£"}},
	{{"ARS", "$AR"}, {"AUD", "$AU"}, {"BMD", "$BM"}, {"BND", "$BN"}, {"BZD", "$BZ"}, {"CAD", "$CA"}, {"CLP", "$CL"}, {"COP", "$CO"}, {"EGP", "£E"}, {"FJD", "$FJ"}, {"FKP", "£FK"}, {"GBP", "£GB"}, {"GIP", "£GI"}, {"HKD", "HKD"}, {"JPY", "JPY"}, {"KMF", "FC"}, {"LBP", "£LB"}, {"MXN", "$MX"}, {"NAD", "$NA"}, {"NZD", "$NZ"}, {"PHP", "PHP"}, {"RON", "L"}, {"RWF", "FR"}, {"SBD", "$SB"}, {"SGD", "$SG"}, {"SRD", "$SR"}, {"TOP", "$T"}, {"TRY", "LT"}, {"TTD", "$TT"}, {"TWD", "TWD"}, {"USD", "$US"}, {"UYU", "$UY"}, {"WST", "$WS"}, {"XCD", "XCD"}, {"XPF", "FCFP"}, {"ZMW", "Kw"}},
	{{"ARS", "ARS"}, {"AUD", "AUD"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BRL", "BRL"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CLP", "CLP"}, {"CNY", "CNY"}, {"COP", "COP"}, {"EGP", "EGP"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GBP"}, {"GIP", "GIP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"LBP", "LBP"}, {"MXN", "MXN"}, {"NAD", "NAD"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"SBD", "SBD"}, {"SGD", "SDG"}, {"SRD", "SRD"}, {"TRY", "TL"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"UYU", "UYU"}, {"WST", "WST"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XOF", "XOF"}, {"XPF", "FCFP"}},
	{{"ETB", "Br"}, {"KES", "Ksh"}},
	{{"GEL", "GEL"}, {"USD", "$"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"XXX", "XXX"}},
	{{"PKR", "ر"}},
	{{"ANG", "XCG"}},
	{{"BYN", "p."}, {"CAD", "KA$"}, {"PHP", "PHP"}},
	{{"AUD", "AUD"}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "lej"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCG", "XCG"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"PHP", "PHP"}, {"PTE", "Esc."}, {"RON", "L"}, {"SYP", "S£"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"PHP", "PHP"}, {"PTE", "\u200b"}, {"RON", "L"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CVE", "\u200b"}, {"PHP", "PHP"}, {"PTE", "PTE"}, {"RON", "L"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"LUF", "F"}, {"PHP", "PHP"}, {"PTE", "\u200b"}, {"RON", "L"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"MOP", "MOP$"}, {"PHP", "PHP"}, {"PTE", "\u200b"}, {"RON", "L"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"MZN", "MTn"}, {"PHP", "PHP"}, {"PTE", "\u200b"}, {"RON", "L"}},
	{{"BBD", "BBG"}, {"BMD", "DBM"}, {"BZD", "DBZ"}, {"CAD", "$CA"}, {"GHS", "GHC"}, {"PEN", "S/"}, {"PHP", "PHP"}, {"USD", "$US"}},
	{{"BBD", "BBG"}, {"BMD", "DBM"}, {"BZD", "DBZ"}, {"CAD", "$CA"}, {"GHS", "GHC"}, {"PEN", "PEN"}, {"PHP", "PHP"}, {"USD", "$US"}},
	{{"BBD", "BBG"}, {"BMD", "DBM"}, {"BZD", "DBZ"}, {"CAD", "$CA"}, {"GHS", "GHC"}, {"PEN", "PEN"}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BIF", "FBu"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MDL", "L"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"BYN", "р."}, {"GEL", "ლ"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"TMT", "ТМТ"}, {"USD", "$"}, {"XCG", "Cg"}, {"XXX", "XXXX"}},
	{{"BYN", "Br"}, {"GEL", "ლ"}, {"JPY", "¥"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "RUR"}, {"TMT", "ТМТ"}, {"USD", "$"}, {"XCG", "Cg"}, {"XXX", "XXXX"}},
	{{"BYN", "р."}, {"GEL", "ლ"}, {"JPY", "¥"}, {"KGS", "сом"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"TMT", "ТМТ"}, {"USD", "$"}, {"XCG", "Cg"}, {"XXX", "XXXX"}},
	{{"BYN", "р."}, {"GEL", "ლ"}, {"JPY", "¥"}, {"MDL", "L"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"TMT", "ТМТ"}, {"USD", "$"}, {"XCG", "Cg"}, {"XXX", "XXXX"}},
	{{"XDR", "DIP"}},
	{{"DKK", "Dkr"}, {"GBP", "GB£"}, {"ISK", "Ikr"}, {"SEK", "Skr"}},
	{{"DKK", "Dkr"}, {"GBP", "GB£"}, {"ISK", "Ikr"}, {"NOK", "Nkr"}},
	{{"BYN", "р."}, {"LKR", "රු."}, {"PHP", "PHP"}, {"XOF", "සිෆ්එ"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "NIS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RUR", "р."}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XCG", "Cg"}, {"XXX", "XXX"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"GBP", "GBP"}, {"JPY", "¥"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"TWD", "TWD"}, {"USD", "$"}, {"XCD", "XCD"}},
	{{"BBD", "DBB"}, {"SOS", "S"}},
	{{"BBD", "DBB"}, {"DJF", "Fdj"}, {"SOS", "S"}},
	{{"BBD", "DBB"}, {"ETB", "Br"}, {"SOS", "S"}},
	{{"BBD", "DBB"}, {"KES", "Ksh"}, {"SOS", "S"}},
	{{"AFN", "AFN"}, {"ALL", "Lekë"}, {"AMD", "AMD"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AUD", "AUD"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BRL", "BRL"}, {"BSD", "BSD"}, {"BWP", "BWP"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CLP", "CLP"}, {"CNY", "CNY"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CZK", "CZK"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"EGP", "EGP"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GBP"}, {"GEL", "GEL"}, {"GIP", "GIP"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HKD", "HKS"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"ILS", "ILS"}, {"INR", "INR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JPY", "JPY"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KRW", "KRW"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"MGA", "MGA"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MUR", "MUR"}, {"MXN", "MXN"}, {"MYR", "MYR"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"RON", "RON"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SBD", "SBD"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"UAH", "UAH"}, {"USD", "USD"}, {"UYU", "UYU"}, {"VND", "VND"}, {"XCD", "XCD"}, {"XCG", "ANG"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}},
	{{"AFN", "AFN"}, {"ALL", "Lekë"}, {"AMD", "AMD"}, {"AOA", "AOA"}, {"ARS", "ARS"}, {"AUD", "AUD"}, {"AZN", "AZN"}, {"BAM", "BAM"}, {"BBD", "BBD"}, {"BDT", "BDT"}, {"BMD", "BMD"}, {"BND", "BND"}, {"BOB", "BOB"}, {"BRL", "BRL"}, {"BSD", "BSD"}, {"BWP", "BWP"}, {"BZD", "BZD"}, {"CAD", "CAD"}, {"CLP", "CLP"}, {"CNY", "CNY"}, {"COP", "COP"}, {"CRC", "CRC"}, {"CUC", "CUC"}, {"CUP", "CUP"}, {"CZK", "CZK"}, {"DKK", "DKK"}, {"DOP", "DOP"}, {"EGP", "EGP"}, {"FJD", "FJD"}, {"FKP", "FKP"}, {"GBP", "GBP"}, {"GEL", "GEL"}, {"GIP", "GIP"}, {"GNF", "GNF"}, {"GTQ", "GTQ"}, {"GYD", "GYD"}, {"HKD", "HKS"}, {"HNL", "HNL"}, {"HRK", "HRK"}, {"HUF", "HUF"}, {"IDR", "IDR"}, {"ILS", "ILS"}, {"INR", "INR"}, {"ISK", "ISK"}, {"JMD", "JMD"}, {"JPY", "JPY"}, {"KHR", "KHR"}, {"KMF", "KMF"}, {"KPW", "KPW"}, {"KRW", "KRW"}, {"KYD", "KYD"}, {"KZT", "KZT"}, {"LAK", "LAK"}, {"LBP", "LBP"}, {"LKR", "LKR"}, {"LRD", "LRD"}, {"MGA", "MGA"}, {"MKD", "den"}, {"MMK", "MMK"}, {"MNT", "MNT"}, {"MUR", "MUR"}, {"MXN", "MXN"}, {"MYR", "MYR"}, {"NAD", "NAD"}, {"NGN", "NGN"}, {"NIO", "NIO"}, {"NOK", "NOK"}, {"NPR", "NPR"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"PKR", "PKR"}, {"PLN", "PLN"}, {"PYG", "PYG"}, {"RON", "RON"}, {"RUB", "RUB"}, {"RWF", "RWF"}, {"SBD", "SBD"}, {"SEK", "SEK"}, {"SGD", "SGD"}, {"SHP", "SHP"}, {"SRD", "SRD"}, {"SSP", "SSP"}, {"STN", "STN"}, {"SYP", "SYP"}, {"TOP", "TOP"}, {"TRY", "TRY"}, {"TTD", "TTD"}, {"TWD", "TWD"}, {"UAH", "UAH"}, {"USD", "USD"}, {"UYU", "UYU"}, {"VND", "VND"}, {"XCD", "XCD"}, {"XCG", "ANG"}, {"ZAR", "ZAR"}, {"ZMW", "ZMW"}},
	{{"AUD", "AUD"}, {"BAM", "КМ"}, {"BYN", "р."}, {"GEL", "ლ"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"VND", "VND"}},
	{{"AUD", "AUD"}, {"BYN", "r."}, {"GEL", "ლ"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"VND", "VND"}},
	{{"SZL", "E"}},
	{{"LSL", "M"}},
	{{"AUD", "AUD"}, {"BBD", "Bds$"}, {"BMD", "BM$"}, {"BRL", "BR$"}, {"BSD", "BS$"}, {"BYN", "р."}, {"BZD", "BZ$"}, {"CNY", "CNY"}, {"DKK", "Dkr"}, {"DOP", "RD$"}, {"EEK", "Ekr"}, {"EGP", "EG£"}, {"ESP", "ESP"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"IEP", "IE£"}, {"INR", "INR"}, {"ISK", "Ikr"}, {"JMD", "JM$"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"NOK", "Nkr"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"TWD", "TWD"}, {"VND", "VND"}, {"XCG", "XCG"}},
	{{"BYN", "р."}, {"KES", "Ksh"}, {"PHP", "PHP"}, {"TZS", "TSh"}},
	{{"BYN", "р."}, {"CDF", "FC"}, {"KES", "Ksh"}, {"PHP", "PHP"}, {"TZS", "TSh"}},
	{{"BYN", "р."}, {"KES", "Ksh"}, {"PHP", "PHP"}, {"TZS", "TSh"}, {"USD", "$"}},
	{{"BYN", "р."}, {"KES", "Ksh"}, {"PHP", "PHP"}, {"TZS", "TSh"}, {"UGX", "USh"}},
	{{"SYP", "ل.س.\u200f"}},
	{{"BYN", "р."}, {"JPY", "¥"}, {"LKR", "Rs."}, {"PHP", "PHP"}, {"USD", "$"}},
	{{"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}, {"SGD", "S$"}, {"USD", "$"}},
	{{"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}},
	{{"KES", "Ksh"}, {"UGX", "USh"}},
	{{"TJS", "сом."}, {"USD", "$"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}, {"XXX", "XXX"}},
	{{"CNY", "CNY"}, {"ETB", "Br"}, {"JPY", "JPY"}},
	{{"CNY", "CNY"}, {"ERN", "Nfk"}, {"ETB", "Br"}, {"JPY", "JPY"}},
	{{"BYN", "р."}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"PHP", "PHP"}, {"XCG", "Kg."}},
	{{"AUD", "AUD$"}, {"FJD", "F$"}, {"NZD", "NZD$"}, {"SBD", "S$"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"USD", "$"}},
	{{"RUR", "р."}, {"USD", "$"}},
	{{"CNY", "￥"}, {"USD", "$"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"BYN", "р."}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "¥"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RUR", "р."}, {"TWD", "TWD"}, {"UAK", "крб."}, {"USD", "USD"}, {"VND", "VND"}, {"XCD", "XCD"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"PKR", "PKR"}, {"USD", "$"}},
	{{"BYN", "р."}, {"PHP", "PHP"}, {"UZS", "soʻm"}},
	{{"UZS", "сўм"}},
	{{"AUD", "AUD"}, {"BRL", "BRL"}, {"CAD", "CAD"}, {"CNY", "CNY"}, {"EUR", "EUR"}, {"GBP", "GBP"}, {"HKD", "HKD"}, {"ILS", "ILS"}, {"INR", "INR"}, {"JPY", "JPY"}, {"KRW", "KRW"}, {"MXN", "MXN"}, {"NZD", "NZD"}, {"PHP", "PHP"}, {"RON", "L"}, {"TWD", "TWD"}, {"USD", "USD"}, {"VND", "VND"}, {"XAF", "XAF"}, {"XCD", "XCD"}, {"XCG", "Cf"}, {"XOF", "XOF"}, {"XPF", "XPF"}},
	{{"CLP", "Vote $"}, {"EGP", "EGPP"}, {"GHS", "GHS."}, {"GIP", "GIIP"}, {"GTQ", "GT Q"}, {"HRK", "HRKS"}, {"HUF", "Vote Ft"}, {"MDL", "Vote MDL"}, {"NGN", "NGN."}, {"RON", "Vote lei"}, {"USD", "$"}},
	{{"CNH", "I-CNH"}, {"JPY", "¥"}, {"USD", "$"}, {"XCG", "ICg."}, {"ZWG", "IZWG"}},
	{{"AUD", "AU$"}, {"BOB", "BUB"}, {"BYN", "p."}, {"PHP", "PHP"}, {"PTE", "Esc."}, {"RON", "L"}, {"SCR", "SCRu"}, {"SYP", "S£"}, {"VES", "Bs.S"}, {"XAF", "FCF"}, {"XOF", "CFA"}, {"XPF", "CFP"}, {"ZMW", "Zk"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"JPY", "¥"}, {"KRW", "￦"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"TWD", "$"}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CNY", "¥"}, {"ILR", "ILS"}, {"JPY", "¥"}, {"KRW", "￦"}, {"PHP", "PHP"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"CNY", "¥"}, {"ILR", "ILS"}, {"PHP", "PHP"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"CNY", "¥"}, {"USD", "$"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"ILR", "ILS"}, {"PHP", "PHP"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"ILR", "ILS"}, {"MOP", "MOP$"}, {"PHP", "PHP"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"JPY", "¥"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"AUD", "AU$"}, {"BYN", "р."}, {"JPY", "¥"}, {"MOP", "MOP$"}, {"PHP", "PHP"}, {"RON", "L"}, {"RUR", "р."}, {"XXX", "XXX"}},
	{{"BYN", "P."}, {"DKK", "Kr"}, {"HRK", "Kn"}, {"ISK", "Kr"}, {"NOK", "Kr"}, {"PHP", "PHP"}, {"PLN", "Zł"}, {"SEK", "Kr"}, {"USD", "$"}},
}

var _metazones = [...]kv{
	{"Africa/Abidjan", "GMT"},
	{"Africa/Accra", "GMT"},
	{"Africa/Addis_Ababa", "Africa_Eastern"},
	{"Africa/Algiers", "Europe_Central"},
	{"Africa/Asmera", "Africa_Eastern"},
	{"Africa/Bamako", "GMT"},
	{"Africa/Bangui", "Africa_Western"},
	{"Africa/Banjul", "GMT"},
	{"Africa/Bissau", "GMT"},
	{"Africa/Blantyre", "Africa_Central"},
	{"Africa/Brazzaville", "Africa_Western"},
	{"Africa/Bujumbura", "Africa_Central"},
	{"Africa/Cairo", "Europe_Eastern"},
	{"Africa/Ceuta", "Europe_Central"},
	{"Africa/Conakry", "GMT"},
	{"Africa/Dakar", "GMT"},
	{"Africa/Dar_es_Salaam", "Africa_Eastern"},
	{"Africa/Djibouti", "Africa_Eastern"},
	{"Africa/Douala", "Africa_Western"},
	{"Africa/Freetown", "GMT"},
	{"Africa/Gaborone", "Africa_Central"},
	{"Africa/Harare", "Africa_Central"},
	{"Africa/Johannesburg", "Africa_Southern"},
	{"Africa/Juba", "Africa_Central"},
	{"Africa/Kampala", "Africa_Eastern"},
	{"Africa/Khartoum", "Africa_Central"},
	{"Africa/Kigali", "Africa_Central"},
	{"Africa/Kinshasa", "Africa_Western"},
	{"Africa/Lagos", "Africa_Western"},
	{"Africa/Libreville", "Africa_Western"},
	{"Africa/Lome", "GMT"},
	{"Africa/Luanda", "Africa_Western"},
	{"Africa/Lubumbashi", "Africa_Central"},
	{"Africa/Lusaka", "Africa_Central"},
	{"Africa/Malabo", "Africa_Western"},
	{"Africa/Maputo", "Africa_Central"},
	{"Africa/Maseru", "Africa_Southern"},
	{"Africa/Mbabane", "Africa_Southern"},
	{"Africa/Mogadishu", "Africa_Eastern"},
	{"Africa/Monrovia", "GMT"},
	{"Africa/Nairobi", "Africa_Eastern"},
	{"Africa/Ndjamena", "Africa_Western"},
	{"Africa/Niamey", "Africa_Western"},
	{"Africa/Nouakchott", "GMT"},
	{"Africa/Ouagadougou", "GMT"},
	{"Africa/Porto-Novo", "Africa_Western"},
	{"Africa/Sao_Tome", "GMT"},
	{"Africa/Tripoli", "Europe_Eastern"},
	{"Africa/Tunis", "Europe_Central"},
	{"Africa/Windhoek", "Africa_Central"},
	{"America/Adak", "Hawaii_Aleutian"},
	{"America/Anchorage", "Alaska"},
	{"America/Anguilla", "Atlantic"},
	{"America/Antigua", "Atlantic"},
	{"America/Araguaina", "Brasilia"},
	{"America/Argentina/La_Rioja", "Argentina"},
	{"America/Argentina/Rio_Gallegos", "Argentina"},
	{"America/Argentina/Salta", "Argentina"},
	{"America/Argentina/San_Juan", "Argentina"},
	{"America/Argentina/San_Luis", "Argentina"},
	{"America/Argentina/Tucuman", "Argentina"},
	{"America/Argentina/Ushuaia", "Argentina"},
	{"America/Aruba", "Atlantic"},
	{"America/Asuncion", "Paraguay"},
	{"America/Bahia", "Brasilia"},
	{"America/Bahia_Banderas", "America_Central"},
	{"America/Barbados", "Atlantic"},
	{"America/Belem", "Brasilia"},
	{"America/Belize", "America_Central"},
	{"America/Blanc-Sablon", "Atlantic"},
	{"America/Boa_Vista", "Amazon"},
	{"America/Bogota", "Colombia"},
	{"America/Boise", "America_Mountain"},
	{"America/Buenos_Aires", "Argentina"},
	{"America/Cambridge_Bay", "America_Mountain"},
	{"America/Campo_Grande", "Amazon"},
	{"America/Cancun", "America_Eastern"},
	{"America/Caracas", "Venezuela"},
	{"America/Catamarca", "Argentina"},
	{"America/Cayenne", "French_Guiana"},
	{"America/Cayman", "America_Eastern"},
	{"America/Chicago", "America_Central"},
	{"America/Chihuahua", "America_Central"},
	{"America/Ciudad_Juarez", "America_Mountain"},
	{"America/Coral_Harbour", "America_Eastern"},
	{"America/Cordoba", "Argentina"},
	{"America/Costa_Rica", "America_Central"},
	{"America/Creston", "America_Mountain"},
	{"America/Cuiaba", "Amazon"},
	{"America/Curacao", "Atlantic"},
	{"America/Danmarkshavn", "GMT"},
	{"America/Dawson", "Yukon"},
	{"America/Dawson_Creek", "America_Mountain"},
	{"America/Denver", "America_Mountain"},
	{"America/Detroit", "America_Eastern"},
	{"America/Dominica", "Atlantic"},
	{"America/Edmonton", "America_Mountain"},
	{"America/Eirunepe", "Acre"},
	{"America/El_Salvador", "America_Central"},
	{"America/Fort_Nelson", "America_Mountain"},
	{"America/Fortaleza", "Brasilia"},
	{"America/Glace_Bay", "Atlantic"},
	{"America/Godthab", "Greenland"},
	{"America/Goose_Bay", "Atlantic"},
	{"America/Grand_Turk", "America_Eastern"},
	{"America/Grenada", "Atlantic"},
	{"America/Guadeloupe", "Atlantic"},
	{"America/Guatemala", "America_Central"},
	{"America/Guayaquil", "Ecuador"},
	{"America/Guyana", "Guyana"},
	{"America/Halifax", "Atlantic"},
	{"America/Havana", "Cuba"},
	{"America/Hermosillo", "Mexico_Pacific"},
	{"America/Indiana/Knox", "America_Central"},
	{"America/Indiana/Marengo", "America_Eastern"},
	{"America/Indiana/Petersburg", "America_Eastern"},
	{"America/Indiana/Tell_City", "America_Central"},
	{"America/Indiana/Vevay", "America_Eastern"},
	{"America/Indiana/Vincennes", "America_Eastern"},
	{"America/Indiana/Winamac", "America_Eastern"},
	{"America/Indianapolis", "America_Eastern"},
	{"America/Inuvik", "America_Mountain"},
	{"America/Iqaluit", "America_Eastern"},
	{"America/Jamaica", "America_Eastern"},
	{"America/Jujuy", "Argentina"},
	{"America/Juneau", "Alaska"},
	{"America/Kentucky/Monticello", "America_Eastern"},
	{"America/Kralendijk", "Atlantic"},
	{"America/La_Paz", "Bolivia"},
	{"America/Lima", "Peru"},
	{"America/Los_Angeles", "America_Pacific"},
	{"America/Louisville", "America_Eastern"},
	{"America/Lower_Princes", "Atlantic"},
	{"America/Maceio", "Brasilia"},
	{"America/Managua", "America_Central"},
	{"America/Manaus", "Amazon"},
	{"America/Marigot", "Atlantic"},
	{"America/Martinique", "Atlantic"},
	{"America/Matamoros", "America_Central"},
	{"America/Mazatlan", "Mexico_Pacific"},
	{"America/Mendoza", "Argentina"},
	{"America/Menominee", "America_Central"},
	{"America/Merida", "America_Central"},
	{"America/Metlakatla", "Alaska"},
	{"America/Mexico_City", "America_Central"},
	{"America/Miquelon", "Pierre_Miquelon"},
	{"America/Moncton", "Atlantic"},
	{"America/Monterrey", "America_Central"},
	{"America/Montevideo", "Uruguay"},
	{"America/Montserrat", "Atlantic"},
	{"America/Nassau", "America_Eastern"},
	{"America/New_York", "America_Eastern"},
	{"America/Nome", "Alaska"},
	{"America/Noronha", "Noronha"},
	{"America/North_Dakota/Beulah", "America_Central"},
	{"America/North_Dakota/Center", "America_Central"},
	{"America/North_Dakota/New_Salem", "America_Central"},
	{"America/Ojinaga", "America_Central"},
	{"America/Panama", "America_Eastern"},
	{"America/Paramaribo", "Suriname"},
	{"America/Phoenix", "America_Mountain"},
	{"America/Port-au-Prince", "America_Eastern"},
	{"America/Port_of_Spain", "Atlantic"},
	{"America/Porto_Velho", "Amazon"},
	{"America/Puerto_Rico", "Atlantic"},
	{"America/Rankin_Inlet", "America_Central"},
	{"America/Recife", "Brasilia"},
	{"America/Regina", "America_Central"},
	{"America/Resolute", "America_Central"},
	{"America/Rio_Branco", "Acre"},
	{"America/Santarem", "Brasilia"},
	{"America/Santiago", "Chile"},
	{"America/Santo_Domingo", "Atlantic"},
	{"America/Sao_Paulo", "Brasilia"},
	{"America/Scoresbysund", "Greenland"},
	{"America/Sitka", "Alaska"},
	{"America/St_Barthelemy", "Atlantic"},
	{"America/St_Johns", "Newfoundland"},
	{"America/St_Kitts", "Atlantic"},
	{"America/St_Lucia", "Atlantic"},
	{"America/St_Thomas", "Atlantic"},
	{"America/St_Vincent", "Atlantic"},
	{"America/Swift_Current", "America_Central"},
	{"America/Tegucigalpa", "America_Central"},
	{"America/Thule", "Atlantic"},
	{"America/Tijuana", "America_Pacific"},
	{"America/Toronto", "America_Eastern"},
	{"America/Tortola", "Atlantic"},
	{"America/Vancouver", "America_Pacific"},
	{"America/Whitehorse", "Yukon"},
	{"America/Winnipeg", "America_Central"},
	{"America/Yakutat", "Alaska"},
	{"Antarctica/Casey", "Australia_Western"},
	{"Antarctica/Davis", "Davis"},
	{"Antarctica/DumontDUrville", "DumontDUrville"},
	{"Antarctica/Macquarie", "Australia_Eastern"},
	{"Antarctica/Mawson", "Mawson"},
	{"Antarctica/McMurdo", "New_Zealand"},
	{"Antarctica/Rothera", "Rothera"},
	{"Antarctica/Syowa", "Syowa"},
	{"Antarctica/Troll", "GMT"},
	{"Antarctica/Vostok", "Vostok"},
	{"Arctic/Longyearbyen", "Europe_Central"},
	{"Asia/Aden", "Arabian"},
	{"Asia/Almaty", "Kazakhstan"},
	{"Asia/Anadyr", "Kamchatka"},
	{"Asia/Aqtau", "Kazakhstan"},
	{"Asia/Aqtobe", "Kazakhstan"},
	{"Asia/Ashgabat", "Turkmenistan"},
	{"Asia/Atyrau", "Kazakhstan"},
	{"Asia/Baghdad", "Arabian"},
	{"Asia/Bahrain", "Arabian"},
	{"Asia/Baku", "Azerbaijan"},
	{"Asia/Bangkok", "Indochina"},
	{"Asia/Barnaul", "Krasnoyarsk"},
	{"Asia/Beirut", "Europe_Eastern"},
	{"Asia/Bishkek", "Kyrgystan"},
	{"Asia/Brunei", "Brunei"},
	{"Asia/Calcutta", "India"},
	{"Asia/Chita", "Yakutsk"},
	{"Asia/Colombo", "India"},
	{"Asia/Dhaka", "Bangladesh"},
	{"Asia/Dili", "East_Timor"},
	{"Asia/Dubai", "Gulf"},
	{"Asia/Dushanbe", "Tajikistan"},
	{"Asia/Famagusta", "Europe_Eastern"},
	{"Asia/Gaza", "Europe_Eastern"},
	{"Asia/Hebron", "Europe_Eastern"},
	{"Asia/Hong_Kong", "Hong_Kong"},
	{"Asia/Hovd", "Hovd"},
	{"Asia/Irkutsk", "Irkutsk"},
	{"Asia/Jakarta", "Indonesia_Western"},
	{"Asia/Jayapura", "Indonesia_Eastern"},
	{"Asia/Jerusalem", "Israel"},
	{"Asia/Kabul", "Afghanistan"},
	{"Asia/Kamchatka", "Kamchatka"},
	{"Asia/Karachi", "Pakistan"},
	{"Asia/Katmandu", "Nepal"},
	{"Asia/Khandyga", "Yakutsk"},
	{"Asia/Krasnoyarsk", "Krasnoyarsk"},
	{"Asia/Kuala_Lumpur", "Malaysia"},
	{"Asia/Kuching", "Malaysia"},
	{"Asia/Kuwait", "Arabian"},
	{"Asia/Macau", "China"},
	{"Asia/Magadan", "Magadan"},
	{"Asia/Makassar", "Indonesia_Central"},
	{"Asia/Manila", "Philippines"},
	{"Asia/Muscat", "Gulf"},
	{"Asia/Nicosia", "Europe_Eastern"},
	{"Asia/Novokuznetsk", "Krasnoyarsk"},
	{"Asia/Novosibirsk", "Krasnoyarsk"},
	{"Asia/Omsk", "Omsk"},
	{"Asia/Oral", "Kazakhstan"},
	{"Asia/Phnom_Penh", "Indochina"},
	{"Asia/Pontianak", "Indonesia_Western"},
	{"Asia/Pyongyang", "Korea"},
	{"Asia/Qatar", "Arabian"},
	{"Asia/Qostanay", "Kazakhstan"},
	{"Asia/Qyzylorda", "Kazakhstan"},
	{"Asia/Rangoon", "Myanmar"},
	{"Asia/Riyadh", "Arabian"},
	{"Asia/Saigon", "Indochina"},
	{"Asia/Sakhalin", "Magadan"},
	{"Asia/Samarkand", "Uzbekistan"},
	{"Asia/Seoul", "Korea"},
	{"Asia/Shanghai", "China"},
	{"Asia/Singapore", "Singapore"},
	{"Asia/Srednekolymsk", "Magadan"},
	{"Asia/Taipei", "Taipei"},
	{"Asia/Tashkent", "Uzbekistan"},
	{"Asia/Tbilisi", "Georgia"},
	{"Asia/Tehran", "Iran"},
	{"Asia/Thimphu", "Bhutan"},
	{"Asia/Tokyo", "Japan"},
	{"Asia/Tomsk", "Krasnoyarsk"},
	{"Asia/Ulaanbaatar", "Mongolia"},
	{"Asia/Urumqi", "Urumqi"},
	{"Asia/Ust-Nera", "Vladivostok"},
	{"Asia/Vientiane", "Indochina"},
	{"Asia/Vladivostok", "Vladivostok"},
	{"Asia/Yakutsk", "Yakutsk"},
	{"Asia/Yekaterinburg", "Yekaterinburg"},
	{"Asia/Yerevan", "Armenia"},
	{"Atlantic/Azores", "Azores"},
	{"Atlantic/Bermuda", "Atlantic"},
	{"Atlantic/Canary", "Europe_Western"},
	{"Atlantic/Cape_Verde", "Cape_Verde"},
	{"Atlantic/Faeroe", "Europe_Western"},
	{"Atlantic/Madeira", "Europe_Western"},
	{"Atlantic/Reykjavik", "GMT"},
	{"Atlantic/South_Georgia", "South_Georgia"},
	{"Atlantic/St_Helena", "GMT"},
	{"Atlantic/Stanley", "Falkland"},
	{"Australia/Adelaide", "Australia_Central"},
	{"Australia/Brisbane", "Australia_Eastern"},
	{"Australia/Broken_Hill", "Australia_Central"},
	{"Australia/Darwin", "Australia_Central"},
	{"Australia/Eucla", "Australia_CentralWestern"},
	{"Australia/Hobart", "Australia_Eastern"},
	{"Australia/Lindeman", "Australia_Eastern"},
	{"Australia/Lord_Howe", "Lord_Howe"},
	{"Australia/Melbourne", "Australia_Eastern"},
	{"Australia/Perth", "Australia_Western"},
	{"Australia/Sydney", "Australia_Eastern"},
	{"Etc/GMT", "GMT"},
	{"Europe/Amsterdam", "Europe_Central"},
	{"Europe/Andorra", "Europe_Central"},
	{"Europe/Astrakhan", "Samara"},
	{"Europe/Athens", "Europe_Eastern"},
	{"Europe/Belgrade", "Europe_Central"},
	{"Europe/Berlin", "Europe_Central"},
	{"Europe/Bratislava", "Europe_Central"},
	{"Europe/Brussels", "Europe_Central"},
	{"Europe/Bucharest", "Europe_Eastern"},
	{"Europe/Budapest", "Europe_Central"},
	{"Europe/Busingen", "Europe_Central"},
	{"Europe/Chisinau", "Europe_Eastern"},
	{"Europe/Copenhagen", "Europe_Central"},
	{"Europe/Dublin", "GMT"},
	{"Europe/Gibraltar", "Europe_Central"},
	{"Europe/Guernsey", "GMT"},
	{"Europe/Helsinki", "Europe_Eastern"},
	{"Europe/Isle_of_Man", "GMT"},
	{"Europe/Istanbul", "Turkey"},
	{"Europe/Jersey", "GMT"},
	{"Europe/Kaliningrad", "Europe_Eastern"},
	{"Europe/Kiev", "Europe_Eastern"},
	{"Europe/Kirov", "Moscow"},
	{"Europe/Lisbon", "Europe_Western"},
	{"Europe/Ljubljana", "Europe_Central"},
	{"Europe/London", "GMT"},
	{"Europe/Luxembourg", "Europe_Central"},
	{"Europe/Madrid", "Europe_Central"},
	{"Europe/Malta", "Europe_Central"},
	{"Europe/Mariehamn", "Europe_Eastern"},
	{"Europe/Minsk", "Moscow"},
	{"Europe/Monaco", "Europe_Central"},
	{"Europe/Moscow", "Moscow"},
	{"Europe/Oslo", "Europe_Central"},
	{"Europe/Paris", "Europe_Central"},
	{"Europe/Podgorica", "Europe_Central"},
	{"Europe/Prague", "Europe_Central"},
	{"Europe/Riga", "Europe_Eastern"},
	{"Europe/Rome", "Europe_Central"},
	{"Europe/Samara", "Samara"},
	{"Europe/San_Marino", "Europe_Central"},
	{"Europe/Sarajevo", "Europe_Central"},
	{"Europe/Saratov", "Samara"},
	{"Europe/Simferopol", "Moscow"},
	{"Europe/Skopje", "Europe_Central"},
	{"Europe/Sofia", "Europe_Eastern"},
	{"Europe/Stockholm", "Europe_Central"},
	{"Europe/Tallinn", "Europe_Eastern"},
	{"Europe/Tirane", "Europe_Central"},
	{"Europe/Ulyanovsk", "Samara"},
	{"Europe/Vaduz", "Europe_Central"},
	{"Europe/Vatican", "Europe_Central"},
	{"Europe/Vienna", "Europe_Central"},
	{"Europe/Vilnius", "Europe_Eastern"},
	{"Europe/Volgograd", "Moscow"},
	{"Europe/Warsaw", "Europe_Central"},
	{"Europe/Zagreb", "Europe_Central"},
	{"Europe/Zurich", "Europe_Central"},
	{"Indian/Antananarivo", "Africa_Eastern"},
	{"Indian/Chagos", "Indian_Ocean"},
	{"Indian/Christmas", "Christmas"},
	{"Indian/Cocos", "Cocos"},
	{"Indian/Comoro", "Africa_Eastern"},
	{"Indian/Kerguelen", "French_Southern"},
	{"Indian/Mahe", "Seychelles"},
	{"Indian/Maldives", "Maldives"},
	{"Indian/Mauritius", "Mauritius"},
	{"Indian/Mayotte", "Africa_Eastern"},
	{"Indian/Reunion", "Reunion"},
	{"Pacific/Apia", "Apia"},
	{"Pacific/Auckland", "New_Zealand"},
	{"Pacific/Chatham", "Chatham"},
	{"Pacific/Easter", "Easter"},
	{"Pacific/Efate", "Vanuatu"},
	{"Pacific/Enderbury", "Phoenix_Islands"},
	{"Pacific/Fakaofo", "Tokelau"},
	{"Pacific/Fiji", "Fiji"},
	{"Pacific/Funafuti", "Tuvalu"},
	{"Pacific/Galapagos", "Galapagos"},
	{"Pacific/Gambier", "Gambier"},
	{"Pacific/Guadalcanal", "Solomon"},
	{"Pacific/Guam", "Chamorro"},
	{"Pacific/Honolulu", "Hawaii"},
	{"Pacific/Kiritimati", "Line_Islands"},
	{"Pacific/Kosrae", "Kosrae"},
	{"Pacific/Kwajalein", "Marshall_Islands"},
	{"Pacific/Majuro", "Marshall_Islands"},
	{"Pacific/Marquesas", "Marquesas"},
	{"Pacific/Midway", "Samoa"},
	{"Pacific/Nauru", "Nauru"},
	{"Pacific/Niue", "Niue"},
	{"Pacific/Norfolk", "Norfolk"},
	{"Pacific/Noumea", "New_Caledonia"},
	{"Pacific/Pago_Pago", "Samoa"},
	{"Pacific/Palau", "Palau"},
	{"Pacific/Pitcairn", "Pitcairn"},
	{"Pacific/Ponape", "Ponape"},
	{"Pacific/Port_Moresby", "Papua_New_Guinea"},
	{"Pacific/Rarotonga", "Cook"},
	{"Pacific/Saipan", "Chamorro"},
	{"Pacific/Tahiti", "Tahiti"},
	{"Pacific/Tarawa", "Gilbert_Islands"},
	{"Pacific/Tongatapu", "Tonga"},
	{"Pacific/Truk", "Truk"},
	{"Pacific/Wake", "Wake"},
	{"Pacific/Wallis", "Wallis"},
}

var _tzNames = [...][2][]kv{
	{nil, nil},
	{
		{{"Acre", "Акри астандартә аамҭа"}, {"Afghanistan", "Афганистан"}, {"Africa_Central", "Агәҭантәи Африка"}, {"Africa_Eastern", "Мрагыларатәи Африка"}, {"Africa_Southern", "Аладатәи Африка"}, {"Africa_Western", "Мраҭашәаратәи Африка"}, {"Alaska", "Алиаска, астандартә аамҭа"}, {"Almaty", "Алма-Ата астандартә аамҭа"}, {"Amazon", "Амазонка, астандартә аамҭа"}, {"America_Central", "Агәҭантәи Америка, астандартә аамҭа"}, {"America_Eastern", "Мрагыларатәи Америка, астандартә аамҭа"}, {"America_Mountain", "Астандартә ашьхатә аамҭа (Аҩадатәи Америка)"}, {"America_Pacific", "Аокеанҭынчтәи астандартә аамҭа"}, {"Anadyr", "Анадыр астандартә аамҭа"}, {"Apia", "Апиа, астандартә аамҭа"}, {"Aqtau", "Актау, астандартә аамҭа"}, {"Aqtobe", "Актобе астандартә аамҭа"}, {"Arabian", "Саудтәи Арабсҭан, астандартә аамҭа"}, {"Argentina", "Аргентина, астандартә аамҭа"}, {"Argentina_Western", "Мраҭашәаратәи Аргентина, астандартә аамҭа"}, {"Armenia", "Ермантәыла, астандартә аамҭа"}, {"Atlantic", "Атлантикатәи астандартә аамҭа"}, {"Australia_Central", "Агәҭантәи Австралиа, астандартә аамҭа"}, {"Australia_CentralWestern", "Агәҭантәи Австралиа, мраҭашәаратәи астандартә аамҭа"}, {"Australia_Eastern", "Мрагыларатәи Австралиа, астандартә аамҭа"}, {"Australia_Western", "Мраҭашәаратәи Австралиа, астандартә аамҭа"}, {"Azerbaijan", "Азербаиџьан, астандартә аамҭа"}, {"Azores", "Азортәи ад-хақәа, астандартә аамҭа"}, {"Bangladesh", "Бангладеш, астандартә аамҭа"}, {"Bhutan", "Бутан"}, {"Bolivia", "Боливиа"}, {"Brasilia", "Бразилиа, астандартә аамҭа"}, {"Brunei", "Брунеи-Даруссалам"}, {"Cape_Verde", "Кабо-Верде, астандартә аамҭа"}, {"Casey", "Кеиси"}, {"Chamorro", "Чаморро"}, {"Chatham", "Чатем, астандартә аамҭа"}, {"Chile", "Чили, астандартә аамҭа"}, {"China", "Китаи, астандартә аамҭа"}, {"Christmas", "ад-ха Қьырса"}, {"Cocos", "Кокостәи ад-хақәа"}, {"Colombia", "Колумбиа, астандартә аамҭа"}, {"Cook", "Кук идгьылбжьахақәа, астандартә аамҭа"}, {"Cuba", "Куба, астандартә аамҭа"}, {"Davis", "Деивис"}, {"DumontDUrville", "Диумон-д’Иурвил"}, {"East_Timor", "Мрагыларатәи Тимор"}, {"Easter", "ад-ха Пасхи, астандартә аамҭа"}, {"Ecuador", "Еквадор"}, {"Europe_Central", "Агәҭантәи Европа, астандартә аамҭа"}, {"Europe_Eastern", "Мрагыларатәи Европа, астандартә аамҭа"}, {"Europe_Further_Eastern", "Московатәи аамҭа"}, {"Europe_Western", "Мраҭашәаратәи Европа, астандартә аамҭа"}, {"Falkland", "Фолклендтәи ад-хақәа, астандартә аамҭа"}, {"Fiji", "Фиџи, астандартә аамҭа"}, {"French_Guiana", "Францызтәи Гвиана"}, {"French_Southern", "Францызтәи Аладатәеи Антарктикатәи рҵакырадгьылқәа"}, {"GMT", "Агринвич Ибжьаратәу Аамҭа"}, {"Galapagos", "Галапагостәи ад-хақәа"}, {"Gambier", "Гамбе"}, {"Georgia", "Қырҭтәыла, астандартә аамҭа"}, {"Gilbert_Islands", "ад-хақәа Гилберта"}, {"Greenland_Eastern", "Мрагыларатәи Гренландиа, стандарное времиа"}, {"Greenland_Western", "Мраҭашәаратәи Гренландиа, астандартә аамҭа"}, {"Guam", "Гуам"}, {"Gulf", "Аџьамтә аӡыбжьахала"}, {"Guyana", "Гаиана"}, {"Hawaii", "Ҳаваи-алеуттәи астандартә аамҭа"}, {"Hawaii_Aleutian", "Ҳаваи-алеуттәи астандартә аамҭа"}, {"Hong_Kong", "Ҳонконг, астандартә аамҭа"}, {"Hovd", "Ховд, астандартә аамҭа"}, {"India", "Индиа"}, {"Indian_Ocean", "Индиатәи аокеан"}, {"Indochina", "Индокитаи"}, {"Indonesia_Central", "Агәҭантәи Индонезиа"}, {"Indonesia_Eastern", "Мрагыларатәи Индонезиа"}, {"Indonesia_Western", "Мраҭашәаратәи Индонезиа"}, {"Iran", "Иран, астандартә аамҭа"}, {"Irkutsk", "Иркутск, астандартә аамҭа"}, {"Israel", "Израиль, астандартә аамҭа"}, {"Japan", "Иапониа, астандартә аамҭа"}, {"Kamchatka", "Петропавловск-Камчаткатәи, астандартә аамҭа"}, {"Kazakhstan_Eastern", "Мрагыларатәи Ҟазахсҭан"}, {"Kazakhstan_Western", "Мраҭашәаратәи Ҟазахсҭан"}, {"Korea", "Кореиа, астандартә аамҭа"}, {"Kosrae", "Косрае"}, {"Krasnoyarsk", "Красноиарск, астандартә аамҭа"}, {"Kyrgystan", "Киргизиа"}, {"Lanka", "Шри-Ланка"}, {"Line_Islands", "ад-хақәа Лаин"}, {"Lord_Howe", "Лорд-Ҳау, астандартә аамҭа"}, {"Macau", "Макао, астандартә аамҭа"}, {"Magadan", "Магадан, астандартә аамҭа"}, {"Malaysia", "Малаизиа"}, {"Maldives", "Мальдив"}, {"Marquesas", "Маркизтәи ад-хақәа"}, {"Marshall_Islands", "Маршаллтәи Адгьылбжьахақәа"}, {"Mauritius", "Маврики, астандартә аамҭа"}, {"Mawson", "Моусон"}, {"Mexico_Pacific", "Аокеанҭынчтәи амексикатә астандартә аамҭа"}, {"Mongolia", "Улан-Батор, астандартә аамҭа"}, {"Moscow", "Москва, астандартә аамҭа"}, {"Myanmar", "Мианма"}, {"Nauru", "Науру"}, {"Nepal", "Непал"}, {"New_Caledonia", "Каледониа ҿыц, астандартә аамҭа"}, {"New_Zealand", "Зеландиа ҿыц, астандартә аамҭа"}, {"Newfoundland", "Ниуфаундленд, астандартә аамҭа"}, {"Niue", "Ниуе"}, {"Norfolk", "Норфолк, астандартә аамҭа"}, {"Noronha", "Фернанду-ди-Норониа, астандартә аамҭа"}, {"North_Mariana", "Аҩадатәи Мариантәи ад-хақәа"}, {"Novosibirsk", "Новосибирск, астандартә аамҭа"}, {"Omsk", "Омск, астандартә аамҭа"}, {"Pakistan", "Пакистан, астандартә аамҭа"}, {"Palau", "Палау"}, {"Papua_New_Guinea", "Папуа — Гвинеиа ҿыц"}, {"Paraguay", "Парагваи, астандартә аамҭа"}, {"Peru", "Перу, астандартә аамҭа"}, {"Philippines", "Филиппин, астандартә аамҭа"}, {"Phoenix_Islands", "ад-хақәа Феникс"}, {"Pierre_Miquelon", "Сен-Пиери Микелони, астандартә аамҭа"}, {"Pitcairn", "Питкерн"}, {"Ponape", "Понпеи"}, {"Pyongyang", "Пхениан"}, {"Qyzylorda", "Кызылорда, астандартә аамҭа*"}, {"Reunion", "Реиунон"}, {"Rothera", "Ротера"}, {"Sakhalin", "Сахалин, астандартә аамҭа"}, {"Samara", "Самартәи астандартә аамҭа"}, {"Samoa", "Самоа, астандартә аамҭа"}, {"Seychelles", "Сеишелтәи адгьылбжьахақәа"}, {"Singapore", "Сингапур"}, {"Solomon", "Соломонтәи адгьылбжьахақәа"}, {"South_Georgia", "Аладатәи Георгиа"}, {"Suriname", "Суринам"}, {"Syowa", "Сиова"}, {"Tahiti", "Таити"}, {"Taipei", "Таиван, астандартә аамҭа"}, {"Tajikistan", "Таџьықсҭан"}, {"Tokelau", "Токелау"}, {"Tonga", "Тонга, астандартә аамҭа"}, {"Truk", "Трук"}, {"Turkmenistan", "Туркменистан, астандартә аамҭа"}, {"Tuvalu", "Тувалу"}, {"Uruguay", "Уругваи, астандартә аамҭа"}, {"Uzbekistan", "Узбеқьисҭан, астандартә аамҭа"}, {"Vanuatu", "Вануату, астандартә аамҭа"}, {"Venezuela", "Венесуела"}, {"Vladivostok", "Владивосток, астандартә аамҭа"}, {"Volgograd", "Волгоград, астандартә аамҭа"}, {"Vostok", "Амрагылара"}, {"Wake", "Уеик"}, {"Wallis", "Уоллиси Футунеи"}, {"Yakutsk", "Иакутск, астандартә аамҭа"}, {"Yekaterinburg", "Екатеринбург, астандартә аамҭа"}, {"Yukon", "Иукон"}},
		{{"Acre", "Акри аԥхынтәи аамҭа"}, {"Alaska", "Алиаска, аԥхынтәи аамҭа"}, {"Almaty", "Алма-Ата аԥхынтәи аамҭа"}, {"Amazon", "Амазонка, аԥхынтәи аамҭа"}, {"America_Central", "Агәҭантәи Америка, аԥхынтәи аамҭа"}, {"America_Eastern", "Мрагыларатәи Америка, аԥхынтәи аамҭа"}, {"America_Mountain", "Аԥхынтәи ашьхатә аамҭа (Аҩадатәи Америка)"}, {"America_Pacific", "Аокеанҭынчтәи аԥхынтәи аамҭа"}, {"Anadyr", "Анадыр аԥхынтәи аамҭа"}, {"Apia", "Апиа, аԥхынтәи аамҭа"}, {"Aqtau", "Актау аԥхынтәи аамҭа"}, {"Aqtobe", "Актобе аԥхынтәи аамҭа"}, {"Arabian", "Саудтәи Арабсҭан, аԥхынтәи аамҭа"}, {"Argentina", "Аргентина, аԥхынтәи аамҭа"}, {"Argentina_Western", "Мраҭашәаратәи Аргентина, аԥхынтәи аамҭа"}, {"Armenia", "Ермантәыла, аԥхынтәи аамҭа"}, {"Atlantic", "Атлантикатәи аԥхынтәи аамҭа"}, {"Australia_Central", "Агәҭантәи Австралиа, аԥхынтәи аамҭа"}, {"Australia_CentralWestern", "Агәҭантәи Австралиа, мраҭашәаратәи аԥхынтәи аамҭа"}, {"Australia_Eastern", "Мрагыларатәи Австралиа, аԥхынтәи аамҭа"}, {"Australia_Western", "Мраҭашәаратәи Австралиа, аԥхынтәи аамҭа"}, {"Azerbaijan", "Азербаиџьан, аԥхынтәи аамҭа"}, {"Azores", "Азортәи ад-хақәа, аԥхынтәи аамҭа"}, {"Bangladesh", "Бангладеш, аԥхынтәи аамҭа"}, {"Brasilia", "Бразилиа, аԥхынтәи аамҭа"}, {"Cape_Verde", "Кабо-Верде, аԥхынтәи аамҭа"}, {"Chatham", "Чатем, аԥхынтәи аамҭа"}, {"Chile", "Чили, аԥхынтәи аамҭа"}, {"China", "Китаи, аԥхынтәи аамҭа"}, {"Colombia", "Колумбиа, аԥхынтәи аамҭа"}, {"Cook", "Кук идгьылбжьахақәа, полуаԥхынтәи аамҭа"}, {"Cuba", "Куба, аԥхынтәи аамҭа"}, {"Easter", "ад-ха Пасхи, аԥхынтәи аамҭа"}, {"Europe_Central", "Агәҭантәи Европа, аԥхынтәи аамҭа"}, {"Europe_Eastern", "Мрагыларатәи Европа, аԥхынтәи аамҭа"}, {"Europe_Western", "Мраҭашәаратәи Европа, аԥхынтәи аамҭа"}, {"Falkland", "Фолклендтәи ад-хақәа, аԥхынтәи аамҭа"}, {"Fiji", "Фиџи, аԥхынтәи аамҭа"}, {"Georgia", "Қырҭтәыла, аԥхынтәи аамҭа"}, {"Greenland_Eastern", "Мрагыларатәи Гренландиа, аԥхынтәи аамҭа"}, {"Greenland_Western", "Мраҭашәаратәи Гренландиа, аԥхынтәи аамҭа"}, {"Hawaii_Aleutian", "Ҳаваи-алеуттәи аԥхынтәи аамҭа"}, {"Hong_Kong", "Ҳонконг, аԥхынтәи аамҭа"}, {"Hovd", "Ховд, аԥхынтәи аамҭа"}, {"Iran", "Иран, аԥхынтәи аамҭа"}, {"Irkutsk", "Иркутск, аԥхынтәи аамҭа"}, {"Israel", "Израиль, аԥхынтәи аамҭа"}, {"Japan", "Иапониа, аԥхынтәи аамҭа"}, {"Kamchatka", "Петропавловск-Камчаткатәи, аԥхынтәи аамҭа"}, {"Korea", "Кореиа, аԥхынтәи аамҭа"}, {"Krasnoyarsk", "Красноиарск, аԥхынтәи аамҭа"}, {"Lord_Howe", "Лорд-Ҳау, аԥхынтәи аамҭа"}, {"Macau", "Макао, аԥхынтәи аамҭа"}, {"Magadan", "Магадан, аԥхынтәи аамҭа"}, {"Mauritius", "Маврики, аԥхынтәи аамҭа"}, {"Mexico_Pacific", "Аокеанҭынчтәи амексикатә аԥхынтәи аамҭа"}, {"Mongolia", "Улан-Батор, аԥхынтәи аамҭа"}, {"Moscow", "Москва, аԥхынтәи аамҭа"}, {"New_Caledonia", "Каледониа ҿыц, аԥхынтәи аамҭа"}, {"New_Zealand", "Зеландиа ҿыц, аԥхынтәи аамҭа"}, {"Newfoundland", "Ниуфаундленд, аԥхынтәи аамҭа"}, {"Norfolk", "Норфолк, аԥхынтәи аамҭа"}, {"Noronha", "Фернанду-ди-Норониа, аԥхынтәи аамҭа"}, {"Novosibirsk", "Новосибирск, аԥхынтәи аамҭа"}, {"Omsk", "Омск, аԥхынтәи аамҭа"}, {"Pakistan", "Пакистан, аԥхынтәи аамҭа"}, {"Paraguay", "Парагваи, аԥхынтәи аамҭа"}, {"Peru", "Перу, аԥхынтәи аамҭа"}, {"Philippines", "Филиппин, аԥхынтәи аамҭа"}, {"Pierre_Miquelon", "Сен-Пиери Микелони, аԥхынтәи аамҭа"}, {"Qyzylorda", "Кызылорда, аԥхынтәи аамҭа*"}, {"Sakhalin", "Сахалин, аԥхынтәи аамҭа"}, {"Samara", "Самартәи аԥхынтәи аамҭа"}, {"Samoa", "Самоа, аԥхынтәи аамҭа"}, {"Taipei", "Таиван, аԥхынтәи аамҭа"}, {"Tonga", "Тонга, аԥхынтәи аамҭа"}, {"Turkmenistan", "Туркменистан, аԥхынтәи аамҭа"}, {"Uruguay", "Уругваи, аԥхынтәи аамҭа"}, {"Uzbekistan", "Узбеқьисҭан, аԥхынтәи аамҭа"}, {"Vanuatu", "Вануату, аԥхынтәи аамҭа"}, {"Vladivostok", "Владивосток, аԥхынтәи аамҭа"}, {"Volgograd", "Волгоград, аԥхынтәи аамҭа"}, {"Yakutsk", "Иакутск, аԥхынтәи аамҭа"}, {"Yekaterinburg", "Екатеринбург, аԥхынтәи аамҭа"}},
	},
	{
		{{"Afghanistan", "Afganistan-tyd"}, {"Africa_Central", "Sentraal-Afrika-tyd"}, {"Africa_Eastern", "Oos-Afrika-tyd"}, {"Africa_Southern", "Suid-Afrika-standaardtyd"}, {"Africa_Western", "Wes-Afrika-tyd"}, {"Alaska", "Alaska-standaardtyd"}, {"Amazon", "Amasone-standaardtyd"}, {"America_Central", "Noord-Amerikaanse sentrale standaardtyd"}, {"America_Eastern", "Noord-Amerikaanse oostelike standaardtyd"}, {"America_Mountain", "Noord-Amerikaanse berg-standaardtyd"}, {"America_Pacific", "Pasifiese standaardtyd"}, {"Anadyr", "Anadyr-standaardtyd"}, {"Apia", "Apia-standaardtyd"}, {"Arabian", "Arabiese standaardtyd"}, {"Argentina", "Argentinië-standaardtyd"}, {"Argentina_Western", "Wes-Argentinië-standaardtyd"}, {"Armenia", "Armenië-standaardtyd"}, {"Atlantic", "Atlantiese standaardtyd"}, {"Australia_Central", "Sentraal-Australiese standaardtyd"}, {"Australia_CentralWestern", "Sentraal-westelike Australiese standaard-tyd"}, {"Australia_Eastern", "Oostelike Australiese standaardtyd"}, {"Australia_Western", "Westelike Australiese standaardtyd"}, {"Azerbaijan", "Azerbeidjan-standaardtyd"}, {"Azores", "Asore-standaardtyd"}, {"Bangladesh", "Bangladesj-standaardtyd"}, {"Bhutan", "Bhoetan-tyd"}, {"Bolivia", "Bolivië-tyd"}, {"Brasilia", "Brasilia-standaardtyd"}, {"Brunei", "Broenei Darussalam-tyd"}, {"Cape_Verde", "Kaap Verde-standaardtyd"}, {"Chamorro", "Chamorro-standaardtyd"}, {"Chatham", "Chatham-standaardtyd"}, {"Chile", "Chili-standaardtyd"}, {"China", "China-standaardtyd"}, {"Christmas", "Christmaseiland-tyd"}, {"Cocos", "Kokoseilande-tyd"}, {"Colombia", "Colombië-standaardtyd"}, {"Cook", "Cookeilande-standaardtyd"}, {"Cuba", "Kuba-standaardtyd"}, {"Davis", "Davis-tyd"}, {"DumontDUrville", "Dumont d’Urville-tyd"}, {"East_Timor", "Oos-Timor-tyd"}, {"Easter", "Paaseiland-standaardtyd"}, {"Ecuador", "Ecuador-tyd"}, {"Europe_Central", "Sentraal-Europese standaardtyd"}, {"Europe_Eastern", "Oos-Europese standaardtyd"}, {"Europe_Further_Eastern", "Verder-oos-Europese tyd"}, {"Europe_Western", "Wes-Europese standaardtyd"}, {"Falkland", "Falklandeilande-standaardtyd"}, {"Fiji", "Fidji-standaardtyd"}, {"French_Guiana", "Frans-Guiana-tyd"}, {"French_Southern", "Franse Suider- en Antarktiese tyd"}, {"GMT", "Greenwich-tyd"}, {"Galapagos", "Galapagos-tyd"}, {"Gambier", "Gambier-tyd"}, {"Georgia", "Georgië-standaardtyd"}, {"Gilbert_Islands", "Gilberteilande-tyd"}, {"Greenland_Eastern", "Oos-Groenland-standaardtyd"}, {"Greenland_Western", "Wes-Groenland-standaardtyd"}, {"Gulf", "Persiese Golf-standaardtyd"}, {"Guyana", "Guiana-tyd"}, {"Hawaii", "Hawaii-Aleoete-standaardtyd"}, {"Hawaii_Aleutian", "Hawaii-Aleoete-standaardtyd"}, {"Hong_Kong", "Hongkong-standaardtyd"}, {"Hovd", "Hovd-standaardtyd"}, {"India", "Indië-standaardtyd"}, {"Indian_Ocean", "Indiese Oseaan-tyd"}, {"Indochina", "Indosjina-tyd"}, {"Indonesia_Central", "Sentraal-Indonesiese tyd"}, {"Indonesia_Eastern", "Oos-Indonesië-tyd"}, {"Indonesia_Western", "Wes-Indonesië-tyd"}, {"Iran", "Iran-standaardtyd"}, {"Irkutsk", "Irkoetsk-standaardtyd"}, {"Israel", "Israel-standaardtyd"}, {"Japan", "Japan-standaardtyd"}, {"Kamchatka", "Petropavlovsk-Kamchatski-standaardtyd"}, {"Kazakhstan", "Kazakstan-tyd"}, {"Kazakhstan_Eastern", "Oos-Kazakstan-tyd"}, {"Kazakhstan_Western", "Wes-Kazakstan-tyd"}, {"Korea", "Koreaanse standaardtyd"}, {"Kosrae", "Kosrae-tyd"}, {"Krasnoyarsk", "Krasnojarsk-standaardtyd"}, {"Kyrgystan", "Kirgistan-tyd"}, {"Line_Islands", "Line-eilande-tyd"}, {"Lord_Howe", "Lord Howe-standaardtyd"}, {"Magadan", "Magadan-standaardtyd"}, {"Malaysia", "Maleisië-tyd"}, {"Maldives", "Maledive-tyd"}, {"Marquesas", "Marquesas-tyd"}, {"Marshall_Islands", "Marshalleilande-tyd"}, {"Mauritius", "Mauritius-standaardtyd"}, {"Mawson", "Mawson-tyd"}, {"Mexico_Pacific", "Meksikaanse Pasifiese standaardtyd"}, {"Mongolia", "Ulaanbaatar-standaardtyd"}, {"Moscow", "Moskou-standaardtyd"}, {"Myanmar", "Mianmar-tyd"}, {"Nauru", "Nauru-tyd"}, {"Nepal", "Nepal-tyd"}, {"New_Caledonia", "Nieu-Kaledonië-standaardtyd"}, {"New_Zealand", "Nieu-Seeland-standaardtyd"}, {"Newfoundland", "Newfoundland-standaardtyd"}, {"Niue", "Niue-tyd"}, {"Norfolk", "Norfolkeiland-standaardtyd"}, {"Noronha", "Fernando de Noronha-standaardtyd"}, {"Novosibirsk", "Novosibirsk-standaardtyd"}, {"Omsk", "Omsk-standaardtyd"}, {"Pakistan", "Pakistan-standaardtyd"}, {"Palau", "Palau-tyd"}, {"Papua_New_Guinea", "Papoea-Nieu-Guinee-tyd"}, {"Paraguay", "Paraguay-standaardtyd"}, {"Peru", "Peru-standaardtyd"}, {"Philippines", "Filippynse standaardtyd"}, {"Phoenix_Islands", "Fenikseilande-tyd"}, {"Pierre_Miquelon", "Sint-Pierre en Miquelon-standaardtyd"}, {"Pitcairn", "Pitcairn-tyd"}, {"Ponape", "Ponape-tyd"}, {"Pyongyang", "Pyongyang-tyd"}, {"Reunion", "Réunion-tyd"}, {"Rothera", "Rothera-tyd"}, {"Sakhalin", "Sakhalin-standaardtyd"}, {"Samara", "Samara-standaardtyd"}, {"Samoa", "Samoa-standaardtyd"}, {"Seychelles", "Seychelle-tyd"}, {"Singapore", "Singapoer-standaardtyd"}, {"Solomon", "Salomonseilande-tyd"}, {"South_Georgia", "Suid-Georgië-tyd"}, {"Suriname", "Suriname-tyd"}, {"Syowa", "Syowa-tyd"}, {"Tahiti", "Tahiti-tyd"}, {"Taipei", "Taipei-standaardtyd"}, {"Tajikistan", "Tadjikistan-tyd"}, {"Tokelau", "Tokelau-tyd"}, {"Tonga", "Tonga-standaardtyd"}, {"Truk", "Chuuk-tyd"}, {"Turkmenistan", "Turkmenistan-standaardtyd"}, {"Tuvalu", "Tuvalu-tyd"}, {"Uruguay", "Uruguay-standaardtyd"}, {"Uzbekistan", "Oesbekistan-standaardtyd"}, {"Vanuatu", "Vanuatu-standaardtyd"}, {"Venezuela", "Venezuela-tyd"}, {"Vladivostok", "Wladiwostok-standaardtyd"}, {"Volgograd", "Wolgograd-standaardtyd"}, {"Vostok", "Wostok-tyd"}, {"Wake", "Wake-eiland-tyd"}, {"Wallis", "Wallis en Futuna-tyd"}, {"Yakutsk", "Jakoetsk-standaardtyd"}, {"Yekaterinburg", "Jekaterinburg-standaardtyd"}, {"Yukon", "Yukontyd"}},
		{{"Alaska", "Alaska-dagligtyd"}, {"Amazon", "Amasone-somertyd"}, {"America_Central", "Noord-Amerikaanse sentrale dagligtyd"}, {"America_Eastern", "Noord-Amerikaanse oostelike dagligtyd"}, {"America_Mountain", "Noord-Amerikaanse berg-dagligtyd"}, {"America_Pacific", "Pasifiese dagligtyd"}, {"Anadyr", "Anadyr-somertyd"}, {"Apia", "Apia-dagligtyd"}, {"Arabian", "Arabiese dagligtyd"}, {"Argentina", "Argentinië-somertyd"}, {"Argentina_Western", "Wes-Argentinië-somertyd"}, {"Armenia", "Armenië-somertyd"}, {"Atlantic", "Atlantiese dagligtyd"}, {"Australia_Central", "Sentraal-Australiese dagligtyd"}, {"Australia_CentralWestern", "Sentraal-westelike Australiese dagligtyd"}, {"Australia_Eastern", "Oostelike Australiese dagligtyd"}, {"Australia_Western", "Westelike Australiese dagligtyd"}, {"Azerbaijan", "Azerbeidjan-somertyd"}, {"Azores", "Asore-somertyd"}, {"Bangladesh", "Bangladesj-somertyd"}, {"Brasilia", "Brasilia-somertyd"}, {"Cape_Verde", "Kaap Verde-somertyd"}, {"Chatham", "Chatham-dagligtyd"}, {"Chile", "Chili-somertyd"}, {"China", "China-dagligtyd"}, {"Colombia", "Colombië-somertyd"}, {"Cook", "Cookeilande-halfsomertyd"}, {"Cuba", "Kuba-dagligtyd"}, {"Easter", "Paaseiland-somertyd"}, {"Europe_Central", "Sentraal-Europese somertyd"}, {"Europe_Eastern", "Oos-Europese somertyd"}, {"Europe_Western", "Wes-Europese somertyd"}, {"Falkland", "Falklandeilande-somertyd"}, {"Fiji", "Fidji-somertyd"}, {"Georgia", "Georgië-somertyd"}, {"Greenland_Eastern", "Oos-Groenland-somertyd"}, {"Greenland_Western", "Wes-Groenland-somertyd"}, {"Hawaii_Aleutian", "Hawaii-Aleoete-dagligtyd"}, {"Hong_Kong", "Hongkong-somertyd"}, {"Hovd", "Hovd-somertyd"}, {"Iran", "Iran-dagligtyd"}, {"Irkutsk", "Irkoetsk-somertyd"}, {"Israel", "Israel-dagligtyd"}, {"Japan", "Japan-dagligtyd"}, {"Kamchatka", "Petropavlovsk-Kamchatski-somertyd"}, {"Korea", "Koreaanse dagligtyd"}, {"Krasnoyarsk", "Krasnojarsk-somertyd"}, {"Lord_Howe", "Lord Howe-dagligtyd"}, {"Magadan", "Magadan-somertyd"}, {"Mauritius", "Mauritius-somertyd"}, {"Mexico_Pacific", "Meksikaanse Pasifiese dagligtyd"}, {"Mongolia", "Ulaanbaatar-somertyd"}, {"Moscow", "Moskou-somertyd"}, {"New_Caledonia", "Nieu-Kaledonië-somertyd"}, {"New_Zealand", "Nieu-Seeland-dagligtyd"}, {"Newfoundland", "Newfoundland-dagligtyd"}, {"Norfolk", "Norfolkeiland-dagligtyd"}, {"Noronha", "Fernando de Noronha-somertyd"}, {"Novosibirsk", "Novosibirsk-somertyd"}, {"Omsk", "Omsk-somertyd"}, {"Pakistan", "Pakistan-somertyd"}, {"Paraguay", "Paraguay-somertyd"}, {"Peru", "Peru-somertyd"}, {"Philippines", "Filippynse somertyd"}, {"Pierre_Miquelon", "Sint-Pierre en Miquelon-dagligtyd"}, {"Sakhalin", "Sakhalin-somertyd"}, {"Samara", "Samara-dagligtyd"}, {"Samoa", "Samoa-dagligtyd"}, {"Taipei", "Taipei-dagligtyd"}, {"Tonga", "Tonga-somertyd"}, {"Turkmenistan", "Turkmenistan-somertyd"}, {"Uruguay", "Uruguay-somertyd"}, {"Uzbekistan", "Oesbekistan-somertyd"}, {"Vanuatu", "Vanuatu-somertyd"}, {"Vladivostok", "Wladiwostok-somertyd"}, {"Volgograd", "Wolgograd-somertyd"}, {"Yakutsk", "Jakoetsk-somertyd"}, {"Yekaterinburg", "Jekaterinburg-somertyd"}},
	},
	{
		{{"Afghanistan", "Afganistan Berɛ"}, {"Africa_Central", "Afrika Finimfin Berɛ"}, {"Africa_Eastern", "Afrika Apueeɛ Berɛ"}, {"Africa_Southern", "Afrika Anaafoɔ Susudua Berɛ"}, {"Africa_Western", "Afrika Atɔeɛ Berɛ"}, {"Alaska", "Alaska Susudua Berɛ"}, {"Amazon", "Amazon Susudua Berɛ"}, {"America_Central", "Mfinimfini Susudua Berɛ"}, {"America_Eastern", "Apueeɛ Susudua Berɛ"}, {"America_Mountain", "Bepɔ Susudua Berɛ"}, {"America_Pacific", "Pasifik Susudua Berɛ"}, {"Apia", "Apia Susudua Berɛ"}, {"Arabian", "Arabia Susudua Berɛ"}, {"Argentina", "Agyɛntina Susudua Berɛ"}, {"Argentina_Western", "Agyɛntina Atɔeeɛ Susudua Berɛ"}, {"Armenia", "Aamenia Susudua Berɛ"}, {"Atlantic", "Atlantik Susudua Berɛ"}, {"Australia_Central", "Ɔstrelia Mfinimfini Susudua Berɛ"}, {"Australia_CentralWestern", "Ɔstrelia Mfinimfini Atɔeeɛ Susudua Berɛ"}, {"Australia_Eastern", "Ɔstrelia Apueeɛ Susudua Berɛ"}, {"Australia_Western", "Ɔstrelia Atɔeeɛ Susudua Berɛ"}, {"Azerbaijan", "Asabegyan Susudua Berɛ"}, {"Azores", "Azores Susudua Berɛ"}, {"Bangladesh", "Bangladɛhye Susudua Berɛ"}, {"Bhutan", "Butan Berɛ"}, {"Bolivia", "Bolivia Berɛ"}, {"Brasilia", "Brasilia Susudua Berɛ"}, {"Brunei", "Brunei Berɛ"}, {"Cape_Verde", "Kepvɛde Susudua Berɛ"}, {"Chamorro", "Kyamoro Susudua Berɛ"}, {"Chatham", "Kyatam Susudua Berɛ"}, {"Chile", "Kyili Susudua Berɛ"}, {"China", "Kyaena Susudua Berɛ"}, {"Christmas", "Buronya Aeland Berɛ"}, {"Cocos", "Kokoso Aeland Berɛ"}, {"Colombia", "Kolombia Susudua Berɛ"}, {"Cook", "Kuk Aeland Susudua Berɛ"}, {"Cuba", "Kuba Susudua Berɛ"}, {"Davis", "Davis Berɛ"}, {"DumontDUrville", "Dumont-d’Urville Berɛ"}, {"East_Timor", "Timɔɔ Apueeɛ Berɛ"}, {"Easter", "Easta Aeland Susudua Berɛ"}, {"Ecuador", "Yikuwedɔ Berɛ"}, {"Europe_Central", "Yuropu Mfinimfini Susudua Berɛ"}, {"Europe_Eastern", "Yuropu Apueeɛ Susudua Berɛ"}, {"Europe_Further_Eastern", "Yuropu Apueeɛ Nohoa Berɛ"}, {"Europe_Western", "Yuropu Atɔeeɛ Susudua Berɛ"}, {"Falkland", "Fɔkman Aeland Susudua Berɛ"}, {"Fiji", "Figyi Susudua Berɛ"}, {"French_Guiana", "Frɛnkye Gayana Berɛ"}, {"French_Southern", "Frɛnkye Anaafoɔ ne Antaatik Berɛ"}, {"GMT", "Greenwich Mean Berɛ"}, {"Galapagos", "Galapagɔs Berɛ"}, {"Gambier", "Gambier Berɛ"}, {"Georgia", "Gyɔgyea Susudua Berɛ"}, {"Gilbert_Islands", "Geebɛt Aeland Berɛ"}, {"Greenland_Eastern", "Greenland Apueeɛ Susudua Berɛ"}, {"Greenland_Western", "Greenland Atɔeɛ Susudua Berɛ"}, {"Gulf", "Gɔɔfo Susudua Berɛ"}, {"Guyana", "Gayana Berɛ"}, {"Hawaii", "Hawaii-Aleutian Susudua Berɛ"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Susudua Berɛ"}, {"Hong_Kong", "Hɔnkɔn Susudua Berɛ"}, {"Hovd", "Hovd Susudua Berɛ"}, {"India", "India Susudua Berɛ"}, {"Indian_Ocean", "India Po Berɛ"}, {"Indochina", "Indɔkyina Berɛ"}, {"Indonesia_Central", "Indɔnehyia Mfinimfini Berɛ"}, {"Indonesia_Eastern", "Indɔnehyia Apueeɛ Berɛ"}, {"Indonesia_Western", "Indɔnehyia Atɔeeɛ Berɛ"}, {"Iran", "Iran Susudua Berɛ"}, {"Irkutsk", "Yiikusk Susudua Berɛ"}, {"Israel", "Israel Susudua Berɛ"}, {"Japan", "Gyapan Susudua Berɛ"}, {"Kazakhstan", "Kazakstan Berɛ"}, {"Kazakhstan_Eastern", "Kazakstan Apueeɛ Berɛ"}, {"Kazakhstan_Western", "Kazakstan Atɔeɛ Berɛ"}, {"Korea", "Korean Susudua Berɛ"}, {"Kosrae", "Kosrae Berɛ"}, {"Krasnoyarsk", "Krasnoyarsk Susudua Berɛ"}, {"Kyrgystan", "Kɛɛgestan Berɛ"}, {"Line_Islands", "Lai Aeland Berɛ"}, {"Lord_Howe", "Lɔd Howe Susudua Berɛ"}, {"Magadan", "Magadan Susudua Berɛ"}, {"Malaysia", "Malehyia Berɛ"}, {"Maldives", "Maldives Berɛ"}, {"Marquesas", "Makesase Berɛ"}, {"Marshall_Islands", "Mahyaa Aeland Berɛ"}, {"Mauritius", "Mɔrihyiɔso Susudua Berɛ"}, {"Mawson", "Mɔɔson Berɛ"}, {"Mexico_Pacific", "Mɛksiko Pasifik Susudua Berɛ"}, {"Mongolia", "Yulanbata Susudua Berɛ"}, {"Moscow", "Mɔsko Susudua Berɛ"}, {"Myanmar", "Mayaama Berɛ"}, {"Nauru", "Nauru Berɛ"}, {"Nepal", "Nɛpal Berɛ"}, {"New_Caledonia", "Kaledonia Foforɔ Susudua Berɛ"}, {"New_Zealand", "Ziland Foforɔ Susudua Berɛ"}, {"Newfoundland", "Newfoundland Susudua Berɛ"}, {"Niue", "Niue Berɛ"}, {"Norfolk", "Nɔɔfɔk Aeland Susudua Berɛ"}, {"Noronha", "Fernando de Noronha Susudua Berɛ"}, {"Novosibirsk", "Novosibirsk Susudua Berɛ"}, {"Omsk", "Omsk Susudua Berɛ"}, {"Pakistan", "Pakistan Susudua Berɛ"}, {"Palau", "Palau Berɛ"}, {"Papua_New_Guinea", "Papua Gini Foforɔ Berɛ"}, {"Paraguay", "Paraguae Susudua Berɛ"}, {"Peru", "Peru Susudua Berɛ"}, {"Philippines", "Filipin Susudua Berɛ"}, {"Phoenix_Islands", "Finise Aeland Berɛ"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Susudua Berɛ"}, {"Pitcairn", "Pitkairn Berɛ"}, {"Ponape", "Ponape Berɛ"}, {"Pyongyang", "Pyongyang Berɛ"}, {"Reunion", "Réunion Berɛ"}, {"Rothera", "Rotera Berɛ"}, {"Sakhalin", "Sakhalin Susudua Berɛ"}, {"Samoa", "Samoa Susudua Berɛ"}, {"Seychelles", "Seyhyɛl Berɛ"}, {"Singapore", "Singapɔ Susudua Berɛ"}, {"Solomon", "Solomon Aeland Berɛ"}, {"South_Georgia", "Gyɔɔgyia Anaafoɔ Berɛ"}, {"Suriname", "Suriname Berɛ"}, {"Syowa", "Syowa Berɛ"}, {"Tahiti", "Tahiti Berɛ"}, {"Taipei", "Taipei Susudua Berɛ"}, {"Tajikistan", "Tagyikistan Berɛ"}, {"Tokelau", "Tokelau Berɛ"}, {"Tonga", "Tonga Susudua Berɛ"}, {"Truk", "Kyuuk Berɛ"}, {"Turkmenistan", "Tɛkmɛnistan Susudua Berɛ"}, {"Tuvalu", "Tuvalu Berɛ"}, {"Uruguay", "Yurugwae Susudua Berɛ"}, {"Uzbekistan", "Usbɛkistan Susudua Berɛ"}, {"Vanuatu", "Vanuatu Susudua Berɛ"}, {"Venezuela", "Venezuela Berɛ"}, {"Vladivostok", "Vladivostok Susudua Berɛ"}, {"Volgograd", "Volgograd Susudua Berɛ"}, {"Vostok", "Vostok Berɛ"}, {"Wake", "Wake Aeland Berɛ"}, {"Wallis", "Wallis ne Futuna Berɛ"}, {"Yakutsk", "Yakutsk Susudua Berɛ"}, {"Yekaterinburg", "Yɛkatɛrinbɛg Susudua Berɛ"}, {"Yukon", "Yukɔn Berɛ"}},
		{{"Alaska", "Alaska Awia Berɛ"}, {"Amazon", "Amazon Awia Berɛ"}, {"America_Central", "Mfinimfini Awia Berɛ"}, {"America_Eastern", "Apueeɛ Awia Berɛ"}, {"America_Mountain", "Bepɔ Awia Berɛ"}, {"America_Pacific", "Pasifik Awia Berɛ"}, {"Apia", "Apia Awia Berɛ"}, {"Arabian", "Arabia Awia Berɛ"}, {"Argentina", "Agyɛntina Awia Berɛ"}, {"Argentina_Western", "Agyɛntina Atɔeeɛ Awia Berɛ"}, {"Armenia", "Aamenia Awia Berɛ"}, {"Atlantic", "Atlantik Awia Berɛ"}, {"Australia_Central", "Ɔstrelia Mfinimfini Awia Berɛ"}, {"Australia_CentralWestern", "Ɔstrelia Mfinimfini Atɔeeɛ Awia Berɛ"}, {"Australia_Eastern", "Ɔstrelia Apueeɛ Awia Berɛ"}, {"Australia_Western", "Ɔstrelia Atɔeeɛ Awia Berɛ"}, {"Azerbaijan", "Asabegyan Awia Berɛ"}, {"Azores", "Azores Awia Berɛ"}, {"Bangladesh", "Bangladɛhye Awia Berɛ"}, {"Brasilia", "Brasilia Awia Berɛ"}, {"Cape_Verde", "Kepvɛde Awia Berɛ"}, {"Chatham", "Kyatam Awia Berɛ"}, {"Chile", "Kyili Awia Berɛ"}, {"China", "Kyaena Awia Berɛ"}, {"Colombia", "Kolombia Awia Berɛ"}, {"Cook", "Kuk Aeland Awia Fa Berɛ"}, {"Cuba", "Kuba Awia Berɛ"}, {"Easter", "Easta Aeland Awia Berɛ"}, {"Europe_Central", "Yuropu Mfinimfini Awia Berɛ"}, {"Europe_Eastern", "Yuropu Apueeɛ Awia Berɛ"}, {"Europe_Western", "Yuropu Atɔeeɛ Awia Berɛ"}, {"Falkland", "Fɔkman Aeland Awia Berɛ"}, {"Fiji", "Figyi Awia Berɛ"}, {"Georgia", "Gyɔgyea Awia Berɛ"}, {"Greenland_Eastern", "Greenland Apueeɛ Awia Berɛ"}, {"Greenland_Western", "Greenland Atɔeɛ Awia Berɛ"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Awia Berɛ"}, {"Hong_Kong", "Hɔnkɔn Awia Berɛ"}, {"Hovd", "Hovd Awia Berɛ"}, {"Iran", "Iran Awia Berɛ"}, {"Irkutsk", "Yiikusk Awia Berɛ"}, {"Israel", "Israel Awia Berɛ"}, {"Japan", "Gyapan Awia Berɛ"}, {"Korea", "Korean Awia Berɛ"}, {"Krasnoyarsk", "Krasnoyarsk Awia Berɛ"}, {"Lord_Howe", "Lɔd Howe Awia Berɛ"}, {"Magadan", "Magadan Awia Berɛ"}, {"Mauritius", "Mɔrihyiɔso Awia Berɛ"}, {"Mexico_Pacific", "Mɛksiko Pasifik Awia Berɛ"}, {"Mongolia", "Yulanbata Awia Berɛ"}, {"Moscow", "Mɔsko Awia Berɛ"}, {"New_Caledonia", "Kaledonia Foforɔ Awia Berɛ"}, {"New_Zealand", "Ziland Foforɔ Awia Berɛ"}, {"Newfoundland", "Newfoundland Awia Berɛ"}, {"Norfolk", "Nɔɔfɔk Aeland Awia Berɛ"}, {"Noronha", "Fernando de Noronha Awia Berɛ"}, {"Novosibirsk", "Novosibirsk Awia Berɛ"}, {"Omsk", "Omsk Awia Berɛ"}, {"Pakistan", "Pakistan Awia Berɛ"}, {"Paraguay", "Paraguae Awia Berɛ"}, {"Peru", "Peru Awia Berɛ"}, {"Philippines", "Filipin Awia Berɛ"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Awia Berɛ"}, {"Sakhalin", "Sakhalin Awia Berɛ"}, {"Samoa", "Samoa Awia Berɛ"}, {"Taipei", "Taipei Awia Berɛ"}, {"Tonga", "Tonga Awia Berɛ"}, {"Turkmenistan", "Tɛkmɛnistan Awia Berɛ"}, {"Uruguay", "Yurugwae Awia Berɛ"}, {"Uzbekistan", "Usbɛkistan Awia Berɛ"}, {"Vanuatu", "Vanuatu Awia Berɛ"}, {"Vladivostok", "Vladivostok Awia Berɛ"}, {"Volgograd", "Volgograd Awia Berɛ"}, {"Yakutsk", "Yakutsk Awia Berɛ"}, {"Yekaterinburg", "Yɛkatɛrinbɛg Awia Berɛ"}},
	},
	{
		{{"Afghanistan", "የአፍጋኒስታን ሰዓት"}, {"Africa_Central", "የመካከለኛው አፍሪካ ሰዓት"}, {"Africa_Eastern", "የምስራቅ አፍሪካ ሰዓት"}, {"Africa_Southern", "የደቡብ አፍሪካ መደበኛ ሰዓት"}, {"Africa_Western", "የምዕራብ አፍሪካ ሰዓት"}, {"Alaska", "የአላስካ መደበኛ የሰዓት አቆጣጠር"}, {"Amazon", "የአማዞን መደበኛ ሰዓት አቆጣጠር"}, {"America_Central", "የሰሜን አሜሪካ የመካከለኛ መደበኛ ሰዓት አቆጣጠር"}, {"America_Eastern", "ምስራቃዊ መደበኛ ሰዓት አቆጣጠር"}, {"America_Mountain", "የተራራ መደበኛ የሰዓት አቆጣጠር"}, {"America_Pacific", "የፓስፊክ መደበኛ ሰዓት አቆጣጠር"}, {"Anadyr", "የአናዲይር ሰዓት አቆጣጠር"}, {"Apia", "የአፒያ መደበኛ ሰዓት"}, {"Arabian", "የዓረቢያ መደበኛ ሰዓት"}, {"Argentina", "የአርጀንቲና መደበኛ ሰዓት አቆጣጠር"}, {"Argentina_Western", "የምዕራባዊ አርጀንቲና መደበኛ ሰዓት አቆጣጠር"}, {"Armenia", "የአርመኒያ መደበኛ ሰዓት"}, {"Atlantic", "የአትላንቲክ መደበኛ የሰዓት አቆጣጠር"}, {"Australia_Central", "የአውስትራሊያ መካከለኛ መደበኛ የሰዓት አቆጣጠር"}, {"Australia_CentralWestern", "የአውስትራሊያ መካከለኛ ምስራቃዊ መደበኛ ሰዓት አቆጣጠር"}, {"Australia_Eastern", "የአውስትራሊያ ምዕራባዊ መደበኛ የሰዓት አቆጣጠር"}, {"Australia_Western", "የአውስትራሊያ ምስራቃዊ መደበኛ ሰዓት አቆጣጠር"}, {"Azerbaijan", "የአዘርባጃን መደበኛ ሰዓት"}, {"Azores", "የአዞረስ መደበኛ ሰዓት"}, {"Bangladesh", "የባንግላዴሽ መደበኛ ሰዓት"}, {"Bhutan", "የቡታን ሰዓት"}, {"Bolivia", "የቦሊቪያ ሰዓት"}, {"Brasilia", "የብራሲሊያ መደበኛ ሰዓት አቆጣጠር"}, {"Brunei", "የብሩኔይ ዳሩሳላም ሰዓት"}, {"Cape_Verde", "የኬፕ ቨርዴ መደበኛ ሰዓት"}, {"Chamorro", "የቻሞሮ መደበኛ ሰዓት"}, {"Chatham", "የቻታም መደበኛ ሰዓት"}, {"Chile", "የቺሊ መደበኛ ሰዓት"}, {"China", "የቻይና መደበኛ ሰዓት"}, {"Christmas", "የገና ደሴት ሰዓት"}, {"Cocos", "የኮኮስ ደሴቶች ሰዓት"}, {"Colombia", "የኮሎምቢያ መደበኛ ሰዓት"}, {"Cook", "የኩክ ደሴቶች መደበኛ ሰዓት"}, {"Cuba", "የኩባ መደበኛ ሰዓት"}, {"Davis", "የዴቪስ ሰዓት"}, {"DumontDUrville", "የዱሞንት-ዱርቪል ሰዓት"}, {"East_Timor", "የምስራቅ ቲሞር ሰዓት"}, {"Easter", "የኢስተር ደሴት መደበኛ ሰዓት"}, {"Ecuador", "የኢኳዶር ሰዓት"}, {"Europe_Central", "የመካከለኛው አውሮፓ መደበኛ ሰዓት"}, {"Europe_Eastern", "የምስራቃዊ አውሮፓ መደበኛ ሰዓት"}, {"Europe_Further_Eastern", "የሩቅ ምስራቅ የአውሮፓ ሰዓት"}, {"Europe_Western", "የምዕራባዊ አውሮፓ መደበኛ ሰዓት"}, {"Falkland", "የፋልክላንድ ደሴቶች መደበኛ ሰዓት"}, {"Fiji", "የፊጂ መደበኛ ሰዓት"}, {"French_Guiana", "የፈረንሳይ ጉያና ሰዓት"}, {"French_Southern", "የፈረንሳይ ደቡባዊ እና አንታርክቲክ ሰዓት"}, {"GMT", "ግሪንዊች ማዕከላዊ ሰዓት"}, {"Galapagos", "የጋላፓጎስ ሰዓት"}, {"Gambier", "የጋምቢየር ሰዓት"}, {"Georgia", "የጂዮርጂያ መደበኛ ሰዓት"}, {"Gilbert_Islands", "የጂልበርት ደሴቶች ሰዓት"}, {"Greenland_Eastern", "የምስራቅ ግሪንላንድ መደበኛ ሰዓት"}, {"Greenland_Western", "የምዕራብ ግሪንላንድ መደበኛ ሰዓት"}, {"Gulf", "የባህረሰላጤ መደበኛ ሰዓት"}, {"Guyana", "የጉያና ሰዓት"}, {"Hawaii", "የሃዋይ አሌኡት መደበኛ ሰዓት አቆጣጠር"}, {"Hawaii_Aleutian", "የሃዋይ አሌኡት መደበኛ ሰዓት አቆጣጠር"}, {"Hong_Kong", "የሆንግ ኮንግ መደበኛ ሰዓት"}, {"Hovd", "የሆቭድ መደበኛ የሰዓት አቆጣጠር"}, {"India", "የህንድ መደበኛ ሰዓት"}, {"Indian_Ocean", "የህንድ ውቅያኖስ ሰዓት"}, {"Indochina", "የኢንዶቻይና ሰዓት"}, {"Indonesia_Central", "የመካከለኛው ኢንዶኔዢያ ሰዓት"}, {"Indonesia_Eastern", "የምስራቃዊ ኢንዶኔዢያ ሰዓት"}, {"Indonesia_Western", "የምዕራባዊ ኢንዶኔዢያ ሰዓት"}, {"Iran", "የኢራን መደበኛ ሰዓት"}, {"Irkutsk", "የኢርኩትስክ መደበኛ የሰዓት አቆጣጠር"}, {"Israel", "የእስራኤል መደበኛ ሰዓት"}, {"Japan", "የጃፓን መደበኛ ሰዓት"}, {"Kamchatka", "የፔትሮፓቭሎስኪ - ካምቻትስኪ ሰዓት አቆጣጠር"}, {"Kazakhstan", "ካዛኪስታን ሰዓት"}, {"Kazakhstan_Eastern", "የምስራቅ ካዛኪስታን ሰዓት"}, {"Kazakhstan_Western", "የምዕራብ ካዛኪስታን ሰዓት"}, {"Korea", "የኮሪያ መደበኛ ሰዓት"}, {"Kosrae", "የኮስራኤ ሰዓት"}, {"Krasnoyarsk", "የክራስኖይአርስክ መደበኛ ሰዓት አቆጣጠር"}, {"Kyrgystan", "የኪርጊስታን ሰዓት"}, {"Line_Islands", "የላይን ደሴቶች ሰዓት"}, {"Lord_Howe", "የሎርድ ሆዌ መደበኛ የሰዓት አቆጣጠር"}, {"Magadan", "የማጋዳን መደበኛ ሰዓት አቆጣጠር"}, {"Malaysia", "የማሌይዢያ ሰዓት"}, {"Maldives", "የማልዲቭስ ሰዓት"}, {"Marquesas", "የማርኴሳስ ሰዓት"}, {"Marshall_Islands", "የማርሻል ደሴቶች ሰዓት"}, {"Mauritius", "የማውሪሺየስ መደበኛ ሰዓት"}, {"Mawson", "የማውሰን ሰዓት"}, {"Mexico_Pacific", "የሜክሲኮ ፓሲፊክ መደበኛ ሰዓት አቆጣጠር"}, {"Mongolia", "የኡላን ባቶር መደበኛ ሰዓት አቆጣጠር"}, {"Moscow", "የሞስኮ መደበኛ ሰዓት አቆጣጠር"}, {"Myanmar", "የሚያንማር ሰዓት"}, {"Nauru", "የናውሩ ሰዓት"}, {"Nepal", "የኔፓል ሰዓት"}, {"New_Caledonia", "የኒው ካሌዶኒያ መደበኛ ሰዓት"}, {"New_Zealand", "የኒው ዚላንድ መደበኛ ሰዓት"}, {"Newfoundland", "የኒውፋውንድላንድ መደበኛ የሰዓት አቆጣጠር"}, {"Niue", "የኒዩዌ ሰዓት"}, {"Norfolk", "የኖርፎልክ ደሴቶች መደበኛ የሰዓት አቆጣጠር"}, {"Noronha", "የፈርናንዶ ዲ ኖሮንቻ መደበኛ ሰዓት አቆጣጠር"}, {"Novosibirsk", "የኖቮሲቢርስክ መደበኛ የሰዓት አቆጣጠር"}, {"Omsk", "የኦምስክ መደበኛ ሰዓት አቆጣጠር"}, {"Pakistan", "የፓኪስታን መደበኛ ሰዓት"}, {"Palau", "የፓላው ሰዓት"}, {"Papua_New_Guinea", "የፓፗ ኒው ጊኒ ሰዓት"}, {"Paraguay", "የፓራጓይ መደበኛ ሰዓት"}, {"Peru", "የፔሩ መደበኛ ሰዓት"}, {"Philippines", "የፊሊፒን መደበኛ ሰዓት"}, {"Phoenix_Islands", "የፊኒክስ ደሴቶች ሰዓት"}, {"Pierre_Miquelon", "ቅዱስ የፒዬር እና ሚኴሎን መደበኛ ሰዓት"}, {"Pitcairn", "የፒትካይርን ሰዓት"}, {"Ponape", "የፖናፔ ሰዓት"}, {"Pyongyang", "የፕዮንግያንግ ሰዓት"}, {"Reunion", "የሬዩኒየን ሰዓት"}, {"Rothera", "የሮቴራ ሰዓት"}, {"Sakhalin", "የሳክሃሊን መደበኛ ሰዓት አቆጣጠር"}, {"Samara", "የሳማራ መደበኛ ሰዓት አቆጣጠር"}, {"Samoa", "የሳሞዋ መደበኛ ሰዓት"}, {"Seychelles", "የሴሸልስ ሰዓት"}, {"Singapore", "የሲንጋፒር መደበኛ ሰዓት"}, {"Solomon", "የሰለሞን ደሴቶች ሰዓት"}, {"South_Georgia", "የደቡብ ጂዮርጂያ ሰዓት"}, {"Suriname", "የሱሪናም ሰዓት"}, {"Syowa", "የሲዮዋ ሰዓት"}, {"Tahiti", "የታሂቲ ሰዓት"}, {"Taipei", "የታይፔይ መደበኛ ሰዓት"}, {"Tajikistan", "የታጂኪስታን ሰዓት"}, {"Tokelau", "የቶኬላው ሰዓት"}, {"Tonga", "የቶንጋ መደበኛ ሰዓት"}, {"Truk", "የቹክ ሰዓት"}, {"Turkmenistan", "የቱርክመኒስታን መደበኛ ሰዓት"}, {"Tuvalu", "የቱቫሉ ሰዓት"}, {"Uruguay", "የኡራጓይ መደበኛ ሰዓት"}, {"Uzbekistan", "የኡዝቤኪስታን መደበኛ ሰዓት"}, {"Vanuatu", "የቫኗቱ መደበኛ ሰዓት"}, {"Venezuela", "የቬኔዝዌላ ሰዓት"}, {"Vladivostok", "የቪላዲቮስቶክ መደበኛ የሰዓት አቆጣጠር"}, {"Volgograd", "የቮልጎራድ መደበኛ ሰዓት አቆጣጠር"}, {"Vostok", "የቮስቶክ ሰዓት"}, {"Wake", "የዌክ ደሴት ሰዓት"}, {"Wallis", "የዋሊስ እና ፉቱና ሰዓት"}, {"Yakutsk", "ያኩትስክ መደበኛ ሰዓት አቆጣጠር"}, {"Yekaterinburg", "የየካተሪንበርግ መደበኛ ሰዓት አቆጣጠር"}, {"Yukon", "የዩኮን ጊዜ"}},
		{{"Alaska", "የአላስካ የቀን ሰዓት አቆጣጠር"}, {"Amazon", "የአማዞን የቀን ሰዓት አቆጣጠር"}, {"America_Central", "የመካከለኛ የቀን ሰዓት አቆጣጠር"}, {"America_Eastern", "ምስራቃዊ የቀን ሰዓት አቆጣጠር"}, {"America_Mountain", "የተራራ የቀንሰዓት አቆጣጠር"}, {"America_Pacific", "የፓስፊክ የቀን ሰዓት አቆጣጠር"}, {"Anadyr", "የአናድይር የበጋ የሰዓት አቆጣጠር"}, {"Apia", "የአፒያ የቀን ጊዜ ሰዓት"}, {"Arabian", "የዓረቢያ የቀን ብርሃን ሰዓት"}, {"Argentina", "የአርጀንቲና የበጋ ሰዓት አቆጣጠር"}, {"Argentina_Western", "የምዕራባዊ አርጀንቲና የበጋ ሰዓት አቆጣጠር"}, {"Armenia", "የአርመኒያ ክረምት ሰዓት"}, {"Atlantic", "የአትላንቲክ የቀን ሰዓት አቆጣጠር"}, {"Australia_Central", "የአውስትራሊያ መካከለኛ የቀን ሰዓት አቆጣጠር"}, {"Australia_CentralWestern", "የአውስትራሊያ መካከለኛው ምስራቅ የቀን ሰዓት አቆጣጠር"}, {"Australia_Eastern", "የአውስትራሊያ ምዕራባዊ የቀን ሰዓት አቆጣጠር"}, {"Australia_Western", "የአውስትራሊያ ምስራቃዊ የቀን ሰዓት አቆጣጠር"}, {"Azerbaijan", "የአዘርባጃን ክረምት ሰዓት"}, {"Azores", "የአዞረስ ክረምት ሰዓት"}, {"Bangladesh", "የባንግላዴሽ ክረምት ሰዓት"}, {"Brasilia", "የብራዚላ የበጋ ሰዓት አቆጣጠር"}, {"Cape_Verde", "የኬፕ ቨርዴ ክረምት ሰዓት"}, {"Chatham", "የቻታም የቀን ብርሃን ሰዓት"}, {"Chile", "የቺሊ ክረምት ሰዓት"}, {"China", "የቻይና የቀን ብርሃን ሰዓት"}, {"Colombia", "የኮሎምቢያ ክረምት ሰዓት"}, {"Cook", "የኩክ ደሴቶች ግማሽ ክረምት ሰዓት"}, {"Cuba", "የኩባ የቀን ብርሃን ሰዓት"}, {"Easter", "የኢስተር ደሴት ክረምት ሰዓት"}, {"Europe_Central", "የመካከለኛው አውሮፓ ክረምት ሰዓት"}, {"Europe_Eastern", "የምስራቃዊ አውሮፓ ክረምት ሰዓት"}, {"Europe_Western", "የምዕራባዊ አውሮፓ ክረምት ሰዓት"}, {"Falkland", "የፋልክላንድ ደሴቶች ክረምት ሰዓት"}, {"Fiji", "የፊጂ ክረምት ሰዓት"}, {"Georgia", "የጂዮርጂያ ክረምት ሰዓት"}, {"Greenland_Eastern", "የምስራቅ ግሪንላንድ ክረምት ሰዓት"}, {"Greenland_Western", "የምዕራብ ግሪንላንድ ክረምት ሰዓት"}, {"Hawaii_Aleutian", "የሃዋይ አሌኡት የቀን ሰዓት አቆጣጠር"}, {"Hong_Kong", "የሆንግ ኮንግ ክረምት ሰዓት"}, {"Hovd", "የሆቭድ የበጋ ሰዓት አቆጣጠር"}, {"Iran", "የኢራን የቀን ብርሃን ሰዓት"}, {"Irkutsk", "ኢርኩትስክ የበጋ የሰዓት አቆጣጠር"}, {"Israel", "የእስራኤል የቀን ብርሃን ሰዓት"}, {"Japan", "የጃፓን የቀን ብርሃን ሰዓት"}, {"Kamchatka", "የፔትሮፓቭሎስኪ - ካምቻትስኪ የበጋ ሰዓት አቆጣጠር"}, {"Korea", "የኮሪያ የቀን ብርሃን ሰዓት"}, {"Krasnoyarsk", "የክራስኖያርስክ የበጋ ሰዓት አቆጣጠር"}, {"Lord_Howe", "የሎርድ ሆዌ የቀን ሰዓት አቆጣጠር"}, {"Magadan", "የማጋዳን በጋ ሰዓት አቆጣጠር"}, {"Mauritius", "የማውሪሺየስ ክረምት ሰዓት"}, {"Mexico_Pacific", "የሜክሲኮ ፓሲፊክ የቀን ሰዓት አቆጣጠር"}, {"Mongolia", "የኡላን ባቶር የበጋ ሰዓት አቆጣጠር"}, {"Moscow", "የሞስኮ የበጋ ሰዓት አቆጣጠር"}, {"New_Caledonia", "የኒው ካሌዶኒያ ክረምት ሰዓት"}, {"New_Zealand", "የኒው ዚላንድ የቀን ብርሃን ሰዓት"}, {"Newfoundland", "የኒውፋውንድላንድ የቀን የሰዓት አቆጣጠር"}, {"Norfolk", "የኖርፎልክ ደሴቶች የቀን የሰዓት አቆጣጠር"}, {"Noronha", "የፈርናንዶ ዲ ኖሮንሃ የበጋ የሰዓት አቆጣጠር"}, {"Novosibirsk", "የኖቮሲብሪስክ የበጋ ሰአት አቆጣጠር"}, {"Omsk", "የኦምስክ የበጋ ሰዓት አቆጣጠር"}, {"Pakistan", "የፓኪስታን ክረምት ሰዓት"}, {"Paraguay", "የፓራጓይ ክረምት ሰዓት"}, {"Peru", "የፔሩ ክረምት ሰዓት"}, {"Philippines", "የፊሊፒን ክረምት ሰዓት"}, {"Pierre_Miquelon", "ቅዱስ የፒዬር እና ሚኴሎን የቀን ብርሃን ሰዓት"}, {"Sakhalin", "የሳክሃሊን የበጋ ሰዓት አቆጣጠር"}, {"Samara", "የሳማራ የበጋ ሰዓት አቆጣጠር"}, {"Samoa", "የሳሞዋ የበጋ ሰዓት"}, {"Taipei", "የታይፔይ የቀን ብርሃን ሰዓት"}, {"Tonga", "የቶንጋ ክረምት ሰዓት"}, {"Turkmenistan", "የቱርክመኒስታን ክረምት ሰዓት"}, {"Uruguay", "የኡራጓይ ክረምት ሰዓት"}, {"Uzbekistan", "የኡዝቤኪስታን ክረምት ሰዓት"}, {"Vanuatu", "የቫኗቱ ክረምት ሰዓት"}, {"Vladivostok", "የቭላዲቮስቶክ የበጋ የሰዓት አቆጣጠር"}, {"Volgograd", "የቫልጎራድ የበጋ ሰዓት አቆጣጠር"}, {"Yakutsk", "የያኩትስክ የበጋ ሰዓት አቆጣጠር"}, {"Yekaterinburg", "የየካተሪንበርግ የበጋ ሰዓት አቆጣጠር"}},
	},
	{
		{{"America_Central", "hora standard central d’America d’o Norte"}, {"America_Eastern", "hora standard oriental d’America d’o Norte"}, {"America_Mountain", "hora standard de montanya d’America d’o Norte"}, {"America_Pacific", "hora standard d’o Pacifico d’America d’o Norte"}, {"Atlantic", "hora standard de l’Atlantico"}, {"Europe_Central", "hora standard d’o centro d’Europa"}, {"Europe_Eastern", "hora standard de l’este d’Europa"}, {"Europe_Western", "hora standard de l’ueste d’Europa"}, {"GMT", "hora d’o meridiano de Greenwich"}},
		{{"America_Central", "hora de verano central d’America d’o Norte"}, {"America_Eastern", "hora de verano oriental d’America d’o Norte"}, {"America_Mountain", "hora de verano de montanya d’America d’o Norte"}, {"America_Pacific", "hora de verano d’o Pacifico d’America d’o Norte"}, {"Atlantic", "hora de verano de l’Atlantico"}, {"Europe_Central", "hora de verano d’o centro d’Europa"}, {"Europe_Eastern", "hora de verano de l’este d’Europa"}, {"Europe_Western", "hora de verano de l’ueste d’Europa"}},
	},
	{
		{{"Afghanistan", "توقيت أفغانستان"}, {"Africa_Central", "توقيت وسط أفريقيا"}, {"Africa_Eastern", "توقيت شرق أفريقيا"}, {"Africa_Southern", "توقيت جنوب أفريقيا"}, {"Africa_Western", "توقيت غرب أفريقيا"}, {"Alaska", "التوقيت الرسمي لألاسكا"}, {"Amazon", "توقيت الأمازون الرسمي"}, {"America_Central", "التوقيت الرسمي المركزي لأمريكا الشمالية"}, {"America_Eastern", "التوقيت الرسمي الشرقي لأمريكا الشمالية"}, {"America_Mountain", "التوقيت الجبلي الرسمي لأمريكا الشمالية"}, {"America_Pacific", "توقيت المحيط الهادي الرسمي"}, {"Anadyr", "توقيت أنادير الرسمي"}, {"Apia", "التوقيت الرسمي لآبيا"}, {"Arabian", "التوقيت العربي الرسمي"}, {"Argentina", "توقيت الأرجنتين الرسمي"}, {"Argentina_Western", "توقيت غرب الأرجنتين الرسمي"}, {"Armenia", "توقيت أرمينيا الرسمي"}, {"Atlantic", "التوقيت الرسمي الأطلسي"}, {"Australia_Central", "توقيت وسط أستراليا الرسمي"}, {"Australia_CentralWestern", "توقيت غرب وسط أستراليا الرسمي"}, {"Australia_Eastern", "توقيت شرق أستراليا الرسمي"}, {"Australia_Western", "توقيت غرب أستراليا الرسمي"}, {"Azerbaijan", "توقيت أذربيجان الرسمي"}, {"Azores", "توقيت أزورس الرسمي"}, {"Bangladesh", "توقيت بنغلاديش الرسمي"}, {"Bhutan", "توقيت بوتان"}, {"Bolivia", "توقيت بوليفيا"}, {"Brasilia", "توقيت برازيليا الرسمي"}, {"Brunei", "توقيت بروناي"}, {"Cape_Verde", "توقيت الرأس الأخضر الرسمي"}, {"Chamorro", "توقيت تشامورو"}, {"Chatham", "توقيت تشاتام الرسمي"}, {"Chile", "توقيت تشيلي الرسمي"}, {"China", "توقيت الصين الرسمي"}, {"Christmas", "توقيت جزر الكريسماس"}, {"Cocos", "توقيت جزر كوكوس"}, {"Colombia", "توقيت كولومبيا الرسمي"}, {"Cook", "توقيت جزر كوك الرسمي"}, {"Cuba", "توقيت كوبا الرسمي"}, {"Davis", "توقيت دافيز"}, {"DumontDUrville", "توقيت دي مونت دو روفيل"}, {"East_Timor", "توقيت تيمور الشرقية"}, {"Easter", "توقيت جزيرة استر الرسمي"}, {"Ecuador", "توقيت الإكوادور"}, {"Europe_Central", "توقيت وسط أوروبا الرسمي"}, {"Europe_Eastern", "توقيت شرق أوروبا الرسمي"}, {"Europe_Further_Eastern", "التوقيت الأوروبي (أكثر شرقًا)"}, {"Europe_Western", "توقيت غرب أوروبا الرسمي"}, {"Falkland", "توقيت جزر فوكلاند الرسمي"}, {"Fiji", "توقيت فيجي الرسمي"}, {"French_Guiana", "توقيت غويانا الفرنسية"}, {"French_Southern", "توقيت المقاطعات الفرنسية الجنوبية والأنتارتيكية"}, {"GMT", "توقيت غرينتش"}, {"Galapagos", "توقيت غلاباغوس"}, {"Gambier", "توقيت جامبير"}, {"Georgia", "توقيت جورجيا الرسمي"}, {"Gilbert_Islands", "توقيت جزر جيلبرت"}, {"Greenland_Eastern", "توقيت شرق غرينلاند الرسمي"}, {"Greenland_Western", "توقيت غرب غرينلاند الرسمي"}, {"Guam", "توقيت غوام"}, {"Gulf", "توقيت الخليج"}, {"Guyana", "توقيت غيانا"}, {"Hawaii", "توقيت هاواي ألوتيان الرسمي"}, {"Hawaii_Aleutian", "توقيت هاواي ألوتيان الرسمي"}, {"Hong_Kong", "توقيت هونغ كونغ الرسمي"}, {"Hovd", "توقيت هوفد الرسمي"}, {"India", "توقيت الهند"}, {"Indian_Ocean", "توقيت المحيط الهندي"}, {"Indochina", "توقيت الهند الصينية"}, {"Indonesia_Central", "توقيت وسط إندونيسيا"}, {"Indonesia_Eastern", "توقيت شرق إندونيسيا"}, {"Indonesia_Western", "توقيت غرب إندونيسيا"}, {"Iran", "توقيت إيران الرسمي"}, {"Irkutsk", "توقيت إركوتسك الرسمي"}, {"Israel", "توقيت إسرائيل الرسمي"}, {"Japan", "توقيت اليابان الرسمي"}, {"Kamchatka", "توقيت بيتروبافلوفسك-كامتشاتسكي"}, {"Kazakhstan", "توقيت كازاخستان"}, {"Kazakhstan_Eastern", "توقيت شرق كازاخستان"}, {"Kazakhstan_Western", "توقيت غرب كازاخستان"}, {"Korea", "توقيت كوريا الرسمي"}, {"Kosrae", "توقيت كوسرا"}, {"Krasnoyarsk", "توقيت كراسنويارسك الرسمي"}, {"Kyrgystan", "توقيت قيرغيزستان"}, {"Line_Islands", "توقيت جزر لاين"}, {"Lord_Howe", "توقيت لورد هاو الرسمي"}, {"Magadan", "توقيت ماغادان الرسمي"}, {"Malaysia", "توقيت ماليزيا"}, {"Maldives", "توقيت جزر المالديف"}, {"Marquesas", "توقيت ماركيساس"}, {"Marshall_Islands", "توقيت جزر مارشال"}, {"Mauritius", "توقيت موريشيوس الرسمي"}, {"Mawson", "توقيت ماوسون"}, {"Mexico_Pacific", "توقيت المحيط الهادي الرسمي للمكسيك"}, {"Mongolia", "توقيت أولان باتور الرسمي"}, {"Moscow", "توقيت موسكو الرسمي"}, {"Myanmar", "توقيت ميانمار"}, {"Nauru", "توقيت ناورو"}, {"Nepal", "توقيت نيبال"}, {"New_Caledonia", "توقيت كاليدونيا الجديدة الرسمي"}, {"New_Zealand", "توقيت نيوزيلندا الرسمي"}, {"Newfoundland", "توقيت نيوفاوندلاند الرسمي"}, {"Niue", "توقيت نيوي"}, {"Norfolk", "توقيت جزيرة نورفولك الرسمي"}, {"Noronha", "توقيت فرناندو دي نورونها الرسمي"}, {"North_Mariana", "توقيت جزر ماريانا الشمالية"}, {"Novosibirsk", "توقيت نوفوسيبيرسك الرسمي"}, {"Omsk", "توقيت أومسك الرسمي"}, {"Pakistan", "توقيت باكستان الرسمي"}, {"Palau", "توقيت بالاو"}, {"Papua_New_Guinea", "توقيت بابوا غينيا الجديدة"}, {"Paraguay", "توقيت باراغواي الرسمي"}, {"Peru", "توقيت بيرو الرسمي"}, {"Philippines", "توقيت الفيلبين الرسمي"}, {"Phoenix_Islands", "توقيت جزر فينكس"}, {"Pierre_Miquelon", "توقيت سانت بيير وميكولون الرسمي"}, {"Pitcairn", "توقيت بيتكيرن"}, {"Ponape", "توقيت بونابي"}, {"Pyongyang", "توقيت بيونغ يانغ"}, {"Reunion", "توقيت روينيون"}, {"Rothera", "توقيت روثيرا"}, {"Sakhalin", "توقيت ساخالين الرسمي"}, {"Samara", "توقيت سمارا"}, {"Samoa", "توقيت ساموا الرسمي"}, {"Seychelles", "توقيت سيشل"}, {"Singapore", "توقيت سنغافورة"}, {"Solomon", "توقيت جزر سليمان"}, {"South_Georgia", "توقيت جنوب جورجيا"}, {"Suriname", "توقيت سورينام"}, {"Syowa", "توقيت سايووا"}, {"Tahiti", "توقيت تاهيتي"}, {"Taipei", "توقيت تايبيه الرسمي"}, {"Tajikistan", "توقيت طاجكستان"}, {"Tokelau", "توقيت توكيلاو"}, {"Tonga", "توقيت تونغا الرسمي"}, {"Truk", "توقيت شوك"}, {"Turkmenistan", "توقيت تركمانستان الرسمي"}, {"Tuvalu", "توقيت توفالو"}, {"Uruguay", "توقيت أوروغواي الرسمي"}, {"Uzbekistan", "توقيت أوزبكستان الرسمي"}, {"Vanuatu", "توقيت فانواتو الرسمي"}, {"Venezuela", "توقيت فنزويلا"}, {"Vladivostok", "توقيت فلاديفوستوك الرسمي"}, {"Volgograd", "توقيت فولغوغراد الرسمي"}, {"Vostok", "توقيت فوستوك"}, {"Wake", "توقيت جزيرة ويك"}, {"Wallis", "توقيت واليس و فوتونا"}, {"Yakutsk", "توقيت ياكوتسك الرسمي"}, {"Yekaterinburg", "توقيت يكاترينبورغ الرسمي"}, {"Yukon", "توقيت يوكون"}},
		{{"Alaska", "توقيت ألاسكا الصيفي"}, {"Amazon", "توقيت الأمازون الصيفي"}, {"America_Central", "التوقيت الصيفي المركزي لأمريكا الشمالية"}, {"America_Eastern", "التوقيت الصيفي الشرقي لأمريكا الشمالية"}, {"America_Mountain", "التوقيت الجبلي الصيفي لأمريكا الشمالية"}, {"America_Pacific", "توقيت المحيط الهادي الصيفي"}, {"Anadyr", "التوقيت الصيفي لأنادير"}, {"Apia", "التوقيت الصيفي لأبيا"}, {"Arabian", "التوقيت العربي الصيفي"}, {"Argentina", "توقيت الأرجنتين الصيفي"}, {"Argentina_Western", "توقيت غرب الأرجنتين الصيفي"}, {"Armenia", "توقيت أرمينيا الصيفي"}, {"Atlantic", "التوقيت الصيفي الأطلسي"}, {"Australia_Central", "توقيت وسط أستراليا الصيفي"}, {"Australia_CentralWestern", "توقيت غرب وسط أستراليا الصيفي"}, {"Australia_Eastern", "توقيت شرق أستراليا الصيفي"}, {"Australia_Western", "توقيت غرب أستراليا الصيفي"}, {"Azerbaijan", "توقيت أذربيجان الصيفي"}, {"Azores", "توقيت أزورس الصيفي"}, {"Bangladesh", "توقيت بنغلاديش الصيفي"}, {"Brasilia", "توقيت برازيليا الصيفي"}, {"Cape_Verde", "توقيت الرأس الأخضر الصيفي"}, {"Chatham", "توقيت تشاتام الصيفي"}, {"Chile", "توقيت تشيلي الصيفي"}, {"China", "توقيت الصين الصيفي"}, {"Colombia", "توقيت كولومبيا الصيفي"}, {"Cook", "توقيت جزر كوك الصيفي"}, {"Cuba", "توقيت كوبا الصيفي"}, {"Easter", "توقيت جزيرة استر الصيفي"}, {"Europe_Central", "توقيت وسط أوروبا الصيفي"}, {"Europe_Eastern", "توقيت شرق أوروبا الصيفي"}, {"Europe_Western", "توقيت غرب أوروبا الصيفي"}, {"Falkland", "توقيت جزر فوكلاند الصيفي"}, {"Fiji", "توقيت فيجي الصيفي"}, {"Georgia", "توقيت جورجيا الصيفي"}, {"Greenland_Eastern", "توقيت شرق غرينلاند الصيفي"}, {"Greenland_Western", "توقيت غرب غرينلاند الصيفي"}, {"Hawaii_Aleutian", "توقيت هاواي ألوتيان الصيفي"}, {"Hong_Kong", "توقيت هونغ كونغ الصيفي"}, {"Hovd", "توقيت هوفد الصيفي"}, {"Iran", "توقيت إيران الصيفي"}, {"Irkutsk", "توقيت إركوتسك الصيفي"}, {"Israel", "توقيت إسرائيل الصيفي"}, {"Japan", "توقيت اليابان الصيفي"}, {"Kamchatka", "توقيت بيتروبافلوفسك-كامتشاتسكي الصيفي"}, {"Korea", "توقيت كوريا الصيفي"}, {"Krasnoyarsk", "التوقيت الصيفي لكراسنويارسك"}, {"Lord_Howe", "التوقيت الصيفي للورد هاو"}, {"Magadan", "توقيت ماغادان الصيفي"}, {"Mauritius", "توقيت موريشيوس الصيفي"}, {"Mexico_Pacific", "توقيت المحيط الهادي الصيفي للمكسيك"}, {"Mongolia", "توقيت أولان باتور الصيفي"}, {"Moscow", "توقيت موسكو الصيفي"}, {"New_Caledonia", "توقيت كاليدونيا الجديدة الصيفي"}, {"New_Zealand", "توقيت نيوزيلندا الصيفي"}, {"Newfoundland", "توقيت نيوفاوندلاند الصيفي"}, {"Norfolk", "توقيت جزيرة نورفولك الصيفي"}, {"Noronha", "توقيت فرناندو دي نورونها الصيفي"}, {"Novosibirsk", "توقيت نوفوسيبيرسك الصيفي"}, {"Omsk", "توقيت أومسك الصيفي"}, {"Pakistan", "توقيت باكستان الصيفي"}, {"Paraguay", "توقيت باراغواي الصيفي"}, {"Peru", "توقيت بيرو الصيفي"}, {"Philippines", "توقيت الفيلبين الصيفي"}, {"Pierre_Miquelon", "توقيت سانت بيير وميكولون الصيفي"}, {"Sakhalin", "توقيت ساخالين الصيفي"}, {"Samara", "توقيت سمارا الصيفي"}, {"Samoa", "توقيت ساموا الصيفي"}, {"Taipei", "توقيت تايبيه الصيفي"}, {"Tonga", "توقيت تونغا الصيفي"}, {"Turkmenistan", "توقيت تركمانستان الصيفي"}, {"Uruguay", "توقيت أوروغواي الصيفي"}, {"Uzbekistan", "توقيت أوزبكستان الصيفي"}, {"Vanuatu", "توقيت فانواتو الصيفي"}, {"Vladivostok", "توقيت فلاديفوستوك الصيفي"}, {"Volgograd", "توقيت فولغوغراد الصيفي"}, {"Yakutsk", "توقيت ياكوتسك الصيفي"}, {"Yekaterinburg", "توقيت يكاترينبورغ الصيفي"}},
	},
	{
		{{"Afghanistan", "আফগানিস্তানৰ সময়"}, {"Africa_Central", "মধ্য আফ্ৰিকাৰ সময়"}, {"Africa_Eastern", "পূব আফ্ৰিকাৰ সময়"}, {"Africa_Southern", "দক্ষিণ আফ্ৰিকাৰ মান সময়"}, {"Africa_Western", "পশ্চিম আফ্ৰিকাৰ সময়"}, {"Alaska", "আলাস্কাৰ মান সময়"}, {"Amazon", "আমাজনৰ মান সময়"}, {"America_Central", "উত্তৰ আমেৰিকাৰ কেন্দ্ৰীয় মান সময়"}, {"America_Eastern", "উত্তৰ আমেৰিকাৰ প্ৰাচ্য মান সময়"}, {"America_Mountain", "উত্তৰ আমেৰিকাৰ পৰ্ব্বতীয় মান সময়"}, {"America_Pacific", "উত্তৰ আমেৰিকাৰ প্ৰশান্ত মান সময়"}, {"Apia", "আপিয়াৰ মান সময়"}, {"Arabian", "আৰবীয় মান সময়"}, {"Argentina", "আৰ্জেণ্টিনাৰ মান সময়"}, {"Argentina_Western", "পাশ্চাত্য আৰ্জেণ্টিনাৰ মান সময়"}, {"Armenia", "আৰ্মেনিয়াৰ মান সময়"}, {"Atlantic", "আটলাণ্টিক মান সময়"}, {"Australia_Central", "অষ্ট্ৰেলিয়াৰ কেন্দ্ৰীয় মান সময়"}, {"Australia_CentralWestern", "অষ্ট্ৰেলিয়াৰ কেন্দ্ৰীয় পাশ্চাত্য মান সময়"}, {"Australia_Eastern", "অষ্ট্ৰেলিয়াৰ প্ৰাচ্য মান সময়"}, {"Australia_Western", "অষ্ট্ৰেলিয়াৰ পাশ্চাত্য মান সময়"}, {"Azerbaijan", "আজেৰবাইজানৰ মান সময়"}, {"Azores", "আজোৰেছৰ মান সময়"}, {"Bangladesh", "বাংলাদেশৰ মান সময়"}, {"Bhutan", "ভুটানৰ সময়"}, {"Bolivia", "বলিভিয়াৰ সময়"}, {"Brasilia", "ব্ৰাজিলিয়াৰ মান সময়"}, {"Brunei", "ব্ৰুনেই ডাৰুছালেমৰ সময়"}, {"Cape_Verde", "কেপ ভাৰ্দেৰ মান সময়"}, {"Chamorro", "চামোৰোৰ মান সময়"}, {"Chatham", "চাথামৰ মান সময়"}, {"Chile", "চিলিৰ মান সময়"}, {"China", "চীনৰ মান সময়"}, {"Christmas", "খ্ৰীষ্টমাছ দ্বীপৰ সময়"}, {"Cocos", "কোকোছ দ্বীপপুঞ্জৰ সময়"}, {"Colombia", "কলম্বিয়াৰ মান সময়"}, {"Cook", "কুক দ্বীপপুঞ্জৰ মান সময়"}, {"Cuba", "কিউবাৰ মান সময়"}, {"Davis", "ডেভিছৰ সময়"}, {"DumontDUrville", "ডুমোণ্ট-ডি আৰ্ভিলৰ সময়"}, {"East_Timor", "পূব তিমোৰৰ সময়"}, {"Easter", "ইষ্টাৰ দ্বীপৰ মান সময়"}, {"Ecuador", "ইকুৱেডৰৰ সময়"}, {"Europe_Central", "মধ্য ইউৰোপীয় মান সময়"}, {"Europe_Eastern", "প্ৰাচ্য ইউৰোপীয় মান সময়"}, {"Europe_Further_Eastern", "অগ্ৰ-প্ৰাচ্য ইউৰোপীয় সময়"}, {"Europe_Western", "পাশ্চাত্য ইউৰোপীয় মান সময়"}, {"Falkland", "ফকলেণ্ড দ্বীপপুঞ্জৰ মান সময়"}, {"Fiji", "ফিজিৰ মান সময়"}, {"French_Guiana", "ফ্ৰান্স গয়ানাৰ সময়"}, {"French_Southern", "দক্ষিণ ফ্ৰান্স আৰু এণ্টাৰ্কটিক সময়"}, {"GMT", "গ্ৰীণউইচ মান সময়"}, {"Galapagos", "গালাপাগোছৰ সময়"}, {"Gambier", "গেম্বিয়াৰ সময়"}, {"Georgia", "জৰ্জিয়াৰ মান সময়"}, {"Gilbert_Islands", "গিলবাৰ্ট দ্বীপপুঞ্জৰ সময়"}, {"Greenland_Eastern", "পূব গ্ৰীণলেণ্ডৰ মান সময়"}, {"Greenland_Western", "পশ্চিম গ্ৰীণলেণ্ডৰ মান সময়"}, {"Gulf", "উপসাগৰীয় মান সময়"}, {"Guyana", "গায়ানাৰ সময়"}, {"Hawaii", "হাৱাই-এলিউশ্বনৰ মান সময়"}, {"Hawaii_Aleutian", "হাৱাই-এলিউশ্বনৰ মান সময়"}, {"Hong_Kong", "হং কঙৰ মান সময়"}, {"Hovd", "হোভ্\u200cডৰ মান সময়"}, {"India", "ভাৰতীয় মান সময়"}, {"Indian_Ocean", "ভাৰত মহাসাগৰীয় সময়"}, {"Indochina", "ইণ্ডোচাইনাৰ সময়"}, {"Indonesia_Central", "মধ্য ইণ্ডোনেচিয়াৰ সময়"}, {"Indonesia_Eastern", "প্ৰাচ্য ইণ্ডোনেচিয়াৰ সময়"}, {"Indonesia_Western", "পাশ্চাত্য ইণ্ডোনেচিয়াৰ সময়"}, {"Iran", "ইৰানৰ মান সময়"}, {"Irkutsk", "ইৰ্কুটস্কৰ মান সময়"}, {"Israel", "ইজৰাইলৰ মান সময়"}, {"Japan", "জাপানৰ মান সময়"}, {"Kazakhstan", "কাজাখস্তানৰ সময়"}, {"Kazakhstan_Eastern", "পূব কাজাখস্তানৰ সময়"}, {"Kazakhstan_Western", "পশ্চিম কাজাখস্তানৰ সময়"}, {"Korea", "কোৰিয়াৰ মান সময়"}, {"Kosrae", "কোছৰায়ে সময়"}, {"Krasnoyarsk", "ক্ৰাছনোয়াৰ্স্কৰ মান সময়"}, {"Kyrgystan", "কিৰ্গিজস্তানৰ সময়"}, {"Line_Islands", "লাইন দ্বীপপুঞ্জৰ সময়"}, {"Lord_Howe", "লৰ্ড হাওৰ মান সময়"}, {"Magadan", "মাগাদানৰ মান সময়"}, {"Malaysia", "মালয়েচিয়াৰ সময়"}, {"Maldives", "মালদ্বীপৰ সময়"}, {"Marquesas", "মাৰ্কছেছৰ সময়"}, {"Marshall_Islands", "মাৰ্শ্বাল দ্বীপপুঞ্জৰ সময়"}, {"Mauritius", "মৰিছাছৰ মান সময়"}, {"Mawson", "মাউছনৰ সময়"}, {"Mexico_Pacific", "মেক্সিকোৰ প্ৰশান্ত মান সময়"}, {"Mongolia", "উলানবাটাৰৰ মান সময়"}, {"Moscow", "মস্কোৰ মান সময়"}, {"Myanmar", "ম্যানমাৰৰ সময়"}, {"Nauru", "নাউৰুৰ সময়"}, {"Nepal", "নেপালৰ সময়"}, {"New_Caledonia", "নিউ কেলিডোনিয়াৰ মান সময়"}, {"New_Zealand", "নিউজিলেণ্ডৰ মান সময়"}, {"Newfoundland", "নিউফাউণ্ডলেণ্ডৰ মান সময়"}, {"Niue", "নিয়ুৰ সময়"}, {"Norfolk", "ন’ৰফ’ক দ্বীপৰ মান্য সময়"}, {"Noronha", "ফাৰ্নাণ্ডো ডে নোৰোন্\u200cহাৰ মান সময়"}, {"Novosibirsk", "নভোছিবিৰ্স্কৰ মান সময়"}, {"Omsk", "ওমস্কৰ মান সময়"}, {"Pakistan", "পাকিস্তানৰ মান সময়"}, {"Palau", "পালাউৰ সময়"}, {"Papua_New_Guinea", "পাপুৱা নিউ গিনিৰ সময়"}, {"Paraguay", "পাৰাগুৱেৰ মান সময়"}, {"Peru", "পেৰুৰ মান সময়"}, {"Philippines", "ফিলিপাইনৰ মান সময়"}, {"Phoenix_Islands", "ফিনিক্স দ্বীপপুঞ্জৰ সময়"}, {"Pierre_Miquelon", "ছেইণ্ট পিয়েৰে আৰু মিকিউৱেলনৰ মান সময়"}, {"Pitcairn", "পিটকেইৰ্ণৰ সময়"}, {"Ponape", "পোনাপেৰ সময়"}, {"Pyongyang", "প্যংয়াংৰ সময়"}, {"Reunion", "ৰিইউনিয়নৰ সময়"}, {"Rothera", "ৰোথেৰাৰ সময়"}, {"Sakhalin", "ছাখালিনৰ মান সময়"}, {"Samoa", "ছামোৱাৰ মান সময়"}, {"Seychelles", "ছিচিলিছৰ সময়"}, {"Singapore", "ছিংগাপুৰৰ মান সময়"}, {"Solomon", "চোলোমোন দ্বীপপুঞ্জৰ সময়"}, {"South_Georgia", "দক্ষিণ জৰ্জিয়াৰ সময়"}, {"Suriname", "ছুৰিনামৰ সময়"}, {"Syowa", "স্বোৱা সময়"}, {"Tahiti", "তাহিতিৰ সময়"}, {"Taipei", "টাইপেইৰ মান সময়"}, {"Tajikistan", "তাজিকিস্তানৰ সময়"}, {"Tokelau", "টোকেলাউৰ সময়"}, {"Tonga", "টংগাৰ মান সময়"}, {"Truk", "চ্চুকৰ সময়"}, {"Turkmenistan", "তুৰ্কমেনিস্তানৰ মান সময়"}, {"Tuvalu", "টুভালাউৰ সময়"}, {"Uruguay", "উৰুগুৱেৰ মান সময়"}, {"Uzbekistan", "উজবেকিস্তানৰ মান সময়"}, {"Vanuatu", "ভানাটুৰ মান সময়"}, {"Venezuela", "ভেনিজুৱেলাৰ সময়"}, {"Vladivostok", "ভ্লাডিভোষ্টোকৰ মান সময়"}, {"Volgograd", "ভোল্গোগ্ৰাডৰ মান সময়"}, {"Vostok", "ভোষ্টকৰ সময়"}, {"Wake", "ৱেক দ্বীপৰ সময়"}, {"Wallis", "ৱালিছ আৰু ফুটুনাৰ সময়"}, {"Yakutsk", "য়াকুত্স্কৰ মান সময়"}, {"Yekaterinburg", "য়েকাটেৰিণবাৰ্গৰ মান সময়"}, {"Yukon", "য়ুকোন সময়"}},
		{{"Alaska", "আলাস্কাৰ ডেলাইট সময়"}, {"Amazon", "আমাজনৰ গ্ৰীষ্মকালীন সময়"}, {"America_Central", "উত্তৰ আমেৰিকাৰ কেন্দ্ৰীয় ডেলাইট সময়"}, {"America_Eastern", "উত্তৰ আমেৰিকাৰ প্ৰাচ্য ডেলাইট সময়"}, {"America_Mountain", "উত্তৰ আমেৰিকাৰ পৰ্ব্বতীয় ডেলাইট সময়"}, {"America_Pacific", "উত্তৰ আমেৰিকাৰ ডেলাইট সময়"}, {"Apia", "আপিয়াৰ ডেলাইট সময়"}, {"Arabian", "আৰবীয় ডেলাইট সময়"}, {"Argentina", "আৰ্জেণ্টিনাৰ গ্ৰীষ্মকালীন সময়"}, {"Argentina_Western", "পাশ্চাত্য আৰ্জেণ্টিনাৰ গ্ৰীষ্মকালীন সময়"}, {"Armenia", "আৰ্মেনিয়াৰ গ্ৰীষ্মকালীন সময়"}, {"Atlantic", "আটলাণ্টিক ডেলাইট সময়"}, {"Australia_Central", "অষ্ট্ৰেলিয়াৰ কেন্দ্ৰীয় ডেলাইট সময়"}, {"Australia_CentralWestern", "অষ্ট্ৰেলিয়াৰ কেন্দ্ৰীয় পাশ্চাত্য ডেলাইট সময়"}, {"Australia_Eastern", "অষ্ট্ৰেলিয়াৰ প্ৰাচ্য ডেলাইট সময়"}, {"Australia_Western", "অষ্ট্ৰেলিয়াৰ পাশ্চাত্য ডেলাইট সময়"}, {"Azerbaijan", "আজেৰবাইজানৰ গ্ৰীষ্মকালীন সময়"}, {"Azores", "আজোৰেছৰ গ্ৰীষ্মকালীন সময়"}, {"Bangladesh", "বাংলাদেশৰ গ্ৰীষ্মকালীন সময়"}, {"Brasilia", "ব্ৰাজিলিয়াৰ গ্ৰীষ্মকালীন সময়"}, {"Cape_Verde", "কেপ ভাৰ্দেৰ গ্ৰীষ্মকালীন সময়"}, {"Chatham", "চাথামৰ ডেলাইট সময়"}, {"Chile", "চিলিৰ গ্ৰীষ্মকালীন সময়"}, {"China", "চীনৰ ডেলাইট সময়"}, {"Colombia", "কলম্বিয়াৰ গ্ৰীষ্মকালীন সময়"}, {"Cook", "কুক দ্বীপপুঞ্জৰ অৰ্ধ গ্ৰীষ্মকালীন সময়"}, {"Cuba", "কিউবাৰ ডেলাইট সময়"}, {"Easter", "ইষ্টাৰ দ্বীপৰ গ্ৰীষ্মকালীন সময়"}, {"Europe_Central", "মধ্য ইউৰোপীয় গ্ৰীষ্মকালীন সময়"}, {"Europe_Eastern", "প্ৰাচ্য ইউৰোপীয় গ্ৰীষ্মকালীন সময়"}, {"Europe_Western", "পাশ্চাত্য ইউৰোপীয় গ্ৰীষ্মকালীন সময়"}, {"Falkland", "ফকলেণ্ড দ্বীপপুঞ্জৰ গ্ৰীষ্মকালীন সময়"}, {"Fiji", "ফিজিৰ গ্ৰীষ্মকালীন সময়"}, {"Georgia", "জৰ্জিয়াৰ গ্ৰীষ্মকালীন সময়"}, {"Greenland_Eastern", "পূব গ্ৰীণলেণ্ডৰ গ্ৰীষ্মকালীন সময়"}, {"Greenland_Western", "পশ্চিম গ্ৰীণলেণ্ডৰ গ্ৰীষ্মকালীন সময়"}, {"Hawaii_Aleutian", "হাৱাই-এলিউশ্বনৰ ডেলাইট সময়"}, {"Hong_Kong", "হং কঙৰ গ্ৰীষ্মকালীন সময়"}, {"Hovd", "হোভ্\u200cডৰ গ্ৰীষ্মকালীন সময়"}, {"Iran", "ইৰানৰ ডেলাইট সময়"}, {"Irkutsk", "ইৰ্কুটস্কৰ গ্ৰীষ্মকালীন সময়"}, {"Israel", "ইজৰাইলৰ ডেলাইট সময়"}, {"Japan", "জাপানৰ ডেলাইট সময়"}, {"Korea", "কোৰিয়াৰ ডেলাইট সময়"}, {"Krasnoyarsk", "ক্ৰাছনোয়াৰ্স্ক গ্ৰীষ্মকালীন সময়"}, {"Lord_Howe", "লৰ্ড হাওৰ ডেলাইট সময়"}, {"Magadan", "মাগাদানৰ গ্ৰীষ্মকালীন সময়"}, {"Mauritius", "মৰিছাছৰ গ্ৰীষ্মকালীন সময়"}, {"Mexico_Pacific", "মেক্সিকোৰ প্ৰশান্ত ডেলাইট সময়"}, {"Mongolia", "উলানবাটাৰৰ গ্ৰীষ্মকালীন সময়"}, {"Moscow", "মস্কোৰ গ্ৰীষ্মকালীন সময়"}, {"New_Caledonia", "নিউ কেলিডোনিয়াৰ গ্ৰীষ্মকালীন সময়"}, {"New_Zealand", "নিউজিলেণ্ডৰ ডেলাইট সময়"}, {"Newfoundland", "নিউফাউণ্ডলেণ্ডৰ ডেলাইট সময়"}, {"Norfolk", "ন’ৰফ’ক দ্বীপৰ গ্ৰীষ্মকালীন সময়"}, {"Noronha", "ফাৰ্নাণ্ডো ডে নোৰোন্\u200cহাৰ গ্ৰীষ্মকালীন সময়"}, {"Novosibirsk", "নভোছিবিৰ্স্কৰ গ্ৰীষ্মকালীন সময়"}, {"Omsk", "ওমস্কৰ গ্ৰীষ্মকালীন সময়"}, {"Pakistan", "পাকিস্তানৰ গ্ৰীষ্মকালীন সময়"}, {"Paraguay", "পাৰাগুৱেৰ গ্ৰীষ্মকালীন সময়"}, {"Peru", "পেৰুৰ গ্ৰীষ্মকালীন সময়"}, {"Philippines", "ফিলিপাইনৰ গ্ৰীষ্মকালীন সময়"}, {"Pierre_Miquelon", "ছেইণ্ট পিয়েৰে আৰু মিকিউৱেলনৰ ডেলাইট সময়"}, {"Sakhalin", "ছাখালিনৰ গ্ৰীষ্মকালীন সময়"}, {"Samoa", "ছামোৱাৰ ডেলাইট সময়"}, {"Taipei", "টাইপেইৰ ডেলাইট সময়"}, {"Tonga", "টংগাৰ গ্ৰীষ্মকালীন সময়"}, {"Turkmenistan", "তুৰ্কমেনিস্তানৰ গ্ৰীষ্মকালীন সময়"}, {"Uruguay", "উৰুগুৱেৰ গ্ৰীষ্মকালীন সময়"}, {"Uzbekistan", "উজবেকিস্তানৰ গ্ৰীষ্মকালীন সময়"}, {"Vanuatu", "ভানাটুৰ গ্ৰীষ্মকালীন সময়"}, {"Vladivostok", "ভ্লাডিভোষ্টোকৰ গ্ৰীষ্মকালীন সময়"}, {"Volgograd", "ভোল্গোগ্ৰাডৰ গ্ৰীষ্মকালীন সময়"}, {"Yakutsk", "য়াকুত্স্কৰ গ্ৰীষ্মকালীন সময়"}, {"Yekaterinburg", "য়েকাটেৰিণবাৰ্গৰ গ্ৰীষ্মকালীন সময়"}},
	},
	{
		{{"Acre", "hora estándar d’Acre"}, {"Afghanistan", "Hora d’Afganistán"}, {"Africa_Central", "Hora d’África central"}, {"Africa_Eastern", "Hora d’África del este"}, {"Africa_Southern", "Hora de Sudáfrica"}, {"Africa_Western", "Hora d’África del oeste"}, {"Alaska", "Hora estándar d’Alaska"}, {"Almaty", "hora estándar d’Almaty"}, {"Amazon", "Hora estándar del Amazonas"}, {"America_Central", "Hora estándar central norteamericana"}, {"America_Eastern", "Hora estándar del este norteamericanu"}, {"America_Mountain", "Hora estándar de les montañes norteamericanes"}, {"America_Pacific", "Hora estándar del Pacíficu norteamericanu"}, {"Anadyr", "hora estándar d’Anadyr"}, {"Apia", "Hora estándar d’Apia"}, {"Aqtau", "Hora estándar d’Aqtau"}, {"Aqtobe", "Hora estándar d’Aqtobe"}, {"Arabian", "Hora estándar d’Arabia"}, {"Argentina", "Hora estándar d’Arxentina"}, {"Argentina_Western", "Hora estándar occidental d’Arxentina"}, {"Armenia", "Hora estándar d’Armenia"}, {"Atlantic", "Hora estándar del Atlánticu"}, {"Australia_Central", "Hora estándar d’Australia central"}, {"Australia_CentralWestern", "Hora estándar d’Australia central del oeste"}, {"Australia_Eastern", "Hora estándar d’Australia del este"}, {"Australia_Western", "Hora estándar d’Australia del oeste"}, {"Azerbaijan", "Hora estándar d’Azerbaixán"}, {"Azores", "Hora estándar de les Azores"}, {"Bangladesh", "Hora estándar de Bangladex"}, {"Bhutan", "Hora de Bután"}, {"Bolivia", "Hora de Bolivia"}, {"Brasilia", "Hora estándar de Brasilia"}, {"Brunei", "Hora de Brunéi Darussalam"}, {"Cape_Verde", "Hora estándar de Cabu Verde"}, {"Casey", "Hora de Casey"}, {"Chamorro", "Hora estándar de Chamorro"}, {"Chatham", "Hora estándar de Chatham"}, {"Chile", "Hora estándar de Chile"}, {"China", "Hora estándar de China"}, {"Christmas", "Hora estándar de la Islla Christmas"}, {"Cocos", "Hora de les Islles Cocos"}, {"Colombia", "Hora estándar de Colombia"}, {"Cook", "Hora estándar de les Islles Cook"}, {"Cuba", "Hora estándar de Cuba"}, {"Davis", "Hora de Davis"}, {"DumontDUrville", "Hora de Dumont-d’Urville"}, {"East_Timor", "Hora de Timor Oriental"}, {"Easter", "Hora estándar de la Islla de Pascua"}, {"Ecuador", "Hora d’Ecuador"}, {"Europe_Central", "Hora estándar d’Europa Central"}, {"Europe_Eastern", "Hora estándar d’Europa del Este"}, {"Europe_Further_Eastern", "Hora d’Europa del estremu este"}, {"Europe_Western", "Hora estándar d’Europa Occidental"}, {"Falkland", "Hora estándar de les Islles Falkland"}, {"Fiji", "Hora estándar de Fixi"}, {"French_Guiana", "Hora de La Guyana Francesa"}, {"French_Southern", "Hora del sur y l’antárticu francés"}, {"GMT", "Hora media de Greenwich"}, {"Galapagos", "Hora de Galápagos"}, {"Gambier", "Hora de Gambier"}, {"Georgia", "Hora estándar de Xeorxa"}, {"Gilbert_Islands", "Hora de les Islles Gilbert"}, {"Greenland_Eastern", "Hora estándar de Groenlandia oriental"}, {"Greenland_Western", "Hora estándar de Groenlandia occidental"}, {"Guam", "Hora estándar de Guam"}, {"Gulf", "Hora estándar del Golfu"}, {"Guyana", "Hora de La Guyana"}, {"Hawaii", "Hora estándar de Hawaii-Aleutianes"}, {"Hawaii_Aleutian", "Hora estándar de Hawaii-Aleutianes"}, {"Hong_Kong", "Hora estándar de Ḥong Kong"}, {"Hovd", "Hora estándar de Hovd"}, {"India", "Hora estándar de la India"}, {"Indian_Ocean", "Hora del Océanu Índicu"}, {"Indochina", "Hora d’Indochina"}, {"Indonesia_Central", "Hora d’Indonesia central"}, {"Indonesia_Eastern", "Hora d’Indonesia del este"}, {"Indonesia_Western", "Hora d’Indonesia del oeste"}, {"Iran", "Hora estándar d’Irán"}, {"Irkutsk", "Hora estándar d’Irkutsk"}, {"Israel", "Hora estándar d’Israel"}, {"Japan", "Hora estándar de Xapón"}, {"Kamchatka", "hora estandar de Petropavlovsk-Kamchatski"}, {"Kazakhstan_Eastern", "Hora del Kazakstán oriental"}, {"Kazakhstan_Western", "Hora del Kazakstán occidental"}, {"Korea", "Hora estándar de Corea"}, {"Kosrae", "Hora de Kosrae"}, {"Krasnoyarsk", "Hora estándar de Krasnoyarsk"}, {"Kyrgystan", "Hora del Kirguistán"}, {"Lanka", "Hora de Lanka"}, {"Line_Islands", "Hora de les Islles Line"}, {"Lord_Howe", "Hora estándar de Lord Howe"}, {"Macau", "Hora estándar de Macáu"}, {"Magadan", "Hora estándar de Magadán"}, {"Malaysia", "Hora de Malasia"}, {"Maldives", "Hora de Les Maldives"}, {"Marquesas", "Hora de les Marqueses"}, {"Marshall_Islands", "Hora de les Islles Marshall"}, {"Mauritius", "Hora estándar de Mauriciu"}, {"Mawson", "Hora de Mawson"}, {"Mexico_Pacific", "Hora estándar del Pacíficu de Méxicu"}, {"Mongolia", "Hora estándar d’Ulán Bátor"}, {"Moscow", "Hora estándar de Moscú"}, {"Myanmar", "Hora de Myanmar"}, {"Nauru", "Hora de Nauru"}, {"Nepal", "Hora del Nepal"}, {"New_Caledonia", "Hora estándar de Nueva Caledonia"}, {"New_Zealand", "Hora estándar de Nueva Zelanda"}, {"Newfoundland", "Hora estándar de Newfoundland"}, {"Niue", "Hora de Niue"}, {"Norfolk", "Hora estándar de la Islla Norfolk"}, {"Noronha", "Hora estándar de Fernando de Noronha"}, {"North_Mariana", "Hora de les Islles Marianes del Norte"}, {"Novosibirsk", "Hora estándar de Novosibirsk"}, {"Omsk", "Hora estándar d’Omsk"}, {"Pakistan", "Hora estándar del Paquistán"}, {"Palau", "Hora de Palau"}, {"Papua_New_Guinea", "Hora de Papúa Nueva Guinea"}, {"Paraguay", "Hora estándar del Paraguái"}, {"Peru", "Hora estándar del Perú"}, {"Philippines", "Hora estándar de Filipines"}, {"Phoenix_Islands", "Hora de les Islles Phoenix"}, {"Pierre_Miquelon", "Hora estándar de Saint Pierre y Miquelon"}, {"Pitcairn", "Hora de Pitcairn"}, {"Ponape", "Hora de Ponape"}, {"Pyongyang", "hora de Pyongyang"}, {"Qyzylorda", "Hora estándar de Qyzylorda"}, {"Reunion", "Hora de Reunión"}, {"Rothera", "Hora de Rothera"}, {"Sakhalin", "Hora estándar de Saxalín"}, {"Samara", "Hora estándar de Samara"}, {"Samoa", "Hora estándar de Samoa"}, {"Seychelles", "Hora de Les Seixeles"}, {"Singapore", "Hora estándar de Singapur"}, {"Solomon", "Hora de les Islles Salomón"}, {"South_Georgia", "Hora de Xeorxa del Sur"}, {"Suriname", "Hora del Surinam"}, {"Syowa", "Hora de Syowa"}, {"Tahiti", "Hora de Tahiti"}, {"Taipei", "Hora estándar de Taipéi"}, {"Tajikistan", "Hora del Taxiquistán"}, {"Tokelau", "Hora de Tokelau"}, {"Tonga", "Hora estándar de Tonga"}, {"Truk", "Hora de Chuuk"}, {"Turkmenistan", "Hora estándar del Turkmenistán"}, {"Tuvalu", "Hora de Tuvalu"}, {"Uruguay", "Hora estándar del Uruguái"}, {"Uzbekistan", "Hora estándar del Uzbequistán"}, {"Vanuatu", "Hora estándar de Vanuatu"}, {"Venezuela", "Hora de Venezuela"}, {"Vladivostok", "Hora estándar de Vladivostok"}, {"Volgograd", "Hora estándar de Volgográu"}, {"Vostok", "Hora de Vostok"}, {"Wake", "Hora de la Islla Wake"}, {"Wallis", "Hora de Wallis y Futuna"}, {"Yakutsk", "Hora estándar de Yakutsk"}, {"Yekaterinburg", "Hora estándar de Yekaterimburgu"}},
		{{"Acre", "hora braniega d’Acre"}, {"Alaska", "Hora braniega d’Alaska"}, {"Almaty", "hora braniega d’Almaty"}, {"Amazon", "Hora braniega del Amazonas"}, {"America_Central", "Hora braniega central norteamericana"}, {"America_Eastern", "Hora braniega del este norteamericanu"}, {"America_Mountain", "Hora braniega de les montañes norteamericanes"}, {"America_Pacific", "Hora braniega del Pacíficu norteamericanu"}, {"Anadyr", "hora braniega d’Anadyr"}, {"Apia", "Hora braniega d’Apia"}, {"Aqtau", "Hora braniega d’Aqtau"}, {"Aqtobe", "Hora braniega d’Aqtobe"}, {"Arabian", "Hora braniega d’Arabia"}, {"Argentina", "Hora braniega d’Arxentina"}, {"Argentina_Western", "Hora braniega occidental d’Arxentina"}, {"Armenia", "Hora braniega d’Armenia"}, {"Atlantic", "Hora braniega del Atlánticu"}, {"Australia_Central", "Hora braniega d’Australia central"}, {"Australia_CentralWestern", "Hora braniega d’Australia central del oeste"}, {"Australia_Eastern", "Hora braniega d’Australia del este"}, {"Australia_Western", "Hora braniega d’Australia del oeste"}, {"Azerbaijan", "Hora braniega d’Azerbaixán"}, {"Azores", "Hora braniega de Les Azores"}, {"Bangladesh", "Hora braniega de Bangladex"}, {"Brasilia", "Hora braniega de Brasilia"}, {"Cape_Verde", "Hora braniega de Cabu Verde"}, {"Chatham", "Hora braniega de Chatham"}, {"Chile", "Hora braniega de Chile"}, {"China", "Hora braniega de China"}, {"Colombia", "Hora braniega de Colombia"}, {"Cook", "Hora media braniega de les Islles Cook"}, {"Cuba", "Hora braniega de Cuba"}, {"Easter", "Hora braniega de la Islla de Pascua"}, {"Europe_Central", "Hora braniega d’Europa Central"}, {"Europe_Eastern", "Hora braniega d’Europa del Este"}, {"Europe_Western", "Hora braniega d’Europa Occidental"}, {"Falkland", "Hora braniega de les Islles Falkland"}, {"Fiji", "Hora braniega de Fixi"}, {"Georgia", "Hora braniega de Xeorxa"}, {"Greenland_Eastern", "Hora braniega de Groenlandia oriental"}, {"Greenland_Western", "Hora braniega de Groenlandia occidental"}, {"Hawaii_Aleutian", "Hora braniega de Hawaii-Aleutianes"}, {"Hong_Kong", "Hora braniega de Ḥong Kong"}, {"Hovd", "Hora braniega de Hovd"}, {"Iran", "Hora braniega d’Irán"}, {"Irkutsk", "Hora braniega d’Irkutsk"}, {"Israel", "Hora braniega d’Israel"}, {"Japan", "Hora braniega de Xapón"}, {"Kamchatka", "hora braniega de Petropavlovsk-Kamchatski"}, {"Korea", "Hora braniega de Corea"}, {"Krasnoyarsk", "Hora braniega de Krasnoyarsk"}, {"Lord_Howe", "Hora braniega de Lord Howe"}, {"Macau", "Hora braniega de Macáu"}, {"Magadan", "Hora braniega de Magadán"}, {"Mauritius", "Hora braniega de Mauriciu"}, {"Mexico_Pacific", "Hora braniega del Pacíficu de Méxicu"}, {"Mongolia", "Hora braniega d’Ulán Bátor"}, {"Moscow", "Hora braniega de Moscú"}, {"New_Caledonia", "Hora braniega de Nueva Caledonia"}, {"New_Zealand", "Hora braniega de Nueva Zelanda"}, {"Newfoundland", "Hora braniega de Newfoundland"}, {"Norfolk", "Hora braniega de la Islla Norfolk"}, {"Noronha", "Hora braniega de Fernando de Noronha"}, {"Novosibirsk", "Hora braniega de Novosibirsk"}, {"Omsk", "Hora braniega d’Omsk"}, {"Pakistan", "Hora braniega del Paquistán"}, {"Paraguay", "Hora braniega del Paraguái"}, {"Peru", "Hora braniega del Perú"}, {"Philippines", "Hora de branu de Filipines"}, {"Pierre_Miquelon", "Hora braniega de Saint Pierre y Miquelon"}, {"Qyzylorda", "Hora braniega de Qyzylorda"}, {"Sakhalin", "Hora braniega de Saxalín"}, {"Samara", "Hora braniega de Samara"}, {"Samoa", "Hora braniega de Samoa"}, {"Taipei", "Hora braniega de Taipéi"}, {"Tonga", "Hora braniega de Tonga"}, {"Turkmenistan", "Hora braniega del Turkmenistán"}, {"Uruguay", "Hora braniega del Uruguái"}, {"Uzbekistan", "Hora braniega del Uzbequistán"}, {"Vanuatu", "Hora braniega de Vanuatu"}, {"Vladivostok", "Hora braniega de Vladivostok"}, {"Volgograd", "Hora braniega de Volgográu"}, {"Yakutsk", "Hora braniega de Yakutsk"}, {"Yekaterinburg", "Hora braniega de Yekaterimburgu"}},
	},
	{
		{{"Afghanistan", "Əfqanıstan Vaxtı"}, {"Africa_Central", "Mərkəzi Afrika Vaxtı"}, {"Africa_Eastern", "Şərqi Afrika Vaxtı"}, {"Africa_Southern", "Cənubi Afrika Vaxtı"}, {"Africa_Western", "Qərbi Afrika Vaxtı"}, {"Alaska", "Alyaska Standart Vaxtı"}, {"Amazon", "Amazon Standart Vaxtı"}, {"America_Central", "Mərkəzi Amerika Standart Vaxtı"}, {"America_Eastern", "Şimali Şərqi Amerika Standart Vaxtı"}, {"America_Mountain", "Şimali Dağlıq Amerika Standart Vaxtı"}, {"America_Pacific", "Şimali Amerika Sakit Okean Standart Vaxtı"}, {"Apia", "Apia Standart Vaxtı"}, {"Arabian", "Ərəbistan Standart Vaxtı"}, {"Argentina", "Argentina Standart Vaxtı"}, {"Argentina_Western", "Qərbi Argentina Standart Vaxtı"}, {"Armenia", "Ermənistan Standart Vaxtı"}, {"Atlantic", "Atlantik Standart Vaxt"}, {"Australia_Central", "Mərkəzi Avstraliya Standart Vaxtı"}, {"Australia_CentralWestern", "Mərkəzi Qərbi Avstraliya Standart Vaxtı"}, {"Australia_Eastern", "Şərqi Avstraliya Standart Vaxtı"}, {"Australia_Western", "Qərbi Avstraliya Standart Vaxtı"}, {"Azerbaijan", "Azərbaycan Standart Vaxtı"}, {"Azores", "Azor Standart Vaxtı"}, {"Bangladesh", "Banqladeş Standart Vaxtı"}, {"Bhutan", "Butan Vaxtı"}, {"Bolivia", "Boliviya Vaxtı"}, {"Brasilia", "Braziliya Standart Vaxtı"}, {"Brunei", "Brunei Darussalam vaxtı"}, {"Cape_Verde", "Kape Verde Standart Vaxtı"}, {"Chamorro", "Çamorro Vaxtı"}, {"Chatham", "Çatham Standart Vaxtı"}, {"Chile", "Çili Standart Vaxtı"}, {"China", "Çin Standart Vaxtı"}, {"Christmas", "Milad Adası Vaxtı"}, {"Cocos", "Kokos Adaları Vaxtı"}, {"Colombia", "Kolumbiya Standart Vaxtı"}, {"Cook", "Kuk Adaları Standart Vaxtı"}, {"Cuba", "Kuba Standart Vaxtı"}, {"Davis", "Devis Vaxtı"}, {"DumontDUrville", "Dümon-d’Ürvil Vaxtı"}, {"East_Timor", "Şərqi Timor Vaxtı"}, {"Easter", "Pasxa Adası Standart Vaxtı"}, {"Ecuador", "Ekvador Vaxtı"}, {"Europe_Central", "Mərkəzi Avropa Standart Vaxtı"}, {"Europe_Eastern", "Şərqi Avropa Standart Vaxtı"}, {"Europe_Further_Eastern", "Kənar Şərqi Avropa Vaxtı"}, {"Europe_Western", "Qərbi Avropa Standart Vaxtı"}, {"Falkland", "Folklend Adaları Standart Vaxtı"}, {"Fiji", "Fici Standart Vaxtı"}, {"French_Guiana", "Fransız Qvianası Vaxtı"}, {"French_Southern", "Fransız Cənubi və Antarktik Vaxtı"}, {"GMT", "Qrinviç Orta Vaxtı"}, {"Galapagos", "Qalapaqos Vaxtı"}, {"Gambier", "Qambier Vaxtı"}, {"Georgia", "Gurcüstan Standart Vaxtı"}, {"Gilbert_Islands", "Gilbert Adaları Vaxtı"}, {"Greenland_Eastern", "Şərqi Qrenlandiya Standart Vaxtı"}, {"Greenland_Western", "Qərbi Qrenlandiya Standart Vaxtı"}, {"Gulf", "Körfəz Vaxtı"}, {"Guyana", "Qayana Vaxtı"}, {"Hawaii", "Havay-Aleut Standart Vaxtı"}, {"Hawaii_Aleutian", "Havay-Aleut Standart Vaxtı"}, {"Hong_Kong", "Honq Konq Standart Vaxtı"}, {"Hovd", "Hovd Standart Vaxtı"}, {"India", "Hindistan Vaxtı"}, {"Indian_Ocean", "Hind Okeanı Vaxtı"}, {"Indochina", "Hindçin Vaxtı"}, {"Indonesia_Central", "Mərkəzi İndoneziya Vaxtı"}, {"Indonesia_Eastern", "Şərqi İndoneziya Vaxtı"}, {"Indonesia_Western", "Qərbi İndoneziya Vaxtı"}, {"Iran", "İran Standart Vaxtı"}, {"Irkutsk", "İrkutsk Standart Vaxtı"}, {"Israel", "İsrail Standart Vaxtı"}, {"Japan", "Yaponiya Standart Vaxtı"}, {"Kazakhstan", "Qazaxıstan vaxtı"}, {"Kazakhstan_Eastern", "Şərqi Qazaxıstan Vaxtı"}, {"Kazakhstan_Western", "Qərbi Qazaxıstan Vaxtı"}, {"Korea", "Koreya Standart Vaxtı"}, {"Kosrae", "Korse Vaxtı"}, {"Krasnoyarsk", "Krasnoyarsk Standart Vaxtı"}, {"Kyrgystan", "Qırğızıstan Vaxtı"}, {"Line_Islands", "Layn Adaları Vaxtı"}, {"Lord_Howe", "Lord Hau Standart Vaxtı"}, {"Magadan", "Maqadan Standart Vaxtı"}, {"Malaysia", "Malayziya Vaxtı"}, {"Maldives", "Maldiv Vaxtı"}, {"Marquesas", "Markesas Vaxtı"}, {"Marshall_Islands", "Marşal Adaları Vaxtı"}, {"Mauritius", "Mavriki Standart Vaxtı"}, {"Mawson", "Mouson Vaxtı"}, {"Mexico_Pacific", "Meksika Sakit Okean Standart Vaxtı"}, {"Mongolia", "Ulanbator Standart Vaxtı"}, {"Moscow", "Moskva Standart Vaxtı"}, {"Myanmar", "Myanma Vaxtı"}, {"Nauru", "Nauru Vaxtı"}, {"Nepal", "Nepal vaxtı"}, {"New_Caledonia", "Yeni Kaledoniya Standart Vaxtı"}, {"New_Zealand", "Yeni Zelandiya Standart Vaxtı"}, {"Newfoundland", "Nyufaundlend Standart Vaxtı"}, {"Niue", "Niue Vaxtı"}, {"Norfolk", "Norfolk Adası Standart Vaxtı"}, {"Noronha", "Fernando de Noronya Standart Vaxtı"}, {"Novosibirsk", "Novosibirsk Standart Vaxtı"}, {"Omsk", "Omsk Standart Vaxtı"}, {"Pakistan", "Pakistan Standart vaxtı"}, {"Palau", "Palau Vaxtı"}, {"Papua_New_Guinea", "Papua Yeni Qvineya Vaxtı"}, {"Paraguay", "Paraqvay Standart Vaxtı"}, {"Peru", "Peru Standart Vaxtı"}, {"Philippines", "Filippin Standart Vaxtı"}, {"Phoenix_Islands", "Feniks Adaları Vaxtı"}, {"Pierre_Miquelon", "Müqəddəs Pyer və Mikelon Standart Vaxtı"}, {"Pitcairn", "Pitkern Vaxtı"}, {"Ponape", "Ponape Vaxtı"}, {"Pyongyang", "Pxenyan Vaxtı"}, {"Reunion", "Reyunyon"}, {"Rothera", "Rotera Vaxtı"}, {"Sakhalin", "Saxalin Standart Vaxtı"}, {"Samara", "Samara standart vaxtı"}, {"Samoa", "Samoa Standart Vaxtı"}, {"Seychelles", "Seyşel Adaları Vaxtı"}, {"Singapore", "Sinqapur Vaxtı"}, {"Solomon", "Solomon Adaları Vaxtı"}, {"South_Georgia", "Cənubi Corciya Vaxtı"}, {"Suriname", "Surinam Vaxtı"}, {"Syowa", "Syova Vaxtı"}, {"Tahiti", "Tahiti Vaxtı"}, {"Taipei", "Taybey Standart Vaxtı"}, {"Tajikistan", "Tacikistan Vaxtı"}, {"Tokelau", "Tokelau Vaxtı"}, {"Tonga", "Tonqa Standart Vaxtı"}, {"Truk", "Çuuk Vaxtı"}, {"Turkey", "Türkiya standart vaxtı"}, {"Turkmenistan", "Türkmənistan Standart Vaxtı"}, {"Tuvalu", "Tuvalu Vaxtı"}, {"Uruguay", "Uruqvay Standart Vaxtı"}, {"Uzbekistan", "Özbəkistan Standart Vaxtı"}, {"Vanuatu", "Vanuatu Standart Vaxtı"}, {"Venezuela", "Venesuela Vaxtı"}, {"Vladivostok", "Vladivostok Standart Vaxtı"}, {"Volgograd", "Volqoqrad Standart Vaxtı"}, {"Vostok", "Vostok Vaxtı"}, {"Wake", "Ueyk Vaxtı"}, {"Wallis", "Uollis və Futuna Vaxtı"}, {"Yakutsk", "Yakutsk Standart Vaxtı"}, {"Yekaterinburg", "Yekaterinburq Standart Vaxtı"}, {"Yukon", "Yukon Vaxtı"}},
		{{"Alaska", "Alyaska Yay Vaxtı"}, {"Amazon", "Amazon Yay Vaxtı"}, {"America_Central", "Mərkəzi Amerika yay vaxtı"}, {"America_Eastern", "Şimali Şərqi Amerika Yay Vaxtı"}, {"America_Mountain", "Şimali Dağlıq Amerika Yay Vaxtı"}, {"America_Pacific", "Şimali Amerika Sakit Okean Yay Vaxtı"}, {"Apia", "Apia Yay Vaxtı"}, {"Arabian", "Ərəbistan Yay Vaxtı"}, {"Argentina", "Argentina Yay Vaxtı"}, {"Argentina_Western", "Qərbi Argentina Yay Vaxtı"}, {"Armenia", "Ermənistan Yay Vaxtı"}, {"Atlantic", "Atlantik Yay Vaxtı"}, {"Australia_Central", "Mərkəzi Avstraliya Yay Vaxtı"}, {"Australia_CentralWestern", "Mərkəzi Qərbi Avstraliya Yay Vaxtı"}, {"Australia_Eastern", "Şərqi Avstraliya Yay Vaxtı"}, {"Australia_Western", "Qərbi Avstraliya Yay Vaxtı"}, {"Azerbaijan", "Azərbaycan Yay Vaxtı"}, {"Azores", "Azor Yay Vaxtı"}, {"Bangladesh", "Banqladeş Yay Vaxtı"}, {"Brasilia", "Braziliya Yay Vaxtı"}, {"Cape_Verde", "Kape Verde Yay Vaxtı"}, {"Chatham", "Çatham Yay Vaxtı"}, {"Chile", "Çili Yay Vaxtı"}, {"China", "Çin Yay Vaxtı"}, {"Colombia", "Kolumbiya Yay Vaxtı"}, {"Cook", "Kuk Adaları Yarım Yay Vaxtı"}, {"Cuba", "Kuba Yay Vaxtı"}, {"Easter", "Pasxa Adası Yay Vaxtı"}, {"Europe_Central", "Mərkəzi Avropa Yay Vaxtı"}, {"Europe_Eastern", "Şərqi Avropa Yay Vaxtı"}, {"Europe_Western", "Qərbi Avropa Yay Vaxtı"}, {"Falkland", "Folklend Adaları Yay Vaxtı"}, {"Fiji", "Fici Yay Vaxtı"}, {"Georgia", "Gurcüstan Yay Vaxtı"}, {"Greenland_Eastern", "Şərqi Qrenlandiya Yay Vaxtı"}, {"Greenland_Western", "Qərbi Qrenlandiya Yay Vaxtı"}, {"Hawaii_Aleutian", "Havay-Aleut Yay Vaxtı"}, {"Hong_Kong", "Honq Konq Yay Vaxtı"}, {"Hovd", "Hovd Yay Vaxtı"}, {"Iran", "İran Yay Vaxtı"}, {"Irkutsk", "İrkutsk Yay Vaxtı"}, {"Israel", "İsrail Yay Vaxtı"}, {"Japan", "Yaponiya Yay Vaxtı"}, {"Korea", "Koreya Yay Vaxtı"}, {"Krasnoyarsk", "Krasnoyarsk Yay Vaxtı"}, {"Lord_Howe", "Lord Hau Yay vaxtı"}, {"Magadan", "Maqadan Yay Vaxtı"}, {"Mauritius", "Mavriki Yay Vaxtı"}, {"Mexico_Pacific", "Meksika Sakit Okean Yay Vaxtı"}, {"Mongolia", "Ulanbator Yay Vaxtı"}, {"Moscow", "Moskva Yay vaxtı"}, {"New_Caledonia", "Yeni Kaledoniya Yay Vaxtı"}, {"New_Zealand", "Yeni Zelandiya Yay Vaxtı"}, {"Newfoundland", "Nyufaundlend Yay Vaxtı"}, {"Norfolk", "Norfolk Adası Yay Vaxtı"}, {"Noronha", "Fernando de Noronya Yay Vaxtı"}, {"Novosibirsk", "Novosibirsk Yay Vaxtı"}, {"Omsk", "Omsk Yay Vaxtı"}, {"Pakistan", "Pakistan Yay Vaxtı"}, {"Paraguay", "Paraqvay Yay Vaxtı"}, {"Peru", "Peru Yay Vaxtı"}, {"Philippines", "Filippin Yay Vaxtı"}, {"Pierre_Miquelon", "Müqəddəs Pyer və Mikelon Yay Vaxtı"}, {"Sakhalin", "Saxalin Yay Vaxtı"}, {"Samara", "Samara yay vaxtı"}, {"Samoa", "Samoa Yay Vaxtı"}, {"Taipei", "Taybey Yay Vaxtı"}, {"Tonga", "Tonqa Yay Vaxtı"}, {"Turkey", "Türkiya yay vaxtı"}, {"Turkmenistan", "Türkmənistan Yay Vaxtı"}, {"Uruguay", "Uruqvay Yay Vaxtı"}, {"Uzbekistan", "Özbəkistan Yay Vaxtı"}, {"Vanuatu", "Vaunatu Yay Vaxtı"}, {"Vladivostok", "Vladivostok Yay Vaxtı"}, {"Volgograd", "Volqoqrad Yay Vaxtı"}, {"Yakutsk", "Yakutsk Yay Vaxtı"}, {"Yekaterinburg", "Yekaterinburq Yay Vaxtı"}},
	},
	{
		{{"Acre", "Акри стандарт ваҡыты"}, {"Afghanistan", "Афғанстан ваҡыты"}, {"Africa_Central", "Үҙәк Африка ваҡыты"}, {"Africa_Eastern", "Көнсығыш Африка ваҡыты"}, {"Africa_Southern", "Көньяҡ Африка стандарт ваҡыты"}, {"Africa_Western", "Көнбайыш Африка ваҡыты"}, {"Alaska", "Аляска стандарт ваҡыты"}, {"Almaty", "Алматы стандарт ваҡыты"}, {"Amazon", "Амазон стандарт ваҡыты"}, {"America_Central", "Үҙәк Америка стандарт ваҡыты"}, {"America_Eastern", "Көнсығыш Америка стандарт ваҡыты"}, {"America_Mountain", "Төньяҡ Америка Тау стандарт ваҡыты"}, {"America_Pacific", "Тымыҡ океан стандарт ваҡыты"}, {"Anadyr", "Анадырь стандарт ваҡыты"}, {"Apia", "Самоа стандарт ваҡыты"}, {"Aqtau", "Аҡтау стандарт ваҡыты"}, {"Aqtobe", "Аҡтүбә стандарт ваҡыты"}, {"Arabian", "Сәғүд Ғәрәбстаны стандарт ваҡыты"}, {"Argentina", "Аргентина стандарт ваҡыты"}, {"Argentina_Western", "Көнбайыш Аргентина стандарт ваҡыты"}, {"Armenia", "Әрмәнстан стандарт ваҡыты"}, {"Atlantic", "Атлантик стандарт ваҡыты"}, {"Australia_Central", "Үҙәк Австралия стандарт ваҡыты"}, {"Australia_CentralWestern", "Үҙәк Австралия көнбайыш стандарт ваҡыты"}, {"Australia_Eastern", "Көнсығыш Австралия стандарт ваҡыты"}, {"Australia_Western", "Көнбайыш Австралия стандарт ваҡыты"}, {"Azerbaijan", "Әзербайжан стандарт ваҡыты"}, {"Azores", "Азор утрауҙары стандарт ваҡыты"}, {"Bangladesh", "Бангладеш стандарт ваҡыты"}, {"Bhutan", "Бутан ваҡыты"}, {"Bolivia", "Боливия ваҡыты"}, {"Brasilia", "Бразилия стандарт ваҡыты"}, {"Brunei", "Бруней ваҡыты"}, {"Cape_Verde", "Кабо-Верде стандарт ваҡыты"}, {"Casey", "Кейси ваҡыты"}, {"Chamorro", "Чаморро стандарт ваҡыты"}, {"Chatham", "Чатем стандарт ваҡыты"}, {"Chile", "Чили стандарт ваҡыты"}, {"China", "Ҡытай стандарт ваҡыты"}, {"Christmas", "Раштыуа утрауы ваҡыты"}, {"Cocos", "Кокос утрауҙары ваҡыты"}, {"Colombia", "Колумбия стандарт ваҡыты"}, {"Cook", "Кук утрауҙары стандарт ваҡыты"}, {"Cuba", "Куба стандарт ваҡыты"}, {"Davis", "Дейвис ваҡыты"}, {"DumontDUrville", "Дюмон-д’Юрвиль ваҡыты"}, {"East_Timor", "Көнсығыш Тимор ваҡыты"}, {"Easter", "Пасха утрауы стандарт ваҡыты"}, {"Ecuador", "Эквадор ваҡыты"}, {"Europe_Central", "Үҙәк Европа стандарт ваҡыты"}, {"Europe_Eastern", "Көнсығыш Европа стандарт ваҡыты"}, {"Europe_Further_Eastern", "Мәскәү ваҡыты"}, {"Europe_Western", "Көнбайыш Европа стандарт ваҡыты"}, {"Falkland", "Фолкленд утрауҙары стандарт ваҡыты"}, {"Fiji", "Фиджи стандарт ваҡыты"}, {"French_Guiana", "Француз Гвианаһы ваҡыты"}, {"French_Southern", "Француз Көньяҡ һәм Антарктика ваҡыты"}, {"GMT", "Гринвич буйынса уртаса ваҡыт"}, {"Galapagos", "Галапагос ваҡыты"}, {"Gambier", "Гамбье ваҡыты"}, {"Georgia", "Грузия стандарт ваҡыты"}, {"Gilbert_Islands", "Гилберт утрауҙары ваҡыты"}, {"Greenland", "Гренландия стандарт ваҡыты"}, {"Greenland_Eastern", "Көнсығыш Гренландия стандарт ваҡыты"}, {"Greenland_Western", "Көнбайыш Гренландия стандарт ваҡыты"}, {"Guam", "Гуам стандарт ваҡыты"}, {"Gulf", "Фарсы ҡултығы стандарт ваҡыты"}, {"Guyana", "Гайана ваҡыты"}, {"Hawaii", "Гавай-Алеут стандарт ваҡыты"}, {"Hawaii_Aleutian", "Гавай-Алеут стандарт ваҡыты"}, {"Hong_Kong", "Гонконг стандарт ваҡыты"}, {"Hovd", "Ховд стандарт ваҡыты"}, {"India", "Һиндостан стандарт ваҡыты"}, {"Indian_Ocean", "Һинд океаны ваҡыты"}, {"Indochina", "Һинд-Ҡытай ваҡыты"}, {"Indonesia_Central", "Үҙәк Индонезия ваҡыты"}, {"Indonesia_Eastern", "Көнсығыш Индонезия"}, {"Indonesia_Western", "Көнбайыш Индонезия"}, {"Iran", "Иран стандарт ваҡыты"}, {"Irkutsk", "Иркутск стандарт ваҡыты"}, {"Israel", "Израиль стандарт ваҡыты"}, {"Japan", "Япония стандарт ваҡыты"}, {"Kamchatka", "Камчатка стандарт ваҡыты"}, {"Kazakhstan", "Ҡаҙағстан ваҡыты"}, {"Kazakhstan_Eastern", "Көнсығыш Ҡаҙағстан ваҡыты"}, {"Kazakhstan_Western", "Көнбайыш Ҡаҙағстан ваҡыты"}, {"Korea", "Корея стандарт ваҡыты"}, {"Kosrae", "Косрае ваҡыты"}, {"Krasnoyarsk", "Красноярск стандарт ваҡыты"}, {"Kyrgystan", "Ҡырғыҙстан ваҡыты"}, {"Lanka", "Ланка ваҡыты"}, {"Line_Islands", "Лайн утрауҙары ваҡыты"}, {"Lord_Howe", "Лорд-Хау стандарт ваҡыты"}, {"Macau", "Макао стандарт ваҡыты"}, {"Magadan", "Магадан стандарт ваҡыты"}, {"Malaysia", "Малайзия ваҡыты"}, {"Maldives", "Мальдив утрауҙары ваҡыты"}, {"Marquesas", "Маркиз утрауҙары ваҡыты"}, {"Marshall_Islands", "Маршалл Утрауҙары ваҡыты"}, {"Mauritius", "Маврикий стандарт ваҡыты"}, {"Mawson", "Моусон ваҡыты"}, {"Mexico_Pacific", "Мексика Тымыҡ океан стандарт ваҡыты"}, {"Mongolia", "Улан-Батор стандарт ваҡыты"}, {"Moscow", "Мәскәү стандарт ваҡыты"}, {"Myanmar", "Мьянма ваҡыты"}, {"Nauru", "Науру ваҡыты"}, {"Nepal", "Непал ваҡыты"}, {"New_Caledonia", "Яңы Каледония стандарт ваҡыты"}, {"New_Zealand", "Яңы Зеландия стандарт ваҡыты"}, {"Newfoundland", "Ньюфаундленд стандарт ваҡыты"}, {"Niue", "Ниуэ ваҡыты"}, {"Norfolk", "Норфолк стандарт ваҡыты"}, {"Noronha", "Фернанду-ди-Норонья стандарт ваҡыты"}, {"Novosibirsk", "Новосибирск стандарт ваҡыты"}, {"Omsk", "Омск стандарт ваҡыты"}, {"Pakistan", "Пакистан стандарт ваҡыты"}, {"Palau", "Палау ваҡыты"}, {"Papua_New_Guinea", "Папуа – Яңы Гвинея"}, {"Paraguay", "Парагвай стандарт ваҡыты"}, {"Peru", "Перу стандарт ваҡыты"}, {"Philippines", "Филиппин стандарт ваҡыты"}, {"Phoenix_Islands", "Феникс утрауҙары ваҡыты"}, {"Pierre_Miquelon", "Сен-Пьер һәм Микелон стандарт ваҡыты"}, {"Pitcairn", "Питкэрн ваҡыты"}, {"Ponape", "Понпеи ваҡыты"}, {"Pyongyang", "Төньяҡ Корея ваҡыты"}, {"Qyzylorda", "Ҡыҙылурҙа стандарт ваҡыты"}, {"Reunion", "Реюньон ваҡыты"}, {"Rothera", "Ротера ваҡыты"}, {"Sakhalin", "Сахалин стандарт ваҡыты"}, {"Samara", "Һамар стандарт ваҡыты"}, {"Samoa", "Америка Самоаһы стандарт ваҡыты"}, {"Seychelles", "Сейшел утрауҙары ваҡыты"}, {"Singapore", "Сингапур стандарт ваҡыты"}, {"Solomon", "Соломон утрауҙары ваҡыты"}, {"South_Georgia", "Көньяҡ Георгия ваҡыты"}, {"Suriname", "Суринам ваҡыты"}, {"Syowa", "Сёва ваҡыты"}, {"Tahiti", "Таити ваҡыты"}, {"Taipei", "Тайвань стандарт ваҡыты"}, {"Tajikistan", "Тажикстан ваҡыты"}, {"Tokelau", "Токелау ваҡыты"}, {"Tonga", "Тонга стандарт ваҡыты"}, {"Truk", "Чуук ваҡыты"}, {"Turkmenistan", "Төркмәнстан стандарт ваҡыты"}, {"Tuvalu", "Тувалу ваҡыты"}, {"Uruguay", "Уругвай стандарт ваҡыты"}, {"Uzbekistan", "Үзбәкстан стандарт ваҡыты"}, {"Vanuatu", "Вануату стандарт ваҡыты"}, {"Venezuela", "Венесуэла ваҡыты"}, {"Vladivostok", "Владивосток стандарт ваҡыты"}, {"Volgograd", "Волгоград стандарт ваҡыты"}, {"Vostok", "Восток ваҡыты"}, {"Wake", "Уэйк утрауы ваҡыты"}, {"Wallis", "Уоллис һәм Футуна ваҡыты"}, {"Yakutsk", "Якутск стандарт ваҡыты"}, {"Yekaterinburg", "Екатеринбург стандарт ваҡыты"}, {"Yukon", "Юкон ваҡыты"}},
		{{"Acre", "Акри йәйге ваҡыты"}, {"Alaska", "Аляска йәйге ваҡыты"}, {"Almaty", "Алматы йәйге ваҡыты"}, {"Amazon", "Амазон йәйге ваҡыты"}, {"America_Central", "Үҙәк Америка йәйге ваҡыты"}, {"America_Eastern", "Көнсығыш Америка йәйге ваҡыты"}, {"America_Mountain", "Төньяҡ Америка Тау йәйге ваҡыты"}, {"America_Pacific", "Тымыҡ океан йәйге ваҡыты"}, {"Anadyr", "Анадырь йәйге ваҡыты"}, {"Apia", "Самоа йәйге ваҡыты"}, {"Aqtau", "Аҡтау йәйге ваҡыты"}, {"Aqtobe", "Аҡтүбә йәйге ваҡыты"}, {"Arabian", "Сәғүд Ғәрәбстаны йәйге ваҡыты"}, {"Argentina", "Аргентина йәйге ваҡыты"}, {"Argentina_Western", "Көнбайыш Аргентина йәйге ваҡыты"}, {"Armenia", "Әрмәнстан йәйге ваҡыты"}, {"Atlantic", "Атлантик йәйге ваҡыты"}, {"Australia_Central", "Үҙәк Австралия йәйге ваҡыты"}, {"Australia_CentralWestern", "Үҙәк Австралия көнбайыш йәйге ваҡыты"}, {"Australia_Eastern", "Көнсығыш Австралия йәйге ваҡыты"}, {"Australia_Western", "Көнбайыш Австралия йәйге ваҡыты"}, {"Azerbaijan", "Әзербайжан йәйге ваҡыты"}, {"Azores", "Азор утрауҙары йәйге ваҡыты"}, {"Bangladesh", "Бангладеш йәйге ваҡыты"}, {"Brasilia", "Бразилия йәйге ваҡыты"}, {"Cape_Verde", "Кабо-Верде йәйге ваҡыты"}, {"Chatham", "Чатем йәйге ваҡыты"}, {"Chile", "Чили йәйге ваҡыты"}, {"China", "Ҡытай йәйге ваҡыты"}, {"Colombia", "Колумбия йәйге ваҡыты"}, {"Cook", "Кук утрауҙары йәйге ваҡыты"}, {"Cuba", "Куба йәйге ваҡыты"}, {"Easter", "Пасха утрауы йәйге ваҡыты"}, {"Europe_Central", "Үҙәк Европа йәйге ваҡыты"}, {"Europe_Eastern", "Көнсығыш Европа йәйге ваҡыты"}, {"Europe_Western", "Көнбайыш Европа йәйге ваҡыты"}, {"Falkland", "Фолкленд утрауҙары йәйге ваҡыты"}, {"Fiji", "Фиджи йәйге ваҡыты"}, {"Georgia", "Грузия йәйге ваҡыты"}, {"Greenland", "Гренландия йәйге ваҡыты"}, {"Greenland_Eastern", "Көнсығыш Гренландия йәйге ваҡыты"}, {"Greenland_Western", "Көнбайыш Гренландия йәйге ваҡыты"}, {"Hawaii_Aleutian", "Гавай-Алеут йәйге ваҡыты"}, {"Hong_Kong", "Гонконг йәйге ваҡыты"}, {"Hovd", "Ховд йәйге ваҡыты"}, {"Iran", "Иран йәйге ваҡыты"}, {"Irkutsk", "Иркутск йәйге ваҡыты"}, {"Israel", "Израиль йәйге ваҡыты"}, {"Japan", "Япония йәйге ваҡыты"}, {"Kamchatka", "Камчатка йәйге ваҡыты"}, {"Korea", "Корея йәйге ваҡыты"}, {"Krasnoyarsk", "Красноярск йәйге ваҡыты"}, {"Lord_Howe", "Лорд-Хау йәйге ваҡыты"}, {"Macau", "Макао йәйге ваҡыты"}, {"Magadan", "Магадан йәйге ваҡыты"}, {"Mauritius", "Маврикий йәйге ваҡыты"}, {"Mexico_Pacific", "Мексика Тымыҡ океан йәйге ваҡыты"}, {"Mongolia", "Улан-Батор йәйге ваҡыты"}, {"Moscow", "Мәскәү йәйге ваҡыты"}, {"New_Caledonia", "Яңы Каледония йәйге ваҡыты"}, {"New_Zealand", "Яңы Зеландия йәйге ваҡыты"}, {"Newfoundland", "Ньюфаундленд йәйге ваҡыты"}, {"Norfolk", "Норфолк йәйге ваҡыты"}, {"Noronha", "Фернанду-ди-Норонья йәйге ваҡыты"}, {"Novosibirsk", "Новосибирск йәйге ваҡыты"}, {"Omsk", "Омск йәйге ваҡыты"}, {"Pakistan", "Пакистан йәйге ваҡыты"}, {"Paraguay", "Парагвай йәйге ваҡыты"}, {"Peru", "Перу йәйге ваҡыты"}, {"Philippines", "Филиппин йәйге ваҡыты"}, {"Pierre_Miquelon", "Сен-Пьер һәм Микелон йәйге ваҡыты"}, {"Qyzylorda", "Ҡыҙылурҙа йәйге ваҡыты"}, {"Sakhalin", "Сахалин йәйге ваҡыты"}, {"Samara", "Һамар йәйге ваҡыты"}, {"Samoa", "Америка Самоаһы йәйге ваҡыты"}, {"Taipei", "Тайвань йәйге ваҡыты"}, {"Tonga", "Тонга йәйге ваҡыты"}, {"Turkmenistan", "Төркмәнстан йәйге ваҡыты"}, {"Uruguay", "Уругвай йәйге ваҡыты"}, {"Uzbekistan", "Үзбәкстан йәйге ваҡыты"}, {"Vanuatu", "Вануату йәйге ваҡыты"}, {"Vladivostok", "Владивосток йәйге ваҡыты"}, {"Volgograd", "Волгоград йәйге ваҡыты"}, {"Yakutsk", "Якутск йәйге ваҡыты"}, {"Yekaterinburg", "Екатеринбург йәйге ваҡыты"}},
	},
	{
		{{"Alaska", "اَلاسکائے گیشّتگێن ساهت"}, {"Amazon", "امازۆنئے گیشّتگێن ساهت"}, {"America_Central", "نیامی امریکائے گیشّتگێن ساهت"}, {"America_Eastern", "رۆدراتکی امریکائے گیشّتگێن ساهت"}, {"America_Mountain", "کۆهستگێن امریکائے گیشّتگێن ساهت"}, {"America_Pacific", "آرامزِری امریکائے گیشّتگێن ساهت"}, {"Argentina", "ارجنتینائے گیشّتگێن ساهت"}, {"Argentina_Western", "رۆنندی ارجنتینائے گیشّتگێن ساهت"}, {"Australia_Central", "نیامی اُسترالیائے گیشّتگێن ساهت"}, {"Australia_CentralWestern", "نیام\u200cرۆنندی اُسترالیائے گیشّتگێن ساهت"}, {"Australia_Eastern", "رۆدراتکی اُسترالیائے گیشّتگێن ساهت"}, {"Australia_Western", "رۆنندی اُسترالیائے گیشّتگێن ساهت"}, {"Brasilia", "برازیلئے گیشّتگێن ساهت"}, {"Europe_Central", "نیامی یورپئے گیشّتگێن ساهت"}, {"Europe_Eastern", "رۆدراتکی یورپئے گیشّتگێن ساهت"}, {"Europe_Further_Eastern", "دێمتری رۆدراتکی یورپئے گیشّتگێن ساهت"}, {"Europe_Western", "رۆنندی یورپئے گیشّتگێن ساهت"}, {"GMT", "گرین\u200cوِچ مین ٹائم"}, {"Hawaii", "هئواییئے گیشّتگێن ساهت"}, {"Hawaii_Aleutian", "هئواییئے گیشّتگێن ساهت"}, {"Indonesia_Central", "نیامی اندۆنیزیائے گیشّتگێن ساهت"}, {"Indonesia_Eastern", "رۆدراتکی اندۆنیزیائے گیشّتگێن ساهت"}, {"Indonesia_Western", "رۆنندی اندۆنیزیائے گیشّتگێن ساهت"}, {"Irkutsk", "ایرکوتسکئے گیشّتگێن ساهت"}, {"Kazakhstan_Eastern", "رۆدراتکی کازکستانئے گیشّتگێن ساهت"}, {"Kazakhstan_Western", "رۆنندی کازکستانئے گیشّتگێن ساهت"}, {"Krasnoyarsk", "کرانسنُیارسکئے گیشّتگێن ساهت"}, {"Lord_Howe", "لۆرڈ هئو اُسترالیائے گیشّتگێن ساهت"}, {"Magadan", "مَگَدَنئے گیشّتگێن ساهت"}, {"Mexico_Pacific", "آرامزِری مِکسیکۆئے گیشّتگێن ساهت"}, {"Moscow", "ماسکۆئے گیشّتگێن ساهت"}, {"Newfoundland", "نیوفاوونڈلئینڈئے گیشّتگێن ساهت"}, {"Noronha", "نُرُنهائے گیشّتگێن ساهت"}, {"Novosibirsk", "نۆوۆسیبیرسکئے گیشّتگێن ساهت"}, {"Omsk", "اۆمسکئے گیشّتگێن ساهت"}, {"Vladivostok", "ولادیوُستُکئے گیشّتگێن ساهت"}, {"Yakutsk", "یاکوتسکئے گیشّتگێن ساهت"}, {"Yekaterinburg", "یێکاترینبورگئے گیشّتگێن ساهت"}},
		{{"Alaska", "اَلاسکائے گرماگی ساهت"}, {"Amazon", "امازۆنئے گرماگی ساهت"}, {"America_Central", "نیامی امریکائے گرماگی ساهت"}, {"America_Eastern", "رۆدراتکی امریکائے گرماگی ساهت"}, {"America_Mountain", "کۆهستگێن امریکائے گرماگی ساهت"}, {"America_Pacific", "آرامزِری امریکائے گرماگی ساهت"}, {"Argentina", "ارجنتینائے گرماگی ساهت"}, {"Argentina_Western", "رۆنندی ارجنتینائے گرماگی ساهت"}, {"Australia_Central", "نیامی اُسترالیائے گرماگی ساهت"}, {"Australia_CentralWestern", "نیام\u200cرۆنندی اُسترالیائے گرماگی ساهت"}, {"Australia_Eastern", "رۆدراتکی اُسترالیائے گرماگی ساهت"}, {"Australia_Western", "رۆنندی اُسترالیائے گرماگی ساهت"}, {"Brasilia", "برازیلئے گرماگی ساهت"}, {"Europe_Central", "نیامی یورپئے گرماگی ساهت"}, {"Europe_Eastern", "رۆدراتکی یورپئے گرماگی ساهت"}, {"Europe_Western", "رۆنندی یورپئے گرماگی ساهت"}, {"Hawaii_Aleutian", "هئواییئے گرماگی ساهت"}, {"Irkutsk", "ایرکوتسکئے گرماگی ساهت"}, {"Krasnoyarsk", "کرانسنُیارسکئے گرماگی ساهت"}, {"Lord_Howe", "لۆرڈ هئو اُسترالیائے گرماگی ساهت"}, {"Magadan", "مَگَدَنئے گرماگی ساهت"}, {"Mexico_Pacific", "آرامزِری مِکسیکۆئے گرماگی ساهت"}, {"Moscow", "ماسکۆئے گرماگی ساهت"}, {"Newfoundland", "نیوفاوونڈلئینڈئے گرماگی ساهت"}, {"Noronha", "نُرُنهائے گرماگی ساهت"}, {"Novosibirsk", "نۆوۆسیبیرسکئے گرماگی ساهت"}, {"Omsk", "اۆمسکئے گرماگی ساهت"}, {"Vladivostok", "ولادیوُستُکئے گرماگی ساهت"}, {"Yakutsk", "یاکوتسکئے گرماگی ساهت"}, {"Yekaterinburg", "یێکاترینبورگئے گرماگی ساهت"}},
	},
	{
		{{"Afghanistan", "Awgánestánay wahd"}, {"Africa_Central", "Delgáhi Aprikáay wahd"}, {"Africa_Eastern", "Ródarátki Aprikáay wahd"}, {"Africa_Southern", "Ródarátki Aprikáay anjári wahd"}, {"Africa_Western", "Rónendi Aprikáay wahd"}, {"Alaska", "Aláskáay anjári wahd"}, {"Amazon", "Amázónay anjári wahd"}, {"America_Central", "Delgáhi Amrikáay anjári wahd"}, {"America_Eastern", "Ródarátki Amrikáay anjári wahd"}, {"America_Mountain", "Kóhestagi Amrikáay anjári wahd"}, {"America_Pacific", "Árámzeri Amrikáay anjári wahd"}, {"Apia", "Apiáay anjári wahd"}, {"Arabian", "Arabi anjári wahd"}, {"Argentina", "Arjentináay anjári wahd"}, {"Argentina_Western", "Rónendi Arjentináay anjári wahd"}, {"Armenia", "Árminiáay anjári wahd"}, {"Atlantic", "Atlantáay anjári wahd"}, {"Australia_Central", "Delgáhi Ástréliáay anjári wahd"}, {"Australia_CentralWestern", "Delgáhirónendi Ástréliáay anjári wahd"}, {"Australia_Eastern", "Ródarátki Ástréliáay anjári wahd"}, {"Australia_Western", "Rónendi Ástréliáay anjári wahd"}, {"Azerbaijan", "Ázerbáijánay anjári wahd"}, {"Azores", "Azóresay anjári wahd"}, {"Bangladesh", "Bangaladéshay anjári wahd"}, {"Bhutan", "Buthánay wahd"}, {"Bolivia", "Boliwiáay wahd"}, {"Brasilia", "Brázilay anjári wahd"}, {"Brunei", "Brunáiay wahd"}, {"Cape_Verde", "Kap Wardéay anjári wahd"}, {"Chamorro", "Chamorróay wahd"}, {"Chatham", "Chatam anjári wahd"}, {"Chile", "Chilayay anjári wahd"}, {"China", "Chinay anjári wahd"}, {"Christmas", "Kresmes Islánday wahd"}, {"Cocos", "Kukus Islánday wahd"}, {"Colombia", "Kolambiáay anjári wahd"}, {"Cook", "Kuk Islánday anjári wahd"}, {"Cuba", "Kyubáay anjári wahd"}, {"Davis", "Dawisay wahd"}, {"DumontDUrville", "Dumont Urwilay wahd"}, {"East_Timor", "Ródarátki Timuray wahd"}, {"Easter", "Isthar Islánday anjári wahd"}, {"Ecuador", "Ekwádóray wahd"}, {"Europe_Central", "Delgáhi Yuropay anjári wahd"}, {"Europe_Eastern", "Ródarátki Yuropay anjári wahd"}, {"Europe_Further_Eastern", "Démterén Ródarátki Yuropay anjári wahd"}, {"Europe_Western", "Rónendi Yuropay anjári wahd"}, {"Falkland", "Palklánd Islánday anjári wahd"}, {"Fiji", "Fijiay anjári wahd"}, {"French_Guiana", "Paránsi Gwináay wahd"}, {"French_Southern", "Zerbári Paransi o Antárktikáay wahd"}, {"GMT", "Grinwech Min Wahd"}, {"Galapagos", "Galapagosay wahd"}, {"Gambier", "Gambiray wahd"}, {"Georgia", "Járjiáay anjári wahd"}, {"Gilbert_Islands", "Gelbart Islánday wahd"}, {"Greenland_Eastern", "Ródarátki Grinlánday anjári wahd"}, {"Greenland_Western", "Rónendi Grinlánday anjári wahd"}, {"Gulf", "Khalijay anjári wahd"}, {"Guyana", "Guyánáay wahd"}, {"Hawaii", "Hawái/Alushiay anjári wahd"}, {"Hawaii_Aleutian", "Hawái/Alushiay anjári wahd"}, {"Hong_Kong", "Háng Kángay anjári wahd"}, {"Hovd", "Hówday anjári wahd"}, {"India", "Henday anjári wahd"}, {"Indian_Ocean", "Hendi zeray wahd"}, {"Indochina", "Hendóchinay wahd"}, {"Indonesia_Central", "Delgáhi Endhonishiáay anjári wahd"}, {"Indonesia_Eastern", "Ródarátki Endhonishiáay anjári wahd"}, {"Indonesia_Western", "Rónendi Endhonishiáay anjári wahd"}, {"Iran", "Éránay anjári wahd"}, {"Irkutsk", "Erkuskay anjári wahd"}, {"Israel", "Esráilay anjári wahd"}, {"Japan", "Jápánay anjári wahd"}, {"Kazakhstan", "Kázakestánay wahd"}, {"Kazakhstan_Eastern", "Ródarátki Kázekestánay anjári wahd"}, {"Kazakhstan_Western", "Rónendi Kázekestánay anjári wahd"}, {"Korea", "Kóriáay anjári wahd"}, {"Kosrae", "Kósraiay wahd"}, {"Krasnoyarsk", "Krasnóyáskay anjári wahd"}, {"Kyrgystan", "Kargezestánay wahd"}, {"Line_Islands", "Liné Islánday wahd"}, {"Lord_Howe", "Ástréliáay, Ládhaway anjári wahd"}, {"Magadan", "Mágadánay anjári wahd"}, {"Malaysia", "Malishiáay wahd"}, {"Maldives", "Máldipay wahd"}, {"Marquesas", "Markésásay wahd"}, {"Marshall_Islands", "Márshal Islánday wahd"}, {"Mauritius", "Muritániáay anjári wahd"}, {"Mawson", "Mawsonay wahd"}, {"Mexico_Pacific", "Árámzeri Meksikóay anjári wahd"}, {"Mongolia", "Ulánbátaray anjári wahd"}, {"Moscow", "Máskóay anjári wahd"}, {"Myanmar", "Myanmáray wahd"}, {"Nauru", "Nauruay wahd"}, {"Nepal", "Népálay wahd"}, {"New_Caledonia", "Nyu Kaledóniáay anjári wahd"}, {"New_Zealand", "Niu Zilánday anjári wahd"}, {"Newfoundland", "Nipándlaynday anjári wahd"}, {"Niue", "Niuay wahd"}, {"Norfolk", "Nurpolk Islánday anjári wahd"}, {"Noronha", "Noronáay anjári wahd"}, {"Novosibirsk", "Nawásibiskay anjári wahd"}, {"Omsk", "Ómskay anjári wahd"}, {"Pakistan", "Pákestánay anjári wahd"}, {"Palau", "Paláuay wahd"}, {"Papua_New_Guinea", "Pápuá Niu Giniáay wahd"}, {"Paraguay", "Paragóayay anjári wahd"}, {"Peru", "Péróay anjári wahd"}, {"Philippines", "Pelpinay anjári wahd"}, {"Phoenix_Islands", "Phoeneks Islánday wahd"}, {"Pierre_Miquelon", "St. Péri o Mikwélin ajári wahd"}, {"Pitcairn", "Pitkarénay wahd"}, {"Ponape", "Pónpiay wahd"}, {"Pyongyang", "Pyongyángay wahd"}, {"Reunion", "Réyunianay wahd"}, {"Rothera", "Rothéráay wahd"}, {"Sakhalin", "Sakhálinay anjári wahd"}, {"Samoa", "Samóáway anjári wahd"}, {"Seychelles", "Séchelesay wahd"}, {"Singapore", "Sengápuray anjári wahd"}, {"Solomon", "Solomán Islánday wahd"}, {"South_Georgia", "Zerbári Járjiáay wahd"}, {"Suriname", "Surinaymay wahd"}, {"Syowa", "Syówáay wahd"}, {"Tahiti", "Tahitiay wahd"}, {"Taipei", "Táipiay anjári wahd"}, {"Tajikistan", "Tájekestánay wahd"}, {"Tokelau", "Tokeláuay wahd"}, {"Tonga", "Tongáay anjári wahd"}, {"Truk", "Chukay wahd"}, {"Turkmenistan", "Torkmenestánay anjári wahd"}, {"Tuvalu", "Tuwáluay wahd"}, {"Uruguay", "Yurógóayay anjári wahd"}, {"Uzbekistan", "Ozbekestánay anjári wahd"}, {"Vanuatu", "Wánuátuay anjári wahd"}, {"Venezuela", "Wenezwéláay wahd"}, {"Vladivostok", "Waládiwástókay anjári wahd"}, {"Volgograd", "Wolgograday anjári wahd"}, {"Vostok", "Wostokay wahd"}, {"Wake", "Wayk Islánday wahd"}, {"Wallis", "Wallis o Futunáay wahd"}, {"Yakutsk", "Yákuskay anjári wahd"}, {"Yekaterinburg", "Yakátrinborgay anjári wahd"}, {"Yukon", "Yukón wahd"}},
		{{"Alaska", "Aláskáay garmági wahd"}, {"Amazon", "Amázónay garmági wahd"}, {"America_Central", "Delgáhi Amrikáay garmági wahd"}, {"America_Eastern", "Ródarátki Amrikáay garmági wahd"}, {"America_Mountain", "Kóhestagi Amrikáay garmági wahd"}, {"America_Pacific", "Árámzeri Amrikáay garmági wahd"}, {"Apia", "Apiáay róchi wahd"}, {"Arabian", "Arabi róchi wahd"}, {"Argentina", "Arjentináay garmági wahd"}, {"Argentina_Western", "Rónendi Arjentináay gramági wahd"}, {"Armenia", "Árminiáay garmági wahd"}, {"Atlantic", "Atlantáay róchi wahd"}, {"Australia_Central", "Delgáhi Ástréliáay garmági wahd"}, {"Australia_CentralWestern", "Delgáhirónendi Ástréliáay garmági wahd"}, {"Australia_Eastern", "Ródarátki Ástréliáay garmági wahd"}, {"Australia_Western", "Rónendi Ástréliáay garmági wahd"}, {"Azerbaijan", "Ázerbáijánay garmági wahd"}, {"Azores", "Azóresay garmági wahd"}, {"Bangladesh", "Bangaladéshay garmági wahd"}, {"Brasilia", "Brázilay garmági wahd"}, {"Cape_Verde", "Kap Wardéay garmági wahd"}, {"Chatham", "Chatam róchi wahd"}, {"Chile", "Chilayay garmági wahd"}, {"China", "Chinay róchi wahd"}, {"Colombia", "Kolambiáay garmági wahd"}, {"Cook", "Kuk Islánday ném-garmági wahd"}, {"Cuba", "Kyubáay róchay wahd"}, {"Easter", "Isthar Islánday garmági wahd"}, {"Europe_Central", "Delgáhi Yuropay garmági wahd"}, {"Europe_Eastern", "Ródarátki Yuropay garmági wahd"}, {"Europe_Western", "Rónendi Yuropay garmági wahd"}, {"Falkland", "Palklánd Islánday garmági wahd"}, {"Fiji", "Fijiay garmági wahd"}, {"Georgia", "Járjiáay garmági wahd"}, {"Greenland_Eastern", "Ródarátki Grinlánday garmági wahd"}, {"Greenland_Western", "Rónendi Grinlánd Garmági Wahd"}, {"Hawaii_Aleutian", "Hawái/Alushiay garmági wahd"}, {"Hong_Kong", "Háng Kángay garmági wahd"}, {"Hovd", "Hówday garmági wahd"}, {"Iran", "Éránay róchi wahd"}, {"Irkutsk", "Erkuskay garmági wahd"}, {"Israel", "Esráilay róchi wahd"}, {"Japan", "Jápánay róchi wahd"}, {"Korea", "Kóriáay róchi wahd"}, {"Krasnoyarsk", "Krasnóyáskay garmági wahd"}, {"Lord_Howe", "Ástréliáay, Ládhaway garmági wahd"}, {"Magadan", "Mágadánay garmági wahd"}, {"Mauritius", "Muritániáay garmági wahd"}, {"Mexico_Pacific", "Árámzeri Meksikóay garmági wahd"}, {"Mongolia", "Ulánbátaray garmági wahd"}, {"Moscow", "Máskóay garmági wahd"}, {"New_Caledonia", "Nyu Kaledóniáay garmági wahd"}, {"New_Zealand", "Niu Zilánday róchi wahd"}, {"Newfoundland", "Nipándlaynday garmági wahd"}, {"Norfolk", "Nurpolk Islánday róchi wahd"}, {"Noronha", "Noronáay garmági wahd"}, {"Novosibirsk", "Nawásibiskay garmági wahd"}, {"Omsk", "Ómskay garmági wahd"}, {"Pakistan", "Pákestánay garmági wahd"}, {"Paraguay", "Paragóayay garmági wahd"}, {"Peru", "Péróay garmági wahd"}, {"Philippines", "Pelpinay garmági wahd"}, {"Pierre_Miquelon", "St. Péri o Mikwélin róchi wahd"}, {"Sakhalin", "Sakhálinay garmági wahd"}, {"Samoa", "Samóáway róchi wahd"}, {"Taipei", "Táipiay róchi wahd"}, {"Tonga", "Tongáay garmági wahd"}, {"Turkmenistan", "Torkmenestánay garmági wahd"}, {"Uruguay", "Yurógóayay garmági wahd"}, {"Uzbekistan", "Ozbekestánay garmági wahd"}, {"Vanuatu", "Wánuátuay garmági wahd"}, {"Vladivostok", "Waládiwástókay garmági wahd"}, {"Volgograd", "Wolgograday garmági wahd"}, {"Yakutsk", "Yákuskay garmági wahd"}, {"Yekaterinburg", "Yakátrinborgay garmági wahd"}},
	},
	{
		{{"Afghanistan", "Афганістанскі час"}, {"Africa_Central", "Цэнтральнаафрыканскі час"}, {"Africa_Eastern", "Усходнеафрыканскі час"}, {"Africa_Southern", "Паўднёваафрыканскі час"}, {"Africa_Western", "Заходнеафрыканскі час"}, {"Alaska", "Стандартны час Аляскі"}, {"Amazon", "Амазонскі стандартны час"}, {"America_Central", "Паўночнаамерыканскі цэнтральны стандартны час"}, {"America_Eastern", "Паўночнаамерыканскі ўсходні стандартны час"}, {"America_Mountain", "Паўночнаамерыканскі горны стандартны час"}, {"America_Pacific", "Ціхаакіянскі стандартны час"}, {"Apia", "Стандартны час Апіі"}, {"Arabian", "Стандартны час Саудаўскай Аравіі"}, {"Argentina", "Аргенцінскі стандартны час"}, {"Argentina_Western", "Стандартны час Заходняй Аргенціны"}, {"Armenia", "Стандартны час Арменіі"}, {"Atlantic", "Атлантычны стандартны час"}, {"Australia_Central", "Стандартны час цэнтральнай Аўстраліі"}, {"Australia_CentralWestern", "Стандартны цэнтральна-заходні час Аўстраліі"}, {"Australia_Eastern", "Стандартны час усходняй Аўстраліі"}, {"Australia_Western", "Стандартны час заходняй Аўстраліі"}, {"Azerbaijan", "Стандартны час Азербайджана"}, {"Azores", "Стандартны час Азорскіх астравоў"}, {"Bangladesh", "Стандартны час Бангладэш"}, {"Bhutan", "Час Бутана"}, {"Bolivia", "Балівійскі час"}, {"Brasilia", "Бразільскі стандартны час"}, {"Brunei", "Час Брунея"}, {"Cape_Verde", "Стандартны час Каба-Вердэ"}, {"Chamorro", "Час Чамора"}, {"Chatham", "Стандартны час Чатэма"}, {"Chile", "Чылійскі стандартны час"}, {"China", "Стандартны час Кітая"}, {"Christmas", "Час вострава Каляд"}, {"Cocos", "Час Какосавых астравоў"}, {"Colombia", "Калумбійскі стандартны час"}, {"Cook", "Стандартны час астравоў Кука"}, {"Cuba", "Стандартны час Кубы"}, {"Davis", "Час станцыі Дэйвіс"}, {"DumontDUrville", "Час станцыі Дзюмон-Дзюрвіль"}, {"East_Timor", "Час Усходняга Тымора"}, {"Easter", "Стандартны час вострава Вялікадня"}, {"Ecuador", "Эквадорскі час"}, {"Europe_Central", "Цэнтральнаеўрапейскі стандартны час"}, {"Europe_Eastern", "Усходнееўрапейскі стандартны час"}, {"Europe_Further_Eastern", "Далёкаўсходнееўрапейскі час"}, {"Europe_Western", "Заходнееўрапейскі стандартны час"}, {"Falkland", "Стандартны час Фалклендскіх астравоў"}, {"Fiji", "Стандартны час Фіджы"}, {"French_Guiana", "Час Французскай Гвіяны"}, {"French_Southern", "Час Французскай паўднёва-антарктычнай тэрыторыі"}, {"GMT", "Час па Грынвічы"}, {"Galapagos", "Стандартны час Галапагоскіх астравоў"}, {"Gambier", "Час астравоў Гамб’е"}, {"Georgia", "Грузінскі стандартны час"}, {"Gilbert_Islands", "Час астравоў Гілберта"}, {"Greenland_Eastern", "Стандартны час Усходняй Грэнландыі"}, {"Greenland_Western", "Стандартны час Заходняй Грэнландыі"}, {"Gulf", "Час Персідскага заліва"}, {"Guyana", "Час Гаяны"}, {"Hawaii", "Гавайска-Алеуцкі стандартны час"}, {"Hawaii_Aleutian", "Гавайска-Алеуцкі стандартны час"}, {"Hong_Kong", "Стандартны час Ганконга"}, {"Hovd", "Стандартны час Хоўда"}, {"India", "Час Індыі"}, {"Indian_Ocean", "Час Індыйскага акіяна"}, {"Indochina", "Індакітайскі час"}, {"Indonesia_Central", "Цэнтральнаінданезійскі час"}, {"Indonesia_Eastern", "Усходнеінданезійскі час"}, {"Indonesia_Western", "Заходнеінданезійскі час"}, {"Iran", "Іранскі стандартны час"}, {"Irkutsk", "Іркуцкі стандартны час"}, {"Israel", "Ізраільскі стандартны час"}, {"Japan", "Стандартны час Японіі"}, {"Kazakhstan", "Казахстанскі час"}, {"Kazakhstan_Eastern", "Усходнеказахстанскі час"}, {"Kazakhstan_Western", "Заходнеказахстанскі час"}, {"Korea", "Стандартны час Карэі"}, {"Kosrae", "Час астравоў Кусаіе"}, {"Krasnoyarsk", "Краснаярскі стандартны час"}, {"Kyrgystan", "Час Кыргызстана"}, {"Line_Islands", "Час астравоў Лайн"}, {"Lord_Howe", "Стандартны час Лорд-Хау"}, {"Magadan", "Магаданскі стандартны час"}, {"Malaysia", "Час Малайзіі"}, {"Maldives", "Час Мальдываў"}, {"Marquesas", "Час Маркізскіх астравоў"}, {"Marshall_Islands", "Час Маршалавых астравоў"}, {"Mauritius", "Стандартны час Маўрыкія"}, {"Mawson", "Час станцыі Моўсан"}, {"Mexico_Pacific", "Мексіканскі ціхаакіянскі стандатны час"}, {"Mongolia", "Стандартны час Улан-Батара"}, {"Moscow", "Маскоўскі стандартны час"}, {"Myanmar", "Час М’янмы"}, {"Nauru", "Час Науру"}, {"Nepal", "Непальскі час"}, {"New_Caledonia", "Стандартны час Новай Каледоніі"}, {"New_Zealand", "Стандартны час Новай Зеландыі"}, {"Newfoundland", "Ньюфаўндлендскі стандартны час"}, {"Niue", "Час Ніуэ"}, {"Norfolk", "Стандартны час вострава Норфалк"}, {"Noronha", "Стандартны час Фернанду-дзі-Наронья"}, {"Novosibirsk", "Новасібірскі стандартны час"}, {"Omsk", "Омскі стандартны час"}, {"Pakistan", "Пакістанскі стандартны час"}, {"Palau", "Час Палау"}, {"Papua_New_Guinea", "Час Папуа-Новай Гвінеі"}, {"Paraguay", "Стандартны час Парагвая"}, {"Peru", "Перуанскі стандартны час"}, {"Philippines", "Філіпінскі стандартны час"}, {"Phoenix_Islands", "Час астравоў Фенікс"}, {"Pierre_Miquelon", "Стандартны час Сен-П’ер і Мікелон"}, {"Pitcairn", "Час вострава Піткэрн"}, {"Ponape", "Час вострава Панпеі"}, {"Pyongyang", "Пхеньянскі час"}, {"Reunion", "Час Рэюньёна"}, {"Rothera", "Час станцыі Ротэра"}, {"Sakhalin", "Сахалінскі стандартны час"}, {"Samoa", "Стандартны час Самоа"}, {"Seychelles", "Час Сейшэльскіх астравоў"}, {"Singapore", "Сінгапурскі час"}, {"Solomon", "Час Саламонавых астравоў"}, {"South_Georgia", "Час Паўднёвай Георгіі"}, {"Suriname", "Час Сурынама"}, {"Syowa", "Час станцыі Сёва"}, {"Tahiti", "Час Таіці"}, {"Taipei", "Стандартны час Тайбэя"}, {"Tajikistan", "Час Таджыкістана"}, {"Tokelau", "Час Такелау"}, {"Tonga", "Стандартны час Тонга"}, {"Truk", "Час Трука"}, {"Turkmenistan", "Стандартны час Туркменістана"}, {"Tuvalu", "Час Тувалу"}, {"Uruguay", "Уругвайскі стандартны час"}, {"Uzbekistan", "Стандартны час Узбекістана"}, {"Vanuatu", "Стандартны час Вануату"}, {"Venezuela", "Венесуэльскі час"}, {"Vladivostok", "Уладзівастоцкі стандартны час"}, {"Volgograd", "Валгаградскі стандартны час"}, {"Vostok", "Час станцыі Васток"}, {"Wake", "Час вострава Уэйк"}, {"Wallis", "Час астравоў Уоліс і Футуна"}, {"Yakutsk", "Якуцкі стандартны час"}, {"Yekaterinburg", "Екацярынбургскі стандартны час"}, {"Yukon", "Час Юкана"}},
		{{"Alaska", "Летні час Аляскі"}, {"Amazon", "Амазонскі летні час"}, {"America_Central", "Паўночнаамерыканскі цэнтральны летні час"}, {"America_Eastern", "Паўночнаамерыканскі ўсходні летні час"}, {"America_Mountain", "Паўночнаамерыканскі горны летні час"}, {"America_Pacific", "Ціхаакіянскі летні час"}, {"Apia", "Летні час Апіі"}, {"Arabian", "Летні час Саудаўскай Аравіі"}, {"Argentina", "Аргенцінскі летні час"}, {"Argentina_Western", "Летні час Заходняй Аргенціны"}, {"Armenia", "Летні час Арменіі"}, {"Atlantic", "Атлантычны летні час"}, {"Australia_Central", "Летні час цэнтральнай Аўстраліі"}, {"Australia_CentralWestern", "Летні цэнтральна-заходні час Аўстраліі"}, {"Australia_Eastern", "Летні час усходняй Аўстраліі"}, {"Australia_Western", "Летні час заходняй Аўстраліі"}, {"Azerbaijan", "Летні час Азербайджана"}, {"Azores", "Летні час Азорскіх астравоў"}, {"Bangladesh", "Летні час Бангладэш"}, {"Brasilia", "Бразільскі летні час"}, {"Cape_Verde", "Летні час Каба-Вердэ"}, {"Chatham", "Летні час Чатэма"}, {"Chile", "Чылійскі летні час"}, {"China", "Летні час Кітая"}, {"Colombia", "Калумбійскі летні час"}, {"Cook", "Паўлетні час астравоў Кука"}, {"Cuba", "Летні час Кубы"}, {"Easter", "Летні час вострава Вялікадня"}, {"Europe_Central", "Цэнтральнаеўрапейскі летні час"}, {"Europe_Eastern", "Усходнееўрапейскі летні час"}, {"Europe_Western", "Заходнееўрапейскі летні час"}, {"Falkland", "Летні час Фалклендскіх астравоў"}, {"Fiji", "Летні час Фіджы"}, {"Georgia", "Грузінскі летні час"}, {"Greenland_Eastern", "Летні час Усходняй Грэнландыі"}, {"Greenland_Western", "Летні час Заходняй Грэнландыі"}, {"Hawaii_Aleutian", "Гавайска-Алеуцкі летні час"}, {"Hong_Kong", "Летні час Ганконга"}, {"Hovd", "Летні час Хоўда"}, {"Iran", "Іранскі летні час"}, {"Irkutsk", "Іркуцкі летні час"}, {"Israel", "Ізраільскі летні час"}, {"Japan", "Летні час Японіі"}, {"Korea", "Летні час Карэі"}, {"Krasnoyarsk", "Краснаярскі летні час"}, {"Lord_Howe", "Летні час Лорд-Хау"}, {"Magadan", "Магаданскі летні час"}, {"Mauritius", "Летні час Маўрыкія"}, {"Mexico_Pacific", "Мексіканскі ціхаакіянскі летні час"}, {"Mongolia", "Летні час Улан-Батара"}, {"Moscow", "Маскоўскі летні час"}, {"New_Caledonia", "Летні час Новай Каледоніі"}, {"New_Zealand", "Летні час Новай Зеландыі"}, {"Newfoundland", "Ньюфаўндлендскі летні час"}, {"Norfolk", "Летні час вострава Норфалк"}, {"Noronha", "Летні час Фернанду-дзі-Наронья"}, {"Novosibirsk", "Новасібірскі летні час"}, {"Omsk", "Омскі летні час"}, {"Pakistan", "Пакістанскі летні час"}, {"Paraguay", "Летні час Парагвая"}, {"Peru", "Перуанскі летні час"}, {"Philippines", "Філіпінскі летні час"}, {"Pierre_Miquelon", "Стандартны летні час Сен-П’ер і Мікелон"}, {"Sakhalin", "Сахалінскі летні час"}, {"Samoa", "Летні час Самоа"}, {"Taipei", "Летні час Тайбэя"}, {"Tonga", "Летні час Тонга"}, {"Turkmenistan", "Летні час Туркменістана"}, {"Uruguay", "Уругвайскі летні час"}, {"Uzbekistan", "Летні час Узбекістана"}, {"Vanuatu", "Летні час Вануату"}, {"Vladivostok", "Уладзівастоцкі летні час"}, {"Volgograd", "Валгаградскі летні час"}, {"Yakutsk", "Якуцкі летні час"}, {"Yekaterinburg", "Екацярынбургскі летні час"}},
	},
	{
		{{"Afghanistan", "Аўганістанскі час"}, {"Africa_Central", "Цэнтральнаафрыканскі час"}, {"Africa_Eastern", "Усходнеафрыканскі час"}, {"Africa_Southern", "Паўднёваафрыканскі час"}, {"Africa_Western", "Заходнеафрыканскі час"}, {"Alaska", "Змоўчны час Аляскі"}, {"Amazon", "Амазонскі змоўчны час"}, {"America_Central", "Паўночнаамэрыканскі цэнтральны змоўчны час"}, {"America_Eastern", "Паўночнаамэрыканскі ўсходні змоўчны час"}, {"America_Mountain", "Паўночнаамэрыканскі горны змоўчны час"}, {"America_Pacific", "Ціхаакіянскі змоўчны час"}, {"Apia", "Змоўчны час Апіі"}, {"Arabian", "Змоўчны час Саудаўскай Арабіі"}, {"Argentina", "Аргентынскі змоўчны час"}, {"Argentina_Western", "Змоўчны час Заходняй Аргэнтыны"}, {"Armenia", "Змоўчны час Армэніі"}, {"Atlantic", "Атлянтычны змоўчны час"}, {"Australia_Central", "Змоўчны час цэнтральнай Аўстраліі"}, {"Australia_CentralWestern", "Змоўчны цэнтральна-заходні час Аўстраліі"}, {"Australia_Eastern", "Змоўчны час усходняй Аўстраліі"}, {"Australia_Western", "змоўчны час заходняй Аўстраліі"}, {"Azerbaijan", "Змоўчны час Азэрбайджану"}, {"Azores", "Змоўчны час Азорскіх астравоў"}, {"Bangladesh", "Змоўчны час Банглядэшу"}, {"Bhutan", "Час Бутану"}, {"Bolivia", "Балівійскі час"}, {"Brasilia", "Бразыльскі змоўчны час"}, {"Brunei", "Час Брунэю"}, {"Cape_Verde", "Змоўчны час Каба-Вэрдэ"}, {"Chamorro", "Час Чамора"}, {"Chatham", "Змоўчны час Чатэму"}, {"Chile", "Чылійскі змоўчны час"}, {"China", "Змоўчны час Кітаю"}, {"Christmas", "Час вострава Раства"}, {"Cocos", "Час Какосавых астравоў"}, {"Colombia", "Калюмбійскі змоўчны час"}, {"Cook", "Змоўчны час астравоў Кука"}, {"Cuba", "Змоўчны час Кубы"}, {"Davis", "Час станцыі Дэйвіс"}, {"DumontDUrville", "Час станцыі Дзюмон-Дзюрвіль"}, {"East_Timor", "Час Усходняга Тымору"}, {"Easter", "Змоўчны час вострава Раства"}, {"Ecuador", "Эквадорскі час"}, {"Europe_Central", "Цэнтральнаэўрапейскі змоўчны час"}, {"Europe_Eastern", "Усходнеэўрапейскі змоўчны час"}, {"Europe_Further_Eastern", "Далёкаўсходнеэўрапейскі час"}, {"Europe_Western", "Заходнеэўрапейскі змоўчны час"}, {"Falkland", "Змоўчны час Фолклэндзкіх астравоў"}, {"Fiji", "Змоўчны час Фіджы"}, {"French_Guiana", "Час Францускай Гвіяны"}, {"French_Southern", "Час Францускай паўднёва-антарктычнай тэрыторыі"}, {"GMT", "Час па Грынвічы"}, {"Galapagos", "Змоўчны час Галяпагоскіх астравоў"}, {"Gambier", "Час астравоў Гамб’е"}, {"Georgia", "Грузінскі змоўчны час"}, {"Gilbert_Islands", "Час астравоў Гілбэрта"}, {"Greenland_Eastern", "Змоўчны час Усходняй Грэнляндыі"}, {"Greenland_Western", "Змоўчны час Заходняй Грэнляндыі"}, {"Gulf", "Час Пэрсыдзкага заліву"}, {"Guyana", "Час Гаяны"}, {"Hawaii", "Гавайска-Алэвуцкі змоўчны час"}, {"Hawaii_Aleutian", "Гавайска-Алэвуцкі змоўчны час"}, {"Hong_Kong", "Змоўчны час Ганконга"}, {"Hovd", "Змоўчны час Хоўда"}, {"India", "Час Індыі"}, {"Indian_Ocean", "Час Індыйскага акіяну"}, {"Indochina", "Індакітайскі час"}, {"Indonesia_Central", "Цэнтральнаінданэзійскі час"}, {"Indonesia_Eastern", "Усходнеінданэзійскі час"}, {"Indonesia_Western", "Заходнеінданэзійскі час"}, {"Iran", "Іранскі змоўчны час"}, {"Irkutsk", "Іркуцкі змоўчны час"}, {"Israel", "Ізраільскі змоўчны час"}, {"Japan", "Змоўчны час Японіі"}, {"Kazakhstan", "Казахстанскі час"}, {"Kazakhstan_Eastern", "Усходнеказахстанскі час"}, {"Kazakhstan_Western", "Заходнеказахстанскі час"}, {"Korea", "Змоўчны час Карэі"}, {"Kosrae", "Час астравоў Касраэ"}, {"Krasnoyarsk", "Краснаярскі змоўчны час"}, {"Kyrgystan", "Час Кыргыстану"}, {"Line_Islands", "Час астравоў Лайн"}, {"Lord_Howe", "Змоўчны час Лорд-Гаў"}, {"Magadan", "Магаданскі змоўчны час"}, {"Malaysia", "Час Малайзіі"}, {"Maldives", "Час Мальдываў"}, {"Marquesas", "Час Маркіскіх астравоў"}, {"Marshall_Islands", "Час Маршалавых астравоў"}, {"Mauritius", "Змоўчны час Маўрыцыю"}, {"Mawson", "Час станцыі Мосан"}, {"Mexico_Pacific", "Мэксыканскі ціхаакіянскі стандатны час"}, {"Mongolia", "Змоўчны час Улан-Батару"}, {"Moscow", "Маскоўскі змоўчны час"}, {"Myanmar", "Час М’янмы"}, {"Nauru", "Час Наўру"}, {"Nepal", "Нэпальскі час"}, {"New_Caledonia", "Змоўчны час Новай Каледоніі"}, {"New_Zealand", "Змоўчны час Новай Зэляндыі"}, {"Newfoundland", "Ньюфаўндлэндзкі змоўчны час"}, {"Niue", "Час Ніуэ"}, {"Norfolk", "Змоўчны час вострава Норфалк"}, {"Noronha", "Змоўчны час Фэрнанду-ды-Наронья"}, {"Novosibirsk", "Новасыбірскі змоўчны час"}, {"Omsk", "Омскі змоўчны час"}, {"Pakistan", "Пакістанскі змоўчны час"}, {"Palau", "Час Палаў"}, {"Papua_New_Guinea", "Час Папуа-Новай Гвінэі"}, {"Paraguay", "Змоўчны час Парагваю"}, {"Peru", "Пэруанскі змоўчны час"}, {"Philippines", "Філіпінскі змоўчны час"}, {"Phoenix_Islands", "Час астравоў Фінікс"}, {"Pierre_Miquelon", "Змоўчны час Сэн-П’еру і Мікелёну"}, {"Pitcairn", "Час вострава Піткэрн"}, {"Ponape", "Час вострава Панпэі"}, {"Pyongyang", "Пхэньянскі час"}, {"Reunion", "Час Рэюньёна"}, {"Rothera", "Час станцыі Ротэра"}, {"Sakhalin", "Сахалінскі змоўчны час"}, {"Samoa", "Змоўчны час Самоа"}, {"Seychelles", "Час Сэйшэлаў"}, {"Singapore", "Сынгапурскі час"}, {"Solomon", "Час Саламонавых астравоў"}, {"South_Georgia", "Час Паўднёвай Георгіі"}, {"Suriname", "Час Сурынаму"}, {"Syowa", "Час станцыі Сёва"}, {"Tahiti", "Час Таіці"}, {"Taipei", "Змоўчны час Тайбэя"}, {"Tajikistan", "Час Таджыкістану"}, {"Tokelau", "Час Такелаў"}, {"Tonga", "Змоўчны час Тонга"}, {"Truk", "Час Трука"}, {"Turkmenistan", "Змоўчны час Туркмэністана"}, {"Tuvalu", "Час Тувалу"}, {"Uruguay", "Уругвайскі змоўчны час"}, {"Uzbekistan", "Змоўчны час Узбэкістану"}, {"Vanuatu", "Змоўчны час Вануату"}, {"Venezuela", "Вэнэсуэльскі час"}, {"Vladivostok", "Уладзівастоцкі змоўчны час"}, {"Volgograd", "Валгаградзкі змоўчны час"}, {"Vostok", "Час станцыі Ўсход"}, {"Wake", "Час вострава Ўэйк"}, {"Wallis", "Час астравоў Ўоліс і Футуна"}, {"Yakutsk", "Якуцкі змоўчны час"}, {"Yekaterinburg", "Екацярынбурскі змоўчны час"}, {"Yukon", "Час Юкана"}},
		{{"Alaska", "Летні час Аляскі"}, {"Amazon", "Амазонскі летні час"}, {"America_Central", "Паўночнаамэрыканскі цэнтральны летні час"}, {"America_Eastern", "Паўночнаамэрыканскі ўсходні летні час"}, {"America_Mountain", "Паўночнаамэрыканскі горны летні час"}, {"America_Pacific", "Ціхаакіянскі летні час"}, {"Apia", "Летні час Апіі"}, {"Arabian", "Летні час Саудаўскай Арабіі"}, {"Argentina", "Аргентынскі летні час"}, {"Argentina_Western", "Летні час Заходняй Аргэнтыны"}, {"Armenia", "Летні час Армэніі"}, {"Atlantic", "Атлянтычны летні час"}, {"Australia_Central", "Летні час цэнтральнай Аўстраліі"}, {"Australia_CentralWestern", "Летні цэнтральна-заходні час Аўстраліі"}, {"Australia_Eastern", "Летні час усходняй Аўстраліі"}, {"Australia_Western", "Летні час заходняй Аўстраліі"}, {"Azerbaijan", "Летні час Азэрбайджану"}, {"Azores", "Летні час Азорскіх астравоў"}, {"Bangladesh", "Летні час Банглядэшу"}, {"Brasilia", "Бразыльскі летні час"}, {"Cape_Verde", "Летні час Каба-Вэрдэ"}, {"Chatham", "Летні час Чатэму"}, {"Chile", "Чылійскі летні час"}, {"China", "Летні час Кітаю"}, {"Colombia", "Калюмбійскі летні час"}, {"Cook", "Паўлетні час астравоў Кука"}, {"Cuba", "Летні час Кубы"}, {"Easter", "Летні час вострава Раства"}, {"Europe_Central", "Цэнтральнаэўрапейскі летні час"}, {"Europe_Eastern", "Усходнеэўрапейскі летні час"}, {"Europe_Western", "Заходнеэўрапейскі летні час"}, {"Falkland", "Летні час Фолклэндзкіх астравоў"}, {"Fiji", "Летні час Фіджы"}, {"Georgia", "Грузінскі летні час"}, {"Greenland_Eastern", "Летні час Усходняй Грэнляндыі"}, {"Greenland_Western", "Летні час Заходняй Грэнляндыі"}, {"Hawaii_Aleutian", "Гавайска-Алевуцкі летні час"}, {"Hong_Kong", "Летні час Ганконгу"}, {"Hovd", "Летні час Хоўда"}, {"Iran", "Іранскі летні час"}, {"Irkutsk", "Іркуцкі летні час"}, {"Israel", "Ізраільскі летні час"}, {"Japan", "Летні час Японіі"}, {"Korea", "Летні час Карэі"}, {"Krasnoyarsk", "Краснаярскі летні час"}, {"Lord_Howe", "Летні час Лорд-Гаў"}, {"Magadan", "Магаданскі летні час"}, {"Mauritius", "Летні час Маўрыцыю"}, {"Mexico_Pacific", "Мэксыканскі ціхаакіянскі летні час"}, {"Mongolia", "Летні час Улан-Батару"}, {"Moscow", "Маскоўскі летні час"}, {"New_Caledonia", "Летні час Новай Каледоніі"}, {"New_Zealand", "Летні час Новай Зэляндыі"}, {"Newfoundland", "Ньюфаўндлэндзкі летні час"}, {"Norfolk", "Летні час вострава Норфалк"}, {"Noronha", "Летні час Фэрнанду-ды-Наронья"}, {"Novosibirsk", "Новасыбірскі летні час"}, {"Omsk", "Омскі летні час"}, {"Pakistan", "Пакістанскі летні час"}, {"Paraguay", "Летні час Парагваю"}, {"Peru", "Пэруанскі летні час"}, {"Philippines", "Філіпінскі летні час"}, {"Pierre_Miquelon", "Змоўчны летні час Сэн-П’еру і Мікелёну"}, {"Sakhalin", "Сахалінскі летні час"}, {"Samoa", "Летні час Самоа"}, {"Taipei", "Летні час Тайбэю"}, {"Tonga", "Летні час Тонга"}, {"Turkmenistan", "Летні час Туркмэністану"}, {"Uruguay", "Уругвайскі летні час"}, {"Uzbekistan", "Летні час Узбэкістану"}, {"Vanuatu", "Летні час Вануату"}, {"Vladivostok", "Уладзівастоцкі летні час"}, {"Volgograd", "Валгаградзкі летні час"}, {"Yakutsk", "Якуцкі летні час"}, {"Yekaterinburg", "Екацярынбурскі летні час"}},
	},
	{
		{{"Acre", "Waktu Pakem Akre"}, {"Afghanistan", "Waktu Apganistan"}, {"Africa_Central", "Waktu Aprika Tenga"}, {"Africa_Eastern", "Waktu Aprika Wètan"}, {"Africa_Southern", "Waktu Pakem Aprika Kidul"}, {"Africa_Western", "Waktu Aprika Kulon"}, {"Alaska", "Waktu Pakem Alaska"}, {"Almaty", "Waktu Pakem Almati"}, {"Amazon", "Waktu Pakem Amason"}, {"America_Central", "Waktu Pakem Tenga"}, {"America_Eastern", "Waktu Pakem Wètan"}, {"America_Mountain", "Waktu Pakem Gunung"}, {"America_Pacific", "Waktu Pakem Teduh"}, {"Anadyr", "Waktu Pakem Anadir"}, {"Apia", "Waktu Pakem Apia"}, {"Aqtau", "Waktu Pakem Akto"}, {"Aqtobe", "Waktu Pakem Aktobé"}, {"Arabian", "Waktu Pakem Arab"}, {"Argentina", "Waktu Pakem Argèntina"}, {"Argentina_Western", "Waktu Pakem Argèntina Kulon"}, {"Armenia", "Waktu Pakem Lemènder"}, {"Atlantic", "Waktu Pakem Atlantik"}, {"Australia_Central", "Waktu Pakem Ostrali Tenga"}, {"Australia_CentralWestern", "Waktu Pakem Ostrali Kulon-tenga"}, {"Australia_Eastern", "Waktu Pakem Ostrali Wètan"}, {"Australia_Western", "Waktu Pakem Ostrali Kulon"}, {"Azerbaijan", "Waktu Pakem Asèrbaijan"}, {"Azores", "Waktu Pakem Asoren"}, {"Bangladesh", "Waktu Pakem Benggaladésa"}, {"Bhutan", "Waktu Butan"}, {"Bolivia", "Waktu Boliwi"}, {"Brasilia", "Waktu Pakem Brasilia"}, {"Brunei", "Waktu Bruné Darussalam"}, {"Cape_Verde", "Waktu Pakem Tanjung Ijo"}, {"Casey", "Waktu Casey"}, {"Chamorro", "Waktu Pakem Camoro"}, {"Chatham", "Waktu Pakem Catham"}, {"Chile", "Waktu Pakem Cili"}, {"China", "Waktu Pakem Tiongkok"}, {"Christmas", "Waktu Pulo Natal"}, {"Cocos", "Waktu Pulo Kokos"}, {"Colombia", "Waktu Pakem Kolombia"}, {"Cook", "Waktu Pakem Pulo Cook"}, {"Cuba", "Waktu Pakem Kuba"}, {"Davis", "Waktu Davis"}, {"DumontDUrville", "Waktu Dumont-d’Urville"}, {"East_Timor", "Waktu Timor Wètan"}, {"Easter", "Waktu Pakem Pulo Paskah"}, {"Ecuador", "Waktu Èkuador"}, {"Europe_Central", "Waktu Pakem Èropa Tenga"}, {"Europe_Eastern", "Waktu Pakem Èropa Wètan"}, {"Europe_Further_Eastern", "Waktu Èropa Wètan-jau"}, {"Europe_Western", "Waktu Pakem Èropa Kulon"}, {"Falkland", "Waktu Pakem Pulo Paklan"}, {"Fiji", "Waktu Pakem Piji"}, {"French_Guiana", "Waktu Guyana Prasman"}, {"French_Southern", "Waktu Wilayah Kulon èn Kutub Kidul Prasman"}, {"GMT", "Waktu Rerata Greenwich"}, {"Galapagos", "Waktu Galapagos"}, {"Gambier", "Waktu Gambier"}, {"Georgia", "Waktu Pakem Géorgi"}, {"Gilbert_Islands", "Waktu Pulo Gilbet"}, {"Greenland_Eastern", "Waktu Pakem Grunlan Wètan"}, {"Greenland_Western", "Waktu Pakem Grunlan Kulon"}, {"Gulf", "Waktu Pakem Telok"}, {"Guyana", "Waktu Guyana"}, {"Hawaii", "Waktu Pakem Hawai-Aléut"}, {"Hawaii_Aleutian", "Waktu Pakem Hawai-Aléut"}, {"Hong_Kong", "Waktu Pakem Hongkong"}, {"Hovd", "Waktu Pakem Houd"}, {"India", "Waktu Hindi Pakem"}, {"Indian_Ocean", "Waktu Laotan Hindi"}, {"Indochina", "Waktu Indocina"}, {"Indonesia_Central", "Waktu Indonésia Tenga"}, {"Indonesia_Eastern", "Waktu Indonésia Wètan"}, {"Indonesia_Western", "Waktu Indonésia Kulon"}, {"Iran", "Waktu Pakem Iran"}, {"Irkutsk", "Waktu Pakem Irkut"}, {"Israel", "Waktu Pakem Israèl"}, {"Japan", "Waktu Pakem Jepang"}, {"Kamchatka", "Waktu Pakem Pètropaulop-Kamcatka"}, {"Kazakhstan_Eastern", "Waktu Kasakstan Wètan"}, {"Kazakhstan_Western", "Waktu Kasakstan Kulon"}, {"Korea", "Waktu Pakem Koréa"}, {"Kosrae", "Waktu Kosaé"}, {"Krasnoyarsk", "Waktu Pakem Krasnoyar"}, {"Kyrgystan", "Waktu Kirgistan"}, {"Lanka", "Waktu Sri Langka"}, {"Line_Islands", "Waktu Pulo Line"}, {"Lord_Howe", "Waktu Pakem Lord Howe"}, {"Macau", "Waktu Pakem Makao"}, {"Magadan", "Waktu Pakem Magadan"}, {"Malaysia", "Waktu Malésia"}, {"Maldives", "Waktu Maladéwa"}, {"Marquesas", "Waktu Markésas"}, {"Marshall_Islands", "Waktu Pulo Marsal"}, {"Mauritius", "Waktu Pakem Moritius"}, {"Mawson", "Waktu Mawson"}, {"Mexico_Pacific", "Waktu Pakem Mèksiko Teduh"}, {"Mongolia", "Waktu Pakem Ulanbator"}, {"Moscow", "Waktu Pakem Mosko"}, {"Myanmar", "Waktu Mianmar"}, {"Nauru", "Waktu Nauru"}, {"Nepal", "Waktu Népal"}, {"New_Caledonia", "Waktu Pakem Kalédoni Baru"}, {"New_Zealand", "Waktu Pakem Sélan Baru"}, {"Newfoundland", "Waktu Pakem Niuponlan"}, {"Niue", "Waktu Niué"}, {"Norfolk", "Waktu Pakem Pulo Norpok"}, {"Noronha", "Waktu Pakem Fernando de Noronha"}, {"Novosibirsk", "Waktu Pakem Nowosibir"}, {"Omsk", "Waktu Pakem Om"}, {"Pakistan", "Waktu Pakem Pakistan"}, {"Palau", "Waktu Palau"}, {"Papua_New_Guinea", "Waktu Papua Giné Baru"}, {"Paraguay", "Waktu Pakem Paragué"}, {"Peru", "Waktu Pakem Péru"}, {"Philippines", "Waktu Pakem Pilipénen"}, {"Phoenix_Islands", "Waktu Pulo Pènik"}, {"Pierre_Miquelon", "Waktu Pakem Sint-Pièr èn Mikélon"}, {"Pitcairn", "Waktu Pitkèren"}, {"Ponape", "Waktu Ponapé"}, {"Pyongyang", "Waktu Piongyang"}, {"Qyzylorda", "Waktu Pakem Kisilorda"}, {"Reunion", "Waktu Réunion"}, {"Rothera", "Waktu Rothera"}, {"Sakhalin", "Waktu Pakem Sahalin"}, {"Samara", "Waktu Pakem Samara"}, {"Samoa", "Waktu Pakem Samoa"}, {"Seychelles", "Waktu Sésèl"}, {"Singapore", "Waktu Pakem Singapur"}, {"Solomon", "Waktu Pulo Suléman"}, {"South_Georgia", "Waktu Géorgi Kidul"}, {"Suriname", "Waktu Suriname"}, {"Syowa", "Waktu Syowa"}, {"Tahiti", "Waktu Tahiti"}, {"Taipei", "Waktu Pakem Taipé"}, {"Tajikistan", "Waktu Tajikistan"}, {"Tokelau", "Waktu Tokelau"}, {"Tonga", "Waktu Pakem Tonga"}, {"Truk", "Waktu Cuk"}, {"Turkmenistan", "Waktu Pakem Turkmènistan"}, {"Tuvalu", "Waktu Tuwalu"}, {"Uruguay", "Waktu Pakem Urugué"}, {"Uzbekistan", "Waktu Pakem Usbèkistan"}, {"Vanuatu", "Waktu Pakem Wanuatu"}, {"Venezuela", "Waktu Bénésuèla"}, {"Vladivostok", "Waktu Pakem Weladiwostok"}, {"Volgograd", "Waktu Pake Wolgograd"}, {"Vostok", "Waktu Wostok"}, {"Wake", "Waktu Pulo Wék"}, {"Wallis", "Waktu Walis èn Putuna"}, {"Yakutsk", "Waktu Pakem Yakut"}, {"Yekaterinburg", "Waktu Pakem Yékatèrinenbereh"}, {"Yukon", "Waktu Yukon"}},
		{{"Acre", "Waktu Musim Pentèr Akre"}, {"Alaska", "Waktu Musim Pentèr Alaska"}, {"Almaty", "Waktu Musim Pentèr Almati"}, {"Amazon", "Waktu Musim Pentèr Amason"}, {"America_Central", "Waktu Musim Pentèr Tenga"}, {"America_Eastern", "Waktu Musim Pentèr Wètan"}, {"America_Mountain", "Waktu Musim Pentèr Gunung"}, {"America_Pacific", "Waktu Musim Pentèr Teduh"}, {"Anadyr", "Waktu Musim Pentèr Anadir"}, {"Apia", "Waktu Musim Pentèr Apia"}, {"Aqtau", "Waktu Musim Pentèr Akto"}, {"Aqtobe", "Waktu Musim Pentèr Aktobé"}, {"Arabian", "Waktu Musim Pentèr Arab"}, {"Argentina", "Waktu Musim Pentèr Argèntina"}, {"Argentina_Western", "Waktu Musim Pentèr Argèntina Kulon"}, {"Armenia", "Waktu Musim Pentèr Lemènder"}, {"Atlantic", "Waktu Musim Pentèr Atlantik"}, {"Australia_Central", "Waktu Musim Pentèr Ostrali Tena"}, {"Australia_CentralWestern", "Waktu Musim Pentèr Ostrali Kulon-tenga"}, {"Australia_Eastern", "Waktu Musim Pentèr Ostrali Wètan"}, {"Australia_Western", "Waktu Musim Pentèr Ostrali Kulon"}, {"Azerbaijan", "Waktu Musim Pentèr Asèrbaijan"}, {"Azores", "Waktu Musim Pentèr Asoren"}, {"Bangladesh", "Waktu Musim Pentèr Benggaladésa"}, {"Brasilia", "Waktu Musim Pentèr Brasilia"}, {"Cape_Verde", "Waktu Musim Pentèr Tanjung Ijo"}, {"Chatham", "Waktu Musim Pentèr Catham"}, {"Chile", "Waktu Musim Pentèr Cili"}, {"China", "Waktu Musim Pentèr Tiongkok"}, {"Colombia", "Waktu Musim Pentèr Kolombia"}, {"Cook", "Waktu Musim Pentèr Pulo Cook"}, {"Cuba", "Waktu Musim Pentèr Kuba"}, {"Easter", "Waktu Musim Pentèr Pulo Paskah"}, {"Europe_Central", "Waktu Musim Pentèr Èropa Tenga"}, {"Europe_Eastern", "Waktu Musim Pentèr Èropa Wètan"}, {"Europe_Western", "Waktu Musim Pentèr Èropa Kulon"}, {"Falkland", "Waktu Musim Pentèr Pulo Paklan"}, {"Fiji", "Waktu Musim Pentèr Piji"}, {"Georgia", "Waktu Musim Pentèr Géorgi"}, {"Greenland_Eastern", "Waktu Musim Pentèr Grunlan Wètan"}, {"Greenland_Western", "Waktu Musim Pentèr Grunlan Kulon"}, {"Hawaii_Aleutian", "Waktu Musim Pentèr Hawai-Aléut"}, {"Hong_Kong", "Waktu Musim Pentèr Hongkong"}, {"Hovd", "Waktu Musim Pentèr Houd"}, {"Iran", "Waktu Musim Pentèr Iran"}, {"Irkutsk", "Waktu Musim Pentèr Irkut"}, {"Israel", "Waktu Musim Pentèr Israèl"}, {"Japan", "Waktu Musim Pentèr Jepang"}, {"Kamchatka", "Waktu Musim Pentèr Pètropaulop-Kamcatka"}, {"Korea", "Waktu Musim Pentèr Koréa"}, {"Krasnoyarsk", "Waktu Musim Pentèr Krasnoyar"}, {"Lord_Howe", "Waktu Musim Pentèr Lord Howe"}, {"Macau", "Waktu Musim Pentèr Makao"}, {"Magadan", "Waktu Musim Pentèr Magadan"}, {"Mauritius", "Waktu Musim Pentèr Moritius"}, {"Mexico_Pacific", "Waktu Musim Pentèr Mèksiko Teduh"}, {"Mongolia", "Waktu Musim Pentèr Ulanbator"}, {"Moscow", "Waktu Musim Pentèr Mosko"}, {"New_Caledonia", "Waktu Musim Pentèr Kalédoni Baru"}, {"New_Zealand", "Waktu Musim Pentèr Sélan Baru"}, {"Newfoundland", "Waktu Musim Pentèr Niuponlan"}, {"Norfolk", "Waktu Musim Pentèr Pulo Norpok"}, {"Noronha", "Waktu Musim Pentèr Fernando de Noronha"}, {"Novosibirsk", "Waktu Musim Pentèr Nowosibir"}, {"Omsk", "Waktu Musim Pentèr Om"}, {"Pakistan", "Waktu Musim Pentèr Pakistan"}, {"Paraguay", "Waktu Musim Pentèr Paragué"}, {"Peru", "Waktu Musim Pentèr Péru"}, {"Philippines", "Waktu Musim Pentèr Pilipénen"}, {"Pierre_Miquelon", "Waktu Musim Pentèr Sint-Pièr èn Mikélon"}, {"Qyzylorda", "Waktu Musim Pentèr Kisilorda"}, {"Sakhalin", "Waktu Musim Pentèr Sahalin"}, {"Samara", "Waktu Musim Pentèr Samara"}, {"Samoa", "Waktu Musim Pentèr Samoa"}, {"Taipei", "Waktu Musim Pentèr Taipé"}, {"Tonga", "Waktu Musim Pentèr Tonga"}, {"Turkmenistan", "Waktu Musim Pentèr Turkmènistan"}, {"Uruguay", "Waktu Musim Pentèr Urugué"}, {"Uzbekistan", "Waktu Musim Pentèr Usbèkistan"}, {"Vanuatu", "Waktu Musim Pentèr Wanuatu"}, {"Vladivostok", "Waktu Musim Pentèr Weladiwostok"}, {"Volgograd", "Waktu Musim Pentèr Wolgograd"}, {"Yakutsk", "Waktu Musim Pentèr Yakut"}, {"Yekaterinburg", "Waktu Musim Pentèr Yékatèrinenbereh"}},
	},
	{
		{{"Afghanistan", "Афганистанско време"}, {"Africa_Central", "Централноафриканско време"}, {"Africa_Eastern", "Източноафриканско време"}, {"Africa_Southern", "Южноафриканско време"}, {"Africa_Western", "Западноафриканско време"}, {"Alaska", "Аляска – стандартно време"}, {"Amazon", "Амазонско стандартно време"}, {"America_Central", "Северноамериканско централно стандартно време"}, {"America_Eastern", "Северноамериканско източно стандартно време"}, {"America_Mountain", "Северноамериканско планинско стандартно време"}, {"America_Pacific", "Северноамериканско тихоокеанско стандартно време"}, {"Anadyr", "Анадир – стандартно време"}, {"Apia", "Апия – стандартно време"}, {"Arabian", "Арабско стандартно време"}, {"Argentina", "Аржентинско стандартно време"}, {"Argentina_Western", "Западноаржентинско стандартно време"}, {"Armenia", "Арменско стандартно време"}, {"Atlantic", "Северноамериканско атлантическо стандартно време"}, {"Australia_Central", "Централноавстралийско стандартно време"}, {"Australia_CentralWestern", "Австралия – западно централно стандартно време"}, {"Australia_Eastern", "Източноавстралийско стандартно време"}, {"Australia_Western", "Западноавстралийско стандартно време"}, {"Azerbaijan", "Азербайджанско стандартно време"}, {"Azores", "Азорски острови – стандартно време"}, {"Bangladesh", "Бангладешко стандартно време"}, {"Bhutan", "Бутанско време"}, {"Bolivia", "Боливийско време"}, {"Brasilia", "Бразилско стандартно време"}, {"Brunei", "Бруней Даруссалам"}, {"Cape_Verde", "Кабо Верде – стандартно време"}, {"Chamorro", "Чаморско време"}, {"Chatham", "Чатъмско стандартно време"}, {"Chile", "Чилийско стандартно време"}, {"China", "Китайско стандартно време"}, {"Christmas", "Остров Рождество"}, {"Cocos", "Кокосови острови"}, {"Colombia", "Колумбийско стандартно време"}, {"Cook", "Острови Кук – стандартно време"}, {"Cuba", "Кубинско стандартно време"}, {"Davis", "Дейвис"}, {"DumontDUrville", "Дюмон Дюрвил"}, {"East_Timor", "Източнотиморско време"}, {"Easter", "Великденски остров – стандартно време"}, {"Ecuador", "Еквадорско време"}, {"Europe_Central", "Централноевропейско стандартно време"}, {"Europe_Eastern", "Източноевропейско стандартно време"}, {"Europe_Further_Eastern", "Далечно източноевропейско време"}, {"Europe_Western", "Западноевропейско стандартно време"}, {"Falkland", "Фолклендски острови – стандартно време"}, {"Fiji", "Фиджийско стандартно време"}, {"French_Guiana", "Френска Гвиана"}, {"French_Southern", "Френски южни и антарктически територии"}, {"GMT", "Средно гринуичко време"}, {"Galapagos", "Галапагоско време"}, {"Gambier", "Гамбие"}, {"Georgia", "Грузинско стандартно време"}, {"Gilbert_Islands", "Острови Гилбърт"}, {"Greenland_Eastern", "Източногренландско стандартно време"}, {"Greenland_Western", "Западногренландско стандартно време"}, {"Gulf", "Персийски залив"}, {"Guyana", "Гаяна"}, {"Hawaii", "Хавайско-алеутско стандартно време"}, {"Hawaii_Aleutian", "Хавайско-алеутско стандартно време"}, {"Hong_Kong", "Хонконгско стандартно време"}, {"Hovd", "Ховдско стандартно време"}, {"India", "Индийско време"}, {"Indian_Ocean", "Индийски океан"}, {"Indochina", "Индокитайско време"}, {"Indonesia_Central", "Централноиндонезийско време"}, {"Indonesia_Eastern", "Източноиндонезийско време"}, {"Indonesia_Western", "Западноиндонезийско време"}, {"Iran", "Иранско стандартно време"}, {"Irkutsk", "Иркутско стандартно време"}, {"Israel", "Израелско стандартно време"}, {"Japan", "Японско стандартно време"}, {"Kamchatka", "Петропавловск-Камчатски стандартно време"}, {"Kazakhstan", "Казахстанско време"}, {"Kazakhstan_Eastern", "Източноказахстанско време"}, {"Kazakhstan_Western", "Западноказахстанско време"}, {"Korea", "Корейско стандартно време"}, {"Kosrae", "Кошрай"}, {"Krasnoyarsk", "Красноярско стандартно време"}, {"Kyrgystan", "Киргизстанско време"}, {"Line_Islands", "Екваториални острови"}, {"Lord_Howe", "Лорд Хау – стандартно време"}, {"Magadan", "Магаданско стандартно време"}, {"Malaysia", "Малайзийско време"}, {"Maldives", "Малдивско време"}, {"Marquesas", "Маркизки острови"}, {"Marshall_Islands", "Маршалови острови"}, {"Mauritius", "Мавриций – стандартно време"}, {"Mawson", "Моусън"}, {"Mexico_Pacific", "Мексиканско тихоокеанско стандартно време"}, {"Mongolia", "Уланбаторско стандартно време"}, {"Moscow", "Московско стандартно време"}, {"Myanmar", "Мианмарско време"}, {"Nauru", "Науру"}, {"Nepal", "Непалско време"}, {"New_Caledonia", "Новокаледонско стандартно време"}, {"New_Zealand", "Новозеландско стандартно време"}, {"Newfoundland", "Нюфаундлендско стандартно време"}, {"Niue", "Ниуе"}, {"Norfolk", "Норфолкско стандартно време"}, {"Noronha", "Фернандо де Нороня – стандартно време"}, {"Novosibirsk", "Новосибирско стандартно време"}, {"Omsk", "Омско стандартно време"}, {"Pakistan", "Пакистанско стандартно време"}, {"Palau", "Палау"}, {"Papua_New_Guinea", "Папуа Нова Гвинея"}, {"Paraguay", "Парагвайско стандартно време"}, {"Peru", "Перуанско стандартно време"}, {"Philippines", "Филипинско стандартно време"}, {"Phoenix_Islands", "Острови Феникс"}, {"Pierre_Miquelon", "Сен Пиер и Микелон – стандартно време"}, {"Pitcairn", "Питкерн"}, {"Ponape", "Понапе"}, {"Pyongyang", "Пхенянско време"}, {"Reunion", "Реюнион"}, {"Rothera", "Ротера"}, {"Sakhalin", "Сахалинско стандартно време"}, {"Samara", "Самара – стандартно време"}, {"Samoa", "Самоанско стандартно време"}, {"Seychelles", "Сейшели"}, {"Singapore", "Сингапурско време"}, {"Solomon", "Соломонови острови"}, {"South_Georgia", "Южна Джорджия"}, {"Suriname", "Суринамско време"}, {"Syowa", "Шова"}, {"Tahiti", "Таитянско време"}, {"Taipei", "Тайпе – стандартно време"}, {"Tajikistan", "Таджикистанско време"}, {"Tokelau", "Токелау"}, {"Tonga", "Тонга – стандартно време"}, {"Truk", "Чуюк"}, {"Turkmenistan", "Туркменистанско стандартно време"}, {"Tuvalu", "Тувалу"}, {"Uruguay", "Уругвайско стандартно време"}, {"Uzbekistan", "Узбекистанско стандартно време"}, {"Vanuatu", "Вануату – стандартно време"}, {"Venezuela", "Венецуелско време"}, {"Vladivostok", "Владивостокско стандартно време"}, {"Volgograd", "Волгоградско стандартно време"}, {"Vostok", "Восток"}, {"Wake", "Остров Уейк"}, {"Wallis", "Уолис и Футуна"}, {"Yakutsk", "Якутскско стандартно време"}, {"Yekaterinburg", "Екатеринбургско стандартно време"}, {"Yukon", "Юкон"}},
		{{"Alaska", "Аляска – лятно часово време"}, {"Amazon", "Амазонско лятно часово време"}, {"America_Central", "Северноамериканско централно лятно часово време"}, {"America_Eastern", "Северноамериканско източно лятно часово време"}, {"America_Mountain", "Северноамериканско планинско лятно часово време"}, {"America_Pacific", "Северноамериканско тихоокеанско лятно часово време"}, {"Anadyr", "Анадир – лятно часово време"}, {"Apia", "Апия – лятно часово време"}, {"Arabian", "Арабско лятно часово време"}, {"Argentina", "Аржентинско лятно часово време"}, {"Argentina_Western", "Западноаржентинско лятно часово време"}, {"Armenia", "Арменско лятно часово време"}, {"Atlantic", "Северноамериканско атлантическо лятно часово време"}, {"Australia_Central", "Централноавстралийско лятно часово време"}, {"Australia_CentralWestern", "Австралия – западно централно лятно часово време"}, {"Australia_Eastern", "Източноавстралийско лятно часово време"}, {"Australia_Western", "Западноавстралийско лятно часово време"}, {"Azerbaijan", "Азербайджанско лятно часово време"}, {"Azores", "Азорски острови – лятно часово време"}, {"Bangladesh", "Бангладешко лятно часово време"}, {"Brasilia", "Бразилско лятно часово време"}, {"Cape_Verde", "Кабо Верде – лятно часово време"}, {"Chatham", "Чатъмско лятно часово време"}, {"Chile", "Чилийско лятно часово време"}, {"China", "Китайско лятно часово време"}, {"Colombia", "Колумбийско лятно часово време"}, {"Cook", "Острови Кук – лятно часово време"}, {"Cuba", "Кубинско лятно часово време"}, {"Easter", "Великденски остров – лятно часово време"}, {"Europe_Central", "Централноевропейско лятно часово време"}, {"Europe_Eastern", "Източноевропейско лятно часово време"}, {"Europe_Western", "Западноевропейско лятно време"}, {"Falkland", "Фолклендски острови – лятно часово време"}, {"Fiji", "Фиджийско лятно часово време"}, {"Georgia", "Грузинско лятно часово време"}, {"Greenland_Eastern", "Източногренландско лятно часово време"}, {"Greenland_Western", "Западногренландско лятно часово време"}, {"Hawaii_Aleutian", "Хавайско-алеутско лятно часово време"}, {"Hong_Kong", "Хонконгско лятно часово време"}, {"Hovd", "Ховдско лятно часово време"}, {"Iran", "Иранско лятно часово време"}, {"Irkutsk", "Иркутско лятно часово време"}, {"Israel", "Израелско лятно часово време"}, {"Japan", "Японско лятно часово време"}, {"Kamchatka", "Петропавловск-Камчатски – лятно часово време"}, {"Korea", "Корейско лятно часово време"}, {"Krasnoyarsk", "Красноярско лятно часово време"}, {"Lord_Howe", "Лорд Хау – лятно часово време"}, {"Magadan", "Магаданско лятно часово време"}, {"Mauritius", "Мавриций – лятно часово време"}, {"Mexico_Pacific", "Мексиканско тихоокеанско лятно часово време"}, {"Mongolia", "Уланбаторско лятно часово време"}, {"Moscow", "Московско лятно часово време"}, {"New_Caledonia", "Новокаледонско лятно часово време"}, {"New_Zealand", "Новозеландско лятно часово време"}, {"Newfoundland", "Нюфаундлендско лятно часово време"}, {"Norfolk", "Норфолкско лятно часово време"}, {"Noronha", "Фернандо де Нороня – лятно часово време"}, {"Novosibirsk", "Новосибирско лятно часово време"}, {"Omsk", "Омско лятно часово време"}, {"Pakistan", "Пакистанско лятно часово време"}, {"Paraguay", "Парагвайско лятно часово време"}, {"Peru", "Перуанско лятно часово време"}, {"Philippines", "Филипинско лятно часово време"}, {"Pierre_Miquelon", "Сен Пиер и Микелон – лятно часово време"}, {"Sakhalin", "Сахалинско лятно часово време"}, {"Samara", "Самара – лятно часово време"}, {"Samoa", "Самоанско лятно часово време"}, {"Taipei", "Тайпе – лятно часово време"}, {"Tonga", "Тонга – лятно часово време"}, {"Turkmenistan", "Туркменистанско лятно часово време"}, {"Uruguay", "Уругвайско лятно часово време"}, {"Uzbekistan", "Узбекистанско лятно часово време"}, {"Vanuatu", "Вануату – лятно часово време"}, {"Vladivostok", "Владивостокско лятно часово време"}, {"Volgograd", "Волгоградско лятно часово време"}, {"Yakutsk", "Якутскско лятно часово време"}, {"Yekaterinburg", "Екатеринбургско лятно часово време"}},
	},
	{
		{{"GMT", "ग्रीनविच मीन टाइम"}},
		{},
	},
	{
		{{"Afghanistan", "اوگانستانی وخت"}, {"Kazakhstan_Eastern", "روچ\u200cدراتین قزاقستانی وخت"}, {"Kazakhstan_Western", "روچ\u200cکپتین قزاقستانی وخت"}, {"Kyrgystan", "قیرغیزستانی وخت"}, {"Tajikistan", "تاجیکستانی وخت"}},
		{},
	},
	{
		{{"Afghanistan", "Afganistan kaakɔŋkɔŋɔ̀"}, {"Africa_Central", "Garɩɖontǝna gɩcɩɩca kaakɔŋkɔŋɔ̀"}, {"Africa_Eastern", "Garɩɖontǝna gajakalaŋ kaakɔŋkɔŋɔ̀"}, {"Africa_Southern", "Garɩɖontǝna gʊnyɩsonolaŋ kaakɔŋkɔŋɔ̀"}, {"Africa_Western", "Garɩɖontǝna gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀"}, {"Alaska", "Alaskaa kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Amazon", "Amasɔn kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"America_Central", "Amalɩka gʊnyɩpɛnɛlaŋ kagɩcɩɩca kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"America_Eastern", "Amalɩka gʊnyɩpɛnɛlaŋ kaajakalaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"America_Mountain", "Amalɩka gʊnyɩpɛnɛlaŋ kabʊnʊ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"America_Pacific", "Amalɩka kapasifika kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Apia", "Samowa kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Arabian", "Galaaributǝna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Argentina", "Arjantin kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Argentina_Western", "Arjantin gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Armenia", "Armenii kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Atlantic", "Amalɩka katǝlantika kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Australia_Central", "Ɔstraliya kagɩcɩɩca kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Australia_CentralWestern", "Ɔstraliya kagɩcɩɩca gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Australia_Eastern", "Ɔstraliya kaajakalaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Australia_Western", "Ɔstraliya kagɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Azerbaijan", "Asɛrbaɩjaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Azores", "Asɔɔr kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Bangladesh", "Baŋglaɖɛɛshɩ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Bhutan", "Butan kaakɔŋkɔŋɔ̀"}, {"Bolivia", "Bolifiya kaakɔŋkɔŋɔ̀"}, {"Brasilia", "Brasiliya kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Brunei", "Brunɛɩ kaakɔŋkɔŋɔ̀"}, {"Cape_Verde", "Kapfɛɛr kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Chamorro", "Shamoroo kaakɔŋkɔŋɔ̀"}, {"Chatham", "Shatam kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Chile", "Shilii kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"China", "Caɩna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Christmas", "Nowɛl kaAtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Cocos", "Kokoos kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Colombia", "Kolɔmbii kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Cook", "Kʊkʊ kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Cuba", "Kubaa kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Davis", "Ɖefis kaakɔŋkɔŋɔ̀"}, {"DumontDUrville", "Ɖimɔn Ɖirfil kaakɔŋkɔŋɔ̀"}, {"East_Timor", "Timɔɔ gajakalaŋ kaakɔŋkɔŋɔ̀"}, {"Easter", "Paakɩ kaAtukǝltǝna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Ecuador", "Ekuwaɖɔɔr kaakɔŋkɔŋɔ̀"}, {"Europe_Central", "Garɩfɔntǝna gɩcɩɩca kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Europe_Eastern", "Garɩfɔntǝna gajakalaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Europe_Further_Eastern", "Garɩfɔntǝna gajakalaŋ kaajakalaŋ kaakɔŋkɔŋɔ̀"}, {"Europe_Western", "Garɩfɔntǝna gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Falkland", "Fɔklanɖ kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Fiji", "Fiji kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"French_Guiana", "Guyanaa Gafɔntǝna kaja kaakɔŋkɔŋɔ̀"}, {"French_Southern", "Gafɔntǝna gʊnyɩsonolaŋ na Gatutaltǝna kaakɔŋkɔŋɔ̀"}, {"GMT", "Griinwish kaakɔŋkɔŋɔ̀"}, {"Galapagos", "Galapagɔs kaakɔŋkɔŋɔ̀"}, {"Gambier", "Gambiyee kaakɔŋkɔŋɔ̀"}, {"Georgia", "Jɔrjiya kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Gilbert_Islands", "Jilbɛɛr kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Greenland_Eastern", "Grinlanɖ gajakalaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Greenland_Western", "Grinlanɖ gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Guam", "Guwam kaakɔŋkɔŋɔ̀"}, {"Gulf", "Gɔlf kaakɔŋkɔŋɔ̀"}, {"Guyana", "Guyanaa kaakɔŋkɔŋɔ̀"}, {"Hawaii", "Awayɩɩ n’Alewutii kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Hawaii_Aleutian", "Awayɩɩ n’Alewutii kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Hong_Kong", "Hɔŋ Kɔŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Hovd", "Kɔfɖǝ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"India", "Inɖiya kaakɔŋkɔŋɔ̀"}, {"Indian_Ocean", "Inɖiya kateŋku kaakɔŋkɔŋɔ̀"}, {"Indochina", "Inɖicaɩna kaakɔŋkɔŋɔ̀"}, {"Indonesia_Central", "Ɛnɖonosii kagɩcɩɩca kaakɔŋkɔŋɔ̀"}, {"Indonesia_Eastern", "Ɛnɖonosii kaajakalaŋ kaakɔŋkɔŋɔ̀"}, {"Indonesia_Western", "Ɛnɖonosii kagɩteŋshilelaŋ kaakɔŋkɔŋɔ̀"}, {"Iran", "Iraŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Irkutsk", "Irkut kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Israel", "Yishraɛl kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Japan", "Japaŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Kazakhstan", "Kasastan kaakɔŋkɔŋɔ̀"}, {"Kazakhstan_Eastern", "Kasastan gajakalaŋ kaakɔŋkɔŋɔ̀"}, {"Kazakhstan_Western", "Kasastan gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀"}, {"Korea", "Koree kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Kosrae", "Kɔsrɛɛ kaakɔŋkɔŋɔ̀"}, {"Krasnoyarsk", "Krasnoyark kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Kyrgystan", "Kirgistan kaakɔŋkɔŋɔ̀"}, {"Lanka", "Siri Laŋkaa kaakɔŋkɔŋɔ̀"}, {"Line_Islands", "Laɩn kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Lord_Howe", "Lɔrɖ Hoo kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Macau", "Makawoo kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Magadan", "Magaɖan kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Malaysia", "Malɛsii kaakɔŋkɔŋɔ̀"}, {"Maldives", "Malɖiifu kaakɔŋkɔŋɔ̀"}, {"Marquesas", "Markesas kaakɔŋkɔŋɔ̀"}, {"Marshall_Islands", "Marshal kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Mauritius", "Imoris kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Mawson", "Mɔsɔn kaakɔŋkɔŋɔ̀"}, {"Mexico_Pacific", "Mɛsik kapasifika kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Mongolia", "Ulanbatɔɔr kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Moscow", "Moskuu kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Myanmar", "Miyanmaa kaakɔŋkɔŋɔ̀"}, {"Nauru", "Nawuru kaakɔŋkɔŋɔ̀"}, {"Nepal", "Neepal kaakɔŋkɔŋɔ̀"}, {"New_Caledonia", "Kaleɖonii afɔlɩ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"New_Zealand", "Selanɖ afɔlɩ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Newfoundland", "Faʊnɖlanɖ afɔlɩ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Niue", "Niwuye kaakɔŋkɔŋɔ̀"}, {"Norfolk", "Nɔrfook kaAtukǝltǝna kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Noronha", "Fɛrnanɖo ɖe Norɔnya kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"North_Mariana", "Mariyan kǝbʊtukǝltǝna gʊnyɩpɛnɛlaŋ kaakɔŋkɔŋɔ̀"}, {"Novosibirsk", "Nofosibirk kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Omsk", "Ɔmsǝkǝ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Pakistan", "Pakistan kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Palau", "Palawoo kaakɔŋkɔŋɔ̀"}, {"Papua_New_Guinea", "Papuasii Ginee afɔlɩ kaakɔŋkɔŋɔ̀"}, {"Paraguay", "Paraguwee kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Peru", "Peruu kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Philippines", "Filipiin kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Phoenix_Islands", "Foeniis kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Pierre_Miquelon", "Sɛŋ-Petrɔs na Mikelɔŋ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Pitcairn", "Pɩtkɛɛn kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Ponape", "Poonpei kaakɔŋkɔŋɔ̀"}, {"Pyongyang", "Koree gʊnyɩpɛnɛlaŋ kaakɔŋkɔŋɔ̀"}, {"Reunion", "Reeniyɔŋ kaakɔŋkɔŋɔ̀"}, {"Rothera", "Roteraa kaakɔŋkɔŋɔ̀"}, {"Sakhalin", "Sakalin kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Samoa", "Samowa Amalɩka kaja kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Seychelles", "Seshɛl kaakɔŋkɔŋɔ̀"}, {"Singapore", "Siŋgapuur kaakɔŋkɔŋɔ̀"}, {"Solomon", "Salomɔɔn kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"South_Georgia", "Jɔrjiya gʊnyɩsono kaakɔŋkɔŋɔ̀"}, {"Suriname", "Surinam kaakɔŋkɔŋɔ̀"}, {"Syowa", "Siyowaa kaakɔŋkɔŋɔ̀"}, {"Tahiti", "Tahitii kaakɔŋkɔŋɔ̀"}, {"Taipei", "Taɩwan kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Tajikistan", "Tajikistan kaakɔŋkɔŋɔ̀"}, {"Tokelau", "Tokelaʊ kaakɔŋkɔŋɔ̀"}, {"Tonga", "Tɔŋga kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Truk", "Cuuk kaakɔŋkɔŋɔ̀"}, {"Turkmenistan", "Turkmenistan kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Tuvalu", "Tufalu kaakɔŋkɔŋɔ̀"}, {"Uruguay", "Uruguwee kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Uzbekistan", "Usbeekistan kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Vanuatu", "Fanuwatu kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Venezuela", "Fenesuwelaa kaakɔŋkɔŋɔ̀"}, {"Vladivostok", "Flaɖifɔstɔk kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Volgograd", "Fɔlgograaɖ kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Vostok", "Fɔstɔk kaakɔŋkɔŋɔ̀"}, {"Wake", "Week kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀"}, {"Wallis", "Walis na Futuna kaakɔŋkɔŋɔ̀"}, {"Yakutsk", "Yakut kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Yekaterinburg", "Yekaterinbuu kaakɔŋkɔŋɔ̀ ɖeiɖei"}, {"Yukon", "Yukɔn kaakɔŋkɔŋɔ̀"}},
		{{"Alaska", "Alaskaa kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Amazon", "Amasɔn kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"America_Central", "Amalɩka gʊnyɩpɛnɛlaŋ kagɩcɩɩca kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"America_Eastern", "Amalɩka gʊnyɩpɛnɛlaŋ kaajakalaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"America_Mountain", "Amalɩka gʊnyɩpɛnɛlaŋ kabʊnʊ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"America_Pacific", "Amalɩka kapasifika kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Apia", "Samowa kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Arabian", "Galaaributǝna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Argentina", "Arjantin kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Argentina_Western", "Arjantin gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Armenia", "Armenii kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Atlantic", "Amalɩka katǝlantika kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Australia_Central", "Ɔstraliya kagɩcɩɩca kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Australia_CentralWestern", "Ɔstraliya kagɩcɩɩca gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Australia_Eastern", "Ɔstraliya kaajakalaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Australia_Western", "Ɔstraliya kagɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Azerbaijan", "Asɛrbaɩjaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Azores", "Asɔɔr kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Bangladesh", "Baŋglaɖɛɛshɩ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Brasilia", "Brasiliya kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Cape_Verde", "Kapfɛɛr kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Chatham", "Shatam kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Chile", "Shilii kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"China", "Caɩna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Colombia", "Kolɔmbii kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Cook", "Kʊkʊ kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Cuba", "Kubaa kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Easter", "Paakɩ kaAtukǝltǝna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Europe_Central", "Garɩfɔntǝna gɩcɩɩca kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Europe_Eastern", "Garɩfɔntǝna gajakalaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Europe_Western", "Garɩfɔntǝna gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Falkland", "Fɔklanɖ kaBʊtukǝltǝna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Fiji", "Fiji kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Georgia", "Jɔrjiya kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Greenland_Eastern", "Grinlanɖ gajakalaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Greenland_Western", "Grinlanɖ gɩteŋshilelaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Hawaii_Aleutian", "Awayɩɩ n’Alewutii kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Hong_Kong", "Hɔŋ Kɔŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Hovd", "Kɔfɖǝ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Iran", "Iraŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Irkutsk", "Irkut kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Israel", "Yishraɛl kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Japan", "Japaŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Korea", "Koree kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Krasnoyarsk", "Krasnoyark kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Lord_Howe", "Lɔrɖ Hoo kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Macau", "akawoo kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Magadan", "Magaɖan kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Mauritius", "Imoris kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Mexico_Pacific", "Mɛsik kapasifika kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Mongolia", "Ulanbatɔɔr kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Moscow", "Moskuu kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"New_Caledonia", "Kaleɖonii afɔlɩ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"New_Zealand", "Selanɖ afɔlɩ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Newfoundland", "Faʊnɖlanɖ afɔlɩ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Norfolk", "Nɔrfook kaAtukǝltǝna kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Noronha", "Fɛrnanɖo ɖe Norɔnya kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Novosibirsk", "Nofosibirk kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Omsk", "Ɔmsǝkǝ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Pakistan", "Pakistan kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Paraguay", "Paraguwee kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Peru", "Peruu kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Philippines", "Filipiin kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Pierre_Miquelon", "Sɛŋ-Petrɔs na Mikelɔŋ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Sakhalin", "Sakalin kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Samoa", "Samowa Amalɩka kaja kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Taipei", "Taɩwan kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Tonga", "Tɔŋga kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Turkmenistan", "Turkmenistan kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Uruguay", "Uruguwee kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Uzbekistan", "Usbeekistan kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Vanuatu", "Fanuwatu kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Vladivostok", "Flaɖifɔstɔk kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Volgograd", "Fɔlgograaɖ kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Yakutsk", "Yakut kaakɔŋkɔŋɔ̀ gafʊbaka"}, {"Yekaterinburg", "Yekaterinbuu kaakɔŋkɔŋɔ̀ gafʊbaka"}},
	},
	{
		{{"Acre", "একর মানক সময়"}, {"Afghanistan", "আফগানিস্তান সময়"}, {"Africa_Central", "মধ্য আফ্রিকা সময়"}, {"Africa_Eastern", "পূর্ব আফ্রিকা সময়"}, {"Africa_Southern", "দক্ষিণ আফ্রিকা মানক সময়"}, {"Africa_Western", "পশ্চিম আফ্রিকা সময়"}, {"Alaska", "আলাস্কা মানক সময়"}, {"Almaty", "আল্মাটি মানক সময়"}, {"Amazon", "আমাজন মানক সময়"}, {"America_Central", "কেন্দ্রীয় মানক সময়"}, {"America_Eastern", "পূর্বাঞ্চলের মানক সময়"}, {"America_Mountain", "পার্বত্য অঞ্চলের মানক সময়"}, {"America_Pacific", "প্রশান্ত মহাসাগরীয় অঞ্চলের মানক সময়"}, {"Anadyr", "অনদ্য্র্ মানক সময়"}, {"Apia", "অপিয়া মানক সময়"}, {"Aqtau", "আকটাও মানক সময়"}, {"Aqtobe", "আকটোব মানক সময়"}, {"Arabian", "আরবি মানক সময়"}, {"Argentina", "আর্জেন্টিনা মানক সময়"}, {"Argentina_Western", "পশ্চিমি আর্জেনটিনার প্রমাণ সময়"}, {"Armenia", "আর্মেনিয়া মানক সময়"}, {"Atlantic", "অতলান্তিক মানক সময়"}, {"Australia_Central", "অস্ট্রেলীয় কেন্দ্রীয় মানক সময়"}, {"Australia_CentralWestern", "অস্ট্রেলীয় কেন্দ্রীয় পশ্চিমি মানক সময়"}, {"Australia_Eastern", "অস্ট্রেলীয় পূর্ব মানক সময়"}, {"Australia_Western", "অস্ট্রেলীয় পশ্চিমি মানক সময়"}, {"Azerbaijan", "আজারবাইজান মানক সময়"}, {"Azores", "এজোরেস মানক সময়"}, {"Bangladesh", "বাংলাদেশ মানক সময়"}, {"Bhutan", "ভুটান সময়"}, {"Bolivia", "বোলিভিয়া সময়"}, {"Brasilia", "ব্রাসিলিয়া মানক সময়"}, {"Brunei", "ব্রুনেই দারুসসালাম সময়"}, {"Cape_Verde", "কেপ ভার্দ মানক সময়"}, {"Chamorro", "চামেরো মানক সময়"}, {"Chatham", "চ্যাথাম মানক সময়"}, {"Chile", "চিলি মানক সময়"}, {"China", "চীন মানক সময়"}, {"Christmas", "ক্রিসমাস দ্বীপ সময়"}, {"Cocos", "কোকোস দ্বীপপুঞ্জ সময়"}, {"Colombia", "কোলোম্বিয়া মানক সময়"}, {"Cook", "কুক দ্বীপপুঞ্জ মানক সময়"}, {"Cuba", "কিউবা মানক সময়"}, {"Davis", "ডেভিস সময়"}, {"DumontDUrville", "ডুমন্ট-দ্য’উরভিলে সময়"}, {"East_Timor", "পূর্ব টিমর সময়"}, {"Easter", "ইস্টার দ্বীপ মানক সময়"}, {"Ecuador", "ইকুয়েডর সময়"}, {"Europe_Central", "মধ্য ইউরোপীয় মানক সময়"}, {"Europe_Eastern", "পূর্ব ইউরোপীয় মানক সময়"}, {"Europe_Further_Eastern", "প্রান্তীয় পূর্ব ইউরোপীয় সময়"}, {"Europe_Western", "পশ্চিম ইউরোপীয় মানক সময়"}, {"Falkland", "ফকল্যান্ড দ্বীপপুঞ্জ মানক সময়"}, {"Fiji", "ফিজি মানক সময়"}, {"French_Guiana", "ফরাসি গায়ানা সময়"}, {"French_Southern", "ফরাসি দক্ষিণ এবং আন্টার্কটিক সময়"}, {"GMT", "গ্রীনিচ মিন টাইম"}, {"Galapagos", "গালাপাগোস সময়"}, {"Gambier", "গ্যাম্বিয়ার সময়"}, {"Georgia", "জর্জিয়া মানক সময়"}, {"Gilbert_Islands", "গিলবার্ট দ্বীপপুঞ্জ সময়"}, {"Greenland_Eastern", "পূর্ব গ্রীনল্যান্ড মানক সময়"}, {"Greenland_Western", "পশ্চিম গ্রীনল্যান্ড মানক সময়"}, {"Guam", "গুয়াম মান সময়"}, {"Gulf", "উপসাগরীয় মানক সময়"}, {"Guyana", "গুয়ানা সময়"}, {"Hawaii", "হাওয়াই-আলেউত মানক সময়"}, {"Hawaii_Aleutian", "হাওয়াই-আলেউত মানক সময়"}, {"Hong_Kong", "হং কং মানক সময়"}, {"Hovd", "হোভড মানক সময়"}, {"India", "ভারতীয় মানক সময়"}, {"Indian_Ocean", "ভারত মহাসাগরীয় সময়"}, {"Indochina", "ইন্দোচীন সময়"}, {"Indonesia_Central", "কেন্দ্রীয় ইন্দোনেশিয়া সময়"}, {"Indonesia_Eastern", "পূর্ব ইন্দোনেশিয়া সময়"}, {"Indonesia_Western", "পশ্চিমী ইন্দোনেশিয়া সময়"}, {"Iran", "ইরান মানক সময়"}, {"Irkutsk", "ইরকুটস্ক মানক সময়"}, {"Israel", "ইজরায়েল মানক সময়"}, {"Japan", "জাপান মানক সময়"}, {"Kamchatka", "পিত্রেপ্যাভলস্ক- ক্যামচ্যাটস্কি মান সময়"}, {"Kazakhstan", "কাজাখাস্তান সময়"}, {"Kazakhstan_Eastern", "পূর্ব কাজাখাস্তান সময়"}, {"Kazakhstan_Western", "পশ্চিম কাজাখাস্তান সময়"}, {"Korea", "কোরিয়ান মানক সময়"}, {"Kosrae", "কোসরেই সময়"}, {"Krasnoyarsk", "ক্রাসনোয়ার্স্কি মানক সময়"}, {"Kyrgystan", "কিরগিস্তান সময়"}, {"Lanka", "লঙ্কা সময়"}, {"Line_Islands", "লাইন দ্বীপপুঞ্জ সময়"}, {"Lord_Howe", "লর্ড হাওয়ে মানক মসয়"}, {"Macau", "মাকাও মান সময়"}, {"Magadan", "ম্যাগাডান মানক সময়"}, {"Malaysia", "মালয়েশিয়া সময়"}, {"Maldives", "মালদ্বীপ সময়"}, {"Marquesas", "মার্কেসাস সময়"}, {"Marshall_Islands", "মার্শাল দ্বীপপুঞ্জ সময়"}, {"Mauritius", "মরিশাস মানক সময়"}, {"Mawson", "মসন সময়"}, {"Mexico_Pacific", "মেক্সিকান প্রশান্ত মহসাগরীয় মানক সময়"}, {"Mongolia", "উলান বাতোর মানক সময়"}, {"Moscow", "মস্কো মানক সময়"}, {"Myanmar", "মায়ানমার সময়"}, {"Nauru", "নাউরু সময়"}, {"Nepal", "নেপাল সময়"}, {"New_Caledonia", "নিউ ক্যালেডোনিয়া মানক সময়"}, {"New_Zealand", "নিউজিল্যান্ড মানক সময়"}, {"Newfoundland", "নিউফাউন্ডল্যান্ড মানক সময়"}, {"Niue", "নিউই সময়"}, {"Norfolk", "নরফোক দ্বীপ মানক সময়"}, {"Noronha", "ফার্নান্দো ডি নোরোনহা মানক সময়"}, {"North_Mariana", "উত্তর মেরিন দ্বীপপুঞ্জ সময়"}, {"Novosibirsk", "নোভোসিবির্স্ক মানক সময়"}, {"Omsk", "ওমস্ক মানক সময়"}, {"Pakistan", "পাকিস্তান মানক সময়"}, {"Palau", "পালাউ সময়"}, {"Papua_New_Guinea", "পাপুয়া নিউ গিনি সময়"}, {"Paraguay", "প্যারাগুয়ে মানক সময়"}, {"Peru", "পেরু মানক সময়"}, {"Philippines", "ফিলিপাইন মানক সময়"}, {"Phoenix_Islands", "ফোনিক্স দ্বীপপুঞ্জ সময়"}, {"Pierre_Miquelon", "সেন্ট পিয়ের ও মিকেলন মানক সময়"}, {"Pitcairn", "পিটকেয়ার্ন সময়"}, {"Ponape", "পোনাপে সময়"}, {"Pyongyang", "পিয়ংইয়াং সময়"}, {"Qyzylorda", "কিজিলোর্ডা মান সময়"}, {"Reunion", "রিইউনিয়ন সময়"}, {"Rothera", "রথেরা সময়"}, {"Sakhalin", "সাখালিন মানক সময়"}, {"Samara", "সামারা মান সময়"}, {"Samoa", "সামোয়া মানক সময়"}, {"Seychelles", "সেশেলস সময়"}, {"Singapore", "সিঙ্গাপুর মানক সময়"}, {"Solomon", "সলোমন দ্বীপপুঞ্জ সময়"}, {"South_Georgia", "দক্ষিণ জর্জিয়া সময়"}, {"Suriname", "সুরিনাম সময়"}, {"Syowa", "সায়োওয়া সময়"}, {"Tahiti", "তাহিতি সময়"}, {"Taipei", "তাইপেই মানক সময়"}, {"Tajikistan", "তাজাখাস্তান সময়"}, {"Tokelau", "টোকেলাউ সময়"}, {"Tonga", "টোঙ্গা মানক সময়"}, {"Truk", "চুক সময়"}, {"Turkey", "তুর্কী মান সময়"}, {"Turkmenistan", "তুর্কমেনিস্তান মানক সময়"}, {"Tuvalu", "টুভালু সময়"}, {"Uruguay", "উরুগুয়ে মানক সময়"}, {"Uzbekistan", "উজবেকিস্তান মানক সময়"}, {"Vanuatu", "ভানুয়াতু মানক সময়"}, {"Venezuela", "ভেনেজুয়েলা সময়"}, {"Vladivostok", "ভ্লাদিভস্তক মানক সময়"}, {"Volgograd", "ভলগোগ্রাড মানক সময়"}, {"Vostok", "ভসটক সময়"}, {"Wake", "ওয়েক দ্বীপ সময়"}, {"Wallis", "ওয়ালিস এবং ফুটুনা সময়"}, {"Yakutsk", "ইয়াকুটাস্ক মানক সময়"}, {"Yekaterinburg", "ইয়েকাতেরিনবুর্গ মানক সময়"}, {"Yukon", "ইউকোন সময়"}},
		{{"Acre", "একর গ্রীষ্মকাল সময়"}, {"Alaska", "আলাস্কা দিবালোক সময়"}, {"Almaty", "আল্মাটি গ্রীষ্মকাল সময়"}, {"Amazon", "আমাজন গ্রীষ্মকালীন সময়"}, {"America_Central", "কেন্দ্রীয় দিবালোক সময়"}, {"America_Eastern", "পূর্বাঞ্চলের দিবালোক সময়"}, {"America_Mountain", "পার্বত্য অঞ্চলের দিনের সময়"}, {"America_Pacific", "প্রশান্ত মহাসাগরীয় অঞ্চলের দিনের সময়"}, {"Anadyr", "অনদ্য্র্ গ্রীষ্মকালীন সময়"}, {"Apia", "অপিয়া দিনের সময়"}, {"Aqtau", "আকটাও গ্রীষ্মকাল সময়"}, {"Aqtobe", "আকটোব গ্রীষ্মকাল সময়"}, {"Arabian", "আরবি দিবালোক সময়"}, {"Argentina", "আর্জেন্টিনা গ্রীষ্মকালীন সময়"}, {"Argentina_Western", "পশ্চিমি আর্জেনটিনা গ্রীষ্মকালীন সময়"}, {"Armenia", "আর্মেনিয়া গ্রীষ্মকালীন সময়"}, {"Atlantic", "অতলান্তিক দিবালোক সময়"}, {"Australia_Central", "অস্ট্রেলীয় কেন্দ্রীয় দিবালোক সময়"}, {"Australia_CentralWestern", "অস্ট্রেলীয় কেন্দ্রীয় পশ্চিমি দিবালোক সময়"}, {"Australia_Eastern", "অস্ট্রেলীয় পূর্ব দিবালোক সময়"}, {"Australia_Western", "অস্ট্রেলীয় পশ্চিমি দিবালোক সময়"}, {"Azerbaijan", "আজারবাইজান গ্রীষ্মকালীন সময়"}, {"Azores", "এজোরেস গ্রীষ্মকালীন সময়"}, {"Bangladesh", "বাংলাদেশ গ্রীষ্মকালীন সময়"}, {"Brasilia", "ব্রাসিলিয়া গ্রীষ্মকালীন সময়"}, {"Cape_Verde", "কেপ ভার্দ গ্রীষ্মকালীন সময়"}, {"Chatham", "চ্যাথাম দিবালোক সময়"}, {"Chile", "চিলি গ্রীষ্মকালীন সময়"}, {"China", "চীন দিবালোক সময়"}, {"Colombia", "কলম্বিয়া গ্রীষ্মকালীন সময়"}, {"Cook", "কুক দ্বীপপুঞ্জ অর্ধেক গ্রীষ্মকালীন সময়"}, {"Cuba", "কিউবা দিবালোক সময়"}, {"Easter", "ইস্টার দ্বীপ গ্রীষ্মকালীন সময়"}, {"Europe_Central", "মধ্য ইউরোপীয় গ্রীষ্মকালীন সময়"}, {"Europe_Eastern", "পূর্ব ইউরোপীয় গ্রীষ্মকালীন সময়"}, {"Europe_Western", "পশ্চিম ইউরোপীয় গ্রীষ্মকালীন সময়"}, {"Falkland", "ফকল্যান্ড দ্বীপপুঞ্জ গ্রীষ্মকালীন সময়"}, {"Fiji", "ফিজি গ্রীষ্মকালীন সময়"}, {"Georgia", "জর্জিয়া গ্রীষ্মকালীন সময়"}, {"Greenland_Eastern", "পূর্ব গ্রীনল্যান্ড গ্রীষ্মকালীন সময়"}, {"Greenland_Western", "পশ্চিম গ্রীনল্যান্ড গ্রীষ্মকালীন সময়"}, {"Hawaii_Aleutian", "হাওয়াই-আলেউত দিবালোক সময়"}, {"Hong_Kong", "হং কং গ্রীষ্মকালীন সময়"}, {"Hovd", "হোভড গ্রীষ্মকালীন সময়"}, {"Iran", "ইরান দিবালোক সময়"}, {"Irkutsk", "ইরকুটস্ক গ্রীষ্মকালীন সময়"}, {"Israel", "ইজরায়েল দিবালোক সময়"}, {"Japan", "জাপান দিবালোক সময়"}, {"Kamchatka", "পিত্রেপ্যাভলস্ক- ক্যামচ্যাটস্কি গৃষ্মকালীন সময়"}, {"Korea", "কোরিয়ান দিবালোক সময়"}, {"Krasnoyarsk", "ক্রাসনোয়ার্স্কি গ্রীষ্মকালীন সময়"}, {"Lord_Howe", "লর্ড হাওয়ে দিবালোক মসয়"}, {"Macau", "মাকাও গ্রীষ্মকাল সময়"}, {"Magadan", "ম্যাগাডান গ্রীষ্মকালীন সময়"}, {"Mauritius", "মরিশাস গ্রীষ্মকালীন সময়"}, {"Mexico_Pacific", "মেক্সিকান প্রশান্ত মহাসাগরীয় দিবালোক সময়"}, {"Mongolia", "উলান বাতোর গ্রীষ্মকালীন সময়"}, {"Moscow", "মস্কো গ্রীষ্মকালীন সময়"}, {"New_Caledonia", "নিউ ক্যালেডোনিয়া গ্রীষ্মকালীন সময়"}, {"New_Zealand", "নিউজিল্যান্ড দিবালোক সময়"}, {"Newfoundland", "নিউফাউন্ডল্যান্ড দিবালোক সময়"}, {"Norfolk", "নরফোক দ্বীপ দিবালোক সময়"}, {"Noronha", "ফার্নান্দো ডি নোরোনহা গ্রীষ্মকালীন সময়"}, {"Novosibirsk", "নোভোসিবির্স্ক গ্রীষ্মকালীন সময়"}, {"Omsk", "ওমস্ক গ্রীষ্মকালীন সময়"}, {"Pakistan", "পাকিস্তান গ্রীষ্মকালীন সময়"}, {"Paraguay", "প্যারাগুয়ে গ্রীষ্মকালীন সময়"}, {"Peru", "পেরু গ্রীষ্মকালীন সময়"}, {"Philippines", "ফিলিপাইন গ্রীষ্মকালীন সময়"}, {"Pierre_Miquelon", "সেন্ট পিয়ের ও মিকেলন দিবালোক সময়"}, {"Qyzylorda", "কিজিলোর্ডা গ্রীষ্মকাল সময়"}, {"Sakhalin", "সাখালিন গ্রীষ্মকালীন সময়"}, {"Samara", "সামারা গৃষ্মকালীন সময়"}, {"Samoa", "সামোয়া দিবালোক সময়"}, {"Taipei", "তাইপেই দিবালোক সময়"}, {"Tonga", "টোঙ্গা গ্রীষ্মকালীন সময়"}, {"Turkey", "তুর্কী গ্রীষ্মকাল সময়"}, {"Turkmenistan", "তুর্কমেনিস্তান গ্রীষ্মকালীন সময়"}, {"Uruguay", "উরুগুয়ে গ্রীষ্মকালীন সময়"}, {"Uzbekistan", "উজবেকিস্তান গ্রীষ্মকালীন সময়"}, {"Vanuatu", "ভানুয়াতু গ্রীষ্মকালীন সময়"}, {"Vladivostok", "ভ্লাদিভস্তক গ্রীষ্মকালীন সময়"}, {"Volgograd", "ভলগোগ্রাড গ্রীষ্মকালীন সময়"}, {"Yakutsk", "ইয়াকুটাস্ক গ্রীষ্মকালীন সময়"}, {"Yekaterinburg", "ইয়েকাতেরিনবুর্গ গ্রীষ্মকালীন সময়"}},
	},
	{
		{{"GMT", "مجال گرینویچ"}},
		{},
	},
	{
		{{"Afghanistan", "eur Afghanistan"}, {"Africa_Central", "eur Kreizafrika"}, {"Africa_Eastern", "eur Afrika ar Reter"}, {"Africa_Southern", "eur cʼhoañv Suafrika"}, {"Africa_Western", "eur Afrika ar Cʼhornôg"}, {"Alaska", "eur cʼhoañv Alaska"}, {"Almaty", "eur cʼhoañv Almaty"}, {"Amazon", "eur cʼhoañv an Amazon"}, {"America_Central", "eur cʼhoañv ar Cʼhreiz"}, {"America_Eastern", "eur cʼhoañv ar Reter"}, {"America_Mountain", "eur cʼhoañv ar Menezioù"}, {"America_Pacific", "eur cʼhoañv an Habask"}, {"Anadyr", "eur cʼhoañv Anadyrʼ"}, {"Apia", "eur cʼhoañv Samoa"}, {"Arabian", "eur cʼhoañv Arabia"}, {"Argentina", "eur cʼhoañv Arcʼhantina"}, {"Argentina_Western", "eur cʼhoañv Arcʼhantina ar Cʼhornôg"}, {"Armenia", "eur cʼhoañv Armenia"}, {"Atlantic", "eur cʼhoañv an Atlantel"}, {"Australia_Central", "eur cʼhoañv Kreizaostralia"}, {"Australia_CentralWestern", "eur cʼhoañv Kreizaostralia ar Cʼhornôg"}, {"Australia_Eastern", "eur cʼhoañv Aostralia ar Reter"}, {"Australia_Western", "eur cʼhoañv Aostralia ar Cʼhornôg"}, {"Azerbaijan", "eur cʼhoañv Azerbaidjan"}, {"Azores", "eur cʼhoañv an Azorez"}, {"Bangladesh", "eur cʼhoañv Bangladesh"}, {"Bhutan", "eur Bhoutan"}, {"Bolivia", "eur Bolivia"}, {"Brasilia", "eur cʼhoañv Brasília"}, {"Brunei", "eur Brunei"}, {"Cape_Verde", "eur cʼhoañv ar Cʼhab-Glas"}, {"Chamorro", "eur Chamorro"}, {"Chatham", "eur cʼhoañv Chatham"}, {"Chile", "eur cʼhoañv Chile"}, {"China", "eur cʼhoañv Sina"}, {"Christmas", "eur Enez Christmas"}, {"Cocos", "eur Inizi Kokoz"}, {"Colombia", "eur cʼhoañv Kolombia"}, {"Cook", "eur cʼhoañv Inizi Cook"}, {"Cuba", "eur cʼhoañv Kuba"}, {"Davis", "eur Davis"}, {"DumontDUrville", "eur Dumont-d’Urville"}, {"East_Timor", "eur Timor ar Reter"}, {"Easter", "eur cʼhoañv Enez Pask"}, {"Ecuador", "eur Ecuador"}, {"Europe_Central", "eur cʼhoañv Kreizeuropa"}, {"Europe_Eastern", "eur cʼhoañv Europa ar Reter"}, {"Europe_Further_Eastern", "eur Kaliningrad"}, {"Europe_Western", "eur cʼhoañv Europa ar Cʼhornôg"}, {"Falkland", "eur cʼhoañv Inizi Falkland"}, {"Fiji", "eur cʼhoañv Fidji"}, {"French_Guiana", "eur Gwiana cʼhall"}, {"French_Southern", "eur Douaroù aostral Frañs hag Antarktika"}, {"GMT", "amzer keitat Greenwich (AKG)"}, {"Galapagos", "eur Inizi Galápagos"}, {"Gambier", "eur Inizi Gambier"}, {"Georgia", "eur cʼhoañv Jorjia"}, {"Gilbert_Islands", "eur Inizi Gilbert"}, {"Greenland_Eastern", "eur cʼhoañv Greunland ar Reter"}, {"Greenland_Western", "eur cʼhoañv Greunland ar Cʼhornôg"}, {"Guam", "eur cʼhoañv Guam"}, {"Gulf", "eur cʼhoañv ar Pleg-mor Arab-ha-Pers"}, {"Guyana", "eur Guyana"}, {"Hawaii", "eur cʼhoañv Hawaii hag an Aleouted"}, {"Hawaii_Aleutian", "eur cʼhoañv Hawaii hag an Aleouted"}, {"Hong_Kong", "eur cʼhoañv Hong Kong"}, {"Hovd", "eur cʼhoañv Khovd"}, {"India", "eur cʼhoañv India"}, {"Indian_Ocean", "eur Meurvor Indez"}, {"Indochina", "eur Indez-Sina"}, {"Indonesia_Central", "eur Kreiz Indonezia"}, {"Indonesia_Eastern", "eur Indonezia ar Reter"}, {"Indonesia_Western", "eur Indonezia ar Cʼhornôg"}, {"Iran", "eur cʼhoañv Iran"}, {"Irkutsk", "eur cʼhoañv Irkutsk"}, {"Israel", "eur cʼhoañv Israel"}, {"Japan", "eur cʼhoañv Japan"}, {"Kazakhstan", "eur Kazakstan"}, {"Kazakhstan_Eastern", "eur Kazakstan ar Reter"}, {"Kazakhstan_Western", "eur Kazakstan ar Cʼhornôg"}, {"Korea", "eur cʼhoañv Korea"}, {"Kosrae", "eur Kosrae"}, {"Krasnoyarsk", "eur cʼhoañv Krasnoyarsk"}, {"Kyrgystan", "eur Kyrgyzstan"}, {"Lanka", "eur Sri Lanka"}, {"Line_Islands", "eur Line Islands"}, {"Lord_Howe", "eur cʼhoañv Lord Howe"}, {"Macau", "eur cʼhoañv Macau"}, {"Magadan", "eur cʼhoañv Magadan"}, {"Malaysia", "eur Malaysia"}, {"Maldives", "eur ar Maldivez"}, {"Marquesas", "eur Inizi Markiz"}, {"Marshall_Islands", "eur Inizi Marshall"}, {"Mauritius", "eur cʼhoañv Moris"}, {"Mawson", "eur Mawson"}, {"Mexico_Pacific", "eur cʼhoañv an Habask mecʼhikan"}, {"Mongolia", "eur cʼhoañv Ulaanbaatar"}, {"Moscow", "eur cʼhoañv Moskov"}, {"Myanmar", "eur Myanmar"}, {"Nauru", "eur Nauru"}, {"Nepal", "eur Nepal"}, {"New_Caledonia", "eur cʼhoañv Kaledonia-Nevez"}, {"New_Zealand", "eur cʼhoañv Zeland-Nevez"}, {"Newfoundland", "eur cʼhoañv Newfoundland"}, {"Niue", "eur Niue"}, {"Norfolk", "eur cʼhoañv Enez Norfolk"}, {"Noronha", "eur cʼhoañv Fernando de Noronha"}, {"North_Mariana", "eur Mariana an Norzh"}, {"Novosibirsk", "eur cʼhoañv Novosibirsk"}, {"Omsk", "eur cʼhoañv Omsk"}, {"Pakistan", "eur cʼhoañv Pakistan"}, {"Palau", "eur Palau"}, {"Papua_New_Guinea", "eur Papoua Ginea-Nevez"}, {"Paraguay", "eur cʼhoañv Paraguay"}, {"Peru", "eur cʼhoañv Perou"}, {"Philippines", "eur cʼhoañv ar Filipinez"}, {"Phoenix_Islands", "eur Inizi Phoenix"}, {"Pierre_Miquelon", "eur cʼhoañv Sant-Pêr-ha-Mikelon"}, {"Pitcairn", "eur Pitcairn"}, {"Ponape", "eur Pohnpei"}, {"Pyongyang", "eur Norzhkorea"}, {"Reunion", "eur ar Reünion"}, {"Rothera", "eur Rothera"}, {"Sakhalin", "eur cʼhoañv Sakhalin"}, {"Samoa", "eur cʼhoañv Samoa Amerikan"}, {"Seychelles", "eur Sechelez"}, {"Singapore", "eur cʼhoañv Singapour"}, {"Solomon", "eur Inizi Salomon"}, {"South_Georgia", "eur Georgia ar Su"}, {"Suriname", "eur Surinam"}, {"Syowa", "eur Syowa"}, {"Tahiti", "eur Tahiti"}, {"Taipei", "eur cʼhoañv Taiwan"}, {"Tajikistan", "eur Tadjikistan"}, {"Tokelau", "eur Tokelau"}, {"Tonga", "eur cʼhoañv Tonga"}, {"Truk", "eur Chuuk"}, {"Turkmenistan", "eur cʼhoañv Turkmenistan"}, {"Tuvalu", "eur Tuvalu"}, {"Uruguay", "eur cʼhoañv Uruguay"}, {"Uzbekistan", "eur cʼhoañv Ouzbekistan"}, {"Vanuatu", "eur cʼhoañv Vanuatu"}, {"Venezuela", "eur Venezuela"}, {"Vladivostok", "eur cʼhoañv Vladivostok"}, {"Volgograd", "eur cʼhoañv Volgograd"}, {"Vostok", "eur Vostok"}, {"Wake", "eur Enez Wake"}, {"Wallis", "eur Wallis-ha-Futuna"}, {"Yakutsk", "eur cʼhoañv Yakutsk"}, {"Yekaterinburg", "eur cʼhoañv Yekaterinbourg"}, {"Yukon", "eur Yukon"}},
		{{"Alaska", "eur hañv Alaska"}, {"Almaty", "eur hañv Almaty"}, {"Amazon", "eur hañv an Amazon"}, {"America_Central", "eur hañv ar Cʼhreiz"}, {"America_Eastern", "eur hañv ar Reter"}, {"America_Mountain", "eur hañv ar Menezioù"}, {"America_Pacific", "eur hañv an Habask"}, {"Anadyr", "eur hañv Anadyrʼ"}, {"Apia", "eur hañv Samoa"}, {"Arabian", "eur hañv Arabia"}, {"Argentina", "eur hañv Arcʼhantina"}, {"Argentina_Western", "eur hañv Arcʼhantina ar Cʼhornôg"}, {"Armenia", "eur hañv Armenia"}, {"Atlantic", "eur hañv an Atlantel"}, {"Australia_Central", "eur hañv Kreizaostralia"}, {"Australia_CentralWestern", "eur hañv Kreizaostralia ar Cʼhornôg"}, {"Australia_Eastern", "eur hañv Aostralia ar Reter"}, {"Australia_Western", "eur hañv Aostralia ar Cʼhornôg"}, {"Azerbaijan", "eur hañv Azerbaidjan"}, {"Azores", "eur hañv an Azorez"}, {"Bangladesh", "eur hañv Bangladesh"}, {"Brasilia", "eur hañv Brasília"}, {"Cape_Verde", "eur hañv ar Cʼhab-Glas"}, {"Chatham", "eur hañv Chatham"}, {"Chile", "eur hañv Chile"}, {"China", "eur hañv Sina"}, {"Colombia", "eur hañv Kolombia"}, {"Cook", "eur hañv Inizi Cook"}, {"Cuba", "eur hañv Kuba"}, {"Easter", "eur hañv Enez Pask"}, {"Europe_Central", "eur hañv Kreizeuropa"}, {"Europe_Eastern", "eur hañv Europa ar Reter"}, {"Europe_Western", "eur hañv Europa ar Cʼhornôg"}, {"Falkland", "eur hañv Inizi Falkland"}, {"Fiji", "eur hañv Fidji"}, {"Georgia", "eur hañv Jorjia"}, {"Greenland_Eastern", "eur hañv Greunland ar Reter"}, {"Greenland_Western", "eur hañv Greunland ar Cʼhornôg"}, {"Hawaii_Aleutian", "eur hañv Hawaii hag an Aleouted"}, {"Hong_Kong", "eur hañv Hong Kong"}, {"Hovd", "eur hañv Khovd"}, {"Iran", "eur hañv Iran"}, {"Irkutsk", "eur hañv Irkutsk"}, {"Israel", "eur hañv Israel"}, {"Japan", "eur hañv Japan"}, {"Korea", "eur hañv Korea"}, {"Krasnoyarsk", "eur hañv Krasnoyarsk"}, {"Lord_Howe", "eur hañv Lord Howe"}, {"Macau", "eur hañv Macau"}, {"Magadan", "eur hañv Magadan"}, {"Mauritius", "eur hañv Moris"}, {"Mexico_Pacific", "eur hañv an Habask mecʼhikan"}, {"Mongolia", "eur hañv Ulaanbaatar"}, {"Moscow", "eur hañv Moskov"}, {"New_Caledonia", "eur hañv Kaledonia-Nevez"}, {"New_Zealand", "eur hañv Zeland-Nevez"}, {"Newfoundland", "eur hañv Newfoundland"}, {"Norfolk", "eur hañv Enez Norfolk"}, {"Noronha", "eur hañv Fernando de Noronha"}, {"Novosibirsk", "eur hañv Novosibirsk"}, {"Omsk", "eur hañv Omsk"}, {"Pakistan", "eur hañv Pakistan"}, {"Paraguay", "eur hañv Paraguay"}, {"Peru", "eur hañv Perou"}, {"Philippines", "eur hañv ar Filipinez"}, {"Pierre_Miquelon", "eur hañv Sant-Pêr-ha-Mikelon"}, {"Sakhalin", "eur hañv Sakhalin"}, {"Samoa", "eur hañv Samoa Amerikan"}, {"Taipei", "eur hañv Taiwan"}, {"Tonga", "eur hañv Tonga"}, {"Turkmenistan", "eur hañv Turkmenistan"}, {"Uruguay", "eur hañv Uruguay"}, {"Uzbekistan", "eur hañv Ouzbekistan"}, {"Vanuatu", "eur hañv Vanuatu"}, {"Vladivostok", "eur hañv Vladivostok"}, {"Volgograd", "eur hañv Volgograd"}, {"Yakutsk", "eur hañv Yakutsk"}, {"Yekaterinburg", "eur hañv Yekaterinbourg"}},
	},
	{
		{{"Acre", "आकर स्टैंडर्ड टाईम"}, {"Afghanistan", "आफगानीस्तान सम"}, {"Africa_Central", "मिरु आफ्रिका सम"}, {"Africa_Eastern", "सानजा आफ्रिका सम"}, {"Africa_Southern", "खोला आफ्रिकानि थाखोआरि सम"}, {"Africa_Western", "सोनाब आफ्रिकानि सम"}, {"Alaska", "आलास्का स्टेन्डार्ड सम"}, {"Almaty", "अलमाटी स्टैंडर्ड टाईम"}, {"Amazon", "आमाजन थाखोआऱि सम"}, {"America_Central", "साहा आमेरिकानि मिरु मानथाखोआरि सम"}, {"America_Eastern", "साहा आमेरिकानि सानजायारि थाखोआरि सम"}, {"America_Mountain", "साहा आमेरिकानि हाजोमायारि थाखोआरि सम"}, {"America_Pacific", "साहा आमेरिकानि पेसिफिक थाखोआरि सम"}, {"Anadyr", "अनादीर स्टैंडर्ड टाईम"}, {"Apia", "आपिया थाखोआरि सम"}, {"Aqtau", "अक़्टाऊ स्टैंडर्ड टाईम"}, {"Aqtobe", "अक़्टोबे स्टैंडर्ड टाईम"}, {"Arabian", "आराबीयान थाखोआरि सम"}, {"Argentina", "आर्जेन्टिना थाखोआरि सम"}, {"Argentina_Western", "सोनाब आर्जेन्टिना थाखोआरि सम"}, {"Armenia", "आर्मेनिया थाखोआरि सम"}, {"Atlantic", "आटलान्टिक थाखोआरि सम"}, {"Australia_Central", "अ’स्ट्रेलियानि मिरु थाखोआरि सम"}, {"Australia_CentralWestern", "अस्ट्रेलियानि मिरु सोनाबारि थाखोआरि सम"}, {"Australia_Eastern", "अस्ट्रेलियानि सानजायारि थाखोआरि सम"}, {"Australia_Western", "अस्ट्रेलियानि सोनाबारि थाखोआरि सम"}, {"Azerbaijan", "आजारबाईजान थाखोआरि सम"}, {"Azores", "आज’र्सनि थाखोआरि सम"}, {"Bangladesh", "बांलादेश थाखोआरि सम"}, {"Bhutan", "भुटान सम"}, {"Bolivia", "बलिभिया सम"}, {"Brasilia", "ब्राजीलिया थाखोआरि सम"}, {"Brunei", "ब्रुनेई दर उस सलाम स्टैंडर्ड टाईम"}, {"Cape_Verde", "केप भेर्दे थाखोआरि सम"}, {"Chamorro", "चाम’र्र’ थाखोआरि सम"}, {"Chatham", "चेथाम थाखोआरि सम"}, {"Chile", "चीले थाखोआरि सम"}, {"China", "चाईना थाखोआरि सम"}, {"Christmas", "ख्रीस्टमास द्वीप सम"}, {"Cocos", "कक’स द्वीपफोरनि सम"}, {"Colombia", "कलम्बिया थाखोआरि सम"}, {"Cook", "कुक द्वीपफोरनि थाखोआरि सम"}, {"Cuba", "किउबा थाखोआरि सम"}, {"Davis", "डेभिस सम"}, {"DumontDUrville", "डुमन्त-डीआर्भील सम"}, {"East_Timor", "ईस्ट टीमोर स्टैंडर्ड टाईम"}, {"Easter", "सानजायारि द्वीप थाखोआरि सम"}, {"Ecuador", "एकुवाडर सम"}, {"Europe_Central", "मिरु युर’पनि थाखोआरि सम"}, {"Europe_Eastern", "सानजायारि युर’पनि थाखोआरि सम"}, {"Europe_Further_Eastern", "गोजानसिन-सानजायारि युर’पनि सम"}, {"Europe_Western", "सोनाब युर’पनि थाखोआरि सम"}, {"Falkland", "फकलेण्ड द्वीपफोरनि थाखोआरि सम"}, {"Fiji", "फीजी थाखोआरि सम"}, {"French_Guiana", "फ्रेन्च गुईयाना सम"}, {"French_Southern", "फ्रेन्च खोलायारि आरो एन्टार्कटिक सम"}, {"GMT", "ग्रिनवीच गेजेरारि सम"}, {"Galapagos", "गालापाग’स सम"}, {"Gambier", "गाम्बियेर सम"}, {"Georgia", "जर्जिया थाखोआरि सम"}, {"Gilbert_Islands", "गीलबार्ट द्वीपफोरनि सम"}, {"Greenland_Eastern", "सानजा ग्रीनलेण्ड थाखोआरि सम"}, {"Greenland_Western", "सोनाब ग्रीनलेण्ड थाखोआरि सम"}, {"Guam", "गुआम स्टैंडर्ड टाईम"}, {"Gulf", "गाल्फ थाखोआरि सम"}, {"Guyana", "गुयाना सम"}, {"Hawaii", "हावाई-एल्युतियान थाखोआरि सम"}, {"Hawaii_Aleutian", "हावाई-एल्युतियान थाखोआरि सम"}, {"Hong_Kong", "हंकं थाखोआरि सम"}, {"Hovd", "ह’भ्द थाखोआरि सम"}, {"India", "भारतारि थाखोआरि सम"}, {"Indian_Ocean", "भारत लैथोमायारि सम"}, {"Indochina", "ईंडो चइना स्टैंडर्ड टाईम"}, {"Indonesia_Central", "ईंडोनीशिया स्टैंडर्ड टाईम"}, {"Indonesia_Eastern", "ईस्टर्न ईंडोनीशिया स्टैंडर्ड टाईम"}, {"Indonesia_Western", "वेस्टर्न ईंडोनीशिया स्टैंडर्ड टाईम"}, {"Iran", "इरान थाखोआरि सम"}, {"Irkutsk", "ईर्कुत्स्क थाखोआरि सम"}, {"Israel", "इज्राईल थाखोआरि सम"}, {"Japan", "जापान थाखोआरि सम"}, {"Kamchatka", "पेत्रोपावलोस्क कामचटका स्टैंडर्ड टाईम"}, {"Kazakhstan_Eastern", "सानजा काजाखस्तान सम"}, {"Kazakhstan_Western", "सोनाब काजाखस्तान सम"}, {"Korea", "क’रिया थाखोआरि सम"}, {"Kosrae", "कस्रे सम"}, {"Krasnoyarsk", "क्रास्न’यार्स्क थाखोआरि सम"}, {"Kyrgystan", "कीर्गीस्तान सम"}, {"Lanka", "लंका स्टैंडर्ड टाईम"}, {"Line_Islands", "लाईन द्वीपफोरनि सम"}, {"Lord_Howe", "लर्ड हावि थाखोआरि सम"}, {"Macau", "माकाऊ स्टैंडर्ड टाईम"}, {"Magadan", "मागादान थाखोआरि सम"}, {"Malaysia", "मलेशिया स्टैंडर्ड टाईम"}, {"Maldives", "मालदीभ्स सम"}, {"Marquesas", "मार्केसास सम"}, {"Marshall_Islands", "मार्शेल द्वीपफोरनि सम"}, {"Mauritius", "म’रिशियासनि थाखोआरि सम"}, {"Mawson", "मौसन सम"}, {"Mexico_Pacific", "मेक्सिक’नि पेसिफिक थाखोआरि सम"}, {"Mongolia", "उलानबातार थाखोआरि सम"}, {"Moscow", "मस्कौ थाखोआरि सम"}, {"Myanmar", "म्यानमार स्टैंडर्ड टाईम"}, {"Nauru", "नाऊरु सम"}, {"Nepal", "नेपाल सम"}, {"New_Caledonia", "गोदान केलेडनिया थाखोआरि सम"}, {"New_Zealand", "निउजिलेण्ड थाखोआरि सम"}, {"Newfoundland", "निउफाउन्दलेन्द थाखोआरि सम"}, {"Niue", "नीऊए सम"}, {"Norfolk", "नरफ’क द्वीप थाखोआरि सम"}, {"Noronha", "फेरनान्द’ दे नर’न्हा थाखोआरि सम"}, {"North_Mariana", "नॉर्थ मारिआना स्टैंडर्ड टाईम"}, {"Novosibirsk", "नभ’सिबिर्स्क थाखोआरि सम"}, {"Omsk", "अम्स्क थाखोआरि सम"}, {"Pakistan", "पाकिस्तान थाखोआरि सम"}, {"Palau", "पालाउ सम"}, {"Papua_New_Guinea", "पापुआ निउ गिनी सम"}, {"Paraguay", "पारागुवे थाखोआरि सम"}, {"Peru", "पेरु थाखोआरि सम"}, {"Philippines", "फीलीपीन्स स्टैंडर्ड टाईम"}, {"Phoenix_Islands", "फनीक्स द्वीपफोरनि सम"}, {"Pierre_Miquelon", "सैन्ट पियेर आरो मिक्वेलन थाखोआरि सम"}, {"Pitcairn", "पीटकैर्न सम"}, {"Ponape", "पनापे सम"}, {"Pyongyang", "प्यंयां सम"}, {"Qyzylorda", "क़ीज़ीलोर्डा स्टैंडर्ड टाईम"}, {"Reunion", "रियूनियन सम"}, {"Rothera", "रथेरा सम"}, {"Sakhalin", "साखालिन थाखोआरि सम"}, {"Samara", "समारा स्टैंडर्ड टाईम"}, {"Samoa", "साम’वा थाखोआरि सम"}, {"Seychelles", "सेशेल्स सम"}, {"Singapore", "सींगापुर स्टैंडर्ड टाईम"}, {"Solomon", "सल’मन द्वीपफोरनि सम"}, {"South_Georgia", "खोला जर्जिया सम"}, {"Suriname", "सुरीनाम सम"}, {"Syowa", "सीअवा सम"}, {"Tahiti", "टाहिटी सम"}, {"Taipei", "ताईपेइ थाखोआरि सम"}, {"Tajikistan", "ताजिकिस्तान सम"}, {"Tokelau", "टकेलौ सम"}, {"Tonga", "टंगा थाखोआरि सम"}, {"Truk", "चूक सम"}, {"Turkey", "तुर्की स्टैंडर्ड टाईम"}, {"Turkmenistan", "तुर्कमेनीस्तान थाखोआरि सम"}, {"Tuvalu", "तुभालु सम"}, {"Uruguay", "उरुगुवे थाखोआरि सम"}, {"Uzbekistan", "उजबेकिस्तान थाखोआरि सम"}, {"Vanuatu", "वानुआटु थाखोआरि सम"}, {"Venezuela", "भेनेजुवेला सम"}, {"Vladivostok", "भ्लादिभस्त’क थाखोआरि सम"}, {"Volgograd", "भल्ग’ग्रेद थाखोआरि सम"}, {"Vostok", "भस्त’क सम"}, {"Wake", "वैक द्वीप सम"}, {"Wallis", "वालीस आरो फुतुना सम"}, {"Yakutsk", "याकुत्स्क थाखोआरि सम"}, {"Yekaterinburg", "येकातेरीनाबुर्ग थाखोआरि सम"}, {"Yukon", "यूकन सम"}},
		{{"Acre", "आकर समर टाईम"}, {"Alaska", "आलास्कानि सानारि सम"}, {"Almaty", "अलमाटी समर टाईम"}, {"Amazon", "आमाजन दैज्लां सम"}, {"America_Central", "साहा आमेरिकानि मिरु सानारि सम"}, {"America_Eastern", "साहा आमेरिकानि सानजायारि सानारि सम"}, {"America_Mountain", "साहा आमोरिकानि हाजोमायारि सानारि सम"}, {"America_Pacific", "साहा आमेरिकानि पेसिफिक सानारि सम"}, {"Anadyr", "अनादीर समर टाईम"}, {"Apia", "आपिया सान सम"}, {"Aqtau", "अक़्टाऊ समर टाईम"}, {"Aqtobe", "अक़्टोबे समर टाईम"}, {"Arabian", "आराबीयान सानारि सम"}, {"Argentina", "आर्जेन्टिना दैज्लां सम"}, {"Argentina_Western", "सोनाब आर्जेन्टिना दैज्लां सम"}, {"Armenia", "आर्मेनिया दैज्लां सम"}, {"Atlantic", "आटलान्टिक सानारि सम"}, {"Australia_Central", "अ’स्ट्रेलिया मिरु सानारि सम"}, {"Australia_CentralWestern", "अस्ट्रेलियानि मिरु सोनाबारि सानारि सम"}, {"Australia_Eastern", "अस्ट्रेलियानि सानजायारि सानारि सम"}, {"Australia_Western", "अस्ट्रेलियानि सोनाबारि सानारि सम"}, {"Azerbaijan", "आजारबाईजान दैज्लां सम"}, {"Azores", "आज’र्सनि दैज्लां सम"}, {"Bangladesh", "बांलादेश दैज्लां सम"}, {"Brasilia", "ब्राजीलिया दैज्लां सम"}, {"Cape_Verde", "केप भेर्दे दैज्लां सम"}, {"Chatham", "चेथाम सानारि सम"}, {"Chile", "चीले दैज्लां सम"}, {"China", "चाईना सानारि सम"}, {"Colombia", "कलम्बिया दैज्लां सम"}, {"Cook", "कुक द्वीपफोरनि खावसे दैज्लां सम"}, {"Cuba", "किउबा सानारि सम"}, {"Easter", "सानजायारि द्वीप दैज्लां सम"}, {"Europe_Central", "मिरु युर’पनि दैज्लां सम"}, {"Europe_Eastern", "सानजायारि युर’पनि दैज्लां सम"}, {"Europe_Western", "सोनाब युर’पनि दैज्लां सम"}, {"Falkland", "फकलेण्ड द्वीपफोरनि दैज्लां सम"}, {"Fiji", "फीजी दैज्लां सम"}, {"Georgia", "जर्जिया दैज्लां सम"}, {"Greenland_Eastern", "सानडजा ग्रीनलेण्ड दैज्लां सम"}, {"Greenland_Western", "सेनाब ग्रीनलेण्ड दैज्लां सम"}, {"Hawaii_Aleutian", "हावाई-एल्युतियान सानारि सम"}, {"Hong_Kong", "हंकं दैज्लां सम"}, {"Hovd", "ह’भ्द दैज्लां सम"}, {"Iran", "इरान सानारि सम"}, {"Irkutsk", "ईर्कुत्स्क दैज्लांं सम"}, {"Israel", "इज्राईल सानारि सम"}, {"Japan", "जापान सानारि सम"}, {"Kamchatka", "पेत्रोपावलोस्क कामचटका समर टाईम"}, {"Korea", "क’रिया सानारि सम"}, {"Krasnoyarsk", "क्रास्न’यार्स्क दैज्लांं सम"}, {"Lord_Howe", "लर्ड हावि सानारि सम"}, {"Macau", "माकाऊ समर टाईम"}, {"Magadan", "मागादान दैज्लां सम"}, {"Mauritius", "म’रिशियासनि दैज्लां सम"}, {"Mexico_Pacific", "मेक्सिक’नि पेसिफिक सानारि सम"}, {"Mongolia", "उलानबातार दैज्लां सम"}, {"Moscow", "मस्कौ दैज्लां सम"}, {"New_Caledonia", "गोदान केलेडनिया दैज्लां सम"}, {"New_Zealand", "निउजिलेण्ड सानारि सम"}, {"Newfoundland", "निउफाउन्दलेन्द सानारि सम"}, {"Norfolk", "नरफ’क द्वीप सानारि सम"}, {"Noronha", "फेरनान्द’ दे नर’न्हा दैज्लां सम"}, {"Novosibirsk", "नभ’सिबिर्स्क दैज्लां सम"}, {"Omsk", "अम्स्क दैज्लां सम"}, {"Pakistan", "पाकिस्तान दैज्लां सम"}, {"Paraguay", "पारागुवे दैज्लां सम"}, {"Peru", "पेरु दैज्लां सम"}, {"Philippines", "फीलीपीन्स समर टाईम"}, {"Pierre_Miquelon", "सैन्ट पियेर आरो मिक्वेलन सानारि सम"}, {"Qyzylorda", "क़ीज़ीलोर्डा समर टाईम"}, {"Sakhalin", "साखालिन दैज्लां सम"}, {"Samara", "समारा समर टाईम"}, {"Samoa", "साम’वा सानारि सम"}, {"Taipei", "ताईपेइ सानारि सम"}, {"Tonga", "टंगा दैज्लां सम"}, {"Turkey", "तुर्की समर टाईम"}, {"Turkmenistan", "तुर्कमेनीस्तान दैज्लां सम"}, {"Uruguay", "उरुगुवे दैज्लां सम"}, {"Uzbekistan", "उजबेकिस्तान दैज्लां सम"}, {"Vanuatu", "वानुआटु दैज्लां सम"}, {"Vladivostok", "भ्लादिभस्त’क दैज्लां सम"}, {"Volgograd", "भल्ग’ग्रेद दैज्लां सम"}, {"Yakutsk", "याकुत्स्क दैज्लां सम"}, {"Yekaterinburg", "येकातेरीनाबुर्ग दैज्लां सम"}},
	},
	{
		{{"Acre", "Acre standardno vreme"}, {"Afghanistan", "Afganistansko vrijeme"}, {"Africa_Central", "Centralnoafričko vrijeme"}, {"Africa_Eastern", "Istočnoafričko vrijeme"}, {"Africa_Southern", "Južnoafričko standardno vrijeme"}, {"Africa_Western", "Zapadnoafričko vrijeme"}, {"Alaska", "Aljaskansko standardno vrijeme"}, {"Almaty", "Almatu standardno vreme"}, {"Amazon", "Amazonsko standardno vrijeme"}, {"America_Central", "Sjevernoameričko centralno standardno vrijeme"}, {"America_Eastern", "Sjevernoameričko istočno standardno vrijeme"}, {"America_Mountain", "Sjevernoameričko planinsko standardno vrijeme"}, {"America_Pacific", "Sjevernoameričko pacifičko standardno vrijeme"}, {"Anadyr", "Anadir standardno vreme"}, {"Apia", "Apijsko standardno vrijeme"}, {"Aqtau", "Akvtau standardno vreme"}, {"Aqtobe", "Akvtobe standardno vreme"}, {"Arabian", "Arabijsko standardno vrijeme"}, {"Argentina", "Argentinsko standardno vrijeme"}, {"Argentina_Western", "Zapadnoargentinsko standardno vrijeme"}, {"Armenia", "Armensko standardno vrijeme"}, {"Atlantic", "Sjevernoameričko atlantsko standardno vrijeme"}, {"Australia_Central", "Centralnoaustralijsko standardno vrijeme"}, {"Australia_CentralWestern", "Australijsko centralnozapadno standardno vrijeme"}, {"Australia_Eastern", "Istočnoaustralijsko standardno vrijeme"}, {"Australia_Western", "Zapadnoaustralijsko standardno vrijeme"}, {"Azerbaijan", "Azerbejdžansko standardno vrijeme"}, {"Azores", "Azorsko standardno vrijeme"}, {"Bangladesh", "Bangladeško standardno vrijeme"}, {"Bhutan", "Butansko vrijeme"}, {"Bolivia", "Bolivijsko vrijeme"}, {"Brasilia", "Brazilijsko standardno vrijeme"}, {"Brunei", "Brunejsko vrijeme"}, {"Cape_Verde", "Zelenortsko standardno vrijeme"}, {"Chamorro", "Čamorsko standardno vrijeme"}, {"Chatham", "Čatamsko standardno vrijeme"}, {"Chile", "Čileansko standardno vrijeme"}, {"China", "Kinesko standardno vrijeme"}, {"Christmas", "Vrijeme na Božićnom Ostrvu"}, {"Cocos", "Vrijeme na Ostrvima Kokos"}, {"Colombia", "Kolumbijsko standardno vrijeme"}, {"Cook", "Standardno vrijeme na Kukovim ostrvima"}, {"Cuba", "Kubansko standardno vrijeme"}, {"Davis", "Vrijeme stanice Davis"}, {"DumontDUrville", "Vrijeme stanice Dumont-d’Urville"}, {"East_Timor", "Istočnotimorsko vrijeme"}, {"Easter", "Uskršnjeostrvsko standardno vrijeme"}, {"Ecuador", "Ekvadorsko vrijeme"}, {"Europe_Central", "Centralnoevropsko standardno vrijeme"}, {"Europe_Eastern", "Istočnoevropsko standardno vrijeme"}, {"Europe_Further_Eastern", "Dalekoistočnoevropsko vrijeme"}, {"Europe_Western", "Zapadnoevropsko standardno vrijeme"}, {"Falkland", "Folklandsko standardno vrijeme"}, {"Fiji", "Standardno vrijeme na Fidžiju"}, {"French_Guiana", "Francuskogvajansko vrijeme"}, {"French_Southern", "Vrijeme na Francuskoj Južnoj Teritoriji i Antarktiku"}, {"GMT", "Griničko vrijeme"}, {"Galapagos", "Galapagosko vrijeme"}, {"Gambier", "Gambijersko vrijeme"}, {"Georgia", "Gruzijsko standardno vrijeme"}, {"Gilbert_Islands", "Vrijeme na Gilbertovim ostrvima"}, {"Greenland_Eastern", "Istočnogrenlandsko standardno vrijeme"}, {"Greenland_Western", "Zapadnogrenlandsko standardno vrijeme"}, {"Guam", "Guam standardno vreme"}, {"Gulf", "Zalivsko standardno vrijeme"}, {"Guyana", "Gvajansko vrijeme"}, {"Hawaii", "Havajsko-aleućansko standardno vrijeme"}, {"Hawaii_Aleutian", "Havajsko-aleućansko standardno vrijeme"}, {"Hong_Kong", "Hongkonško standardno vrijeme"}, {"Hovd", "Hovdsko standardno vrijeme"}, {"India", "Indijsko standardno vrijeme"}, {"Indian_Ocean", "Vrijeme na Indijskom okeanu"}, {"Indochina", "Indokinesko vrijeme"}, {"Indonesia_Central", "Centralnoindonezijsko vrijeme"}, {"Indonesia_Eastern", "Istočnoindonezijsko vrijeme"}, {"Indonesia_Western", "Zapadnoindonezijsko vrijeme"}, {"Iran", "Iransko standardno vrijeme"}, {"Irkutsk", "Irkutsko standardno vrijeme"}, {"Israel", "Izraelsko standardno vrijeme"}, {"Japan", "Japansko standardno vrijeme"}, {"Kamchatka", "Petropavlovsk-Kamčatski standardno vreme"}, {"Kazakhstan", "kazahstansko vrijeme"}, {"Kazakhstan_Eastern", "istočnokazahstansko vrijeme"}, {"Kazakhstan_Western", "zapadnokazahstansko vrijeme"}, {"Korea", "Korejsko standardno vrijeme"}, {"Kosrae", "Vrijeme na Ostrvu Kosrae"}, {"Krasnoyarsk", "Krasnojarsko standardno vrijeme"}, {"Kyrgystan", "kirgistansko vrijeme"}, {"Lanka", "Lanka vreme"}, {"Line_Islands", "Vrijeme na Ostrvima Lajn"}, {"Lord_Howe", "Standardno vrijeme na Ostrvu Lord Hau"}, {"Macau", "Makao standardno vreme"}, {"Magadan", "Magadansko standardno vrijeme"}, {"Malaysia", "Malezijsko vrijeme"}, {"Maldives", "Maldivsko vrijeme"}, {"Marquesas", "Vrijeme na Ostrvima Markiz"}, {"Marshall_Islands", "Vrijeme na Maršalovim ostrvima"}, {"Mauritius", "Mauricijsko standardno vrijeme"}, {"Mawson", "Vrijeme stanice Mawson"}, {"Mexico_Pacific", "Meksičko pacifičko standardno vrijeme"}, {"Mongolia", "Ulanbatorsko standardno vrijeme"}, {"Moscow", "Moskovsko standardno vrijeme"}, {"Myanmar", "Mijanmarsko vrijeme"}, {"Nauru", "Vrijeme na Ostrvu Nauru"}, {"Nepal", "Nepalsko vrijeme"}, {"New_Caledonia", "Novokaledonijsko standardno vrijeme"}, {"New_Zealand", "Novozelandsko standardno vrijeme"}, {"Newfoundland", "Njufaundlendsko standardno vrijeme"}, {"Niue", "Vrijeme na Ostrvu Niue"}, {"Norfolk", "Norfolško standardno vrijeme"}, {"Noronha", "Standardno vrijeme na ostrvu Fernando di Noronja"}, {"North_Mariana", "Severna Marijanska Ostrva vreme"}, {"Novosibirsk", "Novosibirsko standardno vrijeme"}, {"Omsk", "Omsko standardno vrijeme"}, {"Pakistan", "Pakistansko standardno vrijeme"}, {"Palau", "Vrijeme na Ostrvu Palau"}, {"Papua_New_Guinea", "Vrijeme na Papui Novoj Gvineji"}, {"Paraguay", "Paragvajsko standardno vrijeme"}, {"Peru", "Peruansko standardno vrijeme"}, {"Philippines", "Filipinsko standardno vrijeme"}, {"Phoenix_Islands", "Vrijeme na Ostrvima Finiks"}, {"Pierre_Miquelon", "Standardno vrijeme na Ostrvima Sveti Petar i Mikelon"}, {"Pitcairn", "Vrijeme na Ostrvima Pitkern"}, {"Ponape", "Vrijeme na Ostrvu Ponape"}, {"Pyongyang", "Pjongjanško vrijeme"}, {"Qyzylorda", "Kizilorda standardno vreme"}, {"Reunion", "Reunionsko vrijeme"}, {"Rothera", "Vrijeme stanice Rothera"}, {"Sakhalin", "Sahalinsko standardno vrijeme"}, {"Samara", "Samara standardno vreme"}, {"Samoa", "Samoansko standardno vrijeme"}, {"Seychelles", "Sejšelsko vrijeme"}, {"Singapore", "Singapursko standardno vrijeme"}, {"Solomon", "Vrijeme na Solomonskim ostrvima"}, {"South_Georgia", "Južnodžordžijsko vrijeme"}, {"Suriname", "Surinamsko vrijeme"}, {"Syowa", "Vrijeme stanice Syowa"}, {"Tahiti", "Tahićansko vrijeme"}, {"Taipei", "Tajpejsko standardno vrijeme"}, {"Tajikistan", "tadžikistansko vrijeme"}, {"Tokelau", "Vrijeme na Ostrvu Tokelau"}, {"Tonga", "Tongansko standardno vrijeme"}, {"Truk", "Čučko vrijeme"}, {"Turkey", "Turska standardno vreme"}, {"Turkmenistan", "turkmenistansko standardno vrijeme"}, {"Tuvalu", "Tuvaluansko vrijeme"}, {"Uruguay", "Urugvajsko standardno vrijeme"}, {"Uzbekistan", "uzbekistansko standardno vrijeme"}, {"Vanuatu", "Vanuatuansko standardno vrijeme"}, {"Venezuela", "Venecuelansko vrijeme"}, {"Vladivostok", "Vladivostočko standardno vrijeme"}, {"Volgograd", "Volgogradsko standardno vrijeme"}, {"Vostok", "Vrijeme stanice Vostok"}, {"Wake", "Vrijeme na Ostrvu Vejk"}, {"Wallis", "Vrijeme na Ostrvima Valis i Futuna"}, {"Yakutsk", "Jakutsko standardno vrijeme"}, {"Yekaterinburg", "Jekaterinburško standardno vrijeme"}, {"Yukon", "Jukonsko vrijeme"}},
		{{"Acre", "Acre letnje računanje vremena"}, {"Alaska", "Aljaskansko ljetno vrijeme"}, {"Almaty", "Almatu letnje računanje vremena"}, {"Amazon", "Amazonsko ljetno vrijeme"}, {"America_Central", "Sjevernoameričko centralno ljetno vrijeme"}, {"America_Eastern", "Sjevernoameričko istočno ljetno vrijeme"}, {"America_Mountain", "Sjevernoameričko planinsko ljetno vrijeme"}, {"America_Pacific", "Sjevernoameričko pacifičko ljetno vrijeme"}, {"Anadyr", "Anadir letnje računanje vremena"}, {"Apia", "Apijsko ljetno vrijeme"}, {"Aqtau", "Akvtau letnje računanje vremena"}, {"Aqtobe", "Akvtobe letnje računanje vremena"}, {"Arabian", "Arabijsko ljetno vrijeme"}, {"Argentina", "Argentinsko ljetno vrijeme"}, {"Argentina_Western", "Zapadnoargentinsko ljetno vrijeme"}, {"Armenia", "Armensko ljetno vrijeme"}, {"Atlantic", "Sjevernoameričko atlantsko ljetno vrijeme"}, {"Australia_Central", "Centralnoaustralijsko ljetno vrijeme"}, {"Australia_CentralWestern", "Australijsko centralnozapadno ljetno vrijeme"}, {"Australia_Eastern", "Istočnoaustralijsko ljetno vrijeme"}, {"Australia_Western", "Zapadnoaustralijsko ljetno vrijeme"}, {"Azerbaijan", "Azerbejdžansko ljetno vrijeme"}, {"Azores", "Azorsko ljetno vrijeme"}, {"Bangladesh", "Bangladeško ljetno vrijeme"}, {"Brasilia", "Brazilijsko ljetno vrijeme"}, {"Cape_Verde", "Zelenortsko ljetno vrijeme"}, {"Chatham", "Čatamsko ljetno vrijeme"}, {"Chile", "Čileansko ljetno vrijeme"}, {"China", "Kinesko ljetno vrijeme"}, {"Colombia", "Kolumbijsko ljetno vrijeme"}, {"Cook", "Poluljetno vrijeme na Kukovim ostrvima"}, {"Cuba", "Kubansko ljetno vrijeme"}, {"Easter", "Uskršnjeostrvsko ljetno vrijeme"}, {"Europe_Central", "Centralnoevropsko ljetno vrijeme"}, {"Europe_Eastern", "Istočnoevropsko ljetno vrijeme"}, {"Europe_Western", "Zapadnoevropsko ljetno vrijeme"}, {"Falkland", "Folklandsko ljetno vrijeme"}, {"Fiji", "Fidžijsko ljetno vrijeme"}, {"Georgia", "Gruzijsko ljetno vrijeme"}, {"Greenland_Eastern", "Istočnogrenlandsko ljetno vrijeme"}, {"Greenland_Western", "Zapadnogrenlandsko ljetno vrijeme"}, {"Hawaii_Aleutian", "Havajsko-aleućansko ljetno vrijeme"}, {"Hong_Kong", "Hongkonško ljetno vrijeme"}, {"Hovd", "Hovdsko ljetno vrijeme"}, {"Iran", "Iransko ljetno vrijeme"}, {"Irkutsk", "Irkutsko ljetno vrijeme"}, {"Israel", "Izraelsko ljetno vrijeme"}, {"Japan", "Japansko ljetno vrijeme"}, {"Kamchatka", "Petropavlovsk-Kamčatski letnje računanje vremena"}, {"Korea", "Korejsko ljetno vrijeme"}, {"Krasnoyarsk", "Krasnojarsko ljetno vrijeme"}, {"Lord_Howe", "Ljetno vrijeme na Ostrvu Lord Hau"}, {"Macau", "Makao letnje računanje vremena"}, {"Magadan", "Magadansko ljetno vrijeme"}, {"Mauritius", "Mauricijsko ljetno vrijeme"}, {"Mexico_Pacific", "Meksičko pacifičko ljetno vrijeme"}, {"Mongolia", "Ulanbatorsko ljetno vrijeme"}, {"Moscow", "Moskovsko ljetno vrijeme"}, {"New_Caledonia", "Novokaledonijsko ljetno vrijeme"}, {"New_Zealand", "Novozelandsko ljetno vrijeme"}, {"Newfoundland", "Njufaundlendsko ljetno vrijeme"}, {"Norfolk", "Norfolško ljetno vrijeme"}, {"Noronha", "Ljetno vrijeme na ostrvu Fernando di Noronja"}, {"Novosibirsk", "Novosibirsko ljetno vrijeme"}, {"Omsk", "Omsko ljetno vrijeme"}, {"Pakistan", "Pakistansko ljetno vrijeme"}, {"Paraguay", "Paragvajsko ljetno vrijeme"}, {"Peru", "Peruansko ljetno vrijeme"}, {"Philippines", "Filipinsko ljetno vrijeme"}, {"Pierre_Miquelon", "Ljetno vrijeme na Ostrvima Sveti Petar i Mikelon"}, {"Qyzylorda", "Kizilorda letnje računanje vremena"}, {"Sakhalin", "Sahalinsko ljetno vrijeme"}, {"Samara", "Samara letnje računanje vremena"}, {"Samoa", "Samoansko ljetno vrijeme"}, {"Taipei", "Tajpejsko ljetno vrijeme"}, {"Tonga", "Tongansko ljetno vrijeme"}, {"Turkey", "Turska letnje računanje vremena"}, {"Turkmenistan", "turkmenistansko ljetno vrijeme"}, {"Uruguay", "Urugvajsko ljetno vrijeme"}, {"Uzbekistan", "uzbekistansko ljetno vrijeme"}, {"Vanuatu", "Vanuatuansko ljetno vrijeme"}, {"Vladivostok", "Vladivostočko ljetno vrijeme"}, {"Volgograd", "Volgogradsko ljetno vrijeme"}, {"Yakutsk", "Jakutsko ljetno vrijeme"}, {"Yekaterinburg", "Jekaterinburško ljetno vrijeme"}},
	},
	{
		{{"Acre", "Акре стандардно време"}, {"Afghanistan", "Авганистан вријеме"}, {"Africa_Central", "Централно-афричко вријеме"}, {"Africa_Eastern", "Источно-афричко вријеме"}, {"Africa_Southern", "Јужно-афричко вријеме"}, {"Africa_Western", "Западно-афричко вријеме"}, {"Alaska", "Аљаска стандардно вријеме"}, {"Almaty", "Алмати стандардно време"}, {"Amazon", "Амазон стандардно вријеме"}, {"America_Central", "Централно стандардно вријеме"}, {"America_Eastern", "Источно стандардно вријеме"}, {"America_Mountain", "Планинско стандардно вријеме"}, {"America_Pacific", "Пацифичко стандардно вријеме"}, {"Anadyr", "Анадир стандардно време"}, {"Apia", "Апија стандардно вријеме"}, {"Aqtau", "Акватау стандардно време"}, {"Aqtobe", "Акутобе стандардно време"}, {"Arabian", "Арабијско стандардно вријеме"}, {"Argentina", "Аргентина стандардно вријеме"}, {"Argentina_Western", "Западна Аргентина стандардно вријеме"}, {"Armenia", "Арменија стандардно вријеме"}, {"Atlantic", "Атланско стандардно вријеме"}, {"Australia_Central", "Аустралијско централно стандардно вријеме"}, {"Australia_CentralWestern", "Аустралијско централно западно стандардно вријеме"}, {"Australia_Eastern", "Аустралијско источно стандардно вријеме"}, {"Australia_Western", "Аустралијско западно стандардно вријеме"}, {"Azerbaijan", "Азербејџан стандардно вријеме"}, {"Azores", "Азори стандардно вријеме"}, {"Bangladesh", "Бангладеш стандардно вријеме"}, {"Bhutan", "Бутан вријеме"}, {"Bolivia", "Боливија вријеме"}, {"Brasilia", "Бразилија стандардно вријеме"}, {"Brunei", "Брунеј Дарусалам вријеме"}, {"Cape_Verde", "Зеленортско стандардно вријеме"}, {"Chamorro", "Чаморо вријеме"}, {"Chatham", "Чатам стандардно вријеме"}, {"Chile", "Чиле стандардно вријеме"}, {"China", "Кина стандардно вријеме"}, {"Christmas", "Божићна острва вријеме"}, {"Cocos", "Кокос (Келинг) Острва вријеме"}, {"Colombia", "Колумбија стандардно вријеме"}, {"Cook", "Кукова острва стандардно вријеме"}, {"Cuba", "Куба стандардно вријеме"}, {"Davis", "Дејвис вријеме"}, {"DumontDUrville", "Димон д’Урвил вријеме"}, {"East_Timor", "Тимор-Лесте вријеме"}, {"Easter", "Ускршња острва стандардно вријеме"}, {"Ecuador", "Еквадор вријеме"}, {"Europe_Central", "Средњеевропско стандардно вријеме"}, {"Europe_Eastern", "Источноевропско стандардно вријеме"}, {"Europe_Further_Eastern", "Даље источноевропско вријеме"}, {"Europe_Western", "Западноевропско стандардно вријеме"}, {"Falkland", "Фолкландска Острва стандардно вријеме"}, {"Fiji", "Фиџи стандардно вријеме"}, {"French_Guiana", "Француска Гвајана вријеме"}, {"French_Southern", "Француско јужно и антарктичко вријеме"}, {"GMT", "Гриничко средње вријеме"}, {"Galapagos", "Галапагос вријеме"}, {"Gambier", "Гамбијер вријеме"}, {"Georgia", "Грузија стандардно вријеме"}, {"Gilbert_Islands", "Гилберт острва вријеме"}, {"Greenland_Eastern", "Источни Гренланд стандардно вријеме"}, {"Greenland_Western", "Западни Гренланд стандардно вријеме"}, {"Guam", "Гуам стандардно време"}, {"Gulf", "Заливско вријеме"}, {"Guyana", "Гвајана вријеме"}, {"Hawaii", "Хавајско-алеутско стандардно вријеме"}, {"Hawaii_Aleutian", "Хавајско-алеутско стандардно вријеме"}, {"Hong_Kong", "Хонг Конг стандардно вријеме"}, {"Hovd", "Ховд стандардно вријеме"}, {"India", "Индијско стандардно вријеме"}, {"Indian_Ocean", "Индијско океанско вријеме"}, {"Indochina", "Индокина вријеме"}, {"Indonesia_Central", "Централно-индонезијско вријеме"}, {"Indonesia_Eastern", "Источно-индонезијско вријеме"}, {"Indonesia_Western", "Западно-индонезијско вријеме"}, {"Iran", "Иран стандардно вријеме"}, {"Irkutsk", "Иркуцк стандардно вријеме"}, {"Israel", "Израелско стандардно вријеме"}, {"Japan", "Јапанско стандардно вријеме"}, {"Kamchatka", "Петропавловско-камчатско стандардно време"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "Источно-казахстанско вријеме"}, {"Kazakhstan_Western", "Западно-казахстанско вријеме"}, {"Korea", "Корејско стандардно вријеме"}, {"Kosrae", "Кошре вријеме"}, {"Krasnoyarsk", "Краснојарск стандардно вријеме"}, {"Kyrgystan", "Киргистан вријеме"}, {"Lanka", "Шри Ланка време"}, {"Line_Islands", "Лине Острва вријеме"}, {"Lord_Howe", "Лорд Хов стандардно вријеме"}, {"Macau", "Макао стандардно време"}, {"Magadan", "Магадан стандардно вријеме"}, {"Malaysia", "Малезија вријеме"}, {"Maldives", "Малдиви вријеме"}, {"Marquesas", "Маркиз вријеме"}, {"Marshall_Islands", "Маршалска Острва вријеме"}, {"Mauritius", "Маурицијус стандардно вријеме"}, {"Mawson", "Мосон вријеме"}, {"Mexico_Pacific", "Мексичко пацифичко стандардно вријеме"}, {"Mongolia", "Улан Батор стандардно вријеме"}, {"Moscow", "Москва стандардно вријеме"}, {"Myanmar", "Мијанмар вријеме"}, {"Nauru", "Науру вријеме"}, {"Nepal", "Непал вријеме"}, {"New_Caledonia", "Нова Каледонија стандардно вријеме"}, {"New_Zealand", "Нови Зеланд стандардно вријеме"}, {"Newfoundland", "Њуфаундленд стандардно вријеме"}, {"Niue", "Ниуе вријеме"}, {"Norfolk", "Норфолк Острво стандардно вријеме"}, {"Noronha", "Фернандо де Нороња стандардно вријеме"}, {"North_Mariana", "Северна Маријанска Острва време"}, {"Novosibirsk", "Новосибирск стандардно вријеме"}, {"Omsk", "Омск стандардно вријеме"}, {"Pakistan", "Пакистан стандардно вријеме"}, {"Palau", "Палау вријеме"}, {"Papua_New_Guinea", "Папуа Нова Гвинеја вријеме"}, {"Paraguay", "Парагвај стандардно вријеме"}, {"Peru", "Перу стандардно вријеме"}, {"Philippines", "Филипини стандардно вријеме"}, {"Phoenix_Islands", "Феникс острва вријеме"}, {"Pierre_Miquelon", "Сен Пјер и Микелон стандардно вријеме"}, {"Pitcairn", "Питкерн вријеме"}, {"Ponape", "Понапе вријеме"}, {"Pyongyang", "Пјонгјанг вријеме"}, {"Qyzylorda", "Кизилорда стандардно време"}, {"Reunion", "Реинион вријеме"}, {"Rothera", "Ротера вријеме"}, {"Sakhalin", "Сахалин стандардно вријеме"}, {"Samara", "Самара стандардно време"}, {"Samoa", "Самоа стандардно вријеме"}, {"Seychelles", "Сејшели вријеме"}, {"Singapore", "Сингапур стандардно вријеме"}, {"Solomon", "Соломонска Острва вријеме"}, {"South_Georgia", "Јужна Џорџија вријеме"}, {"Suriname", "Суринам вријеме"}, {"Syowa", "Шова вријеме"}, {"Tahiti", "Тахити вријеме"}, {"Taipei", "Тајпеј стандардно вријеме"}, {"Tajikistan", "Таџикистан вријеме"}, {"Tokelau", "Токелау вријеме"}, {"Tonga", "Тонга стандардно вријеме"}, {"Truk", "Трук вријеме"}, {"Turkey", "Турска стандардно време"}, {"Turkmenistan", "Туркменистан стандардно вријеме"}, {"Tuvalu", "Тувалу вријеме"}, {"Uruguay", "Уругвај стандардно вријеме"}, {"Uzbekistan", "Узбекистан стандардно вријеме"}, {"Vanuatu", "Вануату стандардно вријеме"}, {"Venezuela", "Венецуела вријеме"}, {"Vladivostok", "Владивосток стандардно вријеме"}, {"Volgograd", "Волгоград стандардно вријеме"}, {"Vostok", "Восток вријеме"}, {"Wake", "Вејк острво вријеме"}, {"Wallis", "Валис и Футуна Острва вријеме"}, {"Yakutsk", "Јакутск стандардно вријеме"}, {"Yekaterinburg", "Јекатеринбург стандардно вријеме"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Акре летње рачунање времена"}, {"Alaska", "Аљаска љетње вријеме"}, {"Almaty", "Алмати летње рачунање времена"}, {"Amazon", "Амазон љетње рачунање времена"}, {"America_Central", "Централно љетње рачунање времена"}, {"America_Eastern", "Источно љетње рачунање времена"}, {"America_Mountain", "Планинско љетње рачунање времена"}, {"America_Pacific", "Пацифичко љетње рачунање времена"}, {"Anadyr", "Анадир летње рачунање времена"}, {"Apia", "Апија љетње рачунање времена"}, {"Aqtau", "Акватау летње рачунање времена"}, {"Aqtobe", "Акутобе летње рачунање времена"}, {"Arabian", "Арабијско љетње рачунање времена"}, {"Argentina", "Аргентина љетње рачунање времена"}, {"Argentina_Western", "Западна Аргентина љетње рачунање времена"}, {"Armenia", "Арменија љетње рачунање времена"}, {"Atlantic", "Атланско љетње рачунање времена"}, {"Australia_Central", "Аустралијско централно љетње рачунање времена"}, {"Australia_CentralWestern", "Аустралијско централно западно љетње рачунање времена"}, {"Australia_Eastern", "Аустралијско источно љетње рачунање времена"}, {"Australia_Western", "Аустралијско западно љетње рачунање времена"}, {"Azerbaijan", "Азербејџан љетње рачунање времена"}, {"Azores", "Азори љетње рачунање времена"}, {"Bangladesh", "Бангладеш љетње рачунање времена"}, {"Brasilia", "Бразилија љетње рачунање времена"}, {"Cape_Verde", "Зеленортско љетње рачунање времена"}, {"Chatham", "Чатам љетње рачунање времена"}, {"Chile", "Чиле љетње рачунање времена"}, {"China", "Кина љетње рачунање времена"}, {"Colombia", "Колумбија љетње рачунање времена"}, {"Cook", "Кукова острва полу-љетње рачунање времена"}, {"Cuba", "Куба љетње рачунање времена"}, {"Easter", "Ускршња острва љетње рачунање времена"}, {"Europe_Central", "Средњеевропско љетње рачунање времена"}, {"Europe_Eastern", "Источноевропско љетње рачунање времена"}, {"Europe_Western", "Западноевропско љетње рачунање времена"}, {"Falkland", "Фолкландска Острва љетње рачунање времена"}, {"Fiji", "Фиџи љетње рачунање времена"}, {"Georgia", "Грузија љетње рачунање времена"}, {"Greenland_Eastern", "Источни Гренланд љетње рачунање времена"}, {"Greenland_Western", "Западни Гренланд љетње рачунање времена"}, {"Hawaii_Aleutian", "Хавајско-алеутско љетње рачунање времена"}, {"Hong_Kong", "Хонгконшко љетње рачунање времена"}, {"Hovd", "Ховд љетње рачунање времена"}, {"Iran", "Иран љетње рачунање времена"}, {"Irkutsk", "Иркуцк љетње рачунање времена"}, {"Israel", "Израелско љетње рачунање времена"}, {"Japan", "Јапанско љетње рачунање времена"}, {"Kamchatka", "Петропавловско-камчатско летње рачунање вемена"}, {"Korea", "Корејско љетње рачунање времена"}, {"Krasnoyarsk", "Краснојарск љетње рачунање времена"}, {"Lord_Howe", "Лорд Хов љетње рачунање времена"}, {"Macau", "Макао летње рачунање вемена"}, {"Magadan", "Магадан љетње рачунање вемена"}, {"Mauritius", "Маурицијус љетње рачунање времена"}, {"Mexico_Pacific", "Мексичко пацифичко љетње рачунање времена"}, {"Mongolia", "Улан Батор љетње рачунање времена"}, {"Moscow", "Москва љетње рачунање времена"}, {"New_Caledonia", "Нова Каледонија љетње рачунање времена"}, {"New_Zealand", "Нови Зеланд љетње рачунање времена"}, {"Newfoundland", "Њуфаундленд љетње рачунање времена"}, {"Norfolk", "Норфолк Острво љетње рачунање вријеме"}, {"Noronha", "Фернандо де Нороња љетње рачунање времена"}, {"Novosibirsk", "Новосибирск љетње рачунање времена"}, {"Omsk", "Омск љетње рачунање времена"}, {"Pakistan", "Пакистан љетње рачунање времена"}, {"Paraguay", "Парагвај љетње рачунање времена"}, {"Peru", "Перу љетње рачунање времена"}, {"Philippines", "Филипини љетње рачунање времена"}, {"Pierre_Miquelon", "Сен Пјер и Микелон љетње рачунање вемена"}, {"Qyzylorda", "Кизилорда летње рачунање времена"}, {"Sakhalin", "Сахалин љетње рачунање времена"}, {"Samara", "Самара летње рачунање времена"}, {"Samoa", "Самоа љетње рачунање времена"}, {"Taipei", "Тајпеј љетње рачунање времена"}, {"Tonga", "Тонга љетње рачунање времена"}, {"Turkey", "Турска летње рачунање времена"}, {"Turkmenistan", "Туркменистан љетње рачунање времена"}, {"Uruguay", "Уругвај љетње рачунање времена"}, {"Uzbekistan", "Узбекистан љетње рачунање времена"}, {"Vanuatu", "Вануату љетње рачунање времена"}, {"Vladivostok", "Владивосток љетње рачунање времена"}, {"Volgograd", "Волгоград љетње рачунање времена"}, {"Yakutsk", "Јакутск љетње рачунање времена"}, {"Yekaterinburg", "Јекатеринбург љетње рачунање времена"}},
	},
	{
		{{"GMT", "Гринвичын саг"}},
		{},
	},
	{
		{{"Afghanistan", "Hora de l’Afganistan"}, {"Africa_Central", "Hora de l’Àfrica central"}, {"Africa_Eastern", "Hora de l’Àfrica oriental"}, {"Africa_Southern", "Hora estàndard de l’Àfrica meridional"}, {"Africa_Western", "Hora de l’Àfrica occidental"}, {"Alaska", "Hora estàndard d’Alaska"}, {"Amazon", "Hora estàndard de l’Amazones"}, {"America_Central", "Hora estàndard central d’Amèrica del Nord"}, {"America_Eastern", "Hora estàndard oriental d’Amèrica del Nord"}, {"America_Mountain", "Hora estàndard de muntanya d’Amèrica del Nord"}, {"America_Pacific", "Hora estàndard del Pacífic d’Amèrica del Nord"}, {"Anadyr", "Hora estàndard d’Anadyr"}, {"Apia", "Hora estàndard d’Apia"}, {"Arabian", "Hora estàndard àrab"}, {"Argentina", "Hora estàndard de l’Argentina"}, {"Argentina_Western", "Hora estàndard de l’oest de l’Argentina"}, {"Armenia", "Hora estàndard d’Armènia"}, {"Atlantic", "Hora estàndard de l’Atlàntic"}, {"Australia_Central", "Hora estàndard d’Austràlia central"}, {"Australia_CentralWestern", "Hora estàndard d’Austràlia centre-occidental"}, {"Australia_Eastern", "Hora estàndard d’Austràlia oriental"}, {"Australia_Western", "Hora estàndard d’Austràlia occidental"}, {"Azerbaijan", "Hora estàndard de l’Azerbaidjan"}, {"Azores", "Hora estàndard de les Açores"}, {"Bangladesh", "Hora estàndard de Bangladesh"}, {"Bhutan", "Hora de Bhutan"}, {"Bolivia", "Hora de Bolívia"}, {"Brasilia", "Hora estàndard de Brasília"}, {"Brunei", "Hora de Brunei Darussalam"}, {"Cape_Verde", "Hora estàndard de Cap Verd"}, {"Chamorro", "Hora estàndard de Chamorro"}, {"Chatham", "Hora estàndard de Chatham"}, {"Chile", "Hora estàndard de Xile"}, {"China", "Hora estàndard de la Xina"}, {"Christmas", "Hora de Kiritimati"}, {"Cocos", "Hora de les illes Cocos"}, {"Colombia", "Hora estàndard de Colòmbia"}, {"Cook", "Hora estàndard de les illes Cook"}, {"Cuba", "Hora estàndard de Cuba"}, {"Davis", "Hora de Davis"}, {"DumontDUrville", "Hora de Dumont d’Urville"}, {"East_Timor", "Hora de Timor Oriental"}, {"Easter", "Hora estàndard de l’illa de Pasqua"}, {"Ecuador", "Hora de l’Equador"}, {"Europe_Central", "Hora estàndard d’Europa central"}, {"Europe_Eastern", "Hora estàndard d’Europa oriental"}, {"Europe_Further_Eastern", "Hora de l’extrem oriental d’Europa"}, {"Europe_Western", "Hora estàndard d’Europa occidental"}, {"Falkland", "Hora estàndard de les illes Falkland"}, {"Fiji", "Hora estàndard de Fiji"}, {"French_Guiana", "Hora de la Guaiana Francesa"}, {"French_Southern", "Hora d’Antàrtida i de les Terres Australs Antàrtiques Franceses"}, {"GMT", "Hora del meridià de Greenwich"}, {"Galapagos", "Hora de Galápagos"}, {"Gambier", "Hora de Gambier"}, {"Georgia", "Hora estàndard de Geòrgia"}, {"Gilbert_Islands", "Hora de les illes Gilbert"}, {"Greenland", "Hora estàndard de Groenlàndia"}, {"Greenland_Eastern", "Hora estàndard de l’Est de Groenlàndia"}, {"Greenland_Western", "Hora estàndard de l’Oest de Groenlàndia"}, {"Gulf", "Hora estàndard del Golf"}, {"Guyana", "Hora de Guyana"}, {"Hawaii", "Hora estàndard de Hawaii-Aleutianes"}, {"Hawaii_Aleutian", "Hora estàndard de Hawaii-Aleutianes"}, {"Hong_Kong", "Hora estàndard de Hong Kong"}, {"Hovd", "Hora estàndard de Khovd"}, {"India", "Hora de l’Índia"}, {"Indian_Ocean", "Hora de l’oceà Índic"}, {"Indochina", "Hora de l’Indoxina"}, {"Indonesia_Central", "Hora central d’Indonèsia"}, {"Indonesia_Eastern", "Hora de l’est d’Indonèsia"}, {"Indonesia_Western", "Hora de l’oest d’Indonèsia"}, {"Iran", "Hora estàndard de l’Iran"}, {"Irkutsk", "Hora estàndard d’Irkutsk"}, {"Israel", "Hora estàndard d’Israel"}, {"Japan", "Hora estàndard del Japó"}, {"Kamchatka", "Hora estàndard de Petropavlovsk de Kamtxatka"}, {"Kazakhstan", "Hora del Kazakhstan"}, {"Kazakhstan_Eastern", "Hora de l’est del Kazakhstan"}, {"Kazakhstan_Western", "Hora de l’oest del Kazakhstan"}, {"Korea", "Hora estàndard de Corea"}, {"Kosrae", "Hora de Kosrae"}, {"Krasnoyarsk", "Hora estàndard de Krasnoiarsk"}, {"Kyrgystan", "Hora del Kirguizstan"}, {"Line_Islands", "Hora de les illes Line"}, {"Lord_Howe", "Hora estàndard de Lord Howe"}, {"Macau", "Hora estàndard de Macau"}, {"Magadan", "Hora estàndard de Magadan"}, {"Malaysia", "Hora de Malàisia"}, {"Maldives", "Hora de les Maldives"}, {"Marquesas", "Hora de les Marqueses"}, {"Marshall_Islands", "Hora de les illes Marshall"}, {"Mauritius", "Hora estàndard de Maurici"}, {"Mawson", "Hora de Mawson"}, {"Mexico_Pacific", "Hora estàndard del Pacífic de Mèxic"}, {"Mongolia", "Hora estàndard d’Ulaanbaatar"}, {"Moscow", "Hora estàndard de Moscou"}, {"Myanmar", "Hora de Myanmar"}, {"Nauru", "Hora de Nauru"}, {"Nepal", "Hora del Nepal"}, {"New_Caledonia", "Hora estàndard de Nova Caledònia"}, {"New_Zealand", "Hora estàndard de Nova Zelanda"}, {"Newfoundland", "Hora estàndard de Terranova"}, {"Niue", "Hora de Niue"}, {"Norfolk", "Hora estàndard de l’illa Norfolk"}, {"Noronha", "Hora estàndard de Fernando de Noronha"}, {"Novosibirsk", "Hora estàndard de Novossibirsk"}, {"Omsk", "Hora estàndard d’Omsk"}, {"Pakistan", "Hora estàndard del Pakistan"}, {"Palau", "Hora de Palau"}, {"Papua_New_Guinea", "Hora de Papua Nova Guinea"}, {"Paraguay", "Hora estàndard del Paraguai"}, {"Peru", "Hora estàndard del Perú"}, {"Philippines", "Hora estàndard de les Filipines"}, {"Phoenix_Islands", "Hora de les illes Phoenix"}, {"Pierre_Miquelon", "Hora estàndard de Saint-Pierre-et-Miquelon"}, {"Pitcairn", "Hora de Pitcairn"}, {"Ponape", "Hora de Ponape"}, {"Pyongyang", "Hora de Pyongyang"}, {"Reunion", "Hora de Reunió"}, {"Rothera", "Hora de Rothera"}, {"Sakhalin", "Hora estàndard de Sakhalín"}, {"Samara", "Hora estàndard de Samara"}, {"Samoa", "Hora estàndard de Samoa"}, {"Seychelles", "Hora de les Seychelles"}, {"Singapore", "Hora de Singapur"}, {"Solomon", "Hora de les illes Salomó"}, {"South_Georgia", "Hora de Geòrgia del Sud"}, {"Suriname", "Hora de Surinam"}, {"Syowa", "Hora de Syowa"}, {"Tahiti", "Hora de Tahití"}, {"Taipei", "Hora estàndard de Taipei"}, {"Tajikistan", "Hora del Tadjikistan"}, {"Tokelau", "Hora de Tokelau"}, {"Tonga", "Hora estàndard de Tonga"}, {"Truk", "Hora de Chuuk"}, {"Turkey", "Hora estàndard de Turquia"}, {"Turkmenistan", "Hora estàndard del Turkmenistan"}, {"Tuvalu", "Hora de Tuvalu"}, {"Uruguay", "Hora estàndard de l’Uruguai"}, {"Uzbekistan", "Hora estàndard de l’Uzbekistan"}, {"Vanuatu", "Hora estàndard de Vanatu"}, {"Venezuela", "Hora de Veneçuela"}, {"Vladivostok", "Hora estàndard de Vladivostok"}, {"Volgograd", "Hora estàndard de Volgograd"}, {"Vostok", "Hora de Vostok"}, {"Wake", "Hora de les illes Wake"}, {"Wallis", "Hora de Wallis i Futuna"}, {"Yakutsk", "Hora estàndard de Iakutsk"}, {"Yekaterinburg", "Hora estàndard de Iekaterinburg"}, {"Yukon", "Hora de Yukon"}},
		{{"Alaska", "Hora d’estiu d’Alaska"}, {"Amazon", "Hora d’estiu de l’Amazones"}, {"America_Central", "Hora d’estiu central d’Amèrica del Nord"}, {"America_Eastern", "Hora d’estiu oriental d’Amèrica del Nord"}, {"America_Mountain", "Hora d’estiu de muntanya d’Amèrica del Nord"}, {"America_Pacific", "Hora d’estiu del Pacífic d’Amèrica del Nord"}, {"Anadyr", "Horari d’estiu d’Anadyr"}, {"Apia", "Hora d’estiu d’Apia"}, {"Arabian", "Hora d’estiu àrab"}, {"Argentina", "Hora d’estiu de l’Argentina"}, {"Argentina_Western", "Hora d’estiu de l’oest de l’Argentina"}, {"Armenia", "Hora d’estiu d’Armènia"}, {"Atlantic", "Hora d’estiu de l’Atlàntic"}, {"Australia_Central", "Hora d’estiu d’Austràlia central"}, {"Australia_CentralWestern", "Hora d’estiu d’Austràlia centre-occidental"}, {"Australia_Eastern", "Hora d’estiu d’Austràlia oriental"}, {"Australia_Western", "Hora d’estiu d’Austràlia occidental"}, {"Azerbaijan", "Hora d’estiu de l’Azerbaidjan"}, {"Azores", "Hora d’estiu de les Açores"}, {"Bangladesh", "Hora d’estiu de Bangladesh"}, {"Brasilia", "Hora d’estiu de Brasília"}, {"Cape_Verde", "Hora d’estiu de Cap Verd"}, {"Chatham", "Hora d’estiu de Chatham"}, {"Chile", "Hora d’estiu de Xile"}, {"China", "Hora d’estiu de la Xina"}, {"Colombia", "Hora d’estiu de Colòmbia"}, {"Cook", "Hora de mig estiu de les illes Cook"}, {"Cuba", "Hora d’estiu de Cuba"}, {"Easter", "Hora d’estiu de l’illa de Pasqua"}, {"Europe_Central", "Hora d’estiu d’Europa central"}, {"Europe_Eastern", "Hora d’estiu d’Europa oriental"}, {"Europe_Western", "Hora d’estiu d’Europa occidental"}, {"Falkland", "Hora d’estiu de les illes Falkland"}, {"Fiji", "Hora d’estiu de Fiji"}, {"Georgia", "Hora d’estiu de Geòrgia"}, {"Greenland", "Hora d’estiu de Groenlàndia"}, {"Greenland_Eastern", "Hora d’estiu de l’Est de Groenlàndia"}, {"Greenland_Western", "Hora d’estiu de l’Oest de Groenlàndia"}, {"Hawaii_Aleutian", "Hora d’estiu de Hawaii-Aleutianes"}, {"Hong_Kong", "Hora d’estiu de Hong Kong"}, {"Hovd", "Hora d’estiu de Khovd"}, {"Iran", "Hora d’estiu de l’Iran"}, {"Irkutsk", "Hora d’estiu d’Irkutsk"}, {"Israel", "Hora d’estiu d’Israel"}, {"Japan", "Hora d’estiu del Japó"}, {"Kamchatka", "Horari d’estiu de Petropavlovsk de Kamtxatka"}, {"Korea", "Hora d’estiu de Corea"}, {"Krasnoyarsk", "Hora d’estiu de Krasnoiarsk"}, {"Lord_Howe", "Horari d’estiu de Lord Howe"}, {"Macau", "Hora d’estiu de Macau"}, {"Magadan", "Hora d’estiu de Magadan"}, {"Mauritius", "Hora d’estiu de Maurici"}, {"Mexico_Pacific", "Hora d’estiu del Pacífic de Mèxic"}, {"Mongolia", "Hora d’estiu d’Ulaanbaatar"}, {"Moscow", "Hora d’estiu de Moscou"}, {"New_Caledonia", "Hora d’estiu de Nova Caledònia"}, {"New_Zealand", "Hora d’estiu de Nova Zelanda"}, {"Newfoundland", "Hora d’estiu de Terranova"}, {"Norfolk", "Hora d’estiu de l’illa Norfolk"}, {"Noronha", "Hora d’estiu de Fernando de Noronha"}, {"Novosibirsk", "Hora d’estiu de Novossibirsk"}, {"Omsk", "Hora d’estiu d’Omsk"}, {"Pakistan", "Hora d’estiu del Pakistan"}, {"Paraguay", "Hora d’estiu del Paraguai"}, {"Peru", "Hora d’estiu del Perú"}, {"Philippines", "Hora d’estiu de les Filipines"}, {"Pierre_Miquelon", "Hora d’estiu de Saint-Pierre-et-Miquelon"}, {"Sakhalin", "Hora d’estiu de Sakhalín"}, {"Samara", "Hora d’estiu de Samara"}, {"Samoa", "Hora d’estiu de Samoa"}, {"Taipei", "Hora d’estiu de Taipei"}, {"Tonga", "Hora d’estiu de Tonga"}, {"Turkey", "Hora d’estiu de Turquia"}, {"Turkmenistan", "Hora d’estiu del Turkmenistan"}, {"Uruguay", "Hora d’estiu de l’Uruguai"}, {"Uzbekistan", "Hora d’estiu de l’Uzbekistan"}, {"Vanuatu", "Hora d’estiu de Vanatu"}, {"Vladivostok", "Hora d’estiu de Vladivostok"}, {"Volgograd", "Hora d’estiu de Volgograd"}, {"Yakutsk", "Hora d’estiu de Iakutsk"}, {"Yekaterinburg", "Hora d’estiu de Iekaterinburg"}},
	},
	{
		{{"Acre", "𑄃𑄬𑄉𑄧𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Afghanistan", "𑄃𑄛𑄴𑄉𑄚𑄨𑄌𑄴𑄖𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Africa_Central", "𑄟𑄧𑄖𑄴𑄙𑄳𑄠 𑄃𑄜𑄳𑄢𑄨𑄇 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Africa_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄃𑄜𑄳𑄢𑄨𑄇 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Africa_Southern", "𑄘𑄧𑄉𑄨𑄚𑄴 𑄃𑄜𑄳𑄢𑄨𑄇 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Africa_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄃𑄜𑄳𑄢𑄨𑄇 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Alaska", "𑄃𑄣𑄌𑄴𑄇 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Almaty", "𑄃𑄣𑄴𑄟𑄑𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Amazon", "𑄃𑄳𑄠𑄟𑄎𑄧𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Central", "𑄃𑄏𑄧𑄣𑄴 𑄉𑄧𑄢𑄳𑄦 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Eastern", "𑄛𑄪𑄉𑄮 𑄞𑄨𑄘𑄬𑄢𑄴 𑄛𑄳𑄢𑄧𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Mountain", "𑄦𑄨𑄣𑄧𑄧𑄱 𑄞𑄨𑄘𑄬𑄢𑄴 𑄛𑄳𑄢𑄧𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Pacific", "𑄛𑄳𑄢𑄧𑄥𑄚𑄴𑄖𑄧 𑄟𑄧𑄦𑄥𑄉𑄧𑄢𑄧𑄢𑄴 𑄞𑄨𑄘𑄬𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Anadyr", "𑄃𑄧𑄚𑄴𑄘𑄠𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Apia", "𑄃𑄧𑄛𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Aqtau", "𑄃𑄇𑄴𑄑𑄃𑄮 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Aqtobe", "𑄃𑄇𑄴𑄑𑄮𑄝𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Arabian", "𑄃𑄢𑄧𑄝𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Argentina", "𑄃𑄢𑄴𑄎𑄬𑄚𑄴𑄑𑄨𑄚 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Argentina_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄃𑄢𑄴𑄎𑄬𑄚𑄴𑄑𑄨𑄚 𑄛𑄳𑄢𑄧𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Armenia", "𑄃𑄢𑄴𑄟𑄬𑄚𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Atlantic", "𑄃𑄑𑄴𑄣𑄚𑄴𑄖𑄨𑄉𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Central", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄃𑄏𑄧𑄣𑄴 𑄉𑄧𑄢𑄳𑄦𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Eastern", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄛𑄪𑄉𑄬𑄘𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Western", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Azerbaijan", "𑄃𑄎𑄢𑄴𑄝𑄭𑄎𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Azores", "𑄃𑄬𑄎𑄮𑄢𑄬𑄌𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Bangladesh", "𑄝𑄁𑄣𑄘𑄬𑄌𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Bhutan", "𑄞𑄪𑄑𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Bolivia", "𑄝𑄮𑄣𑄨𑄞𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Brasilia", "𑄝𑄳𑄢𑄥𑄨𑄣𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Brunei", "𑄝𑄳𑄢𑄪𑄚𑄬𑄭 𑄘𑄢𑄪𑄌𑄴𑄥𑄣𑄟𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cape_Verde", "𑄇𑄬𑄛𑄴 𑄞𑄢𑄴𑄘𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Chamorro", "𑄌𑄟𑄬𑄢𑄮 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Chatham", "𑄌𑄳𑄠𑄗𑄟𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Chile", "𑄌𑄨𑄣𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"China", "𑄌𑄨𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Christmas", "𑄇𑄳𑄢𑄨𑄌𑄴𑄟𑄥𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cocos", "𑄇𑄮𑄇𑄮𑄌𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Colombia", "𑄇𑄧𑄣𑄧𑄟𑄴𑄝𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cook", "𑄇𑄪𑄇𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cuba", "𑄇𑄨𑄃𑄪𑄝 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Davis", "𑄓𑄬𑄞𑄨𑄥𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"DumontDUrville", "𑄓𑄪𑄟𑄧𑄚𑄴𑄑𑄴-𑄘𑄳𑄠𑄧 𑄅𑄪𑄢𑄴𑄞𑄨𑄣𑄬 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"East_Timor", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄑𑄨𑄟𑄪𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Easter", "𑄃𑄨𑄥𑄴𑄑𑄢𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Ecuador", "𑄃𑄨𑄇𑄪𑄠𑄬𑄓𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Central", "𑄟𑄧𑄖𑄴𑄙𑄳𑄠 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Further_Eastern", "𑄃𑄬𑄇𑄴𑄉𑄮𑄙𑄳𑄆𑄬𑄣𑄴 𑄛𑄪𑄉𑄬𑄘𑄨 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Falkland", "𑄜𑄧𑄇𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Fiji", "𑄜𑄨𑄎𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"French_Guiana", "𑄜𑄧𑄢𑄥𑄨 𑄉𑄠𑄚 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"French_Southern", "𑄜𑄧𑄢𑄥𑄨 𑄘𑄧𑄉𑄨𑄚𑄴 𑄃𑄳𑄃 𑄃𑄚𑄴𑄑𑄢𑄴𑄇𑄧𑄑𑄨𑄇 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"GMT", "𑄉𑄳𑄢𑄨𑄚𑄨𑄌𑄴 𑄟𑄨𑄚𑄴 𑄑𑄬𑄟𑄴"}, {"Galapagos", "𑄉𑄣𑄛𑄉𑄮𑄌𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Gambier", "𑄉𑄳𑄠𑄟𑄴𑄝𑄨𑄠𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Georgia", "𑄎𑄧𑄢𑄴𑄎𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Gilbert_Islands", "𑄉𑄨𑄣𑄴𑄝𑄢𑄴𑄑𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Greenland_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄉𑄳𑄢𑄨𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Greenland_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄉𑄳𑄢𑄨𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Guam", "𑄉𑄪𑄠𑄟𑄴 𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Gulf", "𑄃𑄪𑄛𑄧𑄥𑄉𑄧𑄢𑄨𑄠𑄧 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Guyana", "𑄉𑄪𑄠𑄚 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hawaii", "𑄦𑄧𑄃𑄮𑄠𑄭-𑄃𑄣𑄬𑄃𑄪𑄖𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hawaii_Aleutian", "𑄦𑄧𑄃𑄮𑄠𑄭-𑄃𑄣𑄬𑄃𑄪𑄖𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hong_Kong", "𑄦𑄧𑄁 𑄇𑄧𑄁 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hovd", "𑄦𑄮𑄞𑄧𑄓𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"India", "𑄃𑄨𑄚𑄴𑄘𑄨𑄠𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Indian_Ocean", "𑄃𑄨𑄚𑄴𑄘𑄨𑄠𑄬 𑄟𑄧𑄦𑄥𑄉𑄧𑄢𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Indochina", "𑄃𑄨𑄚𑄴𑄘𑄮𑄌𑄩𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Indonesia_Central", "𑄃𑄏𑄧𑄣𑄴 𑄉𑄢𑄳𑄦 𑄃𑄨𑄚𑄴𑄘𑄮𑄚𑄬𑄥𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Indonesia_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄃𑄨𑄚𑄴𑄘𑄮𑄚𑄬𑄥𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Indonesia_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄃𑄨𑄚𑄴𑄘𑄮𑄚𑄬𑄥𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Iran", "𑄃𑄨𑄢𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Irkutsk", "𑄃𑄨𑄢𑄴𑄖𑄪𑄑𑄴𑄥𑄴𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Israel", "𑄃𑄨𑄎𑄴𑄢𑄠𑄬𑄣𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Japan", "𑄎𑄛𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kamchatka", "𑄛𑄨𑄖𑄳𑄢𑄬𑄛𑄳𑄠𑄞𑄧𑄣𑄧𑄥𑄴𑄇𑄴-𑄇𑄳𑄠𑄟𑄴𑄌𑄳𑄠𑄑𑄴𑄃𑄨𑄥𑄴𑄇𑄨 𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kazakhstan_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄇𑄎𑄇𑄴𑄥𑄳𑄖𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kazakhstan_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄇𑄎𑄇𑄴𑄥𑄳𑄖𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Korea", "𑄇𑄮𑄢𑄨𑄠𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kosrae", "𑄇𑄮𑄌𑄴𑄢𑄳𑄆𑄬 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Krasnoyarsk", "𑄇𑄳𑄢𑄥𑄴𑄚𑄮𑄠𑄢𑄴𑄥𑄴𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kyrgystan", "𑄇𑄨𑄢𑄴𑄉𑄨𑄥𑄴𑄖𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Lanka", "𑄣𑄧𑄁𑄇 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Line_Islands", "𑄣𑄭𑄚𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Lord_Howe", "𑄣𑄧𑄢𑄴𑄓𑄴 𑄦𑄤𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Macau", "𑄟𑄇𑄃𑄮 𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Magadan", "𑄟𑄳𑄠𑄉𑄓𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Malaysia", "𑄟𑄣𑄴𑄠𑄬𑄥𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Maldives", "𑄟𑄣𑄴𑄘𑄨𑄛𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Marquesas", "𑄟𑄢𑄴𑄇𑄬𑄥𑄥𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Marshall_Islands", "𑄟𑄢𑄴𑄥𑄣𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mauritius", "𑄟𑄧𑄢𑄨𑄥𑄥𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mawson", "𑄟𑄧𑄥𑄳𑄦𑄧𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mexico_Pacific", "𑄟𑄬𑄇𑄴𑄥𑄨𑄇𑄚𑄴 𑄛𑄳𑄢𑄧𑄥𑄚𑄴𑄖𑄧 𑄟𑄧𑄦𑄥𑄉𑄧𑄢𑄧𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mongolia", "𑄃𑄪𑄣𑄚𑄴 𑄝𑄖𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Moscow", "𑄟𑄧𑄥𑄴𑄇𑄮 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Myanmar", "𑄟𑄠𑄚𑄴𑄟𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Nauru", "𑄚𑄃𑄪𑄢𑄪 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Nepal", "𑄚𑄬𑄛𑄣𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"New_Caledonia", "𑄚𑄨𑄃𑄪 𑄇𑄳𑄠𑄣𑄬𑄓𑄮𑄚𑄨𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"New_Zealand", "𑄚𑄨𑄃𑄪𑄎𑄨𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Newfoundland", "𑄚𑄨𑄃𑄪𑄜𑄃𑄪𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄨 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Niue", "𑄚𑄨𑄃𑄪𑄃𑄨 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Norfolk", "𑄚𑄧𑄢𑄴𑄜𑄮𑄇𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Noronha", "𑄜𑄢𑄴𑄚𑄚𑄴𑄘𑄮𑄓𑄨 𑄚𑄮𑄢𑄮𑄚𑄴𑄦 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"North_Mariana", "𑄅𑄪𑄖𑄴𑄖𑄮𑄢𑄴 𑄟𑄬𑄢𑄨𑄚𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Novosibirsk", "𑄚𑄮𑄞𑄮𑄥𑄨𑄝𑄨𑄢𑄴𑄥𑄴𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Omsk", "𑄃𑄮𑄟𑄧𑄥𑄴𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pakistan", "𑄛𑄇𑄨𑄥𑄴𑄖𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Palau", "𑄛𑄣𑄅𑄪 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Papua_New_Guinea", "𑄛𑄛𑄪𑄠 𑄚𑄨𑄃𑄪 𑄉𑄨𑄚𑄨 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Paraguay", "𑄛𑄳𑄠𑄢𑄉𑄪𑄠𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Peru", "𑄛𑄬𑄢𑄪 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Philippines", "𑄜𑄨𑄣𑄨𑄛𑄭𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Phoenix_Islands", "𑄜𑄮𑄚𑄨𑄇𑄴𑄥𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pierre_Miquelon", "𑄥𑄬𑄚𑄴𑄑𑄴 𑄛𑄨𑄠𑄬𑄢𑄴 𑄃𑄮 𑄟𑄨𑄇𑄬𑄣𑄧𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pitcairn", "𑄛𑄨𑄖𑄴𑄇𑄬𑄠𑄢𑄴𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Ponape", "𑄛𑄮𑄚𑄛𑄬 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pyongyang", "𑄛𑄨𑄠𑄧𑄁 𑄃𑄨𑄠𑄁 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Qyzylorda", "𑄇𑄨𑄎𑄨𑄣𑄮𑄢𑄴𑄓 𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Reunion", "𑄢𑄨𑄃𑄨𑄃𑄪𑄚𑄨𑄠𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Rothera", "𑄢𑄧𑄗𑄬𑄢 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Sakhalin", "𑄥𑄈𑄣𑄨𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Samara", "𑄥𑄟𑄢 𑄟𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Samoa", "𑄥𑄟𑄮𑄠 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Seychelles", "𑄥𑄬𑄥𑄬𑄣𑄧𑄥𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Singapore", "𑄥𑄨𑄁𑄉𑄛𑄪𑄢 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Solomon", "𑄥𑄧𑄣𑄮𑄟𑄧𑄚𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"South_Georgia", "𑄘𑄧𑄉𑄨𑄚𑄴 𑄎𑄧𑄢𑄴𑄎𑄨𑄠 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Suriname", "𑄥𑄪𑄢𑄨𑄚𑄟𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Syowa", "𑄥𑄠𑄮𑄤 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tahiti", "𑄖𑄦𑄨𑄖𑄨 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Taipei", "𑄖𑄭𑄛𑄳𑄆𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tajikistan", "𑄖𑄎𑄈𑄥𑄴𑄖𑄚𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tokelau", "𑄑𑄮𑄇𑄬𑄣𑄃𑄪 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tonga", "𑄑𑄮𑄋𑄴𑄉 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Truk", "𑄌𑄪𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Turkmenistan", "𑄖𑄪𑄢𑄴𑄇𑄴𑄟𑄬𑄚𑄨𑄌𑄴𑄖𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tuvalu", "𑄑𑄪𑄞𑄣𑄪 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Uruguay", "𑄃𑄪𑄢𑄪𑄉𑄪𑄠𑄬 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Uzbekistan", "𑄃𑄪𑄎𑄴𑄝𑄬𑄇𑄨𑄖𑄚𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Vanuatu", "𑄞𑄚𑄪𑄠𑄖𑄪 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Venezuela", "𑄞𑄬𑄚𑄬𑄎𑄪𑄠𑄬𑄣 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Vladivostok", "𑄝𑄳𑄣𑄘𑄨𑄝𑄮𑄥𑄴𑄖𑄮𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Volgograd", "𑄞𑄧𑄣𑄴𑄉𑄮𑄉𑄳𑄢𑄓𑄴 𑄟𑄧𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Vostok", "𑄞𑄧𑄥𑄴𑄑𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Wake", "𑄃𑄮𑄠𑄬𑄇𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Wallis", "𑄤𑄣𑄨𑄥𑄴 𑄃𑄳𑄃 𑄜𑄪𑄑𑄪𑄚 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Yakutsk", "𑄠𑄇𑄪𑄖𑄴𑄥𑄴𑄇𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Yekaterinburg", "𑄃𑄨𑄠𑄬𑄇𑄖𑄬𑄢𑄨𑄚𑄴𑄝𑄪𑄢𑄴𑄉𑄴 𑄟𑄚𑄧𑄇𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}},
		{{"Acre", "𑄃𑄬𑄉𑄧𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Alaska", "𑄃𑄣𑄌𑄴𑄇 𑄘𑄨𑄚𑄮𑄃𑄣𑄮 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Almaty", "𑄃𑄣𑄴𑄟𑄑𑄨 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Amazon", "𑄃𑄳𑄠𑄟𑄎𑄧𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Central", "𑄃𑄏𑄧𑄣𑄴 𑄉𑄧𑄢𑄳𑄦 𑄘𑄨𑄚𑄮𑄃𑄣𑄮 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Eastern", "𑄛𑄪𑄉𑄮 𑄞𑄨𑄘𑄬𑄢𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Mountain", "𑄦𑄨𑄣𑄧𑄧𑄱 𑄞𑄨𑄘𑄬𑄢𑄴 𑄘𑄨𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"America_Pacific", "𑄛𑄳𑄢𑄧𑄥𑄚𑄴𑄖𑄧 𑄟𑄧𑄦𑄥𑄉𑄧𑄢𑄧𑄢𑄴 𑄞𑄨𑄘𑄬𑄢𑄴 𑄘𑄨𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Anadyr", "𑄃𑄧𑄚𑄴𑄘𑄠𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Apia", "𑄃𑄧𑄛𑄨𑄠 𑄘𑄨𑄚𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Aqtau", "𑄃𑄇𑄴𑄑𑄃𑄮 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Aqtobe", "𑄃𑄇𑄴𑄑𑄮𑄝𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Arabian", "𑄃𑄢𑄧𑄝𑄨 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Argentina", "𑄃𑄢𑄴𑄎𑄬𑄚𑄴𑄑𑄨𑄚 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Argentina_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄃𑄢𑄴𑄎𑄬𑄚𑄴𑄑𑄨𑄚 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Armenia", "𑄃𑄢𑄴𑄟𑄬𑄚𑄨𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Atlantic", "𑄃𑄑𑄴𑄣𑄚𑄴𑄖𑄨𑄉𑄮𑄢𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Central", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄃𑄏𑄧𑄣𑄴 𑄉𑄧𑄢𑄳𑄦𑄢𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Eastern", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄛𑄪𑄉𑄬𑄘𑄨 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Australia_Western", "𑄃𑄧𑄌𑄴𑄑𑄳𑄢𑄬𑄣𑄨𑄠𑄧 𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Azerbaijan", "𑄃𑄎𑄢𑄴𑄝𑄭𑄎𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Azores", "𑄃𑄬𑄎𑄮𑄢𑄬𑄌𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Bangladesh", "𑄝𑄁𑄣𑄘𑄬𑄌𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Brasilia", "𑄝𑄳𑄢𑄥𑄨𑄣𑄨𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cape_Verde", "𑄇𑄬𑄛𑄴 𑄞𑄢𑄴𑄘𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Chatham", "𑄌𑄳𑄠𑄗𑄟𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Chile", "𑄌𑄨𑄣𑄨 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"China", "𑄌𑄨𑄚𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Colombia", "𑄇𑄧𑄣𑄧𑄟𑄴𑄝𑄨𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cook", "𑄇𑄪𑄇𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄃𑄧𑄖𑄴𑄙𑄬𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Cuba", "𑄇𑄨𑄃𑄪𑄝 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Easter", "𑄃𑄨𑄥𑄴𑄑𑄢𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Central", "𑄟𑄧𑄖𑄴𑄙𑄳𑄠 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Europe_Western", "𑄛𑄧𑄏𑄬𑄟𑄬𑄘𑄨 𑄃𑄨𑄃𑄪𑄢𑄮𑄝𑄮𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Falkland", "𑄜𑄧𑄇𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Fiji", "𑄜𑄨𑄎𑄨 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Georgia", "𑄎𑄧𑄢𑄴𑄎𑄨𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Greenland_Eastern", "𑄛𑄪𑄉𑄬𑄘𑄨 𑄉𑄳𑄢𑄨𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Greenland_Western", "𑄛𑄧𑄏𑄨𑄟𑄬𑄘𑄨 𑄉𑄳𑄢𑄨𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hawaii_Aleutian", "𑄦𑄧𑄃𑄮𑄠𑄭-𑄃𑄣𑄬𑄃𑄪𑄖𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hong_Kong", "𑄦𑄧𑄁 𑄇𑄧𑄁 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Hovd", "𑄦𑄮𑄞𑄧𑄓𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Iran", "𑄃𑄨𑄢𑄚𑄴 𑄘𑄨𑄚𑄬𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Irkutsk", "𑄃𑄨𑄢𑄴𑄖𑄪𑄑𑄴𑄥𑄴𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Israel", "𑄃𑄨𑄎𑄴𑄢𑄠𑄬𑄣𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Japan", "𑄎𑄛𑄚𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Kamchatka", "𑄛𑄨𑄖𑄳𑄢𑄬𑄛𑄳𑄠𑄞𑄧𑄣𑄧𑄥𑄴𑄇𑄴-𑄇𑄳𑄠𑄟𑄴𑄌𑄳𑄠𑄑𑄴𑄃𑄨𑄥𑄴𑄇𑄨 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Korea", "𑄇𑄮𑄢𑄨𑄠𑄚𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Krasnoyarsk", "𑄇𑄳𑄢𑄥𑄴𑄚𑄮𑄠𑄢𑄴𑄥𑄴𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Lord_Howe", "𑄣𑄧𑄢𑄴𑄓𑄴 𑄦𑄤𑄬 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Macau", "𑄟𑄇𑄃𑄮 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Magadan", "𑄟𑄳𑄠𑄉𑄓𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mauritius", "𑄟𑄧𑄢𑄨𑄥𑄥𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mexico_Pacific", "𑄟𑄬𑄇𑄴𑄥𑄨𑄇𑄚𑄴 𑄛𑄳𑄢𑄧𑄥𑄚𑄴𑄖𑄧 𑄟𑄧𑄦𑄥𑄉𑄧𑄢𑄧𑄢𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Mongolia", "𑄃𑄪𑄣𑄚𑄴 𑄝𑄖𑄮𑄢𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Moscow", "𑄟𑄧𑄥𑄴𑄇𑄮 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"New_Caledonia", "𑄚𑄨𑄃𑄪 𑄇𑄳𑄠𑄣𑄬𑄓𑄮𑄚𑄨𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"New_Zealand", "𑄚𑄨𑄃𑄪𑄎𑄨𑄣𑄳𑄠𑄚𑄴𑄓𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Newfoundland", "𑄚𑄨𑄃𑄪𑄜𑄃𑄪𑄚𑄴𑄣𑄳𑄠𑄚𑄴𑄓𑄨 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Norfolk", "𑄚𑄧𑄢𑄴𑄜𑄮𑄇𑄴 𑄉𑄭 𑄉𑄭 𑄞𑄨𑄘𑄬𑄉𑄪𑄚𑄮𑄢𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Noronha", "𑄜𑄢𑄴𑄚𑄚𑄴𑄘𑄮𑄓𑄨 𑄚𑄮𑄢𑄮𑄚𑄴𑄦 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Novosibirsk", "𑄚𑄮𑄞𑄮𑄥𑄨𑄝𑄨𑄢𑄴𑄥𑄴𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Omsk", "𑄃𑄮𑄟𑄧𑄥𑄴𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pakistan", "𑄛𑄇𑄨𑄥𑄴𑄖𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Paraguay", "𑄛𑄳𑄠𑄢𑄉𑄪𑄠𑄬 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Peru", "𑄛𑄬𑄢𑄪 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Philippines", "𑄜𑄨𑄣𑄨𑄛𑄭𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Pierre_Miquelon", "𑄥𑄬𑄚𑄴𑄑𑄴 𑄛𑄨𑄠𑄬𑄢𑄴 𑄃𑄮 𑄟𑄨𑄇𑄬𑄣𑄧𑄚𑄴 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Qyzylorda", "𑄇𑄨𑄎𑄨𑄣𑄮𑄢𑄴𑄓 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Sakhalin", "𑄥𑄈𑄣𑄨𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Samara", "𑄥𑄟𑄢 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Samoa", "𑄥𑄟𑄮𑄠 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Taipei", "𑄖𑄭𑄛𑄳𑄆𑄬 𑄘𑄨𑄚𑄮𑄃𑄣𑄮𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Tonga", "𑄑𑄮𑄋𑄴𑄉 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Turkmenistan", "𑄖𑄪𑄢𑄴𑄇𑄴𑄟𑄬𑄚𑄨𑄌𑄴𑄖𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Uruguay", "𑄃𑄪𑄢𑄪𑄉𑄪𑄠𑄬 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Uzbekistan", "𑄃𑄪𑄎𑄴𑄝𑄬𑄇𑄨𑄖𑄚𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Vanuatu", "𑄞𑄚𑄪𑄠𑄖𑄪 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Vladivostok", "𑄝𑄳𑄣𑄘𑄨𑄝𑄮𑄥𑄴𑄖𑄮𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Volgograd", "𑄞𑄧𑄣𑄴𑄉𑄮𑄉𑄳𑄢𑄓𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Yakutsk", "𑄠𑄇𑄪𑄖𑄴𑄥𑄴𑄇𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}, {"Yekaterinburg", "𑄃𑄨𑄠𑄬𑄇𑄖𑄬𑄢𑄨𑄚𑄴𑄝𑄪𑄢𑄴𑄉𑄴 𑄉𑄧𑄢𑄧𑄟𑄴𑄇𑄣𑄧𑄢𑄴 𑄃𑄧𑄇𑄴𑄖𑄧"}},
	},
	{
		{{"Afghanistan", "ОвхӀан"}, {"Africa_Central", "Юккъера Африка"}, {"Africa_Eastern", "Малхбален Африка"}, {"Africa_Southern", "Къилба Африка"}, {"Africa_Western", "Малхбузен Африка"}, {"Alaska", "Аляска, стандартан хан"}, {"Amazon", "Амазонка, стандартан хан"}, {"America_Central", "Юккъера Америка, стандартан хан"}, {"America_Eastern", "Малхбален Америка, стандартан хан"}, {"America_Mountain", "Лаьмнийн стандартан хан (АЦШ)"}, {"America_Pacific", "Тийна океанан стандартан хан"}, {"Apia", "стандартан хан Апиа, Самоа"}, {"Arabian", "СаӀудийн Ӏаьрбийчоьнан стандартан хан"}, {"Argentina", "Аргентина, стандартан хан"}, {"Argentina_Western", "Малхбузен Аргентина, стандартан хан"}, {"Armenia", "Эрмалойчоь, стандартан хан"}, {"Atlantic", "Атлантикан стандартан хан"}, {"Australia_Central", "Юккъера Австрали, стандартан хан"}, {"Australia_CentralWestern", "Юккъера Австрали, малхбузен стандартан хан"}, {"Australia_Eastern", "Малхбален Австрали, стандартан хан"}, {"Australia_Western", "Малхбузен Австрали, стандартан хан"}, {"Azerbaijan", "Азербайджан, стандартан хан"}, {"Azores", "Азоран гӀайренаш, стандартан хан"}, {"Bangladesh", "Бангладеш, стандартан хан"}, {"Bhutan", "Бутан"}, {"Bolivia", "Боливи"}, {"Brasilia", "Бразили, стандартан хан"}, {"Brunei", "Бруней-Даруссалам"}, {"Cape_Verde", "Кабо-Верде, стандартан хан"}, {"Chamorro", "Чаморро"}, {"Chatham", "Чатем, стандартан хан"}, {"Chile", "Чили, стандартан хан"}, {"China", "Цийчоьнан, стандартан хан"}, {"Christmas", "Ӏийса пайхамар винчу ден гӀайре"}, {"Cocos", "Кокосийн, гӀ-наш"}, {"Colombia", "Колумби, стандартан хан"}, {"Cook", "Кукан, гӀ-наш, стандартан хан"}, {"Cuba", "Куба, стандартан хан"}, {"Davis", "Дейвис"}, {"DumontDUrville", "Дюмон-д’Юрвиль"}, {"East_Timor", "Малхбален Тимор"}, {"Easter", "Мархин гӀайре, стандартан хан"}, {"Ecuador", "Эквадор"}, {"Europe_Central", "Юккъера Европа, стандартан хан"}, {"Europe_Eastern", "Малхбален Европа, стандартан хан"}, {"Europe_Further_Eastern", "Калининградера а, Минскера а хан"}, {"Europe_Western", "Малхбузен Европа, стандартан хан"}, {"Falkland", "Фолклендан гӀайренаш, стандартан хан"}, {"Fiji", "Фиджи, стандартан хан"}, {"French_Guiana", "Французийн Гвиана"}, {"French_Southern", "Французийн къилба а, Антарктидан а хан"}, {"GMT", "Гринвичица юкъара хан"}, {"Galapagos", "Галапагосан гӀайренаш"}, {"Gambier", "Гамбье"}, {"Georgia", "Гуьржийчоь, стандартан хан"}, {"Gilbert_Islands", "Гилбертан, гӀ-наш"}, {"Greenland_Eastern", "Малхбален Гренланди, стандартан хан"}, {"Greenland_Western", "Малхбузен Гренланди, стандартан хан"}, {"Gulf", "ГӀажарийн айма"}, {"Guyana", "Гайана"}, {"Hawaii", "Гавайн-алеутийн стандартан хан"}, {"Hawaii_Aleutian", "Гавайн-алеутийн стандартан хан"}, {"Hong_Kong", "Гонконг, стандартан хан"}, {"Hovd", "Ховд, стандартан хан"}, {"India", "ХӀинди"}, {"Indian_Ocean", "Индин океан"}, {"Indochina", "Индокитай"}, {"Indonesia_Central", "Юккъера Индонези"}, {"Indonesia_Eastern", "Малхбален Индонези"}, {"Indonesia_Western", "Малхбузен Индонези"}, {"Iran", "ГӀажарийчоь, стандартан хан"}, {"Irkutsk", "Иркутск, стандартан хан"}, {"Israel", "Израиль, стандартан хан"}, {"Japan", "Япон, стандартан хан"}, {"Kazakhstan_Eastern", "Малхбален Казахстан"}, {"Kazakhstan_Western", "Малхбузен Казахстан"}, {"Korea", "Корей, стандартан хан"}, {"Kosrae", "Косраэ"}, {"Krasnoyarsk", "Красноярск, стандартан хан"}, {"Kyrgystan", "Киргизи"}, {"Line_Islands", "Лайн, гӀ-наш"}, {"Lord_Howe", "Лорд-Хау, стандартан хан"}, {"Magadan", "Магадан, стандартан хан"}, {"Malaysia", "Малайзи"}, {"Maldives", "Мальдиваш"}, {"Marquesas", "Маркизан, гӀ-наш"}, {"Marshall_Islands", "Маршалан , гӀ-наш"}, {"Mauritius", "Маврики, стандартан хан"}, {"Mawson", "Моусон"}, {"Mexico_Pacific", "Тийна океанан Мексикан стандартан хан"}, {"Mongolia", "Улан-Батор, стандартан хан"}, {"Moscow", "Москва, стандартан хан"}, {"Myanmar", "Мьянма"}, {"Nauru", "Науру"}, {"Nepal", "Непал"}, {"New_Caledonia", "Керла Каледони, стандартан хан"}, {"New_Zealand", "Керла Зеланди, стандартан хан"}, {"Newfoundland", "Ньюфаундленд, стандартан хан"}, {"Niue", "Ниуэ"}, {"Norfolk", "Норфолк, стандартан хан"}, {"Noronha", "Фернанду-ди-Норонья, стандартан хан"}, {"Novosibirsk", "Новосибирск, стандартан хан"}, {"Omsk", "Омск, стандартан хан"}, {"Pakistan", "Пакистан, стандартан хан"}, {"Palau", "Палау"}, {"Papua_New_Guinea", "Папуа – Керла Гвиней"}, {"Paraguay", "Парагвай, стандартан хан"}, {"Peru", "Перу, стандартан хан"}, {"Philippines", "Филиппинаш, стандартан хан"}, {"Phoenix_Islands", "Феникс, гӀ-наш"}, {"Pierre_Miquelon", "Сен-Пьер а, Микелон а, стандартан хан"}, {"Pitcairn", "Питкэрн"}, {"Ponape", "Понапе, гӀ-наш"}, {"Pyongyang", "Пхеньян"}, {"Reunion", "Реюньон"}, {"Rothera", "Ротера"}, {"Sakhalin", "Сахалин, стандартан хан"}, {"Samoa", "Самоа, стандартан хан"}, {"Seychelles", "Сейшелан гӀайренаш"}, {"Singapore", "Сингапур"}, {"Solomon", "Соломонан, гӀ-наш"}, {"South_Georgia", "Къилба Георги"}, {"Suriname", "Суринам"}, {"Syowa", "Сёва"}, {"Tahiti", "Таити, гӀ-наш"}, {"Taipei", "Тайвань, стандартан хан"}, {"Tajikistan", "Таджикистан"}, {"Tokelau", "Токелау"}, {"Tonga", "Тонга, стандартан хан"}, {"Truk", "Чуук"}, {"Turkmenistan", "Туркменин стандартан хан"}, {"Tuvalu", "Тувалу"}, {"Uruguay", "Уругвай, стандартан хан"}, {"Uzbekistan", "Узбекистанан стандартан хан"}, {"Vanuatu", "Вануату, стандартан хан"}, {"Venezuela", "Венесуэла"}, {"Vladivostok", "Владивосток, стандартан хан"}, {"Volgograd", "Волгоград, стандартан хан"}, {"Vostok", "Восток"}, {"Wake", "Уэйк, гӀ-е"}, {"Wallis", "Уоллис а, Футуна а"}, {"Yakutsk", "Якутск, стандартан хан"}, {"Yekaterinburg", "Екатеринбург, стандартан хан"}},
		{{"Alaska", "Аляска, аьхкенан хан"}, {"Amazon", "Амазонка, аьхкенан хан"}, {"America_Central", "Юккъера Америка, аьхкенан хан"}, {"America_Eastern", "Малхбален Америка, аьхкенан хан"}, {"America_Mountain", "Лаьмнийн аьхкенан хан (АЦШ)"}, {"America_Pacific", "Тийна океанан аьхкенан хан"}, {"Apia", "аьхкенан хан Апиа, Самоа"}, {"Arabian", "СаӀудийн Ӏаьрбийчоьнан, аьхкенан хан"}, {"Argentina", "Аргентина, аьхкенан хан"}, {"Argentina_Western", "Малхбузен Аргентина, аьхкенан хан"}, {"Armenia", "Эрмалойчоь, аьхкенан хан"}, {"Atlantic", "Атлантикан аьхкенан хан"}, {"Australia_Central", "Юккъера Австрали, аьхкенан хан"}, {"Australia_CentralWestern", "Юккъера Австрали, малхбузен аьхкенан хан"}, {"Australia_Eastern", "Малхбален Австрали, аьхкенан хан"}, {"Australia_Western", "Малхбузен Австрали, аьхкенан хан"}, {"Azerbaijan", "Азербайджан, аьхкенан хан"}, {"Azores", "Азоран гӀайренаш, аьхкенан хан"}, {"Bangladesh", "Бангладеш, аьхкенан хан"}, {"Brasilia", "Бразили, аьхкенан хан"}, {"Cape_Verde", "Кабо-Верде, аьхкенан хан"}, {"Chatham", "Чатем, аьхкенан хан"}, {"Chile", "Чили, аьхкенан хан"}, {"China", "Цийчоьнан, аьхкенан хан"}, {"Colombia", "Колумби, аьхкенан хан"}, {"Cook", "Кукан, гӀ-наш, аьхкенан хан"}, {"Cuba", "Куба, аьхкенан хан"}, {"Easter", "Мархин гӀайре, аьхкенан хан"}, {"Europe_Central", "Юккъера Европа, аьхкенан хан"}, {"Europe_Eastern", "Малхбален Европа, аьхкенан хан"}, {"Europe_Western", "Малхбузен Европа, аьхкенан хан"}, {"Falkland", "Фолклендан гӀайренаш, аьхкенан хан"}, {"Fiji", "Фиджи, аьхкенан хан"}, {"Georgia", "Гуьржийчоь, аьхкенан хан"}, {"Greenland_Eastern", "Малхбален Гренланди, аьхкенан хан"}, {"Greenland_Western", "Малхбузен Гренланди, аьхкенан хан"}, {"Hawaii_Aleutian", "Гавайн-алеутийн аьхкенан хан"}, {"Hong_Kong", "Гонконг, аьхкенан хан"}, {"Hovd", "Ховд, аьхкенан хан"}, {"Iran", "ГӀажарийчоь, аьхкенан хан"}, {"Irkutsk", "Иркутск, аьхкенан хан"}, {"Israel", "Израиль, аьхкенан хан"}, {"Japan", "Япон, аьхкенан хан"}, {"Korea", "Корей, аьхкенан хан"}, {"Krasnoyarsk", "Красноярск, аьхкенан хан"}, {"Lord_Howe", "Лорд-Хау, аьхкенан хан"}, {"Magadan", "Магадан, аьхкенан хан"}, {"Mauritius", "Маврики, аьхкенан хан"}, {"Mexico_Pacific", "Тийна океанан Мексикан аьхкенан хан"}, {"Mongolia", "Улан-Батор, аьхкенан хан"}, {"Moscow", "Москва, аьхкенан хан"}, {"New_Caledonia", "Керла Каледони, аьхкенан хан"}, {"New_Zealand", "Керла Зеланди, аьхкенан хан"}, {"Newfoundland", "Ньюфаундленд, аьхкенан хан"}, {"Norfolk", "Норфолк, аьхкенан хан"}, {"Noronha", "Фернанду-ди-Норонья, аьхкенан хан"}, {"Novosibirsk", "Новосибирск, аьхкенан хан"}, {"Omsk", "Омск, аьхкенан хан"}, {"Pakistan", "Пакистан, аьхкенан хан"}, {"Paraguay", "Парагвай, аьхкенан хан"}, {"Peru", "Перу, аьхкенан хан"}, {"Philippines", "Филиппинаш, аьхкенан хан"}, {"Pierre_Miquelon", "Сен-Пьер а, Микелон а, аьхкенан хан"}, {"Sakhalin", "Сахалин, аьхкенан хан"}, {"Samoa", "Самоа, аьхкенан хан"}, {"Taipei", "Тайвань, аьхкенан хан"}, {"Tonga", "Тонга, аьхкенан хан"}, {"Turkmenistan", "Туркменин аьхкенан хан"}, {"Uruguay", "Уругвай, аьхкенан хан"}, {"Uzbekistan", "Узбекистанан аьхкенан хан"}, {"Vanuatu", "Вануату, аьхкенан хан"}, {"Vladivostok", "Владивосток, аьхкенан хан"}, {"Volgograd", "Волгоград, аьхкенан хан"}, {"Yakutsk", "Якутск, аьхкенан хан"}, {"Yekaterinburg", "Екатеринбург, аьхкенан хан"}},
	},
	{
		{{"Afghanistan", "Oras sa Afghanistan"}, {"Africa_Central", "Oras sa Central Africa"}, {"Africa_Eastern", "Oras sa East Africa"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "Oras sa West Africa"}, {"Alaska", "Standard nga Oras sa Alaska"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Standard nga Oras sa Pasipiko"}, {"Apia", "Samoa Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Standard nga Oras sa Atlantic"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Oras sa Bhutan"}, {"Bolivia", "Oras sa Bolivia"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Oras sa Brunei Darussalam"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Oras sa Christmas Island"}, {"Cocos", "Oras sa Cocos Islands"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Oras sa Davis"}, {"DumontDUrville", "Oras sa Dumont-d’Urville"}, {"East_Timor", "Oras sa East Timor"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Oras sa Ecuador"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Oras sa Further-eastern Europe"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "Oras sa French Guiana"}, {"French_Southern", "Oras sa French Southern ug Antarctic"}, {"GMT", "Oras sa Greenwich Mean"}, {"Galapagos", "Oras sa Galapagos"}, {"Gambier", "Oras sa Gambier"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Oras sa Gilbert Islands"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Oras sa Guyana"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Oras sa Indian Ocean"}, {"Indochina", "Oras sa Indochina"}, {"Indonesia_Central", "Oras sa Central Indonesia"}, {"Indonesia_Eastern", "Oras sa Eastern Indonesia"}, {"Indonesia_Western", "Oras sa Western Indonesia"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kazakhstan", "Oras sa Kazakhstan"}, {"Kazakhstan_Eastern", "Oras sa East Kazakhstan"}, {"Kazakhstan_Western", "Oras sa West Kazakhstan"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Oras sa Kosrae"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Oras sa Kyrgyzstan"}, {"Line_Islands", "Oras sa Line Islands"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Oras sa Malaysia"}, {"Maldives", "Oras sa Maldives"}, {"Marquesas", "Oras sa Marquesas"}, {"Marshall_Islands", "Oras sa Marshall Islands"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Oras sa Mawson"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Oras sa Myanmar"}, {"Nauru", "Oras sa Nauru"}, {"Nepal", "Oras sa Nepal"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Oras sa Niue"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Oras sa Palau"}, {"Papua_New_Guinea", "Oras sa Papua New Guinea"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Oras sa Phoenix Islands"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Standard Time"}, {"Pitcairn", "Oras sa Pitcairn"}, {"Ponape", "Oras sa Ponape"}, {"Pyongyang", "Oras sa Pyongyang"}, {"Reunion", "Oras sa Reunion"}, {"Rothera", "Oras sa Rothera"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Oras sa Seychelles"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Oras sa Solomon Islands"}, {"South_Georgia", "Oras sa South Georgia"}, {"Suriname", "Oras sa Suriname"}, {"Syowa", "Oras sa Syowa"}, {"Tahiti", "Oras sa Tahiti"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Oras sa Tajikistan"}, {"Tokelau", "Oras sa Tokelau"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Oras sa Chuuk"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Oras sa Tuvalu"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Oras sa Venezuela"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Oras sa Vostok"}, {"Wake", "Oras sa Wake Island"}, {"Wallis", "Oras sa Wallis & Futuna"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Oras sa Yukon"}},
		{{"Alaska", "Daylight nga Oras sa Alaska"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Daylight nga Oras sa Pasipiko"}, {"Apia", "Samoa Daylight Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Daylight nga Oras sa Atlantic"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Daylight Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samoa", "American Samoa Daylight Time"}, {"Taipei", "Taiwan Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Afghanistan", "ᎠᏫᎨᏂᏍᏖᏂ ᎠᏟᎢᎵᏒ"}, {"Africa_Central", "ᎠᏰᏟ ᎬᎿᎨᏍᏛ ᎠᏟᎢᎵᏒ"}, {"Africa_Eastern", "ᏗᎧᎸᎬ ᎬᎿᎨᏍᏛ ᎠᏟᎢᎵᏒ"}, {"Africa_Southern", "ᏧᎦᎾᏮ ᎬᎿᎨᏍᏛ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Africa_Western", "ᏭᏕᎵᎬ ᎬᎿᎨᏍᏛ ᎠᏟᎢᎵᏒ"}, {"Alaska", "ᎠᎳᏍᎦ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Amazon", "ᎠᎺᏌᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"America_Central", "ᎠᏰᏟ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"America_Eastern", "ᏗᎧᎸᎬ ᏗᏜ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"America_Mountain", "ᎣᏓᎸ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"America_Pacific", "ᏭᏕᎵᎬ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Apia", "ᎠᏈᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Arabian", "ᎠᎴᏈᏯ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Argentina", "ᎠᏥᏂᏘᏂᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Argentina_Western", "ᏭᏕᎵᎬ ᏗᏜ ᎠᏥᏂᏘᏂᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Armenia", "ᎠᎵᎻᏂᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Atlantic", "ᏗᎧᎸᎬ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Australia_Central", "ᎠᏰᏟ ᎡᎳᏗᏜ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Australia_CentralWestern", "ᎠᏰᏟ ᎡᎳᏗᏜ ᏭᏕᎵᎬ ᏗᏜ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Australia_Eastern", "ᎡᎳᏗᏜ ᏗᎧᎸᎬ ᏗᏜ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Australia_Western", "ᎡᎳᏗᏜ ᏭᏕᎵᎬ ᏗᏜ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Azerbaijan", "ᎠᏏᎵᏆᏌᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Azores", "ᎠᏐᎴᏏ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Bangladesh", "ᏆᏂᎦᎵᏕᏍ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Bhutan", "ᏊᏔᏂ ᎠᏟᎢᎵᏒ"}, {"Bolivia", "ᏉᎵᏫᎠ ᎠᏟᎢᎵᏒ"}, {"Brasilia", "ᏆᏏᎵᏯ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Brunei", "ᏊᎾᎢ ᏓᎷᏌᎳᎻ ᎠᏟᎢᎵᏒ"}, {"Cape_Verde", "ᎢᎬᎾᏕᎾ ᎢᏤᏳᏍᏗ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Chamorro", "ᏣᎼᎶ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Chatham", "ᏣᏝᎻ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Chile", "ᏥᎵ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"China", "ᏓᎶᏂᎨᏍᏛ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Christmas", "ᏓᏂᏍᏓᏲᎯᎲ ᎤᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Cocos", "ᎪᎪᏍ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Colombia", "ᎪᎸᎻᏈᎢᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Cook", "ᎠᏓᏍᏓᏴᎲᏍᎩ ᏚᎦᏚᏛᎢ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Cuba", "ᎫᏆ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Davis", "ᏕᏫᏏ ᎠᏟᎢᎵᏒ"}, {"DumontDUrville", "ᏚᎼᎾᏘ-Ꮧ’ᎤᎵᏫᎵ ᎠᏟᎢᎵᏒ"}, {"East_Timor", "ᏗᎧᎸᎬ ᏘᎼᎵ ᎠᏟᎢᎵᏒ"}, {"Easter", "ᏥᏌᏕᎴᎯᏌᏅ ᎤᎦᏚᏛᎢ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Ecuador", "ᎡᏆᏙᎵ ᎠᏟᎢᎵᏒ"}, {"Europe_Central", "ᎠᏰᏟ ᏳᎳᏈ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Europe_Eastern", "ᏗᎧᎸᎬ ᏗᏜ ᏳᎳᏈ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Europe_Further_Eastern", "ᏗᎧᎸᎬ ᏳᎳᏈ ᎠᏟᎢᎵᏒ"}, {"Europe_Western", "ᏭᏕᎵᎬ ᏗᏜ ᏳᎳᏈ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Falkland", "ᏩᎩ ᏚᎦᏚᏛᎢ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Fiji", "ᏫᏥ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"French_Guiana", "ᎠᏂᎦᎸ ᏈᏯᎾ ᎠᏟᎢᎵᏒ"}, {"French_Southern", "ᎠᏂᎦᎸᏥ ᎤᎦᏃᏮ & ᎤᏁᏍᏓᎶ ᎠᏟᎢᎵᏒ"}, {"GMT", "ᎢᏤ ᎢᏳᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Galapagos", "ᎡᏆ ᏓᎦᏏ ᎤᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Gambier", "ᎦᎻᏇᎵ ᎠᏟᎢᎵᏒ"}, {"Georgia", "ᏣᎠᏥᎢ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Gilbert_Islands", "ᎩᎵᏇᏘ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Greenland_Eastern", "ᏗᎧᎸᎬ ᎢᏤᏍᏛᏱ ᎠᏟᎶᏍᏗ ᎠᎵᎢᎵᏒ"}, {"Greenland_Western", "ᏭᏕᎵᎬ ᎢᏤᏍᏛᏱ ᎠᏟᎶᏍᏗ ᎠᎵᎢᎵᏒ"}, {"Gulf", "ᎡᏉᏄᎸᏗ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Guyana", "ᎦᏯᎾ ᎠᏟᎢᎵᏒ"}, {"Hawaii", "ᎭᏩᏱ-ᎠᎵᏳᏏᎠᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Hawaii_Aleutian", "ᎭᏩᏱ-ᎠᎵᏳᏏᎠᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Hong_Kong", "ᎰᏂᎩ ᎪᏂᎩ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Hovd", "ᎰᏩᏗ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"India", "ᎢᏂᏗᎢᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Indian_Ocean", "ᎠᏂᏴᏫᏯ ᎠᎺᏉᎯ ᎠᏟᎢᎵᏒ"}, {"Indochina", "ᎢᏂᏙᏓᎶᏂᎨᏍᏛ ᎠᏟᎢᎵᏒ"}, {"Indonesia_Central", "ᎠᏰᏟ ᎢᏂᏙᏂᏍᏯ ᎠᏟᎢᎵᏒ"}, {"Indonesia_Eastern", "ᏗᎧᎸᎬ ᏗᏜ ᎢᏂᏙᏂᏍᏯ ᎠᏟᎢᎵᏒ"}, {"Indonesia_Western", "ᏭᏕᎵᎬ ᏗᏜ ᎢᏂᏙᏂᏍᏯ ᎠᏟᎢᎵᏒ"}, {"Iran", "ᎢᎳᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Irkutsk", "ᎢᎫᏥᎧ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Israel", "ᎢᏏᎵᏱ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Japan", "ᏣᏩᏂᏏ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Kazakhstan", "ᎧᏎᎧᏍᏕᏂ ᎠᏟᎢᎵᏒ"}, {"Kazakhstan_Eastern", "ᏗᎧᎸᎬ ᎧᏎᎧᏍᏕᏂ ᎠᏟᎢᎵᏒ"}, {"Kazakhstan_Western", "ᏭᏕᎵᎬ ᎧᏎᎧᏍᏕᏂ ᎠᏟᎢᎵᏒ"}, {"Korea", "ᎪᎵᎠᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Kosrae", "ᎪᏍᎴ ᎠᏟᎢᎵᏒ"}, {"Krasnoyarsk", "ᏝᏍᏃᏯᏍᎧ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Kyrgystan", "ᎩᎵᏣᎢᏍ ᎠᏟᎢᎵᏒ"}, {"Line_Islands", "ᎠᏍᏓᏅᏅ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Lord_Howe", "ᎤᎬᏫᏳᎯ ᎭᏫ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Magadan", "ᎹᎦᏓᏂ ᎠᏟᎢᎵᏒ ᎠᏟᎢᎵᏒ"}, {"Malaysia", "ᎹᎴᏏᎢᎠ ᎠᏟᎢᎵᏒ"}, {"Maldives", "ᎹᎵᏗᏫᏍ ᎠᏟᎢᎵᏒ"}, {"Marquesas", "ᎹᎵᎨᏌᏏ ᎠᏟᎢᎵᏒ"}, {"Marshall_Islands", "ᎹᏌᎵ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Mauritius", "ᎼᎵᏏᎥᏍ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Mawson", "ᎹᏌᏂ ᎠᏟᎢᎵᏒ"}, {"Mexico_Pacific", "ᎠᏂᏍᏆᏂ ᏭᏕᎵᎬ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Mongolia", "ᎤᎳᏂ ᏆᏙᎸ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Moscow", "ᎹᏍᎦᏫ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Myanmar", "ᎹᏯᎹᎵ ᎠᏟᎢᎵᏒ"}, {"Nauru", "ᎾᎷ ᎠᏟᎢᎵᏒ"}, {"Nepal", "ᏁᏆᎵ ᎠᏟᎢᎵᏒ"}, {"New_Caledonia", "ᎢᏤ ᎧᎵᏙᏂᎠᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"New_Zealand", "ᎢᏤ ᏏᎢᎴᏂᏗ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Newfoundland", "ᎢᏤᎤᏂᏩᏛᏓᎦᏙᎯ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Niue", "ᏂᏳ ᎠᏟᎢᎵᏒ"}, {"Norfolk", "ᏃᎵᏬᎵᎩ ᎤᎦᏚᏛᎢ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Noronha", "ᏪᎾᏅᏙ Ꮥ ᏃᎶᎾᎭ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Novosibirsk", "ᏃᏬᏏᏈᏍᎧ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Omsk", "ᎣᎻᏍᎧ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Pakistan", "ᏆᎩᏍᏖᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Palau", "ᏆᎷ ᎠᏟᎢᎵᏒ"}, {"Papua_New_Guinea", "ᏆᏇ ᎢᏤ ᎩᎢᏂ ᎠᏟᎢᎵᏒ"}, {"Paraguay", "ᏆᎵᏇ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Peru", "ᏇᎷ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Philippines", "ᎠᏂᏈᎵᎩᏃ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Phoenix_Islands", "ᏧᎴᎯᏌᏅᎯ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Pierre_Miquelon", "ᎤᏓᏅᏘ ᏈᏰ & ᎻᏇᎶᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Pitcairn", "ᏈᎧᎵᏂ ᎠᏟᎢᎵᏒ"}, {"Ponape", "ᏉᎾᏇ ᎠᏟᎢᎵᏒ"}, {"Pyongyang", "ᏈᏯᏂᎩᏰᏂᎩ ᎠᏟᎢᎵᏒ"}, {"Reunion", "ᎴᏳᏂᎠᏂ ᎠᏟᎢᎵᏒ"}, {"Rothera", "ᎳᏞᎳ ᎠᏟᎢᎵᏒ"}, {"Sakhalin", "ᏌᎧᎵᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Samoa", "ᏌᎼᎠ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Seychelles", "ᏎᏤᎴᏏ ᎠᏟᎢᎵᏒ"}, {"Singapore", "ᏏᏂᎦᏉᎵ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Solomon", "ᏐᎶᎹᏂ ᏚᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"South_Georgia", "ᏧᎦᎾᏮ ᏣᎠᏥᎢ ᎠᏟᎢᎵᏒ"}, {"Suriname", "ᏒᎵᎾᎻ ᎠᏟᎢᎵᏒ"}, {"Syowa", "ᏏᏲᏩ ᎠᏟᎢᎵᏒ"}, {"Tahiti", "ᏔᎯᏘ ᎠᏟᎢᎵᏒ"}, {"Taipei", "ᏔᏱᏇ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Tajikistan", "ᏔᏥᎩᏍᏕᏂ ᎠᏟᎢᎵᏒ"}, {"Tokelau", "ᏙᎨᎳᎤ ᎠᏟᎢᎵᏒ"}, {"Tonga", "ᏙᎾᎦ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Truk", "ᏧᎩ ᎠᏟᎢᎵᏒ"}, {"Turkmenistan", "ᏛᎵᎩᎺᏂᏍᏔᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Tuvalu", "ᏚᏩᎷ ᎠᏟᎢᎵᏒ"}, {"Uruguay", "ᏳᎷᏇ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Uzbekistan", "ᎤᏍᏇᎩᏍᏖᏂ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Vanuatu", "ᏩᏄᏩᏚ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Venezuela", "ᏪᏁᏑᏪᎳ ᎠᏟᎢᎵᏒ"}, {"Vladivostok", "ᏭᎳᏗᏬᏍᏙᎩ ᎠᏟᎢᎵᏒ ᎠᏟᎢᎵᏒ"}, {"Volgograd", "ᏬᎶᎪᏝᏗ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Vostok", "ᏬᏍᏙᎧ ᎠᏟᎢᎵᏒ"}, {"Wake", "ᎤᏰᏨ ᎤᎦᏚᏛᎢ ᎠᏟᎢᎵᏒ"}, {"Wallis", "ᏩᎵᏍ ᎠᎴ ᏊᏚᎾ ᎠᏟᎢᎵᏒ"}, {"Yakutsk", "ᏯᎫᏥᎧ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Yekaterinburg", "ᏰᎧᏖᎵᏂᏊᎦ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Yukon", "ᏳᎧᏂ ᎠᏟᎢᎵᏒ"}},
		{{"Alaska", "ᎠᎳᏍᎦ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Amazon", "ᎠᎺᏌᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"America_Central", "ᎠᏰᏟ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"America_Eastern", "ᏗᎧᎸᎬ ᏗᏜ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"America_Mountain", "ᎣᏓᎸ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"America_Pacific", "ᏭᏕᎵᎬ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Apia", "ᎠᏈᎠ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Arabian", "ᎠᎴᏈᏯ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Argentina", "ᎠᏥᏂᏘᏂᎠ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Argentina_Western", "ᏭᏕᎵᎬ ᏗᏜ ᎠᏥᏂᏘᏂᎠ ᎪᎩ ᎠᏟᎶᏍᏗ ᎠᏟᎢᎵᏒ"}, {"Armenia", "ᎠᎵᎻᏂᎠ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Atlantic", "ᏗᎧᎸᎬ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Australia_Central", "ᎠᏰᏟ ᎡᎳᏗᏜ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Australia_CentralWestern", "ᎠᏰᏟ ᎡᎳᏗᏜ ᏭᏕᎵᎬ ᏗᏜ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Australia_Eastern", "ᎡᎳᏗᏜ ᏗᎧᎸᎬ ᏗᏜ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Australia_Western", "ᎡᎳᏗᏜ ᏭᏕᎵᎬ ᏗᏜ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Azerbaijan", "ᎠᏏᎵᏆᏌᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Azores", "ᎠᏐᎴᏏ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Bangladesh", "ᏆᏂᎦᎵᏕᏍ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Brasilia", "ᏆᏏᎵᏯ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Cape_Verde", "ᎢᎬᎾᏕᎾ ᎢᏤᏳᏍᏗ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Chatham", "ᏣᏝᎻ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Chile", "ᏥᎵ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"China", "ᏓᎶᏂᎨᏍᏛ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Colombia", "ᎪᎸᎻᏈᎢᎠ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Cook", "ᎠᏓᏍᏓᏴᎲᏍᎩ ᏚᎦᏚᏛᎢ ᎠᏰᏟ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Cuba", "ᎫᏆ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Easter", "ᏥᏌᏕᎴᎯᏌᏅ ᎤᎦᏚᏛᎢ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Europe_Central", "ᎠᏰᏟ ᏳᎳᏈ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Europe_Eastern", "ᏗᎧᎸᎬ ᏗᏜ ᏳᎳᏈ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Europe_Western", "ᏭᏕᎵᎬ ᏗᏜ ᏳᎳᏈ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Falkland", "ᏩᎩ ᏚᎦᏚᏛᎢ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Fiji", "ᏫᏥ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Georgia", "ᏣᎠᏥᎢ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Greenland_Eastern", "ᏗᎧᎸᎬ ᎢᏤᏍᏛᏱ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Greenland_Western", "ᏭᏕᎵᎬ ᎢᏤᏍᏛᏱ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Hawaii_Aleutian", "ᎭᏩᏱ-ᎠᎵᏳᏏᎠᏂ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Hong_Kong", "ᎰᏂᎩ ᎪᏂᎩ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Hovd", "ᎰᏩᏗ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Iran", "ᎢᎳᏂ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Irkutsk", "ᎢᎫᏥᎧ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Israel", "ᎢᏏᎵᏱ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒᎩ"}, {"Japan", "ᏣᏩᏂᏏ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Korea", "ᎪᎵᎠᏂ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Krasnoyarsk", "ᏝᏍᏃᏯᏍᎧ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Lord_Howe", "ᎤᎬᏫᏳᎯ ᎭᏫ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Magadan", "ᎹᎦᏓᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Mauritius", "ᎼᎵᏏᎥᏍ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Mexico_Pacific", "ᎠᏂᏍᏆᏂ ᏭᏕᎵᎬ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Mongolia", "ᎤᎳᏂ ᏆᏙᎸ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Moscow", "ᎹᏍᎦᏫ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"New_Caledonia", "ᎢᏤ ᎧᎵᏙᏂᎠᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"New_Zealand", "ᎢᏤ ᏏᎢᎴᏂᏗ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Newfoundland", "ᎢᏤᎤᏂᏩᏛᏓᎦᏙᎯ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Norfolk", "ᏃᎵᏬᎵᎩ ᎤᎦᏚᏛᎢ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Noronha", "ᏪᎾᏅᏙ Ꮥ ᏃᎶᎾᎭ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Novosibirsk", "ᏃᏬᏏᏈᏍᎧ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Omsk", "ᎣᎻᏍᎧ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Pakistan", "ᏆᎩᏍᏖᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Paraguay", "ᏆᎵᏇ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Peru", "ᏇᎷ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Philippines", "ᎠᏂᏈᎵᎩᏃ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Pierre_Miquelon", "ᎤᏓᏅᏘ ᏈᏰ & ᎻᏇᎶᏂ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Sakhalin", "ᏌᎧᎵᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Samoa", "ᏌᎼᎠ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Taipei", "ᏔᏱᏇ ᎪᎯ ᎢᎦ ᎠᏟᎢᎵᏒ"}, {"Tonga", "ᏙᎾᎦ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Turkmenistan", "ᏛᎵᎩᎺᏂᏍᏔᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Uruguay", "ᏳᎷᏇ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Uzbekistan", "ᎤᏍᏇᎩᏍᏖᏂ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Vanuatu", "ᏩᏄᏩᏚ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Vladivostok", "ᏭᎳᏗᏬᏍᏙᎩ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Volgograd", "ᏬᎶᎪᏝᏗ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Yakutsk", "ᏯᎫᏥᎧ ᎪᎩ ᎠᏟᎢᎵᏒ"}, {"Yekaterinburg", "ᏰᎧᏖᎵᏂᏊᎦ ᎪᎩ ᎠᏟᎢᎵᏒ"}},
	},
	{
		{{"GMT", "ora mediana di Greenwich"}},
		{},
	},
	{
		{{"Afghanistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲁϥⲅⲁⲛⲓⲥⲑⲁⲛ"}, {"Africa_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϯⲫⲣⲓⲕⲓⲁ ⲛ̀̀ⲑⲙⲏϯ"}, {"Africa_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϯⲫⲣⲓⲕⲓⲁ ⲙ̀ⲡⲉⲓⲉⲃⲧ"}, {"Africa_Southern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲟϩⲣⲟⲥ ⲛ̀ϯⲫⲣⲓⲕⲓⲁ ⲙ̀ⲡⲓⲣⲏⲥ"}, {"Africa_Western", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ϯⲫⲣⲓⲕⲓⲁ"}, {"Alaska", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲗⲁⲥⲕⲁ"}, {"Amazon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲙⲁⲍⲟⲛ"}, {"America_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲑⲙⲏϯ"}, {"America_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲓⲉⲃⲧ"}, {"America_Mountain", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲓⲧⲱⲟⲩ"}, {"America_Pacific", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲁⲥⲓϥⲓⲕ"}, {"Apia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲥⲁⲙⲟⲁ"}, {"Arabian", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲣⲁⲃⲟⲥ"}, {"Argentina", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲣϫⲉⲛⲑⲓⲛ"}, {"Argentina_Western", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲁⲣϫⲉⲛⲑⲓⲛ"}, {"Armenia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲣⲙⲉⲛⲓⲁ"}, {"Atlantic", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲑⲗⲁⲛⲑⲓⲕ"}, {"Australia_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_CentralWestern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_Western", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Azerbaijan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲍⲉⲣⲃⲁⲓϫⲁⲛ"}, {"Azores", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲁⲍⲟⲣⲉⲥ"}, {"Bangladesh", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲃⲁⲛⲅⲗⲁⲇⲉϣ"}, {"Bhutan", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲃⲟⲩⲑⲁⲛ"}, {"Bolivia", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲃⲟⲗⲓⲃⲓⲁ"}, {"Brasilia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲃⲣⲁⲍⲓⲗ"}, {"Brunei", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲃⲣⲟⲩⲛⲉⲓ"}, {"Cape_Verde", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲕⲁⲡ ⲃⲉⲣⲇⲉ"}, {"Chamorro", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϭⲁⲙⲟⲣⲟ"}, {"Chatham", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϣⲁⲑⲁⲙ"}, {"Chile", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϭⲓⲗⲉ"}, {"China", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϭⲓⲛⲁ"}, {"Christmas", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲙⲟⲩⲓ ⲛ̀ⲭⲣⲓⲥⲧⲙⲁⲥ"}, {"Cocos", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲕⲟⲕⲟⲥ"}, {"Colombia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲕⲟⲗⲟⲙⲃⲓⲁ"}, {"Cook", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲕⲟⲩⲕ"}, {"Cuba", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲧⲉ ⲕⲟⲩⲃⲁ"}, {"Davis", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲇⲁⲩⲓⲥ"}, {"DumontDUrville", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲇⲟⲩⲣⲙⲟⲛⲧ ⲇⲟⲩⲣⲃⲓⲗ"}, {"East_Timor", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲓⲙⲟⲣ-ⲗⲉⲥⲧ"}, {"Easter", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲥⲑⲉⲣ ⲁⲓⲥⲗⲁⲛⲇ"}, {"Ecuador", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲉⲕⲟⲩⲁⲇⲟⲣ"}, {"Europe_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Europe_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Europe_Further_Eastern", "ⲟⲩⲕⲉ ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Europe_Western", "ⲡⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Falkland", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ϥⲟⲕⲗⲁⲛⲇ"}, {"Fiji", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϥⲓⲇϫⲓ"}, {"French_Guiana", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲅⲩⲁⲛⲁ ⲛ̀ϥⲉⲣⲉⲛⲥⲟⲥ"}, {"French_Southern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϥⲉⲣⲉⲛⲥⲟⲥ ⲙ̀ⲡⲓⲣⲏⲥ & ⲁⲛⲧⲁⲣⲕⲑⲓⲕⲁ"}, {"GMT", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲙⲏϯ ⲛ̀ⲅⲣⲓⲛⲓϭ"}, {"Galapagos", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲅⲁⲗⲁⲡⲁⲅⲟⲥ"}, {"Gambier", "̀ⲡⲥⲟⲏⲩ ⲛ̀ⲅⲁⲙⲃⲓⲉⲣ"}, {"Georgia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲅⲉⲟⲣⲅⲓⲁ"}, {"Gilbert_Islands", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ϫⲓⲗⲃⲉⲣⲧ"}, {"Greenland_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲧⲉ ⲅⲣⲓⲛⲗⲁⲛⲇ"}, {"Greenland_Western", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲧⲉ ⲅⲣⲓⲛⲗⲁⲛⲇ"}, {"Gulf", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲅⲟⲗϥ"}, {"Guyana", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲅⲩⲁⲛⲁ"}, {"Hawaii", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϩⲁⲟⲩⲁⲓ-ⲁⲗⲉⲩⲑⲓⲁⲛ"}, {"Hawaii_Aleutian", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϩⲁⲟⲩⲁⲓ-ⲁⲗⲉⲩⲑⲓⲁⲛ"}, {"Hong_Kong", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϩⲟⲛⲅ ⲕⲟⲛⲅ"}, {"Hovd", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϧⲟⲃⲇ"}, {"India", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϩⲉⲛⲧⲟⲩ"}, {"Indian_Ocean", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲱⲕⲉⲁⲛⲟⲥ ⲛ̀ϩⲉⲛⲧⲟⲩ"}, {"Indochina", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲓⲛⲇⲟϭⲓⲛⲁ"}, {"Indonesia_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲓⲛⲇⲟⲛⲉⲥⲓⲁ"}, {"Indonesia_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲓⲛⲇⲟⲛⲉⲥⲓⲁ"}, {"Indonesia_Western", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲓⲛⲇⲟⲛⲉⲥⲓⲁ"}, {"Iran", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲣⲁⲛ"}, {"Irkutsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲣⲕⲟⲩⲧⲥⲕ"}, {"Israel", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲓⲥⲣⲁⲏⲗ"}, {"Japan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲁⲡⲁⲛ"}, {"Kazakhstan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲕⲁⲍⲁϧⲉⲥⲑⲁⲛ"}, {"Kazakhstan_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲕⲁⲍⲁϧⲉⲥⲑⲁⲛ"}, {"Kazakhstan_Western", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲕⲁⲍⲁϧⲉⲥⲑⲁⲛ"}, {"Korea", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲕⲟⲣⲉⲁ"}, {"Kosrae", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲕⲟⲥⲣⲁⲉ"}, {"Krasnoyarsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲕⲣⲁⲥⲛⲟⲓⲁⲣⲥⲕ"}, {"Kyrgystan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲕⲩⲣⲅⲩⲥⲑⲁⲛ"}, {"Line_Islands", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲗⲓⲛ"}, {"Lord_Howe", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲗⲟⲣⲇ ϩⲟⲩⲉ"}, {"Magadan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲙⲁⲅⲁⲇⲁⲛ"}, {"Malaysia", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲁⲗⲉⲍⲓⲁ"}, {"Maldives", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲁⲗⲇⲓⲃ"}, {"Marquesas", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲁⲣⲕⲉⲥⲁⲥ"}, {"Marshall_Islands", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲙ̀ⲙⲁⲣϣⲁⲗ"}, {"Mauritius", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲙⲁⲩⲣⲓⲑⲓⲟⲩⲥ"}, {"Mawson", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲁⲩⲥⲟⲛ"}, {"Mexico_Pacific", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲙⲉⲝⲓⲕⲟ ⲙ̀ⲡⲁⲥⲓϥⲓⲕ"}, {"Mongolia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲟⲩⲗⲁⲛⲃⲁⲧⲁⲣ"}, {"Moscow", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲙⲟⲥⲕⲟ"}, {"Myanmar", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲓⲁⲛⲙⲁⲣ"}, {"Nauru", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲁⲩⲣⲟⲩ"}, {"Nepal", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲉⲡⲁⲗ"}, {"New_Caledonia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲓⲟⲩⲕⲁⲗⲉⲇⲟⲛⲓⲁ"}, {"New_Zealand", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲓⲟⲩⲍⲓⲗⲁⲛⲇⲁ"}, {"Newfoundland", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛⲛ̀ⲓⲟⲩϥⲁⲩⲛⲇⲗⲁⲛⲇ"}, {"Niue", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲟⲩⲉ"}, {"Norfolk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲛⲟⲣϥⲟⲕ"}, {"Noronha", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ϥⲉⲣⲛⲁⲛⲇⲟ ⲇⲉ ⲛⲟⲣⲟⲛϩⲁ"}, {"Novosibirsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲛⲟⲃⲟⲥⲓⲃⲓⲣⲥⲕ"}, {"Omsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲟⲙⲥⲕ"}, {"Pakistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲁⲕⲓⲥⲑⲁⲛ"}, {"Palau", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲙⲡⲁⲗⲁⲩ"}, {"Papua_New_Guinea", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲁⲡⲟⲩⲁ ⲛⲓⲟⲩⲅⲓⲛⲉⲁ"}, {"Paraguay", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲁⲣⲁⲅⲟⲩⲁⲓ"}, {"Peru", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲡⲉⲣⲟⲩ"}, {"Philippines", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲫⲓⲗⲓⲡⲡⲓⲛ"}, {"Phoenix_Islands", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲙ̀ⲫⲓⲛⲓⲝ"}, {"Pierre_Miquelon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲥⲁⲛⲧ ⲡⲓⲉⲣ & ⲙⲓⲕⲉⲗⲟⲛ"}, {"Pitcairn", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲑⲕⲁⲓⲣⲛ"}, {"Ponape", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲟϩⲛⲡⲉⲓ"}, {"Pyongyang", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲕⲟⲣⲉⲁ ⲙ̀ⲡⲉⲙϩⲓⲧ"}, {"Reunion", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲣⲉⲓⲟⲩⲛⲓⲟⲛ"}, {"Rothera", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲣⲟⲑⲉⲣⲁ"}, {"Sakhalin", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲥⲁϧⲁⲗⲓⲛ"}, {"Samoa", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲥⲁⲙⲟⲣⲁ ⲛ̀ⲁⲙⲉⲣⲓⲕⲁⲛⲟⲥ"}, {"Seychelles", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲥⲩϣⲉⲗ"}, {"Singapore", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲥⲓⲛⲅⲁⲡⲟⲣ"}, {"Solomon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲥⲟⲗⲟⲙⲟⲛ"}, {"South_Georgia", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲣⲏⲥ ⲛ̀ⲅⲉⲟⲣⲅⲓⲁ"}, {"Suriname", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲥⲟⲩⲣⲓⲛⲁⲙ"}, {"Syowa", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲥⲩⲟⲩⲁ"}, {"Tahiti", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲁϩⲓⲑⲓ"}, {"Taipei", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲑⲁⲓⲟⲩⲁⲛ"}, {"Tajikistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲁϫⲓⲕⲓⲥⲑⲁⲛ"}, {"Tokelau", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲟⲕⲉⲗⲁⲩ"}, {"Tonga", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲑⲟⲛⲅⲁ"}, {"Truk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϭⲟⲩⲕ"}, {"Turkmenistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲑⲟⲩⲣⲕⲙⲉⲛⲓⲥⲑⲁⲛ"}, {"Tuvalu", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲑⲟⲩⲃⲁⲗⲟⲩ"}, {"Uruguay", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲟⲩⲣⲟⲩⲅⲟⲩⲁⲓ"}, {"Uzbekistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲟⲩⲍⲃⲉⲕⲓⲥⲑⲁⲛ"}, {"Vanuatu", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲃⲁⲛⲁⲑⲟⲩ"}, {"Venezuela", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲃⲉⲛⲉⲍⲟⲩⲉⲗⲁ"}, {"Vladivostok", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲃⲗⲁⲇⲓⲃⲟⲥⲑⲟⲕ"}, {"Volgograd", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲙ̀ⲃⲟⲗⲅⲟⲅⲣⲁⲇ"}, {"Vostok", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲃⲟⲥⲑⲟⲕ"}, {"Wake", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡⲓⲙⲟⲩⲓ ⲛ̀ⲟⲩⲁⲕ"}, {"Wallis", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲟⲩⲁⲗⲓⲥ & ϥⲟⲩⲑⲟⲩⲛⲁ"}, {"Yakutsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲁⲕⲟⲩⲧⲥⲕ"}, {"Yekaterinburg", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϩⲟⲣⲟⲥ ⲛ̀ⲓⲉⲕⲁⲑⲉⲣⲓⲛⲃⲟⲩⲣⲅ"}, {"Yukon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲓⲟⲩⲕⲟⲛ"}},
		{{"Alaska", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲁⲗⲁⲥⲕⲁ"}, {"Amazon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲁⲙⲁⲍⲟⲛ"}, {"America_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲑⲙⲏϯ"}, {"America_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲉⲓⲉⲃⲧ"}, {"America_Mountain", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲓⲧⲱⲟⲩ"}, {"America_Pacific", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲁⲥⲓϥⲓⲕ"}, {"Apia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲥⲁⲙⲟⲁ"}, {"Arabian", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲁⲣⲁⲃⲟⲥ"}, {"Argentina", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲁⲣϫⲉⲛⲑⲓⲛ"}, {"Argentina_Western", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲁⲣϫⲉⲛⲑⲓⲛ"}, {"Armenia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲁⲣⲙⲉⲛⲓⲁ"}, {"Atlantic", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲁⲑⲗⲁⲛⲑⲓⲕ"}, {"Australia_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_CentralWestern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Australia_Western", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲁⲩⲥⲧⲣⲁⲗⲓⲁ"}, {"Azerbaijan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲁⲍⲉⲣⲃⲁⲓϫⲁⲛ"}, {"Azores", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲁⲍⲟⲣⲉⲥ"}, {"Bangladesh", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲃⲁⲛⲅⲗⲁⲇⲉϣ"}, {"Brasilia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲃⲣⲁⲍⲓⲗ"}, {"Cape_Verde", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲕⲁⲡ ⲃⲉⲣⲇⲉ"}, {"Chatham", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ϣⲁⲑⲁⲙ"}, {"Chile", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ϭⲓⲗⲉ"}, {"China", "ⲡ̀ⲥⲏⲟⲩ ⲛⲧⲟⲟⲩⲓ ⲛ̀ϭⲓⲛⲁ"}, {"Colombia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲕⲟⲗⲟⲙⲃⲓⲁ"}, {"Cook", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲕⲟⲩⲕ"}, {"Cuba", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲧⲉ ⲕⲟⲩⲃⲁ"}, {"Easter", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲓⲥⲑⲉⲣ ⲁⲓⲥⲗⲁⲛⲇ"}, {"Europe_Central", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀̀ⲑⲙⲏϯ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Europe_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Europe_Western", "ⲡⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲉⲩⲣⲟⲡⲁ"}, {"Falkland", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ϥⲟⲕⲗⲁⲛⲇ"}, {"Fiji", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ϥⲓⲇϫⲓ"}, {"Georgia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲅⲉⲟⲣⲅⲓⲁ"}, {"Greenland_Eastern", "ⲡ̀ⲥⲏⲟⲩ ⲙ̀ⲡϣⲱⲙ ⲙ̀ⲡⲉⲓⲉⲃⲧ ⲛ̀ⲧⲉ ⲅⲣⲓⲛⲗⲁⲛⲇ"}, {"Greenland_Western", "̀ⲡⲥⲏⲟⲩ ⲙ̀̀ⲡϣⲱⲙ ⲙ̀ⲡⲉⲙⲉⲛⲧ ⲛ̀ⲧⲉ ⲅⲣⲓⲛⲗⲁⲛⲇ"}, {"Hawaii_Aleutian", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ϩⲁⲟⲩⲁⲓ-ⲁⲗⲉⲩⲑⲓⲁⲛ"}, {"Hong_Kong", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ϩⲟⲛⲅ ⲕⲟⲛⲅ"}, {"Hovd", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ϧⲟⲃⲇ"}, {"Iran", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲓⲣⲁⲛ"}, {"Irkutsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲓⲣⲕⲟⲩⲧⲥⲕ"}, {"Israel", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲡⲓⲥⲣⲁⲏⲗ"}, {"Japan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲓⲁⲡⲁⲛ"}, {"Korea", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲕⲟⲣⲉⲁ"}, {"Krasnoyarsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲕⲣⲁⲥⲛⲟⲓⲁⲣⲥⲕ"}, {"Lord_Howe", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲗⲟⲣⲇ ϩⲟⲩⲉ"}, {"Magadan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲙⲁⲅⲁⲇⲁⲛ"}, {"Mauritius", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲙⲁⲩⲣⲓⲑⲓⲟⲩⲥ"}, {"Mexico_Pacific", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲙ̀ⲙⲉⲝⲓⲕⲟ ⲙ̀ⲡⲁⲥⲓϥⲓⲕ"}, {"Mongolia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲟⲩⲗⲁⲛⲃⲁⲧⲁⲣ"}, {"Moscow", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲙⲟⲥⲕⲟ"}, {"New_Caledonia", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲛⲓⲟⲩⲕⲁⲗⲉⲇⲟⲛⲓⲁ"}, {"New_Zealand", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲛⲓⲟⲩⲍⲓⲗⲁⲛⲇⲁ"}, {"Newfoundland", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛⲛ̀ⲓⲟⲩϥⲁⲩⲛⲇⲗⲁⲛⲇ"}, {"Norfolk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲛⲓⲙⲟⲩⲓ ⲛ̀ⲛⲟⲣϥⲟⲕ"}, {"Noronha", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ϥⲉⲣⲛⲁⲛⲇⲟ ⲇⲉ ⲛⲟⲣⲟⲛϩⲁ"}, {"Novosibirsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲛⲟⲃⲟⲥⲓⲃⲓⲣⲥⲕ"}, {"Omsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲟⲙⲥⲕ"}, {"Pakistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲁⲕⲓⲥⲑⲁⲛ"}, {"Paraguay", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲁⲣⲁⲅⲟⲩⲁⲓ"}, {"Peru", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲡⲉⲣⲟⲩ"}, {"Philippines", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲫⲓⲗⲓⲡⲡⲓⲛ"}, {"Pierre_Miquelon", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲥⲁⲛⲧ ⲡⲓⲉⲣ & ⲙⲓⲕⲉⲗⲟⲛ"}, {"Sakhalin", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲥⲁϧⲁⲗⲓⲛ"}, {"Samoa", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲥⲁⲙⲟⲣⲁ ⲛ̀ⲁⲙⲉⲣⲓⲕⲁⲛⲟⲥ"}, {"Taipei", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ⲧⲟⲟⲩⲓ ⲛ̀ⲑⲁⲓⲟⲩⲁⲛ"}, {"Tonga", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲑⲟⲛⲅⲁ"}, {"Turkmenistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲑⲟⲩⲣⲕⲙⲉⲛⲓⲥⲑⲁⲛ"}, {"Uruguay", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲟⲩⲣⲟⲩⲅⲟⲩⲁⲓ"}, {"Uzbekistan", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲟⲩⲍⲃⲉⲕⲓⲥⲑⲁⲛ"}, {"Vanuatu", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲃⲁⲛⲁⲑⲟⲩ"}, {"Vladivostok", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲃⲗⲁⲇⲓⲃⲟⲥⲑⲟⲕ"}, {"Volgograd", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲙ̀ⲃⲟⲗⲅⲟⲅⲣⲁⲇ"}, {"Yakutsk", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲓⲁⲕⲟⲩⲧⲥⲕ"}, {"Yekaterinburg", "ⲡ̀ⲥⲏⲟⲩ ⲛ̀ϣⲱⲙ ⲛ̀ⲓⲉⲕⲁⲑⲉⲣⲓⲛⲃⲟⲩⲣⲅ"}},
	},
	{
		{{"Acre", "acrejský standardní čas"}, {"Afghanistan", "afghánský čas"}, {"Africa_Central", "středoafrický čas"}, {"Africa_Eastern", "východoafrický čas"}, {"Africa_Southern", "jihoafrický čas"}, {"Africa_Western", "západoafrický čas"}, {"Alaska", "aljašský standardní čas"}, {"Almaty", "Almatský standardní čas"}, {"Amazon", "amazonský standardní čas"}, {"America_Central", "severoamerický centrální standardní čas"}, {"America_Eastern", "severoamerický východní standardní čas"}, {"America_Mountain", "severoamerický horský standardní čas"}, {"America_Pacific", "severoamerický pacifický standardní čas"}, {"Anadyr", "anadyrský standardní čas"}, {"Apia", "apijský standardní čas"}, {"Aqtau", "Aktauský standardní čas"}, {"Aqtobe", "Aktobský standardní čas"}, {"Arabian", "arabský standardní čas"}, {"Argentina", "argentinský standardní čas"}, {"Argentina_Western", "západoargentinský standardní čas"}, {"Armenia", "arménský standardní čas"}, {"Atlantic", "atlantický standardní čas"}, {"Australia_Central", "středoaustralský standardní čas"}, {"Australia_CentralWestern", "středozápadní australský standardní čas"}, {"Australia_Eastern", "východoaustralský standardní čas"}, {"Australia_Western", "západoaustralský standardní čas"}, {"Azerbaijan", "ázerbájdžánský standardní čas"}, {"Azores", "azorský standardní čas"}, {"Bangladesh", "bangladéšský standardní čas"}, {"Bhutan", "bhútánský čas"}, {"Bolivia", "bolivijský čas"}, {"Brasilia", "brasilijský standardní čas"}, {"Brunei", "brunejský čas"}, {"Cape_Verde", "kapverdský standardní čas"}, {"Casey", "čas Caseyho stanice"}, {"Chamorro", "chamorrský čas"}, {"Chatham", "chathamský standardní čas"}, {"Chile", "chilský standardní čas"}, {"China", "čínský standardní čas"}, {"Christmas", "čas Vánočního ostrova"}, {"Cocos", "čas Kokosových ostrovů"}, {"Colombia", "kolumbijský standardní čas"}, {"Cook", "standardní čas Cookových ostrovů"}, {"Cuba", "kubánský standardní čas"}, {"Davis", "čas Davisovy stanice"}, {"DumontDUrville", "čas stanice Dumonta d’Urvilla"}, {"East_Timor", "východotimorský čas"}, {"Easter", "standardní čas Velikonočního ostrova"}, {"Ecuador", "ekvádorský čas"}, {"Europe_Central", "středoevropský standardní čas"}, {"Europe_Eastern", "východoevropský standardní čas"}, {"Europe_Further_Eastern", "dálněvýchodoevropský čas"}, {"Europe_Western", "západoevropský standardní čas"}, {"Falkland", "falklandský standardní čas"}, {"Fiji", "fidžijský standardní čas"}, {"French_Guiana", "francouzskoguyanský čas"}, {"French_Southern", "čas Francouzských jižních a antarktických území"}, {"GMT", "greenwichský střední čas"}, {"Galapagos", "galapážský čas"}, {"Gambier", "gambierský čas"}, {"Georgia", "gruzínský standardní čas"}, {"Gilbert_Islands", "čas Gilbertových ostrovů"}, {"Greenland_Eastern", "východogrónský standardní čas"}, {"Greenland_Western", "západogrónský standardní čas"}, {"Guam", "Guamský čas"}, {"Gulf", "standardní čas Perského zálivu"}, {"Guyana", "guyanský čas"}, {"Hawaii", "havajsko-aleutský standardní čas"}, {"Hawaii_Aleutian", "havajsko-aleutský standardní čas"}, {"Hong_Kong", "hongkongský standardní čas"}, {"Hovd", "hovdský standardní čas"}, {"India", "indický čas"}, {"Indian_Ocean", "indickooceánský čas"}, {"Indochina", "indočínský čas"}, {"Indonesia_Central", "středoindonéský čas"}, {"Indonesia_Eastern", "východoindonéský čas"}, {"Indonesia_Western", "západoindonéský čas"}, {"Iran", "íránský standardní čas"}, {"Irkutsk", "irkutský standardní čas"}, {"Israel", "izraelský standardní čas"}, {"Japan", "japonský standardní čas"}, {"Kamchatka", "petropavlovsko-kamčatský standardní čas"}, {"Kazakhstan", "kazachstánský čas"}, {"Kazakhstan_Eastern", "východokazachstánský čas"}, {"Kazakhstan_Western", "západokazachstánský čas"}, {"Korea", "korejský standardní čas"}, {"Kosrae", "kosrajský čas"}, {"Krasnoyarsk", "krasnojarský standardní čas"}, {"Kyrgystan", "kyrgyzský čas"}, {"Lanka", "Srílanský čas"}, {"Line_Islands", "čas Rovníkových ostrovů"}, {"Lord_Howe", "standardní čas ostrova lorda Howa"}, {"Macau", "Macajský standardní čas"}, {"Magadan", "magadanský standardní čas"}, {"Malaysia", "malajský čas"}, {"Maldives", "maledivský čas"}, {"Marquesas", "markézský čas"}, {"Marshall_Islands", "čas Marshallových ostrovů"}, {"Mauritius", "mauricijský standardní čas"}, {"Mawson", "čas Mawsonovy stanice"}, {"Mexico_Pacific", "mexický pacifický standardní čas"}, {"Mongolia", "ulánbátarský standardní čas"}, {"Moscow", "moskevský standardní čas"}, {"Myanmar", "myanmarský čas"}, {"Nauru", "naurský čas"}, {"Nepal", "nepálský čas"}, {"New_Caledonia", "novokaledonský standardní čas"}, {"New_Zealand", "novozélandský standardní čas"}, {"Newfoundland", "newfoundlandský standardní čas"}, {"Niue", "niuejský čas"}, {"Norfolk", "norfolkský standardní čas"}, {"Noronha", "standardní čas souostroví Fernando de Noronha"}, {"North_Mariana", "Severomariánský čas"}, {"Novosibirsk", "novosibirský standardní čas"}, {"Omsk", "omský standardní čas"}, {"Pakistan", "pákistánský standardní čas"}, {"Palau", "palauský čas"}, {"Papua_New_Guinea", "čas Papuy-Nové Guiney"}, {"Paraguay", "paraguayský standardní čas"}, {"Peru", "peruánský standardní čas"}, {"Philippines", "filipínský standardní čas"}, {"Phoenix_Islands", "čas Fénixových ostrovů"}, {"Pierre_Miquelon", "pierre-miquelonský standardní čas"}, {"Pitcairn", "čas Pitcairnových ostrovů"}, {"Ponape", "ponapský čas"}, {"Pyongyang", "pchjongjangský čas"}, {"Qyzylorda", "Kyzylordský standardní čas"}, {"Reunion", "réunionský čas"}, {"Rothera", "čas Rotherovy stanice"}, {"Sakhalin", "sachalinský standardní čas"}, {"Samara", "samarský standardní čas"}, {"Samoa", "samojský standardní čas"}, {"Seychelles", "seychelský čas"}, {"Singapore", "singapurský čas"}, {"Solomon", "čas Šalamounových ostrovů"}, {"South_Georgia", "čas Jižní Georgie"}, {"Suriname", "surinamský čas"}, {"Syowa", "čas stanice Šówa"}, {"Tahiti", "tahitský čas"}, {"Taipei", "tchajpejský standardní čas"}, {"Tajikistan", "tádžický čas"}, {"Tokelau", "tokelauský čas"}, {"Tonga", "tonžský standardní čas"}, {"Truk", "chuukský čas"}, {"Turkey", "Turecký standardní čas"}, {"Turkmenistan", "turkmenský standardní čas"}, {"Tuvalu", "tuvalský čas"}, {"Uruguay", "uruguayský standardní čas"}, {"Uzbekistan", "uzbecký standardní čas"}, {"Vanuatu", "vanuatský standardní čas"}, {"Venezuela", "venezuelský čas"}, {"Vladivostok", "vladivostocký standardní čas"}, {"Volgograd", "volgogradský standardní čas"}, {"Vostok", "čas stanice Vostok"}, {"Wake", "čas ostrova Wake"}, {"Wallis", "čas ostrovů Wallis a Futuna"}, {"Yakutsk", "jakutský standardní čas"}, {"Yekaterinburg", "jekatěrinburský standardní čas"}, {"Yukon", "yukonský čas"}},
		{{"Acre", "acrejský letní čas"}, {"Alaska", "aljašský letní čas"}, {"Almaty", "Almatský letní čas"}, {"Amazon", "amazonský letní čas"}, {"America_Central", "severoamerický centrální letní čas"}, {"America_Eastern", "severoamerický východní letní čas"}, {"America_Mountain", "severoamerický horský letní čas"}, {"America_Pacific", "severoamerický pacifický letní čas"}, {"Anadyr", "anadyrský letní čas"}, {"Apia", "apijský letní čas"}, {"Aqtau", "Aktauský letní čas"}, {"Aqtobe", "Aktobský letní čas"}, {"Arabian", "arabský letní čas"}, {"Argentina", "argentinský letní čas"}, {"Argentina_Western", "západoargentinský letní čas"}, {"Armenia", "arménský letní čas"}, {"Atlantic", "atlantický letní čas"}, {"Australia_Central", "středoaustralský letní čas"}, {"Australia_CentralWestern", "středozápadní australský letní čas"}, {"Australia_Eastern", "východoaustralský letní čas"}, {"Australia_Western", "západoaustralský letní čas"}, {"Azerbaijan", "ázerbájdžánský letní čas"}, {"Azores", "azorský letní čas"}, {"Bangladesh", "bangladéšský letní čas"}, {"Brasilia", "brasilijský letní čas"}, {"Cape_Verde", "kapverdský letní čas"}, {"Chatham", "chathamský letní čas"}, {"Chile", "chilský letní čas"}, {"China", "čínský letní čas"}, {"Colombia", "kolumbijský letní čas"}, {"Cook", "letní čas Cookových ostrovů"}, {"Cuba", "kubánský letní čas"}, {"Easter", "letní čas Velikonočního ostrova"}, {"Europe_Central", "středoevropský letní čas"}, {"Europe_Eastern", "východoevropský letní čas"}, {"Europe_Western", "západoevropský letní čas"}, {"Falkland", "falklandský letní čas"}, {"Fiji", "fidžijský letní čas"}, {"Georgia", "gruzínský letní čas"}, {"Greenland_Eastern", "východogrónský letní čas"}, {"Greenland_Western", "západogrónský letní čas"}, {"Hawaii_Aleutian", "havajsko-aleutský letní čas"}, {"Hong_Kong", "hongkongský letní čas"}, {"Hovd", "hovdský letní čas"}, {"Iran", "íránský letní čas"}, {"Irkutsk", "irkutský letní čas"}, {"Israel", "izraelský letní čas"}, {"Japan", "japonský letní čas"}, {"Kamchatka", "petropavlovsko-kamčatský letní čas"}, {"Korea", "korejský letní čas"}, {"Krasnoyarsk", "krasnojarský letní čas"}, {"Lord_Howe", "letní čas ostrova lorda Howa"}, {"Macau", "Macajský letní čas"}, {"Magadan", "magadanský letní čas"}, {"Mauritius", "mauricijský letní čas"}, {"Mexico_Pacific", "mexický pacifický letní čas"}, {"Mongolia", "ulánbátarský letní čas"}, {"Moscow", "moskevský letní čas"}, {"New_Caledonia", "novokaledonský letní čas"}, {"New_Zealand", "novozélandský letní čas"}, {"Newfoundland", "newfoundlandský letní čas"}, {"Norfolk", "norfolkský letní čas"}, {"Noronha", "letní čas souostroví Fernando de Noronha"}, {"Novosibirsk", "novosibirský letní čas"}, {"Omsk", "omský letní čas"}, {"Pakistan", "pákistánský letní čas"}, {"Paraguay", "paraguayský letní čas"}, {"Peru", "peruánský letní čas"}, {"Philippines", "filipínský letní čas"}, {"Pierre_Miquelon", "pierre-miquelonský letní čas"}, {"Qyzylorda", "Kyzylordský letní čas"}, {"Sakhalin", "sachalinský letní čas"}, {"Samara", "samarský letní čas"}, {"Samoa", "samojský letní čas"}, {"Taipei", "tchajpejský letní čas"}, {"Tonga", "tonžský letní čas"}, {"Turkey", "Turecký letní čas"}, {"Turkmenistan", "turkmenský letní čas"}, {"Uruguay", "uruguayský letní čas"}, {"Uzbekistan", "uzbecký letní čas"}, {"Vanuatu", "vanuatský letní čas"}, {"Vladivostok", "vladivostocký letní čas"}, {"Volgograd", "volgogradský letní čas"}, {"Yakutsk", "jakutský letní čas"}, {"Yekaterinburg", "jekatěrinburský letní čas"}},
	},
	{
		{{"GMT", "ᐠᕑᐁᓂᐨ ᐯᔭᑯᐦᑖᐏᐣ ᐁᐃᐢᐸᓂᐠ"}},
		{},
	},
	{
		{{"America_Central", "среднеамерїка́нское зи́мнее вре́мѧ"}, {"America_Eastern", "восточноамерїка́нское зи́мнее вре́мѧ"}, {"America_Mountain", "а҆мерїка́нское наго́рнее зи́мнее вре́мѧ"}, {"America_Pacific", "тихоѻкеа́нское зи́мнее вре́мѧ"}, {"Atlantic", "а҆тланті́ческое зи́мнее вре́мѧ"}, {"Europe_Central", "среднеєѵрѡпе́йское зи́мнее вре́мѧ"}, {"Europe_Eastern", "восточноєѵрѡпе́йское зи́мнее вре́мѧ"}, {"Europe_Further_Eastern", "вре́мѧ въ калинингра́дѣ и҆ ми́нскѣ"}, {"Europe_Western", "западноєѵрѡпе́йское зи́мнее вре́мѧ"}, {"GMT", "сре́днее вре́мѧ по грі́нꙋичꙋ"}, {"Irkutsk", "и҆ркꙋ́тское зи́мнее вре́мѧ"}, {"Kazakhstan_Eastern", "восто́чный казахста́нъ"}, {"Kazakhstan_Western", "за́падный казахста́нъ"}, {"Krasnoyarsk", "красноѧ́рское зи́мнее вре́мѧ"}, {"Kyrgystan", "кирги́зїа"}, {"Magadan", "магада́нское зи́мнее вре́мѧ"}, {"Moscow", "моско́вское зи́мнее вре́мѧ"}, {"Novosibirsk", "новосиби́рское зи́мнее вре́мѧ"}, {"Omsk", "ѻ҆́мское зи́мнее вре́мѧ"}, {"Sakhalin", "зи́мнее вре́мѧ на сахали́нѣ"}, {"Vladivostok", "владивосто́цкое зи́мнее вре́мѧ"}, {"Volgograd", "волгогра́дское зи́мнее вре́мѧ"}, {"Yakutsk", "ꙗ҆кꙋ́тское зи́мнее вре́мѧ"}, {"Yekaterinburg", "є҆катерїнбꙋ́ржское зи́мнее вре́мѧ"}},
		{{"America_Central", "среднеамерїка́нское лѣ́тнее вре́мѧ"}, {"America_Eastern", "восточноамерїка́нское лѣ́тнее вре́мѧ"}, {"America_Mountain", "а҆мерїка́нское наго́рнее лѣ́тнее вре́мѧ"}, {"America_Pacific", "тихоѻкеа́нское лѣ́тнее вре́мѧ"}, {"Atlantic", "а҆тланті́ческое лѣ́тнее вре́мѧ"}, {"Europe_Central", "среднеєѵрѡпе́йское лѣ́тнее вре́мѧ"}, {"Europe_Eastern", "восточноєѵрѡпе́йское лѣ́тнее вре́мѧ"}, {"Europe_Western", "западноєѵрѡпе́йское лѣ́тнее вре́мѧ"}, {"Irkutsk", "и҆ркꙋ́тское лѣ́тнее вре́мѧ"}, {"Krasnoyarsk", "красноѧ́рское лѣ́тнее вре́мѧ"}, {"Magadan", "магада́нское лѣ́тнее вре́мѧ"}, {"Moscow", "моско́вское лѣ́тнее вре́мѧ"}, {"Novosibirsk", "новосиби́рское лѣ́тнее вре́мѧ"}, {"Omsk", "ѻ҆́мское лѣ́тнее вре́мѧ"}, {"Sakhalin", "лѣ́тнее вре́мѧ на сахали́нѣ"}, {"Vladivostok", "владивосто́цкое лѣ́тнее вре́мѧ"}, {"Volgograd", "волгогра́дское лѣ́тнее вре́мѧ"}, {"Yakutsk", "ꙗ҆кꙋ́тское лѣ́тнее вре́мѧ"}, {"Yekaterinburg", "є҆катерїнбꙋ́ржское лѣ́тнее вре́мѧ"}},
	},
	{
		{{"Acre", "Акри стандартлӑ вӑхӑчӗ"}, {"Afghanistan", "Афганистан вӑхӑчӗ"}, {"Africa_Central", "Вӑта Африка вӑхӑчӗ"}, {"Africa_Eastern", "Тухӑҫ Африка вӑхӑчӗ"}, {"Africa_Southern", "Кӑнтӑр Африка вӑхӑчӗ"}, {"Africa_Western", "Анӑҫ Африка вӑхӑчӗ"}, {"Alaska", "Аляска стандартлӑ вӑхӑчӗ"}, {"Almaty", "Алматы стандартлӑ вӑхӑчӗ"}, {"Amazon", "Амазонка стандартлӑ вӑхӑчӗ"}, {"America_Central", "Вӑта Америка стандартлӑ вӑхӑчӗ"}, {"America_Eastern", "Тухӑҫ Америка стандартлӑ вӑхӑчӗ"}, {"America_Mountain", "Ту стандартлӑ вӑхӑчӗ"}, {"America_Pacific", "Лӑпкӑ океан стандартлӑ вӑхӑчӗ"}, {"Anadyr", "Анадырь стандартлӑ вӑхӑчӗ"}, {"Apia", "Апиа стандартлӑ вӑхӑчӗ"}, {"Aqtau", "Актау стандартлӑ вӑхӑчӗ"}, {"Aqtobe", "Актобе стандартлӑ вӑхӑчӗ"}, {"Arabian", "Сауд Аравийӗ стандартлӑ вӑхӑчӗ"}, {"Argentina", "Аргентина стандартлӑ вӑхӑчӗ"}, {"Argentina_Western", "Анӑҫ Аргентина стандартлӑ вӑхӑчӗ"}, {"Armenia", "Армени стандартлӑ вӑхӑчӗ"}, {"Atlantic", "Атлантика стандартлӑ вӑхӑчӗ"}, {"Australia_Central", "Вӑта Австрали стандартлӑ вӑхӑчӗ"}, {"Australia_CentralWestern", "Анӑҫ Вӑта Австрали стандартлӑ вӑхӑчӗ"}, {"Australia_Eastern", "Тухӑҫ Австрали стандартлӑ вӑхӑчӗ"}, {"Australia_Western", "Анӑҫ Австрали стандартлӑ вӑхӑчӗ"}, {"Azerbaijan", "Азербайджан стандартлӑ вӑхӑчӗ"}, {"Azores", "Азор утравӗсен стандартлӑ вӑхӑчӗ"}, {"Bangladesh", "Бангладеш стандартлӑ вӑхӑчӗ"}, {"Bhutan", "Бутан вӑхӑчӗ"}, {"Bolivia", "Боливи вӑхӑчӗ"}, {"Brasilia", "Бразили стандартлӑ вӑхӑчӗ"}, {"Brunei", "Бруней вӑхӑчӗ"}, {"Cape_Verde", "Кабо-Верде стандартлӑ вӑхӑчӗ"}, {"Casey", "Кейси вӑхӑчӗ"}, {"Chamorro", "Чаморро вӑхӑчӗ"}, {"Chatham", "Чатем стандартлӑ вӑхӑчӗ"}, {"Chile", "Чили стандартлӑ вӑхӑчӗ"}, {"China", "Китай стандартлӑ вӑхӑчӗ"}, {"Christmas", "Раштав утравӗ вӑхӑчӗ"}, {"Cocos", "Кокос утравӗсен вӑхӑчӗ"}, {"Colombia", "Колумби стандартлӑ вӑхӑчӗ"}, {"Cook", "Кук утравӗсен стандартлӑ вӑхӑчӗ"}, {"Cuba", "Куба стандартлӑ вӑхӑчӗ"}, {"Davis", "Дейвис вӑхӑчӗ"}, {"DumontDUrville", "Дюмон-д’Юрвиль вӑхӑчӗ"}, {"East_Timor", "Тухӑҫ Тимор вӑхӑчӗ"}, {"Easter", "Мӑн кун утравӗн стандартлӑ вӑхӑчӗ"}, {"Ecuador", "Эквадор вӑхӑчӗ"}, {"Europe_Central", "Вӑта Европа стандартлӑ вӑхӑчӗ"}, {"Europe_Eastern", "Тухӑҫ Европа стандартлӑ вӑхӑчӗ"}, {"Europe_Further_Eastern", "Инҫет Тухӑҫ Европа вӑхӑчӗ"}, {"Europe_Western", "Анӑҫ Европа стандартлӑ вӑхӑчӗ"}, {"Falkland", "Фолкленд утравӗсен стандартлӑ вӑхӑчӗ"}, {"Fiji", "Фиджи стандартлӑ вӑхӑчӗ"}, {"French_Guiana", "Франци Гвианин вӑхӑчӗ"}, {"French_Southern", "Франци Кӑнтӑрпа Антарктика территорийӗсен вӑхӑчӗ"}, {"GMT", "Гринвич вӑхӑчӗ"}, {"Galapagos", "Галапагос вӑхӑчӗ"}, {"Gambier", "Гамбье вӑхӑчӗ"}, {"Georgia", "Грузи стандартлӑ вӑхӑчӗ"}, {"Gilbert_Islands", "Гилберт утравӗсен вӑхӑчӗ"}, {"Greenland", "Гренланди стандартлӑ вӑхӑчӗ"}, {"Greenland_Eastern", "Тухӑҫ Гренланди стандартлӑ вӑхӑчӗ"}, {"Greenland_Western", "Анӑҫ Гринланди стандартлӑ вӑхӑчӗ"}, {"Guam", "Гуам стандартлӑ вӑхӑчӗ"}, {"Gulf", "Перс кӳлмекӗн вӑхӑчӗ"}, {"Guyana", "Гайана вӑхӑчӗ"}, {"Hawaii", "Гавайи-Алеут стандартлӑ вӑхӑчӗ"}, {"Hawaii_Aleutian", "Гавайи-Алеут стандартлӑ вӑхӑчӗ"}, {"Hong_Kong", "Гонконг стандартлӑ вӑхӑчӗ"}, {"Hovd", "Ховд стандартлӑ вӑхӑчӗ"}, {"India", "Инди вӑхӑчӗ"}, {"Indian_Ocean", "Инди океанӗ вӑхӑчӗ"}, {"Indochina", "Индокитай вӑхӑчӗ"}, {"Indonesia_Central", "Вӑта Индонези вӑхӑчӗ"}, {"Indonesia_Eastern", "Тухӑҫ Индонези вӑхӑчӗ"}, {"Indonesia_Western", "Анӑҫ Индонези вӑхӑчӗ"}, {"Iran", "Иран стандартлӑ вӑхӑчӗ"}, {"Irkutsk", "Иркутск стандартлӑ вӑхӑчӗ"}, {"Israel", "Израиль стандартлӑ вӑхӑчӗ"}, {"Japan", "Япони стандартлӑ вӑхӑчӗ"}, {"Kamchatka", "Петропавловск-Камчатский стандартлӑ вӑхӑчӗ"}, {"Kazakhstan", "Казахстан вӑхӑчӗ"}, {"Kazakhstan_Eastern", "Тухӑҫ Казахстан вӑхӑчӗ"}, {"Kazakhstan_Western", "Анӑҫ Казахстан вӑхӑчӗ"}, {"Korea", "Корея стандартлӑ вӑхӑчӗ"}, {"Kosrae", "Косрае вӑхӑчӗ"}, {"Krasnoyarsk", "Красноярск стандартлӑ вӑхӑчӗ"}, {"Kyrgystan", "Кӑркӑстан вӑхӑчӗ"}, {"Lanka", "Шри-Ланка вӑхӑчӗ"}, {"Line_Islands", "Лайн утравӗсен вӑхӑчӗ"}, {"Lord_Howe", "Лорд-Хау стандартлӑ вӑхӑчӗ"}, {"Macau", "Макао стандартлӑ вӑхӑчӗ"}, {"Magadan", "Магадан стандартлӑ вӑхӑчӗ"}, {"Malaysia", "Малайзи вӑхӑчӗ"}, {"Maldives", "Мальдива вӑхӑчӗ"}, {"Marquesas", "Маркизас утравӗсен вӑхӑчӗ"}, {"Marshall_Islands", "Маршалл утравӗсен вӑхӑчӗ"}, {"Mauritius", "Маврики стандартлӑ вӑхӑчӗ"}, {"Mawson", "Моусон вӑхӑчӗ"}, {"Mexico_Pacific", "Мексика Лӑпкӑ океан стандартлӑ вӑхӑчӗ"}, {"Mongolia", "Улан-Батор стандартлӑ вӑхӑчӗ"}, {"Moscow", "Мускав стандартлӑ вӑхӑчӗ"}, {"Myanmar", "Мьянма вӑхӑчӗ"}, {"Nauru", "Науру вӑхӑчӗ"}, {"Nepal", "Непал вӑхӑчӗ"}, {"New_Caledonia", "Ҫӗнӗ Каледони стандартлӑ вӑхӑчӗ"}, {"New_Zealand", "Ҫӗнӗ Зеланди стандартлӑ вӑхӑчӗ"}, {"Newfoundland", "Ньюфаундленд стандартлӑ вӑхӑчӗ"}, {"Niue", "Ниуэ вӑхӑчӗ"}, {"Norfolk", "Норфолк стандартлӑ вӑхӑчӗ"}, {"Noronha", "Фернанду-ди-Норонья стандартлӑ вӑхӑчӗ"}, {"North_Mariana", "Ҫур ҫӗр Мариана утравӗсен вӑхӑчӗ"}, {"Novosibirsk", "Новосибирск стандартлӑ вӑхӑчӗ"}, {"Omsk", "Омск стандартлӑ вӑхӑчӗ"}, {"Pakistan", "Пакистан стандартлӑ вӑхӑчӗ"}, {"Palau", "Палау вӑхӑчӗ"}, {"Papua_New_Guinea", "Папуа — Ҫӗнӗ Гвинея вӑхӑчӗ"}, {"Paraguay", "Парагвай стандартлӑ вӑхӑчӗ"}, {"Peru", "Перу стандартлӑ вӑхӑчӗ"}, {"Philippines", "Филиппин стандартлӑ вӑхӑчӗ"}, {"Phoenix_Islands", "Феникс утравӗсен вӑхӑчӗ"}, {"Pierre_Miquelon", "Сен-Пьерпа Микелон стандартлӑ вӑхӑчӗ"}, {"Pitcairn", "Питкэрн вӑхӑчӗ"}, {"Ponape", "Понпеи вӑхӑчӗ"}, {"Pyongyang", "Ҫур ҫӗр Корея вӑхӑчӗ"}, {"Qyzylorda", "Кызылорда стандартлӑ вӑхӑчӗ"}, {"Reunion", "Реюньон вӑхӑчӗ"}, {"Rothera", "Ротера вӑхӑчӗ"}, {"Sakhalin", "Сахалин стандартлӑ вӑхӑчӗ"}, {"Samara", "Самар стандартлӑ вӑхӑчӗ"}, {"Samoa", "Самоа стандартлӑ вӑхӑчӗ"}, {"Seychelles", "Сейшел утравӗсен вӑхӑчӗ"}, {"Singapore", "Сингапур вӑхӑчӗ"}, {"Solomon", "Соломон вӑхӑчӗ"}, {"South_Georgia", "Кӑнтӑр Георги вӑхӑчӗ"}, {"Suriname", "Суринам вӑхӑчӗ"}, {"Syowa", "Сёва вӑхӑчӗ"}, {"Tahiti", "Таити вӑхӑчӗ"}, {"Taipei", "Тайвань стандартлӑ вӑхӑчӗ"}, {"Tajikistan", "Таджикистан вӑхӑчӗ"}, {"Tokelau", "Токелау вӑхӑчӗ"}, {"Tonga", "Тонга стандартлӑ вӑхӑчӗ"}, {"Truk", "Чуук вӑхӑчӗ"}, {"Turkmenistan", "Туркменистан стандартлӑ вӑхӑчӗ"}, {"Tuvalu", "Тувалу вӑхӑчӗ"}, {"Uruguay", "Уругвай стандартлӑ вӑхӑчӗ"}, {"Uzbekistan", "Узбекистан стандартлӑ вӑхӑчӗ"}, {"Vanuatu", "Вануату стандартлӑ вӑхӑчӗ"}, {"Venezuela", "Венесуэла вӑхӑчӗ"}, {"Vladivostok", "Владивосток стандартлӑ вӑхӑчӗ"}, {"Volgograd", "Волгоград стандартлӑ вӑхӑчӗ"}, {"Vostok", "Восток вӑхӑчӗ"}, {"Wake", "Уэйк утравӗ вӑхӑчӗ"}, {"Wallis", "Уоллиспа Футуна вӑхӑчӗ"}, {"Yakutsk", "Якутск стандартлӑ вӑхӑчӗ"}, {"Yekaterinburg", "Екатеринбург стандартлӑ вӑхӑчӗ"}, {"Yukon", "Юкон вӑхӑчӗ"}},
		{{"Acre", "Акри ҫуллахи вӑхӑчӗ"}, {"Alaska", "Аляска ҫуллахи вӑхӑчӗ"}, {"Almaty", "Алматы ҫуллахи вӑхӑчӗ"}, {"Amazon", "Амазонка ҫуллахи вӑхӑчӗ"}, {"America_Central", "Вӑта Америка ҫуллахи вӑхӑчӗ"}, {"America_Eastern", "Тухӑҫ Америка ҫуллахи вӑхӑчӗ"}, {"America_Mountain", "Ту ҫуллахи вӑхӑчӗ"}, {"America_Pacific", "Лӑпкӑ океан ҫуллахи вӑхӑчӗ"}, {"Anadyr", "Анадырь ҫуллахи вӑхӑчӗ"}, {"Apia", "Апиа ҫуллахи вӑхӑчӗ"}, {"Aqtau", "Актау ҫуллахи вӑхӑчӗ"}, {"Aqtobe", "Актобе ҫуллахи вӑхӑчӗ"}, {"Arabian", "Сауд Аравийӗ ҫуллахи вӑхӑчӗ"}, {"Argentina", "Аргентина ҫуллахи вӑхӑчӗ"}, {"Argentina_Western", "Анӑҫ Аргентина ҫуллахи вӑхӑчӗ"}, {"Armenia", "Армени ҫуллахи вӑхӑчӗ"}, {"Atlantic", "Атлантика ҫуллахи вӑхӑчӗ"}, {"Australia_Central", "Вӑта Австрали ҫуллахи вӑхӑчӗ"}, {"Australia_CentralWestern", "Анӑҫ Вӑта Австрали ҫуллахи вӑхӑчӗ"}, {"Australia_Eastern", "Тухӑҫ Австрали ҫуллахи вӑхӑчӗ"}, {"Australia_Western", "Анӑҫ Австрали ҫуллахи вӑхӑчӗ"}, {"Azerbaijan", "Азербайджан ҫуллахи вӑхӑчӗ"}, {"Azores", "Азор утравӗсен ҫуллахи вӑхӑчӗ"}, {"Bangladesh", "Бангладеш ҫуллахи вӑхӑчӗ"}, {"Brasilia", "Бразили ҫуллахи вӑхӑчӗ"}, {"Cape_Verde", "Кабо-Верде ҫуллахи вӑхӑчӗ"}, {"Chatham", "Чатем ҫуллахи вӑхӑчӗ"}, {"Chile", "Чили ҫуллахи вӑхӑчӗ"}, {"China", "Китай ҫуллахи вӑхӑчӗ"}, {"Colombia", "Колумби ҫуллахи вӑхӑчӗ"}, {"Cook", "Кук утравӗсен ҫуллахи вӑхӑчӗ"}, {"Cuba", "Куба ҫуллахи вӑхӑчӗ"}, {"Easter", "Мӑн кун утравӗн ҫуллахи вӑхӑчӗ"}, {"Europe_Central", "Вӑта Европа ҫуллахи вӑхӑчӗ"}, {"Europe_Eastern", "Тухӑҫ Европа ҫуллахи вӑхӑчӗ"}, {"Europe_Western", "Анӑҫ Европа ҫуллахи вӑхӑчӗ"}, {"Falkland", "Фолкленд утравӗсен ҫуллахи вӑхӑчӗ"}, {"Fiji", "Фиджи ҫуллахи вӑхӑчӗ"}, {"Georgia", "Грузи ҫуллахи вӑхӑчӗ"}, {"Greenland", "Гренланди ҫуллахи вӑхӑчӗ"}, {"Greenland_Eastern", "Тухӑҫ Гренланди ҫуллахи вӑхӑчӗ"}, {"Greenland_Western", "Анӑҫ ҫуллахи вӑхӑчӗ"}, {"Hawaii_Aleutian", "Гавайи-Алеут ҫуллахи вӑхӑчӗ"}, {"Hong_Kong", "Гонконг ҫуллахи вӑхӑчӗ"}, {"Hovd", "Ховд ҫуллахи вӑхӑчӗ"}, {"Iran", "Иран ҫуллахи вӑхӑчӗ"}, {"Irkutsk", "Иркутск ҫуллахи вӑхӑчӗ"}, {"Israel", "Израиль ҫуллахи вӑхӑчӗ"}, {"Japan", "Япони ҫуллахи вӑхӑчӗ"}, {"Kamchatka", "Петропавловск-Камчатский ҫуллахи вӑхӑчӗ"}, {"Korea", "Корея ҫуллахи вӑхӑчӗ"}, {"Krasnoyarsk", "Красноярск ҫуллахи вӑхӑчӗ"}, {"Lord_Howe", "Лорд-Хау ҫуллахи вӑхӑчӗ"}, {"Macau", "Макао ҫуллахи вӑхӑчӗ"}, {"Magadan", "Магадан ҫуллахи вӑхӑчӗ"}, {"Mauritius", "Маврики ҫуллахи вӑхӑчӗ"}, {"Mexico_Pacific", "Мексика Лӑпкӑ океан ҫуллахи вӑхӑчӗ"}, {"Mongolia", "Улан-Батор ҫуллахи вӑхӑчӗ"}, {"Moscow", "Мускав ҫуллахи вӑхӑчӗ"}, {"New_Caledonia", "Ҫӗнӗ Каледони ҫуллахи вӑхӑчӗ"}, {"New_Zealand", "Ҫӗнӗ Зеланди ҫуллахи вӑхӑчӗ"}, {"Newfoundland", "Ньюфаундленд ҫуллахи вӑхӑчӗ"}, {"Norfolk", "Норфолк ҫуллахи вӑхӑчӗ"}, {"Noronha", "Фернанду-ди-Норонья ҫуллахи вӑхӑчӗ"}, {"Novosibirsk", "Новосибирск ҫуллахи вӑхӑчӗ"}, {"Omsk", "Омск ҫуллахи вӑхӑчӗ"}, {"Pakistan", "Пакистан ҫуллахи вӑхӑчӗ"}, {"Paraguay", "Парагвай ҫуллахи вӑхӑчӗ"}, {"Peru", "Перу ҫуллахи вӑхӑчӗ"}, {"Philippines", "Филиппин ҫуллахи вӑхӑчӗ"}, {"Pierre_Miquelon", "Сен-Пьерпа Микелон ҫуллахи вӑхӑчӗ"}, {"Qyzylorda", "Кызылорда ҫуллахи вӑхӑчӗ"}, {"Sakhalin", "Сахалин ҫуллахи вӑхӑчӗ"}, {"Samara", "Самар ҫуллахи вӑхӑчӗ"}, {"Samoa", "Самоа ҫуллахи вӑхӑчӗ"}, {"Taipei", "Тайвань ҫуллахи вӑхӑчӗ"}, {"Tonga", "Тонга ҫуллахи вӑхӑчӗ"}, {"Turkmenistan", "Туркменистан ҫуллахи вӑхӑчӗ"}, {"Uruguay", "Уругвай ҫуллахи вӑхӑчӗ"}, {"Uzbekistan", "Узбекистан ҫуллахи вӑхӑчӗ"}, {"Vanuatu", "Вануату ҫуллахи вӑхӑчӗ"}, {"Vladivostok", "Владивосток ҫуллахи вӑхӑчӗ"}, {"Volgograd", "Волгоград ҫуллахи вӑхӑчӗ"}, {"Yakutsk", "Якутск ҫуллахи вӑхӑчӗ"}, {"Yekaterinburg", "Екатеринбург ҫуллахи вӑхӑчӗ"}},
	},
	{
		{{"Afghanistan", "Amser Afghanistan"}, {"Africa_Central", "Amser Canolbarth Affrica"}, {"Africa_Eastern", "Amser Dwyrain Affrica"}, {"Africa_Southern", "Amser Safonol De Affrica"}, {"Africa_Western", "Amser Gorllewin Affrica"}, {"Alaska", "Amser Safonol Alaska"}, {"Amazon", "Amser Safonol Amazonas"}, {"America_Central", "Amser Safonol Canolbarth Gogledd America"}, {"America_Eastern", "Amser Safonol Dwyrain Gogledd America"}, {"America_Mountain", "Amser Safonol Mynyddoedd Gogledd America"}, {"America_Pacific", "Amser Safonol Cefnfor Tawel Gogledd America"}, {"Apia", "Amser Safonol Apia"}, {"Arabian", "Amser Safonol Arabaidd"}, {"Argentina", "Amser Safonol Ariannin"}, {"Argentina_Western", "Amser Safonol Gorllewin Ariannin"}, {"Armenia", "Amser Safonol Armenia"}, {"Atlantic", "Amser Safonol Cefnfor yr Iwerydd"}, {"Australia_Central", "Amser Safonol Canolbarth Awstralia"}, {"Australia_CentralWestern", "Amser Safonol Canolbarth Gorllewin Awstralia"}, {"Australia_Eastern", "Amser Safonol Dwyrain Awstralia"}, {"Australia_Western", "Amser Safonol Gorllewin Awstralia"}, {"Azerbaijan", "Amser Safonol Aserbaijan"}, {"Azores", "Amser Safonol yr Azores"}, {"Bangladesh", "Amser Safonol Bangladesh"}, {"Bhutan", "Amser Bhutan"}, {"Bolivia", "Amser Bolifia"}, {"Brasilia", "Amser Safonol Brasília"}, {"Brunei", "Amser Brunei Darussalam"}, {"Cape_Verde", "Amser Safonol Cabo Verde"}, {"Chamorro", "Amser Chamorro"}, {"Chatham", "Amser Safonol Chatham"}, {"Chile", "Amser Safonol Chile"}, {"China", "Amser Safonol Tsieina"}, {"Christmas", "Amser Ynys Y Nadolig"}, {"Cocos", "Amser Ynysoedd Cocos"}, {"Colombia", "Amser Safonol Colombia"}, {"Cook", "Amser Safonol Ynysoedd Cook"}, {"Cuba", "Amser Safonol Ciwba"}, {"Davis", "Amser Davis"}, {"DumontDUrville", "Amser Dumont-d’Urville"}, {"East_Timor", "Amser Dwyrain Timor"}, {"Easter", "Amser Safonol Ynys y Pasg"}, {"Ecuador", "Amser Ecuador"}, {"Europe_Central", "Amser Safonol Canolbarth Ewrop"}, {"Europe_Eastern", "Amser Safonol Dwyrain Ewrop"}, {"Europe_Further_Eastern", "Amser Dwyrain Pell Ewrop"}, {"Europe_Western", "Amser Safonol Gorllewin Ewrop"}, {"Falkland", "Amser Safonol Ynysoedd Falklands/Malvinas"}, {"Fiji", "Amser Safonol Fiji"}, {"French_Guiana", "Amser Guyane Ffrengig"}, {"French_Southern", "Amser Tiroedd Ffrainc yn y De a’r Antarctig"}, {"GMT", "Amser Safonol Greenwich"}, {"Galapagos", "Amser Galapagos"}, {"Gambier", "Amser Gambier"}, {"Georgia", "Amser Safonol Georgia"}, {"Gilbert_Islands", "Amser Ynysoedd Gilbert"}, {"Greenland_Eastern", "Amser Safonol Dwyrain yr Ynys Las"}, {"Greenland_Western", "Amser Safonol Gorllewin yr Ynys Las"}, {"Gulf", "Amser Safonol y Gwlff"}, {"Guyana", "Amser Guyana"}, {"Hawaii", "Amser Safonol Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Amser Safonol Hawaii-Aleutian"}, {"Hong_Kong", "Amser Safonol Hong Kong"}, {"Hovd", "Amser Safonol Hovd"}, {"India", "Amser India"}, {"Indian_Ocean", "Amser Cefnfor India"}, {"Indochina", "Amser Indo-Tsieina"}, {"Indonesia_Central", "Amser Canolbarth Indonesia"}, {"Indonesia_Eastern", "Amser Dwyrain Indonesia"}, {"Indonesia_Western", "Amser Gorllewin Indonesia"}, {"Iran", "Amser Safonol Iran"}, {"Irkutsk", "Amser Safonol Irkutsk"}, {"Israel", "Amser Safonol Israel"}, {"Japan", "Amser Safonol Japan"}, {"Kazakhstan", "Amser Kazakhstan"}, {"Kazakhstan_Eastern", "Amser Dwyrain Kazakhstan"}, {"Kazakhstan_Western", "Amser Gorllewin Kazakhstan"}, {"Korea", "Amser Safonol Corea"}, {"Kosrae", "Amser Kosrae"}, {"Krasnoyarsk", "Amser Safonol Krasnoyarsk"}, {"Kyrgystan", "Amser Kyrgyzstan"}, {"Line_Islands", "Amser Ynysoedd Line"}, {"Lord_Howe", "Amser Safonol yr Arglwydd Howe"}, {"Magadan", "Amser Safonol Magadan"}, {"Malaysia", "Amser Malaysia"}, {"Maldives", "Amser Y Maldives"}, {"Marquesas", "Amser Marquises"}, {"Marshall_Islands", "Amser Ynysoedd Marshall"}, {"Mauritius", "Amser Safonol Mauritius"}, {"Mawson", "Amser Mawson"}, {"Mexico_Pacific", "Amser Safonol Pasiffig Mecsico"}, {"Mongolia", "Amser Safonol Ulan Bator"}, {"Moscow", "Amser Safonol Moscfa"}, {"Myanmar", "Amser Myanmar"}, {"Nauru", "Amser Nauru"}, {"Nepal", "Amser Nepal"}, {"New_Caledonia", "Amser Safonol Caledonia Newydd"}, {"New_Zealand", "Amser Safonol Seland Newydd"}, {"Newfoundland", "Amser Safonol Newfoundland"}, {"Niue", "Amser Niue"}, {"Norfolk", "Amser Safonol Ynys Norfolk"}, {"Noronha", "Amser Safonol Fernando de Noronha"}, {"Novosibirsk", "Amser Safonol Novosibirsk"}, {"Omsk", "Amser Safonol Omsk"}, {"Pakistan", "Amser Safonol Pakistan"}, {"Palau", "Amser Palau"}, {"Papua_New_Guinea", "Amser Papua Guinea Newydd"}, {"Paraguay", "Amser Safonol Paraguay"}, {"Peru", "Amser Safonol Periw"}, {"Philippines", "Amser Safonol Pilipinas"}, {"Phoenix_Islands", "Amser Ynysoedd Phoenix"}, {"Pierre_Miquelon", "Amser Safonol Saint-Pierre-et-Miquelon"}, {"Pitcairn", "Amser Pitcairn"}, {"Ponape", "Amser Pohnpei"}, {"Pyongyang", "Amser Pyongyang"}, {"Reunion", "Amser Réunion"}, {"Rothera", "Amser Rothera"}, {"Sakhalin", "Amser Safonol Sakhalin"}, {"Samara", "Amser Safonol Samara"}, {"Samoa", "Amser Safonol Samoa"}, {"Seychelles", "Amser Seychelles"}, {"Singapore", "Amser Singapore"}, {"Solomon", "Amser Ynysoedd Solomon"}, {"South_Georgia", "Amser De Georgia"}, {"Suriname", "Amser Suriname"}, {"Syowa", "Amser Syowa"}, {"Tahiti", "Amser Tahiti"}, {"Taipei", "Amser Safonol Taipei"}, {"Tajikistan", "Amser Tajicistan"}, {"Tokelau", "Amser Tokelau"}, {"Tonga", "Amser Safonol Tonga"}, {"Truk", "Amser Chuuk"}, {"Turkmenistan", "Amser Safonol Tyrcmenistan"}, {"Tuvalu", "Amser Tuvalu"}, {"Uruguay", "Amser Safonol Uruguay"}, {"Uzbekistan", "Amser Safonol Uzbekistan"}, {"Vanuatu", "Amser Safonol Vanuatu"}, {"Venezuela", "Amser Venezuela"}, {"Vladivostok", "Amser Safonol Vladivostok"}, {"Volgograd", "Amser Safonol Volgograd"}, {"Vostok", "Amser Vostok"}, {"Wake", "Amser Ynys Wake"}, {"Wallis", "Amser Wallis a Futuna"}, {"Yakutsk", "Amser Safonol Yakutsk"}, {"Yekaterinburg", "Amser Safonol Yekaterinburg"}, {"Yukon", "Amser Yukon"}},
		{{"Alaska", "Amser Haf Alaska"}, {"Amazon", "Amser Haf Amazonas"}, {"America_Central", "Amser Haf Canolbarth Gogledd America"}, {"America_Eastern", "Amser Haf Dwyrain Gogledd America"}, {"America_Mountain", "Amser Haf Mynyddoedd Gogledd America"}, {"America_Pacific", "Amser Haf Cefnfor Tawel Gogledd America"}, {"Apia", "Amser Haf Apia"}, {"Arabian", "Amser Haf Arabaidd"}, {"Argentina", "Amser Haf Ariannin"}, {"Argentina_Western", "Amser Haf Gorllewin Ariannin"}, {"Armenia", "Amser Haf Armenia"}, {"Atlantic", "Amser Haf Cefnfor yr Iwerydd"}, {"Australia_Central", "Amser Haf Canolbarth Awstralia"}, {"Australia_CentralWestern", "Amser Haf Canolbarth Gorllewin Awstralia"}, {"Australia_Eastern", "Amser Haf Dwyrain Awstralia"}, {"Australia_Western", "Amser Haf Gorllewin Awstralia"}, {"Azerbaijan", "Amser Haf Aserbaijan"}, {"Azores", "Amser Haf yr Azores"}, {"Bangladesh", "Amser Haf Bangladesh"}, {"Brasilia", "Amser Haf Brasília"}, {"Cape_Verde", "Amser Haf Cabo Verde"}, {"Chatham", "Amser Haf Chatham"}, {"Chile", "Amser Haf Chile"}, {"China", "Amser Haf Tsieina"}, {"Colombia", "Amser Haf Colombia"}, {"Cook", "Amser Hanner Haf Ynysoedd Cook"}, {"Cuba", "Amser Haf Ciwa"}, {"Easter", "Amser Haf Ynys y Pasg"}, {"Europe_Central", "Amser Haf Canolbarth Ewrop"}, {"Europe_Eastern", "Amser Haf Dwyrain Ewrop"}, {"Europe_Western", "Amser Haf Gorllewin Ewrop"}, {"Falkland", "Amser Haf Ynysoedd Falklands/Malvinas"}, {"Fiji", "Amser Haf Fiji"}, {"Georgia", "Amser Haf Georgia"}, {"Greenland_Eastern", "Amser Haf Dwyrain yr Ynys Las"}, {"Greenland_Western", "Amser Haf Gorllewin yr Ynys Las"}, {"Hawaii_Aleutian", "Amser Haf Hawaii-Aleutian"}, {"Hong_Kong", "Amser Haf Hong Kong"}, {"Hovd", "Amser Haf Hovd"}, {"Iran", "Amser Haf Iran"}, {"Irkutsk", "Amser Haf Irkutsk"}, {"Israel", "Amser Haf Israel"}, {"Japan", "Amser Haf Japan"}, {"Korea", "Amser Haf Corea"}, {"Krasnoyarsk", "Amser Haf Krasnoyarsk"}, {"Lord_Howe", "Amser Haf yr Arglwydd Howe"}, {"Magadan", "Amser Haf Magadan"}, {"Mauritius", "Amser Haf Mauritius"}, {"Mexico_Pacific", "Amser Haf Pasiffig Mecsico"}, {"Mongolia", "Amser Haf Ulan Bator"}, {"Moscow", "Amser Haf Moscfa"}, {"New_Caledonia", "Amser Haf Caledonia Newydd"}, {"New_Zealand", "Amser Haf Seland Newydd"}, {"Newfoundland", "Amser Haf Newfoundland"}, {"Norfolk", "Amser Haf Ynys Norfolk"}, {"Noronha", "Amser Haf Fernando de Noronha"}, {"Novosibirsk", "Amser Haf Novosibirsk"}, {"Omsk", "Amser Haf Omsk"}, {"Pakistan", "Amser Haf Pakistan"}, {"Paraguay", "Amser Haf Paraguay"}, {"Peru", "Amser Haf Periw"}, {"Philippines", "Amser Haf Pilipinas"}, {"Pierre_Miquelon", "Amser Haf Saint-Pierre-et-Miquelon"}, {"Sakhalin", "Amser Haf Sakhalin"}, {"Samara", "Amser Haf Samara"}, {"Samoa", "Amser Haf Samoa"}, {"Taipei", "Amser Haf Taipei"}, {"Tonga", "Amser Haf Tonga"}, {"Turkmenistan", "Amser Haf Tyrcmenistan"}, {"Uruguay", "Amser Haf Uruguay"}, {"Uzbekistan", "Amser Haf Uzbekistan"}, {"Vanuatu", "Amser Haf Vanuatu"}, {"Vladivostok", "Amser Haf Vladivostok"}, {"Volgograd", "Amser Haf Volgograd"}, {"Yakutsk", "Amser Haf Yakutsk"}, {"Yekaterinburg", "Amser Haf Yekaterinburg"}},
	},
	{
		{{"Acre", "Acre-normaltid"}, {"Afghanistan", "Afghansk tid"}, {"Africa_Central", "Centralafrikansk tid"}, {"Africa_Eastern", "Østafrikansk tid"}, {"Africa_Southern", "Sydafrikansk tid"}, {"Africa_Western", "Vestafrikansk tid"}, {"Alaska", "Alaska-normaltid"}, {"Almaty", "Almaty-normaltid"}, {"Amazon", "Amazonas-normaltid"}, {"America_Central", "Central-normaltid"}, {"America_Eastern", "Eastern-normaltid"}, {"America_Mountain", "Mountain-normaltid"}, {"America_Pacific", "Pacific-normaltid"}, {"Anadyr", "Anadyr-normaltid"}, {"Apia", "Apia-normaltid"}, {"Aqtau", "Aqtau-normaltid"}, {"Aqtobe", "Aqtobe-normaltid"}, {"Arabian", "Arabisk normaltid"}, {"Argentina", "Argentinsk normaltid"}, {"Argentina_Western", "Vestargentinsk normaltid"}, {"Armenia", "Armensk normaltid"}, {"Atlantic", "Atlantic-normaltid"}, {"Australia_Central", "Centralaustralsk normaltid"}, {"Australia_CentralWestern", "Vestlig centralaustralsk normaltid"}, {"Australia_Eastern", "Østaustralsk normaltid"}, {"Australia_Western", "Vestaustralsk normaltid"}, {"Azerbaijan", "Aserbajdsjansk normaltid"}, {"Azores", "Azorerne-normaltid"}, {"Bangladesh", "Bangladesh-normaltid"}, {"Bhutan", "Bhutan-tid"}, {"Bolivia", "Boliviansk tid"}, {"Brasilia", "Brasiliansk normaltid"}, {"Brunei", "Brunei Darussalam-tid"}, {"Cape_Verde", "Kap Verde-normaltid"}, {"Chamorro", "Chamorro-tid"}, {"Chatham", "Chatham-normaltid"}, {"Chile", "Chilensk normaltid"}, {"China", "Kinesisk normaltid"}, {"Christmas", "Juleøen-normaltid"}, {"Cocos", "Cocosøerne-normaltid"}, {"Colombia", "Colombiansk normaltid"}, {"Cook", "Cookøerne-normaltid"}, {"Cuba", "Cubansk normaltid"}, {"Davis", "Davis-tid"}, {"DumontDUrville", "Dumont-d’Urville-tid"}, {"East_Timor", "Østtimor-tid"}, {"Easter", "Påskeøen-normaltid"}, {"Ecuador", "Ecuadoriansk tid"}, {"Europe_Central", "Centraleuropæisk normaltid"}, {"Europe_Eastern", "Østeuropæisk normaltid"}, {"Europe_Further_Eastern", "Fjernøsteuropæisk tid"}, {"Europe_Western", "Vesteuropæisk normaltid"}, {"Falkland", "Falklandsøerne-normaltid"}, {"Fiji", "Fijiansk normaltid"}, {"French_Guiana", "Fransk Guyana-tid"}, {"French_Southern", "Franske Sydlige og Antarktiske Territorier-tid"}, {"GMT", "GMT"}, {"Galapagos", "Galapagos-tid"}, {"Gambier", "Gambier-tid"}, {"Georgia", "Georgisk normaltid"}, {"Gilbert_Islands", "Gilbertøerne-tid"}, {"Greenland_Eastern", "Østgrønlandsk normaltid"}, {"Greenland_Western", "Vestgrønlandsk normaltid"}, {"Guam", "Guam-normaltid"}, {"Gulf", "Golflandene-normaltid"}, {"Guyana", "Guyana-tid"}, {"Hawaii", "Hawaii-Aleutian-normaltid"}, {"Hawaii_Aleutian", "Hawaii-Aleutian-normaltid"}, {"Hong_Kong", "Hongkong-normaltid"}, {"Hovd", "Hovd-normaltid"}, {"India", "Indisk normaltid"}, {"Indian_Ocean", "Indiske Ocean-normaltid"}, {"Indochina", "Indokina-tid"}, {"Indonesia_Central", "Centralindonesisk tid"}, {"Indonesia_Eastern", "Østindonesisk tid"}, {"Indonesia_Western", "Vestindonesisk tid"}, {"Iran", "Iransk normaltid"}, {"Irkutsk", "Irkutsk-normaltid"}, {"Israel", "Israelsk normaltid"}, {"Japan", "Japansk normaltid"}, {"Kamchatka", "Petropavlovsk-Kamchatski normaltid"}, {"Kazakhstan", "Kasakhstansk tid"}, {"Kazakhstan_Eastern", "Østkasakhstansk tid"}, {"Kazakhstan_Western", "Vestkasakhstansk tid"}, {"Korea", "Koreansk normaltid"}, {"Kosrae", "Kosrae-tid"}, {"Krasnoyarsk", "Krasnojarsk-normaltid"}, {"Kyrgystan", "Kirgisisk tid"}, {"Lanka", "Langa tid"}, {"Line_Islands", "Linjeøerne-tid"}, {"Lord_Howe", "Lord Howe-normaltid"}, {"Macau", "Macao-normaltid"}, {"Magadan", "Magadan-normaltid"}, {"Malaysia", "Malaysia-tid"}, {"Maldives", "Maldiverne-tid"}, {"Marquesas", "Marquesas-tid"}, {"Marshall_Islands", "Marshalløerne-tid"}, {"Mauritius", "Mauritius-normaltid"}, {"Mawson", "Mawson-tid"}, {"Mexico_Pacific", "Mexicansk Pacific-normaltid"}, {"Mongolia", "Ulan Bator-normaltid"}, {"Moscow", "Moskva-normaltid"}, {"Myanmar", "Myanmar-tid"}, {"Nauru", "Nauru-tid"}, {"Nepal", "Nepalesisk tid"}, {"New_Caledonia", "Ny Kaledonien-normaltid"}, {"New_Zealand", "Newzealandsk normaltid"}, {"Newfoundland", "Newfoundlandsk normaltid"}, {"Niue", "Niue-tid"}, {"Norfolk", "Norfolk Island-normaltid"}, {"Noronha", "Fernando de Noronha-normaltid"}, {"North_Mariana", "Nordmarianerne-tid"}, {"Novosibirsk", "Novosibirsk-normaltid"}, {"Omsk", "Omsk-normaltid"}, {"Pakistan", "Pakistansk normaltid"}, {"Palau", "Palau-tid"}, {"Papua_New_Guinea", "Papua Ny Guinea-tid"}, {"Paraguay", "Paraguayansk normaltid"}, {"Peru", "Peruviansk normaltid"}, {"Philippines", "Filippinsk normaltid"}, {"Phoenix_Islands", "Phoenixøen-tid"}, {"Pierre_Miquelon", "Saint Pierre- og Miquelon-normaltid"}, {"Pitcairn", "Pitcairn-tid"}, {"Ponape", "Ponape-tid"}, {"Pyongyang", "Pyongyang-tid"}, {"Qyzylorda", "Qyzylorda-normaltid"}, {"Reunion", "Reunion-tid"}, {"Rothera", "Rothera-tid"}, {"Sakhalin", "Sakhalin-normaltid"}, {"Samara", "Samara-normaltid"}, {"Samoa", "Samoansk normaltid"}, {"Seychelles", "Seychellisk tid"}, {"Singapore", "Singapore-tid"}, {"Solomon", "Salomonøerne-tid"}, {"South_Georgia", "South Georgia-tid"}, {"Suriname", "Surinam-tid"}, {"Syowa", "Syowa-tid"}, {"Tahiti", "Tahiti-tid"}, {"Taipei", "Taipei-normaltid"}, {"Tajikistan", "Tadsjikisk tid"}, {"Tokelau", "Tokelau-tid"}, {"Tonga", "Tongansk normaltid"}, {"Truk", "Chuuk-tid"}, {"Turkey", "Tyrkisk normaltid"}, {"Turkmenistan", "Turkmensk normaltid"}, {"Tuvalu", "Tuvalu-tid"}, {"Uruguay", "Uruguayansk normaltid"}, {"Uzbekistan", "Usbekisk normaltid"}, {"Vanuatu", "Vanuatu-normaltid"}, {"Venezuela", "Venezuelansk tid"}, {"Vladivostok", "Vladivostok-normaltid"}, {"Volgograd", "Volgograd-normaltid"}, {"Vostok", "Vostok-tid"}, {"Wake", "Wake Island-tid"}, {"Wallis", "Wallis og Futuna-tid"}, {"Yakutsk", "Jakutsk-normaltid"}, {"Yekaterinburg", "Jekaterinburg-normaltid"}, {"Yukon", "Yukon-tid"}},
		{{"Acre", "Acre-sommertid"}, {"Alaska", "Alaska-sommertid"}, {"Almaty", "Almaty-sommertid"}, {"Amazon", "Amazonas-sommertid"}, {"America_Central", "Central-sommertid"}, {"America_Eastern", "Eastern-sommertid"}, {"America_Mountain", "Mountain-sommertid"}, {"America_Pacific", "Pacific-sommertid"}, {"Anadyr", "Anadyr-sommertid"}, {"Apia", "Apia-sommertid"}, {"Aqtau", "Aqtau-sommertid"}, {"Aqtobe", "Aqtobe-sommertid"}, {"Arabian", "Arabisk sommertid"}, {"Argentina", "Argentinsk sommertid"}, {"Argentina_Western", "Vestargentinsk sommertid"}, {"Armenia", "Armensk sommertid"}, {"Atlantic", "Atlantic-sommertid"}, {"Australia_Central", "Centralaustralsk sommertid"}, {"Australia_CentralWestern", "Vestlig centralaustralsk sommertid"}, {"Australia_Eastern", "Østaustralsk sommertid"}, {"Australia_Western", "Vestaustralsk sommertid"}, {"Azerbaijan", "Aserbajdsjansk sommertid"}, {"Azores", "Azorerne-sommertid"}, {"Bangladesh", "Bangladesh-sommertid"}, {"Brasilia", "Brasiliansk sommertid"}, {"Cape_Verde", "Kap Verde-sommertid"}, {"Chatham", "Chatham-sommertid"}, {"Chile", "Chilensk sommertid"}, {"China", "Kinesisk sommertid"}, {"Colombia", "Colombiansk sommertid"}, {"Cook", "Cookøerne-sommertid"}, {"Cuba", "Cubansk sommertid"}, {"Easter", "Påskeøen-sommertid"}, {"Europe_Central", "Centraleuropæisk sommertid"}, {"Europe_Eastern", "Østeuropæisk sommertid"}, {"Europe_Western", "Vesteuropæisk sommertid"}, {"Falkland", "Falklandsøerne-sommertid"}, {"Fiji", "Fijiansk sommertid"}, {"Georgia", "Georgisk sommertid"}, {"Greenland_Eastern", "Østgrønlandsk sommertid"}, {"Greenland_Western", "Vestgrønlandsk sommertid"}, {"Hawaii_Aleutian", "Hawaii-Aleutian-sommertid"}, {"Hong_Kong", "Hongkong-sommertid"}, {"Hovd", "Hovd-sommertid"}, {"Iran", "Iransk sommertid"}, {"Irkutsk", "Irkutsk-sommertid"}, {"Israel", "Israelsk sommertid"}, {"Japan", "Japansk sommertid"}, {"Kamchatka", "Petropavlovsk-Kamchatski sommertid"}, {"Korea", "Koreansk sommertid"}, {"Krasnoyarsk", "Krasnojarsk-sommertid"}, {"Lord_Howe", "Lord Howe-sommertid"}, {"Macau", "Macao-sommertid"}, {"Magadan", "Magadan-sommertid"}, {"Mauritius", "Mauritius-sommertid"}, {"Mexico_Pacific", "Mexicansk Pacific-sommertid"}, {"Mongolia", "Ulan Bator-sommertid"}, {"Moscow", "Moskva-sommertid"}, {"New_Caledonia", "Ny Kaledonien-sommertid"}, {"New_Zealand", "Newzealandsk sommertid"}, {"Newfoundland", "Newfoundlandsk sommertid"}, {"Norfolk", "Norfolk Island-sommertid"}, {"Noronha", "Fernando de Noronha-sommertid"}, {"Novosibirsk", "Novosibirsk-sommertid"}, {"Omsk", "Omsk-sommertid"}, {"Pakistan", "Pakistansk sommertid"}, {"Paraguay", "Paraguayansk sommertid"}, {"Peru", "Peruviansk sommertid"}, {"Philippines", "Filippinsk sommertid"}, {"Pierre_Miquelon", "Saint Pierre- og Miquelon-sommertid"}, {"Qyzylorda", "Qyzylorda-sommertid"}, {"Sakhalin", "Sakhalin-sommertid"}, {"Samara", "Samara-sommertid"}, {"Samoa", "Samoansk sommertid"}, {"Taipei", "Taipei-sommertid"}, {"Tonga", "Tongansk sommertid"}, {"Turkey", "Tyrkisk sommertid"}, {"Turkmenistan", "Turkmensk sommertid"}, {"Uruguay", "Uruguayansk sommertid"}, {"Uzbekistan", "Usbekisk sommertid"}, {"Vanuatu", "Vanuatu-sommertid"}, {"Vladivostok", "Vladivostok-sommertid"}, {"Volgograd", "Volgograd-sommertid"}, {"Yakutsk", "Jakutsk-sommertid"}, {"Yekaterinburg", "Jekaterinburg-sommertid"}},
	},
	{
		{{"Acre", "Acre-Normalzeit"}, {"Afghanistan", "Afghanistan-Zeit"}, {"Africa_Central", "Zentralafrikanische Zeit"}, {"Africa_Eastern", "Ostafrikanische Zeit"}, {"Africa_Southern", "Südafrikanische Zeit"}, {"Africa_Western", "Westafrikanische Zeit"}, {"Alaska", "Alaska-Normalzeit"}, {"Almaty", "Almaty-Normalzeit"}, {"Amazon", "Amazonas-Normalzeit"}, {"America_Central", "Nordamerikanische Zentral-Normalzeit"}, {"America_Eastern", "Nordamerikanische Ostküsten-Normalzeit"}, {"America_Mountain", "Rocky-Mountains-Normalzeit"}, {"America_Pacific", "Nordamerikanische Westküsten-Normalzeit"}, {"Anadyr", "Anadyr Normalzeit"}, {"Apia", "Apia-Normalzeit"}, {"Aqtau", "Aqtau-Normalzeit"}, {"Aqtobe", "Aqtöbe-Normalzeit"}, {"Arabian", "Arabische Normalzeit"}, {"Argentina", "Argentinische Normalzeit"}, {"Argentina_Western", "Westargentinische Normalzeit"}, {"Armenia", "Armenische Normalzeit"}, {"Atlantic", "Atlantik-Normalzeit"}, {"Australia_Central", "Zentralaustralische Normalzeit"}, {"Australia_CentralWestern", "Zentral-/Westaustralische Normalzeit"}, {"Australia_Eastern", "Ostaustralische Normalzeit"}, {"Australia_Western", "Westaustralische Normalzeit"}, {"Azerbaijan", "Aserbeidschanische Normalzeit"}, {"Azores", "Azoren-Normalzeit"}, {"Bangladesh", "Bangladesch-Normalzeit"}, {"Bhutan", "Bhutan-Zeit"}, {"Bolivia", "Bolivianische Zeit"}, {"Brasilia", "Brasília-Normalzeit"}, {"Brunei", "Brunei-Darussalam-Zeit"}, {"Cape_Verde", "Cabo-Verde-Normalzeit"}, {"Casey", "Casey-Zeit"}, {"Chamorro", "Chamorro-Zeit"}, {"Chatham", "Chatham-Normalzeit"}, {"Chile", "Chilenische Normalzeit"}, {"China", "Chinesische Normalzeit"}, {"Christmas", "Weihnachtsinsel-Zeit"}, {"Cocos", "Kokosinseln-Zeit"}, {"Colombia", "Kolumbianische Normalzeit"}, {"Cook", "Cookinseln-Normalzeit"}, {"Cuba", "Kubanische Normalzeit"}, {"Davis", "Davis-Zeit"}, {"DumontDUrville", "Dumont-d’Urville-Zeit"}, {"East_Timor", "Osttimor-Zeit"}, {"Easter", "Osterinsel-Normalzeit"}, {"Ecuador", "Ecuadorianische Zeit"}, {"Europe_Central", "Mitteleuropäische Normalzeit"}, {"Europe_Eastern", "Osteuropäische Normalzeit"}, {"Europe_Further_Eastern", "Kaliningrader Zeit"}, {"Europe_Western", "Westeuropäische Normalzeit"}, {"Falkland", "Falklandinseln-Normalzeit"}, {"Fiji", "Fidschi-Normalzeit"}, {"French_Guiana", "Französisch-Guayana-Zeit"}, {"French_Southern", "Französische-Süd-und-Antarktisgebiete-Zeit"}, {"GMT", "Mittlere Greenwich-Zeit"}, {"Galapagos", "Galapagos-Zeit"}, {"Gambier", "Gambier-Zeit"}, {"Georgia", "Georgische Normalzeit"}, {"Gilbert_Islands", "Gilbert-Inseln-Zeit"}, {"Greenland_Eastern", "Ostgrönland-Normalzeit"}, {"Greenland_Western", "Westgrönland-Normalzeit"}, {"Guam", "Guam-Zeit"}, {"Gulf", "Golf-Zeit"}, {"Guyana", "Guyana-Zeit"}, {"Hawaii", "Hawaii-Aleuten-Normalzeit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Normalzeit"}, {"Hong_Kong", "Hongkong-Normalzeit"}, {"Hovd", "Chowd-Normalzeit"}, {"India", "Indische Normalzeit"}, {"Indian_Ocean", "Indischer-Ozean-Zeit"}, {"Indochina", "Indochina-Zeit"}, {"Indonesia_Central", "Zentralindonesische Zeit"}, {"Indonesia_Eastern", "Ostindonesische Zeit"}, {"Indonesia_Western", "Westindonesische Zeit"}, {"Iran", "Iranische Normalzeit"}, {"Irkutsk", "Irkutsker Normalzeit"}, {"Israel", "Israelische Normalzeit"}, {"Japan", "Japanische Normalzeit"}, {"Kamchatka", "Kamtschatka-Normalzeit"}, {"Kazakhstan", "Kasachische Zeit"}, {"Kazakhstan_Eastern", "Ostkasachische Zeit"}, {"Kazakhstan_Western", "Westkasachische Zeit"}, {"Korea", "Koreanische Normalzeit"}, {"Kosrae", "Kosrae-Zeit"}, {"Krasnoyarsk", "Krasnojarsker Normalzeit"}, {"Kyrgystan", "Kirgisische Zeit"}, {"Lanka", "Sri-Lanka-Zeit"}, {"Line_Islands", "Linieninseln-Zeit"}, {"Lord_Howe", "Lord-Howe-Normalzeit"}, {"Macau", "Macau-Normalzeit"}, {"Magadan", "Magadan-Normalzeit"}, {"Malaysia", "Malaysische Zeit"}, {"Maldives", "Malediven-Zeit"}, {"Marquesas", "Marquesas-Zeit"}, {"Marshall_Islands", "Marshallinseln-Zeit"}, {"Mauritius", "Mauritius-Normalzeit"}, {"Mawson", "Mawson-Zeit"}, {"Mexico_Pacific", "Mexikanische Pazifik-Normalzeit"}, {"Mongolia", "Ulaanbaatar-Normalzeit"}, {"Moscow", "Moskauer Normalzeit"}, {"Myanmar", "Myanmar-Zeit"}, {"Nauru", "Nauru-Zeit"}, {"Nepal", "Nepalesische Zeit"}, {"New_Caledonia", "Neukaledonische Normalzeit"}, {"New_Zealand", "Neuseeland-Normalzeit"}, {"Newfoundland", "Neufundland-Normalzeit"}, {"Niue", "Niue-Zeit"}, {"Norfolk", "Norfolkinsel-Normalzeit"}, {"Noronha", "Fernando-de-Noronha-Normalzeit"}, {"North_Mariana", "Nördliche-Marianen-Zeit"}, {"Novosibirsk", "Nowosibirsker Normalzeit"}, {"Omsk", "Omsker Normalzeit"}, {"Pakistan", "Pakistanische Normalzeit"}, {"Palau", "Palau-Zeit"}, {"Papua_New_Guinea", "Papua-Neuguinea-Zeit"}, {"Paraguay", "Paraguayische Normalzeit"}, {"Peru", "Peruanische Normalzeit"}, {"Philippines", "Philippinische Normalzeit"}, {"Phoenix_Islands", "Phoenixinseln-Zeit"}, {"Pierre_Miquelon", "St.-Pierre-und-Miquelon-Normalzeit"}, {"Pitcairn", "Pitcairninseln-Zeit"}, {"Ponape", "Pohnpei-Zeit"}, {"Pyongyang", "Pjöngjang-Zeit"}, {"Qyzylorda", "Quysylorda-Normalzeit"}, {"Reunion", "Réunion-Zeit"}, {"Rothera", "Rothera-Zeit"}, {"Sakhalin", "Sachalin-Normalzeit"}, {"Samara", "Samara-Normalzeit"}, {"Samoa", "Samoa-Normalzeit"}, {"Seychelles", "Seychellen-Zeit"}, {"Singapore", "Singapurische Normalzeit"}, {"Solomon", "Salomonen-Zeit"}, {"South_Georgia", "Südgeorgische Zeit"}, {"Suriname", "Suriname-Zeit"}, {"Syowa", "Syowa-Zeit"}, {"Tahiti", "Tahiti-Zeit"}, {"Taipei", "Taipeh-Normalzeit"}, {"Tajikistan", "Tadschikische Zeit"}, {"Tokelau", "Tokelau-Zeit"}, {"Tonga", "Tongaische Normalzeit"}, {"Truk", "Chuuk-Zeit"}, {"Turkey", "Türkische Normalzeit"}, {"Turkmenistan", "Turkmenische Normalzeit"}, {"Tuvalu", "Tuvalu-Zeit"}, {"Uruguay", "Uruguayische Normalzeit"}, {"Uzbekistan", "Usbekische Normalzeit"}, {"Vanuatu", "Vanuatu-Normalzeit"}, {"Venezuela", "Venezuela-Zeit"}, {"Vladivostok", "Wladiwostoker Normalzeit"}, {"Volgograd", "Wolgograder Normalzeit"}, {"Vostok", "Wostok-Zeit"}, {"Wake", "Wake-Insel-Zeit"}, {"Wallis", "Wallis-und-Futuna-Zeit"}, {"Yakutsk", "Jakutsker Normalzeit"}, {"Yekaterinburg", "Jekaterinburger Normalzeit"}, {"Yukon", "Yukon-Zeit"}},
		{{"Acre", "Acre-Sommerzeit"}, {"Alaska", "Alaska-Sommerzeit"}, {"Almaty", "Almaty-Sommerzeit"}, {"Amazon", "Amazonas-Sommerzeit"}, {"America_Central", "Nordamerikanische Zentral-Sommerzeit"}, {"America_Eastern", "Nordamerikanische Ostküsten-Sommerzeit"}, {"America_Mountain", "Rocky-Mountains-Sommerzeit"}, {"America_Pacific", "Nordamerikanische Westküsten-Sommerzeit"}, {"Anadyr", "Anadyr Sommerzeit"}, {"Apia", "Apia-Sommerzeit"}, {"Aqtau", "Aqtau-Sommerzeit"}, {"Aqtobe", "Aqtöbe-Sommerzeit"}, {"Arabian", "Arabische Sommerzeit"}, {"Argentina", "Argentinische Sommerzeit"}, {"Argentina_Western", "Westargentinische Sommerzeit"}, {"Armenia", "Armenische Sommerzeit"}, {"Atlantic", "Atlantik-Sommerzeit"}, {"Australia_Central", "Zentralaustralische Sommerzeit"}, {"Australia_CentralWestern", "Zentral-/Westaustralische Sommerzeit"}, {"Australia_Eastern", "Ostaustralische Sommerzeit"}, {"Australia_Western", "Westaustralische Sommerzeit"}, {"Azerbaijan", "Aserbaidschanische Sommerzeit"}, {"Azores", "Azoren-Sommerzeit"}, {"Bangladesh", "Bangladesch-Sommerzeit"}, {"Brasilia", "Brasília-Sommerzeit"}, {"Cape_Verde", "Cabo-Verde-Sommerzeit"}, {"Chatham", "Chatham-Sommerzeit"}, {"Chile", "Chilenische Sommerzeit"}, {"China", "Chinesische Sommerzeit"}, {"Colombia", "Kolumbianische Sommerzeit"}, {"Cook", "Cookinseln-Sommerzeit"}, {"Cuba", "Kubanische Sommerzeit"}, {"Easter", "Osterinsel-Sommerzeit"}, {"Europe_Central", "Mitteleuropäische Sommerzeit"}, {"Europe_Eastern", "Osteuropäische Sommerzeit"}, {"Europe_Western", "Westeuropäische Sommerzeit"}, {"Falkland", "Falklandinseln-Sommerzeit"}, {"Fiji", "Fidschi-Sommerzeit"}, {"Georgia", "Georgische Sommerzeit"}, {"Greenland_Eastern", "Ostgrönland-Sommerzeit"}, {"Greenland_Western", "Westgrönland-Sommerzeit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Sommerzeit"}, {"Hong_Kong", "Hongkong-Sommerzeit"}, {"Hovd", "Chowd-Sommerzeit"}, {"Iran", "Iranische Sommerzeit"}, {"Irkutsk", "Irkutsker Sommerzeit"}, {"Israel", "Israelische Sommerzeit"}, {"Japan", "Japanische Sommerzeit"}, {"Kamchatka", "Kamtschatka-Sommerzeit"}, {"Korea", "Koreanische Sommerzeit"}, {"Krasnoyarsk", "Krasnojarsker Sommerzeit"}, {"Lord_Howe", "Lord-Howe-Sommerzeit"}, {"Macau", "Macau-Sommerzeit"}, {"Magadan", "Magadan-Sommerzeit"}, {"Mauritius", "Mauritius-Sommerzeit"}, {"Mexico_Pacific", "Mexikanische Pazifik-Sommerzeit"}, {"Mongolia", "Ulaanbaatar-Sommerzeit"}, {"Moscow", "Moskauer Sommerzeit"}, {"New_Caledonia", "Neukaledonische Sommerzeit"}, {"New_Zealand", "Neuseeland-Sommerzeit"}, {"Newfoundland", "Neufundland-Sommerzeit"}, {"Norfolk", "Norfolkinsel-Sommerzeit"}, {"Noronha", "Fernando-de-Noronha-Sommerzeit"}, {"Novosibirsk", "Nowosibirsker Sommerzeit"}, {"Omsk", "Omsker Sommerzeit"}, {"Pakistan", "Pakistanische Sommerzeit"}, {"Paraguay", "Paraguayische Sommerzeit"}, {"Peru", "Peruanische Sommerzeit"}, {"Philippines", "Philippinische Sommerzeit"}, {"Pierre_Miquelon", "St.-Pierre-und-Miquelon-Sommerzeit"}, {"Qyzylorda", "Qysylorda-Sommerzeit"}, {"Sakhalin", "Sachalin-Sommerzeit"}, {"Samara", "Samara-Sommerzeit"}, {"Samoa", "Samoa-Sommerzeit"}, {"Taipei", "Taipeh-Sommerzeit"}, {"Tonga", "Tongaische Sommerzeit"}, {"Turkey", "Türkische Sommerzeit"}, {"Turkmenistan", "Turkmenische Sommerzeit"}, {"Uruguay", "Uruguayische Sommerzeit"}, {"Uzbekistan", "Usbekische Sommerzeit"}, {"Vanuatu", "Vanuatu-Sommerzeit"}, {"Vladivostok", "Wladiwostoker Sommerzeit"}, {"Volgograd", "Wolgograder Sommerzeit"}, {"Yakutsk", "Jakutsker Sommerzeit"}, {"Yekaterinburg", "Jekaterinburger Sommerzeit"}},
	},
	{
		{{"Acre", "Acre-Normalzeit"}, {"Afghanistan", "Afghanistan-Zeit"}, {"Africa_Central", "Zentralafrikanische Zeit"}, {"Africa_Eastern", "Ostafrikanische Zeit"}, {"Africa_Southern", "Südafrikanische Zeit"}, {"Africa_Western", "Westafrikanische Zeit"}, {"Alaska", "Alaska-Normalzeit"}, {"Almaty", "Almaty-Normalzeit"}, {"Amazon", "Amazonas-Normalzeit"}, {"America_Central", "Nordamerikanische Zentral-Normalzeit"}, {"America_Eastern", "Nordamerikanische Ostküsten-Normalzeit"}, {"America_Mountain", "Rocky-Mountains-Normalzeit"}, {"America_Pacific", "Nordamerikanische Westküsten-Normalzeit"}, {"Anadyr", "Anadyr Normalzeit"}, {"Apia", "Apia-Normalzeit"}, {"Aqtau", "Aqtau-Normalzeit"}, {"Aqtobe", "Aqtöbe-Normalzeit"}, {"Arabian", "Arabische Normalzeit"}, {"Argentina", "Argentinische Normalzeit"}, {"Argentina_Western", "Westargentinische Normalzeit"}, {"Armenia", "Armenische Normalzeit"}, {"Atlantic", "Atlantik-Normalzeit"}, {"Australia_Central", "Zentralaustralische Normalzeit"}, {"Australia_CentralWestern", "Zentral-/Westaustralische Normalzeit"}, {"Australia_Eastern", "Ostaustralische Normalzeit"}, {"Australia_Western", "Westaustralische Normalzeit"}, {"Azerbaijan", "Aserbeidschanische Normalzeit"}, {"Azores", "Azoren-Normalzeit"}, {"Bangladesh", "Bangladesch-Normalzeit"}, {"Bhutan", "Bhutan-Zeit"}, {"Bolivia", "Bolivianische Zeit"}, {"Brasilia", "Brasília-Normalzeit"}, {"Brunei", "Brunei-Zeit"}, {"Cape_Verde", "Cabo-Verde-Normalzeit"}, {"Casey", "Casey-Zeit"}, {"Chamorro", "Chamorro-Zeit"}, {"Chatham", "Chatham-Normalzeit"}, {"Chile", "Chilenische Normalzeit"}, {"China", "Chinesische Normalzeit"}, {"Christmas", "Weihnachtsinsel-Zeit"}, {"Cocos", "Kokosinseln-Zeit"}, {"Colombia", "Kolumbianische Normalzeit"}, {"Cook", "Cookinseln-Normalzeit"}, {"Cuba", "Kubanische Normalzeit"}, {"Davis", "Davis-Zeit"}, {"DumontDUrville", "Dumont-d’Urville-Zeit"}, {"East_Timor", "Osttimor-Zeit"}, {"Easter", "Osterinsel-Normalzeit"}, {"Ecuador", "Ecuadorianische Zeit"}, {"Europe_Central", "Mitteleuropäische Normalzeit"}, {"Europe_Eastern", "Osteuropäische Normalzeit"}, {"Europe_Further_Eastern", "Kaliningrader Zeit"}, {"Europe_Western", "Westeuropäische Normalzeit"}, {"Falkland", "Falklandinseln-Normalzeit"}, {"Fiji", "Fidschi-Normalzeit"}, {"French_Guiana", "Französisch-Guayana-Zeit"}, {"French_Southern", "Französische-Süd-und-Antarktisgebiete-Zeit"}, {"GMT", "Mittlere Greenwich-Zeit"}, {"Galapagos", "Galapagos-Zeit"}, {"Gambier", "Gambier-Zeit"}, {"Georgia", "Georgische Normalzeit"}, {"Gilbert_Islands", "Gilbert-Inseln-Zeit"}, {"Greenland_Eastern", "Ostgrönland-Normalzeit"}, {"Greenland_Western", "Westgrönland-Normalzeit"}, {"Guam", "Guam-Zeit"}, {"Gulf", "Golf-Zeit"}, {"Guyana", "Guyana-Zeit"}, {"Hawaii", "Hawaii-Aleuten-Normalzeit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Normalzeit"}, {"Hong_Kong", "Hongkong-Normalzeit"}, {"Hovd", "Chowd-Normalzeit"}, {"India", "Indische Normalzeit"}, {"Indian_Ocean", "Indischer-Ozean-Zeit"}, {"Indochina", "Indochina-Zeit"}, {"Indonesia_Central", "Zentralindonesische Zeit"}, {"Indonesia_Eastern", "Ostindonesische Zeit"}, {"Indonesia_Western", "Westindonesische Zeit"}, {"Iran", "Iranische Normalzeit"}, {"Irkutsk", "Irkutsker Normalzeit"}, {"Israel", "Israelische Normalzeit"}, {"Japan", "Japanische Normalzeit"}, {"Kamchatka", "Kamtschatka-Normalzeit"}, {"Kazakhstan", "Kasachische Zeit"}, {"Kazakhstan_Eastern", "Ostkasachische Zeit"}, {"Kazakhstan_Western", "Westkasachische Zeit"}, {"Korea", "Koreanische Normalzeit"}, {"Kosrae", "Kosrae-Zeit"}, {"Krasnoyarsk", "Krasnojarsker Normalzeit"}, {"Kyrgystan", "Kirgisische Zeit"}, {"Lanka", "Sri-Lanka-Zeit"}, {"Line_Islands", "Linieninseln-Zeit"}, {"Lord_Howe", "Lord-Howe-Normalzeit"}, {"Macau", "Macau-Normalzeit"}, {"Magadan", "Magadan-Normalzeit"}, {"Malaysia", "Malaysische Zeit"}, {"Maldives", "Malediven-Zeit"}, {"Marquesas", "Marquesas-Zeit"}, {"Marshall_Islands", "Marshallinseln-Zeit"}, {"Mauritius", "Mauritius-Normalzeit"}, {"Mawson", "Mawson-Zeit"}, {"Mexico_Pacific", "Mexikanische Pazifik-Normalzeit"}, {"Mongolia", "Ulaanbaatar-Normalzeit"}, {"Moscow", "Moskauer Normalzeit"}, {"Myanmar", "Myanmar-Zeit"}, {"Nauru", "Nauru-Zeit"}, {"Nepal", "Nepalesische Zeit"}, {"New_Caledonia", "Neukaledonische Normalzeit"}, {"New_Zealand", "Neuseeland-Normalzeit"}, {"Newfoundland", "Neufundland-Normalzeit"}, {"Niue", "Niue-Zeit"}, {"Norfolk", "Norfolkinsel-Normalzeit"}, {"Noronha", "Fernando-de-Noronha-Normalzeit"}, {"North_Mariana", "Nördliche-Marianen-Zeit"}, {"Novosibirsk", "Nowosibirsker Normalzeit"}, {"Omsk", "Omsker Normalzeit"}, {"Pakistan", "Pakistanische Normalzeit"}, {"Palau", "Palau-Zeit"}, {"Papua_New_Guinea", "Papua-Neuguinea-Zeit"}, {"Paraguay", "Paraguayische Normalzeit"}, {"Peru", "Peruanische Normalzeit"}, {"Philippines", "Philippinische Normalzeit"}, {"Phoenix_Islands", "Phoenixinseln-Zeit"}, {"Pierre_Miquelon", "St.-Pierre-und-Miquelon-Normalzeit"}, {"Pitcairn", "Pitcairninseln-Zeit"}, {"Ponape", "Pohnpei-Zeit"}, {"Pyongyang", "Pjöngjang-Zeit"}, {"Qyzylorda", "Quysylorda-Normalzeit"}, {"Reunion", "Réunion-Zeit"}, {"Rothera", "Rothera-Zeit"}, {"Sakhalin", "Sachalin-Normalzeit"}, {"Samara", "Samara-Normalzeit"}, {"Samoa", "Samoa-Normalzeit"}, {"Seychelles", "Seychellen-Zeit"}, {"Singapore", "Singapurische Normalzeit"}, {"Solomon", "Salomoninseln-Zeit"}, {"South_Georgia", "Südgeorgische Zeit"}, {"Suriname", "Suriname-Zeit"}, {"Syowa", "Syowa-Zeit"}, {"Tahiti", "Tahiti-Zeit"}, {"Taipei", "Taipeh-Normalzeit"}, {"Tajikistan", "Tadschikische Zeit"}, {"Tokelau", "Tokelau-Zeit"}, {"Tonga", "Tongaische Normalzeit"}, {"Truk", "Chuuk-Zeit"}, {"Turkey", "Türkische Normalzeit"}, {"Turkmenistan", "Turkmenische Normalzeit"}, {"Tuvalu", "Tuvalu-Zeit"}, {"Uruguay", "Uruguayische Normalzeit"}, {"Uzbekistan", "Usbekische Normalzeit"}, {"Vanuatu", "Vanuatu-Normalzeit"}, {"Venezuela", "Venezuela-Zeit"}, {"Vladivostok", "Wladiwostoker Normalzeit"}, {"Volgograd", "Wolgograder Normalzeit"}, {"Vostok", "Wostok-Zeit"}, {"Wake", "Wake-Insel-Zeit"}, {"Wallis", "Wallis-und-Futuna-Zeit"}, {"Yakutsk", "Jakutsker Normalzeit"}, {"Yekaterinburg", "Jekaterinburger Normalzeit"}, {"Yukon", "Yukon-Zeit"}},
		{{"Acre", "Acre-Sommerzeit"}, {"Alaska", "Alaska-Sommerzeit"}, {"Almaty", "Almaty-Sommerzeit"}, {"Amazon", "Amazonas-Sommerzeit"}, {"America_Central", "Nordamerikanische Zentral-Sommerzeit"}, {"America_Eastern", "Nordamerikanische Ostküsten-Sommerzeit"}, {"America_Mountain", "Rocky-Mountains-Sommerzeit"}, {"America_Pacific", "Nordamerikanische Westküsten-Sommerzeit"}, {"Anadyr", "Anadyr Sommerzeit"}, {"Apia", "Apia-Sommerzeit"}, {"Aqtau", "Aqtau-Sommerzeit"}, {"Aqtobe", "Aqtöbe-Sommerzeit"}, {"Arabian", "Arabische Sommerzeit"}, {"Argentina", "Argentinische Sommerzeit"}, {"Argentina_Western", "Westargentinische Sommerzeit"}, {"Armenia", "Armenische Sommerzeit"}, {"Atlantic", "Atlantik-Sommerzeit"}, {"Australia_Central", "Zentralaustralische Sommerzeit"}, {"Australia_CentralWestern", "Zentral-/Westaustralische Sommerzeit"}, {"Australia_Eastern", "Ostaustralische Sommerzeit"}, {"Australia_Western", "Westaustralische Sommerzeit"}, {"Azerbaijan", "Aserbaidschanische Sommerzeit"}, {"Azores", "Azoren-Sommerzeit"}, {"Bangladesh", "Bangladesch-Sommerzeit"}, {"Brasilia", "Brasília-Sommerzeit"}, {"Cape_Verde", "Cabo-Verde-Sommerzeit"}, {"Chatham", "Chatham-Sommerzeit"}, {"Chile", "Chilenische Sommerzeit"}, {"China", "Chinesische Sommerzeit"}, {"Colombia", "Kolumbianische Sommerzeit"}, {"Cook", "Cookinseln-Sommerzeit"}, {"Cuba", "Kubanische Sommerzeit"}, {"Easter", "Osterinsel-Sommerzeit"}, {"Europe_Central", "Mitteleuropäische Sommerzeit"}, {"Europe_Eastern", "Osteuropäische Sommerzeit"}, {"Europe_Western", "Westeuropäische Sommerzeit"}, {"Falkland", "Falklandinseln-Sommerzeit"}, {"Fiji", "Fidschi-Sommerzeit"}, {"Georgia", "Georgische Sommerzeit"}, {"Greenland_Eastern", "Ostgrönland-Sommerzeit"}, {"Greenland_Western", "Westgrönland-Sommerzeit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Sommerzeit"}, {"Hong_Kong", "Hongkong-Sommerzeit"}, {"Hovd", "Chowd-Sommerzeit"}, {"Iran", "Iranische Sommerzeit"}, {"Irkutsk", "Irkutsker Sommerzeit"}, {"Israel", "Israelische Sommerzeit"}, {"Japan", "Japanische Sommerzeit"}, {"Kamchatka", "Kamtschatka-Sommerzeit"}, {"Korea", "Koreanische Sommerzeit"}, {"Krasnoyarsk", "Krasnojarsker Sommerzeit"}, {"Lord_Howe", "Lord-Howe-Sommerzeit"}, {"Macau", "Macau-Sommerzeit"}, {"Magadan", "Magadan-Sommerzeit"}, {"Mauritius", "Mauritius-Sommerzeit"}, {"Mexico_Pacific", "Mexikanische Pazifik-Sommerzeit"}, {"Mongolia", "Ulaanbaatar-Sommerzeit"}, {"Moscow", "Moskauer Sommerzeit"}, {"New_Caledonia", "Neukaledonische Sommerzeit"}, {"New_Zealand", "Neuseeland-Sommerzeit"}, {"Newfoundland", "Neufundland-Sommerzeit"}, {"Norfolk", "Norfolkinsel-Sommerzeit"}, {"Noronha", "Fernando-de-Noronha-Sommerzeit"}, {"Novosibirsk", "Nowosibirsker Sommerzeit"}, {"Omsk", "Omsker Sommerzeit"}, {"Pakistan", "Pakistanische Sommerzeit"}, {"Paraguay", "Paraguayische Sommerzeit"}, {"Peru", "Peruanische Sommerzeit"}, {"Philippines", "Philippinische Sommerzeit"}, {"Pierre_Miquelon", "St.-Pierre-und-Miquelon-Sommerzeit"}, {"Qyzylorda", "Qysylorda-Sommerzeit"}, {"Sakhalin", "Sachalin-Sommerzeit"}, {"Samara", "Samara-Sommerzeit"}, {"Samoa", "Samoa-Sommerzeit"}, {"Taipei", "Taipeh-Sommerzeit"}, {"Tonga", "Tongaische Sommerzeit"}, {"Turkey", "Türkische Sommerzeit"}, {"Turkmenistan", "Turkmenische Sommerzeit"}, {"Uruguay", "Uruguayische Sommerzeit"}, {"Uzbekistan", "Usbekische Sommerzeit"}, {"Vanuatu", "Vanuatu-Sommerzeit"}, {"Vladivostok", "Wladiwostoker Sommerzeit"}, {"Volgograd", "Wolgograder Sommerzeit"}, {"Yakutsk", "Jakutsker Sommerzeit"}, {"Yekaterinburg", "Jekaterinburger Sommerzeit"}},
	},
	{
		{{"America_Central", "उत्तरी अमरीकी मानक केंदरी समां"}, {"America_Eastern", "उत्तरी अमरीकी मानक पूर्वी समां"}, {"America_Mountain", "उत्तरी अमरीकी मानक माउंटेन समां"}, {"America_Pacific", "उत्तरी अमरीकी मानक प्रशांत समां"}, {"Atlantic", "अटलांटिक मानक समां"}, {"Europe_Central", "केंदरी यूरोपी मानक समां"}, {"Europe_Eastern", "उत्तरी यूरोपी मानक समां"}, {"Europe_Western", "पच्छमी यूरोपी मानक समां"}, {"GMT", "ग्रीनविच मीन टाइम"}},
		{{"America_Central", "उत्तरी अमरीकी डेलाइट केंदरी समां"}, {"America_Eastern", "उत्तरी अमरीकी डेलाइट पूर्वी समां"}, {"America_Mountain", "उत्तरी अमरीकी डेलाइट माउंटेन समां"}, {"America_Pacific", "उत्तरी अमरीकी डेलाइट प्रशांत समां"}, {"Atlantic", "अटलांटिक डेलाइट समां"}, {"Europe_Central", "केंदरी यूरोपी गर्मियें दा समां"}, {"Europe_Eastern", "उत्तरी यूरोपी गर्मियें दा समां"}, {"Europe_Western", "पच्छमी यूरोपी गर्मियें दा समां"}},
	},
	{
		{{"Afghanistan", "Afghaniski cas"}, {"Africa_Central", "Srjejźoafriski cas"}, {"Africa_Eastern", "Pódzajtšnoafriski cas"}, {"Africa_Southern", "Pódpołdnjowoafriski cas"}, {"Africa_Western", "Pódwjacornoafriski cas"}, {"Alaska", "Alaskojski standardny cas"}, {"Amazon", "Amaconaski standardny cas"}, {"America_Central", "Pódpołnocnoameriski centralny standardny cas"}, {"America_Eastern", "Pódpołnocnoameriski pódzajtšny standardny cas"}, {"America_Mountain", "Pódpołnocnoameriski górski standardny cas"}, {"America_Pacific", "Pódpołnocnoameriski pacifiski standardny cas"}, {"Apia", "Apiaski standardny cas"}, {"Arabian", "Arabiski standardny cas"}, {"Argentina", "Argentinski standardny cas"}, {"Argentina_Western", "Pódwjacornoargentinski standardny cas"}, {"Armenia", "Armeński standardny cas"}, {"Atlantic", "Atlantiski standardny cas"}, {"Australia_Central", "Srjejźoawstralski standardny cas"}, {"Australia_CentralWestern", "Srjejźopódwjacorny awstralski standardny cas"}, {"Australia_Eastern", "Pódzajtšnoawstralski standardny cas"}, {"Australia_Western", "Pódwjacornoawstralski standardny cas"}, {"Azerbaijan", "Azerbajdžaniski standardny cas"}, {"Azores", "Acorski standardny cas"}, {"Bangladesh", "Bangladešski standardny cas"}, {"Bhutan", "Bhutański cas"}, {"Bolivia", "Boliwiski cas"}, {"Brasilia", "Brasília standardny cas"}, {"Brunei", "Bruneiski cas"}, {"Cape_Verde", "Kapverdski standardny cas"}, {"Chamorro", "Chamorrski cas"}, {"Chatham", "Chathamski standardny cas"}, {"Chile", "Chilski standardny cas"}, {"China", "Chinski standardny cas"}, {"Christmas", "cas Gódownych kupow"}, {"Cocos", "cas Kokosowych kupow"}, {"Colombia", "Kolumbiski standardny cas"}, {"Cook", "Standardny cas Cookowych kupow"}, {"Cuba", "Kubański standardny cas"}, {"Davis", "Davis cas"}, {"DumontDUrville", "DumontDUrville cas"}, {"East_Timor", "Pódzajtšnotimorski cas"}, {"Easter", "standardny cas Jatšowneje kupy"}, {"Ecuador", "Ekuadorski cas"}, {"Europe_Central", "Srjejźnoeuropejski standardny cas"}, {"Europe_Eastern", "Pódzajtšnoeuropejski standardny cas"}, {"Europe_Further_Eastern", "Kaliningradski cas"}, {"Europe_Western", "Pódwjacornoeuropejski standardny cas"}, {"Falkland", "Falklandski standardny cas"}, {"Fiji", "Fidźiski standardny cas"}, {"French_Guiana", "Francojskoguyański cas"}, {"French_Southern", "cas francojskego pódpołdnjowego a antarktiskeho teritoriuma"}, {"GMT", "Greenwichski cas"}, {"Galapagos", "Galapagoski cas"}, {"Gambier", "Gambierski cas"}, {"Georgia", "Georgiski standardny cas"}, {"Gilbert_Islands", "cas Gilbertowych kupow"}, {"Greenland_Eastern", "Pódzajtšnogrönlandski standardny cas"}, {"Greenland_Western", "Pódwjacornogrönlandski standardny cas"}, {"Gulf", "cas Persiskego golfa"}, {"Guyana", "Guyański cas"}, {"Hawaii", "Hawaiisko-aleutski standardny cas"}, {"Hawaii_Aleutian", "Hawaiisko-aleutski standardny cas"}, {"Hong_Kong", "Hongkongski standardny cas"}, {"Hovd", "Chowdski standardny cas"}, {"India", "Indiski cas"}, {"Indian_Ocean", "Indiskooceaniski cas"}, {"Indochina", "Indochinski cas"}, {"Indonesia_Central", "Srjejźoindoneski cas"}, {"Indonesia_Eastern", "Pódzajtšnoindoneski"}, {"Indonesia_Western", "Pódwjacornoindoneski cas"}, {"Iran", "Irański standardny cas"}, {"Irkutsk", "Irkutski standardny cas"}, {"Israel", "Israelski standardny cas"}, {"Japan", "Japański standardny cas"}, {"Kazakhstan", "kazachiski cas"}, {"Kazakhstan_Eastern", "Pódzajtšnokazachski cas"}, {"Kazakhstan_Western", "Pódwjacornokazachski cas"}, {"Korea", "Korejski standardny cas"}, {"Kosrae", "Kosraeski cas"}, {"Krasnoyarsk", "Krasnojarski standardny cas"}, {"Kyrgystan", "Kirgiski cas"}, {"Line_Islands", "cas Linijowych kupow"}, {"Lord_Howe", "Standardny cas kupy Lord-Howe"}, {"Magadan", "Magadański standardny cas"}, {"Malaysia", "Malajziski cas"}, {"Maldives", "Malediwski cas"}, {"Marquesas", "Marqueski cas"}, {"Marshall_Islands", "cas Marshallowych kupow"}, {"Mauritius", "Mauriciski standardny cas"}, {"Mawson", "Mawson cas"}, {"Mexico_Pacific", "Mexiski pacifiski standardny cas"}, {"Mongolia", "Ulan-Batorski standardny cas"}, {"Moscow", "Moskowski standardny cas"}, {"Myanmar", "Myanmarski cas"}, {"Nauru", "Nauruski cas"}, {"Nepal", "Nepalski cas"}, {"New_Caledonia", "Nowokaledoniski standardny cas"}, {"New_Zealand", "Nowoseelandski standardny cas"}, {"Newfoundland", "Nowofundlandski standardny cas"}, {"Niue", "Niueski cas"}, {"Norfolk", "standardny cas kupy Norfolk"}, {"Noronha", "standardny cas Fernando de Noronha"}, {"Novosibirsk", "Nowosibirski standardny cas"}, {"Omsk", "Omski standardny cas"}, {"Pakistan", "Pakistański standardny cas"}, {"Palau", "Palauski cas"}, {"Papua_New_Guinea", "Papua-Nowoginejski cas"}, {"Paraguay", "Paraguayski standardny cas"}, {"Peru", "Peruski standardny cas"}, {"Philippines", "Filipinski standardny cas"}, {"Phoenix_Islands", "cas Phoenixowych kupow"}, {"Pierre_Miquelon", "St.-Pierre-a-Miqueloński standardny cas"}, {"Pitcairn", "cas Pitcairnowych kupow"}, {"Ponape", "Ponapski cas"}, {"Pyongyang", "Pjöngjangski cas"}, {"Reunion", "Reunionski cas"}, {"Rothera", "cas Rothera"}, {"Sakhalin", "Sachalinski standardny cas"}, {"Samoa", "Samoaski standardny cas"}, {"Seychelles", "Seychelski cas"}, {"Singapore", "Singapurski cas"}, {"Solomon", "Salomoński cas"}, {"South_Georgia", "Pódpołdnjowogeorgiski cas"}, {"Suriname", "Surinamski cas"}, {"Syowa", "Syowa cas"}, {"Tahiti", "Tahitiski cas"}, {"Taipei", "Tchajpejski standardny cas"}, {"Tajikistan", "Tadźikiski cas"}, {"Tokelau", "Tokelauski cas"}, {"Tonga", "Tongaski standardny cas"}, {"Truk", "Chuukski cas"}, {"Turkmenistan", "Turkmeniski standardny cas"}, {"Tuvalu", "Tuvalski cas"}, {"Uruguay", "Uruguayski standardny cas"}, {"Uzbekistan", "Uzbekiski standardny cas"}, {"Vanuatu", "Vanuatski standardny cas"}, {"Venezuela", "Venezuelski cas"}, {"Vladivostok", "Wladiwostokski standardny cas"}, {"Volgograd", "Wolgogradski standardny cas"}, {"Vostok", "cas Wostok"}, {"Wake", "cas kupy Wake"}, {"Wallis", "cas kupow Wallis a Futuna"}, {"Yakutsk", "Jakutski standardny cas"}, {"Yekaterinburg", "Jekaterinburgski standardny cas"}, {"Yukon", "Yukonowy cas"}},
		{{"Alaska", "Alaskojski lěśojski cas"}, {"Amazon", "Amaconaski lěśojski cas"}, {"America_Central", "Pódpołnocnoameriski centralny lěśojski cas"}, {"America_Eastern", "Pódpołnocnoameriski pódzajtšny lěśojski cas"}, {"America_Mountain", "Pódpołnocnoameriski górski lěśojski cas"}, {"America_Pacific", "Pódpołnocnoameriski pacifiski lěśojski cas"}, {"Apia", "Apiaski lěśojski cas"}, {"Arabian", "Arabiski lěśojski cas"}, {"Argentina", "Argentinski lěśojski cas"}, {"Argentina_Western", "Pódwjacornoargentinski lěśojski cas"}, {"Armenia", "Armeński lěśojski cas"}, {"Atlantic", "Atlantiski lěśojski cas"}, {"Australia_Central", "Srjejźoawstralski lěśojski cas"}, {"Australia_CentralWestern", "Srjejźopódwjacorny awstralski lěśojski cas"}, {"Australia_Eastern", "Pódzajtšnoawstralski lěśojski cas"}, {"Australia_Western", "Pódwjacornoawstralski lěśojski cas"}, {"Azerbaijan", "Azerbajdžaniski lěśojski cas"}, {"Azores", "Acorski lěśojski cas"}, {"Bangladesh", "Bangladešski lěśojski cas"}, {"Brasilia", "Brasília lěśojski cas"}, {"Cape_Verde", "Kapverdski lěśojski cas"}, {"Chatham", "Chathamski lěśojski cas"}, {"Chile", "Chilski lěśojski cas"}, {"China", "Chinski lěśojski cas"}, {"Colombia", "Kolumbiski lěśojski cas"}, {"Cook", "lěśojski cas Cookowych kupow"}, {"Cuba", "Kubański lěśojski cas"}, {"Easter", "lěśojski cas Jatšowneje kupy"}, {"Europe_Central", "Srjejźnoeuropejski lěśojski cas"}, {"Europe_Eastern", "Pódzajtšnoeuropejski lěśojski cas"}, {"Europe_Western", "Pódwjacornoeuropejski lěśojski cas"}, {"Falkland", "Falklandski lěśojski cas"}, {"Fiji", "Fidźiski lěśojski cas"}, {"Georgia", "Georgiski lěśojski cas"}, {"Greenland_Eastern", "Pódzajtšnogrönlandski lěśojski cas"}, {"Greenland_Western", "Pódwjacornogrönlandski lěśojski cas"}, {"Hawaii_Aleutian", "Hawaiisko-aleutski lěśojski cas"}, {"Hong_Kong", "Hongkongski lěśojski cas"}, {"Hovd", "Chowdski lěśojski cas"}, {"Iran", "Irański lěśojski cas"}, {"Irkutsk", "Irkutski lěśojski cas"}, {"Israel", "Israelski lěśojski cas"}, {"Japan", "Japański lěśojski cas"}, {"Korea", "Korejski lěśojski cas"}, {"Krasnoyarsk", "Krasnojarski lěśojski cas"}, {"Lord_Howe", "lěśojski cas kupy Lord-Howe"}, {"Magadan", "Magadański lěśojski cas"}, {"Mauritius", "Mauriciski lěśojski cas"}, {"Mexico_Pacific", "Mexiski pacifiski lěśojski cas"}, {"Mongolia", "Ulan-Batorski lěśojski cas"}, {"Moscow", "Moskowski lěśojski cas"}, {"New_Caledonia", "Nowokaledoniski lěśojski cas"}, {"New_Zealand", "Nowoseelandski lěśojski cas"}, {"Newfoundland", "Nowofundlandski lěśojski cas"}, {"Norfolk", "lěśojski cas kupy Norfolk"}, {"Noronha", "lěśojski cas Fernando de Noronha"}, {"Novosibirsk", "Nowosibirski lěśojski cas"}, {"Omsk", "Omski lěśojski cas"}, {"Pakistan", "Pakistański lěśojski cas"}, {"Paraguay", "Paraguayski lěśojski cas"}, {"Peru", "Peruski lěśojski cas"}, {"Philippines", "Filipinski lěśojski cas"}, {"Pierre_Miquelon", "St.-Pierre-a-Miqueloński lěśojski cas"}, {"Sakhalin", "Sachalinski lěśojski cas"}, {"Samoa", "Samoaski lěśojski cas"}, {"Taipei", "Tchajpejski lěśojski cas"}, {"Tonga", "Tongaski lěśojski cas"}, {"Turkmenistan", "Turkmeniski lěśojski cas"}, {"Uruguay", "Uruguayski lěśojski cas"}, {"Uzbekistan", "Uzbekiski lěśojski cas"}, {"Vanuatu", "Vanuatski lěśojski cas"}, {"Vladivostok", "Wladiwostokski lěśojski cas"}, {"Volgograd", "Wolgogradski lěśojski cas"}, {"Yakutsk", "Jakutski lěśojski cas"}, {"Yekaterinburg", "Jekaterinburgski lěśojski cas"}},
	},
	{
		{{"Afghanistan", "ཨཕ་ག་ནི་ས྄ཏཱནཆུ་ཚོད"}, {"Africa_Central", "དབུས་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད"}, {"Africa_Eastern", "ཤར་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད"}, {"Africa_Southern", "ལྷོ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད"}, {"Africa_Western", "ནུབ་ཕྱོགས་ཨཕ་རི་ཀཱ་ཆུ་ཚོད"}, {"Alaska", "ཨ་ལསི་ཀ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Amazon", "ཨེ་མ་ཛཱོན་ཚད་ལྡན་ཆུ་ཚོད"}, {"America_Central", "བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཚད་ལྡན་ཆུ་ཚོད"}, {"America_Eastern", "བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཚད་ལྡན་ཆུ་ཚོད"}, {"America_Mountain", "བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"America_Pacific", "བྱང་ཨ་མི་རི་ཀ་པེ་སི་ཕིག་ཚད་ལྡན་ཆུ་ཚོད"}, {"Arabian", "ཨ་རེ་བྷི་ཡན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Argentina", "ཨར་ཇེན་ཊི་ན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Argentina_Western", "ནུབ་ཕྱོགས་ཨར་ཇེན་ཊི་ན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Armenia", "ཨར་མི་ནི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Atlantic", "ཨེཊ་ལེན་ཊིཀ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Australia_Central", "དབུས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Australia_CentralWestern", "དབུས་ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Australia_Eastern", "ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Australia_Western", "ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Azerbaijan", "ཨ་ཛར་བྷའི་ཇཱན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Azores", "ཨེ་ཛོརས་ཚད་ལྡན་ཆུ་ཚོད"}, {"Bangladesh", "བངྒ་ལ་དེཤ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Bhutan", "འབྲུག་ཡུལ་ཆུ་ཚོད"}, {"Bolivia", "བྷོ་ལི་བི་ཡ་ཆུ་ཚོད"}, {"Brasilia", "བྲ་ཛི་ལི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Cape_Verde", "ཀེཔ་བཱཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Chile", "ཅི་ལི་ཚད་ལྡན་ཆུ་ཚོད"}, {"China", "རྒྱ་ནག་ཚད་ལྡན་ཆུ་ཚོད"}, {"Christmas", "ཁི་རིསྟ་མེས་མཚོ་གླིང་ཆུ་ཚོད"}, {"Colombia", "ཀོ་ལོམ་བྷི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Cuba", "ཀིའུ་བྷ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Easter", "ཨིསི་ཊར་ཨཱའི་ལེནཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Ecuador", "ཨེ་ཀུ་ཌཽ་ཆུ་ཚོད"}, {"Europe_Central", "དབུས་ཕྱོགས་ཡུ་རོ་པེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Europe_Eastern", "ཤར་ཕྱོགས་ཡུ་རོ་པེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Europe_Western", "ནུབ་ཕྱོགས་ཡུ་རོ་པེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Falkland", "ཕལཀ་ལེནཌ་ཨཱའི་ལེནཌས་ཚད་ལྡན་ཆུ་ཚོད"}, {"French_Guiana", "ཕིརེནཅ་གི་ཡ་ན་ཆུ་ཚོད"}, {"GMT", "གིརིན་ཝིཆ་ལུ་ཡོད་པའི་ཆུ་ཚོད"}, {"Galapagos", "ག་ལ་པ་གོསི་ཆུ་ཚོད"}, {"Georgia", "ཇཽ་ཇཱ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Greenland_Eastern", "ཤར་ཕྱོགས་གིརིན་ལེནཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Greenland_Western", "ནུབ་ཕྱོགས་གིརིན་ལེནཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Gulf", "གཱལཕི་ཆུ་ཚོད"}, {"Guyana", "གུ་ཡ་ན་ཆུ་ཚོད"}, {"Hawaii", "ཧ་ཝའི་-ཨེ་ལིའུ་ཤེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Hawaii_Aleutian", "ཧ་ཝའི་-ཨེ་ལིའུ་ཤེན་ཚད་ལྡན་ཆུ་ཚོད"}, {"India", "རྒྱ་གར་ཆུ་ཚོད"}, {"Indian_Ocean", "རྒྱ་གར་གྱི་རྒྱ་མཚོ་ཆུ་ཚོད"}, {"Indochina", "ཨིན་ཌོ་ཅཱའི་ན་ཆུ་ཚོད"}, {"Indonesia_Central", "དབུས་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད"}, {"Indonesia_Eastern", "ཤར་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད"}, {"Indonesia_Western", "ནུབ་ཕྱོགས་ཨིན་ཌོ་ནེ་ཤི་ཡ་ཆུ་ཚོད"}, {"Iran", "ཨི་རཱན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Irkutsk", "ཨར་ཀུཙི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Israel", "ཨིས་རེལ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Japan", "ཇ་པཱན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Korea", "ཀོ་རི་ཡ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Krasnoyarsk", "ཀརསི་ནོ་ཡརསཀི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Magadan", "མ་གྷ་དཱན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Maldives", "མཱལ་དིབས་ཆུ་ཚོད"}, {"Mauritius", "མོ་རི་ཤཱས་ཚད་ལྡན་ཆུ་ཚོད"}, {"Moscow", "མཽས་ཀོ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Nepal", "ནེ་པཱལ་ཆུ་ཚོད"}, {"New_Zealand", "ནིའུ་ཛི་ལེནཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Newfoundland", "ནིའུ་ཕའུནཌ་ལེནཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Noronha", "ཕར་ནེན་ཌོ་ ཌི་ ནོ་རཱོན་ཧ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Novosibirsk", "ནོ་བོ་སི་བིརསཀི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Omsk", "ཨོམསཀི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Pakistan", "པ་ཀི་ས྄ཏཱན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Paraguay", "པ་ར་གུ་ཝའི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Peru", "པ་རུ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Pierre_Miquelon", "པའི་རི་དང་མི་ཀི་ལཱོན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Reunion", "རི་ཡུ་ནི་ཡཱན་ཆུ་ཚོད"}, {"Sakhalin", "ས་ཁ་ལིན་ཚད་ལྡན་ཆུ་ཚོད"}, {"Seychelles", "སེ་ཤཱལས་ཆུ་ཚོད"}, {"Suriname", "སུ་རི་ནཱམ་ཆུ་ཚོད"}, {"Uruguay", "ཡུ་རུ་གུ་ཝཱའི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Venezuela", "བེ་ནི་ཛུ་ཝེ་ལ་ཆུ་ཚོད"}, {"Vladivostok", "བ་ལ་ཌི་བོསི་ཏོཀ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Volgograd", "བཱོལ་གོ་གིརེཌ་ཚད་ལྡན་ཆུ་ཚོད"}, {"Yakutsk", "ཡ་ཀུཙིཀི་ཚད་ལྡན་ཆུ་ཚོད"}, {"Yekaterinburg", "ཡེ་ཀ་ཏེ་རིན་བརག་ཚད་ལྡན་ཆུ་ཚོད"}},
		{{"Alaska", "ཨ་ལསི་ཀ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Amazon", "ཨེ་མ་ཛཱོན་བྱཱར་དུས་ཆུ་ཚོད"}, {"America_Central", "བྱང་ཨ་མི་རི་ཀ་དབུས་ཕྱོགས་ཉིན་སྲུང་ཆུ་ཚོད"}, {"America_Eastern", "བྱང་ཨ་མི་རི་ཀ་ཤར་ཕྱོགས་ཉིན་སྲུང་ཆུ་ཚོད"}, {"America_Mountain", "བྱང་ཨ་མི་རི་ཀ་མའུ་ཊེན་ཉིན་སྲུང་ཆུ་ཚོད"}, {"America_Pacific", "བྱང་ཨ་མི་རི་ཀ་པེ་སི་ཕིག་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Arabian", "ཨ་རེ་བྷི་ཡན་སྲུང་ཆུ་ཚོད"}, {"Argentina", "ཨར་ཇེན་ཊི་ན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Argentina_Western", "ནུབ་ཕྱོགས་ཨར་ཇེན་ཊི་ན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Armenia", "ཨར་མི་ནི་ཡ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Atlantic", "ཨེཊ་ལེན་ཊིཀ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Australia_Central", "དབུས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Australia_CentralWestern", "དབུས་ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Australia_Eastern", "ཤར་ཕྱོགས་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Australia_Western", "ནུབ་ཕྱོགས་ཨཱོས་ཊྲེལ་ལི་ཡ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Azerbaijan", "ཨ་ཛར་བྷའི་ཇཱན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Azores", "ཨེ་ཛོརས་བྱཱར་དུས་ཆུ་ཚོད"}, {"Bangladesh", "བངྒ་ལ་དེཤ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Brasilia", "བྲ་ཛི་ལི་ཡ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Cape_Verde", "ཀེཔ་བཱཌ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Chile", "ཅི་ལི་བྱཱར་དུས་ཆུ་ཚོད"}, {"China", "རྒྱ་ནག་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Colombia", "ཀོ་ལོམ་བྷི་ཡ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Cuba", "ཀིའུ་བྷ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Easter", "ཨིསི་ཊར་ཨཱའི་ལེནཌ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Europe_Central", "དབུས་ཕྱོགས་ཡུ་རོ་པེན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Europe_Eastern", "ཤར་ཕྱོགས་ཡུ་རོ་པེན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Europe_Western", "ནུབ་ཕྱོགས་ཡུ་རོ་པེན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Falkland", "ཕལཀ་ལེནཌ་ཨཱའི་ལེནཌས་བྱཱར་དུས་ཆུ་ཚོད"}, {"Georgia", "ཇཽ་ཇཱ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Greenland_Eastern", "ཤར་ཕྱོགས་གིརིན་ལེནཌ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Greenland_Western", "ནུབ་ཕྱོགས་གིརིན་ལེནཌ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Hawaii_Aleutian", "ཧ་ཝའི་-ཨེ་ལིའུ་ཤེན་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Iran", "ཨི་རཱན་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Irkutsk", "ཨར་ཀུཙི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Israel", "ཨིས་རེལ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Japan", "ཇ་པཱན་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Korea", "ཀོ་རི་ཡ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Krasnoyarsk", "ཀརསི་ནོ་ཡརསཀི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Magadan", "མ་གྷ་དཱན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Mauritius", "མོ་རི་ཤཱས་བྱཱར་དུས་ཆུ་ཚོད"}, {"Moscow", "མཽས་ཀོ་བྱཱར་དུས་ཆུ་ཚོད"}, {"New_Zealand", "ནིའུ་ཛི་ལེནཌ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Newfoundland", "ནིའུ་ཕའུནཌ་ལེནཌ་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Noronha", "ཕར་ནེན་ཌོ་ ཌི་ ནོ་རཱོན་ཧ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Novosibirsk", "ནོ་བོ་སི་བིརསཀི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Omsk", "ཨོམསཀི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Pakistan", "པ་ཀི་ས྄ཏཱན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Paraguay", "པ་ར་གུ་ཝའི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Peru", "པ་རུ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Pierre_Miquelon", "པའི་རི་དང་མི་ཀི་ལཱོན་ཉིན་སྲུང་ཆུ་ཚོད"}, {"Sakhalin", "ས་ཁ་ལིན་བྱཱར་དུས་ཆུ་ཚོད"}, {"Uruguay", "ཡུ་རུ་གུ་ཝཱའི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Vladivostok", "བ་ལ་ཌི་བོསི་ཏོཀ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Volgograd", "བཱོལ་གོ་གིརེཌ་བྱཱར་དུས་ཆུ་ཚོད"}, {"Yakutsk", "ཡ་ཀུཙིཀི་བྱཱར་དུས་ཆུ་ཚོད"}, {"Yekaterinburg", "ཡེ་ཀ་ཏེ་རིན་བརག་བྱཱར་དུས་ཆུ་ཚོད"}},
	},
	{
		{{"Acre", "Eker gaƒoƒoɖoanyime"}, {"Afghanistan", "Afghanistan gaƒoƒo me"}, {"Africa_Central", "Central Africa gaƒoƒo me"}, {"Africa_Eastern", "East Africa gaƒoƒo me"}, {"Africa_Southern", "South Africa nutome gaƒoƒo me"}, {"Africa_Western", "West Africa game"}, {"Alaska", "Alaska nutome gaƒoƒo me"}, {"Almaty", "Almati gaƒoƒoɖoanyime"}, {"Amazon", "Amazon nutome gaƒoƒo me"}, {"America_Central", "Titina America nutome gaƒoƒo me"}, {"America_Eastern", "Eastern America nutome gaƒoƒo me"}, {"America_Mountain", "Mountain nutome gaƒoƒo me"}, {"America_Pacific", "Pacific nutome gaƒoƒo me"}, {"Anadyr", "Anadir gaƒoƒoɖoanyime"}, {"Apia", "Apia nutome gaƒoƒo me"}, {"Aqtau", "Aktau gaƒoƒoɖoanyime"}, {"Aqtobe", "Aktobe gaƒoƒoɖoanyime"}, {"Arabian", "Arabia nutome gaƒoƒo me"}, {"Argentina", "Argentina nutome gaƒoƒo me"}, {"Argentina_Western", "Ɣetoɖoƒe Argentina nutome gaƒoƒo me"}, {"Armenia", "Armenia nutome gaƒoƒo me"}, {"Atlantic", "Atlantic nutome gaƒoƒome"}, {"Australia_Central", "Australian Central nutome gaƒoƒo me"}, {"Australia_CentralWestern", "Australian Central Western nutome gaƒoƒo me"}, {"Australia_Eastern", "Australian Eastern nutome gaƒoƒo me"}, {"Australia_Western", "Australian Western nutome gaƒoƒo me"}, {"Azerbaijan", "Azerbaijan nutome gaƒoƒo me"}, {"Azores", "Azores nutome gaƒoƒo me"}, {"Bangladesh", "Bangladesh nutome gaƒoƒo me"}, {"Bhutan", "Bhutan gaƒoƒo me"}, {"Bolivia", "Bolivia gaƒoƒo me"}, {"Brasilia", "Brasilia nutome gaƒoƒo me"}, {"Brunei", "Brunei Darussalam gaƒoƒo me"}, {"Cape_Verde", "Cape Verde nutome gaƒoƒo me"}, {"Chamorro", "Chamorro gaƒoƒo me"}, {"Chatham", "Chatham nutome gaƒoƒo me"}, {"Chile", "Chile nutome gaƒoƒo me"}, {"China", "China nutome gaƒoƒo me"}, {"Christmas", "Christmas Island gaƒoƒo me"}, {"Cocos", "Cocos Islands gaƒoƒo me"}, {"Colombia", "Colombia nutome gaƒoƒo me"}, {"Cook", "Cook Islands nutome gaƒoƒo me"}, {"Cuba", "Cuba nutome gaƒoƒome"}, {"Davis", "Davis gaƒoƒo me"}, {"DumontDUrville", "Dumont-d’Urville gaƒoƒo me"}, {"East_Timor", "East Timor gaƒoƒo me"}, {"Easter", "Easter Island nutome gaƒoƒo me"}, {"Ecuador", "Ecuador gaƒoƒo me"}, {"Europe_Central", "Central Europe nutome gaƒoƒo me"}, {"Europe_Eastern", "Ɣedzeƒe Europe gaƒoƒoɖoanyime"}, {"Europe_Western", "Western Europe nutome gaƒoƒo me"}, {"Falkland", "Falkland Islands nutome gaƒoƒo me"}, {"Fiji", "Fiji nutome gaƒoƒo me"}, {"French_Guiana", "French Guiana gaƒoƒo me"}, {"French_Southern", "French Southern & Antarctic gaƒoƒo me"}, {"GMT", "Greenwich gaƒoƒo"}, {"Galapagos", "Galapagos gaƒoƒo me"}, {"Gambier", "Gambier gaƒoƒo me"}, {"Georgia", "Georgia nutome gaƒoƒo me"}, {"Gilbert_Islands", "Gilbert Islands gaƒoƒo me"}, {"Greenland_Eastern", "East Greenland nutome gaƒoƒo me"}, {"Greenland_Western", "West Greenland nutome gaƒoƒo me"}, {"Gulf", "Gulf nutome gaƒoƒo me"}, {"Guyana", "Guyana gaƒoƒo me"}, {"Hawaii", "Hawaii-Aleutia nutome gaƒoƒo me"}, {"Hawaii_Aleutian", "Hawaii-Aleutia nutome gaƒoƒo me"}, {"Hong_Kong", "Hong Kong nutome gaƒoƒo me"}, {"Hovd", "Hovd nutome gaƒoƒo me"}, {"India", "India gaƒoƒo me"}, {"Indian_Ocean", "Indian Ocean gaƒoƒo me"}, {"Indochina", "Indonesia gaƒoƒo me"}, {"Indonesia_Central", "Central Indonesia gaƒoƒo me"}, {"Indonesia_Eastern", "Eastern Indonesia gaƒoƒo me"}, {"Indonesia_Western", "Western Indonesia gaƒoƒo me"}, {"Iran", "Iran nutome gaƒoƒo me"}, {"Irkutsk", "Irkutsk nutome gaƒoƒo me"}, {"Israel", "Israel nutome gaƒoƒo me"}, {"Japan", "Japan nutome gaƒoƒo me"}, {"Kamchatka", "Petropavlovsk-Kamtsatski gaƒoƒoɖoanyime"}, {"Kazakhstan_Eastern", "East Kazakhstan gaƒoƒo me"}, {"Kazakhstan_Western", "West Kazakhstan gaƒoƒo me"}, {"Korea", "Korea nutome gaƒoƒo me"}, {"Kosrae", "Kosrae gaƒoƒo me"}, {"Krasnoyarsk", "Krasnoyarsk nutome gaƒoƒo me"}, {"Kyrgystan", "Kyrgystan gaƒoƒo me"}, {"Line_Islands", "Line Islands gaƒoƒo me"}, {"Lord_Howe", "Lord Howe nutome gaƒoƒo me"}, {"Macau", "Makau gaƒoƒoɖoanyime"}, {"Magadan", "Magadan nutome gaƒoƒo me"}, {"Malaysia", "Malaysia gaƒoƒo me"}, {"Maldives", "Maldives gaƒoƒo me"}, {"Marquesas", "Marquesas gaƒoƒo me"}, {"Marshall_Islands", "Marshall Islands gaƒoƒo me"}, {"Mauritius", "Mauritius nutome gaƒoƒo me"}, {"Mawson", "Mawson gaƒoƒo me"}, {"Mexico_Pacific", "Mexican Pacific nutome gaƒoƒo me"}, {"Mongolia", "Ulan Bator nutome gaƒoƒo me"}, {"Moscow", "Moscow nutome gaƒoƒo me"}, {"Myanmar", "Myanmar gaƒoƒo me"}, {"Nauru", "Nauru gaƒoƒo me"}, {"Nepal", "Nepal gaƒoƒo me"}, {"New_Caledonia", "New Caledonia nutome gaƒoƒo me"}, {"New_Zealand", "New Zealand nutome gaƒoƒo me"}, {"Newfoundland", "Newfoundland nutome gaƒoƒome"}, {"Niue", "Niue gaƒoƒo me"}, {"Norfolk", "Norfolk Island nutome gaƒoƒo me"}, {"Noronha", "Fernando de Noronha nutome gaƒoƒo me"}, {"Novosibirsk", "Novosibirsk nutome gaƒoƒo me"}, {"Omsk", "Omsk nutome gaƒoƒo me"}, {"Pakistan", "Pakistan nutome gaƒoƒo me"}, {"Palau", "Palau gaƒoƒo me"}, {"Papua_New_Guinea", "Papua New Guinea gaƒoƒo me"}, {"Paraguay", "Paraguay nutome gaƒoƒo me"}, {"Peru", "Peru nutome gaƒoƒo me"}, {"Philippines", "Philippine nutome gaƒoƒo me"}, {"Phoenix_Islands", "Phoenix Islands gaƒoƒo me"}, {"Pierre_Miquelon", "St. Pierre & Miquelon nutome gaƒoƒome"}, {"Pitcairn", "Pitcairn gaƒoƒo me"}, {"Ponape", "Ponape gaƒoƒo me"}, {"Pyongyang", "Pyongyang gaƒoƒo me"}, {"Qyzylorda", "Kizilɔrda gaƒoƒoɖoanyime"}, {"Reunion", "Reunion gaƒoƒo me"}, {"Rothera", "Rothera gaƒoƒo me"}, {"Sakhalin", "Sakhalin nutome gaƒoƒo me"}, {"Samara", "Samara gaƒoƒoɖoanyime"}, {"Samoa", "Samoa nutome gaƒoƒo me"}, {"Seychelles", "Seychelles gaƒoƒo me"}, {"Singapore", "Singapore nutome gaƒoƒo me"}, {"Solomon", "Solomon Islands gaƒoƒo me"}, {"South_Georgia", "South Georgia gaƒoƒo me"}, {"Suriname", "Suriname gaƒoƒome"}, {"Syowa", "Syowa gaƒoƒo me"}, {"Tahiti", "Tahiti gaƒoƒo me"}, {"Taipei", "Taipei nutome gaƒoƒo me"}, {"Tajikistan", "Tajikistan gaƒoƒo me"}, {"Tokelau", "Tokelau gaƒoƒo me"}, {"Tonga", "Tonga nutome gaƒoƒo me"}, {"Truk", "Chuuk gaƒoƒo me"}, {"Turkey", "Tɛrki gaƒoƒoɖoanyime"}, {"Turkmenistan", "Turkmenistan nutome gaƒoƒo me"}, {"Tuvalu", "Tuvalu gaƒoƒo me"}, {"Uruguay", "Uruguay nutome gaƒoƒo me"}, {"Uzbekistan", "Uzbekistan nutome gaƒoƒo me"}, {"Vanuatu", "Vanuatu nutome gaƒoƒo me"}, {"Venezuela", "Venezuela gaƒoƒo me"}, {"Vladivostok", "Vladivostok nutome gaƒoƒo me"}, {"Volgograd", "Volgograd nutome gaƒoƒo me"}, {"Vostok", "Vostok gaƒoƒo me"}, {"Wake", "Wake Island gaƒoƒo me"}, {"Wallis", "Wallis & Futuna gaƒoƒo me"}, {"Yakutsk", "Yakutsk nutome gaƒoƒo me"}, {"Yekaterinburg", "Yekaterinburg nutome gaƒoƒo me"}},
		{{"Acre", "Eker dzomeŋɔli gaƒoƒome"}, {"Alaska", "Alaska kele gaƒoƒo me"}, {"Almaty", "Almati dzomeŋɔli gaƒoƒome"}, {"Amazon", "Amazon dzomeŋɔli gaƒoƒo me"}, {"America_Central", "Titina America kele gaƒoƒo me"}, {"America_Eastern", "Eastern America kele gaƒoƒo me"}, {"America_Mountain", "Mountain kele gaƒoƒo me"}, {"America_Pacific", "Pacific kele gaƒoƒo me"}, {"Anadyr", "Anadir ŋkekeme gaƒoƒome"}, {"Apia", "Apia kele gaƒoƒo me"}, {"Aqtau", "Aktau dzomeŋɔli gaƒoƒome"}, {"Aqtobe", "Akttobe gaƒoƒome"}, {"Arabian", "Arabia dzomeŋɔli gaƒoƒo me"}, {"Argentina", "Argentina dzomeŋɔli gaƒoƒo me"}, {"Argentina_Western", "Ɣetoɖoƒe Argentina dzomeŋɔli gaƒoƒo me"}, {"Armenia", "Armenia dzomeŋɔli gaƒoƒo me"}, {"Atlantic", "Atlantic kele gaƒoƒome"}, {"Australia_Central", "Australian Central dzomeli gaƒoƒo me"}, {"Australia_CentralWestern", "Australian Central Western kele gaƒoƒo me"}, {"Australia_Eastern", "Australian Eastern kele gaƒoƒo me"}, {"Australia_Western", "Australian Western kele gaƒoƒo me"}, {"Azerbaijan", "Azerbaijan dzomeŋɔli gaƒoƒo me"}, {"Azores", "Azores dzomeŋɔli gaƒoƒo me"}, {"Bangladesh", "Bangladesh dzomeŋɔli gaƒoƒo me"}, {"Brasilia", "Brasilia dzomeŋɔli gaƒoƒo me"}, {"Cape_Verde", "Cape Verde dzomeŋɔli gaƒoƒo me"}, {"Chatham", "Chatham kele gaƒoƒo me"}, {"Chile", "Chile dzomeŋɔli gaƒoƒo me"}, {"China", "China kele gaƒoƒo me"}, {"Colombia", "Colombia dzomeŋɔli gaƒoƒo me"}, {"Cook", "Cook Islands dzomeŋɔli gaƒoƒo me"}, {"Cuba", "Cuba kele gaƒoƒome"}, {"Easter", "Easter Island dzomeŋɔli gaƒoƒo me"}, {"Europe_Central", "Central Europe dzomeŋɔli gaƒoƒo me"}, {"Europe_Eastern", "Ɣedzeƒe Europe ŋkekeme gaƒoƒome"}, {"Europe_Western", "Western Europe dzomeŋɔli gaƒoƒo me"}, {"Falkland", "Falkland Islands dzomeŋɔli gaƒoƒo me"}, {"Fiji", "Fiji dzomeŋɔli gaƒoƒo me"}, {"Georgia", "Georgia dzomeŋɔli gaƒoƒo me"}, {"Greenland_Eastern", "East Greenland dzomeŋɔli gaƒoƒo me"}, {"Greenland_Western", "West Greenland kele gaƒoƒo me"}, {"Hawaii_Aleutian", "Hawaii-Aleutia kele gaƒoƒo me"}, {"Hong_Kong", "Hong Kong dzomeŋɔli gaƒoƒo me"}, {"Hovd", "Hovd dzomeŋɔli gaƒoƒo me"}, {"Iran", "Iran kele gaƒoƒo me"}, {"Irkutsk", "Irkutsk dzomeŋɔli gaƒoƒo me"}, {"Israel", "Israel kele gaƒoƒo me"}, {"Japan", "Japan dzomeŋɔli gaƒoƒo me"}, {"Kamchatka", "Petropavlovsk-Kamtsatski ŋkekeme gaƒoƒome"}, {"Korea", "Korea dzomeŋɔli gaƒoƒo me"}, {"Krasnoyarsk", "Krasnoyarsk dzomeŋɔli gaƒoƒo me"}, {"Lord_Howe", "Lord Howe kele gaƒoƒo me"}, {"Macau", "Makau ŋkekeme gaƒoƒome"}, {"Magadan", "Magadan dzomeŋɔli gaƒoƒo me"}, {"Mauritius", "Mauritius dzomeŋɔli gaƒoƒo me"}, {"Mexico_Pacific", "Mexican Pacific kele gaƒoƒome"}, {"Mongolia", "Ulan Bator dzomeŋɔli gaƒoƒo me"}, {"Moscow", "Moscow dzomeŋɔli gaƒoƒo me"}, {"New_Caledonia", "New Caledonia dzomeŋɔli gaƒoƒo me"}, {"New_Zealand", "New Zealand kele gaƒoƒo me"}, {"Newfoundland", "Newfoundland kele gaƒoƒome"}, {"Norfolk", "Norfolk Island dzomeŋɔli gaƒoƒo me"}, {"Noronha", "Fernando de Noronha dzomeŋɔli gaƒoƒo me"}, {"Novosibirsk", "Novosibirsk dzomeŋɔli gaƒoƒo me"}, {"Omsk", "Omsk dzomeŋɔli gaƒoƒo me"}, {"Pakistan", "Pakistan dzomeŋɔli gaƒoƒo me"}, {"Paraguay", "Paraguay dzomeŋɔli gaƒoƒo me"}, {"Peru", "Peru dzomeŋɔli gaƒoƒome"}, {"Philippines", "Philippine dzomeŋɔli gaƒoƒo me"}, {"Pierre_Miquelon", "St. Pierre & Miquelon kele gaƒoƒome"}, {"Qyzylorda", "Kizilɔrda dzomeŋɔli gaƒoƒome"}, {"Sakhalin", "Sakhalin dzomeŋɔli gaƒoƒo me"}, {"Samara", "Samara ŋkekeme gaƒoƒome"}, {"Samoa", "Samoa kele gaƒoƒo me"}, {"Taipei", "Taipei kele gaƒoƒo me"}, {"Tonga", "Tonga dzomeŋɔli gaƒoƒo me"}, {"Turkey", "Tɛrki dzomeŋɔli gaƒoƒome"}, {"Turkmenistan", "Turkmenistan dzomeŋɔli gaƒoƒo me"}, {"Uruguay", "Uruguay dzomeŋɔli gaƒoƒo me"}, {"Uzbekistan", "Uzbekistan dzomeŋɔli gaƒoƒo me"}, {"Vanuatu", "Vanuatu dzomeŋɔli gaƒoƒo me"}, {"Vladivostok", "Vladivostok dzomeŋɔli gaƒoƒo me"}, {"Volgograd", "Vogograd dzomeŋɔli gaƒoƒo me"}, {"Yakutsk", "Yakutsk dzomeŋɔli gaƒoƒo me"}, {"Yekaterinburg", "Yekaterinburg dzomeŋɔli gaƒoƒo me"}},
	},
	{
		{{"Acre", "Χειμερινή ώρα Άκρε"}, {"Afghanistan", "Ώρα Αφγανιστάν"}, {"Africa_Central", "Ώρα Κεντρικής Αφρικής"}, {"Africa_Eastern", "Ώρα Ανατολικής Αφρικής"}, {"Africa_Southern", "Χειμερινή ώρα Νότιας Αφρικής"}, {"Africa_Western", "Ώρα Δυτικής Αφρικής"}, {"Alaska", "Χειμερινή ώρα Αλάσκας"}, {"Amazon", "Χειμερινή ώρα Αμαζονίου"}, {"America_Central", "Κεντρική χειμερινή ώρα Βόρειας Αμερικής"}, {"America_Eastern", "Ανατολική χειμερινή ώρα Βόρειας Αμερικής"}, {"America_Mountain", "Ορεινή χειμερινή ώρα Βόρειας Αμερικής"}, {"America_Pacific", "Χειμερινή ώρα Ειρηνικού"}, {"Anadyr", "Χειμερινή ώρα Αναντίρ"}, {"Apia", "Χειμερινή ώρα Απία"}, {"Arabian", "Αραβική χειμερινή ώρα"}, {"Argentina", "Χειμερινή ώρα Αργεντινής"}, {"Argentina_Western", "Χειμερινή ώρα Δυτικής Αργεντινής"}, {"Armenia", "Χειμερινή ώρα Αρμενίας"}, {"Atlantic", "Χειμερινή ώρα Ατλαντικού"}, {"Australia_Central", "Χειμερινή ώρα Κεντρικής Αυστραλίας"}, {"Australia_CentralWestern", "Χειμερινή ώρα Κεντροδυτικής Αυστραλίας"}, {"Australia_Eastern", "Χειμερινή ώρα Ανατολικής Αυστραλίας"}, {"Australia_Western", "Χειμερινή ώρα Δυτικής Αυστραλίας"}, {"Azerbaijan", "Χειμερινή ώρα Αζερμπαϊτζάν"}, {"Azores", "Χειμερινή ώρα Αζορών"}, {"Bangladesh", "Χειμερινή ώρα Μπανγκλαντές"}, {"Bhutan", "Ώρα Μπουτάν"}, {"Bolivia", "Ώρα Βολιβίας"}, {"Brasilia", "Χειμερινή ώρα Μπραζίλιας"}, {"Brunei", "Ώρα Μπρουνέι Νταρουσαλάμ"}, {"Cape_Verde", "Χειμερινή ώρα Πράσινου Ακρωτηρίου"}, {"Chamorro", "Ώρα Τσαμόρο"}, {"Chatham", "Χειμερινή ώρα Τσάταμ"}, {"Chile", "Χειμερινή ώρα Χιλής"}, {"China", "Χειμερινή ώρα Κίνας"}, {"Christmas", "Ώρα Νήσου Χριστουγέννων"}, {"Cocos", "Ώρα Νήσων Κόκος"}, {"Colombia", "Χειμερινή ώρα Κολομβίας"}, {"Cook", "Χειμερινή ώρα Νήσων Κουκ"}, {"Cuba", "Χειμερινή ώρα Κούβας"}, {"Davis", "Ώρα Ντέιβις"}, {"DumontDUrville", "Ώρα Ντιμόν ντ’ Ουρβίλ"}, {"East_Timor", "Ώρα Ανατολικού Τιμόρ"}, {"Easter", "Χειμερινή ώρα Νήσου Πάσχα"}, {"Ecuador", "Ώρα Ισημερινού"}, {"Europe_Central", "Χειμερινή ώρα Κεντρικής Ευρώπης"}, {"Europe_Eastern", "Χειμερινή ώρα Ανατολικής Ευρώπης"}, {"Europe_Further_Eastern", "Ώρα περαιτέρω Ανατολικής Ευρώπης"}, {"Europe_Western", "Χειμερινή ώρα Δυτικής Ευρώπης"}, {"Falkland", "Χειμερινή ώρα Νήσων Φόκλαντ"}, {"Fiji", "Χειμερινή ώρα Φίτζι"}, {"French_Guiana", "Ώρα Γαλλικής Γουιάνας"}, {"French_Southern", "Ώρα Γαλλικού Νότου και Ανταρκτικής"}, {"GMT", "Μέση ώρα Γκρίνουιτς"}, {"Galapagos", "Ώρα Γκαλάπαγκος"}, {"Gambier", "Ώρα Γκάμπιερ"}, {"Georgia", "Χειμερινή ώρα Γεωργίας"}, {"Gilbert_Islands", "Ώρα Νήσων Γκίλμπερτ"}, {"Greenland_Eastern", "Χειμερινή ώρα Ανατολικής Γροιλανδίας"}, {"Greenland_Western", "Χειμερινή ώρα Δυτικής Γροιλανδίας"}, {"Guam", "Ώρα Γκουάμ"}, {"Gulf", "Ώρα Κόλπου"}, {"Guyana", "Ώρα Γουιάνας"}, {"Hawaii", "Χειμερινή ώρα Χαβάης-Αλεούτιων Νήσων"}, {"Hawaii_Aleutian", "Χειμερινή ώρα Χαβάης-Αλεούτιων Νήσων"}, {"Hong_Kong", "Χειμερινή ώρα Χονγκ Κονγκ"}, {"Hovd", "Χειμερινή ώρα Χοβντ"}, {"India", "Ώρα Ινδίας"}, {"Indian_Ocean", "Ώρα Ινδικού Ωκεανού"}, {"Indochina", "Ώρα Ινδοκίνας"}, {"Indonesia_Central", "Ώρα Κεντρικής Ινδονησίας"}, {"Indonesia_Eastern", "Ώρα Ανατολικής Ινδονησίας"}, {"Indonesia_Western", "Ώρα Δυτικής Ινδονησίας"}, {"Iran", "Χειμερινή ώρα Ιράν"}, {"Irkutsk", "Χειμερινή ώρα Ιρκούτσκ"}, {"Israel", "Χειμερινή ώρα Ισραήλ"}, {"Japan", "Χειμερινή ώρα Ιαπωνίας"}, {"Kamchatka", "Χειμερινή ώρα Πετροπαβλόβσκ-Καμτσάτσκι"}, {"Kazakhstan", "Ώρα Καζακστάν"}, {"Kazakhstan_Eastern", "Ώρα Ανατολικού Καζακστάν"}, {"Kazakhstan_Western", "Ώρα Δυτικού Καζακστάν"}, {"Korea", "Χειμερινή ώρα Κορέας"}, {"Kosrae", "Ώρα Κόσραϊ"}, {"Krasnoyarsk", "Χειμερινή ώρα Κρασνογιάρσκ"}, {"Kyrgystan", "Ώρα Κιργιστάν"}, {"Line_Islands", "Ώρα Νήσων Λάιν"}, {"Lord_Howe", "Χειμερινή ώρα Λορντ Χάου"}, {"Macau", "Χειμερινή ώρα Μακάο"}, {"Magadan", "Χειμερινή ώρα Μαγκαντάν"}, {"Malaysia", "Ώρα Μαλαισίας"}, {"Maldives", "Ώρα Μαλδίβων"}, {"Marquesas", "Ώρα Μαρκέζας"}, {"Marshall_Islands", "Ώρα Νήσων Μάρσαλ"}, {"Mauritius", "Χειμερινή ώρα Μαυρίκιου"}, {"Mawson", "Ώρα Μόσον"}, {"Mexico_Pacific", "Χειμερινή ώρα Ειρηνικού Μεξικού"}, {"Mongolia", "Χειμερινή ώρα Ουλάν Μπατόρ"}, {"Moscow", "Χειμερινή ώρα Μόσχας"}, {"Myanmar", "Ώρα Μιανμάρ"}, {"Nauru", "Ώρα Ναούρου"}, {"Nepal", "Ώρα Νεπάλ"}, {"New_Caledonia", "Χειμερινή ώρα Νέας Καληδονίας"}, {"New_Zealand", "Χειμερινή ώρα Νέας Ζηλανδίας"}, {"Newfoundland", "Χειμερινή ώρα Νέας Γης"}, {"Niue", "Ώρα Νιούε"}, {"Norfolk", "Χειμερινή ώρα Νήσου Νόρφολκ"}, {"Noronha", "Χειμερινή ώρα Φερνάρντο ντε Νορόνια"}, {"North_Mariana", "Ώρα Νησιών Βόρειες Μαριάνες"}, {"Novosibirsk", "Χειμερινή ώρα Νοβοσιμπίρσκ"}, {"Omsk", "Χειμερινή ώρα Ομσκ"}, {"Pakistan", "Χειμερινή ώρα Πακιστάν"}, {"Palau", "Ώρα Παλάου"}, {"Papua_New_Guinea", "Ώρα Παπούας Νέας Γουινέας"}, {"Paraguay", "Χειμερινή ώρα Παραγουάης"}, {"Peru", "Χειμερινή ώρα Περού"}, {"Philippines", "Χειμερινή ώρα Φιλιππινών"}, {"Phoenix_Islands", "Ώρα Νήσων Φοίνιξ"}, {"Pierre_Miquelon", "Χειμερινή ώρα Σεν Πιερ και Μικελόν"}, {"Pitcairn", "Ώρα Πίτκερν"}, {"Ponape", "Ώρα Πονάπε"}, {"Pyongyang", "Ώρα Πιονγιάνγκ"}, {"Reunion", "Ώρα Ρεϊνιόν"}, {"Rothera", "Ώρα Ρόθερα"}, {"Sakhalin", "Χειμερινή ώρα Σαχαλίνης"}, {"Samara", "Χειμερινή ώρα Σάμαρας"}, {"Samoa", "Χειμερινή ώρα Σαμόα"}, {"Seychelles", "Ώρα Σεϋχελλών"}, {"Singapore", "Ώρα Σιγκαπούρης"}, {"Solomon", "Ώρα Νήσων Σολομώντος"}, {"South_Georgia", "Ώρα Νότιας Γεωργίας"}, {"Suriname", "Ώρα Σουρινάμ"}, {"Syowa", "Ώρα Σίοβα"}, {"Tahiti", "Ώρα Ταϊτής"}, {"Taipei", "Χειμερινή ώρα Ταϊπέι"}, {"Tajikistan", "Ώρα Τατζικιστάν"}, {"Tokelau", "Ώρα Τοκελάου"}, {"Tonga", "Χειμερινή ώρα Τόνγκα"}, {"Truk", "Ώρα Τσουκ"}, {"Turkey", "Χειμερινή ώρα Τουρκίας"}, {"Turkmenistan", "Χειμερινή ώρα Τουρκμενιστάν"}, {"Tuvalu", "Ώρα Τουβαλού"}, {"Uruguay", "Χειμερινή ώρα Ουρουγουάης"}, {"Uzbekistan", "Χειμερινή ώρα Ουζμπεκιστάν"}, {"Vanuatu", "Χειμερινή ώρα Βανουάτου"}, {"Venezuela", "Ώρα Βενεζουέλας"}, {"Vladivostok", "Χειμερινή ώρα Βλαδιβοστόκ"}, {"Volgograd", "Χειμερινή ώρα Βόλγκογκραντ"}, {"Vostok", "Ώρα Βόστοκ"}, {"Wake", "Ώρα Νήσου Γουέικ"}, {"Wallis", "Ώρα Ουάλις και Φουτούνα"}, {"Yakutsk", "Χειμερινή ώρα Γιακούτσκ"}, {"Yekaterinburg", "Χειμερινή ώρα Αικατερινούπολης"}, {"Yukon", "Ώρα Γιούκον"}},
		{{"Acre", "Θερινή ώρα Άκρε"}, {"Alaska", "Θερινή ώρα Αλάσκας"}, {"Amazon", "Θερινή ώρα Αμαζονίου"}, {"America_Central", "Κεντρική θερινή ώρα Βόρειας Αμερικής"}, {"America_Eastern", "Ανατολική θερινή ώρα Βόρειας Αμερικής"}, {"America_Mountain", "Ορεινή θερινή ώρα Βόρειας Αμερικής"}, {"America_Pacific", "Θερινή ώρα Ειρηνικού"}, {"Anadyr", "Θερινή ώρα Αναντίρ"}, {"Apia", "Θερινή ώρα Απία"}, {"Arabian", "Αραβική θερινή ώρα"}, {"Argentina", "Θερινή ώρα Αργεντινής"}, {"Argentina_Western", "Θερινή ώρα Δυτικής Αργεντινής"}, {"Armenia", "Θερινή ώρα Αρμενίας"}, {"Atlantic", "Θερινή ώρα Ατλαντικού"}, {"Australia_Central", "Θερινή ώρα Κεντρικής Αυστραλίας"}, {"Australia_CentralWestern", "Θερινή ώρα Κεντροδυτικής Αυστραλίας"}, {"Australia_Eastern", "Θερινή ώρα Ανατολικής Αυστραλίας"}, {"Australia_Western", "Θερινή ώρα Δυτικής Αυστραλίας"}, {"Azerbaijan", "Θερινή ώρα Αζερμπαϊτζάν"}, {"Azores", "Θερινή ώρα Αζορών"}, {"Bangladesh", "Θερινή ώρα Μπανγκλαντές"}, {"Brasilia", "Θερινή ώρα Μπραζίλιας"}, {"Cape_Verde", "Θερινή ώρα Πράσινου Ακρωτηρίου"}, {"Chatham", "Θερινή ώρα Τσάταμ"}, {"Chile", "Θερινή ώρα Χιλής"}, {"China", "Θερινή ώρα Κίνας"}, {"Colombia", "Θερινή ώρα Κολομβίας"}, {"Cook", "Θερινή ώρα Νήσων Κουκ"}, {"Cuba", "Θερινή ώρα Κούβας"}, {"Easter", "Θερινή ώρα Νήσου Πάσχα"}, {"Europe_Central", "Θερινή ώρα Κεντρικής Ευρώπης"}, {"Europe_Eastern", "Θερινή ώρα Ανατολικής Ευρώπης"}, {"Europe_Western", "Θερινή ώρα Δυτικής Ευρώπης"}, {"Falkland", "Θερινή ώρα Νήσων Φόκλαντ"}, {"Fiji", "Θερινή ώρα Φίτζι"}, {"Georgia", "Θερινή ώρα Γεωργίας"}, {"Greenland_Eastern", "Θερινή ώρα Ανατολικής Γροιλανδίας"}, {"Greenland_Western", "Θερινή ώρα Δυτικής Γροιλανδίας"}, {"Hawaii_Aleutian", "Θερινή ώρα Χαβάης-Αλεούτιων Νήσων"}, {"Hong_Kong", "Θερινή ώρα Χονγκ Κονγκ"}, {"Hovd", "Θερινή ώρα Χοβντ"}, {"Iran", "Θερινή ώρα Ιράν"}, {"Irkutsk", "Θερινή ώρα Ιρκούτσκ"}, {"Israel", "Θερινή ώρα Ισραήλ"}, {"Japan", "Θερινή ώρα Ιαπωνίας"}, {"Kamchatka", "Θερινή ώρα Πετροπαβλόβσκ-Καμτσάτσκι"}, {"Korea", "Θερινή ώρα Κορέας"}, {"Krasnoyarsk", "Θερινή ώρα Κρασνογιάρσκ"}, {"Lord_Howe", "Θερινή ώρα Λορντ Χάου"}, {"Macau", "Θερινή ώρα Μακάο"}, {"Magadan", "Θερινή ώρα Μαγκαντάν"}, {"Mauritius", "Θερινή ώρα Μαυρίκιου"}, {"Mexico_Pacific", "Θερινή ώρα Ειρηνικού Μεξικού"}, {"Mongolia", "Θερινή ώρα Ουλάν Μπατόρ"}, {"Moscow", "Θερινή ώρα Μόσχας"}, {"New_Caledonia", "Θερινή ώρα Νέας Καληδονίας"}, {"New_Zealand", "Θερινή ώρα Νέας Ζηλανδίας"}, {"Newfoundland", "Θερινή ώρα Νέας Γης"}, {"Norfolk", "Θερινή ώρα Νήσου Νόρφολκ"}, {"Noronha", "Θερινή ώρα Φερνάρντο ντε Νορόνια"}, {"Novosibirsk", "Θερινή ώρα Νοβοσιμπίρσκ"}, {"Omsk", "Θερινή ώρα Ομσκ"}, {"Pakistan", "Θερινή ώρα Πακιστάν"}, {"Paraguay", "Θερινή ώρα Παραγουάης"}, {"Peru", "Θερινή ώρα Περού"}, {"Philippines", "Θερινή ώρα Φιλιππινών"}, {"Pierre_Miquelon", "Θερινή ώρα Σεν Πιερ και Μικελόν"}, {"Sakhalin", "Θερινή ώρα Σαχαλίνης"}, {"Samara", "Θερινή ώρα Σαμάρας"}, {"Samoa", "Θερινή ώρα Σαμόα"}, {"Taipei", "Θερινή ώρα Ταϊπέι"}, {"Tonga", "Θερινή ώρα Τόνγκα"}, {"Turkey", "Θερινή ώρα Τουρκίας"}, {"Turkmenistan", "Θερινή ώρα Τουρκμενιστάν"}, {"Uruguay", "Θερινή ώρα Ουρουγουάης"}, {"Uzbekistan", "Θερινή ώρα Ουζμπεκιστάν"}, {"Vanuatu", "Θερινή ώρα Βανουάτου"}, {"Vladivostok", "Θερινή ώρα Βλαδιβοστόκ"}, {"Volgograd", "Θερινή ώρα Βόλγκογκραντ"}, {"Yakutsk", "Θερινή ώρα Γιακούτσκ"}, {"Yekaterinburg", "Θερινή ώρα Αικατερινούπολης"}},
	},
	{
		{{"Acre", "Acre Standard Time"}, {"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Almaty", "Almaty Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Pacific Standard Time"}, {"Anadyr", "Anadyr Standard Time"}, {"Apia", "Samoa Standard Time"}, {"Aqtau", "Aqtau Standard Time"}, {"Aqtobe", "Aqtobe Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Casey", "Casey Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont d’Urville Time"}, {"East_Timor", "Timor-Leste Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern & Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland", "Greenland Standard Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Guam", "Guam Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kamchatka", "Kamchatka Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Lanka", "Lanka Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Macau", "Macao Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"North_Mariana", "Northern Mariana Islands Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Pohnpei Time"}, {"Pyongyang", "North Korea Time"}, {"Qyzylorda", "Kyzylorda Standard Time"}, {"Reunion", "Réunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samara", "Samara Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkey", "Türkiye Standard Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Acre Summer Time"}, {"Alaska", "Alaska Daylight Time"}, {"Almaty", "Almaty Summer Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Pacific Daylight Time"}, {"Anadyr", "Anadyr Summer Time"}, {"Apia", "Samoa Daylight Time"}, {"Aqtau", "Aqtau Summer Time"}, {"Aqtobe", "Aqtobe Summer Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland", "Greenland Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Kamchatka", "Kamchatka Summer Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Macau", "Macao Summer Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Daylight Time"}, {"Qyzylorda", "Kyzylorda Summer Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samara", "Samara Summer Time"}, {"Samoa", "American Samoa Daylight Time"}, {"Taipei", "Taiwan Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkey", "Türkiye Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Acre", "Acre Standard Time"}, {"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Almaty", "Almaty Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Pacific Standard Time"}, {"Anadyr", "Anadyr Standard Time"}, {"Apia", "Samoa Standard Time"}, {"Aqtau", "Aqtau Standard Time"}, {"Aqtobe", "Aqtobe Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Casey", "Casey Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont d’Urville Time"}, {"East_Timor", "Timor-Leste Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern & Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland", "Greenland Standard Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Guam", "Guam Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kamchatka", "Kamchatka Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Lanka", "Lanka Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Macau", "Macao Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"North_Mariana", "Northern Mariana Islands Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "St Pierre & Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Pohnpei Time"}, {"Pyongyang", "North Korea Time"}, {"Qyzylorda", "Kyzylorda Standard Time"}, {"Reunion", "Réunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samara", "Samara Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkey", "Türkiye Standard Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Acre Summer Time"}, {"Alaska", "Alaska Daylight Time"}, {"Almaty", "Almaty Summer Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Pacific Daylight Time"}, {"Anadyr", "Anadyr Summer Time"}, {"Apia", "Samoa Daylight Time"}, {"Aqtau", "Aqtau Summer Time"}, {"Aqtobe", "Aqtobe Summer Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland", "Greenland Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Kamchatka", "Kamchatka Summer Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Macau", "Macao Summer Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St Pierre & Miquelon Daylight Time"}, {"Qyzylorda", "Kyzylorda Summer Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samara", "Samara Summer Time"}, {"Samoa", "American Samoa Daylight Time"}, {"Taipei", "Taiwan Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkey", "Türkiye Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Acre", "Acre Standard Time"}, {"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Almaty", "Almaty Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Pacific Standard Time"}, {"Anadyr", "Anadyr Standard Time"}, {"Apia", "Samoa Standard Time"}, {"Aqtau", "Aqtau Standard Time"}, {"Aqtobe", "Aqtobe Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Casey", "Casey Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont d’Urville Time"}, {"East_Timor", "Timor-Leste Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern and Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland", "Greenland Standard Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Guam", "Guam Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kamchatka", "Kamchatka Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Lanka", "Lanka Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Macau", "Macao Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"North_Mariana", "Northern Mariana Islands Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "St Pierre & Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Pohnpei Time"}, {"Pyongyang", "North Korea Time"}, {"Qyzylorda", "Kyzylorda Standard Time"}, {"Reunion", "Réunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samara", "Samara Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkey", "Türkiye Standard Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Acre Summer Time"}, {"Alaska", "Alaska Daylight Time"}, {"Almaty", "Almaty Summer Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Pacific Daylight Time"}, {"Anadyr", "Anadyr Summer Time"}, {"Apia", "Samoa Daylight Time"}, {"Aqtau", "Aqtau Summer Time"}, {"Aqtobe", "Aqtobe Summer Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Summer Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland", "Greenland Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Summer Time"}, {"Kamchatka", "Kamchatka Summer Time"}, {"Korea", "Korean Summer Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Macau", "Macao Summer Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St Pierre & Miquelon Daylight Time"}, {"Qyzylorda", "Kyzylorda Summer Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samara", "Samara Summer Time"}, {"Samoa", "Samoa Summer Time"}, {"Taipei", "Taipei Summer Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkey", "Türkiye Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Acre", "Acre Standard Time"}, {"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Almaty", "Almaty Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Pacific Standard Time"}, {"Anadyr", "Anadyr Standard Time"}, {"Apia", "Samoa Standard Time"}, {"Aqtau", "Aqtau Standard Time"}, {"Aqtobe", "Aqtobe Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Casey", "Casey Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont d’Urville Time"}, {"East_Timor", "Timor-Leste Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern and Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland", "Greenland Standard Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Guam", "Guam Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kamchatka", "Kamchatka Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Lanka", "Lanka Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Macau", "Macao Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"North_Mariana", "Northern Mariana Islands Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "Saint-Pierre-et-Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Pohnpei Time"}, {"Pyongyang", "North Korea Time"}, {"Qyzylorda", "Kyzylorda Standard Time"}, {"Reunion", "Réunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samara", "Samara Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkey", "Türkiye Standard Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis and Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Acre Summer Time"}, {"Alaska", "Alaska Daylight Time"}, {"Almaty", "Almaty Summer Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Pacific Daylight Time"}, {"Anadyr", "Anadyr Summer Time"}, {"Apia", "Samoa Daylight Time"}, {"Aqtau", "Aqtau Summer Time"}, {"Aqtobe", "Aqtobe Summer Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland", "Greenland Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Kamchatka", "Kamchatka Summer Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Macau", "Macao Summer Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "Saint-Pierre-et-Miquelon Daylight Time"}, {"Qyzylorda", "Kyzylorda Summer Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samara", "Samara Summer Time"}, {"Samoa", "American Samoa Daylight Time"}, {"Taipei", "Taiwan Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkey", "Türkiye Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Alaska", "𐐊𐑊𐐰𐑅𐐿𐐲 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"America_Central", "𐐝𐐯𐑌𐐻𐑉𐐲𐑊 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"America_Eastern", "𐐀𐑅𐐻𐐲𐑉𐑌 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"America_Mountain", "𐐣𐐵𐑌𐐻𐐲𐑌 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"America_Pacific", "𐐑𐐲𐑅𐐮𐑁𐐮𐐿 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"Atlantic", "𐐈𐐻𐑊𐐰𐑌𐐻𐐮𐐿 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"Hong_Kong", "𐐐𐐱𐑍 𐐗𐐱𐑍 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}, {"Newfoundland", "𐐤𐐭𐑁𐐲𐑌𐐼𐑊𐐲𐑌𐐼 𐐝𐐻𐐰𐑌𐐼𐐲𐑉𐐼 𐐓𐐴𐑋"}},
		{{"Alaska", "𐐊𐑊𐐰𐑅𐐿𐐲 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"America_Central", "𐐝𐐯𐑌𐐻𐑉𐐲𐑊 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"America_Eastern", "𐐀𐑅𐐻𐐲𐑉𐑌 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"America_Mountain", "𐐣𐐵𐑌𐐻𐐲𐑌 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"America_Pacific", "𐐑𐐲𐑅𐐮𐑁𐐮𐐿 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"Atlantic", "𐐈𐐻𐑊𐐰𐑌𐐻𐐮𐐿 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"Hong_Kong", "𐐐𐐱𐑍 𐐗𐐱𐑍 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}, {"Newfoundland", "𐐤𐐭𐑁𐐲𐑌𐐼𐑊𐐲𐑌𐐼 𐐔𐐩𐑊𐐴𐐻 𐐓𐐴𐑋"}},
	},
	{
		{{"Europe_Central", "𐑕𐑧𐑯𐑑𐑮𐑩𐑤 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑑𐑨𐑯𐑛𐑼𐑛 𐑑𐑲𐑥"}, {"Europe_Eastern", "𐑰𐑕𐑑𐑼𐑯 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑑𐑨𐑯𐑛𐑼𐑛 𐑑𐑲𐑥"}, {"Europe_Further_Eastern", "𐑓𐑻𐑞𐑼-𐑰𐑕𐑑𐑼𐑯 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑑𐑲𐑥"}, {"Europe_Western", "𐑢𐑧𐑕𐑑𐑼𐑯 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑑𐑨𐑯𐑛𐑼𐑛 𐑑𐑲𐑥"}, {"GMT", "𐑜𐑮𐑧𐑯𐑦𐑗 𐑥𐑰𐑯 𐑑𐑲𐑥"}},
		{{"Europe_Central", "𐑕𐑧𐑯𐑑𐑮𐑩𐑤 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑳𐑥𐑼 𐑑𐑲𐑥"}, {"Europe_Eastern", "𐑰𐑕𐑑𐑼𐑯 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑳𐑥𐑼 𐑑𐑲𐑥"}, {"Europe_Western", "𐑢𐑧𐑕𐑑𐑼𐑯 𐑘𐑫𐑼𐑩𐑐𐑾𐑯 𐑕𐑳𐑥𐑼 𐑑𐑲𐑥"}},
	},
	{
		{{"Afghanistan", "afgana tempo"}, {"Africa_Central", "centrafrika tempo"}, {"Africa_Eastern", "orientafrika tempo"}, {"Africa_Southern", "sudafrika tempo"}, {"Africa_Western", "okcidentafrika tempo"}, {"Alaska", "alaska norma tempo"}, {"Amazon", "amazona norma tempo"}, {"America_Central", "centra nordamerika norma tempo"}, {"America_Eastern", "orienta nordamerika norma tempo"}, {"America_Mountain", "montara nordamerika norma tempo"}, {"America_Pacific", "pacifika nordamerika norma tempo"}, {"Apia", "samoa norma tempo"}, {"Arabian", "araba norma tempo"}, {"Argentina", "argentina norma tempo"}, {"Argentina_Western", "okcident-argentina norma tempo"}, {"Armenia", "armena norma tempo"}, {"Atlantic", "atlantika nordamerika norma tempo"}, {"Australia_Central", "centra aŭstralia norma tempo"}, {"Australia_CentralWestern", "centrokcidenta aŭstralia norma tempo"}, {"Australia_Eastern", "orienta aŭstralia norma tempo"}, {"Australia_Western", "okcidenta aŭstralia norma tempo"}, {"Azerbaijan", "azerbajĝana norma tempo"}, {"Azores", "Acoroj (norma tempo)"}, {"Bangladesh", "bangladeŝa norma tempo"}, {"Bhutan", "butana tempo"}, {"Bolivia", "bolivia tempo"}, {"Brasilia", "brazilja norma tempo"}, {"Brunei", "bruneja tempo"}, {"Cape_Verde", "kaboverda norma tempo"}, {"Chamorro", "ĉamora tempo"}, {"Chatham", "ĉathama norma tempo"}, {"Chile", "ĉilia norma tempo"}, {"China", "ĉina norma tempo"}, {"Christmas", "kristnaskinsula tempo"}, {"Cocos", "kokosinsula tempo"}, {"Colombia", "kolombia norma tempo"}, {"Cook", "kukinsula norma tempo"}, {"Cuba", "Kubo (norma tempo)"}, {"Davis", "tempo: Davis"}, {"DumontDUrville", "tempo: Dumont d’Urville"}, {"East_Timor", "orient-timora tempo"}, {"Easter", "paskinsula norma tempo"}, {"Ecuador", "ekvadora tempo"}, {"Europe_Central", "centreŭropa norma tempo"}, {"Europe_Eastern", "orienteŭropa norma tempo"}, {"Europe_Further_Eastern", "ekstrem-orienteŭropa tempo"}, {"Europe_Western", "okcidenteŭropa norma tempo"}, {"Falkland", "falklanda norma tempo"}, {"Fiji", "fiĝia norma tempo"}, {"French_Guiana", "tempo: Franca Gujano"}, {"French_Southern", "tempo: Francaj Sudaj Teritorioj"}, {"GMT", "universala tempo kunordigita"}, {"Galapagos", "galapaga tempo"}, {"Gambier", "tempo: Gambier"}, {"Georgia", "kartvela norma tempo"}, {"Gilbert_Islands", "gilbertinsula tempo"}, {"Greenland_Eastern", "orienta gronlanda norma tempo"}, {"Greenland_Western", "okcidenta gronlanda norma tempo"}, {"Gulf", "arabgolfa norma tempo"}, {"Guyana", "gujana tempo"}, {"Hawaii", "Havajo-Aleutoj (norma tempo)"}, {"Hawaii_Aleutian", "Havajo-Aleutoj (norma tempo)"}, {"Hong_Kong", "honkonga norma tempo"}, {"Hovd", "ĥovda norma tempo"}, {"India", "hinda norma tempo"}, {"Indian_Ocean", "hindoceana tempo"}, {"Indochina", "hindoĉina tempo"}, {"Indonesia_Central", "centr-indonezia tempo"}, {"Indonesia_Eastern", "orient-indonezia tempo"}, {"Indonesia_Western", "okcident-indonezia tempo"}, {"Iran", "irana norma tempo"}, {"Irkutsk", "irkutska norma tempo"}, {"Israel", "israela norma tempo"}, {"Japan", "japana norma tempo"}, {"Kazakhstan", "kazaĥa tempo"}, {"Kazakhstan_Eastern", "orient-kazaĥa tempo"}, {"Kazakhstan_Western", "okcident-kazaĥa tempo"}, {"Korea", "korea norma tempo"}, {"Kosrae", "tempo: Kosrae"}, {"Krasnoyarsk", "krasnojarska norma tempo"}, {"Kyrgystan", "kirgiza tempo"}, {"Line_Islands", "tempo: Liniaj Insuloj"}, {"Lord_Howe", "Lord Howe (norma tempo)"}, {"Magadan", "magadana norma tempo"}, {"Malaysia", "malajzia tempo"}, {"Maldives", "maldiva tempo"}, {"Marquesas", "markizinsula tempo"}, {"Marshall_Islands", "marŝalinsula tempo"}, {"Mauritius", "maŭricia norma tempo"}, {"Mawson", "tempo: Mawson"}, {"Mexico_Pacific", "pacifika meksika norma tempo"}, {"Mongolia", "ulanbatora norma tempo"}, {"Moscow", "moskva norma tempo"}, {"Myanmar", "birma tempo"}, {"Nauru", "naura tempo"}, {"Nepal", "nepala tempo"}, {"New_Caledonia", "novkaledonia norma tempo"}, {"New_Zealand", "novzelanda norma tempo"}, {"Newfoundland", "Novlando (norma tempo)"}, {"Niue", "niua tempo"}, {"Norfolk", "norfolkinsula norma tempo"}, {"Noronha", "Fernando de Noronha (norma tempo)"}, {"Novosibirsk", "novosibirska norma tempo"}, {"Omsk", "omska norma tempo"}, {"Pakistan", "pakistana norma tempo"}, {"Palau", "palaŭa tempo"}, {"Papua_New_Guinea", "tempo: Papuo-Nov-Gvineo"}, {"Paraguay", "paragvaja norma tempo"}, {"Peru", "perua norma tempo"}, {"Philippines", "filipina norma tempo"}, {"Phoenix_Islands", "feniksinsula tempo"}, {"Pierre_Miquelon", "Sankta Piero kaj Mikelono (norma tempo)"}, {"Pitcairn", "pitkarninsula tempo"}, {"Ponape", "tempo: Ponape"}, {"Pyongyang", "nord-korea tempo"}, {"Reunion", "tempo: Reunio"}, {"Rothera", "tempo: Rothera"}, {"Sakhalin", "saĥalena norma tempo"}, {"Samoa", "uson-samoa norma tempo"}, {"Seychelles", "sejŝela tempo"}, {"Singapore", "singapura norma tempo"}, {"Solomon", "tempo: Salomonoj"}, {"South_Georgia", "tempo: Sud-Georgio"}, {"Suriname", "surinama tempo"}, {"Syowa", "tempo: Showa"}, {"Tahiti", "tahitia tempo"}, {"Taipei", "tajvana norma tempo"}, {"Tajikistan", "taĝika tempo"}, {"Tokelau", "tokelaa tempo"}, {"Tonga", "tonga norma tempo"}, {"Truk", "tempo: Ĉuuk"}, {"Turkmenistan", "turkmena norma tempo"}, {"Tuvalu", "tuvala tempo"}, {"Uruguay", "urugvaja norma tempo"}, {"Uzbekistan", "uzbeka norma tempo"}, {"Vanuatu", "vanuatua norma tempo"}, {"Venezuela", "venezuela tempo"}, {"Vladivostok", "vladivostoka norma tempo"}, {"Volgograd", "volgograda norma tempo"}, {"Vostok", "tempo: Vostok"}, {"Wake", "vejkinsula tempo"}, {"Wallis", "tempo: Valiso kaj Futuno"}, {"Yakutsk", "jakutska norma tempo"}, {"Yekaterinburg", "jekaterinburga norma tempo"}, {"Yukon", "jukonia tempo"}},
		{{"Alaska", "alaska somera tempo"}, {"Amazon", "amazona somera tempo"}, {"America_Central", "centra nordamerika somera tempo"}, {"America_Eastern", "orienta nordamerika somera tempo"}, {"America_Mountain", "montara nordamerika somera tempo"}, {"America_Pacific", "pacifika nordamerika somera tempo"}, {"Apia", "samoa somera tempo"}, {"Arabian", "araba somera tempo"}, {"Argentina", "argentina somera tempo"}, {"Argentina_Western", "okcident-argentina somera tempo"}, {"Armenia", "armena somera tempo"}, {"Atlantic", "atlantika nordamerika somera tempo"}, {"Australia_Central", "centra aŭstralia somera tempo"}, {"Australia_CentralWestern", "centrokcidenta aŭstralia somera tempo"}, {"Australia_Eastern", "orienta aŭstralia somera tempo"}, {"Australia_Western", "okcidenta aŭstralia somera tempo"}, {"Azerbaijan", "azerbajĝana somera tempo"}, {"Azores", "Acoroj (somera tempo)"}, {"Bangladesh", "bangladeŝa somera tempo"}, {"Brasilia", "brazilja somera tempo"}, {"Cape_Verde", "kaboverda somera tempo"}, {"Chatham", "ĉathama somera tempo"}, {"Chile", "ĉilia somera tempo"}, {"China", "ĉina somera tempo"}, {"Colombia", "kolombia somera tempo"}, {"Cook", "kukinsula somera tempo"}, {"Cuba", "Kubo (somera tempo)"}, {"Easter", "paskinsula somera tempo"}, {"Europe_Central", "centreŭropa somera tempo"}, {"Europe_Eastern", "orienteŭropa somera tempo"}, {"Europe_Western", "okcidenteŭropa somera tempo"}, {"Falkland", "falklanda somera tempo"}, {"Fiji", "fiĝia somera tempo"}, {"Georgia", "kartvela somera tempo"}, {"Greenland_Eastern", "orienta gronlanda somera tempo"}, {"Greenland_Western", "okcidenta gronlanda somera tempo"}, {"Hawaii_Aleutian", "Havajo-Aleutoj (somera tempo)"}, {"Hong_Kong", "honkonga somera tempo"}, {"Hovd", "ĥovda somera tempo"}, {"Iran", "irana somera tempo"}, {"Irkutsk", "irkutska somera tempo"}, {"Israel", "israela somera tempo"}, {"Japan", "japana somera tempo"}, {"Korea", "korea somera tempo"}, {"Krasnoyarsk", "krasnojarska somera tempo"}, {"Lord_Howe", "Lord Howe (somera tempo)"}, {"Magadan", "magadana somera tempo"}, {"Mauritius", "maŭricia somera tempo"}, {"Mexico_Pacific", "pacifika meksika somera tempo"}, {"Mongolia", "ulanbatora somera tempo"}, {"Moscow", "moskva somera tempo"}, {"New_Caledonia", "novkaledonia somera tempo"}, {"New_Zealand", "novzelanda somera tempo"}, {"Newfoundland", "Novlando (somera tempo)"}, {"Norfolk", "norfolkinsula somera tempo"}, {"Noronha", "Fernando de Noronha (somera tempo)"}, {"Novosibirsk", "novosibirska somera tempo"}, {"Omsk", "omska somera tempo"}, {"Pakistan", "pakistana somera tempo"}, {"Paraguay", "paragvaja somera tempo"}, {"Peru", "perua somera tempo"}, {"Philippines", "filipina somera tempo"}, {"Pierre_Miquelon", "Sankta Piero kaj Mikelono (somera tempo)"}, {"Sakhalin", "saĥalena somera tempo"}, {"Samoa", "uson-samoa somera tempo"}, {"Taipei", "tajvana somera tempo"}, {"Tonga", "tonga somera tempo"}, {"Turkmenistan", "turkmena somera tempo"}, {"Uruguay", "urugvaja somera tempo"}, {"Uzbekistan", "uzbeka somera tempo"}, {"Vanuatu", "vanuatua somera tempo"}, {"Vladivostok", "vladivostoka somera tempo"}, {"Volgograd", "volgograda somera tempo"}, {"Yakutsk", "jakutska somera tempo"}, {"Yekaterinburg", "jekaterinburga somera tempo"}},
	},
	{
		{{"Acre", "Hora estándar de Acre"}, {"Afghanistan", "hora de Afganistán"}, {"Africa_Central", "hora de África central"}, {"Africa_Eastern", "hora de África oriental"}, {"Africa_Southern", "hora de Sudáfrica"}, {"Africa_Western", "hora de África occidental"}, {"Alaska", "hora estándar de Alaska"}, {"Amazon", "hora estándar del Amazonas"}, {"America_Central", "hora estándar central"}, {"America_Eastern", "hora estándar oriental"}, {"America_Mountain", "hora estándar de las Montañas Rocosas"}, {"America_Pacific", "hora estándar del Pacífico"}, {"Anadyr", "hora estándar de Anadyr"}, {"Apia", "hora estándar de Apia"}, {"Aqtau", "Hora estándar de Aktau"}, {"Aqtobe", "Hora estándar de Aktobe"}, {"Arabian", "hora estándar de Arabia"}, {"Argentina", "hora estándar de Argentina"}, {"Argentina_Western", "hora estándar de Argentina occidental"}, {"Armenia", "hora estándar de Armenia"}, {"Atlantic", "hora estándar del Atlántico"}, {"Australia_Central", "hora estándar de Australia central"}, {"Australia_CentralWestern", "hora estándar de Australia centroccidental"}, {"Australia_Eastern", "hora estándar de Australia oriental"}, {"Australia_Western", "hora estándar de Australia occidental"}, {"Azerbaijan", "hora estándar de Azerbaiyán"}, {"Azores", "hora estándar de las Azores"}, {"Bangladesh", "hora estándar de Bangladés"}, {"Bhutan", "hora de Bután"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora estándar de Brasilia"}, {"Brunei", "hora de Brunéi"}, {"Cape_Verde", "hora estándar de Cabo Verde"}, {"Chamorro", "hora estándar de Chamorro"}, {"Chatham", "hora estándar de Chatham"}, {"Chile", "hora estándar de Chile"}, {"China", "hora estándar de China"}, {"Christmas", "hora de la Isla de Navidad"}, {"Cocos", "hora de las Islas Cocos"}, {"Colombia", "hora estándar de Colombia"}, {"Cook", "hora estándar de las Islas Cook"}, {"Cuba", "hora estándar de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont-d’Urville"}, {"East_Timor", "hora de Timor Oriental"}, {"Easter", "hora estándar de la isla de Pascua"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora estándar de Europa central"}, {"Europe_Eastern", "hora estándar de Europa oriental"}, {"Europe_Further_Eastern", "hora del extremo oriental de Europa"}, {"Europe_Western", "hora estándar de Europa occidental"}, {"Falkland", "hora estándar de las islas Malvinas"}, {"Fiji", "hora estándar de Fiyi"}, {"French_Guiana", "hora de la Guayana Francesa"}, {"French_Southern", "hora de Antártida y Territorios Australes Franceses"}, {"GMT", "hora del meridiano de Greenwich"}, {"Galapagos", "hora de Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora estándar de Georgia"}, {"Gilbert_Islands", "hora de las islas Gilbert"}, {"Greenland_Eastern", "hora estándar de Groenlandia oriental"}, {"Greenland_Western", "hora estándar de Groenlandia occidental"}, {"Guam", "Hora estándar de Guam"}, {"Gulf", "hora estándar del Golfo"}, {"Guyana", "hora de Guyana"}, {"Hawaii", "hora estándar de Hawái-Aleutianas"}, {"Hawaii_Aleutian", "hora estándar de Hawái-Aleutianas"}, {"Hong_Kong", "hora estándar de Hong Kong"}, {"Hovd", "hora estándar de Hovd"}, {"India", "hora estándar de la India"}, {"Indian_Ocean", "hora del océano Índico"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia central"}, {"Indonesia_Eastern", "hora de Indonesia oriental"}, {"Indonesia_Western", "hora de Indonesia occidental"}, {"Iran", "hora estándar de Irán"}, {"Irkutsk", "hora estándar de Irkutsk"}, {"Israel", "hora estándar de Israel"}, {"Japan", "hora estándar de Japón"}, {"Kamchatka", "hora estándar de Kamchatka"}, {"Kazakhstan", "hora de Kazajistán"}, {"Kazakhstan_Eastern", "hora de Kazajistán oriental"}, {"Kazakhstan_Western", "hora de Kazajistán occidental"}, {"Korea", "hora estándar de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora estándar de Krasnoyarsk"}, {"Kyrgystan", "hora de Kirguistán"}, {"Lanka", "Hora de Sri Lanka"}, {"Line_Islands", "hora de las Espóradas Ecuatoriales"}, {"Lord_Howe", "hora estándar de Lord Howe"}, {"Macau", "Hora estándar de Macao"}, {"Magadan", "hora estándar de Magadán"}, {"Malaysia", "hora de Malasia"}, {"Maldives", "hora de Maldivas"}, {"Marquesas", "hora de Marquesas"}, {"Marshall_Islands", "hora de las Islas Marshall"}, {"Mauritius", "hora estándar de Mauricio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora estándar del Pacífico de México"}, {"Mongolia", "hora estándar de Ulán Bator"}, {"Moscow", "hora estándar de Moscú"}, {"Myanmar", "hora de Myanmar"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora estándar de Nueva Caledonia"}, {"New_Zealand", "hora estándar de Nueva Zelanda"}, {"Newfoundland", "hora estándar de Terranova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora estándar de la isla Norfolk"}, {"Noronha", "hora estándar de Fernando de Noronha"}, {"North_Mariana", "Hora de las Islas Marianas del Norte"}, {"Novosibirsk", "hora estándar de Novosibirsk"}, {"Omsk", "hora estándar de Omsk"}, {"Pakistan", "hora estándar de Pakistán"}, {"Palau", "hora de Palaos"}, {"Papua_New_Guinea", "hora de Papúa Nueva Guinea"}, {"Paraguay", "hora estándar de Paraguay"}, {"Peru", "hora estándar de Perú"}, {"Philippines", "hora estándar de Filipinas"}, {"Phoenix_Islands", "hora de las Islas Fénix"}, {"Pierre_Miquelon", "hora estándar de San Pedro y Miquelón"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Pyongyang"}, {"Qyzylorda", "Hora estándar de Qyzylorda"}, {"Reunion", "hora de Reunión"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora estándar de Sajalín"}, {"Samara", "hora estándar de Samara"}, {"Samoa", "hora estándar de Samoa"}, {"Seychelles", "hora de Seychelles"}, {"Singapore", "hora de Singapur"}, {"Solomon", "hora de las Islas Salomón"}, {"South_Georgia", "hora de Georgia del Sur"}, {"Suriname", "hora de Surinam"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahití"}, {"Taipei", "hora estándar de Taipéi"}, {"Tajikistan", "hora de Tayikistán"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora estándar de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkey", "Hora estándar de Turquía"}, {"Turkmenistan", "hora estándar de Turkmenistán"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora estándar de Uruguay"}, {"Uzbekistan", "hora estándar de Uzbekistán"}, {"Vanuatu", "hora estándar de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora estándar de Vladivostok"}, {"Volgograd", "hora estándar de Volgogrado"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora de la isla Wake"}, {"Wallis", "hora de Wallis y Futuna"}, {"Yakutsk", "hora estándar de Yakutsk"}, {"Yekaterinburg", "hora estándar de Ekaterimburgo"}, {"Yukon", "hora de Yukón"}},
		{{"Acre", "Hora de verano de Acre"}, {"Alaska", "hora de verano de Alaska"}, {"Amazon", "hora de verano del Amazonas"}, {"America_Central", "hora de verano central"}, {"America_Eastern", "hora de verano oriental"}, {"America_Mountain", "hora de verano de las Montañas Rocosas"}, {"America_Pacific", "hora de verano del Pacífico"}, {"Anadyr", "hora de verano de Anadyr"}, {"Apia", "horario de verano de Apia"}, {"Aqtau", "Hora de verano de Aktau"}, {"Aqtobe", "Hora de verano de Aktobe"}, {"Arabian", "hora de verano de Arabia"}, {"Argentina", "hora de verano de Argentina"}, {"Argentina_Western", "hora de verano de Argentina occidental"}, {"Armenia", "hora de verano de Armenia"}, {"Atlantic", "hora de verano del Atlántico"}, {"Australia_Central", "hora de verano de Australia central"}, {"Australia_CentralWestern", "hora de verano de Australia centroccidental"}, {"Australia_Eastern", "hora de verano de Australia oriental"}, {"Australia_Western", "hora de verano de Australia occidental"}, {"Azerbaijan", "hora de verano de Azerbaiyán"}, {"Azores", "hora de verano de las Azores"}, {"Bangladesh", "hora de verano de Bangladés"}, {"Brasilia", "hora de verano de Brasilia"}, {"Cape_Verde", "hora de verano de Cabo Verde"}, {"Chatham", "hora de verano de Chatham"}, {"Chile", "hora de verano de Chile"}, {"China", "hora de verano de China"}, {"Colombia", "hora de verano de Colombia"}, {"Cook", "hora de verano media de las Islas Cook"}, {"Cuba", "hora de verano de Cuba"}, {"Easter", "hora de verano de la isla de Pascua"}, {"Europe_Central", "hora de verano de Europa central"}, {"Europe_Eastern", "hora de verano de Europa oriental"}, {"Europe_Western", "hora de verano de Europa occidental"}, {"Falkland", "hora de verano de las islas Malvinas"}, {"Fiji", "hora de verano de Fiyi"}, {"Georgia", "hora de verano de Georgia"}, {"Greenland_Eastern", "hora de verano de Groenlandia oriental"}, {"Greenland_Western", "hora de verano de Groenlandia occidental"}, {"Hawaii_Aleutian", "hora de verano de Hawái-Aleutianas"}, {"Hong_Kong", "hora de verano de Hong Kong"}, {"Hovd", "hora de verano de Hovd"}, {"Iran", "hora de verano de Irán"}, {"Irkutsk", "hora de verano de Irkutsk"}, {"Israel", "hora de verano de Israel"}, {"Japan", "hora de verano de Japón"}, {"Kamchatka", "hora de verano de Kamchatka"}, {"Korea", "hora de verano de Corea"}, {"Krasnoyarsk", "hora de verano de Krasnoyarsk"}, {"Lord_Howe", "hora de verano de Lord Howe"}, {"Macau", "Hora de verano de Macao"}, {"Magadan", "hora de verano de Magadán"}, {"Mauritius", "hora de verano de Mauricio"}, {"Mexico_Pacific", "hora de verano del Pacífico de México"}, {"Mongolia", "hora de verano de Ulán Bator"}, {"Moscow", "hora de verano de Moscú"}, {"New_Caledonia", "hora de verano de Nueva Caledonia"}, {"New_Zealand", "hora de verano de Nueva Zelanda"}, {"Newfoundland", "hora de verano de Terranova"}, {"Norfolk", "hora de verano de la isla Norfolk"}, {"Noronha", "hora de verano de Fernando de Noronha"}, {"Novosibirsk", "hora de verano de Novosibirsk"}, {"Omsk", "hora de verano de Omsk"}, {"Pakistan", "hora de verano de Pakistán"}, {"Paraguay", "hora de verano de Paraguay"}, {"Peru", "hora de verano de Perú"}, {"Philippines", "hora de verano de Filipinas"}, {"Pierre_Miquelon", "hora de verano de San Pedro y Miquelón"}, {"Qyzylorda", "Hora de verano de Qyzylorda"}, {"Sakhalin", "hora de verano de Sajalín"}, {"Samara", "hora de verano de Samara"}, {"Samoa", "hora de verano de Samoa"}, {"Taipei", "hora de verano de Taipéi"}, {"Tonga", "hora de verano de Tonga"}, {"Turkey", "Hora de verano de Turquía"}, {"Turkmenistan", "hora de verano de Turkmenistán"}, {"Uruguay", "hora de verano de Uruguay"}, {"Uzbekistan", "hora de verano de Uzbekistán"}, {"Vanuatu", "hora de verano de Vanuatu"}, {"Vladivostok", "hora de verano de Vladivostok"}, {"Volgograd", "hora de verano de Volgogrado"}, {"Yakutsk", "hora de verano de Yakutsk"}, {"Yekaterinburg", "hora de verano de Ekaterimburgo"}},
	},
	{
		{{"Acre", "Hora estándar de Acre"}, {"Afghanistan", "hora de Afganistán"}, {"Africa_Central", "hora de África central"}, {"Africa_Eastern", "hora de África oriental"}, {"Africa_Southern", "hora de Sudáfrica"}, {"Africa_Western", "hora de África occidental"}, {"Alaska", "hora estándar de Alaska"}, {"Amazon", "hora estándar del Amazonas"}, {"America_Central", "hora estándar central"}, {"America_Eastern", "hora estándar oriental"}, {"America_Mountain", "hora estándar de la montaña"}, {"America_Pacific", "hora estándar del Pacífico"}, {"Anadyr", "hora estándar de Anadyr"}, {"Apia", "hora estándar de Apia"}, {"Aqtau", "Hora estándar de Aktau"}, {"Aqtobe", "Hora estándar de Aktobe"}, {"Arabian", "hora estándar de Arabia"}, {"Argentina", "hora estándar de Argentina"}, {"Argentina_Western", "hora estándar de Argentina occidental"}, {"Armenia", "hora estándar de Armenia"}, {"Atlantic", "hora estándar del Atlántico"}, {"Australia_Central", "hora estándar de Australia central"}, {"Australia_CentralWestern", "hora estándar de Australia centroccidental"}, {"Australia_Eastern", "hora estándar de Australia oriental"}, {"Australia_Western", "hora estándar de Australia occidental"}, {"Azerbaijan", "hora estándar de Azerbaiyán"}, {"Azores", "hora estándar de las Azores"}, {"Bangladesh", "hora estándar de Bangladés"}, {"Bhutan", "hora de Bután"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora estándar de Brasilia"}, {"Brunei", "hora de Brunéi"}, {"Cape_Verde", "hora estándar de Cabo Verde"}, {"Chamorro", "hora estándar de Chamorro"}, {"Chatham", "hora estándar de Chatham"}, {"Chile", "hora estándar de Chile"}, {"China", "hora estándar de China"}, {"Christmas", "hora de la Isla de Navidad"}, {"Cocos", "hora de Islas Cocos"}, {"Colombia", "hora estándar de Colombia"}, {"Cook", "hora estándar de las islas Cook"}, {"Cuba", "hora estándar de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont-d’Urville"}, {"East_Timor", "hora de Timor Oriental"}, {"Easter", "hora estándar de Isla de Pascua"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora estándar de Europa central"}, {"Europe_Eastern", "hora estándar de Europa del Este"}, {"Europe_Further_Eastern", "horario del lejano este de Europa"}, {"Europe_Western", "hora estándar de Europa del Oeste"}, {"Falkland", "hora estándar de las Islas Malvinas"}, {"Fiji", "hora estándar de Fiyi"}, {"French_Guiana", "hora de la Guayana Francesa"}, {"French_Southern", "hora de las Tierras Australes y Antárticas Francesas"}, {"GMT", "hora del meridiano de Greenwich"}, {"Galapagos", "hora de Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora estándar de Georgia"}, {"Gilbert_Islands", "hora de Islas Gilbert"}, {"Greenland_Eastern", "hora estándar de Groenlandia oriental"}, {"Greenland_Western", "hora estándar de Groenlandia occidental"}, {"Guam", "Hora estándar de Guam"}, {"Gulf", "hora estándar del Golfo"}, {"Guyana", "hora de Guyana"}, {"Hawaii", "hora estándar de Hawái-Aleutianas"}, {"Hawaii_Aleutian", "hora estándar de Hawái-Aleutianas"}, {"Hong_Kong", "hora estándar de Hong Kong"}, {"Hovd", "hora estándar de Hovd"}, {"India", "hora de India"}, {"Indian_Ocean", "hora del océano Índico"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia central"}, {"Indonesia_Eastern", "hora de Indonesia oriental"}, {"Indonesia_Western", "hora de Indonesia occidental"}, {"Iran", "hora estándar de Irán"}, {"Irkutsk", "hora estándar de Irkutsk"}, {"Israel", "hora estándar de Israel"}, {"Japan", "hora estándar de Japón"}, {"Kamchatka", "hora estándar de Kamchatka"}, {"Kazakhstan", "hora de Kazajistán"}, {"Kazakhstan_Eastern", "hora de Kazajistán oriental"}, {"Kazakhstan_Western", "hora de Kazajistán occidental"}, {"Korea", "hora estándar de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora estándar de Krasnoyarsk"}, {"Kyrgystan", "hora de Kirguistán"}, {"Lanka", "Hora de Sri Lanka"}, {"Line_Islands", "hora de las Espóradas Ecuatoriales"}, {"Lord_Howe", "hora estándar de Lord Howe"}, {"Macau", "Hora estándar de Macao"}, {"Magadan", "hora estándar de Magadán"}, {"Malaysia", "hora de Malasia"}, {"Maldives", "hora de Maldivas"}, {"Marquesas", "hora de Marquesas"}, {"Marshall_Islands", "hora de Islas Marshall"}, {"Mauritius", "hora estándar de Mauricio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora estándar del Pacífico de México"}, {"Mongolia", "hora estándar de Ulán Bator"}, {"Moscow", "hora estándar de Moscú"}, {"Myanmar", "hora de Myanmar (Birmania)"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora estándar de Nueva Caledonia"}, {"New_Zealand", "hora estándar de Nueva Zelanda"}, {"Newfoundland", "hora estándar de Terranova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora estándar de la Isla Norfolk"}, {"Noronha", "hora estándar de Fernando de Noronha"}, {"North_Mariana", "Hora de las Islas Marianas del Norte"}, {"Novosibirsk", "hora estándar de Novosibirsk"}, {"Omsk", "hora estándar de Omsk"}, {"Pakistan", "hora estándar de Pakistán"}, {"Palau", "hora de Palaos"}, {"Papua_New_Guinea", "hora de Papúa Nueva Guinea"}, {"Paraguay", "hora estándar de Paraguay"}, {"Peru", "hora estándar de Perú"}, {"Philippines", "hora estándar de Filipinas"}, {"Phoenix_Islands", "hora de las Islas Fénix"}, {"Pierre_Miquelon", "hora estándar de San Pedro y Miquelón"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Pionyang"}, {"Qyzylorda", "Hora estándar de Qyzylorda"}, {"Reunion", "hora de Reunión"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora estándar de Sajalín"}, {"Samara", "hora estándar de Samara"}, {"Samoa", "hora estándar de Samoa"}, {"Seychelles", "hora de Seychelles"}, {"Singapore", "hora de Singapur"}, {"Solomon", "hora de Islas Salomón"}, {"South_Georgia", "hora de Georgia del Sur"}, {"Suriname", "hora de Surinam"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahití"}, {"Taipei", "hora estándar de Taipéi"}, {"Tajikistan", "hora de Tayikistán"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora estándar de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkey", "Hora estándar de Turquía"}, {"Turkmenistan", "hora estándar de Turkmenistán"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora estándar de Uruguay"}, {"Uzbekistan", "hora estándar de Uzbekistán"}, {"Vanuatu", "hora estándar de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora estándar de Vladivostok"}, {"Volgograd", "hora estándar de Volgogrado"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora de Isla Wake"}, {"Wallis", "hora de Wallis y Futuna"}, {"Yakutsk", "hora estándar de Yakutsk"}, {"Yekaterinburg", "hora estándar de Ekaterimburgo"}, {"Yukon", "hora de Yukón"}},
		{{"Acre", "Hora de verano de Acre"}, {"Alaska", "hora de verano de Alaska"}, {"Amazon", "hora de verano del Amazonas"}, {"America_Central", "hora de verano central"}, {"America_Eastern", "hora de verano oriental"}, {"America_Mountain", "hora de verano de la montaña"}, {"America_Pacific", "hora de verano del Pacífico"}, {"Anadyr", "hora de verano de Anadyr"}, {"Apia", "horario de verano de Apia"}, {"Aqtau", "Hora de verano de Aktau"}, {"Aqtobe", "Hora de verano de Aktobe"}, {"Arabian", "hora de verano de Arabia"}, {"Argentina", "hora de verano de Argentina"}, {"Argentina_Western", "hora de verano de Argentina occidental"}, {"Armenia", "hora de verano de Armenia"}, {"Atlantic", "hora de verano del Atlántico"}, {"Australia_Central", "hora de verano de Australia central"}, {"Australia_CentralWestern", "hora de verano de Australia centroccidental"}, {"Australia_Eastern", "hora de verano de Australia oriental"}, {"Australia_Western", "hora de verano de Australia occidental"}, {"Azerbaijan", "hora de verano de Azerbaiyán"}, {"Azores", "hora de verano de las Azores"}, {"Bangladesh", "hora de verano de Bangladés"}, {"Brasilia", "hora de verano de Brasilia"}, {"Cape_Verde", "hora de verano de Cabo Verde"}, {"Chatham", "hora de verano de Chatham"}, {"Chile", "hora de verano de Chile"}, {"China", "hora de verano de China"}, {"Colombia", "hora de verano de Colombia"}, {"Cook", "hora de verano media de las islas Cook"}, {"Cuba", "hora de verano de Cuba"}, {"Easter", "hora de verano de la Isla de Pascua"}, {"Europe_Central", "hora de verano de Europa central"}, {"Europe_Eastern", "hora de verano de Europa del Este"}, {"Europe_Western", "hora de verano de Europa del Oeste"}, {"Falkland", "hora de verano de las Islas Malvinas"}, {"Fiji", "hora de verano de Fiyi"}, {"Georgia", "hora de verano de Georgia"}, {"Greenland_Eastern", "hora de verano de Groenlandia oriental"}, {"Greenland_Western", "hora de verano de Groenlandia occidental"}, {"Hawaii_Aleutian", "hora de verano de Hawái-Aleutianas"}, {"Hong_Kong", "hora de verano de Hong Kong"}, {"Hovd", "hora de verano de Hovd"}, {"Iran", "hora de verano de Irán"}, {"Irkutsk", "hora de verano de Irkutsk"}, {"Israel", "hora de verano de Israel"}, {"Japan", "hora de verano de Japón"}, {"Kamchatka", "hora de verano de Kamchatka"}, {"Korea", "hora de verano de Corea"}, {"Krasnoyarsk", "hora de verano de Krasnoyarsk"}, {"Lord_Howe", "hora de verano de Lord Howe"}, {"Macau", "Hora de verano de Macao"}, {"Magadan", "hora de verano de Magadán"}, {"Mauritius", "hora de verano de Mauricio"}, {"Mexico_Pacific", "hora de verano del Pacífico de México"}, {"Mongolia", "hora de verano de Ulán Bator"}, {"Moscow", "hora de verano de Moscú"}, {"New_Caledonia", "hora de verano de Nueva Caledonia"}, {"New_Zealand", "hora de verano de Nueva Zelanda"}, {"Newfoundland", "hora de verano de Terranova"}, {"Norfolk", "hora de verano de la Isla Norfolk"}, {"Noronha", "hora de verano de Fernando de Noronha"}, {"Novosibirsk", "hora de verano de Novosibirsk"}, {"Omsk", "hora de verano de Omsk"}, {"Pakistan", "hora de verano de Pakistán"}, {"Paraguay", "hora de verano de Paraguay"}, {"Peru", "hora de verano de Perú"}, {"Philippines", "hora de verano de Filipinas"}, {"Pierre_Miquelon", "hora de verano de San Pedro y Miquelón"}, {"Qyzylorda", "Hora de verano de Qyzylorda"}, {"Sakhalin", "hora de verano de Sajalín"}, {"Samara", "hora de verano de Samara"}, {"Samoa", "hora de verano de Samoa"}, {"Taipei", "hora de verano de Taipéi"}, {"Tonga", "hora de verano de Tonga"}, {"Turkey", "Hora de verano de Turquía"}, {"Turkmenistan", "hora de verano de Turkmenistán"}, {"Uruguay", "hora de verano de Uruguay"}, {"Uzbekistan", "hora de verano de Uzbekistán"}, {"Vanuatu", "hora de verano de Vanuatu"}, {"Vladivostok", "hora de verano de Vladivostok"}, {"Volgograd", "hora de verano de Volgogrado"}, {"Yakutsk", "hora de verano de Yakutsk"}, {"Yekaterinburg", "hora de verano de Ekaterimburgo"}},
	},
	{
		{{"Acre", "Hora estándar de Acre"}, {"Afghanistan", "hora de Afganistán"}, {"Africa_Central", "hora de África central"}, {"Africa_Eastern", "hora de África oriental"}, {"Africa_Southern", "hora de Sudáfrica"}, {"Africa_Western", "hora de África occidental"}, {"Alaska", "hora estándar de Alaska"}, {"Amazon", "hora estándar del Amazonas"}, {"America_Central", "hora estándar central"}, {"America_Eastern", "hora estándar oriental"}, {"America_Mountain", "hora estándar de la montaña"}, {"America_Pacific", "hora estándar del Pacífico"}, {"Anadyr", "hora estándar de Anadyr"}, {"Apia", "hora estándar de Apia"}, {"Aqtau", "Hora estándar de Aktau"}, {"Aqtobe", "Hora estándar de Aktobe"}, {"Arabian", "hora estándar de Arabia"}, {"Argentina", "hora estándar de Argentina"}, {"Argentina_Western", "hora estándar de Argentina occidental"}, {"Armenia", "hora estándar de Armenia"}, {"Atlantic", "hora estándar del Atlántico"}, {"Australia_Central", "hora estándar de Australia central"}, {"Australia_CentralWestern", "hora estándar de Australia centroccidental"}, {"Australia_Eastern", "hora estándar de Australia oriental"}, {"Australia_Western", "hora estándar de Australia occidental"}, {"Azerbaijan", "hora estándar de Azerbaiyán"}, {"Azores", "hora estándar de las Azores"}, {"Bangladesh", "hora estándar de Bangladés"}, {"Bhutan", "hora de Bután"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora estándar de Brasilia"}, {"Brunei", "hora de Brunéi"}, {"Cape_Verde", "hora estándar de Cabo Verde"}, {"Chamorro", "hora estándar de Chamorro"}, {"Chatham", "hora estándar de Chatham"}, {"Chile", "hora estándar de Chile"}, {"China", "hora estándar de China"}, {"Christmas", "hora de la isla de Navidad"}, {"Cocos", "hora de las Islas Cocos"}, {"Colombia", "hora estándar de Colombia"}, {"Cook", "hora estándar de las Islas Cook"}, {"Cuba", "hora estándar de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont-d’Urville"}, {"East_Timor", "hora de Timor Oriental"}, {"Easter", "hora estándar de la isla de Pascua"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora estándar de Europa central"}, {"Europe_Eastern", "hora estándar de Europa oriental"}, {"Europe_Further_Eastern", "hora del extremo oriental de Europa"}, {"Europe_Western", "hora estándar de Europa occidental"}, {"Falkland", "hora estándar de Islas Malvinas"}, {"Fiji", "hora estándar de Fiyi"}, {"French_Guiana", "hora de la Guayana Francesa"}, {"French_Southern", "hora de las Tierras Australes y Antárticas Francesas"}, {"GMT", "hora del meridiano de Greenwich"}, {"Galapagos", "hora de Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora estándar de Georgia"}, {"Gilbert_Islands", "hora de las Islas Gilbert"}, {"Greenland_Eastern", "hora estándar de Groenlandia oriental"}, {"Greenland_Western", "hora estándar de Groenlandia occidental"}, {"Guam", "Hora estándar de Guam"}, {"Gulf", "hora estándar del Golfo"}, {"Guyana", "hora de Guyana"}, {"Hawaii", "hora estándar de Hawái-Aleutianas"}, {"Hawaii_Aleutian", "hora estándar de Hawái-Aleutianas"}, {"Hong_Kong", "hora estándar de Hong Kong"}, {"Hovd", "hora estándar de Hovd"}, {"India", "hora de India"}, {"Indian_Ocean", "hora del océano Índico"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia central"}, {"Indonesia_Eastern", "hora de Indonesia oriental"}, {"Indonesia_Western", "hora de Indonesia occidental"}, {"Iran", "hora estándar de Irán"}, {"Irkutsk", "hora estándar de Irkutsk"}, {"Israel", "hora estándar de Israel"}, {"Japan", "hora estándar de Japón"}, {"Kamchatka", "hora estándar de Kamchatka"}, {"Kazakhstan", "hora de Kazajistán"}, {"Kazakhstan_Eastern", "hora de Kazajistán oriental"}, {"Kazakhstan_Western", "hora de Kazajistán occidental"}, {"Korea", "hora estándar de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora estándar de Krasnoyarsk"}, {"Kyrgystan", "hora de Kirguistán"}, {"Lanka", "Hora de Sri Lanka"}, {"Line_Islands", "hora de las Espóradas Ecuatoriales"}, {"Lord_Howe", "hora estándar de Lord Howe"}, {"Macau", "Hora estándar de Macao"}, {"Magadan", "hora estándar de Magadán"}, {"Malaysia", "hora de Malasia"}, {"Maldives", "hora de Maldivas"}, {"Marquesas", "hora de Marquesas"}, {"Marshall_Islands", "hora de las Islas Marshall"}, {"Mauritius", "hora estándar de Mauricio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora estándar del Pacífico de México"}, {"Mongolia", "hora estándar de Ulán Bator"}, {"Moscow", "hora estándar de Moscú"}, {"Myanmar", "hora de Myanmar (Birmania)"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora estándar de Nueva Caledonia"}, {"New_Zealand", "hora estándar de Nueva Zelanda"}, {"Newfoundland", "hora estándar de Terranova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora estándar de la Isla Norfolk"}, {"Noronha", "hora estándar de Fernando de Noronha"}, {"North_Mariana", "Hora de las Islas Marianas del Norte"}, {"Novosibirsk", "hora estándar de Novosibirsk"}, {"Omsk", "hora estándar de Omsk"}, {"Pakistan", "hora estándar de Pakistán"}, {"Palau", "hora de Palaos"}, {"Papua_New_Guinea", "hora de Papúa Nueva Guinea"}, {"Paraguay", "hora estándar de Paraguay"}, {"Peru", "hora estándar de Perú"}, {"Philippines", "hora estándar de Filipinas"}, {"Phoenix_Islands", "hora de las Islas Fénix"}, {"Pierre_Miquelon", "hora estándar de San Pedro y Miquelón"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Pyongyang"}, {"Qyzylorda", "Hora estándar de Qyzylorda"}, {"Reunion", "hora de Reunión"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora estándar de Sajalín"}, {"Samara", "hora estándar de Samara"}, {"Samoa", "hora estándar de Samoa"}, {"Seychelles", "hora de Seychelles"}, {"Singapore", "hora de Singapur"}, {"Solomon", "hora de las Islas Salomón"}, {"South_Georgia", "hora de Georgia del Sur"}, {"Suriname", "hora de Surinam"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahití"}, {"Taipei", "hora estándar de Taipéi"}, {"Tajikistan", "hora de Tayikistán"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora estándar de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkey", "Hora estándar de Turquía"}, {"Turkmenistan", "hora estándar de Turkmenistán"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora estándar de Uruguay"}, {"Uzbekistan", "hora estándar de Uzbekistán"}, {"Vanuatu", "hora estándar de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora estándar de Vladivostok"}, {"Volgograd", "hora estándar de Volgogrado"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora de la Isla Wake"}, {"Wallis", "hora de Wallis y Futuna"}, {"Yakutsk", "hora estándar de Yakutsk"}, {"Yekaterinburg", "hora estándar de Ekaterimburgo"}, {"Yukon", "hora de Yukón"}},
		{{"Acre", "Hora de verano de Acre"}, {"Alaska", "hora de verano de Alaska"}, {"Amazon", "hora de verano del Amazonas"}, {"America_Central", "hora de verano central"}, {"America_Eastern", "hora de verano oriental"}, {"America_Mountain", "hora de verano de la montaña"}, {"America_Pacific", "hora de verano del Pacífico"}, {"Anadyr", "hora de verano de Anadyr"}, {"Apia", "hora de verano de Apia"}, {"Aqtau", "Hora de verano de Aktau"}, {"Aqtobe", "Hora de verano de Aktobe"}, {"Arabian", "hora de verano de Arabia"}, {"Argentina", "hora de verano de Argentina"}, {"Argentina_Western", "hora de verano de Argentina occidental"}, {"Armenia", "hora de verano de Armenia"}, {"Atlantic", "hora de verano del Atlántico"}, {"Australia_Central", "hora de verano de Australia central"}, {"Australia_CentralWestern", "hora de verano de Australia centroccidental"}, {"Australia_Eastern", "hora de verano de Australia oriental"}, {"Australia_Western", "hora de verano de Australia occidental"}, {"Azerbaijan", "hora de verano de Azerbaiyán"}, {"Azores", "hora de verano de las Azores"}, {"Bangladesh", "hora de verano de Bangladés"}, {"Brasilia", "hora de verano de Brasilia"}, {"Cape_Verde", "hora de verano de Cabo Verde"}, {"Chatham", "hora de verano de Chatham"}, {"Chile", "hora de verano de Chile"}, {"China", "hora de verano de China"}, {"Colombia", "hora de verano de Colombia"}, {"Cook", "hora de verano media de las Islas Cook"}, {"Cuba", "hora de verano de Cuba"}, {"Easter", "hora de verano de la isla de Pascua"}, {"Europe_Central", "hora de verano de Europa central"}, {"Europe_Eastern", "hora de verano de Europa oriental"}, {"Europe_Western", "hora de verano de Europa occidental"}, {"Falkland", "hora de verano de Islas Malvinas"}, {"Fiji", "hora de verano de Fiyi"}, {"Georgia", "hora de verano de Georgia"}, {"Greenland_Eastern", "hora de verano de Groenlandia oriental"}, {"Greenland_Western", "hora de verano de Groenlandia occidental"}, {"Hawaii_Aleutian", "hora de verano de Hawái-Aleutianas"}, {"Hong_Kong", "hora de verano de Hong Kong"}, {"Hovd", "hora de verano de Hovd"}, {"Iran", "hora de verano de Irán"}, {"Irkutsk", "hora de verano de Irkutsk"}, {"Israel", "hora de verano de Israel"}, {"Japan", "hora de verano de Japón"}, {"Kamchatka", "hora de verano de Kamchatka"}, {"Korea", "hora de verano de Corea"}, {"Krasnoyarsk", "hora de verano de Krasnoyarsk"}, {"Lord_Howe", "hora de verano de Lord Howe"}, {"Macau", "Hora de verano de Macao"}, {"Magadan", "hora de verano de Magadán"}, {"Mauritius", "hora de verano de Mauricio"}, {"Mexico_Pacific", "hora de verano del Pacífico de México"}, {"Mongolia", "hora de verano de Ulán Bator"}, {"Moscow", "hora de verano de Moscú"}, {"New_Caledonia", "hora de verano de Nueva Caledonia"}, {"New_Zealand", "hora de verano de Nueva Zelanda"}, {"Newfoundland", "hora de verano de Terranova"}, {"Norfolk", "hora de verano de la Isla Norfolk"}, {"Noronha", "hora de verano de Fernando de Noronha"}, {"Novosibirsk", "hora de verano de Novosibirsk"}, {"Omsk", "hora de verano de Omsk"}, {"Pakistan", "hora de verano de Pakistán"}, {"Paraguay", "hora de verano de Paraguay"}, {"Peru", "hora de verano de Perú"}, {"Philippines", "hora de verano de Filipinas"}, {"Pierre_Miquelon", "hora de verano de San Pedro y Miquelón"}, {"Qyzylorda", "Hora de verano de Qyzylorda"}, {"Sakhalin", "hora de verano de Sajalín"}, {"Samara", "hora de verano de Samara"}, {"Samoa", "hora de verano de Samoa"}, {"Taipei", "hora de verano de Taipéi"}, {"Tonga", "hora de verano de Tonga"}, {"Turkey", "Hora de verano de Turquía"}, {"Turkmenistan", "hora de verano de Turkmenistán"}, {"Uruguay", "hora de verano de Uruguay"}, {"Uzbekistan", "hora de verano de Uzbekistán"}, {"Vanuatu", "hora de verano de Vanuatu"}, {"Vladivostok", "hora de verano de Vladivostok"}, {"Volgograd", "hora de verano de Volgogrado"}, {"Yakutsk", "hora de verano de Yakutsk"}, {"Yekaterinburg", "hora de verano de Ekaterimburgo"}},
	},
	{
		{{"Acre", "Hora estándar de Acre"}, {"Afghanistan", "hora de Afganistán"}, {"Africa_Central", "hora de África central"}, {"Africa_Eastern", "hora de África oriental"}, {"Africa_Southern", "hora de Sudáfrica"}, {"Africa_Western", "hora de África occidental"}, {"Alaska", "hora estándar de Alaska"}, {"Amazon", "hora estándar del Amazonas"}, {"America_Central", "hora estándar central"}, {"America_Eastern", "hora estándar oriental"}, {"America_Mountain", "hora estándar de las Montañas Rocosas"}, {"America_Pacific", "hora estándar del Pacífico"}, {"Anadyr", "hora estándar de Anadyr"}, {"Apia", "hora estándar de Apia"}, {"Aqtau", "Hora estándar de Aktau"}, {"Aqtobe", "Hora estándar de Aktobe"}, {"Arabian", "hora estándar de Arabia"}, {"Argentina", "hora estándar de Argentina"}, {"Argentina_Western", "hora estándar de Argentina occidental"}, {"Armenia", "hora estándar de Armenia"}, {"Atlantic", "hora estándar del Atlántico"}, {"Australia_Central", "hora estándar de Australia central"}, {"Australia_CentralWestern", "hora estándar de Australia centroccidental"}, {"Australia_Eastern", "hora estándar de Australia oriental"}, {"Australia_Western", "hora estándar de Australia occidental"}, {"Azerbaijan", "hora estándar de Azerbaiyán"}, {"Azores", "hora estándar de las Azores"}, {"Bangladesh", "hora estándar de Bangladés"}, {"Bhutan", "hora de Bután"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora estándar de Brasilia"}, {"Brunei", "hora de Brunéi"}, {"Cape_Verde", "hora estándar de Cabo Verde"}, {"Chamorro", "hora de Chamorro"}, {"Chatham", "hora estándar de Chatham"}, {"Chile", "hora estándar de Chile"}, {"China", "hora estándar de China"}, {"Christmas", "hora de la Isla de Navidad"}, {"Cocos", "hora de las Islas Cocos"}, {"Colombia", "hora estándar de Colombia"}, {"Cook", "hora estándar de las Islas Cook"}, {"Cuba", "hora estándar de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont-d’Urville"}, {"East_Timor", "hora de Timor Oriental"}, {"Easter", "hora estándar de la isla de Pascua"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora estándar de Europa central"}, {"Europe_Eastern", "hora estándar de Europa oriental"}, {"Europe_Further_Eastern", "hora del extremo oriental de Europa"}, {"Europe_Western", "hora estándar de Europa occidental"}, {"Falkland", "hora estándar de las islas Malvinas"}, {"Fiji", "hora estándar de Fiyi"}, {"French_Guiana", "hora de la Guayana Francesa"}, {"French_Southern", "hora de las Tierras Australes y Antárticas Francesas"}, {"GMT", "hora del meridiano de Greenwich"}, {"Galapagos", "hora de Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora estándar de Georgia"}, {"Gilbert_Islands", "hora de las islas Gilbert"}, {"Greenland_Eastern", "hora estándar de Groenlandia oriental"}, {"Greenland_Western", "hora estándar de Groenlandia occidental"}, {"Guam", "Hora estándar de Guam"}, {"Gulf", "hora estándar del Golfo"}, {"Guyana", "hora de Guyana"}, {"Hawaii", "hora estándar de Hawái-Aleutianas"}, {"Hawaii_Aleutian", "hora estándar de Hawái-Aleutianas"}, {"Hong_Kong", "hora estándar de Hong Kong"}, {"Hovd", "hora estándar de Hovd"}, {"India", "hora de India"}, {"Indian_Ocean", "hora del Océano Índico"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia central"}, {"Indonesia_Eastern", "hora de Indonesia oriental"}, {"Indonesia_Western", "hora de Indonesia occidental"}, {"Iran", "hora estándar de Irán"}, {"Irkutsk", "hora estándar de Irkutsk"}, {"Israel", "hora estándar de Israel"}, {"Japan", "hora estándar de Japón"}, {"Kamchatka", "hora estándar de Kamchatka"}, {"Kazakhstan", "hora de Kazajistán"}, {"Kazakhstan_Eastern", "hora de Kazajistán oriental"}, {"Kazakhstan_Western", "hora de Kazajistán occidental"}, {"Korea", "hora estándar de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora estándar de Krasnoyarsk"}, {"Kyrgystan", "hora de Kirguistán"}, {"Lanka", "Hora de Sri Lanka"}, {"Line_Islands", "hora de las Espóradas Ecuatoriales"}, {"Lord_Howe", "hora estándar de Lord Howe"}, {"Macau", "Hora estándar de Macao"}, {"Magadan", "hora estándar de Magadán"}, {"Malaysia", "hora de Malasia"}, {"Maldives", "hora de Maldivas"}, {"Marquesas", "hora de las islas Marquesas"}, {"Marshall_Islands", "hora de las Islas Marshall"}, {"Mauritius", "hora estándar de Mauricio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora estándar del Pacífico de México"}, {"Mongolia", "hora estándar de Ulán Bator"}, {"Moscow", "hora estándar de Moscú"}, {"Myanmar", "hora de Myanmar (Birmania)"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora estándar de Nueva Caledonia"}, {"New_Zealand", "hora estándar de Nueva Zelanda"}, {"Newfoundland", "hora estándar de Terranova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora estándar de la isla Norfolk"}, {"Noronha", "hora estándar de Fernando de Noronha"}, {"North_Mariana", "Hora de las Islas Marianas del Norte"}, {"Novosibirsk", "hora estándar de Novosibirsk"}, {"Omsk", "hora estándar de Omsk"}, {"Pakistan", "hora estándar de Pakistán"}, {"Palau", "hora de Palaos"}, {"Papua_New_Guinea", "hora de Papúa Nueva Guinea"}, {"Paraguay", "hora estándar de Paraguay"}, {"Peru", "hora estándar de Perú"}, {"Philippines", "hora estándar de Filipinas"}, {"Phoenix_Islands", "hora de las islas Fénix"}, {"Pierre_Miquelon", "hora estándar de San Pedro y Miquelón"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Pyongyang"}, {"Qyzylorda", "Hora estándar de Qyzylorda"}, {"Reunion", "hora de Reunión"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora estándar de Sajalín"}, {"Samara", "hora estándar de Samara"}, {"Samoa", "hora estándar de Samoa"}, {"Seychelles", "hora de Seychelles"}, {"Singapore", "hora de Singapur"}, {"Solomon", "hora de las Islas Salomón"}, {"South_Georgia", "hora de Georgia del Sur"}, {"Suriname", "hora de Surinam"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahití"}, {"Taipei", "hora estándar de Taipéi"}, {"Tajikistan", "hora de Tayikistán"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora estándar de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkey", "Hora estándar de Turquía"}, {"Turkmenistan", "hora estándar de Turkmenistán"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora estándar de Uruguay"}, {"Uzbekistan", "hora estándar de Uzbekistán"}, {"Vanuatu", "hora estándar de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora estándar de Vladivostok"}, {"Volgograd", "hora estándar de Volgogrado"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora de la isla Wake"}, {"Wallis", "hora de Wallis y Futuna"}, {"Yakutsk", "hora estándar de Yakutsk"}, {"Yekaterinburg", "hora estándar de Ekaterimburgo"}, {"Yukon", "hora de Yukón"}},
		{{"Acre", "Hora de verano de Acre"}, {"Alaska", "hora de verano de Alaska"}, {"Amazon", "hora de verano del Amazonas"}, {"America_Central", "hora de verano central"}, {"America_Eastern", "hora de verano oriental"}, {"America_Mountain", "hora de verano de las Montañas Rocosas"}, {"America_Pacific", "hora de verano del Pacífico"}, {"Anadyr", "hora de verano de Anadyr"}, {"Apia", "hora de verano de Apia"}, {"Aqtau", "Hora de verano de Aktau"}, {"Aqtobe", "Hora de verano de Aktobe"}, {"Arabian", "hora de verano de Arabia"}, {"Argentina", "hora de verano de Argentina"}, {"Argentina_Western", "hora de verano de Argentina occidental"}, {"Armenia", "hora de verano de Armenia"}, {"Atlantic", "hora de verano del Atlántico"}, {"Australia_Central", "hora de verano de Australia central"}, {"Australia_CentralWestern", "hora de verano de Australia centroccidental"}, {"Australia_Eastern", "hora de verano de Australia oriental"}, {"Australia_Western", "hora de verano de Australia occidental"}, {"Azerbaijan", "hora de verano de Azerbaiyán"}, {"Azores", "hora de verano de las Azores"}, {"Bangladesh", "hora de verano de Bangladés"}, {"Brasilia", "hora de verano de Brasilia"}, {"Cape_Verde", "hora de verano de Cabo Verde"}, {"Chatham", "hora de verano de Chatham"}, {"Chile", "hora de verano de Chile"}, {"China", "hora de verano de China"}, {"Colombia", "hora de verano de Colombia"}, {"Cook", "hora de verano media de las Islas Cook"}, {"Cuba", "hora de verano de Cuba"}, {"Easter", "hora de verano de la isla de Pascua"}, {"Europe_Central", "hora de verano de Europa central"}, {"Europe_Eastern", "hora de verano de Europa oriental"}, {"Europe_Western", "hora de verano de Europa occidental"}, {"Falkland", "hora de verano de las islas Malvinas"}, {"Fiji", "hora de verano de Fiyi"}, {"Georgia", "hora de verano de Georgia"}, {"Greenland_Eastern", "hora de verano de Groenlandia oriental"}, {"Greenland_Western", "hora de verano de Groenlandia occidental"}, {"Hawaii_Aleutian", "hora de verano de Hawái-Aleutianas"}, {"Hong_Kong", "hora de verano de Hong Kong"}, {"Hovd", "hora de verano de Hovd"}, {"Iran", "hora de verano de Irán"}, {"Irkutsk", "hora de verano de Irkutsk"}, {"Israel", "hora de verano de Israel"}, {"Japan", "hora de verano de Japón"}, {"Kamchatka", "hora de verano de Kamchatka"}, {"Korea", "hora de verano de Corea"}, {"Krasnoyarsk", "hora de verano de Krasnoyarsk"}, {"Lord_Howe", "hora de verano de Lord Howe"}, {"Macau", "Hora de verano de Macao"}, {"Magadan", "hora de verano de Magadán"}, {"Mauritius", "hora de verano de Mauricio"}, {"Mexico_Pacific", "hora de verano del Pacífico de México"}, {"Mongolia", "hora de verano de Ulán Bator"}, {"Moscow", "hora de verano de Moscú"}, {"New_Caledonia", "hora de verano de Nueva Caledonia"}, {"New_Zealand", "hora de verano de Nueva Zelanda"}, {"Newfoundland", "hora de verano de Terranova"}, {"Norfolk", "hora de verano de la isla Norfolk"}, {"Noronha", "hora de verano de Fernando de Noronha"}, {"Novosibirsk", "hora de verano de Novosibirsk"}, {"Omsk", "hora de verano de Omsk"}, {"Pakistan", "hora de verano de Pakistán"}, {"Paraguay", "hora de verano de Paraguay"}, {"Peru", "hora de verano de Perú"}, {"Philippines", "hora de verano de Filipinas"}, {"Pierre_Miquelon", "hora de verano de San Pedro y Miquelón"}, {"Qyzylorda", "Hora de verano de Qyzylorda"}, {"Sakhalin", "hora de verano de Sajalín"}, {"Samara", "hora de verano de Samara"}, {"Samoa", "hora de verano de Samoa"}, {"Taipei", "hora de verano de Taipéi"}, {"Tonga", "hora de verano de Tonga"}, {"Turkey", "Hora de verano de Turquía"}, {"Turkmenistan", "hora de verano de Turkmenistán"}, {"Uruguay", "hora de verano de Uruguay"}, {"Uzbekistan", "hora de verano de Uzbekistán"}, {"Vanuatu", "hora de verano de Vanuatu"}, {"Vladivostok", "hora de verano de Vladivostok"}, {"Volgograd", "hora de verano de Volgogrado"}, {"Yakutsk", "hora de verano de Yakutsk"}, {"Yekaterinburg", "hora de verano de Ekaterimburgo"}},
	},
	{
		{{"Acre", "Acre standardaeg"}, {"Afghanistan", "Afganistani aeg"}, {"Africa_Central", "Kesk-Aafrika aeg"}, {"Africa_Eastern", "Ida-Aafrika aeg"}, {"Africa_Southern", "Lõuna-Aafrika standardaeg"}, {"Africa_Western", "Lääne-Aafrika aeg"}, {"Alaska", "Alaska standardaeg"}, {"Almaty", "Almatõ standardaeg"}, {"Amazon", "Amazonase standardaeg"}, {"America_Central", "Kesk-Ameerika standardaeg"}, {"America_Eastern", "Idaranniku standardaeg"}, {"America_Mountain", "Mäestikuvööndi standardaeg"}, {"America_Pacific", "Vaikse ookeani standardaeg"}, {"Anadyr", "Anadõri standardaeg"}, {"Apia", "Apia standardaeg"}, {"Aqtau", "Aktau standardaeg"}, {"Aqtobe", "Aktöbe standardaeg"}, {"Arabian", "Araabia standardaeg"}, {"Argentina", "Argentina standardaeg"}, {"Argentina_Western", "Lääne-Argentina standardaeg"}, {"Armenia", "Armeenia standardaeg"}, {"Atlantic", "Atlandi standardaeg"}, {"Australia_Central", "Kesk-Austraalia standardaeg"}, {"Australia_CentralWestern", "Austraalia Kesk-Lääne standardaeg"}, {"Australia_Eastern", "Ida-Austraalia standardaeg"}, {"Australia_Western", "Lääne-Austraalia standardaeg"}, {"Azerbaijan", "Aserbaidžaani standardaeg"}, {"Azores", "Assooride standardaeg"}, {"Bangladesh", "Bangladeshi standardaeg"}, {"Bhutan", "Bhutani aeg"}, {"Bolivia", "Boliivia aeg"}, {"Brasilia", "Brasiilia standardaeg"}, {"Brunei", "Brunei aeg"}, {"Cape_Verde", "Roheneemesaarte standardaeg"}, {"Casey", "Casey aeg"}, {"Chamorro", "Tšamorro standardaeg"}, {"Chatham", "Chathami standardaeg"}, {"Chile", "Tšiili standardaeg"}, {"China", "Hiina standardaeg"}, {"Christmas", "Jõulusaare aeg"}, {"Cocos", "Kookossaarte aeg"}, {"Colombia", "Colombia standardaeg"}, {"Cook", "Cooki saarte standardaeg"}, {"Cuba", "Kuuba standardaeg"}, {"Davis", "Davise aeg"}, {"DumontDUrville", "Dumont d’Urville’i aeg"}, {"East_Timor", "Ida-Timori aeg"}, {"Easter", "Lihavõttesaare standardaeg"}, {"Ecuador", "Ecuadori aeg"}, {"Europe_Central", "Kesk-Euroopa standardaeg"}, {"Europe_Eastern", "Ida-Euroopa standardaeg"}, {"Europe_Further_Eastern", "Kaliningradi ja Valgevene aeg"}, {"Europe_Western", "Lääne-Euroopa standardaeg"}, {"Falkland", "Falklandi saarte standardaeg"}, {"Fiji", "Fidži standardaeg"}, {"French_Guiana", "Prantsuse Guajaana aeg"}, {"French_Southern", "Prantsuse Antarktiliste ja Lõunaalade aeg"}, {"GMT", "Greenwichi aeg"}, {"Galapagos", "Galapagose aeg"}, {"Gambier", "Gambier’ aeg"}, {"Georgia", "Gruusia standardaeg"}, {"Gilbert_Islands", "Gilberti saarte aeg"}, {"Greenland", "Gröönimaa standardaeg"}, {"Greenland_Eastern", "Ida-Gröönimaa standardaeg"}, {"Greenland_Western", "Lääne-Gröönimaa standardaeg"}, {"Guam", "Guami standardaeg"}, {"Gulf", "Pärsia lahe standardaeg"}, {"Guyana", "Guyana aeg"}, {"Hawaii", "Hawaii-Aleuudi standardaeg"}, {"Hawaii_Aleutian", "Hawaii-Aleuudi standardaeg"}, {"Hong_Kong", "Hongkongi standardaeg"}, {"Hovd", "Hovdi standardaeg"}, {"India", "India aeg"}, {"Indian_Ocean", "India ookeani aeg"}, {"Indochina", "Indohiina aeg"}, {"Indonesia_Central", "Kesk-Indoneesia aeg"}, {"Indonesia_Eastern", "Ida-Indoneesia aeg"}, {"Indonesia_Western", "Lääne-Indoneesia aeg"}, {"Iran", "Iraani standardaeg"}, {"Irkutsk", "Irkutski standardaeg"}, {"Israel", "Iisraeli standardaeg"}, {"Japan", "Jaapani standardaeg"}, {"Kamchatka", "Kamtšatka standardaeg"}, {"Kazakhstan", "Kasahstani aeg"}, {"Kazakhstan_Eastern", "Ida-Kasahstani aeg"}, {"Kazakhstan_Western", "Lääne-Kasahstani aeg"}, {"Korea", "Korea standardaeg"}, {"Kosrae", "Kosrae aeg"}, {"Krasnoyarsk", "Krasnojarski standardaeg"}, {"Kyrgystan", "Kõrgõzstani aeg"}, {"Lanka", "Sri Lanka aeg"}, {"Line_Islands", "Line’i saarte aeg"}, {"Lord_Howe", "Lord Howe’i standardaeg"}, {"Macau", "Macau standardaeg"}, {"Magadan", "Magadani standardaeg"}, {"Malaysia", "Malaisia aeg"}, {"Maldives", "Maldiivi aeg"}, {"Marquesas", "Markiisaarte aeg"}, {"Marshall_Islands", "Marshalli Saarte aeg"}, {"Mauritius", "Mauritiuse standardaeg"}, {"Mawson", "Mawsoni aeg"}, {"Mexico_Pacific", "Mehhiko Vaikse ookeani standardaeg"}, {"Mongolia", "Ulaanbaatari standardaeg"}, {"Moscow", "Moskva standardaeg"}, {"Myanmar", "Birma aeg"}, {"Nauru", "Nauru aeg"}, {"Nepal", "Nepali aeg"}, {"New_Caledonia", "Uus-Kaledoonia standardaeg"}, {"New_Zealand", "Uus-Meremaa standardaeg"}, {"Newfoundland", "Newfoundlandi standardaeg"}, {"Niue", "Niue aeg"}, {"Norfolk", "Norfolki saare standardaeg"}, {"Noronha", "Fernando de Noronha standardaeg"}, {"North_Mariana", "Põhja-Mariaani aeg"}, {"Novosibirsk", "Novosibirski standardaeg"}, {"Omsk", "Omski standardaeg"}, {"Pakistan", "Pakistani standardaeg"}, {"Palau", "Belau aeg"}, {"Papua_New_Guinea", "Paapua Uus-Guinea aeg"}, {"Paraguay", "Paraguay standardaeg"}, {"Peru", "Peruu standardaeg"}, {"Philippines", "Filipiini standardaeg"}, {"Phoenix_Islands", "Fööniksisaarte aeg"}, {"Pierre_Miquelon", "Saint-Pierre’i ja Miqueloni standardaeg"}, {"Pitcairn", "Pitcairni aeg"}, {"Ponape", "Pohnpei aeg"}, {"Pyongyang", "Pyongyangi aeg"}, {"Qyzylorda", "Kõzõlorda standardaeg"}, {"Reunion", "Réunioni aeg"}, {"Rothera", "Rothera aeg"}, {"Sakhalin", "Sahhalini standardaeg"}, {"Samara", "Samara standardaeg"}, {"Samoa", "Samoa standardaeg"}, {"Seychelles", "Seišelli aeg"}, {"Singapore", "Singapuri standardaeg"}, {"Solomon", "Saalomoni Saarte aeg"}, {"South_Georgia", "Lõuna-Georgia aeg"}, {"Suriname", "Suriname aeg"}, {"Syowa", "Syowa aeg"}, {"Tahiti", "Tahiti aeg"}, {"Taipei", "Taipei standardaeg"}, {"Tajikistan", "Tadžikistani aeg"}, {"Tokelau", "Tokelau aeg"}, {"Tonga", "Tonga standardaeg"}, {"Truk", "Chuuki aeg"}, {"Turkey", "Türgi standardaeg"}, {"Turkmenistan", "Türkmenistani standardaeg"}, {"Tuvalu", "Tuvalu aeg"}, {"Uruguay", "Uruguay standardaeg"}, {"Uzbekistan", "Usbekistani standardaeg"}, {"Vanuatu", "Vanuatu standardaeg"}, {"Venezuela", "Venezuela aeg"}, {"Vladivostok", "Vladivostoki standardaeg"}, {"Volgograd", "Volgogradi standardaeg"}, {"Vostok", "Vostoki aeg"}, {"Wake", "Wake’i aeg"}, {"Wallis", "Wallise ja Futuna aeg"}, {"Yakutsk", "Jakutski standardaeg"}, {"Yekaterinburg", "Jekaterinburgi standardaeg"}, {"Yukon", "Yukoni aeg"}},
		{{"Acre", "Acre suveaeg"}, {"Alaska", "Alaska suveaeg"}, {"Almaty", "Almatõ suveaeg"}, {"Amazon", "Amazonase suveaeg"}, {"America_Central", "Kesk-Ameerika suveaeg"}, {"America_Eastern", "Idaranniku suveaeg"}, {"America_Mountain", "Mäestikuvööndi suveaeg"}, {"America_Pacific", "Vaikse ookeani suveaeg"}, {"Anadyr", "Anadõri suveaeg"}, {"Apia", "Apia suveaeg"}, {"Aqtau", "Aktau suveaeg"}, {"Aqtobe", "Aktöbe suveaeg"}, {"Arabian", "Araabia suveaeg"}, {"Argentina", "Argentina suveaeg"}, {"Argentina_Western", "Lääne-Argentina suveaeg"}, {"Armenia", "Armeenia suveaeg"}, {"Atlantic", "Atlandi suveaeg"}, {"Australia_Central", "Kesk-Austraalia suveaeg"}, {"Australia_CentralWestern", "Austraalia Kesk-Lääne suveaeg"}, {"Australia_Eastern", "Ida-Austraalia suveaeg"}, {"Australia_Western", "Lääne-Austraalia suveaeg"}, {"Azerbaijan", "Aserbaidžaani suveaeg"}, {"Azores", "Assooride suveaeg"}, {"Bangladesh", "Bangladeshi suveaeg"}, {"Brasilia", "Brasiilia suveaeg"}, {"Cape_Verde", "Roheneemesaarte suveaeg"}, {"Chatham", "Chathami suveaeg"}, {"Chile", "Tšiili suveaeg"}, {"China", "Hiina suveaeg"}, {"Colombia", "Colombia suveaeg"}, {"Cook", "Cooki saarte osaline suveaeg"}, {"Cuba", "Kuuba suveaeg"}, {"Easter", "Lihavõttesaare suveaeg"}, {"Europe_Central", "Kesk-Euroopa suveaeg"}, {"Europe_Eastern", "Ida-Euroopa suveaeg"}, {"Europe_Western", "Lääne-Euroopa suveaeg"}, {"Falkland", "Falklandi saarte suveaeg"}, {"Fiji", "Fidži suveaeg"}, {"Georgia", "Gruusia suveaeg"}, {"Greenland", "Gröönimaa suveaeg"}, {"Greenland_Eastern", "Ida-Gröönimaa suveaeg"}, {"Greenland_Western", "Lääne-Gröönimaa suveaeg"}, {"Hawaii_Aleutian", "Hawaii-Aleuudi suveaeg"}, {"Hong_Kong", "Hongkongi suveaeg"}, {"Hovd", "Hovdi suveaeg"}, {"Iran", "Iraani suveaeg"}, {"Irkutsk", "Irkutski suveaeg"}, {"Israel", "Iisraeli suveaeg"}, {"Japan", "Jaapani suveaeg"}, {"Kamchatka", "Kamtšatka suveaeg"}, {"Korea", "Korea suveaeg"}, {"Krasnoyarsk", "Krasnojarski suveaeg"}, {"Lord_Howe", "Lord Howe’i suveaeg"}, {"Macau", "Macau suveaeg"}, {"Magadan", "Magadani suveaeg"}, {"Mauritius", "Mauritiuse suveaeg"}, {"Mexico_Pacific", "Mehhiko Vaikse ookeani suveaeg"}, {"Mongolia", "Ulaanbaatari suveaeg"}, {"Moscow", "Moskva suveaeg"}, {"New_Caledonia", "Uus-Kaledoonia suveaeg"}, {"New_Zealand", "Uus-Meremaa suveaeg"}, {"Newfoundland", "Newfoundlandi suveaeg"}, {"Norfolk", "Norfolki saare suveaeg"}, {"Noronha", "Fernando de Noronha suveaeg"}, {"Novosibirsk", "Novosibirski suveaeg"}, {"Omsk", "Omski suveaeg"}, {"Pakistan", "Pakistani suveaeg"}, {"Paraguay", "Paraguay suveaeg"}, {"Peru", "Peruu suveaeg"}, {"Philippines", "Filipiini suveaeg"}, {"Pierre_Miquelon", "Saint-Pierre’i ja Miqueloni suveaeg"}, {"Qyzylorda", "Kõzõlorda suveaeg"}, {"Sakhalin", "Sahhalini suveaeg"}, {"Samara", "Samara suveaeg"}, {"Samoa", "Samoa suveaeg"}, {"Taipei", "Taipei suveaeg"}, {"Tonga", "Tonga suveaeg"}, {"Turkey", "Türgi suveaeg"}, {"Turkmenistan", "Türkmenistani suveaeg"}, {"Uruguay", "Uruguay suveaeg"}, {"Uzbekistan", "Usbekistani suveaeg"}, {"Vanuatu", "Vanuatu suveaeg"}, {"Vladivostok", "Vladivostoki suveaeg"}, {"Volgograd", "Volgogradi suveaeg"}, {"Yakutsk", "Jakutski suveaeg"}, {"Yekaterinburg", "Jekaterinburgi suveaeg"}},
	},
	{
		{{"Acre", "Acreko ordu estandarra"}, {"Afghanistan", "Afganistango ordua"}, {"Africa_Central", "Afrikako erdialdeko ordua"}, {"Africa_Eastern", "Afrikako ekialdeko ordua"}, {"Africa_Southern", "Afrikako hegoaldeko ordua"}, {"Africa_Western", "Afrikako mendebaldeko ordua"}, {"Alaska", "Alaskako ordu estandarra"}, {"Almaty", "Almatyko ordu estandarra"}, {"Amazon", "Amazoniako ordu estandarra"}, {"America_Central", "Ipar Amerikako erdialdeko ordu estandarra"}, {"America_Eastern", "Ipar Amerikako ekialdeko ordu estandarra"}, {"America_Mountain", "Ipar Amerikako mendialdeko ordu estandarra"}, {"America_Pacific", "Ipar Amerikako Pazifikoko ordu estandarra"}, {"Anadyr", "Anadyrreko ordu estandarra"}, {"Apia", "Apiako ordu estandarra"}, {"Aqtau", "Aktauko ordu estandarra"}, {"Aqtobe", "Aktobeko ordu estandarra"}, {"Arabian", "Arabiako ordu estandarra"}, {"Argentina", "Argentinako ordu estandarra"}, {"Argentina_Western", "Argentina mendebaldeko ordu estandarra"}, {"Armenia", "Armeniako ordu estandarra"}, {"Atlantic", "Ipar Amerikako Atlantikoko ordu estandarra"}, {"Australia_Central", "Australiako erdialdeko ordu estandarra"}, {"Australia_CentralWestern", "Australiako erdi-mendebaldeko ordu estandarra"}, {"Australia_Eastern", "Australiako ekialdeko ordu estandarra"}, {"Australia_Western", "Australiako mendebaldeko ordu estandarra"}, {"Azerbaijan", "Azerbaijango ordu estandarra"}, {"Azores", "Azoreetako ordu estandarra"}, {"Bangladesh", "Bangladesheko ordu estandarra"}, {"Bhutan", "Bhutango ordua"}, {"Bolivia", "Boliviako ordua"}, {"Brasilia", "Brasiliako ordu estandarra"}, {"Brunei", "Brunei Darussalamgo ordua"}, {"Cape_Verde", "Cabo Verdeko ordu estandarra"}, {"Casey", "Caseyko ordua"}, {"Chamorro", "Chamorroko ordu estandarra"}, {"Chatham", "Chathamgo ordu estandarra"}, {"Chile", "Txileko ordu estandarra"}, {"China", "Txinako ordu estandarra"}, {"Christmas", "Christmas uharteko ordua"}, {"Cocos", "Cocos uharteetako ordua"}, {"Colombia", "Kolonbiako ordu estandarra"}, {"Cook", "Cook uharteetako ordu estandarra"}, {"Cuba", "Kubako ordu estandarra"}, {"Davis", "Daviseko ordua"}, {"DumontDUrville", "Dumont-d’Urvilleko ordua"}, {"East_Timor", "Ekialdeko Timorreko ordua"}, {"Easter", "Pazko uharteko ordu estandarra"}, {"Ecuador", "Ekuadorreko ordua"}, {"Europe_Central", "Europako erdialdeko ordu estandarra"}, {"Europe_Eastern", "Europako ekialdeko ordu estandarra"}, {"Europe_Further_Eastern", "Europako ekialde urruneko ordua"}, {"Europe_Western", "Europako mendebaldeko ordu estandarra"}, {"Falkland", "Falkland uharteetako ordu estandarra"}, {"Fiji", "Fijiko ordu estandarra"}, {"French_Guiana", "Guyana Frantseseko ordua"}, {"French_Southern", "Frantziaren lurralde austral eta antartikoetako ordutegia"}, {"GMT", "Greenwichko meridianoaren ordua"}, {"Galapagos", "Galapagoetako ordua"}, {"Gambier", "Gambierretako ordua"}, {"Georgia", "Georgiako ordu estandarra"}, {"Gilbert_Islands", "Gilbert uharteetako ordua"}, {"Greenland_Eastern", "Groenlandiako ekialdeko ordu estandarra"}, {"Greenland_Western", "Groenlandiako mendebaldeko ordu estandarra"}, {"Guam", "Guameko ordu estandarra"}, {"Gulf", "Golkoko ordu estandarra"}, {"Guyana", "Guyanako ordua"}, {"Hawaii", "Hawaii-Aleutiar uharteetako ordu estandarra"}, {"Hawaii_Aleutian", "Hawaii-Aleutiar uharteetako ordu estandarra"}, {"Hong_Kong", "Hong Kongo ordu estandarra"}, {"Hovd", "Khovdeko ordu estandarra"}, {"India", "Indiako ordua"}, {"Indian_Ocean", "Indiako ozeanoko ordua"}, {"Indochina", "Indotxinako ordua"}, {"Indonesia_Central", "Indonesiako erdialdeko ordua"}, {"Indonesia_Eastern", "Indonesiako ekialdeko ordua"}, {"Indonesia_Western", "Indonesiako mendebaldeko ordua"}, {"Iran", "Irango ordu estandarra"}, {"Irkutsk", "Irkutskeko ordu estandarra"}, {"Israel", "Israelgo ordu estandarra"}, {"Japan", "Japoniako ordu estandarra"}, {"Kamchatka", "Petropavlovsk-Kamchatskiko ordu estandarra"}, {"Kazakhstan", "Kazakhstango ordua"}, {"Kazakhstan_Eastern", "Kazakhstango ekialdeko ordua"}, {"Kazakhstan_Western", "Kazakhstango mendebaldeko ordua"}, {"Korea", "Koreako ordu estandarra"}, {"Kosrae", "Kosraeko ordua"}, {"Krasnoyarsk", "Krasnoiarskeko ordu estandarra"}, {"Kyrgystan", "Kirgizistango ordua"}, {"Lanka", "Lankako ordua"}, {"Line_Islands", "Line uharteetako ordua"}, {"Lord_Howe", "Lord Howeko ordu estandarra"}, {"Macau", "Macaoko ordu estandarra"}, {"Magadan", "Magadango ordu estandarra"}, {"Malaysia", "Malaysiako ordua"}, {"Maldives", "Maldivetako ordua"}, {"Marquesas", "Markesetako ordua"}, {"Marshall_Islands", "Marshall Uharteetako ordua"}, {"Mauritius", "Maurizioko ordu estandarra"}, {"Mawson", "Mawsoneko ordua"}, {"Mexico_Pacific", "Mexikoko Pazifikoko ordu estandarra"}, {"Mongolia", "Ulan Batorreko ordu estandarra"}, {"Moscow", "Moskuko ordu estandarra"}, {"Myanmar", "Myanmarreko ordua"}, {"Nauru", "Nauruko ordua"}, {"Nepal", "Nepalgo ordua"}, {"New_Caledonia", "Kaledonia Berriko ordu estandarra"}, {"New_Zealand", "Zeelanda Berriko ordu estandarra"}, {"Newfoundland", "Ternuako ordu estandarra"}, {"Niue", "Niueko ordua"}, {"Norfolk", "Norfolk uharteetako ordu estandarra"}, {"Noronha", "Fernando de Noronhako ordu estandarra"}, {"North_Mariana", "Ipar Mariana uharteetako ordua"}, {"Novosibirsk", "Novosibirskeko ordu estandarra"}, {"Omsk", "Omskeko ordu estandarra"}, {"Pakistan", "Pakistango ordu estandarra"}, {"Palau", "Palauko ordua"}, {"Papua_New_Guinea", "Papua Ginea Berriko ordua"}, {"Paraguay", "Paraguaiko ordu estandarra"}, {"Peru", "Peruko ordu estandarra"}, {"Philippines", "Filipinetako ordu estandarra"}, {"Phoenix_Islands", "Phoenix uharteetako ordua"}, {"Pierre_Miquelon", "Saint-Pierre eta Mikeluneko ordu estandarra"}, {"Pitcairn", "Pitcairneko ordua"}, {"Ponape", "Ponapeko ordua"}, {"Pyongyang", "Piongiangeko ordua"}, {"Qyzylorda", "Kyzylordako ordu estandarra"}, {"Reunion", "Reunioneko ordua"}, {"Rothera", "Rotherako ordua"}, {"Sakhalin", "Sakhalingo ordu estandarra"}, {"Samara", "Samarako ordu estandarra"}, {"Samoa", "Samoako ordu estandarra"}, {"Seychelles", "Seychelle uharteetako ordua"}, {"Singapore", "Singapurreko ordu estandarra"}, {"Solomon", "Salomon Uharteetako ordua"}, {"South_Georgia", "Hegoaldeko Georgietako ordua"}, {"Suriname", "Surinamgo ordua"}, {"Syowa", "Syowako ordua"}, {"Tahiti", "Tahitiko ordua"}, {"Taipei", "Taipeiko ordu estandarra"}, {"Tajikistan", "Tadjikistango ordua"}, {"Tokelau", "Tokelauko ordua"}, {"Tonga", "Tongako ordu estandarra"}, {"Truk", "Chuukeko ordua"}, {"Turkmenistan", "Turkmenistango ordu estandarra"}, {"Tuvalu", "Tuvaluko ordua"}, {"Uruguay", "Uruguaiko ordu estandarra"}, {"Uzbekistan", "Uzbekistango ordu estandarra"}, {"Vanuatu", "Vanuatuko ordu estandarra"}, {"Venezuela", "Venezuelako ordua"}, {"Vladivostok", "Vladivostokeko ordu estandarra"}, {"Volgograd", "Volgogradeko ordu estandarra"}, {"Vostok", "Vostokeko ordua"}, {"Wake", "Wake uharteko ordua"}, {"Wallis", "Wallis eta Futunako ordutegia"}, {"Yakutsk", "Jakutskeko ordu estandarra"}, {"Yekaterinburg", "Jekaterinburgeko ordu estandarra"}, {"Yukon", "Yukongo ordua"}},
		{{"Acre", "Acreko udako ordua"}, {"Alaska", "Alaskako udako ordua"}, {"Almaty", "Almatyko udako ordua"}, {"Amazon", "Amazoniako udako ordua"}, {"America_Central", "Ipar Amerikako erdialdeko udako ordua"}, {"America_Eastern", "Ipar Amerikako ekialdeko udako ordua"}, {"America_Mountain", "Ipar Amerikako mendialdeko udako ordua"}, {"America_Pacific", "Ipar Amerikako Pazifikoko udako ordua"}, {"Anadyr", "Anadyrreko udako ordua"}, {"Apia", "Apiako udako ordua"}, {"Aqtau", "Aktauko udako ordua"}, {"Aqtobe", "Aktobeko udako ordua"}, {"Arabian", "Arabiako udako ordua"}, {"Argentina", "Argentinako udako ordua"}, {"Argentina_Western", "Argentina mendebaldeko udako ordua"}, {"Armenia", "Armeniako udako ordua"}, {"Atlantic", "Ipar Amerikako Atlantikoko udako ordua"}, {"Australia_Central", "Australiako erdialdeko udako ordua"}, {"Australia_CentralWestern", "Australiako erdi-mendebaldeko udako ordua"}, {"Australia_Eastern", "Australiako ekialdeko udako ordua"}, {"Australia_Western", "Australiako mendebaldeko udako ordua"}, {"Azerbaijan", "Azerbaijango udako ordua"}, {"Azores", "Azoreetako udako ordua"}, {"Bangladesh", "Bangladesheko udako ordua"}, {"Brasilia", "Brasiliako udako ordua"}, {"Cape_Verde", "Cabo Verdeko udako ordua"}, {"Chatham", "Chathamgo udako ordua"}, {"Chile", "Txileko udako ordua"}, {"China", "Txinako udako ordua"}, {"Colombia", "Kolonbiako udako ordua"}, {"Cook", "Cook uharteetako uda erdialdeko ordua"}, {"Cuba", "Kubako udako ordua"}, {"Easter", "Pazko uharteko udako ordua"}, {"Europe_Central", "Europako erdialdeko udako ordua"}, {"Europe_Eastern", "Europako ekialdeko udako ordua"}, {"Europe_Western", "Europako mendebaldeko udako ordua"}, {"Falkland", "Falkland uharteetako udako ordua"}, {"Fiji", "Fijiko udako ordua"}, {"Georgia", "Georgiako udako ordua"}, {"Greenland_Eastern", "Groenlandiako ekialdeko udako ordua"}, {"Greenland_Western", "Groenlandiako mendebaldeko udako ordua"}, {"Hawaii_Aleutian", "Hawaii-Aleutiar uharteetako udako ordua"}, {"Hong_Kong", "Hong Kongo udako ordua"}, {"Hovd", "Khovdeko udako ordua"}, {"Iran", "Irango udako ordua"}, {"Irkutsk", "Irkutskeko udako ordua"}, {"Israel", "Israelgo udako ordua"}, {"Japan", "Japoniako udako ordua"}, {"Kamchatka", "Petropavlovsk-Kamchatskiko udako ordua"}, {"Korea", "Koreako udako ordua"}, {"Krasnoyarsk", "Krasnoiarskeko udako ordua"}, {"Lord_Howe", "Lord Howeko udako ordua"}, {"Macau", "Macaoko udako ordua"}, {"Magadan", "Magadango udako ordua"}, {"Mauritius", "Maurizioko udako ordua"}, {"Mexico_Pacific", "Mexikoko Pazifikoko udako ordua"}, {"Mongolia", "Ulan Batorreko udako ordua"}, {"Moscow", "Moskuko udako ordua"}, {"New_Caledonia", "Kaledonia Berriko udako ordua"}, {"New_Zealand", "Zeelanda Berriko udako ordua"}, {"Newfoundland", "Ternuako udako ordua"}, {"Norfolk", "Norfolk uharteetako udako ordua"}, {"Noronha", "Fernando de Noronhako udako ordua"}, {"Novosibirsk", "Novosibirskeko udako ordua"}, {"Omsk", "Omskeko udako ordua"}, {"Pakistan", "Pakistango udako ordua"}, {"Paraguay", "Paraguaiko udako ordua"}, {"Peru", "Peruko udako ordua"}, {"Philippines", "Filipinetako udako ordua"}, {"Pierre_Miquelon", "Saint-Pierre eta Mikeluneko udako ordua"}, {"Qyzylorda", "Kyzylordako udako ordua"}, {"Sakhalin", "Sakhalingo udako ordua"}, {"Samara", "Samarako udako ordua"}, {"Samoa", "Samoako udako ordua"}, {"Taipei", "Taipeiko udako ordua"}, {"Tonga", "Tongako udako ordua"}, {"Turkmenistan", "Turkmenistango udako ordua"}, {"Uruguay", "Uruguaiko udako ordua"}, {"Uzbekistan", "Uzbekistango udako ordua"}, {"Vanuatu", "Vanuatuko udako ordua"}, {"Vladivostok", "Vladivostokeko udako ordua"}, {"Volgograd", "Volgogradeko udako ordua"}, {"Yakutsk", "Jakutskeko udako ordua"}, {"Yekaterinburg", "Jekaterinburgeko udako ordua"}},
	},
	{
		{{"Afghanistan", "وقت افغانستان"}, {"Africa_Central", "وقت مرکز آفریقا"}, {"Africa_Eastern", "وقت شرق افریقا"}, {"Africa_Southern", "وقت عادی جنوب افریقا"}, {"Africa_Western", "وقت غرب افریقا"}, {"Alaska", "وقت عادی آلاسکا"}, {"Almaty", "وقت عادی آلماآتا"}, {"Amazon", "وقت عادی آمازون"}, {"America_Central", "وقت عادی مرکز امریکا"}, {"America_Eastern", "وقت عادی شرق امریکا"}, {"America_Mountain", "وقت عادی کوهستانی امریکا"}, {"America_Pacific", "وقت عادی غرب امریکا"}, {"Anadyr", "وقت عادی آنادیر"}, {"Apia", "وقت عادی آپیا"}, {"Arabian", "وقت عادی عربستان"}, {"Argentina", "وقت عادی آرژانتین"}, {"Argentina_Western", "وقت عادی غرب آرژانتین"}, {"Armenia", "وقت عادی ارمنستان"}, {"Atlantic", "وقت عادی آتلانتیک"}, {"Australia_Central", "وقت عادی مرکز استرالیا"}, {"Australia_CentralWestern", "وقت عادی مرکز استرالیای غربی"}, {"Australia_Eastern", "وقت عادی استرالیای شرقی"}, {"Australia_Western", "وقت عادی استرالیای غربی"}, {"Azerbaijan", "وقت عادی جمهوری آذربایجان"}, {"Azores", "وقت عادی آزور"}, {"Bangladesh", "وقت عادی بنگلادش"}, {"Bhutan", "وقت بوتان"}, {"Bolivia", "وقت بولیوی"}, {"Brasilia", "وقت عادی برازیلیا"}, {"Brunei", "وقت برونئی دارالسلام"}, {"Cape_Verde", "وقت عادی کیپ\u200cورد"}, {"Chamorro", "وقت عادی چامورو"}, {"Chatham", "وقت عادی چت\u200cهام"}, {"Chile", "وقت عادی شیلی"}, {"China", "وقت عادی چین"}, {"Christmas", "وقت جزیرهٔ کریسمس"}, {"Cocos", "وقت جزایر کوکوس"}, {"Colombia", "وقت عادی کلمبیا"}, {"Cook", "وقت عادی جزایر کوک"}, {"Cuba", "وقت عادی کوبا"}, {"Davis", "وقت دیویس"}, {"DumontDUrville", "وقت دومون دورویل"}, {"East_Timor", "وقت تیمور شرقی"}, {"Easter", "وقت عادی جزیرهٔ ایستر"}, {"Ecuador", "وقت اکوادور"}, {"Europe_Central", "وقت عادی مرکز اروپا"}, {"Europe_Eastern", "وقت عادی شرق اروپا"}, {"Europe_Further_Eastern", "وقت تابستانی مکان\u200cهای دیگر شرق اروپا"}, {"Europe_Western", "وقت عادی غرب اروپا"}, {"Falkland", "وقت عادی جزایر فالکلند"}, {"Fiji", "وقت عادی فیجی"}, {"French_Guiana", "وقت گویان فرانسه"}, {"French_Southern", "وقت سرزمین\u200cهای جنوبی و جنوبگان فرانسه"}, {"GMT", "وقت گرینویچ"}, {"Galapagos", "وقت گالاپاگوس"}, {"Gambier", "وقت گامبیه"}, {"Georgia", "وقت عادی گرجستان"}, {"Gilbert_Islands", "وقت جزایر گیلبرت"}, {"Greenland_Eastern", "وقت عادی شرق گرینلند"}, {"Greenland_Western", "وقت عادی غرب گرینلند"}, {"Guam", "وقت عادی گوام"}, {"Gulf", "وقت عادی خلیج فارس"}, {"Guyana", "وقت گویان"}, {"Hawaii", "وقت عادی هاوایی‐الوشن"}, {"Hawaii_Aleutian", "وقت عادی هاوایی‐الوشن"}, {"Hong_Kong", "وقت عادی هنگ\u200cکنگ"}, {"Hovd", "وقت عادی خوود"}, {"India", "وقت هند"}, {"Indian_Ocean", "وقت اقیانوس هند"}, {"Indochina", "وقت هندوچین"}, {"Indonesia_Central", "وقت مرکز اندونزی"}, {"Indonesia_Eastern", "وقت شرق اندونزی"}, {"Indonesia_Western", "وقت غرب اندونزی"}, {"Iran", "وقت عادی ایران"}, {"Irkutsk", "وقت عادی ایرکوتسک"}, {"Israel", "وقت عادی اسرائیل"}, {"Japan", "وقت عادی ژاپن"}, {"Kamchatka", "وقت عادی پتروپاولوسک‐کامچاتسکی"}, {"Kazakhstan", "وقت قزاقستان"}, {"Kazakhstan_Eastern", "وقت شرق قزاقستان"}, {"Kazakhstan_Western", "وقت غرب قزاقستان"}, {"Korea", "وقت عادی کره"}, {"Kosrae", "وقت کوسرای"}, {"Krasnoyarsk", "وقت عادی کراسنویارسک"}, {"Kyrgystan", "وقت قرقیزستان"}, {"Lanka", "وقت لانکا"}, {"Line_Islands", "وقت جزایر لاین"}, {"Lord_Howe", "وقت عادی لردهو"}, {"Macau", "وقت عادی ماکائو"}, {"Magadan", "وقت عادی ماگادان"}, {"Malaysia", "وقت مالزی"}, {"Maldives", "وقت مالدیو"}, {"Marquesas", "وقت مارکوئز"}, {"Marshall_Islands", "وقت جزایر مارشال"}, {"Mauritius", "وقت عادی موریس"}, {"Mawson", "وقت ماوسون"}, {"Mexico_Pacific", "وقت عادی شرق مکزیک"}, {"Mongolia", "وقت عادی اولان\u200cباتور"}, {"Moscow", "وقت عادی مسکو"}, {"Myanmar", "وقت میانمار"}, {"Nauru", "وقت نائورو"}, {"Nepal", "وقت نپال"}, {"New_Caledonia", "وقت عادی کالدونیای جدید"}, {"New_Zealand", "وقت عادی نیوزیلند"}, {"Newfoundland", "وقت عادی نیوفاندلند"}, {"Niue", "وقت نیوئه"}, {"Norfolk", "وقت عادی جزیرهٔ نورفولک"}, {"Noronha", "وقت عادی فرناندو دی نورونیا"}, {"North_Mariana", "وقت جزایر ماریانای شمالی"}, {"Novosibirsk", "وقت عادی نووسیبیرسک"}, {"Omsk", "وقت عادی اومسک"}, {"Pakistan", "وقت عادی پاکستان"}, {"Palau", "وقت پالائو"}, {"Papua_New_Guinea", "وقت پاپوا گینهٔ نو"}, {"Paraguay", "وقت عادی پاراگوئه"}, {"Peru", "وقت عادی پرو"}, {"Philippines", "وقت عادی فیلیپین"}, {"Phoenix_Islands", "وقت جزایر فونیکس"}, {"Pierre_Miquelon", "وقت عادی سنت\u200cپیر و میکلون"}, {"Pitcairn", "وقت پیتکایرن"}, {"Ponape", "وقت پوناپه"}, {"Pyongyang", "وقت پیونگ\u200cیانگ"}, {"Qyzylorda", "وقت عادی قیزیل\u200cاوردا"}, {"Reunion", "وقت رئونیون"}, {"Rothera", "وقت روترا"}, {"Sakhalin", "وقت عادی ساخالین"}, {"Samara", "وقت عادی سامارا"}, {"Samoa", "وقت عادی ساموا"}, {"Seychelles", "وقت سیشل"}, {"Singapore", "وقت سنگاپور"}, {"Solomon", "وقت جزایر سلیمان"}, {"South_Georgia", "وقت جورجیای جنوبی"}, {"Suriname", "وقت سورینام"}, {"Syowa", "وقت شووا"}, {"Tahiti", "وقت تاهیتی"}, {"Taipei", "وقت عادی تایپه"}, {"Tajikistan", "وقت تاجیکستان"}, {"Tokelau", "وقت توکلائو"}, {"Tonga", "وقت عادی تونگا"}, {"Truk", "وقت چوئوک"}, {"Turkey", "وقت عادی ترکیه"}, {"Turkmenistan", "وقت عادی ترکمنستان"}, {"Tuvalu", "وقت تووالو"}, {"Uruguay", "وقت عادی اروگوئه"}, {"Uzbekistan", "وقت عادی ازبکستان"}, {"Vanuatu", "وقت عادی واناتو"}, {"Venezuela", "وقت ونزوئلا"}, {"Vladivostok", "وقت عادی ولادی\u200cوستوک"}, {"Volgograd", "وقت عادی ولگاگراد"}, {"Vostok", "وقت وستوک"}, {"Wake", "وقت جزیرهٔ ویک"}, {"Wallis", "وقت والیس و فوتونا"}, {"Yakutsk", "وقت عادی یاکوتسک"}, {"Yekaterinburg", "وقت عادی یکاترینبورگ"}, {"Yukon", "وقت یوکان"}},
		{{"Alaska", "وقت تابستانی آلاسکا"}, {"Almaty", "وقت تابستانی آلماآتا"}, {"Amazon", "وقت تابستانی آمازون"}, {"America_Central", "وقت تابستانی مرکز امریکا"}, {"America_Eastern", "وقت تابستانی شرق امریکا"}, {"America_Mountain", "وقت تابستانی کوهستانی امریکا"}, {"America_Pacific", "وقت تابستانی غرب امریکا"}, {"Anadyr", "وقت تابستانی آنادیر"}, {"Apia", "وقت تابستانی آپیا"}, {"Arabian", "وقت تابستانی عربستان"}, {"Argentina", "وقت تابستانی آرژانتین"}, {"Argentina_Western", "وقت تابستانی غرب آرژانتین"}, {"Armenia", "وقت تابستانی ارمنستان"}, {"Atlantic", "وقت تابستانی آتلانتیک"}, {"Australia_Central", "وقت تابستانی مرکز استرالیا"}, {"Australia_CentralWestern", "وقت تابستانی مرکز استرالیای غربی"}, {"Australia_Eastern", "وقت تابستانی استرالیای شرقی"}, {"Australia_Western", "وقت تابستانی استرالیای غربی"}, {"Azerbaijan", "وقت تابستانی جمهوری آذربایجان"}, {"Azores", "وقت تابستانی آزور"}, {"Bangladesh", "وقت تابستانی بنگلادش"}, {"Brasilia", "وقت تابستانی برازیلیا"}, {"Cape_Verde", "وقت تابستانی کیپ\u200cورد"}, {"Chatham", "وقت تابستانی چت\u200cهام"}, {"Chile", "وقت تابستانی شیلی"}, {"China", "وقت تابستانی چین"}, {"Colombia", "وقت تابستانی کلمبیا"}, {"Cook", "وقت تابستانی جزایر کوک"}, {"Cuba", "وقت تابستانی کوبا"}, {"Easter", "وقت تابستانی جزیرهٔ ایستر"}, {"Europe_Central", "وقت تابستانی مرکز اروپا"}, {"Europe_Eastern", "وقت تابستانی شرق اروپا"}, {"Europe_Western", "وقت تابستانی غرب اروپا"}, {"Falkland", "وقت تابستانی جزایر فالکلند"}, {"Fiji", "وقت تابستانی فیجی"}, {"Georgia", "وقت تابستانی گرجستان"}, {"Greenland_Eastern", "وقت تابستانی شرق گرینلند"}, {"Greenland_Western", "وقت تابستانی غرب گرینلند"}, {"Hawaii_Aleutian", "وقت تابستانی هاوایی‐الوشن"}, {"Hong_Kong", "وقت تابستانی هنگ\u200cکنگ"}, {"Hovd", "وقت تابستانی خوود"}, {"Iran", "وقت تابستانی ایران"}, {"Irkutsk", "وقت تابستانی ایرکوتسک"}, {"Israel", "وقت تابستانی اسرائیل"}, {"Japan", "وقت تابستانی ژاپن"}, {"Kamchatka", "وقت تابستانی پتروپاولوسک‐کامچاتسکی"}, {"Korea", "وقت تابستانی کره"}, {"Krasnoyarsk", "وقت تابستانی کراسنویارسک"}, {"Lord_Howe", "وقت تابستانی لردهو"}, {"Macau", "وقت تابستانی ماکائو"}, {"Magadan", "وقت تابستانی ماگادان"}, {"Mauritius", "وقت تابستانی موریس"}, {"Mexico_Pacific", "وقت تابستانی شرق مکزیک"}, {"Mongolia", "وقت تابستانی اولان\u200cباتور"}, {"Moscow", "وقت تابستانی مسکو"}, {"New_Caledonia", "وقت تابستانی کالدونیای جدید"}, {"New_Zealand", "وقت تابستانی نیوزیلند"}, {"Newfoundland", "وقت تابستانی نیوفاندلند"}, {"Norfolk", "وقت تابستانی جزیرهٔ نورفولک"}, {"Noronha", "وقت تابستانی فرناندو دی نورونیا"}, {"Novosibirsk", "وقت تابستانی نووسیبیرسک"}, {"Omsk", "وقت تابستانی اومسک"}, {"Pakistan", "وقت تابستانی پاکستان"}, {"Paraguay", "وقت تابستانی پاراگوئه"}, {"Peru", "وقت تابستانی پرو"}, {"Philippines", "وقت تابستانی فیلیپین"}, {"Pierre_Miquelon", "وقت تابستانی سنت\u200cپیر و میکلون"}, {"Qyzylorda", "وقت تابستانی قیزیل\u200cاوردا"}, {"Sakhalin", "وقت تابستانی ساخالین"}, {"Samara", "وقت تابستانی سامارا"}, {"Samoa", "وقت تابستانی ساموا"}, {"Taipei", "وقت تابستانی تایپه"}, {"Tonga", "وقت تابستانی تونگا"}, {"Turkey", "وقت تابستانی ترکیه"}, {"Turkmenistan", "وقت تابستانی ترکمنستان"}, {"Uruguay", "وقت تابستانی اروگوئه"}, {"Uzbekistan", "وقت تابستانی ازبکستان"}, {"Vanuatu", "وقت تابستانی واناتو"}, {"Vladivostok", "وقت تابستانی ولادی\u200cوستوک"}, {"Volgograd", "وقت تابستانی ولگاگراد"}, {"Yakutsk", "وقت تابستانی یاکوتسک"}, {"Yekaterinburg", "وقت تابستانی یکاترینبورگ"}},
	},
	{
		{{"Acre", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞥄𞤳𞤭𞤪"}, {"Afghanistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤀𞤬𞤺𞤢𞤲𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Africa_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤀𞤬𞤪𞤭𞤳𞤢𞥄"}, {"Africa_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫 𞤀𞤬𞤪𞤭𞤳𞤢𞥄"}, {"Africa_Southern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤬𞤪𞤭𞤳𞤢𞥄"}, {"Africa_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫 𞤀𞤬𞤪𞤭𞤳𞤢𞥄"}, {"Alaska", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤤𞤢𞤧𞤳𞤢𞥄 𞤲𞤣𞤫𞤲"}, {"Almaty", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤤𞤥𞤢𞤼𞤭"}, {"Amazon", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤥𞤢𞥁𞤮𞥅𞤲"}, {"America_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤵𞤥𞤦𞤮 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Mountain", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤆𞤫𞤤𞥆𞤭𞤲𞤳𞤮𞥅𞤪𞤫 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Pacific", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤁𞤫𞤰𞥆𞤮 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"Anadyr", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤲𞤢𞤣𞤭𞥅𞤪"}, {"Apia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞥄𞤨𞤭𞤴𞤢"}, {"Aqtau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤳𞤼𞤢𞤱"}, {"Aqtobe", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤳𞤼𞤮𞤦𞤭𞥅"}, {"Arabian", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞥄𞤪𞤢𞤦𞤭𞤴𞤢"}, {"Argentina", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤪𞤶𞤢𞤲𞤼𞤭𞤲𞤢𞥄"}, {"Argentina_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤶𞤢𞤲𞤼𞤭𞤲𞤢𞥄"}, {"Armenia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤪𞤥𞤫𞤲𞤭𞤴𞤢𞥄"}, {"Atlantic", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤫𞤳𞤵 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫"}, {"Australia_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_CentralWestern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤵𞤥𞤦𞤮 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Azerbaijan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞤶𞤫𞤪𞤦𞤢𞤴𞤶𞤢𞤲"}, {"Azores", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤀𞥁𞤮𞤪𞤫𞤧"}, {"Bangladesh", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤄𞤢𞤲𞤺𞤭𞤤𞤢𞤣𞤫𞥅𞤧"}, {"Bhutan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤄𞤵𞤼𞤢𞥄𞤲"}, {"Bolivia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤄𞤮𞤤𞤭𞤾𞤭𞤴𞤢𞥄"}, {"Brasilia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤄𞤪𞤢𞤧𞤭𞤤𞤭𞤴𞤢𞥄"}, {"Brunei", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤄𞤵𞤪𞤲𞤢𞤴"}, {"Cape_Verde", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤢𞥄𞤦𞤮 𞤜𞤫𞤪𞤣𞤫"}, {"Casey", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤑𞤢𞥄𞤧𞤫𞤴"}, {"Chamorro", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤕𞤮𞤥𞤮𞥅𞤪𞤮"}, {"Chatham", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤕𞤢𞤼𞤢𞥄𞤥"}, {"Chile", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤕𞤭𞤤𞤫𞥅"}, {"China", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤕𞤢𞤴𞤲𞤢"}, {"Christmas", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤑𞤭𞤪𞤧𞤭𞤥𞤢𞥄𞤧"}, {"Cocos", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤑𞤮𞥅𞤳𞤮𞤧"}, {"Colombia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤮𞤤𞤮𞤥𞤦𞤭𞤴𞤢𞥄"}, {"Cook", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤑𞤵𞥅𞤳"}, {"Cuba", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤵𞥅𞤦𞤢𞥄"}, {"Davis", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤁𞤫𞥅𞤾𞤭𞤧"}, {"DumontDUrville", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤁𞤭𞤥𞤮𞤲𞤼𞤵-𞤁𞤵𞤪𞤾𞤭𞤤"}, {"East_Timor", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤚𞤭𞥅𞤥𞤮𞤪"}, {"Easter", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤋𞤧𞤼𞤮𞥅-𞤀𞤴𞤤𞤢𞤲𞤣"}, {"Ecuador", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤋𞤳𞤵𞤱𞤢𞤣𞤮𞥅𞤪"}, {"Europe_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Europe_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Europe_Further_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞥋𞤺𞤫𞥅𞤪𞤭 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Europe_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Falkland", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤮𞤤𞤳𞤤𞤢𞤲𞤣-𞤀𞤴𞤤𞤢𞤲𞤣"}, {"Fiji", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤭𞤶𞤭𞥅"}, {"French_Guiana", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤘𞤢𞤴𞤢𞤲𞤢𞥄-𞤊𞤪𞤢𞤲𞤧𞤭"}, {"French_Southern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤂𞤫𞤴𞤪𞤭 𞤊𞤪𞤢𞤲𞤧𞤭 & 𞤀𞤪𞤼𞤢𞤲𞤼𞤭𞤳𞤢"}, {"GMT", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤳𞤭𞤲𞤭𞥅𞤲𞥋𞤣𞤫 𞤘𞤪𞤭𞤲𞤱𞤭𞥅𞤧"}, {"Galapagos", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤘𞤢𞤤𞤢𞤨𞤢𞤺𞤮𞥅𞤧"}, {"Gambier", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤘𞤢𞤥𞤦𞤭𞤴𞤫𞤪"}, {"Georgia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤔𞤮𞤪𞤶𞤭𞤴𞤢"}, {"Gilbert_Islands", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤵𞤪𞤭𞥅𞤶𞤫 𞤘𞤭𞤤𞤦𞤫𞤪𞤼𞤵"}, {"Greenland_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤬𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫 𞤘𞤭𞤪𞤤𞤢𞤲𞤣"}, {"Greenland_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫 𞤘𞤭𞤪𞤤𞤢𞤲𞤣"}, {"Guam", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤘𞤵𞤱𞤢𞥄𞤥"}, {"Gulf", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤂𞤮𞥅𞤻𞤵"}, {"Guyana", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤘𞤢𞤴𞤢𞤲𞤢𞥄"}, {"Hawaii", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤢𞤱𞤢𞥄𞤴𞤭𞥅-𞤀𞤤𞤮𞤧𞤭𞤴𞤢𞤲"}, {"Hawaii_Aleutian", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤢𞤱𞤢𞥄𞤴𞤭𞥅-𞤀𞤤𞤮𞤧𞤭𞤴𞤢𞤲"}, {"Hong_Kong", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤮𞤲𞤺 𞤑𞤮𞤲𞤺"}, {"Hovd", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤖𞤮𞤬𞤣𞤵"}, {"India", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤋𞤲𞤣𞤭𞤴𞤢"}, {"Indian_Ocean", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤢𞥄𞤴𞤮 𞤋𞤲𞤣𞤭𞤴𞤢𞤱𞤮"}, {"Indochina", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤋𞤲𞤣𞤮𞤧𞤭𞥅𞤲"}, {"Indonesia_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤮𞤥𞤦𞤮𞥅𞤪𞤭 𞤋𞤲𞤣𞤮𞤲𞤭𞥅𞤧𞤭𞤴𞤢"}, {"Indonesia_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤋𞤲𞤣𞤮𞤲𞤭𞥅𞤧𞤭𞤴𞤢"}, {"Indonesia_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤋𞤲𞤣𞤮𞤲𞤭𞥅𞤧𞤭𞤴𞤢"}, {"Iran", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤋𞤪𞤢𞥄𞤲"}, {"Irkutsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤋𞤪𞤳𞤵𞤼𞤭𞤧𞤳𞤵"}, {"Israel", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤋𞤧𞤪𞤢𞥄𞤭𞥅𞤤𞤵"}, {"Japan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤭𞤨𞥆𞤮𞤲"}, {"Kazakhstan_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤑𞤢𞥁𞤢𞤿𞤢𞤧𞤼𞤢𞥄𞤲"}, {"Kazakhstan_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤭𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤑𞤢𞥁𞤢𞤿𞤢𞤧𞤼𞤢𞥄𞤲"}, {"Korea", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤮𞥅𞤪𞤫𞤴𞤢𞥄"}, {"Kosrae", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤑𞤮𞤧𞤪𞤢𞤴"}, {"Krasnoyarsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤪𞤢𞤧𞤲𞤮𞤴𞤢𞤪𞤧𞤭𞤳"}, {"Kyrgystan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤑𞤭𞤪𞤺𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Line_Islands", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤂𞤢𞤴𞤲𞤵"}, {"Lord_Howe", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤂𞤮𞤪𞤣𞤵-𞤖𞤮𞤱𞤫"}, {"Macau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤃𞤢𞤳𞤢𞤱𞤮𞥅"}, {"Magadan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤃𞤢𞤺𞤢𞤣𞤢𞤲"}, {"Malaysia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤢𞤤𞤫𞥅𞤧𞤭𞤴𞤢"}, {"Maldives", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤢𞤤𞤣𞤢𞥄𞤴𞤭𞤧"}, {"Marquesas", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤢𞤪𞤳𞤫𞤧𞤢𞤧"}, {"Marshall_Islands", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤵𞤪𞤭𞥅𞤶𞤫 𞤃𞤢𞤪𞤧𞤢𞤤"}, {"Mauritius", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤃𞤮𞤪𞤭𞥅𞤧𞤭"}, {"Mawson", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤢𞤱𞤧𞤮𞤲"}, {"Mexico_Pacific", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤁𞤫𞤰𞥆𞤮 𞤃𞤫𞤳𞤧𞤭𞤳𞤮𞥅"}, {"Mongolia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤓𞤤𞤢𞤲𞤦𞤢𞤼𞤢𞤪"}, {"Moscow", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤃𞤮𞤧𞤳𞤮"}, {"Myanmar", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤃𞤭𞤴𞤢𞤥𞤢𞥄𞤪"}, {"Nauru", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤐𞤵𞥅𞤪𞤵"}, {"Nepal", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤐𞤫𞤨𞤢𞤤"}, {"New_Caledonia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤢𞤤𞤭𞤣𞤮𞤲𞤭𞤴𞤢𞥄 𞤖𞤫𞤧𞤮"}, {"New_Zealand", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤫𞤱-𞤟𞤫𞤤𞤢𞤲𞤣𞤭"}, {"Newfoundland", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤫𞤱-𞤊𞤵𞤲𞤣𞤵𞤤𞤢𞤲𞤣"}, {"Niue", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤐𞤵𞥅𞤱𞤭"}, {"Norfolk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤐𞤮𞤪𞤬𞤮𞤤𞤳𞤵"}, {"Noronha", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤫𞤪𞤲𞤢𞤲𞤣𞤮𞥅 𞤣𞤫 𞤐𞤮𞤪𞤮𞤲𞤽𞤢𞥄"}, {"Novosibirsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤐𞤮𞤾𞤮𞤧𞤦𞤭𞤪𞤧𞤭𞤳"}, {"Omsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤌𞤥𞤧𞤵𞤳𞤵"}, {"Pakistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤆𞤢𞤳𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Palau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤮𞤤𞤢𞥄𞤱𞤮"}, {"Papua_New_Guinea", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤢𞤨𞤵𞤱𞤢 𞤘𞤭𞤲𞤫 𞤖𞤫𞤧𞤮"}, {"Paraguay", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤆𞤢𞥄𞤪𞤢𞤺𞤮𞤴"}, {"Peru", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤆𞤫𞤪𞤵𞥅"}, {"Philippines", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤊𞤭𞤤𞤭𞤨𞤭𞥅𞤲"}, {"Phoenix_Islands", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤊𞤫𞤲𞤭𞤳𞤧𞤭"}, {"Pierre_Miquelon", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤅𞤼. 𞤆𞤭𞤴𞤫𞥅𞤪 & 𞤃𞤭𞤳𞤫𞤤𞤮𞤲"}, {"Pitcairn", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤭𞤼𞤳𞤭𞥅𞤪𞤲𞤵"}, {"Ponape", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤮𞤲𞤢𞥄𞤨𞤫"}, {"Pyongyang", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤭𞤴𞤮𞤲𞤴𞤢𞤲"}, {"Qyzylorda", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤑𞤭𞤶𞤤𞤮𞤪𞤣𞤢𞥄"}, {"Reunion", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤈𞤫𞤲𞤭𞤴𞤮𞤲"}, {"Rothera", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤈𞤮𞤼𞤫𞤪𞤢"}, {"Sakhalin", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤅𞤢𞤿𞤢𞤤𞤭𞥅𞤲"}, {"Samoa", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤅𞤢𞤥𞤵𞤱𞤢"}, {"Seychelles", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤫𞤴𞤭𞤧𞤫𞤤"}, {"Singapore", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤅𞤭𞤲𞤺𞤢𞤨𞤵𞥅𞤪"}, {"Solomon", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤅𞤵𞤤𞤢𞤴𞤥𞤢𞥄𞤲"}, {"South_Georgia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤢𞤱𞤬-𞤔𞤮𞤪𞤶𞤭𞤴𞤢𞥄"}, {"Suriname", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤭𞤪𞤭𞤲𞤢𞤥"}, {"Syowa", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤢𞥄𞤴𞤵𞤱𞤢"}, {"Tahiti", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤢𞤸𞤭𞤼𞤭𞥅"}, {"Taipei", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤢𞤴𞤨𞤫𞥅"}, {"Tajikistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤢𞤶𞤭𞤳𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Tokelau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤮𞥅𞤳𞤮𞤤𞤢𞥄𞤱𞤵"}, {"Tonga", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤮𞤲𞤺𞤢 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫"}, {"Truk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤵𞥅𞤳𞤵"}, {"Turkmenistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤚𞤵𞤪𞤳𞤵𞤥𞤫𞤲𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Tuvalu", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤚𞤵𞥅𞤾𞤢𞤤𞤵"}, {"Uruguay", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤒𞤵𞥅𞤪𞤺𞤮𞤴"}, {"Uzbekistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤓𞥁𞤦𞤫𞤳𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Vanuatu", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤜𞤢𞤲𞤵𞤱𞤢𞥄𞤼𞤵"}, {"Venezuela", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤜𞤫𞤲𞤭𞥅𞥁𞤮𞥅𞤤𞤢"}, {"Vladivostok", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤜𞤭𞤤𞤢𞤾𞤮𞤧𞤼𞤮𞤳"}, {"Volgograd", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤜𞤮𞤤𞤺𞤮𞤺𞤢𞤪𞤢𞥄𞤣"}, {"Vostok", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤜𞤮𞤧𞤼𞤮𞤳"}, {"Wake", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤏𞤫𞥅𞤳𞤵"}, {"Wallis", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤏𞤢𞤤𞥆𞤭𞥅𞤧 & 𞤊𞤵𞤼𞤵𞤲𞤢"}, {"Yakutsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤒𞤢𞤳𞤢𞤼𞤭𞤧𞤳𞤵"}, {"Yekaterinburg", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤵𞤲𞥋𞤣𞤫 𞤒𞤫𞤳𞤢𞤼𞤫𞤪𞤭𞤲𞤦𞤵𞤪𞤺𞤵"}},
		{{"Acre", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞥄𞤳𞤭𞤪"}, {"Alaska", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤤𞤢𞤧𞤳𞤢𞥄 𞤲𞤣𞤫𞤲"}, {"Almaty", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤤𞤥𞤢𞤼𞤭"}, {"Amazon", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤥𞤢𞥁𞤮𞥅𞤲"}, {"America_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤵𞤥𞤦𞤮 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Mountain", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"America_Pacific", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤁𞤫𞤰𞥆𞤮 𞤕𞤫𞥅𞤯𞤵 𞤐𞤢𞤲𞥆𞤢𞥄𞤲𞤺𞤫 𞤀𞤥𞤫𞤪𞤭𞤳𞤢𞥄"}, {"Anadyr", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤲𞤢𞤣𞤭𞥅𞤪"}, {"Apia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞥄𞤨𞤭𞤴𞤢"}, {"Aqtau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤳𞤼𞤢𞤱"}, {"Aqtobe", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤳𞤼𞤮𞤦𞤭𞥅"}, {"Arabian", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞥄𞤪𞤢𞤦𞤭𞤴𞤢"}, {"Argentina", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤪𞤶𞤢𞤲𞤼𞤭𞤲𞤢𞥄"}, {"Argentina_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤶𞤢𞤲𞤼𞤭𞤲𞤢𞥄"}, {"Armenia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤪𞤥𞤫𞤲𞤭𞤴𞤢𞥄"}, {"Atlantic", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤫𞤳𞤵"}, {"Australia_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_CentralWestern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤵𞤥𞤦𞤮 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Australia_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫𞥅𞤪𞤭 𞤌𞤧𞤼𞤢𞤪𞤤𞤭𞤴𞤢𞥄"}, {"Azerbaijan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞤶𞤫𞤪𞤦𞤢𞤴𞤶𞤢𞤲"}, {"Azores", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤀𞥁𞤮𞤪𞤫𞤧"}, {"Bangladesh", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤄𞤢𞤲𞤺𞤭𞤤𞤢𞤣𞤫𞥅𞤧"}, {"Brasilia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤄𞤪𞤢𞤧𞤭𞤤𞤭𞤴𞤢𞥄"}, {"Cape_Verde", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤢𞥄𞤦𞤮-𞤜𞤫𞤪𞤣𞤫"}, {"Chatham", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤕𞤢𞥄𞤼𞤢𞤥"}, {"Chile", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤕𞤭𞤤𞤫𞥅"}, {"China", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤕𞤢𞤴𞤲𞤢"}, {"Colombia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤮𞤤𞤮𞤥𞤦𞤭𞤴𞤢𞥄"}, {"Cook", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤆𞤫𞤷𞥆𞤵 𞤕𞤫𞥅𞤯𞤵 𞤕𞤵𞤪𞤭𞥅𞤶𞤫 𞤑𞤵𞥅𞤳"}, {"Cuba", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤵𞥅𞤦𞤢𞥄"}, {"Easter", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤋𞤧𞤼𞤮𞥅-𞤀𞤴𞤤𞤢𞤲𞤣"}, {"Europe_Central", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤵𞤥𞤦𞤮𞥅𞤪𞤭 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Europe_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Europe_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞥋𞤺𞤫 𞤀𞤪𞤮𞥅𞤦𞤢"}, {"Falkland", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤮𞤤𞤳𞤤𞤢𞤲𞤣-𞤀𞤴𞤤𞤢𞤲𞤣"}, {"Fiji", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤭𞤶𞤭𞥅"}, {"Georgia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤔𞤮𞤪𞤶𞤭𞤴𞤢"}, {"Greenland_Eastern", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤵𞤯𞤲𞤢𞥄𞤲𞤺𞤫 𞤘𞤭𞤪𞤤𞤢𞤲𞤣"}, {"Greenland_Western", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤭𞥅𞤪𞤲𞤢𞥄𞤲𞤺𞤫 𞤘𞤭𞤪𞤤𞤢𞤲𞤣"}, {"Hawaii_Aleutian", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤢𞤱𞤢𞥄𞤴𞤭𞥅-𞤀𞤤𞤮𞤧𞤭𞤴𞤢𞤲"}, {"Hong_Kong", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤮𞤲𞤺 𞤑𞤮𞤲𞤺"}, {"Hovd", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤖𞤮𞤬𞤣𞤵"}, {"Iran", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤋𞤪𞤢𞥄𞤲"}, {"Irkutsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤋𞤪𞤳𞤵𞤼𞤭𞤧𞤳𞤵"}, {"Israel", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤋𞤧𞤪𞤢𞥄𞤭𞥅𞤤𞤵"}, {"Japan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤐𞤭𞤨𞥆𞤮𞤲"}, {"Korea", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤮𞥅𞤪𞤫𞤴𞤢𞥄"}, {"Krasnoyarsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤢𞤪𞤢𞤧𞤲𞤮𞤴𞤢𞤪𞤧𞤭𞤳"}, {"Lord_Howe", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤂𞤮𞤪𞤣𞤵-𞤖𞤮𞤱𞤫"}, {"Macau", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤃𞤢𞤳𞤢𞤱𞤮𞥅"}, {"Magadan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤃𞤢𞤺𞤢𞤣𞤢𞤲"}, {"Mauritius", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤃𞤮𞤪𞤭𞥅𞤧𞤭"}, {"Mexico_Pacific", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤁𞤫𞤰𞥆𞤮 𞤃𞤫𞤳𞤧𞤭𞤳𞤮𞥅"}, {"Mongolia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤓𞤤𞤢𞤲𞤦𞤢𞤼𞤢𞤪"}, {"Moscow", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤃𞤮𞤧𞤳𞤮"}, {"New_Caledonia", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤢𞤤𞤭𞤣𞤮𞤲𞤭𞤴𞤢𞥄 𞤖𞤫𞤧𞤮"}, {"New_Zealand", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤐𞤫𞤱-𞤟𞤫𞤤𞤢𞤲𞤣𞤭"}, {"Newfoundland", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤐𞤫𞤱-𞤊𞤵𞤲𞤣𞤵𞤤𞤢𞤲𞤣"}, {"Norfolk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤅𞤵𞤪𞤭𞥅𞤪𞤫 𞤐𞤮𞤪𞤬𞤮𞤤𞤳𞤵"}, {"Noronha", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤫𞤪𞤲𞤢𞤲𞤣𞤮𞥅 𞤣𞤫 𞤐𞤮𞤪𞤮𞤲𞤽𞤢𞥄"}, {"Novosibirsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤐𞤮𞤾𞤮𞤧𞤦𞤭𞤪𞤧𞤭𞤳"}, {"Omsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤌𞤥𞤧𞤵𞤳𞤵"}, {"Pakistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤆𞤢𞤳𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Paraguay", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤆𞤢𞥄𞤪𞤢𞤺𞤮𞤴"}, {"Peru", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤖𞤢𞤱𞤪𞤭𞤼𞤵𞤲𞥋𞤣𞤫 𞤕𞤫𞥅𞤯𞤵 𞤆𞤫𞤪𞤵𞥅"}, {"Philippines", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤊𞤭𞤤𞤭𞤨𞤭𞥅𞤲"}, {"Pierre_Miquelon", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤅𞤼. 𞤆𞤭𞤴𞤫𞥅𞤪 & 𞤃𞤭𞤳𞤫𞤤𞤮𞤲"}, {"Qyzylorda", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤑𞤭𞤶𞤤𞤮𞤪𞤣𞤢𞥄"}, {"Sakhalin", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤅𞤢𞤿𞤢𞤤𞤭𞥅𞤲"}, {"Samoa", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤅𞤢𞤥𞤵𞤱𞤢"}, {"Taipei", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤢𞤴𞤨𞤫𞥅"}, {"Tonga", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤮𞤲𞤺𞤢"}, {"Turkmenistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤚𞤵𞤪𞤳𞤵𞤥𞤫𞤲𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Uruguay", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤒𞤵𞥅𞤪𞤺𞤮𞤴"}, {"Uzbekistan", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤓𞥁𞤦𞤫𞤳𞤭𞤧𞤼𞤢𞥄𞤲"}, {"Vanuatu", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤜𞤢𞤲𞤵𞤱𞤢𞥄𞤼𞤵"}, {"Vladivostok", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤜𞤭𞤤𞤢𞤾𞤮𞤧𞤼𞤮𞤳"}, {"Volgograd", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤜𞤮𞤤𞤺𞤮𞤺𞤢𞤪𞤢𞥄𞤣"}, {"Yakutsk", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤒𞤢𞤳𞤢𞤼𞤭𞤧𞤳𞤵"}, {"Yekaterinburg", "𞤑𞤭𞤶𞤮𞥅𞤪𞤫 𞤕𞤫𞥅𞤯𞤵 𞤒𞤫𞤳𞤢𞤼𞤫𞤪𞤭𞤲𞤦𞤵𞤪𞤺𞤵"}},
	},
	{
		{{"Acre", "Acren normaaliaika"}, {"Afghanistan", "Afganistanin aika"}, {"Africa_Central", "Keski-Afrikan aika"}, {"Africa_Eastern", "Itä-Afrikan aika"}, {"Africa_Southern", "Etelä-Afrikan aika"}, {"Africa_Western", "Länsi-Afrikan aika"}, {"Alaska", "Alaskan normaaliaika"}, {"Almaty", "Almatyn normaaliaika"}, {"Amazon", "Amazonin normaaliaika"}, {"America_Central", "Yhdysvaltain keskinen normaaliaika"}, {"America_Eastern", "Yhdysvaltain itäinen normaaliaika"}, {"America_Mountain", "Kalliovuorten normaaliaika"}, {"America_Pacific", "Yhdysvaltain Tyynenmeren normaaliaika"}, {"Anadyr", "Anadyrin normaaliaika"}, {"Apia", "Apian normaaliaika"}, {"Aqtau", "Aqtaw’n normaaliaika"}, {"Aqtobe", "Aqtöben normaaliaika"}, {"Arabian", "Saudi-Arabian normaaliaika"}, {"Argentina", "Argentiinan normaaliaika"}, {"Argentina_Western", "Länsi-Argentiinan normaaliaika"}, {"Armenia", "Armenian normaaliaika"}, {"Atlantic", "Kanadan Atlantin normaaliaika"}, {"Australia_Central", "Keski-Australian normaaliaika"}, {"Australia_CentralWestern", "Läntisen Keski-Australian normaaliaika"}, {"Australia_Eastern", "Itä-Australian normaaliaika"}, {"Australia_Western", "Länsi-Australian normaaliaika"}, {"Azerbaijan", "Azerbaidžanin normaaliaika"}, {"Azores", "Azorien normaaliaika"}, {"Bangladesh", "Bangladeshin normaaliaika"}, {"Bhutan", "Bhutanin aika"}, {"Bolivia", "Bolivian aika"}, {"Brasilia", "Brasilian normaaliaika"}, {"Brunei", "Brunein aika"}, {"Cape_Verde", "Kap Verden normaaliaika"}, {"Casey", "Caseyn aika"}, {"Chamorro", "Tšamorron aika"}, {"Chatham", "Chathamin normaaliaika"}, {"Chile", "Chilen normaaliaika"}, {"China", "Kiinan normaaliaika"}, {"Christmas", "Joulusaaren aika"}, {"Cocos", "Kookossaarten aika"}, {"Colombia", "Kolumbian normaaliaika"}, {"Cook", "Cookinsaarten normaaliaika"}, {"Cuba", "Kuuban normaaliaika"}, {"Davis", "Davisin aika"}, {"DumontDUrville", "Dumont d’Urvillen aika"}, {"East_Timor", "Itä-Timorin aika"}, {"Easter", "Pääsiäissaaren normaaliaika"}, {"Ecuador", "Ecuadorin aika"}, {"Europe_Central", "Keski-Euroopan normaaliaika"}, {"Europe_Eastern", "Itä-Euroopan normaaliaika"}, {"Europe_Further_Eastern", "Itäisemmän Euroopan aika"}, {"Europe_Western", "Länsi-Euroopan normaaliaika"}, {"Falkland", "Falklandinsaarten normaaliaika"}, {"Fiji", "Fidžin normaaliaika"}, {"French_Guiana", "Ranskan Guayanan aika"}, {"French_Southern", "Ranskan eteläisten ja antarktisten alueiden aika"}, {"GMT", "Greenwichin normaaliaika"}, {"Galapagos", "Galápagossaarten aika"}, {"Gambier", "Gambiersaarten aika"}, {"Georgia", "Georgian normaaliaika"}, {"Gilbert_Islands", "Gilbertsaarten aika"}, {"Greenland_Eastern", "Itä-Grönlannin normaaliaika"}, {"Greenland_Western", "Länsi-Grönlannin normaaliaika"}, {"Guam", "Guamin aika"}, {"Gulf", "Arabiemiirikuntien normaaliaika"}, {"Guyana", "Guyanan aika"}, {"Hawaii", "Havaijin-Aleuttien normaaliaika"}, {"Hawaii_Aleutian", "Havaijin-Aleuttien normaaliaika"}, {"Hong_Kong", "Hongkongin normaaliaika"}, {"Hovd", "Hovdin normaaliaika"}, {"India", "Intian aika"}, {"Indian_Ocean", "Intian valtameren aika"}, {"Indochina", "Indokiinan aika"}, {"Indonesia_Central", "Keski-Indonesian aika"}, {"Indonesia_Eastern", "Itä-Indonesian aika"}, {"Indonesia_Western", "Länsi-Indonesian aika"}, {"Iran", "Iranin normaaliaika"}, {"Irkutsk", "Irkutskin normaaliaika"}, {"Israel", "Israelin normaaliaika"}, {"Japan", "Japanin normaaliaika"}, {"Kamchatka", "Kamtšatkan normaaliaika"}, {"Kazakhstan", "Kazakstanin aika"}, {"Kazakhstan_Eastern", "Itä-Kazakstanin aika"}, {"Kazakhstan_Western", "Länsi-Kazakstanin aika"}, {"Korea", "Korean normaaliaika"}, {"Kosrae", "Kosraen aika"}, {"Krasnoyarsk", "Krasnojarskin normaaliaika"}, {"Kyrgystan", "Kirgisian aika"}, {"Lanka", "Sri Lankan aika"}, {"Line_Islands", "Linesaarten aika"}, {"Lord_Howe", "Lord Howen normaaliaika"}, {"Macau", "Macaon normaaliaika"}, {"Magadan", "Magadanin normaaliaika"}, {"Malaysia", "Malesian aika"}, {"Maldives", "Malediivien aika"}, {"Marquesas", "Marquesassaarten aika"}, {"Marshall_Islands", "Marshallinsaarten aika"}, {"Mauritius", "Mauritiuksen normaaliaika"}, {"Mawson", "Mawsonin aika"}, {"Mexico_Pacific", "Meksikon Tyynenmeren normaaliaika"}, {"Mongolia", "Ulan Batorin normaaliaika"}, {"Moscow", "Moskovan normaaliaika"}, {"Myanmar", "Myanmarin aika"}, {"Nauru", "Naurun aika"}, {"Nepal", "Nepalin aika"}, {"New_Caledonia", "Uuden-Kaledonian normaaliaika"}, {"New_Zealand", "Uuden-Seelannin normaaliaika"}, {"Newfoundland", "Newfoundlandin normaaliaika"}, {"Niue", "Niuen aika"}, {"Norfolk", "Norfolkinsaaren normaaliaika"}, {"Noronha", "Fernando de Noronhan normaaliaika"}, {"North_Mariana", "Pohjois-Mariaanien aika"}, {"Novosibirsk", "Novosibirskin normaaliaika"}, {"Omsk", "Omskin normaaliaika"}, {"Pakistan", "Pakistanin normaaliaika"}, {"Palau", "Palaun aika"}, {"Papua_New_Guinea", "Papua-Uuden-Guinean aika"}, {"Paraguay", "Paraguayn normaaliaika"}, {"Peru", "Perun normaaliaika"}, {"Philippines", "Filippiinien normaaliaika"}, {"Phoenix_Islands", "Phoenixsaarten aika"}, {"Pierre_Miquelon", "Saint-Pierren ja Miquelonin normaaliaika"}, {"Pitcairn", "Pitcairnin aika"}, {"Ponape", "Pohnpein aika"}, {"Pyongyang", "Pjongjangin aika"}, {"Qyzylorda", "Qızılordan normaaliaika"}, {"Reunion", "Réunionin aika"}, {"Rothera", "Rotheran aika"}, {"Sakhalin", "Sahalinin normaaliaika"}, {"Samara", "Samaran normaaliaika"}, {"Samoa", "Samoan normaaliaika"}, {"Seychelles", "Seychellien aika"}, {"Singapore", "Singaporen aika"}, {"Solomon", "Salomonsaarten aika"}, {"South_Georgia", "Etelä-Georgian aika"}, {"Suriname", "Surinamen aika"}, {"Syowa", "Syowan aika"}, {"Tahiti", "Tahitin aika"}, {"Taipei", "Taipein normaaliaika"}, {"Tajikistan", "Tadžikistanin aika"}, {"Tokelau", "Tokelaun aika"}, {"Tonga", "Tongan normaaliaika"}, {"Truk", "Chuukin aika"}, {"Turkey", "Turkin normaaliaika"}, {"Turkmenistan", "Turkmenistanin normaaliaika"}, {"Tuvalu", "Tuvalun aika"}, {"Uruguay", "Uruguayn normaaliaika"}, {"Uzbekistan", "Uzbekistanin normaaliaika"}, {"Vanuatu", "Vanuatun normaaliaika"}, {"Venezuela", "Venezuelan aika"}, {"Vladivostok", "Vladivostokin normaaliaika"}, {"Volgograd", "Volgogradin normaaliaika"}, {"Vostok", "Vostokin aika"}, {"Wake", "Waken aika"}, {"Wallis", "Wallisin ja Futunan aika"}, {"Yakutsk", "Jakutskin normaaliaika"}, {"Yekaterinburg", "Jekaterinburgin normaaliaika"}, {"Yukon", "Yukonin aika"}},
		{{"Acre", "Acren kesäaika"}, {"Alaska", "Alaskan kesäaika"}, {"Almaty", "Almatyn kesäaika"}, {"Amazon", "Amazonin kesäaika"}, {"America_Central", "Yhdysvaltain keskinen kesäaika"}, {"America_Eastern", "Yhdysvaltain itäinen kesäaika"}, {"America_Mountain", "Kalliovuorten kesäaika"}, {"America_Pacific", "Yhdysvaltain Tyynenmeren kesäaika"}, {"Anadyr", "Anadyrin kesäaika"}, {"Apia", "Apian kesäaika"}, {"Aqtau", "Aqtaw’n kesäaika"}, {"Aqtobe", "Aqtöben kesäaika"}, {"Arabian", "Saudi-Arabian kesäaika"}, {"Argentina", "Argentiinan kesäaika"}, {"Argentina_Western", "Länsi-Argentiinan kesäaika"}, {"Armenia", "Armenian kesäaika"}, {"Atlantic", "Kanadan Atlantin kesäaika"}, {"Australia_Central", "Keski-Australian kesäaika"}, {"Australia_CentralWestern", "Läntisen Keski-Australian kesäaika"}, {"Australia_Eastern", "Itä-Australian kesäaika"}, {"Australia_Western", "Länsi-Australian kesäaika"}, {"Azerbaijan", "Azerbaidžanin kesäaika"}, {"Azores", "Azorien kesäaika"}, {"Bangladesh", "Bangladeshin kesäaika"}, {"Brasilia", "Brasilian kesäaika"}, {"Cape_Verde", "Kap Verden kesäaika"}, {"Chatham", "Chathamin kesäaika"}, {"Chile", "Chilen kesäaika"}, {"China", "Kiinan kesäaika"}, {"Colombia", "Kolumbian kesäaika"}, {"Cook", "Cookinsaarten kesäaika"}, {"Cuba", "Kuuban kesäaika"}, {"Easter", "Pääsiäissaaren kesäaika"}, {"Europe_Central", "Keski-Euroopan kesäaika"}, {"Europe_Eastern", "Itä-Euroopan kesäaika"}, {"Europe_Western", "Länsi-Euroopan kesäaika"}, {"Falkland", "Falklandinsaarten kesäaika"}, {"Fiji", "Fidžin kesäaika"}, {"Georgia", "Georgian kesäaika"}, {"Greenland_Eastern", "Itä-Grönlannin kesäaika"}, {"Greenland_Western", "Länsi-Grönlannin kesäaika"}, {"Hawaii_Aleutian", "Havaijin-Aleuttien kesäaika"}, {"Hong_Kong", "Hongkongin kesäaika"}, {"Hovd", "Hovdin kesäaika"}, {"Iran", "Iranin kesäaika"}, {"Irkutsk", "Irkutskin kesäaika"}, {"Israel", "Israelin kesäaika"}, {"Japan", "Japanin kesäaika"}, {"Kamchatka", "Kamtšatkan kesäaika"}, {"Korea", "Korean kesäaika"}, {"Krasnoyarsk", "Krasnojarskin kesäaika"}, {"Lord_Howe", "Lord Howen kesäaika"}, {"Macau", "Macaon kesäaika"}, {"Magadan", "Magadanin kesäaika"}, {"Mauritius", "Mauritiuksen kesäaika"}, {"Mexico_Pacific", "Meksikon Tyynenmeren kesäaika"}, {"Mongolia", "Ulan Batorin kesäaika"}, {"Moscow", "Moskovan kesäaika"}, {"New_Caledonia", "Uuden-Kaledonian kesäaika"}, {"New_Zealand", "Uuden-Seelannin kesäaika"}, {"Newfoundland", "Newfoundlandin kesäaika"}, {"Norfolk", "Norfolkinsaaren kesäaika"}, {"Noronha", "Fernando de Noronhan kesäaika"}, {"Novosibirsk", "Novosibirskin kesäaika"}, {"Omsk", "Omskin kesäaika"}, {"Pakistan", "Pakistanin kesäaika"}, {"Paraguay", "Paraguayn kesäaika"}, {"Peru", "Perun kesäaika"}, {"Philippines", "Filippiinien kesäaika"}, {"Pierre_Miquelon", "Saint-Pierren ja Miquelonin kesäaika"}, {"Qyzylorda", "Qızılordan kesäaika"}, {"Sakhalin", "Sahalinin kesäaika"}, {"Samara", "Samaran kesäaika"}, {"Samoa", "Samoan kesäaika"}, {"Taipei", "Taipein kesäaika"}, {"Tonga", "Tongan kesäaika"}, {"Turkey", "Turkin kesäaika"}, {"Turkmenistan", "Turkmenistanin kesäaika"}, {"Uruguay", "Uruguayn kesäaika"}, {"Uzbekistan", "Uzbekistanin kesäaika"}, {"Vanuatu", "Vanuatun kesäaika"}, {"Vladivostok", "Vladivostokin kesäaika"}, {"Volgograd", "Volgogradin kesäaika"}, {"Yakutsk", "Jakutskin kesäaika"}, {"Yekaterinburg", "Jekaterinburgin kesäaika"}},
	},
	{
		{{"Afghanistan", "Oras sa Afghanistan"}, {"Africa_Central", "Oras sa Gitnang Africa"}, {"Africa_Eastern", "Oras sa Silangang Africa"}, {"Africa_Southern", "Oras sa Timog Africa"}, {"Africa_Western", "Oras sa Kanlurang Africa"}, {"Alaska", "Standard na Oras sa Alaska"}, {"Amazon", "Standard na Oras sa Amazon"}, {"America_Central", "Sentral na Standard na Oras sa North America"}, {"America_Eastern", "Standard na Oras sa Silangan ng Hilangang Amerika"}, {"America_Mountain", "Standard na Oras sa Bundok sa Hilagang Amerika"}, {"America_Pacific", "Standard na Oras sa Pasipiko sa Hilagang Amerika"}, {"Anadyr", "Standard Time sa Anadyr"}, {"Apia", "Standard na Oras sa Apia"}, {"Arabian", "Standard na Oras sa Arabia"}, {"Argentina", "Standard na Oras sa Argentina"}, {"Argentina_Western", "Standard na Oras sa Kanlurang Argentina"}, {"Armenia", "Standard na Oras sa Armenia"}, {"Atlantic", "Standard na Oras sa Atlantiko"}, {"Australia_Central", "Standard na Oras sa Gitnang Australya"}, {"Australia_CentralWestern", "Standard Time ng Gitnang Kanluran ng Australia"}, {"Australia_Eastern", "Standard na Oras sa Silangang Australia"}, {"Australia_Western", "Standard na Oras sa Kanlurang Australia"}, {"Azerbaijan", "Standard na Oras sa Azerbaijan"}, {"Azores", "Standard na Oras sa Azores"}, {"Bangladesh", "Standard na Oras sa Bangladesh"}, {"Bhutan", "Oras sa Bhutan"}, {"Bolivia", "Oras sa Bolivia"}, {"Brasilia", "Standard na Oras sa Brasilia"}, {"Brunei", "Oras sa Brunei Darussalam"}, {"Cape_Verde", "Standard na Oras sa Cape Verde"}, {"Chamorro", "Standard na Oras sa Chamorro"}, {"Chatham", "Standard na Oras sa Chatham"}, {"Chile", "Standard na Oras sa Chile"}, {"China", "Standard na Oras sa China"}, {"Christmas", "Oras sa Christmas Island"}, {"Cocos", "Oras sa Cocos Islands"}, {"Colombia", "Standard na Oras sa Colombia"}, {"Cook", "Standard na Oras sa Cook Islands"}, {"Cuba", "Standard na Oras sa Cuba"}, {"Davis", "Oras sa Davis"}, {"DumontDUrville", "Oras sa Dumont-d’Urville"}, {"East_Timor", "Oras sa East Timor"}, {"Easter", "Standard na Oras sa Easter Island"}, {"Ecuador", "Oras sa Ecuador"}, {"Europe_Central", "Standard na Oras sa Gitnang Europe"}, {"Europe_Eastern", "Standard na Oras sa Silangang Europe"}, {"Europe_Further_Eastern", "Oras sa Pinaka-silangang Europe"}, {"Europe_Western", "Standard na Oras sa Kanlurang Europe"}, {"Falkland", "Standard na Oras sa Falkland Islands"}, {"Fiji", "Standard na Oras sa Fiji"}, {"French_Guiana", "Oras sa French Guiana"}, {"French_Southern", "Oras sa Katimugang France at Antartiko"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Oras sa Galapagos"}, {"Gambier", "Oras sa Gambier"}, {"Georgia", "Standard na Oras sa Georgia"}, {"Gilbert_Islands", "Oras sa Gilbert Islands"}, {"Greenland_Eastern", "Standard na Oras sa Silangang Greenland"}, {"Greenland_Western", "Standard na Oras sa Kanlurang Greenland"}, {"Gulf", "Oras sa Gulf"}, {"Guyana", "Oras sa Guyana"}, {"Hawaii", "Standard na Oras sa Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Standard na Oras sa Hawaii-Aleutian"}, {"Hong_Kong", "Standard na Oras sa Hong Kong"}, {"Hovd", "Standard na Oras sa Hovd"}, {"India", "Standard na Oras sa India"}, {"Indian_Ocean", "Oras sa Indian Ocean"}, {"Indochina", "Oras sa Indochina"}, {"Indonesia_Central", "Oras sa Gitnang Indonesia"}, {"Indonesia_Eastern", "Oras sa Silangang Indonesia"}, {"Indonesia_Western", "Oras sa Kanlurang Indonesia"}, {"Iran", "Standard na Oras sa Iran"}, {"Irkutsk", "Standard na Oras sa Irkutsk"}, {"Israel", "Standard na Oras sa Israel"}, {"Japan", "Standard na Oras sa Japan"}, {"Kamchatka", "Standard Time sa Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Oras ng Kazakhstan"}, {"Kazakhstan_Eastern", "Oras sa Silangang Kazakhstan"}, {"Kazakhstan_Western", "Oras sa Kanlurang Kazakhstan"}, {"Korea", "Standard na Oras sa Korea"}, {"Kosrae", "Oras sa Kosrae"}, {"Krasnoyarsk", "Standard na Oras sa Krasnoyarsk"}, {"Kyrgystan", "Oras sa Kyrgystan"}, {"Line_Islands", "Oras sa Line Islands"}, {"Lord_Howe", "Standard na Oras sa Lord Howe"}, {"Magadan", "Standard na Oras sa Magadan"}, {"Malaysia", "Oras sa Malaysia"}, {"Maldives", "Oras sa Maldives"}, {"Marquesas", "Oras sa Marquesas"}, {"Marshall_Islands", "Oras sa Marshall Islands"}, {"Mauritius", "Standard na Oras sa Mauritius"}, {"Mawson", "Oras sa Mawson"}, {"Mexico_Pacific", "Standard na Oras sa Pasipiko ng Mexico"}, {"Mongolia", "Standard na Oras sa Ulan Bator"}, {"Moscow", "Standard na Oras sa Moscow"}, {"Myanmar", "Oras sa Myanmar"}, {"Nauru", "Oras sa Nauru"}, {"Nepal", "Oras sa Nepal"}, {"New_Caledonia", "Standard na Oras sa New Caledonia"}, {"New_Zealand", "Standard na Oras sa New Zealand"}, {"Newfoundland", "Standard na Oras sa Newfoundland"}, {"Niue", "Oras sa Niue"}, {"Norfolk", "Standard na Oras sa Norfolk Island"}, {"Noronha", "Standard na Oras sa Fernando de Noronha"}, {"Novosibirsk", "Standard na Oras sa Novosibirsk"}, {"Omsk", "Standard na Oras sa Omsk"}, {"Pakistan", "Standard na Oras sa Pakistan"}, {"Palau", "Oras sa Palau"}, {"Papua_New_Guinea", "Oras sa Papua New Guinea"}, {"Paraguay", "Standard na Oras sa Paraguay"}, {"Peru", "Standard na Oras sa Peru"}, {"Philippines", "Standard na Oras sa Pilipinas"}, {"Phoenix_Islands", "Oras sa Phoenix Islands"}, {"Pierre_Miquelon", "Standard na Oras sa Saint Pierre & Miquelon"}, {"Pitcairn", "Oras sa Pitcairn"}, {"Ponape", "Oras sa Ponape"}, {"Pyongyang", "Oras sa Pyongyang"}, {"Reunion", "Oras sa Reunion"}, {"Rothera", "Oras sa Rothera"}, {"Sakhalin", "Standard na Oras sa Sakhalin"}, {"Samara", "Standard Time sa Samara"}, {"Samoa", "Standard na Oras sa Samoa"}, {"Seychelles", "Oras sa Seychelles"}, {"Singapore", "Standard na Oras sa Singapore"}, {"Solomon", "Oras sa Solomon Islands"}, {"South_Georgia", "Oras sa Timog Georgia"}, {"Suriname", "Oras sa Suriname"}, {"Syowa", "Oras sa Syowa"}, {"Tahiti", "Oras sa Tahiti"}, {"Taipei", "Standard na Oras sa Taipei"}, {"Tajikistan", "Oras sa Tajikistan"}, {"Tokelau", "Oras sa Tokelau"}, {"Tonga", "Standard na Oras sa Tonga"}, {"Truk", "Oras sa Chuuk"}, {"Turkmenistan", "Standard na Oras sa Turkmenistan"}, {"Tuvalu", "Oras sa Tuvalu"}, {"Uruguay", "Standard na Oras sa Uruguay"}, {"Uzbekistan", "Standard na Oras sa Uzbekistan"}, {"Vanuatu", "Standard na Oras sa Vanuatu"}, {"Venezuela", "Oras sa Venezuela"}, {"Vladivostok", "Standard na Oras sa Vladivostok"}, {"Volgograd", "Standard na Oras sa Volgograd"}, {"Vostok", "Oras sa Vostok"}, {"Wake", "Oras sa Wake Island"}, {"Wallis", "Oras sa Wallis & Futuna"}, {"Yakutsk", "Standard na Oras sa Yakutsk"}, {"Yekaterinburg", "Standard na Oras sa Yekaterinburg"}, {"Yukon", "Oras sa Yukon"}},
		{{"Alaska", "Daylight Time sa Alaska"}, {"Amazon", "Oras sa Tag-init ng Amazon"}, {"America_Central", "Sentral na Daylight na Oras sa North America"}, {"America_Eastern", "Daylight na Oras sa Silangan ng Hilagang Amerika"}, {"America_Mountain", "Daylight na Oras sa Bundok sa Hilagang Amerika"}, {"America_Pacific", "Daylight na Oras sa Pasipiko sa Hilagang Amerika"}, {"Anadyr", "Summer Time sa Anadyr"}, {"Apia", "Daylight Time sa Apia"}, {"Arabian", "Daylight Time sa Arabia"}, {"Argentina", "Oras sa Tag-init ng Argentina"}, {"Argentina_Western", "Oras sa Tag-init ng Kanlurang Argentina"}, {"Armenia", "Oras sa Tag-init ng Armenia"}, {"Atlantic", "Daylight na Oras sa Atlantiko"}, {"Australia_Central", "Daylight Time sa Gitnang Australya"}, {"Australia_CentralWestern", "Daylight Time sa Gitnang Kanlurang Australia"}, {"Australia_Eastern", "Daylight Time sa Silangang Australia"}, {"Australia_Western", "Daylight Time sa Kanlurang Australia"}, {"Azerbaijan", "Oras sa Tag-init ng Azerbaijan"}, {"Azores", "Oras sa Tag-init ng Azores"}, {"Bangladesh", "Oras sa Tag-init ng Bangladesh"}, {"Brasilia", "Oras sa Tag-init ng Brasilia"}, {"Cape_Verde", "Oras sa Tag-init ng Cape Verde"}, {"Chatham", "Daylight Time sa Chatham"}, {"Chile", "Oras sa Tag-init ng Chile"}, {"China", "Daylight Time sa China"}, {"Colombia", "Oras sa Tag-init ng Colombia"}, {"Cook", "Oras sa Kalahati ng Tag-init ng Cook Islands"}, {"Cuba", "Daylight na Oras sa Cuba"}, {"Easter", "Oras sa Tag-init ng Easter Island"}, {"Europe_Central", "Oras sa Tag-init ng Gitnang Europe"}, {"Europe_Eastern", "Oras sa Tag-init ng Silangang Europe"}, {"Europe_Western", "Oras sa Tag-init ng Kanlurang Europe"}, {"Falkland", "Oras sa Tag-init ng Falkland Islands"}, {"Fiji", "Oras sa Tag-init ng Fiji"}, {"Georgia", "Oras sa Tag-init ng Georgia"}, {"Greenland_Eastern", "Oras sa Tag-init ng Silangang Greenland"}, {"Greenland_Western", "Oras sa Tag-init ng Kanlurang Greenland"}, {"Hawaii_Aleutian", "Oras sa Tag-init ng Hawaii-Aleutian"}, {"Hong_Kong", "Oras sa Tag-init ng Hong Kong"}, {"Hovd", "Oras sa Tag-init ng Hovd"}, {"Iran", "Daylight Time sa Iran"}, {"Irkutsk", "Oras sa Tag-init ng Irkutsk"}, {"Israel", "Daylight Time sa Israel"}, {"Japan", "Daylight Time sa Japan"}, {"Kamchatka", "Summer Time sa Petropavlovsk-Kamchatski"}, {"Korea", "Daylight Time sa Korea"}, {"Krasnoyarsk", "Oras sa Tag-init ng Krasnoyarsk"}, {"Lord_Howe", "Daylight Time sa Lorde Howe"}, {"Magadan", "Oras sa Tag-init ng Magadan"}, {"Mauritius", "Oras sa Tag-init ng Mauritius"}, {"Mexico_Pacific", "Daylight na Oras sa Pasipiko ng Mexico"}, {"Mongolia", "Oras sa Tag-init ng Ulan Bator"}, {"Moscow", "Oras sa Tag-init ng Moscow"}, {"New_Caledonia", "Oras sa Tag-init ng New Caledonia"}, {"New_Zealand", "Daylight Time sa New Zealand"}, {"Newfoundland", "Daylight na Oras sa Newfoundland"}, {"Norfolk", "Daylight Time sa Norfolk Island"}, {"Noronha", "Oras sa Tag-init ng Fernando de Noronha"}, {"Novosibirsk", "Oras sa Tag-init ng Novosibirsk"}, {"Omsk", "Oras sa Tag-init ng Omsk"}, {"Pakistan", "Oras sa Tag-init ng Pakistan"}, {"Paraguay", "Oras sa Tag-init ng Paraguay"}, {"Peru", "Oras sa Tag-init ng Peru"}, {"Philippines", "Oras sa Tag-init ng Pilipinas"}, {"Pierre_Miquelon", "Daylight na Oras sa Saint Pierre & Miquelon"}, {"Sakhalin", "Oras sa Tag-init ng Sakhalin"}, {"Samara", "Samara Daylight"}, {"Samoa", "Daylight Time sa Samoa"}, {"Taipei", "Daylight Time sa Taipei"}, {"Tonga", "Oras sa Tag-init ng Tonga"}, {"Turkmenistan", "Oras sa Tag-init ng Turkmenistan"}, {"Uruguay", "Oras sa Tag-init ng Uruguay"}, {"Uzbekistan", "Oras sa Tag-init ng Uzbekistan"}, {"Vanuatu", "Oras sa Tag-init ng Vanuatu"}, {"Vladivostok", "Oras sa Tag-init ng Vladivostok"}, {"Volgograd", "Oras sa Tag-init ng Volgograd"}, {"Yakutsk", "Oras sa Tag-init ng Yakutsk"}, {"Yekaterinburg", "Oras sa Tag-init ng Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "Afganistan tíð"}, {"Africa_Central", "Miðafrika tíð"}, {"Africa_Eastern", "Eysturafrika tíð"}, {"Africa_Southern", "Suðurafrika vanlig tíð"}, {"Africa_Western", "Vesturafrika tíð"}, {"Alaska", "Alaska vanlig tíð"}, {"Amazon", "Amasona vanlig tíð"}, {"America_Central", "Central vanlig tíð"}, {"America_Eastern", "Eastern vanlig tíð"}, {"America_Mountain", "Mountain vanlig tíð"}, {"America_Pacific", "Pacific vanlig tíð"}, {"Apia", "Apia vanlig tíð"}, {"Arabian", "Arabisk vanlig tíð"}, {"Argentina", "Argentina vanlig tíð"}, {"Argentina_Western", "Vestur Argentina vanlig tíð"}, {"Armenia", "Armenia vanlig tíð"}, {"Atlantic", "Atlantic vanlig tíð"}, {"Australia_Central", "mið Avstralia vanlig tíð"}, {"Australia_CentralWestern", "miðvestur Avstralia vanlig tíð"}, {"Australia_Eastern", "eystur Avstralia vanlig tíð"}, {"Australia_Western", "vestur Avstralia vanlig tíð"}, {"Azerbaijan", "Aserbadjan vanlig tíð"}, {"Azores", "Azorurnar vanlig tíð"}, {"Bangladesh", "Bangladesj vanlig tíð"}, {"Bhutan", "Butan tíð"}, {"Bolivia", "Bolivia tíð"}, {"Brasilia", "Brasilia vanlig tíð"}, {"Brunei", "Brunei Darussalam tíð"}, {"Cape_Verde", "Grønhøvdaoyggjar vanlig tíð"}, {"Chamorro", "Chamorro vanlig tíð"}, {"Chatham", "Chatham vanlig tíð"}, {"Chile", "Kili vanlig tíð"}, {"China", "Kina vanlig tíð"}, {"Christmas", "Jólaoyggj tíð"}, {"Cocos", "Kokosoyggjar tíð"}, {"Colombia", "Kolombia vanlig tíð"}, {"Cook", "Cooksoyggjar vanlig tíð"}, {"Cuba", "Cuba vanlig tíð"}, {"Davis", "Davis tíð"}, {"DumontDUrville", "Dumont-d’Urville tíð"}, {"East_Timor", "Eysturtimor tíð"}, {"Easter", "Páskaoyggin vanlig tíð"}, {"Ecuador", "Ekvador tíð"}, {"Europe_Central", "Miðevropa vanlig tíð"}, {"Europe_Eastern", "Eysturevropa vanlig tíð"}, {"Europe_Further_Eastern", "longri Eysturevropa tíð"}, {"Europe_Western", "Vesturevropa vanlig tíð"}, {"Falkland", "Falklandsoyggjar vanlig tíð"}, {"Fiji", "Fiji vanlig tíð"}, {"French_Guiana", "Franska Gujana tíð"}, {"French_Southern", "Fronsku sunnaru landaøki og Antarktis tíð"}, {"GMT", "Greenwich Mean tíð"}, {"Galapagos", "Galapagos tíð"}, {"Gambier", "Gambier tíð"}, {"Georgia", "Georgia vanlig tíð"}, {"Gilbert_Islands", "Gilbertoyggjar tíð"}, {"Greenland_Eastern", "Eystur grønlendsk vanlig tíð"}, {"Greenland_Western", "Vestur grønlendsk vanlig tíð"}, {"Gulf", "Gulf vanlig tíð"}, {"Guyana", "Gujana tíð"}, {"Hawaii", "Hawaii-Aleutian vanlig tíð"}, {"Hawaii_Aleutian", "Hawaii-Aleutian vanlig tíð"}, {"Hong_Kong", "Hong Kong vanlig tíð"}, {"Hovd", "Hovd vanlig tíð"}, {"India", "India tíð"}, {"Indian_Ocean", "Indiahav tíð"}, {"Indochina", "Indokina tíð"}, {"Indonesia_Central", "Mið Indonesia tíð"}, {"Indonesia_Eastern", "Eystur Indonesia tíð"}, {"Indonesia_Western", "Vestur Indonesia tíð"}, {"Iran", "Iran vanlig tíð"}, {"Irkutsk", "Irkutsk vanlig tíð"}, {"Israel", "Ísrael vanlig tíð"}, {"Japan", "Japan vanlig tíð"}, {"Kazakhstan", "Kasakstan tíð"}, {"Kazakhstan_Eastern", "Eystur Kasakstan tíð"}, {"Kazakhstan_Western", "Vestur Kasakstan tíð"}, {"Korea", "Korea vanlig tíð"}, {"Kosrae", "Kosrae tíð"}, {"Krasnoyarsk", "Krasnoyarsk vanlig tíð"}, {"Kyrgystan", "Kirgisia tíð"}, {"Line_Islands", "Lineoyggjar tíð"}, {"Lord_Howe", "Lord Howe vanlig tíð"}, {"Magadan", "Magadan vanlig tíð"}, {"Malaysia", "Malaisia tíð"}, {"Maldives", "Maldivoyggjar tíð"}, {"Marquesas", "Marquesas tíð"}, {"Marshall_Islands", "Marshalloyggjar tíð"}, {"Mauritius", "Móritius vanlig tíð"}, {"Mawson", "Mawson tíð"}, {"Mexico_Pacific", "Mexican Pacific vanlig tíð"}, {"Mongolia", "Ulan Bator vanlig tíð"}, {"Moscow", "Moskva vanlig tíð"}, {"Myanmar", "Myanmar (Burma) tíð"}, {"Nauru", "Nauru tíð"}, {"Nepal", "Nepal tíð"}, {"New_Caledonia", "Nýkaledónia vanlig tíð"}, {"New_Zealand", "Nýsæland vanlig tíð"}, {"Newfoundland", "Newfoundland vanlig tíð"}, {"Niue", "Niue tíð"}, {"Norfolk", "Norfolkoyggj vanlig tíð"}, {"Noronha", "Fernando de Noronha vanlig tíð"}, {"Novosibirsk", "Novosibirsk vanlig tíð"}, {"Omsk", "Omsk vanlig tíð"}, {"Pakistan", "Pakistan vanlig tíð"}, {"Palau", "Palau tíð"}, {"Papua_New_Guinea", "Papua Nýguinea tíð"}, {"Paraguay", "Paraguai vanlig tíð"}, {"Peru", "Peru vanlig tíð"}, {"Philippines", "Filipsoyggjar vanlig tíð"}, {"Phoenix_Islands", "Phoenixoyggjar tíð"}, {"Pierre_Miquelon", "St. Pierre & Miquelon vanlig tíð"}, {"Pitcairn", "Pitcairnoyggjar tíð"}, {"Ponape", "Ponape tíð"}, {"Pyongyang", "Pyongyang tíð"}, {"Reunion", "Réunion tíð"}, {"Rothera", "Rothera tíð"}, {"Sakhalin", "Sakhalin vanlig tíð"}, {"Samoa", "Samoa vanlig tíð"}, {"Seychelles", "Seyskelloyggjar tíð"}, {"Singapore", "Singapor tíð"}, {"Solomon", "Salomonoyggjar tíð"}, {"South_Georgia", "Suðurgeorgiaoyggjar tíð"}, {"Suriname", "Surinam tíð"}, {"Syowa", "Syowa tíð"}, {"Tahiti", "Tahiti tíð"}, {"Taipei", "Taipei vanlig tíð"}, {"Tajikistan", "Tadsjikistan tíð"}, {"Tokelau", "Tokelau tíð"}, {"Tonga", "Tonga vanlig tíð"}, {"Truk", "Chuuk tíð"}, {"Turkmenistan", "Turkmenistan vanlig tíð"}, {"Tuvalu", "Tuvalu tíð"}, {"Uruguay", "Uruguai vanlig tíð"}, {"Uzbekistan", "Usbekistan vanlig tíð"}, {"Vanuatu", "Vanuatu vanlig tíð"}, {"Venezuela", "Venesuela tíð"}, {"Vladivostok", "Vladivostok vanlig tíð"}, {"Volgograd", "Volgograd vanlig tíð"}, {"Vostok", "Vostok tíð"}, {"Wake", "Wakeoyggj tíð"}, {"Wallis", "Wallis- og Futunaoyggjar tíð"}, {"Yakutsk", "Yakutsk vanlig tíð"}, {"Yekaterinburg", "Yekaterinburg vanlig tíð"}, {"Yukon", "Yukon tíð"}},
		{{"Alaska", "Alaska summartíð"}, {"Amazon", "Amasona summartíð"}, {"America_Central", "Central summartíð"}, {"America_Eastern", "Eastern summartíð"}, {"America_Mountain", "Mountain summartíð"}, {"America_Pacific", "Pacific summartíð"}, {"Apia", "Apia summartíð"}, {"Arabian", "Arabisk summartíð"}, {"Argentina", "Argentina summartíð"}, {"Argentina_Western", "Vestur Argentina summartíð"}, {"Armenia", "Armenia summartíð"}, {"Atlantic", "Atlantic summartíð"}, {"Australia_Central", "mið Avstralia summartíð"}, {"Australia_CentralWestern", "miðvestur Avstralia summartíð"}, {"Australia_Eastern", "eystur Avstralia summartíð"}, {"Australia_Western", "vestur Avstralia summartíð"}, {"Azerbaijan", "Aserbadjan summartíð"}, {"Azores", "Azorurnar summartíð"}, {"Bangladesh", "Bangladesj summartíð"}, {"Brasilia", "Brasilia summartíð"}, {"Cape_Verde", "Grønhøvdaoyggjar summartíð"}, {"Chatham", "Chatham summartíð"}, {"Chile", "Kili summartíð"}, {"China", "Kina summartíð"}, {"Colombia", "Kolombia summartíð"}, {"Cook", "Cooksoyggjar summartíð"}, {"Cuba", "Cuba summartíð"}, {"Easter", "Páskaoyggin summartíð"}, {"Europe_Central", "Miðevropa summartíð"}, {"Europe_Eastern", "Eysturevropa summartíð"}, {"Europe_Western", "Vesturevropa summartíð"}, {"Falkland", "Falklandsoyggjar summartíð"}, {"Fiji", "Fiji summartíð"}, {"Georgia", "Georgia summartíð"}, {"Greenland_Eastern", "Eystur grønlendsk summartíð"}, {"Greenland_Western", "Vestur grønlendsk summartíð"}, {"Hawaii_Aleutian", "Hawaii-Aleutian summartíð"}, {"Hong_Kong", "Hong Kong summartíð"}, {"Hovd", "Hovd summartíð"}, {"Iran", "Iran summartíð"}, {"Irkutsk", "Irkutsk summartíð"}, {"Israel", "Ísrael summartíð"}, {"Japan", "Japan summartíð"}, {"Korea", "Korea summartíð"}, {"Krasnoyarsk", "Krasnoyarsk summartíð"}, {"Lord_Howe", "Lord Howe summartíð"}, {"Magadan", "Magadan summartíð"}, {"Mauritius", "Móritius summartíð"}, {"Mexico_Pacific", "Mexican Pacific summartíð"}, {"Mongolia", "Ulan Bator summartíð"}, {"Moscow", "Moskva summartíð"}, {"New_Caledonia", "Nýkaledónia summartíð"}, {"New_Zealand", "Nýsæland summartíð"}, {"Newfoundland", "Newfoundland summartíð"}, {"Norfolk", "Norfolkoyggj summartíð"}, {"Noronha", "Fernando de Noronha summartíð"}, {"Novosibirsk", "Novosibirsk summartíð"}, {"Omsk", "Omsk summartíð"}, {"Pakistan", "Pakistan summartíð"}, {"Paraguay", "Paraguai summartíð"}, {"Peru", "Peru summartíð"}, {"Philippines", "Filipsoyggjar summartíð"}, {"Pierre_Miquelon", "St. Pierre & Miquelon summartíð"}, {"Sakhalin", "Sakhalin summartíð"}, {"Samoa", "Samoa summartíð"}, {"Taipei", "Taipei summartíð"}, {"Tonga", "Tonga summartíð"}, {"Turkmenistan", "Turkmenistan summartíð"}, {"Uruguay", "Uruguai summartíð"}, {"Uzbekistan", "Usbekistan summartíð"}, {"Vanuatu", "Vanuatu summartíð"}, {"Vladivostok", "Vladivostok summartíð"}, {"Volgograd", "Volgograd summartíð"}, {"Yakutsk", "Yakutsk summartíð"}, {"Yekaterinburg", "Yekaterinburg summartíð"}},
	},
	{
		{{"Acre", "heure normale de l’Acre"}, {"Afghanistan", "heure de l’Afghanistan"}, {"Africa_Central", "heure normale d’Afrique centrale"}, {"Africa_Eastern", "heure normale d’Afrique de l’Est"}, {"Africa_Southern", "heure normale d’Afrique méridionale"}, {"Africa_Western", "heure d’Afrique de l’Ouest"}, {"Alaska", "heure normale de l’Alaska"}, {"Almaty", "heure normale d’Alma Ata"}, {"Amazon", "heure normale de l’Amazonie"}, {"America_Central", "heure normale du centre nord-américain"}, {"America_Eastern", "heure normale de l’Est nord-américain"}, {"America_Mountain", "heure normale des Rocheuses"}, {"America_Pacific", "heure normale du Pacifique nord-américain"}, {"Anadyr", "heure normale d’Anadyr"}, {"Apia", "heure normale d’Apia"}, {"Aqtau", "heure normale d’Aktaou"}, {"Aqtobe", "heure normale d’Aqtöbe"}, {"Arabian", "heure normale de l’Arabie"}, {"Argentina", "heure normale d’Argentine"}, {"Argentina_Western", "heure normale de l’Ouest argentin"}, {"Armenia", "heure normale de l’Arménie"}, {"Atlantic", "heure normale de l’Atlantique"}, {"Australia_Central", "heure normale du centre de l’Australie"}, {"Australia_CentralWestern", "heure normale du centre-ouest de l’Australie"}, {"Australia_Eastern", "heure normale de l’Est de l’Australie"}, {"Australia_Western", "heure normale de l’Ouest de l’Australie"}, {"Azerbaijan", "heure normale de l’Azerbaïdjan"}, {"Azores", "heure normale des Açores"}, {"Bangladesh", "heure normale du Bangladesh"}, {"Bhutan", "heure du Bhoutan"}, {"Bolivia", "heure de Bolivie"}, {"Brasilia", "heure normale de Brasilia"}, {"Brunei", "heure du Brunei"}, {"Cape_Verde", "heure normale du Cap-Vert"}, {"Casey", "heure de Casey"}, {"Chamorro", "heure des Chamorro"}, {"Chatham", "heure normale des îles Chatham"}, {"Chile", "heure normale du Chili"}, {"China", "heure normale de la Chine"}, {"Christmas", "heure de l’île Christmas"}, {"Cocos", "heure des îles Cocos"}, {"Colombia", "heure normale de Colombie"}, {"Cook", "heure normale des îles Cook"}, {"Cuba", "heure normale de Cuba"}, {"Davis", "heure de Davis"}, {"DumontDUrville", "heure de Dumont-d’Urville"}, {"East_Timor", "heure du Timor oriental"}, {"Easter", "heure normale de l’île de Pâques"}, {"Ecuador", "heure de l’Équateur"}, {"Europe_Central", "heure normale d’Europe centrale"}, {"Europe_Eastern", "heure normale d’Europe de l’Est"}, {"Europe_Further_Eastern", "heure de Kaliningrad"}, {"Europe_Western", "heure normale d’Europe de l’Ouest"}, {"Falkland", "heure normale des îles Malouines"}, {"Fiji", "heure normale des îles Fidji"}, {"French_Guiana", "heure de la Guyane française"}, {"French_Southern", "heure des Terres australes et antarctiques françaises"}, {"GMT", "heure moyenne de Greenwich"}, {"Galapagos", "heure des îles Galápagos"}, {"Gambier", "heure des îles Gambier"}, {"Georgia", "heure normale de la Géorgie"}, {"Gilbert_Islands", "heure des îles Gilbert"}, {"Greenland_Eastern", "heure normale de l’Est du Groenland"}, {"Greenland_Western", "heure normale de l’Ouest du Groenland"}, {"Guam", "heure de Guam"}, {"Gulf", "heure du Golfe"}, {"Guyana", "heure du Guyana"}, {"Hawaii", "heure normale d’Hawaï - Aléoutiennes"}, {"Hawaii_Aleutian", "heure normale d’Hawaï - Aléoutiennes"}, {"Hong_Kong", "heure normale de Hong Kong"}, {"Hovd", "heure normale de Hovd"}, {"India", "heure de l’Inde"}, {"Indian_Ocean", "heure de l’Océan Indien"}, {"Indochina", "heure d’Indochine"}, {"Indonesia_Central", "heure du Centre indonésien"}, {"Indonesia_Eastern", "heure de l’Est indonésien"}, {"Indonesia_Western", "heure de l’Ouest indonésien"}, {"Iran", "heure normale d’Iran"}, {"Irkutsk", "heure normale d’Irkoutsk"}, {"Israel", "heure normale d’Israël"}, {"Japan", "heure normale du Japon"}, {"Kamchatka", "heure normale de Petropavlovsk-Kamchatski"}, {"Kazakhstan", "heure du Kazakhstan"}, {"Kazakhstan_Eastern", "heure de l’Est du Kazakhstan"}, {"Kazakhstan_Western", "heure de l’Ouest du Kazakhstan"}, {"Korea", "heure normale de la Corée"}, {"Kosrae", "heure de Kosrae"}, {"Krasnoyarsk", "heure normale de Krasnoïarsk"}, {"Kyrgystan", "heure du Kirghizistan"}, {"Lanka", "heure de Lanka"}, {"Line_Islands", "heure des îles de la Ligne"}, {"Lord_Howe", "heure normale de Lord Howe"}, {"Macau", "heure normale de Macao"}, {"Magadan", "heure normale de Magadan"}, {"Malaysia", "heure de la Malaisie"}, {"Maldives", "heure des Maldives"}, {"Marquesas", "heure des îles Marquises"}, {"Marshall_Islands", "heure des îles Marshall"}, {"Mauritius", "heure normale de Maurice"}, {"Mawson", "heure de Mawson"}, {"Mexico_Pacific", "heure normale du Pacifique mexicain"}, {"Mongolia", "heure normale d’Oulan-Bator"}, {"Moscow", "heure normale de Moscou"}, {"Myanmar", "heure du Myanmar"}, {"Nauru", "heure de Nauru"}, {"Nepal", "heure du Népal"}, {"New_Caledonia", "heure normale de la Nouvelle-Calédonie"}, {"New_Zealand", "heure normale de la Nouvelle-Zélande"}, {"Newfoundland", "heure normale de Terre-Neuve"}, {"Niue", "heure de Niue"}, {"Norfolk", "heure normale de l’île Norfolk"}, {"Noronha", "heure normale de Fernando de Noronha"}, {"North_Mariana", "heure des îles Mariannes du Nord"}, {"Novosibirsk", "heure normale de Novossibirsk"}, {"Omsk", "heure normale de Omsk"}, {"Pakistan", "heure normale du Pakistan"}, {"Palau", "heure des Palaos"}, {"Papua_New_Guinea", "heure de la Papouasie-Nouvelle-Guinée"}, {"Paraguay", "heure normale du Paraguay"}, {"Peru", "heure normale du Pérou"}, {"Philippines", "heure normale des Philippines"}, {"Phoenix_Islands", "heure des îles Phoenix"}, {"Pierre_Miquelon", "heure normale de Saint-Pierre-et-Miquelon"}, {"Pitcairn", "heure des îles Pitcairn"}, {"Ponape", "heure de l’île de Pohnpei"}, {"Pyongyang", "heure de Pyongyang"}, {"Qyzylorda", "heure normale de Kyzylorda"}, {"Reunion", "heure de La Réunion"}, {"Rothera", "heure de Rothera"}, {"Sakhalin", "heure normale de Sakhaline"}, {"Samara", "heure normale de Samara"}, {"Samoa", "heure normale des Samoa"}, {"Seychelles", "heure des Seychelles"}, {"Singapore", "heure de Singapour"}, {"Solomon", "heure des îles Salomon"}, {"South_Georgia", "heure de Géorgie du Sud"}, {"Suriname", "heure du Suriname"}, {"Syowa", "heure de Syowa"}, {"Tahiti", "heure de Tahiti"}, {"Taipei", "heure normale de Taipei"}, {"Tajikistan", "heure du Tadjikistan"}, {"Tokelau", "heure de Tokelau"}, {"Tonga", "heure normale des Tonga"}, {"Truk", "heure de Chuuk"}, {"Turkey", "heure normale de Turquie"}, {"Turkmenistan", "heure normale du Turkménistan"}, {"Tuvalu", "heure des Tuvalu"}, {"Uruguay", "heure normale de l’Uruguay"}, {"Uzbekistan", "heure normale de l’Ouzbékistan"}, {"Vanuatu", "heure normale du Vanuatu"}, {"Venezuela", "heure du Venezuela"}, {"Vladivostok", "heure normale de Vladivostok"}, {"Volgograd", "heure normale de Volgograd"}, {"Vostok", "heure de Vostok"}, {"Wake", "heure de l’île Wake"}, {"Wallis", "heure de Wallis-et-Futuna"}, {"Yakutsk", "heure normale de Iakoutsk"}, {"Yekaterinburg", "heure normale d’Ekaterinbourg"}, {"Yukon", "heure normale du Yukon"}},
		{{"Acre", "heure d’été de l’Acre"}, {"Alaska", "heure d’été de l’Alaska"}, {"Almaty", "heure d’été d’Alma Ata"}, {"Amazon", "heure d’été de l’Amazonie"}, {"America_Central", "heure d’été du centre nord-américain"}, {"America_Eastern", "heure d’été de l’Est nord-américain"}, {"America_Mountain", "heure d’été des Rocheuses"}, {"America_Pacific", "heure d’été du Pacifique nord-américain"}, {"Anadyr", "heure d’été d’Anadyr"}, {"Apia", "heure d’été d’Apia"}, {"Aqtau", "heure d’été d’Aktaou"}, {"Aqtobe", "heure d’été d’Aqtöbe"}, {"Arabian", "heure d’été de l’Arabie"}, {"Argentina", "heure d’été de l’Argentine"}, {"Argentina_Western", "heure d’été de l’Ouest argentin"}, {"Armenia", "heure d’été d’Arménie"}, {"Atlantic", "heure d’été de l’Atlantique"}, {"Australia_Central", "heure d’été du centre de l’Australie"}, {"Australia_CentralWestern", "heure d’été du centre-ouest de l’Australie"}, {"Australia_Eastern", "heure d’été de l’Est de l’Australie"}, {"Australia_Western", "heure d’été de l’Ouest de l’Australie"}, {"Azerbaijan", "heure d’été d’Azerbaïdjan"}, {"Azores", "heure d’été des Açores"}, {"Bangladesh", "heure d’été du Bangladesh"}, {"Brasilia", "heure d’été de Brasilia"}, {"Cape_Verde", "heure d’été du Cap-Vert"}, {"Chatham", "heure d’été des îles Chatham"}, {"Chile", "heure d’été du Chili"}, {"China", "heure d’été de Chine"}, {"Colombia", "heure d’été de Colombie"}, {"Cook", "heure d’été des îles Cook"}, {"Cuba", "heure d’été de Cuba"}, {"Easter", "heure d’été de l’île de Pâques"}, {"Europe_Central", "heure d’été d’Europe centrale"}, {"Europe_Eastern", "heure d’été d’Europe de l’Est"}, {"Europe_Western", "heure d’été d’Europe de l’Ouest"}, {"Falkland", "heure d’été des îles Malouines"}, {"Fiji", "heure d’été des îles Fidji"}, {"Georgia", "heure d’été de Géorgie"}, {"Greenland_Eastern", "heure d’été de l’Est du Groenland"}, {"Greenland_Western", "heure d’été de l’Ouest du Groenland"}, {"Hawaii_Aleutian", "heure d’été d’Hawaï - Aléoutiennes"}, {"Hong_Kong", "heure d’été de Hong Kong"}, {"Hovd", "heure d’été de Hovd"}, {"Iran", "heure d’été d’Iran"}, {"Irkutsk", "heure d’été d’Irkoutsk"}, {"Israel", "heure d’été d’Israël"}, {"Japan", "heure d’été du Japon"}, {"Kamchatka", "heure d’été de Petropavlovsk-Kamchatski"}, {"Korea", "heure d’été de Corée"}, {"Krasnoyarsk", "heure d’été de Krasnoïarsk"}, {"Lord_Howe", "heure d’été de Lord Howe"}, {"Macau", "heure d’été de Macao"}, {"Magadan", "heure d’été de Magadan"}, {"Mauritius", "heure d’été de Maurice"}, {"Mexico_Pacific", "heure d’été du Pacifique mexicain"}, {"Mongolia", "heure d’été d’Oulan-Bator"}, {"Moscow", "heure d’été de Moscou"}, {"New_Caledonia", "heure d’été de Nouvelle-Calédonie"}, {"New_Zealand", "heure d’été de la Nouvelle-Zélande"}, {"Newfoundland", "heure d’été de Terre-Neuve"}, {"Norfolk", "heure d’été de l’île Norfolk"}, {"Noronha", "heure d’été de Fernando de Noronha"}, {"Novosibirsk", "heure d’été de Novossibirsk"}, {"Omsk", "heure d’été de Omsk"}, {"Pakistan", "heure d’été du Pakistan"}, {"Paraguay", "heure d’été du Paraguay"}, {"Peru", "heure d’été du Pérou"}, {"Philippines", "heure d’été des Philippines"}, {"Pierre_Miquelon", "heure d’été de Saint-Pierre-et-Miquelon"}, {"Qyzylorda", "heure d’été de Kyzylorda"}, {"Sakhalin", "heure d’été de Sakhaline"}, {"Samara", "heure d’été de Samara"}, {"Samoa", "heure d’été des Samoa"}, {"Taipei", "heure d’été de Taipei"}, {"Tonga", "heure d’été de Tonga"}, {"Turkey", "heure avancée de Turquie"}, {"Turkmenistan", "heure d’été du Turkménistan"}, {"Uruguay", "heure d’été de l’Uruguay"}, {"Uzbekistan", "heure d’été de l’Ouzbékistan"}, {"Vanuatu", "heure d’été de Vanuatu"}, {"Vladivostok", "heure d’été de Vladivostok"}, {"Volgograd", "heure d’été de Volgograd"}, {"Yakutsk", "heure d’été de Iakoutsk"}, {"Yekaterinburg", "heure d’été d’Ekaterinbourg"}},
	},
	{
		{{"Acre", "heure normale de l’Acre"}, {"Afghanistan", "heure de l’Afghanistan"}, {"Africa_Central", "heure d’Afrique centrale"}, {"Africa_Eastern", "heure d’Afrique orientale"}, {"Africa_Southern", "heure normale d’Afrique du Sud"}, {"Africa_Western", "heure d’Afrique de l’Ouest"}, {"Alaska", "heure normale de l’Alaska"}, {"Almaty", "heure normale d’Alma Ata"}, {"Amazon", "heure normale de l’Amazonie"}, {"America_Central", "heure normale du Centre"}, {"America_Eastern", "heure normale de l’Est"}, {"America_Mountain", "heure normale des Rocheuses"}, {"America_Pacific", "heure normale du Pacifique"}, {"Anadyr", "heure normale d’Anadyr"}, {"Apia", "heure normale d’Apia"}, {"Aqtau", "heure normale d’Aktaou"}, {"Aqtobe", "heure normale d’Aqtöbe"}, {"Arabian", "heure normale de l’Arabie"}, {"Argentina", "heure normale d’Argentine"}, {"Argentina_Western", "heure normale de l’Ouest argentin"}, {"Armenia", "heure normale de l’Arménie"}, {"Atlantic", "heure normale de l’Atlantique"}, {"Australia_Central", "heure normale du centre de l’Australie"}, {"Australia_CentralWestern", "heure normale du centre-ouest de l’Australie"}, {"Australia_Eastern", "heure normale de l’Est de l’Australie"}, {"Australia_Western", "heure normale de l’Ouest de l’Australie"}, {"Azerbaijan", "heure normale de l’Azerbaïdjan"}, {"Azores", "heure normale des Açores"}, {"Bangladesh", "heure normale du Bangladesh"}, {"Bhutan", "heure du Bhoutan"}, {"Bolivia", "heure de Bolivie"}, {"Brasilia", "heure normale de Brasilia"}, {"Brunei", "heure du Brunéi"}, {"Cape_Verde", "heure normale du Cap-Vert"}, {"Casey", "heure de Casey"}, {"Chamorro", "heure des Chamorro"}, {"Chatham", "heure normale des îles Chatham"}, {"Chile", "heure normale du Chili"}, {"China", "heure normale de Chine"}, {"Christmas", "heure de l’île Christmas"}, {"Cocos", "heure des îles Cocos"}, {"Colombia", "heure normale de Colombie"}, {"Cook", "heure normale des îles Cook"}, {"Cuba", "heure normale de Cuba"}, {"Davis", "heure de Davis"}, {"DumontDUrville", "heure de Dumont-d’Urville"}, {"East_Timor", "heure du Timor oriental"}, {"Easter", "heure normale de l’île de Pâques"}, {"Ecuador", "heure de l’Équateur"}, {"Europe_Central", "heure normale de l’Europe centrale"}, {"Europe_Eastern", "heure normale de l’Europe de l’Est"}, {"Europe_Further_Eastern", "heure de Kaliningrad"}, {"Europe_Western", "heure normale de l’Europe de l’Ouest"}, {"Falkland", "heure normale des îles Malouines"}, {"Fiji", "heure normale des îles Fidji"}, {"French_Guiana", "heure de Guyane française"}, {"French_Southern", "heure des Terres australes et antarctiques françaises"}, {"GMT", "heure moyenne de Greenwich"}, {"Galapagos", "heure des îles Galápagos"}, {"Gambier", "heure des îles Gambier"}, {"Georgia", "heure normale de la Géorgie"}, {"Gilbert_Islands", "heure des îles Gilbert"}, {"Greenland_Eastern", "heure normale de l’Est du Groenland"}, {"Greenland_Western", "heure normale de l’Ouest du Groenland"}, {"Guam", "heure de Guam"}, {"Gulf", "heure du Golfe"}, {"Guyana", "heure du Guyana"}, {"Hawaii", "heure normale d’Hawaï-Aléoutiennes"}, {"Hawaii_Aleutian", "heure normale d’Hawaï-Aléoutiennes"}, {"Hong_Kong", "heure normale de Hong Kong"}, {"Hovd", "heure normale de Hovd"}, {"India", "heure de l’Inde"}, {"Indian_Ocean", "heure de l’océan Indien"}, {"Indochina", "heure d’Indochine"}, {"Indonesia_Central", "heure du Centre indonésien"}, {"Indonesia_Eastern", "heure de l’Est indonésien"}, {"Indonesia_Western", "heure de l’Ouest indonésien"}, {"Iran", "heure normale d’Iran"}, {"Irkutsk", "heure normale d’Irkoutsk"}, {"Israel", "heure normale d’Israël"}, {"Japan", "heure normale du Japon"}, {"Kamchatka", "heure normale de Petropavlovsk-Kamchatski"}, {"Kazakhstan", "heure du Kazakhstan"}, {"Kazakhstan_Eastern", "heure de l’Est du Kazakhstan"}, {"Kazakhstan_Western", "heure de l’Ouest du Kazakhstan"}, {"Korea", "heure normale de la Corée"}, {"Kosrae", "heure de Kosrae"}, {"Krasnoyarsk", "heure normale de Krasnoïarsk"}, {"Kyrgystan", "heure du Kirghizistan"}, {"Lanka", "heure de Lanka"}, {"Line_Islands", "heure des îles de la Ligne"}, {"Lord_Howe", "heure normale de Lord Howe"}, {"Macau", "heure normale de Macao"}, {"Magadan", "heure normale de Magadan"}, {"Malaysia", "heure de la Malaisie"}, {"Maldives", "heure des Maldives"}, {"Marquesas", "heure des îles Marquises"}, {"Marshall_Islands", "heure des îles Marshall"}, {"Mauritius", "heure normale de Maurice"}, {"Mawson", "heure de Mawson"}, {"Mexico_Pacific", "heure normale du Pacifique mexicain"}, {"Mongolia", "heure normale d’Oulan-Bator"}, {"Moscow", "heure normale de Moscou"}, {"Myanmar", "heure du Myanmar"}, {"Nauru", "heure de Nauru"}, {"Nepal", "heure du Népal"}, {"New_Caledonia", "heure normale de la Nouvelle-Calédonie"}, {"New_Zealand", "heure normale de la Nouvelle-Zélande"}, {"Newfoundland", "heure normale de Terre-Neuve"}, {"Niue", "heure de Nioué"}, {"Norfolk", "heure normale de l’île Norfolk"}, {"Noronha", "heure normale de Fernando de Noronha"}, {"North_Mariana", "heure des îles Mariannes du Nord"}, {"Novosibirsk", "heure normale de Novossibirsk"}, {"Omsk", "heure normale d’Omsk"}, {"Pakistan", "heure normale du Pakistan"}, {"Palau", "heure des Palaos"}, {"Papua_New_Guinea", "heure de la Papouasie-Nouvelle-Guinée"}, {"Paraguay", "heure normale du Paraguay"}, {"Peru", "heure normale du Pérou"}, {"Philippines", "heure normale des Philippines"}, {"Phoenix_Islands", "heure des îles Phoenix"}, {"Pierre_Miquelon", "heure normale de Saint-Pierre-et-Miquelon"}, {"Pitcairn", "heure des îles Pitcairn"}, {"Ponape", "heure de l’île de Pohnpei"}, {"Pyongyang", "heure de Pyongyang"}, {"Qyzylorda", "heure normale de Kyzylorda"}, {"Reunion", "heure de la Réunion"}, {"Rothera", "heure de Rothera"}, {"Sakhalin", "heure normale de Sakhaline"}, {"Samara", "heure normale de Samara"}, {"Samoa", "heure normale des Samoa"}, {"Seychelles", "heure des Seychelles"}, {"Singapore", "heure de Singapour"}, {"Solomon", "heure des îles Salomon"}, {"South_Georgia", "heure de Géorgie du Sud"}, {"Suriname", "heure du Suriname"}, {"Syowa", "heure de Syowa"}, {"Tahiti", "heure de Tahiti"}, {"Taipei", "heure normale de Taipei"}, {"Tajikistan", "heure du Tadjikistan"}, {"Tokelau", "heure de Tokelau"}, {"Tonga", "heure normale des Tonga"}, {"Truk", "heure de Chuuk"}, {"Turkey", "heure normale de Turquie"}, {"Turkmenistan", "heure normale du Turkménistan"}, {"Tuvalu", "heure des Tuvalu"}, {"Uruguay", "heure normale de l’Uruguay"}, {"Uzbekistan", "heure normale de l’Ouzbékistan"}, {"Vanuatu", "heure normale du Vanuatu"}, {"Venezuela", "heure du Venezuela"}, {"Vladivostok", "heure normale de Vladivostok"}, {"Volgograd", "heure normale de Volgograd"}, {"Vostok", "heure de Vostok"}, {"Wake", "heure de l’île Wake"}, {"Wallis", "heure de Wallis-et-Futuna"}, {"Yakutsk", "heure normale de Iakoutsk"}, {"Yekaterinburg", "heure normale d’Ekaterinbourg"}, {"Yukon", "heure normale du Yukon"}},
		{{"Acre", "heure avancée de l’Acre"}, {"Alaska", "heure avancée de l’Alaska"}, {"Almaty", "heure avancée d’Alma Ata"}, {"Amazon", "heure avancée de l’Amazonie"}, {"America_Central", "heure avancée du Centre"}, {"America_Eastern", "heure avancée de l’Est"}, {"America_Mountain", "heure avancée des Rocheuses"}, {"America_Pacific", "heure avancée du Pacifique"}, {"Anadyr", "heure avancée d’Anadyr"}, {"Apia", "heure avancée d’Apia"}, {"Aqtau", "heure avancée d’Aktaou"}, {"Aqtobe", "heure avancée d’Aqtöbe"}, {"Arabian", "heure avancée de l’Arabie"}, {"Argentina", "heure avancée de l’Argentine"}, {"Argentina_Western", "heure avancée de l’Ouest argentin"}, {"Armenia", "heure avancée d’Arménie"}, {"Atlantic", "heure avancée de l’Atlantique"}, {"Australia_Central", "heure avancée du centre de l’Australie"}, {"Australia_CentralWestern", "heure avancée du centre-ouest de l’Australie"}, {"Australia_Eastern", "heure avancée de l’Est de l’Australie"}, {"Australia_Western", "heure avancée de l’Ouest de l’Australie"}, {"Azerbaijan", "heure avancée d’Azerbaïdjan"}, {"Azores", "heure avancée des Açores"}, {"Bangladesh", "heure avancée du Bangladesh"}, {"Brasilia", "heure avancée de Brasilia"}, {"Cape_Verde", "heure avancée du Cap-Vert"}, {"Chatham", "heure d’été des îles Chatham"}, {"Chile", "heure avancée du Chili"}, {"China", "heure avancée de Chine"}, {"Colombia", "heure avancée de Colombie"}, {"Cook", "heure avancée des îles Cook"}, {"Cuba", "heure avancée de Cuba"}, {"Easter", "heure avancée de l’île de Pâques"}, {"Europe_Central", "heure avancée de l’Europe centrale"}, {"Europe_Eastern", "heure avancée de l’Europe de l’Est"}, {"Europe_Western", "heure avancée de l’Europe de l’Ouest"}, {"Falkland", "heure avancée des îles Malouines"}, {"Fiji", "heure avancée des îles Fidji"}, {"Georgia", "heure avancée de Géorgie"}, {"Greenland_Eastern", "heure avancée de l’Est du Groenland"}, {"Greenland_Western", "heure avancée de l’Ouest du Groenland"}, {"Hawaii_Aleutian", "heure avancée d’Hawaï-Aléoutiennes"}, {"Hong_Kong", "heure avancée de Hong Kong"}, {"Hovd", "heure avancée de Hovd"}, {"Iran", "heure avancée d’Iran"}, {"Irkutsk", "heure avancée d’Irkoutsk"}, {"Israel", "heure avancée d’Israël"}, {"Japan", "heure avancée du Japon"}, {"Kamchatka", "heure avancée de Petropavlovsk-Kamchatski"}, {"Korea", "heure avancée de Corée"}, {"Krasnoyarsk", "heure avancée de Krasnoïarsk"}, {"Lord_Howe", "heure avancée de Lord Howe"}, {"Macau", "heure avancée de Macao"}, {"Magadan", "heure avancée de Magadan"}, {"Mauritius", "heure avancée de Maurice"}, {"Mexico_Pacific", "heure avancée du Pacifique mexicain"}, {"Mongolia", "heure avancée d’Oulan-Bator"}, {"Moscow", "heure avancée de Moscou"}, {"New_Caledonia", "heure avancée de Nouvelle-Calédonie"}, {"New_Zealand", "heure avancée de la Nouvelle-Zélande"}, {"Newfoundland", "heure avancée de Terre-Neuve"}, {"Norfolk", "heure avancée de l’île Norfolk"}, {"Noronha", "heure avancée de Fernando de Noronha"}, {"Novosibirsk", "heure avancée de Novossibirsk"}, {"Omsk", "heure avancée d’Omsk"}, {"Pakistan", "heure avancée du Pakistan"}, {"Paraguay", "heure avancée du Paraguay"}, {"Peru", "heure avancée du Pérou"}, {"Philippines", "heure avancée des Philippines"}, {"Pierre_Miquelon", "heure avancée de Saint-Pierre-et-Miquelon"}, {"Qyzylorda", "heure d’été de Kyzylorda"}, {"Sakhalin", "heure avancée de Sakhaline"}, {"Samara", "heure d’été de Samara"}, {"Samoa", "heure avancée des Samoa"}, {"Taipei", "heure avancée de Taipei"}, {"Tonga", "heure avancée de Tonga"}, {"Turkey", "heure avancée de Turquie"}, {"Turkmenistan", "heure avancée du Turkménistan"}, {"Uruguay", "heure avancée de l’Uruguay"}, {"Uzbekistan", "heure avancée de l’Ouzbékistan"}, {"Vanuatu", "heure avancée de Vanuatu"}, {"Vladivostok", "heure avancée de Vladivostok"}, {"Volgograd", "heure avancée de Volgograd"}, {"Yakutsk", "heure avancée de Iakoutsk"}, {"Yekaterinburg", "heure avancée d’Ekaterinbourg"}},
	},
	{
		{{"Afghanistan", "Afghaanistaan Tidj"}, {"Africa_Central", "Sentraal Afrikoo Tidj"}, {"Africa_Eastern", "Uast Afrikoo Tidj"}, {"Africa_Southern", "Süüdelk Afrikoo Standard Tidj"}, {"Africa_Western", "Waast Afrikoo Tidj"}, {"Alaska", "Alaska Standard Tidj"}, {"Amazon", "Amazonas Standard Tidj"}, {"America_Central", "Sentraal Ameerikoo Standard Tidj"}, {"America_Eastern", "Uast Ameerikoo Standard Tidj"}, {"America_Mountain", "Waast Ameerikoo Standard Tidj"}, {"America_Pacific", "Ameerikoo Pasiifik Standard Tidj"}, {"Apia", "Samoa Standard Tidj"}, {"Arabian", "Araabisk Standard Tidj"}, {"Argentina", "Argentiinien Standard Tidj"}, {"Argentina_Western", "Waast Argentiinien Standard Tidj"}, {"Armenia", "Armeenien Standard Tidj"}, {"Atlantic", "Ameerikoo Atlantik Standard Tidj"}, {"Australia_Central", "Austraalisk Sentraal Standard Tidj"}, {"Australia_CentralWestern", "Austraalisk Madelwaast Standard Tidj"}, {"Australia_Eastern", "Uast Austraalisk Standard Tidj"}, {"Australia_Western", "Waast Austraalisk Standard Tidj"}, {"Azerbaijan", "Aserbaidschaan Standard Tidj"}, {"Azores", "Atsooren Standard Tidj"}, {"Bangladesh", "Bangladesch Standard Tidj"}, {"Bhutan", "Bhuutaan Tidj"}, {"Bolivia", "Boliiwien Tidj"}, {"Brasilia", "Brasiilien Standard Tidj"}, {"Brunei", "Brunei Tidj"}, {"Cape_Verde", "Kapwerden Standard Tidj"}, {"Chamorro", "Chamorro Tidj"}, {"Chatham", "Chatham Standard Tidj"}, {"Chile", "Chiile Standard Tidj"}, {"China", "Schiina Standard Tidj"}, {"Christmas", "Jul Eilun Tidj"}, {"Cocos", "Cocos Eilunen Tidj"}, {"Colombia", "Kolumbien Standard Tidj"}, {"Cook", "Cook Eilunen Standard Tidj"}, {"Cuba", "Kuuba Standard Tidj"}, {"Davis", "Davis Tidj"}, {"DumontDUrville", "Dumont d’Urville Tidj"}, {"East_Timor", "Uast Tiimor Tidj"}, {"Easter", "Puask Eilun Standard Tidj"}, {"Ecuador", "Ekwadoor Tidj"}, {"Europe_Central", "Madeleuropeesk Standard Tidj"}, {"Europe_Eastern", "Uasteuropeesk Standard Tidj"}, {"Europe_Further_Eastern", "Faarder Uasteuropeesk Tidj"}, {"Europe_Western", "Waasteuropeesk Standard Tidj"}, {"Falkland", "Falklun Eilunen Standard Tidj"}, {"Fiji", "Fidschi Standard Tidj"}, {"French_Guiana", "Fransöösk Guayaana Tidj"}, {"French_Southern", "Fransöösk Süüd an Antarktis Tidj"}, {"GMT", "Madel Greenwich Tidj"}, {"Galapagos", "Galapagos Tidj"}, {"Gambier", "Gambier Tidj"}, {"Georgia", "Georgien Standard Tidj"}, {"Gilbert_Islands", "Gilbert Eilunen Tidj"}, {"Greenland_Eastern", "Uast Greenlun Standard Tidj"}, {"Greenland_Western", "Waast Greenlun Standard Tidj"}, {"Gulf", "Golf Tidj"}, {"Guyana", "Guyaana Tidj"}, {"Hawaii", "Hawaii-Aleuten Standard Tidj"}, {"Hawaii_Aleutian", "Hawaii-Aleuten Standard Tidj"}, {"Hong_Kong", "Hongkong Standard Tidj"}, {"Hovd", "Chowd Standard Tidj"}, {"India", "Indisk Tidj"}, {"Indian_Ocean", "Indisk Ooseaan Tidj"}, {"Indochina", "Indoschiina Tidj"}, {"Indonesia_Central", "Sentraal Indoneesien Tidj"}, {"Indonesia_Eastern", "Uast Indoneesien Tidj"}, {"Indonesia_Western", "Waast Indoneesien Tidj"}, {"Iran", "Iraan Standard Tidj"}, {"Irkutsk", "Irkutsk Standard Tidj"}, {"Israel", "Israel Standard Tidj"}, {"Japan", "Jaapan Standard Tidj"}, {"Kazakhstan", "Kasachstaan Tidj"}, {"Kazakhstan_Eastern", "Uast Kasachstaan Tidj"}, {"Kazakhstan_Western", "Waast Kasachstaan Tidj"}, {"Korea", "Korea Standard Tidj"}, {"Kosrae", "Kosrae Tidj"}, {"Krasnoyarsk", "Krasnojarsk Standard Tidj"}, {"Kyrgystan", "Kirgistaan Tidj"}, {"Line_Islands", "Line Eilunen Tidj"}, {"Lord_Howe", "Lord Howe Standard Tidj"}, {"Magadan", "Magadan Standard Tidj"}, {"Malaysia", "Malaysia Tidj"}, {"Maldives", "Malediiwen Tidj"}, {"Marquesas", "Marquesas Tidj"}, {"Marshall_Islands", "Marshall Eilunen Tidj"}, {"Mauritius", "Mauritius Standard Tidj"}, {"Mawson", "Mawson Tidj"}, {"Mexico_Pacific", "Meksiko Pasiifik Standard Tidj"}, {"Mongolia", "Ulaanbaatar Standard Tidj"}, {"Moscow", "Moskau Standard Tidj"}, {"Myanmar", "Mjanmaar Tidj"}, {"Nauru", "Nauru Tidj"}, {"Nepal", "Neepaal Tidj"}, {"New_Caledonia", "Neikaledoonien Standard Tidj"}, {"New_Zealand", "Neisialun Standard Tidj"}, {"Newfoundland", "Neifundlun Standard Tidj"}, {"Niue", "Niue Tidj"}, {"Norfolk", "Norfolk Eilun Standard Tidj"}, {"Noronha", "Fernando de Noronha Standard Tidj"}, {"Novosibirsk", "Nowosibirsk Standard Tidj"}, {"Omsk", "Omsk Standard Tidj"}, {"Pakistan", "Pakistaan Standard Tidj"}, {"Palau", "Palau Tidj"}, {"Papua_New_Guinea", "Papua Neiguinea Tidj"}, {"Paraguay", "Paraguay Standard Tidj"}, {"Peru", "Peruu Standard Tidj"}, {"Philippines", "Filipiinen Standard Tidj"}, {"Phoenix_Islands", "Phoenix Eilunen Tidj"}, {"Pierre_Miquelon", "St. Pierre an Miquelon Standard Tidj"}, {"Pitcairn", "Pitcairn Eilunen Tidj"}, {"Ponape", "Pohnpei Tidj"}, {"Pyongyang", "Pjöngjang Tidj"}, {"Reunion", "Réunion Tidj"}, {"Rothera", "Rothera Tidj"}, {"Sakhalin", "Sachalin Standard Tidj"}, {"Samoa", "Amerikoons Samoa Standard Tidj"}, {"Seychelles", "Seschelen Tidj"}, {"Singapore", "Singapuur Tidj"}, {"Solomon", "Salomoonen Tidj"}, {"South_Georgia", "Süüdgeorgien Tidj"}, {"Suriname", "Suurinam Tidj"}, {"Syowa", "Syowa Tidj"}, {"Tahiti", "Tahiti Tidj"}, {"Taipei", "Taiwan Standard Tidj"}, {"Tajikistan", "Tadjikistaan Tidj"}, {"Tokelau", "Tokelau Tidj"}, {"Tonga", "Tonga Standard Tidj"}, {"Truk", "Chuuk Tidj"}, {"Turkmenistan", "Turkmeenistaan Standard Tidj"}, {"Tuvalu", "Tuwaalu Tidj"}, {"Uruguay", "Uruguay Standard Tidj"}, {"Uzbekistan", "Usbekistaan Standard Tidj"}, {"Vanuatu", "Vanuatu Standard Tidj"}, {"Venezuela", "Weenesuela Tidj"}, {"Vladivostok", "Wladiwostok Standard Tidj"}, {"Volgograd", "Wolgograd Standard Tidj"}, {"Vostok", "Wostok Tidj"}, {"Wake", "Wake Eilun Tidj"}, {"Wallis", "Wallis an Futuna Tidj"}, {"Yakutsk", "Jakutsk Standard Tidj"}, {"Yekaterinburg", "Jekaterinburg Standard Tidj"}, {"Yukon", "Yukon Tidj"}},
		{{"Alaska", "Alaska Somertidj"}, {"Amazon", "Amazonas Somertidj"}, {"America_Central", "Sentraal Ameerikoo Somertidj"}, {"America_Eastern", "Uast Ameerikoo Somertidj"}, {"America_Mountain", "Waast Ameerikoo Somertidj"}, {"America_Pacific", "Ameerikoo Pasiifik Somertidj"}, {"Apia", "Samoa Somertidj"}, {"Arabian", "Araabisk Somertidj"}, {"Argentina", "Argentiinien Somertidj"}, {"Argentina_Western", "Waast Argentiinien Somertidj"}, {"Armenia", "Armeenien Somertidj"}, {"Atlantic", "Ameerikoo Atlantik Somertidj"}, {"Australia_Central", "Austraalisk Sentraal Somertidj"}, {"Australia_CentralWestern", "Austraalisk Madelwaast Somertidj"}, {"Australia_Eastern", "Uast Austraalisk Somertidj"}, {"Australia_Western", "Waast Austraalisk Somertidj"}, {"Azerbaijan", "Aserbaidschaan Somertidj"}, {"Azores", "Atsooren Somertidj"}, {"Bangladesh", "Bangladesch Somertidj"}, {"Brasilia", "Brasiilien Somertidj"}, {"Cape_Verde", "Kapwerden Sommertidj"}, {"Chatham", "Chatham Somertidj"}, {"Chile", "Chiile Somertidj"}, {"China", "Schiina Somertidj"}, {"Colombia", "Kolumbien Somertidj"}, {"Cook", "Cook Eilunen Somertidj"}, {"Cuba", "Kuuba Somertidj"}, {"Easter", "Puask Eilun Somertidj"}, {"Europe_Central", "Madeleuropeesk Somertidj"}, {"Europe_Eastern", "Uasteuropeesk Somertidj"}, {"Europe_Western", "Waasteuropeesk Somertidj"}, {"Falkland", "Falklun Eilunen Somertidj"}, {"Fiji", "Fidschi Somertidj"}, {"Georgia", "Georgien Somertidj"}, {"Greenland_Eastern", "Uast Greenlun Somertidj"}, {"Greenland_Western", "Waast Greenlun Somertidj"}, {"Hawaii_Aleutian", "Hawaii-Aleuten Somertidj"}, {"Hong_Kong", "Hongkong Somertidj"}, {"Hovd", "Chowd Somertidj"}, {"Iran", "Iraan Somertidj"}, {"Irkutsk", "Irkutsk Somertidj"}, {"Israel", "Israel Somertidj"}, {"Japan", "Jaapan Somertidj"}, {"Korea", "Korea Somertidj"}, {"Krasnoyarsk", "Krasnojarsk Somertidj"}, {"Lord_Howe", "Lord Howe Somertidj"}, {"Magadan", "Magadan Somertidj"}, {"Mauritius", "Mauritius Somertidj"}, {"Mexico_Pacific", "Meksiko Pasiifik Somertidj"}, {"Mongolia", "Ulaanbaatar Somertidj"}, {"Moscow", "Moskau Somertidj"}, {"New_Caledonia", "Neikaledoonien Somertidj"}, {"New_Zealand", "Neisialun Somertidj"}, {"Newfoundland", "Neifundlun Somertidj"}, {"Norfolk", "Norfolk Eilun Somertidj"}, {"Noronha", "Fernando de Noronha Somertidj"}, {"Novosibirsk", "Nowosibirsk Somertidj"}, {"Omsk", "Omsk Somertidj"}, {"Pakistan", "Pakistaan Somertidj"}, {"Paraguay", "Paraguay Somertidj"}, {"Peru", "Peruu Somertidj"}, {"Philippines", "Filipiinen Somertidj"}, {"Pierre_Miquelon", "St. Pierre an Miquelon Somertidj"}, {"Sakhalin", "Sachalin Somertidj"}, {"Samoa", "Amerikoons Samoa Somertidj"}, {"Taipei", "Taiwan Somertidj"}, {"Tonga", "Tonga Somertidj"}, {"Turkmenistan", "Turkmeenistaan Somertidj"}, {"Uruguay", "Uruguay Somertidj"}, {"Uzbekistan", "Usbekistaan Somertidj"}, {"Vanuatu", "Vanuatu Somertidj"}, {"Vladivostok", "Wladiwostok Somertidj"}, {"Volgograd", "Wolgograd Somertidj"}, {"Yakutsk", "Jakutsk Somertidj"}, {"Yekaterinburg", "Jekaterinburg Somertidj"}},
	},
	{
		{{"Europe_Central", "Ore standard de Europe centrâl"}, {"Europe_Eastern", "Ore standard de Europe orientâl"}, {"Europe_Western", "Ore standard de Europe ocidentâl"}, {"Moscow", "Ore standard di Mosche"}},
		{{"Europe_Central", "Ore estive de Europe centrâl"}, {"Europe_Eastern", "Ore estive de Europe orientâl"}, {"Europe_Western", "Ore estive de Europe ocidentâl"}, {"Moscow", "Ore estive di Mosche"}},
	},
	{
		{{"Acre", "Acre-standerttiid"}, {"Afghanistan", "Afghaanske tiid"}, {"Africa_Central", "Sintraal-Afrikaanske tiid"}, {"Africa_Eastern", "East-Afrikaanske tiid"}, {"Africa_Southern", "Sûd-Afrikaanske tiid"}, {"Africa_Western", "West-Afrikaanske tiid"}, {"Alaska", "Alaska-standerttiid"}, {"Almaty", "Alma-Ata-standerttiid"}, {"Amazon", "Amazone-standerttiid"}, {"America_Central", "Central-standerttiid"}, {"America_Eastern", "Eastern-standerttiid"}, {"America_Mountain", "Mountain-standerttiid"}, {"America_Pacific", "Pasifik-standerttiid"}, {"Anadyr", "Anadyr-standerttiid"}, {"Apia", "Apia-standerttiid"}, {"Aqtau", "Aqtau-standerttiid"}, {"Aqtobe", "Aqtöbe-standerttiid"}, {"Arabian", "Arabyske standerttiid"}, {"Argentina", "Argentynske standerttiid"}, {"Argentina_Western", "West-Argentynske standerttiid"}, {"Armenia", "Armeense standerttiid"}, {"Atlantic", "Atlantic-standerttiid"}, {"Australia_Central", "Midden-Australyske standerttiid"}, {"Australia_CentralWestern", "Midden-Australyske westelijke standerttiid"}, {"Australia_Eastern", "East-Australyske standerttiid"}, {"Australia_Western", "West-Australyske standerttiid"}, {"Azerbaijan", "Azerbeidzjaanske standerttiid"}, {"Azores", "Azoren-standerttiid"}, {"Bangladesh", "Bengalese standerttiid"}, {"Bhutan", "Bhutaanske tiid"}, {"Bolivia", "Boliviaanske tiid"}, {"Brasilia", "Brazyljaanske standerttiid"}, {"Brunei", "Bruneise tiid"}, {"Cape_Verde", "Kaapverdyske standerttiid"}, {"Chamorro", "Chamorro-tiid"}, {"Chatham", "Chatham standerttiid"}, {"Chile", "Sileenske standerttiid"}, {"China", "Sineeske standerttiid"}, {"Christmas", "Krysteilânske tiid"}, {"Cocos", "Kokoseilânske tiid"}, {"Colombia", "Kolombiaanske standerttiid"}, {"Cook", "Cookeilânse standerttiid"}, {"Cuba", "Kubaanske standerttiid"}, {"Davis", "Davis tiid"}, {"DumontDUrville", "Dumont-d’Urville tiid"}, {"East_Timor", "East-Timorese tiid"}, {"Easter", "Peaskeeilânske standerttiid"}, {"Ecuador", "Ecuadoraanske tiid"}, {"Europe_Central", "Midden-Europeeske standerttiid"}, {"Europe_Eastern", "East-Europeeske standerttiid"}, {"Europe_Further_Eastern", "Fierder-eastlik-Europeeske tiid"}, {"Europe_Western", "West-Europeeske standerttiid"}, {"Falkland", "Falklâneilânske standerttiid"}, {"Fiji", "Fijyske standerttiid"}, {"French_Guiana", "Frâns-Guyaanske tiid"}, {"French_Southern", "Frânske Súdlike en Antarctyske tiid"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagoseilânske tiid"}, {"Gambier", "Gambiereilânske tiid"}, {"Georgia", "Georgyske standerttiid"}, {"Gilbert_Islands", "Gilberteilânske tiid"}, {"Greenland_Eastern", "East-Groenlânske standerttiid"}, {"Greenland_Western", "West-Groenlânske standerttiid"}, {"Guam", "Guamese standerttiid"}, {"Gulf", "Golf standerttiid"}, {"Guyana", "Guyaanske tiid"}, {"Hawaii", "Hawaii-Aleoetyske standerttiid"}, {"Hawaii_Aleutian", "Hawaii-Aleoetyske standerttiid"}, {"Hong_Kong", "Hongkongse standerttiid"}, {"Hovd", "Hovd standerttiid"}, {"India", "Yndiaaske tiid"}, {"Indian_Ocean", "Yndyske Oceaan-tiid"}, {"Indochina", "Yndochinese tiid"}, {"Indonesia_Central", "Sintraal-Yndonezyske tiid"}, {"Indonesia_Eastern", "East-Yndonezyske tiid"}, {"Indonesia_Western", "West-Yndonezyske tiid"}, {"Iran", "Iraanske standerttiid"}, {"Irkutsk", "Irkoetsk-standerttiid"}, {"Israel", "Israëlyske standerttiid"}, {"Japan", "Japanske standerttiid"}, {"Kamchatka", "Petropavlovsk-Kamtsjatski-standerttiid"}, {"Kazakhstan", "Kazakhstan-tiid"}, {"Kazakhstan_Eastern", "East-Kazachse tiid"}, {"Kazakhstan_Western", "West-Kazachse tiid"}, {"Korea", "Koreaanske standerttiid"}, {"Kosrae", "Kosraese tiid"}, {"Krasnoyarsk", "Krasnojarsk-standerttiid"}, {"Kyrgystan", "Kirgizyske tiid"}, {"Lanka", "Lanka-tiid"}, {"Line_Islands", "Line-eilânske tiid"}, {"Lord_Howe", "Lord Howe-eilânske standerttiid"}, {"Macau", "Macause standerttiid"}, {"Magadan", "Magadan-standerttiid"}, {"Malaysia", "Maleisyske tiid"}, {"Maldives", "Maldivyske tiid"}, {"Marquesas", "Marquesaseilânske tiid"}, {"Marshall_Islands", "Marshalleilânske tiid"}, {"Mauritius", "Mauritiaanske standerttiid"}, {"Mawson", "Mawson tiid"}, {"Mexico_Pacific", "Meksikaanske Pacific-standerttiid"}, {"Mongolia", "Ulaanbaatar standerttiid"}, {"Moscow", "Moskou-standerttiid"}, {"Myanmar", "Myanmarese tiid"}, {"Nauru", "Nauruaanske tiid"}, {"Nepal", "Nepalese tiid"}, {"New_Caledonia", "Nij-Kaledonyske standerttiid"}, {"New_Zealand", "Nij-Seelânske standerttiid"}, {"Newfoundland", "Newfoundlânske-standerttiid"}, {"Niue", "Niuese tiid"}, {"Norfolk", "Norfolkeilânske standerttiid"}, {"Noronha", "Fernando de Noronha-standerttiid"}, {"North_Mariana", "Noardlike Mariaanske tiid"}, {"Novosibirsk", "Novosibirsk-standerttiid"}, {"Omsk", "Omsk-standerttiid"}, {"Pakistan", "Pakistaanske standerttiid"}, {"Palau", "Belause tiid"}, {"Papua_New_Guinea", "Papoea-Nij-Guineeske tiid"}, {"Paraguay", "Paraguayaanske standerttiid"}, {"Peru", "Peruaanske standerttiid"}, {"Philippines", "Filipijnse standerttiid"}, {"Phoenix_Islands", "Phoenixeilânske tiid"}, {"Pierre_Miquelon", "Saint Pierre en Miquelon-standerttiid"}, {"Pitcairn", "Pitcairneillânske tiid"}, {"Ponape", "Pohnpei tiid"}, {"Pyongyang", "Pyongyang-tiid"}, {"Qyzylorda", "Qyzylorda-standerttiid"}, {"Reunion", "Réunionse tiid"}, {"Rothera", "Rothera tiid"}, {"Sakhalin", "Sachalin-standerttiid"}, {"Samara", "Samara-standerttiid"}, {"Samoa", "Samoaanske standerttiid"}, {"Seychelles", "Seychelse tiid"}, {"Singapore", "Singaporese standerttiid"}, {"Solomon", "Salomonseilânske tiid"}, {"South_Georgia", "Sûd-Georgyske tiid"}, {"Suriname", "Surinaamske tiid"}, {"Syowa", "Syowa tiid"}, {"Tahiti", "Tahitiaanske tiid"}, {"Taipei", "Taipei standerttiid"}, {"Tajikistan", "Tadzjiekse tiid"}, {"Tokelau", "Tokelau-eilânske tiid"}, {"Tonga", "Tongaanske standerttiid"}, {"Truk", "Chuukse tiid"}, {"Turkmenistan", "Turkmeense standerttiid"}, {"Tuvalu", "Tuvaluaanske tiid"}, {"Uruguay", "Uruguayaanske standerttiid"}, {"Uzbekistan", "Oezbeekse standerttiid"}, {"Vanuatu", "Vanuatuaanske standerttiid"}, {"Venezuela", "Fenezolaanske tiid"}, {"Vladivostok", "Vladivostok-standerttiid"}, {"Volgograd", "Wolgograd-standerttiid"}, {"Vostok", "Vostok tiid"}, {"Wake", "Wake-eilânske tiid"}, {"Wallis", "Wallis en Futunase tiid"}, {"Yakutsk", "Jakoetsk-standerttiid"}, {"Yekaterinburg", "Jekaterinenburg-standerttiid"}, {"Yukon", "Yukon-tiid"}},
		{{"Acre", "Acre-simmertiid"}, {"Alaska", "Alaska-simmertiid"}, {"Almaty", "Alma-Ata-simmertiid"}, {"Amazon", "Amazone-simmertiid"}, {"America_Central", "Central-simmertiid"}, {"America_Eastern", "Eastern-simmertiid"}, {"America_Mountain", "Mountain-simmertiid"}, {"America_Pacific", "Pasifik-simmertiid"}, {"Anadyr", "Anadyr-simmertiid"}, {"Apia", "Apia-simmertiid"}, {"Aqtau", "Aqtau-simmertiid"}, {"Aqtobe", "Aqtöbe-simmertiid"}, {"Arabian", "Arabyske simmertiid"}, {"Argentina", "Argentynske simmertiid"}, {"Argentina_Western", "West-Argentynske simmertiid"}, {"Armenia", "Armeense simmertiid"}, {"Atlantic", "Atlantic-simmertiid"}, {"Australia_Central", "Midden-Australyske simmertiid"}, {"Australia_CentralWestern", "Midden-Australyske westelijke simmertiid"}, {"Australia_Eastern", "East-Australyske simmertiid"}, {"Australia_Western", "West-Australyske simmertiid"}, {"Azerbaijan", "Azerbeidzjaanske simmertiid"}, {"Azores", "Azoren-simmertiid"}, {"Bangladesh", "Bengalese simmertiid"}, {"Brasilia", "Brazyljaanske simmertiid"}, {"Cape_Verde", "Kaapverdyske simmertiid"}, {"Chatham", "Chatham simmertiid"}, {"Chile", "Sileenske simmertiid"}, {"China", "Sineeske simmertiid"}, {"Colombia", "Kolombiaanske simmertiid"}, {"Cook", "Cookeilânse halve simmertiid"}, {"Cuba", "Kubaanske simmertiid"}, {"Easter", "Peaskeeilânske simmertiid"}, {"Europe_Central", "Midden-Europeeske simmertiid"}, {"Europe_Eastern", "East-Europeeske simmertiid"}, {"Europe_Western", "West-Europeeske simmertiid"}, {"Falkland", "Falklâneilânske simmertiid"}, {"Fiji", "Fijyske simmertiid"}, {"Georgia", "Georgyske simmertiid"}, {"Greenland_Eastern", "East-Groenlânske simmertiid"}, {"Greenland_Western", "West-Groenlânske simmertiid"}, {"Hawaii_Aleutian", "Hawaii-Aleoetyske simmertiid"}, {"Hong_Kong", "Hongkongse simmertiid"}, {"Hovd", "Hovd simmertiid"}, {"Iran", "Iraanske simmertiid"}, {"Irkutsk", "Irkoetsk-simmertiid"}, {"Israel", "Israëlyske simmertiid"}, {"Japan", "Japanske simmertiid"}, {"Kamchatka", "Petropavlovsk-Kamtsjatski-simmertiid"}, {"Korea", "Koreaanske simmertiid"}, {"Krasnoyarsk", "Krasnojarsk-simmertiid"}, {"Lord_Howe", "Lord Howe-eilânske simmertiid"}, {"Macau", "Macause simmertiid"}, {"Magadan", "Magadan-simmertiid"}, {"Mauritius", "Mauritiaanske simmertiid"}, {"Mexico_Pacific", "Meksikaanske Pacific-simmerttiid"}, {"Mongolia", "Ulaanbaatar simmertiid"}, {"Moscow", "Moskou-simmertiid"}, {"New_Caledonia", "Nij-Kaledonyske simmertiid"}, {"New_Zealand", "Nij-Seelânske simmertiid"}, {"Newfoundland", "Newfoundlânske-simmertiid"}, {"Norfolk", "Norfolkeilânske simmertiid"}, {"Noronha", "Fernando de Noronha-simmertiid"}, {"Novosibirsk", "Novosibirsk-simmertiid"}, {"Omsk", "Omsk-simmertiid"}, {"Pakistan", "Pakistaanske simmertiid"}, {"Paraguay", "Paraguayaanske simmertiid"}, {"Peru", "Peruaanske simmertiid"}, {"Philippines", "Filipijnse simmertiid"}, {"Pierre_Miquelon", "Saint Pierre en Miquelon-simmertiid"}, {"Qyzylorda", "Qyzylorda-simmertiid"}, {"Sakhalin", "Sachalin-simmertiid"}, {"Samara", "Samara-simmertiid"}, {"Samoa", "Samoaanske simmertiid"}, {"Taipei", "Taipei simmertiid"}, {"Tonga", "Tongaanske simmertiid"}, {"Turkmenistan", "Turkmeense simmertiid"}, {"Uruguay", "Uruguayaanske simmertiid"}, {"Uzbekistan", "Oezbeekse simmertiid"}, {"Vanuatu", "Vanuatuaanske simmertiid"}, {"Vladivostok", "Vladivostok-simmertiid"}, {"Volgograd", "Wolgograd-simmertiid"}, {"Yakutsk", "Jakoetsk-simmertiid"}, {"Yekaterinburg", "Jekaterinenburg-simmertiid"}},
	},
	{
		{{"Acre", "Am Caighdeánach Acre"}, {"Afghanistan", "Am na hAfganastáine"}, {"Africa_Central", "Am na hAfraice Láir"}, {"Africa_Eastern", "Am Oirthear na hAfraice"}, {"Africa_Southern", "Am na hAfraice Theas"}, {"Africa_Western", "Am Iarthar na hAfraice"}, {"Alaska", "Am Caighdeánach Alasca"}, {"Almaty", "Am Caighdeánach Almaty"}, {"Amazon", "Am Caighdeánach na hAmasóine"}, {"America_Central", "Am Caighdeánach Lárnach Mheiriceá Thuaidh"}, {"America_Eastern", "Am Caighdeánach Oirthearach Mheiriceá Thuaidh"}, {"America_Mountain", "Am Caighdeánach Sléibhte Mheiriceá Thuaidh"}, {"America_Pacific", "Am Caighdeánach an Aigéin Chiúin"}, {"Anadyr", "Am Caighdeánach Anadyr"}, {"Apia", "Am Caighdeánach Shamó"}, {"Aqtau", "Am Caighdeánach Aqtau"}, {"Aqtobe", "Am Caighdeánach Aqtobe"}, {"Arabian", "Am Caighdeánach na hAraibe"}, {"Argentina", "Am Caighdeánach na hAirgintíne"}, {"Argentina_Western", "Am Caighdeánach Iartharach na hAirgintíne"}, {"Armenia", "Am Caighdeánach na hAirméine"}, {"Atlantic", "Am Caighdeánach an Atlantaigh"}, {"Australia_Central", "Am Caighdeánach Lár na hAstráile"}, {"Australia_CentralWestern", "Am Caighdeánach Mheániarthar na hAstráile"}, {"Australia_Eastern", "Am Caighdeánach Oirthear na hAstráile"}, {"Australia_Western", "Am Caighdeánach Iarthar na hAstráile"}, {"Azerbaijan", "Am Caighdeánach na hAsarbaiseáine"}, {"Azores", "Am Caighdeánach na nAsór"}, {"Bangladesh", "Am Caighdeánach na Banglaidéise"}, {"Bhutan", "Am na Bútáine"}, {"Bolivia", "Am na Bolaive"}, {"Brasilia", "Am Caighdeánach Bhrasília"}, {"Brunei", "Am Bhrúiné Darasalám"}, {"Cape_Verde", "Am Caighdeánach Rinn Verde"}, {"Casey", "Am Stáisiún Casey"}, {"Chamorro", "Am Caighdeánach Seamórach"}, {"Chatham", "Am Caighdeánach Chatham"}, {"Chile", "Am Caighdeánach na Sile"}, {"China", "Am Caighdeánach na Síne"}, {"Christmas", "Am Oileán na Nollag"}, {"Cocos", "Am Oileáin Cocos"}, {"Colombia", "Am Caighdeánach na Colóime"}, {"Cook", "Am Caighdeánach Oileáin Cook"}, {"Cuba", "Am Caighdeánach Chúba"}, {"Davis", "Am Davis"}, {"DumontDUrville", "Am Dumont-d’Urville"}, {"East_Timor", "Am Thíomór Thoir"}, {"Easter", "Am Caighdeánach Oileán na Cásca"}, {"Ecuador", "Am Eacuadór"}, {"Europe_Central", "Am Caighdeánach Lár na hEorpa"}, {"Europe_Eastern", "Am Caighdeánach Oirthear na hEorpa"}, {"Europe_Further_Eastern", "Am Chianoirthear na hEorpa"}, {"Europe_Western", "Am Caighdeánach Iarthar na hEorpa"}, {"Falkland", "Am Caighdeánach Oileáin Fháclainne"}, {"Fiji", "Am Caighdeánach Fhidsí"}, {"French_Guiana", "Am Ghuáin na Fraince"}, {"French_Southern", "Am Francach Dheisceart an Domhain agus an Antartaigh"}, {"GMT", "Meán-Am Greenwich"}, {"Galapagos", "Am Oileáin Galápagos"}, {"Gambier", "Am Gambier"}, {"Georgia", "Am Caighdeánach na Seoirsia"}, {"Gilbert_Islands", "Am Chiribeas"}, {"Greenland_Eastern", "Am Caighdeánach Oirthear na Graonlainne"}, {"Greenland_Western", "Am Caighdeánach Iarthar na Graonlainne"}, {"Guam", "Am Caighdeánach Ghuam"}, {"Gulf", "Am Caighdeánach na Murascaille"}, {"Guyana", "Am na Guáine"}, {"Hawaii", "Am Caighdeánach Haváí-Ailiúit"}, {"Hawaii_Aleutian", "Am Caighdeánach Haváí-Ailiúit"}, {"Hong_Kong", "Am Caighdeánach Hong Cong"}, {"Hovd", "Am Caighdeánach Khovd"}, {"India", "Am Caighdeánach na hIndia"}, {"Indian_Ocean", "Am an Aigéin Indiaigh"}, {"Indochina", "Am na hInd-Síne"}, {"Indonesia_Central", "Am Lár na hIndinéise"}, {"Indonesia_Eastern", "Am Oirthear na hIndinéise"}, {"Indonesia_Western", "Am Iarthar na hIndinéise"}, {"Iran", "Am Caighdeánach na hIaráine"}, {"Irkutsk", "Am Caighdeánach Irkutsk"}, {"Israel", "Am Caighdeánach Iosrael"}, {"Japan", "Am Caighdeánach na Seapáine"}, {"Kamchatka", "Am Caighdeánach Phetropavlovsk-Kamchatski"}, {"Kazakhstan", "Am na Casacstáine"}, {"Kazakhstan_Eastern", "Am Oirthear na Casacstáine"}, {"Kazakhstan_Western", "Am Iarthar na Casacstáine"}, {"Korea", "Am Caighdeánach na Cóiré"}, {"Kosrae", "Am Kosrae"}, {"Krasnoyarsk", "Am Caighdeánach Krasnoyarsk"}, {"Kyrgystan", "Am na Cirgeastáine"}, {"Lanka", "Am Shrí Lanca"}, {"Line_Islands", "Am Oileáin na Líne"}, {"Lord_Howe", "Am Caighdeánach Lord Howe"}, {"Macau", "Am Caighdeánach Mhacao"}, {"Magadan", "Am Caighdeánach Mhagadan"}, {"Malaysia", "Am na Malaeisia"}, {"Maldives", "Am Oileáin Mhaildíve"}, {"Marquesas", "Am na nOileán Marcasach"}, {"Marshall_Islands", "Am Oileáin Marshall"}, {"Mauritius", "Am Caighdeánach Oileán Mhuirís"}, {"Mawson", "Am Mawson"}, {"Mexico_Pacific", "Am Caighdeánach Meicsiceach an Aigéin Chiúin"}, {"Mongolia", "Am Caighdeánach Ulan Bator"}, {"Moscow", "Am Caighdeánach Mhoscó"}, {"Myanmar", "Am Mhaenmar"}, {"Nauru", "Am Nárú"}, {"Nepal", "Am Neipeal"}, {"New_Caledonia", "Am Caighdeánach na Nua-Chaladóine"}, {"New_Zealand", "Am Caighdeánach na Nua-Shéalainne"}, {"Newfoundland", "Am Caighdeánach Thalamh an Éisc"}, {"Niue", "Am Niue"}, {"Norfolk", "Am Caighdeánach Oileán Norfolk"}, {"Noronha", "Am Caighdeánach Fernando de Noronha"}, {"North_Mariana", "Am na nOileán Máirianach Thuaidh"}, {"Novosibirsk", "Am Caighdeánach Novosibirsk"}, {"Omsk", "Am Caighdeánach Omsk"}, {"Pakistan", "Am Caighdeánach na Pacastáine"}, {"Palau", "Am Oileáin Palau"}, {"Papua_New_Guinea", "Am Nua-Ghuine Phapua"}, {"Paraguay", "Am Caighdeánach Pharagua"}, {"Peru", "Am Caighdeánach Pheiriú"}, {"Philippines", "Am Caighdeánach na nOileán Filipíneach"}, {"Phoenix_Islands", "Am Oileáin an Fhéinics"}, {"Pierre_Miquelon", "Am Caighdeánach Saint-Pierre-et-Miquelon"}, {"Pitcairn", "Am Pitcairn"}, {"Ponape", "Am Pohnpei"}, {"Pyongyang", "Am Pyongyang"}, {"Qyzylorda", "Am Caighdeánach Qyzylorda"}, {"Reunion", "Am Réunion"}, {"Rothera", "Am Rothera"}, {"Sakhalin", "Am Caighdeánach Shacailín"}, {"Samara", "Am Caighdeánach Shamara"}, {"Samoa", "Am Caighdeánach Shamó Mheiriceá"}, {"Seychelles", "Am na Séiséal"}, {"Singapore", "Am Caighdeánach Shingeapór"}, {"Solomon", "Am Oileáin Sholaimh"}, {"South_Georgia", "Am na Seoirsia Theas"}, {"Suriname", "Am Shuranam"}, {"Syowa", "Am Syowa"}, {"Tahiti", "Am Thaihítí"}, {"Taipei", "Am Caighdeánach Taipei"}, {"Tajikistan", "Am na Táidsíceastáine"}, {"Tokelau", "Am Oileáin Thócaláú"}, {"Tonga", "Am Caighdeánach Thonga"}, {"Truk", "Am Chuuk"}, {"Turkmenistan", "Am Caighdeánach na Tuircméanastáine"}, {"Tuvalu", "Am Thúvalú"}, {"Uruguay", "Am Caighdeánach Uragua"}, {"Uzbekistan", "Am Caighdeánach na hÚisbéiceastáine"}, {"Vanuatu", "Am Caighdeánach Vanuatú"}, {"Venezuela", "Am Veiniséala"}, {"Vladivostok", "Am Caighdeánach Vladivostok"}, {"Volgograd", "Am Caighdeánach Volgograd"}, {"Vostok", "Am Vostok"}, {"Wake", "Am Oileán Wake"}, {"Wallis", "Am Wallis agus Futuna"}, {"Yakutsk", "Am Caighdeánach Iacútsc"}, {"Yekaterinburg", "Am Caighdeánach Yekaterinburg"}, {"Yukon", "Am Yukon"}},
		{{"Acre", "Am Samhraidh Acre"}, {"Alaska", "Am Samhraidh Alasca"}, {"Almaty", "Am Samhraidh Almaty"}, {"Amazon", "Am Samhraidh na hAmasóine"}, {"America_Central", "Am Samhraidh Lárnach Mheiriceá Thuaidh"}, {"America_Eastern", "Am Samhraidh Oirthearach Mheiriceá Thuaidh"}, {"America_Mountain", "Am Samhraidh Sléibhte Mheiriceá Thuaidh"}, {"America_Pacific", "Am Samhraidh an Aigéin Chiúin"}, {"Anadyr", "Am Samhraidh Anadyr"}, {"Apia", "Am Samhraidh Shamó"}, {"Aqtau", "Am Samhraidh Aqtau"}, {"Aqtobe", "Am Samhraidh Aqtobe"}, {"Arabian", "Am Samhraidh na hAraibe"}, {"Argentina", "Am Samhraidh na hAirgintíne"}, {"Argentina_Western", "Am Samhraidh Iartharach na hAirgintíne"}, {"Armenia", "Am Samhraidh na hAirméine"}, {"Atlantic", "Am Samhraidh an Atlantaigh"}, {"Australia_Central", "Am Samhraidh Lár na hAstráile"}, {"Australia_CentralWestern", "Am Samhraidh Mheániarthar na hAstráile"}, {"Australia_Eastern", "Am Samhraidh Oirthear na hAstráile"}, {"Australia_Western", "Am Samhraidh Iarthar na hAstráile"}, {"Azerbaijan", "Am Samhraidh na hAsarbaiseáine"}, {"Azores", "Am Samhraidh na nAsór"}, {"Bangladesh", "Am Samhraidh na Banglaidéise"}, {"Brasilia", "Am Samhraidh Bhrasília"}, {"Cape_Verde", "Am Samhraidh Rinn Verde"}, {"Chatham", "Am Samhraidh Chatham"}, {"Chile", "Am Samhraidh na Sile"}, {"China", "Am Samhraidh na Síne"}, {"Colombia", "Am Samhraidh na Colóime"}, {"Cook", "Am Samhraidh Oileáin Cook"}, {"Cuba", "Am Samhraidh Chúba"}, {"Easter", "Am Samhraidh Oileán na Cásca"}, {"Europe_Central", "Am Samhraidh Lár na hEorpa"}, {"Europe_Eastern", "Am Samhraidh Oirthear na hEorpa"}, {"Europe_Western", "Am Samhraidh Iarthar na hEorpa"}, {"Falkland", "Am Samhraidh Oileáin Fháclainne"}, {"Fiji", "Am Samhraidh Fhidsí"}, {"Georgia", "Am Samhraidh na Seoirsia"}, {"Greenland_Eastern", "Am Samhraidh Oirthear na Graonlainne"}, {"Greenland_Western", "Am Samhraidh Iarthar na Graonlainne"}, {"Hawaii_Aleutian", "Am Samhraidh Haváí-Ailiúit"}, {"Hong_Kong", "Am Samhraidh Hong Cong"}, {"Hovd", "Am Samhraidh Khovd"}, {"Iran", "Am Samhraidh na hIaráine"}, {"Irkutsk", "Am Samhraidh Irkutsk"}, {"Israel", "Am Samhraidh Iosrael"}, {"Japan", "Am Samhraidh na Seapáine"}, {"Kamchatka", "Am Samhraidh Phetropavlovsk-Kamchatski"}, {"Korea", "Am Samhraidh na Cóiré"}, {"Krasnoyarsk", "Am Samhraidh Krasnoyarsk"}, {"Lord_Howe", "Am Samhraidh Lord Howe"}, {"Macau", "Am Samhraidh Mhacao"}, {"Magadan", "Am Samhraidh Mhagadan"}, {"Mauritius", "Am Samhraidh Oileán Mhuirís"}, {"Mexico_Pacific", "Am Samhraidh Meicsiceach an Aigéin Chiúin"}, {"Mongolia", "Am Samhraidh Ulan Bator"}, {"Moscow", "Am Samhraidh Mhoscó"}, {"New_Caledonia", "Am Samhraidh na Nua-Chaladóine"}, {"New_Zealand", "Am Samhraidh na Nua-Shéalainne"}, {"Newfoundland", "Am Samhraidh Thalamh an Éisc"}, {"Norfolk", "Am Samhraidh Oileán Norfolk"}, {"Noronha", "Am Samhraidh Fernando de Noronha"}, {"Novosibirsk", "Am Samhraidh Novosibirsk"}, {"Omsk", "Am Samhraidh Omsk"}, {"Pakistan", "Am Samhraidh na Pacastáine"}, {"Paraguay", "Am Samhraidh Pharagua"}, {"Peru", "Am Samhraidh Pheiriú"}, {"Philippines", "Am Samhraidh na nOileán Filipíneach"}, {"Pierre_Miquelon", "Am Samhraidh Saint-Pierre-et-Miquelon"}, {"Qyzylorda", "Am Samhraidh Qyzylorda"}, {"Sakhalin", "Am Samhraidh Shacailín"}, {"Samara", "Am Samhraidh Shamara"}, {"Samoa", "Am Samhraidh Shamó Mheiriceá"}, {"Taipei", "Am Samhraidh Taipei"}, {"Tonga", "Am Samhraidh Thonga"}, {"Turkmenistan", "Am Samhraidh na Tuircméanastáine"}, {"Uruguay", "Am Samhraidh Uragua"}, {"Uzbekistan", "Am Samhraidh na hÚisbéiceastáine"}, {"Vanuatu", "Am Samhraidh Vanuatú"}, {"Vladivostok", "Am Samhraidh Vladivostok"}, {"Volgograd", "Am Samhraidh Volgograd"}, {"Yakutsk", "Am Samhraidh Iacútsc"}, {"Yekaterinburg", "Am Samhraidh Yekaterinburg"}},
	},
	{
		{{"Africa_Central", "Afrika Teŋgbɛ Be"}, {"Africa_Eastern", "Afrika Bokagbɛ Be"}, {"Africa_Southern", "South Afrika Be"}, {"Africa_Western", "Afrika Anaigbɛ Be"}, {"Alaska", "Alaska Be Yɛ Fɛi Beiaŋ"}, {"America_Central", "Amerika Teŋgbɛbii Abe Yɛ Fɛi Beiaŋ"}, {"America_Eastern", "Amerika Bokãgbɛbii Abe Yɛ Fɛi Beiaŋ"}, {"America_Mountain", "Amerika Gɔjianɔbii Abe Yɛ Fɛi Beiaŋ"}, {"America_Pacific", "Pasifik Be Yɛ Fɛi Beiaŋ"}, {"Atlantic", "Atlantik Be Yɛ Fɛi Beiaŋ"}, {"Azores", "Azores Be Yɛ Fɛi Beiaŋ"}, {"Cape_Verde", "Kape Verde Be Yɛ Fɛi Beiaŋ"}, {"Cuba", "Kuba Be Yɛ Fɛi Beiaŋ"}, {"French_Southern", "Antarktik Kɛ Wuoyigbɛbii Ni Wieɔ Frɛntsi Be"}, {"GMT", "Betsɔɔmɔ ni ka ŋɛlɛ kome nɔ"}, {"Greenland_Eastern", "Greenland Bokãgbɛ Be Yɛ Fɛi Beiaŋ"}, {"Greenland_Western", "Greenland Anaigbɛ Be Yɛ Fɛi Beiaŋ"}, {"Hawaii", "Hawaii-Aleutia Be Yɛ Fɛi Beiaŋ"}, {"Hawaii_Aleutian", "Hawaii-Aleutia Be Yɛ Fɛi Beiaŋ"}, {"Indian_Ocean", "Indian Ŋshɔ Lɛ Be"}, {"Mauritius", "Mauritius Be Yɛ Fɛi Beiaŋ"}, {"Mexico_Pacific", "Meziko Pasifik Be Yɛ Fɛi Beiaŋ"}, {"Newfoundland", "Newfoundland Be Yɛ Fɛi Beiaŋ"}, {"Pierre_Miquelon", "St. Pierre Kɛ Mikelon Be Yɛ Fɛi Beiaŋ"}, {"Reunion", "Réunion Be"}, {"Seychelles", "Seyshelles Be"}},
		{{"Alaska", "Alaska Be Yɛ Latsa Beiaŋ"}, {"America_Central", "Amerika Teŋgbɛbii Abe Yɛ Latsa Beiaŋ"}, {"America_Eastern", "Amerika Bokãgbɛbii Abe Yɛ Latsa Beiaŋ"}, {"America_Mountain", "Amerika Gɔjianɔbii Abe Yɛ Latsa Beiaŋ"}, {"America_Pacific", "Pasifik Be Yɛ Latsa Beiaŋ"}, {"Atlantic", "Atlantik Be Yɛ Latsa Beiaŋ"}, {"Azores", "Azores Be Yɛ Latsa Beiaŋ"}, {"Cape_Verde", "Kape Verde Be Yɛ Latsa Beiaŋ"}, {"Cuba", "Kuba Be Yɛ Latsa Beiaŋ"}, {"Greenland_Eastern", "Greenland Bokãgbɛ Be Yɛ Latsa Beiaŋ"}, {"Greenland_Western", "Greenland Anaigbɛ Be Yɛ Latsa Beiaŋ"}, {"Hawaii_Aleutian", "Hawaii-Aleutia Be Yɛ Latsa Beiaŋ"}, {"Mauritius", "Mauritius Be Yɛ Latsa Beiaŋ"}, {"Mexico_Pacific", "Meziko Pasifik Be Yɛ Latsa Beiaŋ"}, {"Newfoundland", "Newfoundland Be Yɛ Latsa Beiaŋ"}, {"Pierre_Miquelon", "St. Pierre Kɛ Mikelon Be Yɛ Latsa Beiaŋ"}},
	},
	{
		{{"Acre", "Bun-àm Acre"}, {"Afghanistan", "Àm Afghanastàin"}, {"Africa_Central", "Àm Meadhan Afraga"}, {"Africa_Eastern", "Àm Afraga an Ear"}, {"Africa_Southern", "Àm Afraga a Deas"}, {"Africa_Western", "Àm Afraga an Iar"}, {"Alaska", "Bun-àm Alaska"}, {"Almaty", "Bun-àm Almaty"}, {"Amazon", "Bun-àm Amasoin"}, {"America_Central", "Bun-àm Meadhan Aimeireaga"}, {"America_Eastern", "Bun-àm Aimeireaga an Ear"}, {"America_Mountain", "Bun-àm Monadh Aimeireaga"}, {"America_Pacific", "Bun-àm a’ Chuain Shèimh"}, {"Anadyr", "Bun-àm Anadyr"}, {"Apia", "Bun-àm Apia"}, {"Aqtau", "Bun-àm Aqtau"}, {"Aqtobe", "Bun-àm Aqtobe"}, {"Arabian", "Bun-àm Arabach"}, {"Argentina", "Bun-àm na h-Argantaine"}, {"Argentina_Western", "Bun-àm na h-Argantaine Siaraich"}, {"Armenia", "Bun-àm Airmeinia"}, {"Atlantic", "Bun-àm a’ Chuain Shiar"}, {"Australia_Central", "Bun-àm Meadhan Astràilia"}, {"Australia_CentralWestern", "Bun-àm Meadhan Astràilia an Iar"}, {"Australia_Eastern", "Bun-àm Astràilia an Ear"}, {"Australia_Western", "Bun-àm Astràilia an Iar"}, {"Azerbaijan", "Bun-àm Asarbaideàin"}, {"Azores", "Bun-àm nan Eileanan Asorach"}, {"Bangladesh", "Bun-àm Bangladais"}, {"Bhutan", "Àm Butàin"}, {"Bolivia", "Àm Boilibhia"}, {"Brasilia", "Bun-àm Bhrasília"}, {"Brunei", "Àm Bhrùnaigh Dàr as-Salàm"}, {"Cape_Verde", "Bun-àm a’ Chip Uaine"}, {"Casey", "Àm Chasey"}, {"Chamorro", "Àm Chamorro"}, {"Chatham", "Bun-àm Chatham"}, {"Chile", "Bun-àm na Sile"}, {"China", "Bun-àm na Sìne"}, {"Christmas", "Àm Eilean na Nollaig"}, {"Cocos", "Àm Eileanan Chocos"}, {"Colombia", "Bun-àm Coloimbia"}, {"Cook", "Bun-àm Eileanan Cook"}, {"Cuba", "Bun-àm Cùba"}, {"Davis", "Àm Dhavis"}, {"DumontDUrville", "Àm Dumont-d’Urville"}, {"East_Timor", "Àm Thìomor an Ear"}, {"Easter", "Bun-àm Eilean na Càisge"}, {"Ecuador", "Àm Eacuadoir"}, {"Europe_Central", "Bun-àm Meadhan na Roinn-Eòrpa"}, {"Europe_Eastern", "Bun-àm na Roinn-Eòrpa an Ear"}, {"Europe_Further_Eastern", "Àm na Roinn-Eòrpa nas fhaide ear"}, {"Europe_Western", "Bun-àm na Roinn-Eòrpa an Iar"}, {"Falkland", "Bun-àm nan Eileanan Fàclannach"}, {"Fiji", "Bun-àm Fìdi"}, {"French_Guiana", "Àm Guidheàna na Frainge"}, {"French_Southern", "Àm Deasach agus Antartaigeach na Frainge"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Àm Ghalapagos"}, {"Gambier", "Àm Ghambier"}, {"Georgia", "Bun-àm na Cairtbheile"}, {"Gilbert_Islands", "Àm Eileanan Ghileabairt"}, {"Greenland", "Bun-àm na Graonlainne"}, {"Greenland_Eastern", "Bun-àm na Graonlainn an Ear"}, {"Greenland_Western", "Bun-àm na Graonlainn an Iar"}, {"Guam", "Àm Guam"}, {"Gulf", "Àm a’ Chamais"}, {"Guyana", "Àm Guidheàna"}, {"Hawaii", "Bun-àm nan Eileanan Hawai’i ’s Aleutach"}, {"Hawaii_Aleutian", "Bun-àm nan Eileanan Hawai’i ’s Aleutach"}, {"Hong_Kong", "Bun-àm Hong Kong"}, {"Hovd", "Bun-àm Khovd"}, {"India", "Àm nan Innseachan"}, {"Indian_Ocean", "Àm Cuan nan Innseachan"}, {"Indochina", "Àm Sìn-Innseanach"}, {"Indonesia_Central", "Àm Meadhan nan Innd-Innse"}, {"Indonesia_Eastern", "Àm nan Innd-Innse an Ear"}, {"Indonesia_Western", "Àm nan Innd-Innse an Iar"}, {"Iran", "Bun-àm Ioràin"}, {"Irkutsk", "Bun-àm Irkutsk"}, {"Israel", "Bun-àm Iosrael"}, {"Japan", "Bun-àm na Seapaine"}, {"Kamchatka", "Bun-àm Petropavlovsk-Kamchatsky"}, {"Kazakhstan", "Àm Casachstàin"}, {"Kazakhstan_Eastern", "Àm Casachstàin an Ear"}, {"Kazakhstan_Western", "Àm Casachstàin an Iar"}, {"Korea", "Bun-àm Choirèa"}, {"Kosrae", "Àm Kosrae"}, {"Krasnoyarsk", "Bun-àm Krasnoyarsk"}, {"Kyrgystan", "Àm Cìorgastain"}, {"Lanka", "Àm Lanca"}, {"Line_Islands", "Àm Eileanan Teraina"}, {"Lord_Howe", "Bun-àm Lord Howe"}, {"Macau", "Bun-àm Macàthu"}, {"Magadan", "Bun-àm Magadan"}, {"Malaysia", "Àm Mhalaidhsea"}, {"Maldives", "Àm nan Eileanan Mhaladaibh"}, {"Marquesas", "Àm Eileanan a’ Mharcais"}, {"Marshall_Islands", "Àm Eileanan Mharshall"}, {"Mauritius", "Bun-àm nan Eileanan Mhoiriseas"}, {"Mawson", "Àm Mhawson"}, {"Mexico_Pacific", "Bun-àm a’ Chuain Shèimh Mheagsago"}, {"Mongolia", "Bun-àm Ulan Bator"}, {"Moscow", "Bun-àm Mhosgo"}, {"Myanmar", "Àm Miànmar"}, {"Nauru", "Àm Nabhru"}, {"Nepal", "Àm Neapàl"}, {"New_Caledonia", "Bun-àm Chailleann Nuaidh"}, {"New_Zealand", "Bun-àm Shealainn Nuaidh"}, {"Newfoundland", "Bun-àm Talamh an Èisg"}, {"Niue", "Àm Niue"}, {"Norfolk", "Bun-àm Eilein Norfolk"}, {"Noronha", "Bun-àm Fernando de Noronha"}, {"North_Mariana", "Àm nan Eileanan Mairianach a Tuath"}, {"Novosibirsk", "Bun-àm Novosibirsk"}, {"Omsk", "Bun-àm Omsk"}, {"Pakistan", "Bun-àm Pagastàin"}, {"Palau", "Àm Palabh"}, {"Papua_New_Guinea", "Àm Gini Nuaidh Paputhaiche"}, {"Paraguay", "Bun-àm Paraguaidh"}, {"Peru", "Bun-àm Pearù"}, {"Philippines", "Bun-àm nan Eilean Filipineach"}, {"Phoenix_Islands", "Àm Eileanan Phoenix"}, {"Pierre_Miquelon", "Bun-àm Saint Pierre agus Miquelon"}, {"Pitcairn", "Àm Peit a’ Chàirn"}, {"Ponape", "Àm Pohnpei"}, {"Pyongyang", "Àm Pyeongyang"}, {"Qyzylorda", "Bun-àm Qızılorda"}, {"Reunion", "Àm Reunion"}, {"Rothera", "Àm Rothera"}, {"Sakhalin", "Bun-àm Sakhalin"}, {"Samara", "Bun-àm Samara"}, {"Samoa", "Bun-àm Samotha"}, {"Seychelles", "Àm nan Eileanan Sheiseall"}, {"Singapore", "Àm Singeapòr"}, {"Solomon", "Àm Eileanan Sholaimh"}, {"South_Georgia", "Àm Seòrsea a Deas"}, {"Suriname", "Àm Suranaim"}, {"Syowa", "Àm Shyowa"}, {"Tahiti", "Àm Tahiti"}, {"Taipei", "Bun-àm Taipei"}, {"Tajikistan", "Àm Taidigeastàin"}, {"Tokelau", "Àm Tokelau"}, {"Tonga", "Bun-àm Tonga"}, {"Truk", "Àm Chuuk"}, {"Turkmenistan", "Bun-àm Turcmanastàin"}, {"Tuvalu", "Àm Tubhalu"}, {"Uruguay", "Bun-àm Uruguaidh"}, {"Uzbekistan", "Bun-àm Usbagastàn"}, {"Vanuatu", "Bun-àm Vanuatu"}, {"Venezuela", "Àm na Bheiniseala"}, {"Vladivostok", "Bun-àm Vladivostok"}, {"Volgograd", "Bun-àm Volgograd"}, {"Vostok", "Àm Vostok"}, {"Wake", "Àm Eilean Wake"}, {"Wallis", "Àm Uallas agus Futuna"}, {"Yakutsk", "Bun-àm Yakutsk"}, {"Yekaterinburg", "Bun-àm Yekaterinburg"}, {"Yukon", "Àm Yukon"}},
		{{"Acre", "Tìde samhraidh Acre"}, {"Alaska", "Tìde samhraidh Alaska"}, {"Almaty", "Tìde samhraidh Almaty"}, {"Amazon", "Tìde samhraidh Amasoin"}, {"America_Central", "Tìde samhraidh Meadhan Aimeireaga"}, {"America_Eastern", "Tìde samhraidh Aimeireaga an Ear"}, {"America_Mountain", "Tìde samhraidh Monadh Aimeireaga"}, {"America_Pacific", "Tìde samhraidh a’ Chuain Shèimh"}, {"Anadyr", "Tìde samhraidh Anadyr"}, {"Apia", "Tìde samhraidh Apia"}, {"Aqtau", "Tìde samhraidh Aqtau"}, {"Aqtobe", "Tìde samhraidh Aqtobe"}, {"Arabian", "Tìde samhraidh Arabach"}, {"Argentina", "Tìde samhraidh na h-Argantaine"}, {"Argentina_Western", "Tìde samhraidh na h-Argantaine Siaraich"}, {"Armenia", "Tìde samhraidh Airmeinia"}, {"Atlantic", "Tìde samhraidh a’ Chuain Shiar"}, {"Australia_Central", "Tìde samhraidh Meadhan Astràilia"}, {"Australia_CentralWestern", "Tìde samhraidh Meadhan Astràilia an Iar"}, {"Australia_Eastern", "Tìde samhraidh Astràilia an Ear"}, {"Australia_Western", "Tìde samhraidh Astràilia an Iar"}, {"Azerbaijan", "Tìde samhraidh Asarbaideàin"}, {"Azores", "Tìde samhraidh nan Eileanan Asorach"}, {"Bangladesh", "Tìde samhraidh Bangladais"}, {"Brasilia", "Tìde samhraidh Bhrasília"}, {"Cape_Verde", "Tìde samhraidh a’ Chip Uaine"}, {"Chatham", "Tìde samhraidh Chatham"}, {"Chile", "Tìde samhraidh na Sile"}, {"China", "Tìde samhraidh na Sìne"}, {"Colombia", "Tìde samhraidh Coloimbia"}, {"Cook", "Leth-thìde samhraidh Eileanan Cook"}, {"Cuba", "Tìde samhraidh Cùba"}, {"Easter", "Tìde samhraidh Eilean na Càisge"}, {"Europe_Central", "Tìde samhraidh Meadhan na Roinn-Eòrpa"}, {"Europe_Eastern", "Tìde samhraidh na Roinn-Eòrpa an Ear"}, {"Europe_Western", "Tìde samhraidh na Roinn-Eòrpa an Iar"}, {"Falkland", "Tìde samhraidh nan Eileanan Fàclannach"}, {"Fiji", "Tìde samhraidh Fìdi"}, {"Georgia", "Tìde samhraidh na Cairtbheile"}, {"Greenland", "Tìde samhraidh na Graonlainne"}, {"Greenland_Eastern", "Tìde samhraidh na Graonlainn an Ear"}, {"Greenland_Western", "Tìde samhraidh na Graonlainn an Iar"}, {"Hawaii_Aleutian", "Tìde Samhraidh nan Eileanan Hawai’i ’s Aleutach"}, {"Hong_Kong", "Tìde samhraidh Hong Kong"}, {"Hovd", "Tìde samhraidh Khovd"}, {"Iran", "Tìde samhraidh Ioràin"}, {"Irkutsk", "Tìde Samhraidh Irkutsk"}, {"Israel", "Tìde samhraidh Iosrael"}, {"Japan", "Tìde samhraidh na Seapaine"}, {"Kamchatka", "Tìde samhraidh Petropavlovsk-Kamchatsky"}, {"Korea", "Tìde samhraidh Choirèa"}, {"Krasnoyarsk", "Tìde samhraidh Krasnoyarsk"}, {"Lord_Howe", "Tìde samhraidh Lord Howe"}, {"Macau", "Tìde samhraidh Macàthu"}, {"Magadan", "Tìde Samhraidh Magadan"}, {"Mauritius", "Tìde samhraidh nan Eileanan Mhoiriseas"}, {"Mexico_Pacific", "Tìde samhraidh a’ Chuain Shèimh Mheagsago"}, {"Mongolia", "Tìde samhraidh Ulan Bator"}, {"Moscow", "Tìde samhraidh Mhosgo"}, {"New_Caledonia", "Tìde samhraidh Chailleann Nuaidh"}, {"New_Zealand", "Tìde samhraidh Shealainn Nuaidh"}, {"Newfoundland", "Tìde samhraidh Talamh an Èisg"}, {"Norfolk", "Tìde samhraidh Eilein Norfolk"}, {"Noronha", "Tìde Samhraidh Fernando de Noronha"}, {"Novosibirsk", "Tìde samhraidh Novosibirsk"}, {"Omsk", "Tìde samhraidh Omsk"}, {"Pakistan", "Tìde samhraidh Pagastàin"}, {"Paraguay", "Tìde samhraidh Paraguaidh"}, {"Peru", "Tìde samhraidh Pearù"}, {"Philippines", "Tìde samhraidh nan Eilean Filipineach"}, {"Pierre_Miquelon", "Tìde Samhraidh Saint Pierre agus Miquelon"}, {"Qyzylorda", "Tìde samhraidh Qızılorda"}, {"Sakhalin", "Tìde samhraidh Sakhalin"}, {"Samara", "Tìde samhraidh Samara"}, {"Samoa", "Tìde samhraidh Samotha"}, {"Taipei", "Tìde samhraidh Taipei"}, {"Tonga", "Tìde samhraidh Tonga"}, {"Turkmenistan", "Tìde samhraidh Turcmanastàin"}, {"Uruguay", "Tìde samhraidh Uruguaidh"}, {"Uzbekistan", "Tìde samhraidh Usbagastàn"}, {"Vanuatu", "Tìde samhraidh Vanuatu"}, {"Vladivostok", "Tìde Samhraidh Vladivostok"}, {"Volgograd", "Tìde samhraidh Volgograd"}, {"Yakutsk", "Tìde samhraidh Yakutsk"}, {"Yekaterinburg", "Tìde samhraidh Yekaterinburg"}},
	},
	{
		{{"Acre", "hora estándar de Acre"}, {"Afghanistan", "hora de Afganistán"}, {"Africa_Central", "hora de África Central"}, {"Africa_Eastern", "hora de África Oriental"}, {"Africa_Southern", "hora de África Meridional"}, {"Africa_Western", "hora de África Occidental"}, {"Alaska", "hora estándar de Alasca"}, {"Amazon", "hora estándar do Amazonas"}, {"America_Central", "hora estándar central, Norteamérica"}, {"America_Eastern", "hora estándar do leste, América do Norte"}, {"America_Mountain", "hora estándar da montaña, América do Norte"}, {"America_Pacific", "hora estándar do Pacífico, América do Norte"}, {"Anadyr", "Horario estándar de Anadir"}, {"Apia", "hora estándar de Apia"}, {"Arabian", "hora estándar árabe"}, {"Argentina", "hora estándar da Arxentina"}, {"Argentina_Western", "hora estándar da Arxentina Occidental"}, {"Armenia", "hora estándar de Armenia"}, {"Atlantic", "hora estándar do Atlántico"}, {"Australia_Central", "hora estándar de Australia Central"}, {"Australia_CentralWestern", "hora estándar de Australia Occidental Central"}, {"Australia_Eastern", "hora estándar de Australia Oriental"}, {"Australia_Western", "hora estándar de Australia Occidental"}, {"Azerbaijan", "hora estándar de Acerbaixán"}, {"Azores", "hora estándar dos Azores"}, {"Bangladesh", "hora estándar de Bangladesh"}, {"Bhutan", "hora de Bután"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora estándar de Brasilia"}, {"Brunei", "hora de Brunei Darussalam"}, {"Cape_Verde", "hora estándar de Cabo Verde"}, {"Chamorro", "hora estándar chamorro"}, {"Chatham", "hora estándar de Chatham"}, {"Chile", "hora estándar de Chile"}, {"China", "hora estándar da China"}, {"Christmas", "hora da Illa Christmas"}, {"Cocos", "hora das Illas Cocos"}, {"Colombia", "hora estándar de Colombia"}, {"Cook", "hora estándar das Illas Cook"}, {"Cuba", "hora estándar de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont-d’Urville"}, {"East_Timor", "hora de Timor Leste"}, {"Easter", "hora estándar da Illa de Pascua"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora estándar de Europa Central"}, {"Europe_Eastern", "hora estándar de Europa Oriental"}, {"Europe_Further_Eastern", "hora do extremo leste europeo"}, {"Europe_Western", "hora estándar de Europa Occidental"}, {"Falkland", "hora estándar das Illas Malvinas"}, {"Fiji", "hora estándar de Fixi"}, {"French_Guiana", "hora da Güiana Francesa"}, {"French_Southern", "hora das Terras Austrais e Antárticas Francesas"}, {"GMT", "hora do meridiano de Greenwich"}, {"Galapagos", "hora das Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora estándar de Xeorxia"}, {"Gilbert_Islands", "hora das Illas Gilbert"}, {"Greenland_Eastern", "hora estándar de Groenlandia Oriental"}, {"Greenland_Western", "hora estándar de Groenlandia Occidental"}, {"Gulf", "hora do Golfo"}, {"Guyana", "hora da Güiana"}, {"Hawaii", "hora estándar de Hawai-illas Aleutianas"}, {"Hawaii_Aleutian", "hora estándar de Hawai-illas Aleutianas"}, {"Hong_Kong", "hora estándar de Hong Kong"}, {"Hovd", "hora estándar de Hovd"}, {"India", "hora da India"}, {"Indian_Ocean", "hora do Océano Índico"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia Central"}, {"Indonesia_Eastern", "hora de Indonesia Oriental"}, {"Indonesia_Western", "hora de Indonesia Occidental"}, {"Iran", "hora estándar de Irán"}, {"Irkutsk", "hora estándar de Irkutsk"}, {"Israel", "hora estándar de Israel"}, {"Japan", "hora estándar do Xapón"}, {"Kamchatka", "Horario estándar de Petropávlovsk-Kamchatski"}, {"Kazakhstan", "hora de Kazakistán"}, {"Kazakhstan_Eastern", "hora de Kazakistán Oriental"}, {"Kazakhstan_Western", "hora de Kazakistán Occidental"}, {"Korea", "hora estándar de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora estándar de Krasnoiarsk"}, {"Kyrgystan", "hora de Kirguizistán"}, {"Line_Islands", "hora das Illas da Liña"}, {"Lord_Howe", "hora estándar de Lord Howe"}, {"Magadan", "hora estándar de Magadan"}, {"Malaysia", "hora de Malaisia"}, {"Maldives", "hora das Maldivas"}, {"Marquesas", "hora das Marquesas"}, {"Marshall_Islands", "hora das Illas Marshall"}, {"Mauritius", "hora estándar de Mauricio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora estándar do Pacífico mexicano"}, {"Mongolia", "hora estándar de Ulaanbaatar"}, {"Moscow", "hora estándar de Moscova"}, {"Myanmar", "hora de Myanmar"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora estándar de Nova Caledonia"}, {"New_Zealand", "hora estándar de Nova Zelandia"}, {"Newfoundland", "hora estándar de Terra Nova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora estándar da Illa Norfolk"}, {"Noronha", "hora estándar de Fernando de Noronha"}, {"Novosibirsk", "hora estándar de Novosibirsk"}, {"Omsk", "hora estándar de Omsk"}, {"Pakistan", "hora estándar de Paquistán"}, {"Palau", "hora de Palau"}, {"Papua_New_Guinea", "hora de Papúa-Nova Guinea"}, {"Paraguay", "hora estándar do Paraguai"}, {"Peru", "hora estándar do Perú"}, {"Philippines", "hora estándar de Filipinas"}, {"Phoenix_Islands", "hora das Illas Fénix"}, {"Pierre_Miquelon", "hora estándar de Saint Pierre et Miquelon"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Pyongyang"}, {"Reunion", "hora de Reunión"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora estándar de Sakhalín"}, {"Samara", "Horario estándar de Samara"}, {"Samoa", "hora estándar de Samoa"}, {"Seychelles", "hora das Seychelles"}, {"Singapore", "hora de Singapur"}, {"Solomon", "hora das Illas Salomón"}, {"South_Georgia", "hora de Xeorxia do Sur"}, {"Suriname", "hora de Suriname"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahití"}, {"Taipei", "hora estándar de Taipei"}, {"Tajikistan", "hora de Taxiquistán"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora estándar de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkmenistan", "hora estándar de Turkmenistán"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora estándar do Uruguai"}, {"Uzbekistan", "hora estándar de Uzbekistán"}, {"Vanuatu", "hora estándar de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora estándar de Vladivostok"}, {"Volgograd", "hora estándar de Volgogrado"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora da Illa Wake"}, {"Wallis", "hora de Wallis e Futuna"}, {"Yakutsk", "hora estándar de Iakutsk"}, {"Yekaterinburg", "hora estándar de Ekaterimburgo"}, {"Yukon", "hora de Yukon"}},
		{{"Acre", "hora de verán de Acre"}, {"Alaska", "hora de verán de Alasca"}, {"Amazon", "hora de verán do Amazonas"}, {"America_Central", "hora de verán central, Norteamérica"}, {"America_Eastern", "hora de verán do leste, América do Norte"}, {"America_Mountain", "hora de verán da montaña, América do Norte"}, {"America_Pacific", "hora de verán do Pacífico, América do Norte"}, {"Anadyr", "Horario de verán de Anadir"}, {"Apia", "hora de verán de Apia"}, {"Arabian", "hora de verán árabe"}, {"Argentina", "hora de verán da Arxentina"}, {"Argentina_Western", "hora de verán da Arxentina Occidental"}, {"Armenia", "hora de verán de Armenia"}, {"Atlantic", "hora de verán do Atlántico"}, {"Australia_Central", "hora de verán de Australia Central"}, {"Australia_CentralWestern", "hora de verán de Australia Occidental Central"}, {"Australia_Eastern", "hora de verán de Australia Oriental"}, {"Australia_Western", "hora de verán de Australia Occidental"}, {"Azerbaijan", "hora de verán de Acerbaixán"}, {"Azores", "hora de verán dos Azores"}, {"Bangladesh", "hora de verán de Bangladesh"}, {"Brasilia", "hora de verán de Brasilia"}, {"Cape_Verde", "hora de verán de Cabo Verde"}, {"Chatham", "hora de verán de Chatham"}, {"Chile", "hora de verán de Chile"}, {"China", "hora de verán da China"}, {"Colombia", "hora de verán de Colombia"}, {"Cook", "hora de verán medio das Illas Cook"}, {"Cuba", "hora de verán de Cuba"}, {"Easter", "hora de verán da Illa de Pascua"}, {"Europe_Central", "hora de verán de Europa Central"}, {"Europe_Eastern", "hora de verán de Europa Oriental"}, {"Europe_Western", "hora de verán de Europa Occidental"}, {"Falkland", "hora de verán das Illas Malvinas"}, {"Fiji", "hora de verán de Fixi"}, {"Georgia", "hora de verán de Xeorxia"}, {"Greenland_Eastern", "hora de verán de Groenlandia Oriental"}, {"Greenland_Western", "hora de verán de Groenlandia Occidental"}, {"Hawaii_Aleutian", "hora de verán de Hawai-illas Aleutianas"}, {"Hong_Kong", "hora de verán de Hong Kong"}, {"Hovd", "hora de verán de Hovd"}, {"Iran", "hora de verán de Irán"}, {"Irkutsk", "hora de verán de Irkutsk"}, {"Israel", "hora de verán de Israel"}, {"Japan", "hora de verán do Xapón"}, {"Kamchatka", "Horario de verán de Petropávlovsk-Kamchatski"}, {"Korea", "hora de verán de Corea"}, {"Krasnoyarsk", "hora de verán de Krasnoiarsk"}, {"Lord_Howe", "hora de verán de Lord Howe"}, {"Magadan", "hora de verán de Magadan"}, {"Mauritius", "hora de verán de Mauricio"}, {"Mexico_Pacific", "hora de verán do Pacífico mexicano"}, {"Mongolia", "hora de verán de Ulaanbaatar"}, {"Moscow", "hora de verán de Moscova"}, {"New_Caledonia", "hora de verán de Nova Caledonia"}, {"New_Zealand", "hora de verán de Nova Zelandia"}, {"Newfoundland", "hora de verán de Terra Nova"}, {"Norfolk", "hora de verán da Illa Norfolk"}, {"Noronha", "hora de verán de Fernando de Noronha"}, {"Novosibirsk", "hora de verán de Novosibirsk"}, {"Omsk", "hora de verán de Omsk"}, {"Pakistan", "hora de verán de Paquistán"}, {"Paraguay", "hora de verán do Paraguai"}, {"Peru", "hora de verán do Perú"}, {"Philippines", "hora de verán de Filipinas"}, {"Pierre_Miquelon", "hora de verán de Saint Pierre et Miquelon"}, {"Sakhalin", "hora de verán de Sakhalín"}, {"Samara", "Horario de verán de Samara"}, {"Samoa", "hora de verán de Samoa"}, {"Taipei", "hora de verán de Taipei"}, {"Tonga", "hora de verán de Tonga"}, {"Turkmenistan", "hora de verán de Turkmenistán"}, {"Uruguay", "hora de verán do Uruguai"}, {"Uzbekistan", "hora de verán de Uzbekistán"}, {"Vanuatu", "hora de verán de Vanuatu"}, {"Vladivostok", "hora de verán de Vladivostok"}, {"Volgograd", "hora de verán de Volgogrado"}, {"Yakutsk", "hora de verán de Iakutsk"}, {"Yekaterinburg", "hora de verán de Ekaterimburgo"}},
	},
	{
		{{"Bolivia", "Bolivia óra"}, {"Ecuador", "Ecuador óra"}, {"Galapagos", "Galápagos óra"}, {"Venezuela", "Venezuela óra"}},
		{},
	},
	{
		{{"Acre", "Acre-Schtandardziit"}, {"Afghanistan", "Afghanischtan-Ziit"}, {"Africa_Central", "Zentralafrikanischi Ziit"}, {"Africa_Eastern", "Oschtafrikanischi Ziit"}, {"Africa_Southern", "Süüdafrikanischi ziit"}, {"Africa_Western", "Weschtafrikanischi Ziit"}, {"Alaska", "Alaska-Schtandardziit"}, {"Almaty", "Almaty-Schtandardziit"}, {"Amazon", "Amazonas-Schtandardziit"}, {"America_Central", "Amerika-Zentraal Schtandardziit"}, {"Europe_Central", "Mitteleuropäischi Schtandardziit"}, {"Europe_Eastern", "Oschteuropäischi Schtandardziit"}, {"Europe_Western", "Weschteuropäischi Schtandardziit"}, {"Moscow", "Moskauer Schtandardziit"}},
		{{"Acre", "Acre-Summerziit"}, {"Alaska", "Alaska-Summerziit"}, {"Almaty", "Almaty-Summerziit"}, {"Amazon", "Amazonas-Summerziit"}, {"America_Central", "Amerika-Zentraal Summerziit"}, {"Europe_Central", "Mitteleuropäischi Summerziit"}, {"Europe_Eastern", "Oschteuropäischi Summerziit"}, {"Europe_Western", "Weschteuropäischi Summerziit"}, {"Moscow", "Moskauer Summerziit"}},
	},
	{
		{{"Acre", "એકર પ્રમાણભૂત સમય"}, {"Afghanistan", "અફઘાનિસ્તાન સમય"}, {"Africa_Central", "મધ્ય આફ્રિકા સમય"}, {"Africa_Eastern", "પૂર્વ આફ્રિકા સમય"}, {"Africa_Southern", "દક્ષિણ આફ્રિકા માનક સમય"}, {"Africa_Western", "પશ્ચિમ આફ્રિકા સમય"}, {"Alaska", "અલાસ્કા માનક સમય"}, {"Almaty", "અલ્માટી પ્રમાણભૂત સમય"}, {"Amazon", "એમેઝોન માનક સમય"}, {"America_Central", "ઉત્તર અમેરિકન કેન્દ્રીય માનક સમય"}, {"America_Eastern", "ઉત્તર અમેરિકન પૂર્વી માનક સમય"}, {"America_Mountain", "ઉત્તર અમેરિકન માઉન્ટન માનક સમય"}, {"America_Pacific", "ઉત્તર અમેરિકન પેસિફિક માનક સમય"}, {"Anadyr", "અનાડિર પ્રમાણભૂત સમય"}, {"Apia", "એપિયા માનક સમય"}, {"Aqtau", "અક્તાઉ પ્રમાણભૂત સમય"}, {"Aqtobe", "ઍક્ટોબ પ્રમાણભૂત સમય"}, {"Arabian", "અરેબિયન માનક સમય"}, {"Argentina", "આર્જેન્ટીના માનક સમય"}, {"Argentina_Western", "પશ્ચિમી આર્જેન્ટીના માનક સમય"}, {"Armenia", "આર્મેનિયા માનક સમય"}, {"Atlantic", "એટલાન્ટિક માનક સમય"}, {"Australia_Central", "ઑસ્ટ્રેલિયન કેન્દ્રીય માનક સમય"}, {"Australia_CentralWestern", "ઑસ્ટ્રેલિયન કેન્દ્રીય પશ્ચિમી માનક સમય"}, {"Australia_Eastern", "ઑસ્ટ્રેલિયન પૂર્વીય માનક સમય"}, {"Australia_Western", "ઑસ્ટ્રેલિયન પશ્ચિમી માનક સમય"}, {"Azerbaijan", "અઝરબૈજાન માનક સમય"}, {"Azores", "એઝોર્સ માનક સમય"}, {"Bangladesh", "બાંગ્લાદેશ માનક સમય"}, {"Bhutan", "ભૂટાન સમય"}, {"Bolivia", "બોલિવિયા સમય"}, {"Brasilia", "બ્રાઝિલિયા માનક સમય"}, {"Brunei", "બ્રુનેઇ દરુસલામ સમય"}, {"Cape_Verde", "કૅપ વર્ડે માનક સમય"}, {"Chamorro", "કેમોરો માનક સમય"}, {"Chatham", "ચેતહામ માનક સમય"}, {"Chile", "ચિલી માનક સમય"}, {"China", "ચીન માનક સમય"}, {"Christmas", "ક્રિસમસ આઇલેન્ડ સમય"}, {"Cocos", "કોકોઝ આઇલેન્ડ્સ સમય"}, {"Colombia", "કોલમ્બિયા માનક સમય"}, {"Cook", "કુક આઇલેન્ડ્સ માનક સમય"}, {"Cuba", "ક્યુબા માનક સમય"}, {"Davis", "ડેવિસ સમય"}, {"DumontDUrville", "ડ્યુમોન્ટ-ડી‘ઉર્વિલ સમય"}, {"East_Timor", "પૂર્વ તિમોર સમય"}, {"Easter", "ઇસ્ટર આઇલેન્ડ માનક સમય"}, {"Ecuador", "એક્વાડોર સમય"}, {"Europe_Central", "મધ્ય યુરોપિયન માનક સમય"}, {"Europe_Eastern", "પૂર્વી યુરોપિયન માનક સમય"}, {"Europe_Further_Eastern", "સુદૂર-પૂર્વી યુરોપિયન સમય"}, {"Europe_Western", "પશ્ચિમી યુરોપિયન માનક સમય"}, {"Falkland", "ફૉકલેન્ડ આઇલેન્ડ્સ માનક સમય"}, {"Fiji", "ફીજી માનક સમય"}, {"French_Guiana", "ફ્રેંચ ગયાના સમય"}, {"French_Southern", "ફ્રેંચ સધર્ન અને એન્ટાર્કટિક સમય"}, {"GMT", "ગ્રીનવિચ મધ્યમ સમય"}, {"Galapagos", "ગાલાપાગોસ સમય"}, {"Gambier", "ગેમ્બિયર સમય"}, {"Georgia", "જ્યોર્જિયા માનક સમય"}, {"Gilbert_Islands", "ગિલબર્ટ આઇલેન્ડ્સ સમય"}, {"Greenland_Eastern", "પૂર્વ ગ્રીનલેન્ડ માનક સમય"}, {"Greenland_Western", "પશ્ચિમ ગ્રીનલેન્ડ માનક સમય"}, {"Guam", "ગ્વામ પ્રમાણભૂત સમય"}, {"Gulf", "ગલ્ફ માનક સમય"}, {"Guyana", "ગયાના સમય"}, {"Hawaii", "હવાઇ-એલ્યુશિઅન માનક સમય"}, {"Hawaii_Aleutian", "હવાઇ-એલ્યુશિઅન માનક સમય"}, {"Hong_Kong", "હોંગ કોંગ માનક સમય"}, {"Hovd", "હોવ્ડ માનક સમય"}, {"India", "ભારતીય માનક સમય"}, {"Indian_Ocean", "ભારતીય મહાસાગર સમય"}, {"Indochina", "ઇન્ડોચાઇના સમય"}, {"Indonesia_Central", "મધ્ય ઇન્ડોનેશિયા સમય"}, {"Indonesia_Eastern", "પૂર્વીય ઇન્ડોનેશિયા સમય"}, {"Indonesia_Western", "પશ્ચિમી ઇન્ડોનેશિયા સમય"}, {"Iran", "ઈરાન માનક સમય"}, {"Irkutsk", "ઇર્કુત્સ્ક માનક સમય"}, {"Israel", "ઇઝરાઇલ માનક સમય"}, {"Japan", "જાપાન માનક સમય"}, {"Kamchatka", "પેટ્રોપેવલોવ્સ્ક-કામચતસ્કી પ્રમાણભૂત સમય"}, {"Kazakhstan", "કઝાકિસ્તાન સમય"}, {"Kazakhstan_Eastern", "પૂર્વ કઝાકિસ્તાન સમય"}, {"Kazakhstan_Western", "પશ્ચિમ કઝાકિસ્તાન સમય"}, {"Korea", "કોરિયન માનક સમય"}, {"Kosrae", "કોસરે સમય"}, {"Krasnoyarsk", "ક્રેસ્નોયાર્સ્ક માનક સમય"}, {"Kyrgystan", "કિર્ગિઝ્સ્તાન સમય"}, {"Lanka", "લંકા સમય"}, {"Line_Islands", "લાઇન આઇલેન્ડ્સ સમય"}, {"Lord_Howe", "લોર્ડ હોવ માનક સમય"}, {"Macau", "મકાઉ પ્રમાણભૂત સમય"}, {"Magadan", "મગાડન માનક સમય"}, {"Malaysia", "મલેશિયા સમય"}, {"Maldives", "માલદીવ સમય"}, {"Marquesas", "માર્ક્યૂસસ સમય"}, {"Marshall_Islands", "માર્શલ આઇલેન્ડ્સ સમય"}, {"Mauritius", "મોરિશિયસ માનક સમય"}, {"Mawson", "મોસન સમય"}, {"Mexico_Pacific", "મેક્સિકન પેસિફિક માનક સમય"}, {"Mongolia", "ઉલાન બાટોર માનક સમય"}, {"Moscow", "મોસ્કો માનક સમય"}, {"Myanmar", "મ્યાનમાર સમય"}, {"Nauru", "નૌરુ સમય"}, {"Nepal", "નેપાળ સમય"}, {"New_Caledonia", "ન્યુ સેલેડોનિયા માનક સમય"}, {"New_Zealand", "ન્યુઝીલેન્ડ માનક સમય"}, {"Newfoundland", "ન્યૂફાઉન્ડલેન્ડ માનક સમય"}, {"Niue", "નીયુ સમય"}, {"Norfolk", "નોરફૉક આઇલેન્ડ માનક સમય"}, {"Noronha", "ફર્નાન્ડો ડી નોરોન્હા માનક સમય"}, {"North_Mariana", "ઉત્તર મારિયાના આઇલેન્ડ્સ"}, {"Novosibirsk", "નોવસિબિર્સ્ક માનક સમય"}, {"Omsk", "ઓમ્સ્ક માનક સમય"}, {"Pakistan", "પાકિસ્તાન માનક સમય"}, {"Palau", "પલાઉ સમય"}, {"Papua_New_Guinea", "પાપુઆ ન્યુ ગિની સમય"}, {"Paraguay", "પેરાગ્વે માનક સમય"}, {"Peru", "પેરુ માનક સમય"}, {"Philippines", "ફિલિપાઇન માનક સમય"}, {"Phoenix_Islands", "ફોનિક્સ આઇલેન્ડ્સ સમય"}, {"Pierre_Miquelon", "સેંટ પીએરી અને મિક્યુલોન માનક સમય"}, {"Pitcairn", "પિટકેયર્ન સમય"}, {"Ponape", "પોનપે સમય"}, {"Pyongyang", "પ્યોંગયાંગ સમય"}, {"Qyzylorda", "કિઝિલોર્ડા પ્રમાણભૂત સમય"}, {"Reunion", "રીયુનિયન સમય"}, {"Rothera", "રોથેરા સમય"}, {"Sakhalin", "સખાલિન માનક સમય"}, {"Samara", "સમારા પ્રમાણભૂત સમય"}, {"Samoa", "સમોઆ માનક સમય"}, {"Seychelles", "સેશલ્સ સમય"}, {"Singapore", "સિંગાપુર માનક સમય"}, {"Solomon", "સોલોમન આઇલેન્ડ્સ સમય"}, {"South_Georgia", "દક્ષિણ જ્યોર્જિયા સમય"}, {"Suriname", "સુરીનામ સમય"}, {"Syowa", "સ્યોવા સમય"}, {"Tahiti", "તાહિતી સમય"}, {"Taipei", "તાઇપેઇ માનક સમય"}, {"Tajikistan", "તાજીકિસ્તાન સમય"}, {"Tokelau", "ટોકલાઉ સમય"}, {"Tonga", "ટોંગા માનક સમય"}, {"Truk", "ચુઉક સમય"}, {"Turkmenistan", "તુર્કમેનિસ્તાન માનક સમય"}, {"Tuvalu", "ટવાલૂ સમય"}, {"Uruguay", "ઉરુગ્વે માનક સમય"}, {"Uzbekistan", "ઉઝ્બેકિસ્તાન માનક સમય"}, {"Vanuatu", "વનાતૂ માનક સમય"}, {"Venezuela", "વેનેઝુએલા સમય"}, {"Vladivostok", "વ્લાડિવોસ્ટોક માનક સમય"}, {"Volgograd", "વોલ્ગોગ્રેડ માનક સમય"}, {"Vostok", "વોસ્ટોક સમય"}, {"Wake", "વૅક આઇલેન્ડ સમય"}, {"Wallis", "વૉલિસ અને ફ્યુચુના સમય"}, {"Yakutsk", "યાકુત્સ્ક માનક સમય"}, {"Yekaterinburg", "યેકાટેરિનબર્ગ માનક સમય"}, {"Yukon", "યુકોન સમય"}},
		{{"Acre", "એકર ગ્રીષ્મ સમય"}, {"Alaska", "અલાસ્કા દિવસ સમય"}, {"Almaty", "અલ્માટી ગ્રીષ્મ સમય"}, {"Amazon", "એમેઝોન ગ્રીષ્મ સમય"}, {"America_Central", "ઉત્તર અમેરિકન કેન્દ્રીય દિવસ સમય"}, {"America_Eastern", "ઉત્તર અમેરિકન પૂર્વી દિવસ સમય"}, {"America_Mountain", "ઉત્તર અમેરિકન માઉન્ટન દિવસ સમય"}, {"America_Pacific", "ઉત્તર અમેરિકન પેસિફિક દિવસ સમય"}, {"Anadyr", "અનાડિર ગ્રીષ્મ સમય"}, {"Apia", "એપિયા દિવસ સમય"}, {"Aqtau", "અક્તાઉ ગ્રીષ્મ સમય"}, {"Aqtobe", "ઍક્ટોબ ગ્રીષ્મ સમય"}, {"Arabian", "અરેબિયન દિવસ સમય"}, {"Argentina", "આર્જેન્ટીના ગ્રીષ્મ સમય"}, {"Argentina_Western", "પશ્ચિમી આર્જેન્ટીના ગ્રીષ્મ સમય"}, {"Armenia", "આર્મેનિયા ગ્રીષ્મ સમય"}, {"Atlantic", "એટલાન્ટિક દિવસ સમય"}, {"Australia_Central", "ઓસ્ટ્રેલિયન કેન્દ્રીય દિવસ સમય"}, {"Australia_CentralWestern", "ઓસ્ટ્રેલિયન કેન્દ્રીય પશ્ચિમી દિવસ સમય"}, {"Australia_Eastern", "ઓસ્ટ્રેલિયન પૂર્વીય દિવસ સમય"}, {"Australia_Western", "ઓસ્ટ્રેલિયન પશ્ચિમી દિવસ સમય"}, {"Azerbaijan", "અઝરબૈજાન ગ્રીષ્મ સમય"}, {"Azores", "એઝોર્સ ગ્રીષ્મ સમય"}, {"Bangladesh", "બાંગ્લાદેશ ગ્રીષ્મ સમય"}, {"Brasilia", "બ્રાઝિલિયા ગ્રીષ્મ સમય"}, {"Cape_Verde", "કૅપ વર્ડે ગ્રીષ્મ સમય"}, {"Chatham", "ચેતહામ દિવસ સમય"}, {"Chile", "ચિલી ગ્રીષ્મ સમય"}, {"China", "ચીન દિવસ સમય"}, {"Colombia", "કોલમ્બિયા ગ્રીષ્મ સમય"}, {"Cook", "કુક આઇલેન્ડ્સ અર્ધ ગ્રીષ્મ સમય"}, {"Cuba", "ક્યુબા દિવસ સમય"}, {"Easter", "ઇસ્ટર આઇલેન્ડ ગ્રીષ્મ સમય"}, {"Europe_Central", "મધ્ય યુરોપિયન ગ્રીષ્મ સમય"}, {"Europe_Eastern", "પૂર્વી યુરોપિયન ગ્રીષ્મ સમય"}, {"Europe_Western", "પશ્ચિમી યુરોપિયન ગ્રીષ્મ સમય"}, {"Falkland", "ફૉકલેન્ડ આઇલેન્ડ્સ ગ્રીષ્મ સમય"}, {"Fiji", "ફીજી ગ્રીષ્મ સમય"}, {"Georgia", "જ્યોર્જિયા ગ્રીષ્મ સમય"}, {"Greenland_Eastern", "પૂર્વ ગ્રીનલેન્ડ ગ્રીષ્મ સમય"}, {"Greenland_Western", "પશ્ચિમ ગ્રીનલેન્ડ ગ્રીષ્મ સમય"}, {"Hawaii_Aleutian", "હવાઇ-એલ્યુશિઅન દિવસ સમય"}, {"Hong_Kong", "હોંગ કોંગ ગ્રીષ્મ સમય"}, {"Hovd", "હોવ્ડ ગ્રીષ્મ સમય"}, {"Iran", "ઈરાન દિવસ સમય"}, {"Irkutsk", "ઇર્કુત્સ્ક ગ્રીષ્મ સમય"}, {"Israel", "ઇઝરાઇલ દિવસ સમય"}, {"Japan", "જાપાન દિવસ સમય"}, {"Kamchatka", "પેટ્રોપેવલોવ્સ્ક-કામચતસ્કી ગ્રીષ્મ સમય"}, {"Korea", "કોરિયન દિવસ સમય"}, {"Krasnoyarsk", "ક્રેસ્નોયાર્સ્ક ગ્રીષ્મ સમય"}, {"Lord_Howe", "લોર્ડ હોવ દિવસ સમય"}, {"Macau", "મકાઉ ગ્રીષ્મ સમય"}, {"Magadan", "મગાડન ગ્રીષ્મ સમય"}, {"Mauritius", "મોરિશિયસ ગ્રીષ્મ સમય"}, {"Mexico_Pacific", "મેક્સિકન પેસિફિક દિવસ સમય"}, {"Mongolia", "ઉલાન બટોર ગ્રીષ્મ સમય"}, {"Moscow", "મોસ્કો ગ્રીષ્મ સમય"}, {"New_Caledonia", "ન્યુ સેલેડોનિયા ગ્રીષ્મ સમય"}, {"New_Zealand", "ન્યુઝીલેન્ડ દિવસ સમય"}, {"Newfoundland", "ન્યૂફાઉન્ડલેન્ડ દિવસ સમય"}, {"Norfolk", "નોરફૉક આઇલેન્ડ દિવસ સમય"}, {"Noronha", "ફર્નાન્ડો દે નોરોહા ગ્રીષ્મ સમય"}, {"Novosibirsk", "નોવસિબિર્સ્ક ગ્રીષ્મ સમય"}, {"Omsk", "ઓમ્સ્ક ગ્રીષ્મ સમય"}, {"Pakistan", "પાકિસ્તાન ગ્રીષ્મ સમય"}, {"Paraguay", "પેરાગ્વે ગ્રીષ્મ સમય"}, {"Peru", "પેરુ ગ્રીષ્મ સમય"}, {"Philippines", "ફિલિપાઇન ગ્રીષ્મ સમય"}, {"Pierre_Miquelon", "સેંટ પીએરી અને મિક્યુલોન દિવસ સમય"}, {"Qyzylorda", "કિઝિલોર્ડા ગ્રીષ્મ સમય"}, {"Sakhalin", "સખાલિન ગ્રીષ્મ સમય"}, {"Samara", "સમારા ગ્રીષ્મ સમય"}, {"Samoa", "સમોઆ ગ્રીષ્મ સમય"}, {"Taipei", "તાઇપેઇ દિવસ સમય"}, {"Tonga", "ટોંગા ગ્રીષ્મ સમય"}, {"Turkmenistan", "તુર્કમેનિસ્તાન ગ્રીષ્મ સમય"}, {"Uruguay", "ઉરુગ્વે ગ્રીષ્મ સમય"}, {"Uzbekistan", "ઉઝ્બેકિસ્તાન ગ્રીષ્મ સમય"}, {"Vanuatu", "વનાતૂ ગ્રીષ્મ સમય"}, {"Vladivostok", "વ્લાડિવોસ્ટોક ગ્રીષ્મ સમય"}, {"Volgograd", "વોલ્ગોગ્રેડ ગ્રીષ્મ સમય"}, {"Yakutsk", "યાકુત્સ્ક ગ્રીષ્મ સમય"}, {"Yekaterinburg", "યેકાટેરિનબર્ગ ગ્રીષ્મ સમય"}},
	},
	{
		{{"Afghanistan", "Lokacin Afghanistan"}, {"Africa_Central", "Lokacin Afirka ta Tsakiya"}, {"Africa_Eastern", "Lokacin Gabashin Afirka"}, {"Africa_Southern", "Tsayayyen Lokacin Afirka ta Kudu"}, {"Africa_Western", "Lokacin Afirka ta Yamma"}, {"Alaska", "Tsayayyen Lokacin Alaska"}, {"Amazon", "Tsayayyen Lokacin Amazon"}, {"America_Central", "Tsayayyen Lokaci dake Arewacin Amurika ta Tsakiya"}, {"America_Eastern", "Tsayayyen Lokacin Gabas dake Arewacin Amurika"}, {"America_Mountain", "Lokaci tsayayye na tsauni a Arewacin Amurica"}, {"America_Pacific", "Lokaci Tsayayye na Arewacin Amurika"}, {"Apia", "Tsayayyen Lokacin Apia"}, {"Arabian", "Tsayayyen lokacin Arebiya"}, {"Argentina", "Tsayayyen Lokacin Argentina"}, {"Argentina_Western", "Tsayayyen Lokacin Yammacin Argentina"}, {"Armenia", "Tsayayyen Lokacin Armenia"}, {"Atlantic", "Tsayayyen Lokacin Kanada, Puerto Rico da Virgin Islands"}, {"Australia_Central", "Tsayayyen Lokacin Tsakiyar Austiraliya"}, {"Australia_CentralWestern", "Tsayayyen Lokacin Yammacin Tsakiyar Austiraliya"}, {"Australia_Eastern", "Tsayayyen lokacin Gabashin Australia"}, {"Australia_Western", "Tsayayyen Lokacin Yammacin Austiralia"}, {"Azerbaijan", "Tsayayyen Lokacin Azerbaijan"}, {"Azores", "Lokacin Azores Daidaitacce"}, {"Bangladesh", "Tsayayyen Lokacin Bangladesh"}, {"Bhutan", "Lokacin Bhutan"}, {"Bolivia", "Lokacin Bolivia"}, {"Brasilia", "Tsayayyen Lokacin Brasillia"}, {"Brunei", "Lokacin Brunei Darussalam"}, {"Cape_Verde", "Tsayayyen lokacin Cape Verde"}, {"Chamorro", "Tsayayyen Lokacin Chamorro"}, {"Chatham", "Tsayayyen Lokacin Chatham"}, {"Chile", "Tsayayyen Lokacin Chile"}, {"China", "Tsayayyen Lokacin Sin"}, {"Christmas", "Lokacin Christmas Island"}, {"Cocos", "Lokacin Cocos Islands"}, {"Colombia", "Tsayayyen Lokacin Colombia"}, {"Cook", "Tsayayyen Lokacin Cook Islands"}, {"Cuba", "Tsayayyen Lokacin Kuba"}, {"Davis", "Lokacin Davis"}, {"DumontDUrville", "Lokacin Dumont-d’Urville"}, {"East_Timor", "Lokacin East Timor"}, {"Easter", "Tsayayyen Lokacin Easter Island"}, {"Ecuador", "Lokacin Ecuador"}, {"Europe_Central", "Ida Tsakiyar a Lokaci Turai"}, {"Europe_Eastern", "Ida lokaci a turai gabas"}, {"Europe_Further_Eastern", "Lokacin Gabashin Turai mai Nisa"}, {"Europe_Western", "Ida lokaci ta yammacin turai"}, {"Falkland", "Tsayayyen Lokacin Falkland Islands"}, {"Fiji", "Tsayayyen Lokacin Fiji"}, {"French_Guiana", "Lokacin French Guiana"}, {"French_Southern", "Lokacin Kudancin Faransa da Antarctic"}, {"GMT", "Lokacin Greenwich a Ingila"}, {"Galapagos", "Lokacin Galapagos"}, {"Gambier", "Lokacin Gambier"}, {"Georgia", "Tsayayyen Lokacin Georgia"}, {"Gilbert_Islands", "Lokacin Gilbert Islands"}, {"Greenland_Eastern", "Tsayayyen Lokacin Gabashin Greenland"}, {"Greenland_Western", "Tsayayyen Lokacin Yammacin Greenland"}, {"Gulf", "Tsayayyen lokacin Gulf"}, {"Guyana", "Lokacin Guyana"}, {"Hawaii", "Tsayayyen Lokacin Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Tsayayyen Lokacin Hawaii-Aleutian"}, {"Hong_Kong", "Tsayayyen Lokacin Hong Kong"}, {"Hovd", "Tsayayyen Lokacin Hovd"}, {"India", "Tsayayyen lokacin Indiya"}, {"Indian_Ocean", "Lokacin Tekun Indiya"}, {"Indochina", "Lokacin Indochina"}, {"Indonesia_Central", "Lokacin Indonesia ta Tsakiya"}, {"Indonesia_Eastern", "Lokacin Gabashin Indonesia"}, {"Indonesia_Western", "Lokacin Yammacin Indonesia"}, {"Iran", "Tsayayyen Lokacin Iran"}, {"Irkutsk", "Tsayayyen Lokacin Irkutsk"}, {"Israel", "Tsayayyen lokacin Israʼila"}, {"Japan", "Tsayayyen lokacin Japan"}, {"Kazakhstan", "Lokacin Kazakhstan"}, {"Kazakhstan_Eastern", "Lokacin Gabashin Kazakhstan"}, {"Kazakhstan_Western", "Lokacin Yammacin Kazakhstan"}, {"Korea", "Tsayayyen Lokacin Koriya"}, {"Kosrae", "Lokacin Kosrae"}, {"Krasnoyarsk", "Tsayayyen Lokacin Krasnoyarsk"}, {"Kyrgystan", "Lokacin Kyrgyzstan"}, {"Line_Islands", "Lokacin Line Islands"}, {"Lord_Howe", "Tsayayyen Lokacin Lord Howe"}, {"Magadan", "Tsayayyen Lokacin Magadan"}, {"Malaysia", "Lokacin Malaysia"}, {"Maldives", "Lokacin Maldives"}, {"Marquesas", "Lokacin Marquesas"}, {"Marshall_Islands", "Lokacin Marshall Islands"}, {"Mauritius", "Tsayayyen Lokacin Mauritius"}, {"Mawson", "Lokacin Mawson"}, {"Mexico_Pacific", "Tsayayyen Lokacin Mekziko Pacific"}, {"Mongolia", "Tsayayyen Lokacin Ulaanbaatar"}, {"Moscow", "Tsayayyen Lokacin Moscow"}, {"Myanmar", "Lokacin Myanmar"}, {"Nauru", "Lokacin Nauru"}, {"Nepal", "Lokacin Nepal"}, {"New_Caledonia", "Tsayayyen Lokacin New Caledonia"}, {"New_Zealand", "Tsayayyen Lokacin New Zealand"}, {"Newfoundland", "Lokaci Tsayayye ta Newfoundland"}, {"Niue", "Lokacin Niue"}, {"Norfolk", "Tsayayyen Lokacin Norfolk Island"}, {"Noronha", "Tsayayyen Lokacin Fernando de Noronha"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Tsayayyen Lokacin Omsk"}, {"Pakistan", "Tsayayyen Lokacin Pakistan"}, {"Palau", "Lokacin Palau"}, {"Papua_New_Guinea", "Lokacin Papua New Guinea"}, {"Paraguay", "Tsayayyen Lokacin Paraguay"}, {"Peru", "Tsayayyen Lokacin Peru"}, {"Philippines", "Tsayayyen Lokacin Philippine"}, {"Phoenix_Islands", "Lokacin Phoenix Islands"}, {"Pierre_Miquelon", "Tsayayyen Lokacin St. Pierre da Miquelon"}, {"Pitcairn", "Lokacin Pitcairn"}, {"Ponape", "Lokacin Ponape"}, {"Pyongyang", "Lokacin Pyongyang"}, {"Reunion", "Lokacin Réunion"}, {"Rothera", "Lokacin Rothera"}, {"Sakhalin", "Tsayayyen Lokacin Sakhalin"}, {"Samoa", "Tsayayyen Lokacin Samoa"}, {"Seychelles", "Lokacin Seychelles"}, {"Singapore", "Tsayayyen Lokacin Singapore"}, {"Solomon", "Lokacin Rana na Solomon"}, {"South_Georgia", "Lokacin Kudancin Georgia"}, {"Suriname", "Lokacin Suriname"}, {"Syowa", "Lokacin Syowa"}, {"Tahiti", "Lokacin Tahiti"}, {"Taipei", "Tsayayyen Lokacin Taipei"}, {"Tajikistan", "Lokacin Tajikistan"}, {"Tokelau", "Lokacin Tokelau"}, {"Tonga", "Tsayayyen Lokacin Tonga"}, {"Truk", "Lokacin Chuuk"}, {"Turkmenistan", "Tsayayyen Lokacin Turkmenistan"}, {"Tuvalu", "Lokacin Tuvalu"}, {"Uruguay", "Tsayayyen Lokacin Uruguay"}, {"Uzbekistan", "Tsayayyen Lokacin Uzbekistan"}, {"Vanuatu", "Tsayayyen Lokacin Vanuatu"}, {"Venezuela", "Lokacin Venezuela"}, {"Vladivostok", "Tsayayyen Lokacin Vladivostok"}, {"Volgograd", "Tsayayyen Lokacin Volgograd"}, {"Vostok", "Lokacin Vostok"}, {"Wake", "Lokacin Wake Island"}, {"Wallis", "Lokacin Wallis da Futuna"}, {"Yakutsk", "Tsayayyen Lokacin Yakutsk"}, {"Yekaterinburg", "Tsayayyen Lokacin Yekaterinburg"}, {"Yukon", "Lokacin Yukon"}},
		{{"Alaska", "Lokacin Rana na Alaska"}, {"Amazon", "Lokacin Bazara na Amazon"}, {"America_Central", "Lokacin Rana dake Arewacin Amurika ta Tsakiya"}, {"America_Eastern", "Lokacin Rana ta Gabas dake Arewacin Amurika"}, {"America_Mountain", "Lokacin Rana na Tsaunin Arewacin Amurka"}, {"America_Pacific", "Lokacin Rana na Arewacin Amurka"}, {"Apia", "Lokacin Rana na Apia"}, {"Arabian", "Lokacin Rana na Arebiya"}, {"Argentina", "Lokacin Bazara na Argentina"}, {"Argentina_Western", "Lokacin Bazara na Yammacin Argentina"}, {"Armenia", "Lokacin Bazara na Armenia"}, {"Atlantic", "Lokacin Rana na Kanada, Puerto Rico da Virgin Islands"}, {"Australia_Central", "Lokacin Rana na Tsakiyar Austiraliya"}, {"Australia_CentralWestern", "Lokacin Rana na Yammacin Tsakiyar Austiraliya"}, {"Australia_Eastern", "Lokacin Rana na Gabashin Austiraliya"}, {"Australia_Western", "Lokacin Rana na Yammacin Austiralia"}, {"Azerbaijan", "Lokacin Bazara na Azerbaijan"}, {"Azores", "Lokacin Azure na Bazara"}, {"Bangladesh", "Lokacin Bazara na Bangladesh"}, {"Brasilia", "Lokacin Bazara na Brasillia"}, {"Cape_Verde", "Lokacin Bazara na Cape Verde"}, {"Chatham", "Lokacin Rana na Chatham"}, {"Chile", "Lokacin Bazara na Chile"}, {"China", "Lokacin Rana na Sin"}, {"Colombia", "Lokacin Bazara na Colombia"}, {"Cook", "Rabin Lokacin Bazara na Cook Islands"}, {"Cuba", "Lokacin Rana na Kuba"}, {"Easter", "Lokacin Bazara na Easter Island"}, {"Europe_Central", "Tsakiyar bazara a lokaci turai"}, {"Europe_Eastern", "Gabas a lokaci turai da bazara"}, {"Europe_Western", "Ida lokaci ta yammacin turai da bazara"}, {"Falkland", "Lokacin Bazara na Falkland Islands"}, {"Fiji", "Lokacin Bazara na Fiji"}, {"Georgia", "Lokacin Bazara na Georgia"}, {"Greenland_Eastern", "Lokacin Rana na Gabashin Greenland"}, {"Greenland_Western", "Lokacin Rana na Yammacin Greenland"}, {"Hawaii_Aleutian", "Lokacin Rana na Hawaii-Aleutian"}, {"Hong_Kong", "Lokacin Bazara na Hong Kong"}, {"Hovd", "Lokacin Bazara na Hovd"}, {"Iran", "Lokacin Rana na Iran"}, {"Irkutsk", "Lokacin Bazara na Irkutsk"}, {"Israel", "Lokacin Hasken Rana na Israʼila"}, {"Japan", "Lokacin Hasken Rana na Japan"}, {"Korea", "Lokacin Rana na Koriya"}, {"Krasnoyarsk", "Lokacin Bazara na Krasnoyarsk"}, {"Lord_Howe", "Lokacin Rana na Vote Lord Howe"}, {"Magadan", "Lokacin Bazara na Magadan"}, {"Mauritius", "Lokacin Bazara na Mauritius"}, {"Mexico_Pacific", "Lokacin Rana na Mekziko Pacific"}, {"Mongolia", "Lokacin Bazara na Ulaanbaatar"}, {"Moscow", "Lokacin Bazara na Moscow"}, {"New_Caledonia", "Lokacin Bazara na New Caledonia"}, {"New_Zealand", "Lokacin Rana na New Zealand"}, {"Newfoundland", "Lokaci rana ta Newfoundland"}, {"Norfolk", "Lokacin Rana na Norfolk Island"}, {"Noronha", "Lokacin Bazara na Fernando de Noronha"}, {"Novosibirsk", "Lokacin Bazara na Novosibirsk"}, {"Omsk", "Lokacin Bazara na Omsk"}, {"Pakistan", "Lokacin Bazara na Pakistan"}, {"Paraguay", "Lokacin Bazara na Paraguay"}, {"Peru", "Lokacin Bazara na Peru"}, {"Philippines", "Lokacin Bazara na Philippine"}, {"Pierre_Miquelon", "Lokacin Rana na St. Pierre da Miquelon"}, {"Sakhalin", "Lokacin Bazara na Sakhalin"}, {"Samoa", "Lokacin Rana na Vote Samoa"}, {"Taipei", "Lokacin Rana na Taipei"}, {"Tonga", "Lokacin Bazara na Tonga"}, {"Turkmenistan", "Lokacin Bazara na Turkmenistan"}, {"Uruguay", "Lokacin Bazara na Uruguay"}, {"Uzbekistan", "Lokacin Bazara na Uzbekistan"}, {"Vanuatu", "Lokacin Bazara na Vanuatu"}, {"Vladivostok", "Lokacin Bazara na Vladivostok"}, {"Volgograd", "Lokacin Bazara na Volgograd"}, {"Yakutsk", "Lokacin Bazara na Yakutsk"}, {"Yekaterinburg", "Lokacin Bazara na Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "שעון אפגניסטן"}, {"Africa_Central", "שעון מרכז אפריקה"}, {"Africa_Eastern", "שעון מזרח אפריקה"}, {"Africa_Southern", "שעון דרום אפריקה"}, {"Africa_Western", "שעון מערב אפריקה"}, {"Alaska", "שעון אלסקה (חורף)"}, {"Almaty", "שעון אלמטי (חורף)"}, {"Amazon", "שעון אמזונס (חורף)"}, {"America_Central", "שעון מרכז ארה״ב (חורף)"}, {"America_Eastern", "שעון החוף המזרחי (חורף)"}, {"America_Mountain", "שעון אזור ההרים בארה״ב (חורף)"}, {"America_Pacific", "שעון מערב ארה״ב (חורף)"}, {"Anadyr", "שעון רגיל אנדיר"}, {"Apia", "שעון אפיה (חורף)"}, {"Aqtau", "שעון אקטאו (חורף)"}, {"Aqtobe", "שעון אוקטובה (חורף)"}, {"Arabian", "שעון חצי האי ערב (חורף)"}, {"Argentina", "שעון ארגנטינה (חורף)"}, {"Argentina_Western", "שעון מערב ארגנטינה (חורף)"}, {"Armenia", "שעון ארמניה (חורף)"}, {"Atlantic", "שעון האוקיינוס האטלנטי (חורף)"}, {"Australia_Central", "שעון מרכז אוסטרליה (חורף)"}, {"Australia_CentralWestern", "שעון מרכז-מערב אוסטרליה (חורף)"}, {"Australia_Eastern", "שעון מזרח אוסטרליה (חורף)"}, {"Australia_Western", "שעון מערב אוסטרליה (חורף)"}, {"Azerbaijan", "שעון אזרבייג׳ן (חורף)"}, {"Azores", "שעון האיים האזוריים (חורף)"}, {"Bangladesh", "שעון בנגלדש (חורף)"}, {"Bhutan", "שעון בהוטן"}, {"Bolivia", "שעון בוליביה"}, {"Brasilia", "שעון ברזיליה (חורף)"}, {"Brunei", "שעון ברוניי דארוסלאם"}, {"Cape_Verde", "שעון כף ורדה (חורף)"}, {"Chamorro", "שעון צ׳אמורו"}, {"Chatham", "שעון צ׳טהאם (חורף)"}, {"Chile", "שעון צ׳ילה (חורף)"}, {"China", "שעון סין (חורף)"}, {"Christmas", "שעון האי כריסטמס"}, {"Cocos", "שעון איי קוקוס"}, {"Colombia", "שעון קולומביה (חורף)"}, {"Cook", "שעון איי קוק (חורף)"}, {"Cuba", "שעון קובה (חורף)"}, {"Davis", "שעון דיוויס"}, {"DumontDUrville", "שעון דומון ד׳אורוויל"}, {"East_Timor", "שעון מזרח טימור"}, {"Easter", "שעון אי הפסחא (חורף)"}, {"Ecuador", "שעון אקוודור"}, {"Europe_Central", "שעון מרכז אירופה (חורף)"}, {"Europe_Eastern", "שעון מזרח אירופה (חורף)"}, {"Europe_Further_Eastern", "שעון מינסק"}, {"Europe_Western", "שעון מערב אירופה (חורף)"}, {"Falkland", "שעון איי פוקלנד (חורף)"}, {"Fiji", "שעון פיג׳י (חורף)"}, {"French_Guiana", "שעון גיאנה הצרפתית"}, {"French_Southern", "שעון הארצות הדרומיות והאנטארקטיות של צרפת"}, {"GMT", "שעון גריניץ׳\u200f"}, {"Galapagos", "שעון איי גלאפגוס"}, {"Gambier", "שעון איי גמבייה"}, {"Georgia", "שעון גאורגיה (חורף)"}, {"Gilbert_Islands", "שעון איי גילברט"}, {"Greenland_Eastern", "שעון מזרח גרינלנד (חורף)"}, {"Greenland_Western", "שעון מערב גרינלנד (חורף)"}, {"Guam", "שעון גואם"}, {"Gulf", "שעון מדינות המפרץ"}, {"Guyana", "שעון גיאנה"}, {"Hawaii", "שעון האיים האלאוטיים הוואי (חורף)"}, {"Hawaii_Aleutian", "שעון האיים האלאוטיים הוואי (חורף)"}, {"Hong_Kong", "שעון הונג קונג (חורף)"}, {"Hovd", "שעון חובד (חורף)"}, {"India", "שעון הודו"}, {"Indian_Ocean", "שעון האוקיינוס ההודי"}, {"Indochina", "שעון הודו-סין"}, {"Indonesia_Central", "שעון מרכז אינדונזיה"}, {"Indonesia_Eastern", "שעון מזרח אינדונזיה"}, {"Indonesia_Western", "שעון מערב אינדונזיה"}, {"Iran", "שעון איראן (חורף)"}, {"Irkutsk", "שעון אירקוטסק (חורף)"}, {"Israel", "שעון ישראל (חורף)"}, {"Japan", "שעון יפן (חורף)"}, {"Kamchatka", "שעון רגיל פטרופבלובסק-קמצ׳טסקי"}, {"Kazakhstan", "שעון קזחסטן"}, {"Kazakhstan_Eastern", "שעון מזרח קזחסטן"}, {"Kazakhstan_Western", "שעון מערב קזחסטן"}, {"Korea", "שעון קוריאה (חורף)"}, {"Kosrae", "שעון קוסראה"}, {"Krasnoyarsk", "שעון קרסנויארסק (חורף)"}, {"Kyrgystan", "שעון קירגיזסטן"}, {"Line_Islands", "שעון איי ליין"}, {"Lord_Howe", "שעון אי הלורד האו (חורף)"}, {"Macau", "שעון חורף מקאו"}, {"Magadan", "שעון מגדן (חורף)"}, {"Malaysia", "שעון מלזיה"}, {"Maldives", "שעון האיים המלדיביים"}, {"Marquesas", "שעון איי מרקיז"}, {"Marshall_Islands", "שעון איי מרשל"}, {"Mauritius", "שעון מאוריציוס (חורף)"}, {"Mawson", "שעון מאוסון"}, {"Mexico_Pacific", "שעון מערב מקסיקו (חורף)"}, {"Mongolia", "שעון אולאן באטור (חורף)"}, {"Moscow", "שעון מוסקבה (חורף)"}, {"Myanmar", "שעון מיאנמר"}, {"Nauru", "שעון נאורו"}, {"Nepal", "שעון נפאל"}, {"New_Caledonia", "שעון קלדוניה החדשה (חורף)"}, {"New_Zealand", "שעון ניו זילנד (חורף)"}, {"Newfoundland", "שעון ניופאונדלנד (חורף)"}, {"Niue", "שעון ניואה"}, {"Norfolk", "שעון האי נורפוק (חורף)"}, {"Noronha", "שעון פרננדו די נורוניה (חורף)"}, {"North_Mariana", "שעון איי מריאנה הצפוניים"}, {"Novosibirsk", "שעון נובוסיבירסק (חורף)"}, {"Omsk", "שעון אומסק (חורף)"}, {"Pakistan", "שעון פקיסטן (חורף)"}, {"Palau", "שעון פלאו"}, {"Papua_New_Guinea", "שעון פפואה גיניאה החדשה"}, {"Paraguay", "שעון פרגוואי (חורף)"}, {"Peru", "שעון פרו (חורף)"}, {"Philippines", "שעון הפיליפינים (חורף)"}, {"Phoenix_Islands", "שעון איי פיניקס"}, {"Pierre_Miquelon", "שעון סנט פייר ומיקלון (חורף)"}, {"Pitcairn", "שעון פיטקרן"}, {"Ponape", "שעון פונאפי"}, {"Pyongyang", "שעון פיונגיאנג"}, {"Qyzylorda", "שעון קיזילורדה (חורף)"}, {"Reunion", "שעון ראוניון"}, {"Rothera", "שעון רות׳רה"}, {"Sakhalin", "שעון סחלין (חורף)"}, {"Samara", "שעון רגיל סמרה"}, {"Samoa", "שעון סמואה (חורף)"}, {"Seychelles", "שעון איי סיישל"}, {"Singapore", "שעון סינגפור"}, {"Solomon", "שעון איי שלמה"}, {"South_Georgia", "שעון דרום ג׳ורג׳יה"}, {"Suriname", "שעון סורינאם"}, {"Syowa", "שעון סייווה"}, {"Tahiti", "שעון טהיטי"}, {"Taipei", "שעון טאיפיי (חורף)"}, {"Tajikistan", "שעון טג׳יקיסטן"}, {"Tokelau", "שעון טוקלאו"}, {"Tonga", "שעון טונגה (חורף)"}, {"Truk", "שעון צ׳וק"}, {"Turkey", "שעון רגיל טורקיה"}, {"Turkmenistan", "שעון טורקמניסטן (חורף)"}, {"Tuvalu", "שעון טובאלו"}, {"Uruguay", "שעון אורוגוואי (חורף)"}, {"Uzbekistan", "שעון אוזבקיסטן (חורף)"}, {"Vanuatu", "שעון ונואטו (חורף)"}, {"Venezuela", "שעון ונצואלה"}, {"Vladivostok", "שעון ולדיווסטוק (חורף)"}, {"Volgograd", "שעון וולגוגרד (חורף)"}, {"Vostok", "שעון ווסטוק"}, {"Wake", "שעון האי וייק"}, {"Wallis", "שעון וואליס ופוטונה"}, {"Yakutsk", "שעון יקוטסק (חורף)"}, {"Yekaterinburg", "שעון יקטרינבורג (חורף)"}, {"Yukon", "שעון יוקון"}},
		{{"Alaska", "שעון אלסקה (קיץ)"}, {"Almaty", "שעון אלמטי (קיץ)"}, {"Amazon", "שעון אמזונס (קיץ)"}, {"America_Central", "שעון מרכז ארה״ב (קיץ)"}, {"America_Eastern", "שעון החוף המזרחי (קיץ)"}, {"America_Mountain", "שעון אזור ההרים בארה״ב (קיץ)"}, {"America_Pacific", "שעון מערב ארה״ב (קיץ)"}, {"Anadyr", "שעון קיץ אנדיר"}, {"Apia", "שעון אפיה (קיץ)"}, {"Aqtau", "שעון אקטאו (קיץ)"}, {"Aqtobe", "שעון אוקטובה (קיץ)"}, {"Arabian", "שעון חצי האי ערב (קיץ)"}, {"Argentina", "שעון ארגנטינה (קיץ)"}, {"Argentina_Western", "שעון מערב ארגנטינה (קיץ)"}, {"Armenia", "שעון ארמניה (קיץ)"}, {"Atlantic", "שעון האוקיינוס האטלנטי (קיץ)"}, {"Australia_Central", "שעון מרכז אוסטרליה (קיץ)"}, {"Australia_CentralWestern", "שעון מרכז-מערב אוסטרליה (קיץ)"}, {"Australia_Eastern", "שעון מזרח אוסטרליה (קיץ)"}, {"Australia_Western", "שעון מערב אוסטרליה (קיץ)"}, {"Azerbaijan", "שעון אזרבייג׳ן (קיץ)"}, {"Azores", "שעון האיים האזוריים (קיץ)"}, {"Bangladesh", "שעון בנגלדש (קיץ)"}, {"Brasilia", "שעון ברזיליה (קיץ)"}, {"Cape_Verde", "שעון כף ורדה (קיץ)"}, {"Chatham", "שעון צ׳טהאם (קיץ)"}, {"Chile", "שעון צ׳ילה (קיץ)"}, {"China", "שעון סין (קיץ)"}, {"Colombia", "שעון קולומביה (קיץ)"}, {"Cook", "שעון איי קוק (מחצית הקיץ)"}, {"Cuba", "שעון קובה (קיץ)"}, {"Easter", "שעון אי הפסחא (קיץ)"}, {"Europe_Central", "שעון מרכז אירופה (קיץ)"}, {"Europe_Eastern", "שעון מזרח אירופה (קיץ)"}, {"Europe_Western", "שעון מערב אירופה (קיץ)"}, {"Falkland", "שעון איי פוקלנד (קיץ)"}, {"Fiji", "שעון פיג׳י (קיץ)"}, {"Georgia", "שעון גאורגיה (קיץ)"}, {"Greenland_Eastern", "שעון מזרח גרינלנד (קיץ)"}, {"Greenland_Western", "שעון מערב גרינלנד (קיץ)"}, {"Hawaii_Aleutian", "שעון האיים האלאוטיים הוואי (קיץ)"}, {"Hong_Kong", "שעון הונג קונג (קיץ)"}, {"Hovd", "שעון חובד (קיץ)"}, {"Iran", "שעון איראן (קיץ)"}, {"Irkutsk", "שעון אירקוסטק (קיץ)"}, {"Israel", "שעון ישראל (קיץ)"}, {"Japan", "שעון יפן (קיץ)"}, {"Kamchatka", "שעון קיץ פטרופבלובסק-קמצ׳טסקי"}, {"Korea", "שעון קוריאה (קיץ)"}, {"Krasnoyarsk", "שעון קרסנויארסק (קיץ)"}, {"Lord_Howe", "שעון אי הלורד האו (קיץ)"}, {"Macau", "שעון קיץ מקאו"}, {"Magadan", "שעון מגדן (קיץ)"}, {"Mauritius", "שעון מאוריציוס (קיץ)"}, {"Mexico_Pacific", "שעון מערב מקסיקו (קיץ)"}, {"Mongolia", "שעון אולאן באטור (קיץ)"}, {"Moscow", "שעון מוסקבה (קיץ)"}, {"New_Caledonia", "שעון קלדוניה החדשה (קיץ)"}, {"New_Zealand", "שעון ניו זילנד (קיץ)"}, {"Newfoundland", "שעון ניופאונדלנד (קיץ)"}, {"Norfolk", "שעון האי נורפוק (קיץ)"}, {"Noronha", "שעון פרננדו די נורוניה (קיץ)"}, {"Novosibirsk", "שעון נובוסיבירסק (קיץ)"}, {"Omsk", "שעון אומסק (קיץ)"}, {"Pakistan", "שעון פקיסטן (קיץ)"}, {"Paraguay", "שעון פרגוואי (קיץ)"}, {"Peru", "שעון פרו (קיץ)"}, {"Philippines", "שעון הפיליפינים (קיץ)"}, {"Pierre_Miquelon", "שעון סנט פייר ומיקלון (קיץ)"}, {"Qyzylorda", "שעון קיזילורדה (קיץ)"}, {"Sakhalin", "שעון סחלין (קיץ)"}, {"Samara", "שעון קיץ סמרה"}, {"Samoa", "שעון סמואה (קיץ)"}, {"Taipei", "שעון טאיפיי (קיץ)"}, {"Tonga", "שעון טונגה (קיץ)"}, {"Turkey", "שעון קיץ טורקיה"}, {"Turkmenistan", "שעון טורקמניסטן (קיץ)"}, {"Uruguay", "שעון אורוגוואי (קיץ)"}, {"Uzbekistan", "שעון אוזבקיסטן (קיץ)"}, {"Vanuatu", "שעון ונואטו (קיץ)"}, {"Vladivostok", "שעון ולדיווסטוק (קיץ)"}, {"Volgograd", "שעון וולגוגרד (קיץ)"}, {"Yakutsk", "שעון יקוטסק (קיץ)"}, {"Yekaterinburg", "שעון יקטרינבורג (קיץ)"}},
	},
	{
		{{"Afghanistan", "अफ़गानिस्तान समय"}, {"Africa_Central", "मध्य अफ़्रीका समय"}, {"Africa_Eastern", "पूर्वी अफ़्रीका समय"}, {"Africa_Southern", "दक्षिण अफ़्रीका मानक समय"}, {"Africa_Western", "पश्चिम अफ़्रीका समय"}, {"Alaska", "अलास्\u200dका मानक समय"}, {"Amazon", "अमेज़न मानक समय"}, {"America_Central", "उत्तरी अमेरिकी केंद्रीय मानक समय"}, {"America_Eastern", "उत्तरी अमेरिकी पूर्वी मानक समय"}, {"America_Mountain", "उत्तरी अमेरिकी माउंटेन मानक समय"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांत मानक समय"}, {"Anadyr", "एनाडीयर मानक समय"}, {"Apia", "एपिआ मानक समय"}, {"Arabian", "अरब मानक समय"}, {"Argentina", "अर्जेंटीना मानक समय"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना मानक समय"}, {"Armenia", "आर्मेनिया मानक समय"}, {"Atlantic", "अटलांटिक मानक समय"}, {"Australia_Central", "ऑस्\u200dट्रेलियाई केंद्रीय मानक समय"}, {"Australia_CentralWestern", "ऑस्\u200dट्रेलियाई केंद्रीय पश्चिमी मानक समय"}, {"Australia_Eastern", "ऑस्\u200dट्रेलियाई पूर्वी मानक समय"}, {"Australia_Western", "ऑस्ट्रेलियाई पश्चिमी मानक समय"}, {"Azerbaijan", "अज़रबैजान मानक समय"}, {"Azores", "अज़ोरेस मानक समय"}, {"Bangladesh", "बांग्लादेश मानक समय"}, {"Bhutan", "भूटान समय"}, {"Bolivia", "बोलीविया समय"}, {"Brasilia", "ब्राज़ीलिया मानक समय"}, {"Brunei", "ब्रूनेई दारूस्सलम समय"}, {"Cape_Verde", "केप वर्ड मानक समय"}, {"Chamorro", "चामोरो मानक समय"}, {"Chatham", "चैथम मानक समय"}, {"Chile", "चिली मानक समय"}, {"China", "चीन मानक समय"}, {"Christmas", "क्रिसमस द्वीप समय"}, {"Cocos", "कोकोस द्वीपसमूह समय"}, {"Colombia", "कोलंबिया मानक समय"}, {"Cook", "कुक द्वीपसमूह मानक समय"}, {"Cuba", "क्यूबा मानक समय"}, {"Davis", "डेविस समय"}, {"DumontDUrville", "ड्यूमोंट डी अर्विले समय"}, {"East_Timor", "पूर्वी तिमोर समय"}, {"Easter", "ईस्टर द्वीप मानक समय"}, {"Ecuador", "इक्वाडोर समय"}, {"Europe_Central", "मध्य यूरोपीय मानक समय"}, {"Europe_Eastern", "पूर्वी यूरोपीय मानक समय"}, {"Europe_Further_Eastern", "अग्र पूर्वी यूरोपीय समय"}, {"Europe_Western", "पश्चिमी यूरोपीय मानक समय"}, {"Falkland", "फ़ॉकलैंड द्वीपसमूह मानक समय"}, {"Fiji", "फ़िजी मानक समय"}, {"French_Guiana", "फ़्रेंच गुयाना समय"}, {"French_Southern", "दक्षिणी फ़्रांस और अंटार्कटिक समय"}, {"GMT", "ग्रीनविच मीन टाइम"}, {"Galapagos", "गैलापेगोस का समय"}, {"Gambier", "गैंबियर समय"}, {"Georgia", "जॉर्जिया मानक समय"}, {"Gilbert_Islands", "गिल्बर्ट द्वीपसमूह समय"}, {"Greenland_Eastern", "पूर्वी ग्रीनलैंड मानक समय"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड मानक समय"}, {"Gulf", "खाड़ी मानक समय"}, {"Guyana", "गुयाना समय"}, {"Hawaii", "हवाई–आल्यूशन मानक समय"}, {"Hawaii_Aleutian", "हवाई–आल्यूशन मानक समय"}, {"Hong_Kong", "हाँग काँग मानक समय"}, {"Hovd", "होव्ड मानक समय"}, {"India", "भारतीय मानक समय"}, {"Indian_Ocean", "हिंद महासागर समय"}, {"Indochina", "इंडोचाइना समय"}, {"Indonesia_Central", "मध्य इंडोनेशिया समय"}, {"Indonesia_Eastern", "पूर्वी इंडोनेशिया समय"}, {"Indonesia_Western", "पश्चिमी इंडोनेशिया समय"}, {"Iran", "ईरान मानक समय"}, {"Irkutsk", "इर्कुत्स्क मानक समय"}, {"Israel", "इज़राइल मानक समय"}, {"Japan", "जापान मानक समय"}, {"Kamchatka", "पेट्रोपेवलास्क-कैमचात्सकी मानक समय"}, {"Kazakhstan", "कज़ाखस्तान समय"}, {"Kazakhstan_Eastern", "पूर्व कज़ाखस्तान समय"}, {"Kazakhstan_Western", "पश्चिम कज़ाखस्तान समय"}, {"Korea", "कोरियाई मानक समय"}, {"Kosrae", "कोसराए समय"}, {"Krasnoyarsk", "क्रास्नोयार्स्क मानक समय"}, {"Kyrgystan", "किर्गिस्\u200dतान समय"}, {"Line_Islands", "लाइन द्वीपसमूह समय"}, {"Lord_Howe", "लॉर्ड होवे मानक समय"}, {"Magadan", "मागादान मानक समय"}, {"Malaysia", "मलेशिया समय"}, {"Maldives", "मालदीव समय"}, {"Marquesas", "मार्केसस समय"}, {"Marshall_Islands", "मार्शल द्वीपसमूह समय"}, {"Mauritius", "मॉरीशस मानक समय"}, {"Mawson", "माव्सन समय"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत मानक समय"}, {"Mongolia", "उलान बटोर मानक समय"}, {"Moscow", "मॉस्को मानक समय"}, {"Myanmar", "म्यांमार समय"}, {"Nauru", "नौरू समय"}, {"Nepal", "नेपाल समय"}, {"New_Caledonia", "न्यू कैलेडोनिया मानक समय"}, {"New_Zealand", "न्यूज़ीलैंड मानक समय"}, {"Newfoundland", "न्यूफ़ाउंडलैंड मानक समय"}, {"Niue", "नीयू समय"}, {"Norfolk", "नॉरफ़ॉक द्वीप मानक समय"}, {"Noronha", "फ़र्नांर्डो डे नोरोन्हा मानक समय"}, {"Novosibirsk", "नोवोसिबिर्स्क मानक समय"}, {"Omsk", "ओम्स्क मानक समय"}, {"Pakistan", "पाकिस्तान मानक समय"}, {"Palau", "पलाउ समय"}, {"Papua_New_Guinea", "पापुआ न्यू गिनी समय"}, {"Paraguay", "पैराग्वे मानक समय"}, {"Peru", "पेरू मानक समय"}, {"Philippines", "फ़िलिपीन मानक समय"}, {"Phoenix_Islands", "फ़ीनिक्स द्वीपसमूह समय"}, {"Pierre_Miquelon", "सेंट पिएरे और मिक्वेलान मानक समय"}, {"Pitcairn", "पिटकैर्न समय"}, {"Ponape", "पोनापे समय"}, {"Pyongyang", "प्योंगयांग समय"}, {"Reunion", "रीयूनियन समय"}, {"Rothera", "रोथेरा समय"}, {"Sakhalin", "सखालिन मानक समय"}, {"Samara", "समारा मानक समय"}, {"Samoa", "समोआ मानक समय"}, {"Seychelles", "सेशेल्स समय"}, {"Singapore", "सिंगापुर समय"}, {"Solomon", "सोलोमन द्वीपसमूह समय"}, {"South_Georgia", "दक्षिणी जॉर्जिया समय"}, {"Suriname", "सूरीनाम समय"}, {"Syowa", "स्योवा समय"}, {"Tahiti", "ताहिती समय"}, {"Taipei", "ताइपे मानक समय"}, {"Tajikistan", "ताजिकिस्तान समय"}, {"Tokelau", "टोकेलाऊ समय"}, {"Tonga", "टोंगा मानक समय"}, {"Truk", "चुक समय"}, {"Turkmenistan", "तुर्कमेनिस्तान मानक समय"}, {"Tuvalu", "तुवालू समय"}, {"Uruguay", "उरुग्वे मानक समय"}, {"Uzbekistan", "उज़्बेकिस्तान मानक समय"}, {"Vanuatu", "वनुआतू मानक समय"}, {"Venezuela", "वेनेज़ुएला समय"}, {"Vladivostok", "व्लादिवोस्तोक मानक समय"}, {"Volgograd", "वोल्गोग्राड मानक समय"}, {"Vostok", "वोस्तोक समय"}, {"Wake", "वेक द्वीप समय"}, {"Wallis", "वालिस और फ़्यूचूना समय"}, {"Yakutsk", "याकुत्स्क मानक समय"}, {"Yekaterinburg", "येकातेरिनबर्ग मानक समय"}, {"Yukon", "युकॉन समय"}},
		{{"Alaska", "अलास्\u200dका डेलाइट समय"}, {"Amazon", "अमेज़न ग्रीष्मकालीन समय"}, {"America_Central", "उत्तरी अमेरिकी केंद्रीय डेलाइट समय"}, {"America_Eastern", "उत्तरी अमेरिकी पूर्वी डेलाइट समय"}, {"America_Mountain", "उत्तरी अमेरिकी माउंटेन डेलाइट समय"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांत डेलाइट समय"}, {"Anadyr", "एनाडीयर ग्रीष्मकालीन समय"}, {"Apia", "एपिआ डेलाइट समय"}, {"Arabian", "अरब डेलाइट समय"}, {"Argentina", "अर्जेंटीना ग्रीष्मकालीन समय"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना ग्रीष्मकालीन समय"}, {"Armenia", "आर्मेनिया ग्रीष्मकालीन समय"}, {"Atlantic", "अटलांटिक डेलाइट समय"}, {"Australia_Central", "ऑस्\u200dट्रेलियाई केंद्रीय डेलाइट समय"}, {"Australia_CentralWestern", "ऑस्\u200dट्रेलियाई केंद्रीय पश्चिमी डेलाइट समय"}, {"Australia_Eastern", "ऑस्\u200dट्रेलियाई पूर्वी डेलाइट समय"}, {"Australia_Western", "ऑस्ट्रेलियाई पश्चिमी डेलाइट समय"}, {"Azerbaijan", "अज़रबैजान ग्रीष्मकालीन समय"}, {"Azores", "अज़ोरेस ग्रीष्मकालीन समय"}, {"Bangladesh", "बांग्लादेश ग्रीष्मकालीन समय"}, {"Brasilia", "ब्राज़ीलिया ग्रीष्मकालीन समय"}, {"Cape_Verde", "केप वर्ड ग्रीष्मकालीन समय"}, {"Chatham", "चैथम डेलाइट समय"}, {"Chile", "चिली ग्रीष्मकालीन समय"}, {"China", "चीन डेलाइट समय"}, {"Colombia", "कोलंबिया ग्रीष्मकालीन समय"}, {"Cook", "कुक द्वीपसमूह अर्द्ध ग्रीष्मकालीन समय"}, {"Cuba", "क्यूबा डेलाइट समय"}, {"Easter", "ईस्टर द्वीप ग्रीष्मकालीन समय"}, {"Europe_Central", "मध्\u200dय यूरोपीय ग्रीष्\u200dमकालीन समय"}, {"Europe_Eastern", "पूर्वी यूरोपीय ग्रीष्मकालीन समय"}, {"Europe_Western", "पश्चिमी यूरोपीय ग्रीष्\u200dमकालीन समय"}, {"Falkland", "फ़ॉकलैंड द्वीपसमूह ग्रीष्मकालीन समय"}, {"Fiji", "फ़िजी ग्रीष्मकालीन समय"}, {"Georgia", "जॉर्जिया ग्रीष्मकालीन समय"}, {"Greenland_Eastern", "पूर्वी ग्रीनलैंड ग्रीष्मकालीन समय"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड ग्रीष्मकालीन समय"}, {"Hawaii_Aleutian", "हवाई–आल्यूशन डेलाइट समय"}, {"Hong_Kong", "हाँग काँग ग्रीष्मकालीन समय"}, {"Hovd", "होव्ड ग्रीष्मकालीन समय"}, {"Iran", "ईरान डेलाइट समय"}, {"Irkutsk", "इर्कुत्स्क ग्रीष्मकालीन समय"}, {"Israel", "इज़राइल डेलाइट समय"}, {"Japan", "जापान डेलाइट समय"}, {"Kamchatka", "पेट्रोपेवलास्क-कैमचात्सकी ग्रीष्मकालीन समय"}, {"Korea", "कोरियाई डेलाइट समय"}, {"Krasnoyarsk", "क्रास्नोयार्स्क ग्रीष्मकालीन समय"}, {"Lord_Howe", "लॉर्ड होवे डेलाइट समय"}, {"Magadan", "मागादान ग्रीष्मकालीन समय"}, {"Mauritius", "मॉरीशस ग्रीष्मकालीन समय"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत डेलाइट समय"}, {"Mongolia", "उलान बटोर ग्रीष्मकालीन समय"}, {"Moscow", "मॉस्को ग्रीष्मकालीन समय"}, {"New_Caledonia", "न्यू कैलेडोनिया ग्रीष्मकालीन समय"}, {"New_Zealand", "न्यूज़ीलैंड डेलाइट समय"}, {"Newfoundland", "न्यूफ़ाउंडलैंड डेलाइट समय"}, {"Norfolk", "नॉरफ़ॉक द्वीप डेलाइट समय"}, {"Noronha", "फ़र्नांर्डो डे नोरोन्हा ग्रीष्मकालीन समय"}, {"Novosibirsk", "नोवोसिबिर्स्क ग्रीष्मकालीन समय"}, {"Omsk", "ओम्स्क ग्रीष्मकालीन समय"}, {"Pakistan", "पाकिस्तान ग्रीष्मकालीन समय"}, {"Paraguay", "पैराग्वे ग्रीष्मकालीन समय"}, {"Peru", "पेरू ग्रीष्मकालीन समय"}, {"Philippines", "फ़िलिपीन ग्रीष्मकालीन समय"}, {"Pierre_Miquelon", "सेंट पिएरे और मिक्वेलान डेलाइट समय"}, {"Sakhalin", "सखालिन ग्रीष्मकालीन समय"}, {"Samara", "समारा ग्रीष्मकालीन समय"}, {"Samoa", "समोआ डेलाइट समय"}, {"Taipei", "ताइपे डेलाइट समय"}, {"Tonga", "टोंगा ग्रीष्मकालीन समय"}, {"Turkmenistan", "तुर्कमेनिस्तान ग्रीष्मकालीन समय"}, {"Uruguay", "उरुग्वे ग्रीष्मकालीन समय"}, {"Uzbekistan", "उज़्बेकिस्तान ग्रीष्मकालीन समय"}, {"Vanuatu", "वनुआतू ग्रीष्मकालीन समय"}, {"Vladivostok", "व्लादिवोस्तोक ग्रीष्मकालीन समय"}, {"Volgograd", "वोल्गोग्राड ग्रीष्मकालीन समय"}, {"Yakutsk", "याकुत्स्क ग्रीष्मकालीन समय"}, {"Yekaterinburg", "येकातेरिनबर्ग ग्रीष्मकालीन समय"}},
	},
	{
		{{"Acre", "Acre Standard Time"}, {"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Almaty", "Almaty Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "North America Central Standard Time"}, {"America_Eastern", "North America Eastern Standard Time"}, {"America_Mountain", "North America Mountain Standard Time"}, {"America_Pacific", "North America Pacific Standard Time"}, {"Anadyr", "Anadyr Standard Time"}, {"Apia", "Samoa Standard Time"}, {"Aqtau", "Aqtau Standard Time"}, {"Aqtobe", "Aqtobe Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Casey", "Casey Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont d’Urville Time"}, {"East_Timor", "Timor-Leste Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern & Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland", "Greenland Standard Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Guam", "Guam Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Khovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kamchatka", "Kamchatka Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Lanka", "Lanka Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Macau", "Macao Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"North_Mariana", "Northern Mariana Islands Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Pohnpei Time"}, {"Pyongyang", "North Korea Time"}, {"Qyzylorda", "Kyzylorda Standard Time"}, {"Reunion", "Reunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samara", "Samara Standard Time"}, {"Samoa", "American Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taiwan Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkey", "Türkiye Standard Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Acre", "Acre Summer Time"}, {"Alaska", "Alaska Daylight Time"}, {"Almaty", "Almaty Summer Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "North America Central Daylight Time"}, {"America_Eastern", "North America Eastern Daylight Time"}, {"America_Mountain", "North America Mountain Daylight Time"}, {"America_Pacific", "North America Pacific Daylight Time"}, {"Anadyr", "Anadyr Summer Time"}, {"Apia", "Samoa Daylight Time"}, {"Aqtau", "Aqtau Summer Time"}, {"Aqtobe", "Aqtobe Summer Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland", "Greenland Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Khovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Kamchatka", "Kamchatka Summer Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Macau", "Macao Summer Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Daylight Time"}, {"Qyzylorda", "Kyzylorda Summer Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samara", "Samara Summer Time"}, {"Samoa", "American Samoa Daylight Time"}, {"Taipei", "Taiwan Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkey", "Türkiye Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Acre", "Acre standardno vrijeme"}, {"Afghanistan", "afganistansko vrijeme"}, {"Africa_Central", "srednjoafričko vrijeme"}, {"Africa_Eastern", "istočnoafričko vrijeme"}, {"Africa_Southern", "južnoafričko vrijeme"}, {"Africa_Western", "zapadnoafričko vrijeme"}, {"Alaska", "aljaško standardno vrijeme"}, {"Almaty", "altmajsko standardno vrijeme"}, {"Amazon", "amazonsko standardno vrijeme"}, {"America_Central", "središnje standardno vrijeme"}, {"America_Eastern", "istočno standardno vrijeme"}, {"America_Mountain", "planinsko standardno vrijeme"}, {"America_Pacific", "pacifičko standardno vrijeme"}, {"Anadyr", "anadirsko standardno vrijeme"}, {"Apia", "standardno vrijeme Apije"}, {"Aqtau", "standardno vrijeme grada Aktau"}, {"Aqtobe", "standardno vrijeme grada Aktobe"}, {"Arabian", "arapsko standardno vrijeme"}, {"Argentina", "argentinsko standardno vrijeme"}, {"Argentina_Western", "zapadnoargentinsko standardno vrijeme"}, {"Armenia", "armensko standardno vrijeme"}, {"Atlantic", "atlantsko standardno vrijeme"}, {"Australia_Central", "srednjoaustralsko standardno vrijeme"}, {"Australia_CentralWestern", "australsko središnje zapadno standardno vrijeme"}, {"Australia_Eastern", "istočnoaustralsko standardno vrijeme"}, {"Australia_Western", "zapadnoaustralsko standardno vrijeme"}, {"Azerbaijan", "azerbajdžansko standardno vrijeme"}, {"Azores", "azorsko standardno vrijeme"}, {"Bangladesh", "bangladeško standardno vrijeme"}, {"Bhutan", "butansko vrijeme"}, {"Bolivia", "bolivijsko vrijeme"}, {"Brasilia", "brazilsko standardno vrijeme"}, {"Brunei", "vrijeme za Brunej Darussalam"}, {"Cape_Verde", "standardno vrijeme Zelenortskog otočja"}, {"Casey", "vrijeme Caseyja"}, {"Chamorro", "standardno vrijeme Chamorra"}, {"Chatham", "standardno vrijeme Chathama"}, {"Chile", "čileansko standardno vrijeme"}, {"China", "kinesko standardno vrijeme"}, {"Christmas", "vrijeme Božićnog Otoka"}, {"Cocos", "vrijeme Kokosovih Otoka"}, {"Colombia", "kolumbijsko standardno vrijeme"}, {"Cook", "standardno vrijeme Cookovih Otoka"}, {"Cuba", "kubansko standardno vrijeme"}, {"Davis", "vrijeme Davisa"}, {"DumontDUrville", "vrijeme Dumont-d’Urvillea"}, {"East_Timor", "istočnotimorsko vrijeme"}, {"Easter", "standardno vrijeme Uskršnjeg otoka"}, {"Ecuador", "ekvadorsko vrijeme"}, {"Europe_Central", "srednjoeuropsko standardno vrijeme"}, {"Europe_Eastern", "istočnoeuropsko standardno vrijeme"}, {"Europe_Further_Eastern", "dalekoistočno europsko vrijeme"}, {"Europe_Western", "zapadnoeuropsko standardno vrijeme"}, {"Falkland", "falklandsko standardno vrijeme"}, {"Fiji", "standardno vrijeme Fidžija"}, {"French_Guiana", "vrijeme Francuske Gijane"}, {"French_Southern", "vrijeme Francuskih Južnih Teritorija"}, {"GMT", "univerzalno vrijeme"}, {"Galapagos", "vrijeme Galapagosa"}, {"Gambier", "vrijeme Gambiera"}, {"Georgia", "gruzijsko standardno vrijeme"}, {"Gilbert_Islands", "vrijeme Gilbertovih otoka"}, {"Greenland_Eastern", "istočnogrenlandsko standardno vrijeme"}, {"Greenland_Western", "zapadnogrenlandsko standardno vrijeme"}, {"Guam", "guamsko standardno vrijeme"}, {"Gulf", "zaljevsko standardno vrijeme"}, {"Guyana", "gvajansko vrijeme"}, {"Hawaii", "havajsko-aleutsko standardno vrijeme"}, {"Hawaii_Aleutian", "havajsko-aleutsko standardno vrijeme"}, {"Hong_Kong", "hongkonško standardno vrijeme"}, {"Hovd", "hovdsko standardno vrijeme"}, {"India", "indijsko vrijeme"}, {"Indian_Ocean", "vrijeme Indijskog oceana"}, {"Indochina", "indokinesko vrijeme"}, {"Indonesia_Central", "srednjoindonezijsko vrijeme"}, {"Indonesia_Eastern", "istočnoindonezijsko vrijeme"}, {"Indonesia_Western", "zapadnoindonezijsko vrijeme"}, {"Iran", "iransko standardno vrijeme"}, {"Irkutsk", "irkutsko standardno vrijeme"}, {"Israel", "izraelsko standardno vrijeme"}, {"Japan", "japansko standardno vrijeme"}, {"Kamchatka", "Petropavlovsk-kamčatsko standardno vrijeme"}, {"Kazakhstan", "kazahstansko vrijeme"}, {"Kazakhstan_Eastern", "istočnokazahstansko vrijeme"}, {"Kazakhstan_Western", "zapadnokazahstansko vrijeme"}, {"Korea", "korejsko standardno vrijeme"}, {"Kosrae", "vrijeme Kosrae"}, {"Krasnoyarsk", "krasnojarsko standardno vrijeme"}, {"Kyrgystan", "kirgistansko vrijeme"}, {"Lanka", "lankansko vrijeme"}, {"Line_Islands", "vrijeme Ekvatorskih otoka"}, {"Lord_Howe", "standardno vrijeme otoka Lord Howe"}, {"Macau", "standardno makaosko vrijeme"}, {"Magadan", "magadansko standardno vrijeme"}, {"Malaysia", "malezijsko vrijeme"}, {"Maldives", "maldivsko vrijeme"}, {"Marquesas", "markižansko vrijeme"}, {"Marshall_Islands", "vrijeme Maršalovih Otoka"}, {"Mauritius", "standardno vrijeme Mauricijusa"}, {"Mawson", "mawsonsko vrijeme"}, {"Mexico_Pacific", "meksičko pacifičko standardno vrijeme"}, {"Mongolia", "ulanbatorsko standardno vrijeme"}, {"Moscow", "moskovsko standardno vrijeme"}, {"Myanmar", "mjanmarsko vrijeme"}, {"Nauru", "vrijeme Naurua"}, {"Nepal", "nepalsko vrijeme"}, {"New_Caledonia", "standardno vrijeme Nove Kaledonije"}, {"New_Zealand", "novozelandsko standardno vrijeme"}, {"Newfoundland", "newfoundlandsko standardno vrijeme"}, {"Niue", "vrijeme Niuea"}, {"Norfolk", "standardno vrijeme Otoka Norfolk"}, {"Noronha", "standardno vrijeme grada Fernando de Noronha"}, {"North_Mariana", "vrijeme Sjevernomarijanskih Otoka"}, {"Novosibirsk", "novosibirsko standardno vrijeme"}, {"Omsk", "omsko standardno vrijeme"}, {"Pakistan", "pakistansko standardno vrijeme"}, {"Palau", "vrijeme Palaua"}, {"Papua_New_Guinea", "vrijeme Papue Nove Gvineje"}, {"Paraguay", "paragvajsko standardno vrijeme"}, {"Peru", "peruansko standardno vrijeme"}, {"Philippines", "filipinsko standardno vrijeme"}, {"Phoenix_Islands", "vrijeme Otoka Phoenix"}, {"Pierre_Miquelon", "standardno vrijeme za Sveti Petar i Mikelon"}, {"Pitcairn", "pitcairnsko vrijeme"}, {"Ponape", "ponapejsko vrijeme"}, {"Pyongyang", "pjongjanško vrijeme"}, {"Qyzylorda", "standardno vrijeme grada Kizilorde"}, {"Reunion", "vrijeme Reuniona"}, {"Rothera", "vrijeme Rothere"}, {"Sakhalin", "sahalinsko standardno vrijeme"}, {"Samara", "samarsko standardno vrijeme"}, {"Samoa", "samoansko standardno vrijeme"}, {"Seychelles", "sejšelsko vrijeme"}, {"Singapore", "singapursko vrijeme"}, {"Solomon", "vrijeme Salomonovih Otoka"}, {"South_Georgia", "vrijeme Južne Georgije"}, {"Suriname", "surinamsko vrijeme"}, {"Syowa", "vrijeme Syowe"}, {"Tahiti", "vrijeme Tahitija"}, {"Taipei", "tajpeško standardno vrijeme"}, {"Tajikistan", "tadžikistansko vrijeme"}, {"Tokelau", "vrijeme Tokelaua"}, {"Tonga", "standardno vrijeme Tonge"}, {"Truk", "vrijeme Chuuka"}, {"Turkey", "tursko standardno vrijeme"}, {"Turkmenistan", "turkmenistansko standardno vrijeme"}, {"Tuvalu", "vrijeme Tuvalua"}, {"Uruguay", "urugvajsko standardno vrijeme"}, {"Uzbekistan", "uzbekistansko standardno vrijeme"}, {"Vanuatu", "standardno vrijeme Vanuatua"}, {"Venezuela", "venezuelsko vrijeme"}, {"Vladivostok", "vladivostočko standardno vrijeme"}, {"Volgograd", "volgogradsko standardno vrijeme"}, {"Vostok", "vostočko vrijeme"}, {"Wake", "vrijeme Otoka Wake"}, {"Wallis", "vrijeme Otoka Wallis i Futuna"}, {"Yakutsk", "jakutsko standardno vrijeme"}, {"Yekaterinburg", "jekaterinburško standardno vrijeme"}, {"Yukon", "jukonško vrijeme"}},
		{{"Acre", "Acre ljetno vrijeme"}, {"Alaska", "aljaško ljetno vrijeme"}, {"Almaty", "altmajsko ljetno vrijeme"}, {"Amazon", "amazonsko ljetno vrijeme"}, {"America_Central", "središnje ljetno vrijeme"}, {"America_Eastern", "istočno ljetno vrijeme"}, {"America_Mountain", "planinsko ljetno vrijeme"}, {"America_Pacific", "pacifičko ljetno vrijeme"}, {"Anadyr", "anadirsko ljetno vrijeme"}, {"Apia", "ljetno vrijeme Apije"}, {"Aqtau", "ljetno vrijeme grada Aktau"}, {"Aqtobe", "ljetno vrijeme grada Aktobe"}, {"Arabian", "arapsko ljetno vrijeme"}, {"Argentina", "argentinsko ljetno vrijeme"}, {"Argentina_Western", "zapadnoargentinsko ljetno vrijeme"}, {"Armenia", "armensko ljetno vrijeme"}, {"Atlantic", "atlantsko ljetno vrijeme"}, {"Australia_Central", "srednjoaustralsko ljetno vrijeme"}, {"Australia_CentralWestern", "australsko središnje zapadno ljetno vrijeme"}, {"Australia_Eastern", "istočnoaustralsko ljetno vrijeme"}, {"Australia_Western", "zapadnoaustralsko ljetno vrijeme"}, {"Azerbaijan", "azerbajdžansko ljetno vrijeme"}, {"Azores", "azorsko ljetno vrijeme"}, {"Bangladesh", "bangladeško ljetno vrijeme"}, {"Brasilia", "brazilsko ljetno vrijeme"}, {"Cape_Verde", "ljetno vrijeme Zelenortskog otočja"}, {"Chatham", "ljetno vrijeme Chathama"}, {"Chile", "čileansko ljetno vrijeme"}, {"China", "kinesko ljetno vrijeme"}, {"Colombia", "kolumbijsko ljetno vrijeme"}, {"Cook", "Cookovi Otoci, polusatni pomak, ljetno vrijeme"}, {"Cuba", "kubansko ljetno vrijeme"}, {"Easter", "ljetno vrijeme Uskršnjeg otoka"}, {"Europe_Central", "srednjoeuropsko ljetno vrijeme"}, {"Europe_Eastern", "istočnoeuropsko ljetno vrijeme"}, {"Europe_Western", "zapadnoeuropsko ljetno vrijeme"}, {"Falkland", "falklandsko ljetno vrijeme"}, {"Fiji", "ljetno vrijeme Fidžija"}, {"Georgia", "gruzijsko ljetno vrijeme"}, {"Greenland_Eastern", "istočnogrenlandsko ljetno vrijeme"}, {"Greenland_Western", "zapadnogrenlandsko ljetno vrijeme"}, {"Hawaii_Aleutian", "havajsko-aleutsko ljetno vrijeme"}, {"Hong_Kong", "hongkonško ljetno vrijeme"}, {"Hovd", "hovdsko ljetno vrijeme"}, {"Iran", "iransko ljetno vrijeme"}, {"Irkutsk", "irkutsko ljetno vrijeme"}, {"Israel", "izraelsko ljetno vrijeme"}, {"Japan", "japansko ljetno vrijeme"}, {"Kamchatka", "Petropavlovsk-kamčatsko ljetno vrijeme"}, {"Korea", "korejsko ljetno vrijeme"}, {"Krasnoyarsk", "krasnojarsko ljetno vrijeme"}, {"Lord_Howe", "ljetno vrijeme otoka Lord Howe"}, {"Macau", "ljetno makaosko vrijeme"}, {"Magadan", "magadansko ljetno vrijeme"}, {"Mauritius", "ljetno vrijeme Mauricijusa"}, {"Mexico_Pacific", "meksičko pacifičko ljetno vrijeme"}, {"Mongolia", "ulanbatorsko ljetno vrijeme"}, {"Moscow", "moskovsko ljetno vrijeme"}, {"New_Caledonia", "ljetno vrijeme Nove Kaledonije"}, {"New_Zealand", "novozelandsko ljetno vrijeme"}, {"Newfoundland", "newfoundlandsko ljetno vrijeme"}, {"Norfolk", "ljetno vrijeme Otoka Norfolk"}, {"Noronha", "ljetno vrijeme grada Fernando de Noronha"}, {"Novosibirsk", "novosibirsko ljetno vrijeme"}, {"Omsk", "omsko ljetno vrijeme"}, {"Pakistan", "pakistansko ljetno vrijeme"}, {"Paraguay", "paragvajsko ljetno vrijeme"}, {"Peru", "peruansko ljetno vrijeme"}, {"Philippines", "filipinsko ljetno vrijeme"}, {"Pierre_Miquelon", "ljetno vrijeme za Sveti Petar i Mikelon"}, {"Qyzylorda", "ljetno vrijeme grada Kizilorde"}, {"Sakhalin", "sahalinsko ljetno vrijeme"}, {"Samara", "samarsko ljetno vrijeme"}, {"Samoa", "samoansko ljetno vrijeme"}, {"Taipei", "tajpeško ljetno vrijeme"}, {"Tonga", "ljetno vrijeme Tonge"}, {"Turkey", "tursko ljetno vrijeme"}, {"Turkmenistan", "turkmenistansko ljetno vrijeme"}, {"Uruguay", "urugvajsko ljetno vrijeme"}, {"Uzbekistan", "uzbekistansko ljetno vrijeme"}, {"Vanuatu", "ljetno vrijeme Vanuatua"}, {"Vladivostok", "vladivostočko ljetno vrijeme"}, {"Volgograd", "volgogradsko ljetno vrijeme"}, {"Yakutsk", "jakutsko ljetno vrijeme"}, {"Yekaterinburg", "jekaterinburško ljetno vrijeme"}},
	},
	{
		{{"Afghanistan", "afghanski čas"}, {"Africa_Central", "centralnoafriski čas"}, {"Africa_Eastern", "wuchodoafriski čas"}, {"Africa_Southern", "južnoafriski čas"}, {"Africa_Western", "zapadoafriski čas"}, {"Alaska", "alaskaski standardny čas"}, {"Amazon", "Amaconaski standardny čas"}, {"America_Central", "sewjeroameriski centralny standardny čas"}, {"America_Eastern", "sewjeroameriski wuchodny standardny čas"}, {"America_Mountain", "sewjeroameriski hórski standardny čas"}, {"America_Pacific", "sewjeroameriski pacifiski standardny čas"}, {"Apia", "Apiaski standardny čas"}, {"Arabian", "arabski standardny čas"}, {"Argentina", "argentinski standardny čas"}, {"Argentina_Western", "zapadoargentinski standardny čas"}, {"Armenia", "armenski standardny čas"}, {"Atlantic", "atlantiski standardny čas"}, {"Australia_Central", "srjedźoawstralski standardny čas"}, {"Australia_CentralWestern", "srjedźozapadny awstralski standardny čas"}, {"Australia_Eastern", "wuchodoawstralski standardny čas"}, {"Australia_Western", "zapadoawstralski standardny čas"}, {"Azerbaijan", "azerbajdźanski standardny čas"}, {"Azores", "acorski standardny čas"}, {"Bangladesh", "bangladešski standardny čas"}, {"Bhutan", "bhutanski čas"}, {"Bolivia", "boliwiski čas"}, {"Brasilia", "Brasiliski standardny čas"}, {"Brunei", "bruneiski čas"}, {"Cape_Verde", "kapverdski standardny čas"}, {"Chamorro", "chamorroski čas"}, {"Chatham", "chathamski standardny čas"}, {"Chile", "chilski standardny čas"}, {"China", "chinski standardny čas"}, {"Christmas", "čas Hodowneje kupy"}, {"Cocos", "čas Kokosowych kupow"}, {"Colombia", "kolumbiski standardny čas"}, {"Cook", "standardny čas Cookowych kupow"}, {"Cuba", "kubaski standardny čas"}, {"Davis", "Daviski čas"}, {"DumontDUrville", "Dumont d´ Urvilleski čas"}, {"East_Timor", "wuchodnotimorski čas"}, {"Easter", "standardny čas Jutrowneje kupy"}, {"Ecuador", "ekwadorski čas"}, {"Europe_Central", "srjedźoeuropski standardny čas"}, {"Europe_Eastern", "wuchodoeuropski standardny čas"}, {"Europe_Further_Eastern", "Kaliningradski čas"}, {"Europe_Western", "zapadoeuropski standardny čas"}, {"Falkland", "falklandski standardny čas"}, {"Fiji", "fidźiski standardny čas"}, {"French_Guiana", "francoskoguyanski čas"}, {"French_Southern", "čas Francoskeho južneho a antarktiskeho teritorija"}, {"GMT", "Greenwichski čas"}, {"Galapagos", "galapagoski čas"}, {"Gambier", "gambierski čas"}, {"Georgia", "georgiski standardny čas"}, {"Gilbert_Islands", "čas Gilbertowych kupow"}, {"Greenland_Eastern", "wuchodogrönlandski standardny čas"}, {"Greenland_Western", "zapadogrönlandski standardny čas"}, {"Gulf", "čas Persiskeho golfa"}, {"Guyana", "guyanski čas"}, {"Hawaii", "hawaiisko-aleutski standardny čas"}, {"Hawaii_Aleutian", "hawaiisko-aleutski standardny čas"}, {"Hong_Kong", "Hongkongski standardny čas"}, {"Hovd", "Chowdski standardny čas"}, {"India", "indiski čas"}, {"Indian_Ocean", "indiskooceanski čas"}, {"Indochina", "indochinski čas"}, {"Indonesia_Central", "srjedźoindoneski čas"}, {"Indonesia_Eastern", "wuchodoindoneski"}, {"Indonesia_Western", "zapadoindoneski čas"}, {"Iran", "iranski standardny čas"}, {"Irkutsk", "Irkutski standardny čas"}, {"Israel", "israelski standardny čas"}, {"Japan", "japanski standardny čas"}, {"Kazakhstan", "kazachski čas"}, {"Kazakhstan_Eastern", "wuchodnokazachski čas"}, {"Kazakhstan_Western", "zapadnokazachski čas"}, {"Korea", "korejski standardny čas"}, {"Kosrae", "kosraeski čas"}, {"Krasnoyarsk", "Krasnojarski standardny čas"}, {"Kyrgystan", "kirgiski čas"}, {"Line_Islands", "čas Linijowych kupow"}, {"Lord_Howe", "standardny čas kupy Lord-Howe"}, {"Magadan", "Magadanski standardny čas"}, {"Malaysia", "malajziski čas"}, {"Maldives", "malediwski čas"}, {"Marquesas", "marquesaski čas"}, {"Marshall_Islands", "čas Marshallowych kupow"}, {"Mauritius", "mauritiuski standardny čas"}, {"Mawson", "Mawsonski čas"}, {"Mexico_Pacific", "mexiski pacifiski standardny čas"}, {"Mongolia", "Ulan-Batorski standardny čas"}, {"Moscow", "Moskowski standardny čas"}, {"Myanmar", "myanmarski čas"}, {"Nauru", "nauruski čas"}, {"Nepal", "nepalski čas"}, {"New_Caledonia", "nowokaledonski standardny čas"}, {"New_Zealand", "nowoseelandski standardny čas"}, {"Newfoundland", "nowofundlandski standardny čas"}, {"Niue", "niueski čas"}, {"Norfolk", "standardny čas kupy Norfolk"}, {"Noronha", "standardny čas kupow Fernando de Noronha"}, {"Novosibirsk", "Nowosibirski standardny čas"}, {"Omsk", "Omski standardny čas"}, {"Pakistan", "pakistanski standardny čas"}, {"Palau", "palauski čas"}, {"Papua_New_Guinea", "papua-nowoginejski čas"}, {"Paraguay", "Paraguayski standardny čas"}, {"Peru", "peruski standardny čas"}, {"Philippines", "filipinski standardny čas"}, {"Phoenix_Islands", "čas Phoenixowych kupow"}, {"Pierre_Miquelon", "standardny čas kupow St. Pierre a Miquelon"}, {"Pitcairn", "čas Pitcairnowych kupow"}, {"Ponape", "ponapeski čas"}, {"Pyongyang", "Pjöngjangski čas"}, {"Reunion", "reunionski čas"}, {"Rothera", "Rotheraski čas"}, {"Sakhalin", "sachalinski standardny čas"}, {"Samoa", "samoaski standardny čas"}, {"Seychelles", "seychellski čas"}, {"Singapore", "Singapurski čas"}, {"Solomon", "čas Salomonskich kupow"}, {"South_Georgia", "južnogeorgiski čas"}, {"Suriname", "surinamski čas"}, {"Syowa", "Syowaski čas"}, {"Tahiti", "tahitiski čas"}, {"Taipei", "Taipehski standardny čas"}, {"Tajikistan", "tadźikski čas"}, {"Tokelau", "tokelauski čas"}, {"Tonga", "tongaski standardny čas"}, {"Truk", "chuukski čas"}, {"Turkmenistan", "turkmenski standardny čas"}, {"Tuvalu", "tuvaluski čas"}, {"Uruguay", "uruguayski standardny čas"}, {"Uzbekistan", "uzbekski standardny čas"}, {"Vanuatu", "vanuatuski standardny čas"}, {"Venezuela", "venezuelski čas"}, {"Vladivostok", "Wladiwostokski standardny čas"}, {"Volgograd", "Wolgogradski standardny čas"}, {"Vostok", "Wostokski čas"}, {"Wake", "čas kupy Wake"}, {"Wallis", "čas kupow Wallis a Futuna"}, {"Yakutsk", "Jakutski standardny čas"}, {"Yekaterinburg", "Jekaterinburgski standardny čas"}, {"Yukon", "Yukonowy čas"}},
		{{"Alaska", "alaskaski lětni čas"}, {"Amazon", "Amaconaski lětni čas"}, {"America_Central", "sewjeroameriski centralny lětni čas"}, {"America_Eastern", "sewjeroameriski wuchodny lětni čas"}, {"America_Mountain", "sewjeroameriski hórski lětni čas"}, {"America_Pacific", "sewjeroameriski pacifiski lětni čas"}, {"Apia", "Apiaski lětni čas"}, {"Arabian", "arabski lětni čas"}, {"Argentina", "argentinski lětni čas"}, {"Argentina_Western", "zapadoargentinski lětni čas"}, {"Armenia", "armenski lětni čas"}, {"Atlantic", "atlantiski lětni čas"}, {"Australia_Central", "srjedźoawstralski lětni čas"}, {"Australia_CentralWestern", "sjedźozapadny awstralski lětni čas"}, {"Australia_Eastern", "wuchodoawstralski lětni čas"}, {"Australia_Western", "zapadoawstralski lětni čas"}, {"Azerbaijan", "azerbajdźanski lětni čas"}, {"Azores", "acorski lětni čas"}, {"Bangladesh", "bangladešski lětni čas"}, {"Brasilia", "Brasiliski lětni čas"}, {"Cape_Verde", "kapverdski lětni čas"}, {"Chatham", "chathamski lětni čas"}, {"Chile", "chilski lětni čas"}, {"China", "chinski lětni čas"}, {"Colombia", "kolumbiski lětni čas"}, {"Cook", "lětni čas Cookowych kupow"}, {"Cuba", "kubaski lětni čas"}, {"Easter", "lětni čas Jutrowneje kupy"}, {"Europe_Central", "srjedźoeuropski lětni čas"}, {"Europe_Eastern", "wuchodoeuropski lětni čas"}, {"Europe_Western", "zapadoeuropski lětni čas"}, {"Falkland", "falklandski lětni čas"}, {"Fiji", "fidźiski lětni čas"}, {"Georgia", "georgiski lětni čas"}, {"Greenland_Eastern", "wuchodogrönlandski lětni čas"}, {"Greenland_Western", "zapadogrönlandski lětni čas"}, {"Hawaii_Aleutian", "hawaiisko-aleutski lětni čas"}, {"Hong_Kong", "Hongkongski lětni čas"}, {"Hovd", "Chowdski lětni čas"}, {"Iran", "iranski lětni čas"}, {"Irkutsk", "Irkutski lětni čas"}, {"Israel", "israelski lětni čas"}, {"Japan", "japanski lětni čas"}, {"Korea", "korejski lětni čas"}, {"Krasnoyarsk", "Krasnojarski lětni čas"}, {"Lord_Howe", "lětni čas kupy Lord-Howe"}, {"Magadan", "Magadanski lětni čas"}, {"Mauritius", "mauritiuski lětni čas"}, {"Mexico_Pacific", "mexiski pacifiski lětni čas"}, {"Mongolia", "Ulan-Batorski lětni čas"}, {"Moscow", "Moskowski lětni čas"}, {"New_Caledonia", "nowokaledonski lětni čas"}, {"New_Zealand", "nowoseelandski lětni čas"}, {"Newfoundland", "nowofundlandski lětni čas"}, {"Norfolk", "lětni čas kupy Norfolk"}, {"Noronha", "lětni čas kupow Fernando de Noronha"}, {"Novosibirsk", "Nowosibirski lětni čas"}, {"Omsk", "Omski lětni čas"}, {"Pakistan", "pakistanski lětni čas"}, {"Paraguay", "Paraguayski lětni čas"}, {"Peru", "peruski lětni čas"}, {"Philippines", "filipinski lětni čas"}, {"Pierre_Miquelon", "lětni čas kupow St. Pierre a Miquelon"}, {"Sakhalin", "sachalinski lětni čas"}, {"Samoa", "samoaski lětni čas"}, {"Taipei", "Taipehski lětni čas"}, {"Tonga", "tongaski lětni čas"}, {"Turkmenistan", "turkmenski lětni čas"}, {"Uruguay", "uruguayski lětni čas"}, {"Uzbekistan", "uzbekski lětni čas"}, {"Vanuatu", "vanuatuski lětni čas"}, {"Vladivostok", "Wladiwostokski lětni čas"}, {"Volgograd", "Wolgogradski lětni čas"}, {"Yakutsk", "Jakutski lětni čas"}, {"Yekaterinburg", "Jekaterinburgski lětni čas"}},
	},
	{
		{{"Acre", "Acre zónaidő"}, {"Afghanistan", "afganisztáni idő"}, {"Africa_Central", "közép-afrikai téli idő"}, {"Africa_Eastern", "kelet-afrikai téli idő"}, {"Africa_Southern", "dél-afrikai téli idő"}, {"Africa_Western", "nyugat-afrikai időzóna"}, {"Alaska", "alaszkai zónaidő"}, {"Almaty", "Almati zónaidő"}, {"Amazon", "amazóniai téli idő"}, {"America_Central", "középső államokbeli zónaidő"}, {"America_Eastern", "keleti államokbeli zónaidő"}, {"America_Mountain", "hegyvidéki zónaidő"}, {"America_Pacific", "csendes-óceáni zónaidő"}, {"Anadyr", "Anadíri zónaidő"}, {"Apia", "apiai téli idő"}, {"Aqtau", "Aqtaui zónaidő"}, {"Aqtobe", "Aqtobei zónaidő"}, {"Arabian", "arab téli idő"}, {"Argentina", "argentínai téli idő"}, {"Argentina_Western", "nyugat-argentínai téli idő"}, {"Armenia", "örményországi téli idő"}, {"Atlantic", "atlanti-óceáni zónaidő"}, {"Australia_Central", "közép-ausztráliai téli idő"}, {"Australia_CentralWestern", "közép-nyugat-ausztráliai téli idő"}, {"Australia_Eastern", "kelet-ausztráliai téli idő"}, {"Australia_Western", "nyugat-ausztráliai téli idő"}, {"Azerbaijan", "azerbajdzsáni téli idő"}, {"Azores", "azori téli idő"}, {"Bangladesh", "bangladesi téli idő"}, {"Bhutan", "butáni idő"}, {"Bolivia", "bolíviai téli idő"}, {"Brasilia", "brazíliai téli idő"}, {"Brunei", "Brunei Darussalam-i idő"}, {"Cape_Verde", "zöld-foki-szigeteki téli idő"}, {"Casey", "casey-i idő"}, {"Chamorro", "chamorrói téli idő"}, {"Chatham", "chathami téli idő"}, {"Chile", "chilei téli idő"}, {"China", "kínai téli idő"}, {"Christmas", "karácsony-szigeti idő"}, {"Cocos", "kókusz-szigeteki téli idő"}, {"Colombia", "kolumbiai téli idő"}, {"Cook", "cook-szigeteki téli idő"}, {"Cuba", "kubai téli idő"}, {"Davis", "davisi idő"}, {"DumontDUrville", "dumont-d’Urville-i idő"}, {"East_Timor", "kelet-timori téli idő"}, {"Easter", "húsvét-szigeti téli idő"}, {"Ecuador", "ecuadori téli idő"}, {"Europe_Central", "közép-európai téli idő"}, {"Europe_Eastern", "kelet-európai téli idő"}, {"Europe_Further_Eastern", "minszki idő"}, {"Europe_Western", "nyugat-európai téli idő"}, {"Falkland", "falkland-szigeteki téli idő"}, {"Fiji", "fidzsi téli idő"}, {"French_Guiana", "francia-guyanai idő"}, {"French_Southern", "francia déli és antarktiszi idő"}, {"GMT", "greenwichi középidő, téli idő"}, {"Galapagos", "galápagosi téli idő"}, {"Gambier", "gambieri idő"}, {"Georgia", "grúziai téli idő"}, {"Gilbert_Islands", "gilbert-szigeteki idő"}, {"Greenland_Eastern", "kelet-grönlandi téli idő"}, {"Greenland_Western", "nyugat-grönlandi téli idő"}, {"Guam", "Guami zónaidő"}, {"Gulf", "öbölbeli téli idő"}, {"Guyana", "guyanai téli idő"}, {"Hawaii", "hawaii-aleuti téli idő"}, {"Hawaii_Aleutian", "hawaii-aleuti téli idő"}, {"Hong_Kong", "hongkongi téli idő"}, {"Hovd", "hovdi téli idő"}, {"India", "indiai téli idő"}, {"Indian_Ocean", "indiai-óceáni idő"}, {"Indochina", "indokínai idő"}, {"Indonesia_Central", "közép-indonéziai idő"}, {"Indonesia_Eastern", "kelet-indonéziai idő"}, {"Indonesia_Western", "nyugat-indonéziai téli idő"}, {"Iran", "iráni téli idő"}, {"Irkutsk", "irkutszki téli idő"}, {"Israel", "izraeli téli idő"}, {"Japan", "japán téli idő"}, {"Kamchatka", "Petropavlovszk-kamcsatkai zónaidő"}, {"Kazakhstan", "kazahsztáni idő"}, {"Kazakhstan_Eastern", "kelet-kazahsztáni idő"}, {"Kazakhstan_Western", "nyugat-kazahsztáni idő"}, {"Korea", "koreai téli idő"}, {"Kosrae", "kosraei idő"}, {"Krasnoyarsk", "krasznojarszki téli idő"}, {"Kyrgystan", "kirgizisztáni idő"}, {"Lanka", "Lankai idő"}, {"Line_Islands", "sor-szigeteki idő"}, {"Lord_Howe", "Lord Howe-szigeti téli idő"}, {"Macau", "Macaui zónaidő"}, {"Magadan", "magadani téli idő"}, {"Malaysia", "malajziai idő"}, {"Maldives", "maldív-szigeteki idő"}, {"Marquesas", "marquises-szigeteki idő"}, {"Marshall_Islands", "marshall-szigeteki idő"}, {"Mauritius", "mauritiusi téli idő"}, {"Mawson", "mawsoni idő"}, {"Mexico_Pacific", "mexikói csendes-óceáni zónaidő"}, {"Mongolia", "ulánbátori téli idő"}, {"Moscow", "moszkvai téli idő"}, {"Myanmar", "mianmari idő"}, {"Nauru", "naurui idő"}, {"Nepal", "nepáli idő"}, {"New_Caledonia", "új-kaledóniai téli idő"}, {"New_Zealand", "új-zélandi téli idő"}, {"Newfoundland", "új-fundlandi zónaidő"}, {"Niue", "niuei idő"}, {"Norfolk", "norfolk-szigeteki téli idő"}, {"Noronha", "Fernando de Noronha-i téli idő"}, {"North_Mariana", "Észak-mariana-szigeteki idő"}, {"Novosibirsk", "novoszibirszki téli idő"}, {"Omsk", "omszki téli idő"}, {"Pakistan", "pakisztáni téli idő"}, {"Palau", "palaui idő"}, {"Papua_New_Guinea", "pápua új-guineai idő"}, {"Paraguay", "paraguayi téli idő"}, {"Peru", "perui téli idő"}, {"Philippines", "fülöp-szigeteki téli idő"}, {"Phoenix_Islands", "phoenix-szigeteki téli idő"}, {"Pierre_Miquelon", "Saint-Pierre és Miquelon-i zónaidő"}, {"Pitcairn", "pitcairn-szigeteki idő"}, {"Ponape", "ponape-szigeti idő"}, {"Pyongyang", "phenjani idő"}, {"Qyzylorda", "Qyzylordai zónaidő"}, {"Reunion", "réunioni idő"}, {"Rothera", "rotherai idő"}, {"Sakhalin", "szahalini téli idő"}, {"Samara", "Szamarai zónaidő"}, {"Samoa", "szamoai téli idő"}, {"Seychelles", "seychelle-szigeteki idő"}, {"Singapore", "szingapúri téli idő"}, {"Solomon", "salamon-szigeteki idő"}, {"South_Georgia", "déli-georgiai idő"}, {"Suriname", "szurinámi idő"}, {"Syowa", "syowai idő"}, {"Tahiti", "tahiti idő"}, {"Taipei", "taipei téli idő"}, {"Tajikistan", "tádzsikisztáni idő"}, {"Tokelau", "tokelaui idő"}, {"Tonga", "tongai téli idő"}, {"Truk", "truki idő"}, {"Turkey", "Törökországi zónaidő"}, {"Turkmenistan", "türkmenisztáni téli idő"}, {"Tuvalu", "tuvalui idő"}, {"Uruguay", "uruguayi téli idő"}, {"Uzbekistan", "üzbegisztáni téli idő"}, {"Vanuatu", "vanuatui téli idő"}, {"Venezuela", "venezuelai idő"}, {"Vladivostok", "vlagyivosztoki téli idő"}, {"Volgograd", "volgográdi téli idő"}, {"Vostok", "vosztoki idő"}, {"Wake", "wake-szigeti idő"}, {"Wallis", "Wallis és Futuna-i idő"}, {"Yakutsk", "jakutszki téli idő"}, {"Yekaterinburg", "jekatyerinburgi téli idő"}, {"Yukon", "yukoni idő"}},
		{{"Acre", "Acre nyári idő"}, {"Alaska", "alaszkai nyári idő"}, {"Almaty", "Almati nyári idő"}, {"Amazon", "amazóniai nyári idő"}, {"America_Central", "középső államokbeli nyári idő"}, {"America_Eastern", "keleti államokbeli nyári idő"}, {"America_Mountain", "hegyvidéki nyári idő"}, {"America_Pacific", "csendes-óceáni nyári idő"}, {"Anadyr", "Anadíri nyári idő"}, {"Apia", "apiai nyári idő"}, {"Aqtau", "Aqtaui nyári idő"}, {"Aqtobe", "Aqtobei nyári idő"}, {"Arabian", "arab nyári idő"}, {"Argentina", "argentínai nyári idő"}, {"Argentina_Western", "nyugat-argentínai nyári idő"}, {"Armenia", "örményországi nyári idő"}, {"Atlantic", "atlanti-óceáni nyári idő"}, {"Australia_Central", "közép-ausztráliai nyári idő"}, {"Australia_CentralWestern", "közép-nyugat-ausztráliai nyári idő"}, {"Australia_Eastern", "kelet-ausztráliai nyári idő"}, {"Australia_Western", "nyugat-ausztráliai nyári idő"}, {"Azerbaijan", "azerbajdzsáni nyári idő"}, {"Azores", "azori nyári idő"}, {"Bangladesh", "bangladesi nyári idő"}, {"Brasilia", "brazíliai nyári idő"}, {"Cape_Verde", "zöld-foki-szigeteki nyári idő"}, {"Chatham", "chathami nyári idő"}, {"Chile", "chilei nyári idő"}, {"China", "kínai nyári idő"}, {"Colombia", "kolumbiai nyári idő"}, {"Cook", "cook-szigeteki fél nyári idő"}, {"Cuba", "kubai nyári idő"}, {"Easter", "húsvét-szigeti nyári idő"}, {"Europe_Central", "közép-európai nyári idő"}, {"Europe_Eastern", "kelet-európai nyári idő"}, {"Europe_Western", "nyugat-európai nyári idő"}, {"Falkland", "falkland-szigeteki nyári idő"}, {"Fiji", "fidzsi nyári idő"}, {"Georgia", "grúziai nyári idő"}, {"Greenland_Eastern", "kelet-grönlandi nyári idő"}, {"Greenland_Western", "nyugat-grönlandi nyári idő"}, {"Hawaii_Aleutian", "hawaii-aleuti nyári idő"}, {"Hong_Kong", "hongkongi nyári idő"}, {"Hovd", "hovdi nyári idő"}, {"Iran", "iráni nyári idő"}, {"Irkutsk", "irkutszki nyári idő"}, {"Israel", "izraeli nyári idő"}, {"Japan", "japán nyári idő"}, {"Kamchatka", "Petropavlovszk-kamcsatkai nyári idő"}, {"Korea", "koreai nyári idő"}, {"Krasnoyarsk", "krasznojarszki nyári idő"}, {"Lord_Howe", "Lord Howe-szigeti nyári idő"}, {"Macau", "Macaui nyári idő"}, {"Magadan", "magadáni nyári idő"}, {"Mauritius", "mauritiusi nyári idő"}, {"Mexico_Pacific", "mexikói csendes-óceáni nyári idő"}, {"Mongolia", "ulánbátori nyári idő"}, {"Moscow", "moszkvai nyári idő"}, {"New_Caledonia", "új-kaledóniai nyári idő"}, {"New_Zealand", "új-zélandi nyári idő"}, {"Newfoundland", "új-fundlandi nyári idő"}, {"Norfolk", "norfolk-szigeteki nyári idő"}, {"Noronha", "Fernando de Noronha-i nyári idő"}, {"Novosibirsk", "novoszibirszki nyári idő"}, {"Omsk", "omszki nyári idő"}, {"Pakistan", "pakisztáni nyári idő"}, {"Paraguay", "paraguayi nyári idő"}, {"Peru", "perui nyári idő"}, {"Philippines", "fülöp-szigeteki nyári idő"}, {"Pierre_Miquelon", "Saint-Pierre és Miquelon-i nyári idő"}, {"Qyzylorda", "Qyzylordai nyári idő"}, {"Sakhalin", "szahalini nyári idő"}, {"Samara", "Szamarai nyári idő"}, {"Samoa", "szamoai nyári idő"}, {"Taipei", "taipei nyári idő"}, {"Tonga", "tongai nyári idő"}, {"Turkey", "Törökországi nyári idő"}, {"Turkmenistan", "türkmenisztáni nyári idő"}, {"Uruguay", "uruguayi nyári idő"}, {"Uzbekistan", "üzbegisztáni nyári idő"}, {"Vanuatu", "vanuatui nyári idő"}, {"Vladivostok", "vlagyivosztoki nyári idő"}, {"Volgograd", "volgográdi nyári idő"}, {"Yakutsk", "jakutszki nyári idő"}, {"Yekaterinburg", "jekatyerinburgi nyári idő"}},
	},
	{
		{{"Afghanistan", "Աֆղանստանի ժամանակ"}, {"Africa_Central", "Կենտրոնական Աֆրիկայի ժամանակ"}, {"Africa_Eastern", "Արևելյան Աֆրիկայի ժամանակ"}, {"Africa_Southern", "Հարավային Աֆրիկայի ժամանակ"}, {"Africa_Western", "Արևմտյան Աֆրիկայի ժամանակ"}, {"Alaska", "Ալյասկայի ստանդարտ ժամանակ"}, {"Amazon", "Ամազոնյան ստանդարտ ժամանակ"}, {"America_Central", "Կենտրոնական Ամերիկայի ստանդարտ ժամանակ"}, {"America_Eastern", "Արևելյան Ամերիկայի ստանդարտ ժամանակ"}, {"America_Mountain", "Լեռնային ստանդարտ ժամանակ (ԱՄՆ)"}, {"America_Pacific", "Խաղաղօվկիանոսյան ստանդարտ ժամանակ"}, {"Apia", "Ապիայի ստանդարտ ժամանակ"}, {"Arabian", "Սաուդյան Արաբիայի ստանդարտ ժամանակ"}, {"Argentina", "Արգենտինայի ստնադարտ ժամանակ"}, {"Argentina_Western", "Արևմտյան Արգենտինայի ստնադարտ ժամանակ"}, {"Armenia", "Հայաստանի ստանդարտ ժամանակ"}, {"Atlantic", "Ատլանտյան ստանդարտ ժամանակ"}, {"Australia_Central", "Կենտրոնական Ավստրալիայի ստանդարտ ժամանակ"}, {"Australia_CentralWestern", "Կենտրոնական Ավստրալիայի արևմտյան ստանդարտ ժամանակ"}, {"Australia_Eastern", "Արևելյան Ավստրալիայի ստանդարտ ժամանակ"}, {"Australia_Western", "Արևմտյան Ավստրալիայի ստանդարտ ժամանակ"}, {"Azerbaijan", "Ադրբեջանի ստանդարտ ժամանակ"}, {"Azores", "Ազորյան կղզիների ստանդարտ ժամանակ"}, {"Bangladesh", "Բանգլադեշի ստանդարտ ժամանակ"}, {"Bhutan", "Բութանի ժամանակ"}, {"Bolivia", "Բոլիվիայի ժամանակ"}, {"Brasilia", "Բրազիլիայի ստանդարտ ժամանակ"}, {"Brunei", "Բրունեյի ժամանակ"}, {"Cape_Verde", "Կաբո Վերդեի ստանդարտ ժամանակ"}, {"Chamorro", "Չամոռոյի ժամանակ"}, {"Chatham", "Չաթեմ կղզու ստանդարտ ժամանակ"}, {"Chile", "Չիլիի ստանդարտ ժամանակ"}, {"China", "Չինաստանի ստանդարտ ժամանակ"}, {"Christmas", "Սուրբ Ծննդյան կղզու ժամանակ"}, {"Cocos", "Կոկոսյան կղզիների ժամանակ"}, {"Colombia", "Կոլումբիայի ստանդարտ ժամանակ"}, {"Cook", "Կուկի կղզիների ստանդարտ ժամանակ"}, {"Cuba", "Կուբայի ստանդարտ ժամանակ"}, {"Davis", "Դեյվիսի ժամանակ"}, {"DumontDUrville", "Դյումոն դ’Յուրվիլի ժամանակ"}, {"East_Timor", "Արևելյան Թիմորի ժամանակ"}, {"Easter", "Զատկի կղզու ստանդարտ ժամանակ"}, {"Ecuador", "Էկվադորի ժամանակ"}, {"Europe_Central", "Կենտրոնական Եվրոպայի ստանդարտ ժամանակ"}, {"Europe_Eastern", "Արևելյան Եվրոպայի ստանդարտ ժամանակ"}, {"Europe_Further_Eastern", "Մինսկի ժամանակ"}, {"Europe_Western", "Արևմտյան Եվրոպայի ստանդարտ ժամանակ"}, {"Falkland", "Ֆոլքլենդյան կղզիների ստանդարտ ժամանակ"}, {"Fiji", "Ֆիջիի ստանդարտ ժամանակ"}, {"French_Guiana", "Ֆրանսիական Գվիանայի ժամանակ"}, {"French_Southern", "Ֆրանսիական հարավային և անտարկտիդյան ժամանակ"}, {"GMT", "Գրինվիչի ժամանակ"}, {"Galapagos", "Գալապագոսյան կղզիների ժամանակ"}, {"Gambier", "Գամբյե կղզիների ժամանակ"}, {"Georgia", "Վրաստանի ստանդարտ ժամանակ"}, {"Gilbert_Islands", "Ջիլբերթի կղզիների ժամանակ"}, {"Greenland_Eastern", "Արևելյան Գրենլանդիայի ստանդարտ ժամանակ"}, {"Greenland_Western", "Արևմտյան Գրենլանդիայի ստանդարտ ժամանակ"}, {"Gulf", "Պարսից ծոցի ստանդարտ ժամանակ"}, {"Guyana", "Գայանայի ժամանակ"}, {"Hawaii", "Հավայան-ալեության ստանդարտ ժամանակ"}, {"Hawaii_Aleutian", "Հավայան-ալեության ստանդարտ ժամանակ"}, {"Hong_Kong", "Հոնկոնգի ստանդարտ ժամանակ"}, {"Hovd", "Հովդի ստանդարտ ժամանակ"}, {"India", "Հնդկաստանի ստանդարտ ժամանակ"}, {"Indian_Ocean", "Հնդկական օվկիանոսի ժամանակ"}, {"Indochina", "Հնդկաչինական ժամանակ"}, {"Indonesia_Central", "Կենտրոնական Ինդոնեզիայի ժամանակ"}, {"Indonesia_Eastern", "Արևելյան Ինդոնեզիայի ժամանակ"}, {"Indonesia_Western", "Արևմտյան Ինդոնեզիայի ժամանակ"}, {"Iran", "Իրանի ստանդարտ ժամանակ"}, {"Irkutsk", "Իրկուտսկի ստանդարտ ժամանակ"}, {"Israel", "Իսրայելի ստանդարտ ժամանակ"}, {"Japan", "Ճապոնիայի ստանդարտ ժամանակ"}, {"Kazakhstan", "Ղազախստանի ժամանակ"}, {"Kazakhstan_Eastern", "Արևելյան Ղազախստանի ժամանակ"}, {"Kazakhstan_Western", "Արևմտյան Ղազախստանի ժամանակ"}, {"Korea", "Կորեայի ստանդարտ ժամանակ"}, {"Kosrae", "Կոսրաեյի ժամանակ"}, {"Krasnoyarsk", "Կրասնոյարսկի ստանդարտ ժամանակ"}, {"Kyrgystan", "Ղրղզստանի ժամանակ"}, {"Line_Islands", "Լայն կղզիների ժամանակ"}, {"Lord_Howe", "Լորդ Հաուի ստանդարտ ժամանակ"}, {"Magadan", "Մագադանի ստանդարտ ժամանակ"}, {"Malaysia", "Մալայզիայի ժամանակ"}, {"Maldives", "Մալդիվների ժամանակ"}, {"Marquesas", "Մարկիզյան կղզիների ժամանակ"}, {"Marshall_Islands", "Մարշալյան կղզիների ժամանակ"}, {"Mauritius", "Մավրիկիոսի ստանդարտ ժամանակ"}, {"Mawson", "Մոուսոնի ժամանակ"}, {"Mexico_Pacific", "Մեքսիկայի խաղաղօվկիանոսյան ստանդարտ ժամանակ"}, {"Mongolia", "Ուլան Բատորի ստանդարտ ժամանակ"}, {"Moscow", "Մոսկվայի ստանդարտ ժամանակ"}, {"Myanmar", "Մյանմայի ժամանակ"}, {"Nauru", "Նաուրուի ժամանակ"}, {"Nepal", "Նեպալի ժամանակ"}, {"New_Caledonia", "Նոր Կալեդոնիայի ստանդարտ ժամանակ"}, {"New_Zealand", "Նոր Զելանդիայի ստանդարտ ժամանակ"}, {"Newfoundland", "Նյուֆաունդլենդի ստանդարտ ժամանակ"}, {"Niue", "Նիուեյի ժամանակ"}, {"Norfolk", "Նորֆոլկ կղզու ստանդարտ ժամանակ"}, {"Noronha", "Ֆերնանդու դի Նորոնյայի ստանդարտ ժամանակ"}, {"Novosibirsk", "Նովոսիբիրսկի ստանդարտ ժամանակ"}, {"Omsk", "Օմսկի ստանդարտ ժամանակ"}, {"Pakistan", "Պակիստանի ստանդարտ ժամանակ"}, {"Palau", "Պալաույի ժամանակ"}, {"Papua_New_Guinea", "Պապուա Նոր Գվինեայի ժամանակ"}, {"Paraguay", "Պարագվայի ստանդարտ ժամանակ"}, {"Peru", "Պերուի ստանդարտ ժամանակ"}, {"Philippines", "Ֆիլիպինների ստանդարտ ժամանակ"}, {"Phoenix_Islands", "Ֆինիքս կղզիների ժամանակ"}, {"Pierre_Miquelon", "Սեն Պիեռ և Միքելոնի ստանդարտ ժամանակ"}, {"Pitcairn", "Պիտկեռնի ժամանակ"}, {"Ponape", "Պոնապե կղզու ժամանակ"}, {"Pyongyang", "Փխենյանի ժամանակ"}, {"Reunion", "Ռեյունիոնի ժամանակ"}, {"Rothera", "Ռոտերայի ժամանակ"}, {"Sakhalin", "Սախալինի ստանդարտ ժամանակ"}, {"Samoa", "Սամոայի ստանդարտ ժամանակ"}, {"Seychelles", "Սեյշելյան կղզիների ժամանակ"}, {"Singapore", "Սինգապուրի ժամանակ"}, {"Solomon", "Սողոմոնի կղզիների ժամանակ"}, {"South_Georgia", "Հարավային Ջորջիայի ժամանակ"}, {"Suriname", "Սուրինամի ժամանակ"}, {"Syowa", "Սյովայի ժամանակ"}, {"Tahiti", "Թաիթիի ժամանակ"}, {"Taipei", "Թայպեյի ստանդարտ ժամանակ"}, {"Tajikistan", "Տաջիկստանի ժամանակ"}, {"Tokelau", "Տոկելաույի ժամանակ"}, {"Tonga", "Տոնգայի ստանդարտ ժամանակ"}, {"Truk", "Տրուկի ժամանակ"}, {"Turkmenistan", "Թուրքմենստանի ստանդարտ ժամանակ"}, {"Tuvalu", "Տուվալույի ժամանակ"}, {"Uruguay", "Ուրուգվայի ստանդարտ ժամանակ"}, {"Uzbekistan", "Ուզբեկստանի ստանդարտ ժամանակ"}, {"Vanuatu", "Վանուատույի ստանդարտ ժամանակ"}, {"Venezuela", "Վենեսուելայի ժամանակ"}, {"Vladivostok", "Վլադիվոստոկի ստանդարտ ժամանակ"}, {"Volgograd", "Վոլգոգրադի ստանդարտ ժամանակ"}, {"Vostok", "Վոստոկի ժամանակ"}, {"Wake", "Ուեյք կղզու ժամանակ"}, {"Wallis", "Ուոլիս և Ֆուտունայի ժամանակ"}, {"Yakutsk", "Յակուտսկի ստանդարտ ժամանակ"}, {"Yekaterinburg", "Եկատերինբուրգի ստանդարտ ժամանակ"}, {"Yukon", "Յուկոնի ժամանակ"}},
		{{"Alaska", "Ալյասկայի ամառային ժամանակ"}, {"Amazon", "Ամազոնյան ամառային ժամանակ"}, {"America_Central", "Կենտրոնական Ամերիկայի ամառային ժամանակ"}, {"America_Eastern", "Արևելյան Ամերիկայի ամառային ժամանակ"}, {"America_Mountain", "Լեռնային ամառային ժամանակ (ԱՄՆ)"}, {"America_Pacific", "Խաղաղօվկիանոսյան ամառային ժամանակ"}, {"Apia", "Ապիայի ամառային ժամանակ"}, {"Arabian", "Սաուդյան Արաբիայի ամառային ժամանակ"}, {"Argentina", "Արգենտինայի ամառային ժամանակ"}, {"Argentina_Western", "Արևմտյան Արգենտինայի ամառային ժամանակ"}, {"Armenia", "Հայաստանի ամառային ժամանակ"}, {"Atlantic", "Ատլանտյան ամառային ժամանակ"}, {"Australia_Central", "Կենտրոնական Ավստրալիայի ամառային ժամանակ"}, {"Australia_CentralWestern", "Կենտրոնական Ավստրալիայի արևմտյան ամառային ժամանակ"}, {"Australia_Eastern", "Արևելյան Ավստրալիայի ամառային ժամանակ"}, {"Australia_Western", "Արևմտյան Ավստրալիայի ամառային ժամանակ"}, {"Azerbaijan", "Ադրբեջանի ամառային ժամանակ"}, {"Azores", "Ազորյան կղզիների ամառային ժամանակ"}, {"Bangladesh", "Բանգլադեշի ամառային ժամանակ"}, {"Brasilia", "Բրազիլիայի ամառային ժամանակ"}, {"Cape_Verde", "Կաբո Վերդեի ամառային ժամանակ"}, {"Chatham", "Չաթեմ կղզու ամառային ժամանակ"}, {"Chile", "Չիլիի ամառային ժամանակ"}, {"China", "Չինաստանի ամառային ժամանակ"}, {"Colombia", "Կոլումբիայի ամառային ժամանակ"}, {"Cook", "Կուկի կղզիների կիսաամառային ժամանակ"}, {"Cuba", "Կուբայի ամառային ժամանակ"}, {"Easter", "Զատկի կղզու ամառային ժամանակ"}, {"Europe_Central", "Կենտրոնական Եվրոպայի ամառային ժամանակ"}, {"Europe_Eastern", "Արևելյան Եվրոպայի ամառային ժամանակ"}, {"Europe_Western", "Արևմտյան Եվրոպայի ամառային ժամանակ"}, {"Falkland", "Ֆոլքլենդյան կղզիների ամառային ժամանակ"}, {"Fiji", "Ֆիջիի ամառային ժամանակ"}, {"Georgia", "Վրաստանի ամառային ժամանակ"}, {"Greenland_Eastern", "Արևելյան Գրենլանդիայի ամառային ժամանակ"}, {"Greenland_Western", "Արևմտյան Գրենլանդիայի ամառային ժամանակ"}, {"Hawaii_Aleutian", "Հավայան-ալեության ամառային ժամանակ"}, {"Hong_Kong", "Հոնկոնգի ամառային ժամանակ"}, {"Hovd", "Հովդի ամառային ժամանակ"}, {"Iran", "Իրանի ամառային ժամանակ"}, {"Irkutsk", "Իրկուտսկի ամառային ժամանակ"}, {"Israel", "Իսրայելի ամառային ժամանակ"}, {"Japan", "Ճապոնիայի ամառային ժամանակ"}, {"Korea", "Կորեայի ամառային ժամանակ"}, {"Krasnoyarsk", "Կրասնոյարսկի ամառային ժամանակ"}, {"Lord_Howe", "Լորդ Հաուի ամառային ժամանակ"}, {"Magadan", "Մագադանի ամառային ժամանակ"}, {"Mauritius", "Մավրիկիոսի ամառային ժամանակ"}, {"Mexico_Pacific", "Մեքսիկայի խաղաղօվկիանոսյան ամառային ժամանակ"}, {"Mongolia", "Ուլան Բատորի ամառային ժամանակ"}, {"Moscow", "Մոսկվայի ամառային ժամանակ"}, {"New_Caledonia", "Նոր Կալեդոնիայի ամառային ժամանակ"}, {"New_Zealand", "Նոր Զելանդիայի ամառային ժամանակ"}, {"Newfoundland", "Նյուֆաունդլենդի ամառային ժամանակ"}, {"Norfolk", "Նորֆոլկ կղզու ամառային ժամանակ"}, {"Noronha", "Ֆերնանդու դի Նորոնյայի ամառային ժամանակ"}, {"Novosibirsk", "Նովոսիբիրսկի ամառային ժամանակ"}, {"Omsk", "Օմսկի ամառային ժամանակ"}, {"Pakistan", "Պակիստանի ամառային ժամանակ"}, {"Paraguay", "Պարագվայի ամառային ժամանակ"}, {"Peru", "Պերուի ամառային ժամանակ"}, {"Philippines", "Ֆիլիպինների ամառային ժամանակ"}, {"Pierre_Miquelon", "Սեն Պիեռ և Միքելոնի ամառային ժամանակ"}, {"Sakhalin", "Սախալինի ամառային ժամանակ"}, {"Samoa", "Սամոայի ամառային ժամանակ"}, {"Taipei", "Թայպեյի ամառային ժամանակ"}, {"Tonga", "Տոնգայի ամառային ժամանակ"}, {"Turkmenistan", "Թուրքմենստանի ամառային ժամանակ"}, {"Uruguay", "Ուրուգվայի ամառային ժամանակ"}, {"Uzbekistan", "Ուզբեկստանի ամառային ժամանակ"}, {"Vanuatu", "Վանուատույի ամառային ժամանակ"}, {"Vladivostok", "Վլադիվոստոկի ամառային ժամանակ"}, {"Volgograd", "Վոլգոգրադի ամառային ժամանակ"}, {"Yakutsk", "Յակուտսկի ամառային ժամանակ"}, {"Yekaterinburg", "Եկատերինբուրգի ամառային ժամանակ"}},
	},
	{
		{{"Acre", "hora normal de Acre"}, {"Afghanistan", "hora de Afghanistan"}, {"Africa_Central", "hora de Africa Central"}, {"Africa_Eastern", "hora de Africa del Est"}, {"Africa_Southern", "hora normal de Africa del Sud"}, {"Africa_Western", "hora de Africa del West"}, {"Alaska", "hora normal de Alaska"}, {"Amazon", "hora normal de Amazonia"}, {"America_Central", "hora normal central"}, {"America_Eastern", "hora normal del est"}, {"America_Mountain", "hora normal del montanias"}, {"America_Pacific", "hora normal pacific"}, {"Apia", "hora normal de Samoa"}, {"Arabian", "hora normal arabe"}, {"Argentina", "hora normal de Argentina"}, {"Argentina_Western", "hora normal de Argentina occidental"}, {"Armenia", "hora normal de Armenia"}, {"Atlantic", "hora normal atlantic"}, {"Australia_Central", "hora normal de Australia central"}, {"Australia_CentralWestern", "hora normal de Australia centro-occidental"}, {"Australia_Eastern", "hora normal de Australia oriental"}, {"Australia_Western", "hora normal de Australia occidental"}, {"Azerbaijan", "hora normal de Azerbeidzhan"}, {"Azores", "hora normal del Azores"}, {"Bangladesh", "hora normal de Bangladesh"}, {"Bhutan", "hora de Bhutan"}, {"Bolivia", "hora de Bolivia"}, {"Brasilia", "hora normal de Brasilia"}, {"Brunei", "hora de Brunei"}, {"Cape_Verde", "hora normal de Capo Verde"}, {"Chamorro", "hora normal de Chamorro"}, {"Chatham", "hora normal de Chatham"}, {"Chile", "hora normal de Chile"}, {"China", "hora normal de China"}, {"Christmas", "hora del Insula de Natal"}, {"Cocos", "hora del Insulas Cocos"}, {"Colombia", "hora normal de Colombia"}, {"Cook", "hora normal del Insulas Cook"}, {"Cuba", "hora normal de Cuba"}, {"Davis", "hora de Davis"}, {"DumontDUrville", "hora de Dumont d’Urville"}, {"East_Timor", "hora de Timor Oriental"}, {"Easter", "hora normal del Insula de Pascha"}, {"Ecuador", "hora de Ecuador"}, {"Europe_Central", "hora normal de Europa central"}, {"Europe_Eastern", "hora normal de Europa oriental"}, {"Europe_Further_Eastern", "hora de Europa ultra-oriental"}, {"Europe_Western", "hora normal de Europa occidental"}, {"Falkland", "hora normal del Insulas Falkland"}, {"Fiji", "hora normal de Fiji"}, {"French_Guiana", "hora de Guiana Francese"}, {"French_Southern", "hora francese meridional e antarctic"}, {"GMT", "hora medie de Greenwich"}, {"Galapagos", "hora del Galápagos"}, {"Gambier", "hora de Gambier"}, {"Georgia", "hora normal de Georgia"}, {"Gilbert_Islands", "hora del Insulas Gilbert"}, {"Greenland", "hora normal de Groenlandia"}, {"Greenland_Eastern", "hora normal de Groenlandia oriental"}, {"Greenland_Western", "hora normal de Groenlandia occidental"}, {"Guam", "hora normal de Guam"}, {"Gulf", "hora normal del Golfo"}, {"Guyana", "hora de Guyana"}, {"Hawaii", "hora normal de Hawaii-Aleutianas"}, {"Hawaii_Aleutian", "hora normal de Hawaii-Aleutianas"}, {"Hong_Kong", "hora normal de Hongkong"}, {"Hovd", "hora normal de Khovd"}, {"India", "hora normal de India"}, {"Indian_Ocean", "hora del Oceano Indian"}, {"Indochina", "hora de Indochina"}, {"Indonesia_Central", "hora de Indonesia Central"}, {"Indonesia_Eastern", "hora de Indonesia del Est"}, {"Indonesia_Western", "hora de Indonesia del West"}, {"Iran", "hora normal de Iran"}, {"Irkutsk", "hora normal de Irkutsk"}, {"Israel", "hora normal de Israel"}, {"Japan", "hora normal de Japon"}, {"Kazakhstan", "hora de Kazakhstan"}, {"Kazakhstan_Eastern", "hora de Kazakhstan del Est"}, {"Kazakhstan_Western", "hora de Kazakhstan del West"}, {"Korea", "hora normal de Corea"}, {"Kosrae", "hora de Kosrae"}, {"Krasnoyarsk", "hora normal de Krasnoyarsk"}, {"Kyrgystan", "hora de Kirghizistan"}, {"Line_Islands", "hora del Insulas del Linea"}, {"Lord_Howe", "hora normal de Lord Howe"}, {"Magadan", "hora normal de Magadan"}, {"Malaysia", "hora de Malaysia"}, {"Maldives", "hora del Maldivas"}, {"Marquesas", "hora de Marquesas"}, {"Marshall_Islands", "hora del Insulas Marshall"}, {"Mauritius", "hora normal de Mauritio"}, {"Mawson", "hora de Mawson"}, {"Mexico_Pacific", "hora normal del Pacifico mexican"}, {"Mongolia", "hora normal de Ulan Bator"}, {"Moscow", "hora normal de Moscova"}, {"Myanmar", "hora de Myanmar"}, {"Nauru", "hora de Nauru"}, {"Nepal", "hora de Nepal"}, {"New_Caledonia", "hora normal de Nove Caledonia"}, {"New_Zealand", "hora normal de Nove Zelanda"}, {"Newfoundland", "hora normal de Terranova"}, {"Niue", "hora de Niue"}, {"Norfolk", "hora normal del Insula Norfolk"}, {"Noronha", "hora normal de Fernando de Noronha"}, {"Novosibirsk", "hora normal de Novosibirsk"}, {"Omsk", "hora normal de Omsk"}, {"Pakistan", "hora normal de Pakistan"}, {"Palau", "hora de Palau"}, {"Papua_New_Guinea", "hora de Papua Nove Guinea"}, {"Paraguay", "hora normal de Paraguay"}, {"Peru", "hora normal de Peru"}, {"Philippines", "hora normal del Philippinas"}, {"Phoenix_Islands", "hora del Insulas Phenice"}, {"Pierre_Miquelon", "hora normal de Saint-Pierre e Miquelon"}, {"Pitcairn", "hora de Pitcairn"}, {"Ponape", "hora de Pohnpei"}, {"Pyongyang", "hora de Corea del Nord"}, {"Reunion", "hora de Réunion"}, {"Rothera", "hora de Rothera"}, {"Sakhalin", "hora normal de Sachalin"}, {"Samoa", "hora normal de Samoa american"}, {"Seychelles", "hora del Seychelles"}, {"Singapore", "hora normal de Singapore"}, {"Solomon", "hora del Insulas Solomon"}, {"South_Georgia", "hora de Georgia del Sud"}, {"Suriname", "hora de Suriname"}, {"Syowa", "hora de Syowa"}, {"Tahiti", "hora de Tahiti"}, {"Taipei", "hora normal de Taiwan"}, {"Tajikistan", "hora de Tajikistan"}, {"Tokelau", "hora de Tokelau"}, {"Tonga", "hora normal de Tonga"}, {"Truk", "hora de Chuuk"}, {"Turkmenistan", "hora normal de Turkmenistan"}, {"Tuvalu", "hora de Tuvalu"}, {"Uruguay", "hora normal de Uruguay"}, {"Uzbekistan", "hora normal de Uzbekistan"}, {"Vanuatu", "hora normal de Vanuatu"}, {"Venezuela", "hora de Venezuela"}, {"Vladivostok", "hora normal de Vladivostok"}, {"Volgograd", "hora normal de Volgograd"}, {"Vostok", "hora de Vostok"}, {"Wake", "hora del Insula Wake"}, {"Wallis", "hora de Wallis e Futuna"}, {"Yakutsk", "hora normal de Yakutsk"}, {"Yekaterinburg", "hora normal de Ekaterinburg"}, {"Yukon", "hora de Yukon"}},
		{{"Acre", "hora estive de Acre"}, {"Alaska", "hora estive de Alaska"}, {"Amazon", "hora estive de Amazonia"}, {"America_Central", "hora estive central"}, {"America_Eastern", "hora estive del est"}, {"America_Mountain", "hora estive del montanias"}, {"America_Pacific", "hora estive pacific"}, {"Apia", "hora estive de Samoa"}, {"Arabian", "hora estive arabe"}, {"Argentina", "hora estive de Argentina"}, {"Argentina_Western", "hora estive de Argentina occidental"}, {"Armenia", "hora estive de Armenia"}, {"Atlantic", "hora estive atlantic"}, {"Australia_Central", "hora estive de Australia central"}, {"Australia_CentralWestern", "hora estive de Australia centro-occidental"}, {"Australia_Eastern", "hora estive de Australia oriental"}, {"Australia_Western", "hora estive de Australia occidental"}, {"Azerbaijan", "hora estive de Azerbeidzhan"}, {"Azores", "hora estive del Azores"}, {"Bangladesh", "hora estive de Bangladesh"}, {"Brasilia", "hora estive de Brasilia"}, {"Cape_Verde", "hora estive de Capo Verde"}, {"Chatham", "hora estive de Chatham"}, {"Chile", "hora estive de Chile"}, {"China", "hora estive de China"}, {"Colombia", "hora estive de Colombia"}, {"Cook", "hora estive del Insulas Cook"}, {"Cuba", "hora estive de Cuba"}, {"Easter", "hora estive del Insula de Pascha"}, {"Europe_Central", "hora estive de Europa central"}, {"Europe_Eastern", "hora estive de Europa oriental"}, {"Europe_Western", "hora estive de Europa occidental"}, {"Falkland", "hora estive del Insulas Falkland"}, {"Fiji", "hora estive de Fiji"}, {"Georgia", "hora estive de Georgia"}, {"Greenland", "hora estive de Groenlandia"}, {"Greenland_Eastern", "hora estive de Groenlandia oriental"}, {"Greenland_Western", "hora estive de Groenlandia occidental"}, {"Hawaii_Aleutian", "hora estive de Hawaii-Aleutianas"}, {"Hong_Kong", "hora estive de Hongkong"}, {"Hovd", "hora estive de Khovd"}, {"Iran", "hora estive de Iran"}, {"Irkutsk", "hora estive de Irkutsk"}, {"Israel", "hora estive de Israel"}, {"Japan", "hora estive de Japon"}, {"Korea", "hora estive de Corea"}, {"Krasnoyarsk", "hora estive de Krasnoyarsk"}, {"Lord_Howe", "hora estive de Lord Howe"}, {"Magadan", "hora estive de Magadan"}, {"Mauritius", "hora estive de Mauritio"}, {"Mexico_Pacific", "hora estive del Pacifico mexican"}, {"Mongolia", "hora estive de Ulan Bator"}, {"Moscow", "hora estive de Moscova"}, {"New_Caledonia", "hora estive de Nove Caledonia"}, {"New_Zealand", "hora estive de Nove Zelanda"}, {"Newfoundland", "hora estive de Terranova"}, {"Norfolk", "hora estive del Insula Norfolk"}, {"Noronha", "hora estive de Fernando de Noronha"}, {"Novosibirsk", "hora estive de Novosibirsk"}, {"Omsk", "hora estive de Omsk"}, {"Pakistan", "hora estive de Pakistan"}, {"Paraguay", "hora estive de Paraguay"}, {"Peru", "hora estive de Peru"}, {"Philippines", "hora estive del Philippinas"}, {"Pierre_Miquelon", "hora estive de Saint-Pierre e Miquelon"}, {"Sakhalin", "hora estive de Sachalin"}, {"Samoa", "hora estive de Samoa american"}, {"Taipei", "hora estive de Taiwan"}, {"Tonga", "hora estive de Tonga"}, {"Turkmenistan", "hora estive de Turkmenistan"}, {"Uruguay", "hora estive de Uruguay"}, {"Uzbekistan", "hora estive de Uzbekistan"}, {"Vanuatu", "hora estive de Vanuatu"}, {"Vladivostok", "hora estive de Vladivostok"}, {"Volgograd", "hora estive de Volgograd"}, {"Yakutsk", "hora estive de Yakutsk"}, {"Yekaterinburg", "hora estive de Ekaterinburg"}},
	},
	{
		{{"Acre", "Waktu Standar Acre"}, {"Afghanistan", "Waktu Afganistan"}, {"Africa_Central", "Waktu Afrika Tengah"}, {"Africa_Eastern", "Waktu Afrika Timur"}, {"Africa_Southern", "Waktu Standar Afrika Selatan"}, {"Africa_Western", "Waktu Afrika Barat"}, {"Alaska", "Waktu Standar Alaska"}, {"Almaty", "Waktu Standar Almaty"}, {"Amazon", "Waktu Standar Amazon"}, {"America_Central", "Waktu Standar Tengah"}, {"America_Eastern", "Waktu Standar Timur"}, {"America_Mountain", "Waktu Standar Pegunungan"}, {"America_Pacific", "Waktu Standar Pasifik"}, {"Anadyr", "Waktu Standar Anadyr"}, {"Apia", "Waktu Standar Apia"}, {"Aqtau", "Waktu Standar Aqtau"}, {"Aqtobe", "Waktu Standar Aqtobe"}, {"Arabian", "Waktu Standar Arab"}, {"Argentina", "Waktu Standar Argentina"}, {"Argentina_Western", "Waktu Standar Argentina Bagian Barat"}, {"Armenia", "Waktu Standar Armenia"}, {"Atlantic", "Waktu Standar Atlantik"}, {"Australia_Central", "Waktu Standar Tengah Australia"}, {"Australia_CentralWestern", "Waktu Standar Barat Tengah Australia"}, {"Australia_Eastern", "Waktu Standar Timur Australia"}, {"Australia_Western", "Waktu Standar Barat Australia"}, {"Azerbaijan", "Waktu Standar Azerbaijan"}, {"Azores", "Waktu Standar Azores"}, {"Bangladesh", "Waktu Standar Bangladesh"}, {"Bhutan", "Waktu Bhutan"}, {"Bolivia", "Waktu Bolivia"}, {"Brasilia", "Waktu Standar Brasil"}, {"Brunei", "Waktu Brunei Darussalam"}, {"Cape_Verde", "Waktu Standar Tanjung Verde"}, {"Casey", "Waktu Casey"}, {"Chamorro", "Waktu Standar Chamorro"}, {"Chatham", "Waktu Standar Chatham"}, {"Chile", "Waktu Standar Cile"}, {"China", "Waktu Standar Tiongkok"}, {"Christmas", "Waktu Pulau Natal"}, {"Cocos", "Waktu Kepulauan Cocos"}, {"Colombia", "Waktu Standar Kolombia"}, {"Cook", "Waktu Standar Kep. Cook"}, {"Cuba", "Waktu Standar Kuba"}, {"Davis", "Waktu Davis"}, {"DumontDUrville", "Waktu Dumont-d’Urville"}, {"East_Timor", "Waktu Timor Leste"}, {"Easter", "Waktu Standar Pulau Paskah"}, {"Ecuador", "Waktu Ekuador"}, {"Europe_Central", "Waktu Standar Eropa Tengah"}, {"Europe_Eastern", "Waktu Standar Eropa Timur"}, {"Europe_Further_Eastern", "Waktu Eropa Timur Jauh"}, {"Europe_Western", "Waktu Standar Eropa Barat"}, {"Falkland", "Waktu Standar Kepulauan Falkland"}, {"Fiji", "Waktu Standar Fiji"}, {"French_Guiana", "Waktu Guyana Prancis"}, {"French_Southern", "Waktu Wilayah Selatan dan Antarktika Prancis"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Waktu Galapagos"}, {"Gambier", "Waktu Gambier"}, {"Georgia", "Waktu Standar Georgia"}, {"Gilbert_Islands", "Waktu Kep. Gilbert"}, {"Greenland_Eastern", "Waktu Standar Greenland Timur"}, {"Greenland_Western", "Waktu Standar Greenland Barat"}, {"Guam", "Waktu Guam"}, {"Gulf", "Waktu Standar Teluk"}, {"Guyana", "Waktu Guyana"}, {"Hawaii", "Waktu Standar Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Waktu Standar Hawaii-Aleutian"}, {"Hong_Kong", "Waktu Standar Hong Kong"}, {"Hovd", "Waktu Standar Hovd"}, {"India", "Waktu India"}, {"Indian_Ocean", "Waktu Samudera Hindia"}, {"Indochina", "Waktu Indochina"}, {"Indonesia_Central", "Waktu Indonesia Tengah"}, {"Indonesia_Eastern", "Waktu Indonesia Timur"}, {"Indonesia_Western", "Waktu Indonesia Barat"}, {"Iran", "Waktu Standar Iran"}, {"Irkutsk", "Waktu Standar Irkutsk"}, {"Israel", "Waktu Standar Israel"}, {"Japan", "Waktu Standar Jepang"}, {"Kamchatka", "Waktu Standar Petropavlovsk-Kamchatsky"}, {"Kazakhstan", "Waktu Kazakhstan"}, {"Kazakhstan_Eastern", "Waktu Kazakhstan Timur"}, {"Kazakhstan_Western", "Waktu Kazakhstan Barat"}, {"Korea", "Waktu Standar Korea"}, {"Kosrae", "Waktu Kosrae"}, {"Krasnoyarsk", "Waktu Standar Krasnoyarsk"}, {"Kyrgystan", "Waktu Kirgizstan"}, {"Lanka", "Waktu Lanka"}, {"Line_Islands", "Waktu Kep. Line"}, {"Lord_Howe", "Waktu Standar Lord Howe"}, {"Macau", "Waktu Standar Makau"}, {"Magadan", "Waktu Standar Magadan"}, {"Malaysia", "Waktu Malaysia"}, {"Maldives", "Waktu Maladewa"}, {"Marquesas", "Waktu Marquesas"}, {"Marshall_Islands", "Waktu Kep. Marshall"}, {"Mauritius", "Waktu Standar Mauritius"}, {"Mawson", "Waktu Mawson"}, {"Mexico_Pacific", "Waktu Standar Pasifik Meksiko"}, {"Mongolia", "Waktu Standar Ulan Bator"}, {"Moscow", "Waktu Standar Moskow"}, {"Myanmar", "Waktu Myanmar"}, {"Nauru", "Waktu Nauru"}, {"Nepal", "Waktu Nepal"}, {"New_Caledonia", "Waktu Standar Kaledonia Baru"}, {"New_Zealand", "Waktu Standar Selandia Baru"}, {"Newfoundland", "Waktu Standar Newfoundland"}, {"Niue", "Waktu Niue"}, {"Norfolk", "Waktu Standar Pulau Norfolk"}, {"Noronha", "Waktu Standar Fernando de Noronha"}, {"North_Mariana", "Waktu Kep. Mariana Utara"}, {"Novosibirsk", "Waktu Standar Novosibirsk"}, {"Omsk", "Waktu Standar Omsk"}, {"Pakistan", "Waktu Standar Pakistan"}, {"Palau", "Waktu Palau"}, {"Papua_New_Guinea", "Waktu Papua Nugini"}, {"Paraguay", "Waktu Standar Paraguay"}, {"Peru", "Waktu Standar Peru"}, {"Philippines", "Waktu Standar Filipina"}, {"Phoenix_Islands", "Waktu Kepulauan Phoenix"}, {"Pierre_Miquelon", "Waktu Standar Saint Pierre dan Miquelon"}, {"Pitcairn", "Waktu Pitcairn"}, {"Ponape", "Waktu Ponape"}, {"Pyongyang", "Waktu Pyongyang"}, {"Qyzylorda", "Waktu Standar Qyzylorda"}, {"Reunion", "Waktu Reunion"}, {"Rothera", "Waktu Rothera"}, {"Sakhalin", "Waktu Standar Sakhalin"}, {"Samara", "Waktu Standar Samara"}, {"Samoa", "Waktu Standar Samoa"}, {"Seychelles", "Waktu Seychelles"}, {"Singapore", "Waktu Standar Singapura"}, {"Solomon", "Waktu Kepulauan Solomon"}, {"South_Georgia", "Waktu Georgia Selatan"}, {"Suriname", "Waktu Suriname"}, {"Syowa", "Waktu Syowa"}, {"Tahiti", "Waktu Tahiti"}, {"Taipei", "Waktu Standar Taipei"}, {"Tajikistan", "Waktu Tajikistan"}, {"Tokelau", "Waktu Tokelau"}, {"Tonga", "Waktu Standar Tonga"}, {"Truk", "Waktu Chuuk"}, {"Turkey", "Waktu Standar Turki"}, {"Turkmenistan", "Waktu Standar Turkmenistan"}, {"Tuvalu", "Waktu Tuvalu"}, {"Uruguay", "Waktu Standar Uruguay"}, {"Uzbekistan", "Waktu Standar Uzbekistan"}, {"Vanuatu", "Waktu Standar Vanuatu"}, {"Venezuela", "Waktu Venezuela"}, {"Vladivostok", "Waktu Standar Vladivostok"}, {"Volgograd", "Waktu Standar Volgograd"}, {"Vostok", "Waktu Vostok"}, {"Wake", "Waktu Kepulauan Wake"}, {"Wallis", "Waktu Wallis dan Futuna"}, {"Yakutsk", "Waktu Standar Yakutsk"}, {"Yekaterinburg", "Waktu Standar Yekaterinburg"}, {"Yukon", "Waktu Yukon"}},
		{{"Acre", "Waktu Musim Panas Acre"}, {"Alaska", "Waktu Musim Panas Alaska"}, {"Almaty", "Waktu Musim Panas Almaty"}, {"Amazon", "Waktu Musim Panas Amazon"}, {"America_Central", "Waktu Musim Panas Tengah"}, {"America_Eastern", "Waktu Musim Panas Timur"}, {"America_Mountain", "Waktu Musim Panas Pegunungan"}, {"America_Pacific", "Waktu Musim Panas Pasifik"}, {"Anadyr", "Waktu Musim Panas Anadyr"}, {"Apia", "Waktu Musim Panas Apia"}, {"Aqtau", "Waktu Musim Panas Aqtau"}, {"Aqtobe", "Waktu Musim Panas Aqtobe"}, {"Arabian", "Waktu Musim Panas Arab"}, {"Argentina", "Waktu Musim Panas Argentina"}, {"Argentina_Western", "Waktu Musim Panas Argentina Bagian Barat"}, {"Armenia", "Waktu Musim Panas Armenia"}, {"Atlantic", "Waktu Musim Panas Atlantik"}, {"Australia_Central", "Waktu Musim Panas Tengah Australia"}, {"Australia_CentralWestern", "Waktu Musim Panas Barat Tengah Australia"}, {"Australia_Eastern", "Waktu Musim Panas Timur Australia"}, {"Australia_Western", "Waktu Musim Panas Barat Australia"}, {"Azerbaijan", "Waktu Musim Panas Azerbaijan"}, {"Azores", "Waktu Musim Panas Azores"}, {"Bangladesh", "Waktu Musim Panas Bangladesh"}, {"Brasilia", "Waktu Musim Panas Brasil"}, {"Cape_Verde", "Waktu Musim Panas Tanjung Verde"}, {"Chatham", "Waktu Musim Panas Chatham"}, {"Chile", "Waktu Musim Panas Cile"}, {"China", "Waktu Musim Panas Tiongkok"}, {"Colombia", "Waktu Musim Panas Kolombia"}, {"Cook", "Waktu Tengah Musim Panas Kep. Cook"}, {"Cuba", "Waktu Musim Panas Kuba"}, {"Easter", "Waktu Musim Panas Pulau Paskah"}, {"Europe_Central", "Waktu Musim Panas Eropa Tengah"}, {"Europe_Eastern", "Waktu Musim Panas Eropa Timur"}, {"Europe_Western", "Waktu Musim Panas Eropa Barat"}, {"Falkland", "Waktu Musim Panas Kepulauan Falkland"}, {"Fiji", "Waktu Musim Panas Fiji"}, {"Georgia", "Waktu Musim Panas Georgia"}, {"Greenland_Eastern", "Waktu Musim Panas Greenland Timur"}, {"Greenland_Western", "Waktu Musim Panas Greenland Barat"}, {"Hawaii_Aleutian", "Waktu Musim Panas Hawaii-Aleutian"}, {"Hong_Kong", "Waktu Musim Panas Hong Kong"}, {"Hovd", "Waktu Musim Panas Hovd"}, {"Iran", "Waktu Musim Panas Iran"}, {"Irkutsk", "Waktu Musim Panas Irkutsk"}, {"Israel", "Waktu Musim Panas Israel"}, {"Japan", "Waktu Musim Panas Jepang"}, {"Kamchatka", "Waktu Musim Panas Petropavlovsk-Kamchatski"}, {"Korea", "Waktu Musim Panas Korea"}, {"Krasnoyarsk", "Waktu Musim Panas Krasnoyarsk"}, {"Lord_Howe", "Waktu Musim Panas Lord Howe"}, {"Macau", "Waktu Musim Panas Makau"}, {"Magadan", "Waktu Musim Panas Magadan"}, {"Mauritius", "Waktu Musim Panas Mauritius"}, {"Mexico_Pacific", "Waktu Musim Panas Pasifik Meksiko"}, {"Mongolia", "Waktu Musim Panas Ulan Bator"}, {"Moscow", "Waktu Musim Panas Moskow"}, {"New_Caledonia", "Waktu Musim Panas Kaledonia Baru"}, {"New_Zealand", "Waktu Musim Panas Selandia Baru"}, {"Newfoundland", "Waktu Musim Panas Newfoundland"}, {"Norfolk", "Waktu Musim Panas Pulau Norfolk"}, {"Noronha", "Waktu Musim Panas Fernando de Noronha"}, {"Novosibirsk", "Waktu Musim Panas Novosibirsk"}, {"Omsk", "Waktu Musim Panas Omsk"}, {"Pakistan", "Waktu Musim Panas Pakistan"}, {"Paraguay", "Waktu Musim Panas Paraguay"}, {"Peru", "Waktu Musim Panas Peru"}, {"Philippines", "Waktu Musim Panas Filipina"}, {"Pierre_Miquelon", "Waktu Musim Panas Saint Pierre dan Miquelon"}, {"Qyzylorda", "Waktu Musim Panas Qyzylorda"}, {"Sakhalin", "Waktu Musim Panas Sakhalin"}, {"Samara", "Waktu Musim Panas Samara"}, {"Samoa", "Waktu Musim Panas Samoa"}, {"Taipei", "Waktu Musim Panas Taipei"}, {"Tonga", "Waktu Musim Panas Tonga"}, {"Turkey", "Waktu Musim Panas Turki"}, {"Turkmenistan", "Waktu Musim Panas Turkmenistan"}, {"Uruguay", "Waktu Musim Panas Uruguay"}, {"Uzbekistan", "Waktu Musim Panas Uzbekistan"}, {"Vanuatu", "Waktu Musim Panas Vanuatu"}, {"Vladivostok", "Waktu Musim Panas Vladivostok"}, {"Volgograd", "Waktu Musim Panas Volgograd"}, {"Yakutsk", "Waktu Musim Panas Yakutsk"}, {"Yekaterinburg", "Waktu Musim Panas Yekaterinburg"}},
	},
	{
		{{"Alaska", "alaskan standard témpor"}, {"America_Central", "american central standard témpor"}, {"America_Eastern", "american oriental standard témpor"}, {"America_Mountain", "american montan standard témpor"}, {"America_Pacific", "american pacific standard témpor"}, {"Atlantic", "american atlantic standard témpor"}, {"Azores", "témpor standard del Azores"}, {"Christmas", "témpor del Insul Christmas"}, {"Cocos", "témpor del Insules Cocos"}, {"Cook", "témpor standard del Insules Cook"}, {"Cuba", "cuban standard témpor"}, {"Europe_Central", "central europan standard témpor"}, {"Europe_Eastern", "ost-europan standard témpor"}, {"Europe_Western", "west-europan standard témpor"}, {"Falkland", "témpor standard del Insules Falkland"}, {"GMT", "témpore medial de Greenwich"}, {"Gilbert_Islands", "témpor del Insules Gilbert"}, {"Greenland_Eastern", "témpor standard del oriental Greenland"}, {"Greenland_Western", "témpor standard del occidental Greenland"}, {"Hawaii", "témpor standard de Hawai e Aleutes"}, {"Hawaii_Aleutian", "témpor standard de Hawai e Aleutes"}, {"Line_Islands", "témpor del Insules Line"}, {"Lord_Howe", "témpor standard del Insul Lord-Howe"}, {"Marquesas", "témpor del Marquesas"}, {"Marshall_Islands", "témpor del Insules Marshall"}, {"Mexico_Pacific", "mexican pacific standard témpor"}, {"Newfoundland", "témpor standard de Newfoundland"}, {"Norfolk", "témpor standard del Insul Norfolk"}, {"Phoenix_Islands", "témpor del Insules Fénix"}, {"Solomon", "témpor del Insules Solomon"}, {"Wake", "témpor del Insul Wake"}, {"Yukon", "témpor de Yukon"}},
		{{"Alaska", "alaskan estival témpor"}, {"America_Central", "american central estival témpor"}, {"America_Eastern", "american oriental estival témpor"}, {"America_Mountain", "american montan estival témpor"}, {"America_Pacific", "american pacific estival témpor"}, {"Atlantic", "american atlantic estival témpor"}, {"Azores", "témpor estival del Azores"}, {"Cook", "témpor demíestival del Insules Cook"}, {"Cuba", "cuban estival témpor"}, {"Europe_Central", "central europan estival témpor"}, {"Europe_Eastern", "ost-europan estival témpor"}, {"Europe_Western", "west-europan estival témpor"}, {"Falkland", "témpor estival del Insules Falkland"}, {"Greenland_Eastern", "témpor estival del oriental Greenland"}, {"Greenland_Western", "témpor estival del occidental Greenland"}, {"Hawaii_Aleutian", "témpor estival de Hawai e Aleutes"}, {"Lord_Howe", "témpor estival del Insul Lord-Howe"}, {"Mexico_Pacific", "mexican pacific estival témpor"}, {"Newfoundland", "témpor estival de Newfoundland"}, {"Norfolk", "témpor estival del Insul Norfolk"}},
	},
	{
		{{"Afghanistan", "Oge Afghanistan"}, {"Africa_Central", "Oge Etiti Afrịka"}, {"Africa_Eastern", "Oge Mpaghara Ọwụwa Anyanwụ Afrịka"}, {"Africa_Southern", "Oge Izugbe Mpaghara Mgbada Ugwu Afrịka"}, {"Africa_Western", "Oge Mpaghara Ọdịda Anyanwụ Afrịka"}, {"Alaska", "Oge Izugbe Alaska"}, {"Amazon", "Oge Izugbe Amazon"}, {"America_Central", "Oge Izugbe Mpaghara Etiti"}, {"America_Eastern", "Oge Izugbe Mpaghara Ọwụwa Anyanwụ"}, {"America_Mountain", "Oge Izugbe Mpaghara Ugwu"}, {"America_Pacific", "Oge Izugbe Mpaghara Pacific"}, {"Apia", "Oge Izugbe Apia"}, {"Arabian", "Oge Izugbe Arab"}, {"Argentina", "Oge Izugbe Argentina"}, {"Argentina_Western", "Oge Izugbe Mpaghara Ọdịda Anyanwụ Argentina"}, {"Armenia", "Oge Izugbe Armenia"}, {"Atlantic", "Oge Izugbe Mpaghara Atlantic"}, {"Australia_Central", "Oge Izugbe Etiti Australia"}, {"Australia_CentralWestern", "Oge Izugbe Mpaghara Ọdịda Anyanwụ Etiti Australia"}, {"Australia_Eastern", "Oge Izugbe Mpaghara Ọwụwa Anyanwụ Australia"}, {"Australia_Western", "Oge Izugbe Mpaghara Ọdịda Anyanwụ Australia"}, {"Azerbaijan", "Oge Izugbe Azerbaijan"}, {"Azores", "Oge Izugbe Azores"}, {"Bangladesh", "Oge Izugbe Bangladesh"}, {"Bhutan", "Oge Bhutan"}, {"Bolivia", "Oge Bolivia"}, {"Brasilia", "Oge Izugbe Brasilia"}, {"Brunei", "Oge Brunei Darussalam"}, {"Cape_Verde", "Oge Izugbe Cape Verde"}, {"Chamorro", "Oge Izugbe Chamorro"}, {"Chatham", "Oge Izugbe Chatham"}, {"Chile", "Oge Izugbe Chile"}, {"China", "Oge Izugbe China"}, {"Christmas", "Oge Ekeresimesi Island"}, {"Cocos", "Oge Cocos Islands"}, {"Colombia", "Oge Izugbe Columbia"}, {"Cook", "Oge Izugbe Cook Islands"}, {"Cuba", "Oge Izugbe Cuba"}, {"Davis", "Oge Davis"}, {"DumontDUrville", "Oge Dumont-d’Urville"}, {"East_Timor", "Oge Mpaghara Ọwụwa Anyanwụ Timor"}, {"Easter", "Oge Izugbe Mpaghara Ọwụwa Anyanwụ Island"}, {"Ecuador", "Oge Ecuador"}, {"Europe_Central", "Oge Izugbe Mpaghara Etiti Europe"}, {"Europe_Eastern", "Oge Izugbe Mpaghara Ọwụwa Anyanwụ Europe"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Oge Izugbe Mpaghara Ọdịda Anyanwụ Europe"}, {"Falkland", "Oge Izugbe Falkland Islands"}, {"Fiji", "Oge Izugbe Fiji"}, {"French_Guiana", "Oge French Guiana"}, {"French_Southern", "Oge French Southern & Antarctic"}, {"GMT", "Oge Mpaghara Greemwich Mean"}, {"Galapagos", "Oge Galapagos"}, {"Gambier", "Oge Gambier"}, {"Georgia", "Oge Izugbe Georgia"}, {"Gilbert_Islands", "Oge Gilbert Islands"}, {"Greenland_Eastern", "Oge Izugbe Mpaghara Ọwụwa Anyanwụ Greenland"}, {"Greenland_Western", "Oge Izugbe Mpaghara Ọdịda Anyanwụ Greenland"}, {"Gulf", "Oge Izugbe Gulf"}, {"Guyana", "Oge Guyana"}, {"Hawaii", "Oge Izugbe Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Oge Izugbe Hawaii-Aleutian"}, {"Hong_Kong", "Oge Izugbe Hong Kong"}, {"Hovd", "Oge Izugbe Hovd"}, {"India", "Oge Izugbe India"}, {"Indian_Ocean", "Oge Osimiri India"}, {"Indochina", "Oge Indochina"}, {"Indonesia_Central", "Oge Etiti Indonesia"}, {"Indonesia_Eastern", "Oge Mpaghara Ọwụwa Anyanwụ Indonesia"}, {"Indonesia_Western", "Oge Mpaghara Ọdịda Anyanwụ Indonesia"}, {"Iran", "Oge Izugbe Iran"}, {"Irkutsk", "Oge Izugbe Irkutsk"}, {"Israel", "Oge Izugbe Israel"}, {"Japan", "Oge Izugbe Japan"}, {"Kazakhstan", "Oge Kazakhstan"}, {"Kazakhstan_Eastern", "Oge Mpaghara Ọwụwa Anyanwụ Kazakhstan"}, {"Kazakhstan_Western", "Oge Mpaghara Ọdịda Anyanwụ Kazakhstan"}, {"Korea", "Oge Izugbe Korea"}, {"Kosrae", "Oge Kosrae"}, {"Krasnoyarsk", "Oge Izugbe Krasnoyarsk"}, {"Kyrgystan", "Oge Kyrgyzstan"}, {"Line_Islands", "Oge Line Islands"}, {"Lord_Howe", "Oge Izugbe Lord Howe"}, {"Magadan", "Oge Izugbe Magadan"}, {"Malaysia", "Oge Malaysia"}, {"Maldives", "Oge Maldives"}, {"Marquesas", "Oge Marquesas"}, {"Marshall_Islands", "Oge Marshall Islands"}, {"Mauritius", "Oge Izugbe Mauritius"}, {"Mawson", "Oge Mawson"}, {"Mexico_Pacific", "Oge Izugbe Mexican Pacific"}, {"Mongolia", "Oge Izugbe Ulaanbaatar"}, {"Moscow", "Oge Izugbe Moscow"}, {"Myanmar", "Oge Myanmar"}, {"Nauru", "Oge Nauru"}, {"Nepal", "Oge Nepal"}, {"New_Caledonia", "Oge Izugbe New Caledonia"}, {"New_Zealand", "Oge Izugbe New Zealand"}, {"Newfoundland", "Oge Izugbe Newfoundland"}, {"Niue", "Oge Niue"}, {"Norfolk", "Oge Izugbe Norfolk Island"}, {"Noronha", "Oge Izugbe Fernando de Noronha"}, {"Novosibirsk", "Oge Izugbe Novosibirsk"}, {"Omsk", "Oge Izugbe Omsk"}, {"Pakistan", "Oge Izugbe Pakistan"}, {"Palau", "Oge Palau"}, {"Papua_New_Guinea", "Oge Papua New Guinea"}, {"Paraguay", "Oge Izugbe Paraguay"}, {"Peru", "Oge Izugbe Peru"}, {"Philippines", "Oge Izugbe Philippine"}, {"Phoenix_Islands", "Oge Phoenix Islands"}, {"Pierre_Miquelon", "Oge Izugbe St. Pierre & Miquelon"}, {"Pitcairn", "Oge Pitcairn"}, {"Ponape", "Oge Ponape"}, {"Pyongyang", "Oge Pyongyang"}, {"Reunion", "Oge Réunion"}, {"Rothera", "Oge Rothera"}, {"Sakhalin", "Oge Izugbe Sakhalin"}, {"Samoa", "Oge Izugbe Samoa"}, {"Seychelles", "Oge Seychelles"}, {"Singapore", "Oge Izugbe Singapore"}, {"Solomon", "Oge Solomon Islands"}, {"South_Georgia", "Oge South Georgia"}, {"Suriname", "Oge Suriname"}, {"Syowa", "Oge Syowa"}, {"Tahiti", "Oge Tahiti"}, {"Taipei", "Oge Izugbe Taipei"}, {"Tajikistan", "Oge Tajikistan"}, {"Tokelau", "Oge Tokelau"}, {"Tonga", "Oge Izugbe Tonga"}, {"Truk", "Oge Chuuk"}, {"Turkmenistan", "Oge Izugbe Turkmenist"}, {"Tuvalu", "Oge Tuvalu"}, {"Uruguay", "Oge Izugbe Uruguay"}, {"Uzbekistan", "Oge Izugbe Uzbekist"}, {"Vanuatu", "Oge Izugbe Vanuatu"}, {"Venezuela", "Oge Venezuela"}, {"Vladivostok", "Oge Izugbe Vladivostok"}, {"Volgograd", "Oge Izugbe Volgograd"}, {"Vostok", "Oge Vostok"}, {"Wake", "Oge Wake Island"}, {"Wallis", "Oge Wallis & Futuna"}, {"Yakutsk", "Oge Izugbe Yakutsk"}, {"Yekaterinburg", "Oge Izugbe Yekaterinburg"}, {"Yukon", "Oge Yukon"}},
		{{"Alaska", "Oge Ihe Alaska"}, {"Amazon", "Oge Okpomọkụ Amazon"}, {"America_Central", "Oge Ihe Mpaghara Etiti"}, {"America_Eastern", "Oge Ihe Mpaghara Ọwụwa Anyanwụ"}, {"America_Mountain", "Oge Ihe Mpaghara Ugwu"}, {"America_Pacific", "Oge Ihe Mpaghara Pacific"}, {"Apia", "Oge Ihe Apia"}, {"Arabian", "Oge Ihe Arab"}, {"Argentina", "Oge Okpomọkụ Argentina"}, {"Argentina_Western", "Oge Okpomọkụ Mpaghara Ọdịda Anyanwụ Argentina"}, {"Armenia", "Oge Okpomọkụ Armenia"}, {"Atlantic", "Oge Ihe Mpaghara Atlantic"}, {"Australia_Central", "Oge Ihe Etiti Australia"}, {"Australia_CentralWestern", "Oge Ihe Mpaghara Ọdịda Anyanwụ Etiti Australia"}, {"Australia_Eastern", "Oge Ihe Mpaghara Ọwụwa Anyanwụ Australia"}, {"Australia_Western", "Oge Ihe Mpaghara Ọdịda Anyanwụ Australia"}, {"Azerbaijan", "Oge Okpomọkụ Azerbaijan"}, {"Azores", "Oge Okpomọkụ Azores"}, {"Bangladesh", "Oge Okpomọkụ Bangladesh"}, {"Brasilia", "Oge Okpomọkụ Brasilia"}, {"Cape_Verde", "Oge Okpomọkụ Cape Verde"}, {"Chatham", "Oge Ihe Chatham"}, {"Chile", "Oge Okpomọkụ Chile"}, {"China", "Oge Ihe China"}, {"Colombia", "Oge Okpomọkụ Columbia"}, {"Cook", "Oge Ọkara Okpomọkụ Cook Islands"}, {"Cuba", "Oge Ihe Mpaghara Cuba"}, {"Easter", "Oge Okpomọkụ Mpaghara Ọwụwa Anyanwụ Island"}, {"Europe_Central", "Oge Okpomọkụ Mpaghara Etiti Europe"}, {"Europe_Eastern", "Oge Okpomọkụ Mpaghara Ọwụwa Anyanwụ Europe"}, {"Europe_Western", "Oge Okpomọkụ Mpaghara Ọdịda Anyanwụ Europe"}, {"Falkland", "Oge Okpomọkụ Falkland Islands"}, {"Fiji", "Oge Okpomọkụ Fiji"}, {"Georgia", "Oge Okpomọkụ Georgia"}, {"Greenland_Eastern", "Oge Okpomọkụ Mpaghara Ọwụwa Anyanwụ Greenland"}, {"Greenland_Western", "Oge Okpomọkụ Mpaghara Ọdịda Anyanwụ Greenland"}, {"Hawaii_Aleutian", "Oge Ihe Hawaii-Aleutian"}, {"Hong_Kong", "Oge Okpomọkụ Hong Kong"}, {"Hovd", "Oge Okpomọkụ Hovd"}, {"Iran", "Oge Ihe Iran"}, {"Irkutsk", "Oge Okpomọkụ Irkutsk"}, {"Israel", "Oge Ihe Israel"}, {"Japan", "Oge Ihe Japan"}, {"Korea", "Oge Ihe Korea"}, {"Krasnoyarsk", "Oge Okpomọkụ Krasnoyarsk"}, {"Lord_Howe", "Oge Ihe Lord Howe"}, {"Magadan", "Oge Okpomọkụ Magadan"}, {"Mauritius", "Oge Okpomọkụ Mauritius"}, {"Mexico_Pacific", "Oge Ihe Mexican Pacific"}, {"Mongolia", "Oge Okpomọkụ Ulaanbaatar"}, {"Moscow", "Oge Okpomọkụ Moscow"}, {"New_Caledonia", "Oge Okpomọkụ New Caledonia"}, {"New_Zealand", "Oge Ihe New Zealand"}, {"Newfoundland", "Oge Ihe Newfoundland"}, {"Norfolk", "Oge Okpomọkụ Norfolk Island"}, {"Noronha", "Oge Okpomọkụ Fernando de Noronha"}, {"Novosibirsk", "Oge Okpomọkụ Novosibirsk"}, {"Omsk", "Oge Okpomọkụ Omsk"}, {"Pakistan", "Oge Okpomọkụ Pakistan"}, {"Paraguay", "Oge Okpomọkụ Paraguay"}, {"Peru", "Oge Okpomọkụ Peru"}, {"Philippines", "Oge Okpomọkụ Philippine"}, {"Pierre_Miquelon", "Oge Ihe St. Pierre & Miquelon"}, {"Sakhalin", "Oge Okpomọkụ Sakhalin"}, {"Samoa", "Oge Ihe Samoa"}, {"Taipei", "Oge Ihe Taipei"}, {"Tonga", "Oge Okpomọkụ Tonga"}, {"Turkmenistan", "Oge Okpomọkụ Turkmenist"}, {"Uruguay", "Oge Okpomọkụ Uruguay"}, {"Uzbekistan", "Oge Okpomọkụ Uzbekist"}, {"Vanuatu", "Oge Okpomọkụ Vanuatu"}, {"Vladivostok", "Oge Okpomọkụ Vladivostok"}, {"Volgograd", "Oge Okpomọkụ Volgograd"}, {"Yakutsk", "Oge Okpomọkụ Yakutsk"}, {"Yekaterinburg", "Oge Okpomọkụ Yekaterinburg"}},
	},
	{
		{{"GMT", "ꋧꃅꎕꏦꄮꈉ"}},
		{},
	},
	{
		{{"Afghanistan", "Afganistantími"}, {"Africa_Central", "Mið-Afríkutími"}, {"Africa_Eastern", "Austur-Afríkutími"}, {"Africa_Southern", "Suður-Afríkutími"}, {"Africa_Western", "Vestur-Afríkutími"}, {"Alaska", "Staðaltími í Alaska"}, {"Amazon", "Staðaltími á Amasónsvæðinu"}, {"America_Central", "Staðaltími í miðhluta Bandaríkjanna og Kanada"}, {"America_Eastern", "Staðaltími í austurhluta Bandaríkjanna og Kanada"}, {"America_Mountain", "Staðaltími í Klettafjöllum"}, {"America_Pacific", "Staðaltími á Kyrrahafssvæðinu"}, {"Anadyr", "Staðaltími í Anadyr"}, {"Apia", "Staðaltími í Apía"}, {"Arabian", "Staðaltími í Arabíu"}, {"Argentina", "Staðaltími í Argentínu"}, {"Argentina_Western", "Staðaltími í Vestur-Argentínu"}, {"Armenia", "Staðaltími í Armeníu"}, {"Atlantic", "Staðaltími á Atlantshafssvæðinu"}, {"Australia_Central", "Staðaltími í Mið-Ástralíu"}, {"Australia_CentralWestern", "Staðaltími í miðvesturhluta Ástralíu"}, {"Australia_Eastern", "Staðaltími í Austur-Ástralíu"}, {"Australia_Western", "Staðaltími í Vestur-Ástralíu"}, {"Azerbaijan", "Staðaltími í Aserbaídsjan"}, {"Azores", "Staðaltími á Asóreyjum"}, {"Bangladesh", "Staðaltími í Bangladess"}, {"Bhutan", "Bútantími"}, {"Bolivia", "Bólivíutími"}, {"Brasilia", "Staðaltími í Brasilíu"}, {"Brunei", "Brúneitími"}, {"Cape_Verde", "Staðaltími á Grænhöfðaeyjum"}, {"Chamorro", "Chamorro-staðaltími"}, {"Chatham", "Staðaltími í Chatham"}, {"Chile", "Staðaltími í Síle"}, {"China", "Staðaltími í Kína"}, {"Christmas", "Jólaeyjartími"}, {"Cocos", "Kókoseyjatími"}, {"Colombia", "Staðaltími í Kólumbíu"}, {"Cook", "Staðaltími á Cooks-eyjum"}, {"Cuba", "Staðaltími á Kúbu"}, {"Davis", "Davis-tími"}, {"DumontDUrville", "Tími á Dumont-d’Urville"}, {"East_Timor", "Tími á Tímor-Leste"}, {"Easter", "Staðaltími á Páskaeyju"}, {"Ecuador", "Ekvadortími"}, {"Europe_Central", "Staðaltími í Mið-Evrópu"}, {"Europe_Eastern", "Staðaltími í Austur-Evrópu"}, {"Europe_Further_Eastern", "Staðartími Kalíníngrad"}, {"Europe_Western", "Staðaltími í Vestur-Evrópu"}, {"Falkland", "Staðaltími á Falklandseyjum"}, {"Fiji", "Staðaltími á Fídjíeyjum"}, {"French_Guiana", "Tími í Frönsku Gvæjana"}, {"French_Southern", "Tími á frönsku suðurhafssvæðum og Suðurskautslandssvæði"}, {"GMT", "Greenwich-staðaltími"}, {"Galapagos", "Galapagos-tími"}, {"Gambier", "Gambier-tími"}, {"Georgia", "Staðaltími í Georgíu"}, {"Gilbert_Islands", "Tími á Gilbert-eyjum"}, {"Greenland_Eastern", "Staðaltími á Austur-Grænlandi"}, {"Greenland_Western", "Staðaltími á Vestur-Grænlandi"}, {"Gulf", "Staðaltími við Persaflóa"}, {"Guyana", "Gvæjanatími"}, {"Hawaii", "Staðaltími á Havaí og Aleúta"}, {"Hawaii_Aleutian", "Staðaltími á Havaí og Aleúta"}, {"Hong_Kong", "Staðaltími í Hong Kong"}, {"Hovd", "Staðaltími í Hovd"}, {"India", "Indlandstími"}, {"Indian_Ocean", "Indlandshafstími"}, {"Indochina", "Indókínatími"}, {"Indonesia_Central", "Mið-Indónesíutími"}, {"Indonesia_Eastern", "Austur-Indónesíutími"}, {"Indonesia_Western", "Vestur-Indónesíutími"}, {"Iran", "Staðaltími í Íran"}, {"Irkutsk", "Staðaltími í Irkutsk"}, {"Israel", "Staðaltími í Ísrael"}, {"Japan", "Staðaltími í Japan"}, {"Kamchatka", "Staðaltími í Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Tími í Kasakstan"}, {"Kazakhstan_Eastern", "Tími í Austur-Kasakstan"}, {"Kazakhstan_Western", "Tími í Vestur-Kasakstan"}, {"Korea", "Staðaltími í Kóreu"}, {"Kosrae", "Kosrae-tími"}, {"Krasnoyarsk", "Staðaltími í Krasnoyarsk"}, {"Kyrgystan", "Kirgistan-tími"}, {"Line_Islands", "Línueyja-tími"}, {"Lord_Howe", "Staðaltími á Lord Howe-eyju"}, {"Magadan", "Staðaltími í Magadan"}, {"Malaysia", "Malasíutími"}, {"Maldives", "Maldíveyja-tími"}, {"Marquesas", "Tími á Markgreifafrúreyjum"}, {"Marshall_Islands", "Tími á Marshall-eyjum"}, {"Mauritius", "Staðaltími á Máritíus"}, {"Mawson", "Mawson-tími"}, {"Mexico_Pacific", "Staðaltími í Mexíkó á Kyrrahafssvæðinu"}, {"Mongolia", "Staðaltími í Úlan Bator"}, {"Moscow", "Staðaltími í Moskvu"}, {"Myanmar", "Mjanmar-tími"}, {"Nauru", "Nárú-tími"}, {"Nepal", "Nepaltími"}, {"New_Caledonia", "Staðaltími í Nýju-Kaledóníu"}, {"New_Zealand", "Staðaltími á Nýja-Sjálandi"}, {"Newfoundland", "Staðaltími á Nýfundnalandi"}, {"Niue", "Niue-tími"}, {"Norfolk", "Staðaltími á Norfolk-eyju"}, {"Noronha", "Staðaltími í Fernando de Noronha"}, {"Novosibirsk", "Staðaltími í Novosibirsk"}, {"Omsk", "Staðaltími í Omsk"}, {"Pakistan", "Staðaltími í Pakistan"}, {"Palau", "Palátími"}, {"Papua_New_Guinea", "Tími á Papúa Nýju-Gíneu"}, {"Paraguay", "Staðaltími í Paragvæ"}, {"Peru", "Staðaltími í Perú"}, {"Philippines", "Staðaltími á Filippseyjum"}, {"Phoenix_Islands", "Fönixeyjatími"}, {"Pierre_Miquelon", "Staðaltími á Sankti Pierre og Miquelon"}, {"Pitcairn", "Pitcairn-tími"}, {"Ponape", "Ponape-tími"}, {"Pyongyang", "Tími í Pjongjang"}, {"Reunion", "Réunion-tími"}, {"Rothera", "Rothera-tími"}, {"Sakhalin", "Staðaltími á Sakhalin"}, {"Samara", "Staðaltími í Samara"}, {"Samoa", "Staðaltími á Samóa"}, {"Seychelles", "Seychelles-eyjatími"}, {"Singapore", "Singapúrtími"}, {"Solomon", "Salómonseyjatími"}, {"South_Georgia", "Suður-Georgíutími"}, {"Suriname", "Súrinamtími"}, {"Syowa", "Syowa-tími"}, {"Tahiti", "Tahítí-tími"}, {"Taipei", "Staðaltími í Taipei"}, {"Tajikistan", "Tadsjíkistan-tími"}, {"Tokelau", "Tókelá-tími"}, {"Tonga", "Staðaltími á Tonga"}, {"Truk", "Chuuk-tími"}, {"Turkmenistan", "Staðaltími í Túrkmenistan"}, {"Tuvalu", "Túvalútími"}, {"Uruguay", "Staðaltími í Úrúgvæ"}, {"Uzbekistan", "Staðaltími í Úsbekistan"}, {"Vanuatu", "Staðaltími á Vanúatú"}, {"Venezuela", "Venesúelatími"}, {"Vladivostok", "Staðaltími í Vladivostok"}, {"Volgograd", "Staðaltími í Volgograd"}, {"Vostok", "Vostok-tími"}, {"Wake", "Tími á Wake-eyju"}, {"Wallis", "Tími á Wallis- og Fútúnaeyjum"}, {"Yakutsk", "Staðaltími í Yakutsk"}, {"Yekaterinburg", "Staðaltími í Yekaterinborg"}, {"Yukon", "Tími í Júkon"}},
		{{"Alaska", "Sumartími í Alaska"}, {"Amazon", "Sumartími á Amasónsvæðinu"}, {"America_Central", "Sumartími í miðhluta Bandaríkjanna og Kanada"}, {"America_Eastern", "Sumartími í austurhluta Bandaríkjanna og Kanada"}, {"America_Mountain", "Sumartími í Klettafjöllum"}, {"America_Pacific", "Sumartími á Kyrrahafssvæðinu"}, {"Anadyr", "Sumartími í Anadyr"}, {"Apia", "Sumartími í Apía"}, {"Arabian", "Sumartími í Arabíu"}, {"Argentina", "Sumartími í Argentínu"}, {"Argentina_Western", "Sumartími í Vestur-Argentínu"}, {"Armenia", "Sumartími í Armeníu"}, {"Atlantic", "Sumartími á Atlantshafssvæðinu"}, {"Australia_Central", "Sumartími í Mið-Ástralíu"}, {"Australia_CentralWestern", "Sumartími í miðvesturhluta Ástralíu"}, {"Australia_Eastern", "Sumartími í Austur-Ástralíu"}, {"Australia_Western", "Sumartími í Vestur-Ástralíu"}, {"Azerbaijan", "Sumartími í Aserbaídsjan"}, {"Azores", "Sumartími á Asóreyjum"}, {"Bangladesh", "Sumartími í Bangladess"}, {"Brasilia", "Sumartími í Brasilíu"}, {"Cape_Verde", "Sumartími á Grænhöfðaeyjum"}, {"Chatham", "Sumartími í Chatham"}, {"Chile", "Sumartími í Síle"}, {"China", "Sumartími í Kína"}, {"Colombia", "Sumartími í Kólumbíu"}, {"Cook", "Hálfsumartími á Cooks-eyjum"}, {"Cuba", "Sumartími á Kúbu"}, {"Easter", "Sumartími á Páskaeyju"}, {"Europe_Central", "Sumartími í Mið-Evrópu"}, {"Europe_Eastern", "Sumartími í Austur-Evrópu"}, {"Europe_Western", "Sumartími í Vestur-Evrópu"}, {"Falkland", "Sumartími á Falklandseyjum"}, {"Fiji", "Sumartími á Fídjíeyjum"}, {"Georgia", "Sumartími í Georgíu"}, {"Greenland_Eastern", "Sumartími á Austur-Grænlandi"}, {"Greenland_Western", "Sumartími á Vestur-Grænlandi"}, {"Hawaii_Aleutian", "Sumartími á Havaí og Aleúta"}, {"Hong_Kong", "Sumartími í Hong Kong"}, {"Hovd", "Sumartími í Hovd"}, {"Iran", "Sumartími í Íran"}, {"Irkutsk", "Sumartími í Irkutsk"}, {"Israel", "Sumartími í Ísrael"}, {"Japan", "Sumartími í Japan"}, {"Kamchatka", "Sumartími í Petropavlovsk-Kamchatski"}, {"Korea", "Sumartími í Kóreu"}, {"Krasnoyarsk", "Sumartími í Krasnoyarsk"}, {"Lord_Howe", "Sumartími á Lord Howe-eyju"}, {"Magadan", "Sumartími í Magadan"}, {"Mauritius", "Sumartími á Máritíus"}, {"Mexico_Pacific", "Sumartími í Mexíkó á Kyrrahafssvæðinu"}, {"Mongolia", "Sumartími í Úlan Bator"}, {"Moscow", "Sumartími í Moskvu"}, {"New_Caledonia", "Sumartími í Nýju-Kaledóníu"}, {"New_Zealand", "Sumartími á Nýja-Sjálandi"}, {"Newfoundland", "Sumartími á Nýfundnalandi"}, {"Norfolk", "Sumartími á Norfolk-eyju"}, {"Noronha", "Sumartími í Fernando de Noronha"}, {"Novosibirsk", "Sumartími í Novosibirsk"}, {"Omsk", "Sumartími í Omsk"}, {"Pakistan", "Sumartími í Pakistan"}, {"Paraguay", "Sumartími í Paragvæ"}, {"Peru", "Sumartími í Perú"}, {"Philippines", "Sumartími á Filippseyjum"}, {"Pierre_Miquelon", "Sumartími á Sankti Pierre og Miquelon"}, {"Sakhalin", "Sumartími á Sakhalin"}, {"Samara", "Sumartími í Samara"}, {"Samoa", "Sumartími á Samóa"}, {"Taipei", "Sumartími í Taipei"}, {"Tonga", "Sumartími á Tonga"}, {"Turkmenistan", "Sumartími í Túrkmenistan"}, {"Uruguay", "Sumartími í Úrúgvæ"}, {"Uzbekistan", "Sumartími í Úsbekistan"}, {"Vanuatu", "Sumartími á Vanúatú"}, {"Vladivostok", "Sumartími í Vladivostok"}, {"Volgograd", "Sumartími í Volgograd"}, {"Yakutsk", "Sumartími í Yakutsk"}, {"Yekaterinburg", "Sumartími í Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "Ora dell’Afghanistan"}, {"Africa_Central", "Ora dell’Africa centrale"}, {"Africa_Eastern", "Ora dell’Africa orientale"}, {"Africa_Southern", "Ora dell’Africa meridionale"}, {"Africa_Western", "Ora dell’Africa occidentale"}, {"Alaska", "Ora standard dell’Alaska"}, {"Amazon", "Ora standard dell’Amazzonia"}, {"America_Central", "Ora standard centrale USA"}, {"America_Eastern", "Ora standard orientale USA"}, {"America_Mountain", "Ora standard Montagne Rocciose USA"}, {"America_Pacific", "Ora standard del Pacifico USA"}, {"Anadyr", "Ora standard di Anadyr"}, {"Apia", "Ora standard di Apia"}, {"Arabian", "Ora standard araba"}, {"Argentina", "Ora standard dell’Argentina"}, {"Argentina_Western", "Ora standard dell’Argentina occidentale"}, {"Armenia", "Ora standard dell’Armenia"}, {"Atlantic", "Ora standard dell’Atlantico"}, {"Australia_Central", "Ora standard dell’Australia centrale"}, {"Australia_CentralWestern", "Ora standard dell’Australia centroccidentale"}, {"Australia_Eastern", "Ora standard dell’Australia orientale"}, {"Australia_Western", "Ora standard dell’Australia occidentale"}, {"Azerbaijan", "Ora standard dell’Azerbaigian"}, {"Azores", "Ora standard delle Azzorre"}, {"Bangladesh", "Ora standard del Bangladesh"}, {"Bhutan", "Ora del Bhutan"}, {"Bolivia", "Ora della Bolivia"}, {"Brasilia", "Ora standard di Brasilia"}, {"Brunei", "Ora del Brunei Darussalam"}, {"Cape_Verde", "Ora standard di Capo Verde"}, {"Chamorro", "Ora di Chamorro"}, {"Chatham", "Ora standard delle Chatham"}, {"Chile", "Ora standard del Cile"}, {"China", "Ora standard della Cina"}, {"Christmas", "Ora dell’Isola Christmas"}, {"Cocos", "Ora delle Isole Cocos"}, {"Colombia", "Ora standard della Colombia"}, {"Cook", "Ora standard delle isole Cook"}, {"Cuba", "Ora standard di Cuba"}, {"Davis", "Ora di Davis"}, {"DumontDUrville", "Ora di Dumont-d’Urville"}, {"East_Timor", "Ora di Timor Est"}, {"Easter", "Ora standard dell’Isola di Pasqua"}, {"Ecuador", "Ora dell’Ecuador"}, {"Europe_Central", "Ora standard dell’Europa centrale"}, {"Europe_Eastern", "Ora standard dell’Europa orientale"}, {"Europe_Further_Eastern", "Ora dell’Europa orientale (Kaliningrad)"}, {"Europe_Western", "Ora standard dell’Europa occidentale"}, {"Falkland", "Ora standard delle Isole Falkland"}, {"Fiji", "Ora standard delle Figi"}, {"French_Guiana", "Ora della Guiana francese"}, {"French_Southern", "Ora delle Terre australi e antartiche francesi"}, {"GMT", "Ora del meridiano di Greenwich"}, {"Galapagos", "Ora delle Galapagos"}, {"Gambier", "Ora di Gambier"}, {"Georgia", "Ora standard della Georgia"}, {"Gilbert_Islands", "Ora delle isole Gilbert"}, {"Greenland_Eastern", "Ora standard della Groenlandia orientale"}, {"Greenland_Western", "Ora standard della Groenlandia occidentale"}, {"Gulf", "Ora del Golfo"}, {"Guyana", "Ora della Guyana"}, {"Hawaii", "Ora standard delle Isole Hawaii-Aleutine"}, {"Hawaii_Aleutian", "Ora standard delle Isole Hawaii-Aleutine"}, {"Hong_Kong", "Ora standard di Hong Kong"}, {"Hovd", "Ora standard di Hovd"}, {"India", "Ora standard dell’India"}, {"Indian_Ocean", "Ora dell’Oceano Indiano"}, {"Indochina", "Ora dell’Indocina"}, {"Indonesia_Central", "Ora dell’Indonesia centrale"}, {"Indonesia_Eastern", "Ora dell’Indonesia orientale"}, {"Indonesia_Western", "Ora dell’Indonesia occidentale"}, {"Iran", "Ora standard dell’Iran"}, {"Irkutsk", "Ora standard di Irkutsk"}, {"Israel", "Ora standard di Israele"}, {"Japan", "Ora standard del Giappone"}, {"Kamchatka", "Ora standard di Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Ora del Kazakistan"}, {"Kazakhstan_Eastern", "Ora del Kazakistan orientale"}, {"Kazakhstan_Western", "Ora del Kazakistan occidentale"}, {"Korea", "Ora standard coreana"}, {"Kosrae", "Ora del Kosrae"}, {"Krasnoyarsk", "Ora standard di Krasnoyarsk"}, {"Kyrgystan", "Ora del Kirghizistan"}, {"Line_Islands", "Ora delle Sporadi equatoriali"}, {"Lord_Howe", "Ora standard di Lord Howe"}, {"Macau", "Ora standard di Macao"}, {"Magadan", "Ora standard di Magadan"}, {"Malaysia", "Ora della Malesia"}, {"Maldives", "Ora delle Maldive"}, {"Marquesas", "Ora delle Marchesi"}, {"Marshall_Islands", "Ora delle Isole Marshall"}, {"Mauritius", "Ora standard delle Mauritius"}, {"Mawson", "Ora di Mawson"}, {"Mexico_Pacific", "Ora standard del Pacifico (Messico)"}, {"Mongolia", "Ora standard di Ulan Bator"}, {"Moscow", "Ora standard di Mosca"}, {"Myanmar", "Ora della Birmania"}, {"Nauru", "Ora di Nauru"}, {"Nepal", "Ora del Nepal"}, {"New_Caledonia", "Ora standard della Nuova Caledonia"}, {"New_Zealand", "Ora standard della Nuova Zelanda"}, {"Newfoundland", "Ora standard di Terranova"}, {"Niue", "Ora di Niue"}, {"Norfolk", "Ora standard delle Isole Norfolk"}, {"Noronha", "Ora standard di Fernando de Noronha"}, {"Novosibirsk", "Ora standard di Novosibirsk"}, {"Omsk", "Ora standard di Omsk"}, {"Pakistan", "Ora standard del Pakistan"}, {"Palau", "Ora di Palau"}, {"Papua_New_Guinea", "Ora della Papua Nuova Guinea"}, {"Paraguay", "Ora standard del Paraguay"}, {"Peru", "Ora standard del Perù"}, {"Philippines", "Ora standard delle Filippine"}, {"Phoenix_Islands", "Ora delle Isole della Fenice"}, {"Pierre_Miquelon", "Ora standard di Saint-Pierre e Miquelon"}, {"Pitcairn", "Ora delle Pitcairn"}, {"Ponape", "Ora di Pohnpei"}, {"Pyongyang", "Ora di Pyongyang"}, {"Reunion", "Ora di Riunione"}, {"Rothera", "Ora di Rothera"}, {"Sakhalin", "Ora standard di Sakhalin"}, {"Samara", "Ora standard di Samara"}, {"Samoa", "Ora standard di Samoa"}, {"Seychelles", "Ora delle Seychelles"}, {"Singapore", "Ora di Singapore"}, {"Solomon", "Ora delle Isole Salomone"}, {"South_Georgia", "Ora della Georgia del Sud"}, {"Suriname", "Ora del Suriname"}, {"Syowa", "Ora di Syowa"}, {"Tahiti", "Ora di Tahiti"}, {"Taipei", "Ora standard di Taipei"}, {"Tajikistan", "Ora del Tagikistan"}, {"Tokelau", "Ora di Tokelau"}, {"Tonga", "Ora standard di Tonga"}, {"Truk", "Ora del Chuuk"}, {"Turkmenistan", "Ora standard del Turkmenistan"}, {"Tuvalu", "Ora di Tuvalu"}, {"Uruguay", "Ora standard dell’Uruguay"}, {"Uzbekistan", "Ora standard dell’Uzbekistan"}, {"Vanuatu", "Ora standard del Vanuatu"}, {"Venezuela", "Ora del Venezuela"}, {"Vladivostok", "Ora standard di Vladivostok"}, {"Volgograd", "Ora standard di Volgograd"}, {"Vostok", "Ora di Vostok"}, {"Wake", "Ora dell’Isola di Wake"}, {"Wallis", "Ora di Wallis e Futuna"}, {"Yakutsk", "Ora standard di Yakutsk"}, {"Yekaterinburg", "Ora standard di Ekaterinburg"}, {"Yukon", "Ora dello Yukon"}},
		{{"Alaska", "Ora legale dell’Alaska"}, {"Amazon", "Ora legale dell’Amazzonia"}, {"America_Central", "Ora legale centrale USA"}, {"America_Eastern", "Ora legale orientale USA"}, {"America_Mountain", "Ora legale Montagne Rocciose USA"}, {"America_Pacific", "Ora legale del Pacifico USA"}, {"Anadyr", "Ora legale di Anadyr"}, {"Apia", "Ora legale di Apia"}, {"Arabian", "Ora legale araba"}, {"Argentina", "Ora legale dell’Argentina"}, {"Argentina_Western", "Ora legale dell’Argentina occidentale"}, {"Armenia", "Ora legale dell’Armenia"}, {"Atlantic", "Ora legale dell’Atlantico"}, {"Australia_Central", "Ora legale dell’Australia centrale"}, {"Australia_CentralWestern", "Ora legale dell’Australia centroccidentale"}, {"Australia_Eastern", "Ora legale dell’Australia orientale"}, {"Australia_Western", "Ora legale dell’Australia occidentale"}, {"Azerbaijan", "Ora legale dell’Azerbaigian"}, {"Azores", "Ora legale delle Azzorre"}, {"Bangladesh", "Ora legale del Bangladesh"}, {"Brasilia", "Ora legale di Brasilia"}, {"Cape_Verde", "Ora legale di Capo Verde"}, {"Chatham", "Ora legale delle Chatham"}, {"Chile", "Ora legale del Cile"}, {"China", "Ora legale della Cina"}, {"Colombia", "Ora legale della Colombia"}, {"Cook", "Ora legale media delle isole Cook"}, {"Cuba", "Ora legale di Cuba"}, {"Easter", "Ora legale dell’Isola di Pasqua"}, {"Europe_Central", "Ora legale dell’Europa centrale"}, {"Europe_Eastern", "Ora legale dell’Europa orientale"}, {"Europe_Western", "Ora legale dell’Europa occidentale"}, {"Falkland", "Ora legale delle Isole Falkland"}, {"Fiji", "Ora legale delle Figi"}, {"Georgia", "Ora legale della Georgia"}, {"Greenland_Eastern", "Ora legale della Groenlandia orientale"}, {"Greenland_Western", "Ora legale della Groenlandia occidentale"}, {"Hawaii_Aleutian", "Ora legale delle Isole Hawaii-Aleutine"}, {"Hong_Kong", "Ora legale di Hong Kong"}, {"Hovd", "Ora legale di Hovd"}, {"Iran", "Ora legale dell’Iran"}, {"Irkutsk", "Ora legale di Irkutsk"}, {"Israel", "Ora legale di Israele"}, {"Japan", "Ora legale del Giappone"}, {"Kamchatka", "Ora legale di Petropavlovsk-Kamchatski"}, {"Korea", "Ora legale coreana"}, {"Krasnoyarsk", "Ora legale di Krasnoyarsk"}, {"Lord_Howe", "Ora legale di Lord Howe"}, {"Macau", "Ora legale di Macao"}, {"Magadan", "Ora legale di Magadan"}, {"Mauritius", "Ora legale delle Mauritius"}, {"Mexico_Pacific", "Ora legale del Pacifico (Messico)"}, {"Mongolia", "Ora legale di Ulan Bator"}, {"Moscow", "Ora legale di Mosca"}, {"New_Caledonia", "Ora legale della Nuova Caledonia"}, {"New_Zealand", "Ora legale della Nuova Zelanda"}, {"Newfoundland", "Ora legale di Terranova"}, {"Norfolk", "Ora legale delle Isole Norfolk"}, {"Noronha", "Ora legale di Fernando de Noronha"}, {"Novosibirsk", "Ora legale di Novosibirsk"}, {"Omsk", "Ora legale di Omsk"}, {"Pakistan", "Ora legale del Pakistan"}, {"Paraguay", "Ora legale del Paraguay"}, {"Peru", "Ora legale del Perù"}, {"Philippines", "Ora legale delle Filippine"}, {"Pierre_Miquelon", "Ora legale di Saint-Pierre e Miquelon"}, {"Sakhalin", "Ora legale di Sakhalin"}, {"Samara", "Ora legale di Samara"}, {"Samoa", "Ora legale di Samoa"}, {"Taipei", "Ora legale di Taipei"}, {"Tonga", "Ora legale di Tonga"}, {"Turkmenistan", "Ora legale del Turkmenistan"}, {"Uruguay", "Ora legale dell’Uruguay"}, {"Uzbekistan", "Ora legale dell’Uzbekistan"}, {"Vanuatu", "Ora legale del Vanuatu"}, {"Vladivostok", "Ora legale di Vladivostok"}, {"Volgograd", "Ora legale di Volgograd"}, {"Yakutsk", "Ora legale di Yakutsk"}, {"Yekaterinburg", "Ora legale di Ekaterinburg"}},
	},
	{
		{{"Acre", "アクレ標準時"}, {"Afghanistan", "アフガニスタン時間"}, {"Africa_Central", "中央アフリカ時間"}, {"Africa_Eastern", "東アフリカ時間"}, {"Africa_Southern", "南アフリカ標準時"}, {"Africa_Western", "西アフリカ時間"}, {"Alaska", "アラスカ標準時"}, {"Almaty", "アルトマイ標準時"}, {"Amazon", "アマゾン標準時"}, {"America_Central", "米国中部標準時"}, {"America_Eastern", "米国東部標準時"}, {"America_Mountain", "米国山岳標準時"}, {"America_Pacific", "米国太平洋標準時"}, {"Anadyr", "アナディリ標準時"}, {"Apia", "サモア標準時"}, {"Aqtau", "アクタウ標準時"}, {"Aqtobe", "アクトベ標準時"}, {"Arabian", "アラビア標準時"}, {"Argentina", "アルゼンチン標準時"}, {"Argentina_Western", "西部アルゼンチン標準時"}, {"Armenia", "アルメニア標準時"}, {"Atlantic", "大西洋標準時"}, {"Australia_Central", "オーストラリア中部標準時"}, {"Australia_CentralWestern", "オーストラリア中西部標準時"}, {"Australia_Eastern", "オーストラリア東部標準時"}, {"Australia_Western", "オーストラリア西部標準時"}, {"Azerbaijan", "アゼルバイジャン標準時"}, {"Azores", "アゾレス標準時"}, {"Bangladesh", "バングラデシュ標準時"}, {"Bhutan", "ブータン時間"}, {"Bolivia", "ボリビア時間"}, {"Brasilia", "ブラジリア標準時"}, {"Brunei", "ブルネイ時間"}, {"Cape_Verde", "カーボベルデ標準時"}, {"Casey", "ケイシー基地時間"}, {"Chamorro", "チャモロ時間"}, {"Chatham", "チャタム標準時"}, {"Chile", "チリ標準時"}, {"China", "中国標準時"}, {"Christmas", "クリスマス島時間"}, {"Cocos", "ココス諸島時間"}, {"Colombia", "コロンビア標準時"}, {"Cook", "クック諸島標準時"}, {"Cuba", "キューバ標準時"}, {"Davis", "デービス基地時間"}, {"DumontDUrville", "デュモン・デュルヴィル基地時間"}, {"East_Timor", "東ティモール時間"}, {"Easter", "イースター島標準時"}, {"Ecuador", "エクアドル時間"}, {"Europe_Central", "中央ヨーロッパ標準時"}, {"Europe_Eastern", "東ヨーロッパ標準時"}, {"Europe_Further_Eastern", "極東ヨーロッパ時間"}, {"Europe_Western", "西ヨーロッパ標準時"}, {"Falkland", "フォークランド諸島標準時"}, {"Fiji", "フィジー標準時"}, {"French_Guiana", "仏領ギアナ時間"}, {"French_Southern", "仏領南方南極時間"}, {"GMT", "グリニッジ標準時"}, {"Galapagos", "ガラパゴス時間"}, {"Gambier", "ガンビエ諸島時間"}, {"Georgia", "ジョージア標準時"}, {"Gilbert_Islands", "ギルバート諸島時間"}, {"Greenland_Eastern", "グリーンランド東部標準時"}, {"Greenland_Western", "グリーンランド西部標準時"}, {"Guam", "グアム時間"}, {"Gulf", "湾岸標準時"}, {"Guyana", "ガイアナ時間"}, {"Hawaii", "ハワイ・アリューシャン標準時"}, {"Hawaii_Aleutian", "ハワイ・アリューシャン標準時"}, {"Hong_Kong", "香港標準時"}, {"Hovd", "ホブド標準時"}, {"India", "インド標準時"}, {"Indian_Ocean", "インド洋時間"}, {"Indochina", "インドシナ時間"}, {"Indonesia_Central", "インドネシア中部時間"}, {"Indonesia_Eastern", "インドネシア東部時間"}, {"Indonesia_Western", "インドネシア西部時間"}, {"Iran", "イラン標準時"}, {"Irkutsk", "イルクーツク標準時"}, {"Israel", "イスラエル標準時"}, {"Japan", "日本標準時"}, {"Kamchatka", "ペトロパブロフスク・カムチャツキー標準時"}, {"Kazakhstan", "カザフスタン時間"}, {"Kazakhstan_Eastern", "東カザフスタン時間"}, {"Kazakhstan_Western", "西カザフスタン時間"}, {"Korea", "韓国標準時"}, {"Kosrae", "コスラエ時間"}, {"Krasnoyarsk", "クラスノヤルスク標準時"}, {"Kyrgystan", "キルギス時間"}, {"Lanka", "ランカ時間"}, {"Line_Islands", "ライン諸島時間"}, {"Lord_Howe", "ロードハウ標準時"}, {"Macau", "マカオ標準時"}, {"Magadan", "マガダン標準時"}, {"Malaysia", "マレーシア時間"}, {"Maldives", "モルディブ時間"}, {"Marquesas", "マルキーズ時間"}, {"Marshall_Islands", "マーシャル諸島時間"}, {"Mauritius", "モーリシャス標準時"}, {"Mawson", "モーソン基地時間"}, {"Mexico_Pacific", "メキシコ太平洋標準時"}, {"Mongolia", "ウランバートル標準時"}, {"Moscow", "モスクワ標準時"}, {"Myanmar", "ミャンマー時間"}, {"Nauru", "ナウル時間"}, {"Nepal", "ネパール時間"}, {"New_Caledonia", "ニューカレドニア標準時"}, {"New_Zealand", "ニュージーランド標準時"}, {"Newfoundland", "ニューファンドランド標準時"}, {"Niue", "ニウエ時間"}, {"Norfolk", "ノーフォーク島標準時"}, {"Noronha", "フェルナンド・デ・ノローニャ標準時"}, {"North_Mariana", "北マリアナ諸島時間"}, {"Novosibirsk", "ノヴォシビルスク標準時"}, {"Omsk", "オムスク標準時"}, {"Pakistan", "パキスタン標準時"}, {"Palau", "パラオ時間"}, {"Papua_New_Guinea", "パプアニューギニア時間"}, {"Paraguay", "パラグアイ標準時"}, {"Peru", "ペルー標準時"}, {"Philippines", "フィリピン標準時"}, {"Phoenix_Islands", "フェニックス諸島時間"}, {"Pierre_Miquelon", "サンピエール島・ミクロン島標準時"}, {"Pitcairn", "ピトケアン時間"}, {"Ponape", "ポンペイ時間"}, {"Pyongyang", "北朝鮮時間"}, {"Qyzylorda", "クズロルダ標準時"}, {"Reunion", "レユニオン時間"}, {"Rothera", "ロゼラ基地時間"}, {"Sakhalin", "サハリン標準時"}, {"Samara", "サマラ標準時"}, {"Samoa", "米領サモア標準時"}, {"Seychelles", "セーシェル時間"}, {"Singapore", "シンガポール標準時"}, {"Solomon", "ソロモン諸島時間"}, {"South_Georgia", "サウスジョージア時間"}, {"Suriname", "スリナム時間"}, {"Syowa", "昭和基地時間"}, {"Tahiti", "タヒチ時間"}, {"Taipei", "台湾標準時"}, {"Tajikistan", "タジキスタン時間"}, {"Tokelau", "トケラウ時間"}, {"Tonga", "トンガ標準時"}, {"Truk", "チューク時間"}, {"Turkey", "トルコ標準時"}, {"Turkmenistan", "トルクメニスタン標準時"}, {"Tuvalu", "ツバル時間"}, {"Uruguay", "ウルグアイ標準時"}, {"Uzbekistan", "ウズベキスタン標準時"}, {"Vanuatu", "バヌアツ標準時"}, {"Venezuela", "ベネズエラ時間"}, {"Vladivostok", "ウラジオストク標準時"}, {"Volgograd", "ボルゴグラード標準時"}, {"Vostok", "ボストーク基地時間"}, {"Wake", "ウェーク島時間"}, {"Wallis", "ウォリス・フツナ時間"}, {"Yakutsk", "ヤクーツク標準時"}, {"Yekaterinburg", "エカテリンブルグ標準時"}, {"Yukon", "ユーコン時間"}},
		{{"Acre", "アクレ夏時間"}, {"Alaska", "アラスカ夏時間"}, {"Almaty", "アルマトイ夏時間"}, {"Amazon", "アマゾン夏時間"}, {"America_Central", "米国中部夏時間"}, {"America_Eastern", "米国東部夏時間"}, {"America_Mountain", "米国山岳夏時間"}, {"America_Pacific", "米国太平洋夏時間"}, {"Anadyr", "アナディリ夏時間"}, {"Apia", "サモア夏時間"}, {"Aqtau", "アクタウ夏時間"}, {"Aqtobe", "アクトベ夏時間"}, {"Arabian", "アラビア夏時間"}, {"Argentina", "アルゼンチン夏時間"}, {"Argentina_Western", "西部アルゼンチン夏時間"}, {"Armenia", "アルメニア夏時間"}, {"Atlantic", "大西洋夏時間"}, {"Australia_Central", "オーストラリア中部夏時間"}, {"Australia_CentralWestern", "オーストラリア中西部夏時間"}, {"Australia_Eastern", "オーストラリア東部夏時間"}, {"Australia_Western", "オーストラリア西部夏時間"}, {"Azerbaijan", "アゼルバイジャン夏時間"}, {"Azores", "アゾレス夏時間"}, {"Bangladesh", "バングラデシュ夏時間"}, {"Brasilia", "ブラジリア夏時間"}, {"Cape_Verde", "カーボベルデ夏時間"}, {"Chatham", "チャタム夏時間"}, {"Chile", "チリ夏時間"}, {"China", "中国夏時間"}, {"Colombia", "コロンビア夏時間"}, {"Cook", "クック諸島夏時間"}, {"Cuba", "キューバ夏時間"}, {"Easter", "イースター島夏時間"}, {"Europe_Central", "中央ヨーロッパ夏時間"}, {"Europe_Eastern", "東ヨーロッパ夏時間"}, {"Europe_Western", "西ヨーロッパ夏時間"}, {"Falkland", "フォークランド諸島夏時間"}, {"Fiji", "フィジー夏時間"}, {"Georgia", "ジョージア夏時間"}, {"Greenland_Eastern", "グリーンランド東部夏時間"}, {"Greenland_Western", "グリーンランド西部夏時間"}, {"Hawaii_Aleutian", "ハワイ・アリューシャン夏時間"}, {"Hong_Kong", "香港夏時間"}, {"Hovd", "ホブド夏時間"}, {"Iran", "イラン夏時間"}, {"Irkutsk", "イルクーツク夏時間"}, {"Israel", "イスラエル夏時間"}, {"Japan", "日本夏時間"}, {"Kamchatka", "ペトロパブロフスク・カムチャツキー夏時間"}, {"Korea", "韓国夏時間"}, {"Krasnoyarsk", "クラスノヤルスク夏時間"}, {"Lord_Howe", "ロードハウ夏時間"}, {"Macau", "マカオ夏時間"}, {"Magadan", "マガダン夏時間"}, {"Mauritius", "モーリシャス夏時間"}, {"Mexico_Pacific", "メキシコ太平洋夏時間"}, {"Mongolia", "ウランバートル夏時間"}, {"Moscow", "モスクワ夏時間"}, {"New_Caledonia", "ニューカレドニア夏時間"}, {"New_Zealand", "ニュージーランド夏時間"}, {"Newfoundland", "ニューファンドランド夏時間"}, {"Norfolk", "ノーフォーク島夏時間"}, {"Noronha", "フェルナンド・デ・ノローニャ夏時間"}, {"Novosibirsk", "ノヴォシビルスク夏時間"}, {"Omsk", "オムスク夏時間"}, {"Pakistan", "パキスタン夏時間"}, {"Paraguay", "パラグアイ夏時間"}, {"Peru", "ペルー夏時間"}, {"Philippines", "フィリピン夏時間"}, {"Pierre_Miquelon", "サンピエール島・ミクロン島夏時間"}, {"Qyzylorda", "クズロルダ夏時間"}, {"Sakhalin", "サハリン夏時間"}, {"Samara", "サマラ夏時間"}, {"Samoa", "米領サモア夏時間"}, {"Taipei", "台湾夏時間"}, {"Tonga", "トンガ夏時間"}, {"Turkey", "トルコ夏時間"}, {"Turkmenistan", "トルクメニスタン夏時間"}, {"Uruguay", "ウルグアイ夏時間"}, {"Uzbekistan", "ウズベキスタン夏時間"}, {"Vanuatu", "バヌアツ夏時間"}, {"Vladivostok", "ウラジオストク夏時間"}, {"Volgograd", "ボルゴグラード夏時間"}, {"Yakutsk", "ヤクーツク夏時間"}, {"Yekaterinburg", "エカテリンブルグ夏時間"}},
	},
	{
		{{"Colombia", "nalcistcika fo la kolombiias"}, {"Galapagos", "tcika fo la galapagos"}},
		{{"Colombia", "cistcika fo la kolombiias"}},
	},
	{
		{{"Afghanistan", "Wektu Afghanistan"}, {"Africa_Central", "Wektu Afrika Tengah"}, {"Africa_Eastern", "Wektu Afrika Wetan"}, {"Africa_Southern", "Wektu Standar Afrika Kidul"}, {"Africa_Western", "Wektu Afrika Kulon"}, {"Alaska", "Wektu Standar Alaska"}, {"Amazon", "Wektu Standar Amazon"}, {"America_Central", "Wektu Standar Tengah"}, {"America_Eastern", "Wektu Standar sisih Wetan"}, {"America_Mountain", "Wektu Standar Giri"}, {"America_Pacific", "Wektu Standar Pasifik"}, {"Apia", "Wektu Standar Samoa"}, {"Arabian", "Wektu Standar Arab"}, {"Argentina", "Wektu Standar Argentina"}, {"Argentina_Western", "Wektu Standar Argentina sisih Kulon"}, {"Armenia", "Wektu Standar Armenia"}, {"Atlantic", "Wektu Standar Atlantik"}, {"Australia_Central", "Wektu Standar Australia Tengah"}, {"Australia_CentralWestern", "Wektu Standar Australia Tengah sisih Kulon"}, {"Australia_Eastern", "Wektu Standar Australia sisih Wetan"}, {"Australia_Western", "Wektu Standar Australia sisih Kulon"}, {"Azerbaijan", "Wektu Standar Azerbaijan"}, {"Azores", "Wektu Standar Azores"}, {"Bangladesh", "Wektu Standar Bangladesh"}, {"Bhutan", "Wektu Bhutan"}, {"Bolivia", "Wektu Bolivia"}, {"Brasilia", "Wektu Standar Brasilia"}, {"Brunei", "Wektu Brunei"}, {"Cape_Verde", "Wektu Standar Tanjung Verde"}, {"Chamorro", "Wektu Chamorro"}, {"Chatham", "Wektu Standar Chatham"}, {"Chile", "Wektu Standar Chili"}, {"China", "Wektu Standar Cina"}, {"Christmas", "Wektu Pulo Natal"}, {"Cocos", "Wektu Kepuloan Cocos"}, {"Colombia", "Wektu Standar Kolombia"}, {"Cook", "Wektu Standar Kepuloan Cook"}, {"Cuba", "Wektu Standar Kuba"}, {"Davis", "Wektu Davis"}, {"DumontDUrville", "Wektu Dumont d’Urville"}, {"East_Timor", "Wektu Timor-Leste"}, {"Easter", "Wektu Standar Pulo Paskah"}, {"Ecuador", "Wektu Ekuador"}, {"Europe_Central", "Wektu Standar Eropa Tengah"}, {"Europe_Eastern", "Wektu Standar Eropa sisih Wetan"}, {"Europe_Further_Eastern", "Wektu Eropa sisih Wetan seng Luwih Adoh"}, {"Europe_Western", "Wektu Standar Eropa sisih Kulon"}, {"Falkland", "Wektu Standar Kepuloan Falkland"}, {"Fiji", "Wektu Standar Fiji"}, {"French_Guiana", "Wektu Guiana Prancis"}, {"French_Southern", "Wektu Antartika lan Prancis sisih Kidul"}, {"GMT", "Wektu Rerata Greenwich"}, {"Galapagos", "Wektu Galapagos"}, {"Gambier", "Wektu Gambier"}, {"Georgia", "Wektu Standar Georgia"}, {"Gilbert_Islands", "Wektu Kepuloan Gilbert"}, {"Greenland_Eastern", "Wektu Standar Grinland Wetan"}, {"Greenland_Western", "Wektu Standar Grinland Kulon"}, {"Gulf", "Wektu Standar Teluk"}, {"Guyana", "Wektu Guyana"}, {"Hawaii", "Wektu Standar Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Wektu Standar Hawaii-Aleutian"}, {"Hong_Kong", "Wektu Standar Hong Kong"}, {"Hovd", "Wektu Standar Hovd"}, {"India", "Wektu Standar India"}, {"Indian_Ocean", "Wektu Segoro Hindia"}, {"Indochina", "Wektu Indocina"}, {"Indonesia_Central", "Wektu Indonesia Tengah"}, {"Indonesia_Eastern", "Wektu Indonesia sisih Wetan"}, {"Indonesia_Western", "Wektu Indonesia sisih Kulon"}, {"Iran", "Wektu Standar Iran"}, {"Irkutsk", "Wektu Standar Irkutsk"}, {"Israel", "Wektu Standar Israel"}, {"Japan", "Wektu Standar Jepang"}, {"Kazakhstan", "Wektu Kazakhstan"}, {"Kazakhstan_Eastern", "Wektu Kazakhstan Wetan"}, {"Kazakhstan_Western", "Wektu Kazakhstan Kulon"}, {"Korea", "Wektu Standar Korea"}, {"Kosrae", "Wektu Kosrae"}, {"Krasnoyarsk", "Wektu Standar Krasnoyarsk"}, {"Kyrgystan", "Wektu Kirgizstan"}, {"Line_Islands", "Wektu Kepuloan Line"}, {"Lord_Howe", "Wektu Standar Lord Howe"}, {"Magadan", "Wektu Standar Magadan"}, {"Malaysia", "Wektu Malaysia"}, {"Maldives", "Wektu Maladewa"}, {"Marquesas", "Wektu Marquesas"}, {"Marshall_Islands", "Wektu Kepuloan Marshall"}, {"Mauritius", "Wektu Standar Mauritius"}, {"Mawson", "Wektu Mawson"}, {"Mexico_Pacific", "Wektu Standar Pasifik Meksiko"}, {"Mongolia", "Wektu Standar Ulaanbaatar"}, {"Moscow", "Wektu Standar Moscow"}, {"Myanmar", "Wektu Myanmar"}, {"Nauru", "Wektu Nauru"}, {"Nepal", "Wektu Nepal"}, {"New_Caledonia", "Wektu Standar Kaledonia Anyar"}, {"New_Zealand", "Wektu Standar Selandia Anyar"}, {"Newfoundland", "Wektu Standar Newfoundland"}, {"Niue", "Wektu Niue"}, {"Norfolk", "Wektu Standar Pulo Norfolk"}, {"Noronha", "Wektu Standar Fernando de Noronha"}, {"Novosibirsk", "Wektu Standar Novosibirsk"}, {"Omsk", "Wektu Standar Omsk"}, {"Pakistan", "Wektu Standar Pakistan"}, {"Palau", "Wektu Palau"}, {"Papua_New_Guinea", "Wektu Papua Nugini"}, {"Paraguay", "Wektu Standar Paraguay"}, {"Peru", "Wektu Standar Peru"}, {"Philippines", "Wektu Standar Filipina"}, {"Phoenix_Islands", "Wektu Kepuloan Phoenix"}, {"Pierre_Miquelon", "Wektu Standar Santa Pierre lan Miquelon"}, {"Pitcairn", "Wektu Pitcairn"}, {"Ponape", "Wektu Pohnpei"}, {"Pyongyang", "Wektu Korea Lor"}, {"Reunion", "Wektu Reunion"}, {"Rothera", "Wektu Rothera"}, {"Sakhalin", "Wektu Standar Sakhalin"}, {"Samoa", "Wektu Standar Samoa Amerika"}, {"Seychelles", "Wektu Seichelles"}, {"Singapore", "Wektu Singapura"}, {"Solomon", "Wektu Kepuloan Solomon"}, {"South_Georgia", "Wektu Georgia Kidul"}, {"Suriname", "Wektu Suriname"}, {"Syowa", "Wektu Syowa"}, {"Tahiti", "Wektu Tahiti"}, {"Taipei", "Wektu Standar Taiwan"}, {"Tajikistan", "Wektu Tajikistan"}, {"Tokelau", "Wektu Tokelau"}, {"Tonga", "Wektu Standar Tonga"}, {"Truk", "Wektu Chuuk"}, {"Turkmenistan", "Wektu Standar Turkmenistan"}, {"Tuvalu", "Wektu Tuvalu"}, {"Uruguay", "Wektu Standar Uruguay"}, {"Uzbekistan", "Wektu Standar Usbekistan"}, {"Vanuatu", "Wektu Standar Vanuatu"}, {"Venezuela", "Wektu Venezuela"}, {"Vladivostok", "Wektu Standar Vladivostok"}, {"Volgograd", "Wektu Standar Volgograd"}, {"Vostok", "Wektu Vostok"}, {"Wake", "Wektu Pulo Wake"}, {"Wallis", "Wektu Wallis lan Futuna"}, {"Yakutsk", "Wektu Standar Yakutsk"}, {"Yekaterinburg", "Wektu Standar Yekaterinburg"}, {"Yukon", "Wektu Yukon"}},
		{{"Alaska", "Wektu Ketigo Alaska"}, {"Amazon", "Wektu Ketigo Amazon"}, {"America_Central", "Wektu Ketigo Tengah"}, {"America_Eastern", "Wektu Ketigo sisih Wetah"}, {"America_Mountain", "Wektu Ketigo Giri"}, {"America_Pacific", "Wektu Ketigo Pasifik"}, {"Apia", "Wektu Ketiga Samoa"}, {"Arabian", "Wektu Ketigo Arab"}, {"Argentina", "Wektu Ketigo Argentina"}, {"Argentina_Western", "Wektu Ketigo Argentina sisih Kulon"}, {"Armenia", "Wektu Ketigo Armenia"}, {"Atlantic", "Wektu Ketigo Atlantik"}, {"Australia_Central", "Wektu Ketigo Australia Tengah"}, {"Australia_CentralWestern", "Wektu Ketigo Australia Tengah sisih Kulon"}, {"Australia_Eastern", "Wektu Ketigo Australia sisih Wetan"}, {"Australia_Western", "Wektu Ketigo Australia sisih Kulon"}, {"Azerbaijan", "Wektu Ketigo Azerbaijan"}, {"Azores", "Wektu Ketigo Azores"}, {"Bangladesh", "Wektu Ketigo Bangladesh"}, {"Brasilia", "Wektu Ketigo Brasilia"}, {"Cape_Verde", "Wektu Ketigo Tanjung Verde"}, {"Chatham", "Wektu Ketigo Chatham"}, {"Chile", "Wektu Ketigo Chili"}, {"China", "Wektu Ketiga Cina"}, {"Colombia", "Wektu Ketigo Kolombia"}, {"Cook", "Wektu Ketiga Kepuloan Cook"}, {"Cuba", "Wektu Ketigo Kuba"}, {"Easter", "Wektu Ketigo Pulo Paskah"}, {"Europe_Central", "Wektu Ketigo Eropa Tengah"}, {"Europe_Eastern", "Wektu Ketigo Eropa sisih Wetan"}, {"Europe_Western", "Wektu Ketigo Eropa sisih Kulon"}, {"Falkland", "Wektu Ketigo Kepuloan Falkland"}, {"Fiji", "Wektu Ketigo Fiji"}, {"Georgia", "Wektu Ketigo Georgia"}, {"Greenland_Eastern", "Wektu Ketigo Grinland Wetan"}, {"Greenland_Western", "Wektu Ketigo Grinland Kulon"}, {"Hawaii_Aleutian", "Wektu Ketigo Hawaii-Aleutian"}, {"Hong_Kong", "Wektu Ketiga Hong Kong"}, {"Hovd", "Wektu Ketiga Hovd"}, {"Iran", "Wektu Ketigo Iran"}, {"Irkutsk", "Wektu Ketigo Irkutsk"}, {"Israel", "Wektu Ketigo Israel"}, {"Japan", "Wektu Ketiga Jepang"}, {"Korea", "Wektu Ketiga Korea"}, {"Krasnoyarsk", "Wektu Ketigo Krasnoyarsk"}, {"Lord_Howe", "Wektu Ketigo Lord Howe"}, {"Magadan", "Wektu Ketigo Magadan"}, {"Mauritius", "Wektu Ketigo Mauritius"}, {"Mexico_Pacific", "Wektu Ketigo Pasifik Meksiko"}, {"Mongolia", "Wektu Ketiga Ulaanbaatar"}, {"Moscow", "Wektu Ketigo Moscow"}, {"New_Caledonia", "Wektu Ketigo Kaledonia Anyar"}, {"New_Zealand", "Wektu Ketigo Selandia Anyar"}, {"Newfoundland", "Wektu Ketigo Newfoundland"}, {"Norfolk", "Wektu Ketigo Pulo Norfolk"}, {"Noronha", "Wektu Ketigo Fernando de Noronha"}, {"Novosibirsk", "Wektu Ketigo Novosibirsk"}, {"Omsk", "Wektu Ketigo Omsk"}, {"Pakistan", "Wektu Ketigo Pakistan"}, {"Paraguay", "Wektu Ketigo Paraguay"}, {"Peru", "Wektu Ketigo Peru"}, {"Philippines", "Wektu Ketigo Filipina"}, {"Pierre_Miquelon", "Wektu Ketigo Santa Pierre lan Miquelon"}, {"Sakhalin", "Wektu Ketigo Sakhalin"}, {"Samoa", "Wektu Ketiga Samoa Amerika"}, {"Taipei", "Wektu Awan Taiwan"}, {"Tonga", "Wektu Ketigo Tonga"}, {"Turkmenistan", "Wektu Ketigo Turkmenistan"}, {"Uruguay", "Wektu Ketigo Uruguay"}, {"Uzbekistan", "Wektu Ketigo Usbekistan"}, {"Vanuatu", "Wektu Ketigo Vanuatu"}, {"Vladivostok", "Wektu Ketigo Vladivostok"}, {"Volgograd", "Wektu Ketigo Volgograd"}, {"Yakutsk", "Wektu Ketigo Yakutsk"}, {"Yekaterinburg", "Wektu Ketigo Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "ავღანეთის დრო"}, {"Africa_Central", "ცენტრალური აფრიკის დრო"}, {"Africa_Eastern", "აღმოსავლეთ აფრიკის დრო"}, {"Africa_Southern", "სამხრეთ აფრიკის დრო"}, {"Africa_Western", "დასავლეთ აფრიკის დრო"}, {"Alaska", "ალასკის სტანდარტული დრო"}, {"Amazon", "ამაზონიის სტანდარტული დრო"}, {"America_Central", "ჩრდილოეთ ამერიკის ცენტრალური სტანდარტული დრო"}, {"America_Eastern", "ჩრდილოეთ ამერიკის აღმოსავლეთის სტანდარტული დრო"}, {"America_Mountain", "ჩრდილოეთ ამერიკის მაუნთინის სტანდარტული დრო"}, {"America_Pacific", "ჩრდილოეთ ამერიკის წყნარი ოკეანის სტანდარტული დრო"}, {"Apia", "აპიას სტანდარტული დრო"}, {"Arabian", "არაბეთის სტანდარტული დრო"}, {"Argentina", "არგენტინის სტანდარტული დრო"}, {"Argentina_Western", "დასავლეთ არგენტინის სტანდარტული დრო"}, {"Armenia", "სომხეთის სტანდარტული დრო"}, {"Atlantic", "ატლანტიკის ოკეანის სტანდარტული დრო"}, {"Australia_Central", "ავსტრალიის ცენტრალური სტანდარტული დრო"}, {"Australia_CentralWestern", "ცენტრალური და დასავლეთ ავსტრალიის სტანდარტული დრო"}, {"Australia_Eastern", "აღმოსავლეთ ავსტრალიის სტანდარტული დრო"}, {"Australia_Western", "დასავლეთ ავსტრალიის სტანდარტული დრო"}, {"Azerbaijan", "აზერბაიჯანის სტანდარტული დრო"}, {"Azores", "აზორის კუნძულების სტანდარტული დრო"}, {"Bangladesh", "ბანგლადეშის სტანდარტული დრო"}, {"Bhutan", "ბუტანის დრო"}, {"Bolivia", "ბოლივიის დრო"}, {"Brasilia", "ბრაზილიის სტანდარტული დრო"}, {"Brunei", "ბრუნეი-დარუსალამის დრო"}, {"Cape_Verde", "კაბო-ვერდეს სტანდარტული დრო"}, {"Chamorro", "ჩამოროს დრო"}, {"Chatham", "ჩატემის სტანდარტული დრო"}, {"Chile", "ჩილეს სტანდარტული დრო"}, {"China", "ჩინეთის სტანდარტული დრო"}, {"Christmas", "შობის კუნძულის დრო"}, {"Cocos", "ქოქოსის კუნძულების დრო"}, {"Colombia", "კოლუმბიის სტანდარტული დრო"}, {"Cook", "კუკის კუნძულების სტანდარტული დრო"}, {"Cuba", "კუბის სტანდარტული დრო"}, {"Davis", "დევისის დრო"}, {"DumontDUrville", "დუმონ-დურვილის დრო"}, {"East_Timor", "აღმოსავლეთ ტიმორის დრო"}, {"Easter", "აღდგომის კუნძულის სტანდარტული დრო"}, {"Ecuador", "ეკვადორის დრო"}, {"Europe_Central", "ცენტრალური ევროპის სტანდარტული დრო"}, {"Europe_Eastern", "აღმოსავლეთ ევროპის სტანდარტული დრო"}, {"Europe_Further_Eastern", "შორეული აღმოსავლეთ ევროპის დრო"}, {"Europe_Western", "დასავლეთ ევროპის სტანდარტული დრო"}, {"Falkland", "ფოლკლენდის კუნძულების სტანდარტული დრო"}, {"Fiji", "ფიჯის სტანდარტული დრო"}, {"French_Guiana", "საფრანგეთის გვიანის დრო"}, {"French_Southern", "ფრანგული სამხრეთის და ანტარქტიკის დრო"}, {"GMT", "გრინვიჩის საშუალო დრო"}, {"Galapagos", "გალაპაგოსის დრო"}, {"Gambier", "გამბიერის დრო"}, {"Georgia", "საქართველოს სტანდარტული დრო"}, {"Gilbert_Islands", "გილბერტის კუნძულების დრო"}, {"Greenland_Eastern", "აღმოსავლეთ გრენლანდიის სტანდარტული დრო"}, {"Greenland_Western", "დასავლეთ გრენლანდიის სტანდარტული დრო"}, {"Gulf", "სპარსეთის ყურის სტანდარტული დრო"}, {"Guyana", "გაიანის დრო"}, {"Hawaii", "ჰავაისა და ალეუტის სტანდარტული დრო"}, {"Hawaii_Aleutian", "ჰავაისა და ალეუტის სტანდარტული დრო"}, {"Hong_Kong", "ჰონკონგის სტანდარტული დრო"}, {"Hovd", "ჰოვდის სტანდარტული დრო"}, {"India", "ინდოეთის დრო"}, {"Indian_Ocean", "ინდოეთის ოკეანის კუნძულების დრო"}, {"Indochina", "ინდოჩინეთის დრო"}, {"Indonesia_Central", "ცენტრალური ინდონეზიის დრო"}, {"Indonesia_Eastern", "აღმოსავლეთ ინდონეზიის დრო"}, {"Indonesia_Western", "დასავლეთ ინდონეზიის დრო"}, {"Iran", "ირანის სტანდარტული დრო"}, {"Irkutsk", "ირკუტსკის სტანდარტული დრო"}, {"Israel", "ისრაელის სტანდარტული დრო"}, {"Japan", "იაპონიის სტანდარტული დრო"}, {"Kazakhstan", "ყაზახეთის დრო"}, {"Kazakhstan_Eastern", "აღმოსავლეთ ყაზახეთის დრო"}, {"Kazakhstan_Western", "დასავლეთ ყაზახეთის დრო"}, {"Korea", "კორეის სტანდარტული დრო"}, {"Kosrae", "კოსრეს დრო"}, {"Krasnoyarsk", "კრასნოიარსკის სტანდარტული დრო"}, {"Kyrgystan", "ყირგიზეთის დრო"}, {"Line_Islands", "ლაინის კუნძულების დრო"}, {"Lord_Howe", "ლორდ-ჰაუს სტანდარტული დრო"}, {"Magadan", "მაგადანის სტანდარტული დრო"}, {"Malaysia", "მალაიზიის დრო"}, {"Maldives", "მალდივების დრო"}, {"Marquesas", "მარკიზის კუნძულების დრო"}, {"Marshall_Islands", "მარშალის კუნძულების დრო"}, {"Mauritius", "მავრიკის სტანდარტული დრო"}, {"Mawson", "მოუსონის დრო"}, {"Mexico_Pacific", "მექსიკის წყნარი ოკეანის სტანდარტული დრო"}, {"Mongolia", "ულან-ბატორის სტანდარტული დრო"}, {"Moscow", "მოსკოვის სტანდარტული დრო"}, {"Myanmar", "მიანმარის დრო"}, {"Nauru", "ნაურუს დრო"}, {"Nepal", "ნეპალის დრო"}, {"New_Caledonia", "ახალი კალედონიის სტანდარტული დრო"}, {"New_Zealand", "ახალი ზელანდიის სტანდარტული დრო"}, {"Newfoundland", "ნიუფაუნდლენდის სტანდარტული დრო"}, {"Niue", "ნიუეს დრო"}, {"Norfolk", "ნორფოლკის კუნძულის სტანდარტული დრო"}, {"Noronha", "ფერნანდო-დე-ნორონიას სტანდარტული დრო"}, {"Novosibirsk", "ნოვოსიბირსკის სტანდარტული დრო"}, {"Omsk", "ომსკის სტანდარტული დრო"}, {"Pakistan", "პაკისტანის სტანდარტული დრო"}, {"Palau", "პალაუს დრო"}, {"Papua_New_Guinea", "პაპუა-ახალი გვინეის დრო"}, {"Paraguay", "პარაგვაის სტანდარტული დრო"}, {"Peru", "პერუს სტანდარტული დრო"}, {"Philippines", "ფილიპინების სტანდარტული დრო"}, {"Phoenix_Islands", "ფენიქსის კუნძულების დრო"}, {"Pierre_Miquelon", "სენ-პიერის და მიკელონის სტანდარტული დრო"}, {"Pitcairn", "პიტკერნის დრო"}, {"Ponape", "პონაპეს დრო"}, {"Pyongyang", "ფხენიანის დრო"}, {"Reunion", "რეიუნიონის დრო"}, {"Rothera", "როთერის დრო"}, {"Sakhalin", "სახალინის სტანდარტული დრო"}, {"Samoa", "სამოას სტანდარტული დრო"}, {"Seychelles", "სეიშელის კუნძულების დრო"}, {"Singapore", "სინგაპურის დრო"}, {"Solomon", "სოლომონის კუნძულების დრო"}, {"South_Georgia", "სამხრეთ გეორგიის დრო"}, {"Suriname", "სურინამის დრო"}, {"Syowa", "სიოვას დრო"}, {"Tahiti", "ტაიტის დრო"}, {"Taipei", "ტაიბეის სტანდარტული დრო"}, {"Tajikistan", "ტაჯიკეთის დრო"}, {"Tokelau", "ტოკელაუს დრო"}, {"Tonga", "ტონგის სტანდარტული დრო"}, {"Truk", "ჩუუკის დრო"}, {"Turkmenistan", "თურქმენეთის სტანდარტული დრო"}, {"Tuvalu", "ტუვალუს დრო"}, {"Uruguay", "ურუგვაის სტანდარტული დრო"}, {"Uzbekistan", "უზბეკეთის სტანდარტული დრო"}, {"Vanuatu", "ვანუატუს სტანდარტული დრო"}, {"Venezuela", "ვენესუელის დრო"}, {"Vladivostok", "ვლადივოსტოკის სტანდარტული დრო"}, {"Volgograd", "ვოლგოგრადის სტანდარტული დრო"}, {"Vostok", "ვოსტოკის დრო"}, {"Wake", "ვეიკის კუნძულის დრო"}, {"Wallis", "ვოლისი და ფუტუნას დრო"}, {"Yakutsk", "იაკუტსკის სტანდარტული დრო"}, {"Yekaterinburg", "ეკატერინბურგის სტანდარტული დრო"}, {"Yukon", "იუკონის დრო"}},
		{{"Alaska", "ალასკის ზაფხულის დრო"}, {"Amazon", "ამაზონიის ზაფხულის დრო"}, {"America_Central", "ჩრდილოეთ ამერიკის ცენტრალური ზაფხულის დრო"}, {"America_Eastern", "ჩრდილოეთ ამერიკის აღმოსავლეთის ზაფხულის დრო"}, {"America_Mountain", "ჩრდილოეთ ამერიკის მაუნთინის ზაფხულის დრო"}, {"America_Pacific", "ჩრდილოეთ ამერიკის წყნარი ოკეანის ზაფხულის დრო"}, {"Apia", "აპიას ზაფხულის დრო"}, {"Arabian", "არაბეთის ზაფხულის დრო"}, {"Argentina", "არგენტინის ზაფხულის დრო"}, {"Argentina_Western", "დასავლეთ არგენტინის ზაფხულის დრო"}, {"Armenia", "სომხეთის ზაფხულის დრო"}, {"Atlantic", "ატლანტიკის ოკეანის ზაფხულის დრო"}, {"Australia_Central", "ავსტრალიის ცენტრალური ზაფხულის დრო"}, {"Australia_CentralWestern", "ცენტრალური და დასავლეთ ავსტრალიის ზაფხულის დრო"}, {"Australia_Eastern", "აღმოსავლეთ ავსტრალიის ზაფხულის დრო"}, {"Australia_Western", "დასავლეთ ავსტრალიის ზაფხულის დრო"}, {"Azerbaijan", "აზერბაიჯანის ზაფხულის დრო"}, {"Azores", "აზორის კუნძულების ზაფხულის დრო"}, {"Bangladesh", "ბანგლადეშის ზაფხულის დრო"}, {"Brasilia", "ბრაზილიის ზაფხულის დრო"}, {"Cape_Verde", "კაბო-ვერდეს ზაფხულის დრო"}, {"Chatham", "ჩატემის ზაფხულის დრო"}, {"Chile", "ჩილეს ზაფხულის დრო"}, {"China", "ჩინეთის დროის სარტყელი"}, {"Colombia", "კოლუმბიის ზაფხულის დრო"}, {"Cook", "კუკის კუნძულების ნახევრად ზაფხულის დრო"}, {"Cuba", "კუბის ზაფხულის დრო"}, {"Easter", "აღდგომის კუნძულის ზაფხულის დრო"}, {"Europe_Central", "ცენტრალური ევროპის ზაფხულის დრო"}, {"Europe_Eastern", "აღმოსავლეთ ევროპის ზაფხულის დრო"}, {"Europe_Western", "დასავლეთ ევროპის ზაფხულის დრო"}, {"Falkland", "ფოლკლენდის კუნძულების ზაფხულის დრო"}, {"Fiji", "ფიჯის ზაფხულის დრო"}, {"Georgia", "საქართველოს ზაფხულის დრო"}, {"Greenland_Eastern", "აღმოსავლეთ გრენლანდიის ზაფხულის დრო"}, {"Greenland_Western", "დასავლეთ გრენლანდიის ზაფხულის დრო"}, {"Hawaii_Aleutian", "ჰავაისა და ალეუტის ზაფხულის დრო"}, {"Hong_Kong", "ჰონკონგის ზაფხულის დრო"}, {"Hovd", "ჰოვდის ზაფხულის დრო"}, {"Iran", "ირანის დროის სარტყელი"}, {"Irkutsk", "ირკუტსკის ზაფხულის დრო"}, {"Israel", "ისრაელის ზაფხულის დრო"}, {"Japan", "იაპონიის ზაფხულის დრო"}, {"Korea", "კორეის ზაფხულის დრო"}, {"Krasnoyarsk", "კრასნოიარსკის ზაფხულის დრო"}, {"Lord_Howe", "ლორდ-ჰაუს ზაფხულის დრო"}, {"Magadan", "მაგადანის ზაფხულის დრო"}, {"Mauritius", "მავრიკის ზაფხულის დრო"}, {"Mexico_Pacific", "მექსიკის წყნარი ოკეანის ზაფხულის დრო"}, {"Mongolia", "ულან-ბატორის ზაფხულის დრო"}, {"Moscow", "მოსკოვის ზაფხულის დრო"}, {"New_Caledonia", "ახალი კალედონიის ზაფხულის დრო"}, {"New_Zealand", "ახალი ზელანდიის ზაფხულის დრო"}, {"Newfoundland", "ნიუფაუნდლენდის ზაფხულის დრო"}, {"Norfolk", "ნორფოლკის კუნძულის ზაფხულის დრო"}, {"Noronha", "ფერნანდო-დე-ნორონიას ზაფხულის დრო"}, {"Novosibirsk", "ნოვოსიბირსკის ზაფხულის დრო"}, {"Omsk", "ომსკის ზაფხულის დრო"}, {"Pakistan", "პაკისტანის ზაფხულის დრო"}, {"Paraguay", "პარაგვაის ზაფხულის დრო"}, {"Peru", "პერუს ზაფხულის დრო"}, {"Philippines", "ფილიპინების ზაფხულის დრო"}, {"Pierre_Miquelon", "სენ-პიერის და მიკელონის ზაფხულის დრო"}, {"Sakhalin", "სახალინის ზაფხულის დრო"}, {"Samoa", "სამოას ზაფხულის დრო"}, {"Taipei", "ტაიბეის ზაფხულის დრო"}, {"Tonga", "ტონგის ზაფხულის დრო"}, {"Turkmenistan", "თურქმენეთის ზაფხულის დრო"}, {"Uruguay", "ურუგვაის ზაფხულის დრო"}, {"Uzbekistan", "უზბეკეთის ზაფხულის დრო"}, {"Vanuatu", "ვანუატუს ზაფხულის დრო"}, {"Vladivostok", "ვლადივოსტოკის ზაფხულის დრო"}, {"Volgograd", "ვოლგოგრადის ზაფხულის დრო"}, {"Yakutsk", "იაკუტსკის ზაფხულის დრო"}, {"Yekaterinburg", "ეკატერინბურგის ზაფხულის დრო"}},
	},
	{
		{{"Afghanistan", "Akud n Afɣanistan"}, {"Africa_Central", "Akud n Tefriqt talemmast"}, {"Africa_Eastern", "Akud n Tefriqt n usammar"}, {"Africa_Southern", "Akud amezday n Tefriqt n unẓul"}, {"Africa_Western", "Akud n Tefriqt n umalu"}, {"Alaska", "Akud amezday n Alaska"}, {"Amazon", "Akud amezday n Amaẓun"}, {"America_Central", "Akud amezday n tlemmast n Marikan"}, {"America_Eastern", "Akud amezday n usammar n Marikan"}, {"America_Mountain", "Akud amezday n yidurar n Marikan"}, {"America_Pacific", "Akud amezday amelwi"}, {"Apia", "Akud amezday n Apya"}, {"Arabian", "Akud aɛrab amezday"}, {"Argentina", "Akud amezday n Arjuntin"}, {"Argentina_Western", "Akud amezday n Arjuntin n usammar"}, {"Armenia", "Akud amezday n Arminya"}, {"Atlantic", "Akud amezday aṭlasan"}, {"Australia_Central", "Akud amezday n Ustralya talemmast"}, {"Australia_CentralWestern", "Akud amezday n tlemmast n umalu n Ustralya"}, {"Australia_Eastern", "Akud amezday n Ustralya n usammar"}, {"Australia_Western", "Akud amezday n Ustralya n umalu"}, {"Azerbaijan", "Akud amezday n Aẓerbiǧan"}, {"Azores", "Akud amezday n Aẓuris"}, {"Bangladesh", "Akud amezday n Bangladac"}, {"Bhutan", "Akud n Butan"}, {"Bolivia", "Akud n Bulivi"}, {"Brasilia", "Akud amezday n Braẓilya"}, {"Brunei", "Akud n Brunay Dar Salam"}, {"Cape_Verde", "Akud amezday n Kap Vir"}, {"Chamorro", "Akud amezday n Camurru"}, {"Chatham", "Akud amezday n Catham"}, {"Chile", "Akud amezday n Cili"}, {"China", "Akud amezday n Cin"}, {"Christmas", "Akud n Tegzirin n Krismas"}, {"Cocos", "Akud n Tegzirin n Kuku"}, {"Colombia", "Akud amezday n Kulumbya"}, {"Cook", "Akud amezday n Tegzirin n Kuk"}, {"Cuba", "Akud amezday n Kuba"}, {"Davis", "Akud n Davis"}, {"DumontDUrville", "Akud n Dumont-d’Urville"}, {"East_Timor", "Akud n Timur n usammar"}, {"Easter", "Akud amezday n Island n usammar"}, {"Ecuador", "Akud n Ikwaṭur"}, {"Europe_Central", "Akud amezday n Turuft talemmast"}, {"Europe_Eastern", "Akud amezday n Turuft n usammar"}, {"Europe_Further_Eastern", "Akud nniḍen n Turuft n Usammar"}, {"Europe_Western", "Akud amezday n Turuft n umalu"}, {"Falkland", "Akud amezday n Tegzirin n Falkland"}, {"Fiji", "Akud amezday n Fiji"}, {"French_Guiana", "Akud n Gwiyan tafransist"}, {"French_Southern", "Akud n wakal n unẓul d Antarktik n Fransa"}, {"GMT", "Akud alemmas n Greenwich"}, {"Galapagos", "Akud n Galapagus"}, {"Gambier", "Akud n Tegzirin Gambyer"}, {"Georgia", "Akud amezday n Jyurjya"}, {"Gilbert_Islands", "Akud n Tegzirin Jilbar"}, {"Greenland_Eastern", "Akud amezday n Grinland n usammar"}, {"Greenland_Western", "Akud amezday n Grinland n umalu"}, {"Gulf", "Akud amezday n Gulf"}, {"Guyana", "Akud n Gwiyan"}, {"Hawaii", "Akud amezday n Haway-Aliwsyan"}, {"Hawaii_Aleutian", "Akud amezday n Haway-Aliwsyan"}, {"Hong_Kong", "Akud amezday n Hung Kung"}, {"Hovd", "Akud amezday n Huvd"}, {"India", "Akud amezday n Lhend"}, {"Indian_Ocean", "Akud n Ugaraw Ahendi"}, {"Indochina", "Akud n Inducin"}, {"Indonesia_Central", "Akud n tlemmast n Andunisya"}, {"Indonesia_Eastern", "Akud n usammar n Andunisya"}, {"Indonesia_Western", "Akud n umalu n Andunisya"}, {"Iran", "Akud amezday n Iran"}, {"Irkutsk", "Akud amezday n Irkutsk"}, {"Israel", "Akud amezday n Izrayil"}, {"Japan", "Akud amezday n Japun"}, {"Kazakhstan_Eastern", "Akud n usammar n Kazaxistan"}, {"Kazakhstan_Western", "Akud n n umalu n Kazaxistan"}, {"Korea", "Akud amezday n Kurya"}, {"Kosrae", "Akud n Kosrae"}, {"Krasnoyarsk", "Akud amagnu n Krasnoyarsk"}, {"Kyrgystan", "Akud n Kirigistan"}, {"Line_Islands", "Akud n Tegzirin Lin"}, {"Lord_Howe", "Akud Amagnu n Lord Howe"}, {"Magadan", "Akud amezday n Magadan"}, {"Malaysia", "Akud n Malizya"}, {"Maldives", "Akud n Maldiv"}, {"Marquesas", "Akud n Tegzirin Markiz"}, {"Marshall_Islands", "Akud n Tegzirin Marcal"}, {"Mauritius", "Akud amezday n Muris"}, {"Mawson", "Akud n Mawsun"}, {"Mexico_Pacific", "Akud amezday amelwi n Miksik"}, {"Mongolia", "Akud amezday n Ulanbatar"}, {"Moscow", "Akud amezday n Muṣku"}, {"Myanmar", "Akud n Myanmar"}, {"Nauru", "Akud n Nuru"}, {"Nepal", "Akud n Nipal"}, {"New_Caledonia", "Akud amezday n Kalidunya Tamaynut"}, {"New_Zealand", "Akud amezday n Ziland Tamaynut"}, {"Newfoundland", "Akud amezday n Wakal amaynut"}, {"Niue", "Akud n Niyu"}, {"Norfolk", "Akud amezday n Tegzirt n Nurfulk"}, {"Noronha", "Akud amezday n Firnandu n Nurunha"}, {"Novosibirsk", "Akud Amagnu n Novosibirsk"}, {"Omsk", "Akud Amagnu n Omsk"}, {"Pakistan", "Akud amezday n Pakistan"}, {"Palau", "Akud n Palau"}, {"Papua_New_Guinea", "Akud n Papwa n Ɣinya Tamaynut"}, {"Paraguay", "Akud amezday n Paragway"}, {"Peru", "Akud amezday n Piru"}, {"Philippines", "Akud amezday n Filipin"}, {"Phoenix_Islands", "Akud n Tegzirin n Finiks"}, {"Pierre_Miquelon", "Akud amezday n San Pyir & Miklun"}, {"Pitcairn", "Akud n Pitkarn"}, {"Ponape", "Akud n Ponape"}, {"Pyongyang", "Akud n Pyungyung"}, {"Reunion", "Akud n Riyunyun"}, {"Rothera", "Akud n Rothera"}, {"Sakhalin", "Akud Amagnu n Sakhalin"}, {"Samoa", "Akud amezday n Samwa"}, {"Seychelles", "Akud n Saycal"}, {"Singapore", "Akud amezday n Sangapur"}, {"Solomon", "Akud n Tegzirin Salumun"}, {"South_Georgia", "Akud n Jyurjya n unẓul"}, {"Suriname", "Akud n Surinam"}, {"Syowa", "Akud n Syuwa"}, {"Tahiti", "Akud n Tahiti"}, {"Taipei", "Akud amezday n Taypay"}, {"Tajikistan", "Akud n Ṭajikistan"}, {"Tokelau", "Akud n Ṭukilaw"}, {"Tonga", "Akud amezday n Ṭunga"}, {"Truk", "Akud n Chuuk"}, {"Turkmenistan", "Akud amezday n Turkmanistan"}, {"Tuvalu", "Akud n Tuvalu"}, {"Uruguay", "Akud amezday n Urugway"}, {"Uzbekistan", "Akud amezday n Uzbikistan"}, {"Vanuatu", "Akud Amagnu n Vanuyatu"}, {"Venezuela", "Akud n Vinizwila"}, {"Vladivostok", "Akud Amagnu n Vladivostok"}, {"Volgograd", "Akud Amagnu n Volgograd"}, {"Vostok", "Akud n Vostok"}, {"Wake", "Akud n Tegzirin n Wake"}, {"Wallis", "Akud n Wallis akked Futuna"}, {"Yakutsk", "Akud amezday n Yakutsk"}, {"Yekaterinburg", "Akud Amagnu n Yekaterinburg"}},
		{{"Alaska", "Akud n unebdu n Alaska"}, {"Amazon", "Akud n unebdu n Amaẓun"}, {"America_Central", "Akud n unebdu n tlemmast n Marikan"}, {"America_Eastern", "Akud n unebdu n usammar n Marikan"}, {"America_Mountain", "Akud n unebdu n yidurar n Marikan"}, {"America_Pacific", "Akud umelwi n unebdu"}, {"Apia", "Akud n unebdu n Apya"}, {"Arabian", "Akud aɛrab n unebdu"}, {"Argentina", "Akud n unebdu n Arjuntin"}, {"Argentina_Western", "Akud n unebdu n Arjuntin n usammar"}, {"Armenia", "Akud n unebdu n Arminya"}, {"Atlantic", "Akud aṭlasan n unebdu"}, {"Australia_Central", "Akud n unebdu n Ustralya talemmast"}, {"Australia_CentralWestern", "Akud n unebdu n tlemmast n umalu n Ustralya"}, {"Australia_Eastern", "Akud n unebdu n Ustralya n usammar"}, {"Australia_Western", "Akud n unebdu Ustralya n umalu"}, {"Azerbaijan", "Akud n unebdu n Aẓerbiǧan"}, {"Azores", "Akud n unebdu n Aẓuris"}, {"Bangladesh", "Akud n unebdu n Bangladac"}, {"Brasilia", "Akud n unebdu n Braẓilya"}, {"Cape_Verde", "Akud n unebdu n Kap Vir"}, {"Chatham", "Akud n unebdu Catham"}, {"Chile", "Akud n unebdu n Cili"}, {"China", "Akud n unebdu n Cin"}, {"Colombia", "Akud n unebdu n Kulumbya"}, {"Cook", "Akud n unebdu n Tegzirin n Kuk"}, {"Cuba", "Akud n unebdu n Kuba"}, {"Easter", "Akud n unebdu n Island n usammar"}, {"Europe_Central", "Akud n unebdu n Turuft talemmast"}, {"Europe_Eastern", "Akud n unebdu n Turuft n usammar"}, {"Europe_Western", "Akud n unebdu Turuft n umalu"}, {"Falkland", "Akud n unebdu Tegzirin n Falkland"}, {"Fiji", "Akud n unebdu n Fiji"}, {"Georgia", "Akud n unebdu n Jyurjya"}, {"Greenland_Eastern", "Akud n unebdu n Grinland n usammar"}, {"Greenland_Western", "Akud n unebdu n Grinland n umalu"}, {"Hawaii_Aleutian", "Akud n unebdu n Haway-Aliwsyan"}, {"Hong_Kong", "Akud n unebdu n Hung Kung"}, {"Hovd", "Akud n unebdu n Huvd"}, {"Iran", "Akud n unebdu Iran"}, {"Irkutsk", "Akud n unebdu n Irkutsk"}, {"Israel", "Akud n unebdu n Izrayil"}, {"Japan", "Akud n unebdu n Japun"}, {"Korea", "Akud n unebdu n Kurya"}, {"Krasnoyarsk", "Akud n unebdu n Krasnoyarsk"}, {"Lord_Howe", "Akud n Unebdu n Lord Howe"}, {"Magadan", "Akud n unebdu n Magadan"}, {"Mauritius", "Akud n unebdu n Muris"}, {"Mexico_Pacific", "Akud amelwi n unebdu n Miksik"}, {"Mongolia", "Akud n unebdu n Ulanbatar"}, {"Moscow", "Akud n unebdu n Muṣku"}, {"New_Caledonia", "Akud n unebdu n Kalidunya Tamaynut"}, {"New_Zealand", "Akud n unebdu Ziland Tamaynut"}, {"Newfoundland", "Akud n unebdu n Wakal amaynut"}, {"Norfolk", "Akud n unebdu n Tegzirt n Nurfulk"}, {"Noronha", "Akud n unebdu n Firnandu n Nurunha"}, {"Novosibirsk", "Akud n Unebdu n Novosibirsk"}, {"Omsk", "Akud n Unebdu n Omsk"}, {"Pakistan", "Akud n unebdu n Pakistan"}, {"Paraguay", "Akud n unebdu n Paragway"}, {"Peru", "Akud n unebdu n Piru"}, {"Philippines", "Akud n unebdu n Filipin"}, {"Pierre_Miquelon", "Akud n unebdu n San Pyir & Miklun"}, {"Sakhalin", "Akud n Unebdu n Sakhalin"}, {"Samoa", "Akud n unebdu n Samwa"}, {"Taipei", "Akud n unebdu n Taypay"}, {"Tonga", "Akud n unebdu n Ṭunga"}, {"Turkmenistan", "Akud n unebdu n Turkmanistan"}, {"Uruguay", "Akud n unebdu n Urugway"}, {"Uzbekistan", "Akud n unebdu n Uzbikistan"}, {"Vanuatu", "Akud n Unebdu n Vanuyatu"}, {"Vladivostok", "Akud n Unebdu n Vladivostok"}, {"Volgograd", "Akud n Unebdu n Volgograd"}, {"Yakutsk", "Akud n unebdu n Yakutsk"}, {"Yekaterinburg", "Akud n Unebdu n Yekaterinburg"}},
	},
	{
		{{"Africa_Central", "Ora di Áfrika Sentral"}, {"Africa_Eastern", "Ora di Áfrika Oriental"}, {"Africa_Southern", "Ora di Sul di Áfrika"}, {"Africa_Western", "Ora di Áfrika Osidental"}, {"Alaska", "Ora Padron di Alaska"}, {"America_Central", "Ora Padron Sentral"}, {"America_Eastern", "Ora Padron Oriental"}, {"America_Mountain", "Ora Padron di Montanha"}, {"America_Pacific", "Ora Padron di Pasífiku"}, {"Atlantic", "Ora Padron di Atlántiku"}, {"Australia_Central", "Ora Padron di Austrália Sentral"}, {"Australia_CentralWestern", "Ora Padron di Austrália Sentru-Osidental"}, {"Australia_Eastern", "Ora Padron di Austrália Oriental"}, {"Australia_Western", "Ora Padron di Austrália Osidental"}, {"Cape_Verde", "Ora Padron di Kabu Verdi"}, {"Cuba", "Ora Padron di Kuba"}, {"Europe_Central", "Ora Padron di Europa Sentral"}, {"Europe_Eastern", "Ora Padron di Europa Oriental"}, {"Europe_Western", "Ora Padron di Europa Osidental"}, {"GMT", "Ora Médiu di Greenwich"}, {"Greenland_Eastern", "Ora Padron di Gronelándia Oriental"}, {"Greenland_Western", "Ora Padron di Gronelándia Osidental"}, {"Hawaii", "Ora Padron di Avaí i Aleutas"}, {"Hawaii_Aleutian", "Ora Padron di Avaí i Aleutas"}, {"Mexico_Pacific", "Ora Padron di Pasífiku Mexikanu"}, {"Newfoundland", "Ora Padron di Tera Nova"}, {"Pierre_Miquelon", "Ora Padron di San Pedru i Mikelon"}, {"Yukon", "Ora di Yukon"}},
		{{"Alaska", "Ora di Veron di Alaska"}, {"America_Central", "Ora di Veron Sentral"}, {"America_Eastern", "Ora di Veron Oriental"}, {"America_Mountain", "Ora di Veron di Montanha"}, {"America_Pacific", "Ora di Veron di Pasífiku"}, {"Atlantic", "Ora di Veron di Atlántiku"}, {"Australia_Central", "Ora di Veron di Austrália Sentral"}, {"Australia_CentralWestern", "Ora di Veron di Austrália Sentru-Osidental"}, {"Australia_Eastern", "Ora di Veron di Austrália Oriental"}, {"Australia_Western", "Ora di Veron di Austrália Osidental"}, {"Cape_Verde", "Ora di Veron de Kabu Verdi"}, {"Cuba", "Ora di Veron di Kuba"}, {"Europe_Central", "Ora di Veron di Europa Sentral"}, {"Europe_Eastern", "Ora di Veron di Europa Oriental"}, {"Europe_Western", "Ora di Veron di Europa Osidental"}, {"Greenland_Eastern", "Ora di Veron di Gronelándia Oriental"}, {"Greenland_Western", "Ora di Veron di Gronelándia Osidental"}, {"Hawaii_Aleutian", "Ora di Veron di Avaí i Aleutas"}, {"Mexico_Pacific", "Ora di Veron di Pasífiku Mexikanu"}, {"Newfoundland", "Ora di Veron di Tera Nova"}, {"Pierre_Miquelon", "Ora di Veron di San Pedru i Mikelon"}},
	},
	{
		{{"Acre", "Óra Pã Akre tá"}, {"Afghanistan", "Afeganĩtã tá óra"}, {"Africa_Central", "Afrika-Kuju tá óra"}, {"Africa_Eastern", "Afrika Rãjur tá óra"}, {"Africa_Southern", "Sur-Afrika tá óra"}, {"Africa_Western", "Afrika Rãpur tá óra"}, {"Alaska", "Óra Pã Aranhka tá"}, {"Almaty", "Óra Pã Armaty tá"}, {"Amazon", "Óra Pã Amỹjonỹ tá"}, {"America_Central", "Óra Pã Kuju tá"}, {"America_Eastern", "Óra Pã Rãjur tá"}, {"America_Mountain", "Óra Pã Krĩ tá"}, {"America_Pacific", "Óra Pã Rãpur tá"}, {"Anadyr", "Óra Pã Anỹnhyr tá"}, {"Apia", "Óra Pã Apija tá"}, {"Aqtau", "Óra Pã Agtav tá"}, {"Aqtobe", "Óra Pã Agtóme tá"}, {"Arabian", "Óra Pã Aramija tá"}, {"Argentina", "Óra Opã Arjẽtĩnỹ tá"}, {"Argentina_Western", "Óra Pã Arjẽtĩnỹ Rãpur tá"}, {"Armenia", "Óra Pã Armẽnĩja tá"}, {"Atlantic", "Óra Pã Atrỹtiku tá"}, {"Australia_Central", "Óra Pã Avotyraria Kuju tá"}, {"Australia_CentralWestern", "Óra Pã Avotyraria Kuju-Rãpur tá"}, {"Australia_Eastern", "Óra Pã Avotyraria Rãjur tá"}, {"Australia_Western", "Óra Pã Avotyraria Rãpur tá"}, {"Azerbaijan", "Óra Pã Ajermajjáv tá"}, {"Azores", "Óra Pã Asor-ag tá"}, {"Bangladesh", "Óra Pã Mỹngranési tá"}, {"Bhutan", "Óra Mutỹv tá"}, {"Bolivia", "Óra Morivia tá"}, {"Brasilia", "Óra Pã Mrasirja tá"}, {"Brunei", "Óra Mrunẽj Narusarỹ tá"}, {"Cape_Verde", "Óra Pã Pu Tánh tá"}, {"Chamorro", "Óra Samãho tá"}, {"Chatham", "Óra Pã San-hỹm tá"}, {"Chile", "Óra Pã Sire tá"}, {"China", "Óra Pã Sĩnỹ tá"}, {"Christmas", "Óra Krĩtimỹnh Goj-vẽso tá"}, {"Cocos", "Óra Kokonh Goj-vẽso tá"}, {"Colombia", "Óra Pã Korãmija tá"}, {"Cook", "Óra Pã Kuki Goj-vẽso tá"}, {"Cuba", "Óra Pã Kuma tá"}, {"Davis", "Óra Navinh tá"}, {"DumontDUrville", "Óra Numã-Nurviri tá"}, {"East_Timor", "Óra Tĩmãr-Rãjur tá"}, {"Easter", "Óra Pã Panhkuva Goj-vẽso tá"}, {"Ecuador", "Óra Ekuvanor tá"}, {"Europe_Central", "Óra Pã Orópa Kuju tá"}, {"Europe_Eastern", "Óra Pã Orópa Rãjur tá"}, {"Europe_Further_Eastern", "Rãjur tỹ Orópa jã há tá óra"}, {"Europe_Western", "Óra Pã Orópa Rãpur tá"}, {"Falkland", "Ór Pã Mỹrvĩnỹ Goj-vẽso tá"}, {"Fiji", "Óra Pã Fiji tá"}, {"French_Guiana", "Óra Frỹsa Gijanỹ tá"}, {"French_Southern", "Óra Frỹsa Ga Sur kar Ỹtartina tá"}, {"GMT", "Óra Mirinjỹnũ Grinũvisi tá"}, {"Galapagos", "Óra Gara Pago tá"}, {"Gambier", "Óra Gỹmmijer"}, {"Georgia", "Óra Pã Jeórja tá"}, {"Gilbert_Islands", "Óra Jirmértu Goj-vẽso tá"}, {"Greenland_Eastern", "Óra Pã Groẽrỹnija Rãjur tá"}, {"Greenland_Western", "Óra Pã Groẽrỹnija Rãpur tá"}, {"Guam", "Óra Pã Guvỹm tá"}, {"Gulf", "Óra Gorfu tá"}, {"Guyana", "Óra Gijỹnỹ tá"}, {"Hawaii", "Óra Pã Hava’i kar Arevta Goj-vẽso tá"}, {"Hawaii_Aleutian", "Óra Pã Hava’i kar Arevta Goj-vẽso tá"}, {"Hong_Kong", "Óra Pã Hãg Kãg tá"}, {"Hovd", "Óra Pã Hóvin tá"}, {"India", "Óra Pã Ĩnija tá"}, {"Indian_Ocean", "Óra Osiỹno Ĩniko tá"}, {"Indochina", "Óra Ĩnosĩnỹ tá"}, {"Indonesia_Central", "Óra Ĩnonẽja Kuju tá"}, {"Indonesia_Eastern", "Óra Ĩnonẽja Rãjur tá"}, {"Indonesia_Western", "Óra Ĩnonẽja Rãpur tá"}, {"Iran", "Óra Pã Irỹ tá"}, {"Irkutsk", "Óra Pã Irkutinhki tá"}, {"Israel", "Óra Pã Isihaé tá"}, {"Japan", "Óra Pã Japã tá"}, {"Kamchatka", "Óra Pã Petrupaviróvinhki-Kỹmsatinhki"}, {"Kazakhstan_Eastern", "Óra Kajakinhtỹv Rãjur tá"}, {"Kazakhstan_Western", "Óra Kajakinhtỹv Rãpur tá"}, {"Korea", "Óra Pã Koréja tá"}, {"Kosrae", "Óra de Kosiraje tá"}, {"Krasnoyarsk", "Óra Pã Kranhnãjarki tá"}, {"Kyrgystan", "Óra Kirginhtỹv tá"}, {"Lanka", "Óra Rỹnka tá"}, {"Line_Islands", "Óra Vãfe Goj-vẽso tỹ tá"}, {"Lord_Howe", "Óra Pã Rórni Hove tá"}, {"Macau", "Óra Pã Mỹkav tá"}, {"Magadan", "Óra Pã Mỹganan tá"}, {"Malaysia", "Óra Mỹraja tá"}, {"Maldives", "Óra Goj Vẽso Mỹrniva tá"}, {"Marquesas", "Óra Mỹrkeja Fag tá"}, {"Marshall_Islands", "Óra MỹrSar Goj-vẽso tá"}, {"Mauritius", "Óra Pã Mãriso tá"}, {"Mawson", "Óra Mỹusãn tá"}, {"Mexico_Pacific", "Óra Pã Mẽsiku Pasifiku tá"}, {"Mongolia", "Óra Pã Uran Mator tá"}, {"Moscow", "Óra Pã Mãnhkov tá"}, {"Myanmar", "Óra Mĩjỹmỹr tá"}, {"Nauru", "Óra Nỹvuru tá"}, {"Nepal", "Óra Nẽpar tá"}, {"New_Caledonia", "Óra Pã Karenonĩja Tãg tá"}, {"New_Zealand", "Óra Pã Jerỹnija Tãg tá"}, {"Newfoundland", "Óra Pã Ga Tãg tá"}, {"Niue", "Óra Nĩve tá"}, {"Norfolk", "Óra Pã Nãrforki Goj-vẽso tá"}, {"Noronha", "Óra Pã Fernỹnu Nãrãja-tá tá"}, {"North_Mariana", "Óra Nãrti-Mỹrijỹnỹ Goj-vẽso tá"}, {"Novosibirsk", "Óra Pã Simirsiki Tãg tá"}, {"Omsk", "Óra Pã Omĩnhki tá"}, {"Pakistan", "Óra Pã Pakinhtỹv tá"}, {"Palau", "Óra Paravu tá"}, {"Papua_New_Guinea", "Óra Papuva-Ginẽ Tãg tá"}, {"Paraguay", "Óra Pã Paraguvaj tá"}, {"Peru", "Óra Pã Piru tá"}, {"Philippines", "Óra Pã Firipinỹ tá"}, {"Phoenix_Islands", "Óra Fẽnĩg Goj-vẽso tá"}, {"Pierre_Miquelon", "Óra Pã Sỹ Pedro kar Mĩkerỹv tá"}, {"Pitcairn", "Óra Pinkajir Goj-vẽso tá"}, {"Ponape", "Óra Ponỹpe tá"}, {"Pyongyang", "Óra Piãgiỹng tá"}, {"Qyzylorda", "Óra Pã Kysyrorna tá"}, {"Reunion", "Óra Hujáv tá"}, {"Rothera", "Óra Rotera tá"}, {"Sakhalin", "Óra Pã Sakarinỹ tá"}, {"Samara", "Óra Pã Samỹra tá"}, {"Samoa", "Óra Pã Samãva tá"}, {"Seychelles", "Óra Sejserenh tá"}, {"Singapore", "Óra Pã Sĩgapura tá"}, {"Solomon", "Óra Saromỹv Goj-vẽso tá"}, {"South_Georgia", "Óra Jiórja tỹ Sur tá"}, {"Suriname", "Óra Surinỹmĩ tá"}, {"Syowa", "Óra Siova tá"}, {"Tahiti", "Óra Tajti tá"}, {"Taipei", "Óra Pã Tajpej tá"}, {"Tajikistan", "Óra Tajikinhtỹv tá"}, {"Tokelau", "Óra Tokeravu tá"}, {"Tonga", "Óra Pã Tãga tá"}, {"Truk", "Óra Suuki tá"}, {"Turkmenistan", "Óra Pã Turkomẽnĩnhtỹv tá"}, {"Tuvalu", "Óra Tuvaru tá"}, {"Uruguay", "Óra Pã Uruguvaj tá"}, {"Uzbekistan", "Óra Pã Unnmekinhtỹv tá"}, {"Vanuatu", "Óra Pã Vanũvatu tá"}, {"Venezuela", "Óra Venẽjuvéra tá"}, {"Vladivostok", "Óra Pã Uranivónhtóki tá"}, {"Volgograd", "Óra Pã Vorgugrano tá"}, {"Vostok", "Óra Vonhtóki tá"}, {"Wake", "Óra Vejki Goj-vẽso tá"}, {"Wallis", "Óra Varinh kar Futunỹ tá"}, {"Yakutsk", "Óra Pã Yjakutinhkii tá"}, {"Yekaterinburg", "Óra Pã Ekaterĩnmurgu tá"}},
		{{"Acre", "Rỹ Kã óra Akre tá"}, {"Alaska", "Rỹ Kã óra Aranhka tá"}, {"Almaty", "Rỹ Kã óra Armaty tá"}, {"Amazon", "Rỹ Kã óra Amỹjonỹ tá"}, {"America_Central", "Rỹ Kã óra Kuju tá"}, {"America_Eastern", "Rỹ Kã óra Rãjur tá"}, {"America_Mountain", "Rỹ Kã óra Krĩ tá"}, {"America_Pacific", "Rỹ Kã óra Rãpur tá"}, {"Anadyr", "Rỹ Kã óra Anỹnhyr tá"}, {"Apia", "Rỹ Kã óra Apija tá"}, {"Aqtau", "Rỹ Kã óra Agtav tá"}, {"Aqtobe", "Rỹ Kã óra Agtóme tá"}, {"Arabian", "Rỹ Kã óra Aramija tá"}, {"Argentina", "Rỹ Kã óra Arjẽtĩnỹ tá"}, {"Argentina_Western", "Rỹ Kã óra Arjẽtĩnỹ Rãpurtá"}, {"Armenia", "Rỹ Kã óra Armẽnĩja tá"}, {"Atlantic", "Rỹ Kã óra Atrỹtiku tá"}, {"Australia_Central", "Rỹ Kã óra Avotyraria Kuju tá"}, {"Australia_CentralWestern", "Rỹ Kã óra Avotyraria Kuju-Rãpur tá"}, {"Australia_Eastern", "Rỹ Kã óra Avotyraria Rãjur tá"}, {"Australia_Western", "Rỹ Kã óra Avotyraria Rãpur tá"}, {"Azerbaijan", "Rỹ Kã óra Ajermajjáv tá"}, {"Azores", "Rỹ Kã óra Aso-ag tá"}, {"Bangladesh", "Rỹ Kã óra Mỹngranési tá"}, {"Brasilia", "Rỹ Kã óra Mrasirja tá"}, {"Cape_Verde", "Rỹ Kã óra Pu tánh tá"}, {"Chatham", "Rỹ Kã óra San-hỹm tá"}, {"Chile", "Rỹ Kã óra Sire tá"}, {"China", "Rỹ Kã óra Sĩnỹ tá"}, {"Colombia", "Rỹ Kã óra Korãmija tá"}, {"Cook", "Rỹ Kã óra Kuki Goj-vẽso tá"}, {"Cuba", "Rỹ Kã óra Kuma tá"}, {"Easter", "Rỹ Kã óra Panhkuva Goj-vẽso tá"}, {"Europe_Central", "Rỹ Kã óra Orópa Kuju tá"}, {"Europe_Eastern", "Rỹ Kã óra Orópa Rãjur tá"}, {"Europe_Western", "Rỹ Kã óra Orópa Rãpur tá"}, {"Falkland", "Rỹ Kã óra Mỹrvĩnỹ Goj-vẽso tá"}, {"Fiji", "Rỹ Kã óra Fiji tá"}, {"Georgia", "Rỹ Kã óra Jeórja tá"}, {"Greenland_Eastern", "Rỹ Kã óra Groẽrỹnija Rãjur tá"}, {"Greenland_Western", "Rỹ Kã óra Groẽrỹnija Rãpur tá"}, {"Hawaii_Aleutian", "Rỹ kã óra Hava’i kar Arevta Goj-vẽso tá"}, {"Hong_Kong", "Rỹ Kã óra Hãg Kãg tá"}, {"Hovd", "Rỹ kã óra Hóvin tá"}, {"Iran", "Rỹ Kã óra Irỹ tá"}, {"Irkutsk", "Rỹ Kã óra Irkutinhki tá"}, {"Israel", "Rỹ Kã óra Isihaé tá"}, {"Japan", "Rỹ Kã óra Japã tá"}, {"Kamchatka", "Rỹ Kã óra Petrupaviróvinhki-Kỹmsatinhki"}, {"Korea", "Rỹ Kã óra Koréja tá"}, {"Krasnoyarsk", "Rỹ Kã óra Kranhnãjarki tá"}, {"Lord_Howe", "Rỹ Kã óra Rórni Hove tá"}, {"Macau", "Rỹ kã óra Mỹkav tá"}, {"Magadan", "Rỹ Kã óra Mỹganan tá"}, {"Mauritius", "Rỹ Kã óra Mãriso tá"}, {"Mexico_Pacific", "Rỹ Kã óra Mẽsiku Pasifiku tá"}, {"Mongolia", "Rỹ Kã óra Uran Mator tá"}, {"Moscow", "Rỹ Kã óra Mãnhkov tá"}, {"New_Caledonia", "Rỹ Kã óra Karenonĩja Tãg tá"}, {"New_Zealand", "Rỹ Kã óra Jerỹnija Tãg tá"}, {"Newfoundland", "Rỹ Kã óra Ga Tãg tá"}, {"Norfolk", "Rỹ Kã óra Nãrforki Goj-vẽso tá"}, {"Noronha", "Rỹ Kã óra Fernỹnu Nãrãja-tá tá"}, {"Novosibirsk", "Rỹ Kã óra Pã Simirsiki Tãg tá"}, {"Omsk", "Rỹ Kã óra Omĩnhki tá"}, {"Pakistan", "Rỹ Kã óra Pakinhtỹv tá"}, {"Paraguay", "Rỹ Kã óra Paraguvaj tá"}, {"Peru", "Rỹ Kã óra Piru tá"}, {"Philippines", "Rỹ Kã Firipinỹ tá"}, {"Pierre_Miquelon", "Rỹ Kã óra Sỹ Pedro kar Mĩkerỹv tá"}, {"Qyzylorda", "Rỹ Kã óra Kysyrorna tá"}, {"Sakhalin", "Rỹ Kã óra Sakarinỹ tá"}, {"Samara", "Rỹ Kã óra Samỹra tá"}, {"Samoa", "Rỹ Kã óra Samãva tá"}, {"Taipei", "Rỹ Kã óra Tajpej tá"}, {"Tonga", "Rỹ Kã óra Tãga tá"}, {"Turkmenistan", "Rỹ Kã óra Turkomẽnĩnhtỹv tá"}, {"Uruguay", "Rỹ Kã óra Uruguvaj tá"}, {"Uzbekistan", "Rỹ Kã óra Unhmekinhtỹv tá"}, {"Vanuatu", "Rỹ Kã óra Vanũvatu tá"}, {"Vladivostok", "Rỹ Kã óra Uranivónhtóki tá"}, {"Volgograd", "Rỹ Kã óra Vorgugrano tá"}, {"Yakutsk", "Rỹ Kã óra Yjakutinhki tá"}, {"Yekaterinburg", "Rỹ Kã óra Ekaterĩnmurgu tá"}},
	},
	{
		{{"Afghanistan", "Ауғанстан уақыты"}, {"Africa_Central", "Орталық Африка уақыты"}, {"Africa_Eastern", "Шығыс Африка уақыты"}, {"Africa_Southern", "Оңтүстік Африка стандартты уақыты"}, {"Africa_Western", "Батыс Африка уақыты"}, {"Alaska", "Аляска стандартты уақыты"}, {"Amazon", "Амазонка стандартты уақыты"}, {"America_Central", "Солтүстік Америка стандартты орталық уақыты"}, {"America_Eastern", "Солтүстік Америка стандартты шығыс уақыты"}, {"America_Mountain", "Солтүстік Америка стандартты тау уақыты"}, {"America_Pacific", "Солтүстік Америка стандартты Тынық мұхиты уақыты"}, {"Apia", "Апиа стандартты уақыты"}, {"Arabian", "Сауд Арабиясы стандартты уақыты"}, {"Argentina", "Аргентина стандартты уақыты"}, {"Argentina_Western", "Батыс Аргентина стандартты уақыты"}, {"Armenia", "Армения стандартты уақыты"}, {"Atlantic", "Атлантика стандартты уақыты"}, {"Australia_Central", "Аустралия стандартты орталық уақыты"}, {"Australia_CentralWestern", "Аустралия стандартты орталық-батыс уақыты"}, {"Australia_Eastern", "Аустралия стандартты шығыс уақыты"}, {"Australia_Western", "Аустралия стандартты батыс уақыты"}, {"Azerbaijan", "Әзірбайжан стандартты уақыты"}, {"Azores", "Азор аралдары стандартты уақыты"}, {"Bangladesh", "Бангладеш стандартты уақыты"}, {"Bhutan", "Бутан уақыты"}, {"Bolivia", "Боливия уақыты"}, {"Brasilia", "Бразилия стандартты уақыты"}, {"Brunei", "Бруней-Даруссалам уақыты"}, {"Cape_Verde", "Кабо-Верде стандартты уақыты"}, {"Chamorro", "Чаморро стандартты уақыты"}, {"Chatham", "Чатем стандартты уақыты"}, {"Chile", "Чили стандартты уақыты"}, {"China", "Қытай стандартты уақыты"}, {"Christmas", "Рождество аралының уақыты"}, {"Cocos", "Кокос аралдарының уақыты"}, {"Colombia", "Колумбия стандартты уақыты"}, {"Cook", "Кук аралдарының стандартты уақыты"}, {"Cuba", "Куба стандартты уақыты"}, {"Davis", "Дейвис уақыты"}, {"DumontDUrville", "Дюмон-д’Юрвиль уақыты"}, {"East_Timor", "Шығыс Тимор уақыты"}, {"Easter", "Пасха аралы стандартты уақыты"}, {"Ecuador", "Эквадор уақыты"}, {"Europe_Central", "Орталық Еуропа стандартты уақыты"}, {"Europe_Eastern", "Шығыс Еуропа стандартты уақыты"}, {"Europe_Further_Eastern", "Қиыр Шығыс Еуропа уақыты"}, {"Europe_Western", "Батыс Еуропа стандартты уақыты"}, {"Falkland", "Фолкленд аралдары стандартты уақыты"}, {"Fiji", "Фиджи стандартты уақыты"}, {"French_Guiana", "Француз Гвианасы уақыты"}, {"French_Southern", "Францияның оңтүстік аймағы және Антарктика уақыты"}, {"GMT", "Гринвич уақыты"}, {"Galapagos", "Галапагос уақыты"}, {"Gambier", "Гамбье уақыты"}, {"Georgia", "Грузия стандартты уақыты"}, {"Gilbert_Islands", "Гилберт аралдарының уақыты"}, {"Greenland_Eastern", "Шығыс Гренландия стандартты уақыты"}, {"Greenland_Western", "Батыс Гренландия стандартты уақыты"}, {"Gulf", "Парсы шығанағы стандартты уақыты"}, {"Guyana", "Гайана уақыты"}, {"Hawaii", "Гавай және Алеут аралдары стандартты уақыты"}, {"Hawaii_Aleutian", "Гавай және Алеут аралдары стандартты уақыты"}, {"Hong_Kong", "Гонконг стандартты уақыты"}, {"Hovd", "Ховд стандартты уақыты"}, {"India", "Үндістан стандартты уақыты"}, {"Indian_Ocean", "Үнді мұхиты уақыты"}, {"Indochina", "Үндіқытай уақыты"}, {"Indonesia_Central", "Орталық Индонезия уақыты"}, {"Indonesia_Eastern", "Шығыс Индонезия уақыты"}, {"Indonesia_Western", "Батыс Индонезия уақыты"}, {"Iran", "Иран стандартты уақыты"}, {"Irkutsk", "Иркутск стандартты уақыты"}, {"Israel", "Израиль стандартты уақыты"}, {"Japan", "Жапония стандартты уақыты"}, {"Kazakhstan", "Қазақстан уақыты"}, {"Kazakhstan_Eastern", "Шығыс Қазақстан уақыты"}, {"Kazakhstan_Western", "Батыс Қазақстан уақыты"}, {"Korea", "Корея стандартты уақыты"}, {"Kosrae", "Кусаие уақыты"}, {"Krasnoyarsk", "Красноярск стандартты уақыты"}, {"Kyrgystan", "Қырғызстан уақыты"}, {"Line_Islands", "Лайн аралдары уақыты"}, {"Lord_Howe", "Лорд-Хау стандартты уақыты"}, {"Magadan", "Магадан стандартты уақыты"}, {"Malaysia", "Малайзия уақыты"}, {"Maldives", "Мальдив аралдары уақыты"}, {"Marquesas", "Маркиз аралдары уақыты"}, {"Marshall_Islands", "Маршалл аралдары уақыты"}, {"Mauritius", "Маврикий стандартты уақыты"}, {"Mawson", "Моусон уақыты"}, {"Mexico_Pacific", "Мексика стандартты Тынық мұхит уақыты"}, {"Mongolia", "Ұланбатыр стандартты уақыты"}, {"Moscow", "Мәскеу стандартты уақыты"}, {"Myanmar", "Мьянма уақыты"}, {"Nauru", "Науру уақыты"}, {"Nepal", "Непал уақыты"}, {"New_Caledonia", "Жаңа Каледония стандартты уақыты"}, {"New_Zealand", "Жаңа Зеландия стандартты уақыты"}, {"Newfoundland", "Ньюфаундленд стандартты уақыты"}, {"Niue", "Ниуэ уақыты"}, {"Norfolk", "Норфолк аралы стандартты уақыты"}, {"Noronha", "Фернанду-ди-Норонья стандартты уақыты"}, {"Novosibirsk", "Новосібір стандартты уақыты"}, {"Omsk", "Омбы стандартты уақыты"}, {"Pakistan", "Пәкістан стандартты уақыты"}, {"Palau", "Палау уақыты"}, {"Papua_New_Guinea", "Папуа – Жаңа Гвинея уақыты"}, {"Paraguay", "Парагвай стандартты уақыты"}, {"Peru", "Перу стандартты уақыты"}, {"Philippines", "Филиппин аралдары стандартты уақыты"}, {"Phoenix_Islands", "Феникс аралдары уақыты"}, {"Pierre_Miquelon", "Сен-Пьер және Микелон стандартты уақыты"}, {"Pitcairn", "Питкэрн уақыты"}, {"Ponape", "Понпеи уақыты"}, {"Pyongyang", "Пхеньян уақыты"}, {"Reunion", "Реюньон уақыты"}, {"Rothera", "Ротера уақыты"}, {"Sakhalin", "Сахалин стандартты уақыты"}, {"Samoa", "Самоа стандартты уақыты"}, {"Seychelles", "Сейшель аралдары уақыты"}, {"Singapore", "Сингапур стандартты уақыты"}, {"Solomon", "Соломон аралдары уақыты"}, {"South_Georgia", "Оңтүстік Георгия уақыты"}, {"Suriname", "Суринам уақыты"}, {"Syowa", "Сёва уақыты"}, {"Tahiti", "Таити уақыты"}, {"Taipei", "Тайбэй стандартты уақыты"}, {"Tajikistan", "Тәжікстан уақыты"}, {"Tokelau", "Токелау уақыты"}, {"Tonga", "Тонга стандартты уақыты"}, {"Truk", "Трук уақыты"}, {"Turkmenistan", "Түрікменстан стандартты уақыты"}, {"Tuvalu", "Тувалу уақыты"}, {"Uruguay", "Уругвай стандартты уақыты"}, {"Uzbekistan", "Өзбекстан стандартты уақыты"}, {"Vanuatu", "Вануату стандартты уақыты"}, {"Venezuela", "Венесуэла уақыты"}, {"Vladivostok", "Владивосток стандартты уақыты"}, {"Volgograd", "Волгоград стандартты уақыты"}, {"Vostok", "Восток уақыты"}, {"Wake", "Уэйк аралы уақыты"}, {"Wallis", "Уоллис және Футуна уақыты"}, {"Yakutsk", "Якутск стандартты уақыты"}, {"Yekaterinburg", "Екатеринбург стандартты уақыты"}, {"Yukon", "Юкон уақыты"}},
		{{"Alaska", "Аляска жазғы уақыты"}, {"Amazon", "Амазонка жазғы уақыты"}, {"America_Central", "Солтүстік Америка жазғы орталық уақыты"}, {"America_Eastern", "Солтүстік Америка жазғы шығыс уақыты"}, {"America_Mountain", "Солтүстік Америка жазғы тау уақыты"}, {"America_Pacific", "Солтүстік Америка жазғы Тынық мұхиты уақыты"}, {"Apia", "Апиа жазғы уақыты"}, {"Arabian", "Сауд Арабиясы жазғы уақыты"}, {"Argentina", "Аргентина жазғы уақыты"}, {"Argentina_Western", "Батыс Аргентина жазғы уақыты"}, {"Armenia", "Армения жазғы уақыты"}, {"Atlantic", "Атлантика жазғы уақыты"}, {"Australia_Central", "Аустралия жазғы орталық уақыты"}, {"Australia_CentralWestern", "Аустралия жазғы орталық-батыс уақыты"}, {"Australia_Eastern", "Аустралия жазғы шығыс уақыты"}, {"Australia_Western", "Аустралия жазғы батыс уақыты"}, {"Azerbaijan", "Әзірбайжан жазғы уақыты"}, {"Azores", "Азор аралдары жазғы уақыты"}, {"Bangladesh", "Бангладеш жазғы уақыты"}, {"Brasilia", "Бразилия жазғы уақыты"}, {"Cape_Verde", "Кабо-Верде жазғы уақыты"}, {"Chatham", "Чатем жазғы уақыты"}, {"Chile", "Чили жазғы уақыты"}, {"China", "Қытай жазғы уақыты"}, {"Colombia", "Колумбия жазғы уақыты"}, {"Cook", "Кук аралдарының жартылай жазғы уақыты"}, {"Cuba", "Куба жазғы уақыты"}, {"Easter", "Пасха аралы жазғы уақыты"}, {"Europe_Central", "Орталық Еуропа жазғы уақыты"}, {"Europe_Eastern", "Шығыс Еуропа жазғы уақыты"}, {"Europe_Western", "Батыс Еуропа жазғы уақыты"}, {"Falkland", "Фолкленд аралдары жазғы уақыты"}, {"Fiji", "Фиджи жазғы уақыты"}, {"Georgia", "Грузия жазғы уақыты"}, {"Greenland_Eastern", "Шығыс Гренландия жазғы уақыты"}, {"Greenland_Western", "Батыс Гренландия жазғы уақыты"}, {"Hawaii_Aleutian", "Гавай және Алеут аралдары жазғы уақыты"}, {"Hong_Kong", "Гонконг жазғы уақыты"}, {"Hovd", "Ховд жазғы уақыты"}, {"Iran", "Иран жазғы уақыты"}, {"Irkutsk", "Иркутск жазғы уақыты"}, {"Israel", "Израиль жазғы уақыты"}, {"Japan", "Жапония жазғы уақыты"}, {"Korea", "Корея жазғы уақыты"}, {"Krasnoyarsk", "Красноярск жазғы уақыты"}, {"Lord_Howe", "Лорд-Хау жазғы уақыты"}, {"Magadan", "Магадан жазғы уақыты"}, {"Mauritius", "Маврикий жазғы уақыты"}, {"Mexico_Pacific", "Мексика жазғы Тынық мұхит уақыты"}, {"Mongolia", "Ұланбатыр жазғы уақыты"}, {"Moscow", "Мәскеу жазғы уақыты"}, {"New_Caledonia", "Жаңа Каледония жазғы уақыты"}, {"New_Zealand", "Жаңа Зеландия жазғы уақыты"}, {"Newfoundland", "Ньюфаундленд жазғы уақыты"}, {"Norfolk", "Норфолк аралы жазғы уақыты"}, {"Noronha", "Фернанду-ди-Норонья жазғы уақыты"}, {"Novosibirsk", "Новосібір жазғы уақыты"}, {"Omsk", "Омбы жазғы уақыты"}, {"Pakistan", "Пәкістан жазғы уақыты"}, {"Paraguay", "Парагвай жазғы уақыты"}, {"Peru", "Перу жазғы уақыты"}, {"Philippines", "Филиппин аралдары жазғы уақыты"}, {"Pierre_Miquelon", "Сен-Пьер және Микелон жазғы уақыты"}, {"Sakhalin", "Сахалин жазғы уақыты"}, {"Samoa", "Самоа жазғы уақыты"}, {"Taipei", "Тайбэй жазғы уақыты"}, {"Tonga", "Тонга жазғы уақыты"}, {"Turkmenistan", "Түрікменстан жазғы уақыты"}, {"Uruguay", "Уругвай жазғы уақыты"}, {"Uzbekistan", "Өзбекстан жазғы уақыты"}, {"Vanuatu", "Вануату жазғы уақыты"}, {"Vladivostok", "Владивосток жазғы уақыты"}, {"Volgograd", "Волгоград жазғы уақыты"}, {"Yakutsk", "Якутск жазғы уақыты"}, {"Yekaterinburg", "Екатеринбург жазғы уақыты"}},
	},
	{
		{{"Afghanistan", "اۋعانستان ۋاقىتى"}, {"Africa_Central", "ورتالىق افريكا ۋاقىتى"}, {"Africa_Eastern", "شىعىس افريكا ۋاقىتى"}, {"Africa_Southern", "وڭتۇستىك افريكا ستاندارتتى ۋاقىتى"}, {"Africa_Western", "باتىس افريكا ۋاقىتى"}, {"Alaska", "الاسكا ستاندارتتى ۋاقىتى"}, {"Almaty", "الماتى ستاندارتتى ۋاقىتى"}, {"Amazon", "امازون ستاندارتتى ۋاقىتى"}, {"America_Central", "سولتۇستىك امەريكا ستاندارتتى ورتالىق ۋاقىتى"}, {"America_Eastern", "سولتۇستىك امەريكا ستاندارتتى شىعىس ۋاقىتى"}, {"America_Mountain", "سولتۇستىك امەريكا ستاندارتتى تاۋ ۋاقىتى"}, {"America_Pacific", "سولتۇستىك امەريكا ستاندارتتى تىنىق مۇحيتى ۋاقىتى"}, {"Apia", "اپيا ستاندارتتى ۋاقىتى"}, {"Aqtau", "اقتاۋ ستاندارتتى ۋاقىتى"}, {"Aqtobe", "اقتوبە ستاندارتتى ۋاقىتى"}, {"Arabian", "ساۋد ارابياسى ستاندارتتى ۋاقىتى"}, {"Argentina", "ارگەنتينا ستاندارتتى ۋاقىتى"}, {"Argentina_Western", "باتىس ارگەنتينا ستاندارتتى ۋاقىتى"}, {"Armenia", "ارمەنيا ستاندارتتى ۋاقىتى"}, {"Atlantic", "اتلانتيكا ستاندارتتى ۋاقىتى"}, {"Australia_Central", "اۋستراليا ستاندارتتى ورتالىق ۋاقىتى"}, {"Australia_CentralWestern", "اۋستراليا ستاندارتتى ورتالىق-باتىس ۋاقىتى"}, {"Australia_Eastern", "اۋستراليا ستاندارتتى شىعىس ۋاقىتى"}, {"Australia_Western", "اۋستراليا ستاندارتتى باتىس ۋاقىتى"}, {"Azerbaijan", "ءازىربايجان ستاندارتتى ۋاقىتى"}, {"Azores", "ازور ارالدارى ستاندارتتى ۋاقىتى"}, {"Bangladesh", "بانگلادەش ستاندارتتى ۋاقىتى"}, {"Bhutan", "بۋتان ۋاقىتى"}, {"Bolivia", "بوليۆيا ۋاقىتى"}, {"Brasilia", "برازيليا ستاندارتتى ۋاقىتى"}, {"Brunei", "برۋنەي-دارۋسسالام ۋاقىتى"}, {"Cape_Verde", "كابو-ۆەردە ستاندارتتى ۋاقىتى"}, {"Chamorro", "چاموررو ستاندارتتى ۋاقىتى"}, {"Chatham", "چاتەم ستاندارتتى ۋاقىتى"}, {"Chile", "چيلي ستاندارتتى ۋاقىتى"}, {"China", "قىتاي ستاندارتتى ۋاقىتى"}, {"Christmas", "كريستماس ارالىنىڭ ۋاقىتى"}, {"Cocos", "كوكوس ارالدارىنىڭ ۋاقىتى"}, {"Colombia", "كولۋمبيا ستاندارتتى ۋاقىتى"}, {"Cook", "كۋك ارالدارىنىڭ ستاندارتتى ۋاقىتى"}, {"Cuba", "كۋبا ستاندارتتى ۋاقىتى"}, {"Davis", "دەيۆيس ۋاقىتى"}, {"DumontDUrville", "ديۋمون-ديۋرۆيل ۋاقىتى"}, {"East_Timor", "شىعىس تيمور ۋاقىتى"}, {"Easter", "پاسحا ارالى ستاندارتتى ۋاقىتى"}, {"Ecuador", "ەكۆادور ۋاقىتى"}, {"Europe_Central", "ورتالىق ەۋروپا ستاندارتتى ۋاقىتى"}, {"Europe_Eastern", "شىعىس ەۋروپا ستاندارتتى ۋاقىتى"}, {"Europe_Further_Eastern", "قيىر شىعىس ەۋروپا ۋاقىتى"}, {"Europe_Western", "باتىس ەۋروپا ستاندارتتى ۋاقىتى"}, {"Falkland", "فولكلەند ارالدارى ستاندارتتى ۋاقىتى"}, {"Fiji", "فيجي ستاندارتتى ۋاقىتى"}, {"French_Guiana", "فرانتسۋز گۆياناسى ۋاقىتى"}, {"French_Southern", "فرانتسيانىڭ وڭتۇستىك ايماعى جانە انتاركتيكا ۋاقىتى"}, {"GMT", "گرينۆيچ ۋاقىتى"}, {"Galapagos", "گالاپاگوس ۋاقىتى"}, {"Gambier", "گامبە ۋاقىتى"}, {"Georgia", "گرۋزيا ستاندارتتى ۋاقىتى"}, {"Gilbert_Islands", "گيلبەرت ارالدارىنىڭ ۋاقىتى"}, {"Greenland_Eastern", "شىعىس گرەنلانديا ستاندارتتى ۋاقىتى"}, {"Greenland_Western", "باتىس گرەنلانديا ستاندارتتى ۋاقىتى"}, {"Gulf", "پارسى شىعاناعى ستاندارتتى ۋاقىتى"}, {"Guyana", "گايانا ۋاقىتى"}, {"Hawaii", "گاۆاي جانە الەۋت ارالدارى ستاندارتتى ۋاقىتى"}, {"Hawaii_Aleutian", "گاۆاي جانە الەۋت ارالدارى ستاندارتتى ۋاقىتى"}, {"Hong_Kong", "حوڭكوڭ ستاندارتتى ۋاقىتى"}, {"Hovd", "حوۆد ستاندارتتى ۋاقىتى"}, {"India", "ءۇندىستان ستاندارتتى ۋاقىتى"}, {"Indian_Ocean", "ءۇندى مۇحيتى ۋاقىتى"}, {"Indochina", "ءۇندى-قىتاي ۋاقىتى"}, {"Indonesia_Central", "ورتالىق يندونەزيا ۋاقىتى"}, {"Indonesia_Eastern", "شىعىس يندونەزيا ۋاقىتى"}, {"Indonesia_Western", "باتىس يندونەزيا ۋاقىتى"}, {"Iran", "يران ستاندارتتى ۋاقىتى"}, {"Irkutsk", "يركۋتسك ستاندارتتى ۋاقىتى"}, {"Israel", "يزرايل ستاندارتتى ۋاقىتى"}, {"Japan", "جاپونيا ستاندارتتى ۋاقىتى"}, {"Kazakhstan", "قازاق ەلى ۋاقىتى"}, {"Kazakhstan_Eastern", "شىعىس قازاق ەلى ۋاقىتى"}, {"Kazakhstan_Western", "باتىس قازاق ەلى ۋاقىتى"}, {"Korea", "كورەيا ستاندارتتى ۋاقىتى"}, {"Kosrae", "كۋسايە ۋاقىتى"}, {"Krasnoyarsk", "كراسنويارسك ستاندارتتى ۋاقىتى"}, {"Kyrgystan", "قىرعىزستان ۋاقىتى"}, {"Line_Islands", "لاين ارالدارى ۋاقىتى"}, {"Lord_Howe", "لورد-حاۋ ستاندارتتى ۋاقىتى"}, {"Magadan", "ماگادان ستاندارتتى ۋاقىتى"}, {"Malaysia", "مالايزيا ۋاقىتى"}, {"Maldives", "مالديۆ ارالدارى ۋاقىتى"}, {"Marquesas", "ماركيز ارالدارى ۋاقىتى"}, {"Marshall_Islands", "مارشال ارالدارى ۋاقىتى"}, {"Mauritius", "ماۆريكيي ستاندارتتى ۋاقىتى"}, {"Mawson", "موۋسون ۋاقىتى"}, {"Mexico_Pacific", "مەكسيكا ستاندارتتى تىنىق مۇحيت ۋاقىتى"}, {"Mongolia", "ۇلانباتىر ستاندارتتى ۋاقىتى"}, {"Moscow", "ماسكەۋ ستاندارتتى ۋاقىتى"}, {"Myanmar", "ميانما ۋاقىتى"}, {"Nauru", "ناۋرۋ ۋاقىتى"}, {"Nepal", "نەپال ۋاقىتى"}, {"New_Caledonia", "جاڭا كالەدونيا ستاندارتتى ۋاقىتى"}, {"New_Zealand", "جاڭا زەلانديا ستاندارتتى ۋاقىتى"}, {"Newfoundland", "نيۋفاۋندلەند ستاندارتتى ۋاقىتى"}, {"Niue", "نيۋە ۋاقىتى"}, {"Norfolk", "نورفولك ارالى ستاندارتتى ۋاقىتى"}, {"Noronha", "فەرناندۋ-دي-نورونيا ستاندارتتى ۋاقىتى"}, {"Novosibirsk", "جاڭاسىبىر ستاندارتتى ۋاقىتى"}, {"Omsk", "ومبى ستاندارتتى ۋاقىتى"}, {"Pakistan", "پاكىستان ستاندارتتى ۋاقىتى"}, {"Palau", "پالاۋ ۋاقىتى"}, {"Papua_New_Guinea", "پاپۋا – جاڭا گۆينەيا ۋاقىتى"}, {"Paraguay", "پاراگۆاي ستاندارتتى ۋاقىتى"}, {"Peru", "پەرۋ ستاندارتتى ۋاقىتى"}, {"Philippines", "فيليپين ارالدارى ستاندارتتى ۋاقىتى"}, {"Phoenix_Islands", "فەنيكس ارالدارى ۋاقىتى"}, {"Pierre_Miquelon", "سەن-پەر جانە ميكەلون ستاندارتتى ۋاقىتى"}, {"Pitcairn", "پيتكەرن ۋاقىتى"}, {"Ponape", "پونپەي ۋاقىتى"}, {"Pyongyang", "پحەنيان ۋاقىتى"}, {"Qyzylorda", "قىزىلوردا ستاندارتتى ۋاقىتى"}, {"Reunion", "رەيۋنون ۋاقىتى"}, {"Rothera", "روتەرا ۋقىتى"}, {"Sakhalin", "ساحالين ستاندارتتى ۋاقىتى"}, {"Samoa", "ساموا ستاندارتتى ۋاقىتى"}, {"Seychelles", "سەيشەل ارالدارى ۋاقىتى"}, {"Singapore", "سينگاپۋر ستاندارتتى ۋاقىتى"}, {"Solomon", "سولومون ارالدارى ۋاقىتى"}, {"South_Georgia", "وڭتۇستىك گەورگيا ۋاقىتى"}, {"Suriname", "سۋرينام ۋاقىتى"}, {"Syowa", "سەۆا ۋاقىتى"}, {"Tahiti", "تايتي ۋاقىتى"}, {"Taipei", "تايبەي ستاندارتتى ۋاقىتى"}, {"Tajikistan", "تاجىكستان ۋاقىتى"}, {"Tokelau", "توكەلاۋ ۋاقىتى"}, {"Tonga", "تونگا ستاندارتتى ۋاقىتى"}, {"Truk", "ترۋك ۋاقىتى"}, {"Turkmenistan", "تۇرىكمەنستان ستاندارتتى ۋاقىتى"}, {"Tuvalu", "تۋۆالۋ ۋاقىتى"}, {"Uruguay", "ۋرۋگۆاي ستاندارتتى ۋاقىتى"}, {"Uzbekistan", "وزبەكستان ستاندارتتى ۋاقىتى"}, {"Vanuatu", "ۆانۋاتۋ ستاندارتتى ۋاقىتى"}, {"Venezuela", "ۆەنەسۋەلا ۋاقىتى"}, {"Vladivostok", "ۆلاديۆوستوك ستاندارتتى ۋاقىتى"}, {"Volgograd", "ۆولگوگراد ستاندارتتى ۋاقىتى"}, {"Vostok", "ۆوستوك ۋاقىتى"}, {"Wake", "ۋەيك ارالى ۋاقىتى"}, {"Wallis", "ۋولليس جانە فۋتۋنا ۋاقىتى"}, {"Yakutsk", "ياكۋتسك ستاندارتتى ۋاقىتى"}, {"Yekaterinburg", "ەكاتەرينبۋرگ ستاندارتتى ۋاقىتى"}, {"Yukon", "يۋكون ۋاقىتى"}},
		{{"Alaska", "الاسكا جازعى ۋاقىتى"}, {"Almaty", "الماتى جازعى ۋاقىتى"}, {"Amazon", "امازون جازعى ۋاقىتى"}, {"America_Central", "ولتۇستىك امەريكا جازعى ورتالىق ۋاقىتى"}, {"America_Eastern", "سولتۇستىك امەريكا جازعى شىعىس ۋاقىتى"}, {"America_Mountain", "سولتۇستىك امەريكا جازعى تاۋ ۋاقىتى"}, {"America_Pacific", "سولتۇستىك امەريكا جازعى تىنىق مۇحيتى ۋاقىتى"}, {"Apia", "اپيا جازعى ۋاقىتى"}, {"Aqtau", "اقتاۋ جازعى ۋاقىتى"}, {"Aqtobe", "اقتوبە جازعى ۋاقىتى"}, {"Arabian", "ساۋد ارابياسى جازعى ۋاقىتى"}, {"Argentina", "ارگەنتينا جازعى ۋاقىتى"}, {"Argentina_Western", "باتىس ارگەنتينا جازعى ۋاقىتى"}, {"Armenia", "ارمەنيا جازعى ۋاقىتى"}, {"Atlantic", "اتلانتيكا جازعى ۋاقىتى"}, {"Australia_Central", "اۋستراليا جازعى ورتالىق ۋاقىتى"}, {"Australia_CentralWestern", "اۋستراليا جازعى ورتالىق-باتىس ۋاقىتى"}, {"Australia_Eastern", "اۋستراليا جازعى شىعىس ۋاقىتى"}, {"Australia_Western", "اۋستراليا جازعى باتىس ۋاقىتى"}, {"Azerbaijan", "ءازىربايجان جازعى ۋاقىتى"}, {"Azores", "ازور ارالدارى جازعى ۋاقىتى"}, {"Bangladesh", "بانگلادەش جازعى ۋاقىتى"}, {"Brasilia", "برازيليا جازعى ۋاقىتى"}, {"Cape_Verde", "كابو-ۆەردە جازعى ۋاقىتى"}, {"Chatham", "چاتەم جازعى ۋاقىتى"}, {"Chile", "چيلي جازعى ۋاقىتى"}, {"China", "قىتاي جازعى ۋاقىتى"}, {"Colombia", "كولۋمبيا جازعى ۋاقىتى"}, {"Cook", "كۋك ارالدارىنىڭ جازعى ۋاقىتى"}, {"Cuba", "كۋبا جازعى ۋاقىتى"}, {"Easter", "پاسحا ارالى جازعى ۋاقىتى"}, {"Europe_Central", "ورتالىق ەۋروپا جازعى ۋاقىتى"}, {"Europe_Eastern", "شىعىس ەۋروپا جازعى ۋاقىتى"}, {"Europe_Western", "باتىس ەۋروپا جازعى ۋاقىتى"}, {"Falkland", "فولكلەند ارالدارى جازعى ۋاقىتى"}, {"Fiji", "فيجي جازعى ۋاقىتى"}, {"Georgia", "گرۋزيا جازعى ۋاقىتى"}, {"Greenland_Eastern", "شىعىس گرەنلانديا جازعى ۋاقىتى"}, {"Greenland_Western", "باتىس گرەنلانديا جازعى ۋاقىتى"}, {"Hawaii_Aleutian", "گاۆاي جانە الەۋت ارالدارى جازعى ۋاقىتى"}, {"Hong_Kong", "حوڭكوڭ جازعى ۋاقىتى"}, {"Hovd", "حوۆد جازعى ۋاقىتى"}, {"Iran", "يران جازعى ۋاقىتى"}, {"Irkutsk", "يركۋتسك جازعى ۋاقىتى"}, {"Israel", "يزرايل جازعى ۋاقىتى"}, {"Japan", "جاپونيا جازعى ۋاقىتى"}, {"Korea", "كورەيا جازعى ۋاقىتى"}, {"Krasnoyarsk", "كراسنويارسك جازعى ۋاقىتى"}, {"Lord_Howe", "لورد-حاۋ جازعى ۋاقىتى"}, {"Magadan", "ماگادان جازعى ۋاقىتى"}, {"Mauritius", "ماۆريكيي جازعى ۋاقىتى"}, {"Mexico_Pacific", "مەكسيكا جازعى تىنىق مۇحيت ۋاقىتى"}, {"Mongolia", "ۇلانباتىر جازعى ۋاقىتى"}, {"Moscow", "ماسكەۋ جازعى ۋاقىتى"}, {"New_Caledonia", "جاڭا كالەدونيا جازعى ۋاقىتى"}, {"New_Zealand", "جاڭا زەلانديا جازعى ۋاقىتى"}, {"Newfoundland", "نيۋفاۋندلەند جازعى ۋاقىتى"}, {"Norfolk", "نورفولك ارالى جازعى ۋاقىتى"}, {"Noronha", "فەرناندۋ-دي-نورونيا جازعى ۋاقىتى"}, {"Novosibirsk", "جاڭاسىبىر جازعى ۋاقىتى"}, {"Omsk", "ومبى جازعى ۋاقىتى"}, {"Pakistan", "پاكىستان جازعى ۋاقىتى"}, {"Paraguay", "پاراگۆاي جازعى ۋاقىتى"}, {"Peru", "پەرۋ جازعى ۋاقىتى"}, {"Philippines", "فيليپين ارالدارى جازعى ۋاقىتى"}, {"Pierre_Miquelon", "سەن-پەر جانە ميكەلون جازعى ۋاقىتى"}, {"Qyzylorda", "قىزىلوردا جازعى ۋاقىتى"}, {"Sakhalin", "ساحالين جازعى ۋاقىتى"}, {"Samoa", "ساموا جازعى ۋاقىتى"}, {"Taipei", "تايبەي جازعى ۋاقىتى"}, {"Tonga", "تونگا كازعى ۋاقىتى"}, {"Turkmenistan", "تۇرىكمەنستان جازعى ۋاقىتى"}, {"Uruguay", "ۋرۋگۆاي جازعى ۋاقىتى"}, {"Uzbekistan", "وزبەكستان جازعى ۋاقىتى"}, {"Vanuatu", "ۆانۋاتۋ جازعى ۋاقىتى"}, {"Vladivostok", "ۆلاديۆوستوك جازعى ۋاقىتى"}, {"Volgograd", "ۆولگوگراد جازعى ۋاقىتى"}, {"Yakutsk", "ياكۋتسك جازعى ۋاقىتى"}, {"Yekaterinburg", "ەكاتەرينبۋرگ جازعى ۋاقىتى"}},
	},
	{
		{{"Afghanistan", "ម៉ោង\u200bនៅ\u200bអាហ្វហ្គានីស្ថាន"}, {"Africa_Central", "ម៉ោង\u200bនៅ\u200bអាហ្វ្រិក\u200bកណ្ដាល"}, {"Africa_Eastern", "ម៉ោង\u200bនៅ\u200bអាហ្វ្រិក\u200bខាង\u200bកើត"}, {"Africa_Southern", "ម៉ោង\u200bនៅ\u200bអាហ្វ្រិក\u200bខាង\u200bត្បូង"}, {"Africa_Western", "ម៉ោង\u200bនៅ\u200bអាហ្វ្រិក\u200bខាង\u200bលិច"}, {"Alaska", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអាឡាស្កា"}, {"Amazon", "ម៉ោងស្តង់ដារ\u200bនៅ\u200bអាម៉ាហ្សូន"}, {"America_Central", "ម៉ោង\u200b\u200bស្តង់ដារនៅ\u200bទ្វីបអាមេរិក\u200bខាង\u200bជើងភាគកណ្តាល"}, {"America_Eastern", "ម៉ោងស្តង់ដារនៅទ្វីបអាមេរិកខាងជើងភាគខាងកើត"}, {"America_Mountain", "ម៉ោងស្តង់ដារ\u200bនៅតំបន់ភ្នំនៃទ្វីប\u200bអាមេរិក\u200b\u200b\u200bខាង\u200bជើង"}, {"America_Pacific", "ម៉ោងស្ដង់ដារនៅប៉ាស៊ីហ្វិកអាមេរិក"}, {"Apia", "ម៉ោង\u200bស្តង់ដា\u200bនៅ\u200bអាប្យា"}, {"Arabian", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអារ៉ាប់"}, {"Argentina", "ម៉ោងស្តង់ដារ\u200bនៅ\u200bអាហ្សង់ទីន"}, {"Argentina_Western", "ម៉ោងស្តង់ដារ\u200bនៅ\u200bអាហ្សង់ទីនភាគខាងលិច"}, {"Armenia", "ម៉ោង\u200bស្ដង់ដារ\u200bនៅ\u200bអាមេនី"}, {"Atlantic", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអាត្លង់ទិក"}, {"Australia_Central", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអូស្ត្រាលី\u200bកណ្ដាល"}, {"Australia_CentralWestern", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bភាគ\u200bខាង\u200bលិច\u200bនៃ\u200bអូស្ត្រាលី\u200bកណ្ដាល"}, {"Australia_Eastern", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអូស្ត្រាលី\u200bខាង\u200bកើត"}, {"Australia_Western", "ម៉ោង\u200b\u200bស្តង់ដារ\u200bនៅ\u200bអូស្ត្រាលី\u200bខាង\u200bលិច"}, {"Azerbaijan", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអាស៊ែបៃហ្សង់"}, {"Azores", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអេហ្សស"}, {"Bangladesh", "ម៉ោង\u200bស្ដង់ដារ\u200bនៅ\u200bបង់ក្លាដែស"}, {"Bhutan", "ម៉ោងនៅប៊ូតង់"}, {"Bolivia", "ម៉ោង\u200bនៅ\u200bបូលីវី"}, {"Brasilia", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bប្រាស៊ីលីយ៉ា"}, {"Brunei", "ម៉ោងនៅព្រុយណេដារូសាឡឹម"}, {"Cape_Verde", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bកាប់វែរ"}, {"Chamorro", "ម៉ោង\u200bស្តង់ដារនៅ\u200bចាំម៉ូរ៉ូ"}, {"Chatham", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bចាថាំ"}, {"Chile", "ម៉ោងស្តង់ដារនៅស៊ីលី"}, {"China", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bចិន"}, {"Christmas", "ម៉ោង\u200bនៅ\u200bកោះ\u200bគ្រីស្មាស"}, {"Cocos", "ម៉ោង\u200bនៅ\u200bប្រជុំកោះ\u200bកូកូស"}, {"Colombia", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bកូឡុំប៊ី"}, {"Cook", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bប្រជុំកោះ\u200bខូក"}, {"Cuba", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bគុយបា"}, {"Davis", "ម៉ោង\u200bនៅ\u200bដាវីស"}, {"DumontDUrville", "ម៉ោង\u200bនៅ\u200bឌុយម៉ុងដឺអ៊ុយវីល"}, {"East_Timor", "ម៉ោង\u200bនៅ\u200b\u200bទីម័រ\u200bខាង\u200bកើត"}, {"Easter", "ម៉ោងស្តង់ដារនៅកោះអ៊ីស្ទ័រ"}, {"Ecuador", "ម៉ោង\u200bនៅ\u200bអេក្វាទ័រ"}, {"Europe_Central", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអឺរ៉ុប\u200bកណ្ដាល"}, {"Europe_Eastern", "ម៉ោង\u200bស្តង់ដារ\u200b\u200bនៅ\u200bអឺរ៉ុប\u200b\u200bខាង\u200bកើត\u200b"}, {"Europe_Further_Eastern", "ម៉ោង\u200bនៅ\u200bចុងបូព៌ានៃទ្វីប\u200bអឺរ៉ុប\u200b"}, {"Europe_Western", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអឺរ៉ុប\u200bខាង\u200bលិច"}, {"Falkland", "ម៉ោង\u200bស្តង់ដារ\u200bនៅប្រជុំ\u200bកោះ\u200bហ្វក់ឡែន"}, {"Fiji", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bហ្វីជី"}, {"French_Guiana", "ម៉ោង\u200bនៅ\u200bហ្គីយ៉ាន\u200bបារាំង"}, {"French_Southern", "ម៉ោងនៅបារាំងខាងត្បូង និងនៅអង់តាំងទិក"}, {"GMT", "ម៉ោងនៅគ្រីនវិច"}, {"Galapagos", "ម៉ោង\u200bនៅ\u200bកាឡាប៉ាកូស"}, {"Gambier", "ម៉ោង\u200bនៅ\u200bកាំបៀ"}, {"Georgia", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bហ្សកហ្ស៊ី"}, {"Gilbert_Islands", "ម៉ោង\u200bនៅ\u200bកោះ\u200bកីប៊ឺត"}, {"Greenland_Eastern", "ម៉ោង\u200b\u200b\u200bស្តង់ដារ\u200bនៅ\u200b\u200bហ្គ្រីនលែន\u200bខាង\u200bកើត"}, {"Greenland_Western", "ម៉ោងស្តង់ដារនៅហ្គ្រីនលែនខាងលិច"}, {"Gulf", "ម៉ោង\u200bស្តង់ដា\u200bនៅ\u200bកាល់"}, {"Guyana", "ម៉ោង\u200bនៅ\u200bហ្គីយ៉ាន"}, {"Hawaii", "ម៉ោង\u200bស្តង់ដារ\u200b\u200bនៅ\u200bហាវៃ-អាល់ដ្យូសិន"}, {"Hawaii_Aleutian", "ម៉ោង\u200bស្តង់ដារ\u200b\u200bនៅ\u200bហាវៃ-អាល់ដ្យូសិន"}, {"Hong_Kong", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bហុងកុង"}, {"Hovd", "ម៉ោង\u200bស្តង់ដារ\u200bនៅហូវ"}, {"India", "ម៉ោង\u200bស្តង់ដារនៅ\u200bឥណ្ឌា"}, {"Indian_Ocean", "ម៉ោង\u200bនៅ\u200bមហាសមុទ្រ\u200bឥណ្ឌា"}, {"Indochina", "ម៉ោង\u200bនៅ\u200bឥណ្ឌូចិន"}, {"Indonesia_Central", "ម៉ោង\u200bនៅ\u200bឥណ្ឌូណេស៊ី\u200b\u200b\u200bកណ្ដាល"}, {"Indonesia_Eastern", "ម៉ោង\u200bនៅ\u200bឥណ្ឌូណេស៊ី\u200b\u200bខាង\u200bកើត"}, {"Indonesia_Western", "ម៉ោង\u200bនៅ\u200bឥណ្ឌូណេស៊ី\u200b\u200bខាង\u200bលិច"}, {"Iran", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ីរ៉ង់"}, {"Irkutsk", "ម៉ោងស្តង់ដារនៅអៀរគុតស្កិ៍"}, {"Israel", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ីស្រាអែល"}, {"Japan", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bជប៉ុន"}, {"Kazakhstan", "ពេលវេលានៅកាហ្សាក់ស្ថាន"}, {"Kazakhstan_Eastern", "ម៉ោង\u200bកាហ្សាក់ស្ថាន\u200b\u200bខាង\u200bកើត"}, {"Kazakhstan_Western", "ម៉ោង\u200bនៅ\u200bកាហ្សាក់ស្ថាន\u200bខាង\u200b\u200b\u200bលិច"}, {"Korea", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bកូរ៉េ"}, {"Kosrae", "ម៉ោង\u200bនៅ\u200bកូស្រៃ"}, {"Krasnoyarsk", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bក្រាណូយ៉ាស"}, {"Kyrgystan", "ម៉ោងនៅកៀហ្ស៊ីស៊ីស្ថាន"}, {"Line_Islands", "ម៉ោង\u200bនៅ\u200bកោះ\u200bឡាញ"}, {"Lord_Howe", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bឡត\u200bហៅ"}, {"Magadan", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bម៉ាហ្កាដាន"}, {"Malaysia", "ម៉ោង\u200bនៅ\u200bម៉ាឡេស៊ី"}, {"Maldives", "ម៉ោង\u200bនៅ\u200bម៉ាល់ឌីវ"}, {"Marquesas", "ម៉ោង\u200bនៅ\u200bកោះ\u200bម៉ាគឺសាស់"}, {"Marshall_Islands", "ម៉ោង\u200bនៅ\u200bម៉ាសាល"}, {"Mauritius", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bម៉ូរីស"}, {"Mawson", "ម៉ោង\u200bនៅ\u200bម៉ៅ\u200bសាន់"}, {"Mexico_Pacific", "ម៉ោង\u200bស្តង់ដា\u200bនៅ\u200bប៉ាស៊ីហ្វិក\u200bម៉ិកស៊ិក"}, {"Mongolia", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ូឡាន\u200bបាទូ"}, {"Moscow", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bមូស្គូ"}, {"Myanmar", "ម៉ោង\u200bនៅ\u200bមីយ៉ាន់ម៉ា"}, {"Nauru", "ម៉ោង\u200bនៅ\u200bណូរូ"}, {"Nepal", "ម៉ោងនៅនេប៉ាល់"}, {"New_Caledonia", "ម៉ោងស្តង់ដារ\u200bនៅណូវ៉ែលកាឡេដូនៀ"}, {"New_Zealand", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bនូវែលសេឡង់"}, {"Newfoundland", "ម៉ោង\u200b\u200bស្តង់ដារ\u200b\u200bនៅ\u200bញូវហ្វោនឡែន"}, {"Niue", "ម៉ោងនៅ\u200bនីវ៉េ"}, {"Norfolk", "ម៉ោង\u200bស្ដង់ដារ\u200bនៅ\u200bណ័រហ្វក់"}, {"Noronha", "ម៉ោង\u200bស្តង់ដារនៅហ្វ៊ែណាន់ដូ\u200bដឺណូរ៉ូញ៉ា"}, {"Novosibirsk", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bណូវ៉ូស៊ីប៊ីក"}, {"Omsk", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអូម"}, {"Pakistan", "ម៉ោង\u200bស្ដង់ដារ\u200bនៅ\u200bប៉ាគីស្ថាន"}, {"Palau", "ម៉ោង\u200bនៅ\u200bផាឡៅ"}, {"Papua_New_Guinea", "ម៉ោង\u200bនៅប៉ាពូអាស៊ី នូវែលហ្គីណេ"}, {"Paraguay", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bប៉ារ៉ាហ្គាយ"}, {"Peru", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bប៉េរូ"}, {"Philippines", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bហ្វីលីពីន"}, {"Phoenix_Islands", "ម៉ោង\u200bនៅ\u200bកោះ\u200bផូនីក"}, {"Pierre_Miquelon", "ម៉ោង\u200bស្តង់ដារ\u200bនៅសង់\u200bព្យែរ និង\u200bមីគុយឡុង"}, {"Pitcairn", "ម៉ោង\u200bនៅ\u200bភីឃឺន"}, {"Ponape", "ម៉ោង\u200bនៅ\u200bប៉ូណាប់"}, {"Pyongyang", "ម៉ោងនៅព្យុងយ៉ាង"}, {"Reunion", "ម៉ោងនៅរេអ៊ុយ៉ុង"}, {"Rothera", "ម៉ោង\u200bនៅ\u200bរ៉ូធឺរ៉ា"}, {"Sakhalin", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bសាក់ខាលីន"}, {"Samoa", "ម៉ោង\u200bស្តង់ដារនៅ\u200bសាម័រ"}, {"Seychelles", "ម៉ោង\u200bនៅ\u200bសីស្ហែល"}, {"Singapore", "ម៉ោង\u200bនៅ\u200bសិង្ហបូរី"}, {"Solomon", "ម៉ោង\u200bនៅ\u200bកោះ\u200bសូឡូម៉ុន"}, {"South_Georgia", "ម៉ោង\u200bនៅ\u200bកោះ\u200bហ្សកហ្ស៊ី"}, {"Suriname", "ម៉ោង\u200bនៅ\u200bសូរីណាម"}, {"Syowa", "ម៉ោង\u200bនៅ\u200bស៊ីអូវ៉ា"}, {"Tahiti", "ម៉ោង\u200bនៅ\u200bតាហិទី"}, {"Taipei", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bតៃប៉ិ"}, {"Tajikistan", "ម៉ោងនៅតាជីគីស្ថាន"}, {"Tokelau", "ម៉ោង\u200bនៅ\u200bតូខេឡៅ"}, {"Tonga", "ម៉ោង\u200bស្តង់ដារ\u200b\u200bនៅ\u200bតុងហ្គា"}, {"Truk", "ម៉ោង\u200bនៅ\u200bចូអុក"}, {"Turkmenistan", "ម៉ោង\u200bស្តង់ដារ\u200bនៅតួកម៉េនីស្ថាន"}, {"Tuvalu", "ម៉ោង\u200bនៅ\u200bទុយវ៉ាលូ"}, {"Uruguay", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ុយរូហ្គាយ"}, {"Uzbekistan", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ូសបេគីស្ថាន"}, {"Vanuatu", "ម៉ោង\u200b\u200bស្តង់ដារ\u200bនៅ\u200bវ៉ានូទូ"}, {"Venezuela", "ម៉ោង\u200bនៅ\u200bវ៉េណេស៊ុយអេឡា"}, {"Vladivostok", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bវ៉្លាឌីវ៉ូស្តុក"}, {"Volgograd", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bវ៉ូហ្កោក្រាដ"}, {"Vostok", "ម៉ោង\u200bនៅ\u200bវ័រស្តុក"}, {"Wake", "ម៉ោង\u200bនៅ\u200bកោះវេក"}, {"Wallis", "ម៉ោង\u200bនៅ\u200bវ៉ាលីស និងហ្វ៊ុទូណា"}, {"Yakutsk", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bយ៉ាគុតស្កិ៍"}, {"Yekaterinburg", "ម៉ោង\u200bស្តង់ដារ\u200bនៅ\u200bអ៊ិខាធឺរីនប៊័ក"}, {"Yukon", "ម៉ោងនៅយូខន់"}},
		{{"Alaska", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200b\u200bអាឡាស្កា"}, {"Amazon", "ម៉ោង\u200bនៅ\u200bអាម៉ាហ្សូននារដូវក្តៅ"}, {"America_Central", "ម៉ោង\u200b\u200bពេលថ្ងៃនៅ\u200bទ្វីបអាមេរិក\u200bខាង\u200bជើងភាគកណ្តាល"}, {"America_Eastern", "ម៉ោងពេលថ្ងៃនៅទ្វីបអាមេរិកខាងជើងភាគខាងកើត"}, {"America_Mountain", "ម៉ោង\u200bពេលថ្ងៃនៅតំបន់ភ្នំនៃទ្វីប\u200bអាមេរិក\u200b\u200b\u200bខាង\u200bជើង"}, {"America_Pacific", "ម៉ោងពេលថ្ងៃនៅប៉ាស៊ីហ្វិកអាមេរិក"}, {"Apia", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអាប្យា"}, {"Arabian", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអារ៉ាប់"}, {"Argentina", "ម៉ោង\u200bនៅ\u200bអាហ្សង់ទីននារដូវក្តៅ"}, {"Argentina_Western", "ម៉ោង\u200bនៅ\u200bអាហ្សង់ទីនភាគខាងលិចនារដូវក្តៅ"}, {"Armenia", "ម៉ោង\u200bនៅ\u200bអាមេនីនារដូវ\u200bក្ដៅ\u200b"}, {"Atlantic", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអាត្លង់ទិក"}, {"Australia_Central", "ម៉ោង\u200bពេលថ្ងៃ\u200b\u200b\u200b\u200bនៅ\u200bអូស្ត្រាលី\u200bកណ្ដាល"}, {"Australia_CentralWestern", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200b\u200bភាគ\u200bខាង\u200bលិច\u200bនៃ\u200bអូស្ត្រាលី\u200bកណ្ដាល"}, {"Australia_Eastern", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអូស្ត្រាលី\u200bខាង\u200bកើត"}, {"Australia_Western", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអូស្ត្រាលី\u200bខាង\u200bលិច"}, {"Azerbaijan", "ម៉ោង\u200b\u200bនៅ\u200bអាស៊ែបៃហ្សង់នារដូវ\u200bក្ដៅ"}, {"Azores", "ម៉ោង\u200b\u200bនៅ\u200bអេហ្សសនារដូវ\u200bក្ដៅ"}, {"Bangladesh", "ម៉ោង\u200b\u200bនៅ\u200bបង់ក្លាដែសនារដូវ\u200bក្ដៅ"}, {"Brasilia", "ម៉ោង\u200bនៅ\u200bប្រាស៊ីលីយ៉ានា\u200b\u200bរដូវ\u200bក្ដៅ"}, {"Cape_Verde", "ម៉ោង\u200b\u200bនៅ\u200bកាប់វែរនារដូវ\u200bក្ដៅ"}, {"Chatham", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bចាថាំ"}, {"Chile", "ម៉ោងនៅស៊ីលីនារដូវក្តៅ"}, {"China", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bចិន"}, {"Colombia", "ម៉ោង\u200bនៅ\u200bកូឡុំប៊ីនា\u200bរដូវ\u200bក្ដៅ"}, {"Cook", "ម៉ោង\u200bនៅប្រជុំ\u200bកោះ\u200bខូកនាពាក់កណ្ដាល\u200bរដូវ\u200b\u200b\u200bក្ដៅ"}, {"Cuba", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bគុយបា"}, {"Easter", "ម៉ោងនៅកោះអ៊ីស្ទ័រនារដូវក្តៅ"}, {"Europe_Central", "ម៉ោង\u200bនៅ\u200bអឺរ៉ុប\u200bកណ្ដាលនា\u200bរដូវ\u200bក្ដៅ"}, {"Europe_Eastern", "ម៉ោង\u200bនៅ\u200bអឺរ៉ុប\u200b\u200bខាង\u200bកើត\u200bនា\u200bរដូវ\u200bក្ដៅ"}, {"Europe_Western", "ម៉ោង\u200bនៅ\u200bអឺរ៉ុប\u200bខាង\u200bលិចនារដូវ\u200bក្ដៅ\u200b"}, {"Falkland", "ម៉ោង\u200b\u200bនៅប្រជុំ\u200bកោះ\u200bហ្វក់ឡែននារដូវ\u200bក្ដៅ"}, {"Fiji", "ម៉ោង\u200bនៅ\u200b\u200bហ្វីជីនា\u200b\u200bរដូវ\u200bក្ដៅ"}, {"Georgia", "ម៉ោង\u200bនៅ\u200bហ្សកហ្ស៊ីនា\u200b\u200bរដូវ\u200bក្ដៅ"}, {"Greenland_Eastern", "ម៉ោង\u200bនៅ\u200bហ្គ្រីនលែនខាង\u200bកើតនា\u200bរដូវ\u200bក្ដៅ"}, {"Greenland_Western", "ម៉ោងនៅហ្គ្រីនលែនខាងលិចនារដូវក្តៅ"}, {"Hawaii_Aleutian", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bហាវៃ-អាល់ដ្យូសិន"}, {"Hong_Kong", "ម៉ោងនៅ\u200bហុងកុងនា\u200bរដូវ\u200bក្ដៅ\u200b"}, {"Hovd", "ម៉ោងនៅ\u200bហូវនា\u200bរដូវ\u200bក្ដៅ\u200b"}, {"Iran", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអ៊ីរ៉ង់"}, {"Irkutsk", "ម៉ោងនៅអៀរគុតស្កិ៍នារដូវក្តៅ"}, {"Israel", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bអ៊ីស្រាអែល"}, {"Japan", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅជប៉ុន"}, {"Korea", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bកូរ៉េ"}, {"Krasnoyarsk", "ម៉ោង\u200bនៅ\u200bក្រាណូយ៉ាសនា\u200bរដូវ\u200bក្ដៅ"}, {"Lord_Howe", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bឡតហៅ"}, {"Magadan", "ម៉ោង\u200bនៅ\u200bម៉ាហ្កាដាន\u200bនារដូវ\u200bក្ដៅ"}, {"Mauritius", "ម៉ោង\u200b\u200bរដូវ\u200bក្ដៅនៅ\u200bម៉ូរីស"}, {"Mexico_Pacific", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bប៉ាស៊ីហ្វិក\u200bម៉ិកស៊ិក"}, {"Mongolia", "ម៉ោងនៅ\u200bអ៊ូឡាន\u200bបាទូនា\u200bរដូវ\u200bក្ដៅ\u200b"}, {"Moscow", "ម៉ោង\u200bនៅ\u200bមូស្គូ\u200bនារដូវ\u200bក្ដៅ"}, {"New_Caledonia", "ម៉ោង\u200bនៅណូវ៉ែលកាឡេដូនៀនារដូវក្តៅ"}, {"New_Zealand", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bនូវែលសេឡង់"}, {"Newfoundland", "ម៉ោង\u200bពេលថ្ងៃ\u200bនៅ\u200bញូវហ្វោនឡែន"}, {"Norfolk", "ម៉ោងនៅ\u200bណ័រហ្វក់នា\u200bរដូវ\u200bក្ដៅ"}, {"Noronha", "ម៉ោង\u200bនៅហ្វ៊ែណាន់ដូ\u200bដឺណូរ៉ូញ៉ានារដូវក្តៅ"}, {"Novosibirsk", "ម៉ោង\u200bនៅ\u200bណូវ៉ូស៊ីប៊ីកនា\u200bរដូវ\u200bក្ដៅ"}, {"Omsk", "ម៉ោង\u200bនៅ\u200bអូមនា\u200bរដូវ\u200bក្ដៅ"}, {"Pakistan", "ម៉ោងនៅ\u200bប៉ាគីស្ថាននា\u200bរដូវ\u200bក្ដៅ\u200b"}, {"Paraguay", "ម៉ោង\u200bនៅប៉ារ៉ាហ្គាយនា\u200bរដូវ\u200bក្ដៅ"}, {"Peru", "ម៉ោង\u200b\u200bនៅ\u200bប៉េរូនារដូវ\u200bក្ដៅ"}, {"Philippines", "ម៉ោង\u200b\u200bនៅ\u200bហ្វីលីពីននា\u200bរដូវ\u200bក្ដៅ"}, {"Pierre_Miquelon", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅសង់\u200bព្យែរ និង\u200bមីគុយឡុង"}, {"Sakhalin", "ម៉ោង\u200bនៅ\u200bសាក់ខាលីននា\u200bរដូវ\u200bក្ដៅ"}, {"Samoa", "ម៉ោង\u200bនៅ\u200bសាម័រនារដូវក្តៅ"}, {"Taipei", "ម៉ោង\u200bពេល\u200bថ្ងៃ\u200bនៅ\u200bតៃប៉ិ"}, {"Tonga", "ម៉ោង\u200b\u200bនៅ\u200bតុងហ្គានារដូវ\u200bក្ដៅ"}, {"Turkmenistan", "ម៉ោង\u200bរដូវ\u200bក្ដៅ\u200bនៅ\u200bតួកម៉េនីស្ថាន\u200b"}, {"Uruguay", "ម៉ោង\u200bនៅ\u200bអ៊ុយរូហ្គាយនា\u200b\u200bរដូវ\u200bក្ដៅ"}, {"Uzbekistan", "ម៉ោង\u200bនៅ\u200bអ៊ូសបេគីស្ថាននារដូវ\u200bក្ដៅ\u200b"}, {"Vanuatu", "ម៉ោង\u200bនៅ\u200bវ៉ានូទូនារដូវ\u200bក្ដៅ\u200b"}, {"Vladivostok", "ម៉ោង\u200bនៅ\u200bវ៉្លាឌីវ៉ូស្តុកនា\u200bរដូវ\u200bក្ដៅ"}, {"Volgograd", "ម៉ោង\u200bនៅ\u200bវ៉ូហ្កោក្រាដនា\u200bរដូវ\u200bក្ដៅ"}, {"Yakutsk", "ម៉ោង\u200bនៅ\u200bយ៉ាគុតស្កិ៍នា\u200bរដូវ\u200bក្ដៅ"}, {"Yekaterinburg", "ម៉ោង\u200bនៅ\u200bអ៊ិខាធឺរីនប៊័កនា\u200bរដូវ\u200b\u200bក្ដៅ"}},
	},
	{
		{{"Acre", "ಏಕರ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Afghanistan", "ಅಫಘಾನಿಸ್ತಾನ ಸಮಯ"}, {"Africa_Central", "ಮಧ್ಯ ಆಫ್ರಿಕಾ ಸಮಯ"}, {"Africa_Eastern", "ಪೂರ್ವ ಆಫ್ರಿಕಾ ಸಮಯ"}, {"Africa_Southern", "ದಕ್ಷಿಣ ಆಫ್ರಿಕಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Africa_Western", "ಪಶ್ಚಿಮ ಆಫ್ರಿಕಾ ಸಮಯ"}, {"Alaska", "ಅಲಾಸ್ಕಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Almaty", "ಆಲ್ಮೇಟಿ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Amazon", "ಅಮೆಜಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"America_Central", "ಉತ್ತರ ಅಮೆರಿಕದ ಕೇಂದ್ರ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"America_Eastern", "ಉತ್ತರ ಅಮೆರಿಕದ ಪೂರ್ವದ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"America_Mountain", "ಉತ್ತರ ಅಮೆರಿಕದ ಪರ್ವತ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"America_Pacific", "ಉತ್ತರ ಅಮೆರಿಕದ ಪೆಸಿಫಿಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Anadyr", "ಅನಡೀರ್\u200c ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Apia", "ಅಪಿಯಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Aqtau", "ಅಕ್ಟೌ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Aqtobe", "ಅಕ್ಟೋಬೆ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Arabian", "ಅರೇಬಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Argentina", "ಅರ್ಜೆಂಟೀನಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Argentina_Western", "ಪಶ್ಚಿಮ ಅರ್ಜೆಂಟೀನಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Armenia", "ಅರ್ಮೇನಿಯ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Atlantic", "ಅಟ್ಲಾಂಟಿಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Australia_Central", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಕೇಂದ್ರ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Australia_CentralWestern", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಕೇಂದ್ರ ಪಶ್ಚಿಮ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Australia_Eastern", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಪೂರ್ವ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Australia_Western", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಪಶ್ಚಿಮ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Azerbaijan", "ಅಜರ್ಬೈಜಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Azores", "ಅಜೋರಸ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Bangladesh", "ಬಾಂಗ್ಲಾದೇಶ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Bhutan", "ಭೂತಾನ್ ಸಮಯ"}, {"Bolivia", "ಬೊಲಿವಿಯಾ ಸಮಯ"}, {"Brasilia", "ಬ್ರೆಸಿಲಿಯಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Brunei", "ಬ್ರೂನಿ ದಾರುಸಲೆಮ್ ಸಮಯ"}, {"Cape_Verde", "ಕೇಪ್ ವರ್ಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Chamorro", "ಚಮೋರೋ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Chatham", "ಚಥಾಮ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Chile", "ಚಿಲಿ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"China", "ಚೀನಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Christmas", "ಕ್ರಿಸ್ಮಸ್ ದ್ವೀಪ ಸಮಯ"}, {"Cocos", "ಕೋಕೋಸ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Colombia", "ಕೊಲಂಬಿಯಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Cook", "ಕುಕ್ ದ್ವೀಪಗಳ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Cuba", "ಕ್ಯೂಬಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Davis", "ಡೇವಿಸ್ ಸಮಯ"}, {"DumontDUrville", "ಡುಮಂಟ್-ಡಿ ಉರ್ವಿಲೆ ಸಮಯ"}, {"East_Timor", "ಪೂರ್ವ ಟಿಮೋರ್ ಸಮಯ"}, {"Easter", "ಈಸ್ಟರ್ ದ್ವೀಪ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Ecuador", "ಈಕ್ವೆಡಾರ್ ಸಮಯ"}, {"Europe_Central", "ಮಧ್ಯ ಯುರೋಪಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Europe_Eastern", "ಪೂರ್ವ ಯುರೋಪಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Europe_Further_Eastern", "ಮತ್ತಷ್ಟು-ಪೂರ್ವ ಯುರೋಪಿಯನ್ ಸಮಯ"}, {"Europe_Western", "ಪಶ್ಚಿಮ ಯುರೋಪಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Falkland", "ಫಾಕ್\u200cಲ್ಯಾಂಡ್ ದ್ವೀಪಗಳ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Fiji", "ಫಿಜಿ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"French_Guiana", "ಫ್ರೆಂಚ್ ಗಯಾನಾ ಸಮಯ"}, {"French_Southern", "ದಕ್ಷಿಣ ಫ್ರೆಂಚ್ ಮತ್ತು ಅಂಟಾರ್ಟಿಕಾ ಸಮಯ"}, {"GMT", "ಗ್ರೀನ್\u200cವಿಚ್ ಸರಾಸರಿ ಕಾಲಮಾನ"}, {"Galapagos", "ಗಾಲಾಪಾಗೋಸ್ ಸಮಯ"}, {"Gambier", "ಗ್ಯಾಂಬಿಯರ್ ಸಮಯ"}, {"Georgia", "ಜಾರ್ಜಿಯಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Gilbert_Islands", "ಗಿಲ್ಬರ್ಟ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Greenland_Eastern", "ಪೂರ್ವ ಗ್ರೀನ್\u200cಲ್ಯಾಂಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Greenland_Western", "ಪಶ್ಚಿಮ ಗ್ರೀನ್\u200cಲ್ಯಾಂಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Guam", "ಗುವಾಮ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Gulf", "ಗಲ್ಫ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Guyana", "ಗಯಾನಾ ಸಮಯ"}, {"Hawaii", "ಹವಾಯಿ-ಅಲ್ಯುಟಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Hawaii_Aleutian", "ಹವಾಯಿ-ಅಲ್ಯುಟಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Hong_Kong", "ಹಾಂಗ್ ಕಾಂಗ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Hovd", "ಹವ್ಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"India", "ಭಾರತೀಯ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Indian_Ocean", "ಹಿಂದೂ ಮಹಾಸಾಗರ ಸಮಯ"}, {"Indochina", "ಇಂಡೊಚೈನಾ ಸಮಯ"}, {"Indonesia_Central", "ಮಧ್ಯ ಇಂಡೋನೇಷಿಯಾ ಸಮಯ"}, {"Indonesia_Eastern", "ಪೂರ್ವ ಇಂಡೋನೇಷಿಯಾ ಸಮಯ"}, {"Indonesia_Western", "ಪಶ್ಚಿಮ ಇಂಡೋನೇಷಿಯ ಸಮಯ"}, {"Iran", "ಇರಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Irkutsk", "ಇರ್\u200cಕುಟಸ್ಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Israel", "ಇಸ್ರೇಲ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Japan", "ಜಪಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Kamchatka", "ಪೆತ್ರೋಪಾವ್ಲೋಸ್ಕ್\u200c\u200c-ಕಮ್ಚತ್ಸ್\u200cಕೀ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Kazakhstan", "ಕಝಾಖ್\u200cಸ್ತಾನ್ ಸಮಯ"}, {"Kazakhstan_Eastern", "ಪೂರ್ವ ಕಝಾಖ್\u200cಸ್ತಾನ್ ಸಮಯ"}, {"Kazakhstan_Western", "ಪಶ್ಚಿಮ ಕಝಾಖ್\u200cಸ್ತಾನ್ ಸಮಯ"}, {"Korea", "ಕೊರಿಯನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Kosrae", "ಕೊಸರೆ ಸಮಯ"}, {"Krasnoyarsk", "ಕ್ರಾಸ್\u200cನೊಯಾರ್ಸ್ಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Kyrgystan", "ಕಿರ್ಗಿಸ್ತಾನ್ ಸಮಯ"}, {"Lanka", "ಲಂಕಾ ಸಮಯ"}, {"Line_Islands", "ಲೈನ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Lord_Howe", "ಲಾರ್ಡ್ ಹೋವ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Macau", "ಮಕಾವ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Magadan", "ಮಗಡಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Malaysia", "ಮಲೇಷಿಯಾ ಸಮಯ"}, {"Maldives", "ಮಾಲ್ಡೀವ್ಸ್ ಸಮಯ"}, {"Marquesas", "ಮಾರ್ಕ್ಯುಸಸ್ ಸಮಯ"}, {"Marshall_Islands", "ಮಾರ್ಷಲ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Mauritius", "ಮಾರಿಷಸ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Mawson", "ಮಾವ್\u200cಸನ್ ಸಮಯ"}, {"Mexico_Pacific", "ಮೆಕ್ಸಿಕನ್ ಪೆಸಿಫಿಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Mongolia", "ಉಲಾನ್ ಬತೊರ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Moscow", "ಮಾಸ್ಕೋ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Myanmar", "ಮ್ಯಾನ್ಮಾರ್ ಸಮಯ"}, {"Nauru", "ನೌರು ಸಮಯ"}, {"Nepal", "ನೇಪಾಳ ಸಮಯ"}, {"New_Caledonia", "ಹೊಸ ಕ್ಯಾಲೆಡೋನಿಯಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"New_Zealand", "ನ್ಯೂಜಿಲ್ಯಾಂಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Newfoundland", "ನ್ಯೂಫೌಂಡ್\u200cಲ್ಯಾಂಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Niue", "ನಿಯು ಸಮಯ"}, {"Norfolk", "ನಾರ್ಫೋಕ್ ದ್ವೀಪ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Noronha", "ಫೆರ್ನಾಂಡೋ ಡೆ ನೊರೊನ್ಹಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"North_Mariana", "ಉತ್ತರ ಮರಿಯಾನಾ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Novosibirsk", "ನೊವೊಸಿಬಿರ್\u200cಸ್ಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Omsk", "ಒಮಾಸ್ಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Pakistan", "ಪಾಕಿಸ್ತಾನ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Palau", "ಪಾಲಾವ್ ಸಮಯ"}, {"Papua_New_Guinea", "ಪಪುವಾ ನ್ಯೂ ಗಿನಿಯಾ ಸಮಯ"}, {"Paraguay", "ಪರಾಗ್ವೇ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Peru", "ಪೆರು ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Philippines", "ಫಿಲಿಫೈನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Phoenix_Islands", "ಫಿನಿಕ್ಸ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"Pierre_Miquelon", "ಸೇಂಟ್ ಪಿಯರ್ ಮತ್ತು ಮಿಕ್ವೆಲನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Pitcairn", "ಪಿಟ್\u200cಕೈರ್ನ್ ಸಮಯ"}, {"Ponape", "ಪೊನಾಪೆ ಸಮಯ"}, {"Pyongyang", "ಪ್ಯೊಂಗ್ಯಾಂಗ್ ಸಮಯ"}, {"Qyzylorda", "ಕೋಜಿಲೋರ್ಡಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Reunion", "ರಿಯೂನಿಯನ್ ಸಮಯ"}, {"Rothera", "ರೊತೇರಾ ಸಮಯ"}, {"Sakhalin", "ಸ್ಯಾಕ್\u200cಹಲಿನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Samara", "ಸಮರ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Samoa", "ಸಮೋವಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Seychelles", "ಸೀಷೆಲ್ಸ್ ಸಮಯ"}, {"Singapore", "ಸಿಂಗಪುರ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Solomon", "ಸಾಲಮನ್ ದ್ವೀಪಗಳ ಸಮಯ"}, {"South_Georgia", "ದಕ್ಷಿಣ ಜಾರ್ಜಿಯಾ ಸಮಯ"}, {"Suriname", "ಸುರಿನೇಮ್ ಸಮಯ"}, {"Syowa", "ಸ್ಯೊವಾ ಸಮಯ"}, {"Tahiti", "ತಾಹಿತಿ ಸಮಯ"}, {"Taipei", "ತೈಪೆ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Tajikistan", "ತಝಕಿಸ್ತಾನ್ ಸಮಯ"}, {"Tokelau", "ಟೊಕೆಲಾವ್ ಸಮಯ"}, {"Tonga", "ಟೊಂಗಾ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Truk", "ಚುಕ್ ಸಮಯ"}, {"Turkey", "ಟರ್ಕಿ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Turkmenistan", "ತುರ್ಕ್\u200cಮೇನಿಸ್ತಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Tuvalu", "ತುವಾಲು ಸಮಯ"}, {"Uruguay", "ಉರುಗ್ವೇ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Uzbekistan", "ಉಜ್ಬೇಕಿಸ್ತಾನ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Vanuatu", "ವನೌತು ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Venezuela", "ವೆನಿಜುವೆಲಾ ಸಮಯ"}, {"Vladivostok", "ವ್ಲಾಡಿವೋಸ್ಟೋಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Volgograd", "ವೋಲ್ಗೋಗಾರ್ಡ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Vostok", "ವೋಸ್ಟೊಕ್ ಸಮಯ"}, {"Wake", "ವೇಕ್ ದ್ವೀಪ ಸಮಯ"}, {"Wallis", "ವ್ಯಾಲೀಸ್ ಮತ್ತು ಫ್ಯುಟುನಾ ಸಮಯ"}, {"Yakutsk", "ಯಾಕುಟ್ಸಕ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Yekaterinburg", "ಯೇಕಟರಿನ್\u200cಬರ್ಗ್ ಪ್ರಮಾಣಿತ ಸಮಯ"}, {"Yukon", "ಯುಕಾನ್ ಸಮಯ"}},
		{{"Acre", "ಏಕರ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Alaska", "\u200cಅಲಾಸ್ಕಾ ಹಗಲು ಸಮಯ"}, {"Almaty", "ಆಲ್ಮೇಟಿ ಬೇಸಿಗೆ ಸಮಯ"}, {"Amazon", "ಅಮೆಜಾನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"America_Central", "ಉತ್ತರ ಅಮೆರಿಕದ ಕೇಂದ್ರೀಯ ದಿನದ ಸಮಯ"}, {"America_Eastern", "ಉತ್ತರ ಅಮೆರಿಕದ ಪೂರ್ವದ ದಿನದ ಸಮಯ"}, {"America_Mountain", "ಉತ್ತರ ಅಮೆರಿಕದ ಪರ್ವತ ದಿನದ ಸಮಯ"}, {"America_Pacific", "ಉತ್ತರ ಅಮೆರಿಕದ ಪೆಸಿಫಿಕ್ ದಿನದ ಸಮಯ"}, {"Anadyr", "ಅನಡೀರ್\u200c ಹಗಲು ಸಮಯ"}, {"Apia", "ಅಪಿಯಾ ಹಗಲು ಸಮಯ"}, {"Aqtau", "ಅಕ್ಟೌ ಬೇಸಿಗೆ ಸಮಯ"}, {"Aqtobe", "ಅಕ್ಟೋಬೆ ಬೇಸಿಗೆ ಸಮಯ"}, {"Arabian", "ಅರೇಬಿಯನ್ ಹಗಲು ಸಮಯ"}, {"Argentina", "ಅರ್ಜೆಂಟಿನಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Argentina_Western", "ಪಶ್ಚಿಮ ಅರ್ಜೆಂಟೀನಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Armenia", "ಅರ್ಮೇನಿಯ ಬೇಸಿಗೆ ಸಮಯ"}, {"Atlantic", "ಅಟ್ಲಾಂಟಿಕ್ ದಿನದ ಸಮಯ"}, {"Australia_Central", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಕೇಂದ್ರ ಹಗಲು ಸಮಯ"}, {"Australia_CentralWestern", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಕೇಂದ್ರ ಪಶ್ಚಿಮ ಹಗಲು ಸಮಯ"}, {"Australia_Eastern", "ಪೂರ್ವ ಆಸ್ಟ್ರೇಲಿಯಾದ ಹಗಲು ಸಮಯ"}, {"Australia_Western", "ಆಸ್ಟ್ರೇಲಿಯಾದ ಪಶ್ಚಿಮ ಹಗಲು ಸಮಯ"}, {"Azerbaijan", "ಅಜರ್ಬೈಜಾನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Azores", "ಅಜೋರಸ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Bangladesh", "ಬಾಂಗ್ಲಾದೇಶ ಬೇಸಿಗೆ ಸಮಯ"}, {"Brasilia", "ಬ್ರೆಸಿಲಿಯಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Cape_Verde", "ಕೇಪ್ ವರ್ಡ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Chatham", "ಚಥಾಮ್ ಹಗಲು ಸಮಯ"}, {"Chile", "ಚಿಲಿ ಬೇಸಿಗೆ ಸಮಯ"}, {"China", "ಚೀನಾ ಹಗಲು ಸಮಯ"}, {"Colombia", "ಕೊಲಂಬಿಯಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Cook", "ಕುಕ್ ದ್ವೀಪಗಳ ಮಧ್ಯಕಾಲೀನ ಬೇಸಿಗೆ ಸಮಯ"}, {"Cuba", "ಕ್ಯೂಬಾ ದಿನದ ಸಮಯ"}, {"Easter", "ಈಸ್ಟರ್ ದ್ವೀಪ ಬೇಸಿಗೆ ಸಮಯ"}, {"Europe_Central", "ಮಧ್ಯ ಯುರೋಪಿಯನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Europe_Eastern", "ಪೂರ್ವ ಯುರೋಪಿಯನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Europe_Western", "ಪಶ್ಚಿಮ ಯುರೋಪಿಯನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Falkland", "ಫಾಕ್\u200cಲ್ಯಾಂಡ್ ದ್ವೀಪಗಳ ಬೇಸಿಗೆ ಸಮಯ"}, {"Fiji", "ಫಿಜಿ ಬೇಸಿಗೆ ಸಮಯ"}, {"Georgia", "ಜಾರ್ಜಿಯಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Greenland_Eastern", "ಪೂರ್ವ ಗ್ರೀನ್\u200cಲ್ಯಾಂಡ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Greenland_Western", "ಪಶ್ಚಿಮ ಗ್ರೀನ್\u200cಲ್ಯಾಂಡ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Hawaii_Aleutian", "ಹವಾಯಿ-ಅಲ್ಯುಟಿಯನ್ ಹಗಲು ಸಮಯ"}, {"Hong_Kong", "ಹಾಂಗ್ ಕಾಂಗ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Hovd", "ಹವ್ಡ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Iran", "ಇರಾನ್ ಹಗಲು ಸಮಯ"}, {"Irkutsk", "ಇರ್\u200cಕುಟಸ್ಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Israel", "ಇಸ್ರೇಲ್ ಹಗಲು ಸಮಯ"}, {"Japan", "ಜಪಾನ್ ಹಗಲು ಸಮಯ"}, {"Kamchatka", "ಪೆತ್ರೋಪಾವ್ಲೋಸ್ಕ್\u200c\u200c-ಕಮ್ಚತ್ಸ್\u200cಕೀ ಬೇಸಿಗೆ ಸಮಯ"}, {"Korea", "ಕೊರಿಯನ್ ಹಗಲು ಸಮಯ"}, {"Krasnoyarsk", "ಕ್ರಾಸ್\u200cನೊಯಾರ್ಸ್ಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Lord_Howe", "ಲಾರ್ಡ್ ಹೋವ್ ಬೆಳಗಿನ ಸಮಯ"}, {"Macau", "ಮಕಾವ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Magadan", "ಮಗಡಾನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Mauritius", "ಮಾರಿಷಸ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Mexico_Pacific", "ಮೆಕ್ಸಿಕನ್ ಪೆಸಿಫಿಕ್ ಹಗಲು ಸಮಯ"}, {"Mongolia", "ಉಲಾನ್ ಬತೊರ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Moscow", "ಮಾಸ್ಕೋ ಬೇಸಿಗೆ ಸಮಯ"}, {"New_Caledonia", "ಹೊಸ ಕ್ಯಾಲೆಡೋನಿಯಾ ಬೇಸಿಗೆಯ ಸಮಯ"}, {"New_Zealand", "ನ್ಯೂಜಿಲ್ಯಾಂಡ್ ಹಗಲು ಸಮಯ"}, {"Newfoundland", "ನ್ಯೂಫೌಂಡ್\u200cಲ್ಯಾಂಡ್ ದಿನದ ಸಮಯ"}, {"Norfolk", "ನಾರ್ಫೋಕ್ ದ್ವೀಪ ಬೇಸಿಗೆ ಸಮಯ"}, {"Noronha", "ಫರ್ನಾಂಡೋ ದೆ ನೊರೊನ್ಹಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Novosibirsk", "ನೊವೊಸಿಬಿರ್\u200cಸ್ಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Omsk", "ಒಮಾಸ್ಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Pakistan", "ಪಾಕಿಸ್ತಾನ ಬೇಸಿಗೆ ಸಮಯ"}, {"Paraguay", "ಪರಾಗ್ವೇ ಬೇಸಿಗೆ ಸಮಯ"}, {"Peru", "ಪೆರು ಬೇಸಿಗೆ ಸಮಯ"}, {"Philippines", "ಫಿಲಿಫೈನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Pierre_Miquelon", "ಸೇಂಟ್ ಪಿಯರ್ ಮತ್ತು ಮಿಕ್ವೆಲನ್ ಹಗಲು ಸಮಯ"}, {"Qyzylorda", "ಕೋಜಿಲೋರ್ಡಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Sakhalin", "ಸ್ಯಾಕ್\u200cಹಲಿನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Samara", "ಸಮರ ಬೇಸಿಗೆ ಸಮಯ"}, {"Samoa", "ಸಮೋವಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Taipei", "ತೈಪೆ ಹಗಲು ಸಮಯ"}, {"Tonga", "ಟೊಂಗಾ ಬೇಸಿಗೆ ಸಮಯ"}, {"Turkey", "ಟರ್ಕಿ ಬೇಸಿಗೆ ಸಮಯ"}, {"Turkmenistan", "ತುರ್ಕ್\u200cಮೇನಿಸ್ತಾನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Uruguay", "ಉರುಗ್ವೇ ಬೇಸಿಗೆ ಸಮಯ"}, {"Uzbekistan", "ಉಜ್ಬೇಕಿಸ್ತಾನ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Vanuatu", "ವನೌತು ಬೇಸಿಗೆ ಸಮಯ"}, {"Vladivostok", "ವ್ಲಾಡಿವೋಸ್ಟೋಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Volgograd", "ವೋಲ್ಗೋಗಾರ್ಡ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Yakutsk", "ಯಾಕುಟ್ಸಕ್ ಬೇಸಿಗೆ ಸಮಯ"}, {"Yekaterinburg", "ಯೇಕೇಟರಿನ್\u200dಬರ್ಗ್ ಬೇಸಿಗೆ ಸಮಯ"}},
	},
	{
		{{"Acre", "아크레 표준시"}, {"Afghanistan", "아프가니스탄 시간"}, {"Africa_Central", "중앙아프리카 시간"}, {"Africa_Eastern", "동아프리카 시간"}, {"Africa_Southern", "남아프리카 시간"}, {"Africa_Western", "서아프리카 시간"}, {"Alaska", "알래스카 표준시"}, {"Almaty", "알마티 표준 표준시"}, {"Amazon", "아마존 표준시"}, {"America_Central", "미 중부 표준시"}, {"America_Eastern", "미 동부 표준시"}, {"America_Mountain", "미 산악 표준시"}, {"America_Pacific", "미 태평양 표준시"}, {"Anadyr", "아나디리 표준시"}, {"Apia", "아피아 표준시"}, {"Aqtau", "악타우 표준 표준시"}, {"Aqtobe", "악퇴베 표준 표준시"}, {"Arabian", "아라비아 표준시"}, {"Argentina", "아르헨티나 표준시"}, {"Argentina_Western", "아르헨티나 서부 표준시"}, {"Armenia", "아르메니아 표준시"}, {"Atlantic", "대서양 표준시"}, {"Australia_Central", "호주 중부 표준시"}, {"Australia_CentralWestern", "호주 중서부 표준시"}, {"Australia_Eastern", "호주 동부 표준시"}, {"Australia_Western", "호주 서부 표준시"}, {"Azerbaijan", "아제르바이잔 표준시"}, {"Azores", "아조레스 표준시"}, {"Bangladesh", "방글라데시 표준시"}, {"Bhutan", "부탄 시간"}, {"Bolivia", "볼리비아 시간"}, {"Brasilia", "브라질리아 표준시"}, {"Brunei", "브루나이 시간"}, {"Cape_Verde", "카보 베르데 표준시"}, {"Casey", "케이시 시간"}, {"Chamorro", "차모로 시간"}, {"Chatham", "채텀 표준시"}, {"Chile", "칠레 표준시"}, {"China", "중국 표준시"}, {"Christmas", "크리스마스섬 시간"}, {"Cocos", "코코스 제도 시간"}, {"Colombia", "콜롬비아 표준시"}, {"Cook", "쿡 제도 표준시"}, {"Cuba", "쿠바 표준시"}, {"Davis", "데이비스 시간"}, {"DumontDUrville", "뒤몽뒤르빌 시간"}, {"East_Timor", "동티모르 시간"}, {"Easter", "이스터섬 표준시"}, {"Ecuador", "에콰도르 시간"}, {"Europe_Central", "중부유럽 표준시"}, {"Europe_Eastern", "동유럽 표준시"}, {"Europe_Further_Eastern", "극동유럽 표준시"}, {"Europe_Western", "서유럽 표준시"}, {"Falkland", "포클랜드 제도 표준시"}, {"Fiji", "피지 표준시"}, {"French_Guiana", "프랑스령 가이아나 시간"}, {"French_Southern", "프랑스령 남부 식민지 및 남극 시간"}, {"GMT", "그리니치 표준시"}, {"Galapagos", "갈라파고스 시간"}, {"Gambier", "감비에 시간"}, {"Georgia", "조지아 표준시"}, {"Gilbert_Islands", "길버트 제도 시간"}, {"Greenland_Eastern", "그린란드 동부 표준시"}, {"Greenland_Western", "그린란드 서부 표준시"}, {"Guam", "괌 표준 시간"}, {"Gulf", "걸프만 표준시"}, {"Guyana", "가이아나 시간"}, {"Hawaii", "하와이 알류샨 표준시"}, {"Hawaii_Aleutian", "하와이 알류샨 표준시"}, {"Hong_Kong", "홍콩 표준시"}, {"Hovd", "호브드 표준시"}, {"India", "인도 표준시"}, {"Indian_Ocean", "인도양 시간"}, {"Indochina", "인도차이나 시간"}, {"Indonesia_Central", "중부 인도네시아 시간"}, {"Indonesia_Eastern", "동부 인도네시아 시간"}, {"Indonesia_Western", "서부 인도네시아 시간"}, {"Iran", "이란 표준시"}, {"Irkutsk", "이르쿠츠크 표준시"}, {"Israel", "이스라엘 표준시"}, {"Japan", "일본 표준시"}, {"Kamchatka", "페트로파블롭스크-캄차츠키 표준시"}, {"Kazakhstan", "카자흐스탄 시간"}, {"Kazakhstan_Eastern", "동부 카자흐스탄 시간"}, {"Kazakhstan_Western", "서부 카자흐스탄 시간"}, {"Korea", "한국 표준시"}, {"Kosrae", "코스라에섬 시간"}, {"Krasnoyarsk", "크라스노야르스크 표준시"}, {"Kyrgystan", "키르기스스탄 시간"}, {"Lanka", "랑카 표준 시간"}, {"Line_Islands", "라인 제도 시간"}, {"Lord_Howe", "로드 하우 표준시"}, {"Macau", "마카오 표준 시간"}, {"Magadan", "마가단 표준시"}, {"Malaysia", "말레이시아 시간"}, {"Maldives", "몰디브 시간"}, {"Marquesas", "마르키즈 제도 시간"}, {"Marshall_Islands", "마셜 제도 시간"}, {"Mauritius", "모리셔스 표준시"}, {"Mawson", "모슨 시간"}, {"Mexico_Pacific", "멕시코 태평양 표준시"}, {"Mongolia", "울란바토르 표준시"}, {"Moscow", "모스크바 표준시"}, {"Myanmar", "미얀마 시간"}, {"Nauru", "나우루 시간"}, {"Nepal", "네팔 시간"}, {"New_Caledonia", "뉴칼레도니아 표준시"}, {"New_Zealand", "뉴질랜드 표준시"}, {"Newfoundland", "뉴펀들랜드 표준시"}, {"Niue", "니우에 시간"}, {"Norfolk", "노퍽섬 표준시"}, {"Noronha", "페르난도 데 노로냐 표준시"}, {"North_Mariana", "북마리아나 제도 표준 시간"}, {"Novosibirsk", "노보시비르스크 표준시"}, {"Omsk", "옴스크 표준시"}, {"Pakistan", "파키스탄 표준시"}, {"Palau", "팔라우 시간"}, {"Papua_New_Guinea", "파푸아뉴기니 시간"}, {"Paraguay", "파라과이 표준시"}, {"Peru", "페루 표준시"}, {"Philippines", "필리핀 표준시"}, {"Phoenix_Islands", "피닉스 제도 시간"}, {"Pierre_Miquelon", "세인트피에르 미클롱 표준시"}, {"Pitcairn", "핏케언 시간"}, {"Ponape", "포나페 시간"}, {"Pyongyang", "평양 시간"}, {"Qyzylorda", "키질로르다 표준 시간"}, {"Reunion", "레위니옹 시간"}, {"Rothera", "로데라 시간"}, {"Sakhalin", "사할린 표준시"}, {"Samara", "사마라 표준시"}, {"Samoa", "사모아 표준시"}, {"Seychelles", "세이셸 시간"}, {"Singapore", "싱가포르 표준시"}, {"Solomon", "솔로몬 제도 시간"}, {"South_Georgia", "사우스 조지아 시간"}, {"Suriname", "수리남 시간"}, {"Syowa", "쇼와 시간"}, {"Tahiti", "타히티 시간"}, {"Taipei", "대만 표준시"}, {"Tajikistan", "타지키스탄 시간"}, {"Tokelau", "토켈라우 시간"}, {"Tonga", "통가 표준시"}, {"Truk", "추크 시간"}, {"Turkey", "터키 표준시"}, {"Turkmenistan", "투르크메니스탄 표준시"}, {"Tuvalu", "투발루 시간"}, {"Uruguay", "우루과이 표준시"}, {"Uzbekistan", "우즈베키스탄 표준시"}, {"Vanuatu", "바누아투 표준시"}, {"Venezuela", "베네수엘라 시간"}, {"Vladivostok", "블라디보스토크 표준시"}, {"Volgograd", "볼고그라드 표준시"}, {"Vostok", "보스톡 시간"}, {"Wake", "웨이크섬 시간"}, {"Wallis", "월리스푸투나 제도 시간"}, {"Yakutsk", "야쿠츠크 표준시"}, {"Yekaterinburg", "예카테린부르크 표준시"}, {"Yukon", "유콘 시간"}},
		{{"Acre", "아크레 하계 표준시"}, {"Alaska", "알래스카 하계 표준시"}, {"Almaty", "알마티 하계 표준시"}, {"Amazon", "아마존 하계 표준시"}, {"America_Central", "미 중부 하계 표준시"}, {"America_Eastern", "미 동부 하계 표준시"}, {"America_Mountain", "미 산지 하계 표준시"}, {"America_Pacific", "미 태평양 하계 표준시"}, {"Anadyr", "아나디리 하계 표준시"}, {"Apia", "아피아 하계 표준시"}, {"Aqtau", "악타우 하계 표준시"}, {"Aqtobe", "악퇴베 하계 표준시"}, {"Arabian", "아라비아 하계 표준시"}, {"Argentina", "아르헨티나 하계 표준시"}, {"Argentina_Western", "아르헨티나 서부 하계 표준시"}, {"Armenia", "아르메니아 하계 표준시"}, {"Atlantic", "대서양 하계 표준시"}, {"Australia_Central", "호주 중부 하계 표준시"}, {"Australia_CentralWestern", "호주 중서부 하계 표준시"}, {"Australia_Eastern", "호주 동부 하계 표준시"}, {"Australia_Western", "호주 서부 하계 표준시"}, {"Azerbaijan", "아제르바이잔 하계 표준시"}, {"Azores", "아조레스 하계 표준시"}, {"Bangladesh", "방글라데시 하계 표준시"}, {"Brasilia", "브라질리아 하계 표준시"}, {"Cape_Verde", "카보 베르데 하계 표준시"}, {"Chatham", "채텀 하계 표준시"}, {"Chile", "칠레 하계 표준시"}, {"China", "중국 하계 표준시"}, {"Colombia", "콜롬비아 하계 표준시"}, {"Cook", "쿡 제도 절반 하계 표준시"}, {"Cuba", "쿠바 하계 표준시"}, {"Easter", "이스터섬 하계 표준시"}, {"Europe_Central", "중부유럽 하계 표준시"}, {"Europe_Eastern", "동유럽 하계 표준시"}, {"Europe_Western", "서유럽 하계 표준시"}, {"Falkland", "포클랜드 제도 하계 표준시"}, {"Fiji", "피지 하계 표준시"}, {"Georgia", "조지아 하계 표준시"}, {"Greenland_Eastern", "그린란드 동부 하계 표준시"}, {"Greenland_Western", "그린란드 서부 하계 표준시"}, {"Hawaii_Aleutian", "하와이 알류샨 하계 표준시"}, {"Hong_Kong", "홍콩 하계 표준시"}, {"Hovd", "호브드 하계 표준시"}, {"Iran", "이란 하계 표준시"}, {"Irkutsk", "이르쿠츠크 하계 표준시"}, {"Israel", "이스라엘 하계 표준시"}, {"Japan", "일본 하계 표준시"}, {"Kamchatka", "페트로파블롭스크-캄차츠키 하계 표준시"}, {"Korea", "한국 하계 표준시"}, {"Krasnoyarsk", "크라스노야르스크 하계 표준시"}, {"Lord_Howe", "로드 하우 하계 표준시"}, {"Macau", "마카오 하계 표준시"}, {"Magadan", "마가단 하계 표준시"}, {"Mauritius", "모리셔스 하계 표준시"}, {"Mexico_Pacific", "멕시코 태평양 하계 표준시"}, {"Mongolia", "울란바토르 하계 표준시"}, {"Moscow", "모스크바 하계 표준시"}, {"New_Caledonia", "뉴칼레도니아 하계 표준시"}, {"New_Zealand", "뉴질랜드 하계 표준시"}, {"Newfoundland", "뉴펀들랜드 하계 표준시"}, {"Norfolk", "노퍽섬 하계 표준시"}, {"Noronha", "페르난도 데 노로냐 하계 표준시"}, {"Novosibirsk", "노보시비르스크 하계 표준시"}, {"Omsk", "옴스크 하계 표준시"}, {"Pakistan", "파키스탄 하계 표준시"}, {"Paraguay", "파라과이 하계 표준시"}, {"Peru", "페루 하계 표준시"}, {"Philippines", "필리핀 하계 표준시"}, {"Pierre_Miquelon", "세인트피에르 미클롱 하계 표준시"}, {"Qyzylorda", "키질로르다 하계 표준시"}, {"Sakhalin", "사할린 하계 표준시"}, {"Samara", "사마라 하계 표준시"}, {"Samoa", "사모아 하계 표준시"}, {"Taipei", "대만 하계 표준시"}, {"Tonga", "통가 하계 표준시"}, {"Turkey", "터키 일광 절약 시간"}, {"Turkmenistan", "투르크메니스탄 하계 표준시"}, {"Uruguay", "우루과이 하계 표준시"}, {"Uzbekistan", "우즈베키스탄 하계 표준시"}, {"Vanuatu", "바누아투 하계 표준시"}, {"Vladivostok", "블라디보스토크 하계 표준시"}, {"Volgograd", "볼고그라드 하계 표준시"}, {"Yakutsk", "야쿠츠크 하계 표준시"}, {"Yekaterinburg", "예카테린부르크 하계 표준시"}},
	},
	{
		{{"Acre", "아크레 표준시"}, {"Afghanistan", "아프가니스탄 시간"}, {"Africa_Central", "중앙아프리카 시간"}, {"Africa_Eastern", "동아프리카 시간"}, {"Africa_Southern", "남아프리카 시간"}, {"Africa_Western", "서아프리카 시간"}, {"Alaska", "알래스카 표준시"}, {"Almaty", "알마티 표준 표준시"}, {"Amazon", "아마존 표준시"}, {"America_Central", "미 중부 표준시"}, {"America_Eastern", "미 동부 표준시"}, {"America_Mountain", "미 산악 표준시"}, {"America_Pacific", "미 태평양 표준시"}, {"Anadyr", "아나디리 표준시"}, {"Apia", "아피아 표준시"}, {"Aqtau", "악타우 표준 표준시"}, {"Aqtobe", "악퇴베 표준 표준시"}, {"Arabian", "아라비아 표준시"}, {"Argentina", "아르헨티나 표준시"}, {"Argentina_Western", "아르헨티나 서부 표준시"}, {"Armenia", "아르메니아 표준시"}, {"Atlantic", "대서양 표준시"}, {"Australia_Central", "호주 중부 표준시"}, {"Australia_CentralWestern", "호주 중서부 표준시"}, {"Australia_Eastern", "호주 동부 표준시"}, {"Australia_Western", "호주 서부 표준시"}, {"Azerbaijan", "아제르바이잔 표준시"}, {"Azores", "아조레스 표준시"}, {"Bangladesh", "방글라데시 표준시"}, {"Bhutan", "부탄 시간"}, {"Bolivia", "볼리비아 시간"}, {"Brasilia", "브라질리아 표준시"}, {"Brunei", "브루나이 시간"}, {"Cape_Verde", "카보 베르데 표준시"}, {"Casey", "케이시 시간"}, {"Chamorro", "차모로 시간"}, {"Chatham", "채텀 표준시"}, {"Chile", "칠레 표준시"}, {"China", "중국 표준시"}, {"Christmas", "크리스마스섬 시간"}, {"Cocos", "코코스 제도 시간"}, {"Colombia", "콜롬비아 표준시"}, {"Cook", "쿡 제도 표준시"}, {"Cuba", "쿠바 표준시"}, {"Davis", "데이비스 시간"}, {"DumontDUrville", "뒤몽뒤르빌 시간"}, {"East_Timor", "동티모르 시간"}, {"Easter", "이스터섬 표준시"}, {"Ecuador", "에콰도르 시간"}, {"Europe_Central", "중부유럽 표준시"}, {"Europe_Eastern", "동유럽 표준시"}, {"Europe_Further_Eastern", "극동유럽 표준시"}, {"Europe_Western", "서유럽 표준시"}, {"Falkland", "포클랜드 제도 표준시"}, {"Fiji", "피지 표준시"}, {"French_Guiana", "프랑스령 가이아나 시간"}, {"French_Southern", "프랑스령 남부 식민지 및 남극 시간"}, {"GMT", "그리니치 표준시"}, {"Galapagos", "갈라파고스 시간"}, {"Gambier", "감비에 시간"}, {"Georgia", "조지아 표준시"}, {"Gilbert_Islands", "길버트 제도 시간"}, {"Greenland_Eastern", "그린란드 동부 표준시"}, {"Greenland_Western", "그린란드 서부 표준시"}, {"Guam", "괌 표준 시간"}, {"Gulf", "걸프만 표준시"}, {"Guyana", "가이아나 시간"}, {"Hawaii", "하와이 알류샨 표준시"}, {"Hawaii_Aleutian", "하와이 알류샨 표준시"}, {"Hong_Kong", "홍콩 표준시"}, {"Hovd", "호브드 표준시"}, {"India", "인도 표준시"}, {"Indian_Ocean", "인도양 시간"}, {"Indochina", "인도차이나 시간"}, {"Indonesia_Central", "중부 인도네시아 시간"}, {"Indonesia_Eastern", "동부 인도네시아 시간"}, {"Indonesia_Western", "서부 인도네시아 시간"}, {"Iran", "이란 표준시"}, {"Irkutsk", "이르쿠츠크 표준시"}, {"Israel", "이스라엘 표준시"}, {"Japan", "일본 표준시"}, {"Kamchatka", "페트로파블롭스크-캄차츠키 표준시"}, {"Kazakhstan", "카자흐스탄 시간"}, {"Kazakhstan_Eastern", "동부 카자흐스탄 시간"}, {"Kazakhstan_Western", "서부 카자흐스탄 시간"}, {"Korea", "조선 표준시"}, {"Kosrae", "코스라에섬 시간"}, {"Krasnoyarsk", "크라스노야르스크 표준시"}, {"Kyrgystan", "키르기스스탄 시간"}, {"Lanka", "랑카 표준 시간"}, {"Line_Islands", "라인 제도 시간"}, {"Lord_Howe", "로드 하우 표준시"}, {"Macau", "마카오 표준 시간"}, {"Magadan", "마가단 표준시"}, {"Malaysia", "말레이시아 시간"}, {"Maldives", "몰디브 시간"}, {"Marquesas", "마르키즈 제도 시간"}, {"Marshall_Islands", "마셜 제도 시간"}, {"Mauritius", "모리셔스 표준시"}, {"Mawson", "모슨 시간"}, {"Mexico_Pacific", "멕시코 태평양 표준시"}, {"Mongolia", "울란바토르 표준시"}, {"Moscow", "모스크바 표준시"}, {"Myanmar", "미얀마 시간"}, {"Nauru", "나우루 시간"}, {"Nepal", "네팔 시간"}, {"New_Caledonia", "뉴칼레도니아 표준시"}, {"New_Zealand", "뉴질랜드 표준시"}, {"Newfoundland", "뉴펀들랜드 표준시"}, {"Niue", "니우에 시간"}, {"Norfolk", "노퍽섬 표준시"}, {"Noronha", "페르난도 데 노로냐 표준시"}, {"North_Mariana", "북마리아나 제도 표준 시간"}, {"Novosibirsk", "노보시비르스크 표준시"}, {"Omsk", "옴스크 표준시"}, {"Pakistan", "파키스탄 표준시"}, {"Palau", "팔라우 시간"}, {"Papua_New_Guinea", "파푸아뉴기니 시간"}, {"Paraguay", "파라과이 표준시"}, {"Peru", "페루 표준시"}, {"Philippines", "필리핀 표준시"}, {"Phoenix_Islands", "피닉스 제도 시간"}, {"Pierre_Miquelon", "세인트피에르 미클롱 표준시"}, {"Pitcairn", "핏케언 시간"}, {"Ponape", "포나페 시간"}, {"Pyongyang", "평양 시간"}, {"Qyzylorda", "키질로르다 표준 시간"}, {"Reunion", "레위니옹 시간"}, {"Rothera", "로데라 시간"}, {"Sakhalin", "사할린 표준시"}, {"Samara", "사마라 표준시"}, {"Samoa", "사모아 표준시"}, {"Seychelles", "세이셸 시간"}, {"Singapore", "싱가포르 표준시"}, {"Solomon", "솔로몬 제도 시간"}, {"South_Georgia", "사우스 조지아 시간"}, {"Suriname", "수리남 시간"}, {"Syowa", "쇼와 시간"}, {"Tahiti", "타히티 시간"}, {"Taipei", "대만 표준시"}, {"Tajikistan", "타지키스탄 시간"}, {"Tokelau", "토켈라우 시간"}, {"Tonga", "통가 표준시"}, {"Truk", "추크 시간"}, {"Turkey", "터키 표준시"}, {"Turkmenistan", "투르크메니스탄 표준시"}, {"Tuvalu", "투발루 시간"}, {"Uruguay", "우루과이 표준시"}, {"Uzbekistan", "우즈베키스탄 표준시"}, {"Vanuatu", "바누아투 표준시"}, {"Venezuela", "베네수엘라 시간"}, {"Vladivostok", "블라디보스토크 표준시"}, {"Volgograd", "볼고그라드 표준시"}, {"Vostok", "보스톡 시간"}, {"Wake", "웨이크섬 시간"}, {"Wallis", "월리스푸투나 제도 시간"}, {"Yakutsk", "야쿠츠크 표준시"}, {"Yekaterinburg", "예카테린부르크 표준시"}, {"Yukon", "유콘 시간"}},
		{{"Acre", "아크레 하계 표준시"}, {"Alaska", "알래스카 하계 표준시"}, {"Almaty", "알마티 하계 표준시"}, {"Amazon", "아마존 하계 표준시"}, {"America_Central", "미 중부 하계 표준시"}, {"America_Eastern", "미 동부 하계 표준시"}, {"America_Mountain", "미 산지 하계 표준시"}, {"America_Pacific", "미 태평양 하계 표준시"}, {"Anadyr", "아나디리 하계 표준시"}, {"Apia", "아피아 하계 표준시"}, {"Aqtau", "악타우 하계 표준시"}, {"Aqtobe", "악퇴베 하계 표준시"}, {"Arabian", "아라비아 하계 표준시"}, {"Argentina", "아르헨티나 하계 표준시"}, {"Argentina_Western", "아르헨티나 서부 하계 표준시"}, {"Armenia", "아르메니아 하계 표준시"}, {"Atlantic", "대서양 하계 표준시"}, {"Australia_Central", "호주 중부 하계 표준시"}, {"Australia_CentralWestern", "호주 중서부 하계 표준시"}, {"Australia_Eastern", "호주 동부 하계 표준시"}, {"Australia_Western", "호주 서부 하계 표준시"}, {"Azerbaijan", "아제르바이잔 하계 표준시"}, {"Azores", "아조레스 하계 표준시"}, {"Bangladesh", "방글라데시 하계 표준시"}, {"Brasilia", "브라질리아 하계 표준시"}, {"Cape_Verde", "카보 베르데 하계 표준시"}, {"Chatham", "채텀 하계 표준시"}, {"Chile", "칠레 하계 표준시"}, {"China", "중국 하계 표준시"}, {"Colombia", "콜롬비아 하계 표준시"}, {"Cook", "쿡 제도 절반 하계 표준시"}, {"Cuba", "쿠바 하계 표준시"}, {"Easter", "이스터섬 하계 표준시"}, {"Europe_Central", "중부유럽 하계 표준시"}, {"Europe_Eastern", "동유럽 하계 표준시"}, {"Europe_Western", "서유럽 하계 표준시"}, {"Falkland", "포클랜드 제도 하계 표준시"}, {"Fiji", "피지 하계 표준시"}, {"Georgia", "조지아 하계 표준시"}, {"Greenland_Eastern", "그린란드 동부 하계 표준시"}, {"Greenland_Western", "그린란드 서부 하계 표준시"}, {"Hawaii_Aleutian", "하와이 알류샨 하계 표준시"}, {"Hong_Kong", "홍콩 하계 표준시"}, {"Hovd", "호브드 하계 표준시"}, {"Iran", "이란 하계 표준시"}, {"Irkutsk", "이르쿠츠크 하계 표준시"}, {"Israel", "이스라엘 하계 표준시"}, {"Japan", "일본 하계 표준시"}, {"Kamchatka", "페트로파블롭스크-캄차츠키 하계 표준시"}, {"Korea", "조선 하계 표준시"}, {"Krasnoyarsk", "크라스노야르스크 하계 표준시"}, {"Lord_Howe", "로드 하우 하계 표준시"}, {"Macau", "마카오 하계 표준시"}, {"Magadan", "마가단 하계 표준시"}, {"Mauritius", "모리셔스 하계 표준시"}, {"Mexico_Pacific", "멕시코 태평양 하계 표준시"}, {"Mongolia", "울란바토르 하계 표준시"}, {"Moscow", "모스크바 하계 표준시"}, {"New_Caledonia", "뉴칼레도니아 하계 표준시"}, {"New_Zealand", "뉴질랜드 하계 표준시"}, {"Newfoundland", "뉴펀들랜드 하계 표준시"}, {"Norfolk", "노퍽섬 하계 표준시"}, {"Noronha", "페르난도 데 노로냐 하계 표준시"}, {"Novosibirsk", "노보시비르스크 하계 표준시"}, {"Omsk", "옴스크 하계 표준시"}, {"Pakistan", "파키스탄 하계 표준시"}, {"Paraguay", "파라과이 하계 표준시"}, {"Peru", "페루 하계 표준시"}, {"Philippines", "필리핀 하계 표준시"}, {"Pierre_Miquelon", "세인트피에르 미클롱 하계 표준시"}, {"Qyzylorda", "키질로르다 하계 표준시"}, {"Sakhalin", "사할린 하계 표준시"}, {"Samara", "사마라 하계 표준시"}, {"Samoa", "사모아 하계 표준시"}, {"Taipei", "대만 하계 표준시"}, {"Tonga", "통가 하계 표준시"}, {"Turkey", "터키 일광 절약 시간"}, {"Turkmenistan", "투르크메니스탄 하계 표준시"}, {"Uruguay", "우루과이 하계 표준시"}, {"Uzbekistan", "우즈베키스탄 하계 표준시"}, {"Vanuatu", "바누아투 하계 표준시"}, {"Vladivostok", "블라디보스토크 하계 표준시"}, {"Volgograd", "볼고그라드 하계 표준시"}, {"Yakutsk", "야쿠츠크 하계 표준시"}, {"Yekaterinburg", "예카테린부르크 하계 표준시"}},
	},
	{
		{{"Afghanistan", "अफगानिस्तान वेळ"}, {"Africa_Central", "मध्य आफ्रिका वेळ"}, {"Africa_Eastern", "उदेंत आफ्रिका वेळ"}, {"Africa_Southern", "दक्षिण आफ्रिका प्रमाणित वेळ"}, {"Africa_Western", "अस्तंत आफ्रिका वेळ"}, {"Alaska", "अलास्का प्रमाणीत वेळ"}, {"Amazon", "अमेझोन प्रमाणित वेळ"}, {"America_Central", "मध्य प्रमाणित वेळ"}, {"America_Eastern", "उदेंत प्रमाणित वेळ"}, {"America_Mountain", "पर्वतीय प्रमाणित वेळ"}, {"America_Pacific", "प्रशांत प्रमाणित वेळ"}, {"Apia", "अपिया प्रमाणित वेळ"}, {"Arabian", "अरबी प्रमाणित वेळ"}, {"Argentina", "अर्जेंटिना प्रमाणित वेळ"}, {"Argentina_Western", "अस्तंत अर्जेंटिना प्रमाणित वेळ"}, {"Armenia", "आर्मेनिया प्रमाणित वेळ"}, {"Atlantic", "अटलांटीक प्रमाणित वेळ"}, {"Australia_Central", "ऑस्ट्रेलीयन मध्य प्रमाणित वेळ"}, {"Australia_CentralWestern", "ऑस्ट्रेलीयन मध्य अस्तंत प्रमाणित वेळ"}, {"Australia_Eastern", "ऑस्ट्रेलीयन उदेंत प्रमाणित वेळ"}, {"Australia_Western", "ऑस्ट्रेलीयन अस्तंत प्रमाणित वेळ"}, {"Azerbaijan", "अजरबैजान प्रमाणित वेळ"}, {"Azores", "अझोरेस प्रमाणित वेळ"}, {"Bangladesh", "बांगलादेश प्रमाणित वेळ"}, {"Bhutan", "भूतान"}, {"Bolivia", "बोलिव्हिया वेळ"}, {"Brasilia", "ब्राझिलिया प्रमाणित वेळ"}, {"Brunei", "ब्रुनेइ दारूस्सलाम वेळ"}, {"Cape_Verde", "केप वर्दे प्रमाणित वेळ"}, {"Chamorro", "कॅमोरा प्रमाणित वेळ"}, {"Chatham", "चॅथम प्रमाणित वेळ"}, {"Chile", "चिली प्रमाणित वेळ"}, {"China", "चीन प्रमाणित वेळ"}, {"Christmas", "क्रिसमस आयलँड वेळ"}, {"Cocos", "कोकोस आयलँड वेळ"}, {"Colombia", "कोलंबिया प्रमाणित वेळ"}, {"Cook", "कूक आयलँड प्रमाणित वेळ"}, {"Cuba", "क्युबा प्रमाणीत वेळ"}, {"Davis", "डेव्हीस वेळ"}, {"DumontDUrville", "द्युमाँ दूरवीय वेळ"}, {"East_Timor", "उदेंत तिमोर वेळ"}, {"Easter", "ईस्टर आयलँड प्रमाणित वेळ"}, {"Ecuador", "इक्वेडोर वेळ"}, {"Europe_Central", "मध्य युरोपियन प्रमाणित वेळ"}, {"Europe_Eastern", "उदेंत युरोपियन प्रमाणित वेळ"}, {"Europe_Further_Eastern", "आनीक-उदेंत युरोपियन वेळ"}, {"Europe_Western", "अस्तंत युरोपियन प्रमाणित वेळ"}, {"Falkland", "फॉकलँड आयलँड्स प्रमाणित वेळ"}, {"Fiji", "फिजी प्रमाणित वेळ"}, {"French_Guiana", "फ्रेंच गयाना वेळ"}, {"French_Southern", "फ्रेन्च दक्षिण आनी अंटार्क्टिक वेळ"}, {"GMT", "ग्रीनविच मध्य वेळ"}, {"Galapagos", "गालापागोस वेळ"}, {"Gambier", "गाम्बियर वेळ"}, {"Georgia", "जॉर्जिया प्रमाणित वेळ"}, {"Gilbert_Islands", "गिल्बर्ट आयलँड वेळ"}, {"Greenland_Eastern", "उदेंत ग्रीनलँड प्रमाणीत वेळ"}, {"Greenland_Western", "अस्तंत ग्रीनलँड प्रमाणीत वेळ"}, {"Gulf", "गल्फ प्रमाणित वेळ"}, {"Guyana", "गुयाना वेळ"}, {"Hawaii", "हवाई-अलेयुशिन प्रमाणीत वेळ"}, {"Hawaii_Aleutian", "हवाई-अलेयुशिन प्रमाणीत वेळ"}, {"Hong_Kong", "हाँग काँग प्रमाणित वेळ"}, {"Hovd", "होव्हड प्रमाणित वेळ"}, {"India", "भारतीय प्रमाणित वेळ"}, {"Indian_Ocean", "हिंद म्हासागर वेळ"}, {"Indochina", "इंडोचीन वेळ"}, {"Indonesia_Central", "मध्य इंडोनेशिया वेळ"}, {"Indonesia_Eastern", "उदेंत इंडोनेशिया वेळ"}, {"Indonesia_Western", "अस्तंत इंडोनेशिया वेळ"}, {"Iran", "इरान प्रमाणित वेळ"}, {"Irkutsk", "ईर्कुटस्क प्रमाणित वेळ"}, {"Israel", "इज़राइल प्रमाणित वेळ"}, {"Japan", "जपान प्रमाणित वेळ"}, {"Kazakhstan", "कझाखस्तान वेळ"}, {"Kazakhstan_Eastern", "उदेंत कझाकस्तान वेळ"}, {"Kazakhstan_Western", "अस्तंत कझाकस्तान वेळ"}, {"Korea", "कोरियन प्रमाणित वेळ"}, {"Kosrae", "कोसरे वेळ"}, {"Krasnoyarsk", "क्रास्नोयार्स्क प्रमाणित वेळ"}, {"Kyrgystan", "किर्गिज़स्तान वेळ"}, {"Line_Islands", "लायन आयलँड वेळ"}, {"Lord_Howe", "लॉर्ड होवे प्रमाणित वेळ"}, {"Magadan", "मगादान प्रमाणित वेळ"}, {"Malaysia", "मलेशिया वेळ"}, {"Maldives", "मालदीव वेळ"}, {"Marquesas", "मार्किसस वेळ"}, {"Marshall_Islands", "मार्शल आयलँड वेळ"}, {"Mauritius", "मॉरिशस प्रमाणित वेळ"}, {"Mawson", "मॉसन वेळ"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत प्रमाणीत वेळ"}, {"Mongolia", "उलानबतार प्रमाणित वेळ"}, {"Moscow", "मॉस्को प्रमाणित वेळ"}, {"Myanmar", "म्यानमार वेळ"}, {"Nauru", "नरू वेळ"}, {"Nepal", "नेपाळ वेळ"}, {"New_Caledonia", "न्यु कॅलेडोनिया प्रमाणित वेळ"}, {"New_Zealand", "न्युझीलॅन्ड प्रमाणित वेळ"}, {"Newfoundland", "न्युफावंडलँड प्रमाणीत वेळ"}, {"Niue", "न्युए वेळ"}, {"Norfolk", "नॉरफॉक आयलँड प्रमाणित वेळ"}, {"Noronha", "फर्नांडो दी नोरोन्हा प्रमाणित वेळ"}, {"Novosibirsk", "नोवोसिबिर्स्क प्रमाणित वेळ"}, {"Omsk", "ओम्स्क प्रमाणित वेळ"}, {"Pakistan", "पाकिस्तान प्रमाणित वेळ"}, {"Palau", "पलाऊ वेळ"}, {"Papua_New_Guinea", "पापुआ न्यु गिनी वेळ"}, {"Paraguay", "परागुआ प्रमाणित वेळ"}, {"Peru", "पेरू प्रमाणित वेळ"}, {"Philippines", "फिलिपायन प्रमाणित वेळ"}, {"Phoenix_Islands", "फिनिक्स आयलँड वेळ"}, {"Pierre_Miquelon", "सेंट पियर आनी मिकलान प्रमाणीत वेळ"}, {"Pitcairn", "पिटकॅरन वेळ"}, {"Ponape", "पोनेप वेळ"}, {"Pyongyang", "प्योंगयांग वेळ"}, {"Reunion", "रियुनियन वेळ"}, {"Rothera", "रोथेरा वेळ"}, {"Sakhalin", "सखलिन प्रमाणित वेळ"}, {"Samoa", "सामोआ प्रमाणित वेळ"}, {"Seychelles", "सेशेल्स वेळ"}, {"Singapore", "सिंगापूर प्रमाणित वेळ"}, {"Solomon", "सोलोमन आइलँड्स वेळ"}, {"South_Georgia", "दक्षिण जॉर्जिया वेळ"}, {"Suriname", "सुरिनाम वेळ"}, {"Syowa", "स्योवा वेळ"}, {"Tahiti", "ताहिती वेळ"}, {"Taipei", "तैपेई प्रमाणित वेळ"}, {"Tajikistan", "तजीकिस्तान वेळ"}, {"Tokelau", "टोकलाऊ वेळ"}, {"Tonga", "टोंगा प्रमाणित वेळ"}, {"Truk", "चुक वेळ"}, {"Turkmenistan", "तुर्कमेनिस्तान प्रमाणित वेळ"}, {"Tuvalu", "टुवालू वेळ"}, {"Uruguay", "उरुग्वे प्रमाणित वेळ"}, {"Uzbekistan", "उज़्बेकिस्तान प्रमाणित वेळ"}, {"Vanuatu", "वनातू प्रमाणित वेळ"}, {"Venezuela", "वेनेझुएला वेळ"}, {"Vladivostok", "व्लादिवोस्तोक प्रमाणित वेळ"}, {"Volgograd", "व्होल्गोग्राड प्रमाणित वेळ"}, {"Vostok", "वोस्तोक वेळ"}, {"Wake", "वैक आयलँड वेळ"}, {"Wallis", "वालिस आनी फ्यूचूना वेळ"}, {"Yakutsk", "यकुत्स्क प्रमाणित वेळ"}, {"Yekaterinburg", "येकातेरिनबर्ग प्रमाणित वेळ"}, {"Yukon", "युकॉन वेळ"}},
		{{"Alaska", "अलास्का डेलायट वेळ"}, {"Amazon", "अमेझोन ग्रीष्म वेळ"}, {"America_Central", "मध्य डेलायट वेळ"}, {"America_Eastern", "उदेंत डेलायट वेळ"}, {"America_Mountain", "पर्वतीय डेलायट वेळ"}, {"America_Pacific", "प्रशांत डेलायट वेळ"}, {"Apia", "अपिया डेलायट वेळ"}, {"Arabian", "अरबी डेलायट वेळ"}, {"Argentina", "अर्जेंटिना ग्रीष्म वेळ"}, {"Argentina_Western", "अस्तंत अर्जेंटिना ग्रीष्म वेळ"}, {"Armenia", "आर्मेनिया ग्रीष्म वेळ"}, {"Atlantic", "अटलांटीक डेलायट वेळ"}, {"Australia_Central", "ऑस्ट्रेलीयन मध्य डेलायट वेळ"}, {"Australia_CentralWestern", "ऑस्ट्रेलीयन मध्य अस्तंत डेलायट वेळ"}, {"Australia_Eastern", "ऑस्ट्रेलीयन उदेंत डेलायट वेळ"}, {"Australia_Western", "ऑस्ट्रेलीयन अस्तंत डेलायट वेळ"}, {"Azerbaijan", "अजरबैजान ग्रीष्म वेळ"}, {"Azores", "अझोरेस ग्रीष्म वेळ"}, {"Bangladesh", "बांगलादेश ग्रीष्म वेळ"}, {"Brasilia", "ब्राझिलिया ग्रीष्म वेळ"}, {"Cape_Verde", "केप वर्दे ग्रीष्म वेळ"}, {"Chatham", "चॅथम डेलायट वेळ"}, {"Chile", "चिली ग्रीष्म वेळ"}, {"China", "चीन डेलायट वेळ"}, {"Colombia", "कोलंबिया ग्रीष्म वेळ"}, {"Cook", "कूक आयलँड अर्द ग्रीष्म वेळ"}, {"Cuba", "क्युबा डेलायट वेळ"}, {"Easter", "ईस्टर आयलँड ग्रीष्म वेळ"}, {"Europe_Central", "मध्य युरोपियन ग्रीष्म वेळ"}, {"Europe_Eastern", "उदेंत युरोपियन ग्रीष्म वेळ"}, {"Europe_Western", "अस्तंत युरोपियन ग्रीष्म वेळ"}, {"Falkland", "फॉकलँड आयलँड्स ग्रीष्म वेळ"}, {"Fiji", "फिजी ग्रीष्म वेळ"}, {"Georgia", "जॉर्जिया ग्रीष्म वेळ"}, {"Greenland_Eastern", "उदेंत ग्रीनलँड ग्रीष्म वेळ"}, {"Greenland_Western", "अस्तंत ग्रीनलँड ग्रीष्म वेळ"}, {"Hawaii_Aleutian", "हवाई-अलेयुशिन डेलायट वेळ"}, {"Hong_Kong", "हाँग काँग ग्रीष्म वेळ"}, {"Hovd", "होव्हड ग्रीष्म वेळ"}, {"Iran", "इरान डेलायट वेळ"}, {"Irkutsk", "ईर्कुटस्क ग्रीष्म वेळ"}, {"Israel", "इज़राइल डेलायट वेळ"}, {"Japan", "जपान डेलायट वेळ"}, {"Korea", "कोरियन डेलायट वेळ"}, {"Krasnoyarsk", "क्रास्नोयार्स्क ग्रीष्म वेळ"}, {"Lord_Howe", "लॉर्ड होवे डेलायट वेळ"}, {"Magadan", "मगादान ग्रीष्म वेळ"}, {"Mauritius", "मॉरिशस ग्रीष्म वेळ"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत डेलायट वेळ"}, {"Mongolia", "उलानबतार ग्रीष्म वेळ"}, {"Moscow", "मॉस्को ग्रीष्म वेळ"}, {"New_Caledonia", "न्यु कॅलेडोनिया ग्रीष्म वेळ"}, {"New_Zealand", "न्युझीलॅन्ड डेलायट वेळ"}, {"Newfoundland", "न्युफावंडलँड डेलायट वेळ"}, {"Norfolk", "नॉरफॉक आयलँड ग्रीष्म वेळ"}, {"Noronha", "फर्नांडो दी नोरोन्हा ग्रीष्म वेळ"}, {"Novosibirsk", "नोवोसिबिर्स्क ग्रीष्म वेळ"}, {"Omsk", "ओम्स्क ग्रीष्म वेळ"}, {"Pakistan", "पाकिस्तान ग्रीष्म वेळ"}, {"Paraguay", "परागुआ ग्रीष्म वेळ"}, {"Peru", "पेरू ग्रीष्म वेळ"}, {"Philippines", "फिलिपायन ग्रीष्म वेळ"}, {"Pierre_Miquelon", "सेंट पियर आनी मिकलान डेलायट वेळ"}, {"Sakhalin", "सखलिन ग्रीष्म वेळ"}, {"Samoa", "सामोआ डेलायट वेळ"}, {"Taipei", "तैपेई डेलायट वेळ"}, {"Tonga", "टोंगा ग्रीष्म वेळ"}, {"Turkmenistan", "तुर्कमेनिस्तान ग्रीष्म वेळ"}, {"Uruguay", "उरुग्वे ग्रीष्म वेळ"}, {"Uzbekistan", "उज़्बेकिस्तान ग्रीष्म वेळ"}, {"Vanuatu", "वनातू ग्रीष्म वेळ"}, {"Vladivostok", "व्लादिवोस्तोक ग्रीष्म वेळ"}, {"Volgograd", "व्होल्गोग्राड ग्रीष्म वेळ"}, {"Yakutsk", "यकुत्स्क ग्रीष्म वेळ"}, {"Yekaterinburg", "येकातेरिनबर्ग ग्रीष्म वेळ"}},
	},
	{
		{{"GMT", "Greenwich Promann Vell"}},
		{},
	},
	{
		{{"Acre", "اؠکرے سٹینڑاڑ ٹایِم"}, {"Afghanistan", "افغانِستان ٹایِم"}, {"Africa_Central", "مرکزی افریٖقا ٹایِم"}, {"Africa_Eastern", "مشرقی افریٖقا ٹایِم"}, {"Africa_Southern", "جنوٗبی افریقا ٹایِم"}, {"Africa_Western", "مغربی افریٖقا ٹایِم"}, {"Alaska", "اؠلاسکا سٹینڑاڑ ٹایِم"}, {"Almaty", "اؠلمؠٹی سٹینڑاڑ ٹایِم"}, {"Amazon", "اؠمَزَن سٹینڑاڑ ٹایِم"}, {"America_Central", "مرکزی سٹینڑاڑ ٹایِم"}, {"America_Eastern", "مشرقی سٹینڑاڑ ٹایِم"}, {"America_Mountain", "ماونٹین سٹینڑاڑ ٹایِم"}, {"America_Pacific", "پیسِفِک سٹینڑاڑ ٹایِم"}, {"Anadyr", "اؠنَڑیٖر سٹینڑاڑ ٹایِم"}, {"Aqtau", "اؠکٹاؤ سٹینڑاڑ ٹایِم"}, {"Aqtobe", "اؠکٹوب سٹینڑاڑ ٹایِم"}, {"Arabian", "ارؠبِیَن سٹینڈرڈ ٹایِم"}, {"Argentina", "ارجؠنٹیٖنا سٹینڑاڑ ٹایِم"}, {"Argentina_Western", "مغربی ارجؠنٹیٖنا سٹینڑاڑ ٹایِم"}, {"Armenia", "ارمیٖنِیا سٹینڈرڈ ٹایِم"}, {"Atlantic", "اؠٹلانٹِک سٹینڑاڑ ٹایِم"}, {"Australia_Central", "آسٹریلِیَن مرکزی سٹینڑاڑ ٹایِم"}, {"Australia_CentralWestern", "آسٹریلِیَن مرکزی مغربی سٹینڑاڑ ٹایِم"}, {"Australia_Eastern", "آسٹریلِیَن مشرقی سٹینڑاڑ ٹایِم"}, {"Australia_Western", "آسٹریلِیَن مغرِبی سٹینڑاڑ ٹایِم"}, {"Azerbaijan", "ازربائیجان سٹینڈرڈ ٹائم"}, {"Azores", "اؠزورٕس سٹینڑاڑ ٹایِم"}, {"Bangladesh", "بَنگلادیش سٹینڑاڑ ٹایِم"}, {"Bhutan", "بوٗٹان ٹایِم"}, {"Bolivia", "بولِوِیا ٹایِم"}, {"Brasilia", "برؠسِلِیا سٹینڑاڑ ٹایِم"}, {"Brunei", "بروٗنَے دَروٗسَلَم ٹایِم"}, {"Cape_Verde", "کیپ ؤرڑو سٹینڈرڈ ٹایِم"}, {"Chamorro", "کؠمورو سٹینڑاڑ ٹایِم"}, {"Chatham", "کؠتھَم سٹینڑاڑ ٹایِم"}, {"Chile", "چِلی سٹینڑاڑ ٹایِم"}, {"China", "چَینا سٹینڈرڈ ٹایِم"}, {"Christmas", "کرسمَس ٹایِم"}, {"Cocos", "کوکوز اَیلینڑز ٹایِم"}, {"Colombia", "کولومبِیا سٹینڑاڑ ٹایِم"}, {"Cook", "کُک اَیلینڑز سٹینڑاڑ ٹایِم"}, {"Cuba", "کیوٗبا سٹینڑاڑ ٹایِم"}, {"Davis", "ڑیوِس ٹایِم"}, {"DumontDUrville", "ڑمانٹ ڈی اُرویٖل ٹایِم"}, {"East_Timor", "ایٖسٹ ٹیٖمَر ٹایِم"}, {"Easter", "ایٖسٹَر جزیرٕ سٹینڈرڈ ٹایِم"}, {"Ecuador", "اِکویڑَر ٹایِم"}, {"Europe_Central", "مرکزی یوٗرپی سٹینڑاڑ ٹایِم"}, {"Europe_Eastern", "مشرقی یوٗرپی سٹینڑاڑ ٹایِم"}, {"Europe_Further_Eastern", "مزید مشرقی یورپی ٹائم"}, {"Europe_Western", "مغرِبی یوٗرپی سٹینڈرڈ ٹایِم"}, {"Falkland", "فالک لینڈ جزیرٕ سٹینڈرڈ ٹائم"}, {"Fiji", "فیٖجی سٹینڑاڑ ٹایِم"}, {"French_Guiana", "فرؠنچ گیوٗؠنا ٹایِم"}, {"French_Southern", "فرینچ جنوبی تٕہ انٹارکٹِک ٹائم"}, {"GMT", "گریٖن وِچ میٖن ٹایِم"}, {"Galapagos", "گؠلؠپیگوز ٹایِم"}, {"Gambier", "گؠمبِیَر ٹایِم"}, {"Georgia", "جورجِیاہُک سٹینڈرڈ ٹایِم"}, {"Gilbert_Islands", "گِلبٲٹ ججیٖرُک ٹایِم"}, {"Greenland_Eastern", "مشرِقی گریٖن لینڈُک سٹینڑاڑ ٹایِم"}, {"Greenland_Western", "مغرِبی گریٖن لینڈُک سٹینڑاڑ ٹایِم"}, {"Guam", "گُوؠم ٹایِم"}, {"Gulf", "گَلف سٹینڈرڈ ٹایِم"}, {"Guyana", "گُیَنا ٹایِم"}, {"Hawaii", "حَواے اؠلیوٗٹِیَن سٹینڑاڑ ٹایِم"}, {"Hawaii_Aleutian", "حَواے اؠلیوٗٹِیَن سٹینڑاڑ ٹایِم"}, {"Hong_Kong", "ہانگ کانگ سٹینڈرڈ ٹائم"}, {"Hovd", "حووڑ سٹینڈرڈ ٹایِم"}, {"India", "ہِندوستان"}, {"Indian_Ocean", "ہِندوستٲنؠ اوشَن ٹائم"}, {"Indochina", "اِنڑوچَینا ٹایِم"}, {"Indonesia_Central", "مرکزی اِنڑونیشِیا ٹایِم"}, {"Indonesia_Eastern", "مشرِقی اِنڑونیشِیا ٹایِم"}, {"Indonesia_Western", "مغرِبی اِنڑونیشِیا ٹایِم"}, {"Iran", "اِیٖرٲنؠ سٹینڑاڑ ٹایِم"}, {"Irkutsk", "اِرکُٹسک سٹینڈرڈ ٹایِم"}, {"Israel", "اِسرٲیِلی سٹینڈرڈ ٹایِم"}, {"Japan", "جاپٲنؠ سٹینڈرڈ ٹایِم"}, {"Kamchatka", "کَمچَٹکا سٹینڑاڑ ٹایِم"}, {"Kazakhstan_Eastern", "مشرقی قازقستان ٹائم"}, {"Kazakhstan_Western", "مغربی قازقستان ٹائم"}, {"Korea", "کورِیا سٹینڈرڈ ٹایِم"}, {"Kosrae", "کورسَے ٹایِم"}, {"Krasnoyarsk", "کرؠسنوےیارسک سٹینڈرڈ ٹایِم"}, {"Kyrgystan", "کرغزستان ٹائم"}, {"Lanka", "لَنکا ٹایِم"}, {"Line_Islands", "لایِٔن ججیٖرُک ٹایِم"}, {"Lord_Howe", "لعاڑ حووے سٹینڑاڑ ٹایِم"}, {"Macau", "مَکَعوٗ سٹینڑاڑ ٹایِم"}, {"Magadan", "مَگَدَن سٹینڈرڈ ٹایِم"}, {"Malaysia", "مَلیشِیا ٹایِم"}, {"Maldives", "مالدیٖوٕز ٹایِم"}, {"Marquesas", "مارقیوٗسَس ٹایِم"}, {"Marshall_Islands", "مارشَل ججیٖرُک ٹایِم"}, {"Mauritius", "مورِشَس سٹینڈرڈ ٹایِم"}, {"Mawson", "ماسَن ٹایِم"}, {"Mexico_Pacific", "میکسیکن پیسیفک سٹینڈرڈ ٹائم"}, {"Mongolia", "اولن باٹر سٹینڈرڈ ٹائم"}, {"Moscow", "ماسکو سٹینڈرڈ ٹایِم"}, {"Myanmar", "مِیانمَر ٹایِم"}, {"Nauru", "نَعوٗروٗ ٹایِم"}, {"Nepal", "نؠپٲلؠ ٹایِم"}, {"New_Caledonia", "نِو کیلؠڑونِیا سٹینڑاڑ ٹایِم"}, {"New_Zealand", "نِوزِلینڑ سٹینڑاڑ ٹایِم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ سٹینڈرڈ ٹائم"}, {"Niue", "نِیوٗ ٹایِم"}, {"Norfolk", "نورفعاک سٹینڑاڑ ٹایِم"}, {"Noronha", "فرنینڈو ڈی نورونہا سٹینڈرڈ ٹائم"}, {"North_Mariana", "شُمٲلی مَرِیانا ٹایِم"}, {"Novosibirsk", "نۄوۄسِبٔرسک سٹینڈرڈ ٹایِم"}, {"Omsk", "اۄمسک سٹینڈرڈ ٹایِم"}, {"Pakistan", "پاکِستان سٹینڑاڑ ٹایِم"}, {"Palau", "پَلاو ٹایِم"}, {"Papua_New_Guinea", "پاپُعا نیوٗ گؠنی ٹایِم"}, {"Paraguay", "پیرؠگوے سٹینڈرڈ ٹایِم"}, {"Peru", "پٔروٗ سٹینڈرڈ ٹایِم"}, {"Philippines", "پھِلِپایِن سٹینڑاڑ ٹایِم"}, {"Phoenix_Islands", "پھونِکس ججیٖرُک ٹایِم"}, {"Pierre_Miquelon", "سینٹ پَیری مِقیوٗلَن سٹینڑاڑ ٹایِم"}, {"Pitcairn", "پِٹکیرٕن ٹایِم"}, {"Ponape", "پونیپ ٹایِم"}, {"Pyongyang", "یونگ یانگ ٹائم"}, {"Qyzylorda", "قِزلوڑا سٹینڑاڑ ٹایِم"}, {"Reunion", "رِیوٗنِیَن ٹایِم"}, {"Rothera", "روتھؠرا ٹایِم"}, {"Sakhalin", "سَکھؠلِن سٹینڈرڈ ٹایِم"}, {"Samara", "سمؠرا سٹینڑاڑ ٹایِم"}, {"Samoa", "سؠموآ سٹینڑاڑ ٹایِم"}, {"Seychelles", "سیشؠلٕز ٹایِم"}, {"Singapore", "سِنگاپوٗر ٹایِم"}, {"Solomon", "سولومَن ججیٖرَن ہُند ٹایِم"}, {"South_Georgia", "شُمٲلی جورجِیا ٹایِم"}, {"Suriname", "سُرِنام ٹایِم"}, {"Syowa", "سیووا ٹایِم"}, {"Tahiti", "ٹاہِٹی ٹایِم"}, {"Taipei", "ٹے پے سٹینڈرڈ ٹائم"}, {"Tajikistan", "تاجکستان ٹائم"}, {"Tokelau", "ٹوکؠلو ٹایِم"}, {"Tonga", "ٹعانگا سٹینڑاڑ ٹایِم"}, {"Truk", "ٹٔرک ٹایِم"}, {"Turkey", "تُرکی سٹینڑاڑ ٹایِم"}, {"Turkmenistan", "ترکمانستان سٹینڈرڈ ٹائم"}, {"Tuvalu", "ٹوٗوَلوٗ ٹایِم"}, {"Uruguay", "یوٗرؠگوَے سٹینڈرڈ ٹایِم"}, {"Uzbekistan", "اُزبیکِستان سٹینڈرڈ ٹایِم"}, {"Vanuatu", "وَنوٗاَٹوٗ سٹینڑاڑ ٹایِم"}, {"Venezuela", "وؠنؠزیوٗلا ٹایِم"}, {"Vladivostok", "ولاڑِووسٹوک سٹینڈرڈ ٹایِم"}, {"Volgograd", "وولگوگریڑ سٹینڈرڈ ٹایِم"}, {"Vostok", "ووسٹوک ٹایِم"}, {"Wake", "ویک ججیٖرُک ٹایِم"}, {"Wallis", "والِس تہٕ فیوٗٹیوٗنا ٹایِم"}, {"Yakutsk", "یَکُٹسک سٹینڈرڈ ٹایِم"}, {"Yekaterinburg", "یؠکَٹٔرِنبٔرگ سٹینڈرڈ ٹایِم"}, {"Yukon", "یوکون ٹائم"}},
		{{"Acre", "اؠکرے سَمَر ٹایِم"}, {"Alaska", "اؠلاسکا ڈےلایِٔٹ ٹایِم"}, {"Almaty", "اؠلمؠٹی سَمَر ٹایِم"}, {"Amazon", "اؠمَزَن سَمَر ٹایِم"}, {"America_Central", "مرکزی ڈےلایِٔٹ ٹایِم"}, {"America_Eastern", "مشرقی ڈےلایِٔٹ ٹایِم"}, {"America_Mountain", "ماونٹین ڈےلایِٔٹ ٹایِم"}, {"America_Pacific", "پیسِفِک ڈےلایِٔٹ ٹایِم"}, {"Anadyr", "اؠنڑیٖر سَمَر ٹایِم"}, {"Aqtau", "اؠکٹاؤ سَمَر ٹایِم"}, {"Aqtobe", "اؠکٹوب سَمَر ٹایِم"}, {"Arabian", "ارؠبِیَن ڈےلایِٔٹ ٹایِم"}, {"Argentina", "ارجؠنٹیٖنا سَمَر ٹایِم"}, {"Argentina_Western", "مغربی ارجؠنٹیٖنا سَمَر ٹایِم"}, {"Armenia", "ارمیٖنِیا سَمَر ٹایِم"}, {"Atlantic", "اؠٹلانٹِک ڈےلایِٔٹ ٹایِم"}, {"Australia_Central", "آسٹریلِیَن مرکزی ڈےلایِٔٹ ٹایِم"}, {"Australia_CentralWestern", "آسٹریلِیَن مرکزی مغربی ڈےلایِٔٹ ٹایِم"}, {"Australia_Eastern", "آسٹریلِیَن مشرقی ڈےلایِٔٹ ٹایِم"}, {"Australia_Western", "آسٹریلِیَن مغرِبیٖ ڈےلایٔٹ ٹایِم"}, {"Azerbaijan", "ازربائیجان سَمَر ٹائم"}, {"Azores", "اؠزورٕس سَمَر ٹائم"}, {"Bangladesh", "بَنگلادیش سَمَر ٹایِم"}, {"Brasilia", "برؠسِلِیا سَمَر ٹایِم"}, {"Cape_Verde", "کیپ سَمَر ٹایِم"}, {"Chatham", "چؠتھَم سَمَر ٹایِم"}, {"Chile", "چِلی سَمَر ٹایِم"}, {"China", "چَینا ڈےلایِٔٹ ٹایِم"}, {"Colombia", "کولومبِیا سَمَر ٹایِم"}, {"Cook", "کُک اَیلینڑز حاف سَمَر ٹایِم"}, {"Cuba", "کیوٗبا ڈےلایِٔٹ ٹایِم"}, {"Easter", "ایٖسٹَر جزیرٕ سَمَر ٹایِم"}, {"Europe_Central", "مرکزی یوٗرپی سَمَر ٹایِم"}, {"Europe_Eastern", "مشرقی یوٗرپی سَمَر ٹایِم"}, {"Europe_Western", "مغرِبی یوٗرِپی سَمَر ٹایِم"}, {"Falkland", "فالک لینڈ جزیرٕ سَمَر ٹائم"}, {"Fiji", "فیٖجی سَمَر ٹایِم"}, {"Georgia", "جورجِیاہُک سَمَر ٹایِم"}, {"Greenland_Eastern", "مشرِقی گریٖن لینڈُک سَمَر ٹایِم"}, {"Greenland_Western", "مغرِبی گریٖن لینڈُک سَمَر ٹایِم"}, {"Hawaii_Aleutian", "حَواے اؠلیوٗٹِیَن سَمَر ٹایِم"}, {"Hong_Kong", "ہانگ کانگ سَمر ٹائم"}, {"Hovd", "حووڑ سَمَر ٹایِم"}, {"Iran", "اِیٖرٲنی سَمَر ٹایِم"}, {"Irkutsk", "اِرکُٹسک سَمَر ٹایِم"}, {"Israel", "اِسرٲیِلی ڑےلایِٔٹ ٹایِم"}, {"Japan", "جاپٲنؠ ڑےلایِٔٹ ٹایِم"}, {"Kamchatka", "کَمچَٹکا سَمَر ٹایِم"}, {"Korea", "کورِیا ڑےلایِٔٹ ٹایِم"}, {"Krasnoyarsk", "کرؠسنوےیارسک سَمَر ٹایِم"}, {"Lord_Howe", "لعاڑ ڑےلایٔٹ ٹایِم"}, {"Macau", "مَکَعوٗ سَمَر ٹایِم"}, {"Magadan", "مَگَدَن سَمَر ٹایِم"}, {"Mauritius", "مورِشَس سَمَر ٹایِم"}, {"Mexico_Pacific", "میکسیکن پیسیفک ڈے لائٹ ٹائم"}, {"Mongolia", "اولن باٹر سَمَر ٹایِم"}, {"Moscow", "ماسکو سَمَر ٹایِم"}, {"New_Caledonia", "نِو کیلؠڑونِیس سَمَر ٹایِم"}, {"New_Zealand", "نِوزِلینڑ ڑےلایٔٹ ٹایِم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ ڈے لائٹ ٹائم"}, {"Norfolk", "نورفعاک سَمَر ٹایِم"}, {"Noronha", "فرنینڈو ڈی نورونہا سَمَر ٹائم"}, {"Novosibirsk", "نۄوۄسِبٔرسک سَمَر ٹایِم"}, {"Omsk", "اۄمسک سَمَر ٹایِم"}, {"Pakistan", "پاکِستان سَمَر ٹایِم"}, {"Paraguay", "پیرؠگوے سَمَر ٹایِم"}, {"Peru", "پٔروٗ سَمَر ٹایِم"}, {"Philippines", "پھِلِپایِن سَمَر ٹایِم"}, {"Pierre_Miquelon", "سینٹ پَیری مِقیوٗلَن ڑےلایِٔٹ ٹایِم"}, {"Qyzylorda", "قِزلوڑا سَمَر ٹایِم"}, {"Sakhalin", "سَکھؠلِن سَمَر ٹایِم"}, {"Samara", "سمؠرا سَمَر ٹایِم"}, {"Samoa", "سؠموآ سَمَر ٹایِم"}, {"Taipei", "ٹے پے ڈے لائٹ ٹائم"}, {"Tonga", "ٹعانگا سَمَر ٹایِم"}, {"Turkey", "تُرکی سَمَر ٹایِم"}, {"Turkmenistan", "تُرکمؠنِستان سَمَر ٹایِم"}, {"Uruguay", "یوٗرؠگوَے سَمَر ٹایِم"}, {"Uzbekistan", "اُزبیکِستانُک سَمَر ٹایِم"}, {"Vanuatu", "وَنوٗاَٹوٗ سَمَر ٹایِم"}, {"Vladivostok", "ولاڑِووسٹوک سَمَر ٹایِم"}, {"Volgograd", "وولگوگریڑ سَمَر ٹایِم"}, {"Yakutsk", "یَکُٹُسک سَمَر ٹایِم"}, {"Yekaterinburg", "یؠکَٹرِنبٔرگ سَمَر ٹایِم"}},
	},
	{
		{{"America_Central", "सेंट्रल स्टैन्डर्ड वख"}, {"America_Eastern", "मशरिकी स्टैन्डर्ड वख"}, {"America_Mountain", "माउंटेन स्टैन्डर्ड वख"}, {"America_Pacific", "पेसिफिक स्टैन्डर्ड वख"}, {"Atlantic", "अटलांटिक स्टैन्डर्ड वख"}, {"Europe_Central", "मरकज़ी यूरपी स्टैन्डर्ड वख"}, {"Europe_Eastern", "मशरिकी यूरपी स्टैन्डर्ड वख"}, {"Europe_Western", "मगरीबी यूरपी स्टैन्डर्ड वख"}, {"GMT", "ग्रीनविच ओसत वख"}},
		{{"America_Central", "सेंट्रल डे लाइट वख"}, {"America_Eastern", "मशरिकी डे लाइट वख"}, {"America_Mountain", "माउंटेन डे लाइट वख"}, {"America_Pacific", "पेसिफिक डे लाइट वख"}, {"Atlantic", "अटलांटिक डे लाइट वख"}, {"Europe_Central", "मरकज़ी यूरपी समर वख"}, {"Europe_Eastern", "मशरिकी यूरपी समर वख"}, {"Europe_Western", "मगरीबी यूरपी समर वख"}},
	},
	{
		{{"Africa_Central", "Zentraal-Affrekaanesche Zigg"}, {"Africa_Eastern", "Oß-Affrekaanesche Zigg"}, {"Africa_Southern", "Söd-Affrekaanesche Zigg"}, {"Africa_Western", "Wäß-Affrekaanesche Zigg"}, {"Azores", "de Azore ier jewöhnlijje Zick"}, {"Cape_Verde", "Jewöhnlijje Kapvärdejaansche Zigg"}, {"Europe_Central", "Meddel-Europpa sing jewöhnlijje Zick"}, {"Europe_Eastern", "Oß-Europpa sing jewöhnlijje Zick"}, {"Europe_Western", "Weß-Europpa sing jewöhnlijje Zick"}, {"GMT", "Greenwich sing Standat-Zick"}, {"Indian_Ocean", "dem Indische Ozejan sing Zick"}, {"Mauritius", "Jewöhnlijje Zigg vun Mauritius"}, {"Reunion", "Zigg vun Reunion"}, {"Seychelles", "Zigg vun de Seischälle"}},
		{{"Azores", "de Azore ier Summerzick"}, {"Cape_Verde", "Kapvärdejaansche Sommerzigg"}, {"Europe_Central", "Meddel-Europpa sing Summerzick"}, {"Europe_Eastern", "Oß-Europpa sing Summerzick"}, {"Europe_Western", "Weß-Europpa sing Summerzick"}, {"Mauritius", "Summerzigg vun Mauritius"}},
	},
	{
		{{"Acre", "Saeta Acreyê ya Standard"}, {"Afghanistan", "Saeta Efxanistanê"}, {"Africa_Central", "Saeta Afrîkaya Navîn"}, {"Africa_Eastern", "Saeta Afrîkaya Rojhilat"}, {"Africa_Southern", "Saeta Standard a Afrîkaya Başûr"}, {"Africa_Western", "Saeta Afrîkaya Rojava"}, {"Alaska", "Saeta Standard a Alaskayê"}, {"Almaty", "Saeta Almatîyê ya Standard"}, {"Amazon", "Saeta Amazonê ya Standard"}, {"America_Central", "Saeta Merkezî ya Standard a Amerîkaya Bakur"}, {"America_Eastern", "Saeta Standard a Rojhilat ya Amerîkaya Bakur"}, {"America_Mountain", "Saeta Standard a Çîyayî ya Amerîkaya Bakur"}, {"America_Pacific", "Saeta Standard a Pasîfîkê ya Amerîkaya Bakur"}, {"Anadyr", "Saeta Anadyrê ya Standard"}, {"Apia", "Saeta Apiayê ya Standard"}, {"Aqtau", "Saeta Aqtauyê ya Standard"}, {"Aqtobe", "Saeta Aqtobeyê ya Standard"}, {"Arabian", "Saeta Erebistanê ya Standard"}, {"Argentina", "Saeta Arjantînê ya Standard"}, {"Argentina_Western", "Saeta Standard a Arjantîna Rojava"}, {"Armenia", "Saeta Ermenistanê ya Standard"}, {"Atlantic", "Saeta Standard a Atlantîkê"}, {"Australia_Central", "Saeta Standard a Awistralyaya Navîn"}, {"Australia_CentralWestern", "Saeta Standard a Rojavaya Navîn a Awistralyayê"}, {"Australia_Eastern", "Saeta Standard a Awistralyaya Rojhilat"}, {"Australia_Western", "Saeta Standard a Awistralyaya Rojava"}, {"Azerbaijan", "Saeta Azerbeycanê ya Standard"}, {"Azores", "Saeta Azoran a Standard"}, {"Bangladesh", "Saeta Bengladeşê ya Standard"}, {"Bhutan", "Saeta Bûtanê"}, {"Bolivia", "Saeta Bolîvyayê"}, {"Brasilia", "Saeta Brasîlyayê ya Standard"}, {"Brunei", "Saeta Brûneyê"}, {"Cape_Verde", "Saeta Standard a Cape Verdeyê"}, {"Casey", "Saeta Caseyê"}, {"Chamorro", "Saeta Chamorroyê ya Standard"}, {"Chatham", "Saeta Standard a Chathamê"}, {"Chile", "Saeta Şîlîyê ya Standard"}, {"China", "Saeta Çînê ya Standard"}, {"Christmas", "Saeta Girava Christmasê"}, {"Cocos", "Saeta Giravên Cocosê"}, {"Colombia", "Saeta Kolombîyayê ya Standard"}, {"Cook", "Saeta Giravên Cookê ya Standard"}, {"Cuba", "Saeta Standard a Kubayê"}, {"Davis", "Saeta Davîsê"}, {"DumontDUrville", "Saeta Dumont-d’Urvilleyê"}, {"East_Timor", "Saeta Tîmûra Rojhilat"}, {"Easter", "Saeta Standard a Girava Paskalyayê"}, {"Ecuador", "Saeta Ekwadorê"}, {"Europe_Central", "Saeta Standard a Ewropaya Navîn"}, {"Europe_Eastern", "Saeta Standard a Ewropaya Rojhilat"}, {"Europe_Further_Eastern", "Saeta Ewropaya Rojhilat a Pêştir"}, {"Europe_Western", "Saeta Standard a Ewropaya Rojava"}, {"Falkland", "Saeta Giravên Falklandê ya Standard"}, {"Fiji", "Saeta Fîjîyê ya Standard"}, {"French_Guiana", "Saeta Guiyanaya Fransî"}, {"French_Southern", "Saeta Antarktîka û Başûrê Fransayê"}, {"GMT", "Saeta Navînî ya Greenwichê"}, {"Galapagos", "Saeta Galapagosê"}, {"Gambier", "Saeta Gambierê"}, {"Georgia", "Saeta Gurcistanê ya Standard"}, {"Gilbert_Islands", "Saeta Giravên Gilbertê"}, {"Greenland", "Saeta Grînlendayê ya Standard"}, {"Greenland_Eastern", "Saeta Standard a Grînlanda Rojhilat"}, {"Greenland_Western", "Saeta Standard a Grînlanda Rojava"}, {"Guam", "Saeta Guamê ya Standard"}, {"Gulf", "Saeta Kendavê ya Standard"}, {"Guyana", "Saeta Guyanayê"}, {"Hawaii", "Saeta Standard a Hawaii-Aleutianê"}, {"Hawaii_Aleutian", "Saeta Standard a Hawaii-Aleutianê"}, {"Hong_Kong", "Saeta Hong Kongê ya Standard"}, {"Hovd", "Saeta Hovdê ya Standard"}, {"India", "Saeta Hindistanê ya Standard"}, {"Indian_Ocean", "Saeta Okyanûsa Hindê"}, {"Indochina", "Saeta Hindiçînê"}, {"Indonesia_Central", "Saeta Endonezyaya Navîn"}, {"Indonesia_Eastern", "Saeta Endonezyaya Rojhilat"}, {"Indonesia_Western", "Saeta Endonezyaya Rojava"}, {"Iran", "Saeta Îranê ya Standard"}, {"Irkutsk", "Saeta Irkutskê ya Standard"}, {"Israel", "Saeta Îsraîlê ya Standard"}, {"Japan", "Saeta Japonyayê ya Standard"}, {"Kamchatka", "Saeta Kamchatkayê ya Standard"}, {"Kazakhstan", "Saeta Qazaxistanê"}, {"Kazakhstan_Eastern", "Saeta Qazaxistana Rojhilat"}, {"Kazakhstan_Western", "Saeta Qazaxistana Rojava"}, {"Korea", "Saeta Koreyê ya Standard"}, {"Kosrae", "Saeta Kosraeyê"}, {"Krasnoyarsk", "Saeta Krasnoyarskê ya Standard"}, {"Kyrgystan", "Saeta Qirxizistanê"}, {"Lanka", "Saeta Lankayê"}, {"Line_Islands", "Saeta Giravên Lîneyê"}, {"Lord_Howe", "Saeta Standard a Lord Howeyê"}, {"Macau", "Saeta Macaoyê ya Standard"}, {"Magadan", "Saeta Magadanê ya Standard"}, {"Malaysia", "Saeta Malezyayê"}, {"Maldives", "Saeta Maldîvan"}, {"Marquesas", "Saeta Marquesasê"}, {"Marshall_Islands", "Saeta Giravên Marşalê"}, {"Mauritius", "Saeta Mauritiusê ya Standard"}, {"Mawson", "Saeta Mawsonê"}, {"Mexico_Pacific", "Saeta Standard a Pasîfîka Meksîkayê"}, {"Mongolia", "Saeta Ûlanbatarê ya Standard"}, {"Moscow", "Saeta Moskovayê ya Standard"}, {"Myanmar", "Saeta Myanmarê"}, {"Nauru", "Saeta Naûrûyê"}, {"Nepal", "Saeta Nepalê"}, {"New_Caledonia", "Saeta Standard a Kaledonyaya Nû"}, {"New_Zealand", "Saeta Standard a Zelandaya Nû"}, {"Newfoundland", "Saeta Standard a Newfoundlandê"}, {"Niue", "Saeta Niueyê"}, {"Norfolk", "Saeta Standard a Girava Norfolkê"}, {"Noronha", "Saeta Fernando de Noronhayê ya Standard"}, {"North_Mariana", "Saeta Giravên Marianaya Bakur"}, {"Novosibirsk", "Saeta Novosibirskê ya Standard"}, {"Omsk", "Saeta Omskê ya Standard"}, {"Pakistan", "Saeta Pakistanê ya Standard"}, {"Palau", "Saeta Palauyê"}, {"Papua_New_Guinea", "Saeta Gîneya Nû ya Papûayê"}, {"Paraguay", "Saeta Paragûayê ya Standard"}, {"Peru", "Saeta Perûyê ya Standard"}, {"Philippines", "Saeta Fîlîpînê ya Standard"}, {"Phoenix_Islands", "Saeta Giravên Phoenîks"}, {"Pierre_Miquelon", "Saeta Standard a Saint Pierre û Miquelonê"}, {"Pitcairn", "Saeta Pitcairnê"}, {"Ponape", "Saeta Ponapeyê"}, {"Pyongyang", "Saeta Pyongyangê"}, {"Qyzylorda", "Saeta Qizilordayê ya Standard"}, {"Reunion", "Saeta Réunionê"}, {"Rothera", "Saeta Rotherayê"}, {"Sakhalin", "Saeta Saxalînê ya Standard"}, {"Samara", "Saeta Samarayê ya Standard"}, {"Samoa", "Saeta Samoayê ya Standard"}, {"Seychelles", "Saeta Seyşelerê"}, {"Singapore", "Saeta Sîngapûrê ya Standard"}, {"Solomon", "Saeta Giravên Solomonê"}, {"South_Georgia", "Saeta Georgiaya Başûr"}, {"Suriname", "Saeta Surînamê"}, {"Syowa", "Saeta Syowayê"}, {"Tahiti", "Saeta Tahîtîyê"}, {"Taipei", "Saeta Taîpeîyê ya Standard"}, {"Tajikistan", "Saeta Tacikistanê"}, {"Tokelau", "Saeta Tokelauyê"}, {"Tonga", "Saeta Tongayê ya Standard"}, {"Truk", "Saeta Chuukê"}, {"Turkmenistan", "Saeta Tirkmenistanê ya Standard"}, {"Tuvalu", "Saeta Tûvalûyê"}, {"Uruguay", "Saeta Ûrûgûayê ya Standard"}, {"Uzbekistan", "Saeta Ozbekistanê ya Standard"}, {"Vanuatu", "Saeta Vanûatûyê ya Standard"}, {"Venezuela", "Saeta Venezûelayê"}, {"Vladivostok", "Saeta Vladivostokê ya Standard"}, {"Volgograd", "Saeta Volgogradê ya Standard"}, {"Vostok", "Saeta Vostokê"}, {"Wake", "Saeta Girava Wakeyê"}, {"Wallis", "Saeta Wallis û Futunayê"}, {"Yakutsk", "Saeta Yakutskê ya Standard"}, {"Yekaterinburg", "Saeta Yekaterinburgê ya Standard"}, {"Yukon", "Saeta Yukonê"}},
		{{"Acre", "Saeta Havînê ya Acreyê"}, {"Alaska", "Saeta Havînê ya Alaskayê"}, {"Almaty", "Saeta Havînê ya Almatîyê"}, {"Amazon", "Saeta Havînê ya Amazonê"}, {"America_Central", "Saeta Havînê ya Merkezî ya Amerîkaya Bakur"}, {"America_Eastern", "Saeta Havînê ya Rojhilat ya Amerîkaya Bakur"}, {"America_Mountain", "Saeta Havînê ya Çîyayî ya Amerîkaya Bakur"}, {"America_Pacific", "Saeta Havînê ya Pasîfîkê ya Amerîkaya Bakur"}, {"Anadyr", "Saeta Havînê ya Anadyrê"}, {"Apia", "Saeta Havînê ya Apiayê"}, {"Aqtau", "Saeta Havînê ya Aqtauyê"}, {"Aqtobe", "Saeta Havînê ya Aqtobeyê"}, {"Arabian", "Saeta Havînê ya Erebistanê"}, {"Argentina", "Saeta Havînê ya Arjantînê"}, {"Argentina_Western", "Saeta Havînê ya Arjantîna Rojava"}, {"Armenia", "Saeta Havînê ya Ermenistanê"}, {"Atlantic", "Saeta Havînê ya Atlantîkê"}, {"Australia_Central", "Saeta Havînê ya Awistralyaya Navîn"}, {"Australia_CentralWestern", "Saeta Havînê ya Rojavaya Navîn a Awistralyayê"}, {"Australia_Eastern", "Saeta Havînê ya Awistralyaya Rojhilat"}, {"Australia_Western", "Saeta Havînê ya Awistralyaya Rojava"}, {"Azerbaijan", "Saeta Havînê ya Azerbeycanê"}, {"Azores", "Saeta Havînê ya Azoran"}, {"Bangladesh", "Saeta Havînê ya Bengladeşê"}, {"Brasilia", "Saeta Havînê ya Brasîlyayê"}, {"Cape_Verde", "Saeta Havînê ya Cape Verdeyê"}, {"Chatham", "Saeta Havînê ya Chathamê"}, {"Chile", "Saeta Havînê ya Şîlîyê"}, {"China", "Saeta Havînê ya Çînê"}, {"Colombia", "Saeta Havînê ya Kolombîyayê"}, {"Cook", "Saeta Havînê ya Giravên Cookê"}, {"Cuba", "Saeta Havînê ya Kubayê"}, {"Easter", "Saeta Havînê ya Girava Paskalyayê"}, {"Europe_Central", "Saeta Havînê ya Ewropaya Navîn"}, {"Europe_Eastern", "Saeta Havînê ya Ewropaya Rojhilat"}, {"Europe_Western", "Saeta Havînê ya Ewropaya Rojava"}, {"Falkland", "Saeta Havînê ya Giravên Falklandê"}, {"Fiji", "Saeta Havînê ya Fîjîyê"}, {"Georgia", "Saeta Havînê ya Gurcistanê"}, {"Greenland", "Saeta Havînê ya Grînlendayê"}, {"Greenland_Eastern", "Saeta Havînê ya Grînlanda Rojhilat"}, {"Greenland_Western", "Saeta Havînê ya Grînlanda Rojava"}, {"Hawaii_Aleutian", "Saeta Havînê ya Hawaii-Aleutianê"}, {"Hong_Kong", "Saeta Havînê ya Hong Kongê"}, {"Hovd", "Saeta Havînê ya Hovdê"}, {"Iran", "Saeta Havînê ya Îranê"}, {"Irkutsk", "Saeta Havînê ya Irkutskê"}, {"Israel", "Saeta Havînê ya Îsraîlê"}, {"Japan", "Saeta Havînê ya Japonyayê"}, {"Kamchatka", "Saeta Havînê ya Kamchatkayê"}, {"Korea", "Saeta Havînê ya Koreyê"}, {"Krasnoyarsk", "Saeta Havînê ya Krasnoyarskê"}, {"Lord_Howe", "Saeta Havînê ya Lord Howeyê"}, {"Macau", "Saeta Havînê ya Macaoyê"}, {"Magadan", "Saeta Havînê ya Magadanê"}, {"Mauritius", "Saeta Havînê ya Mauritiusê"}, {"Mexico_Pacific", "Saeta Havînê ya Pasîfîka Meksîkayê"}, {"Mongolia", "Saeta Havînê ya Ûlanbatarê"}, {"Moscow", "Saeta Havînê ya Moskovayê"}, {"New_Caledonia", "Saeta Havînê ya Kaledonyaya Nû"}, {"New_Zealand", "Saeta Havînê ya Zelandaya Nû"}, {"Newfoundland", "Saeta Havînê ya Newfoundlandê"}, {"Norfolk", "Saeta Havînê ya Girava Norfolkê"}, {"Noronha", "Saeta Havînê ya Fernando de Noronhayê"}, {"Novosibirsk", "Saeta Havînê ya Novosibirskê"}, {"Omsk", "Saeta Havînê ya Omskê"}, {"Pakistan", "Saeta Havînê ya Pakistanê"}, {"Paraguay", "Saeta Havînê ya Paragûayê"}, {"Peru", "Saeta Havînê ya Perûyê"}, {"Philippines", "Saeta Havînê ya Fîlîpînê"}, {"Pierre_Miquelon", "Saeta Havînê ya Saint Pierre û Miquelonê"}, {"Qyzylorda", "Saeta Havînê ya Qizilordayê"}, {"Sakhalin", "Saeta Havînê ya Saxalînê"}, {"Samara", "Saeta Havînê ya Samarayê"}, {"Samoa", "Saeta Havînê ya Samoayê"}, {"Taipei", "Saeta Havînê ya Taîpeîyê"}, {"Tonga", "Saeta Havînê ya Tongayê"}, {"Turkmenistan", "Saeta Havînê ya Tirkmenistanê"}, {"Uruguay", "Saeta Havînê ya Ûrûgûayê"}, {"Uzbekistan", "Saeta Havînê ya Ozbekistanê"}, {"Vanuatu", "Saeta Havînê ya Vanûatûyê"}, {"Vladivostok", "Saeta Havînê ya Vladivostokê"}, {"Volgograd", "Saeta Havînê ya Volgogradê"}, {"Yakutsk", "Saeta Havînê ya Yakutskê"}, {"Yekaterinburg", "Saeta Havînê ya Yekaterinburgê"}},
	},
	{
		{{"Afghanistan", "aapganistan belā"}, {"Africa_Central", "madini aaprika belā"}, {"Africa_Eastern", "veḍa hpu aaprika belā"}, {"Africa_Southern", "dkiṇ aaprika belā"}, {"Africa_Western", "veḍakuṇpu aaprika belā"}, {"Alaska", "alaska mananka belā"}, {"Amazon", "āmajon mananka belā"}, {"America_Central", "madinī mananka belā"}, {"America_Eastern", "veḍahapu mananka belā"}, {"America_Mountain", "harka mananka belā"}, {"America_Pacific", "pesipik mananka belā"}, {"Apia", "epiā mānānka belā"}, {"Arabian", "ārbiāti mānānka belā"}, {"Argentina", "ārjenṭinā mānānka belā"}, {"Argentina_Western", "weḍākuṇupu ārjenṭinā mānānka belā"}, {"Armenia", "ārmeniā mānānka belā"}, {"Atlantic", "āṭlānṭik mānānka belā"}, {"Australia_Central", "āstreliāti mādinā mānānka belā"}, {"Australia_CentralWestern", "āstreliāti mādinā weḍākuṇupu mānānka belā"}, {"Australia_Eastern", "āstreliāti weḍāhapū mānānka belā"}, {"Australia_Western", "āstreliāti weḍākuṇupū mānānka belā"}, {"Azerbaijan", "ājerbāijān mānānka belā"}, {"Azores", "ajores mānānka belā"}, {"Bangladesh", "bānglādes mānānka belā"}, {"Bhutan", "butān belā"}, {"Bolivia", "bolwiā belā"}, {"Brasilia", "brājiliā mānānka belā"}, {"Brunei", "bruneti dārusālām belā"}, {"Cape_Verde", "kep bḍ mānānka belā"}, {"Chamorro", "cāmor mānānka belā"}, {"Chatham", "cyātām mānānka belā"}, {"Chile", "cili mānānka belā"}, {"China", "cin mānānka belā"}, {"Christmas", "krismās dīp belā"}, {"Cocos", "kokos dīp belā"}, {"Colombia", "kolombiā mānānka belā"}, {"Cook", "kuk dīp mānānka belā"}, {"Cuba", "kubā mānānka belā"}, {"Davis", "debis belā"}, {"DumontDUrville", "ḍumonṭ ḍi arwilē belā"}, {"East_Timor", "weḍāhapu timor belā"}, {"Easter", "isṭr dīp mānānka belā"}, {"Ecuador", "iquāḍor belā"}, {"Europe_Central", "mādinā yuropiāti mānānka belā"}, {"Europe_Eastern", "weḍāhpu yuropiāti mānānka belā"}, {"Europe_Further_Eastern", "ar weḍāhpu yuropati belā"}, {"Europe_Western", "weḍākūṇūpu yuropiāti mānānka belā"}, {"Falkland", "paklyānḍ dīpati mānānka belā"}, {"Fiji", "piji mānānka belā"}, {"French_Guiana", "prench guyān belā"}, {"French_Southern", "prenc dakiṇ aḍe aanṭārkṭik belā"}, {"GMT", "grinwic mīn belā"}, {"Galapagos", "gālāpogs belā"}, {"Gambier", "gambiyr belā"}, {"Georgia", "jarjiā mānānka belā"}, {"Gilbert_Islands", "gīlbrṭ dīp belā"}, {"Greenland_Eastern", "weḍāhpu grinlānd mānānka belā"}, {"Greenland_Western", "weḍakūṇūp grinlānḍ mānānka belā"}, {"Gulf", "galp mānānka belā"}, {"Guyana", "guyān belā"}, {"Hawaii", "hāwāe- aalūsān mānānka belā"}, {"Hawaii_Aleutian", "hāwāe- aalūsān mānānka belā"}, {"Hong_Kong", "hang kang mānānka belā"}, {"Hovd", "hwoḍ mānānka belā"}, {"India", "bārat mānānka belā"}, {"Indian_Ocean", "bārat kājā belā"}, {"Indochina", "inḍocinā belā"}, {"Indonesia_Central", "mādini inḍnesiā belā"}, {"Indonesia_Eastern", "weḍāhpu inḍnesiā belā"}, {"Indonesia_Western", "weḍākūṇpū inḍnesiā belā"}, {"Iran", "irān mānānka belā"}, {"Irkutsk", "īrkustak mānānka belā"}, {"Israel", "ijarāīl mānānka belā"}, {"Japan", "jāpān mānānka belā"}, {"Kazakhstan", "kājākstān belā"}, {"Kazakhstan_Eastern", "weḍāhapu kājākstān belā"}, {"Kazakhstan_Western", "weḍākūṇpū kājākstān belā"}, {"Korea", "koriān mānānka belā"}, {"Kosrae", "kasrāē belā"}, {"Krasnoyarsk", "krāsnōrska mānānka belā"}, {"Kyrgystan", "kirgijstān belā"}, {"Line_Islands", "lāin dīp belā"}, {"Lord_Howe", "laṛ hawe mānānka belā"}, {"Magadan", "māgādan mānānka belā"}, {"Malaysia", "malesiā belā"}, {"Maldives", "māldīp belā"}, {"Marquesas", "mārksas belā"}, {"Marshall_Islands", "mārsāl dīpa belā"}, {"Mauritius", "marīsasmānānka belā"}, {"Mawson", "māwosn belā"}, {"Mexico_Pacific", "meksikān pesipic mānānka belā"}, {"Mongolia", "ūlānbator mānānka belā"}, {"Moscow", "mosko mānānka belā"}, {"Myanmar", "miñyāmār belā"}, {"Nauru", "nāurū belā"}, {"Nepal", "nepāl belā"}, {"New_Caledonia", "niū keleḍoniā mānānka belā"}, {"New_Zealand", "niūjilānḍ mānānka belā"}, {"Newfoundland", "niūpaunḍlyānḍ mānānka belā"}, {"Niue", "niū belā"}, {"Norfolk", "norpok dīp mānānka belā"}, {"Noronha", "parn~ḍo ḍe norohā~ mānānka belā"}, {"Novosibirsk", "nawosibisrk mānānka belā"}, {"Omsk", "amska mānānka belā"}, {"Pakistan", "pākistān mānānka belā"}, {"Palau", "pālāu belā"}, {"Papua_New_Guinea", "pāpuā niu gunīā belā"}, {"Paraguay", "pārāguyē mānānka belā"}, {"Peru", "peru mānānka belā"}, {"Philippines", "pilipin mānānka belā"}, {"Phoenix_Islands", "piniksa dīpati belā"}, {"Pierre_Miquelon", "sēnṭ pierē aḍē mekwēlān mānānka belā"}, {"Pitcairn", "piṭkēran belā"}, {"Ponape", "ponāpe belā"}, {"Pyongyang", "pyongayāng belā"}, {"Reunion", "rīūnīan belā"}, {"Rothera", "rotērā belā"}, {"Sakhalin", "sakālin mānānka belā"}, {"Samoa", "saāmoa mānānka belā"}, {"Seychelles", "sēsels belā"}, {"Singapore", "singāpur mānānka belā"}, {"Solomon", "soloman dīpati belā"}, {"South_Georgia", "dkīṇa jarjīā belā"}, {"Suriname", "surīnām belā"}, {"Syowa", "sawā belā"}, {"Tahiti", "tāhiti belā"}, {"Taipei", "tāipē mānānka belā"}, {"Tajikistan", "tājikistān belā"}, {"Tokelau", "ṭokelāu belā"}, {"Tonga", "ṭangā mānānka belā"}, {"Truk", "cuk belā"}, {"Turkmenistan", "turkmenistān manaka belā"}, {"Tuvalu", "tuwalū belā"}, {"Uruguay", "urguwē manaka belā"}, {"Uzbekistan", "uzwēkistān mānānka belā"}, {"Vanuatu", "wanuātū mānānka belā"}, {"Venezuela", "wenēzuelā belā"}, {"Vladivostok", "wlādiwostak mānānka belā"}, {"Volgograd", "wālgogrāḍ mānānka belā"}, {"Vostok", "wostak belā"}, {"Wake", "waka dīpa belā"}, {"Wallis", "walis aḍē puṭunā"}, {"Yakutsk", "yakustuk mānānka belā"}, {"Yekaterinburg", "yakaterinbarg mānānka belā"}},
		{{"Alaska", "alaska ḍelaaiṭ belā"}, {"Amazon", "āmajon karã masa belā"}, {"America_Central", "madinī ḍelaaiṭ belā"}, {"America_Eastern", "veḍahapu ḍelaaiṭ belā"}, {"America_Mountain", "harka ḍelaaiṭ belā"}, {"America_Pacific", "pesipik ḍelaaiṭ belā"}, {"Apia", "epiā delāit belā"}, {"Arabian", "ārbiāti delāit belā"}, {"Argentina", "ārjenṭinā kār~ā belā belā"}, {"Argentina_Western", "weḍākuṇupu ārjenṭinā kār~ā belā"}, {"Armenia", "ārmeniā kār~ā belā"}, {"Atlantic", "āṭlānṭik delāiṭ belā"}, {"Australia_Central", "āstreliāti mādinā delāiṭ belā"}, {"Australia_CentralWestern", "āstreliāti mādinā weḍākuṇupu delāiṭ belā"}, {"Australia_Eastern", "āstreliāti weḍāhapū delāiti belā"}, {"Australia_Western", "āstreliāti weḍākuṇupū delāiṭ belā"}, {"Azerbaijan", "ājerbāijān kār~ā belā"}, {"Azores", "ajores kār~ā belā"}, {"Bangladesh", "bānglādes kār~ā belā"}, {"Brasilia", "brājiliā kār~ā belā"}, {"Cape_Verde", "kep bḍ kār~ā belā"}, {"Chatham", "cyātām delāit belā"}, {"Chile", "cili kār~ā belā"}, {"China", "cin delāiṭ belā"}, {"Colombia", "kolombiā kār~ā belā"}, {"Cook", "kuk dīp ādā kār~ā belā"}, {"Cuba", "kubā delāiṭ belā"}, {"Easter", "isṭr dīp kār~ā belā"}, {"Europe_Central", "mādinā yuropiāti kār~ā belā"}, {"Europe_Eastern", "weḍāhpu yuropiāti kār~ā belā"}, {"Europe_Western", "weḍākūṇūpu yuropiāti kār~ā belā"}, {"Falkland", "paklyānḍ dīpati kār~ā belā"}, {"Fiji", "piji kār~ā belā"}, {"Georgia", "jarjiā kār~ā belā"}, {"Greenland_Eastern", "weḍāhpu grinlānd kār~ā belā"}, {"Greenland_Western", "weḍakūṇūp grinlānḍ kār~ā belā"}, {"Hawaii_Aleutian", "hāwāe- aalūsān ḍelāiṭ belā"}, {"Hong_Kong", "hang kang kār~ā belā"}, {"Hovd", "hwoḍ kār~ā belā"}, {"Iran", "irān ḍelāiṭ belā"}, {"Irkutsk", "īrkustak kār~ā belā"}, {"Israel", "ijarāīl ḍelāiṭ belā"}, {"Japan", "jāpān ḍelāiṭ belā"}, {"Korea", "koriān ḍelāiṭ belā"}, {"Krasnoyarsk", "krāsnōrska kār~ā belā"}, {"Lord_Howe", "laṛ hawe ḍelāiṭ belā"}, {"Magadan", "māgādan ḍelāit belā"}, {"Mauritius", "marīsas kār~ā belā"}, {"Mexico_Pacific", "meksikān pesipic ḍelāiṭ belā"}, {"Mongolia", "ūlānbator kār~ā belā"}, {"Moscow", "mosko kār~ā belā"}, {"New_Caledonia", "niū keleḍoniā kār~ā belā"}, {"New_Zealand", "niūjilānḍ ḍelāiṭ belā"}, {"Newfoundland", "niūpaunḍlyānḍ ḍelāiṭ belā"}, {"Norfolk", "norpok dīp ḍelāiṭ belā"}, {"Noronha", "parn~ḍo ḍe norohā~ kār~ā belā"}, {"Novosibirsk", "nawosibisrk kār~ā belā"}, {"Omsk", "amska kār~ā belā"}, {"Pakistan", "pākistān kār~ā belā"}, {"Paraguay", "pārāguyē kār~ā belā"}, {"Peru", "peru kār~ā belā"}, {"Philippines", "pilipin kār~ā belā"}, {"Pierre_Miquelon", "sēnṭ pierē aḍē mekwēlān ḍelāiṭ belā"}, {"Sakhalin", "sakālin kār~ā belā"}, {"Samoa", "saāmoa ḍelāiṭ"}, {"Taipei", "tāipē ḍelāiṭ"}, {"Tonga", "ṭangā kār~ā belā"}, {"Turkmenistan", "turkmenistān kār~ā belā"}, {"Uruguay", "urguwē kār~ā belā"}, {"Uzbekistan", "uzwēkistān kārã belā"}, {"Vanuatu", "wanuātū kār~ā belā"}, {"Vladivostok", "wlādiwostak kār~ā belā"}, {"Volgograd", "wālgogrāḍ kār~ā belā"}, {"Yakutsk", "yakustuk kār~ā belā"}, {"Yekaterinburg", "yakaterinbarg kār~ā belā"}},
	},
	{
		{{"Afghanistan", "आपगानिस्तान बेला"}, {"Africa_Central", "मादिनी आप्रिका बेला"}, {"Africa_Eastern", "वेड़ा हॉपु आप्रिका बेला"}, {"Africa_Southern", "दक्षिण आप्रिका मानांकॉ बेला"}, {"Africa_Western", "पस्चिम आप्रिका बेला"}, {"Alaska", "अलास्\u200dका मानांकॉ बेला"}, {"Amazon", "अमेज़न मानांकॉ बेला"}, {"America_Central", "मादिनी मानांकॉ बेला"}, {"America_Eastern", "वेड़ा हॉपु मानांकॉ बेला"}, {"America_Mountain", "हॉर्का मानांकॉ बेला"}, {"America_Pacific", "पेसिपिक मानांकॉ बेला"}, {"Apia", "एपिआ मानांकॉ बेला"}, {"Arabian", "अरब मानांकॉ बेला"}, {"Argentina", "अर्जेंटीना मानांकॉ बेला"}, {"Argentina_Western", "वेड़ा कुण्पु अर्जेंटीना मानांकॉ बेला"}, {"Armenia", "आर्मेनिया मानांकॉ बेला"}, {"Atlantic", "अटलांटिक मानांकॉ बेला"}, {"Australia_Central", "ऑस्\u200dट्रेलियाति मादिनी मानांकॉ बेला"}, {"Australia_CentralWestern", "ऑस्\u200dट्रेलियाति मादिनी वेड़ा कुण्पु मानांकॉ बेला"}, {"Australia_Eastern", "ऑस्\u200dट्रेलियाति वेड़ा हॉपु मानांकॉ बेला"}, {"Australia_Western", "ऑस्ट्रेलियाति वेड़ा कुण्पु मानांकॉ बेला"}, {"Azerbaijan", "अजरबाइजान मानांकॉ बेला"}, {"Azores", "अज़ोरेस मानांकॉ बेला"}, {"Bangladesh", "बांग्लादेस मानांकॉ बेला"}, {"Bhutan", "बुटान बेला"}, {"Bolivia", "बोलीविया बेला"}, {"Brasilia", "ब्राज़ीलिया मानांकॉ बेला"}, {"Brunei", "ब्रूनेति दारूस्सलम बेला"}, {"Cape_Verde", "केप वर्ड मानांकॉ बेला"}, {"Chamorro", "चामोरो मानांकॉ बेला"}, {"Chatham", "च्याताम मानांकॉ बेला"}, {"Chile", "चिली मानांकॉ बेला"}, {"China", "चीन मानांकॉ बेला"}, {"Christmas", "क्रिसमस द्वीप बेला"}, {"Cocos", "कोकोस द्वीप बेला"}, {"Colombia", "कोलंबिया मानांकॉ बेला"}, {"Cook", "कुक द्वीप मानांकॉ बेला"}, {"Cuba", "क्यूबा मानांकॉ बेला"}, {"Davis", "डेविस बेला"}, {"DumontDUrville", "ड्यूमोंट डी अर्विले बेला"}, {"East_Timor", "वेड़ा हॉपु तिमोर बेला"}, {"Easter", "ईस्टर द्वीप मानांकॉ बेला"}, {"Ecuador", "इक्वाडोर बेला"}, {"Europe_Central", "मादिनी युरोप-ति मानांकॉ बेला"}, {"Europe_Eastern", "वेड़ा हॉपु युरोप-ति मानांकॉ बेला"}, {"Europe_Further_Eastern", "ऑरॉ वेड़ा हॉपु युरोप-ति बेला"}, {"Europe_Western", "वेड़ा कुण्पु युरोप-ति मानांकॉ बेला"}, {"Falkland", "पाक -लेंड द्वीप मानांकॉ बेला"}, {"Fiji", "प़िजी मानांकॉ बेला"}, {"French_Guiana", "प़्रेंच गुयाना बेला"}, {"French_Southern", "दॉकिणॉ प़्रांस ऑड़े अंटार्कटिक बेला"}, {"GMT", "ग्रीनविच मीन बेला"}, {"Galapagos", "ग्यालापागोस ति बेला"}, {"Gambier", "ग्याम्बीयर बेला"}, {"Georgia", "जॉर्जिया मानांकॉ बेला"}, {"Gilbert_Islands", "गिल्बर्ट द्वीप बेला"}, {"Greenland_Eastern", "वेड़ा हॉपु ग्रीनलेंड मानांकॉ बेला"}, {"Greenland_Western", "वेड़ा कुण्पु ग्रीनलेंड मानांकॉ बेला"}, {"Gulf", "गल्प मानांकॉ बेला"}, {"Guyana", "गुयाना बेला"}, {"Hawaii", "हवाति–आल्यूसन मानांकॉ बेला"}, {"Hawaii_Aleutian", "हवाति–आल्यूसन मानांकॉ बेला"}, {"Hong_Kong", "हाँग काँग मानांकॉ बेला"}, {"Hovd", "होव्ड मानांकॉ बेला"}, {"India", "बारतीय मानांकॉ बेला"}, {"Indian_Ocean", "बारॉत काजा सामुद्री बेला"}, {"Indochina", "इंडोचाइना बेला"}, {"Indonesia_Central", "मध्य इंडोनेसिया बेला"}, {"Indonesia_Eastern", "वेड़ा हॉपु इंडोनेसिया बेला"}, {"Indonesia_Western", "वेड़ा कुण्पु इंडोनेसिया बेला"}, {"Iran", "इरान मानांकॉ बेला"}, {"Irkutsk", "इर्कुत्स्क मानांकॉ बेला"}, {"Israel", "इज़राइल मानांकॉ बेला"}, {"Japan", "जापान मानांकॉ बेला"}, {"Kazakhstan", "कज़ाकस्तान बेला"}, {"Kazakhstan_Eastern", "वेड़ा हॉपु कज़ाकस्तान बेला"}, {"Kazakhstan_Western", "वेड़ा कुण्पु कज़ाकस्तान बेला"}, {"Korea", "कोरियाति मानांकॉ बेला"}, {"Kosrae", "कोसराए बेला"}, {"Krasnoyarsk", "क्रास्नोयार्स्क मानांकॉ बेला"}, {"Kyrgystan", "किर्गिस्\u200dतान बेला"}, {"Line_Islands", "लाइन द्वीप बेला"}, {"Lord_Howe", "लॉर्ड होवे मानांकॉ बेला"}, {"Magadan", "मागादान मानांकॉ बेला"}, {"Malaysia", "मलेसिया बेला"}, {"Maldives", "मालदीव बेला"}, {"Marquesas", "मार्केसस बेला"}, {"Marshall_Islands", "मार्सल द्वीप बेला"}, {"Mauritius", "मॉरीसस मानांकॉ बेला"}, {"Mawson", "माव्सन बेला"}, {"Mexico_Pacific", "मेक्सिकोति पेसिपिक मानांकॉ बेला"}, {"Mongolia", "उलान बटोर मानांकॉ बेला"}, {"Moscow", "मॉस्को मानांकॉ बेला"}, {"Myanmar", "म्यांमार बेला"}, {"Nauru", "नॉउरु बेला"}, {"Nepal", "नेपाल बेला"}, {"New_Caledonia", "न्यू केलेडोनिया मानांकॉ बेला"}, {"New_Zealand", "न्यूज़ीलेंड मानांकॉ बेला"}, {"Newfoundland", "न्यूप़ाउंडलेंड मानांकॉ बेला"}, {"Niue", "नीयू बेला"}, {"Norfolk", "नॉरप़ॉक द्वीप मानांकॉ बेला"}, {"Noronha", "प़र्नांर्डो डे नोरोन्हा मानांकॉ बेला"}, {"Novosibirsk", "नोवोसिबिर्स्क मानांकॉ बेला"}, {"Omsk", "ओम्स्क मानांकॉ बेला"}, {"Pakistan", "पाकिस्तान मानांकॉ बेला"}, {"Palau", "पलाउ बेला"}, {"Papua_New_Guinea", "पापुआ न्यू गिनी बेला"}, {"Paraguay", "पैराग्वे मानांकॉ बेला"}, {"Peru", "पेरू मानांकॉ बेला"}, {"Philippines", "प़िलिपीन मानांकॉ बेला"}, {"Phoenix_Islands", "प़ीनिक्स द्वीप बेला"}, {"Pierre_Miquelon", "सेंट पिएरे ऑड़े मिक्वेलान मानांकॉ बेला"}, {"Pitcairn", "पिटकेर्न बेला"}, {"Ponape", "पोनापे बेला"}, {"Pyongyang", "प्योंगयांग बेला"}, {"Reunion", "रीयूनियन बेला"}, {"Rothera", "रोथेरा बेला"}, {"Sakhalin", "सकालिन मानांकॉ बेला"}, {"Samoa", "समोआ मानांकॉ बेला"}, {"Seychelles", "सेसेल्स बेला"}, {"Singapore", "सिंगापुर बेला"}, {"Solomon", "सोलोमन द्वीप बेला"}, {"South_Georgia", "दक्षिणी जॉर्जिया बेला"}, {"Suriname", "सूरीनाम बेला"}, {"Syowa", "स्योवा बेला"}, {"Tahiti", "ताहिती बेला"}, {"Taipei", "ताइपे मानांकॉ बेला"}, {"Tajikistan", "ताजिकिस्तान बेला"}, {"Tokelau", "टोकेलाऊ बेला"}, {"Tonga", "टोंगा मानांकॉ बेला"}, {"Truk", "चुक बेला"}, {"Turkmenistan", "तुर्कमेनिस्तान मानांकॉ बेला"}, {"Tuvalu", "तुवालू बेला"}, {"Uruguay", "उरुग्वे मानांकॉ बेला"}, {"Uzbekistan", "उज़्बेकिस्तान मानांकॉ बेला"}, {"Vanuatu", "वनुआतू मानांकॉ बेला"}, {"Venezuela", "वेनेज़ुएला बेला"}, {"Vladivostok", "व्लादिवोस्तोक मानांकॉ बेला"}, {"Volgograd", "वोल्गोग्राड मानांकॉ बेला"}, {"Vostok", "वोस्तोक बेला"}, {"Wake", "वेक द्वीप बेला"}, {"Wallis", "वालिस ऑड़े प़्यूचूना बेला"}, {"Yakutsk", "याकुत्स्क मानांकॉ बेला"}, {"Yekaterinburg", "येकातेरिनबर्ग मानांकॉ बेला"}},
		{{"Alaska", "अलास्\u200dका डेलाइट बेला"}, {"Amazon", "अमेज़न काराँ मासा बेला"}, {"America_Central", "मादिनी डेलाइट बेला"}, {"America_Eastern", "वेड़ा हॉपु डेलाइट बेला"}, {"America_Mountain", "हॉर्का डेलाइट बेला"}, {"America_Pacific", "पेसिपिक डेलाइट बेला"}, {"Apia", "एपिआ डेलाइट बेला"}, {"Arabian", "अरब डेलाइट बेला"}, {"Argentina", "अर्जेंटीना काराँ मासा बेला"}, {"Argentina_Western", "वेड़ा कुण्पु अर्जेंटीना काराँ मासा बेला"}, {"Armenia", "आर्मेनिया काराँ मासा बेला"}, {"Atlantic", "अटलांटिक डेलाइट बेला"}, {"Australia_Central", "ऑस्\u200dट्रेलियाति मादिनी डेलाइट बेला"}, {"Australia_CentralWestern", "ऑस्\u200dट्रेलियाति मादिनी वेड़ा कुण्पु डेलाइट बेला"}, {"Australia_Eastern", "ऑस्\u200dट्रेलियाति वेड़ा हॉपु डेलाइट बेला"}, {"Australia_Western", "ऑस्ट्रेलियाति वेड़ा कुण्पु डेलाइट बेला"}, {"Azerbaijan", "अजरबाइजान काराँ मासा बेला"}, {"Azores", "अज़ोरेस काराँ मासा बेला"}, {"Bangladesh", "बांग्लादेस काराँ मासा बेला"}, {"Brasilia", "ब्राज़ीलिया काराँ मासा बेला"}, {"Cape_Verde", "केप वर्ड काराँ मासा बेला"}, {"Chatham", "च्याताम डेलाइट बेला"}, {"Chile", "चिली काराँ मासा बेला"}, {"China", "चीन डेलाइट बेला"}, {"Colombia", "कोलंबिया काराँ मासा बेला"}, {"Cook", "कुक द्वीप आधा काराँ मासा बेला"}, {"Cuba", "क्यूबा डेलाइट बेला"}, {"Easter", "ईस्टर द्वीप काराँ मासा बेला"}, {"Europe_Central", "मादिनी युरोप-ति काराँ मासा बेला"}, {"Europe_Eastern", "वेड़ा हॉपु युरोप-ति काराँ मासा बेला"}, {"Europe_Western", "वेड़ा कुण्पु युरोप-ति काराँ मासा बेला"}, {"Falkland", "पाक -लेंड द्वीप काराँ मासा बेला"}, {"Fiji", "प़िजी काराँ मासा बेला"}, {"Georgia", "जॉर्जिया काराँ मासा बेला"}, {"Greenland_Eastern", "वेड़ा हॉपु ग्रीनलेंड काराँ मासा बेला"}, {"Greenland_Western", "वेड़ा कुण्पु ग्रीनलेंड काराँ मासा बेला"}, {"Hawaii_Aleutian", "हवाति–आल्यूसन डेलाइट बेला"}, {"Hong_Kong", "हाँग काँग काराँ मासा बेला"}, {"Hovd", "होव्ड काराँ मासा बेला"}, {"Iran", "तिरान डेलाइट बेला"}, {"Irkutsk", "इर्कुत्स्क काराँ मासा बेला"}, {"Israel", "इज़राइल डेलाइट बेला"}, {"Japan", "जापान डेलाइट बेला"}, {"Korea", "कोरियाति डेलाइट बेला"}, {"Krasnoyarsk", "क्रास्नोयार्स्क काराँ मासा बेला"}, {"Lord_Howe", "लॉर्ड होवे डेलाइट बेला"}, {"Magadan", "मागादान काराँ मासा बेला"}, {"Mauritius", "मॉरीसस काराँ मासा बेला"}, {"Mexico_Pacific", "मेक्सिकोति पेसिपिक डेलाइट बेला"}, {"Mongolia", "उलान बटोर काराँ मासा बेला"}, {"Moscow", "मॉस्को काराँ मासा बेला"}, {"New_Caledonia", "न्यू केलेडोनिया काराँ मासा बेला"}, {"New_Zealand", "न्यूज़ीलेंड डेलाइट बेला"}, {"Newfoundland", "न्यूप़ाउंडलेंड डेलाइट बेला"}, {"Norfolk", "नॉरप़ॉक द्वीप डेलाइट बेला"}, {"Noronha", "प़र्नांर्डो डे नोरोन्हा काराँ मासा बेला"}, {"Novosibirsk", "नोवोसिबिर्स्क काराँ मासा बेला"}, {"Omsk", "ओम्स्क काराँ मासा बेला"}, {"Pakistan", "पाकिस्तान काराँ मासा बेला"}, {"Paraguay", "पैराग्वे काराँ मासा बेला"}, {"Peru", "पेरू काराँ मासा बेला"}, {"Philippines", "प़िलिपीन काराँ मासा बेला"}, {"Pierre_Miquelon", "सेंट पिएरे ऑड़े मिक्वेलान डेलाइट बेला"}, {"Sakhalin", "सकालिन काराँ मासा बेला"}, {"Samoa", "समोआ डेलाइट बेला"}, {"Taipei", "ताइपे डेलाइट बेला"}, {"Tonga", "टोंगा काराँ मासा बेला"}, {"Turkmenistan", "तुर्कमेनिस्तान काराँ मासा बेला"}, {"Uruguay", "उरुग्वे काराँ मासा बेला"}, {"Uzbekistan", "उज़्बेकिस्तान काराँ मासा बेला"}, {"Vanuatu", "वनुआतू काराँ मासा बेला"}, {"Vladivostok", "व्लादिवोस्तोक काराँ मासा बेला"}, {"Volgograd", "वोल्गोग्राड काराँ मासा बेला"}, {"Yakutsk", "याकुत्स्क काराँ मासा बेला"}, {"Yekaterinburg", "येकातेरिनबर्ग काराँ मासा बेला"}},
	},
	{
		{{"Afghanistan", "ଆପଗାନିସ୍ତାନ ବେଲା"}, {"Africa_Central", "ମାଦିନି ଆପ୍ରିକା ବେଲା"}, {"Africa_Eastern", "ୱେଡ଼ା ହପୁ ଆପ୍ରିକା ବେଲା"}, {"Africa_Southern", "ଦକିଣ ଆପ୍ରିକା ମାନାଙ୍କ ବେଲା"}, {"Africa_Western", "ୱେଡ଼ା କୁଣ୍\u200dପୁ ଆପ୍ରିକା ବେଲା"}, {"Alaska", "ଆଲାସ୍କା ମାନାଙ୍କ ବେଲା"}, {"Amazon", "ଆମେଜନ ମାନାଙ୍କ ବେଲା"}, {"America_Central", "ମାଦିନି ମାନାଙ୍କ ବେଲା"}, {"America_Eastern", "ୱେଡ଼ାହପୁ ଜାଗାତି ମାନାଙ୍କ ବେଲା"}, {"America_Mountain", "ହର୍କା ମାନାଙ୍କ ବେଲା"}, {"America_Pacific", "ପେସିପିକ୍ ମାନାଙ୍କ ବେଲା"}, {"Apia", "ଏପିଆ ମାନାଙ୍କ ବେଲା"}, {"Arabian", "ଆରବିଆତି ମାନାଙ୍କ ବେଲା"}, {"Argentina", "ଆରଜେଣ୍ଟିନା ମାନାଙ୍କ ବେଲା"}, {"Argentina_Western", "ୱେଡ଼ା କୁଣ୍\u200dପୁ ଆର୍ଜେଣ୍ଟିନା ମାନାଙ୍କ ବେଲା"}, {"Armenia", "ଆରମେନିୟା ମାନାଙ୍କ ବେଲା"}, {"Atlantic", "ଆଟ୍ଲାଣ୍ଟିକ ମାନାଙ୍କ ବେଲା"}, {"Australia_Central", "ଅସ୍ଟ୍ରେଲିଆତି ମାଦିନି ମାନାଙ୍କ ବେଲା"}, {"Australia_CentralWestern", "ଅସ୍ଟ୍ରେଲିଆତି ମାଦିନି ୱେଡ଼ାକୁଣ୍\u200dପୁ ମାନାଙ୍କ ବେଲା"}, {"Australia_Eastern", "ଅସ୍ଟ୍ରେଲିଆତି ୱେଡ଼ାହପୁ ମାନାଙ୍କ ବେଲା"}, {"Australia_Western", "ଅସ୍ଟ୍ରେଲିଆତି ୱେଡ଼ାକୁଣ୍\u200dପୁ ମାନାଙ୍କ ବେଲା"}, {"Azerbaijan", "ଅଜେରବାଇଜାନ ମାନାଙ୍କ ବେଲା"}, {"Azores", "ଅଜରେସ ମାନାଙ୍କ ବେଲା"}, {"Bangladesh", "ବାଙ୍ଗଲାଦେସ୍ ମାନାଙ୍କ ବେଲା"}, {"Bhutan", "ବୁଟାନ ବେଲା"}, {"Bolivia", "ବଲୱିଆ ବେଲା"}, {"Brasilia", "ବ୍ରାଜିଲିୟା ମାନକ ବେଲା"}, {"Brunei", "ବ୍ରୁନେଇ ଦାରୁସାଲାମ ବେଲା"}, {"Cape_Verde", "କେପ ବର୍ଡ ମାନାଙ୍କ ବେଲା"}, {"Chamorro", "ଚମର ମାନାଙ୍କ ବେଲା"}, {"Chatham", "ଚାତାମ ମାନାଙ୍କ ବେଲା"}, {"Chile", "ଚିଲି ମାନାଙ୍କ ବେଲା"}, {"China", "ଚିନ ମାନାଙ୍କ ବେଲା"}, {"Christmas", "କ୍ରିସମାସ ଦିପ ବେଲା"}, {"Cocos", "କକସ ଦିପ ବେଲା"}, {"Colombia", "କଲମ୍ବିୟା ମାନାଙ୍କ ବେଲା"}, {"Cook", "କୁକ ଦିପ ମାନାଙ୍କ ବେଲା"}, {"Cuba", "କ୍ୱୁବା ମାନାଙ୍କ ବେଲା"}, {"Davis", "ଡେବିସ୍ ବେଲା"}, {"DumontDUrville", "ଡ୍ୟୁମାଣ୍ଟ ଡି ଅରୱିଲେ ବେଲା"}, {"East_Timor", "ୱେଡ଼ାହପୁ ତିମର ବେଲା"}, {"Easter", "ଇସ୍ଟର ଦିପ ମାନାଙ୍କ ବେଲା"}, {"Ecuador", "ଇକ୍ୱାଡର ବେଲା"}, {"Europe_Central", "ମାଦିନି ୟୁରପିଆ ତି ମାନାଙ୍କ ବେଲା"}, {"Europe_Eastern", "ୱେଡ଼ାହପୁ ୟୁରପିଆତି ମାନାଙ୍କ ବେଲା"}, {"Europe_Further_Eastern", "ଅର ୱେଡ଼ାହପୁ ୟୁରପତି ବେଲା"}, {"Europe_Western", "ୱେଡ଼ାକୁଣ୍\u200dପୁ ୟୁରପିଆତି ମାନାଙ୍କ ବେଲା"}, {"Falkland", "ପାକଲାଣ୍ଡ ଦିପତି ମାନାଙ୍କ ବେଲା"}, {"Fiji", "ପିଜି ମାନାଙ୍କ ବେଲା"}, {"French_Guiana", "ପ୍ରେଞ୍ଚ୍ ଗୁୟାନା ବେଲା"}, {"French_Southern", "ପ୍ରେଞ୍ଚ ଦକିଣ ଅଡ଼େ ଆଣ୍ଟାରଟିକ ବେଲା"}, {"GMT", "ଗ୍ରିନୱିଚ ମିନ ବେଲା"}, {"Galapagos", "ଗଲାପଗସ ତି ବେଲା"}, {"Gambier", "ଗମ୍ବିୟର ବେଲା"}, {"Georgia", "ଜର୍ଜିୟା ମାନାଙ୍କ ବେଲା"}, {"Gilbert_Islands", "ଗିଲ୍\u200dବେର୍ଟ ଦିପତି ବେଲା"}, {"Greenland_Eastern", "ୱେଡ଼ା ହପୁ ଗ୍ରିନଲାଣ୍ଡ ମାନାଙ୍କ ବେଲା"}, {"Greenland_Western", "ୱେଡ଼ା କୁଣ୍\u200dପୁ ଗ୍ରିନଲାଣ୍ଡ ମାନାଙ୍କ ବେଲା"}, {"Gulf", "ଗଲ୍ପ ମାନାଙ୍କ ବେଲା"}, {"Guyana", "ଗୁୟାନା ବେଲା"}, {"Hawaii", "ହାୱାଇ-ଆଲ୍ୟୁସାନ ମାନାଙ୍କ ବେଲା"}, {"Hawaii_Aleutian", "ହାୱାଇ-ଆଲ୍ୟୁସାନ ମାନାଙ୍କ ବେଲା"}, {"Hong_Kong", "ହଙ୍ଗ କଙ୍ଗ ମାନାଙ୍କ ବେଲା"}, {"Hovd", "ହୱଡ ମାନାଙ୍କ ବେଲା"}, {"India", "ବାରତ ମାନାଙ୍କ ବେଲା"}, {"Indian_Ocean", "ବାରତ କାଜା ସାମୁଦ୍ରି ବେଲା"}, {"Indochina", "ଇଣ୍ଡଚିନା ବେଲା"}, {"Indonesia_Central", "ମାଦିନି ଇଣ୍ଡନେସିଆ ବେଲା"}, {"Indonesia_Eastern", "ୱେଡ଼ାହପୁ ଇଣ୍ଡନେସିଆ ବେଲା"}, {"Indonesia_Western", "ୱେଡ଼ାକୁଣ୍ପୁ ଇଣ୍ଡନେସିଆ ବେଲା"}, {"Iran", "ଇରାନ ମାନାଙ୍କ ବେଲା"}, {"Irkutsk", "ଇରକୁସ୍ତକ ମାନାଙ୍କ ବେଲା"}, {"Israel", "ଇଜରାଇଲ ମାନାଙ୍କ ବେଲା"}, {"Japan", "ଜାପାନ ମାନାଙ୍କ ବେଲା"}, {"Kazakhstan", "କଜାକାସ୍ତାନ ବେଲା"}, {"Kazakhstan_Eastern", "ୱେଡ଼ାହପୁ କଜାକାସ୍ତାନ ବେଲା"}, {"Kazakhstan_Western", "ୱେଡ଼ାକୁଣ୍ପୁ କଜାକାସ୍ତାନ ବେଲା"}, {"Korea", "କରିୟା ମାନାଙ୍କ ବେଲା"}, {"Kosrae", "କସରାଏ ବେଲା"}, {"Krasnoyarsk", "କ୍ରାସ୍ନୟାର୍ସ୍କ ମାନାଙ୍କ ବେଲା"}, {"Kyrgystan", "କିର୍ଗିସ୍ତାନ ବେଲା"}, {"Line_Islands", "ଲାଇନ ଦିପତି ବେଲା"}, {"Lord_Howe", "ଲର୍ଡ ହୱେ ମାନାଙ୍କ ବେଲା"}, {"Magadan", "ମାଗାଦାନ ମାନଙ୍କ ବେଲା"}, {"Malaysia", "ମାଲେସିୟା ବେଲା"}, {"Maldives", "ମାଲଡ୍ୱିସ୍ ବେଲା"}, {"Marquesas", "ମାର୍କସସ ବେଲା"}, {"Marshall_Islands", "ମାର୍ସାଲ ଦିପ ବେଲା"}, {"Mauritius", "ମରିସସ୍ ମାନାଙ୍କ ବେଲା"}, {"Mawson", "ମାୱସନ ବେଲା"}, {"Mexico_Pacific", "ମେକ୍ସିକତି ପେସିପିକ ମାନାଙ୍କ ବେଲା"}, {"Mongolia", "ଉଲାନ ବଟର ମାନାଙ୍କ ବେଲା"}, {"Moscow", "ମସ୍କ ମାନାଙ୍କ ବେଲା"}, {"Myanmar", "ମ୍ୟାଁମାର ବେଲା"}, {"Nauru", "ନଉରୁ ବେଲା"}, {"Nepal", "ନେପାଲ ବେଲା"}, {"New_Caledonia", "ନ୍ୟୁ କେଲେଡନିୟା ମାନାଙ୍କ ବେଲା"}, {"New_Zealand", "ନ୍ୟୁଜିଲାଣ୍ଡ ମାନାଙ୍କ ବେଲା"}, {"Newfoundland", "ନ୍ୟୁପାଉଣ୍ଡଲାଣ୍ଡ ମାନାଙ୍କ ବେଲା"}, {"Niue", "ନିୟୁ ବେଲା"}, {"Norfolk", "ନରପାଁକ ଦିପ ମାନାଙ୍କ ବେଲା"}, {"Noronha", "ପର୍ନାଡ ଡେ ନରହ୍ନ ମାନାଙ୍କା ବେଲା"}, {"Novosibirsk", "ନୱସିବିର୍ସ୍କ ମାନାଙ୍କ ବେଲା"}, {"Omsk", "ଅମସ୍କ ମାନାଙ୍କ ବେଲା"}, {"Pakistan", "ପାକିସ୍ତାନ ମାନାଙ୍କ ବେଲା"}, {"Palau", "ପଲାଉ ବେଲା"}, {"Papua_New_Guinea", "ପାପୁୟା ପୁନି ଗିନି ବେଲା"}, {"Paraguay", "ପେରାଗ୍ୱେ ମାନାଙ୍କ ବେଲା"}, {"Peru", "ପେରୁ ମାନାଙ୍କ ବେଲା"}, {"Philippines", "ପିଲିପିନ ମାନାଙ୍କ ବେଲା"}, {"Phoenix_Islands", "ପିନିକ୍ସ ଦିପତି ବେଲା"}, {"Pierre_Miquelon", "ସେଣ୍ଟ ପିଏରେ ଅଡ଼େ ମିକ୍ୱେଲାନ ମାନାଙ୍କ ବେଲା"}, {"Pitcairn", "ପିଟକଇରନ ବେଲା"}, {"Ponape", "ପନାପେ ବେଲା"}, {"Pyongyang", "ପ୍ୟଙ୍ଗୟାଙ୍କ ବେଲା"}, {"Reunion", "ରିୟୁନିୟନ ବେଲା"}, {"Rothera", "ରତେରା ବେଲା"}, {"Sakhalin", "ସକାଲିନ ମାନାଙ୍କ ବେଲା"}, {"Samoa", "ସାମଆ ମାନାଙ୍କ ବେଲା"}, {"Seychelles", "ସେସେଲ୍ସ ବେଲା"}, {"Singapore", "ସାଙ୍ଗାପୁର ମାନାଙ୍କ ବେଲା"}, {"Solomon", "ସଲମନ ଦିପତି ବେଲା"}, {"South_Georgia", "ଦକିଣ ଜର୍ଜିୟା ବେଲା"}, {"Suriname", "ସୁରିନାମ ବେଲା"}, {"Syowa", "ସୱା ବେଲା"}, {"Tahiti", "ତାହିତି ବେଲା"}, {"Taipei", "ତାଇପେ ମାନଙ୍କ ବେଲା"}, {"Tajikistan", "ତାଜିକିସ୍ତାନ ବେଲା"}, {"Tokelau", "ଟକେଲାଉ ବେଲା"}, {"Tonga", "ଟଙ୍ଗା ମାନାଙ୍କ ବେଲା"}, {"Truk", "ଚୁକ ବେଲା"}, {"Turkmenistan", "ତୁର୍କମେନିସ୍ତାନ ମାନାଙ୍କ ବେଲା"}, {"Tuvalu", "ତୁୱାଲୁ ବେଲା"}, {"Uruguay", "ଉରୁଗ୍ୱେ ମାନାଙ୍କ ବେଲା"}, {"Uzbekistan", "ଉଜ୍ୱେକିସ୍ତାନ ମାନାଙ୍କ ବେଲା"}, {"Vanuatu", "ୱନୁଆତୁ ମାନାଙ୍କ ବେଲା"}, {"Venezuela", "ୱେନେଜୁଏଲା ବେଲା"}, {"Vladivostok", "ୱ୍ଲାଦିୱସ୍ତକ ମାନାଙ୍କ ବେଲା"}, {"Volgograd", "ୱାଲଗଗ୍ରାଡ ମାନାଙ୍କ ବେଲା"}, {"Vostok", "ୱସ୍ତକ ବେଲା"}, {"Wake", "ୱେକ ଦିପ ବେଲା"}, {"Wallis", "ୱାଲିସ ଅଡ଼େ ପୁଟୁନା ବେଲା"}, {"Yakutsk", "ୟାକୁତ୍ସକ ମାନାଙ୍କ ବେଲା"}, {"Yekaterinburg", "ୟେକାତେରିନବର୍ଗ ମାନାଙ୍କ ବେଲା"}},
		{{"Alaska", "ଆଲାସ୍କା ଡେଲାଇଟ୍ ବେଲା"}, {"Amazon", "ଆମେଜନ କାରାଁ ବେଲା"}, {"America_Central", "ମାଦିନି ଡେଲାଇଟ୍ ବେଲା"}, {"America_Eastern", "ୱେଡ଼ାହପୁ ଜାଗାତି ଡେଲାଇଟ୍ ବେଲା"}, {"America_Mountain", "ହର୍କା ଡେଲାଇଟ୍ ବେଲା"}, {"America_Pacific", "ପେସିପିକ୍ ଡେଲାଇଟ୍ ବେଲା"}, {"Apia", "ଏପିଆ ଡେଲାଇଟ୍ ବେଲା"}, {"Arabian", "ଆରବିଆତି ମେଦାଣା ବେଲା"}, {"Argentina", "ଆର୍ଜେଣ୍ଟିନା କାରା ବେଲା"}, {"Argentina_Western", "ୱେଡ଼ା କୁଣ୍\u200dପୁ ଆର୍ଜେଣ୍ଟିନା କାରାଁ ବେଲା"}, {"Armenia", "ଆରମେନିୟା କାରାଁ ବେଲା"}, {"Atlantic", "ଆଟ୍ଲାଣ୍ଟିକ ଡେଲାଇଟ ବେଲା"}, {"Australia_Central", "ଅସ୍ଟ୍ରେଲିଆତି ମାଦିନି ଡେଲାଇଟ ବେଲା"}, {"Australia_CentralWestern", "ଅସ୍ଟ୍ରେଲିଆତି ମାଦିନି ୱେଡ଼ାକୁଣ୍\u200dପୁ ଡେଲାଇଟ ବେଲା"}, {"Australia_Eastern", "ଅସ୍ଟ୍ରେଲିଆତି ୱେଡ଼ାହପୁ ଡେଲାଇଟ ବେଲା"}, {"Australia_Western", "ଅସ୍ଟ୍ରେଲିଆତି ୱେଡ଼ାକୁଣ୍\u200dପୁ ଡେଲାଇଟ ବେଲା"}, {"Azerbaijan", "ଅଜେରବାଇଜାନ କାରାଁ ବେଲା"}, {"Azores", "ଅଜରେସ କାରାଁ ବେଲା"}, {"Bangladesh", "ବାଙ୍ଗଲାଦେସ୍ କାରାଁ ବେଲା"}, {"Brasilia", "ବ୍ରାଜିଲିୟା କାରାଁ ବେଲା"}, {"Cape_Verde", "କେପ ବର୍ଡ କାରାଁ ବେଲା"}, {"Chatham", "ଚାତାମ ଡେଲାଇଟ୍ ବେଲା"}, {"Chile", "ଚିଲି କାରାଁ ବେଲା"}, {"China", "ଚିନ ଡେଲାଇଟ ବେଲା"}, {"Colombia", "କଲମ୍ବିୟା କାରାଁ ବେଲା"}, {"Cook", "କୁକ ଦିପ ଆଦ୍ଦା କାରାଁ ବେଲା"}, {"Cuba", "କ୍ୱୁବା ଡେଲାଇଟ୍ ବେଲା"}, {"Easter", "ଇସ୍ଟର ଦିପ କାରାଁ ବେଲା"}, {"Europe_Central", "ମାଦିନା ୟୁରପିଆ ତି କାରାଁ ବେଲା"}, {"Europe_Eastern", "ୱେଡ଼ାହପୁ ୟୁରପିଆତି କାରାଁ ବେଲା"}, {"Europe_Western", "ୱେଡ଼ାକୁଣ୍\u200dପୁ ୟୁରପତି କାରାଁ ବେଲା"}, {"Falkland", "ପାକଲାଣ୍ଡ ଦିପତି କାରାଁ ବେଲା"}, {"Fiji", "ପିଜି କାରାଁ ବେଲା"}, {"Georgia", "ଜର୍ଜିୟା କାରା ବେଲା"}, {"Greenland_Eastern", "ୱେଡ଼ା ହପୁ ଗ୍ରିନଲାଣ୍ଡ କାରାଁ ବେଲା"}, {"Greenland_Western", "ୱେଡ଼ା କୁଣ୍\u200dପୁ ଗ୍ରିନଲାଣ୍ଡ କାରାଁ ବେଲା"}, {"Hawaii_Aleutian", "ହାୱାଇ-ଆଲ୍ୟୁସାନ ଡେ ଲାଇଟ ବେଲା"}, {"Hong_Kong", "ହଙ୍ଗ କଙ୍କ କାରାଁ ବେଲା"}, {"Hovd", "ହୱଡ କାରାଁ ବେଲା"}, {"Iran", "ଇରାନ ଡେ ଲାଇଟ୍ ବେଲା"}, {"Irkutsk", "ଇରକୁସ୍ତକ କାରାଁ ବେଲା"}, {"Israel", "ଇଜରାଇଲ ଡେଲାଇଟ ବେଲା"}, {"Japan", "ଜାପାନ ଡେଲାଇଟ ବେଲା"}, {"Korea", "କରିୟା ଡେଲାଇଟ ବେଲା"}, {"Krasnoyarsk", "କ୍ରାସ୍ନାର୍ସ୍କ କାରାଁ ବେଲା"}, {"Lord_Howe", "ଲର୍ଡ ହୱେ ଡେଲାଇଟ୍ ବେଲା"}, {"Magadan", "ମାଗାଦାନ କାରାଁ ବେଲା"}, {"Mauritius", "ମରିସସ୍ କାରାଁ ବେଲା"}, {"Mexico_Pacific", "ମେକ୍ସିକତି ପେସିପିକ ଡେଲାଇଟ ବେଲା"}, {"Mongolia", "ଉଲାନ ବଟର କାରାଁ ବେଲା"}, {"Moscow", "ମସ୍କ କାରାଁ ବେଲା"}, {"New_Caledonia", "ନ୍ୟୁ କେଲେଡନିୟା କାରାଁ ବେଲା"}, {"New_Zealand", "ନ୍ୟୁଜିଲାଣ୍ଡ ଡେଲାଇଟ ବେଲା"}, {"Newfoundland", "ନ୍ୟୁପାଉଣ୍ଡଲାଣ୍ଡ ଡେଲାଇଟ ବେଲା"}, {"Norfolk", "ନରପାଁକ ଦିପ ଡେଲାଇଟ ବେଲା"}, {"Noronha", "ପର୍ନାଡ ଡେ ନରହ୍ନ କାରାଁ ବେଲା"}, {"Novosibirsk", "ନୱସିବର୍ସ୍କ କାରାଁ ବଲା"}, {"Omsk", "ଅମସ୍କ କାରାଁ ବେଲା"}, {"Pakistan", "ପାକିସ୍ତାନ କାରାଁ ବେଲା"}, {"Paraguay", "ପେରାଗ୍ୱେ କାରାଁ ବେଲା"}, {"Peru", "ପେରୁ କାରାଁ ବେଲା"}, {"Philippines", "ପିଲିପିନ କାରାଁ ବେଲା"}, {"Pierre_Miquelon", "ସେଣ୍ଟ ପିଏରେ ଅଡ଼େ ମିକ୍ୱେଲାନ ଡେଲାଇଟ ବେଲା"}, {"Sakhalin", "ସକାଲିନ କାରାଁ ବେଲା"}, {"Samoa", "ସାମଆ ଡେଲାଇଟ ବେଲା"}, {"Taipei", "ତାଇପେ ଡେଲାଇଟ ବେଲା"}, {"Tonga", "ଟଙ୍ଗା କାରାଁ ବେଲା"}, {"Turkmenistan", "ତୁର୍କମେନିସ୍ତାନ କାରାଁ ବେଲା"}, {"Uruguay", "ଉରୁଗ୍ୱେ କାରାଁ ବେଲା"}, {"Uzbekistan", "ଉଜ୍ୱେକିସ୍ତାନ କାରାଁ ବେଲା"}, {"Vanuatu", "ୱନୁଆତୁ କାରାଁ ବେଲା"}, {"Vladivostok", "ୱ୍ଲାଦିୱସ୍ତକ କାରାଁ ବେଲା"}, {"Volgograd", "ୱାଲଗଗ୍ରାଡ କାରାଁ ବେଲା"}, {"Yakutsk", "ୟାକୁତ୍ସକ କାରାଁ ବେଲା"}, {"Yekaterinburg", "ୟେକାତେରିନବର୍ଗ କାରାଁ ବେଲା"}},
	},
	{
		{{"Afghanistan", "ఆఫ్ఘనిస్తాన్ సమయం"}, {"Africa_Central", "సెంట్రల్ ఆఫ్రికా సమయం"}, {"Africa_Eastern", "తూర్పు ఆఫ్రికా సమయం"}, {"Africa_Southern", "దక్షిణ ఆఫ్రికా ప్రామాణిక సమయం"}, {"Africa_Western", "పశ్చిమ ఆఫ్రికా సమయం"}, {"Alaska", "అలాస్కా ప్రామాణిక సమయం"}, {"Amazon", "అమెజాన్ ప్రామాణిక సమయం"}, {"America_Central", "మధ్యమ ప్రామాణిక సమయం"}, {"America_Eastern", "తూర్పు ప్రామాణిక సమయం"}, {"America_Mountain", "మౌంటెయిన్ ప్రామాణిక సమయం"}, {"America_Pacific", "పసిఫిక్ ప్రామాణిక సమయం"}, {"Apia", "ఏపియా ప్రామాణిక సమయం"}, {"Arabian", "అరేబియన్ ప్రామాణిక సమయం"}, {"Argentina", "అర్జెంటీనా ప్రామాణిక సమయం"}, {"Argentina_Western", "పశ్చిమ అర్జెంటీనా ప్రామాణిక సమయం"}, {"Armenia", "ఆర్మేనియా ప్రామాణిక సమయం"}, {"Atlantic", "అట్లాంటిక్ ప్రామాణిక సమయం"}, {"Australia_Central", "ఆస్ట్రేలియా మధ్యమ ప్రామాణిక సమయం"}, {"Australia_CentralWestern", "మధ్యమ ఆస్ట్రేలియన్ పశ్చిమ ప్రామాణిక సమయం"}, {"Australia_Eastern", "ఆస్ట్రేలియన్ తూర్పు ప్రామాణిక సమయం"}, {"Australia_Western", "ఆస్ట్రేలియన్ పశ్చిమ ప్రామాణిక సమయం"}, {"Azerbaijan", "అజర్బైజాన్ ప్రామాణిక సమయం"}, {"Azores", "అజోర్స్ ప్రామాణిక సమయం"}, {"Bangladesh", "బంగ్లాదేశ్ ప్రామాణిక సమయం"}, {"Bhutan", "భూటాన్ సమయం"}, {"Bolivia", "బొలీవియా సమయం"}, {"Brasilia", "బ్రెజిలియా ప్రామాణిక సమయం"}, {"Brunei", "బ్రూనే దరుసలామ్ సమయం"}, {"Cape_Verde", "కేప్ వెర్డె ప్రామాణిక సమయం"}, {"Chamorro", "చామర్రో ప్రామాణిక సమయం"}, {"Chatham", "చాథమ్ ప్రామాణిక సమయం"}, {"Chile", "చిలీ ప్రామాణిక సమయం"}, {"China", "చైనా ప్రామాణిక సమయం"}, {"Christmas", "క్రిస్మస్ దీవి సమయం"}, {"Cocos", "కోకోస్ దీవుల సమయం"}, {"Colombia", "కొలంబియా ప్రామాణిక సమయం"}, {"Cook", "కుక్ దీవుల ప్రామాణిక సమయం"}, {"Cuba", "క్యూబా ప్రామాణిక సమయం"}, {"Davis", "డేవిస్ సమయం"}, {"DumontDUrville", "డ్యూమాంట్-డి’ఉర్విల్లే సమయం"}, {"East_Timor", "తూర్పు తైమూర్ సమయం"}, {"Easter", "ఈస్టర్ దీవి ప్రామాణిక సమయం"}, {"Ecuador", "ఈక్వడార్ సమయం"}, {"Europe_Central", "సెంట్రల్ యూరోపియన్ ప్రామాణిక సమయం"}, {"Europe_Eastern", "తూర్పు యూరోపియన్ ప్రామాణిక సమయం"}, {"Europe_Further_Eastern", "సుదూర-తూర్పు యూరోపియన్ సమయం"}, {"Europe_Western", "పశ్చిమ యూరోపియన్ ప్రామాణిక సమయం"}, {"Falkland", "ఫాక్\u200cల్యాండ్ దీవుల ప్రామాణిక సమయం"}, {"Fiji", "ఫిజీ ప్రామాణిక సమయం"}, {"French_Guiana", "ఫ్రెంచ్ గయానా సమయం"}, {"French_Southern", "ఫ్రెంచ్ దక్షిణ మరియు అంటార్కిటిక్ సమయం"}, {"GMT", "గ్రీన్\u200cవిచ్ సగటు సమయం"}, {"Galapagos", "గాలాపాగోస్ సమయం"}, {"Gambier", "గాంబియర్ సమయం"}, {"Georgia", "జార్జియా ప్రామాణిక సమయం"}, {"Gilbert_Islands", "గిల్బర్ట్ దీవుల సమయం"}, {"Greenland_Eastern", "తూర్పు గ్రీన్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Greenland_Western", "పశ్చిమ గ్రీన్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Gulf", "గల్ఫ్ ప్రామాణిక సమయం"}, {"Guyana", "గయానా సమయం"}, {"Hawaii", "హవాయ్-అల్యూషియన్ ప్రామాణిక సమయం"}, {"Hawaii_Aleutian", "హవాయ్-అల్యూషియన్ ప్రామాణిక సమయం"}, {"Hong_Kong", "హాంకాంగ్ ప్రామాణిక సమయం"}, {"Hovd", "హోవ్డ్ ప్రామాణిక సమయం"}, {"India", "భారతదేశ ప్రామాణిక సమయం"}, {"Indian_Ocean", "హిందూ మహా సముద్ర సమయం"}, {"Indochina", "ఇండోచైనా సమయం"}, {"Indonesia_Central", "సెంట్రల్ ఇండోనేషియా సమయం"}, {"Indonesia_Eastern", "తూర్పు ఇండోనేషియా సమయం"}, {"Indonesia_Western", "పశ్చిమ ఇండోనేషియా సమయం"}, {"Iran", "ఇరాన్ ప్రామాణిక సమయం"}, {"Irkutsk", "ఇర్కుట్స్క్ ప్రామాణిక సమయం"}, {"Israel", "ఇజ్రాయిల్ ప్రామాణిక సమయం"}, {"Japan", "జపాన్ ప్రామాణిక సమయం"}, {"Kazakhstan", "కజకి\u200cస్తాన్ సమయం"}, {"Kazakhstan_Eastern", "తూర్పు కజకి\u200cస్తాన్ సమయం"}, {"Kazakhstan_Western", "పశ్చిమ కజకిస్తాన్ సమయం"}, {"Korea", "కొరియన్ ప్రామాణిక సమయం"}, {"Kosrae", "కోస్రాయి సమయం"}, {"Krasnoyarsk", "క్రాస్నోయార్స్క్ ప్రామాణిక సమయం"}, {"Kyrgystan", "కిర్గిస్తాన్ సమయం"}, {"Line_Islands", "లైన్ దీవుల సమయం"}, {"Lord_Howe", "లార్డ్ హోవ్ ప్రామాణిక సమయం"}, {"Magadan", "మగడాన్ ప్రామాణిక సమయం"}, {"Malaysia", "మలేషియా సమయం"}, {"Maldives", "మాల్దీవుల సమయం"}, {"Marquesas", "మార్క్వేసాస్ సమయం"}, {"Marshall_Islands", "మార్షల్ దీవుల సమయం"}, {"Mauritius", "మారిషస్ ప్రామాణిక సమయం"}, {"Mawson", "మాసన్ సమయం"}, {"Mexico_Pacific", "మెక్సికన్ పసిఫిక్ ప్రామాణిక సమయం"}, {"Mongolia", "ఉలన్ బతోర్ ప్రామాణిక సమయం"}, {"Moscow", "మాస్కో ప్రామాణిక సమయం"}, {"Myanmar", "మయన్మార్ సమయం"}, {"Nauru", "నౌరు సమయం"}, {"Nepal", "నేపాల్ సమయం"}, {"New_Caledonia", "న్యూ కాలెడోనియా ప్రామాణిక సమయం"}, {"New_Zealand", "న్యూజిల్యాండ్ ప్రామాణిక సమయం"}, {"Newfoundland", "న్యూఫౌండ్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Niue", "నియూ సమయం"}, {"Norfolk", "నార్ఫోక్ దీవి ప్రామాణిక సమయం"}, {"Noronha", "ఫెర్నాండో డి నొరోన్హా ప్రామాణిక సమయం"}, {"Novosibirsk", "నోవోసిబిర్క్స్ ప్రామాణిక సమయం"}, {"Omsk", "ఓమ్స్క్ ప్రామాణిక సమయం"}, {"Pakistan", "పాకిస్తాన్ ప్రామాణిక సమయం"}, {"Palau", "పాలావ్ సమయం"}, {"Papua_New_Guinea", "పాపువా న్యూ గినియా సమయం"}, {"Paraguay", "పరాగ్వే ప్రామాణిక సమయం"}, {"Peru", "పెరూ ప్రామాణిక సమయం"}, {"Philippines", "ఫిలిప్పైన్ ప్రామాణిక సమయం"}, {"Phoenix_Islands", "ఫినిక్స్ దీవుల సమయం"}, {"Pierre_Miquelon", "సెయింట్ పియెర్ మరియు మిక్వెలాన్ ప్రామాణిక సమయం"}, {"Pitcairn", "పిట్\u200cకైర్న్ సమయం"}, {"Ponape", "పొనేప్ సమయం"}, {"Pyongyang", "ప్యోంగాంగ్ సమయం"}, {"Reunion", "రీయూనియన్ సమయం"}, {"Rothera", "రొతేరా సమయం"}, {"Sakhalin", "సఖాలిన్ ప్రామాణిక సమయం"}, {"Samoa", "సమోవా ప్రామాణిక సమయం"}, {"Seychelles", "సీషెల్స్ సమయం"}, {"Singapore", "సింగపూర్ ప్రామాణిక సమయం"}, {"Solomon", "సోలమన్ దీవుల సమయం"}, {"South_Georgia", "దక్షిణ జార్జియా సమయం"}, {"Suriname", "సూరినామ్ సమయం"}, {"Syowa", "స్యోవా సమయం"}, {"Tahiti", "తహితి సమయం"}, {"Taipei", "తైపీ ప్రామాణిక సమయం"}, {"Tajikistan", "తజికిస్తాన్ సమయం"}, {"Tokelau", "టోకెలావ్ సమయం"}, {"Tonga", "టాంగా ప్రామాణిక సమయం"}, {"Truk", "చక్ సమయం"}, {"Turkmenistan", "తుర్క్\u200cమెనిస్తాన్ ప్రామాణిక సమయం"}, {"Tuvalu", "తువాలు సమయం"}, {"Uruguay", "ఉరుగ్వే ప్రామాణిక సమయం"}, {"Uzbekistan", "ఉజ్బెకిస్తాన్ ప్రామాణిక సమయం"}, {"Vanuatu", "వనౌటు ప్రామాణిక సమయం"}, {"Venezuela", "వెనిజులా సమయం"}, {"Vladivostok", "వ్లాడివోస్టోక్ ప్రామాణిక సమయం"}, {"Volgograd", "వోల్గోగ్రాడ్ ప్రామాణిక సమయం"}, {"Vostok", "వోస్టోక్ సమయం"}, {"Wake", "వేక్ దీవి సమయం"}, {"Wallis", "వాలీస్ మరియు ఫుటునా సమయం"}, {"Yakutsk", "యాకుట్స్క్ ప్రామాణిక సమయం"}, {"Yekaterinburg", "యెకటెరిన్\u200cబర్గ్ ప్రామాణిక సమయం"}},
		{{"Alaska", "అలాస్కా పగటి వెలుతురు సమయం"}, {"Amazon", "అమెజాన్ వేసవి సమయం"}, {"America_Central", "మధ్యమ పగటి వెలుతురు సమయం"}, {"America_Eastern", "తూర్పు పగటి వెలుతురు సమయం"}, {"America_Mountain", "మౌంటెయిన్ పగటి వెలుతురు సమయం"}, {"America_Pacific", "పసిఫిక్ పగటి వెలుతురు సమయం"}, {"Apia", "ఏపియా పగటి సమయం"}, {"Arabian", "అరేబియన్ పగటి వెలుతురు సమయం"}, {"Argentina", "ఆర్జెంటీనా వేసవి సమయం"}, {"Argentina_Western", "పశ్చిమ అర్జెంటీనా వేసవి సమయం"}, {"Armenia", "ఆర్మేనియా వేసవి సమయం"}, {"Atlantic", "అట్లాంటిక్ పగటి వెలుతురు సమయం"}, {"Australia_Central", "ఆస్ట్రేలియా మధ్యమ పగటి వెలుతురు సమయం"}, {"Australia_CentralWestern", "ఆస్ట్రేలియా మధ్యమ పశ్చిమ పగటి వెలుతురు సమయం"}, {"Australia_Eastern", "ఆస్ట్రేలియన్ తూర్పు పగటి వెలుతురు సమయం"}, {"Australia_Western", "ఆస్ట్రేలియన్ పశ్చిమ పగటి వెలుతురు సమయం"}, {"Azerbaijan", "అజర్బైజాన్ వేసవి సమయం"}, {"Azores", "అజోర్స్ వేసవి సమయం"}, {"Bangladesh", "బంగ్లాదేశ్ వేసవి సమయం"}, {"Brasilia", "బ్రెజిలియా వేసవి సమయం"}, {"Cape_Verde", "కేప్ వెర్డె వేసవి సమయం"}, {"Chatham", "చాథమ్ పగటి వెలుతురు సమయం"}, {"Chile", "చిలీ వేసవి సమయం"}, {"China", "చైనా పగటి వెలుతురు సమయం"}, {"Colombia", "కొలంబియా వేసవి సమయం"}, {"Cook", "కుక్ దీవుల అర్ధ వేసవి సమయం"}, {"Cuba", "క్యూబా పగటి వెలుతురు సమయం"}, {"Easter", "ఈస్టర్ దీవి వేసవి సమయం"}, {"Europe_Central", "సెంట్రల్ యూరోపియన్ వేసవి సమయం"}, {"Europe_Eastern", "తూర్పు యూరోపియన్ వేసవి సమయం"}, {"Europe_Western", "పశ్చిమ యూరోపియన్ వేసవి సమయం"}, {"Falkland", "ఫాక్\u200cల్యాండ్ దీవుల వేసవి సమయం"}, {"Fiji", "ఫిజీ వేసవి సమయం"}, {"Georgia", "జార్జియా వేసవి సమయం"}, {"Greenland_Eastern", "తూర్పు గ్రీన్\u200cల్యాండ్ వేసవి సమయం"}, {"Greenland_Western", "పశ్చిమ గ్రీన్\u200cల్యాండ్ వేసవి సమయం"}, {"Hawaii_Aleutian", "హవాయ్-అల్యూషియన్ పగటి వెలుతురు సమయం"}, {"Hong_Kong", "హాంకాంగ్ వేసవి సమయం"}, {"Hovd", "హోవ్డ్ వేసవి సమయం"}, {"Iran", "ఇరాన్ పగటి వెలుతురు సమయం"}, {"Irkutsk", "ఇర్కుట్స్క్ వేసవి సమయం"}, {"Israel", "ఇజ్రాయిల్ పగటి వెలుతురు సమయం"}, {"Japan", "జపాన్ పగటి వెలుతురు సమయం"}, {"Korea", "కొరియన్ పగటి వెలుతురు సమయం"}, {"Krasnoyarsk", "క్రాస్నోయార్స్క్ వేసవి సమయం"}, {"Lord_Howe", "లార్డ్ హోవ్ పగటి సమయం"}, {"Magadan", "మగడాన్ వేసవి సమయం"}, {"Mauritius", "మారిషస్ వేసవి సమయం"}, {"Mexico_Pacific", "మెక్సికన్ పసిఫిక్ పగటి వెలుతురు సమయం"}, {"Mongolia", "ఉలన్ బతోర్ వేసవి సమయం"}, {"Moscow", "మాస్కో వేసవి సమయం"}, {"New_Caledonia", "న్యూ కాలెడోనియా వేసవి సమయం"}, {"New_Zealand", "న్యూజిల్యాండ్ పగటి వెలుతురు సమయం"}, {"Newfoundland", "న్యూఫౌండ్\u200cల్యాండ్ పగటి వెలుతురు సమయం"}, {"Norfolk", "నార్ఫోక్ దీవి పగటి సమయం"}, {"Noronha", "ఫెర్నాండో డి నొరోన్హా వేసవి సమయం"}, {"Novosibirsk", "నోవోసిబిర్స్క్ వేసవి సమయం"}, {"Omsk", "ఓమ్స్క్ వేసవి సమయం"}, {"Pakistan", "పాకిస్తాన్ వేసవి సమయం"}, {"Paraguay", "పరాగ్వే వేసవి సమయం"}, {"Peru", "పెరూ వేసవి సమయం"}, {"Philippines", "ఫిలిప్పైన్ వేసవి సమయం"}, {"Pierre_Miquelon", "సెయింట్ పియర్ మరియు మిక్వెలాన్ పగటి వెలుతురు సమయం"}, {"Sakhalin", "సఖాలిన్ వేసవి సమయం"}, {"Samoa", "సమోవా పగటి వెలుతురు సమయం"}, {"Taipei", "తైపీ పగటి వెలుతురు సమయం"}, {"Tonga", "టాంగా వేసవి సమయం"}, {"Turkmenistan", "తుర్క్\u200cమెనిస్తాన్ వేసవి సమయం"}, {"Uruguay", "ఉరుగ్వే వేసవి సమయం"}, {"Uzbekistan", "ఉజ్బెకిస్తాన్ వేసవి సమయం"}, {"Vanuatu", "వనౌటు వేసవి సమయం"}, {"Vladivostok", "వ్లాడివోస్టోక్ వేసవి సమయం"}, {"Volgograd", "వోల్గోగ్రాడ్ వేసవి సమయం"}, {"Yakutsk", "యాకుట్స్క్ వేసవి సమయం"}, {"Yekaterinburg", "యెకటెరిన్\u200cబర్గ్ వేసవి సమయం"}},
	},
	{
		{{"Afghanistan", "Афганистан убактысы"}, {"Africa_Central", "Борбордук Африка убактысы"}, {"Africa_Eastern", "Чыгыш Африка убактысы"}, {"Africa_Southern", "Түштүк Африка убактысы"}, {"Africa_Western", "Батыш Африка убактысы"}, {"Alaska", "Аляска кышкы убактысы"}, {"Amazon", "Амазон кышкы убактысы"}, {"America_Central", "Түндүк Америка, борбордук кышкы убактысы"}, {"America_Eastern", "Түндүк Америка, чыгыш кышкы убактысы"}, {"America_Mountain", "Түндүк Америка, тоо кышкы убактысы"}, {"America_Pacific", "Түндүк Америка, Тынч океан кышкы убактысы"}, {"Apia", "Апиа кышкы убактысы"}, {"Arabian", "Арабия кышкы убакыты"}, {"Argentina", "Аргентина кышкы убактысы"}, {"Argentina_Western", "Батыш Аргентина кышкы убактысы"}, {"Armenia", "Армения кышкы убакыты"}, {"Atlantic", "Атлантика кышкы убактысы"}, {"Australia_Central", "Австралия борбордук кышкы убакыты"}, {"Australia_CentralWestern", "Австралия борбордук батыш кышкы убакыты"}, {"Australia_Eastern", "Австралия чыгыш кышкы убакыты"}, {"Australia_Western", "Австралия батыш кышкы убакыты"}, {"Azerbaijan", "Азербайжан кышкы убакыты"}, {"Azores", "Азор кышкы убакыты"}, {"Bangladesh", "Бангладеш кышкы убакыты"}, {"Bhutan", "Бутан убактысы"}, {"Bolivia", "Боливия убактысы"}, {"Brasilia", "Бразилия кышкы убактысы"}, {"Brunei", "Бруней Даруссалам убактысы"}, {"Cape_Verde", "Капе Верде кышкы убакыты"}, {"Chamorro", "Чаморро убактысы"}, {"Chatham", "Чатам кышкы убакыт"}, {"Chile", "Чили кышкы убактысы"}, {"China", "Кытай кышкы убакыты"}, {"Christmas", "Крисмас аралынын убактысы"}, {"Cocos", "Кокос аралдарынын убактысы"}, {"Colombia", "Колумбия кышкы убактысы"}, {"Cook", "Кук аралдарынын кышкы убактысы"}, {"Cuba", "Куба кышкы убактысы"}, {"Davis", "Дэвис убактысы"}, {"DumontDUrville", "Дюмон-д-Урвил убактысы"}, {"East_Timor", "Чыгыш Тимор убактысы"}, {"Easter", "Истер аралынын кышкы убакыты"}, {"Ecuador", "Экуадор убактысы"}, {"Europe_Central", "Борбордук Европа кышкы убакыты"}, {"Europe_Eastern", "Чыгыш Европа кышкы убакыты"}, {"Europe_Further_Eastern", "Калининград жана Минск убактысы"}, {"Europe_Western", "Батыш Европа кышкы убакыты"}, {"Falkland", "Фолкленд аралдарынын кышкы убакыты"}, {"Fiji", "Фижи кышкы убакыты"}, {"French_Guiana", "Француз Гвиана убактысы"}, {"French_Southern", "Француз Түштүгү жана Антарктика убактысы"}, {"GMT", "Гринвич боюнча орточо убакыт"}, {"Galapagos", "Галапагос убактысы"}, {"Gambier", "Гамбие убактысы"}, {"Georgia", "Грузия кышкы убакыты"}, {"Gilbert_Islands", "Гилберт убактысы"}, {"Greenland_Eastern", "Чыгыш Гренландия кышкы убактысы"}, {"Greenland_Western", "Батыш Гренландия кышкы убактысы"}, {"Gulf", "Булуңдун стандарттык убакыты"}, {"Guyana", "Гвиана убактысы"}, {"Hawaii", "Гавайи-Алеут кышкы убактысы"}, {"Hawaii_Aleutian", "Гавайи-Алеут кышкы убактысы"}, {"Hong_Kong", "Гонконг кышкы убакыты"}, {"Hovd", "Ховд кышкы убакыты"}, {"India", "Индия убактысы"}, {"Indian_Ocean", "Инди океан убактысы"}, {"Indochina", "Индокытай убактысы"}, {"Indonesia_Central", "Борбордук Индонезия убактысы"}, {"Indonesia_Eastern", "Чыгыш Индонезия убактысы"}, {"Indonesia_Western", "Батыш Индонезия убактысы"}, {"Iran", "Иран кышкы убакыты"}, {"Irkutsk", "Иркутск кышкы убакыты"}, {"Israel", "Израиль кышкы убакыты"}, {"Japan", "Жапон кышкы убакыты"}, {"Kazakhstan", "Казакстан убактысы"}, {"Kazakhstan_Eastern", "Чыгыш Казакстан убактысы"}, {"Kazakhstan_Western", "Батыш Казакстан убактысы"}, {"Korea", "Корея кышкы убакыты"}, {"Kosrae", "Косрае убактысы"}, {"Krasnoyarsk", "Красноярск кышкы убакыты"}, {"Kyrgystan", "Кыргызстан убактысы"}, {"Line_Islands", "Лайн аралдарынын убактысы"}, {"Lord_Howe", "Лорд Хау кышкы убакыты"}, {"Magadan", "Магадан кышкы убакыты"}, {"Malaysia", "Малайзия убактысы"}, {"Maldives", "Мальдив убактысы"}, {"Marquesas", "Маркезас убактысы"}, {"Marshall_Islands", "Маршалл аралдарынын убактысы"}, {"Mauritius", "Маврикий кышкы убакыты"}, {"Mawson", "Моусон убактысы"}, {"Mexico_Pacific", "Мексика, Тынч океан кышкы убактысы"}, {"Mongolia", "Улан Батор кышкы убакыты"}, {"Moscow", "Москва кышкы убакыты"}, {"Myanmar", "Мйанмар убактысы"}, {"Nauru", "Науру убактысы"}, {"Nepal", "Непал убактысы"}, {"New_Caledonia", "Жаңы Каледония кышкы убактысы"}, {"New_Zealand", "Жаңы Зеландия кышкы убакыты"}, {"Newfoundland", "Нюфаундлэнд кышкы убактысы"}, {"Niue", "Ниуэ убактысы"}, {"Norfolk", "Норфолк кышкы убактысы"}, {"Noronha", "Фернандо де Норонья кышкы убактысы"}, {"Novosibirsk", "Новосибирск кышкы убакыты"}, {"Omsk", "Омск кышкы убакыты"}, {"Pakistan", "Пакистан кышкы убакыты"}, {"Palau", "Палау убактысы"}, {"Papua_New_Guinea", "Папуа-Жаңы Гвинея убакыты"}, {"Paraguay", "Парагвай кышкы убактысы"}, {"Peru", "Перу кышкы убактысы"}, {"Philippines", "Филиппин аралдарынын кышкы убактысы"}, {"Phoenix_Islands", "Феникс аралдарынын убактысы"}, {"Pierre_Miquelon", "Сен Пьер жана Микелон кышкы убактысы"}, {"Pitcairn", "Питкэрнг убактысы"}, {"Ponape", "Понапе убактысы"}, {"Pyongyang", "Пхеньян убакыты"}, {"Reunion", "Реюнион убактысы"}, {"Rothera", "Ротера убактысы"}, {"Sakhalin", "Сахалин кышкы убакыты"}, {"Samoa", "Самоа кышкы убактысы"}, {"Seychelles", "Сейшел убактысы"}, {"Singapore", "Сингапур убактысы"}, {"Solomon", "Соломон аралдарынын убактысы"}, {"South_Georgia", "Түштүк Жоржия убактысы"}, {"Suriname", "Суринаме убактысы"}, {"Syowa", "Саоа убактысы"}, {"Tahiti", "Таити убактысы"}, {"Taipei", "Тайпей кышкы убакыты"}, {"Tajikistan", "Тажикстан убактысы"}, {"Tokelau", "Токелау убактысы"}, {"Tonga", "Тонга кышкы убактысы"}, {"Truk", "Чуук убактысы"}, {"Turkmenistan", "Түркмөнстан кышкы убакыты"}, {"Tuvalu", "Тувалу убактысы"}, {"Uruguay", "Уругвай кышкы убактысы"}, {"Uzbekistan", "Өзбекстан кышкы убакыты"}, {"Vanuatu", "Вануату кышкы убакыты"}, {"Venezuela", "Венесуэла убактысы"}, {"Vladivostok", "Владивосток кышкы убакыты"}, {"Volgograd", "Волгоград кышкы убакыты"}, {"Vostok", "Восток убактысы"}, {"Wake", "Уейк аралдарынын убактысы"}, {"Wallis", "Уолис жана Футуна убактысы"}, {"Yakutsk", "Якутск кышкы убакыты"}, {"Yekaterinburg", "Екатеринбург кышкы убакыты"}, {"Yukon", "Юкон убактысы"}},
		{{"Alaska", "Аляска жайкы убактысы"}, {"Amazon", "Амазон жайкы убактысы"}, {"America_Central", "Түндүк Америка, борбордук жайкы убактысы"}, {"America_Eastern", "Түндүк Америка, чыгыш жайкы убактысы"}, {"America_Mountain", "Түндүк Америка, тоо жайкы убактысы"}, {"America_Pacific", "Түндүк Америка, Тынч океан жайкы убактысы"}, {"Apia", "Апиа жайкы убактысы"}, {"Arabian", "Арабия жайкы убакыты"}, {"Argentina", "Аргентина жайкы убактысы"}, {"Argentina_Western", "Батыш Аргентина жайкы убактысы"}, {"Armenia", "Армения жайкы убактысы"}, {"Atlantic", "Атлантика жайкы убактысы"}, {"Australia_Central", "Австралия борбордук жайкы убактысы"}, {"Australia_CentralWestern", "Австралия борбордук чыгыш жайкы убактысы"}, {"Australia_Eastern", "Австралия чыгыш жайкы убактысы"}, {"Australia_Western", "Австралия батыш жайкы убактысы"}, {"Azerbaijan", "Азербайжан жайкы убактысы"}, {"Azores", "Азорс жайкы убактысы"}, {"Bangladesh", "Бангладеш жайкы убактысы"}, {"Brasilia", "Бразилия жайкы убактысы"}, {"Cape_Verde", "Капе Верде жайкы убактысы"}, {"Chatham", "Чатам жайкы убактысы"}, {"Chile", "Чили жайкы убактысы"}, {"China", "Кытай жайкы убакыты"}, {"Colombia", "Колумбия жайкы убактысы"}, {"Cook", "Кук аралдарынын жарым жайкы убактысы"}, {"Cuba", "Куба жайкы убактысы"}, {"Easter", "Истер аралынын жайкы убакыты"}, {"Europe_Central", "Борбордук Европа жайкы убактысы"}, {"Europe_Eastern", "Чыгыш Европа жайкы убактысы"}, {"Europe_Western", "Батыш Европа жайкы убактысы"}, {"Falkland", "Фолкленд аралдарынын жайкы убактысы"}, {"Fiji", "Фижи жайкы убактысы"}, {"Georgia", "Грузия жайкы убактысы"}, {"Greenland_Eastern", "Чыгыш Гренландия жайкы убактысы"}, {"Greenland_Western", "Батыш Гренландия жайкы убактысы"}, {"Hawaii_Aleutian", "Гавайи-Алеут жайкы убактысы"}, {"Hong_Kong", "Гонконг жайкы убактысы"}, {"Hovd", "Ховд жайкы убактысы"}, {"Iran", "Иран күндүзгү убактысы"}, {"Irkutsk", "Иркутск жайкы убакыты"}, {"Israel", "Израиль жайкы убакыты"}, {"Japan", "Жапон жайкы убактысы"}, {"Korea", "Корея жайкы убактысы"}, {"Krasnoyarsk", "Красноярск жайкы убактысы"}, {"Lord_Howe", "Лорд Хау жайкы убактысы"}, {"Magadan", "Магадан жайкы убактысы"}, {"Mauritius", "Маврикий жайкы убактысы"}, {"Mexico_Pacific", "Мексика, Тынч океан жайкы убактысы"}, {"Mongolia", "Улан Батор жайкы убактысы"}, {"Moscow", "Москва жайкы убактысы"}, {"New_Caledonia", "Жаңы Каледония жайкы убактысы"}, {"New_Zealand", "Жаңы Зеландия жайкы убакыты"}, {"Newfoundland", "Нюфаундлэнд жайкы убактысы"}, {"Norfolk", "Норфолк жайкы убактысы"}, {"Noronha", "Фернандо де Норонья жайкы убактысы"}, {"Novosibirsk", "Новосибирск жайкы убактысы"}, {"Omsk", "Омск жайкы убактысы"}, {"Pakistan", "Пакистан жайкы убактысы"}, {"Paraguay", "Парагвай жайкы убактысы"}, {"Peru", "Перу жайкы убактысы"}, {"Philippines", "Филиппин аралдарынын жайкы убактысы"}, {"Pierre_Miquelon", "Сен Пьер жана Микелон жайкы убактысы"}, {"Sakhalin", "Сахалин жайкы убактысы"}, {"Samoa", "Самоа жайкы убактысы"}, {"Taipei", "Тайпей жайкы убакыты"}, {"Tonga", "Тонга жайкы убактысы"}, {"Turkmenistan", "Түркмөнстан жайкы убактысы"}, {"Uruguay", "Уругвай жайкы убактысы"}, {"Uzbekistan", "Өзбекстан жайкы убактысы"}, {"Vanuatu", "Вануату жайкы убактысы"}, {"Vladivostok", "Владивосток жайкы убактысы"}, {"Volgograd", "Волгоград жайкы убактысы"}, {"Yakutsk", "Якутск жайкы убактысы"}, {"Yekaterinburg", "Екатеринбург жайкы убактысы"}},
	},
	{
		{{"Acre", "Acre-Normalzäit"}, {"Afghanistan", "Afghanistan-Zäit"}, {"Africa_Central", "Zentralafrikanesch Zäit"}, {"Africa_Eastern", "Ostafrikanesch Zäit"}, {"Africa_Southern", "Südafrikanesch Zäit"}, {"Africa_Western", "Westafrikanesch Zäit"}, {"Alaska", "Alaska-Normalzäit"}, {"Almaty", "Almaty-Normalzäit"}, {"Amazon", "Amazonas-Normalzäit"}, {"America_Central", "Nordamerikanesch Inland-Normalzäit"}, {"America_Eastern", "Nordamerikanesch Ostküsten-Normalzäit"}, {"America_Mountain", "Rocky-Mountain-Normalzäit"}, {"America_Pacific", "Nordamerikanesch Westküsten-Normalzäit"}, {"Anadyr", "Anadyr-Normalzäit"}, {"Arabian", "Arabesch Normalzäit"}, {"Argentina", "Argentinesch Normalzäit"}, {"Argentina_Western", "Westargentinesch Normalzäit"}, {"Armenia", "Armenesch Normalzäit"}, {"Atlantic", "Atlantik-Normalzäit"}, {"Australia_Central", "Zentralaustralesch Normalzäit"}, {"Australia_CentralWestern", "Zentral-/Westaustralesch Normalzäit"}, {"Australia_Eastern", "Ostaustralesch Normalzäit"}, {"Australia_Western", "Westaustralesch Normalzäit"}, {"Azerbaijan", "Aserbeidschanesch Normalzäit"}, {"Azores", "Azoren-Normalzäit"}, {"Bangladesh", "Bangladesch-Normalzäit"}, {"Bhutan", "Bhutan-Zäit"}, {"Bolivia", "Bolivianesch Zäit"}, {"Brasilia", "Brasília-Normalzäit"}, {"Brunei", "Brunei-Zäit"}, {"Cape_Verde", "Kap-Verde-Normalzäit"}, {"Chamorro", "Chamorro-Zäit"}, {"Chatham", "Chatham-Normalzäit"}, {"Chile", "Chilenesch Normalzäit"}, {"China", "Chinesesch Normalzäit"}, {"Christmas", "Chrëschtdagsinsel-Zäit"}, {"Cocos", "Kokosinselen-Zäit"}, {"Colombia", "Kolumbianesch Normalzäit"}, {"Cook", "Cookinselen-Normalzäit"}, {"Cuba", "Kubanesch Normalzäit"}, {"Davis", "Davis-Zäit"}, {"DumontDUrville", "Dumont-d’Urville-Zäit"}, {"East_Timor", "Osttimor-Zäit"}, {"Easter", "Ouschterinsel-Normalzäit"}, {"Ecuador", "Ecuadorianesch Zäit"}, {"Europe_Central", "Mëtteleuropäesch Normalzäit"}, {"Europe_Eastern", "Osteuropäesch Normalzäit"}, {"Europe_Western", "Westeuropäesch Normalzäit"}, {"Falkland", "Falklandinselen-Normalzäit"}, {"Fiji", "Fidschi-Normalzäit"}, {"French_Guiana", "Franséisch-Guayane-Zäit"}, {"French_Southern", "Franséisch Süd- an Antarktisgebidder-Zäit"}, {"GMT", "Mëttler Greenwich-Zäit"}, {"Galapagos", "Galapagos-Zäit"}, {"Gambier", "Gambier-Zäit"}, {"Georgia", "Georgesch Normalzäit"}, {"Gilbert_Islands", "Gilbert-Inselen-Zäit"}, {"Greenland_Eastern", "Ostgrönland-Normalzäit"}, {"Greenland_Western", "Westgrönland-Normalzäit"}, {"Guam", "Guam-Zäit"}, {"Gulf", "Golf-Zäit"}, {"Guyana", "Guyana-Zäit"}, {"Hawaii", "Hawaii-Aleuten-Normalzäit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Normalzäit"}, {"Hong_Kong", "Hong-Kong-Normalzäit"}, {"Hovd", "Hovd-Normalzäit"}, {"India", "Indesch Zäit"}, {"Indian_Ocean", "Indeschen Ozean-Zäit"}, {"Indochina", "Indochina-Zäit"}, {"Indonesia_Central", "Zentralindonesesch Zäit"}, {"Indonesia_Eastern", "Ostindonesesch Zäit"}, {"Indonesia_Western", "Westindonesesch Zäit"}, {"Iran", "Iranesch Normalzäit"}, {"Irkutsk", "Irkutsk-Normalzäit"}, {"Israel", "Israelesch Normalzäit"}, {"Japan", "Japanesch Normalzäit"}, {"Kamchatka", "Kamtschatka-Normalzäit"}, {"Kazakhstan_Eastern", "Ostkasachesch Zäit"}, {"Kazakhstan_Western", "Westkasachesch Zäit"}, {"Korea", "Koreanesch Normalzäit"}, {"Kosrae", "Kosrae-Zäit"}, {"Krasnoyarsk", "Krasnojarsk-Normalzäit"}, {"Kyrgystan", "Kirgisistan-Zäit"}, {"Line_Islands", "Linneninselen-Zäit"}, {"Lord_Howe", "Lord-Howe-Normalzäit"}, {"Magadan", "Magadan-Normalzäit"}, {"Malaysia", "Malaysesch Zäit"}, {"Maldives", "Maldiven-Zäit"}, {"Marquesas", "Marquesas-Zäit"}, {"Marshall_Islands", "Marshallinselen-Zäit"}, {"Mauritius", "Mauritius-Normalzäit"}, {"Mawson", "Mawson-Zäit"}, {"Mexico_Pacific", "Mexikanesch Pazifik-Normalzäit"}, {"Mongolia", "Ulaanbaatar-Normalzäit"}, {"Moscow", "Moskauer Normalzäit"}, {"Myanmar", "Myanmar-Zäit"}, {"Nauru", "Nauru-Zäit"}, {"Nepal", "Nepalesesch Zäit"}, {"New_Caledonia", "Neikaledonesch Normalzäit"}, {"New_Zealand", "Neiséiland-Normalzäit"}, {"Newfoundland", "Neifundland-Normalzäit"}, {"Niue", "Niue-Zäit"}, {"Norfolk", "Norfolkinselen-Normalzäit"}, {"Noronha", "Fernando-de-Noronha-Normalzäit"}, {"Novosibirsk", "Nowosibirsk-Normalzäit"}, {"Omsk", "Omsk-Normalzäit"}, {"Pakistan", "Pakistanesch Normalzäit"}, {"Palau", "Palau-Zäit"}, {"Papua_New_Guinea", "Papua-Neiguinea-Zäit"}, {"Paraguay", "Paraguayanesch Normalzäit"}, {"Peru", "Peruanesch Normalzäit"}, {"Philippines", "Philippinnesch Normalzäit"}, {"Phoenix_Islands", "Phoenixinselen-Zäit"}, {"Pierre_Miquelon", "Saint-Pierre-a-Miquelon-Normalzäit"}, {"Pitcairn", "Pitcairninselen-Zäit"}, {"Ponape", "Ponape-Zäit"}, {"Reunion", "Réunion-Zäit"}, {"Rothera", "Rothera-Zäit"}, {"Sakhalin", "Sakhalin-Normalzäit"}, {"Samara", "Samara-Normalzäit"}, {"Samoa", "Samoa-Normalzäit"}, {"Seychelles", "Seychellen-Zäit"}, {"Singapore", "Singapur-Standardzäit"}, {"Solomon", "Salomoninselen-Zäit"}, {"South_Georgia", "Südgeorgesch Zäit"}, {"Suriname", "Suriname-Zäit"}, {"Syowa", "Syowa-Zäit"}, {"Tahiti", "Tahiti-Zäit"}, {"Taipei", "Taipei-Normalzäit"}, {"Tajikistan", "Tadschikistan-Zäit"}, {"Tokelau", "Tokelau-Zäit"}, {"Tonga", "Tonganesch Normalzäit"}, {"Truk", "Chuuk-Zäit"}, {"Turkmenistan", "Turkmenistan-Normalzäit"}, {"Tuvalu", "Tuvalu-Zäit"}, {"Uruguay", "Uruguyanesch Normalzäit"}, {"Uzbekistan", "Usbekistan-Normalzäit"}, {"Vanuatu", "Vanuatu-Normalzäit"}, {"Venezuela", "Venezuela-Zäit"}, {"Vladivostok", "Wladiwostok-Normalzäit"}, {"Volgograd", "Wolgograd-Normalzäit"}, {"Vostok", "Wostok-Zäit"}, {"Wake", "Wake-Insel-Zäit"}, {"Wallis", "Wallis-a-Futuna-Zäit"}, {"Yakutsk", "Jakutsk-Normalzäit"}, {"Yekaterinburg", "Jekaterinbuerg-Normalzäit"}},
		{{"Acre", "Acre-Summerzäit"}, {"Alaska", "Alaska-Summerzäit"}, {"Almaty", "Almaty-Summerzäit"}, {"Amazon", "Amazonas-Summerzäit"}, {"America_Central", "Nordamerikanesch Inland-Summerzäit"}, {"America_Eastern", "Nordamerikanesch Ostküsten-Summerzäit"}, {"America_Mountain", "Rocky-Mountain-Summerzäit"}, {"America_Pacific", "Nordamerikanesch Westküsten-Summerzäit"}, {"Anadyr", "Anadyr-Summerzäit"}, {"Arabian", "Arabesch Summerzäit"}, {"Argentina", "Argentinesch Summerzäit"}, {"Argentina_Western", "Westargentinesch Summerzäit"}, {"Armenia", "Armenesch Summerzäit"}, {"Atlantic", "Atlantik-Summerzäit"}, {"Australia_Central", "Zentralaustralesch Summerzäit"}, {"Australia_CentralWestern", "Zentral-/Westaustralesch Summerzäit"}, {"Australia_Eastern", "Ostaustralesch Summerzäit"}, {"Australia_Western", "Westaustralesch Summerzäit"}, {"Azerbaijan", "Aserbaidschanesch Summerzäit"}, {"Azores", "Azoren-Summerzäit"}, {"Bangladesh", "Bangladesch-Summerzäit"}, {"Brasilia", "Brasília-Summerzäit"}, {"Cape_Verde", "Kap-Verde-Summerzäit"}, {"Chatham", "Chatham-Summerzäit"}, {"Chile", "Chilenesch Summerzäit"}, {"China", "Chinesesch Summerzäit"}, {"Colombia", "Kolumbianesch Summerzäit"}, {"Cook", "Cookinselen-Summerzäit"}, {"Cuba", "Kubanesch Summerzäit"}, {"Easter", "Ouschterinsel-Summerzäit"}, {"Europe_Central", "Mëtteleuropäesch Summerzäit"}, {"Europe_Eastern", "Osteuropäesch Summerzäit"}, {"Europe_Western", "Westeuropäesch Summerzäit"}, {"Falkland", "Falklandinselen-Summerzäit"}, {"Fiji", "Fidschi-Summerzäit"}, {"Georgia", "Georgesch Summerzäit"}, {"Greenland_Eastern", "Ostgrönland-Summerzäit"}, {"Greenland_Western", "Westgrönland-Summerzäit"}, {"Hawaii_Aleutian", "Hawaii-Aleuten-Summerzäit"}, {"Hong_Kong", "Hong-Kong-Summerzäit"}, {"Hovd", "Hovd-Summerzäit"}, {"Iran", "Iranesch Summerzäit"}, {"Irkutsk", "Irkutsk-Summerzäit"}, {"Israel", "Israelesch Summerzäit"}, {"Japan", "Japanesch Summerzäit"}, {"Kamchatka", "Kamtschatka-Summerzäit"}, {"Korea", "Koreanesch Summerzäit"}, {"Krasnoyarsk", "Krasnojarsk-Summerzäit"}, {"Lord_Howe", "Lord-Howe-Summerzäit"}, {"Magadan", "Magadan-Summerzäit"}, {"Mauritius", "Mauritius-Summerzäit"}, {"Mexico_Pacific", "Mexikanesch Pazifik-Summerzäit"}, {"Mongolia", "Ulaanbaatar-Summerzäit"}, {"Moscow", "Moskauer Summerzäit"}, {"New_Caledonia", "Neikaledonesch Summerzäit"}, {"New_Zealand", "Neiséiland-Summerzäit"}, {"Newfoundland", "Neifundland-Summerzäit"}, {"Norfolk", "Norfolkinselen-Summerzäit"}, {"Noronha", "Fernando-de-Noronha-Summerzäit"}, {"Novosibirsk", "Nowosibirsk-Summerzäit"}, {"Omsk", "Omsk-Summerzäit"}, {"Pakistan", "Pakistanesch Summerzäit"}, {"Paraguay", "Paraguayanesch Summerzäit"}, {"Peru", "Peruanesch Summerzäit"}, {"Philippines", "Philippinnesch Summerzäit"}, {"Pierre_Miquelon", "Saint-Pierre-a-Miquelon-Summerzäit"}, {"Sakhalin", "Sakhalin-Summerzäit"}, {"Samara", "Samara-Summerzäit"}, {"Samoa", "Samoa-Summerzäit"}, {"Taipei", "Taipei-Summerzäit"}, {"Tonga", "Tonganesch Summerzäit"}, {"Turkmenistan", "Turkmenistan-Summerzäit"}, {"Uruguay", "Uruguayanesch Summerzäit"}, {"Uzbekistan", "Usbekistan-Summerzäit"}, {"Vanuatu", "Vanuatu-Summerzäit"}, {"Vladivostok", "Wladiwostok-Summerzäit"}, {"Volgograd", "Wolgograd-Summerzäit"}, {"Yakutsk", "Jakutsk-Summerzäit"}, {"Yekaterinburg", "Jekaterinbuerg-Summerzäit"}},
	},
	{
		{{"Afghanistan", "oa de l’Afghanistan"}, {"Africa_Central", "oa de l’Africa do mezo"}, {"Africa_Eastern", "oa de l’Africa de levante"}, {"Africa_Southern", "oa de l’Africa do meridion"}, {"Africa_Western", "oa de l’Africa de ponente"}, {"Alaska", "oa standard de l’Alaska"}, {"Amazon", "oa standard de l’Amassònia"}, {"America_Central", "oa standard do mezo nordamericaña"}, {"America_Eastern", "oa standard do levante nordamericaña"}, {"America_Mountain", "oa standard de Montagne Alliggiæ"}, {"America_Pacific", "oa standard do Paçifico nordamericaña"}, {"Apia", "oa standard de Apia"}, {"Arabian", "oa standard de l’Arabia"}, {"Argentina", "oa standard de l’Argentiña"}, {"Argentina_Western", "oa standard de l’Argentiña de ponente"}, {"Armenia", "oa standard de l’Ermenia"}, {"Atlantic", "oa standard de l’Atlantico nordamericaña"}, {"Australia_Central", "oa standard de l’Australia de mezo"}, {"Australia_CentralWestern", "oa standard de l’Australia do mezo-ponente"}, {"Australia_Eastern", "oa standard de l’Australia de levante"}, {"Australia_Western", "oa standard de l’Australia de ponente"}, {"Azerbaijan", "oa standard de l’Azerbaigian"}, {"Azores", "oa standard de Azore"}, {"Bangladesh", "oa standard do Bangladesh"}, {"Bhutan", "oa do Bhutan"}, {"Bolivia", "oa da Bolivia"}, {"Brasilia", "oa standard de Brasilia"}, {"Brunei", "oa do Brunei Darussalam"}, {"Cape_Verde", "oa standard de Cappo Verde"}, {"Chamorro", "oa de Chamorro"}, {"Chatham", "oa standard de Chatham"}, {"Chile", "oa standard do Cile"}, {"China", "oa standard da Ciña"}, {"Christmas", "oa de l’isoa Christmas"}, {"Cocos", "oa de isoe Cocos"}, {"Colombia", "oa standard da Colombia"}, {"Cook", "oa standard de isoe Cook"}, {"Cuba", "oa standard de Cubba"}, {"Davis", "oa de Davis"}, {"DumontDUrville", "oa de Dumont-d’Urville"}, {"East_Timor", "oa de Timor Est"}, {"Easter", "oa standard de l’isoa de Pasqua"}, {"Ecuador", "oa de l’Ecuador"}, {"Europe_Central", "oa standard de l’Euröpa do mezo"}, {"Europe_Eastern", "oa standard de l’Euröpa de levante"}, {"Europe_Further_Eastern", "oa de Kaliningrad"}, {"Europe_Western", "oa standard de l’Euröpa de ponente"}, {"Falkland", "oa standard de isoe Malviñe"}, {"Fiji", "oa standard de Figi"}, {"French_Guiana", "oa da Guyana franseise"}, {"French_Southern", "oa de Tære australe e antartiche franseixi"}, {"GMT", "oa do meridian de Greenwich"}, {"Galapagos", "oa de Galapagos"}, {"Gambier", "oa de Gambier"}, {"Georgia", "oa standard da Geòrgia"}, {"Gilbert_Islands", "oa de isoe Gilbert"}, {"Greenland_Eastern", "oa standard da Groenlandia de levante"}, {"Greenland_Western", "oa standard da Groenlandia de ponente"}, {"Gulf", "oa standard do Gorfo"}, {"Guyana", "oa da Guyana"}, {"Hawaii", "oa standard de Hawaii-Aleutiñe"}, {"Hawaii_Aleutian", "oa standard de Hawaii-Aleutiñe"}, {"Hong_Kong", "oa standard de Hong Kong"}, {"Hovd", "oa standard de Hovd"}, {"India", "oa de l’India"}, {"Indian_Ocean", "oa de l’Oçeano Indian"}, {"Indochina", "oa de l’Indociña"}, {"Indonesia_Central", "oa de l’Indonesia de mezo"}, {"Indonesia_Eastern", "oa de l’Indonesia de levante"}, {"Indonesia_Western", "oa de l’Indonesia de ponente"}, {"Iran", "oa standard de l’Iran"}, {"Irkutsk", "oa standard de Irkutsk"}, {"Israel", "oa standard d’Israele"}, {"Japan", "oa standard do Giappon"}, {"Kazakhstan_Eastern", "oa do Kazakistan de levante"}, {"Kazakhstan_Western", "oa do Kazakistan de ponente"}, {"Korea", "oa standard da Corea"}, {"Kosrae", "oa do Kosrae"}, {"Krasnoyarsk", "oa standard de Krasnoyarsk"}, {"Kyrgystan", "oa do Kirghizistan"}, {"Line_Islands", "oa de isoe da Linia"}, {"Lord_Howe", "oa standard de Lord Howe"}, {"Magadan", "oa standard de Magadan"}, {"Malaysia", "oa da Malesia"}, {"Maldives", "oa de Maldive"}, {"Marquesas", "oa de Marcheixi"}, {"Marshall_Islands", "oa de isoe Marshall"}, {"Mauritius", "oa standard de Mauritius"}, {"Mawson", "oa de Mawson"}, {"Mexico_Pacific", "oa standard do Paçifico mescicaña"}, {"Mongolia", "oa standard d’Ulan Bator"}, {"Moscow", "oa standard de Mosca"}, {"Myanmar", "oa da Birmania"}, {"Nauru", "oa de Nauru"}, {"Nepal", "oa do Nepal"}, {"New_Caledonia", "oa standard da Neuva Caledònia"}, {"New_Zealand", "oa standard da Neuva Zelanda"}, {"Newfoundland", "oa standard de Tæraneuva"}, {"Niue", "oa de Niue"}, {"Norfolk", "oa standard de l’isoa Norfolk"}, {"Noronha", "oa standard de Fernando de Noronha"}, {"Novosibirsk", "oa standard de Novosibirsk"}, {"Omsk", "oa standard de Òmsk"}, {"Pakistan", "oa standard do Pakistan"}, {"Palau", "oa de Palau"}, {"Papua_New_Guinea", "oa da Papua Neuva Guinea"}, {"Paraguay", "oa standard do Paraguay"}, {"Peru", "oa standard do Perù"}, {"Philippines", "oa standard de Filipiñe"}, {"Phoenix_Islands", "oa de isoe Phoenix"}, {"Pierre_Miquelon", "oa standard de San Pê e Miquelon"}, {"Pitcairn", "oa de Pitcairn"}, {"Ponape", "oa de Pohnpei"}, {"Pyongyang", "oa de Pyongyang"}, {"Reunion", "oa da Réunion"}, {"Rothera", "oa de Rothera"}, {"Sakhalin", "oa standard de Sakhalin"}, {"Samoa", "oa standard de Samoa"}, {"Seychelles", "oa de Seychelles"}, {"Singapore", "oa de Scingapô"}, {"Solomon", "oa de isoe Solomon"}, {"South_Georgia", "oa da Geòrgia do sud"}, {"Suriname", "oa do Suriname"}, {"Syowa", "oa de Syowa"}, {"Tahiti", "oa de Tahiti"}, {"Taipei", "oa standard de Taipei"}, {"Tajikistan", "oa do Tagikistan"}, {"Tokelau", "oa de Tokelau"}, {"Tonga", "oa standard de Tonga"}, {"Truk", "oa do Chuuk"}, {"Turkmenistan", "oa standard do Turkmenistan"}, {"Tuvalu", "oa de Tuvalu"}, {"Uruguay", "oa standard de l’Uruguay"}, {"Uzbekistan", "oa standard de l’Uzbekistan"}, {"Vanuatu", "oa standard de Vanuatu"}, {"Venezuela", "oa do Venessuela"}, {"Vladivostok", "oa standard de Vladivostok"}, {"Volgograd", "oa standard de Volgograd"}, {"Vostok", "oa de Vostok"}, {"Wake", "oa de l’isoa de Wake"}, {"Wallis", "oa de Wallis e Futuna"}, {"Yakutsk", "oa standard de Yakutsk"}, {"Yekaterinburg", "oa standard d’Ekaterinburg"}, {"Yukon", "oa do Yukon"}},
		{{"Alaska", "oa de stæ de l’Alaska"}, {"Amazon", "oa de stæ de l’Amassònia"}, {"America_Central", "oa de stæ do mezo nordamericaña"}, {"America_Eastern", "oa de stæ do levante nordamericaña"}, {"America_Mountain", "oa de stæ de Montagne Alliggiæ"}, {"America_Pacific", "oa de stæ do Paçifico nordamericaña"}, {"Apia", "oa de stæ de Apia"}, {"Arabian", "oa de stæ de l’Arabia"}, {"Argentina", "oa de stæ de l’Argentiña"}, {"Argentina_Western", "oa de stæ de l’Argentiña de ponente"}, {"Armenia", "oa de stæ de l’Ermenia"}, {"Atlantic", "oa de stæ de l’Atlantico nordamericaña"}, {"Australia_Central", "oa de stæ de l’Australia de mezo"}, {"Australia_CentralWestern", "oa de stæ de l’Australia do mezo-ponente"}, {"Australia_Eastern", "oa de stæ de l’Australia de levante"}, {"Australia_Western", "oa de stæ de l’Australia de ponente"}, {"Azerbaijan", "oa de stæ de l’Azerbaigian"}, {"Azores", "oa de stæ de Azore"}, {"Bangladesh", "oa de stæ do Bangladesh"}, {"Brasilia", "oa de stæ de Brasilia"}, {"Cape_Verde", "oa de stæ de Cappo Verde"}, {"Chatham", "oa de stæ de Chatham"}, {"Chile", "oa de stæ do Cile"}, {"China", "oa de stæ da Ciña"}, {"Colombia", "oa de stæ da Colombia"}, {"Cook", "oa de stæ de isoe Cook"}, {"Cuba", "oa de stæ de Cubba"}, {"Easter", "oa de stæ de l’isoa de Pasqua"}, {"Europe_Central", "oa de stæ de l’Euröpa do mezo"}, {"Europe_Eastern", "oa de stæ de l’Euröpa de levante"}, {"Europe_Western", "oa de stæ de l’Euröpa de ponente"}, {"Falkland", "oa de stæ de isoe Malviñe"}, {"Fiji", "oa de stæ de Figi"}, {"Georgia", "oa de stæ da Geòrgia"}, {"Greenland_Eastern", "oa de stæ da Groenlandia de levante"}, {"Greenland_Western", "oa de stæ da Groenlandia de ponente"}, {"Hawaii_Aleutian", "oa de stæ de Hawaii-Aleutiñe"}, {"Hong_Kong", "oa de stæ de Hong Kong"}, {"Hovd", "oa de stæ de Hovd"}, {"Iran", "oa de stæ de l’Iran"}, {"Irkutsk", "oa de stæ de Irkutsk"}, {"Israel", "oa de stæ d’Israele"}, {"Japan", "oa de stæ do Giappon"}, {"Korea", "oa de stæ da Corea"}, {"Krasnoyarsk", "oa de stæ de Krasnoyarsk"}, {"Lord_Howe", "oa de stæ de Lord Howe"}, {"Magadan", "oa de stæ de Magadan"}, {"Mauritius", "oa de stæ de Mauritius"}, {"Mexico_Pacific", "oa de stæ do Paçifico mescicaña"}, {"Mongolia", "oa de stæ d’Ulan Bator"}, {"Moscow", "oa de stæ de Mosca"}, {"New_Caledonia", "oa de stæ da Neuva Caledònia"}, {"New_Zealand", "oa de stæ da Neuva Zelanda"}, {"Newfoundland", "oa de stæ de Tæraneuva"}, {"Norfolk", "oa de stæ de l’isoa Norfolk"}, {"Noronha", "oa de stæ de Fernando de Noronha"}, {"Novosibirsk", "oa de stæ de Novosibirsk"}, {"Omsk", "oa de stæ de Òmsk"}, {"Pakistan", "oa de stæ do Pakistan"}, {"Paraguay", "oa de stæ do Paraguay"}, {"Peru", "oa de stæ do Perù"}, {"Philippines", "oa de stæ de Filipiñe"}, {"Pierre_Miquelon", "oa de stæ de San Pê e Miquelon"}, {"Sakhalin", "oa de stæ de Sakhalin"}, {"Samoa", "oa de stæ de Samoa"}, {"Taipei", "oa de stæ de Taipei"}, {"Tonga", "oa de stæ de Tonga"}, {"Turkmenistan", "oa de stæ do Turkmenistan"}, {"Uruguay", "oa de stæ de l’Uruguay"}, {"Uzbekistan", "oa de stæ de l’Uzbekistan"}, {"Vanuatu", "oa de stæ de Vanuatu"}, {"Vladivostok", "oa de stæ de Vladivostok"}, {"Volgograd", "oa de stæ de Volgograd"}, {"Yakutsk", "oa de stæ de Yakutsk"}, {"Yekaterinburg", "oa de stæ d’Ekaterinburg"}},
	},
	{
		{{"Afghanistan", "Ora dl Afghanistan"}, {"Africa_Central", "Ora dl’Africa zentrala"}, {"Africa_Eastern", "Ora dl’Africa orientala"}, {"Africa_Southern", "Ora dl’Africa dl Süd"}, {"Africa_Western", "Ora dl’Africa ozidentala"}, {"Alaska", "Ora standard dl’Alaska"}, {"Amazon", "Ora standard dl’Amazonia"}, {"America_Central", "Ora standard dl’America dl Nord zentrala"}, {"America_Eastern", "Ora standard dl’America dl Nord orientala"}, {"America_Mountain", "Ora standard dles Rocky Mountains"}, {"America_Pacific", "Ora standard dl’America dl Nord ozidentala"}, {"Apia", "Ora standard de Apia"}, {"Arabian", "Ora standard dl’Arabia"}, {"Argentina", "Ora standard dl’Argentina"}, {"Argentina_Western", "Ora standard dl’Argentina ozidentala"}, {"Armenia", "Ora standard dl’Armenia"}, {"Atlantic", "Ora standard dl Atlantich"}, {"Australia_Central", "Ora standard dl’Australia zentrala"}, {"Australia_CentralWestern", "Ora standard dl’Australia zënter-ozidentala"}, {"Australia_Eastern", "Ora standard dl’Australia orientala"}, {"Australia_Western", "Ora standard dl’Australia ozidentala"}, {"Azerbaijan", "Ora standard dl Azerbaijan"}, {"Azores", "Ora standard dles Azores"}, {"Bangladesh", "Ora standard dl Bangladesc"}, {"Bhutan", "Ora dl Bhutan"}, {"Bolivia", "Ora dla Bolivia"}, {"Brasilia", "Ora standard de Brasilia"}, {"Brunei", "Ora dl Brunei Darussalam"}, {"Cape_Verde", "Ora standard de Capo Verde"}, {"Chamorro", "Ora standard de Chamorro"}, {"Chatham", "Ora standard dles Isoles Chatham"}, {"Chile", "Ora standard dl Cile"}, {"China", "Ora standard dla Cina"}, {"Christmas", "Ora dla Isola dl Nadé"}, {"Cocos", "Ora dles Isoles Cocos"}, {"Colombia", "Ora standard dla Colombia"}, {"Cook", "Ora standard dles Isoles Cook"}, {"Cuba", "Ora standard de Cuba"}, {"Davis", "Ora de Davis"}, {"DumontDUrville", "Ora de Dumont-d’Urville"}, {"East_Timor", "Ora de Timor Ost"}, {"Easter", "Ora standard dl’Isola de Pasca"}, {"Ecuador", "Ora dl Ecuador"}, {"Europe_Central", "Ora standard dl’Europa zentrala"}, {"Europe_Eastern", "Ora standard dl’Europa orientala"}, {"Europe_Further_Eastern", "Ora de Kaliningrad"}, {"Europe_Western", "Ora standard dl’Europa ozidentala"}, {"Falkland", "Ora standard dles Isoles Falkland"}, {"Fiji", "Ora standard dles Fiji"}, {"French_Guiana", "Ora dla Guyana franzeja"}, {"French_Southern", "Ora di Teritori franzesc dl Süd y dl’Antartica"}, {"GMT", "Ora mesana de Greenwich"}, {"Galapagos", "Ora dles Galapagos"}, {"Gambier", "Ora dles Isoles Gambier"}, {"Georgia", "Ora standard dla Georgia"}, {"Gilbert_Islands", "Ora dles Isoles Gilbert"}, {"Greenland_Eastern", "Ora standard dla Groenlandia orientala"}, {"Greenland_Western", "Ora standard dla Groenlandia ozidentala"}, {"Gulf", "Ora standard dl Golf"}, {"Guyana", "Ora dla Guyana"}, {"Hawaii", "Ora standard dles Isoles Hawaii-Aleutines"}, {"Hawaii_Aleutian", "Ora standard dles Isoles Hawaii-Aleutines"}, {"Hong_Kong", "Ora standard de Hong Kong"}, {"Hovd", "Ora standard de Hovd"}, {"India", "Ora standard dl’India"}, {"Indian_Ocean", "Ora dl Ozean indian"}, {"Indochina", "Ora dl’Indocina"}, {"Indonesia_Central", "Ora dl’Indonesia zentrala"}, {"Indonesia_Eastern", "Ora dl’Indonesia orientala"}, {"Indonesia_Western", "Ora dl’Indonesia ozidentala"}, {"Iran", "Ora standard dl Iran"}, {"Irkutsk", "Ora standard de Irkutsk"}, {"Israel", "Ora standard d’Israel"}, {"Japan", "Ora standard dl Iapan"}, {"Kazakhstan", "Ora dl Kazakhstan"}, {"Kazakhstan_Eastern", "Ora dl Kazakhstan oriental"}, {"Kazakhstan_Western", "Ora dl Kazakhstan ozidental"}, {"Korea", "Ora standard dla Corea"}, {"Kosrae", "Ora dl’Isola Kosrae"}, {"Krasnoyarsk", "Ora standard de Krasnoyarsk"}, {"Kyrgystan", "Ora dl Kyrgystan"}, {"Line_Islands", "Ora dles Isoles Line"}, {"Lord_Howe", "Ora standard de Lord Howe"}, {"Magadan", "Ora standard de Magadan"}, {"Malaysia", "Ora dla Malesia"}, {"Maldives", "Ora dles Maldives"}, {"Marquesas", "Ora dles Isoles Marchejes"}, {"Marshall_Islands", "Ore dles Isoles Marshall"}, {"Mauritius", "Ora standard de Mauritius"}, {"Mawson", "Ora de Mawson"}, {"Mexico_Pacific", "Ora standard dla spona pazifica dl Messich"}, {"Mongolia", "Ora standard de Ulan Bator"}, {"Moscow", "Ora standard de Mosca"}, {"Myanmar", "Ora dl Myanmar"}, {"Nauru", "Ora de Nauru"}, {"Nepal", "Ora dl Nepal"}, {"New_Caledonia", "Ora standard dla Nöia Caledonia"}, {"New_Zealand", "Ora standard dla Nöia Zelanda"}, {"Newfoundland", "Ora standard de Newfoundland"}, {"Niue", "Ora de Niue"}, {"Norfolk", "Ora standard dl’Isola Norfolk"}, {"Noronha", "Ora standard de Fernando de Noronha"}, {"Novosibirsk", "Ora standard de Novosibirsk"}, {"Omsk", "Ora standard de Omsk"}, {"Pakistan", "Ora standard dl Pakistan"}, {"Palau", "Ora de Palau"}, {"Papua_New_Guinea", "Ora dla Papua Nöia Guinea"}, {"Paraguay", "Ora standard dl Paraguay"}, {"Peru", "Ora standard dl Perú"}, {"Philippines", "Ora standard dles Filipines"}, {"Phoenix_Islands", "Ora dles Isoles Phoenix"}, {"Pierre_Miquelon", "Ora standard de St. Pierre y Miquelon"}, {"Pitcairn", "Ora dles Isoles Pitcairn"}, {"Ponape", "Ora de Ponape"}, {"Pyongyang", "Ora de Pjöngjang"}, {"Reunion", "Ora de Réunion"}, {"Rothera", "Ora de Rothera"}, {"Sakhalin", "Ora standard de Sachalin"}, {"Samoa", "Ora standard de Samoa"}, {"Seychelles", "Ora dles Seychelles"}, {"Singapore", "Ora standard de Singapur"}, {"Solomon", "Ora dles Isoles Salomon"}, {"South_Georgia", "Ora dla Georgia dl Süd"}, {"Suriname", "Ora dl Surinam"}, {"Syowa", "Ora de Syowa"}, {"Tahiti", "Ora de Tahiti"}, {"Taipei", "Ora standard de Taipei"}, {"Tajikistan", "Ora dl Tajikistan"}, {"Tokelau", "Ora de Tokelau"}, {"Tonga", "Ora standard de Tonga"}, {"Truk", "Ora dl Chuuk"}, {"Turkmenistan", "Ora standard dl Turkmenistan"}, {"Tuvalu", "Ora de Tuvalu"}, {"Uruguay", "Ora standard dl Uruguay"}, {"Uzbekistan", "Ora standard dl Uzbekistan"}, {"Vanuatu", "Ora standard dl Vanuatu"}, {"Venezuela", "Ora dl Venezuela"}, {"Vladivostok", "Ora standard de Vladivostok"}, {"Volgograd", "Ora standard de Volgograd"}, {"Vostok", "Ora de Vostok"}, {"Wake", "Ora dl’Isola de Wake"}, {"Wallis", "Ora de Wallis y Futuna"}, {"Yakutsk", "Ora standard de Iakutsk"}, {"Yekaterinburg", "Ora standard de Yekaterinburg"}, {"Yukon", "Ora dl Yukon"}},
		{{"Alaska", "Ora da d’isté dl’Alaska"}, {"Amazon", "Ora da d’isté dl’Amazonia"}, {"America_Central", "Ora da d’isté dl’America dl Nord zentrala"}, {"America_Eastern", "Ora da d’isté dl’America dl Nord orientala"}, {"America_Mountain", "Ora da d’isté dles Rocky Mountains"}, {"America_Pacific", "Ora da d’isté dl’America dl Nord ozidentala"}, {"Apia", "Ora da d’isté de Apia"}, {"Arabian", "Ora da d’isté dl’Arabia"}, {"Argentina", "Ora da d’isté dl’Argentina"}, {"Argentina_Western", "Ora da d’isté dl’Argentina ozidentala"}, {"Armenia", "Ora da d’isté dl’Armenia"}, {"Atlantic", "Ora da d’isté dl Atlantich"}, {"Australia_Central", "Ora da d’isté dl’Australia zentrala"}, {"Australia_CentralWestern", "Ora da d’isté dl’Australia zënter-ozidentala"}, {"Australia_Eastern", "Ora da d’isté dl’Australia orientala"}, {"Australia_Western", "Ora da d’isté dl’Australia ozidentala"}, {"Azerbaijan", "Ora da d’isté dl Azerbaijan"}, {"Azores", "Ora da d’isté dles Azores"}, {"Bangladesh", "Ora da d’isté dl Bangladesc"}, {"Brasilia", "Ora da d’isté de Brasilia"}, {"Cape_Verde", "Ora da d’isté de Capo Verde"}, {"Chatham", "Ora da d’isté dles Isoles Chatham"}, {"Chile", "Ora da d’isté dl Cile"}, {"China", "Ora da d’isté dla Cina"}, {"Colombia", "Ora da d’isté dla Colombia"}, {"Cook", "Ora da d’isté mesana dles Isoles Cook"}, {"Cuba", "Ora da d’isté de Cuba"}, {"Easter", "Ora da d’isté dl’Isola de Pasca"}, {"Europe_Central", "Ora da d’isté dl’Europa zentrala"}, {"Europe_Eastern", "Ora da d’isté dl’Europa orientala"}, {"Europe_Western", "Ora da d’isté dl’Europa ozidentala"}, {"Falkland", "Ora da d’isté dles Isoles Falkland"}, {"Fiji", "Ora da d’isté dles Fiji"}, {"Georgia", "Ora da d’isté dla Georgia"}, {"Greenland_Eastern", "Ora da d’isté dla Groenlandia orientala"}, {"Greenland_Western", "Ora da d’isté dla Groenlandia ozidentala"}, {"Hawaii_Aleutian", "Ora da d’isté dles Isoles Hawaii-Aleutines"}, {"Hong_Kong", "Ora da d’isté de Hong Kong"}, {"Hovd", "Ora da d’isté de Hovd"}, {"Iran", "Ora da d’isté dl Iran"}, {"Irkutsk", "Ora da d’isté de Irkutsk"}, {"Israel", "Ora da d’isté d’Israel"}, {"Japan", "Ora da d’isté dl Iapan"}, {"Korea", "Ora da d’isté dla Corea"}, {"Krasnoyarsk", "Ora da d’isté de Krasnoyarsk"}, {"Lord_Howe", "Ora da d’isté de Lord Howe"}, {"Magadan", "Ora da d’isté de Magadan"}, {"Mauritius", "Ora da d’isté de Mauritius"}, {"Mexico_Pacific", "Ora da d’isté dla spona pazifica dl Messich"}, {"Mongolia", "Ora da d’isté de Ulan Bator"}, {"Moscow", "Ora da d’isté de Mosca"}, {"New_Caledonia", "Ora da d’isté dla Nöia Caledonia"}, {"New_Zealand", "Ora da d’isté dla Nöia Zelanda"}, {"Newfoundland", "Ora da d’isté de Newfoundland"}, {"Norfolk", "Ora da d’isté dl’Isola Norfolk"}, {"Noronha", "Ora da d’isté de Fernando de Noronha"}, {"Novosibirsk", "Ora da d’isté de Novosibirsk"}, {"Omsk", "Ora da d’isté de Omsk"}, {"Pakistan", "Ora da d’isté dl Pakistan"}, {"Paraguay", "Ora da d’isté dl Paraguay"}, {"Peru", "Ora da d’isté dl Perú"}, {"Philippines", "Ora da d’isté dles Filipines"}, {"Pierre_Miquelon", "Ora da d’isté de St. Pierre y Miquelon"}, {"Sakhalin", "Ora da d’isté de Sachalin"}, {"Samoa", "Ora da d’isté de Samoa"}, {"Taipei", "Ora da d’isté de Taipei"}, {"Tonga", "Ora da d’isté de Tonga"}, {"Turkmenistan", "Ora da d’isté dl Turkmenistan"}, {"Uruguay", "Ora da d’isté dl Uruguay"}, {"Uzbekistan", "Ora da d’isté dl Uzbekistan"}, {"Vanuatu", "Ora da d’isté dl Vanuatu"}, {"Vladivostok", "Ora da d’isté de Vladivostok"}, {"Volgograd", "Ora da d’isté de Volgograd"}, {"Yakutsk", "Ora da d’isté de Iakutsk"}, {"Yekaterinburg", "Ora da d’isté de Yekaterinburg"}},
	},
	{
		{{"GMT", "Temp Medi de Greenwich"}},
		{},
	},
	{
		{{"Africa_Central", "Ntángo ya Lubumbashi"}, {"Africa_Eastern", "Ntángo ya Afríka ya Ɛ́sita"}, {"Africa_Southern", "Ntángo ya Afríka ya Sidi"}, {"GMT", "Ntángo ya Londoni"}, {"Seychelles", "Ntángo ya Seyshel"}},
		{},
	},
	{
		{{"Acre", "ເວລາມາດຕະຖານຂອງອາເກຣ"}, {"Afghanistan", "ເວລາ ອັຟການິສຖານ"}, {"Africa_Central", "ເວ\u200bລາ\u200bອາ\u200bຟຣິ\u200bກາ\u200bກາງ"}, {"Africa_Eastern", "ເວ\u200bລາ\u200bອາ\u200bຟຣິ\u200bກາ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Africa_Southern", "ເວ\u200bລາ\u200bອາ\u200bຟຣິ\u200bກາ\u200bໃຕ້"}, {"Africa_Western", "ເວ\u200bລາ\u200bອາ\u200bຟຣິ\u200bກາ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Alaska", "ເວລາມາດຕະຖານອະລັສກາ"}, {"Almaty", "ເວລາມາດຕະຖານອໍມາຕີ"}, {"Amazon", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອາ\u200bເມ\u200bຊອນ"}, {"America_Central", "ເວລາມາດຕະຖານກາງ"}, {"America_Eastern", "ເວລາມາດຕະຖານຕາເວັນອອກ"}, {"America_Mountain", "ເວລາມາດຕະຖານແຖບພູເຂົາ"}, {"America_Pacific", "ເວລາມາດຕະຖານແປຊິຟິກ"}, {"Apia", "ເວລາມາດຕະຖານເອເພຍ"}, {"Aqtau", "ເວລາມາດຕະຖານອັດຕາອູ"}, {"Aqtobe", "ເວລາມາດຕະຖານອັດໂຕເບ"}, {"Arabian", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອາ\u200bຣາ\u200bບຽນ"}, {"Argentina", "\u200bເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອາ\u200bເຈນ\u200bທິ\u200bນາ"}, {"Argentina_Western", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານເວ\u200bສ\u200bເທິນອາ\u200bເຈນ\u200bທິ\u200bນາ"}, {"Armenia", "ເວລາມາດຕະຖານອາເມເນຍ"}, {"Atlantic", "ເວລາມາດຕະຖານຂອງອາແລນຕິກ"}, {"Australia_Central", "ເວ\u200bລາມາດ\u200bຕະ\u200bຖານອອ\u200bສ\u200bເຕຣ\u200bເລຍ\u200bກ\u200bາງ"}, {"Australia_CentralWestern", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອອສ\u200bເຕຣ\u200bລຽນ\u200bກາງ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Australia_Eastern", "ເວ\u200bລາ\u200bມາດຕະຖານ\u200b\u200b\u200bອອສ\u200bເຕຣ\u200bລຽນ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Australia_Western", "ເວ\u200bລາ\u200bມາ\u200bດ\u200bຕະ\u200bຖານອອສ\u200bເຕຣ\u200bລຽນ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Azerbaijan", "ເວລາມາດຕະຖານອັສເຊີໄບຈັນ"}, {"Azores", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອາ\u200bໂຊ\u200bເຣ\u200bສ"}, {"Bangladesh", "ເວລາມາດຕະຖານ ບັງກະລາເທດ"}, {"Bhutan", "ເວ\u200bລາ\u200bພູ\u200bຖານ"}, {"Bolivia", "ເວ\u200bລາ\u200bໂບ\u200bລິ\u200bເວຍ"}, {"Brasilia", "ເວລາມາດຕາຖານເບຣຊີເລຍ"}, {"Brunei", "\u200bເວ\u200bລາບຣູ\u200bໄນດາ\u200bຣຸສ\u200bຊາ\u200bລາມ"}, {"Cape_Verde", "\u200bເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bເຄບ\u200bເວີດ"}, {"Casey", "ເວລາເຄຊີ"}, {"Chamorro", "ເວ\u200bລາ\u200bຈາ\u200bໂມ\u200bໂຣ"}, {"Chatham", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຊາ\u200bທາມ"}, {"Chile", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານຊິ\u200bລີ"}, {"China", "ເວລາມາດຕະຖານຈີນ"}, {"Christmas", "ເວ\u200bລາ\u200bເກາະ\u200bຄ\u200bຣິສ\u200bມາສ"}, {"Cocos", "ເວລາຫມູ່ເກາະໂກໂກສ"}, {"Colombia", "ເວລາມາດຕະຖານໂຄລຳເບຍ"}, {"Cook", "ເວລາມາດຕະຖານຫມູ່ເກາະຄຸກ"}, {"Cuba", "ເວລາມາດຕະຖານຂອງຄິວບາ"}, {"Davis", "ເວລາເດວິດ"}, {"DumontDUrville", "ເວລາດູມອງດູວິລ"}, {"East_Timor", "ເວລາຕີມໍຕາເວັນອອກ"}, {"Easter", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານເກາະ\u200bອີ\u200bສ\u200bເຕີ"}, {"Ecuador", "ເວ\u200bລາ\u200bເອ\u200bກົວ\u200bດໍ"}, {"Europe_Central", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຢູ\u200bໂຣບກາງ"}, {"Europe_Eastern", "ເວ\u200bລາ\u200bມາ\u200bດ\u200bຕະ\u200bຖານ\u200bຢູ\u200bໂຣບ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Europe_Further_Eastern", "ເວ\u200bລາ\u200b\u200bຢູ\u200bໂຣ\u200bປຽນ\u200bຕາ\u200bເວັນ\u200bອອກ\u200bໄກ"}, {"Europe_Western", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານຢູ\u200bໂຣບ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Falkland", "\u200bເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານໝູ່\u200bເກາະ\u200bຟອ\u200bລ໌ກ\u200bແລນ"}, {"Fiji", "ເວລາມາດຕະຖານຟິຈິ"}, {"French_Guiana", "ເວ\u200bລາ\u200bເຟ\u200bຣນ\u200bຊ໌\u200bເກຍ\u200bນາ"}, {"French_Southern", "ເວລາຝຣັ່ງຕອນໃຕ້ ແລະ ແອນຕາກຕິກ"}, {"GMT", "ເວ\u200bລາກຣີນ\u200bວິ\u200bຊ"}, {"Galapagos", "ເວ\u200bລາ\u200bກາ\u200bລາ\u200bປາ\u200bກອ\u200bສ"}, {"Gambier", "ເວລາແກມເບຍ"}, {"Georgia", "ເວລາມາດຕະຖານຈໍເຈຍ"}, {"Gilbert_Islands", "ເວລາຫມູ່ເກາະກິລເບີດ"}, {"Greenland_Eastern", "ເວລາມາດຕະຖານຕາເວັນອອກກຣີນແລນ"}, {"Greenland_Western", "ເວລາມາດຕະຖານກຣີນແລນຕາເວັນຕົກ"}, {"Guam", "ເວລາກວມ"}, {"Gulf", "ເວ\u200bລາ\u200bກູ\u200bລ\u200b໌ຟ"}, {"Guyana", "ເວລາກາຍອານາ"}, {"Hawaii", "ເວລາມາດຕະຖານຮາວາຍ-ເອລູທຽນ"}, {"Hawaii_Aleutian", "ເວລາມາດຕະຖານຮາວາຍ-ເອລູທຽນ"}, {"Hong_Kong", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຮອງ\u200bກົງ"}, {"Hovd", "\u200bເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຮອບ\u200bດ໌"}, {"India", "ເວລາ ອິນເດຍ"}, {"Indian_Ocean", "ເວລາມະຫາສະຫມຸດອິນເດຍ"}, {"Indochina", "ເວລາອິນດູຈີນ"}, {"Indonesia_Central", "ເວ\u200bລາ\u200bອິນ\u200bໂດ\u200bເນ\u200bເຊຍ\u200bກາງ"}, {"Indonesia_Eastern", "ເວ\u200bລາ\u200bອິນ\u200bໂດ\u200bເນ\u200bເຊຍ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Indonesia_Western", "ເວ\u200bລາ\u200bອິນ\u200bໂດ\u200bເນ\u200bເຊຍ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Iran", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອີ\u200bຣານ"}, {"Irkutsk", "ເວ\u200bລາມາດ\u200bຕະ\u200bຖານອີ\u200bຄຸດ\u200bສ\u200bຄ໌"}, {"Israel", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອິ\u200bສ\u200bຣາ\u200bເອວ"}, {"Japan", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຍີ່\u200bປຸ່ນ"}, {"Kazakhstan", "ເວລາຄາຊັກສຖານ"}, {"Kazakhstan_Eastern", "ເວ\u200bລາ\u200bຄາ\u200bຊັກ\u200bສ\u200bຖານ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Kazakhstan_Western", "ເວ\u200bລາ\u200bຄາ\u200bຊັກ\u200bສ\u200bຖານ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Korea", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bເກົາ\u200bຫລີ"}, {"Kosrae", "ເວລາຄອສແຣ"}, {"Krasnoyarsk", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານຄຣັສ\u200bໂນ\u200bຢາ\u200bສ\u200bຄ໌"}, {"Kyrgystan", "ເວລາເຄຍກິສຖານ"}, {"Lanka", "ເວລາລັງກາ"}, {"Line_Islands", "ເວ\u200bລາ\u200bໝູ່\u200bເກາະ\u200bລາຍ"}, {"Lord_Howe", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bລອດ\u200bເຮົາ"}, {"Macau", "ເວລາມາດຕະຖານມາເກົາ"}, {"Magadan", "ເວລາມາດຕະຖານເມັກກາເດນ"}, {"Malaysia", "ເວ\u200bລາ\u200bມາ\u200bເລ\u200bເຊຍ"}, {"Maldives", "ເວລາມັລດີຟ"}, {"Marquesas", "ເວລາມາເຄີຊັສ"}, {"Marshall_Islands", "ເວ\u200bລາ\u200bໝູ່\u200bເກາະ\u200bມາ\u200bແຊວ"}, {"Mauritius", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານມົວ\u200bຣິ\u200bຊຽ\u200bສ"}, {"Mawson", "ເວລາມໍສັນ"}, {"Mexico_Pacific", "ເວລາມາດຕະຖານແປຊິຟິກເມັກຊິກັນ"}, {"Mongolia", "ເວລາມາດຕະຖານ ອູລານບາເຕີ"}, {"Moscow", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານມອ\u200bສ\u200bໂຄ"}, {"Myanmar", "ເວລາມຽນມາ"}, {"Nauru", "ເວ\u200bລາ\u200bນາ\u200bອູ\u200bຣຸ"}, {"Nepal", "\u200bເວ\u200bລາ\u200bເນ\u200bປານ"}, {"New_Caledonia", "ເວລາມາດຕະຖານນິວແຄລິໂດເນຍ"}, {"New_Zealand", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານນິວ\u200bຊີ\u200bແລນ"}, {"Newfoundland", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bນິວ\u200bຟາວ\u200bແລນ"}, {"Niue", "ເວລານິອູເອ"}, {"Norfolk", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານເກາະ\u200bນໍ\u200bຟອ\u200bລ໌ກ"}, {"Noronha", "ເວລາມາດຕະຖານເຟນັນໂດເດໂນຮອນຮາ"}, {"North_Mariana", "ເວລາຫມູ່ເກາະມາເຣຍນາເຫນືອ"}, {"Novosibirsk", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານໂນ\u200bໂບ\u200bຊິ\u200bບິ\u200bສ\u200bຄ໌"}, {"Omsk", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານອອມ\u200bສ\u200bຄ໌"}, {"Pakistan", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bປາ\u200bກີສຖານ"}, {"Palau", "ເວລາປາເລົາ"}, {"Papua_New_Guinea", "ເວລາປາປົວກິນີ"}, {"Paraguay", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bປາ\u200bຣາ\u200bກວຍ"}, {"Peru", "ເວ\u200bລາ\u200b\u200bມາ\u200bດ\u200bຕະ\u200bຖານເປ\u200bຣູ"}, {"Philippines", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bຟິ\u200bລິບ\u200bປິນ"}, {"Phoenix_Islands", "ເວລາຫມູ່ເກາະຟີນິກ"}, {"Pierre_Miquelon", "\u200bເວ\u200bລາມາດ\u200bຕະ\u200bຖານເຊນ\u200bປີ\u200bແອ ແລະ\u200bມິ\u200bກົວ\u200bລອນ"}, {"Pitcairn", "ເວລາພິດແຄຣ໌ນ"}, {"Ponape", "ເວລາໂປເນບ"}, {"Pyongyang", "ເວລາປຽງຢາງ"}, {"Qyzylorda", "ເວລາມາດຕະຖານຄີວລໍດາ"}, {"Reunion", "ເວ\u200bລາ\u200bເຣ\u200bອູ\u200bນິ\u200bຢົງ"}, {"Rothera", "ເວລາ ໂຣທີຕາ"}, {"Sakhalin", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານຊາ\u200bຮາ\u200bລິນ"}, {"Samoa", "ເວລາມາດຕະຖານຊາມົວ"}, {"Seychelles", "ເວ\u200bລາ\u200bເຊ\u200bເຊ\u200bລ\u200bສ໌"}, {"Singapore", "ເວ\u200bລາ\u200bສິງ\u200bກະ\u200bໂປ"}, {"Solomon", "ເວລາຫມູ່ເກາະໂຊໂລມອນ"}, {"South_Georgia", "ເວລາຈໍເຈຍໃຕ້"}, {"Suriname", "ເວ\u200bລາ\u200bຊຸ\u200bຣິ\u200bນາມ"}, {"Syowa", "ເວລາ ໂຊວາ"}, {"Tahiti", "ເວລາທາຮິຕິ"}, {"Taipei", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bໄທ\u200bເປ"}, {"Tajikistan", "ເວລາທາຈິກິສຖານ"}, {"Tokelau", "ເວລາໂຕເກເລົາ"}, {"Tonga", "ເວລາມາດຕະຖານຕອງກາ"}, {"Truk", "ເວລາຊຸກ"}, {"Turkmenistan", "ເວລາມາດຕະຖານຕວກເມນິສຖານ"}, {"Tuvalu", "ເວລາຕູວາລູ"}, {"Uruguay", "ເວ\u200bລາ\u200bມາດ\u200bຕະ\u200bຖານ\u200bອູ\u200bຣູ\u200bກວຍ"}, {"Uzbekistan", "ເວລາມາດຕະຖານອຸສເບກິດສະຖານ"}, {"Vanuatu", "ເວລາມາດຕະຖານວານູອາຕູ"}, {"Venezuela", "ເວ\u200bລາ\u200bເວ\u200bເນ\u200bຊູ\u200bເອ\u200bລາ"}, {"Vladivostok", "ເວລາມາດຕະຖານລາດີໂວສຕົກ"}, {"Volgograd", "ເວລາມາດຕະຖານໂວໂກກຣາດ"}, {"Vostok", "ເວລາ ວອສໂຕກ"}, {"Wake", "ເວລາເກາະເວກ"}, {"Wallis", "ເວລາວາລລິສ ແລະ ຟູຕູນາ"}, {"Yakutsk", "ເວລາມາດຕະຖານຢາກູດສ"}, {"Yekaterinburg", "ເວລາມາດຕະຖານເຢກາເຕລິນເບີກ"}, {"Yukon", "ເວລາຢູຄອນ"}},
		{{"Acre", "ເວລາລະດູຮ້ອນຂອງອາເກຣ"}, {"Alaska", "ເວລາກາງເວັນອະລັສກາ"}, {"Almaty", "ເວລາລະດູຮ້ອນອໍມາຕີ"}, {"Amazon", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນອາ\u200bເມ\u200bຊອນ"}, {"America_Central", "ເວລາກາງເວັນກາງ"}, {"America_Eastern", "ເວລາກາງເວັນຕາເວັນອອກ"}, {"America_Mountain", "ເວລາກາງເວັນແຖບພູເຂົາ"}, {"America_Pacific", "ເວລາກາງເວັນແປຊິຟິກ"}, {"Apia", "ເວລາກາງເວັນອາເພຍ"}, {"Aqtau", "ເວລາລະດູຮ້ອນອັດຕາອູ"}, {"Aqtobe", "ເວລາລະດູຮ້ອນອັດໂຕເບ"}, {"Arabian", "ເວລາກາງເວັນອາຣາບຽນ"}, {"Argentina", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bອາ\u200bເຈນ\u200bທິ\u200bນາ"}, {"Argentina_Western", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນເວ\u200bສ\u200bເທິນອາ\u200bເຈນ\u200bທິ\u200bນາ"}, {"Armenia", "ເວລາລະດູຮ້ອນອາເມເນຍ"}, {"Atlantic", "ເວລາກາງເວັນຂອງອາແລນຕິກ"}, {"Australia_Central", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bອອສ\u200bເຕຣ\u200bເລຍ\u200bກາງ"}, {"Australia_CentralWestern", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bອອສ\u200bເຕຣ\u200bລຽນ\u200bກາງ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Australia_Eastern", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bອອສ\u200bເຕຣ\u200bລຽນ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Australia_Western", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bອອສ\u200bເຕຣ\u200bລຽນ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Azerbaijan", "ເວລາລະດູຮ້ອນອັສເຊີໄບຈັນ"}, {"Azores", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນອາ\u200bໂຊ\u200bເຣ\u200bສ"}, {"Bangladesh", "ເວລາ ລະດູຮ້ອນ ບັງກະລາເທດ"}, {"Brasilia", "ເວລາຕາມເຂດລະດູຮ້ອນຕາມເຂດບຣາຊີເລຍ"}, {"Cape_Verde", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bເຄບ\u200bເວີດ"}, {"Chatham", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bຊາ\u200bທາມ"}, {"Chile", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນຊິ\u200bລີ"}, {"China", "\u200bເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bຈີນ"}, {"Colombia", "ເວລາລະດູຮ້ອນໂຄລໍາເບຍ"}, {"Cook", "ເວ\u200bລາ\u200bເຄິ່ງ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bໝູ່\u200bເກາະ\u200bຄຸກ"}, {"Cuba", "ເວລາກາງເວັນຄິວບາ"}, {"Easter", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນເກາະ\u200bອີ\u200bສ\u200bເຕີ"}, {"Europe_Central", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bຢູ\u200bໂຣບ\u200bກາງ"}, {"Europe_Eastern", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນຢູ\u200bໂຣບ\u200bຕາ\u200bເວັນ\u200bອອກ"}, {"Europe_Western", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນຢູ\u200bໂຣບ\u200bຕາ\u200bເວັນ\u200bຕົກ"}, {"Falkland", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນໝູ່\u200bເກາະ\u200bຟອ\u200bລ໌ກ\u200bແລນ"}, {"Fiji", "ເວລາລະດູຮ້ອນຟິຈິ"}, {"Georgia", "ເວລາລະດູຮ້ອນຈໍເຈຍ"}, {"Greenland_Eastern", "ເວລາລະດູຮ້ອນກຣີນແລນຕາເວັນອອກ"}, {"Greenland_Western", "ເວລາຕອນທ່ຽງກຣີນແລນຕາເວັນຕົກ"}, {"Hawaii_Aleutian", "ເວລາຕອນທ່ຽງຮາວາຍ-ເອລູທຽນ"}, {"Hong_Kong", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bຮອງ\u200bກົງ"}, {"Hovd", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bຮອບ\u200bດ໌"}, {"Iran", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bອີ\u200bຣາ\u200bນ"}, {"Irkutsk", "ເວ\u200bລາລະ\u200bດູ\u200bຮ້ອນອີ\u200bຄຸດ\u200bສ\u200bຄ໌"}, {"Israel", "ເວລາກາງເວັນອິສຣາເອວ"}, {"Japan", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bຍີ່\u200bປຸ່ນ"}, {"Korea", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bເກົາ\u200bຫລີ"}, {"Krasnoyarsk", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນຄຣັສ\u200bໂນ\u200bຢາ\u200bສ\u200bຄ໌"}, {"Lord_Howe", "\u200bເວ\u200bລ\u200bສາ\u200bຕອນ\u200b\u200bທ່ຽງ\u200bລອດ\u200bເຮົາ\u200b"}, {"Macau", "ເວລາລະດູຮ້ອນມາເກົາ"}, {"Magadan", "ເວລາລະດູຮ້ອນເມັກກາເດນ"}, {"Mauritius", "\u200bເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bເມົາ\u200bຣິ\u200bທຽ\u200bສ"}, {"Mexico_Pacific", "ເວລາກາງເວັນແປຊິຟິກເມັກຊິກັນ"}, {"Mongolia", "ເວລາລະດູຮ້ອນອູລານບາເຕີ"}, {"Moscow", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນມອ\u200bສ\u200bໂຄ"}, {"New_Caledonia", "ເວລາລະດູຮ້ອນນິວແຄລິໂດເນຍ"}, {"New_Zealand", "ເວ\u200bລາ\u200bຕອນ\u200bທ່ຽງ\u200bນິວ\u200bຊີ\u200bແລນ"}, {"Newfoundland", "ເວລາກາງເວັນນິວຟາວແລນ"}, {"Norfolk", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນເກາະ\u200bນໍ\u200bຟອ\u200bລ໌ກ"}, {"Noronha", "ເວລາລະດູຮ້ອນເຟນັນໂດເດໂນຮອນຮາ"}, {"Novosibirsk", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນໂນ\u200bໂບ\u200bຊິ\u200bບິ\u200bສ\u200bຄ໌"}, {"Omsk", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນອອມ\u200bສ\u200bຄ໌"}, {"Pakistan", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bປາ\u200bກີ\u200bສ\u200bຖານ"}, {"Paraguay", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bປາ\u200bຣາ\u200bກວຍ"}, {"Peru", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bເປ\u200bຣູ"}, {"Philippines", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bຟິ\u200bລິບ\u200bປິນ"}, {"Pierre_Miquelon", "\u200bເວ\u200bລາຕອນ\u200bທ່ຽງເຊນ\u200bປີ\u200bແອ ແລະ\u200bມິ\u200bກົວ\u200bລອນ"}, {"Qyzylorda", "ເວລາລະດູຮ້ອນຄີວລໍດາ"}, {"Sakhalin", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນຊາ\u200bຮາ\u200bລິນ"}, {"Samoa", "ເວລາລະດູຮ້ອນຊາມົວ"}, {"Taipei", "ເວ\u200bລາ\u200bຕອນ\u200bທ່\u200bຽງ\u200bໄທ\u200bເປ"}, {"Tonga", "ເວລາລະດູຮ້ອນຕອງກາ"}, {"Turkmenistan", "ເວລາລະດູຮ້ອນຕວກເມນິສຖານ"}, {"Uruguay", "ເວ\u200bລາ\u200bລະ\u200bດູ\u200bຮ້ອນ\u200bອູ\u200bຣູ\u200bກວຍ"}, {"Uzbekistan", "ເວລາລະດູຮ້ອນອຸສເບກິດສະຖານ"}, {"Vanuatu", "ເວລາລະດູຮ້ອນວານູອາຕູ"}, {"Vladivostok", "ເວລາລະດູຮ້ອນລາດີໂວສຕົກ"}, {"Volgograd", "ເວລາລະດູຮ້ອນໂວໂກກຣາດ"}, {"Yakutsk", "ເວລາລະດູຮ້ອນຢາກູດສ"}, {"Yekaterinburg", "ເວລາລະດູຮ້ອນເຢກາເຕລິນເບີກ"}},
	},
	{
		{{"America_Central", "گاٛت مینجاٛیی ئستاٛنداٛرد"}},
		{{"America_Central", "روٙشنایی نئهادار روٙز"}},
	},
	{
		{{"Acre", "Ako standartinis laikas"}, {"Afghanistan", "Afganistano laikas"}, {"Africa_Central", "Centrinės Afrikos laikas"}, {"Africa_Eastern", "Rytų Afrikos laikas"}, {"Africa_Southern", "Pietų Afrikos laikas"}, {"Africa_Western", "Vakarų Afrikos laikas"}, {"Alaska", "Aliaskos žiemos laikas"}, {"Almaty", "Almatos žiemos laikas"}, {"Amazon", "Amazonės žiemos laikas"}, {"America_Central", "Šiaurės Amerikos centro žiemos laikas"}, {"America_Eastern", "Šiaurės Amerikos rytų žiemos laikas"}, {"America_Mountain", "Šiaurės Amerikos kalnų žiemos laikas"}, {"America_Pacific", "Šiaurės Amerikos Ramiojo vandenyno žiemos laikas"}, {"Anadyr", "Anadyrės žiemos laikas"}, {"Apia", "Apijos žiemos laikas"}, {"Aqtau", "Aktau žiemos laikas"}, {"Aqtobe", "Aktobės žiemos laikas"}, {"Arabian", "Arabijos žiemos laikas"}, {"Argentina", "Argentinos žiemos laikas"}, {"Argentina_Western", "Vakarų Argentinos žiemos laikas"}, {"Armenia", "Armėnijos žiemos laikas"}, {"Atlantic", "Atlanto žiemos laikas"}, {"Australia_Central", "Centrinės Australijos žiemos laikas"}, {"Australia_CentralWestern", "Centrinės vakarų Australijos žiemos laikas"}, {"Australia_Eastern", "Rytų Australijos žiemos laikas"}, {"Australia_Western", "Vakarų Australijos žiemos laikas"}, {"Azerbaijan", "Azerbaidžano žiemos laikas"}, {"Azores", "Azorų Salų žiemos laikas"}, {"Bangladesh", "Bangladešo žiemos laikas"}, {"Bhutan", "Butano laikas"}, {"Bolivia", "Bolivijos laikas"}, {"Brasilia", "Brazilijos žiemos laikas"}, {"Brunei", "Brunėjaus Darusalamo laikas"}, {"Cape_Verde", "Žaliojo Kyšulio žiemos laikas"}, {"Casey", "Keisio laikas"}, {"Chamorro", "Čamoro laikas"}, {"Chatham", "Čatamo žiemos laikas"}, {"Chile", "Čilės žiemos laikas"}, {"China", "Kinijos žiemos laikas"}, {"Christmas", "Kalėdų Salos laikas"}, {"Cocos", "Kokosų Salų laikas"}, {"Colombia", "Kolumbijos žiemos laikas"}, {"Cook", "Kuko Salų žiemos laikas"}, {"Cuba", "Kubos žiemos laikas"}, {"Davis", "Deiviso laikas"}, {"DumontDUrville", "Diumono d’Urvilio laikas"}, {"East_Timor", "Rytų Timoro laikas"}, {"Easter", "Velykų salos žiemos laikas"}, {"Ecuador", "Ekvadoro laikas"}, {"Europe_Central", "Vidurio Europos žiemos laikas"}, {"Europe_Eastern", "Rytų Europos žiemos laikas"}, {"Europe_Further_Eastern", "Tolimųjų rytų Europos laikas"}, {"Europe_Western", "Vakarų Europos žiemos laikas"}, {"Falkland", "Folklandų Salų žiemos laikas"}, {"Fiji", "Fidžio žiemos laikas"}, {"French_Guiana", "Prancūzijos Gvianos laikas"}, {"French_Southern", "Pietų Prancūzijos ir antarktinis laikas"}, {"GMT", "Grinvičo laikas"}, {"Galapagos", "Galapagų laikas"}, {"Gambier", "Gambyro laikas"}, {"Georgia", "Gruzijos žiemos laikas"}, {"Gilbert_Islands", "Gilberto Salų laikas"}, {"Greenland_Eastern", "Grenlandijos rytų žiemos laikas"}, {"Greenland_Western", "Grenlandijos vakarų žiemos laikas"}, {"Guam", "Guamo laikas"}, {"Gulf", "Persijos įlankos laikas"}, {"Guyana", "Gajanos laikas"}, {"Hawaii", "Havajų–Aleutų žiemos laikas"}, {"Hawaii_Aleutian", "Havajų–Aleutų žiemos laikas"}, {"Hong_Kong", "Honkongo žiemos laikas"}, {"Hovd", "Hovdo žiemos laikas"}, {"India", "Indijos laikas"}, {"Indian_Ocean", "Indijos vandenyno laikas"}, {"Indochina", "Indokinijos laikas"}, {"Indonesia_Central", "Centrinės Indonezijos laikas"}, {"Indonesia_Eastern", "Rytų Indonezijos laikas"}, {"Indonesia_Western", "Vakarų Indonezijos laikas"}, {"Iran", "Irano žiemos laikas"}, {"Irkutsk", "Irkutsko žiemos laikas"}, {"Israel", "Izraelio žiemos laikas"}, {"Japan", "Japonijos žiemos laikas"}, {"Kamchatka", "Kamčiatkos Petropavlovsko žiemos laikas"}, {"Kazakhstan", "Kazachstano laikas"}, {"Kazakhstan_Eastern", "Rytų Kazachstano laikas"}, {"Kazakhstan_Western", "Vakarų Kazachstano laikas"}, {"Korea", "Korėjos žiemos laikas"}, {"Kosrae", "Kosrajė laikas"}, {"Krasnoyarsk", "Krasnojarsko žiemos laikas"}, {"Kyrgystan", "Kirgistano laikas"}, {"Lanka", "Lankos laikas"}, {"Line_Islands", "Laino Salų laikas"}, {"Lord_Howe", "Lordo Hau žiemos laikas"}, {"Macau", "Makau žiemos laikas"}, {"Magadan", "Magadano žiemos laikas"}, {"Malaysia", "Malaizijos laikas"}, {"Maldives", "Maldyvų laikas"}, {"Marquesas", "Markizo Salų laikas"}, {"Marshall_Islands", "Maršalo Salų laikas"}, {"Mauritius", "Mauricijaus žiemos laikas"}, {"Mawson", "Mosono laikas"}, {"Mexico_Pacific", "Meksikos Ramiojo vandenyno žiemos laikas"}, {"Mongolia", "Ulan Batoro žiemos laikas"}, {"Moscow", "Maskvos žiemos laikas"}, {"Myanmar", "Mianmaro laikas"}, {"Nauru", "Nauru laikas"}, {"Nepal", "Nepalo laikas"}, {"New_Caledonia", "Naujosios Kaledonijos žiemos laikas"}, {"New_Zealand", "Naujosios Zelandijos žiemos laikas"}, {"Newfoundland", "Niufaundlendo žiemos laikas"}, {"Niue", "Niujė laikas"}, {"Norfolk", "Norfolko Salų žiemos laikas"}, {"Noronha", "Fernando de Noronjos žiemos laikas"}, {"North_Mariana", "Šiaurės Marianos Salų laikas"}, {"Novosibirsk", "Novosibirsko žiemos laikas"}, {"Omsk", "Omsko žiemos laikas"}, {"Pakistan", "Pakistano žiemos laikas"}, {"Palau", "Palau laikas"}, {"Papua_New_Guinea", "Papua Naujosios Gvinėjos laikas"}, {"Paraguay", "Paragvajaus žiemos laikas"}, {"Peru", "Peru žiemos laikas"}, {"Philippines", "Filipinų žiemos laikas"}, {"Phoenix_Islands", "Fenikso Salų laikas"}, {"Pierre_Miquelon", "Sen Pjero ir Mikelono žiemos laikas"}, {"Pitcairn", "Pitkerno laikas"}, {"Ponape", "Ponapės laikas"}, {"Pyongyang", "Pchenjano laikas"}, {"Qyzylorda", "Kyzylordos žiemos laikas"}, {"Reunion", "Reunjono laikas"}, {"Rothera", "Roteros laikas"}, {"Sakhalin", "Sachalino žiemos laikas"}, {"Samara", "Samaros žiemos laikas"}, {"Samoa", "Samoa žiemos laikas"}, {"Seychelles", "Seišelių laikas"}, {"Singapore", "Singapūro laikas"}, {"Solomon", "Saliamono Salų laikas"}, {"South_Georgia", "Pietų Džordžijos laikas"}, {"Suriname", "Surinamo laikas"}, {"Syowa", "Siovos laikas"}, {"Tahiti", "Tahičio laikas"}, {"Taipei", "Taipėjaus žiemos laikas"}, {"Tajikistan", "Tadžikistano laikas"}, {"Tokelau", "Tokelau laikas"}, {"Tonga", "Tongos žiemos laikas"}, {"Truk", "Čuko laikas"}, {"Turkey", "Turkijos žiemos laikas"}, {"Turkmenistan", "Turkmėnistano žiemos laikas"}, {"Tuvalu", "Tuvalu laikas"}, {"Uruguay", "Urugvajaus žiemos laikas"}, {"Uzbekistan", "Uzbekistano žiemos laikas"}, {"Vanuatu", "Vanuatu žiemos laikas"}, {"Venezuela", "Venesuelos laikas"}, {"Vladivostok", "Vladivostoko žiemos laikas"}, {"Volgograd", "Volgogrado žiemos laikas"}, {"Vostok", "Vostoko laikas"}, {"Wake", "Veiko Salos laikas"}, {"Wallis", "Voliso ir Futūnos laikas"}, {"Yakutsk", "Jakutsko žiemos laikas"}, {"Yekaterinburg", "Jekaterinburgo žiemos laikas"}, {"Yukon", "Jukono laikas"}},
		{{"Acre", "Ako vasaros laikas"}, {"Alaska", "Aliaskos vasaros laikas"}, {"Almaty", "Almatos vasaros laikas"}, {"Amazon", "Amazonės vasaros laikas"}, {"America_Central", "Šiaurės Amerikos centro vasaros laikas"}, {"America_Eastern", "Šiaurės Amerikos rytų vasaros laikas"}, {"America_Mountain", "Šiaurės Amerikos kalnų vasaros laikas"}, {"America_Pacific", "Šiaurės Amerikos Ramiojo vandenyno vasaros laikas"}, {"Anadyr", "Anadyrės vasaros laikas"}, {"Apia", "Apijos vasaros laikas"}, {"Aqtau", "Aktau vasaros laikas"}, {"Aqtobe", "Aktobės vasaros laikas"}, {"Arabian", "Arabijos vasaros laikas"}, {"Argentina", "Argentinos vasaros laikas"}, {"Argentina_Western", "Vakarų Argentinos vasaros laikas"}, {"Armenia", "Armėnijos vasaros laikas"}, {"Atlantic", "Atlanto vasaros laikas"}, {"Australia_Central", "Centrinės Australijos vasaros laikas"}, {"Australia_CentralWestern", "Centrinės vakarų Australijos vasaros laikas"}, {"Australia_Eastern", "Rytų Australijos vasaros laikas"}, {"Australia_Western", "Vakarų Australijos vasaros laikas"}, {"Azerbaijan", "Azerbaidžano vasaros laikas"}, {"Azores", "Azorų Salų vasaros laikas"}, {"Bangladesh", "Bangladešo vasaros laikas"}, {"Brasilia", "Brazilijos vasaros laikas"}, {"Cape_Verde", "Žaliojo Kyšulio vasaros laikas"}, {"Chatham", "Čatamo vasaros laikas"}, {"Chile", "Čilės vasaros laikas"}, {"China", "Kinijos vasaros laikas"}, {"Colombia", "Kolumbijos vasaros laikas"}, {"Cook", "Kuko Salų pusės vasaros laikas"}, {"Cuba", "Kubos vasaros laikas"}, {"Easter", "Velykų Salos vasaros laikas"}, {"Europe_Central", "Vidurio Europos vasaros laikas"}, {"Europe_Eastern", "Rytų Europos vasaros laikas"}, {"Europe_Western", "Vakarų Europos vasaros laikas"}, {"Falkland", "Folklando Salų vasaros laikas"}, {"Fiji", "Fidžio vasaros laikas"}, {"Georgia", "Gruzijos vasaros laikas"}, {"Greenland_Eastern", "Grenlandijos rytų vasaros laikas"}, {"Greenland_Western", "Grenlandijos vakarų vasaros laikas"}, {"Hawaii_Aleutian", "Havajų–Aleutų vasaros laikas"}, {"Hong_Kong", "Honkongo vasaros laikas"}, {"Hovd", "Hovdo vasaros laikas"}, {"Iran", "Irano vasaros laikas"}, {"Irkutsk", "Irkutsko vasaros laikas"}, {"Israel", "Izraelio vasaros laikas"}, {"Japan", "Japonijos vasaros laikas"}, {"Kamchatka", "Kamčiatkos Petropavlovsko vasaros laikas"}, {"Korea", "Korėjos vasaros laikas"}, {"Krasnoyarsk", "Krasnojarsko vasaros laikas"}, {"Lord_Howe", "Lordo Hau vasaros laikas"}, {"Macau", "Makau vasaros laikas"}, {"Magadan", "Magadano vasaros laikas"}, {"Mauritius", "Mauricijaus vasaros laikas"}, {"Mexico_Pacific", "Meksikos Ramiojo vandenyno vasaros laikas"}, {"Mongolia", "Ulan Batoro vasaros laikas"}, {"Moscow", "Maskvos vasaros laikas"}, {"New_Caledonia", "Naujosios Kaledonijos vasaros laikas"}, {"New_Zealand", "Naujosios Zelandijos vasaros laikas"}, {"Newfoundland", "Niufaundlendo vasaros laikas"}, {"Norfolk", "Norfolko Salų vasaros laikas"}, {"Noronha", "Fernando de Noronjos vasaros laikas"}, {"Novosibirsk", "Novosibirsko vasaros laikas"}, {"Omsk", "Omsko vasaros laikas"}, {"Pakistan", "Pakistano vasaros laikas"}, {"Paraguay", "Paragvajaus vasaros laikas"}, {"Peru", "Peru vasaros laikas"}, {"Philippines", "Filipinų vasaros laikas"}, {"Pierre_Miquelon", "Sen Pjero ir Mikelono vasaros laikas"}, {"Qyzylorda", "Kyzylordos vasaros laikas"}, {"Sakhalin", "Sachalino vasaros laikas"}, {"Samara", "Samaros vasaros laikas"}, {"Samoa", "Samoa vasaros laikas"}, {"Taipei", "Taipėjaus vasaros laikas"}, {"Tonga", "Tongos vasaros laikas"}, {"Turkey", "Turkijos vasaros laikas"}, {"Turkmenistan", "Turkmėnistano vasaros laikas"}, {"Uruguay", "Urugvajaus vasaros laikas"}, {"Uzbekistan", "Uzbekistano vasaros laikas"}, {"Vanuatu", "Vanuatu vasaros laikas"}, {"Vladivostok", "Vladivostoko vasaros laikas"}, {"Volgograd", "Volgogrado vasaros laikas"}, {"Yakutsk", "Jakutsko vasaros laikas"}, {"Yekaterinburg", "Jekaterinburgo vasaros laikas"}},
	},
	{
		{{"GMT", "Griničys laiks"}},
		{},
	},
	{
		{{"Afghanistan", "Afganistānas laiks"}, {"Africa_Central", "Centrālāfrikas laiks"}, {"Africa_Eastern", "Austrumāfrikas laiks"}, {"Africa_Southern", "Dienvidāfrikas ziemas laiks"}, {"Africa_Western", "Rietumāfrikas laiks"}, {"Alaska", "Aļaskas ziemas laiks"}, {"Amazon", "Amazones ziemas laiks"}, {"America_Central", "Centrālais ziemas laiks"}, {"America_Eastern", "Austrumu ziemas laiks"}, {"America_Mountain", "Kalnu ziemas laiks"}, {"America_Pacific", "Klusā okeāna ziemas laiks"}, {"Anadyr", "Anadiras ziemas laiks"}, {"Apia", "Apijas ziemas laiks"}, {"Arabian", "Arābijas pussalas ziemas laiks"}, {"Argentina", "Argentīnas ziemas laiks"}, {"Argentina_Western", "Rietumargentīnas ziemas laiks"}, {"Armenia", "Armēnijas ziemas laiks"}, {"Atlantic", "Atlantijas ziemas laiks"}, {"Australia_Central", "Austrālijas centrālais ziemas laiks"}, {"Australia_CentralWestern", "Austrālijas centrālais rietumu ziemas laiks"}, {"Australia_Eastern", "Austrālijas austrumu ziemas laiks"}, {"Australia_Western", "Austrālijas rietumu ziemas laiks"}, {"Azerbaijan", "Azerbaidžānas ziemas laiks"}, {"Azores", "Azoru salu ziemas laiks"}, {"Bangladesh", "Bangladešas ziemas laiks"}, {"Bhutan", "Butānas laiks"}, {"Bolivia", "Bolīvijas laiks"}, {"Brasilia", "Brazīlijas ziemas laiks"}, {"Brunei", "Brunejas Darusalamas laiks"}, {"Cape_Verde", "Kaboverdes ziemas laiks"}, {"Chamorro", "Čamorra ziemas laiks"}, {"Chatham", "Četemas ziemas laiks"}, {"Chile", "Čīles ziemas laiks"}, {"China", "Ķīnas ziemas laiks"}, {"Christmas", "Ziemsvētku salas laiks"}, {"Cocos", "Kokosu (Kīlinga) salu laiks"}, {"Colombia", "Kolumbijas ziemas laiks"}, {"Cook", "Kuka salu ziemas laiks"}, {"Cuba", "Kubas ziemas laiks"}, {"Davis", "Deivisas laiks"}, {"DumontDUrville", "Dimondirvilas laiks"}, {"East_Timor", "Austrumtimoras laiks"}, {"Easter", "Lieldienu salas ziemas laiks"}, {"Ecuador", "Ekvadoras laiks"}, {"Europe_Central", "Centrāleiropas ziemas laiks"}, {"Europe_Eastern", "Austrumeiropas ziemas laiks"}, {"Europe_Further_Eastern", "Austrumeiropas laika josla (FET)"}, {"Europe_Western", "Rietumeiropas ziemas laiks"}, {"Falkland", "Folklenda (Malvinu) salu ziemas laiks"}, {"Fiji", "Fidži ziemas laiks"}, {"French_Guiana", "Francijas Gviānas laiks"}, {"French_Southern", "Francijas Dienvidjūru un Antarktikas teritorijas laiks"}, {"GMT", "Griničas laiks"}, {"Galapagos", "Galapagu laiks"}, {"Gambier", "Gambjē salu laiks"}, {"Georgia", "Gruzijas ziemas laiks"}, {"Gilbert_Islands", "Gilberta salu laiks"}, {"Greenland_Eastern", "Austrumgrenlandes ziemas laiks"}, {"Greenland_Western", "Rietumgrenlandes ziemas laiks"}, {"Gulf", "Persijas līča laiks"}, {"Guyana", "Gajānas laiks"}, {"Hawaii", "Havaju–Aleutu ziemas laiks"}, {"Hawaii_Aleutian", "Havaju–Aleutu ziemas laiks"}, {"Hong_Kong", "Honkongas ziemas laiks"}, {"Hovd", "Hovdas ziemas laiks"}, {"India", "Indijas ziemas laiks"}, {"Indian_Ocean", "Indijas okeāna laiks"}, {"Indochina", "Indoķīnas laiks"}, {"Indonesia_Central", "Centrālindonēzijas laiks"}, {"Indonesia_Eastern", "Austrumindonēzijas laiks"}, {"Indonesia_Western", "Rietumindonēzijas laiks"}, {"Iran", "Irānas ziemas laiks"}, {"Irkutsk", "Irkutskas ziemas laiks"}, {"Israel", "Izraēlas ziemas laiks"}, {"Japan", "Japānas ziemas laiks"}, {"Kamchatka", "Petropavlovskas-Kamčatskas ziemas laiks"}, {"Kazakhstan", "Kazahstānas laiks"}, {"Kazakhstan_Eastern", "Austrumkazahstānas laiks"}, {"Kazakhstan_Western", "Rietumkazahstānas laiks"}, {"Korea", "Korejas ziemas laiks"}, {"Kosrae", "Kosrae laiks"}, {"Krasnoyarsk", "Krasnojarskas ziemas laiks"}, {"Kyrgystan", "Kirgizstānas laiks"}, {"Line_Islands", "Lainas salu laiks"}, {"Lord_Howe", "Lorda Hava salas ziemas laiks"}, {"Magadan", "Magadanas ziemas laiks"}, {"Malaysia", "Malaizijas laiks"}, {"Maldives", "Maldīvijas laiks"}, {"Marquesas", "Marķīza salu laiks"}, {"Marshall_Islands", "Māršala salu laiks"}, {"Mauritius", "Maurīcijas ziemas laiks"}, {"Mawson", "Mosonas laiks"}, {"Mexico_Pacific", "Meksikas Klusā okeāna piekrastes ziemas laiks"}, {"Mongolia", "Ulanbatoras ziemas laiks"}, {"Moscow", "Maskavas ziemas laiks"}, {"Myanmar", "Mjanmas laiks"}, {"Nauru", "Nauru laiks"}, {"Nepal", "Nepālas laiks"}, {"New_Caledonia", "Jaunkaledonijas ziemas laiks"}, {"New_Zealand", "Jaunzēlandes ziemas laiks"}, {"Newfoundland", "Ņūfaundlendas ziemas laiks"}, {"Niue", "Niues laiks"}, {"Norfolk", "Norfolkas salas ziemas laiks"}, {"Noronha", "Fernandu di Noroņas ziemas laiks"}, {"Novosibirsk", "Novosibirskas ziemas laiks"}, {"Omsk", "Omskas ziemas laiks"}, {"Pakistan", "Pakistānas ziemas laiks"}, {"Palau", "Palau laiks"}, {"Papua_New_Guinea", "Papua-Jaungvinejas laiks"}, {"Paraguay", "Paragvajas ziemas laiks"}, {"Peru", "Peru ziemas laiks"}, {"Philippines", "Filipīnu ziemas laiks"}, {"Phoenix_Islands", "Fēniksa salu laiks"}, {"Pierre_Miquelon", "Senpjēras un Mikelonas ziemas laiks"}, {"Pitcairn", "Pitkērnas laiks"}, {"Ponape", "Ponapē laiks"}, {"Pyongyang", "Phenjanas laiks"}, {"Reunion", "Reinjonas laiks"}, {"Rothera", "Roteras laiks"}, {"Sakhalin", "Sahalīnas ziemas laiks"}, {"Samara", "Samaras ziemas laiks"}, {"Samoa", "Samoa ziemas laiks"}, {"Seychelles", "Seišeļu salu laiks"}, {"Singapore", "Singapūras laiks"}, {"Solomon", "Zālamana salu laiks"}, {"South_Georgia", "Dienviddžordžijas laiks"}, {"Suriname", "Surinamas laiks"}, {"Syowa", "Šovas laiks"}, {"Tahiti", "Taiti laiks"}, {"Taipei", "Taibei ziemas laiks"}, {"Tajikistan", "Tadžikistānas laiks"}, {"Tokelau", "Tokelau laiks"}, {"Tonga", "Tongas ziemas laiks"}, {"Truk", "Čūkas laiks"}, {"Turkey", "Turcijas ziemas laiks"}, {"Turkmenistan", "Turkmenistānas ziemas laiks"}, {"Tuvalu", "Tuvalu laiks"}, {"Uruguay", "Urugvajas ziemas laiks"}, {"Uzbekistan", "Uzbekistānas ziemas laiks"}, {"Vanuatu", "Vanuatu ziemas laiks"}, {"Venezuela", "Venecuēlas laiks"}, {"Vladivostok", "Vladivostokas ziemas laiks"}, {"Volgograd", "Volgogradas ziemas laiks"}, {"Vostok", "Vostokas laiks"}, {"Wake", "Veika salas laiks"}, {"Wallis", "Volisas un Futunas laiks"}, {"Yakutsk", "Jakutskas ziemas laiks"}, {"Yekaterinburg", "Jekaterinburgas ziemas laiks"}, {"Yukon", "Jukonas laiks"}},
		{{"Alaska", "Aļaskas vasaras laiks"}, {"Amazon", "Amazones vasaras laiks"}, {"America_Central", "Centrālais vasaras laiks"}, {"America_Eastern", "Austrumu vasaras laiks"}, {"America_Mountain", "Kalnu vasaras laiks"}, {"America_Pacific", "Klusā okeāna vasaras laiks"}, {"Anadyr", "Anadiras vasaras laiks"}, {"Apia", "Apijas vasaras laiks"}, {"Arabian", "Arābijas pussalas vasaras laiks"}, {"Argentina", "Argentīnas vasaras laiks"}, {"Argentina_Western", "Rietumargentīnas vasaras laiks"}, {"Armenia", "Armēnijas vasaras laiks"}, {"Atlantic", "Atlantijas vasaras laiks"}, {"Australia_Central", "Austrālijas centrālais vasaras laiks"}, {"Australia_CentralWestern", "Austrālijas centrālais rietumu vasaras laiks"}, {"Australia_Eastern", "Austrālijas austrumu vasaras laiks"}, {"Australia_Western", "Austrālijas rietumu vasaras laiks"}, {"Azerbaijan", "Azerbaidžānas vasaras laiks"}, {"Azores", "Azoru salu vasaras laiks"}, {"Bangladesh", "Bangladešas vasaras laiks"}, {"Brasilia", "Brazīlijas vasaras laiks"}, {"Cape_Verde", "Kaboverdes vasaras laiks"}, {"Chatham", "Četemas vasaras laiks"}, {"Chile", "Čīles vasaras laiks"}, {"China", "Ķīnas vasaras laiks"}, {"Colombia", "Kolumbijas vasaras laiks"}, {"Cook", "Kuka salu vasaras laiks"}, {"Cuba", "Kubas vasaras laiks"}, {"Easter", "Lieldienu salas vasaras laiks"}, {"Europe_Central", "Centrāleiropas vasaras laiks"}, {"Europe_Eastern", "Austrumeiropas vasaras laiks"}, {"Europe_Western", "Rietumeiropas vasaras laiks"}, {"Falkland", "Folklenda (Malvinu) salu vasaras laiks"}, {"Fiji", "Fidži vasaras laiks"}, {"Georgia", "Gruzijas vasaras laiks"}, {"Greenland_Eastern", "Austrumgrenlandes vasaras laiks"}, {"Greenland_Western", "Rietumgrenlandes vasaras laiks"}, {"Hawaii_Aleutian", "Havaju–Aleutu vasaras laiks"}, {"Hong_Kong", "Honkongas vasaras laiks"}, {"Hovd", "Hovdas vasaras laiks"}, {"Iran", "Irānas vasaras laiks"}, {"Irkutsk", "Irkutskas vasaras laiks"}, {"Israel", "Izraēlas vasaras laiks"}, {"Japan", "Japānas vasaras laiks"}, {"Kamchatka", "Petropavlovskas-Kamčatskas vasaras laiks"}, {"Korea", "Korejas vasaras laiks"}, {"Krasnoyarsk", "Krasnojarskas vasaras laiks"}, {"Lord_Howe", "Lorda Hava salas vasaras laiks"}, {"Magadan", "Magadanas vasaras laiks"}, {"Mauritius", "Maurīcijas vasaras laiks"}, {"Mexico_Pacific", "Meksikas Klusā okeāna piekrastes vasaras laiks"}, {"Mongolia", "Ulanbatoras vasaras laiks"}, {"Moscow", "Maskavas vasaras laiks"}, {"New_Caledonia", "Jaunkaledonijas vasaras laiks"}, {"New_Zealand", "Jaunzēlandes vasaras laiks"}, {"Newfoundland", "Ņūfaundlendas vasaras laiks"}, {"Norfolk", "Norfolkas salas vasaras laiks"}, {"Noronha", "Fernandu di Noroņas vasaras laiks"}, {"Novosibirsk", "Novosibirskas vasaras laiks"}, {"Omsk", "Omskas vasaras laiks"}, {"Pakistan", "Pakistānas vasaras laiks"}, {"Paraguay", "Paragvajas vasaras laiks"}, {"Peru", "Peru vasaras laiks"}, {"Philippines", "Filipīnu vasaras laiks"}, {"Pierre_Miquelon", "Senpjēras un Mikelonas vasaras laiks"}, {"Sakhalin", "Sahalīnas vasaras laiks"}, {"Samara", "Samaras vasaras laiks"}, {"Samoa", "Samoa vasaras laiks"}, {"Taipei", "Taibei vasaras laiks"}, {"Tonga", "Tongas vasaras laiks"}, {"Turkey", "Turcijas vasaras laiks"}, {"Turkmenistan", "Turkmenistānas vasaras laiks"}, {"Uruguay", "Urugvajas vasaras laiks"}, {"Uzbekistan", "Uzbekistānas vasaras laiks"}, {"Vanuatu", "Vanuatu vasaras laiks"}, {"Vladivostok", "Vladivostokas vasaras laiks"}, {"Volgograd", "Volgogradas vasaras laiks"}, {"Yakutsk", "Jakutskas vasaras laiks"}, {"Yekaterinburg", "Jekaterinburgas vasaras laiks"}},
	},
	{
		{{"Afghanistan", "अफगानिस्तान टाइम"}, {"Africa_Central", "मध्य अफ्रीका टाइम"}, {"Africa_Eastern", "पूरब अफ्रीका टाइम"}, {"Africa_Southern", "दक्षिण अफ्रीका मानक टाइम"}, {"Africa_Western", "पश्चिम अफ्रीका टाइम"}, {"Alaska", "अलास्का मानक समय"}, {"Amazon", "अमेजन मानक टाइम"}, {"America_Central", "उत्तरी अमेरिकी केंद्रीय मानक समय"}, {"America_Eastern", "उत्तरी अमेरिकी पूर्वी मानक समय"}, {"America_Mountain", "उत्तरी अमेरिकी माउंटेन मानक समय"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांत मानक समय"}, {"Arabian", "अरेबियन मानक टाइम"}, {"Argentina", "अर्जेंटीना मानक टाइम"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना मानक टाइम"}, {"Armenia", "आर्मेनिया मानक टाइम"}, {"Atlantic", "अटलांटिक मानक समय"}, {"Australia_Central", "आस्ट्रेलियाई मध्य मानक टाइम"}, {"Australia_CentralWestern", "आस्ट्रेलियाई मध्य पश्चिमी मानक टाइम"}, {"Australia_Eastern", "आस्ट्रेलियाई पूरबी मानक टाइम"}, {"Australia_Western", "आस्ट्रेलियाई पश्चिमी मानक टाइम"}, {"Azerbaijan", "अजरबैजान मानक टाइम"}, {"Azores", "एजोर्स मानक टाइम"}, {"Bangladesh", "बंगलादेश मानक टाइम"}, {"Bhutan", "भूटान टाइम"}, {"Bolivia", "बोलीबिया टाइम"}, {"Brasilia", "ब्राजीलिया मानक टाइम"}, {"Brunei", "ब्रूनेई दारेसलाम टाइम"}, {"Cape_Verde", "केप बर्ड मानक टाइम"}, {"Chatham", "चेथम मानक टाइम"}, {"Chile", "चिली मानक टाइम"}, {"China", "चीनी मानत टाइम"}, {"Christmas", "क्रिसमस द्वीप टाइम"}, {"Cocos", "कोकोस द्वीप टाइम"}, {"Colombia", "कोलंबिया मानक टाइम"}, {"Cuba", "क्यूबा मानक टाइम"}, {"East_Timor", "पूरबी तिमोर टाइम"}, {"Easter", "ईस्टर आइलैंड मानक टाइम"}, {"Ecuador", "इक्वाडोर टाइम"}, {"Europe_Central", "मध्य यूरोपीय मानक समय"}, {"Europe_Eastern", "पूर्वी यूरोपीय मानक समय"}, {"Europe_Further_Eastern", "फरदर-ईस्टर्न यूरोपीयन टाइम"}, {"Europe_Western", "पश्चिमी यूरोपीय मानक समय"}, {"Falkland", "फाकलैंड आइलैंड मानक टाइम"}, {"French_Guiana", "फ्रेंच गुयाना टाइम"}, {"French_Southern", "फ़्रांसीसी दक्षिणी क्षेत्र आ अंटार्कटिक टाइम"}, {"GMT", "ग्रीनविच मीन टाइम"}, {"Galapagos", "गैलापैगोस टाइम"}, {"Georgia", "जार्जिया मानक टाइम"}, {"Greenland_Eastern", "पूरबी ग्रीनलैंड मानक टाइम"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड मानक टाइम"}, {"Gulf", "गल्फ मानक समय"}, {"Guyana", "गुयाना टाइम"}, {"Hawaii", "हवाई-एल्यूटियन मानक टाइम"}, {"Hawaii_Aleutian", "हवाई-एल्यूटियन मानक टाइम"}, {"Hong_Kong", "हांग कांग मानक टाइम"}, {"Hovd", "होब्द मानक टाइम"}, {"India", "भारतीय मानक समय"}, {"Indian_Ocean", "हिंद महासागर टाइम"}, {"Indochina", "इंडोचाइना टाइम"}, {"Indonesia_Central", "मध्य इंडोनेशिया टाइम"}, {"Indonesia_Eastern", "पूरबी इंडोनेशिया टाइम"}, {"Indonesia_Western", "पश्चिमी इंडोनेशिया टाइम"}, {"Iran", "ईरानमानक टाइम"}, {"Irkutsk", "इरकूत्स्क मानक टाइम"}, {"Israel", "इजरायल मानक टाइम"}, {"Japan", "जापान मानक टाइम"}, {"Kazakhstan_Eastern", "पूरब कजाखस्तान टाइम"}, {"Kazakhstan_Western", "पश्चिम कजाखस्तान टाइम"}, {"Korea", "कोरियन मानक टाइम"}, {"Krasnoyarsk", "क्रेस्नोयार्स्क मानक टाइम"}, {"Kyrgystan", "किर्गिस्तान"}, {"Lord_Howe", "लार्ड होबे मानक टाइम"}, {"Magadan", "मगादान मानक टाइम"}, {"Malaysia", "मलेशिया टाइम"}, {"Maldives", "मालदीव टाइम"}, {"Mauritius", "मारीशस मानक टाइम"}, {"Mexico_Pacific", "मैक्सिकन पेसिफिक मानक टाइम"}, {"Mongolia", "उलनबटोर मानक टाइम"}, {"Moscow", "मास्को मानक टाइम"}, {"Myanmar", "मयनमार टाइम"}, {"Nepal", "नेपाल टाइम"}, {"New_Caledonia", "न्यू केलैडोनिया मानक समय"}, {"New_Zealand", "न्यूजीलैंड मानक टाइम"}, {"Newfoundland", "न्यूफाउंडलैंड मानक टाइम"}, {"Norfolk", "फाकलैंड द्वीप मानक टाइम"}, {"Noronha", "फर्नेंडो डी नोरोन्हा मानक टाइम"}, {"Novosibirsk", "नोबोसिबिर्स्क मानक टाइम"}, {"Omsk", "ओम्स्क मानक टाइम"}, {"Pakistan", "पाकिस्तान मानक टाइम"}, {"Paraguay", "पराग्वे मानक टाइम"}, {"Peru", "पेरू मानक टाइम"}, {"Philippines", "फिलिपीन मानक टाइम"}, {"Pierre_Miquelon", "सेंट पिएरे आ मिक्वेलान मानक टाइम"}, {"Pyongyang", "प्योंगयांग टाइम"}, {"Reunion", "रियूनियन टाइम"}, {"Sakhalin", "सकालिन मानक टाइम"}, {"Samoa", "सामोआ मानक समय"}, {"Seychelles", "सेशेल्स टाइम"}, {"Singapore", "सिंगापुर मानक टाइम"}, {"South_Georgia", "दक्षिण जार्जिया टाइम"}, {"Suriname", "सुरीनाम टाइम"}, {"Taipei", "ताइपेई मानक टाइम"}, {"Tajikistan", "ताजिकिस्तान"}, {"Turkmenistan", "तुर्कमेनिस्तान मानक टाइम"}, {"Uruguay", "उरुग्वे मानक टाइम"}, {"Uzbekistan", "उजबेकिस्तान मानक टाइम"}, {"Venezuela", "बेनेजुएला टाइम"}, {"Vladivostok", "ब्लादिबोस्तोक मानक टाइम"}, {"Volgograd", "बोल्गोग्राद मानक टाइम"}, {"Yakutsk", "यकुत्स्क मानक टाइम"}, {"Yekaterinburg", "येकाटैरिनबर्ग मानक टाइम"}, {"Yukon", "यूकोन टाइम"}},
		{{"Alaska", "अलास्का डेलाइट समय"}, {"Amazon", "अमेजन समर टाइम"}, {"America_Central", "उत्तरी अमेरिकी केंद्रीय डेलाइट समय"}, {"America_Eastern", "उत्तरी अमेरिकी पूर्वी डेलाइट समय"}, {"America_Mountain", "उत्तरी अमेरिकी माउंटेन डेलाइट समय"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांत डेलाइट समय"}, {"Arabian", "अरेबियन डेलाइट टाइम"}, {"Argentina", "अर्जेंटीना समर टाइम"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना समर टाइम"}, {"Armenia", "आर्मेनिया समर टाइम"}, {"Atlantic", "अटलांटिक डेलाइट समय"}, {"Australia_Central", "आस्ट्रेलियाई मध्य डेलाइट टाइम"}, {"Australia_CentralWestern", "आस्ट्रेलियाई मध्य पश्चिमी डेलाइट टाइम"}, {"Australia_Eastern", "आस्ट्रेलियाई पूरबी डेलाइट टाइम"}, {"Australia_Western", "आस्ट्रेलियाई पश्चिमी डेलाइट टाइम"}, {"Azerbaijan", "अजरबैजान समर टाइम"}, {"Azores", "एजोर्स समर टाइम"}, {"Bangladesh", "बंगलादेश समर टाइम"}, {"Brasilia", "ब्राजीलिया समर टाइम"}, {"Cape_Verde", "केप बर्ड समर टाइम"}, {"Chatham", "चेथम डेलाइट टाइम"}, {"Chile", "चिली समर टाइम"}, {"China", "चीनी डेलाइट टाइम"}, {"Colombia", "कोलंबिया समर टाइम"}, {"Cuba", "क्यूबा डेलाइट टाइम"}, {"Easter", "ईस्टर आइलैंड समर टाइम"}, {"Europe_Central", "मध्\u200dय यूरोपीय ग्रीष्\u200dमकालीन समय"}, {"Europe_Eastern", "पूर्वी यूरोपीय ग्रीष्मकालीन समय"}, {"Europe_Western", "पश्चिमी यूरोपीय ग्रीष्\u200dमकालीन समय"}, {"Falkland", "फाकलैंड आइलैंड समर टाइम"}, {"Georgia", "जार्जिया समर टाइम"}, {"Greenland_Eastern", "पूरबी ग्रीनलैंड समर टाइम"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड समर टाइम"}, {"Hawaii_Aleutian", "हवाई-एल्यूटियन डेलाइट टाइम"}, {"Hong_Kong", "हांग कांग समर टाइम"}, {"Hovd", "होब्द समर टाइम"}, {"Iran", "ईरान डेलाइट टाइम"}, {"Irkutsk", "इरकूत्स्क समर टाइम"}, {"Israel", "इजरायल डेलाइट टाइम"}, {"Japan", "जापान डेलाइट टाइम"}, {"Korea", "कोरियन डेलाइट टाइम"}, {"Krasnoyarsk", "क्रेस्नोयार्स्क समर टाइम"}, {"Lord_Howe", "लार्ड होबे डेलाइट टाइम"}, {"Magadan", "मगादान समर टाइम"}, {"Mauritius", "मारीशस समर टाइम"}, {"Mexico_Pacific", "मैक्सिकन पेसिफिक डेलाइट टाइम"}, {"Mongolia", "उलनबटोर समर टाइम"}, {"Moscow", "मास्को समर टाइम"}, {"New_Caledonia", "न्यू केलैडोनिया डेलाइट समय"}, {"New_Zealand", "न्यूजीलैंड डेलाइट टाइम"}, {"Newfoundland", "न्यूफाउंडलैंड डेलाइट टाइम"}, {"Norfolk", "फाकलैंड द्वीप डेलाइट टाइम"}, {"Noronha", "फर्नेंडो डी नोरोन्हा समर टाइम"}, {"Novosibirsk", "नोबोसिबिर्स्क समर टाइम"}, {"Omsk", "ओम्स्क समर टाइम"}, {"Pakistan", "पाकिस्तान समर टाइम"}, {"Paraguay", "पराग्वे समर टाइम"}, {"Peru", "पेरू समर टाइम"}, {"Philippines", "फिलिपीन समर टाइम"}, {"Pierre_Miquelon", "सेंट पिएरे आ मिक्वेलान डेलाइट टाइम"}, {"Sakhalin", "सकालिन समर टाइम"}, {"Samoa", "सामोआ समर समय"}, {"Taipei", "ताइपेई डेलाइट टाइम"}, {"Turkmenistan", "तुर्कमेनिस्तान समर टाइम"}, {"Uruguay", "उरुग्वे समर टाइम"}, {"Uzbekistan", "उजबेकिस्तान समर टाइम"}, {"Vladivostok", "ब्लादिबोस्तोक समर टाइम"}, {"Volgograd", "बोल्गोग्राद समर टाइम"}, {"Yakutsk", "यकुत्स्क समर टाइम"}, {"Yekaterinburg", "येकाटैरिनबर्ग समर टाइम"}},
	},
	{
		{{"Afghanistan", "Wā Awhekenetāna"}, {"Africa_Central", "Wā o Te Puku o Āwherika"}, {"Africa_Eastern", "Wā o Āwherika ki te rāwhiti"}, {"Africa_Southern", "Wā Arowhānui o Āwherika ki te tonga"}, {"Africa_Western", "Wā o Āwherika ki te uru"}, {"Alaska", "Wā Arowhānui Alaska"}, {"Amazon", "Wā Amahona Arowhānui"}, {"America_Central", "Wā Arowhānui Waenga"}, {"America_Eastern", "Wā Arowhānui Rāwhiti"}, {"America_Mountain", "Wā Arowhānui Maunga"}, {"America_Pacific", "Wā Arowhānui Kiwa"}, {"Apia", "Wā Āpia Arowhānui"}, {"Arabian", "Wā Arāpia Arowhānui"}, {"Argentina", "Wā Āketina Arowhānui"}, {"Argentina_Western", "Wā Āketina ki te uru Arowhānui"}, {"Armenia", "Wā Āmenia Arowhānui"}, {"Atlantic", "Wā Arowhānui Ranatiki"}, {"Australia_Central", "Wā Ahitereiria Waenga Arowhānui"}, {"Australia_CentralWestern", "Wā Ahitereiria Waenga-Uru Arowhānui"}, {"Australia_Eastern", "Wā Ahitereiria ki te Rāwhiti Arowhānui"}, {"Australia_Western", "Wā Ahitereiria ki te Uru Arowhānui"}, {"Azerbaijan", "Wā Atepaihānia Arowhānui"}, {"Azores", "Wā Azores Arowhānui"}, {"Bangladesh", "Wā Pākaratēhi Arowhānui"}, {"Bhutan", "Wā Pūtana"}, {"Bolivia", "Wā Poriwia"}, {"Brasilia", "Wā Parīhia Arowhānui"}, {"Brunei", "Wā Poronai Darussalam"}, {"Cape_Verde", "Wā Arowhānui o Te Kūrae Matomato"}, {"Chamorro", "Wā Chamorro Arowhānui"}, {"Chatham", "Wā Rēkohu Arowhānui"}, {"Chile", "Wā Hiri Arowhānui"}, {"China", "Wā Haina Arowhānui"}, {"Christmas", "Wā o Te Moutere Kirihimete"}, {"Cocos", "Wā o Ngā Moutere Kokohi"}, {"Colombia", "Wā Koromōpia Arowhānui"}, {"Cook", "Wā Kuki Airani Arowhānui"}, {"Cuba", "Wā Arowhānui Kiupa"}, {"Davis", "Wā Rēweti"}, {"DumontDUrville", "Wā Dumont-d’Urville"}, {"East_Timor", "Wā o Timoa ki te Rāwhiti"}, {"Easter", "Wā ki te Moutere Aranga Arowhānui"}, {"Ecuador", "Wā Ekuatoa"}, {"Europe_Central", "Wā Arowhānui Uropi Waenga"}, {"Europe_Eastern", "Wā Arowhānui Uropi Rāwhiti"}, {"Europe_Further_Eastern", "Wā Ūropi ki te rāwhiti rawa"}, {"Europe_Western", "Wā Arowhānui Uropi Uru"}, {"Falkland", "Wā ki Ngā Motu Whākana Arowhānui"}, {"Fiji", "Wā Whītī Arowhānui"}, {"French_Guiana", "Wā Kiāna Wīwī"}, {"French_Southern", "Wā Wīwī o Te Tonga me te Kōpakatanga ki te Tonga"}, {"GMT", "Wā Toharite Kiriwīti"}, {"Galapagos", "Wā Galapagos"}, {"Gambier", "Wā Gambier"}, {"Georgia", "Wā Hōria Arowhānui"}, {"Gilbert_Islands", "Wā Kiripati"}, {"Greenland_Eastern", "Wā Arowhānui o Whenuakāriki ki te rāwhiti"}, {"Greenland_Western", "Wā Arowhānui o Whenuakāriki ki te uru"}, {"Gulf", "Wā Whanga Arowhānui"}, {"Guyana", "Wā Kaiana"}, {"Hawaii", "Wā Arowhānui Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Wā Arowhānui Hawaii-Aleutian"}, {"Hong_Kong", "Wā Hongipua Arowhānui"}, {"Hovd", "Wā Hovd Arowhānui"}, {"India", "Wā Īnia"}, {"Indian_Ocean", "Wā o Te Moana Īnia"}, {"Indochina", "Wā Īniahaina"}, {"Indonesia_Central", "Wā Initonīhia Waenga"}, {"Indonesia_Eastern", "Wā Initonīhia ki te rāwhiti"}, {"Indonesia_Western", "Wā Initonīhia ki te uru"}, {"Iran", "Wā Irāna Arowhānui"}, {"Irkutsk", "Wā Irkutsk Arowhānui"}, {"Israel", "Wā Iharaira Arowhānui"}, {"Japan", "Wā Hapani Arowhānui"}, {"Kazakhstan", "Wā Katatānga"}, {"Kazakhstan_Eastern", "Wā Katatānga ki te Rāwhiti"}, {"Kazakhstan_Western", "Wā Katatānga ki te Uru"}, {"Korea", "Wā Kōrea Arowhānui"}, {"Kosrae", "Wā Kosrae"}, {"Krasnoyarsk", "Wā Krasnoyarsk Arowhānui"}, {"Kyrgystan", "Wā Kikitānga"}, {"Line_Islands", "Wā o Ngā Mouter o Te Raina"}, {"Lord_Howe", "Wā Lord Howe Arowhānui"}, {"Magadan", "Wā Magadan Arowhānui"}, {"Malaysia", "Wā Mareia"}, {"Maldives", "Wā Māratiri"}, {"Marquesas", "Wā Marquesas"}, {"Marshall_Islands", "Wā o Ngā Motu Māhara"}, {"Mauritius", "Wā Marihi Arowhānui"}, {"Mawson", "Wā Mawson"}, {"Mexico_Pacific", "Wā Arowhānui Mēhiko Kiwa"}, {"Mongolia", "Wā Ulaanbaatar Arowhānui"}, {"Moscow", "Wā Mohikau Arowhānui"}, {"Myanmar", "Wā Pēma"}, {"Nauru", "Wā Nauru"}, {"Nepal", "Wā Nepōra"}, {"New_Caledonia", "Wā Whenua Kanaki Arowhānui"}, {"New_Zealand", "Wā Aotearoa Arowhānui"}, {"Newfoundland", "Wā Arowhānui Newfoundland"}, {"Niue", "Wā Niue"}, {"Norfolk", "Wā o Te Moutere Nōpoke Arowhānui"}, {"Noronha", "Wā Fernando de Noronha Arowhānui"}, {"Novosibirsk", "Wā Novosibirsk Arowhānui"}, {"Omsk", "Wā Omsk Arowhānui"}, {"Pakistan", "Wā Pakitāne Arowhānui"}, {"Palau", "Wā Pārau"}, {"Papua_New_Guinea", "Wā Papua Nūkini"}, {"Paraguay", "Wā Parakai Arowhānui"}, {"Peru", "Wā Peru Arowhānui"}, {"Philippines", "Wā Piripīni Arowhānui"}, {"Phoenix_Islands", "Wā o Ngā Moutere Phoenix"}, {"Pierre_Miquelon", "Wā Arowhānui o St. Pierre me Miquelon"}, {"Pitcairn", "Wā Pitcairn"}, {"Ponape", "Wā Ponape"}, {"Pyongyang", "Wā Pyongyang"}, {"Reunion", "Wā Reunion"}, {"Rothera", "Wā Rothera"}, {"Sakhalin", "Wā Sakhalin Arowhānui"}, {"Samoa", "Wā Hāmoa Arowhānui"}, {"Seychelles", "Wā Heikere"}, {"Singapore", "Wā Hingapoa Arowhānui"}, {"Solomon", "Wā o Ngā Motu Horomona"}, {"South_Georgia", "Wā Hōria ki te Tonga"}, {"Suriname", "Wā Huriname"}, {"Syowa", "Wā Syowa"}, {"Tahiti", "Wā Tahiti"}, {"Taipei", "Wā Taipei Arowhānui"}, {"Tajikistan", "Wā Takiritānga"}, {"Tokelau", "Wā Tokerau"}, {"Tonga", "Wā Tonga Arowhānui"}, {"Truk", "Wā Chuuk"}, {"Turkmenistan", "Wā Tukumanatānga Arowhānui"}, {"Tuvalu", "Wā Tūwaru"}, {"Uruguay", "Wā Urukoi Arowhānui"}, {"Uzbekistan", "Wā Uhipeketāne Arowhānui"}, {"Vanuatu", "Wā Whenuatū Arowhānui"}, {"Venezuela", "Wā Penehūera"}, {"Vladivostok", "Wā Vladivostok Arowhānui"}, {"Volgograd", "Wā Volgograd Arowhānui"}, {"Vostok", "Wā Vostok"}, {"Wake", "Wā o Te Motu Wake"}, {"Wallis", "Wā Wārihi me Whutuna"}, {"Yakutsk", "Wā Yakutsk Arowhānui"}, {"Yekaterinburg", "Wā Yekaterinburg Arowhānui"}, {"Yukon", "Wā Yukon"}},
		{{"Alaska", "Wā Awatea Alaska"}, {"Amazon", "Wā Amahona Raumati"}, {"America_Central", "Wā Awatea Waenga"}, {"America_Eastern", "Wā Awatea Rāwhiti"}, {"America_Mountain", "Wā Awatea Maunga"}, {"America_Pacific", "Wā Awatea Kiwa"}, {"Apia", "Wā Āpia Awatea"}, {"Arabian", "Wā Arāpia Awatea"}, {"Argentina", "Wā Āketina Raumati"}, {"Argentina_Western", "Wā Āketina ki te uru Raumati"}, {"Armenia", "Wā Āmenia Raumati"}, {"Atlantic", "Wā Awatea Ranatiki"}, {"Australia_Central", "Wā Ahitereiria Waenga Awatea"}, {"Australia_CentralWestern", "Wā Ahitereiria Waenga-Uru Awatea"}, {"Australia_Eastern", "Wā Ahitereiria ki te Rāwhiti Awatea"}, {"Australia_Western", "Wā Ahitereiria ki te Uru Awatea"}, {"Azerbaijan", "Wā Atepaihānia Raumati"}, {"Azores", "Wā Azores Raumati"}, {"Bangladesh", "Wā Pākaratēhi Raumati"}, {"Brasilia", "Wā Parīhia Raumati"}, {"Cape_Verde", "Wā Raumati o Te Kūrae Matomato"}, {"Chatham", "Wā Rēkohu Awatea"}, {"Chile", "Wā Hiri Raumati"}, {"China", "Wā Haina Awatea"}, {"Colombia", "Wā Koromōpia Raumati"}, {"Cook", "Wā Kuki Airani Raumati Haurua"}, {"Cuba", "Wā Awatea Kiupa"}, {"Easter", "Wā ki te Moutere Aranga Raumati"}, {"Europe_Central", "Wā Raumati Uropi Waenga"}, {"Europe_Eastern", "Wā Raumati Uropi Rāwhiti"}, {"Europe_Western", "Wā Raumati Uropi Uru"}, {"Falkland", "Wā ki Ngā Motu Whākana Raumati"}, {"Fiji", "Wā Whītī Raumati"}, {"Georgia", "Wā Hōria Raumati"}, {"Greenland_Eastern", "Wā Raumati o Whenuakāriki ki te rāwhiti"}, {"Greenland_Western", "Wā Raumati o Whenuakāriki ki te uru"}, {"Hawaii_Aleutian", "Wā Awatea Hawaii-Aleutian"}, {"Hong_Kong", "Wā Hongipua Raumati"}, {"Hovd", "Wā Hovd Raumati"}, {"Iran", "Wā Irāna Awatea"}, {"Irkutsk", "Wā Irkutsk Raumati"}, {"Israel", "Wā Iharaira Awatea"}, {"Japan", "Wā Hapani Awatea"}, {"Korea", "Wā Kōrea Awatea"}, {"Krasnoyarsk", "Wā Krasnoyarsk Raumati"}, {"Lord_Howe", "Wā Lord Howe Awatea"}, {"Magadan", "Wā Magadan Raumati"}, {"Mauritius", "Wā Marihi Raumati"}, {"Mexico_Pacific", "Wā Awatea Mēhiko Kiwa"}, {"Mongolia", "Wā Ulaanbaatar Raumati"}, {"Moscow", "Wā Mohikau Raumati"}, {"New_Caledonia", "Wā Whenua Kanaki Raumati"}, {"New_Zealand", "Wā Aotearoa Awatea"}, {"Newfoundland", "Wā Awatea Newfoundland"}, {"Norfolk", "Wā o Te Moutere Nōpoke Awatea"}, {"Noronha", "Wā Fernando de Noronha Raumati"}, {"Novosibirsk", "Wā Novosibirsk Raumati"}, {"Omsk", "Wā Omsk Raumati"}, {"Pakistan", "Wā Pakitāne Raumati"}, {"Paraguay", "Wā Parakai Raumati"}, {"Peru", "Wā Peru Raumati"}, {"Philippines", "Wā Piripīni Raumati"}, {"Pierre_Miquelon", "Wā Awatea o St. Pierre me Miquelon"}, {"Sakhalin", "Wā Sakhalin Raumati"}, {"Samoa", "Wā Hāmoa Awatea"}, {"Taipei", "Wā Taipei Awatea"}, {"Tonga", "Wā Tonga Raumati"}, {"Turkmenistan", "Wā Tukumanatānga Raumati"}, {"Uruguay", "Wā Urukoi Raumati"}, {"Uzbekistan", "Wā Uhipeketāne Raumati"}, {"Vanuatu", "Wā Whenuatū Raumati"}, {"Vladivostok", "Wā Vladivostok Raumati"}, {"Volgograd", "Wā Volgograd Raumati"}, {"Yakutsk", "Wā Yakutsk Raumati"}, {"Yekaterinburg", "Wā Yekaterinburg Raumati"}},
	},
	{
		{{"Acre", "Акре стандардно време"}, {"Afghanistan", "Време во Авганистан"}, {"Africa_Central", "Средноафриканско време"}, {"Africa_Eastern", "Источноафриканско време"}, {"Africa_Southern", "Време во Јужноафриканска Република"}, {"Africa_Western", "Западноафриканско време"}, {"Alaska", "Стандардно време во Алјаска"}, {"Amazon", "Стандардно време во Амазон"}, {"America_Central", "Централно стандардно време во Северна Америка"}, {"America_Eastern", "Источно стандардно време во Северна Америка"}, {"America_Mountain", "Планинско стандардно време во Северна Америка"}, {"America_Pacific", "Пацифичко стандардно време во Северна Америка"}, {"Anadyr", "Анадирско стандардно време"}, {"Apia", "Стандардно време во Апија"}, {"Arabian", "Стандардно арапско време"}, {"Argentina", "Стандардно време во Аргентина"}, {"Argentina_Western", "Стандардно време во западна Аргентина"}, {"Armenia", "Стандардно време во Ерменија"}, {"Atlantic", "Атлантско стандардно време"}, {"Australia_Central", "Стандардно време во Централна Австралија"}, {"Australia_CentralWestern", "Стандардно време во Централна и Западна Австралија"}, {"Australia_Eastern", "Стандардно време во Источна Австралија"}, {"Australia_Western", "Стандардно време во Западна Австралија"}, {"Azerbaijan", "Стандардно време во Азербејџан"}, {"Azores", "Стандардно време во Азорски Острови"}, {"Bangladesh", "Стандардно време во Бангладеш"}, {"Bhutan", "Време во Бутан"}, {"Bolivia", "Време во Боливија"}, {"Brasilia", "Стандардно време во Бразилија"}, {"Brunei", "Време во Брунеј Дарусалам"}, {"Cape_Verde", "Стандардно време во Кабо Верде"}, {"Chamorro", "Време во Чаморо"}, {"Chatham", "Стандардно време во Чатам"}, {"Chile", "Стандардно време во Чиле"}, {"China", "Стандардно време во Кина"}, {"Christmas", "Време во Божиќен Остров"}, {"Cocos", "Време во Кокосови Острови"}, {"Colombia", "Стандардно време во Колумбија"}, {"Cook", "Стандардно време во Кукови Острови"}, {"Cuba", "Стандардно време во Куба"}, {"Davis", "Време во Дејвис"}, {"DumontDUrville", "Време во Димон Дирвил"}, {"East_Timor", "Време во Источен Тимор"}, {"Easter", "Стандардно време во Велигденски Остров"}, {"Ecuador", "Време во Еквадор"}, {"Europe_Central", "Средноевропско стандардно време"}, {"Europe_Eastern", "Источноевропско стандардно време"}, {"Europe_Further_Eastern", "Калининградско време"}, {"Europe_Western", "Западноевропско стандардно време"}, {"Falkland", "Стандардно време во Фолкландски Острови"}, {"Fiji", "Стандардно време во Фиџи"}, {"French_Guiana", "Време во Француска Гвајана"}, {"French_Southern", "Француско јужно и антарктичко време"}, {"GMT", "Средно време по Гринич"}, {"Galapagos", "Време во Галапагос"}, {"Gambier", "Време во Гамбе"}, {"Georgia", "Стандардно време во Грузија"}, {"Gilbert_Islands", "Време во Гилбертови Острови"}, {"Greenland_Eastern", "Стандардно време во Источен Гренланд"}, {"Greenland_Western", "Стандардно време во Западен Гренланд"}, {"Gulf", "Време во Персиски Залив"}, {"Guyana", "Време во Гвајана"}, {"Hawaii", "Стандардно време во Хаваи - Алеутски острови"}, {"Hawaii_Aleutian", "Стандардно време во Хаваи - Алеутски острови"}, {"Hong_Kong", "Стандардно време во Хонг Конг"}, {"Hovd", "Стандардно време во Ховд"}, {"India", "Време во Индија"}, {"Indian_Ocean", "Време во Индиски океан"}, {"Indochina", "Време во Индокина"}, {"Indonesia_Central", "Време во Централна Индонезија"}, {"Indonesia_Eastern", "Време во Источна Индонезија"}, {"Indonesia_Western", "Време во Западна Индонезија"}, {"Iran", "Стандардно време во Иран"}, {"Irkutsk", "Стандардно време во Иркутск"}, {"Israel", "Стандардно време во Израел"}, {"Japan", "Стандардно време во Јапонија"}, {"Kazakhstan", "Време во Казахстан"}, {"Kazakhstan_Eastern", "Време во Источен Казахстан"}, {"Kazakhstan_Western", "Време во Западен Казахстан"}, {"Korea", "Стандардно време во Кореја"}, {"Kosrae", "Време во Косра"}, {"Krasnoyarsk", "Стандардно време во Краснојарск"}, {"Kyrgystan", "Време во Киргистан"}, {"Line_Islands", "Време во Линиски Острови"}, {"Lord_Howe", "Стандардно време во Лорд Хау"}, {"Magadan", "Стандардно време во Магадан"}, {"Malaysia", "Време во Малезија"}, {"Maldives", "Време во Малдиви"}, {"Marquesas", "Време во Маркесас"}, {"Marshall_Islands", "Време во Маршалски Острови"}, {"Mauritius", "Стандардно време во Маврициус"}, {"Mawson", "Време во Мосон"}, {"Mexico_Pacific", "Стандардно пацифичко време во Мексико"}, {"Mongolia", "Стандардно време во Улан Батор"}, {"Moscow", "Стандардно време во Москва"}, {"Myanmar", "Време во Мјанмар"}, {"Nauru", "Време во Науру"}, {"Nepal", "Време во Непал"}, {"New_Caledonia", "Стандардно време во Нова Каледонија"}, {"New_Zealand", "Стандардно време во Нов Зеланд"}, {"Newfoundland", "Стандардно време во Њуфаундленд"}, {"Niue", "Време во Ниуе"}, {"Norfolk", "Стандардно време во Норфолшки Остров"}, {"Noronha", "Стандардно време во Фернандо де Нороња"}, {"Novosibirsk", "Стандардно време во Новосибирск"}, {"Omsk", "Стандардно време во Омск"}, {"Pakistan", "Стандардно време во Пакистан"}, {"Palau", "Време во Палау"}, {"Papua_New_Guinea", "Време во Папуа Нова Гвинеја"}, {"Paraguay", "Стандардно време во Парагвај"}, {"Peru", "Стандардно време во Перу"}, {"Philippines", "Стандардно време во Филипини"}, {"Phoenix_Islands", "Време во Островите Феникс"}, {"Pierre_Miquelon", "Стандардно време во Сент Пјер и Микелан"}, {"Pitcairn", "Време во Питкерн"}, {"Ponape", "Време во Понапе"}, {"Pyongyang", "Време во Пјонгјанг"}, {"Reunion", "Време во Рејунион"}, {"Rothera", "Време во Ротера"}, {"Sakhalin", "Стандардно време во Сакалин"}, {"Samara", "Самара стандардно време"}, {"Samoa", "Стандардно време во Самоа"}, {"Seychelles", "Време во Сејшели"}, {"Singapore", "Време во Сингапур"}, {"Solomon", "Време во Соломонски Острови"}, {"South_Georgia", "Време во Јужна Џорџија"}, {"Suriname", "Време во Суринам"}, {"Syowa", "Време во Сајова"}, {"Tahiti", "Време во Тахити"}, {"Taipei", "Стандардно време во Тајпеј"}, {"Tajikistan", "Време во Таџикистан"}, {"Tokelau", "Време во Токелау"}, {"Tonga", "Стандардно време во Тонга"}, {"Truk", "Време во Чуук"}, {"Turkey", "Турција стандардно време"}, {"Turkmenistan", "Стандардно време во Туркменистан"}, {"Tuvalu", "Време во Тувалу"}, {"Uruguay", "Стандардно време во Уругвај"}, {"Uzbekistan", "Стандардно време во Узбекистан"}, {"Vanuatu", "Стандардно време во Вануату"}, {"Venezuela", "Време во Венецуела"}, {"Vladivostok", "Стандардно време во Владивосток"}, {"Volgograd", "Стандардно време во Волгоград"}, {"Vostok", "Време во Восток"}, {"Wake", "Време во Островот Вејк"}, {"Wallis", "Време во Валис и Футуна"}, {"Yakutsk", "Стандардно време во Јакутск"}, {"Yekaterinburg", "Стандардно време во Екатеринбург"}, {"Yukon", "Време во Јукон"}},
		{{"Acre", "Акре летно сметање на времето"}, {"Alaska", "Летно сметање на времето во Алјаска"}, {"Amazon", "Летно сметање на времето во Амазон"}, {"America_Central", "Централно летно сметање на времето во Северна Америка"}, {"America_Eastern", "Источно летно сметање на времето во Северна Америка"}, {"America_Mountain", "Планинско летно сметање на времето во Северна Америка"}, {"America_Pacific", "Пацифичко летно сметање на времето во Северна Америка"}, {"Anadyr", "Анадирско летно време"}, {"Apia", "Летно време во Апија"}, {"Arabian", "Арапско летно сметање на времето"}, {"Argentina", "Летно сметање на времето во Аргентина"}, {"Argentina_Western", "Летно сметање на времето во западна Аргентина"}, {"Armenia", "Летно време во Ерменија"}, {"Atlantic", "Атлантско летно сметање на времето"}, {"Australia_Central", "Летно сметање на времето во Централна Австралија"}, {"Australia_CentralWestern", "Летно сметање на времето во Централна и Западна Австралија"}, {"Australia_Eastern", "Летно сметање на времето во Источна Австралија"}, {"Australia_Western", "Летно сметање на времето во Западна Австралија"}, {"Azerbaijan", "Летно време во Азербејџан"}, {"Azores", "Летно време во Азорски Острови"}, {"Bangladesh", "Летно време во Бангладеш"}, {"Brasilia", "Летно сметање на времето во Бразилија"}, {"Cape_Verde", "Летно сметање на времето во Кабо Верде"}, {"Chatham", "Летно сметање на времето во Чатам"}, {"Chile", "Летно сметање на времето во Чиле"}, {"China", "Летно сметање на времето во Кина"}, {"Colombia", "Летно сметање на времето во Колумбија"}, {"Cook", "Летно време во Кукови Острови"}, {"Cuba", "Летно сметање на времето во Куба"}, {"Easter", "Летно време во Велигденски Остров"}, {"Europe_Central", "Средноевропско летно време"}, {"Europe_Eastern", "Источноевропско летно време"}, {"Europe_Western", "Западноевропско летно време"}, {"Falkland", "Летно сметање на времето во Фолкландски Острови"}, {"Fiji", "Летно време во Фиџи"}, {"Georgia", "Летно време во Грузија"}, {"Greenland_Eastern", "Летно сметање на времето во Источен Гренланд"}, {"Greenland_Western", "Летно сметање на времето во Западен Гренланд"}, {"Hawaii_Aleutian", "Летно сметање на времето во Хаваи - Алеутски острови"}, {"Hong_Kong", "Летно време во Хонг Конг"}, {"Hovd", "Летно време во Ховд"}, {"Iran", "Летно сметање на времето во Иран"}, {"Irkutsk", "Летно време во Иркутск"}, {"Israel", "Летно сметање на времето во Израел"}, {"Japan", "Летно сметање на времето во Јапонија"}, {"Korea", "Летно сметање на времето во Кореја"}, {"Krasnoyarsk", "Летно време во Краснојарск"}, {"Lord_Howe", "Летно сметање на времето во Лорд Хау"}, {"Magadan", "Летно време во Магадан"}, {"Mauritius", "Летно сметање на времето во Маврициус"}, {"Mexico_Pacific", "Летно пацифичко време во Мексико"}, {"Mongolia", "Летно време во Улан Батор"}, {"Moscow", "Летно сметање на времето во Москва"}, {"New_Caledonia", "Летно време во Нова Каледонија"}, {"New_Zealand", "Летно сметање на времето во Нов Зеланд"}, {"Newfoundland", "Летно сметање на времето во Њуфаундленд"}, {"Norfolk", "Летно сметање на времето во Норфолшки Остров"}, {"Noronha", "Летно сметање на времето во Фернандо де Нороња"}, {"Novosibirsk", "Летно време во Новосибирск"}, {"Omsk", "Летно време во Омск"}, {"Pakistan", "Летно време во Пакистан"}, {"Paraguay", "Летно сметање на времето во Парагвај"}, {"Peru", "Летно сметање на времето во Перу"}, {"Philippines", "Летно време во Филипини"}, {"Pierre_Miquelon", "Летно сметање на времето во Сент Пјер и Микелан"}, {"Sakhalin", "Летно време во Сакалин"}, {"Samara", "Самара летно сметање на времето"}, {"Samoa", "Летно време во Самоа"}, {"Taipei", "Летно сметање на времето во Тајпеј"}, {"Tonga", "Летно време во Тонга"}, {"Turkey", "Турција летно сметање на времето"}, {"Turkmenistan", "Летно време во Туркменистан"}, {"Uruguay", "Летно сметање на времето во Уругвај"}, {"Uzbekistan", "Летно време во Узбекистан"}, {"Vanuatu", "Летно време во Вануату"}, {"Vladivostok", "Летно време во Владивосток"}, {"Volgograd", "Летно сметање на времето во Волгоград"}, {"Yakutsk", "Летно време во Јакутск"}, {"Yekaterinburg", "Летно време во Екатеринбург"}},
	},
	{
		{{"Acre", "എയ്ക്കർ സ്റ്റാൻഡേർഡ് സമയം"}, {"Afghanistan", "അഫ്\u200cഗാനിസ്ഥാൻ സമയം"}, {"Africa_Central", "മധ്യ ആഫ്രിക്ക സമയം"}, {"Africa_Eastern", "കിഴക്കൻ ആഫ്രിക്ക സമയം"}, {"Africa_Southern", "ദക്ഷിണാഫ്രിക്ക സ്റ്റാൻഡേർഡ് സമയം"}, {"Africa_Western", "പടിഞ്ഞാറൻ ആഫ്രിക്ക സമയം"}, {"Alaska", "അലാസ്ക സ്റ്റാൻഡേർഡ് സമയം"}, {"Almaty", "അൽമതി സ്റ്റാൻഡേർഡ് സമയം"}, {"Amazon", "ആമസോൺ സ്റ്റാൻഡേർഡ് സമയം"}, {"America_Central", "സെൻട്രൽ സ്റ്റാൻഡേർഡ് സമയം"}, {"America_Eastern", "ഈസ്റ്റേൺ സ്റ്റാൻഡേർഡ് സമയം"}, {"America_Mountain", "മൗണ്ടൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"America_Pacific", "പസഫിക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Anadyr", "അനാഡിർ സ്റ്റാൻഡേർഡ് സമയം"}, {"Apia", "അപിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"Aqtau", "അഖ്തൌ സ്റ്റാൻഡേർഡ് സമയം"}, {"Aqtobe", "അഖ്തോബ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Arabian", "അറേബ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Argentina", "അർജന്റീന സ്റ്റാൻഡേർഡ് സമയം"}, {"Argentina_Western", "പടിഞ്ഞാറൻ അർജന്റീന സ്റ്റാൻഡേർഡ് സമയം"}, {"Armenia", "അർമേനിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"Atlantic", "അറ്റ്\u200cലാന്റിക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Australia_Central", "ഓസ്ട്രേലിയൻ സെൻട്രൽ സ്റ്റാൻഡേർഡ് സമയം"}, {"Australia_CentralWestern", "ഓസ്ട്രേലിയൻ സെൻട്രൽ പടിഞ്ഞാറൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Australia_Eastern", "ഓസ്\u200cട്രേലിയൻ കിഴക്കൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Australia_Western", "ഓസ്\u200cട്രേലിയൻ പടിഞ്ഞാറൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Azerbaijan", "അസർബൈജാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Azores", "അസോർസ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Bangladesh", "ബംഗ്ലാദേശ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Bhutan", "ഭൂട്ടാൻ സമയം"}, {"Bolivia", "ബൊളീവിയ സമയം"}, {"Brasilia", "ബ്രസീലിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"Brunei", "ബ്രൂണൈ ദാറുസ്സലാം സമയം"}, {"Cape_Verde", "കേപ് വെർദെ സ്റ്റാൻഡേർഡ് സമയം"}, {"Chamorro", "ചമോറോ സ്റ്റാൻഡേർഡ് സമയം"}, {"Chatham", "ചാത്തം സ്റ്റാൻഡേർഡ് സമയം"}, {"Chile", "ചിലി സ്റ്റാൻഡേർഡ് സമയം"}, {"China", "ചൈന സ്റ്റാൻഡേർഡ് സമയം"}, {"Christmas", "ക്രിസ്\u200cമസ് ദ്വീപ് സമയം"}, {"Cocos", "കൊക്കോസ് ദ്വീപുകൾ സമയം"}, {"Colombia", "കൊളംബിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"Cook", "കുക്ക് ദ്വീപുകൾ സ്റ്റാൻഡേർഡ് സമയം"}, {"Cuba", "ക്യൂബ സ്റ്റാൻഡേർഡ് സമയം"}, {"Davis", "ഡേവിസ് സമയം"}, {"DumontDUrville", "ഡുമോണ്ട് ഡി ഉർവില്ലെ സമയം"}, {"East_Timor", "കിഴക്കൻ തിമോർ സമയം"}, {"Easter", "ഈസ്റ്റർ ദ്വീപ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Ecuador", "ഇക്വഡോർ സമയം"}, {"Europe_Central", "സെൻട്രൽ യൂറോപ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Europe_Eastern", "കിഴക്കൻ യൂറോപ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Europe_Further_Eastern", "കിഴക്കേയറ്റത്തെ യൂറോപ്യൻ സമയം"}, {"Europe_Western", "പടിഞ്ഞാറൻ യൂറോപ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Falkland", "ഫാക്ക്\u200cലാൻഡ് ദ്വീപുകൾ സ്റ്റാൻഡേർഡ് സമയം"}, {"Fiji", "ഫിജി സ്റ്റാൻഡേർഡ് സമയം"}, {"French_Guiana", "ഫ്രഞ്ച് ഗയാന സമയം"}, {"French_Southern", "ഫ്രഞ്ച് സതേൺ, അന്റാർട്ടിക് സമയം"}, {"GMT", "ഗ്രീൻവിച്ച് മീൻ സമയം"}, {"Galapagos", "ഗാലപ്പഗോസ് സമയം"}, {"Gambier", "ഗാമ്പിയർ സമയം"}, {"Georgia", "ജോർജ്ജിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"Gilbert_Islands", "ഗിൽബേർട്ട് ദ്വീപുകൾ സമയം"}, {"Greenland_Eastern", "കിഴക്കൻ ഗ്രീൻലാൻഡ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Greenland_Western", "പടിഞ്ഞാറൻ ഗ്രീൻലാൻഡ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Guam", "ഗ്വാം സ്റ്റാൻഡേർഡ് സമയം"}, {"Gulf", "ഗൾഫ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Guyana", "ഗയാന സമയം"}, {"Hawaii", "ഹവായ്-അലൂഷ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Hawaii_Aleutian", "ഹവായ്-അലൂഷ്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Hong_Kong", "ഹോങ്കോങ്ങ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Hovd", "ഹോഡ് സ്റ്റാൻഡേർഡ് സമയം"}, {"India", "ഇന്ത്യൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Indian_Ocean", "ഇന്ത്യൻ മഹാസമുദ്ര സമയം"}, {"Indochina", "ഇൻഡോചൈന സമയം"}, {"Indonesia_Central", "മധ്യ ഇന്തോനേഷ്യ സമയം"}, {"Indonesia_Eastern", "കിഴക്കൻ ഇന്തോനേഷ്യ സമയം"}, {"Indonesia_Western", "പടിഞ്ഞാറൻ ഇന്തോനേഷ്യ സമയം"}, {"Iran", "ഇറാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Irkutsk", "ഇർകസ്ക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Israel", "ഇസ്രായേൽ സ്റ്റാൻഡേർഡ് സമയം"}, {"Japan", "ജപ്പാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Kamchatka", "പെട്രോപാവ്\u200cലോസ്ക് കംചാസ്കി സ്റ്റാൻഡേർഡ് സമയം"}, {"Kazakhstan", "കസാഖിസ്ഥാൻ സമയം"}, {"Kazakhstan_Eastern", "കിഴക്കൻ കസാഖിസ്ഥാൻ സമയം"}, {"Kazakhstan_Western", "പടിഞ്ഞാറൻ കസാഖിസ്ഥാൻ സമയം"}, {"Korea", "കൊറിയൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Kosrae", "കൊസ്ര സമയം"}, {"Krasnoyarsk", "ക്രാസ്\u200cനോയാർസ്\u200cക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Kyrgystan", "കിർഗിസ്ഥാൻ സമയം"}, {"Lanka", "ലങ്ക സമയം"}, {"Line_Islands", "ലൈൻ ദ്വീപുകൾ സമയം"}, {"Lord_Howe", "ലോർഡ് ഹോവ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Macau", "മകൌ സ്റ്റാൻഡേർഡ് സമയം"}, {"Magadan", "മഗാദൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Malaysia", "മലേഷ്യ സമയം"}, {"Maldives", "മാലിദ്വീപ് സമയം"}, {"Marquesas", "മർക്കസസ് സമയം"}, {"Marshall_Islands", "മാർഷൽ ദ്വീപുകൾ സമയം"}, {"Mauritius", "മൗറീഷ്യസ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Mawson", "മാസൺ സമയം"}, {"Mexico_Pacific", "മെക്\u200cസിക്കൻ പസഫിക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Mongolia", "ഉലാൻബാത്തർ സ്റ്റാൻഡേർഡ് സമയം"}, {"Moscow", "മോസ്കോ സ്റ്റാൻഡേർഡ് സമയം"}, {"Myanmar", "മ്യാൻമാർ സമയം"}, {"Nauru", "നൗറു സമയം"}, {"Nepal", "നേപ്പാൾ സമയം"}, {"New_Caledonia", "ന്യൂ കാലിഡോണിയ സ്റ്റാൻഡേർഡ് സമയം"}, {"New_Zealand", "ന്യൂസിലാൻഡ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Newfoundland", "ന്യൂഫൗണ്ട്\u200cലാന്റ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Niue", "ന്യൂയി സമയം"}, {"Norfolk", "നോർ\u200cഫോക്ക് ദ്വീപ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Noronha", "ഫെർണാഡോ ഡി നൊറോൻഹ സ്റ്റാൻഡേർഡ് സമയം"}, {"North_Mariana", "നോർത്ത് മറിയാനാ ദ്വീപുകൾ സമയം"}, {"Novosibirsk", "നോവോസിബിർസ്ക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Omsk", "ഓംസ്\u200cക്ക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Pakistan", "പാക്കിസ്ഥാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Palau", "പലാവു സമയം"}, {"Papua_New_Guinea", "പാപ്പുവ ന്യൂ ഗിനിയ സമയം"}, {"Paraguay", "പരാഗ്വേ സ്റ്റാൻഡേർഡ് സമയം"}, {"Peru", "പെറു സ്റ്റാൻഡേർഡ് സമയം"}, {"Philippines", "ഫിലിപ്പൈൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Phoenix_Islands", "ഫിനിക്\u200cസ് ദ്വീപ് സമയം"}, {"Pierre_Miquelon", "സെന്റ് പിയറി ആൻഡ് മിക്വലൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Pitcairn", "പിറ്റ്കേൻ സമയം"}, {"Ponape", "പൊനാപ്പ് സമയം"}, {"Pyongyang", "പ്യോംഗ്\u200cയാംഗ് സമയം"}, {"Qyzylorda", "ഖിസിലോർഡ സ്റ്റാൻഡേർഡ് സമയം"}, {"Reunion", "റീയൂണിയൻ സമയം"}, {"Rothera", "റോഥെറ സമയം"}, {"Sakhalin", "സഖാലിൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Samara", "സമാറ സ്റ്റാൻഡേർഡ് സമയം"}, {"Samoa", "സമോവ സ്റ്റാൻഡേർഡ് സമയം"}, {"Seychelles", "സീഷെൽസ് സമയം"}, {"Singapore", "സിംഗപ്പൂർ സ്റ്റാൻഡേർഡ് സമയം"}, {"Solomon", "സോളമൻ ദ്വീപുകൾ സമയം"}, {"South_Georgia", "ദക്ഷിണ ജോർജ്ജിയൻ സമയം"}, {"Suriname", "സുരിനെയിം സമയം"}, {"Syowa", "സയോവ സമയം"}, {"Tahiti", "താഹിതി സമയം"}, {"Taipei", "തായ്\u200cപെയ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Tajikistan", "താജിക്കിസ്ഥാൻ സമയം"}, {"Tokelau", "ടോക്കെലൂ സമയം"}, {"Tonga", "ടോംഗ സ്റ്റാൻഡേർഡ് സമയം"}, {"Truk", "ചൂക്ക് സമയം"}, {"Turkey", "തുർക്കി സ്റ്റാൻഡേർഡ് സമയം"}, {"Turkmenistan", "തുർക്ക്\u200cമെനിസ്ഥാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Tuvalu", "ടുവാലു സമയം"}, {"Uruguay", "ഉറുഗ്വേ സ്റ്റാൻഡേർഡ് സമയം"}, {"Uzbekistan", "ഉസ്\u200cബെക്കിസ്ഥാൻ സ്റ്റാൻഡേർഡ് സമയം"}, {"Vanuatu", "വന്വാതു സ്റ്റാൻഡേർഡ് സമയം"}, {"Venezuela", "വെനിസ്വേല സമയം"}, {"Vladivostok", "വ്ലാഡിവോസ്റ്റോക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Volgograd", "വോൾഗോഗ്രാഡ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Vostok", "വോസ്റ്റോക് സമയം"}, {"Wake", "വേക്ക് ദ്വീപ് സമയം"}, {"Wallis", "വാലിസ് ആന്റ് ഫ്യൂച്യുന സമയം"}, {"Yakutsk", "യാകസ്\u200cക്ക് സ്റ്റാൻഡേർഡ് സമയം"}, {"Yekaterinburg", "യെക്കാറ്റരിൻബർഗ് സ്റ്റാൻഡേർഡ് സമയം"}, {"Yukon", "യൂക്കോൺ സമയം"}},
		{{"Acre", "എയ്ക്കർ വേനൽക്കാല സമയം"}, {"Alaska", "അലാസ്\u200cക ഡേലൈറ്റ് സമയം"}, {"Almaty", "അൽമതി വേനൽക്കാല സമയം"}, {"Amazon", "ആമസോൺ ഗ്രീഷ്\u200cമകാല സമയം"}, {"America_Central", "സെൻട്രൽ ഡേലൈറ്റ് സമയം"}, {"America_Eastern", "ഈസ്റ്റേൺ ഡേലൈറ്റ് സമയം"}, {"America_Mountain", "മൗണ്ടൻ ഡേലൈറ്റ് സമയം"}, {"America_Pacific", "പസഫിക് ഡേലൈറ്റ് സമയം"}, {"Anadyr", "അനാഡിർ വേനൽക്കാല സമയം"}, {"Apia", "അപിയ ഡേലൈറ്റ് സമയം"}, {"Aqtau", "അഖ്തൌ വേനൽക്കാല സമയം"}, {"Aqtobe", "അഖ്തോബ് വേനൽക്കാല സമയം"}, {"Arabian", "അറേബ്യൻ ഡേലൈറ്റ് സമയം"}, {"Argentina", "അർജന്റീന ഗ്രീഷ്\u200cമകാല സമയം"}, {"Argentina_Western", "പടിഞ്ഞാറൻ അർജന്റീന ഗ്രീഷ്\u200cമകാല സമയം"}, {"Armenia", "അർമേനിയ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Atlantic", "അറ്റ്\u200cലാന്റിക് ഡേലൈറ്റ് സമയം"}, {"Australia_Central", "ഓസ്ട്രേലിയൻ സെൻട്രൽ ഡേലൈറ്റ് സമയം"}, {"Australia_CentralWestern", "ഓസ്ട്രേലിയൻ സെൻട്രൽ പടിഞ്ഞാറൻ ഡേലൈറ്റ് സമയം"}, {"Australia_Eastern", "ഓസ്\u200cട്രേലിയൻ കിഴക്കൻ ഡേലൈറ്റ് സമയം"}, {"Australia_Western", "ഓസ്\u200cട്രേലിയൻ പടിഞ്ഞാറൻ ഡേലൈറ്റ് സമയം"}, {"Azerbaijan", "അസർബൈജാൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Azores", "അസോർസ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Bangladesh", "ബംഗ്ലാദേശ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Brasilia", "ബ്രസീലിയ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Cape_Verde", "കേപ് വെർദെ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Chatham", "ചാത്തം ഗ്രീഷ്\u200cമകാല സമയം"}, {"Chile", "ചിലി ഗ്രീഷ്\u200cമകാല സമയം"}, {"China", "ചൈന ഡേലൈറ്റ് സമയം"}, {"Colombia", "കൊളംബിയ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Cook", "കുക്ക് ദ്വീപുകൾ അർദ്ധ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Cuba", "ക്യൂബ ഡേലൈറ്റ് സമയം"}, {"Easter", "ഈസ്റ്റർ ദ്വീപ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Europe_Central", "സെൻട്രൽ യൂറോപ്യൻ ഗ്രീഷ്മകാല സമയം"}, {"Europe_Eastern", "കിഴക്കൻ യൂറോപ്യൻ ഗ്രീഷ്മകാല സമയം"}, {"Europe_Western", "പടിഞ്ഞാറൻ യൂറോപ്യൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Falkland", "ഫാക്ക്\u200cലാൻഡ് ദ്വീപുകൾ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Fiji", "ഫിജി ഗ്രീഷ്\u200cമകാല സമയം"}, {"Georgia", "ജോർജ്ജിയ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Greenland_Eastern", "കിഴക്കൻ ഗ്രീൻലാൻഡ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Greenland_Western", "പടിഞ്ഞാറൻ ഗ്രീൻലാൻഡ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Hawaii_Aleutian", "ഹവായ്-അലൂഷ്യൻ ഡേലൈറ്റ് സമയം"}, {"Hong_Kong", "ഹോങ്കോങ്ങ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Hovd", "ഹോഡ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Iran", "ഇറാൻ ഡേലൈറ്റ് സമയം"}, {"Irkutsk", "ഇർകസ്\u200cക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Israel", "ഇസ്രായേൽ ഡേലൈറ്റ് സമയം"}, {"Japan", "ജപ്പാൻ ഡേലൈറ്റ് സമയം"}, {"Kamchatka", "പെട്രോപാവ്\u200cലോസ്ക് കംചാസ്കി വേനൽക്കാല സമയം"}, {"Korea", "കൊറിയൻ ഡേലൈറ്റ് സമയം"}, {"Krasnoyarsk", "ക്രാസ്\u200cനോയാർസ്\u200cക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Lord_Howe", "ലോർഡ് ഹോവ് ഡേലൈറ്റ് സമയം"}, {"Macau", "മകൌ വേനൽക്കാല സമയം"}, {"Magadan", "മഗാദൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Mauritius", "മൗറീഷ്യസ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Mexico_Pacific", "മെക്സിക്കൻ പസഫിക് ഡേലൈറ്റ് സമയം"}, {"Mongolia", "ഉലാൻബാത്തർ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Moscow", "മോസ്\u200cകോ ഗ്രീഷ്\u200cമകാല സമയം"}, {"New_Caledonia", "ന്യൂ കാലിഡോണിയ ഗ്രീഷ്\u200cമകാല സമയം"}, {"New_Zealand", "ന്യൂസിലാൻഡ് ഡേലൈറ്റ് സമയം"}, {"Newfoundland", "ന്യൂഫൗണ്ട്\u200cലാന്റ് ഡേലൈറ്റ് സമയം"}, {"Norfolk", "നോർ\u200cഫോക്ക് ദ്വീപ് ഡേലൈറ്റ് സമയം"}, {"Noronha", "ഫെർണാഡോ ഡി നൊറോൻഹ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Novosibirsk", "നോവോസിബിർസ്\u200cക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Omsk", "ഓംസ്\u200cക്ക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Pakistan", "പാക്കിസ്ഥാൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Paraguay", "പരാഗ്വേ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Peru", "പെറു ഗ്രീഷ്\u200cമകാല സമയം"}, {"Philippines", "ഫിലിപ്പൈൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Pierre_Miquelon", "സെന്റ് പിയറി ആൻഡ് മിക്വലൻ ഡേലൈറ്റ് സമയം"}, {"Qyzylorda", "ഖിസിലോർഡ വേനൽക്കാല സമയം"}, {"Sakhalin", "സഖാലിൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Samara", "സമാറ വേനൽക്കാല സമയം"}, {"Samoa", "സമോവാ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Taipei", "തായ്\u200cപെയ് ഡേലൈറ്റ് സമയം"}, {"Tonga", "ടോംഗ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Turkey", "തുർക്കി വേനൽക്കാല സമയം"}, {"Turkmenistan", "തുർക്ക്\u200cമെനിസ്ഥാൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Uruguay", "ഉറുഗ്വേ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Uzbekistan", "ഉസ്\u200cബെക്കിസ്ഥാൻ ഗ്രീഷ്\u200cമകാല സമയം"}, {"Vanuatu", "വന്വാതു ഗ്രീഷ്\u200cമകാല സമയം"}, {"Vladivostok", "വ്ലാഡിവോസ്റ്റോക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Volgograd", "വോൾഗോഗ്രാഡ് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Yakutsk", "യാകസ്\u200cക്ക് ഗ്രീഷ്\u200cമകാല സമയം"}, {"Yekaterinburg", "യെക്കാറ്റരിൻബർഗ് ഗ്രീഷ്\u200cമകാല സമയം"}},
	},
	{
		{{"Afghanistan", "Афганистаны цаг"}, {"Africa_Central", "Төв Африкийн цаг"}, {"Africa_Eastern", "Зүүн Африкийн цаг"}, {"Africa_Southern", "Өмнөд Африкийн стандарт цаг"}, {"Africa_Western", "Баруун Африкийн цаг"}, {"Alaska", "Аляскийн стандарт цаг"}, {"Amazon", "Амазоны стандарт цаг"}, {"America_Central", "Төв стандарт цаг"}, {"America_Eastern", "Зүүн эргийн стандарт цаг"}, {"America_Mountain", "Уулын стандарт цаг"}, {"America_Pacific", "Номхон далайн стандарт цаг"}, {"Apia", "Апиагийн стандарт цаг"}, {"Arabian", "Арабын стандарт цаг"}, {"Argentina", "Аргентины стандарт цаг"}, {"Argentina_Western", "Баруун Аргентины стандарт цаг"}, {"Armenia", "Арменийн стандарт цаг"}, {"Atlantic", "Атлантын стандарт цаг"}, {"Australia_Central", "Төв Австралийн стандарт цаг"}, {"Australia_CentralWestern", "Австралийн төв баруун эргийн стандарт цаг"}, {"Australia_Eastern", "Австралийн зүүн эргийн стандарт цаг"}, {"Australia_Western", "Австралийн баруун эргийн стандарт цаг"}, {"Azerbaijan", "Азербайжаны стандарт цаг"}, {"Azores", "Азорын стандарт цаг"}, {"Bangladesh", "Бангладешийн стандарт цаг"}, {"Bhutan", "Бутаны цаг"}, {"Bolivia", "Боливийн цаг"}, {"Brasilia", "Бразилийн стандарт цаг"}, {"Brunei", "Бруней Даруссаламын цаг"}, {"Cape_Verde", "Кабо-Вердийн стандарт цаг"}, {"Chamorro", "Чаморрогийн цаг"}, {"Chatham", "Чатемын стандарт цаг"}, {"Chile", "Чилийн стандарт цаг"}, {"China", "Хятадын стандарт цаг"}, {"Christmas", "Крисмас арлын цаг"}, {"Cocos", "Кокос арлын цаг"}, {"Colombia", "Колумбын стандарт цаг"}, {"Cook", "Күүкийн арлуудын стандарт цаг"}, {"Cuba", "Кубын стандарт цаг"}, {"Davis", "Дэвисийн цаг"}, {"DumontDUrville", "Дюмон д’Юрвилийн цаг"}, {"East_Timor", "Зүүн Тиморын цаг"}, {"Easter", "Зүүн Исландын стандарт цаг"}, {"Ecuador", "Эквадорын цаг"}, {"Europe_Central", "Төв Европын стандарт цаг"}, {"Europe_Eastern", "Зүүн Европын стандарт цаг"}, {"Europe_Further_Eastern", "Алс дорнод Европын цаг"}, {"Europe_Western", "Баруун Европын стандарт цаг"}, {"Falkland", "Фолклендийн арлуудын стандарт цаг"}, {"Fiji", "Фижигийн стандарт цаг"}, {"French_Guiana", "Францын Гвианагийн цаг"}, {"French_Southern", "Францын Өмнөд ба Антарктидийн цаг"}, {"GMT", "Гринвичийн цаг"}, {"Galapagos", "Галапагосын цаг"}, {"Gambier", "Гамбьегийн цаг"}, {"Georgia", "Гүржийн стандарт цаг"}, {"Gilbert_Islands", "Гильбертийн арлуудын цаг"}, {"Greenland_Eastern", "Зүүн Гренландын стандарт цаг"}, {"Greenland_Western", "Баруун Гренландын стандарт цаг"}, {"Gulf", "Персийн булангийн цаг"}, {"Guyana", "Гайанагийн цаг"}, {"Hawaii", "Хавай-Алеутын стандарт цаг"}, {"Hawaii_Aleutian", "Хавай-Алеутын стандарт цаг"}, {"Hong_Kong", "Хонг Конгийн стандарт цаг"}, {"Hovd", "Ховдын стандарт цаг"}, {"India", "Энэтхэгийн цаг"}, {"Indian_Ocean", "Энэтхэгийн далайн цаг"}, {"Indochina", "Энэтхэг-Хятадын хойгийн цаг"}, {"Indonesia_Central", "Төв Индонезийн цаг"}, {"Indonesia_Eastern", "Зүүн Индонезийн цаг"}, {"Indonesia_Western", "Баруун Индонезийн цаг"}, {"Iran", "Ираны стандарт цаг"}, {"Irkutsk", "Эрхүүгийн стандарт цаг"}, {"Israel", "Израилийн стандарт цаг"}, {"Japan", "Японы стандарт цаг"}, {"Kazakhstan", "Казахстаны цаг"}, {"Kazakhstan_Eastern", "Зүүн Казахстаны цаг"}, {"Kazakhstan_Western", "Баруун Казахстаны цаг"}, {"Korea", "Солонгосын стандарт цаг"}, {"Kosrae", "Косрэгийн цаг"}, {"Krasnoyarsk", "Красноярскийн стандарт цаг"}, {"Kyrgystan", "Киргизийн цаг"}, {"Line_Islands", "Лайн арлуудын цаг"}, {"Lord_Howe", "Лорд Хоугийн стандарт цаг"}, {"Magadan", "Магаданы стандарт цаг"}, {"Malaysia", "Малайзын цаг"}, {"Maldives", "Мальдивийн цаг"}, {"Marquesas", "Маркезын арлуудын цаг"}, {"Marshall_Islands", "Маршаллын арлуудын цаг"}, {"Mauritius", "Маврикийн стандарт цаг"}, {"Mawson", "Моусоны цаг"}, {"Mexico_Pacific", "Мексик-Номхон далайн стандарт цаг"}, {"Mongolia", "Улаанбаатарын стандарт цаг"}, {"Moscow", "Москвагийн стандарт цаг"}, {"Myanmar", "Мьянмарын цаг"}, {"Nauru", "Науругийн цаг"}, {"Nepal", "Балбын цаг"}, {"New_Caledonia", "Шинэ Каледонийн стандарт цаг"}, {"New_Zealand", "Шинэ Зеландын стандарт цаг"}, {"Newfoundland", "Нью-Фаундлендын стандарт цаг"}, {"Niue", "Ниуэгийн цаг"}, {"Norfolk", "Норфолк арлын стандарт цаг"}, {"Noronha", "Фернандо де Норонагийн стандарт цаг"}, {"Novosibirsk", "Новосибирскийн стандарт цаг"}, {"Omsk", "Омскийн стандарт цаг"}, {"Pakistan", "Пакистаны стандарт цаг"}, {"Palau", "Палаугийн цаг"}, {"Papua_New_Guinea", "Папуа Шинэ Гвинейн цаг"}, {"Paraguay", "Парагвайн стандарт цаг"}, {"Peru", "Перугийн стандарт цаг"}, {"Philippines", "Филиппиний стандарт цаг"}, {"Phoenix_Islands", "Феникс арлын цаг"}, {"Pierre_Miquelon", "Сент-Пьер ба Микелоны стандарт цаг"}, {"Pitcairn", "Питкернийн цаг"}, {"Ponape", "Понапегийн цаг"}, {"Pyongyang", "Пёньяны цаг"}, {"Reunion", "Реюнионы цаг"}, {"Rothera", "Ротерагийн цаг"}, {"Sakhalin", "Сахалины стандарт цаг"}, {"Samoa", "Самоагийн стандарт цаг"}, {"Seychelles", "Сейшелийн арлуудын цаг"}, {"Singapore", "Сингапурын цаг"}, {"Solomon", "Соломоны арлуудын цаг"}, {"South_Georgia", "Өмнөд Жоржиагийн цаг"}, {"Suriname", "Суринамын цаг"}, {"Syowa", "Сёвагийн цаг"}, {"Tahiti", "Таитигийн цаг"}, {"Taipei", "Тайпейн стандарт цаг"}, {"Tajikistan", "Тажикистаны цаг"}, {"Tokelau", "Токелаугийн цаг"}, {"Tonga", "Тонгагийн стандарт цаг"}, {"Truk", "Чүүкийн цаг"}, {"Turkmenistan", "Туркменистаны стандарт цаг"}, {"Tuvalu", "Тувалугийн цаг"}, {"Uruguay", "Уругвайн стандарт цаг"}, {"Uzbekistan", "Узбекистаны стандарт цаг"}, {"Vanuatu", "Вануатугийн стандарт цаг"}, {"Venezuela", "Венесуэлийн цаг"}, {"Vladivostok", "Владивостокийн стандарт цаг"}, {"Volgograd", "Волгоградын стандарт цаг"}, {"Vostok", "Востокийн цаг"}, {"Wake", "Уэкийн арлуудын цаг"}, {"Wallis", "Уоллис ба Футунагийн цаг"}, {"Yakutsk", "Якутын стандарт цаг"}, {"Yekaterinburg", "Екатеринбургийн стандарт цаг"}, {"Yukon", "Юкон цагийн бүс"}},
		{{"Alaska", "Аляскийн зуны цаг"}, {"Amazon", "Амазоны зуны цаг"}, {"America_Central", "Төв зуны цаг"}, {"America_Eastern", "Зүүн эргийн зуны цаг"}, {"America_Mountain", "Уулын зуны цаг"}, {"America_Pacific", "Номхон далайн зуны цаг"}, {"Apia", "Апиагийн зуны цаг"}, {"Arabian", "Арабын зуны цаг"}, {"Argentina", "Аргентины зуны цаг"}, {"Argentina_Western", "Баруун Аргентины зуны цаг"}, {"Armenia", "Арменийн зуны цаг"}, {"Atlantic", "Атлантын зуны цаг"}, {"Australia_Central", "Төв Австралийн зуны цаг"}, {"Australia_CentralWestern", "Австралийн төв баруун эргийн зуны цаг"}, {"Australia_Eastern", "Австралийн зүүн эргийн зуны цаг"}, {"Australia_Western", "Австралийн баруун эргийн зуны цаг"}, {"Azerbaijan", "Азербайжаны зуны цаг"}, {"Azores", "Азорын зуны цаг"}, {"Bangladesh", "Бангладешийн зуны цаг"}, {"Brasilia", "Бразилийн зуны цаг"}, {"Cape_Verde", "Кабо-Вердийн зуны цаг"}, {"Chatham", "Чатемын зуны цаг"}, {"Chile", "Чилийн зуны цаг"}, {"China", "Хятадын зуны цаг"}, {"Colombia", "Колумбын зуны цаг"}, {"Cook", "Күүкийн арлуудын хагас зуны цаг"}, {"Cuba", "Кубын зуны цаг"}, {"Easter", "Зүүн Исландын зуны цаг"}, {"Europe_Central", "Төв Европын зуны цаг"}, {"Europe_Eastern", "Зүүн Европын зуны цаг"}, {"Europe_Western", "Баруун Европын зуны цаг"}, {"Falkland", "Фолклендийн арлуудын зуны цаг"}, {"Fiji", "Фижигийн зуны цаг"}, {"Georgia", "Гүржийн зуны цаг"}, {"Greenland_Eastern", "Зүүн Гренландын зуны цаг"}, {"Greenland_Western", "Баруун Гренландын зуны цаг"}, {"Hawaii_Aleutian", "Хавай-Алеутын зуны цаг"}, {"Hong_Kong", "Хонг Конгийн зуны цаг"}, {"Hovd", "Ховдын зуны цаг"}, {"Iran", "Ираны зуны цаг"}, {"Irkutsk", "Эрхүүгийн зуны цаг"}, {"Israel", "Израилийн зуны цаг"}, {"Japan", "Японы зуны цаг"}, {"Korea", "Солонгосын зуны цаг"}, {"Krasnoyarsk", "Красноярскийн зуны цаг"}, {"Lord_Howe", "Лорд Хоугийн зуны цаг"}, {"Magadan", "Магаданы зуны цаг"}, {"Mauritius", "Маврикийн зуны цаг"}, {"Mexico_Pacific", "Мексик-Номхон далайн зуны цаг"}, {"Mongolia", "Улаанбаатарын зуны цаг"}, {"Moscow", "Москвагийн зуны цаг"}, {"New_Caledonia", "Шинэ Каледонийн зуны цаг"}, {"New_Zealand", "Шинэ Зеландын зуны цаг"}, {"Newfoundland", "Нью-Фаундлендын зуны цаг"}, {"Norfolk", "Норфолк арлын зуны цаг"}, {"Noronha", "Фернандо де Норонагийн зуны цаг"}, {"Novosibirsk", "Новосибирскийн зуны цаг"}, {"Omsk", "Омскийн зуны цаг"}, {"Pakistan", "Пакистаны зуны цаг"}, {"Paraguay", "Парагвайн зуны цаг"}, {"Peru", "Перугийн зуны цаг"}, {"Philippines", "Филиппиний зуны цаг"}, {"Pierre_Miquelon", "Сент-Пьер ба Микелоны зуны цаг"}, {"Sakhalin", "Сахалины зуны цаг"}, {"Samoa", "Самоагийн зуны цаг"}, {"Taipei", "Тайпейн зуны цаг"}, {"Tonga", "Тонгагийн зуны цаг"}, {"Turkmenistan", "Туркменистаны зуны цаг"}, {"Uruguay", "Уругвайн зуны цаг"}, {"Uzbekistan", "Узбекистаны зуны цаг"}, {"Vanuatu", "Вануатугийн зуны цаг"}, {"Vladivostok", "Владивостокийн зуны цаг"}, {"Volgograd", "Волгоградын зуны цаг"}, {"Yakutsk", "Якутын зуны цаг"}, {"Yekaterinburg", "Екатеринбургийн зуны цаг"}},
	},
	{
		{{"America_Central", "ᠳᠥᠪ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"America_Eastern", "ᠵᠡᠭᠥᠨ ᠡᠷᠭᠡ \u180eᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"America_Mountain", "ᠠᠭᠣᠯᠠ \u180e\u180e\u180e\u180eᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"America_Pacific", "ᠨᠣᠮᠣᠬᠠᠨ ᠳᠠᠯᠠᠢ \u180e\u180eᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"Atlantic", "ᠠᠲ᠋ᠯᠠᠨ᠋ᠲ \u180eᠤᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"Europe_Central", "ᠲᠥᠪ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"Europe_Eastern", "ᠵᠡᠭᠦᠨ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"Europe_Western", "ᠪᠠᠷᠠᠭᠣᠨ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠰᠲ᠋ᠠᠨ᠋ᠳᠠᠷᠳ᠋ ᠴᠠᠭ"}, {"GMT", "ᠭᠷᠢᠨ᠋ᠸᠢᠴᠢ ᠢᠢᠨ ᠴᠠᠭ"}},
		{{"America_Central", "ᠲᠥᠪ ᠵᠣᠨ \u180e\u180e\u180eᠤ ᠴᠠᠭ"}, {"America_Eastern", "ᠵᠡᠭᠥᠨ ᠡᠷᠭᠡ \u180eᠢᠢᠨ ᠵᠣᠨ \u180e\u180e\u180eᠤ ᠴᠠᠭ"}, {"America_Mountain", "ᠠᠭᠣᠯᠠ \u180eᠢᠢᠨ ᠵᠣᠨ \u180e\u180eᠤ ᠴᠠᠭ"}, {"America_Pacific", "ᠨᠣᠮᠣᠬᠠᠨ ᠳᠠᠯᠠᠢ \u180eᠢᠢᠨ ᠵᠣᠨ \u180e\u180e\u180eᠪ ᠴᠠᠭ"}, {"Atlantic", "ᠠᠲ᠋ᠯᠠᠨ᠋ᠲ \u180eᠤᠨ ᠵᠣᠨ \u180eᠪ ᠴᠠᠭ"}, {"Europe_Central", "ᠲᠥᠪ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠵᠣᠨ \u180e\u180eᠤ ᠴᠠᠭ"}, {"Europe_Eastern", "ᠵᠡᠭᠦᠨ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠵᠣᠨ \u180e\u180eᠤ ᠴᠠᠭ"}, {"Europe_Western", "ᠪᠠᠷᠠᠭᠣᠨ ᠡᠸᠣᠢᠷᠤᠫᠠ ᠢᠢᠨ ᠵᠣᠨ \u180e\u180eᠤ ᠴᠠᠭ"}},
	},
	{
		{{"America_Central", "নোর্থ অমেরিকান সেন্ত্রেল ষ্টেন্দর্দ টাইম"}, {"America_Eastern", "নোর্থ অমেরিকান ইষ্টর্ন ষ্টেন্দর্দ টাইম"}, {"America_Mountain", "নোর্থ অমেরিকান মাউন্টেন ষ্টেন্দর্দ টাইম"}, {"America_Pacific", "নোর্থ অমেরিকান পেসিফিক ষ্টেন্দর্দ টাইম"}, {"Atlantic", "অটলান্টিক ষ্টেন্দর্দ টাইম"}, {"Europe_Central", "সেন্ত্রেল য়ুরোপিয়ান ষ্টেন্দর্দ টাইম"}, {"Europe_Eastern", "ইষ্টর্ন য়ুরোপিয়ান ষ্টেন্দর্দ টাইম"}, {"Europe_Western", "ৱেষ্টর্ন য়ুরোপিয়ান ষ্টেন্দর্দ টাইম"}, {"GMT", "গ্রিনৱিচ মিন টাইম"}},
		{{"America_Central", "নোর্থ অমেরিকান সেন্ত্রেল দেলাইট টাইম"}, {"America_Eastern", "নোর্থ অমেরিকান ইষ্টর্ন দেলাইট টাইম"}, {"America_Mountain", "নোর্থ অমেরিকান মাউন্টেন দেলাইট টাইম"}, {"America_Pacific", "নোর্থ অমেরিকান পেসিফিক দেলাইট টাইম"}, {"Atlantic", "অটলান্টিক দেলাইট টাইম"}, {"Europe_Central", "সেন্ত্রেল য়ুরোপিয়ান সমর টাইম"}, {"Europe_Eastern", "ইষ্টর্ন য়ুরোপিয়ান সমর টাইম"}, {"Europe_Western", "ৱেষ্টর্ন য়ুরোপিয়ান সমর টাইম"}},
	},
	{
		{{"Acre", "एकर प्रमाणवेळ"}, {"Afghanistan", "अफगाणिस्तान वेळ"}, {"Africa_Central", "मध्\u200dय आफ्रिका वेळ"}, {"Africa_Eastern", "पूर्व आफ्रिका वेळ"}, {"Africa_Southern", "दक्षिण आफ्रिका प्रमाण वेळ"}, {"Africa_Western", "पश्चिम आफ्रिका वेळ"}, {"Alaska", "अलास्का प्रमाण वेळ"}, {"Almaty", "अल्माटी प्रमाणवेळ"}, {"Amazon", "अ\u200dॅमेझॉन प्रमाण वेळ"}, {"America_Central", "केंद्रीय प्रमाण वेळ"}, {"America_Eastern", "पौर्वात्य प्रमाण वेळ"}, {"America_Mountain", "पर्वतीय प्रमाण वेळ"}, {"America_Pacific", "पॅसिफिक प्रमाण वेळ"}, {"Anadyr", "अनादीर प्रमाणवेळ"}, {"Apia", "एपिया प्रमाण वेळ"}, {"Aqtau", "अ\u200dॅक्ताउ प्रमाणवेळ"}, {"Aqtobe", "अ\u200dॅक्टोबे प्रमाणवेळ"}, {"Arabian", "अरेबियन प्रमाण वेळ"}, {"Argentina", "अर्जेंटिना प्रमाण वेळ"}, {"Argentina_Western", "पश्चिमी अर्जेंटिना प्रमाण वेळ"}, {"Armenia", "आर्मेनिया प्रमाण वेळ"}, {"Atlantic", "अटलांटिक प्रमाण वेळ"}, {"Australia_Central", "ऑस्ट्रेलियन मध्य प्रमाण वेळ"}, {"Australia_CentralWestern", "ऑस्ट्रेलियन मध्य-पश्चिम प्रमाण वेळ"}, {"Australia_Eastern", "ऑस्ट्रेलियन पूर्व प्रमाण वेळ"}, {"Australia_Western", "ऑस्ट्रेलियन पश्चिम प्रमाण वेळ"}, {"Azerbaijan", "अझरबैजान प्रमाण वेळ"}, {"Azores", "अ\u200dॅझोरेस प्रमाण वेळ"}, {"Bangladesh", "बांगलादेश प्रमाण वेळ"}, {"Bhutan", "भूतान वेळ"}, {"Bolivia", "बोलिव्हिया वेळ"}, {"Brasilia", "ब्राझिलिया प्रमाण वेळ"}, {"Brunei", "ब्रुनेई दारूसलाम वेळ"}, {"Cape_Verde", "केप व्हर्डे प्रमाण वेळ"}, {"Chamorro", "चामोरो प्रमाण वेळ"}, {"Chatham", "चॅथम प्रमाण वेळ"}, {"Chile", "चिली प्रमाण वेळ"}, {"China", "चीनी प्रमाण वेळ"}, {"Christmas", "ख्रिसमस बेट वेळ"}, {"Cocos", "कॉकोस बेटे वेळ"}, {"Colombia", "कोलंबिया प्रमाण वेळ"}, {"Cook", "कुक बेटे प्रमाण वेळ"}, {"Cuba", "क्यूबा प्रमाण वेळ"}, {"Davis", "डेव्हिस वेळ"}, {"DumontDUrville", "ड्युमॉन्ट-ड्युर्विल वेळ"}, {"East_Timor", "पूर्व तिमोर वेळ"}, {"Easter", "इस्टर बेट प्रमाण वेळ"}, {"Ecuador", "इक्वेडोर वेळ"}, {"Europe_Central", "मध्\u200dय युरोपियन प्रमाण वेळ"}, {"Europe_Eastern", "पूर्व युरोपियन प्रमाण वेळ"}, {"Europe_Further_Eastern", "अग्र-पौर्वात्य युरोपीयन वेळ"}, {"Europe_Western", "पश्चिम युरोपियन प्रमाण वेळ"}, {"Falkland", "फॉकलंड बेटे प्रमाण वेळ"}, {"Fiji", "फिजी प्रमाण वेळ"}, {"French_Guiana", "फ्रेंच गयाना वेळ"}, {"French_Southern", "फ्रेंच दक्षिण आणि अंटार्क्टिक वेळ"}, {"GMT", "ग्रीनिच प्रमाण वेळ"}, {"Galapagos", "गॅलापागोस वेळ"}, {"Gambier", "गॅम्बियर वेळ"}, {"Georgia", "जॉर्जिया प्रमाण वेळ"}, {"Gilbert_Islands", "गिल्बर्ट बेटे वेळ"}, {"Greenland_Eastern", "पूर्व ग्रीनलँड प्रमाण वेळ"}, {"Greenland_Western", "पश्चिम ग्रीनलँड प्रमाण वेळ"}, {"Guam", "गुआम प्रमाणवेळ"}, {"Gulf", "खाडी प्रमाण वेळ"}, {"Guyana", "गयाना वेळ"}, {"Hawaii", "हवाई-अलूशन प्रमाण वेळ"}, {"Hawaii_Aleutian", "हवाई-अलूशन प्रमाण वेळ"}, {"Hong_Kong", "हाँग काँग प्रमाण वेळ"}, {"Hovd", "होव्ह्ड प्रमाण वेळ"}, {"India", "भारतीय प्रमाण वेळ"}, {"Indian_Ocean", "हिंद महासागर वेळ"}, {"Indochina", "इंडोचायना वेळ"}, {"Indonesia_Central", "मध्\u200dय इंडोनेशिया वेळ"}, {"Indonesia_Eastern", "पौर्वात्य इंडोनेशिया वेळ"}, {"Indonesia_Western", "पश्चिमी इंडोनेशिया वेळ"}, {"Iran", "इराण प्रमाण वेळ"}, {"Irkutsk", "इर्कुत्सक प्रमाण वेळ"}, {"Israel", "इस्रायल प्रमाण वेळ"}, {"Japan", "जपान प्रमाण वेळ"}, {"Kamchatka", "पेट्रोपाव्हलोस्क- कामचाट्स्की प्रमाणवेळ"}, {"Kazakhstan", "कझाकस्तान वेळ"}, {"Kazakhstan_Eastern", "पूर्व कझाकस्तान वेळ"}, {"Kazakhstan_Western", "पश्चिम कझाकस्तान वेळ"}, {"Korea", "कोरियन प्रमाण वेळ"}, {"Kosrae", "कोस्राई वेळ"}, {"Krasnoyarsk", "क्रास्नोयार्स्क प्रमाण वेळ"}, {"Kyrgystan", "किरगिस्तान वेळ"}, {"Lanka", "लंका वेळ"}, {"Line_Islands", "लाइन बेटे वेळ"}, {"Lord_Howe", "लॉर्ड होवे प्रमाण वेळ"}, {"Macau", "मकाऊ प्रमाणवेळ"}, {"Magadan", "मॅगाडन प्रमाण वेळ"}, {"Malaysia", "मलेशिया वेळ"}, {"Maldives", "मालदिव वेळ"}, {"Marquesas", "मार्क्वेसास वेळ"}, {"Marshall_Islands", "मार्शल बेटे वेळ"}, {"Mauritius", "मॉरीशस प्रमाण वेळ"}, {"Mawson", "मॉसन वेळ"}, {"Mexico_Pacific", "मेक्सिको पॅसिफिक प्रमाण वेळ"}, {"Mongolia", "उलान बाटोर प्रमाण वेळ"}, {"Moscow", "मॉस्को प्रमाण वेळ"}, {"Myanmar", "म्यानमार वेळ"}, {"Nauru", "नउरु वेळ"}, {"Nepal", "नेपाळ वेळ"}, {"New_Caledonia", "न्यू कॅलेडोनिया प्रमाण वेळ"}, {"New_Zealand", "न्यूझीलंड प्रमाण वेळ"}, {"Newfoundland", "न्यू फाउंडलंड प्रमाण वेळ"}, {"Niue", "न्युए वेळ"}, {"Norfolk", "नॉरफोक बेट प्रमाण वेळ"}, {"Noronha", "फर्नांडो दी नोरोन्हा प्रमाण वेळ"}, {"North_Mariana", "उत्तर मरिना बेटे वेळ"}, {"Novosibirsk", "नोवोसिबिर्स्क प्रमाण वेळ"}, {"Omsk", "ओम्स्क प्रमाण वेळ"}, {"Pakistan", "पाकिस्तान प्रमाण वेळ"}, {"Palau", "पलाऊ वेळ"}, {"Papua_New_Guinea", "पापुआ न्यू गिनी वेळ"}, {"Paraguay", "पॅराग्वे प्रमाण वेळ"}, {"Peru", "पेरु प्रमाण वेळ"}, {"Philippines", "फिलिपाइन प्रमाण वेळ"}, {"Phoenix_Islands", "\u200dफोनिक्स बेटे वेळ"}, {"Pierre_Miquelon", "सेंट पियरे आणि मिक्वेलोन प्रमाण वेळ"}, {"Pitcairn", "पिटकैर्न वेळ"}, {"Ponape", "पोनॅपे वेळ"}, {"Pyongyang", "प्योंगयांग वेळ"}, {"Qyzylorda", "क़िझीलोर्डा प्रमाणवेळ"}, {"Reunion", "रियुनियन वेळ"}, {"Rothera", "रोथेरा वेळ"}, {"Sakhalin", "सखलिन प्रमाण वेळ"}, {"Samara", "सामरा प्रमाणवेळ"}, {"Samoa", "सामोआ प्रमाण वेळ"}, {"Seychelles", "सेशेल्स वेळ"}, {"Singapore", "सिंगापूर प्रमाण वेळ"}, {"Solomon", "सोलोमॉन बेटे वेळ"}, {"South_Georgia", "दक्षिण जॉर्जिया वेळ"}, {"Suriname", "सुरिनाम वेळ"}, {"Syowa", "स्योवा वेळ"}, {"Tahiti", "ताहिती वेळ"}, {"Taipei", "तैपेई प्रमाण वेळ"}, {"Tajikistan", "ताजिकिस्तान वेळ"}, {"Tokelau", "टोकेलाऊ वेळ"}, {"Tonga", "टोंगा प्रमाण वेळ"}, {"Truk", "चूक वेळ"}, {"Turkey", "तुर्की प्रमाणवेळ"}, {"Turkmenistan", "तुर्कमेनिस्तान प्रमाण वेळ"}, {"Tuvalu", "तुवालू वेळ"}, {"Uruguay", "उरुग्वे प्रमाण वेळ"}, {"Uzbekistan", "उझबेकिस्तान प्रमाण वेळ"}, {"Vanuatu", "वानुआतु प्रमाण वेळ"}, {"Venezuela", "व्हेनेझुएला वेळ"}, {"Vladivostok", "व्लादिवोस्तोक प्रमाण वेळ"}, {"Volgograd", "व्होल्गोग्राड प्रमाण वेळ"}, {"Vostok", "व्होस्टॉक वेळ"}, {"Wake", "वेक बेट वेळ"}, {"Wallis", "वॉलिस आणि फुटुना वेळ"}, {"Yakutsk", "याकुत्सक प्रमाण वेळ"}, {"Yekaterinburg", "येकतरिनबर्ग प्रमाण वेळ"}, {"Yukon", "युकोन वेळ"}},
		{{"Acre", "ऐकर ग्रीष्म वेळ"}, {"Alaska", "अलास्का सूर्यप्रकाश वेळ"}, {"Almaty", "अल्माटी ग्रीष्मकालीन वेळ"}, {"Amazon", "अ\u200dॅमेझॉन उन्हाळी वेळ"}, {"America_Central", "केंद्रीय सूर्यप्रकाश वेळ"}, {"America_Eastern", "पौर्वात्य सूर्यप्रकाश वेळ"}, {"America_Mountain", "पर्वतीय सूर्यप्रकाश वेळ"}, {"America_Pacific", "पॅसिफिक सूर्यप्रकाश वेळ"}, {"Anadyr", "अनादीर ग्रीष्मकालीन वेळ"}, {"Apia", "एपिया सूर्यप्रकाश वेळ"}, {"Aqtau", "अ\u200dॅक्ताउ ग्रीष्मकालीन वेळ"}, {"Aqtobe", "अ\u200dॅक्टोबे ग्रीष्मकालीन वेळ"}, {"Arabian", "अरेबियन सूर्यप्रकाश वेळ"}, {"Argentina", "अर्जेंटिना उन्हाळी वेळ"}, {"Argentina_Western", "पश्चिमी अर्जेंटिना उन्हाळी प्रमाणवेळ"}, {"Armenia", "आर्मेनिया उन्हाळी वेळ"}, {"Atlantic", "अटलांटिक सूर्यप्रकाश वेळ"}, {"Australia_Central", "ऑस्ट्रेलियन मध्य सूर्यप्रकाश वेळ"}, {"Australia_CentralWestern", "ऑस्ट्रेलियन मध्य-पश्चिम सूर्यप्रकाश वेळ"}, {"Australia_Eastern", "ऑस्ट्रेलियन पूर्व सूर्यप्रकाश वेळ"}, {"Australia_Western", "ऑस्ट्रेलियन पश्चिम सूर्यप्रकाश वेळ"}, {"Azerbaijan", "अझरबैजान उन्हाळी वेळ"}, {"Azores", "अ\u200dॅझोरेस उन्हाळी वेळ"}, {"Bangladesh", "बांगलादेश उन्हाळी वेळ"}, {"Brasilia", "ब्राझिलिया उन्हाळी वेळ"}, {"Cape_Verde", "केप व्हर्डे उन्हाळी वेळ"}, {"Chatham", "चॅथम सूर्यप्रकाश वेळ"}, {"Chile", "चिली उन्हाळी वेळ"}, {"China", "चीनी सूर्यप्रकाश वेळ"}, {"Colombia", "कोलंबिया उन्हाळी वेळ"}, {"Cook", "कुक बेटे उन्हाळी प्रमाणवेळ"}, {"Cuba", "क्यूबा सूर्यप्रकाश वेळ"}, {"Easter", "इस्टर बेट उन्हाळी वेळ"}, {"Europe_Central", "मध्\u200dय युरोपियन उन्हाळी वेळ"}, {"Europe_Eastern", "पूर्व युरोपियन उन्हाळी वेळ"}, {"Europe_Western", "पश्चिम युरोपियन उन्हाळी वेळ"}, {"Falkland", "फॉकलंड बेटे उन्हाळी वेळ"}, {"Fiji", "फिजी उन्हाळी वेळ"}, {"Georgia", "जॉर्जिया उन्हाळी वेळ"}, {"Greenland_Eastern", "पूर्व ग्रीनलँड उन्हाळी वेळ"}, {"Greenland_Western", "पश्चिम ग्रीनलँड उन्हाळी वेळ"}, {"Hawaii_Aleutian", "हवाई-अलूशन सूर्यप्रकाश वेळ"}, {"Hong_Kong", "हाँग काँग उन्हाळी वेळ"}, {"Hovd", "होव्ह्ड उन्हाळी वेळ"}, {"Iran", "इराण सूर्यप्रकाश वेळ"}, {"Irkutsk", "इर्कुत्सक उन्हाळी वेळ"}, {"Israel", "इस्रायल सूर्यप्रकाश वेळ"}, {"Japan", "जपान सूर्यप्रकाश वेळ"}, {"Kamchatka", "पेट्रोपाव्हलोस्क- कामचाट्स्की ग्रीष्मकालीन वेळ"}, {"Korea", "कोरियन सूर्यप्रकाश वेळ"}, {"Krasnoyarsk", "क्रास्नोयार्क्स उन्हाळी वेळ"}, {"Lord_Howe", "लॉर्ड होवे सूर्यप्रकाश वेळ"}, {"Macau", "मकाऊ ग्रीष्मकालीन वेळ"}, {"Magadan", "मॅगाडन उन्हाळी वेळ"}, {"Mauritius", "मॉरीशस उन्हाळी वेळ"}, {"Mexico_Pacific", "मेक्सिको पॅसिफिक सूर्यप्रकाश वेळ"}, {"Mongolia", "उलान बाटोर उन्हाळी वेळ"}, {"Moscow", "मॉस्को उन्हाळी वेळ"}, {"New_Caledonia", "न्यू कॅलेडोनिया उन्हाळी प्रमाणवेळ"}, {"New_Zealand", "न्यूझीलंड सूर्यप्रकाश वेळ"}, {"Newfoundland", "न्यू फाउंडलंड सूर्यप्रकाश वेळ"}, {"Norfolk", "नॉरफोक बेट सूर्यप्रकाश वेळ"}, {"Noronha", "फर्नांडो दी नोरोन्हा उन्हाळी प्रमाणवेळ"}, {"Novosibirsk", "नोवोसिबिर्स्क उन्हाळी वेळ"}, {"Omsk", "ओम्स्क उन्हाळी वेळ"}, {"Pakistan", "पाकिस्तान उन्हाळी वेळ"}, {"Paraguay", "पॅराग्वे उन्हाळी प्रमाणवेळ"}, {"Peru", "पेरु उन्हाळी प्रमाणवेळ"}, {"Philippines", "फिलिपाइन उन्हाळी वेळ"}, {"Pierre_Miquelon", "सेंट पियरे आणि मिक्वेलोन सूर्यप्रकाश वेळ"}, {"Qyzylorda", "क़िझीलोर्डा ग्रीष्मकालीन वेळ"}, {"Sakhalin", "सखलिन उन्हाळी वेळ"}, {"Samara", "सामरा ग्रीष्मकालीन वेळ"}, {"Samoa", "सामोआ सूर्यप्रकाश वेळ"}, {"Taipei", "तैपेई सूर्यप्रकाश वेळ"}, {"Tonga", "टोंगा उन्हाळी प्रमाणवेळ"}, {"Turkey", "तुर्की ग्रीष्मकालीन वेळ"}, {"Turkmenistan", "तुर्कमेनिस्तान उन्हाळी वेळ"}, {"Uruguay", "उरुग्वे उन्हाळी प्रमाणवेळ"}, {"Uzbekistan", "उझबेकिस्तान उन्हाळी वेळ"}, {"Vanuatu", "वानुआतु उन्हाळी वेळ"}, {"Vladivostok", "व्लादिवोस्तोक उन्हाळी वेळ"}, {"Volgograd", "व्होल्गोग्राड उन्हाळी वेळ"}, {"Yakutsk", "याकुत्सक उन्हाळी वेळ"}, {"Yekaterinburg", "येकतरिनबर्ग उन्हाळी वेळ"}},
	},
	{
		{{"Acre", "Waktu Piawai Acre"}, {"Afghanistan", "Waktu Afghanistan"}, {"Africa_Central", "Waktu Afrika Tengah"}, {"Africa_Eastern", "Waktu Afrika Timur"}, {"Africa_Southern", "Waktu Piawai Afrika Selatan"}, {"Africa_Western", "Waktu Afrika Barat"}, {"Alaska", "Waktu Piawai Alaska"}, {"Almaty", "Waktu Piawai Almaty"}, {"Amazon", "Waktu Piawai Amazon"}, {"America_Central", "Waktu Piawai Pusat"}, {"America_Eastern", "Waktu Piawai Timur"}, {"America_Mountain", "Waktu Piawai Pergunungan"}, {"America_Pacific", "Waktu Piawai Pasifik"}, {"Anadyr", "Waktu Piawai Anadyr"}, {"Apia", "Waktu Piawai Apia"}, {"Aqtau", "Waktu Standard Aqtau"}, {"Aqtobe", "Waktu Piawai Aqtobe"}, {"Arabian", "Waktu Piawai Arab"}, {"Argentina", "Waktu Piawai Argentina"}, {"Argentina_Western", "Waktu Piawai Argentina Barat"}, {"Armenia", "Waktu Piawai Armenia"}, {"Atlantic", "Waktu Piawai Atlantik"}, {"Australia_Central", "Waktu Piawai Australia Tengah"}, {"Australia_CentralWestern", "Waktu Piawai Barat Tengah Australia"}, {"Australia_Eastern", "Waktu Piawai Timur Australia"}, {"Australia_Western", "Waktu Piawai Australia Barat"}, {"Azerbaijan", "Waktu Piawai Azerbaijan"}, {"Azores", "Waktu Piawai Azores"}, {"Bangladesh", "Waktu Piawai Bangladesh"}, {"Bhutan", "Waktu Bhutan"}, {"Bolivia", "Waktu Bolivia"}, {"Brasilia", "Waktu Piawai Brasilia"}, {"Brunei", "Waktu Brunei Darussalam"}, {"Cape_Verde", "Waktu Piawai Tanjung Verde"}, {"Chamorro", "Waktu Piawai Chamorro"}, {"Chatham", "Waktu Piawai Chatham"}, {"Chile", "Waktu Piawai Chile"}, {"China", "Waktu Piawai China"}, {"Christmas", "Waktu Pulau Christmas"}, {"Cocos", "Waktu Kepulauan Cocos"}, {"Colombia", "Waktu Piawai Colombia"}, {"Cook", "Waktu Piawai Kepulauan Cook"}, {"Cuba", "Waktu Piawai Cuba"}, {"Davis", "Waktu Davis"}, {"DumontDUrville", "Waktu Dumont-d’Urville"}, {"East_Timor", "Waktu Timor Timur"}, {"Easter", "Waktu Piawai Pulau Easter"}, {"Ecuador", "Waktu Ecuador"}, {"Europe_Central", "Waktu Piawai Eropah Tengah"}, {"Europe_Eastern", "Waktu Piawai Eropah Timur"}, {"Europe_Further_Eastern", "Waktu Eropah ceruk timur"}, {"Europe_Western", "Waktu Piawai Eropah Barat"}, {"Falkland", "Waktu Piawai Kepulauan Falkland"}, {"Fiji", "Waktu Piawai Fiji"}, {"French_Guiana", "Waktu Guyana Perancis"}, {"French_Southern", "Waktu Perancis Selatan dan Antartika"}, {"GMT", "Waktu Min Greenwich"}, {"Galapagos", "Waktu Galapagos"}, {"Gambier", "Waktu Gambier"}, {"Georgia", "Waktu Piawai Georgia"}, {"Gilbert_Islands", "Waktu Kepulauan Gilbert"}, {"Greenland_Eastern", "Waktu Piawai Greenland Timur"}, {"Greenland_Western", "Waktu Piawai Greenland Barat"}, {"Gulf", "Waktu Piawai Teluk"}, {"Guyana", "Waktu Guyana"}, {"Hawaii", "Waktu Piawai Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Waktu Piawai Hawaii-Aleutian"}, {"Hong_Kong", "Waktu Piawai Hong Kong"}, {"Hovd", "Waktu Piawai Hovd"}, {"India", "Waktu Piawai India"}, {"Indian_Ocean", "Waktu Lautan Hindi"}, {"Indochina", "Waktu Indochina"}, {"Indonesia_Central", "Waktu Indonesia Tengah"}, {"Indonesia_Eastern", "Waktu Indonesia Timur"}, {"Indonesia_Western", "Waktu Indonesia Barat"}, {"Iran", "Waktu Piawai Iran"}, {"Irkutsk", "Waktu Piawai Irkutsk"}, {"Israel", "Waktu Piawai Israel"}, {"Japan", "Waktu Piawai Jepun"}, {"Kamchatka", "Waktu Piawai Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Waktu Kazakhstan"}, {"Kazakhstan_Eastern", "Waktu Kazakhstan Timur"}, {"Kazakhstan_Western", "Waktu Kazakhstan Barat"}, {"Korea", "Waktu Piawai Korea"}, {"Kosrae", "Waktu Kosrae"}, {"Krasnoyarsk", "Waktu Piawai Krasnoyarsk"}, {"Kyrgystan", "Waktu Kyrgystan"}, {"Line_Islands", "Waktu Kepulauan Line"}, {"Lord_Howe", "Waktu Piawai Lord Howe"}, {"Macau", "Waktu Piawai Macao"}, {"Magadan", "Waktu Piawai Magadan"}, {"Malaysia", "Waktu Malaysia"}, {"Maldives", "Waktu Maldives"}, {"Marquesas", "Waktu Marquesas"}, {"Marshall_Islands", "Waktu Kepulauan Marshall"}, {"Mauritius", "Waktu Piawai Mauritius"}, {"Mawson", "Waktu Mawson"}, {"Mexico_Pacific", "Waktu Piawai Pasifik Mexico"}, {"Mongolia", "Waktu Piawai Ulan Bator"}, {"Moscow", "Waktu Piawai Moscow"}, {"Myanmar", "Waktu Myanmar"}, {"Nauru", "Waktu Nauru"}, {"Nepal", "Waktu Nepal"}, {"New_Caledonia", "Waktu Piawai New Caledonia"}, {"New_Zealand", "Waktu Piawai New Zealand"}, {"Newfoundland", "Waktu Piawai Newfoundland"}, {"Niue", "Waktu Niue"}, {"Norfolk", "Waktu Piawai Kepulauan Norfolk"}, {"Noronha", "Waktu Piawai Fernando de Noronha"}, {"North_Mariana", "Waktu Kepulauan Mariana Utara"}, {"Novosibirsk", "Waktu Piawai Novosibirsk"}, {"Omsk", "Waktu Piawai Omsk"}, {"Pakistan", "Waktu Piawai Pakistan"}, {"Palau", "Waktu Palau"}, {"Papua_New_Guinea", "Waktu Papua New Guinea"}, {"Paraguay", "Waktu Piawai Paraguay"}, {"Peru", "Waktu Piawai Peru"}, {"Philippines", "Waktu Piawai Filipina"}, {"Phoenix_Islands", "Waktu Kepulauan Phoenix"}, {"Pierre_Miquelon", "Waktu Piawai Saint Pierre dan Miquelon"}, {"Pitcairn", "Waktu Pitcairn"}, {"Ponape", "Waktu Ponape"}, {"Pyongyang", "Waktu Pyongyang"}, {"Qyzylorda", "Waktu Piawai Qyzylorda"}, {"Reunion", "Waktu Reunion"}, {"Rothera", "Waktu Rothera"}, {"Sakhalin", "Waktu Piawai Sakhalin"}, {"Samara", "Waktu Piawai Samara"}, {"Samoa", "Waktu Piawai Samoa"}, {"Seychelles", "Waktu Seychelles"}, {"Singapore", "Waktu Piawai Singapura"}, {"Solomon", "Waktu Kepulauan Solomon"}, {"South_Georgia", "Waktu Georgia Selatan"}, {"Suriname", "Waktu Suriname"}, {"Syowa", "Waktu Syowa"}, {"Tahiti", "Waktu Tahiti"}, {"Taipei", "Waktu Piawai Taipei"}, {"Tajikistan", "Waktu Tajikistan"}, {"Tokelau", "Waktu Tokelau"}, {"Tonga", "Waktu Piawai Tonga"}, {"Truk", "Waktu Chuuk"}, {"Turkmenistan", "Waktu Piawai Turkmenistan"}, {"Tuvalu", "Waktu Tuvalu"}, {"Uruguay", "Waktu Piawai Uruguay"}, {"Uzbekistan", "Waktu Piawai Uzbekistan"}, {"Vanuatu", "Waktu Piawai Vanuatu"}, {"Venezuela", "Waktu Venezuela"}, {"Vladivostok", "Waktu Piawai Vladivostok"}, {"Volgograd", "Waktu Piawai Volgograd"}, {"Vostok", "Waktu Vostok"}, {"Wake", "Waktu Pulau Wake"}, {"Wallis", "Waktu Wallis dan Futuna"}, {"Yakutsk", "Waktu Piawai Yakutsk"}, {"Yekaterinburg", "Waktu Piawai Yekaterinburg"}, {"Yukon", "Masa Yukon"}},
		{{"Acre", "Waktu Musim Panas Acre"}, {"Alaska", "Waktu Siang Alaska"}, {"Almaty", "Waktu Musim Panas Almaty"}, {"Amazon", "Waktu Musim Panas Amazon"}, {"America_Central", "Waktu Siang Tengah"}, {"America_Eastern", "Waktu Siang Timur"}, {"America_Mountain", "Waktu Siang Pergunungan"}, {"America_Pacific", "Waktu Siang Pasifik"}, {"Anadyr", "Waktu Musim Panas Anadyr"}, {"Apia", "Waktu Siang Apia"}, {"Aqtau", "Waktu Musim Panas Aqtau"}, {"Aqtobe", "Waktu Musim Panas Aqtobe"}, {"Arabian", "Waktu Siang Arab"}, {"Argentina", "Waktu Musim Panas Argentina"}, {"Argentina_Western", "Waktu Musim Panas Argentina Barat"}, {"Armenia", "Waktu Musim Panas Armenia"}, {"Atlantic", "Waktu Siang Atlantik"}, {"Australia_Central", "Waktu Siang Australia Tengah"}, {"Australia_CentralWestern", "Waktu Siang Barat Tengah Australia"}, {"Australia_Eastern", "Waktu Siang Australia Timur"}, {"Australia_Western", "Waktu Siang Australia Barat"}, {"Azerbaijan", "Waktu Musim Panas Azerbaijan"}, {"Azores", "Waktu Musim Panas Azores"}, {"Bangladesh", "Waktu Musim Panas Bangladesh"}, {"Brasilia", "Waktu Musim Panas Brasilia"}, {"Cape_Verde", "Waktu Musim Panas Tanjung Verde"}, {"Chatham", "Waktu Siang Chatham"}, {"Chile", "Waktu Musim Panas Chile"}, {"China", "Waktu Siang China"}, {"Colombia", "Waktu Musim Panas Colombia"}, {"Cook", "Waktu Musim Panas Separuh Kepulauan Cook"}, {"Cuba", "Waktu Siang Cuba"}, {"Easter", "Waktu Musim Panas Pulau Easter"}, {"Europe_Central", "Waktu Musim Panas Eropah Tengah"}, {"Europe_Eastern", "Waktu Musim Panas Eropah Timur"}, {"Europe_Western", "Waktu Musim Panas Eropah Barat"}, {"Falkland", "Waktu Musim Panas Kepulauan Falkland"}, {"Fiji", "Waktu Musim Panas Fiji"}, {"Georgia", "Waktu Musim Panas Georgia"}, {"Greenland_Eastern", "Waktu Musim Panas Greenland Timur"}, {"Greenland_Western", "Waktu Musim Panas Greenland Barat"}, {"Hawaii_Aleutian", "Waktu Siang Hawaii-Aleutian"}, {"Hong_Kong", "Waktu Musim Panas Hong Kong"}, {"Hovd", "Waktu Musim Panas Hovd"}, {"Iran", "Waktu Siang Iran"}, {"Irkutsk", "Waktu Musim Panas Irkutsk"}, {"Israel", "Waktu Siang Israel"}, {"Japan", "Waktu Siang Jepun"}, {"Kamchatka", "Waktu Musim Panas Petropavlovsk-Kamchatski"}, {"Korea", "Waktu Siang Korea"}, {"Krasnoyarsk", "Waktu Musim Panas Krasnoyarsk"}, {"Lord_Howe", "Waktu Siang Lord Howe"}, {"Macau", "Waktu Musim Panas Macao"}, {"Magadan", "Waktu Musim Panas Magadan"}, {"Mauritius", "Waktu Musim Panas Mauritius"}, {"Mexico_Pacific", "Waktu Siang Pasifik Mexico"}, {"Mongolia", "Waktu Musim Panas Ulan Bator"}, {"Moscow", "Waktu Musim Panas Moscow"}, {"New_Caledonia", "Waktu Musim Panas New Caledonia"}, {"New_Zealand", "Waktu Siang New Zealand"}, {"Newfoundland", "Waktu Siang Newfoundland"}, {"Norfolk", "Waktu Siang Kepulauan Norfolk"}, {"Noronha", "Waktu Musim Panas Fernando de Noronha"}, {"Novosibirsk", "Waktu Musim Panas Novosibirsk"}, {"Omsk", "Waktu Musim Panas Omsk"}, {"Pakistan", "Waktu Musim Panas Pakistan"}, {"Paraguay", "Waktu Musim Panas Paraguay"}, {"Peru", "Waktu Musim Panas Peru"}, {"Philippines", "Waktu Musim Panas Filipina"}, {"Pierre_Miquelon", "Waktu Siang Saint Pierre dan Miquelon"}, {"Qyzylorda", "Waktu Musim Panas Qyzylorda"}, {"Sakhalin", "Waktu Musim Panas Sakhalin"}, {"Samara", "Waktu Musim Panas Samara"}, {"Samoa", "Waktu Musim Panas Samoa"}, {"Taipei", "Waktu Siang Taipei"}, {"Tonga", "Waktu Musim Panas Tonga"}, {"Turkmenistan", "Waktu Musim Panas Turkmenistan"}, {"Uruguay", "Waktu Musim Panas Uruguay"}, {"Uzbekistan", "Waktu Musim Panas Uzbekistan"}, {"Vanuatu", "Waktu Musim Panas Vanuatu"}, {"Vladivostok", "Waktu Musim Panas Vladivostok"}, {"Volgograd", "Waktu Musim Panas Volgograd"}, {"Yakutsk", "Waktu Musim Panas Yakutsk"}, {"Yekaterinburg", "Waktu Musim Panas Yekaterinburg"}},
	},
	{
		{{"Brunei", "وقتو بروني دارالسلام"}, {"India", "وقتو ڤياواي اينديا"}, {"Indian_Ocean", "وقتو لاٴوتن هيندي"}, {"Indonesia_Central", "وقتو إندونيسيا تڠه"}, {"Indonesia_Eastern", "وقتو إندونيسيا تيمور"}, {"Indonesia_Western", "وقتو إندونيسيا بارات"}, {"Malaysia", "وقتو مليسيا"}},
		{},
	},
	{
		{{"Europe_Central", "Ħin Ċentrali Ewropew Standard"}, {"GMT", "Greenwich Mean Time"}},
		{{"Europe_Central", "Ħin Ċentrali Ewropew tas-Sajf"}},
	},
	{
		{{"Afghanistan", "အာဖဂန်နစ္စတန် အချိန်"}, {"Africa_Central", "အလယ်အာဖရိက အချိန်"}, {"Africa_Eastern", "အရှေ့အာဖရိက အချိန်"}, {"Africa_Southern", "တောင်အာဖရိက အချိန်"}, {"Africa_Western", "အနောက်အာဖရိက အချိန်"}, {"Alaska", "အလာစကာ စံတော်ချိန်"}, {"Amazon", "အမေဇုံ စံတော်ချိန်"}, {"America_Central", "အလယ်ပိုင်းစံတော်ချိန်"}, {"America_Eastern", "အရှေ့ပိုင်းစံတော်ချိန်"}, {"America_Mountain", "တောင်တန်းစံတော်ချိန်"}, {"America_Pacific", "ပစိဖိတ်စံတော်ချိန်"}, {"Apia", "အပီယာ စံတော်ချိန်"}, {"Arabian", "အာရေဗျ စံတော်ချိန်"}, {"Argentina", "အာဂျင်တီးနား စံတော်ချိန်"}, {"Argentina_Western", "အနောက် အာဂျင်တီးနား စံတော်ချိန်"}, {"Armenia", "အာမေးနီးယား စံတော်ချိန်"}, {"Atlantic", "အတ္တလန်တစ် စံတော်ချိန်"}, {"Australia_Central", "ဩစတြေးလျ အလယ်ပိုင်း စံတော်ချိန်"}, {"Australia_CentralWestern", "သြစတြေးလျား အနောက်အလယ်ပိုင်း စံတော်ချိန်"}, {"Australia_Eastern", "အရှေ့ဩစတြေးလျ စံတော်ချိန်"}, {"Australia_Western", "အနောက်ဩစတြေးလျ စံတော်ချိန်"}, {"Azerbaijan", "အဇာဘိုင်ဂျန် စံတော်ချိန်"}, {"Azores", "အေဇိုးရီးစ် စံတော်ချိန်"}, {"Bangladesh", "ဘင်္ဂလားဒေ့ရှ် စံတော်ချိန်"}, {"Bhutan", "ဘူတန် အချိန်"}, {"Bolivia", "ဘိုလီးဘီးယား အချိန်"}, {"Brasilia", "ဘရာဇီး စံတော်ချိန်"}, {"Brunei", "ဘရူနိုင်း စံတော်ချိန်"}, {"Cape_Verde", "ကိတ်ဗာဒီ စံတော်ချိန်"}, {"Chamorro", "ချာမိုရို အချိန်"}, {"Chatham", "ချားသမ်စံတော်ချိန်"}, {"Chile", "ချီလီ စံတော်ချိန်"}, {"China", "တရုတ် စံတော်ချိန်"}, {"Christmas", "ခရစ်စမတ်ကျွန်း အချိန်"}, {"Cocos", "ကိုကိုးကျွန်း အချိန်"}, {"Colombia", "ကိုလံဘီယာ စံတော်ချိန်"}, {"Cook", "ကွတ်ကျွန်းစု စံတော်ချိန်"}, {"Cuba", "ကျူးဘား စံတော်ချိန်"}, {"Davis", "ဒေးဗစ် အချိန်"}, {"DumontDUrville", "ဒူးမော့တ် ဒါရ်ဗီးလ် အချိန်"}, {"East_Timor", "အရှေ့တီမော အချိန်"}, {"Easter", "အီစတာကျွန်း စံတော်ချိန်"}, {"Ecuador", "အီကွေဒေါ အချိန်"}, {"Europe_Central", "ဥရောပအလယ်ပိုင်း စံတော်ချိန်"}, {"Europe_Eastern", "အရှေ့ဥရောပ စံတော်ချိန်"}, {"Europe_Further_Eastern", "အရှေ့ဖျား ဥရောပဒေသ အချိန်"}, {"Europe_Western", "အနောက်ဥရောပ စံတော်ချိန်"}, {"Falkland", "ဖော့ကလန်ကျွန်းစု စံတော်ချိန်"}, {"Fiji", "ဖီဂျီ စံတော်ချိန်"}, {"French_Guiana", "ပြင်သစ် ဂီအားနား အချိန်"}, {"French_Southern", "ပြင်သစ်တောင်ပိုင်းနှင့် အန္တာတိတ် အချိန်"}, {"GMT", "ဂရင်းနစ် စံတော်ချိန်"}, {"Galapagos", "ဂါလားပါဂိုးစ် အချိန်"}, {"Gambier", "ဂမ်ဘီယာ အချိန်"}, {"Georgia", "ဂျော်ဂျီယာ စံတော်ချိန်"}, {"Gilbert_Islands", "ဂီလ်ဘတ်ကျွန်းစု အချိန်"}, {"Greenland_Eastern", "အရှေ့ဂရင်းလန်း စံတော်ချိန်"}, {"Greenland_Western", "အနောက် ဂရင်းလန်း စံတော်ချိန်"}, {"Gulf", "ပင်လယ်ကွေ့ အချိန်"}, {"Guyana", "ဂိုင်ယာနာ အချိန်"}, {"Hawaii", "ဟာဝိုင်ယီ အယ်လူးရှန်း စံတော်ချိန်"}, {"Hawaii_Aleutian", "ဟာဝိုင်ယီ အယ်လူးရှန်း စံတော်ချိန်"}, {"Hong_Kong", "ဟောင်ကောင် စံတော်ချိန်"}, {"Hovd", "ဟိုးဗ် စံတော်ချိန်"}, {"India", "အိန္ဒိယ စံတော်ချိန်"}, {"Indian_Ocean", "အိန္ဒိယသမုဒ္ဒရာ အချိန်"}, {"Indochina", "အင်ဒိုချိုင်းနား အချိန်"}, {"Indonesia_Central", "အလယ်ပိုင်း အင်ဒိုနီးရှား အချိန်"}, {"Indonesia_Eastern", "အရှေ့ပိုင်း အင်ဒိုနီးရှား အချိန်"}, {"Indonesia_Western", "အနောက်ပိုင်း အင်ဒိုနီးရှား အချိန်"}, {"Iran", "အီရန် စံတော်ချိန်"}, {"Irkutsk", "အီရူခူတ် စံတော်ချိန်"}, {"Israel", "အစ္စရေး စံတော်ချိန်"}, {"Japan", "ဂျပန် စံတော်ချိန်"}, {"Kazakhstan", "ကာဇက်စတန် အချိန်"}, {"Kazakhstan_Eastern", "အရှေ့ကာဇက်စတန် အချိန်"}, {"Kazakhstan_Western", "အနောက်ကာဇက်စတန် အချိန်"}, {"Korea", "ကိုရီးယား စံတော်ချိန်"}, {"Kosrae", "ခိုစ်ရိုင် အချိန်"}, {"Krasnoyarsk", "ခရာ့စ်နိုရာစ် စံတော်ချိန်"}, {"Kyrgystan", "ကာဂျစ္စတန် အချိန်"}, {"Lanka", "သီရိလင်္ကာ အချိန်"}, {"Line_Islands", "လိုင်းကျွန်းစု အချိန်"}, {"Lord_Howe", "လော့ဒ်ဟောင် စံတော်ချိန်"}, {"Macau", "မကာအို စံတော်ချိန်"}, {"Magadan", "မာဂါဒန်း စံတော်ချိန်"}, {"Malaysia", "မလေးရှား အချိန်"}, {"Maldives", "မော်လဒိုက် အချိန်"}, {"Marquesas", "မာခေးအပ်စ် အချိန်"}, {"Marshall_Islands", "မာရှယ်ကျွန်းစု အချိန်"}, {"Mauritius", "မောရစ်ရှ စံတော်ချိန်"}, {"Mawson", "မော်စွန် အချိန်"}, {"Mexico_Pacific", "မက္ကဆီကန် ပစိဖိတ် စံတော်ချိန်"}, {"Mongolia", "ဥလန်ဘာတော စံတော်ချိန်"}, {"Moscow", "မော်စကို စံတော်ချိန်"}, {"Myanmar", "မြန်မာ အချိန်"}, {"Nauru", "နာဥူရူ အချိန်"}, {"Nepal", "နီပေါ အချိန်"}, {"New_Caledonia", "နယူးကာလီဒိုးနီးယား စံတော်ချိန်"}, {"New_Zealand", "နယူးဇီလန် စံတော်ချိန်"}, {"Newfoundland", "နယူးဖောင်လန် စံတော်ချိန်"}, {"Niue", "နီဥူအေ အချိန်"}, {"Norfolk", "နောဖော့ခ်ကျွန်း စံတော်ချိန်"}, {"Noronha", "ဖာနန်ဒိုးဒီနိုးရိုးညာ စံတော်ချိန်"}, {"Novosibirsk", "နိုဗိုစဲဘီအဲယ်စ် စံတော်ချိန်"}, {"Omsk", "အွမ်းစ်ခ် စံတော်ချိန်"}, {"Pakistan", "ပါကစ္စတန် စံတော်ချိန်"}, {"Palau", "ပလာအို အချိန်"}, {"Papua_New_Guinea", "ပါပူအာနယူးဂီနီ အချိန်"}, {"Paraguay", "ပါရာဂွေး စံတော်ချိန်"}, {"Peru", "ပီရူး စံတော်ချိန်"}, {"Philippines", "ဖိလစ်ပိုင် စံတော်ချိန်"}, {"Phoenix_Islands", "ဖီးနစ်ကျွန်းစု အချိန်"}, {"Pierre_Miquelon", "စိန့်ပီအဲနှင့်မီခွီလွန်စံတော်ချိန်"}, {"Pitcairn", "ပါတ်ကယ်ရင် အချိန်"}, {"Ponape", "ဖိုနာဖဲအ် အချိန်"}, {"Pyongyang", "ပြုံယန်း အချိန်"}, {"Reunion", "ရီယူနီယံ အချိန်"}, {"Rothera", "ရိုသီရာ အချိန်"}, {"Sakhalin", "ဆာခါလင် စံတော်ချိန်"}, {"Samoa", "ဆမိုအာ စံတော်ချိန်"}, {"Seychelles", "ဆေးရှဲ အချိန်"}, {"Singapore", "စင်္ကာပူ အချိန်"}, {"Solomon", "ဆော်လမွန်ကျွန်းစု အချိန်"}, {"South_Georgia", "တောင်ဂျော်ဂျီယာ အချိန်"}, {"Suriname", "စူးရီနာမ်အချိန်"}, {"Syowa", "ရှိုဝါ အချိန်"}, {"Tahiti", "တဟီတီ အချိန်"}, {"Taipei", "ထိုင်ပေ စံတော်ချိန်"}, {"Tajikistan", "တာဂျစ်ကစ္စတန် အချိန်"}, {"Tokelau", "တိုကီလာဥ အချိန်"}, {"Tonga", "တွန်ဂါ စံတော်ချိန်"}, {"Truk", "ချုခ် အချိန်"}, {"Turkey", "တူရကီ စံတော်ချိန်"}, {"Turkmenistan", "တာ့ခ်မင်နစ္စတန် စံတော်ချိန်"}, {"Tuvalu", "တူဗားလူ အချိန်"}, {"Uruguay", "ဥရုဂွေး စံတော်ချိန်"}, {"Uzbekistan", "ဥဇဘက်ကစ္စတန် စံတော်ချိန်"}, {"Vanuatu", "ဗနွားတူ စံတော်ချိန်"}, {"Venezuela", "ဗင်နီဇွဲလား အချိန်"}, {"Vladivostok", "ဗလာဒီဗော့စတော့ခ် စံတော်ချိန်"}, {"Volgograd", "ဗိုလ်ဂိုဂရက် စံတော်ချိန်"}, {"Vostok", "ဗိုစ်တိုခ် အချိန်"}, {"Wake", "ဝိတ်ခ်ကျွန်း အချိန်"}, {"Wallis", "ဝေါလီစ်နှင့် ဖူကျူနာ အချိန်"}, {"Yakutsk", "ယူခူးတ်စ် စံတော်ချိန်"}, {"Yekaterinburg", "ရယ်ခါးတီရင်ဘားခ် စံတော်ချိန်"}, {"Yukon", "ယူကွန်း အချိန်"}},
		{{"Alaska", "အလာစကာ နွေရာသီ စံတော်ချိန်"}, {"Amazon", "အမေဇုံ နွေရာသီအချိန်"}, {"America_Central", "အလယ်ပိုင်း နွေရာသီစံတော်ချိန်"}, {"America_Eastern", "အရှေ့ပိုင်း နွေရာသီစံတော်ချိန်"}, {"America_Mountain", "တောင်တန်း နွေရာသီစံတော်ချိန်"}, {"America_Pacific", "ပစိဖိတ် နွေရာသီစံတော်ချိန်"}, {"Apia", "အပီယာ နွေရာသီ အချိန်"}, {"Arabian", "အာရေဗျ နွေရာသီ အချိန်"}, {"Argentina", "အာဂျင်တီးနား နွေရာသီအချိန်"}, {"Argentina_Western", "အနောက် အာဂျင်တီးနား နွေရာသီ အချိန်"}, {"Armenia", "အာမေးနီးယား နွေရာသီ အချိန်"}, {"Atlantic", "အတ္တလန်တစ် နွေရာသီ စံတော်ချိန်"}, {"Australia_Central", "ဩစတြေးလျ အလယ်ပိုင်း နွေရာသီ အချိန်"}, {"Australia_CentralWestern", "သြစတြေးလျား အနောက်အလယ်ပိုင်း နွေရာသီ အချိန်"}, {"Australia_Eastern", "အရှေ့ဩစတြေးလျ နွေရာသီ အချိန်"}, {"Australia_Western", "ဩစတြေးလျ နွေရာသီ အချိန်"}, {"Azerbaijan", "အဇာဘိုင်ဂျန် နွေရာသီ အချိန်"}, {"Azores", "အေဇိုးရီးစ် နွေရာသီ အချိန်"}, {"Bangladesh", "ဘင်္ဂလားဒေ့ရှ် နွေရာသီ အချိန်"}, {"Brasilia", "ဘရာဇီး နွေရာသီ အချိန်"}, {"Cape_Verde", "ကိတ်ဗာဒီ နွေရာသီ အချိန်"}, {"Chatham", "ချားသမ် နွေရာသီ အချိန်"}, {"Chile", "ချီလီ နွေရာသီ အချိန်"}, {"China", "တရုတ် နွေရာသီ အချိန်"}, {"Colombia", "ကိုလံဘီယာ နွေရာသီ အချိန်"}, {"Cook", "ကွတ်ကျွန်းစု နွေရာသီ အချိန်"}, {"Cuba", "ကျူးဘား နွေရာသီ စံတော်ချိန်"}, {"Easter", "အီစတာကျွန်း နွေရာသီ အချိန်"}, {"Europe_Central", "ဥရောပအလယ်ပိုင်း နွေရာသီ အချိန်"}, {"Europe_Eastern", "အရှေ့ဥရောပ နွေရာသီ အချိန်"}, {"Europe_Western", "အနောက်ဥရောပ နွေရာသီ အချိန်"}, {"Falkland", "ဖော့ကလန်ကျွန်းစု နွေရာသီ အချိန်"}, {"Fiji", "ဖီဂျီ နွေရာသီ အချိန်"}, {"Georgia", "ဂျော်ဂျီယာ နွေရာသီ အချိန်"}, {"Greenland_Eastern", "အရှေ့ဂရင်းလန် နွေရာသီ စံတော်ချိန်"}, {"Greenland_Western", "အနောက် ဂရင်းလန် နွေရာသီ စံတော်ချိန်"}, {"Hawaii_Aleutian", "ဟာဝိုင်ယီ အယ်လူးရှန်း နွေရာသီ စံတော်ချိန်"}, {"Hong_Kong", "ဟောင်ကောင် နွေရာသီ အချိန်"}, {"Hovd", "ဟိုးဗ် နွေရာသီ အချိန်"}, {"Iran", "အီရန် နွေရာသီ အချိန်"}, {"Irkutsk", "အီရူခူတ် နွေရာသီ အချိန်"}, {"Israel", "အစ္စရေး နွေရာသီ အချိန်"}, {"Japan", "ဂျပန် နွေရာသီ အချိန်"}, {"Korea", "ကိုရီးယား နွေရာသီ အချိန်"}, {"Krasnoyarsk", "ခရာ့စ်နိုရာစ် နွေရာသီ အချိန်"}, {"Lord_Howe", "လော့ဒ်ဟောင် နွေရာသီ အချိန်"}, {"Macau", "မကာအို နွေရာသီ အချိန်"}, {"Magadan", "မာဂါဒန်း နွေရာသီ အချိန်"}, {"Mauritius", "မောရစ်ရှ နွေရာသီ အချိန်"}, {"Mexico_Pacific", "မက္ကစီကန် ပစိဖိတ် နွေရာသီ စံတော်ချိန်"}, {"Mongolia", "ဥလန်ဘာတော နွေရာသီ အချိန်"}, {"Moscow", "မော်စကို နွေရာသီ အချိန်"}, {"New_Caledonia", "နယူးကာလီဒိုးနီးယား နွေရာသီ အချိန်"}, {"New_Zealand", "နယူးဇီလန် နွေရာသီ အချိန်"}, {"Newfoundland", "နယူးဖောင်လန် နွေရာသီ စံတော်ချိန်"}, {"Norfolk", "နောဖော့ခ်ကျွန်း နွေရာသီ စံတော်ချိန်"}, {"Noronha", "ဖာနန်ဒိုးဒီနိုးရိုးညာ နွေရာသီ အချိန်"}, {"Novosibirsk", "နိုဗိုစဲဘီအဲယ်စ် နွေရာသီ အချိန်"}, {"Omsk", "အွမ်းစ်ခ် နွေရာသီ အချိန်"}, {"Pakistan", "ပါကစ္စတန် နွေရာသီ အချိန်"}, {"Paraguay", "ပါရာဂွေး နွေရာသီ အချိန်"}, {"Peru", "ပီရူး နွေရာသီ အချိန်"}, {"Philippines", "ဖိလစ်ပိုင် နွေရာသီ အချိန်"}, {"Pierre_Miquelon", "စိန့်ပီအဲနှင့် မီခွီလွန် နွေရာသီ စံတော်ချိန်"}, {"Sakhalin", "ဆာခါလင် နွေရာသီ အချိန်"}, {"Samoa", "ဆမိုးအား နွေရာသီ အချိန်"}, {"Taipei", "ထိုင်ပေ နွေရာသီ အချိန်"}, {"Tonga", "တွန်ဂါ နွေရာသီ အချိန်"}, {"Turkey", "တူရကီ နွေရာသီ အချိန်"}, {"Turkmenistan", "တာ့ခ်မင်နစ္စတန် နွေရာသီ အချိန်"}, {"Uruguay", "ဥရုဂွေး နွေရာသီ အချိန်"}, {"Uzbekistan", "ဥဇဘက်ကစ္စတန် နွေရာသီ အချိန်"}, {"Vanuatu", "ဗနွားတူ နွေရာသီ အချိန်"}, {"Vladivostok", "ဗလာဒီဗော့စတော့ခ် နွေရာသီ အချိန်"}, {"Volgograd", "ဗိုဂိုဂရက် နွေရာသီ အချိန်"}, {"Yakutsk", "ယူခူးတ်စ် နွေရာသီ အချိန်"}, {"Yekaterinburg", "ရယ်ခါးတီရင်ဘာခ် နွေရာသီ အချိန်"}},
	},
	{
		{{"Afghanistan", "Афганистанонь шка"}, {"Alaska", "Аляскань свалонь шка"}, {"Amazon", "Амазононь свалонь шка"}, {"Bangladesh", "Бангладешень свалонь шка"}, {"Brasilia", "Базилиянь свалонь шка"}, {"Chile", "Чилинь свалонь шка"}, {"China", "Китаень свалонь шка"}, {"Cuba", "Кубань свалонь шка"}, {"India", "Индиянь свалонь шка"}, {"Iran", "Иранонь свалонь шка"}, {"Japan", "Япониянь свалонь шка"}, {"Kazakhstan_Eastern", "Казахстанонь чилисемань шка"}, {"Kazakhstan_Western", "Казахстанонь чивалгомань шка"}, {"Korea", "Кореань свалонь шка"}, {"Moscow", "Московонь свалонь шка"}, {"New_Zealand", "Од Зеландиянь свалонь шка"}, {"Omsk", "Омскоень свалонь шка"}, {"Paraguay", "Парагваень свалонь шка"}, {"Sakhalin", "Сахалинэнь свалонь шка"}, {"Uruguay", "Уругваень свалонь шка"}, {"Uzbekistan", "Узбекистанонь свалонь шка"}},
		{{"Alaska", "Аляскань кизэнь шка"}, {"Amazon", "Амазононь кизэнь шка"}, {"Bangladesh", "Бангладешень кизэнь шка"}, {"Brasilia", "Базилиянь кизэнь шка"}, {"Chile", "Чилинь кизэнь шка"}, {"China", "Китаень кизэнь шка"}, {"Cuba", "Кубань кизэнь шка"}, {"Iran", "Иранонь кизэнь шка"}, {"Japan", "Япониянь кизэнь шка"}, {"Korea", "Кореань кизэнь шка"}, {"Moscow", "Московонь кизэнь шка"}, {"New_Zealand", "Од Зеландиянь кизэнь шка"}, {"Omsk", "Омскоень кизэнь шка"}, {"Paraguay", "Парагваень кизэнь шка"}, {"Sakhalin", "Сахалинэнь кизэнь шка"}, {"Uruguay", "Уругваень кизэнь шка"}, {"Uzbekistan", "Узбекистанонь кизэнь шка"}},
	},
	{
		{{"Acre", "Acre normaltid"}, {"Afghanistan", "afghansk tid"}, {"Africa_Central", "sentralafrikansk tid"}, {"Africa_Eastern", "østafrikansk tid"}, {"Africa_Southern", "sørafrikansk tid"}, {"Africa_Western", "vestafrikansk tid"}, {"Alaska", "alaskisk normaltid"}, {"Almaty", "Almaty, standardtid"}, {"Amazon", "normaltid for Amazonas"}, {"America_Central", "normaltid for det sentrale Nord-Amerika"}, {"America_Eastern", "normaltid for den nordamerikanske østkysten"}, {"America_Mountain", "normaltid for Rocky Mountains (USA)"}, {"America_Pacific", "normaltid for den nordamerikanske Stillehavskysten"}, {"Anadyr", "Russisk (Anadyr) normaltid"}, {"Apia", "normaltid for Apia"}, {"Aqtau", "Aqtau, standardtid"}, {"Aqtobe", "Aqtobe, standardtid"}, {"Arabian", "arabisk standardtid"}, {"Argentina", "argentinsk normaltid"}, {"Argentina_Western", "vestargentinsk normaltid"}, {"Armenia", "armensk normaltid"}, {"Atlantic", "normaltid for den nordamerikanske atlanterhavskysten"}, {"Australia_Central", "sentralaustralsk normaltid"}, {"Australia_CentralWestern", "vest-sentralaustralsk normaltid"}, {"Australia_Eastern", "østaustralsk normaltid"}, {"Australia_Western", "vestaustralsk normaltid"}, {"Azerbaijan", "aserbajdsjansk normaltid"}, {"Azores", "asorisk normaltid"}, {"Bangladesh", "bangladeshisk normaltid"}, {"Bhutan", "bhutansk tid"}, {"Bolivia", "boliviansk tid"}, {"Brasilia", "normaltid for Brasilia"}, {"Brunei", "tidssone for Brunei Darussalam"}, {"Cape_Verde", "kappverdisk normaltid"}, {"Casey", "Casey-tid"}, {"Chamorro", "tidssone for Chamorro"}, {"Chatham", "normaltid for Chatham"}, {"Chile", "chilensk normaltid"}, {"China", "kinesisk normaltid"}, {"Christmas", "tidssone for Christmasøya"}, {"Cocos", "tidssone for Kokosøyene"}, {"Colombia", "colombiansk normaltid"}, {"Cook", "normaltid for Cookøyene"}, {"Cuba", "cubansk normaltid"}, {"Davis", "tidssone for Davis"}, {"DumontDUrville", "tidssone for Dumont d’Urville"}, {"East_Timor", "østtimoresisk tid"}, {"Easter", "normaltid for Påskeøya"}, {"Ecuador", "ecuadoriansk tid"}, {"Europe_Central", "sentraleuropeisk normaltid"}, {"Europe_Eastern", "østeuropeisk normaltid"}, {"Europe_Further_Eastern", "fjern-østeuropeisk tid"}, {"Europe_Western", "vesteuropeisk normaltid"}, {"Falkland", "normaltid for Falklandsøyene"}, {"Fiji", "fijiansk normaltid"}, {"French_Guiana", "tidssone for Fransk Guyana"}, {"French_Southern", "tidssone for De franske sørterritorier"}, {"GMT", "Greenwich middeltid"}, {"Galapagos", "tidssone for Galápagosøyene"}, {"Gambier", "tidssone for Gambier"}, {"Georgia", "georgisk normaltid"}, {"Gilbert_Islands", "tidssone for Gilbertøyene"}, {"Greenland_Eastern", "østgrønlandsk normaltid"}, {"Greenland_Western", "vestgrønlandsk normaltid"}, {"Guam", "Guam-tid"}, {"Gulf", "tidssone for Persiabukta"}, {"Guyana", "guyansk tid"}, {"Hawaii", "normaltid for Hawaii og Aleutene"}, {"Hawaii_Aleutian", "normaltid for Hawaii og Aleutene"}, {"Hong_Kong", "normaltid for Hongkong"}, {"Hovd", "normaltid for Khovd"}, {"India", "indisk tid"}, {"Indian_Ocean", "tidssone for Indiahavet"}, {"Indochina", "indokinesisk tid"}, {"Indonesia_Central", "sentralindonesisk tid"}, {"Indonesia_Eastern", "østindonesisk tid"}, {"Indonesia_Western", "vestindonesisk tid"}, {"Iran", "iransk normaltid"}, {"Irkutsk", "normaltid for Irkutsk"}, {"Israel", "israelsk normaltid"}, {"Japan", "japansk normaltid"}, {"Kamchatka", "Russisk (Petropavlovsk-Kamtsjatskij) normaltid"}, {"Kazakhstan", "kasakhstansk tid"}, {"Kazakhstan_Eastern", "østkasakhstansk tid"}, {"Kazakhstan_Western", "vestkasakhstansk tid"}, {"Korea", "koreansk normaltid"}, {"Kosrae", "tidssone for Kosrae"}, {"Krasnoyarsk", "normaltid for Krasnojarsk"}, {"Kyrgystan", "kirgisisk tid"}, {"Lanka", "Lanka-tid"}, {"Line_Islands", "tidssone for Linjeøyene"}, {"Lord_Howe", "normaltid for Lord Howe-øya"}, {"Macau", "Macau, standardtid"}, {"Magadan", "normaltid for Magadan"}, {"Malaysia", "malaysisk tid"}, {"Maldives", "maldivisk tid"}, {"Marquesas", "tidssone for Marquesasøyene"}, {"Marshall_Islands", "marshallesisk tid"}, {"Mauritius", "mauritisk normaltid"}, {"Mawson", "tidssone for Mawson"}, {"Mexico_Pacific", "normaltid for den meksikanske Stillehavskysten"}, {"Mongolia", "normaltid for Ulan Bator"}, {"Moscow", "normaltid for Moskva"}, {"Myanmar", "myanmarsk tid"}, {"Nauru", "naurisk tid"}, {"Nepal", "nepalsk tid"}, {"New_Caledonia", "kaledonsk normaltid"}, {"New_Zealand", "newzealandsk normaltid"}, {"Newfoundland", "normaltid for Newfoundland"}, {"Niue", "tidssone for Niue"}, {"Norfolk", "normaltid for Norfolkøya"}, {"Noronha", "normaltid for Fernando de Noronha"}, {"North_Mariana", "Nord-Marianene-tid"}, {"Novosibirsk", "normaltid for Novosibirsk"}, {"Omsk", "normaltid for Omsk"}, {"Pakistan", "pakistansk normaltid"}, {"Palau", "palauisk tid"}, {"Papua_New_Guinea", "papuansk tid"}, {"Paraguay", "paraguayansk normaltid"}, {"Peru", "peruansk normaltid"}, {"Philippines", "filippinsk normaltid"}, {"Phoenix_Islands", "tidssone for Phoenixøyene"}, {"Pierre_Miquelon", "normaltid for Saint-Pierre-et-Miquelon"}, {"Pitcairn", "tidssone for Pitcairn"}, {"Ponape", "tidssone for Pohnpei"}, {"Pyongyang", "tidssone for Pyongyang"}, {"Qyzylorda", "Qyzylorda, standardtid"}, {"Reunion", "tidssone for Réunion"}, {"Rothera", "tidssone for Rothera"}, {"Sakhalin", "normaltid for Sakhalin"}, {"Samara", "Russisk (Samara) normaltid"}, {"Samoa", "samoansk normaltid"}, {"Seychelles", "seychellisk tid"}, {"Singapore", "singaporsk tid"}, {"Solomon", "salomonsk tid"}, {"South_Georgia", "tidssone for Sør-Georgia"}, {"Suriname", "surinamsk tid"}, {"Syowa", "tidssone for Syowa"}, {"Tahiti", "tahitisk tid"}, {"Taipei", "normaltid for Taipei"}, {"Tajikistan", "tadsjikisk tid"}, {"Tokelau", "tidssone for Tokelau"}, {"Tonga", "tongansk normaltid"}, {"Truk", "tidssone for Chuukøyene"}, {"Turkmenistan", "turkmensk normaltid"}, {"Tuvalu", "tuvalsk tid"}, {"Uruguay", "uruguayansk normaltid"}, {"Uzbekistan", "usbekisk normaltid"}, {"Vanuatu", "vanuatisk normaltid"}, {"Venezuela", "venezuelansk tid"}, {"Vladivostok", "normaltid for Vladivostok"}, {"Volgograd", "normaltid for Volgograd"}, {"Vostok", "tidssone for Vostok"}, {"Wake", "tidssone for Wake Island"}, {"Wallis", "tidssone for Wallis- og Futunaøyene"}, {"Yakutsk", "normaltid for Jakutsk"}, {"Yekaterinburg", "normaltid for Jekaterinburg"}, {"Yukon", "tidssone for Yukon"}},
		{{"Acre", "Acre sommertid"}, {"Alaska", "alaskisk sommertid"}, {"Almaty", "Almaty, sommertid"}, {"Amazon", "sommertid for Amazonas"}, {"America_Central", "sommertid for det sentrale Nord-Amerika"}, {"America_Eastern", "sommertid for den nordamerikanske østkysten"}, {"America_Mountain", "sommertid for Rocky Mountains (USA)"}, {"America_Pacific", "sommertid for den nordamerikanske Stillehavskysten"}, {"Anadyr", "Russisk (Anadyr) sommertid"}, {"Apia", "sommertid for Apia"}, {"Aqtau", "Aqtau, sommertid"}, {"Aqtobe", "Aqtobe, sommertid"}, {"Arabian", "arabisk sommertid"}, {"Argentina", "argentinsk sommertid"}, {"Argentina_Western", "vestargentinsk sommertid"}, {"Armenia", "armensk sommertid"}, {"Atlantic", "sommertid for den nordamerikanske atlanterhavskysten"}, {"Australia_Central", "sentralaustralsk sommertid"}, {"Australia_CentralWestern", "vest-sentralaustralsk sommertid"}, {"Australia_Eastern", "østaustralsk sommertid"}, {"Australia_Western", "vestaustralsk sommertid"}, {"Azerbaijan", "aserbajdsjansk sommertid"}, {"Azores", "asorisk sommertid"}, {"Bangladesh", "bangladeshisk sommertid"}, {"Brasilia", "sommertid for Brasilia"}, {"Cape_Verde", "kappverdisk sommertid"}, {"Chatham", "sommertid for Chatham"}, {"Chile", "chilensk sommertid"}, {"China", "kinesisk sommertid"}, {"Colombia", "colombiansk sommertid"}, {"Cook", "halv sommertid for Cookøyene"}, {"Cuba", "cubansk sommertid"}, {"Easter", "sommertid for Påskeøya"}, {"Europe_Central", "sentraleuropeisk sommertid"}, {"Europe_Eastern", "østeuropeisk sommertid"}, {"Europe_Western", "vesteuropeisk sommertid"}, {"Falkland", "sommertid for Falklandsøyene"}, {"Fiji", "fijiansk sommertid"}, {"Georgia", "georgisk sommertid"}, {"Greenland_Eastern", "østgrønlandsk sommertid"}, {"Greenland_Western", "vestgrønlandsk sommertid"}, {"Hawaii_Aleutian", "sommertid for Hawaii og Aleutene"}, {"Hong_Kong", "sommertid for Hongkong"}, {"Hovd", "sommertid for Khovd"}, {"Iran", "iransk sommertid"}, {"Irkutsk", "sommertid for Irkutsk"}, {"Israel", "israelsk sommertid"}, {"Japan", "japansk sommertid"}, {"Kamchatka", "Russisk (Petropavlovsk-Kamtsjatskij) sommertid"}, {"Korea", "koreansk sommertid"}, {"Krasnoyarsk", "sommertid for Krasnojarsk"}, {"Lord_Howe", "sommertid for Lord Howe-øya"}, {"Macau", "Macau, sommertid"}, {"Magadan", "sommertid for Magadan"}, {"Mauritius", "mauritisk sommertid"}, {"Mexico_Pacific", "sommertid for den meksikanske Stillehavskysten"}, {"Mongolia", "sommertid for Ulan Bator"}, {"Moscow", "sommertid for Moskva"}, {"New_Caledonia", "kaledonsk sommertid"}, {"New_Zealand", "newzealandsk sommertid"}, {"Newfoundland", "sommertid for Newfoundland"}, {"Norfolk", "sommertid for Norfolkøya"}, {"Noronha", "sommertid for Fernando de Noronha"}, {"Novosibirsk", "sommertid for Novosibirsk"}, {"Omsk", "sommertid for Omsk"}, {"Pakistan", "pakistansk sommertid"}, {"Paraguay", "paraguayansk sommertid"}, {"Peru", "peruansk sommertid"}, {"Philippines", "filippinsk sommertid"}, {"Pierre_Miquelon", "sommertid for Saint-Pierre-et-Miquelon"}, {"Qyzylorda", "Qyzylorda, sommertid"}, {"Sakhalin", "sommertid for Sakhalin"}, {"Samara", "Russisk (Samara) sommertid"}, {"Samoa", "samoansk sommertid"}, {"Taipei", "sommertid for Taipei"}, {"Tonga", "tongansk sommertid"}, {"Turkmenistan", "turkmensk sommertid"}, {"Uruguay", "uruguayansk sommertid"}, {"Uzbekistan", "usbekisk sommertid"}, {"Vanuatu", "vanuatisk sommertid"}, {"Vladivostok", "sommertid for Vladivostok"}, {"Volgograd", "sommertid for Volgograd"}, {"Yakutsk", "sommertid for Jakutsk"}, {"Yekaterinburg", "sommertid for Jekaterinburg"}},
	},
	{
		{{"Africa_Central", "Zentraalafrikaansch Tiet"}, {"Africa_Eastern", "Oostafrikaansch Tiet"}, {"Africa_Southern", "Söödafrikaansch Tiet"}, {"Africa_Western", "Westafrikaansch Tiet"}, {"America_Central", "Noordamerikaansch zentraal Standardtiet"}, {"America_Eastern", "Noordamerikaansch oosten Standardtiet"}, {"America_Mountain", "Noordamerikaansch Barg-Standardtiet"}, {"America_Pacific", "Noordamerikaansch Pazifik-Standardtiet"}, {"Arabian", "Araabsch Standardtiet"}, {"Atlantic", "Noordamerikaansch Atlantik-Standardtiet"}, {"Australia_Central", "Zentraalaustraalsch Standardtiet"}, {"Australia_CentralWestern", "Westzentraalaustraalsch Standardtiet"}, {"Australia_Eastern", "Oostaustraalsch Standardtiet"}, {"Australia_Western", "Westaustraalsch Standardtiet"}, {"China", "China-Standardtiet"}, {"Europe_Central", "Zentraaleuropääsch Standardtiet"}, {"Europe_Eastern", "Oosteuropääsch Standardtiet"}, {"Europe_Western", "Westeuropääsch Standardtiet"}, {"GMT", "Gröönwisch-Welttiet"}, {"India", "Indien-Tiet"}, {"Indochina", "Söödoostasiaatsch Tiet"}, {"Indonesia_Central", "Indoneesch Zentraaltiet"}, {"Indonesia_Eastern", "Oostindoneesch Tiet"}, {"Indonesia_Western", "Westindoneesch Tiet"}, {"Israel", "Israel-Standardtiet"}, {"Japan", "Japaansch Standardtiet"}, {"Korea", "Koreaansch Standardtiet"}, {"Moscow", "Moskau-Standardtiet"}},
		{{"America_Central", "Noordamerikaansch zentraal Summertiet"}, {"America_Eastern", "Noordamerikaansch oosten Summertiet"}, {"America_Mountain", "Noordamerikaansch Barg-Summertiet"}, {"America_Pacific", "Noordamerikaansch Pazifik-Summertiet"}, {"Arabian", "Araabsch Summertiet"}, {"Atlantic", "Noordamerikaansch Atlantik-Summertiet"}, {"Australia_Central", "Zentraalaustraalsch Summertiet"}, {"Australia_CentralWestern", "Westzentraalaustraalsch Summertiet"}, {"Australia_Eastern", "Oostaustraalsch Summertiet"}, {"Australia_Western", "Westaustraalsch Summertiet"}, {"China", "China-Summertiet"}, {"Europe_Central", "Zentraaleuropääsch Summertiet"}, {"Europe_Eastern", "Oosteuropääsch Summertiet"}, {"Europe_Western", "Westeuropääsch Summertiet"}, {"Israel", "Israel-Summertiet"}, {"Japan", "Japaansch Summertiet"}, {"Korea", "Koreaansch Summertiet"}, {"Moscow", "Moskau-Summertiet"}},
	},
	{
		{{"Afghanistan", "अफगानिस्तान समय"}, {"Africa_Central", "केन्द्रीय अफ्रिकी समय"}, {"Africa_Eastern", "पूर्वी अफ्रिकी समय"}, {"Africa_Southern", "दक्षिण अफ्रिकी समय"}, {"Africa_Western", "पश्चिम अफ्रिकी समय"}, {"Alaska", "अलस्काको मानक समय"}, {"Amazon", "एमाजोन मानक समय"}, {"America_Central", "केन्द्रीय मानक समय"}, {"America_Eastern", "पूर्वी मानक समय"}, {"America_Mountain", "हिमाली मानक समय"}, {"America_Pacific", "प्यासिफिक मानक समय"}, {"Apia", "आपिया मानक समय"}, {"Arabian", "अरबी मानक समय"}, {"Argentina", "अर्जेनटिनी मानक समय"}, {"Argentina_Western", "पश्चिमी अर्जेनटिनी मानक समय"}, {"Armenia", "अर्मेनिया मानक समय"}, {"Atlantic", "एट्लान्टिक मानक समय"}, {"Australia_Central", "केन्द्रीय अस्ट्रेलिया मानक समय"}, {"Australia_CentralWestern", "केन्द्रीय पश्चिमी अस्ट्रेलिया मानक समय"}, {"Australia_Eastern", "पूर्वी अस्ट्रेलिया मानक समय"}, {"Australia_Western", "पश्चिमी अस्ट्रेलिया मानक समय"}, {"Azerbaijan", "अजरबैजान मानक समय"}, {"Azores", "एजोरेस् मानक समय"}, {"Bangladesh", "बंगलादेशी मानक समय"}, {"Bhutan", "भुटानी समय"}, {"Bolivia", "बोलिभिया समय"}, {"Brasilia", "ब्राजिलिया मानक समय"}, {"Brunei", "ब्रुनाइ दारूस्सलम समय"}, {"Cape_Verde", "केप भर्दे मानक समय"}, {"Chamorro", "चामोर्रो मानक समय"}, {"Chatham", "चाथाम मानक समय"}, {"Chile", "चिली मानक समय"}, {"China", "चीन मानक समय"}, {"Christmas", "क्रिस्मस टापु समय"}, {"Cocos", "कोकोस टापु समय"}, {"Colombia", "कोलम्बियाली मानक समय"}, {"Cook", "कुक टापु मानक समय"}, {"Cuba", "क्यूबाको मानक समय"}, {"Davis", "डेभिस समय"}, {"DumontDUrville", "डुमोन्ट-डी‘ उर्भिले समय"}, {"East_Timor", "पूर्वी टिमोर समय"}, {"Easter", "इस्टर टापू मानक समय"}, {"Ecuador", "ईक्वोडोर समय"}, {"Europe_Central", "केन्द्रीय युरोपेली मानक समय"}, {"Europe_Eastern", "पूर्वी युरोपेली मानक समय"}, {"Europe_Further_Eastern", "थप-पूर्वी युरोपेली समय"}, {"Europe_Western", "पश्चिमी युरोपेली मानक समय"}, {"Falkland", "फल्कल्यान्ड टापू मानक समय"}, {"Fiji", "फिजी मानक समय"}, {"French_Guiana", "फ्रेन्च ग्वाना समय"}, {"French_Southern", "फ्रेन्च दक्षिणी र अन्टार्टिक समय"}, {"GMT", "ग्रीनविच मिन समय"}, {"Galapagos", "गालापागोस् समय"}, {"Gambier", "ग्याम्बियर समय"}, {"Georgia", "जर्जिया मानक समय"}, {"Gilbert_Islands", "गिल्बर्ट टापु समय"}, {"Greenland_Eastern", "पूर्वी ग्रीनल्यान्डको मानक समय"}, {"Greenland_Western", "पश्चिमी ग्रीनल्यान्डको मानक समय"}, {"Gulf", "खाडी मानक समय"}, {"Guyana", "गुयाना समय"}, {"Hawaii", "हवाई-एलुटियन मानक समय"}, {"Hawaii_Aleutian", "हवाई-एलुटियन मानक समय"}, {"Hong_Kong", "हङकङ मानक समय"}, {"Hovd", "होब्ड मानक समय"}, {"India", "भारतीय मानक समय"}, {"Indian_Ocean", "हिन्द महासागर समय"}, {"Indochina", "इन्डोचाइना समय"}, {"Indonesia_Central", "केन्द्रीय इन्डोनेशिया समय"}, {"Indonesia_Eastern", "पूर्वी इन्डोनेशिया समय"}, {"Indonesia_Western", "पश्चिमी इन्डोनेशिया समय"}, {"Iran", "इरानी मानक समय"}, {"Irkutsk", "ईर्कुट्स्क मानक समय"}, {"Israel", "इजरायल मानक समय"}, {"Japan", "जापान मानक समय"}, {"Kazakhstan", "काजकस्तानको समय"}, {"Kazakhstan_Eastern", "पूर्वी काजकस्तान समय"}, {"Kazakhstan_Western", "पश्चिम काजकस्तान समय"}, {"Korea", "कोरियाली मानक समय"}, {"Kosrae", "कोसराए समय"}, {"Krasnoyarsk", "क्रासनोयार्क मानक समय"}, {"Kyrgystan", "किर्गिस्तान समय"}, {"Line_Islands", "लाइन टापु समय"}, {"Lord_Howe", "लर्ड हावे मानक समय"}, {"Magadan", "मागादान मानक समय"}, {"Malaysia", "मलेसिया समय"}, {"Maldives", "माल्दिभ्स समय"}, {"Marquesas", "मार्किसस समय"}, {"Marshall_Islands", "मार्शल टापु समय"}, {"Mauritius", "मउरिटस मानक समय"}, {"Mawson", "म्वसन समय"}, {"Mexico_Pacific", "मेक्सिकन प्यासिफिक मानक समय"}, {"Mongolia", "उलान बाटोर मानक समय"}, {"Moscow", "मस्को मानक समय"}, {"Myanmar", "म्यानमार समय"}, {"Nauru", "नाउरु समय"}, {"Nepal", "नेपाली समय"}, {"New_Caledonia", "नयाँ कालेदोनिया मानक समय"}, {"New_Zealand", "न्यूजिल्यान्ड मानक समय"}, {"Newfoundland", "न्यूफाउनडल्यान्डको मानक समय"}, {"Niue", "निउए समय"}, {"Norfolk", "नोर्फोक टापूको मानक समय"}, {"Noronha", "फर्नान्डो डे नोरोन्हा मानक समय"}, {"Novosibirsk", "नोभोसिविर्स्क मानक समय"}, {"Omsk", "ओम्स्क मानक समय"}, {"Pakistan", "पाकिस्तानी मानक समय"}, {"Palau", "पालाउ समय"}, {"Papua_New_Guinea", "पपूवा न्यू गिनी समय"}, {"Paraguay", "पाराग्वे मानक समय"}, {"Peru", "पेरू मानक समय"}, {"Philippines", "फिलिपिनी मानक समय"}, {"Phoenix_Islands", "फिनिक्स टापु समय"}, {"Pierre_Miquelon", "सेन्ट पियर्रे र मिक्युलोनको मानक समय"}, {"Pitcairn", "पिटकैरण समय"}, {"Ponape", "पोनापे समय"}, {"Pyongyang", "प्योङयाङ समय"}, {"Reunion", "रियुनियन समय"}, {"Rothera", "रोथेरा समय"}, {"Sakhalin", "साखालिन मानक समय"}, {"Samoa", "सामोअ मानक समय"}, {"Seychelles", "सेयेचेलास् समय"}, {"Singapore", "सिंगापुर मानक समय"}, {"Solomon", "सोलोमोन टापु समय"}, {"South_Georgia", "दक्षिण जर्जिया समय"}, {"Suriname", "सुरिनामा समय"}, {"Syowa", "स्योवा समय"}, {"Tahiti", "ताहिती समय"}, {"Taipei", "ताइपेइ मानक समय"}, {"Tajikistan", "ताजिकस्तान समय"}, {"Tokelau", "तोकेलाउ समय"}, {"Tonga", "टोंगा मानक समय"}, {"Truk", "चुउक समय"}, {"Turkmenistan", "तुर्कमेनिस्तान मानक समय"}, {"Tuvalu", "टुभालु समय"}, {"Uruguay", "उरूग्वे मानक समय"}, {"Uzbekistan", "उज्बेकिस्तान मानक समय"}, {"Vanuatu", "भानुआतु मानक समय"}, {"Venezuela", "भेनेज्युएला समय"}, {"Vladivostok", "भ्लादिभास्टोक मानक समय"}, {"Volgograd", "भोल्गाग्राद मानक समय"}, {"Vostok", "भास्टोक समय"}, {"Wake", "वेक टापु समय"}, {"Wallis", "वालिस् र फुटुना समय"}, {"Yakutsk", "याकुस्ट मानक समय"}, {"Yekaterinburg", "येकाटेरिनबर्ग मानक समय"}, {"Yukon", "युकोनको समय"}},
		{{"Alaska", "अलस्काको दिवा समय"}, {"Amazon", "एमाजोन ग्रीष्मकालीन समय"}, {"America_Central", "केन्द्रीय दिवा समय"}, {"America_Eastern", "पूर्वी दिवा समय"}, {"America_Mountain", "हिमाली दिवा समय"}, {"America_Pacific", "प्यासिफिक दिवा समय"}, {"Apia", "आपिया दिवा समय"}, {"Arabian", "अरबी दिवा समय"}, {"Argentina", "अर्जेनटिनी ग्रीष्मकालीन समय"}, {"Argentina_Western", "पश्चिमी अर्जेनटिनी ग्रीष्मकालीन समय"}, {"Armenia", "अर्मेनिया ग्रीष्मकालीन समय"}, {"Atlantic", "एट्लान्टिक दिवा समय"}, {"Australia_Central", "केन्द्रीय अस्ट्रेलिया दिवा समय"}, {"Australia_CentralWestern", "केन्द्रीय पश्चिमी अस्ट्रेलिया दिवा समय"}, {"Australia_Eastern", "पूर्वी अस्ट्रेलिया दिवा समय"}, {"Australia_Western", "पश्चिमी अस्ट्रेलिया दिवा समय"}, {"Azerbaijan", "अजरबैजान ग्रीष्मकालीन समय"}, {"Azores", "एजोरेस् ग्रीष्मकालीन समय"}, {"Bangladesh", "बंगलादेशी ग्रीष्मकालीन समय"}, {"Brasilia", "ब्राजिलीया ग्रीष्मकालीन समय"}, {"Cape_Verde", "केप भर्दे ग्रीष्मकालीन समय"}, {"Chatham", "चाथाम दिवा समय"}, {"Chile", "चिली ग्रीष्मकालीन समय"}, {"China", "चीन दिवा समय"}, {"Colombia", "कोलम्बियाली ग्रीष्मकालीन समय"}, {"Cook", "कुक टापु आधा ग्रीष्मकालीन समय"}, {"Cuba", "क्यूबाको दिवा समय"}, {"Easter", "इस्टर टापू ग्रीष्म समय"}, {"Europe_Central", "केन्द्रीय युरोपेली ग्रीष्मकालीन समय"}, {"Europe_Eastern", "पूर्वी युरोपेली ग्रीष्मकालीन समय"}, {"Europe_Western", "युरोपेली ग्रीष्मकालीन समय"}, {"Falkland", "फल्कल्यान्ड टापू ग्रीष्मकालीन समय"}, {"Fiji", "फिजी ग्रीष्मकालीन समय"}, {"Georgia", "जर्जिया ग्रीष्मकालीन समय"}, {"Greenland_Eastern", "पूर्वी ग्रीनल्यान्डको ग्रीष्मकालीन समय"}, {"Greenland_Western", "पश्चिमी ग्रीनल्यान्डको ग्रीष्मकालीन समय"}, {"Hawaii_Aleutian", "हवाई-एलुटियन दिवा समय"}, {"Hong_Kong", "हङकङ ग्रीष्मकालीन समय"}, {"Hovd", "होब्ड ग्रीष्मकालीन समय"}, {"Iran", "इरानी दिवा समय"}, {"Irkutsk", "ईर्कुट्स्क ग्रीष्मकालीन समय"}, {"Israel", "इजरायल दिवा समय"}, {"Japan", "जापान दिवा समय"}, {"Korea", "कोरियाली दिवा समय"}, {"Krasnoyarsk", "क्रासनोयार्क ग्रीष्मकालीन समय"}, {"Lord_Howe", "लर्ड हावे दिवा समय"}, {"Magadan", "मागादान ग्रीष्मकालीन समय"}, {"Mauritius", "मउरिटस ग्रीष्मकालीन समय"}, {"Mexico_Pacific", "मेक्सिकन प्यासिफिक दिवा समय"}, {"Mongolia", "उलान बाटोर ग्रीष्मकालीन समय"}, {"Moscow", "मस्को ग्रीष्मकालीन समय"}, {"New_Caledonia", "नयाँ कालेदोनिया ग्रीष्मकालीन समय"}, {"New_Zealand", "न्यूजिल्यान्ड दिवा समय"}, {"Newfoundland", "न्यूफाउनल्यान्डको दिवा समय"}, {"Norfolk", "नोर्फोक टापूको ग्रीष्मकालीन समय"}, {"Noronha", "फर्नान्डो डे नोरोन्हा ग्रीष्मकालीन समय"}, {"Novosibirsk", "नोभोसिविर्स्क ग्रीष्मकालीन समय"}, {"Omsk", "ओम्स्क ग्रीष्मकालीन समय"}, {"Pakistan", "पाकिस्तानी ग्रीष्मकालीन समय"}, {"Paraguay", "पाराग्वे ग्रीष्मकालीन समय"}, {"Peru", "पेरु ग्रीष्मकालीन समय"}, {"Philippines", "फिलिपिनी ग्रीष्मकालीन समय"}, {"Pierre_Miquelon", "सेन्ट पियर्रे र मिक्युलोनको दिवा समय"}, {"Sakhalin", "साखालिन ग्रीष्मकालीन समय"}, {"Samoa", "सामोअ दिवा समय"}, {"Taipei", "ताइपेइ दिवा समय"}, {"Tonga", "टोंगा ग्रीष्मकालीन समय"}, {"Turkmenistan", "तुर्कमेनिस्तान ग्रीष्मकालीन मानक समय"}, {"Uruguay", "उरुग्वे ग्रीष्मकालीन समय"}, {"Uzbekistan", "उज्बेकिस्तान ग्रीष्मकालीन समय"}, {"Vanuatu", "भानुआतु ग्रीष्मकालीन समय"}, {"Vladivostok", "भ्लादिभास्टोक ग्रीष्मकालीन समय"}, {"Volgograd", "भोल्गाग्राद ग्रीष्मकालीन समय"}, {"Yakutsk", "याकुस्ट ग्रीष्मकालीन समय"}, {"Yekaterinburg", "येकाटेरिनबर्ग ग्रीष्मकालीन समय"}},
	},
	{
		{{"Acre", "Acre-standaardtijd"}, {"Afghanistan", "Afghaanse tijd"}, {"Africa_Central", "Centraal-Afrikaanse tijd"}, {"Africa_Eastern", "Oost-Afrikaanse tijd"}, {"Africa_Southern", "Zuid-Afrikaanse tijd"}, {"Africa_Western", "West-Afrikaanse tijd"}, {"Alaska", "Alaska-standaardtijd"}, {"Almaty", "Alma-Ata-standaardtijd"}, {"Amazon", "Amazone-standaardtijd"}, {"America_Central", "Central-standaardtijd"}, {"America_Eastern", "Eastern-standaardtijd"}, {"America_Mountain", "Mountain-standaardtijd"}, {"America_Pacific", "Pacific-standaardtijd"}, {"Anadyr", "Anadyr-standaardtijd"}, {"Apia", "Apia-standaardtijd"}, {"Aqtau", "Aqtau-standaardtijd"}, {"Aqtobe", "Aqtöbe-standaardtijd"}, {"Arabian", "Arabische standaardtijd"}, {"Argentina", "Argentijnse standaardtijd"}, {"Argentina_Western", "West-Argentijnse standaardtijd"}, {"Armenia", "Armeense standaardtijd"}, {"Atlantic", "Atlantic-standaardtijd"}, {"Australia_Central", "Midden-Australische standaardtijd"}, {"Australia_CentralWestern", "Midden-Australische westelijke standaardtijd"}, {"Australia_Eastern", "Oost-Australische standaardtijd"}, {"Australia_Western", "West-Australische standaardtijd"}, {"Azerbaijan", "Azerbeidzjaanse standaardtijd"}, {"Azores", "Azoren-standaardtijd"}, {"Bangladesh", "Bengalese standaardtijd"}, {"Bhutan", "Bhutaanse tijd"}, {"Bolivia", "Boliviaanse tijd"}, {"Brasilia", "Braziliaanse standaardtijd"}, {"Brunei", "Bruneise tijd"}, {"Cape_Verde", "Kaapverdische standaardtijd"}, {"Casey", "Casey tijd"}, {"Chamorro", "Chamorro-tijd"}, {"Chatham", "Chatham-standaardtijd"}, {"Chile", "Chileense standaardtijd"}, {"China", "Chinese standaardtijd"}, {"Christmas", "Christmaseilandse tijd"}, {"Cocos", "Cocoseilandse tijd"}, {"Colombia", "Colombiaanse standaardtijd"}, {"Cook", "Cookeilandse standaardtijd"}, {"Cuba", "Cubaanse standaardtijd"}, {"Davis", "Davis-tijd"}, {"DumontDUrville", "Dumont-d’Urville-tijd"}, {"East_Timor", "Oost-Timorese tijd"}, {"Easter", "Paaseilandse standaardtijd"}, {"Ecuador", "Ecuadoraanse tijd"}, {"Europe_Central", "Midden-Europese standaardtijd"}, {"Europe_Eastern", "Oost-Europese standaardtijd"}, {"Europe_Further_Eastern", "Verder-oostelijk-Europese tijd"}, {"Europe_Western", "West-Europese standaardtijd"}, {"Falkland", "Falklandeilandse standaardtijd"}, {"Fiji", "Fijische standaardtijd"}, {"French_Guiana", "Frans-Guyaanse tijd"}, {"French_Southern", "Franse zuidelijke en Antarctische tijd"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagoseilandse tijd"}, {"Gambier", "Gambiereilandse tijd"}, {"Georgia", "Georgische standaardtijd"}, {"Gilbert_Islands", "Gilberteilandse tijd"}, {"Greenland", "Groenlandse standaardtijd"}, {"Greenland_Eastern", "Oost-Groenlandse standaardtijd"}, {"Greenland_Western", "West-Groenlandse standaardtijd"}, {"Guam", "Guamese standaardtijd"}, {"Gulf", "Golf-standaardtijd"}, {"Guyana", "Guyaanse tijd"}, {"Hawaii", "Hawaii-Aleoetische standaardtijd"}, {"Hawaii_Aleutian", "Hawaii-Aleoetische standaardtijd"}, {"Hong_Kong", "Hongkongse standaardtijd"}, {"Hovd", "Hovd-standaardtijd"}, {"India", "Indiase tijd"}, {"Indian_Ocean", "Indische Oceaan-tijd"}, {"Indochina", "Indochinese tijd"}, {"Indonesia_Central", "Centraal-Indonesische tijd"}, {"Indonesia_Eastern", "Oost-Indonesische tijd"}, {"Indonesia_Western", "West-Indonesische tijd"}, {"Iran", "Iraanse standaardtijd"}, {"Irkutsk", "Irkoetsk-standaardtijd"}, {"Israel", "Israëlische standaardtijd"}, {"Japan", "Japanse standaardtijd"}, {"Kamchatka", "Petropavlovsk-Kamtsjatski-standaardtijd"}, {"Kazakhstan", "Kazachse tijd"}, {"Kazakhstan_Eastern", "Oost-Kazachse tijd"}, {"Kazakhstan_Western", "West-Kazachse tijd"}, {"Korea", "Koreaanse standaardtijd"}, {"Kosrae", "Kosraese tijd"}, {"Krasnoyarsk", "Krasnojarsk-standaardtijd"}, {"Kyrgystan", "Kirgizische tijd"}, {"Lanka", "Lanka-tijd"}, {"Line_Islands", "Line-eilandse tijd"}, {"Lord_Howe", "Lord Howe-eilandse standaardtijd"}, {"Macau", "Macause standaardtijd"}, {"Magadan", "Magadan-standaardtijd"}, {"Malaysia", "Maleisische tijd"}, {"Maldives", "Maldivische tijd"}, {"Marquesas", "Marquesaseilandse tijd"}, {"Marshall_Islands", "Marshalleilandse tijd"}, {"Mauritius", "Mauritiaanse standaardtijd"}, {"Mawson", "Mawson-tijd"}, {"Mexico_Pacific", "Mexicaanse Pacific-standaardtijd"}, {"Mongolia", "Ulaanbaatar-standaardtijd"}, {"Moscow", "Moskou-standaardtijd"}, {"Myanmar", "Myanmarese tijd"}, {"Nauru", "Nauruaanse tijd"}, {"Nepal", "Nepalese tijd"}, {"New_Caledonia", "Nieuw-Caledonische standaardtijd"}, {"New_Zealand", "Nieuw-Zeelandse standaardtijd"}, {"Newfoundland", "Newfoundland-standaardtijd"}, {"Niue", "Niuese tijd"}, {"Norfolk", "Norfolkeilandse standaardtijd"}, {"Noronha", "Fernando de Noronha-standaardtijd"}, {"North_Mariana", "Noordelijk Mariaanse tijd"}, {"Novosibirsk", "Novosibirsk-standaardtijd"}, {"Omsk", "Omsk-standaardtijd"}, {"Pakistan", "Pakistaanse standaardtijd"}, {"Palau", "Belause tijd"}, {"Papua_New_Guinea", "Papoea-Nieuw-Guineese tijd"}, {"Paraguay", "Paraguayaanse standaardtijd"}, {"Peru", "Peruaanse standaardtijd"}, {"Philippines", "Filipijnse standaardtijd"}, {"Phoenix_Islands", "Phoenixeilandse tijd"}, {"Pierre_Miquelon", "Saint Pierre en Miquelon-standaardtijd"}, {"Pitcairn", "Pitcairneilandse tijd"}, {"Ponape", "Pohnpei-tijd"}, {"Pyongyang", "Pyongyang-tijd"}, {"Qyzylorda", "Qyzylorda-standaardtijd"}, {"Reunion", "Réunionse tijd"}, {"Rothera", "Rothera-tijd"}, {"Sakhalin", "Sachalin-standaardtijd"}, {"Samara", "Samara-standaardtijd"}, {"Samoa", "Samoaanse standaardtijd"}, {"Seychelles", "Seychelse tijd"}, {"Singapore", "Singaporese standaardtijd"}, {"Solomon", "Salomonseilandse tijd"}, {"South_Georgia", "Zuid-Georgische tijd"}, {"Suriname", "Surinaamse tijd"}, {"Syowa", "Syowa-tijd"}, {"Tahiti", "Tahitiaanse tijd"}, {"Taipei", "Taipei-standaardtijd"}, {"Tajikistan", "Tadzjiekse tijd"}, {"Tokelau", "Tokelau-eilandse tijd"}, {"Tonga", "Tongaanse standaardtijd"}, {"Truk", "Chuukse tijd"}, {"Turkey", "Turkse standaardtijd"}, {"Turkmenistan", "Turkmeense standaardtijd"}, {"Tuvalu", "Tuvaluaanse tijd"}, {"Uruguay", "Uruguayaanse standaardtijd"}, {"Uzbekistan", "Oezbeekse standaardtijd"}, {"Vanuatu", "Vanuatuaanse standaardtijd"}, {"Venezuela", "Venezolaanse tijd"}, {"Vladivostok", "Vladivostok-standaardtijd"}, {"Volgograd", "Wolgograd-standaardtijd"}, {"Vostok", "Vostok-tijd"}, {"Wake", "Wake-eilandse tijd"}, {"Wallis", "Wallis en Futunase tijd"}, {"Yakutsk", "Jakoetsk-standaardtijd"}, {"Yekaterinburg", "Jekaterinenburg-standaardtijd"}, {"Yukon", "Yukon-tijd"}},
		{{"Acre", "Acre-zomertijd"}, {"Alaska", "Alaska-zomertijd"}, {"Almaty", "Alma-Ata-zomertijd"}, {"Amazon", "Amazone-zomertijd"}, {"America_Central", "Central-zomertijd"}, {"America_Eastern", "Eastern-zomertijd"}, {"America_Mountain", "Mountain-zomertijd"}, {"America_Pacific", "Pacific-zomertijd"}, {"Anadyr", "Anadyr-zomertijd"}, {"Apia", "Apia-zomertijd"}, {"Aqtau", "Aqtau-zomertijd"}, {"Aqtobe", "Aqtöbe-zomertijd"}, {"Arabian", "Arabische zomertijd"}, {"Argentina", "Argentijnse zomertijd"}, {"Argentina_Western", "West-Argentijnse zomertijd"}, {"Armenia", "Armeense zomertijd"}, {"Atlantic", "Atlantic-zomertijd"}, {"Australia_Central", "Midden-Australische zomertijd"}, {"Australia_CentralWestern", "Midden-Australische westelijke zomertijd"}, {"Australia_Eastern", "Oost-Australische zomertijd"}, {"Australia_Western", "West-Australische zomertijd"}, {"Azerbaijan", "Azerbeidzjaanse zomertijd"}, {"Azores", "Azoren-zomertijd"}, {"Bangladesh", "Bengalese zomertijd"}, {"Brasilia", "Braziliaanse zomertijd"}, {"Cape_Verde", "Kaapverdische zomertijd"}, {"Chatham", "Chatham-zomertijd"}, {"Chile", "Chileense zomertijd"}, {"China", "Chinese zomertijd"}, {"Colombia", "Colombiaanse zomertijd"}, {"Cook", "Cookeilandse halve zomertijd"}, {"Cuba", "Cubaanse zomertijd"}, {"Easter", "Paaseilandse zomertijd"}, {"Europe_Central", "Midden-Europese zomertijd"}, {"Europe_Eastern", "Oost-Europese zomertijd"}, {"Europe_Western", "West-Europese zomertijd"}, {"Falkland", "Falklandeilandse zomertijd"}, {"Fiji", "Fijische zomertijd"}, {"Georgia", "Georgische zomertijd"}, {"Greenland", "Groenlandse zomertijd"}, {"Greenland_Eastern", "Oost-Groenlandse zomertijd"}, {"Greenland_Western", "West-Groenlandse zomertijd"}, {"Hawaii_Aleutian", "Hawaii-Aleoetische zomertijd"}, {"Hong_Kong", "Hongkongse zomertijd"}, {"Hovd", "Hovd-zomertijd"}, {"Iran", "Iraanse zomertijd"}, {"Irkutsk", "Irkoetsk-zomertijd"}, {"Israel", "Israëlische zomertijd"}, {"Japan", "Japanse zomertijd"}, {"Kamchatka", "Petropavlovsk-Kamtsjatski-zomertijd"}, {"Korea", "Koreaanse zomertijd"}, {"Krasnoyarsk", "Krasnojarsk-zomertijd"}, {"Lord_Howe", "Lord Howe-eilandse zomertijd"}, {"Macau", "Macause zomertijd"}, {"Magadan", "Magadan-zomertijd"}, {"Mauritius", "Mauritiaanse zomertijd"}, {"Mexico_Pacific", "Mexicaanse Pacific-zomertijd"}, {"Mongolia", "Ulaanbaatar-zomertijd"}, {"Moscow", "Moskou-zomertijd"}, {"New_Caledonia", "Nieuw-Caledonische zomertijd"}, {"New_Zealand", "Nieuw-Zeelandse zomertijd"}, {"Newfoundland", "Newfoundland-zomertijd"}, {"Norfolk", "Norfolkeilandse zomertijd"}, {"Noronha", "Fernando de Noronha-zomertijd"}, {"Novosibirsk", "Novosibirsk-zomertijd"}, {"Omsk", "Omsk-zomertijd"}, {"Pakistan", "Pakistaanse zomertijd"}, {"Paraguay", "Paraguayaanse zomertijd"}, {"Peru", "Peruaanse zomertijd"}, {"Philippines", "Filipijnse zomertijd"}, {"Pierre_Miquelon", "Saint Pierre en Miquelon-zomertijd"}, {"Qyzylorda", "Qyzylorda-zomertijd"}, {"Sakhalin", "Sachalin-zomertijd"}, {"Samara", "Samara-zomertijd"}, {"Samoa", "Samoaanse zomertijd"}, {"Taipei", "Taipei-zomertijd"}, {"Tonga", "Tongaanse zomertijd"}, {"Turkey", "Turkse zomertijd"}, {"Turkmenistan", "Turkmeense zomertijd"}, {"Uruguay", "Uruguayaanse zomertijd"}, {"Uzbekistan", "Oezbeekse zomertijd"}, {"Vanuatu", "Vanuatuaanse zomertijd"}, {"Vladivostok", "Vladivostok-zomertijd"}, {"Volgograd", "Wolgograd-zomertijd"}, {"Yakutsk", "Jakoetsk-zomertijd"}, {"Yekaterinburg", "Jekaterinenburg-zomertijd"}},
	},
	{
		{{"Acre", "Acre normaltid"}, {"Afghanistan", "afghansk tid"}, {"Africa_Central", "sentralafrikansk tid"}, {"Africa_Eastern", "austafrikansk tid"}, {"Africa_Southern", "sørafrikansk tid"}, {"Africa_Western", "vestafrikansk tid"}, {"Alaska", "alaskisk normaltid"}, {"Almaty", "Almaty, standardtid"}, {"Amazon", "normaltid for Amazonas"}, {"America_Central", "normaltid for sentrale Nord-Amerika"}, {"America_Eastern", "normaltid for den nordamerikanske austkysten"}, {"America_Mountain", "normaltid for Rocky Mountains (USA)"}, {"America_Pacific", "normaltid for den nordamerikanske stillehavskysten"}, {"Anadyr", "Russisk (Anadyr) normaltid"}, {"Apia", "normaltid for Apia"}, {"Aqtau", "Aqtau, standardtid"}, {"Aqtobe", "Aqtobe, standardtid"}, {"Arabian", "arabisk normaltid"}, {"Argentina", "argentinsk normaltid"}, {"Argentina_Western", "vestargentinsk normaltid"}, {"Armenia", "armensk normaltid"}, {"Atlantic", "normaltid for den nordamerikanske atlanterhavskysten"}, {"Australia_Central", "sentralaustralsk standardtid"}, {"Australia_CentralWestern", "vest-sentralaustralsk standardtid"}, {"Australia_Eastern", "austaustralsk standardtid"}, {"Australia_Western", "vestaustralsk standardtid"}, {"Azerbaijan", "aserbajdsjansk normaltid"}, {"Azores", "asorisk normaltid"}, {"Bangladesh", "bangladeshisk normaltid"}, {"Bhutan", "bhutansk tid"}, {"Bolivia", "boliviansk tid"}, {"Brasilia", "normaltid for Brasilia"}, {"Brunei", "tidssone for Brunei Darussalam"}, {"Cape_Verde", "kappverdisk normaltid"}, {"Casey", "Casey-tid"}, {"Chamorro", "tidssone for Chamorro"}, {"Chatham", "normaltid for Chatham"}, {"Chile", "chilensk normaltid"}, {"China", "kinesisk normaltid"}, {"Christmas", "tidssone for Christmasøya"}, {"Cocos", "tidssone for Kokosøyane"}, {"Colombia", "kolombiansk normaltid"}, {"Cook", "normaltid for Cookøyane"}, {"Cuba", "kubansk normaltid"}, {"Davis", "tidssone for Davis"}, {"DumontDUrville", "tidssone for Dumont-d’Urville"}, {"East_Timor", "austtimoresisk tid"}, {"Easter", "normaltid for Påskeøya"}, {"Ecuador", "ecuadoriansk tid"}, {"Europe_Central", "sentraleuropeisk standardtid"}, {"Europe_Eastern", "austeuropeisk standardtid"}, {"Europe_Further_Eastern", "fjern-austeuropeisk tid"}, {"Europe_Western", "vesteuropeisk standardtid"}, {"Falkland", "normaltid for Falklandsøyane"}, {"Fiji", "fijiansk normaltid"}, {"French_Guiana", "tidssone for Fransk Guyana"}, {"French_Southern", "tidssone for Dei franske sørterritoria"}, {"GMT", "Greenwich middeltid"}, {"Galapagos", "tidssone for Galápagosøyane"}, {"Gambier", "tidssone for Gambier"}, {"Georgia", "georgisk normaltid"}, {"Gilbert_Islands", "tidssone for Gilbertøyane"}, {"Greenland_Eastern", "austgrønlandsk normaltid"}, {"Greenland_Western", "vestgrønlandsk normaltid"}, {"Guam", "Guam-tid"}, {"Gulf", "tidssone for Persiabukta"}, {"Guyana", "guyansk tid"}, {"Hawaii", "normaltid for Hawaii og Aleutene"}, {"Hawaii_Aleutian", "normaltid for Hawaii og Aleutene"}, {"Hong_Kong", "hongkongkinesisk normaltid"}, {"Hovd", "normaltid for Khovd"}, {"India", "indisk tid"}, {"Indian_Ocean", "tidssone for Indiahavet"}, {"Indochina", "indokinesisk tid"}, {"Indonesia_Central", "sentralindonesisk tid"}, {"Indonesia_Eastern", "austindonesisk tid"}, {"Indonesia_Western", "vestindonesisk tid"}, {"Iran", "iransk normaltid"}, {"Irkutsk", "normaltid for Irkutsk"}, {"Israel", "israelsk normaltid"}, {"Japan", "japansk normaltid"}, {"Kamchatka", "Russisk (Petropavlovsk-Kamtsjatskij) normaltid"}, {"Kazakhstan", "kasakhstansk tid"}, {"Kazakhstan_Eastern", "austkasakhstansk tid"}, {"Kazakhstan_Western", "vestkasakhstansk tid"}, {"Korea", "koreansk normaltid"}, {"Kosrae", "tidssone for Kosrae"}, {"Krasnoyarsk", "normaltid for Krasnojarsk"}, {"Kyrgystan", "kirgisisk tid"}, {"Lanka", "Lanka-tid"}, {"Line_Islands", "tidssone for Lineøyane"}, {"Lord_Howe", "normaltid for Lord Howe-øya"}, {"Macau", "Macau, standardtid"}, {"Magadan", "normaltid for Magadan"}, {"Malaysia", "malaysisk tid"}, {"Maldives", "maldivisk tid"}, {"Marquesas", "tidssone for Marquesasøyane"}, {"Marshall_Islands", "marshallesisk tid"}, {"Mauritius", "mauritisk normaltid"}, {"Mawson", "tidssone for Mawson"}, {"Mexico_Pacific", "normaltid for den meksikanske stillehavskysten"}, {"Mongolia", "normaltid for Ulan Bator"}, {"Moscow", "normaltid for Moskva"}, {"Myanmar", "myanmarsk tid"}, {"Nauru", "naurisk tid"}, {"Nepal", "nepalsk tid"}, {"New_Caledonia", "kaledonsk normaltid"}, {"New_Zealand", "nyzealandsk normaltid"}, {"Newfoundland", "normaltid for Newfoundland"}, {"Niue", "tidssone for Niue"}, {"Norfolk", "normaltid for Norfolkøya"}, {"Noronha", "normaltid for Fernando de Noronha"}, {"North_Mariana", "Nord-Marianene-tid"}, {"Novosibirsk", "normaltid for Novosibirsk"}, {"Omsk", "normaltid for Omsk"}, {"Pakistan", "pakistansk normaltid"}, {"Palau", "palauisk tid"}, {"Papua_New_Guinea", "papuansk tid"}, {"Paraguay", "paraguayansk normaltid"}, {"Peru", "peruansk normaltid"}, {"Philippines", "filippinsk normaltid"}, {"Phoenix_Islands", "tidssone for Phoenixøyane"}, {"Pierre_Miquelon", "normaltid for Saint-Pierre-et-Miquelon"}, {"Pitcairn", "tidssone for Pitcairn"}, {"Ponape", "tidssone for Pohnpei"}, {"Pyongyang", "tidssone for Pyongyang"}, {"Qyzylorda", "Qyzylorda, standardtid"}, {"Reunion", "tidssone for Réunion"}, {"Rothera", "tidssone for Rothera"}, {"Sakhalin", "normaltid for Sakhalin"}, {"Samara", "Russisk (Samara) normaltid"}, {"Samoa", "samoansk normaltid"}, {"Seychelles", "seychellisk tid"}, {"Singapore", "singaporsk tid"}, {"Solomon", "salomonsk tid"}, {"South_Georgia", "tidssone for Sør-Georgia"}, {"Suriname", "surinamsk tid"}, {"Syowa", "tidssone for Syowa"}, {"Tahiti", "tahitisk tid"}, {"Taipei", "normaltid for Taipei"}, {"Tajikistan", "tadsjikisk tid"}, {"Tokelau", "tidssone for Tokelau"}, {"Tonga", "tongansk normaltid"}, {"Truk", "tidssone for Chuukøyane"}, {"Turkmenistan", "turkmensk normaltid"}, {"Tuvalu", "tuvalsk tid"}, {"Uruguay", "uruguayansk normaltid"}, {"Uzbekistan", "usbekisk normaltid"}, {"Vanuatu", "vanuatisk normaltid"}, {"Venezuela", "venezuelansk tid"}, {"Vladivostok", "normaltid for Vladivostok"}, {"Volgograd", "normaltid for Volgograd"}, {"Vostok", "tidssone for Vostok"}, {"Wake", "tidssone for Wake Island"}, {"Wallis", "tidssone for Wallis- og Futunaøyane"}, {"Yakutsk", "normaltid for Jakutsk"}, {"Yekaterinburg", "normaltid for Jekaterinburg"}, {"Yukon", "tidssone for Yukon"}},
		{{"Acre", "Acre sommertid"}, {"Alaska", "alaskisk sommartid"}, {"Almaty", "Almaty, sommertid"}, {"Amazon", "sommartid for Amazonas"}, {"America_Central", "sommartid for sentrale Nord-Amerika"}, {"America_Eastern", "sommartid for den nordamerikanske austkysten"}, {"America_Mountain", "sommartid for Rocky Mountains (USA)"}, {"America_Pacific", "sommartid for den nordamerikanske stillehavskysten"}, {"Anadyr", "Russisk (Anadyr) sommertid"}, {"Apia", "sommartid for Apia"}, {"Aqtau", "Aqtau, sommertid"}, {"Aqtobe", "Aqtobe, sommertid"}, {"Arabian", "arabisk sommartid"}, {"Argentina", "argentinsk sommartid"}, {"Argentina_Western", "vestargentinsk sommartid"}, {"Armenia", "armensk sommartid"}, {"Atlantic", "sommartid for den nordamerikanske atlanterhavskysten"}, {"Australia_Central", "sentralaustralsk sommartid"}, {"Australia_CentralWestern", "vest-sentralaustralsk sommartid"}, {"Australia_Eastern", "austaustralsk sommartid"}, {"Australia_Western", "vestaustralsk sommartid"}, {"Azerbaijan", "aserbajdsjansk sommartid"}, {"Azores", "asorisk sommartid"}, {"Bangladesh", "bangladeshisk sommartid"}, {"Brasilia", "sommartid for Brasilia"}, {"Cape_Verde", "kappverdisk sommartid"}, {"Chatham", "sommartid for Chatham"}, {"Chile", "chilensk sommartid"}, {"China", "kinesisk sommartid"}, {"Colombia", "kolombiansk sommartid"}, {"Cook", "sommartid for Cookøyane"}, {"Cuba", "kubansk sommartid"}, {"Easter", "sommartid for Påskeøya"}, {"Europe_Central", "sentraleuropeisk sommartid"}, {"Europe_Eastern", "austeuropeisk sommartid"}, {"Europe_Western", "vesteuropeisk sommartid"}, {"Falkland", "sommartid for Falklandsøyane"}, {"Fiji", "fijiansk sommartid"}, {"Georgia", "georgisk sommartid"}, {"Greenland_Eastern", "austgrønlandsk sommartid"}, {"Greenland_Western", "vestgrønlandsk sommartid"}, {"Hawaii_Aleutian", "sommartid for Hawaii og Aleutene"}, {"Hong_Kong", "hongkongkinesisk sommartid"}, {"Hovd", "sommartid for Khovd"}, {"Iran", "iransk sommartid"}, {"Irkutsk", "sommartid for Irkutsk"}, {"Israel", "israelsk sommartid"}, {"Japan", "japansk sommartid"}, {"Kamchatka", "Russisk (Petropavlovsk-Kamtsjatskij) sommertid"}, {"Korea", "koreansk sommartid"}, {"Krasnoyarsk", "sommartid for Krasnojarsk"}, {"Lord_Howe", "sommartid for Lord Howe-øya"}, {"Macau", "Macau, sommertid"}, {"Magadan", "sommartid for Magadan"}, {"Mauritius", "mauritisk sommartid"}, {"Mexico_Pacific", "sommartid for den meksikanske stillehavskysten"}, {"Mongolia", "sommartid for Ulan Bator"}, {"Moscow", "sommartid for Moskva"}, {"New_Caledonia", "kaledonsk sommartid"}, {"New_Zealand", "nyzealandsk sommartid"}, {"Newfoundland", "sommartid for Newfoundland"}, {"Norfolk", "sommartid for Norfolkøya"}, {"Noronha", "sommartid for Fernando de Noronha"}, {"Novosibirsk", "sommartid for Novosibirsk"}, {"Omsk", "sommartid for Omsk"}, {"Pakistan", "pakistansk sommartid"}, {"Paraguay", "paraguayansk sommartid"}, {"Peru", "peruansk sommartid"}, {"Philippines", "filippinsk sommartid"}, {"Pierre_Miquelon", "sommartid for Saint-Pierre-et-Miquelon"}, {"Qyzylorda", "Qyzylorda, sommertid"}, {"Sakhalin", "sommartid for Sakhalin"}, {"Samara", "Russisk (Samara) sommertid"}, {"Samoa", "samoansk sommartid"}, {"Taipei", "sommartid for Taipei"}, {"Tonga", "tongansk sommartid"}, {"Turkmenistan", "turkmensk sommartid"}, {"Uruguay", "uruguayansk sommartid"}, {"Uzbekistan", "usbekisk sommartid"}, {"Vanuatu", "vanuatisk sommartid"}, {"Vladivostok", "sommartid for Vladivostok"}, {"Volgograd", "sommartid for Volgograd"}, {"Yakutsk", "sommartid for Jakutsk"}, {"Yekaterinburg", "sommartid for Jekaterinburg"}},
	},
	{
		{{"Afghanistan", "ߊߝߑߜ߭ߊߣߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Africa_Central", "ߝߘߊ߬ߝߌ߲߬ߠߊ߫ ߕߊ߲ߓߊ߲ ߕߎ߬ߡߊ߬ߙߋ߲ ߢߊߓߘߍ"}, {"Africa_Eastern", "ߝߘߊ߬ߌ߲߬ߠߊ߫ ߓߟߋ߬ߓߐ ߕߎ߬ߡߊ߬ߙߋ߲ ߢߊߓߘߍ"}, {"Africa_Southern", "ߝߘߌ߬ߝߌ߲߬ߠߊ߫ ߥߙߏ߬ߘߎ߮ ߕߎ߬ߡߊ߬ߙߋ߲ ߢߊߓߘߍ"}, {"Africa_Western", "ߝߘߊ߬ߝߌ߲߬ߠߊ߫ ߕߟߋ߬ߓߋ ߕߎ߬ߡߊ߬ߙߋ߲"}, {"Alaska", "ߊߟߊߛߑߞߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Amazon", "ߊߡߊߖ߭ߏߣ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"America_Central", "ߞߐ߰ߘߎ߮ ߊߡߋߙߌߞߌ߬ ߕߎ߬ߡߘߊ߬ ߕߊ߲ߓߊ߲߫ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"America_Eastern", "ߞߐ߰ߘߎ߰ ߊߡߋߙߌߞߌ߬ ߕߟߋ߬ߓߐ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"America_Mountain", "ߞߙߎ߬ߞߊ߲߬ߠߊ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"America_Pacific", "ߞߐ߰ߘߎ߰ ߊߡߋߙߌߞߌ߬ ߣߌ߫ ߖߐ߫ ߟߌ߲ߓߊ߲ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Apia", "ߊߔߌߦߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Arabian", "ߊߙߊߓߎߟߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Argentina", "ߊߙߑߖ߭ߊ߲ߕߌ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Argentina_Western", "ߕߟߋ߬ߓߋ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Armenia", "ߊߙߑߡߋߣߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Atlantic", "ߟߌ߲ߓߊ߲-ߡߊ߲ߞߊ߲ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Australia_Central", "ߕߍߡߟߊ ߏߛߑߕߙߊߟߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Australia_CentralWestern", "ߏߛߑߕߙߊߟߌ߫ ߕߟߋ߬ߓߋ߬-ߕߍߡߟߊ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Australia_Eastern", "ߕߟߋ߬ߓߐ ߏߛߑߕߙߊߟߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Australia_Western", "ߕߟߋ߬ߓߋ ߏߛߑߕߙߊߟߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Azerbaijan", "ߊߖ߭ߍߙߑߓߊߦߑߖߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Azores", "ߊߗ߭ߐߙߛ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Bangladesh", "ߓߊ߲ߜ߭ߑߟߊߘߍߛ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Bhutan", "ߓߎߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Bolivia", "ߓߏߟߝ߭ߌ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Brasilia", "ߓߙߋߖ߭ߟߌߟߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Brunei", "ߓߙߎߣߋߦߌ߫ ߘߊߙߎߛߑߛߟߊߡ ߕߎ߬ߡߊ߬ߘߊ"}, {"Cape_Verde", "ߜߙߋߞߎ߲߫-ߝߙߌߛߌ߫ ߕߎ߬ߡߊ߬ߙߋ߲ ߢߊߓߘߍ"}, {"Chamorro", "ߛ߭ߊߡߏߙߏ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Chatham", "ߗߕߊߡ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Chile", "ߛߟߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"China", "ߛߌߣ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Christmas", "ߞߑߙߌߛߑߡߊߛ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Cocos", "ߞߏߞߏߛ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Colombia", "ߞߐߟߐ߲ߓߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Cook", "ߞߏߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Cuba", "ߞߎ߳ߓߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Davis", "ߘߊߝ߭ߌߛ ߕߎ߬ߡߊ߬ߘߊ"}, {"DumontDUrville", "ߘߎߡߐ߲-ߘߎߙߑߝ߭ߌߟ ߕߎ߬ߡߊ߬ߘߊ"}, {"East_Timor", "ߕߟߋ߬ߓߐ ߕߌߡߐߙ ߕߎ߬ߡߊ߬ߘߊ"}, {"Easter", "ߌߛߑߟߊ߲ߘ ߕߌ߲ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Ecuador", "ߌߞߑߥߊߘߐߙ ߕߎ߬ߡߊ߬ߘߊ"}, {"Europe_Central", "ߕߍߡߊ߫ ߋߙߐߔߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Europe_Eastern", "ߕߟߋ߬ߓߐ ߋߙߐߔߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Europe_Further_Eastern", "ߞߊߟߌߣߌ߲ߜ߭ߙߊߘ ߕߎ߬ߡߊ߬ߘߊ"}, {"Europe_Western", "ߕߟߋ߬ߓߋ ߋߙߐߔߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Falkland", "ߝߊߟߞߑߟߊ߲ߘ ߕߌ߲ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Fiji", "ߝߖߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"French_Guiana", "ߝߊ߬ߙߊ߲߬ߛߌ߬ ߜ߭ߎߦߣߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"French_Southern", "ߐߛߑߕߙߊߟߌ߫ ߘߎ߰ߞߟߏ ߣߌ߫ ߝߊ߬ߙߊ߲߬ߛߌ߫ ߊ߲ߕߊߙߑߕߌߞ ߕߎ߬ߡߊ߬ߙߋ"}, {"GMT", "ߜ߭ߙߋߣߍߕ ߕߎ߬ߡߊ߬ߘߊ"}, {"Galapagos", "ߜ߭ߟߊߔߊߜ߭ߐߛ ߕߎ߬ߡߊ߬ߘߊ߫"}, {"Gambier", "ߜ߭ߊ߲ߓߌߦߍߙ ߕߎ߬ߡߊ߬ߘߊ"}, {"Georgia", "ߖ߭ߐߙߑߖ߭ߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Gilbert_Islands", "ߖ߭ߌߟߑߓߍߙߕ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Greenland_Eastern", "ߕߟߋ߬ߓߐ ߜ߭ߌߙߌ߲ߟߊ߲ߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Greenland_Western", "ߕߟߋ߬ߓߋ ߜ߭ߌߙߌ߲ߟߊ߲ߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Gulf", "ߖߌߘߏ߮ ߕߎ߬ߡߊ߬ߘߊ"}, {"Guyana", "ߜ߭ߎߦߣߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Hawaii", "ߤߥߊߦ - ߊߟߋߦߎߕߌߦߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Hawaii_Aleutian", "ߤߥߊߦ - ߊߟߋߦߎߕߌߦߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Hong_Kong", "ߤߐ߲ߜ߭ ߞߐ߲ߜ߭ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Hovd", "ߤߏߝ߭ߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"India", "ߤߌ߲ߘߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Indian_Ocean", "ߍ߲ߘߎ߫ ߟߌ߲ߓߊ߲ߘߎ߯ ߕߎ߬ߡߊ߬ߙߋ߲"}, {"Indochina", "ߤߌ߲ߘߏ߫ ߛߌߣ ߕߎ߬ߡߊ߬ߘߊ"}, {"Indonesia_Central", "ߕߍߡߊ߫ ߍ߲ߘߋߣߏߛߌ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Indonesia_Eastern", "ߕߟߋ߬ߓߐ ߍ߲ߘߋߣߏߛߌ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Indonesia_Western", "ߕߟߋ߬ߓߋ ߍ߲ߘߋߣߏߛߌ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Iran", "ߌߙߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Irkutsk", "ߌߙߑߞߎߕߑߛߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Israel", "ߌߛߌ߬ߙߊ߬ߦߌߟߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Japan", "ߖ߭ߊߔߐ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Kazakhstan", "ߞߖ߭ߊߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Kazakhstan_Eastern", "ߕߟߋ߬ߓߐ ߞߖ߭ߊߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Kazakhstan_Western", "ߕߟߋ߬ߓߋ ߞߖ߭ߊߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Korea", "ߞߏߙߋ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Kosrae", "ߞߏߛߑߙߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Krasnoyarsk", "ߞߙߊߛߑߣߏߦߌߦߊߙߑߛߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Kyrgystan", "ߜ߭ߌߙߑߜ߭ߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Line_Islands", "ߌߛߑߟߊ߲ߘ ߞߍ߬ߙߍ߲߬ߘߍ ߕߎ߬ߡߊ߬ߘߊ"}, {"Lord_Howe", "ߟߐߙߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Magadan", "ߡߜ߭ߊߘߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Malaysia", "ߡߊߟߍߘߎ߯ ߕߎ߬ߡߊ߬ߘߊ"}, {"Maldives", "ߡߊߟߑߘߌߝ߭ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Marquesas", "ߡߊߙߞߌߛߊߛ ߕߎ߬ߡߊ߬ߘߊ"}, {"Marshall_Islands", "ߡߊߙߑߛߊߟ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Mauritius", "ߡߏߙߌߛ ߕߎ߬ߡߊ߬ߙߋ߲ ߢߊߓߘߍ"}, {"Mawson", "ߡߊߥߑߛߐ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Mexico_Pacific", "ߖߐ߫ ߟߌ߲ߓߊ߲ ߡߍ߲ߞߛߌߞߌ߬ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Mongolia", "ߎߟߊ߲ߓߕߐߙ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Moscow", "ߡߏߛߑߞߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Myanmar", "ߡߌߦߊߣߑߡߊߙ ߕߎ߬ߡߊ߬ߘߊ"}, {"Nauru", "ߣߊߥߙߎ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Nepal", "ߣߋߔߊߟ ߕߎ߬ߡߊ߬ߘߊ"}, {"New_Caledonia", "ߞߊߟߌߘߏߣߌ߫ ߞߎߘߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"New_Zealand", "ߣߌߦߎ-ߖ߭ߋߟߊ߲ߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Newfoundland", "ߝߎ߲ߘߑߟߊ߲ߘ ߞߎߘߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Niue", "ߣߌߦߋ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Norfolk", "ߣߐߙ ߝߐߟߞ ߕߌ߲ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Noronha", "ߝߍߙߑߣߊ߲ߘߏ߫ ߘߋ߬ ߣߏߙߏ߲ߤߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Novosibirsk", "ߣߝ߭ߏߛߓߌߙߑߛߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Omsk", "ߏߡߑߛߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Pakistan", "ߔߊߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Palau", "ߔߟߊߐߛ ߕߎ߬ߡߊ߬ߘߊ"}, {"Papua_New_Guinea", "ߔߊߓߎߥߊ߫ ߖߌ߬ߣߍ߬ ߞߎߘߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Paraguay", "ߔߙߊߜ߭ߎߥߋ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Peru", "ߔߋߙߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Philippines", "ߝߟߌߔߌ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Phoenix_Islands", "ߝߏߣߌߞߛ ߕߌ߲ ߠߎ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Pierre_Miquelon", "ߛߍ߲-ߔߌߦߍߙ-ߋ-ߡߌߞߋߟߐ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Pitcairn", "ߔߌߕߑߞߍߙߣ ߕߎ߬ߡߊ߬ߘߊ"}, {"Ponape", "ߔߏ߲ߔߍ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Pyongyang", "ߔߌߦߐ߲ߜ߭ߦߊ߲ߜ߭ ߕߎ߬ߡߊ߬ߘߊ"}, {"Reunion", "ߙߋߣߌߦߐ߲߫ ߕߎ߬ߡߊ߬ߙߋ߲"}, {"Rothera", "ߙߏߕߋߙߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Sakhalin", "ߛߊߞߑߤߊߟߌߣ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Samoa", "ߛߊߡߏߥߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Seychelles", "ߛߋߦߌߛߌߟ ߕߎ߬ߡߊ߬ߙߋ߲"}, {"Singapore", "ߛߍ߲ߜ߭ߊߔߎߙ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Solomon", "ߛߟߏ߬ߡߣߊ߬ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"South_Georgia", "ߥߙߏ߬ߘߎ߮ ߖ߭ߐߙߑߖ߭ߌߟߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Suriname", "ߛߎߙߌߣߊߡ ߕߎ߬ߡߊ߬ߘߊ"}, {"Syowa", "ߛߦߏߥߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Tahiti", "ߕߊߤߕߌ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Taipei", "ߕߊߦߌߔߋ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Tajikistan", "ߕߊߖߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Tokelau", "ߕߏߞߋߟߏ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Tonga", "ߕߏ߲ߜ߭ߊ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Truk", "ߗߎߞ ߕߎ߬ߡߊ߬ߘߊ"}, {"Turkmenistan", "ߕߎߙߞߌߡߋߣߌߛߑߕߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Tuvalu", "ߕߎߝ߭ߊߟߎ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Uruguay", "ߎ߳ߙߑߜ߭ߋߦߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Uzbekistan", "ߏߖ߭ߑߓߊߞߌߛߑߕߊ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Vanuatu", "ߝ߭ߊߣߎߥߊߕߎ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Venezuela", "ߝ߭ߣߋߖ߭ߎߦߋߟߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Vladivostok", "ߝ߭ߟߊߘߑߝ߭ߏߛߑߕߏߞ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Volgograd", "ߝ߭ߏߟߑߜ߭ߏߜ߭ߙߊߘ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Vostok", "ߝ߭ߐߛߑߕߐߞ ߕߎ߬ߡߊ߬ߘߊ"}, {"Wake", "ߥߊߞ ߕߌ߲ ߕߎ߬ߡߊ߬ߘߊ"}, {"Wallis", "ߥߊߟߌߛ ߣߌ߫ ߝߕߎߣߊ߫ ߕߎ߬ߡߊ߬ߘߊ"}, {"Yakutsk", "ߦߊߞߎߕߑߛߞߌ߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Yekaterinburg", "ߦߋߞߊߕߋߙߌ߲ߓߎߙߜ߭ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Yukon", "ߦߎߞߐ߲߫ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}},
		{{"Alaska", "ߊߟߊߛߑߞߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Amazon", "ߊߡߊߖ߭ߏ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"America_Central", "ߞߐ߰ߘߎ߮ ߊߡߋߙߌߞߌ߬ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"America_Eastern", "ߞߐ߰ߘߎ߰ ߊߡߋߙߌߞߌ߬ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"America_Mountain", "ߞߙߎ߬ߞߊ߲߬ߠߊ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"America_Pacific", "ߞߐ߰ߘߎ߰ ߊߡߋߙߌߞߌ߬ ߣߌ߫ ߖߐ߫ ߟߌ߲ߓߊ߲ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Apia", "ߊߔߌߦߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Arabian", "ߊߙߊߓߎߟߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Argentina", "ߊߙߑߖ߭ߊ߲ߕߌ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Argentina_Western", "ߕߟߋ߬ߓߋ ߊߙߑߖ߭ߊ߲ߕߌ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Armenia", "ߊߙߑߡߋߣߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Atlantic", "ߟߌ߲ߓߊ߲-ߡߊ߲ߞߊ߲ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Australia_Central", "ߕߍߡߟߊ ߏߛߑߕߙߊߟߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Australia_CentralWestern", "ߏߛߑߕߙߊߟߌ߫ ߕߟߋ߬ߓߋ߬-ߕߍߡߟߊ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Australia_Eastern", "ߕߟߋ߬ߓߐ ߏߛߑߕߙߊߟߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Australia_Western", "ߕߟߋ߬ߓߋ ߏߛߑߕߙߊߟߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Azerbaijan", "ߊߖ߭ߍߙߑߓߊߦߑߖߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Azores", "ߊߗ߭ߐߙߛ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Bangladesh", "ߓߊ߲ߜ߭ߑߟߊߘߍߛ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Brasilia", "ߓߙߋߖ߭ߟߌߟߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Cape_Verde", "ߜߙߋߞߎ߲߫-ߝߙߌߛߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߊߙߋ߲"}, {"Chatham", "ߗߕߊߡ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Chile", "ߛߟߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"China", "ߛߌߣ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Colombia", "ߞߐߟߐ߲ߓߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Cook", "ߞߏߞ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Cuba", "ߞߎ߱ߓߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Easter", "ߌߛߑߟߊ߲ߘ ߕߌ߲ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Europe_Central", "ߕߍߡߊ߫ ߋߙߐߔߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Europe_Eastern", "ߕߟߋ߬ߓߐ ߋߙߐߔߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Europe_Western", "ߕߟߋ߬ߓߋ ߋߙߐߔߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Falkland", "ߝߊߟߞߑߟߊ߲ߘ ߕߌ߲ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Fiji", "ߝߖߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߘߊ߬ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Georgia", "ߖ߭ߐߙߑߖ߭ߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Greenland_Eastern", "ߕߟߋ߬ߓߐ ߜ߭ߌߙߌ߲ߟߊ߲ߘ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Greenland_Western", "ߕߟߋ߬ߓߋ ߜ߭ߌߙߌ߲ߟߊ߲ߘ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Hawaii_Aleutian", "ߤߥߊߦ - ߊߟߋߦߎߕߌߦߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Hong_Kong", "ߤߐ߲ߜ߭ ߞߐ߲ߜ߭ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Hovd", "ߤߏߝ߭ߘ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Iran", "ߌߙߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Irkutsk", "ߌߙߑߞߎߕߑߛߞ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Israel", "ߌߛߌ߬ߙߊ߬ߦߌߟߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ ߛߎߡߊ߲ߘߊ߲ߕߊ"}, {"Japan", "ߖ߭ߊߔߐ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Korea", "ߞߏߙߋ߫ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Krasnoyarsk", "ߞߙߊߛߑߣߏߦߌߦߊߙߑߛߞ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Lord_Howe", "ߟߐߙߘ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Magadan", "ߡߜ߭ߊߘߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Mauritius", "ߡߏߙߌߛ ߕߟߋ߬ߡߊ߬ ߕߎߡߊߙߋ߲"}, {"Mexico_Pacific", "ߖߐ߫ ߟߌ߲ߓߊ߲ ߡߍ߲ߞߛߌߞߌ߬ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Mongolia", "ߎߟߊ߲ߓߕߐߙ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Moscow", "ߡߏߛߑߞߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"New_Caledonia", "ߞߊߟߌߘߏߣߌ߫ ߞߎߘߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"New_Zealand", "ߣߌߦߎ-ߖ߭ߋߟߊ߲ߘ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Newfoundland", "ߝߎ߲ߘߑߟߊ߲ߘ ߞߎߘߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Norfolk", "ߣߐߙ ߝߐߟߞ ߕߌ߲ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Noronha", "ߝߍߙߑߣߊ߲ߘߏ߫ ߘߋ߬ ߣߏߙߏ߲ߤߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Novosibirsk", "ߣߝ߭ߏߛߓߌߙߑߛߞ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Omsk", "ߏߡߑߛߞ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Pakistan", "ߔߊߞߌߛߑߕߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Paraguay", "ߔߙߊߜ߭ߎߥߋ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Peru", "ߔߋߙߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Philippines", "ߝߟߌߔߌ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Pierre_Miquelon", "ߛߍ߲-ߔߌߦߍߙ-ߋ-ߡߌߞߋߟߐ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Sakhalin", "ߛߊߞߑߤߊߟߌߣ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Samoa", "ߛߊߡߏߥߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Taipei", "ߕߊߦߌߔߋ߫ ߕߟߋ߬ߡߊ߬ ߕߎ߬ߡߊ߬ߘߊ"}, {"Tonga", "ߕߏ߲ߜ߭ߊ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Turkmenistan", "ߕߎߙߞߑߡߋߣߌߛߑߕߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Uruguay", "ߎ߳ߙߑߜ߭ߋߦߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Uzbekistan", "ߏߖ߭ߑߓߊߞߌߛߑߕߊ߲߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Vanuatu", "ߝ߭ߊߣߎߥߊߕߎ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Vladivostok", "ߝ߭ߟߊߘߑߝ߭ߏߛߑߕߏߞ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Volgograd", "ߝ߭ߏߟߑߜ߭ߏߜ߭ߙߊߘ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Yakutsk", "ߦߊߞߎߕߑߛߞߌ߫ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}, {"Yekaterinburg", "ߦߋߞߊߕߋߙߌ߲ߓߎߙߜ߭ ߕߟߋ߬ߡߊ߬ ߕߎߡߘߊ"}},
	},
	{
		{{"GMT", "Nako Palogare ya Greenwich"}},
		{},
	},
	{
		{{"Afghanistan", "ora d’Afganistan"}, {"Africa_Central", "ora d’Africa centrala"}, {"Africa_Eastern", "ora d’Africa de l’èst"}, {"Africa_Southern", "ora de Sudafrica"}, {"Africa_Western", "ora d’Africa occidentala"}, {"Alaska", "ora estandarda d’Alaska"}, {"Amazon", "ora estandarda de l’Amazònes"}, {"America_Central", "ora estandarda centrala"}, {"America_Eastern", "ora estandarda de l’Èst"}, {"America_Mountain", "ora estandarda de la montanha"}, {"America_Pacific", "ora estandarda del Pacific"}, {"Apia", "ora estandard d’Apia"}, {"Arabian", "ora estandarda d’Arabia"}, {"Argentina", "ora estandarda d’Argentina"}, {"Argentina_Western", "ora estandarda de l’oèst d’Argentina"}, {"Armenia", "ora estandarda d’Armenia"}, {"Atlantic", "ora estandarda de l’Atlantic"}, {"Australia_Central", "ora estandard d’Austràlia centrala"}, {"Australia_CentralWestern", "ora estandard d’Austràlia centreoccidentala"}, {"Australia_Eastern", "ora estandard d’Austràlia oriental"}, {"Australia_Western", "ora estandard d’Austràlia occidental"}, {"Azerbaijan", "ora estandarda d’Azerbaijan"}, {"Azores", "ora estandarda de les Açòres"}, {"Bangladesh", "ora estandarda de Bangaldesh"}, {"Bhutan", "ora de Butan"}, {"Bolivia", "ora de Bolivia"}, {"Brasilia", "ora estandarda de Brasilia"}, {"Brunei", "ora de Brunei"}, {"Cape_Verde", "ora estandarda de Cap-Verd"}, {"Chamorro", "ora estandard de Chamorro"}, {"Chatham", "ora estandard de Chatham"}, {"Chile", "ora estandarda de Chile"}, {"China", "ora estandarda de China"}, {"Christmas", "ora de l’illa de Nadau"}, {"Cocos", "ora de l’illa de Cocos"}, {"Colombia", "ora estandarda de Colòmbia"}, {"Cook", "ora estandard de les Illes Cook"}, {"Cuba", "ora estandarda de Cuba"}, {"Davis", "ora de Davis"}, {"DumontDUrville", "ora de Dumont-d’Urville"}, {"East_Timor", "ora de Timòr oriental"}, {"Easter", "ora estandarda de la illa de Pascua"}, {"Ecuador", "ora d’Equator"}, {"Europe_Central", "ora estandarda d’Euròpa centrala"}, {"Europe_Eastern", "ora estandarda d’Euròpa de l’èst"}, {"Europe_Further_Eastern", "ora de l’extrem d’Euròpa orientala"}, {"Europe_Western", "ora estandarda d’Euròpa de l’oèst"}, {"Falkland", "ora estandarda de les illes Malvinas"}, {"Fiji", "ora estandard de Fiji"}, {"French_Guiana", "ora de la Guayana Francesa"}, {"French_Southern", "ora de les Terres australes e antartiques franceses"}, {"GMT", "ora al meridian de Greenwich"}, {"Galapagos", "ora de Galapagos"}, {"Gambier", "ora de Gambier"}, {"Georgia", "ora estandarda de Geòrgia"}, {"Gilbert_Islands", "ora de les illes Gilbert"}, {"Greenland_Eastern", "ora estandarda de l’èst de Groenlandia"}, {"Greenland_Western", "ora estandarda de l’oèst de Groenlandia"}, {"Gulf", "ora estandard deth Gòlf"}, {"Guyana", "ora de la Guayana"}, {"Hawaii", "ora estandarda de Hawai-Aleutianes"}, {"Hawaii_Aleutian", "ora estandarda de Hawai-Aleutianes"}, {"Hong_Kong", "ora estandarda de Hong Kong"}, {"Hovd", "ora estandarda de Hovd"}, {"India", "ora estandarda de l’India"}, {"Indian_Ocean", "ora de l’Ocean Indic"}, {"Indochina", "ora d’Indochina"}, {"Indonesia_Central", "hora d’Indonesia centrala"}, {"Indonesia_Eastern", "hora d’Indonesia orientala"}, {"Indonesia_Western", "hora d’Indonesia occidentala"}, {"Iran", "ora estandarda d’Iran"}, {"Irkutsk", "ora estandarda d’Irkutsk"}, {"Israel", "ora estandarda d’Israèl"}, {"Japan", "ora estandarda de Japon"}, {"Kazakhstan_Eastern", "ora de Kazajistan orientala"}, {"Kazakhstan_Western", "ora de Kazajistan occidenatal"}, {"Korea", "ora estandarda de Corèa"}, {"Kosrae", "ora de Kosrae"}, {"Krasnoyarsk", "ora estandarda de Krasnoyarsk"}, {"Kyrgystan", "ora de Kirguistan"}, {"Line_Islands", "ora de les Espòrades Equatorials"}, {"Lord_Howe", "ora estandard de Lord Howe"}, {"Magadan", "ora estandarda de Magadan"}, {"Malaysia", "ora de Malàisia"}, {"Maldives", "ora des Malvines"}, {"Marquesas", "ora de les Illes Marqueses"}, {"Marshall_Islands", "ora de les Illes Marshall"}, {"Mauritius", "ora estandarda de Maurici"}, {"Mawson", "ora de Mawson"}, {"Mexico_Pacific", "ora estandarda del Pacific Mexican"}, {"Mongolia", "ora estandarda de Ulan Bator"}, {"Moscow", "ora estandarda de Moscòu"}, {"Myanmar", "ora de Myanmar"}, {"Nauru", "ora de Nauru"}, {"Nepal", "ora de Nepal"}, {"New_Caledonia", "ora estandard de Nòva Caledònia"}, {"New_Zealand", "ora estandard de Nòva Zelanda"}, {"Newfoundland", "ora estandarda de Terra-Nòva"}, {"Niue", "ora de Niue"}, {"Norfolk", "ora estandard de l’illa Norfolk"}, {"Noronha", "ora estandarda de Fernando Noronha"}, {"Novosibirsk", "ora estandarda de Novosibirsk"}, {"Omsk", "ora estandarda d’Omsk"}, {"Pakistan", "ora estandarda de Pakistan"}, {"Palau", "ora de Palaos"}, {"Papua_New_Guinea", "ora de Papua Nòva Guinèa"}, {"Paraguay", "ora estandarda de Paraguay"}, {"Peru", "ora estandarda de Perú"}, {"Philippines", "ora estandarda de Filipines"}, {"Phoenix_Islands", "ora de les Illes Fénix"}, {"Pierre_Miquelon", "ora estandarda de St. Pierre e Miquelon"}, {"Pitcairn", "ora de Pitcairn"}, {"Ponape", "ora de Pohnpei"}, {"Pyongyang", "ora de Pyongyang"}, {"Reunion", "ora de la Reünion"}, {"Rothera", "ora de Rothera"}, {"Sakhalin", "ora estandarda de Sajalin"}, {"Samoa", "ora estandard de Samoa"}, {"Seychelles", "ora de Seychelles"}, {"Singapore", "ora de Singapor"}, {"Solomon", "ora de les Illes Salomon"}, {"South_Georgia", "ora de Geòrgia del Sud"}, {"Suriname", "ora de Surinam"}, {"Syowa", "ora de Syowa"}, {"Tahiti", "ora de Tahiti"}, {"Taipei", "ora estandarda de Taipei"}, {"Tajikistan", "ora de Tajikistan"}, {"Tokelau", "ora de Tokelau"}, {"Tonga", "ora estandard de Tonga"}, {"Truk", "ora de Chuuk"}, {"Turkmenistan", "ora estandarda de Turkmenistan"}, {"Tuvalu", "ora de Tuvalu"}, {"Uruguay", "ora estandarda de l’Uruguay"}, {"Uzbekistan", "ora estandarda de Uzbekistan"}, {"Vanuatu", "ora estandard de Vanuatu"}, {"Venezuela", "ora de Veneçuela"}, {"Vladivostok", "ora estandarda de Vladivostok"}, {"Volgograd", "ora estandarda de Volgograd"}, {"Vostok", "ora de Vostok"}, {"Wake", "ora de l’Illa Wake"}, {"Wallis", "ora de Wallis e Futuna"}, {"Yakutsk", "ora estandarda de Yakutsk"}, {"Yekaterinburg", "ora estandarda d’Ekaterimburg"}, {"Yukon", "ora de Yukon"}},
		{{"Alaska", "ora d’estiu d’Alaska"}, {"Amazon", "ora d’estiu l’Amazònes"}, {"America_Central", "ora d’estiu centrala"}, {"America_Eastern", "ora d’estiu de l’Èst"}, {"America_Mountain", "ora d’estiu de la montanha"}, {"America_Pacific", "ora d’estiu del Pacific"}, {"Apia", "ora d’esiu d’Apia"}, {"Arabian", "ora d’estiu d’Arabia"}, {"Argentina", "ora d’estiu d’Argentina"}, {"Argentina_Western", "ora d’estiu de l’oèst d’Argentina"}, {"Armenia", "ora d’estiu d’Armenia"}, {"Atlantic", "ora d’estiu de l’Atlantic"}, {"Australia_Central", "ora d’estiu d’Austràlia centrala"}, {"Australia_CentralWestern", "ora d’estiu estandarda d’Austràlia centreoccidentala"}, {"Australia_Eastern", "ora d’estiu d’Austràlia oriental"}, {"Australia_Western", "ora d’estiu d’Austràlia occidental"}, {"Azerbaijan", "ora d’estiu d’Azerbaijan"}, {"Azores", "ora d’estiu de les Açòres"}, {"Bangladesh", "ora d’estiu de Bangaldesh"}, {"Brasilia", "ora d’estiu de Brasilia"}, {"Cape_Verde", "ora d’estiu de Cap-Verd"}, {"Chatham", "ora d’estiu de Chatham"}, {"Chile", "ora d’estiu de Chile"}, {"China", "ora d’estiu de China"}, {"Colombia", "ora d’estiu de Colòmbia"}, {"Cook", "ora d’estiu mieja de les Illes Cook"}, {"Cuba", "ora d’estiu de Cuba"}, {"Easter", "ora d’estiu de la illa de Pascua"}, {"Europe_Central", "ora d’estiu d’Euròpa centrala"}, {"Europe_Eastern", "ora d’estiu d’Euròpa de l’èst"}, {"Europe_Western", "ora d’estiu d’Euròpa de l’oèst"}, {"Falkland", "ora d’estiu de les illes Malvinas"}, {"Fiji", "ora d’estiu de Fiji"}, {"Georgia", "ora d’estiu de Geòrgia"}, {"Greenland_Eastern", "ora d’estiu de l’èst de Groenlandia"}, {"Greenland_Western", "ora d’estiu de l’oèst de Groenlandia"}, {"Hawaii_Aleutian", "ora d’estiu de Hawai-Aleutianes"}, {"Hong_Kong", "ora d’estiu de Hong Kong"}, {"Hovd", "ora d’estiu de Hovd"}, {"Iran", "ora d’estiu d’Iran"}, {"Irkutsk", "ora d’estiu d’Irkutsk"}, {"Israel", "ora d’estiu d’Israèl"}, {"Japan", "ora d’estiu de Japon"}, {"Korea", "ora d’estiu de Corèa"}, {"Krasnoyarsk", "ora d’estiu de Krasnoyarsk"}, {"Lord_Howe", "ora d’estiu de Lord Howe"}, {"Magadan", "ora d’estiu de Magadan"}, {"Mauritius", "ora d’estiu de Maurici"}, {"Mexico_Pacific", "ora d’estiu del Pacific Mexican"}, {"Mongolia", "ora d’estiu de Ulan Bator"}, {"Moscow", "ora d’estiu de Moscòu"}, {"New_Caledonia", "ora d’estiu de Nòva Caledònia"}, {"New_Zealand", "ora d’estiu de Nòva Zelanda"}, {"Newfoundland", "ora d’estiu de Terra-Nòva"}, {"Norfolk", "ora d’estiu de l’illa Norfolk"}, {"Noronha", "ora d’estiu de Fernando Noronha"}, {"Novosibirsk", "ora d’estiu de Novosibirsk"}, {"Omsk", "ora d’estiu d’Omsk"}, {"Pakistan", "ora d’estiu de Pakistan"}, {"Paraguay", "ora d’estiu de Paraguay"}, {"Peru", "ora d’estiu de Perú"}, {"Philippines", "ora d’estiu de Filipines"}, {"Pierre_Miquelon", "ora d’estiu de St. Pierre e Miquelon"}, {"Sakhalin", "ora d’estiu de Sajalin"}, {"Samoa", "ora d’estiu de Samoa"}, {"Taipei", "ora d’estiu de Taipei"}, {"Tonga", "ora d’estiu de Tonga"}, {"Turkmenistan", "ora d’estiu de Turkmenistan"}, {"Uruguay", "ora d’estiu de l’Uruguay"}, {"Uzbekistan", "ora d’estiu de Uzbekistan"}, {"Vanuatu", "ora d’estiu de Vanuatu"}, {"Vladivostok", "ora d’estiu de Vladivostok"}, {"Volgograd", "ora d’estiu de Volgograd"}, {"Yakutsk", "ora d’estiu de Yakutsk"}, {"Yekaterinburg", "ora d’estiu d’Ekaterimburg"}},
	},
	{
		{{"Afghanistan", "ora d’Afganistan"}, {"Africa_Central", "ora d’Africa central"}, {"Africa_Eastern", "ora d’Africa orientau"}, {"Africa_Southern", "ora de Sudafrica"}, {"Africa_Western", "ora d’Africa occidentau"}, {"Alaska", "ora estandard d’Alaska"}, {"Amazon", "ora estandard der Amazònes"}, {"America_Central", "ora estandard centrau"}, {"America_Eastern", "ora estandard orientau"}, {"America_Mountain", "ora estandard des Montanhes Rocoses"}, {"America_Pacific", "ora estandard deth Pacific"}, {"Apia", "ora estandard d’Apia"}, {"Arabian", "ora estandard d’Arabia"}, {"Argentina", "ora estandard d’Argentina"}, {"Argentina_Western", "ora estandard d’Argentina occidentau"}, {"Armenia", "ora estandard d’Armenia"}, {"Atlantic", "ora estandard der Atlantic"}, {"Australia_Central", "ora estandard d’Austràlia centrau"}, {"Australia_CentralWestern", "ora estandarda d’Austràlia centreoccidentau"}, {"Australia_Eastern", "ora estandard d’Austràlia orientau"}, {"Australia_Western", "ora estandard d’Australia occidentau"}, {"Azerbaijan", "ora estandard d’Azerbaijan"}, {"Azores", "ora estandard des Açòres"}, {"Bangladesh", "ora estandard de Bangladesh"}, {"Bhutan", "ora de Butan"}, {"Bolivia", "ora de Bolivia"}, {"Brasilia", "ora estandard de Brasilia"}, {"Brunei", "ora de Brunei"}, {"Cape_Verde", "ora estandard de Cap-Verd"}, {"Chamorro", "ora estandard de Chamorro"}, {"Chatham", "ora estandard de Chatham"}, {"Chile", "ora estandard de Chile"}, {"China", "ora estandard de China"}, {"Christmas", "ora dera isla de Nadau"}, {"Cocos", "ora dera isla de Cocos"}, {"Colombia", "ora estandard de Colòmbia"}, {"Cook", "ora estandard des Isles Cook"}, {"Cuba", "ora estandard de Cuba"}, {"Davis", "ora de Davis"}, {"DumontDUrville", "ora de Dumont-d’Urville"}, {"East_Timor", "ora de Timòr orientau"}, {"Easter", "ora estandard dera isla de Pasqua"}, {"Ecuador", "ora d’Equator"}, {"Europe_Central", "ora estandard d’Euròpa centrau"}, {"Europe_Eastern", "ora estandard d’Euròpa de l’èst"}, {"Europe_Further_Eastern", "ora der extrem d’Euròpa orientau"}, {"Europe_Western", "ora estandard d’Euròpa occidentau"}, {"Falkland", "ora estandard des isles Maldives"}, {"Fiji", "ora estandard de Fiji"}, {"French_Guiana", "ora dera Guayana Francesa"}, {"French_Southern", "hora d’Antartida e Territoris Australs Francesi"}, {"GMT", "ora deth meridian de Greenwich"}, {"Galapagos", "ora de Galápagos"}, {"Gambier", "ora de Gambier"}, {"Georgia", "ora estandard de Geòrgia"}, {"Gilbert_Islands", "ora des Isles Gilbert"}, {"Greenland_Eastern", "ora estandard de Groenlandia orientau"}, {"Greenland_Western", "ora estandard de Groenlandia occidentau"}, {"Gulf", "ora estandard deth Gòlf"}, {"Guyana", "ora de Guyana"}, {"Hawaii", "ora estandard de Hawai-Aleutianes"}, {"Hawaii_Aleutian", "ora estandard de Hawai-Aleutianes"}, {"Hong_Kong", "ora estandard de Hong Kong"}, {"Hovd", "ora estandard de Hovd"}, {"India", "ora estandard dera India"}, {"Indian_Ocean", "ora der Ocean Indic"}, {"Indochina", "ora d’Indochina"}, {"Indonesia_Central", "hora d’Indonesia centrau"}, {"Indonesia_Eastern", "hora d’Indonesia orientau"}, {"Indonesia_Western", "hora d’Indonesia occidentau"}, {"Iran", "ora estandard d’Iran"}, {"Irkutsk", "ora estandard d’Irkutsk"}, {"Israel", "ora estàndard d’Israèl"}, {"Japan", "ora estandard de Japon"}, {"Kazakhstan", "ora de Kazajistan"}, {"Kazakhstan_Eastern", "ora de Kazajistan orientau"}, {"Kazakhstan_Western", "ora de Kazajistan occidentau"}, {"Korea", "ora estandard de Corèa"}, {"Kosrae", "ora de Kosrae"}, {"Krasnoyarsk", "ora estandard de Krasnoyarsk"}, {"Kyrgystan", "ora de Kirguistan"}, {"Line_Islands", "ora des Espòrades Equatorials"}, {"Lord_Howe", "ora estandard de Lord Howe"}, {"Magadan", "ora estandard de Magadan"}, {"Malaysia", "ora de Malàisia"}, {"Maldives", "ora des Malvines"}, {"Marquesas", "ora des Isles Marqueses"}, {"Marshall_Islands", "ora des Isles Marshall"}, {"Mauritius", "ora estandard de Maurici"}, {"Mawson", "ora de Mawson"}, {"Mexico_Pacific", "ora estandard deth Pacífic de Mexic"}, {"Mongolia", "ora estandard de Ulan Bator"}, {"Moscow", "ora estandard de Moscòu"}, {"Myanmar", "ora de Myanmar"}, {"Nauru", "ora de Nauru"}, {"Nepal", "ora deth Nepal"}, {"New_Caledonia", "ora estandard de Nòva Caledònia"}, {"New_Zealand", "ora estandard de Nòva Zelanda"}, {"Newfoundland", "ora estandard de Terra-Nòva"}, {"Niue", "ora de Niue"}, {"Norfolk", "ora estandard dera Isla Norfolk"}, {"Noronha", "ora estandard de Fernando de Noronha"}, {"Novosibirsk", "ora estandard de Novosibirsk"}, {"Omsk", "ora estandard d’Omsk"}, {"Pakistan", "ora estandard deth Pakistan"}, {"Palau", "ora de Palaos"}, {"Papua_New_Guinea", "ora de Papua Nòva Guinèa"}, {"Paraguay", "ora estandard de Paraguay"}, {"Peru", "ora estandard de Perú"}, {"Philippines", "ora estandard de Filipines"}, {"Phoenix_Islands", "ora des Isles Fénix"}, {"Pierre_Miquelon", "ora estandard de St. Pierre e Miquelon"}, {"Pitcairn", "ora de Pitcairn"}, {"Ponape", "ora de Pohnpei"}, {"Pyongyang", "ora de Pyongyang"}, {"Reunion", "ora de Reünion"}, {"Rothera", "ora de Rothera"}, {"Sakhalin", "ora estandard de Sajalin"}, {"Samoa", "ora estandard de Samoa"}, {"Seychelles", "ora de Seychelles"}, {"Singapore", "ora de Singapor"}, {"Solomon", "ora des Isles Salomon"}, {"South_Georgia", "ora de Geòrgia deth Sud"}, {"Suriname", "ora de Surinam"}, {"Syowa", "ora de Syowa"}, {"Tahiti", "ora de Tahiti"}, {"Taipei", "ora estandard de Taipei"}, {"Tajikistan", "ora de Tajikistan"}, {"Tokelau", "ora de Tokelau"}, {"Tonga", "ora estandard de Tonga"}, {"Truk", "ora de Chuuk"}, {"Turkmenistan", "ora estandard de Turkmenistan"}, {"Tuvalu", "ora de Tuvalu"}, {"Uruguay", "ora estandard d’Uruguay"}, {"Uzbekistan", "ora estandard de Uzbekistan"}, {"Vanuatu", "ora estandard de Vanuatu"}, {"Venezuela", "ora de Veneçuela"}, {"Vladivostok", "ora estandard de Vladivostok"}, {"Volgograd", "ora estandard de Volgograd"}, {"Vostok", "ora de Vostok"}, {"Wake", "ora dera Isla Wake"}, {"Wallis", "ora de Wallis e Futuna"}, {"Yakutsk", "ora estandard de Yakutsk"}, {"Yekaterinburg", "ora estandard d’Ekaterimburg"}, {"Yukon", "ora de Yukon"}},
		{{"Alaska", "ora d’estiu d’Alaska"}, {"Amazon", "ora d’estiu der Amazònes"}, {"America_Central", "ora d’estiu centrau"}, {"America_Eastern", "ora d’estiu orientau"}, {"America_Mountain", "ora d’estiu des Montanhes Rocoses"}, {"America_Pacific", "ora d’estiu deth Pacific"}, {"Apia", "ora d’esiu d’Apia"}, {"Arabian", "ora d’estiu d’arabia"}, {"Argentina", "ora d’estiu d’Argentina"}, {"Argentina_Western", "ora d’estiu d’Argentina occidentau"}, {"Armenia", "ora d’estiu d’Armenia"}, {"Atlantic", "ora d’estiu der Atlantic"}, {"Australia_Central", "ora d’estiu d’Austràlia centrau"}, {"Australia_CentralWestern", "ora d’estiu d’Austràlia centreoccidentau"}, {"Australia_Eastern", "ora d’estiu d’Austràlia orientau"}, {"Australia_Western", "ora d’estiu estandard d’Australia occidentau"}, {"Azerbaijan", "ora d’estiu d’Azerbaijan"}, {"Azores", "ora d’estiu des Açòres"}, {"Bangladesh", "ora d’estiu de Bangladesh"}, {"Brasilia", "ora d’estiu de Brasilia"}, {"Cape_Verde", "ora d’estiu de Cap-Verd"}, {"Chatham", "ora d’estiu de Chatham"}, {"Chile", "ora d’estiu de Chile"}, {"China", "ora d’estiu de China"}, {"Colombia", "ora d’estiu de Colòmbia"}, {"Cook", "ora d’estiu mieja des Isles Cook"}, {"Cuba", "ora d’estiu de Cuba"}, {"Easter", "ora d’estiu dera isla de Pasqua"}, {"Europe_Central", "ora d’estiu d’Euròpa centrau"}, {"Europe_Eastern", "ora d’estiu d’Euròpa de l’èst"}, {"Europe_Western", "ora d’estiu d’Euròpa occidentau"}, {"Falkland", "ora d’estiu des isles Maldives"}, {"Fiji", "ora d’estiu de Fiji"}, {"Georgia", "ora d’estiu de Geòrgia"}, {"Greenland_Eastern", "ora d’estiu de Groenlandia orientau"}, {"Greenland_Western", "ora d’estiu de Groenlandia occidentau"}, {"Hawaii_Aleutian", "ora d’estiu de Hawai-Aleutianes"}, {"Hong_Kong", "ora d’estiu de Hong Kong"}, {"Hovd", "ora d’estiu de Hovd"}, {"Iran", "ora d’estiu d’Iran"}, {"Irkutsk", "ora d’estiu d’Irkutsk"}, {"Israel", "ora d’estiu d’Israèl"}, {"Japan", "ora d’estiu de Japon"}, {"Korea", "ora d’estiu de Corèa"}, {"Krasnoyarsk", "ora d’estiu de Krasnoyarsk"}, {"Lord_Howe", "ora d’estiu de Lord Howe"}, {"Magadan", "ora d’estiu de Magadan"}, {"Mauritius", "ora d’estiu de Maurici"}, {"Mexico_Pacific", "ora d’estiu deth Pacífic de Mexic"}, {"Mongolia", "ora d’estiu de Ulan Bator"}, {"Moscow", "ora d’estiu de Moscòu"}, {"New_Caledonia", "ora d’estiu de Nòva Caledònia"}, {"New_Zealand", "ora d’estiu de Nòva Zelanda"}, {"Newfoundland", "ora d’estiu de Terra-Nòva"}, {"Norfolk", "ora d’estiu dera Isla Norfolk"}, {"Noronha", "ora d’estiu de Fernando de Noronha"}, {"Novosibirsk", "ora d’estiu de Novosibirsk"}, {"Omsk", "ora d’estiu d’Omsk"}, {"Pakistan", "ora d’estiu deth Pakistan"}, {"Paraguay", "ora d’estiu de Paraguay"}, {"Peru", "ora d’estu de Perú"}, {"Philippines", "ora d’estiu de Filipines"}, {"Pierre_Miquelon", "ora d’estiu de St. Pierre e Miquelon"}, {"Sakhalin", "ora d’estiu de Sajalin"}, {"Samoa", "ora d’estiu de Samoa"}, {"Taipei", "ora d’estiu de Taipei"}, {"Tonga", "ora d’estiu de Tonga"}, {"Turkmenistan", "ora d’estiu de Turkmenistan"}, {"Uruguay", "ora d’estiu d’Uruguay"}, {"Uzbekistan", "ora d’estiu de Uzbekistan"}, {"Vanuatu", "ora d’estiu de Vanuatu"}, {"Vladivostok", "ora d’estiu de Vladivostok"}, {"Volgograd", "ora d’estiu de Volgograd"}, {"Yakutsk", "ora d’estiu de Yakutsk"}, {"Yekaterinburg", "ora d’estiu d’Ekaterimburg"}},
	},
	{
		{{"Afghanistan", "Sa’aatii Afgaanistaan"}, {"Africa_Central", "Sa’aatii Afrikaa Gidduugaleessaa"}, {"Africa_Eastern", "Sa’aatii Baha Afrikaa"}, {"Africa_Southern", "Sa’aatii Istaandaardii Afrikaa Kibbaa"}, {"Africa_Western", "Sa’aatii Afrikaa Dhihaa"}, {"Alaska", "Sa’aatii Istaandaardii Alaaskaa"}, {"Amazon", "Sa’aatii Istaandaardii Amazoon"}, {"America_Central", "Sa’aatii Istaandaardii Gidduugaleessaa"}, {"America_Eastern", "Sa’aatii Istaandaardii Bahaa"}, {"America_Mountain", "Sa’aatii Istaandaardii Maawonteen"}, {"America_Pacific", "Sa’aatii Istaandaardii Paasfiik"}, {"Apia", "Sa’aatii Istaandaardii Apia"}, {"Arabian", "Sa’aatii Istaandaardii Arabaa"}, {"Argentina", "Sa’aatii Istaandaardii Arjentiinaa"}, {"Argentina_Western", "Sa’aatii Istaandaardii Arjentiinaa Dhihaa"}, {"Armenia", "Sa’aatii Istaandaardii Armaaniyaa"}, {"Atlantic", "Sa’aatii Istaandaardii Atilaantiik"}, {"Australia_Central", "Sa’aatii Istaandaardii Awustiraaliyaa Gidduugaleessaa"}, {"Australia_CentralWestern", "Sa’aatii Istaandaardii Dhiha Awustiraaliyaa Gidduugaleessaa"}, {"Australia_Eastern", "Sa’aatii Istaandaardii Awustiraaliyaa Bahaa"}, {"Australia_Western", "Sa’aatii Sa’aatii Istaandaardii Awustiraaliyaa DhihaaDhiha Awustiraaliyaa"}, {"Azerbaijan", "Sa’aatii Istaandaardii Azerbaajiyaan"}, {"Azores", "Sa’aatii Istaandaardii Azeeroos"}, {"Bangladesh", "Sa’aatii Istaandaardii Baangilaadish"}, {"Bhutan", "Sa’aatii Bihutaan"}, {"Bolivia", "Sa’aatii Boliiviyaa"}, {"Brasilia", "Sa’aatii Istaandaardii Biraaziliyaa"}, {"Brunei", "Sa’aatii Bruunee Darusalaam"}, {"Cape_Verde", "Sa’aatii Istaandaardii Keep Veerdee"}, {"Chamorro", "Sa’aatii Istaandaardii Kamoroo"}, {"Chatham", "Sa’aatii Istaandaardii Chatham"}, {"Chile", "Sa’aatii Istaandaardii Chiilii"}, {"China", "Sa’aatii Istaandaardii Chaayinaa"}, {"Christmas", "Sa’aatii Odola Kirismaas"}, {"Cocos", "Sa’aatii Odoloota Kokos"}, {"Colombia", "Sa’aatii Istaandaardii Kolombiyaa"}, {"Cook", "Sa’aatii Istaandaardii Odoloota Kuuk"}, {"Cuba", "Sa’aatii Istaandaardii Kuubaa"}, {"Davis", "Sa’aatii Daaviis"}, {"DumontDUrville", "Sa’aatii Dumont-d’Urville"}, {"East_Timor", "Sa’aatii Tiimoor Bahaa"}, {"Easter", "Sa’aatii Istaandaardii Odola Bahaa"}, {"Ecuador", "Sa’aatii Ikkuwaadoor"}, {"Europe_Central", "Sa’aatii Istaandaardii Awurooppaa Gidduugaleessaa"}, {"Europe_Eastern", "Sa’aatii Istaandaardii Awurooppaa Bahaa"}, {"Europe_Further_Eastern", "Sa’aatii Awurooppaa Bahaa Dabalataa"}, {"Europe_Western", "Sa’aatii Istaandaardii Awurooppaa Dhihaa"}, {"Falkland", "Sa’aatii Istaandaardii Odoloota Faalklaand"}, {"Fiji", "Sa’aatii Istaandaardii Fiijii"}, {"French_Guiana", "Sa’aatii Fireench Guyinaa"}, {"French_Southern", "Sa’aatii Firaans Kibbaa fi Antaarktikaa"}, {"GMT", "Sa’aatii Giriinwiich Gidduugaleessaa"}, {"Galapagos", "Sa’aatii Galaapagoos"}, {"Gambier", "Sa’aatii Gaambiyeer"}, {"Georgia", "Sa’aatii Istaandaardii Joorjiyaa"}, {"Gilbert_Islands", "Sa’aatii Odoloota Giilbert"}, {"Greenland_Eastern", "Sa’aatii Istaandaardii Giriinlaand Bahaa"}, {"Greenland_Western", "Sa’aatii Istaandaardii Giriinlaand Dhihaa"}, {"Guam", "Sa’aatii Istaandaardii Guwaam"}, {"Gulf", "Sa’aatii Istaandaardii Gaalfii"}, {"Guyana", "Sa’aatii Guyaanaa"}, {"Hawaii", "Sa’aatii Istaandaardii Haawayi-Alewutiyan"}, {"Hawaii_Aleutian", "Sa’aatii Istaandaardii Haawayi-Alewutiyan"}, {"Hong_Kong", "Sa’aatii Istaandaardii Hoong Koong"}, {"Hovd", "Sa’aatii Istaandaardii Hoovd"}, {"India", "Sa’aatii Istaandaardii Hindii"}, {"Indian_Ocean", "Sa’aatii Galaana Hindii"}, {"Indochina", "Sa’aatii IndooChaayinaa"}, {"Indonesia_Central", "Sa’aatii Indooneeshiyaa Gidduugaleessaa"}, {"Indonesia_Eastern", "Sa’aatii Indooneshiyaa Bahaa"}, {"Indonesia_Western", "Sa’aatii Indooneeshiyaa Dhihaa"}, {"Iran", "Sa’aatii Istaandaardii Iraan"}, {"Irkutsk", "Sa’aatii Istaandaardii Irkutsk"}, {"Israel", "Sa’aatii Istaandaardii Israa’eel"}, {"Japan", "Sa’aatii Istaandaardii Jaappaan"}, {"Kazakhstan", "Sa’aatii Kaazaakistaan"}, {"Kazakhstan_Eastern", "Sa’aatii Kaazaakistaan Bahaa"}, {"Kazakhstan_Western", "Sa’aatii Kaazaakistaan Dhihaa"}, {"Korea", "Sa’aatii Istaandaardii Kooriyaa"}, {"Kosrae", "Sa’aatii Koosreyaa"}, {"Krasnoyarsk", "Sa’aatii Istaandaardii Krasnoyarsk"}, {"Kyrgystan", "Sa’aatii Kiyirigiyistan"}, {"Lanka", "Sa’aatii Laankaa"}, {"Line_Islands", "Sa’aatii Odoloota Line"}, {"Lord_Howe", "Sa’aatii Istaandaardii Lord Howe"}, {"Magadan", "Sa’aatii Istaandaardii Magadan"}, {"Malaysia", "Sa’aatii Maaleeshiyaa"}, {"Maldives", "Sa’aatii Maaldiivs"}, {"Marquesas", "Sa’aatii Marquesas"}, {"Marshall_Islands", "Sa’aatii Odoloota Maarshaal"}, {"Mauritius", "Sa’aatii Istaandaardii Moorishiyees"}, {"Mawson", "Sa’aatii Mawson"}, {"Mexico_Pacific", "Sa’aatii Istaandaardii Paasfiik Meksiikaan"}, {"Mongolia", "Sa’aatii Istaandaardii Ulaanbaatar"}, {"Moscow", "Sa’aatii Istaandaardii Mooskoo"}, {"Myanmar", "Sa’aatii Maayinaamaar"}, {"Nauru", "Sa’aatii Naawuruu"}, {"Nepal", "Sa’aatii Neeppaal"}, {"New_Caledonia", "Sa’aatii Istaandaardii Kaaledooniyaa Haaraa"}, {"New_Zealand", "Sa’aatii Istaandaardii New Zealand"}, {"Newfoundland", "Sa’aatii Istaandaardii Newufaawondlaand"}, {"Niue", "Sa’aatii Niue"}, {"Norfolk", "Sa’aatii Istaandaardii Norfolk Island"}, {"Noronha", "Sa’aatii Istaandaardii Fernando de Noronha"}, {"North_Mariana", "Sa’aatii Odoloota Maariyaanaa Kaabaa"}, {"Novosibirsk", "Sa’aatii Istaandaardii Novosibirisk"}, {"Omsk", "Sa’aatii Istaandaardii Omsk"}, {"Pakistan", "Sa’aatii Istaandaardii Paakistaan"}, {"Palau", "Sa’aatii Palawu"}, {"Papua_New_Guinea", "Sa’aatii Paapuwaa Giinii Haaraa"}, {"Paraguay", "Sa’aatii Istaandaardii Paaraaguwaayi"}, {"Peru", "Sa’aatii Istaandaardii Peeruu"}, {"Philippines", "Sa’aatii Istaandaardii Filippiins"}, {"Phoenix_Islands", "Sa’aatii Odoloota Fooneeks"}, {"Pierre_Miquelon", "Sa’aatii Istaandaardii Ql. Piyeeree fi Mikuyelo"}, {"Pitcairn", "Sa’aatii Pitcairn"}, {"Ponape", "Sa’aatii Ponape"}, {"Pyongyang", "Sa’aatii Piyoongyaang"}, {"Reunion", "Sa’aatii Riiyuuniyeen"}, {"Rothera", "Sa’aatii Rothera"}, {"Sakhalin", "Sa’aatii Istaandaardii Sakhalin"}, {"Samoa", "Sa’aatii Istaandaardii Saamowaa"}, {"Seychelles", "Sa’aatii Siisheels"}, {"Singapore", "Sa’aatii Istaandaardii Singaapoor"}, {"Solomon", "Sa’aatii Odoloota Solomoon"}, {"South_Georgia", "Sa’aatii Joorjiyaa Kibbaa"}, {"Suriname", "Sa’aatii Surinaame"}, {"Syowa", "Sa’aatii Syowa"}, {"Tahiti", "Sa’aatii Tahiti"}, {"Taipei", "Sa’aatii Istaandaardii Tayipeyi"}, {"Tajikistan", "Sa’aatii Tajikistaan"}, {"Tokelau", "Sa’aatii Takelawu"}, {"Tonga", "Sa’aatii Istaandaardii Tonga"}, {"Truk", "Sa’aatii Chuuk"}, {"Turkmenistan", "Sa’aatii Istaandaardii Turkemenistaan"}, {"Tuvalu", "Sa’aatii Tuvalu"}, {"Uruguay", "Sa’aatii Istaandaardii Yuraagaayi"}, {"Uzbekistan", "Sa’aatii Istaandaardii Uzbeekistaan"}, {"Vanuatu", "Sa’aatii Istaandaardii Vanuwatu"}, {"Venezuela", "Sa’aatii Veenzuweelaa"}, {"Vladivostok", "Sa’aatii Istaandaardii Vladivostok"}, {"Volgograd", "Sa’aatii Istaandaardii Volgograd"}, {"Vostok", "Sa’aatii Vostok"}, {"Wake", "Sa’aatii Odola Wake"}, {"Wallis", "Sa’aatii Wallis fi Futuna"}, {"Yakutsk", "Sa’aatii Istaandardii Yakutsk"}, {"Yekaterinburg", "Sa’aatii Istaandaardii Yekaterinburg"}, {"Yukon", "Sa’aatii Yuukoon"}},
		{{"Alaska", "Sa’aatii Guyyaa Alaaskaa"}, {"Amazon", "Sa’aatii Bonaa Amazoon"}, {"America_Central", "Sa’aatii Guyyaa Gidduugaleessaa"}, {"America_Eastern", "Sa’aatii Guyyaa Bahaa"}, {"America_Mountain", "Sa’aatii Guyyaa Maawonteen"}, {"America_Pacific", "Sa’aatii Guyyaa Paasfiik"}, {"Apia", "Sa’aatii Guyyaa Apia"}, {"Arabian", "Sa’aatii Guyyaa Arabaa"}, {"Argentina", "Sa’aatii Bonaa Arjentiinaa"}, {"Argentina_Western", "Sa’aatii Bonaa Arjentiinaa Dhihaa"}, {"Armenia", "Sa’aatii Bonaa Armaaniyaa"}, {"Atlantic", "Sa’aatii Guyyaa Atilaantiik"}, {"Australia_Central", "Sa’aatii Guyyaa Awustiraaliyaa Gidduugaleessaa"}, {"Australia_CentralWestern", "Sa’aatii Guyyaa Dhiha Awustiraaliyaa Gidduugaleessaa"}, {"Australia_Eastern", "Sa’aatii Guyyaa Awustiraaliyaa Bahaa"}, {"Australia_Western", "Sa’aatii Guuyyaa Awustiraaliyaa Dhihaa"}, {"Azerbaijan", "Sa’aatii Bonaa Azerbaajiyaan"}, {"Azores", "Sa’aatii Bonaa Azeeroos"}, {"Bangladesh", "Sa’aatii Bonaa Baangilaadish"}, {"Brasilia", "Sa’aatii Bonaa Biraaziliyaa"}, {"Cape_Verde", "Sa’aatii Bonaa Keep Veerdee"}, {"Chatham", "Sa’aatii Guyyaa Chatham"}, {"Chile", "Sa’aatii Bonaa Chiilii"}, {"China", "Sa’aatii Guyyaa Chaayinaa"}, {"Colombia", "Sa’aatii Bonaa Kolombiyaa"}, {"Cook", "Sa’aatii Bona Walakkaa Odoloota Kuuk"}, {"Cuba", "Sa’aatii Guyyaa Kuubaa"}, {"Easter", "Sa’aatii Bonaa Odola Bahaa"}, {"Europe_Central", "Sa’aatii Bonaa Awurooppaa Gidduugaleessaa"}, {"Europe_Eastern", "Sa’aatii Bonaa Awurooppaa Bahaa"}, {"Europe_Western", "Sa’aatii Bonaa Awurooppaa Dhihaa"}, {"Falkland", "Sa’aatii Bonaa Odoloota Faalklaand"}, {"Fiji", "Sa’aatii Bonaa Fiijii"}, {"Georgia", "Sa’aatii Bonaa Joorjiyaa"}, {"Greenland_Eastern", "Sa’aatii Bonaa Giriinlaand Bahaa"}, {"Greenland_Western", "Sa’aatii Bonaa Giriinlaand Dhihaa"}, {"Hawaii_Aleutian", "Sa’aatii Guyyaa Haawayi-Alewutiyan"}, {"Hong_Kong", "Sa’aatii Bonaa Hoong Koong"}, {"Hovd", "Sa’aatii Bonaa Hoovd"}, {"Iran", "Sa’aatii Guyyaa Iraan"}, {"Irkutsk", "Sa’aatii Bonaa Irkutsk"}, {"Israel", "Sa’aatii Guyyaa Israa’eel"}, {"Japan", "Sa’aatii Guyyaa Jaappaan"}, {"Korea", "Sa’aatii Guyyaa Kooriyaa"}, {"Krasnoyarsk", "Sa’aatii Bonaa Krasnoyarsk"}, {"Lord_Howe", "Sa’aatii Guyyaa Lord Howe"}, {"Magadan", "Sa’aatii Bonaa Magadan"}, {"Mauritius", "Sa’aatii Bonaa Moorishiyees"}, {"Mexico_Pacific", "Sa’aatii Guyyaa Paasfiik Meksiikaan"}, {"Mongolia", "Sa’aatii Bonaa Ulaanbaatar"}, {"Moscow", "Sa’aatii Bonaa Mooskoo"}, {"New_Caledonia", "Sa’aatii Bonaa Kaaledooniyaa Haaraa"}, {"New_Zealand", "Sa’aatii Guyyaa New Zealand"}, {"Newfoundland", "Sa’aatii Guyyaa Newufaawondlaand"}, {"Norfolk", "Sa’aatii Guyyaa Norfolk Island"}, {"Noronha", "Sa’aatii Bonaa Fernando de Noronha"}, {"Novosibirsk", "Sa’aatii Bonaa Novosibirisk"}, {"Omsk", "Sa’aatii Bonaa Omsk"}, {"Pakistan", "Sa’aatii Bonaa Paakistaan"}, {"Paraguay", "Sa’aatii Bonaa Paaraaguwaayi"}, {"Peru", "Sa’aatii Bonaa Peeruu"}, {"Philippines", "Sa’aatii Bonaa Filippiins"}, {"Pierre_Miquelon", "Sa’aatii Guyyaa Ql. Piyeeree fi Mikuyelo"}, {"Sakhalin", "Sa’aatii Bonaa Sakhalin"}, {"Samoa", "Sa’aatii Guyyaa Saamowaa"}, {"Taipei", "Sa’aatii Guyyaa Tayipeyi"}, {"Tonga", "Sa’aatii Bonaa Tonga"}, {"Turkmenistan", "Sa’aatii Bonaa Turkemenistaan"}, {"Uruguay", "Sa’aatii Bonaa Yuraagaayi"}, {"Uzbekistan", "Sa’aatii Bonaa Uzbeekistaan"}, {"Vanuatu", "Sa’aatii Bonaa Vanuwatu"}, {"Vladivostok", "Sa’aatii Bonaa Vladivostok"}, {"Volgograd", "Sa’aatii Bonaa Volgograd"}, {"Yakutsk", "Sa’aatii Bonaa Yakutsk"}, {"Yekaterinburg", "Sa’aatii Bonaa Yekaterinburg"}},
	},
	{
		{{"Acre", "ଆକା ମାନକ ସମୟ"}, {"Afghanistan", "ଆଫଗାନିସ୍ତାନ ସମୟ"}, {"Africa_Central", "ମଧ୍ୟ ଆଫ୍ରିକା ସମୟ"}, {"Africa_Eastern", "ପୂର୍ବ ଆଫ୍ରିକା ସମୟ"}, {"Africa_Southern", "ଦକ୍ଷିଣ ଆଫ୍ରିକା ମାନାଙ୍କ ସମୟ"}, {"Africa_Western", "ପଶ୍ଚିମ ଆଫ୍ରିକା ସମୟ"}, {"Alaska", "ଆଲାସ୍କା ମାନାଙ୍କ ସମୟ"}, {"Almaty", "ଅଲମାଟୀ ମାନକ ସମୟ"}, {"Amazon", "ଆମାଜନ୍ ମାନାଙ୍କ ସମୟ"}, {"America_Central", "କେନ୍ଦ୍ରୀୟ ମାନାଙ୍କ ସମୟ"}, {"America_Eastern", "ପୂର୍ବାଞ୍ଚଳ ମାନାଙ୍କ ସମୟ"}, {"America_Mountain", "ପାର୍ବତ୍ୟ ମାନାଙ୍କ ସମୟ"}, {"America_Pacific", "ପାସିଫିକ୍ ମାନାଙ୍କ ସମୟ"}, {"Anadyr", "ଅନାଡିର୍ ମାନକ ସମୟ"}, {"Apia", "ଆପିଆ ମାନାଙ୍କ ସମୟ"}, {"Aqtau", "ଅକତୌ ମାନକ ସମୟ"}, {"Aqtobe", "ଅକ୍ତୋବ ମାନକ ସମୟ"}, {"Arabian", "ଆରବୀୟ ମାନାଙ୍କ ସମୟ"}, {"Argentina", "ଆର୍ଜେଣ୍ଟିନା ମାନାଙ୍କ ସମୟ"}, {"Argentina_Western", "ପଶ୍ଚିମ ଆର୍ଜେଣ୍ଟିନା ମାନାଙ୍କ ସମୟ"}, {"Armenia", "ଆର୍ମେନିଆ ମାନାଙ୍କ ସମୟ"}, {"Atlantic", "ଆଟଲାଣ୍ଟିକ୍ ମାନାଙ୍କ ସମୟ"}, {"Australia_Central", "ଅଷ୍ଟ୍ରେଲିୟ ମଧ୍ୟ ମାନାଙ୍କ ସମୟ"}, {"Australia_CentralWestern", "ଅଷ୍ଟ୍ରେଲିୟ ମଧ୍ୟ ପଶ୍ଚିମ ମାନାଙ୍କ ସମୟ"}, {"Australia_Eastern", "ଅଷ୍ଟ୍ରେଲିୟ ପୂର୍ବ ମାନାଙ୍କ ସମୟ"}, {"Australia_Western", "ଅଷ୍ଟ୍ରେଲିୟ ପଶ୍ଚିମ ମାନାଙ୍କ ସମୟ"}, {"Azerbaijan", "ଆଜେରବାଇଜାନ ମାନଙ୍କ ସମୟ"}, {"Azores", "ଆଜୋରେସ୍ ମାନକ ସମୟ"}, {"Bangladesh", "ବାଂଲାଦେଶ ମାନାଙ୍କ ସମୟ"}, {"Bhutan", "ଭୁଟାନ ସମୟ"}, {"Bolivia", "ବଲିଭିଆ ସମୟ"}, {"Brasilia", "ବ୍ରାସିଲିଆ ମାନାଙ୍କ ସମୟ"}, {"Brunei", "ବ୍ରୁନେଇ ଡାରୁସାଲାମ ସମୟ"}, {"Cape_Verde", "କେପ୍\u200c ଭର୍ଦେ ମାନାଙ୍କ ସମୟ"}, {"Chamorro", "ଚାମୋରୋ ମାନକ ସମୟ"}, {"Chatham", "ଚାଥାମ୍\u200c ମାନାଙ୍କ ସମୟ"}, {"Chile", "ଚିଲି ମାନାଙ୍କ ସମୟ"}, {"China", "ଚୀନ ମାନକ ସମୟ"}, {"Christmas", "ଖ୍ରୀଷ୍ଟମାସ ଦ୍ୱୀପ ସମୟ"}, {"Cocos", "କୋକୋସ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"Colombia", "କଲମ୍ବିଆ ମାନକ ସମୟ"}, {"Cook", "କୁକ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ମାନକ ସମୟ"}, {"Cuba", "କ୍ୟୁବା ମାନାଙ୍କ ସମୟ"}, {"Davis", "ଡେଭିସ୍\u200c ସମୟ"}, {"DumontDUrville", "ଡୁମୋଣ୍ଟ-ଡି‘ଉରଭିଲ୍ଲେ ସମୟ"}, {"East_Timor", "ପୂର୍ବ ତିମୋର୍\u200c ସମୟ"}, {"Easter", "ଇଷ୍ଟର୍\u200c ଆଇଲ୍ୟାଣ୍ଡ ମାନାଙ୍କ ସମୟ"}, {"Ecuador", "ଇକ୍ୱେଡର ସମୟ"}, {"Europe_Central", "କେନ୍ଦ୍ର ୟୁରୋପୀୟ ମାନକ ସମୟ"}, {"Europe_Eastern", "ପୂର୍ବାଞ୍ଚଳ ୟୁରୋପୀୟ ମାନାଙ୍କ ସମୟ"}, {"Europe_Further_Eastern", "ପରବର୍ତ୍ତୀ-ପୂର୍ବାଞ୍ଚଳ ୟୁରୋପୀୟ ସମୟ"}, {"Europe_Western", "ପଶ୍ଚିମାଞ୍ଚଳ ୟୁରୋପୀୟ ମାନାଙ୍କ ସମୟ"}, {"Falkland", "ଫକଲ୍ୟାଣ୍ଡ ଦ୍ୱୀପପୁଞ୍ଜ ମାନକ ସମୟ"}, {"Fiji", "ଫିଜି ମାନକ ସମୟ"}, {"French_Guiana", "ଫ୍ରେଞ୍ଚ ଗୁଆନା ସମୟ"}, {"French_Southern", "ଫ୍ରେଞ୍ଚ ଦକ୍ଷିଣ ଏବଂ ଆଣ୍ଟାର୍କାଟିକ୍\u200c ସମୟ"}, {"GMT", "ଗ୍ରୀନୱିଚ୍ ମିନ୍ ସମୟ"}, {"Galapagos", "ଗାଲାପାଗୋସ୍ ସମୟ"}, {"Gambier", "ଗାମ୍ବିୟର୍ ସମୟ"}, {"Georgia", "ଜର୍ଜିଆ ମାନାଙ୍କ ସମୟ"}, {"Gilbert_Islands", "ଗିଲବର୍ଟ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"Greenland_Eastern", "ପୂର୍ବ ଗ୍ରୀନଲ୍ୟାଣ୍ଡ୍ ମାନାଙ୍କ ସମୟ"}, {"Greenland_Western", "ପଶ୍ଚିମ ଗ୍ରୀନଲ୍ୟାଣ୍ଡ୍ ମାନାଙ୍କ ସମୟ"}, {"Gulf", "ଗଲ୍ଫ ମାନାଙ୍କ ସମୟ"}, {"Guyana", "ଗୁଏନା ସମୟ"}, {"Hawaii", "ହୱାଇ-ଆଲେଉଟିୟ ମାନାଙ୍କ ସମୟ"}, {"Hawaii_Aleutian", "ହୱାଇ-ଆଲେଉଟିୟ ମାନାଙ୍କ ସମୟ"}, {"Hong_Kong", "ହଂ କଂ ମାନକ ସମୟ"}, {"Hovd", "ହୋଭଡ୍\u200c ମାନକ ସମୟ"}, {"India", "ଭାରତୀୟ ମାନକ ସମୟ"}, {"Indian_Ocean", "ଭାରତ ମାହାସାଗର ସମୟ"}, {"Indochina", "ଇଣ୍ଡୋଚାଇନା ସମୟ"}, {"Indonesia_Central", "ମଧ୍ୟ ଇଣ୍ଡୋନେସିଆ ସମୟ"}, {"Indonesia_Eastern", "ପୂର୍ବ ଇଣ୍ଡୋନେସିଆ ସମୟ"}, {"Indonesia_Western", "ପଶ୍ଚିମ ଇଣ୍ଡୋନେସିଆ ସମୟ"}, {"Iran", "ଇରାନ ମାନକ ସମୟ"}, {"Irkutsk", "ଇର୍କୁଟସ୍କ୍ ମାନକ ସମୟ"}, {"Israel", "ଇସ୍ରାଏଲ ମାନାଙ୍କ ସମୟ"}, {"Japan", "ଜାପାନ ମାନକ ସମୟ"}, {"Kamchatka", "ପେଟ୍ରୋପାଭଲୋଭ୍ସକ-କମଚଟସ୍କି ମାନକ ସମୟ"}, {"Kazakhstan", "କାଜାକସ୍ତାନ୍ ସମୟ"}, {"Kazakhstan_Eastern", "ପୂର୍ବ କାଜାକସ୍ତାନ୍ ସମୟ"}, {"Kazakhstan_Western", "ପଶ୍ଚିମ କାଜାକସ୍ତାନ ସମୟ"}, {"Korea", "କୋରିୟ ମାନକ ସମୟ"}, {"Kosrae", "କୋସରେଇ ସମୟ"}, {"Krasnoyarsk", "କ୍ରାସନୋୟାରସ୍କ ମାନାଙ୍କ ସମୟ"}, {"Kyrgystan", "କିର୍ଗିସ୍ତାନ ସମୟ"}, {"Lanka", "ଲଙ୍କା ସମୟ"}, {"Line_Islands", "ଲାଇନ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"Lord_Howe", "ଲର୍ଡ ହୋୱେ ମାନାଙ୍କ ସମୟ"}, {"Macau", "ମକାଓ ମାନକ ସମୟ"}, {"Magadan", "ମାଗାଡାନ୍ ମାନାଙ୍କ ସମୟ"}, {"Malaysia", "ମାଲେସିଆ ସମୟ"}, {"Maldives", "ମାଳଦ୍ୱୀପ ସମୟ"}, {"Marquesas", "ମାର୍କ୍ୱେସାସ୍\u200c ସମୟ"}, {"Marshall_Islands", "ମାର୍ଶାଲ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"Mauritius", "ମୌରିସସ୍\u200c ମାନାଙ୍କ ସମୟ"}, {"Mawson", "ମାୱସନ୍\u200c ସମୟ"}, {"Mexico_Pacific", "ମେକ୍ସିକାନ୍ ପାସିଫିକ୍ ମାନାଙ୍କ ସମୟ"}, {"Mongolia", "ଉଲାନ୍\u200cବାଟର୍\u200c ମାନକ ସମୟ"}, {"Moscow", "ମସ୍କୋ ମାନାଙ୍କ ସମୟ"}, {"Myanmar", "ମିଆଁମାର୍\u200c ସମୟ"}, {"Nauru", "ନାଉରୁ ସମୟ"}, {"Nepal", "ନେପାଳ ସମୟ"}, {"New_Caledonia", "ନ୍ୟୁ କାଲେଡୋନିଆ ମାନାଙ୍କ ସମୟ"}, {"New_Zealand", "ନ୍ୟୁଜିଲାଣ୍ଡ ମାନାଙ୍କ ସମୟ"}, {"Newfoundland", "ନ୍ୟୁଫାଉଣ୍ଡଲ୍ୟାଣ୍ଡ୍ ମାନାଙ୍କ ସମୟ"}, {"Niue", "ନିୟୁ ସମୟ"}, {"Norfolk", "ନରଫୋକ୍\u200c ଦ୍ୱୀପ ମାନାଙ୍କ ସମୟ"}, {"Noronha", "ଫର୍ଣ୍ଣାଣ୍ଡୋ ଡି ନୋରୋନ୍ନା ମାନାଙ୍କ ସମୟ"}, {"North_Mariana", "ଉତ୍ତର ମେରିଆନା ଆଇଲ୍ୟାଣ୍ଡ ସମୟ"}, {"Novosibirsk", "ନୋଭୋସିବିରସ୍କ ମାନାଙ୍କ ସମୟ"}, {"Omsk", "ଓମସ୍କ ମାନାଙ୍କ ସମୟ"}, {"Pakistan", "ପାକିସ୍ତାନ ମାନକ ସମୟ"}, {"Palau", "ପାଲାଉ ସମୟ"}, {"Papua_New_Guinea", "ପପୁଆ ନ୍ୟୁ ଗିନି ସମୟ"}, {"Paraguay", "ପାରାଗୁଏ ମାନାଙ୍କ ସମୟ"}, {"Peru", "ପେରୁ ମାନାଙ୍କ ସମୟ"}, {"Philippines", "ଫିଲିପାଇନ୍\u200c ମାନାଙ୍କ ସମୟ"}, {"Phoenix_Islands", "ଫିନିକ୍ସ ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"Pierre_Miquelon", "ସେଣ୍ଟ. ପିଏରେ ଏବଂ ମିକ୍ୟୁଲୋନ୍ ମାନାଙ୍କ ସମୟ"}, {"Pitcairn", "ପିଟକେର୍ନ୍\u200c ସମୟ"}, {"Ponape", "ପୋନାପେ ସମୟ"}, {"Pyongyang", "ପୋୟଙ୍ଗୟାଙ୍ଗ ସମୟ"}, {"Qyzylorda", "କାଜିଲୋର୍ଡା ମାନକ ସମୟ"}, {"Reunion", "ରିୟୁନିଅନ୍\u200c ସମୟ"}, {"Rothera", "ରୋଥେରା ସମୟ"}, {"Sakhalin", "ସଖାଲିନ୍ ମାନାଙ୍କ ସମୟ"}, {"Samara", "ସମାରା ମାନକ ସମୟ"}, {"Samoa", "ସାମୋଆ ମାନକ ସମୟ"}, {"Seychelles", "ସେଚେଲ୍ଲେସ୍\u200c ସମୟ"}, {"Singapore", "ସିଙ୍ଗାପୁର୍\u200c ମାନକ ସମୟ"}, {"Solomon", "ସୋଲୋମନ ଦ୍ୱୀପପୁଞ୍ଜ ସମୟ"}, {"South_Georgia", "ଦକ୍ଷିଣ ଜର୍ଜିଆ ସମୟ"}, {"Suriname", "ସୁରିନେମ୍\u200c ସମୟ"}, {"Syowa", "ସୋୱା ସମୟ"}, {"Tahiti", "ତାହିତି ସମୟ"}, {"Taipei", "ତାଇପେଇ ମାନକ ସମୟ"}, {"Tajikistan", "ତାଜିକିସ୍ତାନ ସମୟ"}, {"Tokelau", "ଟୋକେଲାଉ ସମୟ"}, {"Tonga", "ଟୋଙ୍ଗା ମାନକ ସମୟ"}, {"Truk", "ଚୂକ୍\u200c ସମୟ"}, {"Turkmenistan", "ତୁର୍କମେନିସ୍ତାନ ମାନାଙ୍କ ସମୟ"}, {"Tuvalu", "ତୁଭାଲୁ ସମୟ"}, {"Uruguay", "ଉରୁଗୁଏ ମାନାଙ୍କ ସମୟ"}, {"Uzbekistan", "ଉଜବେକିସ୍ତାନ ମାନକ ସମୟ"}, {"Vanuatu", "ଭାନୁଆଟୁ ମାନକ ସମୟ"}, {"Venezuela", "ଭେନିଜୁଏଲା ସମୟ"}, {"Vladivostok", "ଭ୍ଲାଡିଭୋଷ୍ଟୋକ୍ ମାନାଙ୍କ ସମୟ"}, {"Volgograd", "ଭୋଲଗୋଗ୍ରାଡ୍ ମାନାଙ୍କ ସମୟ"}, {"Vostok", "ଭୋଷ୍ଟୋକ୍\u200c ସମୟ"}, {"Wake", "ୱେକ୍\u200c ଦ୍ୱୀପ ସମୟ"}, {"Wallis", "ୱାଲିସ୍\u200c ଏବଂ ଫୁଟୁନା ସମୟ"}, {"Yakutsk", "ୟାକୁଟସ୍କ ମାନାଙ୍କ ସମୟ"}, {"Yekaterinburg", "ୟେକାଟେରିନବର୍ଗ୍ ମାନାଙ୍କ ସମୟ"}, {"Yukon", "ୟୁକୋନ୍ ସମୟ"}},
		{{"Acre", "ଆକା ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Alaska", "ଆଲାସ୍କା ଦିବାଲୋକ ସମୟ"}, {"Almaty", "ଅଲମାଟୀ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Amazon", "ଆମାଜନ୍ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"America_Central", "କେନ୍ଦ୍ରୀୟ ଦିବାଲୋକ ସମୟ"}, {"America_Eastern", "ପୂର୍ବାଞ୍ଚଳ ଦିବାଲୋକ ସମୟ"}, {"America_Mountain", "ପାର୍ବତ୍ୟ ଦିବାଲୋକ ସମୟ"}, {"America_Pacific", "ପାସିଫିକ୍ ଦିବାଲୋକ ସମୟ"}, {"Anadyr", "ଅନାଡିର୍ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Apia", "ଆପିଆ ଦିବାଲୋକ ସମୟ"}, {"Aqtau", "ଅକତୌ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Aqtobe", "ଅକ୍ତୋବ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Arabian", "ଆରବୀୟ ଦିବାଲୋକ ସମୟ"}, {"Argentina", "ଆର୍ଜେଣ୍ଟିନା ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Argentina_Western", "ପଶ୍ଚିମ ଆର୍ଜେଣ୍ଟିନା ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Armenia", "ଆର୍ମେନିଆ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Atlantic", "ଆଟଲାଣ୍ଟିକ୍ ଦିବାଲୋକ ସମୟ"}, {"Australia_Central", "ଅଷ୍ଟ୍ରେଲିୟ ମଧ୍ୟ ଦିବାଲୋକ ସମୟ"}, {"Australia_CentralWestern", "ଅଷ୍ଟ୍ରେଲିୟ ମଧ୍ୟ ପଶ୍ଚିମ ଦିବାଲୋକ ସମୟ"}, {"Australia_Eastern", "ଅଷ୍ଟ୍ରେଲିୟ ପୂର୍ବ ଦିବାଲୋକ ସମୟ"}, {"Australia_Western", "ଅଷ୍ଟ୍ରେଲିୟ ପଶ୍ଚିମ ଦିବାଲୋକ ସମୟ"}, {"Azerbaijan", "ଆଜେରବାଇଜାନ ଖରାଦିନ ସମୟ"}, {"Azores", "ଆଜୋରେସ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Bangladesh", "ବାଂଲାଦେଶ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Brasilia", "ବ୍ରାସିଲିଆ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Cape_Verde", "କେପ୍\u200c ଭର୍ଦେ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Chatham", "ଚାଥାମ୍\u200c ଦିବାଲୋକ ସମୟ"}, {"Chile", "ଚିଲି ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"China", "ଚୀନ ଦିବାଲୋକ ସମୟ"}, {"Colombia", "କଲମ୍ବିଆ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Cook", "କୁକ୍\u200c ଦ୍ୱୀପପୁଞ୍ଜ ଅର୍ଦ୍ଧ ଗୀଷ୍ମକାଳୀନ ସମୟ"}, {"Cuba", "କ୍ୟୁବା ଦିବାଲୋକ ସମୟ"}, {"Easter", "ଇଷ୍ଟର୍\u200c ଆଇଲ୍ୟାଣ୍ଡ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Europe_Central", "କେନ୍ଦ୍ରୀୟ ୟୁରୋପୀୟ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Europe_Eastern", "ପୂର୍ବାଞ୍ଚଳ ୟୁରୋପୀୟ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Europe_Western", "ପଶ୍ଚିମାଞ୍ଚଳ ୟୁରୋପୀୟ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Falkland", "ଫକଲ୍ୟାଣ୍ଡ ଦ୍ୱୀପପୁଞ୍ଜ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Fiji", "ଫିଜି ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Georgia", "ଜର୍ଜିଆ ଖରାଦିନ ସମୟ"}, {"Greenland_Eastern", "ପୂର୍ବ ଗ୍ରୀନଲ୍ୟାଣ୍ଡ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Greenland_Western", "ପଶ୍ଚିମ ଗ୍ରୀନଲ୍ୟାଣ୍ଡ୍ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Hawaii_Aleutian", "ହୱାଇ-ଆଲେଉଟିୟ ଦିବାଲୋକ ସମୟ"}, {"Hong_Kong", "ହଂ କଂ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Hovd", "ହୋଭଡ୍\u200c ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Iran", "ଇରାନ ଦିବାଲୋକ ସମୟ"}, {"Irkutsk", "ଇର୍କୁଟସ୍କ୍ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Israel", "ଇସ୍ରାଏଲ ଦିବାଲୋକ ସମୟ"}, {"Japan", "ଜାପାନ ଦିବାଲୋକ ସମୟ"}, {"Kamchatka", "ପେଟ୍ରୋପାଭଲୋଭ୍ସକ-କମଚଟସ୍କି ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Korea", "କୋରିୟ ଦିବାଲୋକ ସମୟ"}, {"Krasnoyarsk", "କ୍ରାସନୋୟାରସ୍କ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Lord_Howe", "ଲର୍ଡ ହୋୱେ ଦିବାଲୋକ ସମୟ"}, {"Macau", "ମକାଓ ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Magadan", "ମାଗାଡାନ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Mauritius", "ମୌରିସସ୍\u200c ଖରାଦିନ ସମୟ"}, {"Mexico_Pacific", "ମେକ୍ସିକାନ୍ ପାସିଫିକ୍ ଦିବାଲୋକ ସମୟ"}, {"Mongolia", "ଉଲାନ୍\u200cବାଟର୍\u200c ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Moscow", "ମସ୍କୋ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"New_Caledonia", "ନ୍ୟୁ କାଲେଡୋନିଆ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"New_Zealand", "ନ୍ୟୁଜିଲାଣ୍ଡ ଦିବାଲୋକ ସମୟ"}, {"Newfoundland", "ନ୍ୟୁଫାଉଣ୍ଡଲ୍ୟାଣ୍ଡ୍ ଦିବାଲୋକ ସମୟ"}, {"Norfolk", "ନରଫୋକ୍\u200c ଦ୍ୱୀପ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Noronha", "ଫର୍ଣ୍ଣାଣ୍ଡୋ ଡି ନୋରୋନ୍ନା ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Novosibirsk", "ନୋଭୋସିବିରସ୍କ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Omsk", "ଓମସ୍କ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Pakistan", "ପାକିସ୍ତାନ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Paraguay", "ପାରାଗୁଏ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Peru", "ପେରୁ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Philippines", "ଫିଲିପାଇନ୍\u200c ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Pierre_Miquelon", "ସେଣ୍ଟ. ପିଏରେ ଏବଂ ମିକ୍ୟୁଲୋନ୍ ଦିବାଲୋକ ସମୟ"}, {"Qyzylorda", "କାଜିଲୋର୍ଡା ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Sakhalin", "ସଖାଲିନ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Samara", "ସମାରା ଗ୍ରୀଷ୍ମ ସମୟ"}, {"Samoa", "ସାମୋଆ ଦିବାଲୋକ ସମୟ"}, {"Taipei", "ତାଇପେଇ ଦିବାଲୋକ ସମୟ"}, {"Tonga", "ଟୋଙ୍ଗା ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Turkmenistan", "ତୁର୍କମେନିସ୍ତାନ ଖରାଦିନ ସମୟ"}, {"Uruguay", "ଉରୁଗୁଏ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Uzbekistan", "ଉଜବେକିସ୍ତାନ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Vanuatu", "ଭାନୁଆଟୁ ଗ୍ରୀଷ୍ମକାଳୀନ ସମୟ"}, {"Vladivostok", "ଭ୍ଲାଡିଭୋଷ୍ଟୋକ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Volgograd", "ଭୋଲଗୋଗ୍ରାଡ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Yakutsk", "ୟାକୁଟସ୍କ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}, {"Yekaterinburg", "ୟେକାଟେରିନବର୍ଗ୍ ଗ୍ରୀଷ୍ମକାଳ ସମୟ"}},
	},
	{
		{{"Europe_Central", "Астӕуккаг Европӕйаг стандартон рӕстӕг"}, {"Europe_Eastern", "Скӕсӕн Европӕйаг стандартон рӕстӕг"}, {"Europe_Western", "Ныгъуылӕн Европӕйаг стандартон рӕстӕг"}, {"GMT", "Гринвичы рӕстӕмбис рӕстӕг"}, {"Georgia", "Гуырдзыстоны стандартон рӕстӕг"}, {"Moscow", "Мӕскуыйы стандартон рӕстӕг"}},
		{{"Europe_Central", "Астӕуккаг Европӕйаг сӕрдыгон рӕстӕг"}, {"Europe_Eastern", "Скӕсӕн Европӕйаг сӕрдыгон рӕстӕг"}, {"Europe_Western", "Ныгъуылӕн Европӕйаг сӕрдыгон рӕстӕг"}, {"Georgia", "Гуырдзыстоны сӕрдыгон рӕстӕг"}, {"Moscow", "Мӕскуыйы сӕрдыгон рӕстӕг"}},
	},
	{
		{{"Afghanistan", "ਅਫ਼ਗਾਨਿਸਤਾਨ ਵੇਲਾ"}, {"Africa_Central", "ਕੇਂਦਰੀ ਅਫ਼ਰੀਕਾ ਵੇਲਾ"}, {"Africa_Eastern", "ਪੂਰਬੀ ਅਫ਼ਰੀਕਾ ਵੇਲਾ"}, {"Africa_Southern", "ਦੱਖਣੀ ਅਫ਼ਰੀਕਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Africa_Western", "ਪੱਛਮੀ ਅਫਰੀਕਾ ਵੇਲਾ"}, {"Alaska", "ਅਲਾਸਕਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Almaty", "ਅਲਮਾਟੀ ਮਿਆਰੀ ਸਮਾਂ"}, {"Amazon", "ਅਮੇਜ਼ਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"America_Central", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਕੇਂਦਰੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"America_Eastern", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਪੂਰਬੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"America_Mountain", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਮਾਉਂਟੇਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"America_Pacific", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਪੈਸਿਫਿਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Apia", "ਐਪੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Aqtau", "ਅਕਤਾਉ ਮਿਆਰੀ ਸਮਾਂ"}, {"Aqtobe", "ਅਕਤੋਬ ਮਿਆਰੀ ਸਮਾਂ"}, {"Arabian", "ਅਰਬੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Argentina", "ਅਰਜਨਟੀਨਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Argentina_Western", "ਪੱਛਮੀ ਅਰਜਨਟੀਨਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Armenia", "ਅਰਮੀਨੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Atlantic", "ਅਟਲਾਂਟਿਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Australia_Central", "ਆਸਟ੍ਰੇਲੀਆਈ ਕੇਂਦਰੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Australia_CentralWestern", "ਆਸਟ੍ਰੇਲੀਆਈ ਕੇਂਦਰੀ ਪੱਛਮੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Australia_Eastern", "ਆਸਟ੍ਰੇਲੀਆਈ ਪੂਰਬੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Australia_Western", "ਆਸਟ੍ਰੇਲੀਆਈ ਪੱਛਮੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Azerbaijan", "ਅਜ਼ਰਬਾਈਜਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Azores", "ਅਜੋਰੇਸ ਮਿਆਰੀ ਵੇਲਾ"}, {"Bangladesh", "ਬੰਗਲਾਦੇਸ਼ ਮਿਆਰੀ ਵੇਲਾ"}, {"Bhutan", "ਭੂਟਾਨ ਵੇਲਾ"}, {"Bolivia", "ਬੋਲੀਵੀਆ ਵੇਲਾ"}, {"Brasilia", "ਬ੍ਰਾਜ਼ੀਲੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Brunei", "ਬਰੂਨੇਈ ਦਾਰੂਸਲਾਮ ਵੇਲਾ"}, {"Cape_Verde", "ਕੇਪ ਵਰਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Casey", "ਕੇਸੀ ਸਮਾਂ"}, {"Chamorro", "ਚਾਮੋਰੋ ਮਿਆਰੀ ਵੇਲਾ"}, {"Chatham", "ਚੈਥਮ ਮਿਆਰੀ ਵੇਲਾ"}, {"Chile", "ਚਿਲੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"China", "ਚੀਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Christmas", "ਕ੍ਰਿਸਮਸ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Cocos", "ਕੋਕਸ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Colombia", "ਕੋਲੰਬੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Cook", "ਕੁੱਕ ਆਈਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Cuba", "ਕਿਊਬਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Davis", "ਡੇਵਿਸ ਵੇਲਾ"}, {"DumontDUrville", "ਡਿਉਮੋਂਟ ਡਿਉਰਵਿਲੇ ਵੇਲਾ"}, {"East_Timor", "ਪੂਰਬੀ ਤਿਮੂਰ ਵੇਲਾ"}, {"Easter", "ਈਸਟਰ ਆਈਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Ecuador", "ਇਕਵੇਡੋਰ ਵੇਲਾ"}, {"Europe_Central", "ਮੱਧ ਯੂਰਪੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Europe_Eastern", "ਪੂਰਬੀ ਯੂਰਪੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Europe_Further_Eastern", "ਹੋਰ-ਪੂਰਬੀ ਯੂਰਪੀ ਵੇਲਾ"}, {"Europe_Western", "ਪੱਛਮੀ ਯੂਰਪੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Falkland", "ਫ਼ਾਕਲੈਂਡ ਆਈਲੈਂਡਸ ਮਿਆਰੀ ਵੇਲਾ"}, {"Fiji", "ਫ਼ਿਜ਼ੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"French_Guiana", "ਫ੍ਰੈਂਚ ਗੁਏਨਾ ਵੇਲਾ"}, {"French_Southern", "ਫ੍ਰੈਂਚ ਦੱਖਣੀ ਅਤੇ ਐਂਟਾਰਟਿਕ ਵੇਲਾ"}, {"GMT", "ਗ੍ਰੀਨਵਿਚ ਮੀਨ ਵੇਲਾ"}, {"Galapagos", "ਗਲਾਪਾਗੋਸ ਵੇਲਾ"}, {"Gambier", "ਗੈਂਬੀਅਰ ਵੇਲਾ"}, {"Georgia", "ਜਾਰਜੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Gilbert_Islands", "ਗਿਲਬਰਟ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Greenland_Eastern", "ਪੂਰਬੀ ਗ੍ਰੀਨਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Greenland_Western", "ਪੱਛਮੀ ਗ੍ਰੀਨਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Guam", "ਗੁਆਮ ਸਮਾਂ"}, {"Gulf", "ਖਾੜੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Guyana", "ਗੁਯਾਨਾ ਵੇਲਾ"}, {"Hawaii", "ਹਵਾਈ-ਅਲੇਯੂਸ਼ਿਅਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Hawaii_Aleutian", "ਹਵਾਈ-ਅਲੇਯੂਸ਼ਿਅਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Hong_Kong", "ਹਾਂਗ ਕਾਂਗ ਮਿਆਰੀ ਵੇਲਾ"}, {"Hovd", "ਹੋਵਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"India", "ਭਾਰਤੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Indian_Ocean", "ਹਿੰਦ ਮਹਾਂਸਾਗਰ ਵੇਲਾ"}, {"Indochina", "ਇੰਡੋਚਾਈਨਾ ਵੇਲਾ"}, {"Indonesia_Central", "ਮੱਧ ਇੰਡੋਨੇਸ਼ੀਆਈ ਵੇਲਾ"}, {"Indonesia_Eastern", "ਪੂਰਬੀ ਇੰਡੋਨੇਸ਼ੀਆ ਵੇਲਾ"}, {"Indonesia_Western", "ਪੱਛਮੀ ਇੰਡੋਨੇਸ਼ੀਆ ਵੇਲਾ"}, {"Iran", "ਈਰਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Irkutsk", "ਇਰਕੁਤਸਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Israel", "ਇਜ਼ਰਾਈਲ ਮਿਆਰੀ ਵੇਲਾ"}, {"Japan", "ਜਪਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Kazakhstan", "ਕਜ਼ਾਖ਼ਸਤਾਨ ਵੇਲਾ"}, {"Kazakhstan_Eastern", "ਪੂਰਬੀ ਕਜ਼ਾਖ਼ਸਤਾਨ ਵੇਲਾ"}, {"Kazakhstan_Western", "ਪੱਛਮੀ ਕਜ਼ਾਖ਼ਸਤਾਨ ਵੇਲਾ"}, {"Korea", "ਕੋਰੀਆਈ ਮਿਆਰੀ ਵੇਲਾ"}, {"Kosrae", "ਕੋਸਰੇ ਵੇਲਾ"}, {"Krasnoyarsk", "ਕ੍ਰਾਸਨੋਯਾਰਸਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Kyrgystan", "ਕਿਰਗਿਸਤਾਨ ਵੇਲਾ"}, {"Lanka", "ਲੰਕਾ ਸਮਾਂ"}, {"Line_Islands", "ਲਾਈਨ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Lord_Howe", "ਲੌਰਡ ਹੋਵੇ ਮਿਆਰੀ ਵੇਲਾ"}, {"Macau", "ਮਕਾਉ ਮਿਆਰੀ ਸਮਾਂ"}, {"Magadan", "ਮੈਗੇਡਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Malaysia", "ਮਲੇਸ਼ੀਆ ਵੇਲਾ"}, {"Maldives", "ਮਾਲਦੀਵ ਵੇਲਾ"}, {"Marquesas", "ਮਾਰਕਿਸਾਸ ਵੇਲਾ"}, {"Marshall_Islands", "ਮਾਰਸ਼ਲ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Mauritius", "ਮੌਰਿਸ਼ਸ ਮਿਆਰੀ ਵੇਲਾ"}, {"Mawson", "ਮੌਸਨ ਵੇਲਾ"}, {"Mexico_Pacific", "ਮੈਕਸੀਕਨ ਪੈਸਿਫਿਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Mongolia", "ਉਲਨ ਬਟੋਰ ਮਿਆਰੀ ਵੇਲਾ"}, {"Moscow", "ਮਾਸਕੋ ਮਿਆਰੀ ਵੇਲਾ"}, {"Myanmar", "ਮਿਆਂਮਾਰ ਵੇਲਾ"}, {"Nauru", "ਨਾਉਰੂ ਵੇਲਾ"}, {"Nepal", "ਨੇਪਾਲ ਵੇਲਾ"}, {"New_Caledonia", "ਨਿਊ ਕੈਲੇਡੋਨੀਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"New_Zealand", "ਨਿਊਜ਼ੀਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Newfoundland", "ਨਿਊਫਾਉਂਡਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Niue", "ਨੀਊ ਵੇਲਾ"}, {"Norfolk", "ਨੋਰਫੌਕ ਆਈਲੈਂਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Noronha", "ਫਰਨਾਂਡੋ ਡੇ ਨੋਰੋਨਹਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"North_Mariana", "ਉੱਤਰੀ ਮਰਿਆਨਾ ਆਈਲੈਂਡ ਸਮਾਂ"}, {"Novosibirsk", "ਨੌਵੋਸਿਬੀਰਸਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Omsk", "ਓਮਸਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Pakistan", "ਪਾਕਿਸਤਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Palau", "ਪਲਾਉ ਵੇਲਾ"}, {"Papua_New_Guinea", "ਪਾਪੂਆ ਨਿਊ ਗਿਨੀ ਵੇਲਾ"}, {"Paraguay", "ਪੈਰਾਗਵੇ ਮਿਆਰੀ ਵੇਲਾ"}, {"Peru", "ਪੇਰੂ ਮਿਆਰੀ ਵੇਲਾ"}, {"Philippines", "ਫਿਲਿਪੀਨੀ ਮਿਆਰੀ ਵੇਲਾ"}, {"Phoenix_Islands", "ਫਿਨਿਕਸ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Pierre_Miquelon", "ਸੈਂਟ ਪੀਅਰੇ ਅਤੇ ਮਿਕੇਲਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Pitcairn", "ਪਿਟਕੈਰਨ ਵੇਲਾ"}, {"Ponape", "ਪੋਨਾਪੇ ਵੇਲਾ"}, {"Pyongyang", "ਪਯੋਂਗਯਾਂਗ ਵੇਲਾ"}, {"Qyzylorda", "ਕਿਜ਼ਲੋਰਡਾ ਮਿਆਰੀ ਸਮਾਂ"}, {"Reunion", "ਰਿਯੂਨੀਅਨ ਵੇਲਾ"}, {"Rothera", "ਰੋਥੇਰਾ ਵੇਲਾ"}, {"Sakhalin", "ਸਖਲੀਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Samoa", "ਸਾਮੋਆ ਮਿਆਰੀ ਵੇਲਾ"}, {"Seychelles", "ਸੇਸ਼ਲਸ ਵੇਲਾ"}, {"Singapore", "ਸਿੰਗਾਪੁਰ ਮਿਆਰੀ ਵੇਲਾ"}, {"Solomon", "ਸੋਲੋਮਨ ਆਈਲੈਂਡਸ ਵੇਲਾ"}, {"South_Georgia", "ਦੱਖਣੀ ਜਾਰਜੀਆ ਵੇਲਾ"}, {"Suriname", "ਸੂਰੀਨਾਮ ਵੇਲਾ"}, {"Syowa", "ਸਿਓਵਾ ਵੇਲਾ"}, {"Tahiti", "ਤਾਹੀਤੀ ਵੇਲਾ"}, {"Taipei", "ਤੈਪਈ ਮਿਆਰੀ ਵੇਲਾ"}, {"Tajikistan", "ਤਾਜਿਕਿਸਤਾਨ ਵੇਲਾ"}, {"Tokelau", "ਟੋਕੇਲਾਉ ਵੇਲਾ"}, {"Tonga", "ਟੋਂਗਾ ਮਿਆਰੀ ਵੇਲਾ"}, {"Truk", "ਚੂਕ ਵੇਲਾ"}, {"Turkmenistan", "ਤੁਰਕਮੇਨਿਸਤਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Tuvalu", "ਟੁਵਾਲੂ ਵੇਲਾ"}, {"Uruguay", "ਉਰੂਗਵੇ ਮਿਆਰੀ ਵੇਲਾ"}, {"Uzbekistan", "ਉਜ਼ਬੇਕਿਸਤਾਨ ਮਿਆਰੀ ਵੇਲਾ"}, {"Vanuatu", "ਵਾਨੂਆਟੂ ਮਿਆਰੀ ਵੇਲਾ"}, {"Venezuela", "ਵੈਨੇਜ਼ੂਏਲਾ ਵੇਲਾ"}, {"Vladivostok", "ਵਲਾਦੀਵੋਸਤਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Volgograd", "ਵੋਲਗੋਗ੍ਰੇਡ ਮਿਆਰੀ ਵੇਲਾ"}, {"Vostok", "ਵੋਸਟੋਕ ਵੇਲਾ"}, {"Wake", "ਵੇਕ ਆਈਲੈਂਡ ਵੇਲਾ"}, {"Wallis", "ਵਾਲਿਸ ਅਤੇ ਫੁਟੂਨਾ ਵੇਲਾ"}, {"Yakutsk", "ਯਕੁਤਸਕ ਮਿਆਰੀ ਵੇਲਾ"}, {"Yekaterinburg", "ਯਕੇਤਰਿਨਬਰਗ ਮਿਆਰੀ ਵੇਲਾ"}, {"Yukon", "ਯੂਕੋਨ ਸਮਾਂ"}},
		{{"Alaska", "ਅਲਾਸਕਾ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Almaty", "ਅਲਮਾਟੀ ਗਰਮੀ-ਰੁੱਤ ਸਮਾਂ"}, {"Amazon", "ਅਮੇਜ਼ਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"America_Central", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਕੇਂਦਰੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"America_Eastern", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਪੂਰਬੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"America_Mountain", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਮਾਉਂਟੇਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"America_Pacific", "ਉੱਤਰੀ ਅਮਰੀਕੀ ਪੈਸਿਫਿਕ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Apia", "ਐਪੀਆ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Aqtau", "ਅਕਤਾਉ ਗਰਮੀ-ਰੁੱਤ ਸਮਾਂ"}, {"Aqtobe", "ਅਕਤੋਬ ਗਰਮੀ-ਰੁੱਤ ਸਮਾਂ"}, {"Arabian", "ਅਰਬੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Argentina", "ਅਰਜਨਟੀਨਾ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Argentina_Western", "ਪੱਛਮੀ ਅਰਜਨਟੀਨਾ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Armenia", "ਅਰਮੀਨੀਆ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Atlantic", "ਅਟਲਾਂਟਿਕ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Australia_Central", "ਆਸਟ੍ਰੇਲੀਆਈ ਕੇਂਦਰੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Australia_CentralWestern", "ਆਸਟ੍ਰੇਲੀਆਈ ਕੇਂਦਰੀ ਪੱਛਮੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Australia_Eastern", "ਆਸਟ੍ਰੇਲੀਆਈ ਪੂਰਬੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Australia_Western", "ਆਸਟ੍ਰੇਲੀਆਈ ਪੱਛਮੀ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Azerbaijan", "ਅਜ਼ਰਬਾਈਜਾਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Azores", "ਅਜੋਰੇਸ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Bangladesh", "ਬੰਗਲਾਦੇਸ਼ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Brasilia", "ਬ੍ਰਾਜ਼ੀਲੀਆ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Cape_Verde", "ਕੇਪ ਵਰਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Chatham", "ਚੈਥਮ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Chile", "ਚਿਲੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"China", "ਚੀਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Colombia", "ਕੋਲੰਬੀਆ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Cook", "ਕੁੱਕ ਆਈਲੈਂਡ ਅੱਧ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Cuba", "ਕਿਊਬਾ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Easter", "ਈਸਟਰ ਆਈਲੈਂਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Europe_Central", "ਮੱਧ ਯੂਰਪੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Europe_Eastern", "ਪੂਰਬੀ ਯੂਰਪੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Europe_Western", "ਪੱਛਮੀ ਯੂਰਪੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Falkland", "ਫ਼ਾਕਲੈਂਡ ਆਈਲੈਂਡਸ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Fiji", "ਫ਼ਿਜ਼ੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Georgia", "ਜਾਰਜੀਆ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Greenland_Eastern", "ਪੂਰਬੀ ਗ੍ਰੀਨਲੈਂਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Greenland_Western", "ਪੱਛਮੀ ਗ੍ਰੀਨਲੈਂਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Hawaii_Aleutian", "ਹਵਾਈ-ਅਲੇਯੂਸ਼ਿਅਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Hong_Kong", "ਹਾਂਗ ਕਾਂਗ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Hovd", "ਹੋਵਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Iran", "ਈਰਾਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Irkutsk", "ਇਰਕੁਤਸਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Israel", "ਇਜ਼ਰਾਈਲ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Japan", "ਜਪਾਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Korea", "ਕੋਰੀਆਈ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Krasnoyarsk", "ਕ੍ਰਾਸਨੋਯਾਰਸਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Lord_Howe", "ਲੌਰਡ ਹੋਵੇ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Macau", "ਮਕਾਉ ਗਰਮੀ-ਰੁੱਤ ਸਮਾਂ"}, {"Magadan", "ਮੈਗੇਡਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Mauritius", "ਮੌਰਿਸ਼ਸ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Mexico_Pacific", "ਮੈਕਸੀਕਨ ਪੈਸਿਫਿਕ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Mongolia", "ਉਲਨ ਬਟੋਰ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Moscow", "ਮਾਸਕੋ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"New_Caledonia", "ਨਿਊ ਕੈਲੇਡੋਨੀਆ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"New_Zealand", "ਨਿਊਜ਼ੀਲੈਂਡ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Newfoundland", "ਨਿਊਫਾਉਂਡਲੈਂਡ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Norfolk", "ਨੋਰਫੌਕ ਆਈਲੈਂਡ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Noronha", "ਫਰਨਾਂਡੋ ਡੇ ਨੋਰੋਨਹਾ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Novosibirsk", "ਨੌਵੋਸਿਬੀਰਸਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Omsk", "ਓਮਸਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Pakistan", "ਪਾਕਿਸਤਾਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Paraguay", "ਪੈਰਾਗਵੇ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Peru", "ਪੇਰੂ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Philippines", "ਫਿਲਿਪੀਨੀ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Pierre_Miquelon", "ਸੈਂਟ ਪੀਅਰੇ ਅਤੇ ਮਿਕੇਲਨ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Qyzylorda", "ਕਿਜ਼ਲੋਰਡਾ ਗਰਮੀ-ਰੁੱਤ ਸਮਾਂ"}, {"Sakhalin", "ਸਖਲੀਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Samoa", "ਸਾਮੋਆ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Taipei", "ਤੈਪਈ ਪ੍ਰਕਾਸ਼ ਵੇਲਾ"}, {"Tonga", "ਟੋਂਗਾ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Turkmenistan", "ਤੁਰਕਮੇਨਿਸਤਾਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Uruguay", "ਉਰੂਗਵੇ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Uzbekistan", "ਉਜ਼ਬੇਕਿਸਤਾਨ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Vanuatu", "ਵਾਨੂਆਟੂ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Vladivostok", "ਵਲਾਦੀਵੋਸਤਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Volgograd", "ਵੋਲਗੋਗ੍ਰੇਡ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Yakutsk", "ਯਕੁਤਸਕ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}, {"Yekaterinburg", "ਯਕੇਤਰਿਨਬਰਗ ਗਰਮੀਆਂ ਦਾ ਵੇਲਾ"}},
	},
	{
		{{"Africa_Central", "Ora di Áfrika Sentral"}, {"Africa_Eastern", "Ora di Ost Áfrika"}, {"Azores", "Ora Normal di Azóres"}, {"Europe_Central", "Ora Normal di Europa Sentral"}, {"Europe_Eastern", "Ora Normal di Ost Europa"}, {"Europe_Western", "Ora Normal di Wèst Europa"}, {"GMT", "Greenwich Mean Time"}},
		{{"Azores", "Ora di Azóres Temporada di Zomer"}, {"Europe_Central", "Ora di Europa Sentral Temporada di Zomer"}, {"Europe_Eastern", "Ora di Ost Europa Temporada di Zomer"}, {"Europe_Western", "Ora di Wèst Europa Temporada di Zomer"}},
	},
	{
		{{"Afghanistan", "Afgánístan Taim"}, {"Africa_Central", "Mídúl Áfríká Taim"}, {"Africa_Eastern", "Íst Áfríká Taim"}, {"Africa_Southern", "Saút Áfríká Fíksd Taim"}, {"Africa_Western", "Wẹ́st Áfríká Taim"}, {"Alaska", "Aláská Fíksd Taim"}, {"Amazon", "Ámázọn Fíksd Taim"}, {"America_Central", "Nọ́t Amẹ́ríká Mídúl Ériá Fíksd Taim"}, {"America_Eastern", "Nọ́t Amẹ́ríká Ístán Ériá Fíksd Taim"}, {"America_Mountain", "Nọ́t Amẹ́ríká Maúntin Ériá Fíksd Taim"}, {"America_Pacific", "Nọ́t Amẹ́ríká Pasífík Ériá Fíksd Taim"}, {"Apia", "Ápia Fíksd Taim"}, {"Arabian", "Arébiá Fíksd Taim"}, {"Argentina", "Ajẹntína Fíksd Taim"}, {"Argentina_Western", "Wẹ́stán Ajẹntína Fíksd Taim"}, {"Armenia", "Armẹ́nia Fíksd Taim"}, {"Atlantic", "Atlántík Fíksd Taim"}, {"Australia_Central", "Ọstrélia Mídúl Fíksd Taim"}, {"Australia_CentralWestern", "Ọstrélia Mídúl Wẹ́stán Fíksd Taim"}, {"Australia_Eastern", "Ọstrélia Ístán Fíksd Taim"}, {"Australia_Western", "Ọstrélia Wẹ́stán Fíksd Taim"}, {"Azerbaijan", "Azẹrbaijan Fíksd Taim"}, {"Azores", "Azọz Fíksd Taim"}, {"Bangladesh", "Bangladẹsh Fíksd Taim"}, {"Bhutan", "Butan Taim"}, {"Bolivia", "Bolívia Fíksd Taim"}, {"Brasilia", "Brasília Fíksd Taim"}, {"Brunei", "Brunẹi Darúsalam Taim"}, {"Cape_Verde", "Kep Vẹ́d Fíksd Taim"}, {"Chamorro", "Chamóro Fíksd Taim"}, {"Chatham", "Chátam Fíksd Taim"}, {"Chile", "Chílẹ Fíksd Taim"}, {"China", "Chaína Fíksd Taim"}, {"Christmas", "Krísmás Aíland Taim"}, {"Cocos", "Kókós Aílands Taim"}, {"Colombia", "Kolómbia Fíksd Taim"}, {"Cook", "Kúk Aílands Fíksd Taim"}, {"Cuba", "Kúba Fíksd Taim"}, {"Davis", "Dévis Taim"}, {"DumontDUrville", "Diúmọ́n-d’Uvil Taim"}, {"East_Timor", "Íst Tímọ Taim"}, {"Easter", "Ísta Fíksd Taim"}, {"Ecuador", "Ẹ́kwuádọ Taim"}, {"Europe_Central", "Mídúl Yúrop Fíksd Taim"}, {"Europe_Eastern", "Ístán Yúrop Fíksd Taim"}, {"Europe_Further_Eastern", "Faá-Ístán Yúrop Taim"}, {"Europe_Western", "Wẹ́stán Yúrop Fíksd Taim"}, {"Falkland", "Fọ́lkland Fíksd Taim"}, {"Fiji", "Fíji Fíksd Taim"}, {"French_Guiana", "Frẹ́nch Giána Taim"}, {"French_Southern", "Frẹ́nch Saútan an Antátík Taim"}, {"GMT", "Grínwích Mín Taim"}, {"Galapagos", "Galápágọs Taim"}, {"Gambier", "Gámbiẹr Taim"}, {"Georgia", "Jọ́jia Fíksd Taim"}, {"Gilbert_Islands", "Gílbat Aílands Taim"}, {"Greenland_Eastern", "Íist Grínlánd Fíksd Taim"}, {"Greenland_Western", "Wẹ́st Grínlánd Fíksd Taim"}, {"Gulf", "Gọ́lf Fíksd Taim"}, {"Guyana", "Gayána Taim"}, {"Hawaii", "Hawaií-Elúshián Fíksd Taim"}, {"Hawaii_Aleutian", "Hawaií-Elúshián Fíksd Taim"}, {"Hong_Kong", "Họng Kọng Fíksd Taim"}, {"Hovd", "Hovd Fíksd Taim"}, {"India", "Índia Fíksd Taim"}, {"Indian_Ocean", "Índián Óshẹ́n Taim"}, {"Indochina", "Indochaína Taim"}, {"Indonesia_Central", "Mídúl Indonẹ́shia Taim"}, {"Indonesia_Eastern", "Ístán Indonẹ́shia Taim"}, {"Indonesia_Western", "Wẹ́stán Indonẹ́shia Taim"}, {"Iran", "Iran Fíksd Taim"}, {"Irkutsk", "Irkútsk Fíksd Taim"}, {"Israel", "Ízrẹl Fíksd Taim"}, {"Japan", "Japan Fíksd Taim"}, {"Kazakhstan", "Kazékstan Taim"}, {"Kazakhstan_Eastern", "Íst Kazékstan Taim"}, {"Kazakhstan_Western", "Wẹ́st Kazékstan Taim"}, {"Korea", "Koria Fíksd Taim"}, {"Kosrae", "Kọ́sraẹ Taim"}, {"Krasnoyarsk", "Krasnoyask Fíksd Taim"}, {"Kyrgystan", "Kẹgistan Taim"}, {"Line_Islands", "Laín Aílands Taim"}, {"Lord_Howe", "Lọd Haú Fíksd Taim"}, {"Magadan", "Mágádan Fíksd Taim"}, {"Malaysia", "Maléshia Taim"}, {"Maldives", "Mọ́divs Taim"}, {"Marquesas", "Makwẹ́sas Taim"}, {"Marshall_Islands", "Máshal Aílands Taim"}, {"Mauritius", "Mọríshọs Fíksd Taim"}, {"Mawson", "Mọ́sọn Taim"}, {"Mexico_Pacific", "Mẹ́ksíkó Pasífík Fíksd Taim"}, {"Mongolia", "Mọngólia Fíksd Taim"}, {"Moscow", "Mọ́sko Fíksd Taim"}, {"Myanmar", "Miánma Taim"}, {"Nauru", "Naúru Taim"}, {"Nepal", "Nẹpọl Taim"}, {"New_Caledonia", "Niú Kalẹdónia Fíksd Taim"}, {"New_Zealand", "Niú Ziland Fíksd Taim"}, {"Newfoundland", "Niúfaúndlánd Fíksd Taim"}, {"Niue", "Niúẹ Taim"}, {"Norfolk", "Nọ́rfọ́lk Aíland Fíksd Taim"}, {"Noronha", "Fẹrnándó di Nọrónia Fíksd Taim"}, {"Novosibirsk", "Novosibisk Fíksd Taim"}, {"Omsk", "Ọmsk Fíksd Taim"}, {"Pakistan", "Pákístan Fíksd Taim"}, {"Palau", "Paláu Taim"}, {"Papua_New_Guinea", "Pápuá Niú Gíni Taim"}, {"Paraguay", "Párágwue Fíksd Taim"}, {"Peru", "Pẹru Fíksd Taim"}, {"Philippines", "Fílípin Fíksd Taim"}, {"Phoenix_Islands", "Fíniks Aílands Taim"}, {"Pierre_Miquelon", "Sent Piẹr an Míkẹlọn Fíksd Taim"}, {"Pitcairn", "Pítkan Taim"}, {"Ponape", "Pónápẹ Taim"}, {"Pyongyang", "Piọngyang Taim"}, {"Reunion", "Riyúniọn Taim"}, {"Rothera", "Rotẹ́ra Taim"}, {"Sakhalin", "Sákhalin Fíksd Taim"}, {"Samoa", "Sámoá Fíksd Taim"}, {"Seychelles", "Sẹ́chẹls Taim"}, {"Singapore", "Singapọ Taim"}, {"Solomon", "Sólómọ́n Aílands Taim"}, {"South_Georgia", "Saút Jọ́jia Taim"}, {"Suriname", "Súrínam Taim"}, {"Syowa", "Siówa Taim"}, {"Tahiti", "Tahíti Taim"}, {"Taipei", "Taipẹi Fíksd Taim"}, {"Tajikistan", "Tajíkistan Taim"}, {"Tokelau", "Tokẹláu Taim"}, {"Tonga", "Tọ́nga Fíksd Taim"}, {"Truk", "Chuk Taim"}, {"Turkmenistan", "Tọkmẹnistan Fíksd Taim"}, {"Tuvalu", "Tuválu Taim"}, {"Uruguay", "Yúrugwue Fíksd Taim"}, {"Uzbekistan", "Uzbẹkistan Fíksd Taim"}, {"Vanuatu", "Vanuátu Fíksd Taim"}, {"Venezuela", "Vẹnẹzuẹ́la Taim"}, {"Vladivostok", "Vladivọstọk Fíksd Taim"}, {"Volgograd", "Volvógrad Fíksd Taim"}, {"Vostok", "Vọ́stọk Taim"}, {"Wake", "Wék Aíland Taim"}, {"Wallis", "Wális an Fútúna Taim"}, {"Yakutsk", "Yékútsk Fíksd Taim"}, {"Yekaterinburg", "Yẹketẹrínbug Fíksd Taim"}, {"Yukon", "Yukón Taim"}},
		{{"Alaska", "Aláská Délaít Taim"}, {"Amazon", "Ámázọn Họ́t Sízín Taim"}, {"America_Central", "Nọ́t Amẹ́ríká Mídúl Ériá Délaít Taim"}, {"America_Eastern", "Nọ́t Amẹ́ríká Ístán Ériá Délaít Taim"}, {"America_Mountain", "Nọ́t Amẹ́ríká Maúntin Ériá Délaít Taim"}, {"America_Pacific", "Nọ́t Amẹ́ríká Pasífík Ériá Délaít Taim"}, {"Apia", "Ápia Délaít Taim"}, {"Arabian", "Arébiá Délaít Taim"}, {"Argentina", "Ajẹntína Họ́t Sízín Taim"}, {"Argentina_Western", "Wẹ́stán Ajẹntína Họ́t Sízín Taim"}, {"Armenia", "Armẹ́nia Họ́t Sízin Taim"}, {"Atlantic", "Atlántík Délaít Taim"}, {"Australia_Central", "Ọstrélia Mídúl Délaít Taim"}, {"Australia_CentralWestern", "Ọstrélia Mídúl Wẹ́stán Délaít Taim"}, {"Australia_Eastern", "Ọstrélia Ístán Délaít Taim"}, {"Australia_Western", "Ọstrélia Wẹ́stán Délaít Taim"}, {"Azerbaijan", "Azẹrbaijan Họ́t Sízin Taim"}, {"Azores", "Azọz Họ́t Sízin Taim"}, {"Bangladesh", "Bangladẹsh Délaít Taim"}, {"Brasilia", "Brasília Họ́t Sízín Taim"}, {"Cape_Verde", "Kep Vẹ́d Họ́t Sízin Taim"}, {"Chatham", "Chátam Délaít Taim"}, {"Chile", "Chílẹ Họ́t Sízín Taim"}, {"China", "Chaína Délaít Taim"}, {"Colombia", "Kolómbia Họ́t Sízín Taim"}, {"Cook", "Kúk Aílands Haf Họ́t Sízin Taim"}, {"Cuba", "Kúba Délaít Taim"}, {"Easter", "Ísta Họ́t Sízín Taim"}, {"Europe_Central", "Mídúl Yúrop Họ́t Sízin Taim"}, {"Europe_Eastern", "Ístán Yúrop Họ́t Sízin Taim"}, {"Europe_Western", "Wẹ́stán Yúrop Họ́t Sízin Taim"}, {"Falkland", "Fọ́lkland Họ́t Sízín Taim"}, {"Fiji", "Fíji Họ́t Sízín Taim"}, {"Georgia", "Jọ́jia Họ́t Sízin Taim"}, {"Greenland_Eastern", "Íist Grínlánd Họ́t Sízin Taim"}, {"Greenland_Western", "Wẹ́st Grínlánd Họ́t Sízin Taim"}, {"Hawaii_Aleutian", "Hawaií-Elúshián Délaít Taim"}, {"Hong_Kong", "Họng Kọng Họ́t Sízin Taim"}, {"Hovd", "Hovd Họ́t Sízin Taim"}, {"Iran", "Iran Délaít Taim"}, {"Irkutsk", "Irkútsk Họ́t Sízin Taim"}, {"Israel", "Ízrẹl Délaít Taim"}, {"Japan", "Japan Délaít Taim"}, {"Korea", "Koria Délaít Taim"}, {"Krasnoyarsk", "Krasnoyask Họ́t Sízin Taim"}, {"Lord_Howe", "Lọd Haú Délaít Taim"}, {"Magadan", "Mágádan Họ́t Sízin Taim"}, {"Mauritius", "Mọríshọs Họ́t Sízin Taim"}, {"Mexico_Pacific", "Mẹ́ksíkó Pasífík Délaít Taim"}, {"Mongolia", "Mọngólia Họ́t Sízin Taim"}, {"Moscow", "Mọ́sko Họ́t Sízin Taim"}, {"New_Caledonia", "Niú Kalẹdónia Họ́t Sízin Taim"}, {"New_Zealand", "Niú Ziland Délaít Taim"}, {"Newfoundland", "Niúfaúndlánd Délaít Taim"}, {"Norfolk", "Nọ́rfọ́lk Aíland Délaít Taim"}, {"Noronha", "Fẹrnándó di Nọrónia Họ́t Sízín Taim"}, {"Novosibirsk", "Novosibisk Họ́t Sízin Taim"}, {"Omsk", "Ọmsk Họ́t Sízin Taim"}, {"Pakistan", "Pákístan Họ́t Sízin Taim"}, {"Paraguay", "Párágwue Họ́t Sízín Taim"}, {"Peru", "Pẹru Họ́t Sízín Taim"}, {"Philippines", "Fílípin Họt Sízin Taim"}, {"Pierre_Miquelon", "Sent Piẹr an Míkẹlọn Délaít Taim"}, {"Sakhalin", "Sákhalin Họ́t Sízin Taim"}, {"Samoa", "Sámoá Délaít Taim"}, {"Taipei", "Taipẹi Délaít Taim"}, {"Tonga", "Tọ́nga Họ́t Sízin Taim"}, {"Turkmenistan", "Tọkmẹnistan Họ́t Sízin Taim"}, {"Uruguay", "Yúrugwue Họ́t Sízín Taim"}, {"Uzbekistan", "Uzbẹkistan Họ́t Sízin Taim"}, {"Vanuatu", "Vanuátu Sízin Taim"}, {"Vladivostok", "Vladivostok Họ́t Sízin Taim"}, {"Volgograd", "Volvógrad Họ́t Sízin Taim"}, {"Yakutsk", "Yékútsk Họ́t Sízin Taim"}, {"Yekaterinburg", "Yẹketẹrínbug Họ́t Sízin Taim"}},
	},
	{
		{{"GMT", "Grinis Min Taem"}},
		{},
	},
	{
		{{"Afghanistan", "czas Afganistan"}, {"Africa_Central", "czas środkowoafrykański"}, {"Africa_Eastern", "czas wschodnioafrykański"}, {"Africa_Southern", "czas południowoafrykański"}, {"Africa_Western", "czas zachodnioafrykański"}, {"Alaska", "Alaska (czas standardowy)"}, {"Almaty", "czas ałmacki standardowy"}, {"Amazon", "czas amazoński standardowy"}, {"America_Central", "czas środkowoamerykański standardowy"}, {"America_Eastern", "czas wschodnioamerykański standardowy"}, {"America_Mountain", "czas górski standardowy"}, {"America_Pacific", "czas pacyficzny standardowy"}, {"Anadyr", "czas standardowy Anadyr"}, {"Apia", "Apia (czas standardowy)"}, {"Aqtau", "czas auktaucki standardowy"}, {"Aqtobe", "czas aktiubiński standardowy"}, {"Arabian", "Półwysep Arabski (czas standardowy)"}, {"Argentina", "Argentyna (czas standardowy)"}, {"Argentina_Western", "Argentyna Zachodnia (czas standardowy)"}, {"Armenia", "Armenia (czas standardowy)"}, {"Atlantic", "czas atlantycki standardowy"}, {"Australia_Central", "czas środkowoaustralijski standardowy"}, {"Australia_CentralWestern", "czas środkowo-zachodnioaustralijski standardowy"}, {"Australia_Eastern", "czas wschodnioaustralijski standardowy"}, {"Australia_Western", "czas zachodnioaustralijski standardowy"}, {"Azerbaijan", "Azerbejdżan (czas standardowy)"}, {"Azores", "Azory (czas standardowy)"}, {"Bangladesh", "Bangladesz (czas standardowy)"}, {"Bhutan", "czas Bhutan"}, {"Bolivia", "czas Boliwia"}, {"Brasilia", "Brasília (czas standardowy)"}, {"Brunei", "czas Brunei"}, {"Cape_Verde", "Wyspy Zielonego Przylądka (czas standardowy)"}, {"Chamorro", "czas Czamorro"}, {"Chatham", "Chatham (czas standardowy)"}, {"Chile", "Chile (czas standardowy)"}, {"China", "Chiny (czas standardowy)"}, {"Christmas", "czas Wyspa Bożego Narodzenia"}, {"Cocos", "czas Wyspy Kokosowe"}, {"Colombia", "Kolumbia (czas standardowy)"}, {"Cook", "Wyspy Cooka (czas standardowy)"}, {"Cuba", "Kuba (czas standardowy)"}, {"Davis", "czas Davis"}, {"DumontDUrville", "czas Dumont-d’Urville"}, {"East_Timor", "czas Timor Wschodni"}, {"Easter", "Wyspa Wielkanocna (czas standardowy)"}, {"Ecuador", "czas Ekwador"}, {"Europe_Central", "czas środkowoeuropejski standardowy"}, {"Europe_Eastern", "czas wschodnioeuropejski standardowy"}, {"Europe_Further_Eastern", "czas wschodnioeuropejski dalszy"}, {"Europe_Western", "czas zachodnioeuropejski standardowy"}, {"Falkland", "Falklandy (czas standardowy)"}, {"Fiji", "Fidżi (czas standardowy)"}, {"French_Guiana", "czas Gujana Francuska"}, {"French_Southern", "czas Francuskie Terytoria Południowe i Antarktyczne"}, {"GMT", "czas uniwersalny"}, {"Galapagos", "czas Galapagos"}, {"Gambier", "czas Wyspy Gambiera"}, {"Georgia", "Gruzja (czas standardowy)"}, {"Gilbert_Islands", "czas Wyspy Gilberta"}, {"Greenland_Eastern", "Grenlandia Wschodnia (czas standardowy)"}, {"Greenland_Western", "Grenlandia Zachodnia (czas standardowy)"}, {"Gulf", "czas Zatoka Perska"}, {"Guyana", "czas Gujana"}, {"Hawaii", "Hawaje-Aleuty (czas standardowy)"}, {"Hawaii_Aleutian", "Hawaje-Aleuty (czas standardowy)"}, {"Hong_Kong", "Hongkong (czas standardowy)"}, {"Hovd", "Kobdo (czas standardowy)"}, {"India", "czas indyjski standardowy"}, {"Indian_Ocean", "czas Ocean Indyjski"}, {"Indochina", "czas indochiński"}, {"Indonesia_Central", "czas Indonezja Środkowa"}, {"Indonesia_Eastern", "czas Indonezja Wschodnia"}, {"Indonesia_Western", "czas Indonezja Zachodnia"}, {"Iran", "Iran (czas standardowy)"}, {"Irkutsk", "Irkuck (czas standardowy)"}, {"Israel", "Izrael (czas standardowy)"}, {"Japan", "Japonia (czas standardowy)"}, {"Kamchatka", "czas standardowy Pietropawłowsk Kamczacki"}, {"Kazakhstan", "czas Kazachstan"}, {"Kazakhstan_Eastern", "czas Kazachstan Wschodni"}, {"Kazakhstan_Western", "czas Kazachstan Zachodni"}, {"Korea", "Korea (czas standardowy)"}, {"Kosrae", "czas Kosrae"}, {"Krasnoyarsk", "Krasnojarsk (czas standardowy)"}, {"Kyrgystan", "czas Kirgistan"}, {"Line_Islands", "czas Line Islands"}, {"Lord_Howe", "Lord Howe (czas standardowy)"}, {"Magadan", "Magadan (czas standardowy)"}, {"Malaysia", "czas Malezja"}, {"Maldives", "czas Malediwy"}, {"Marquesas", "czas Markizy"}, {"Marshall_Islands", "czas Wyspy Marshalla"}, {"Mauritius", "Mauritius (czas standardowy)"}, {"Mawson", "czas Mawson"}, {"Mexico_Pacific", "Meksyk (czas pacyficzny standardowy)"}, {"Mongolia", "Ułan Bator (czas standardowy)"}, {"Moscow", "Moskwa (czas standardowy)"}, {"Myanmar", "czas Mjanma"}, {"Nauru", "czas Nauru"}, {"Nepal", "czas Nepal"}, {"New_Caledonia", "Nowa Kaledonia (czas standardowy)"}, {"New_Zealand", "Nowa Zelandia (czas standardowy)"}, {"Newfoundland", "Nowa Fundlandia (czas standardowy)"}, {"Niue", "czas Niue"}, {"Norfolk", "Norfolk (czas standardowy)"}, {"Noronha", "Fernando de Noronha (czas standardowy)"}, {"Novosibirsk", "Nowosybirsk (czas standardowy)"}, {"Omsk", "Omsk (czas standardowy)"}, {"Pakistan", "Pakistan (czas standardowy)"}, {"Palau", "czas Palau"}, {"Papua_New_Guinea", "czas Papua-Nowa Gwinea"}, {"Paraguay", "Paragwaj (czas standardowy)"}, {"Peru", "Peru (czas standardowy)"}, {"Philippines", "Filipiny (czas standardowy)"}, {"Phoenix_Islands", "czas Feniks"}, {"Pierre_Miquelon", "Saint-Pierre i Miquelon (czas standardowy)"}, {"Pitcairn", "czas Pitcairn"}, {"Ponape", "czas Pohnpei"}, {"Pyongyang", "czas Pjongjang"}, {"Qyzylorda", "czas kyzyłordzki standardowy"}, {"Reunion", "czas Reunion"}, {"Rothera", "czas Rothera"}, {"Sakhalin", "Sachalin (czas standardowy)"}, {"Samara", "czas standardowy Samara"}, {"Samoa", "Samoa (czas standardowy)"}, {"Seychelles", "czas Seszele"}, {"Singapore", "czas Singapur"}, {"Solomon", "czas Wyspy Salomona"}, {"South_Georgia", "czas Georgia Południowa"}, {"Suriname", "czas Surinam"}, {"Syowa", "czas Syowa"}, {"Tahiti", "czas Tahiti"}, {"Taipei", "Tajpej (czas standardowy)"}, {"Tajikistan", "czas Tadżykistan"}, {"Tokelau", "czas Tokelau"}, {"Tonga", "Tonga (czas standardowy)"}, {"Truk", "czas Chuuk"}, {"Turkmenistan", "Turkmenistan (czas standardowy)"}, {"Tuvalu", "czas Tuvalu"}, {"Uruguay", "Urugwaj (czas standardowy)"}, {"Uzbekistan", "Uzbekistan (czas standardowy)"}, {"Vanuatu", "Vanuatu (czas standardowy)"}, {"Venezuela", "czas Wenezuela"}, {"Vladivostok", "Władywostok (czas standardowy)"}, {"Volgograd", "Wołgograd (czas standardowy)"}, {"Vostok", "czas Wostok"}, {"Wake", "czas Wake"}, {"Wallis", "czas Wallis i Futuna"}, {"Yakutsk", "Jakuck (czas standardowy)"}, {"Yekaterinburg", "Jekaterynburg (czas standardowy)"}, {"Yukon", "czas Jukon"}},
		{{"Alaska", "Alaska (czas letni)"}, {"Almaty", "czas ałmacki letni"}, {"Amazon", "czas amazoński letni"}, {"America_Central", "czas środkowoamerykański letni"}, {"America_Eastern", "czas wschodnioamerykański letni"}, {"America_Mountain", "czas górski letni"}, {"America_Pacific", "czas pacyficzny letni"}, {"Anadyr", "czas Anadyr letni"}, {"Apia", "Apia (czas letni)"}, {"Aqtau", "czas auktaucki letni"}, {"Aqtobe", "czas aktiubiński letni"}, {"Arabian", "Półwysep Arabski (czas letni)"}, {"Argentina", "Argentyna (czas letni)"}, {"Argentina_Western", "Argentyna Zachodnia (czas letni)"}, {"Armenia", "Armenia (czas letni)"}, {"Atlantic", "czas atlantycki letni"}, {"Australia_Central", "czas środkowoaustralijski letni"}, {"Australia_CentralWestern", "czas środkowo-zachodnioaustralijski letni"}, {"Australia_Eastern", "czas wschodnioaustralijski letni"}, {"Australia_Western", "czas zachodnioaustralijski letni"}, {"Azerbaijan", "Azerbejdżan (czas letni)"}, {"Azores", "Azory (czas letni)"}, {"Bangladesh", "Bangladesz (czas letni)"}, {"Brasilia", "Brasília (czas letni)"}, {"Cape_Verde", "Wyspy Zielonego Przylądka (czas letni)"}, {"Chatham", "Chatham (czas letni)"}, {"Chile", "Chile (czas letni)"}, {"China", "Chiny (czas letni)"}, {"Colombia", "Kolumbia (czas letni)"}, {"Cook", "Wyspy Cooka (czas letni)"}, {"Cuba", "Kuba (czas letni)"}, {"Easter", "Wyspa Wielkanocna (czas letni)"}, {"Europe_Central", "czas środkowoeuropejski letni"}, {"Europe_Eastern", "czas wschodnioeuropejski letni"}, {"Europe_Western", "czas zachodnioeuropejski letni"}, {"Falkland", "Falklandy (czas letni)"}, {"Fiji", "Fidżi (czas letni)"}, {"Georgia", "Gruzja (czas letni)"}, {"Greenland_Eastern", "Grenlandia Wschodnia (czas letni)"}, {"Greenland_Western", "Grenlandia Zachodnia (czas letni)"}, {"Hawaii_Aleutian", "Hawaje-Aleuty (czas letni)"}, {"Hong_Kong", "Hongkong (czas letni)"}, {"Hovd", "Kobdo (czas letni)"}, {"Iran", "Iran (czas letni)"}, {"Irkutsk", "Irkuck (czas letni)"}, {"Israel", "Izrael (czas letni)"}, {"Japan", "Japonia (czas letni)"}, {"Kamchatka", "czas Pietropawłowsk Kamczacki letni"}, {"Korea", "Korea (czas letni)"}, {"Krasnoyarsk", "Krasnojarsk (czas letni)"}, {"Lord_Howe", "Lord Howe (czas letni)"}, {"Magadan", "Magadan (czas letni)"}, {"Mauritius", "Mauritius (czas letni)"}, {"Mexico_Pacific", "Meksyk (czas pacyficzny letni)"}, {"Mongolia", "Ułan Bator (czas letni)"}, {"Moscow", "Moskwa (czas letni)"}, {"New_Caledonia", "Nowa Kaledonia (czas letni)"}, {"New_Zealand", "Nowa Zelandia (czas letni)"}, {"Newfoundland", "Nowa Fundlandia (czas letni)"}, {"Norfolk", "Norfolk (czas letni)"}, {"Noronha", "Fernando de Noronha (czas letni)"}, {"Novosibirsk", "Nowosybirsk (czas letni)"}, {"Omsk", "Omsk (czas letni)"}, {"Pakistan", "Pakistan (czas letni)"}, {"Paraguay", "Paragwaj (czas letni)"}, {"Peru", "Peru (czas letni)"}, {"Philippines", "Filipiny (czas letni)"}, {"Pierre_Miquelon", "Saint-Pierre i Miquelon (czas letni)"}, {"Qyzylorda", "czas kyzyłordzki letni"}, {"Sakhalin", "Sachalin (czas letni)"}, {"Samara", "czas Samara letni"}, {"Samoa", "Samoa (czas letni)"}, {"Taipei", "Tajpej (czas letni)"}, {"Tonga", "Tonga (czas letni)"}, {"Turkmenistan", "Turkmenistan (czas letni)"}, {"Uruguay", "Urugwaj (czas letni)"}, {"Uzbekistan", "Uzbekistan (czas letni)"}, {"Vanuatu", "Vanuatu (czas letni)"}, {"Vladivostok", "Władywostok (czas letni)"}, {"Volgograd", "Wołgograd (czas letni)"}, {"Yakutsk", "Jakuck (czas letni)"}, {"Yekaterinburg", "Jekaterynburg (czas letni)"}},
	},
	{
		{{"GMT", "Ora dë Greenwich"}},
		{},
	},
	{
		{{"America_Central", "Centrālas Amērikas zēimas kerdā"}, {"America_Eastern", "Dēiniskas Amērikas zēimas kerdā"}, {"America_Mountain", "Amērikas gārban zēimas kerdā"}, {"America_Pacific", "Pacīfiskas Amērikas zēimas kerdā"}, {"Atlantic", "Atlāntiska zēimas kerdā"}, {"Europe_Central", "Centrālas Eurōpas zēimas kerdā"}, {"Europe_Eastern", "Dēiniskas Eurōpas zēimas kerdā"}, {"Europe_Western", "Wakkariskas Eurōpas zēimas kerdā"}, {"GMT", "Greenwich kerdā"}},
		{{"America_Central", "Centrālas Amērikas daggas kerdā"}, {"America_Eastern", "Dēiniskas Amērikas daggas kerdā"}, {"America_Mountain", "Amērikas gārban daggas kerdā"}, {"America_Pacific", "Pacīfiskas Amērikas daggas kerdā"}, {"Atlantic", "Atlāntiska daggas kerdā"}, {"Europe_Central", "Centrālas Eurōpas daggas kerdā"}, {"Europe_Eastern", "Dēiniskas Eurōpas daggas kerdā"}, {"Europe_Western", "Wakkariskas Eurōpas daggas kerdā"}},
	},
	{
		{{"Afghanistan", "افغانستان وخت"}, {"Africa_Central", "منځنی افريقا وخت"}, {"Africa_Eastern", "ختيځ افريقا وخت"}, {"Africa_Southern", "جنوبي افريقا معياري وخت"}, {"Africa_Western", "لوېديځ افريقا وخت"}, {"Alaska", "الاسکا معياري وخت"}, {"Amazon", "ایمیزون معیاری وخت"}, {"America_Central", "مرکزي معياري وخت"}, {"America_Eastern", "ختيځ معياري وخت"}, {"America_Mountain", "د غره معياري وخت"}, {"America_Pacific", "د پیسفک معياري وخت"}, {"Apia", "اپیا معياري وخت"}, {"Arabian", "عربي معیاري وخت"}, {"Argentina", "ارجنټاین معیاری وخت"}, {"Argentina_Western", "لوېديځ ارجنټاين معياري وخت"}, {"Armenia", "ارمنستان معياري وخت"}, {"Atlantic", "اتلانتیک معياري وخت"}, {"Australia_Central", "آسترالوي مرکزي معياري وخت"}, {"Australia_CentralWestern", "آسترالوي مرکزي لوېديځ معياري وخت"}, {"Australia_Eastern", "آسترالوي ختيځ معياري وخت"}, {"Australia_Western", "آسترالوي لوېديځ معياري وخت"}, {"Azerbaijan", "آذربايجان معياري وخت"}, {"Azores", "ايزورس معياري وخت"}, {"Bangladesh", "بنګلادیش معیاري وخت"}, {"Bhutan", "بهوټان وخت"}, {"Bolivia", "بولیویا وخت"}, {"Brasilia", "برسلیا معیاری وخت"}, {"Brunei", "برونايي دارالسلام وخت"}, {"Cape_Verde", "کیپ وردډ معياري وخت"}, {"Chamorro", "چمارو معياري وخت"}, {"Chatham", "چاتام معياري وخت"}, {"Chile", "چلی معیاری وخت"}, {"China", "چین معیاري وخت"}, {"Christmas", "کريسمس ټاپو وخت"}, {"Cocos", "کوکوز ټاپوګانو وخت"}, {"Colombia", "کولمبیا معیاری وخت"}, {"Cook", "کوک ټاپوګانو معياري وخت"}, {"Cuba", "کیوبا معياري وخت"}, {"Davis", "ډيوس وخت"}, {"DumontDUrville", "ډومونټ ډي ارول"}, {"East_Timor", "ختيځ تيمور وخت"}, {"Easter", "ايستر ټاپو معياري وخت"}, {"Ecuador", "د اکوادور وخت"}, {"Europe_Central", "د مرکزي اروپا معیاري وخت"}, {"Europe_Eastern", "ختيځ اروپايي معياري وخت"}, {"Europe_Further_Eastern", "لرې ختيځ اروپايي وخت"}, {"Europe_Western", "لوېديځ اروپايي معياري وخت"}, {"Falkland", "د فوکلنډ ټاپو معیاری وخت"}, {"Fiji", "فجی معياري وخت"}, {"French_Guiana", "د فرانسوي ګانا وخت"}, {"French_Southern", "د فرانسې سویل او انټارټيک وخت"}, {"GMT", "ګرينويچ معياري وخت"}, {"Galapagos", "ګالپګوس وخت"}, {"Gambier", "ګيمبير وخت"}, {"Georgia", "جورجیا معیاري وخت"}, {"Gilbert_Islands", "جلبرټ ټاپوګانو وخت"}, {"Greenland_Eastern", "د ختیځ ګرینلینډ معياري وخت"}, {"Greenland_Western", "لویدیځ ګرینلینډ معياري وخت"}, {"Gulf", "خلیج معياري وخت"}, {"Guyana", "د ګوانانا وخت"}, {"Hawaii", "هوایی الیوتین معیاری وخت"}, {"Hawaii_Aleutian", "هوایی الیوتین معیاری وخت"}, {"Hong_Kong", "هانګ کانګ معياري وخت"}, {"Hovd", "هاوډ معیاری وخت"}, {"India", "هند معیاري وخت"}, {"Indian_Ocean", "د هند سمندر وخت"}, {"Indochina", "انډوچاینه وخت"}, {"Indonesia_Central", "مرکزي ادونيزيا وخت"}, {"Indonesia_Eastern", "اندونیزیا وخت"}, {"Indonesia_Western", "لویدیځ اندونیزیا وخت"}, {"Iran", "ایران معياري وخت"}, {"Irkutsk", "ارکوټسک معياري وخت"}, {"Israel", "اسراییل معياري وخت"}, {"Japan", "جاپان معياري وخت"}, {"Kazakhstan", "قزاقستان وخت"}, {"Kazakhstan_Eastern", "ختيځ قازقستان وخت"}, {"Kazakhstan_Western", "لویدیځ قزاقستان وخت"}, {"Korea", "کوريايي معياري وخت"}, {"Kosrae", "کوسراي وخت"}, {"Krasnoyarsk", "کريسنويارسک معياري وخت"}, {"Kyrgystan", "کرغیزستان وخت"}, {"Line_Islands", "لاين ټاپوګانو وخت"}, {"Lord_Howe", "لارډ هوي معياري وخت"}, {"Magadan", "ميګډان معياري وخت"}, {"Malaysia", "ملائیشیا وخت"}, {"Maldives", "مالديپ وخت"}, {"Marquesas", "مارکسس وخت"}, {"Marshall_Islands", "مارشل ټاپوګانو وخت"}, {"Mauritius", "ماریشیس معياري وخت"}, {"Mawson", "ماوسن وخت"}, {"Mexico_Pacific", "مکسیکن پیسفک معیاری وخت"}, {"Mongolia", "اولان باټر معیاري وخت"}, {"Moscow", "ماسکو معياري وخت"}, {"Myanmar", "میانمار وخت"}, {"Nauru", "ناورو وخت"}, {"Nepal", "نیپال وخت"}, {"New_Caledonia", "نيو کالیډونیا معياري وخت"}, {"New_Zealand", "نيوزي لېنډ معياري وخت"}, {"Newfoundland", "د نوي فیلډلینډ معیاری وخت"}, {"Niue", "نییو وخت"}, {"Norfolk", "د نورفکاس ټاپو معياري وخت"}, {"Noronha", "فرنانڈو دي نورونها معیاری وخت"}, {"Novosibirsk", "نووسيبرسک معياري وخت"}, {"Omsk", "اومسک معياري وخت"}, {"Pakistan", "پاکستان معیاري وخت"}, {"Palau", "پالاو وخت"}, {"Papua_New_Guinea", "پاپوا نیو ګنی وخت"}, {"Paraguay", "پیراګوای معياري وخت"}, {"Peru", "پیرو معياري وخت"}, {"Philippines", "فلپاين معياري وخت"}, {"Phoenix_Islands", "د فینکس ټاپو وخت"}, {"Pierre_Miquelon", "سینټ پییرا و ميکلين معیاری وخت"}, {"Pitcairn", "پیټ کارین وخت"}, {"Ponape", "پونيپ وخت"}, {"Pyongyang", "پيانګ يانګ وخت"}, {"Reunion", "ري يونين وخت"}, {"Rothera", "رودرا وخت"}, {"Sakhalin", "سخلین معياري وخت"}, {"Samoa", "سموا معياري وخت"}, {"Seychelles", "سیچیلس وخت"}, {"Singapore", "سنګاپور معیاري وخت"}, {"Solomon", "سلیمان ټاپوګانو وخت"}, {"South_Georgia", "د سویل جورجیا وخت"}, {"Suriname", "سورینام وخت"}, {"Syowa", "سیوا وخت"}, {"Tahiti", "ټهيټي وخت"}, {"Taipei", "تايپي معياري وخت"}, {"Tajikistan", "تاجکستان وخت"}, {"Tokelau", "توکیلاو وخت"}, {"Tonga", "د ټونګ معياري وخت"}, {"Truk", "چوک وخت"}, {"Turkmenistan", "ترکمنستان معياري وخت"}, {"Tuvalu", "تووالو وخت"}, {"Uruguay", "یوروګوای معياري وخت"}, {"Uzbekistan", "ازبکستان معياري وخت"}, {"Vanuatu", "ونواتو معياري وخت"}, {"Venezuela", "وینزویلا وخت"}, {"Vladivostok", "ولاديوستاک معياري وخت"}, {"Volgograd", "والګوګراد معياري وخت"}, {"Vostok", "واستوک وخت"}, {"Wake", "ويک تاپو وخت"}, {"Wallis", "والس او فوتونا وخت"}, {"Yakutsk", "ياکوټسک معياري وخت"}, {"Yekaterinburg", "د ياکيټرنبرګ معياري وخت"}, {"Yukon", "د یوکون وخت"}},
		{{"Alaska", "الاسکا د ورځې روښانه وخت"}, {"Amazon", "ایمیزون اوړي وخت"}, {"America_Central", "مرکزي د ورځې روښانه وخت"}, {"America_Eastern", "ختيځ د رڼا ورځې وخت"}, {"America_Mountain", "د غره د رڼا ورځې وخت"}, {"America_Pacific", "پیسفک د رڼا ورځې وخت"}, {"Apia", "اپيا د ورځې روښانه وخت"}, {"Arabian", "عربي د ورځې روښانه وخت"}, {"Argentina", "ارجنټاین اوړي وخت"}, {"Argentina_Western", "لوېديځ ارجنټاين اوړي وخت"}, {"Armenia", "ارمنستان اوړي وخت"}, {"Atlantic", "اتلانتیک د رڼا ورځې وخت"}, {"Australia_Central", "آسترالوي مرکزي د ورځې روښانه وخت"}, {"Australia_CentralWestern", "آسترالوي مرکزي لوېديځ د ورځې روښانه وخت"}, {"Australia_Eastern", "آسترالوي ختيځ د ورځې روښانه وخت"}, {"Australia_Western", "آسترالوي لوېديځ د ورځې روښانه وخت"}, {"Azerbaijan", "اذرباییجان اوړي وخت"}, {"Azores", "ايزورس اوړي وخت"}, {"Bangladesh", "بنګله ديش اوړي وخت"}, {"Brasilia", "برسلیا اوړي وخت"}, {"Cape_Verde", "کیپ وردډ سمر وخت"}, {"Chatham", "چاتام د ورځې روښانه وخت"}, {"Chile", "چلی اوړي وخت"}, {"China", "د چين د رڼا ورځې وخت"}, {"Colombia", "کولمبیا اوړي وخت"}, {"Cook", "کوک ټاپوګانو نيم اوړي وخت"}, {"Cuba", "کیوبا د رڼا ورځې وخت"}, {"Easter", "ايستر ټاپو اوړي وخت"}, {"Europe_Central", "مرکزي اروپايياوړي وخت"}, {"Europe_Eastern", "ختيځ اروپايي اوړي وخت"}, {"Europe_Western", "لوېديځ اروپايي اوړي وخت"}, {"Falkland", "د فوکلنډ ټاپو اوړي وخت"}, {"Fiji", "فجي د اوړي وخت"}, {"Georgia", "د جورجيا د اوړي وخت"}, {"Greenland_Eastern", "د ختیځ ګرینلینډ اوړي وخت"}, {"Greenland_Western", "لویدیځ ګرینلینډ اوړي وخت"}, {"Hawaii_Aleutian", "هوایی الیوتین رڼا ورځې وخت"}, {"Hong_Kong", "هانګ کانګ اوړي وخت"}, {"Hovd", "هاوډ اوړي وخت"}, {"Iran", "ايران د ورځې روښانه وخت"}, {"Irkutsk", "ارکوټسک اوړي وخت"}, {"Israel", "اسراييل د ورځې روښانه وخت"}, {"Japan", "جاپان د ورځې روښانه وخت"}, {"Korea", "کوريايي د ورځې روښانه وخت"}, {"Krasnoyarsk", "کريسنويارسک اوړي وخت"}, {"Lord_Howe", "لارډ هوي د ورځې روښانه وخت"}, {"Magadan", "ميګډان اوړي وخت"}, {"Mauritius", "ماريشيس د اوړي وخت"}, {"Mexico_Pacific", "مکسیکن پیسفک رڼا ورځې وخت"}, {"Mongolia", "اولان باټر اوړي وخت"}, {"Moscow", "ماسکو سمر وخت"}, {"New_Caledonia", "نيو کايډونيا اوړي وخت"}, {"New_Zealand", "نيوزي لېنډ د ورځې روښانه وخت"}, {"Newfoundland", "نيو فاونډلېنډ د ورځې روښانه وخت"}, {"Norfolk", "د نورفکاس ټاپو اوړي وخت"}, {"Noronha", "فرنانڈو دي نورونھا اوړي وخت"}, {"Novosibirsk", "نووسيبرسک اوړي وخت"}, {"Omsk", "اومسک اوړي وخت"}, {"Pakistan", "پاکستان اوړي وخت"}, {"Paraguay", "پاراګوای اوړي وخت"}, {"Peru", "پیرو اوړي وخت"}, {"Philippines", "فلپاين اوړي وخت"}, {"Pierre_Miquelon", "سینټ پییرا و ميکلين رڼا ورځې وخت"}, {"Sakhalin", "سخلين اوړي وخت"}, {"Samoa", "سموا د ورځې روښانه وخت"}, {"Taipei", "تايپي د ورځې روښانه وخت"}, {"Tonga", "ټونګا اوړي وخت"}, {"Turkmenistan", "ترکمنستان اوړي وخت"}, {"Uruguay", "یوروګوای اوړي وخت"}, {"Uzbekistan", "ازبکستان اوړي وخت"}, {"Vanuatu", "ونواتو اوړي وخت"}, {"Vladivostok", "ولاديوستاک اوړي وخت"}, {"Volgograd", "والګوګراد اوړي وخت"}, {"Yakutsk", "ياکوټسک د اوړي وخت"}, {"Yekaterinburg", "د ياکټرنبرګ د اوړي وخت"}},
	},
	{
		{{"Afghanistan", "افغانستان وخت"}, {"Africa_Central", "منځنی افريقا وخت"}, {"Africa_Eastern", "ختيځ افريقا وخت"}, {"Africa_Southern", "جنوبي افريقا معياري وخت"}, {"Africa_Western", "لوېديځ افريقا وخت"}, {"Amazon", "ایمیزون معیاری وخت"}, {"Argentina", "ارجنټاین معیاری وخت"}, {"Argentina_Western", "لوېديځ ارجنټاين معياري وخت"}, {"Armenia", "ارمنستان معياري وخت"}, {"Australia_Central", "آسترالوي مرکزي معياري وخت"}, {"Australia_CentralWestern", "آسترالوي مرکزي لوېديځ معياري وخت"}, {"Australia_Eastern", "آسترالوي ختيځ معياري وخت"}, {"Azerbaijan", "آذربايجان معياري وخت"}, {"Azores", "ايزورس معياري وخت"}, {"Bangladesh", "بنګلادیش معیاري وخت"}, {"Bhutan", "بهوټان وخت"}, {"Bolivia", "بولیویا وخت"}, {"Brasilia", "برسلیا معیاری وخت"}, {"Brunei", "برونايي دارالسلام وخت"}, {"Cape_Verde", "کیپ وردډ معياري وخت"}, {"Chamorro", "چمارو معياري وخت"}, {"Chatham", "چاتام معياري وخت"}, {"Chile", "چلی معیاری وخت"}, {"Christmas", "کريسمس ټاپو وخت"}, {"Cocos", "کوکوز ټاپوګانو وخت"}, {"Colombia", "کولمبیا معیاری وخت"}, {"Cook", "کوک ټاپوګانو معياري وخت"}, {"Davis", "ډيوس وخت"}, {"DumontDUrville", "ډومونټ ډي ارول"}, {"East_Timor", "ختيځ تيمور وخت"}, {"Easter", "ايستر ټاپو معياري وخت"}, {"Ecuador", "د اکوادور وخت"}, {"Europe_Central", "د مرکزي اروپا معیاري وخت"}, {"Europe_Eastern", "ختيځ اروپايي معياري وخت"}, {"Europe_Further_Eastern", "لرې ختيځ اروپايي وخت"}, {"Europe_Western", "د لودیځے اروپا معیاري وخت"}, {"Falkland", "د فوکلنډ ټاپو معیاری وخت"}, {"Fiji", "فجی معياري وخت"}, {"French_Guiana", "د فرانسوي ګانا وخت"}, {"French_Southern", "د فرانسے سویل او انټارټيک وخت"}, {"GMT", "ګرينويچ معياري وخت"}, {"Galapagos", "ګالپګوس وخت"}, {"Gambier", "ګيمبير وخت"}, {"Georgia", "جورجیا معیاري وخت"}, {"Gilbert_Islands", "جلبرټ ټاپوګانو وخت"}, {"Greenland_Eastern", "د ختیځ ګرینلینډ معياري وخت"}, {"Greenland_Western", "لویدیځ ګرینلینډ معياري وخت"}, {"Gulf", "خلیج معياري وخت"}, {"Guyana", "د ګوانانا وخت"}, {"Hong_Kong", "هانګ کانګ معياري وخت"}, {"Hovd", "هاوډ معیاری وخت"}, {"India", "هند معیاري وخت"}, {"Indian_Ocean", "د هند سمندر وخت"}, {"Indochina", "انډوچاینه وخت"}, {"Indonesia_Central", "مرکزي ادونيزيا وخت"}, {"Indonesia_Eastern", "اندونیزیا وخت"}, {"Indonesia_Western", "لویدیځ اندونیزیا وخت"}, {"Irkutsk", "ارکوټسک معياري وخت"}, {"Kazakhstan", "قزاقستان وخت"}, {"Kazakhstan_Eastern", "ختيځ قازقستان وخت"}, {"Kazakhstan_Western", "لویدیځ قزاقستان وخت"}, {"Kosrae", "کوسراي وخت"}, {"Krasnoyarsk", "کريسنويارسک معياري وخت"}, {"Kyrgystan", "کرغیزستان وخت"}, {"Line_Islands", "د کرښے ټاټوبي وخت"}, {"Magadan", "ميګډان معياري وخت"}, {"Malaysia", "ملائیشیا وخت"}, {"Maldives", "مالديپ وخت"}, {"Marquesas", "مارکسس وخت"}, {"Marshall_Islands", "مارشل ټاپوګانو وخت"}, {"Mauritius", "ماریشیس معياري وخت"}, {"Mawson", "ماوسن وخت"}, {"Mongolia", "اولان باټر معیاري وخت"}, {"Moscow", "ماسکو معياري وخت"}, {"Myanmar", "میانمار وخت"}, {"Nauru", "ناورو وخت"}, {"Nepal", "نیپال وخت"}, {"New_Caledonia", "نيو کالیډونیا معياري وخت"}, {"Niue", "نییو وخت"}, {"Norfolk", "د نورفکاس ټاپو معياري وخت"}, {"Noronha", "فرنانڈو دي نورونها معیاری وخت"}, {"Novosibirsk", "نووسيبرسک معياري وخت"}, {"Omsk", "اومسک معياري وخت"}, {"Pakistan", "پاکستان معیاري وخت"}, {"Palau", "پالاو وخت"}, {"Papua_New_Guinea", "پاپوا نیو ګنی وخت"}, {"Paraguay", "پیراګوای معياري وخت"}, {"Peru", "پیرو معياري وخت"}, {"Philippines", "فلپاين معياري وخت"}, {"Phoenix_Islands", "د فینکس ټاپو وخت"}, {"Pitcairn", "پیټ کارین وخت"}, {"Ponape", "پونيپ وخت"}, {"Pyongyang", "پيانګ يانګ وخت"}, {"Reunion", "ري يونين وخت"}, {"Rothera", "رودرا وخت"}, {"Sakhalin", "سخلین معياري وخت"}, {"Seychelles", "سیچیلس وخت"}, {"Singapore", "سنګاپور معیاري وخت"}, {"Solomon", "سلیمان ټاپوګانو وخت"}, {"South_Georgia", "د سویل جورجیا وخت"}, {"Suriname", "سورینام وخت"}, {"Syowa", "سیوا وخت"}, {"Tahiti", "ټهيټي وخت"}, {"Taipei", "تايپي معياري وخت"}, {"Tajikistan", "تاجکستان وخت"}, {"Tokelau", "توکیلاو وخت"}, {"Tonga", "د ټونګ معياري وخت"}, {"Truk", "چوک وخت"}, {"Turkmenistan", "ترکمنستان معياري وخت"}, {"Tuvalu", "تووالو وخت"}, {"Uruguay", "یوروګوای معياري وخت"}, {"Uzbekistan", "ازبکستان معياري وخت"}, {"Vanuatu", "ونواتو معياري وخت"}, {"Venezuela", "وینزویلا وخت"}, {"Vladivostok", "ولاديوستاک معياري وخت"}, {"Volgograd", "والګوګراد معياري وخت"}, {"Vostok", "واستوک وخت"}, {"Wake", "ويک تاپو وخت"}, {"Wallis", "والس او فوتونا وخت"}, {"Yakutsk", "ياکوټسک معياري وخت"}, {"Yekaterinburg", "د ياکيټرنبرګ معياري وخت"}, {"Yukon", "د یوکون وخت"}},
		{{"Alaska", "د الاسکا د ورځے روښانه کول"}, {"Amazon", "ایمیزون اوړي وخت"}, {"America_Central", "مرکزي رڼا ورځے وخت"}, {"America_Eastern", "ختيځ د رڼا ورځے وخت"}, {"America_Mountain", "د غره د رڼا ورځے وخت"}, {"America_Pacific", "پیسفک د رڼا ورځے وخت"}, {"Apia", "د اپیا د ورځے وخت"}, {"Arabian", "د عربي ورځپاڼے وخت"}, {"Argentina", "ارجنټاین اوړي وخت"}, {"Argentina_Western", "لوېديځ ارجنټاين اوړي وخت"}, {"Armenia", "ارمنستان اوړي وخت"}, {"Atlantic", "اتلانتیک د رڼا ورځے وخت"}, {"Australia_Central", "آسترالوي مرکزي د ورځې روښانه وخت"}, {"Australia_CentralWestern", "آسترالوي مرکزي لوېديځ د ورځې روښانه وخت"}, {"Australia_Eastern", "آسترالوي ختيځ د ورځې روښانه وخت"}, {"Australia_Western", "د اسټرالیا لویدیځ د ورځے وخت"}, {"Azerbaijan", "اذرباییجان اوړي وخت"}, {"Azores", "ايزورس اوړي وخت"}, {"Bangladesh", "بنګله ديش اوړي وخت"}, {"Brasilia", "برسلیا اوړي وخت"}, {"Cape_Verde", "کیپ وردډ سمر وخت"}, {"Chatham", "چاتام د ورځې روښانه وخت"}, {"Chile", "چلی اوړي وخت"}, {"China", "د چين د رڼا ورځے وخت"}, {"Colombia", "کولمبیا اوړي وخت"}, {"Cook", "کوک ټاپوګانو نيم اوړي وخت"}, {"Cuba", "کیوبا د رڼا ورځے وخت"}, {"Easter", "ايستر ټاپو اوړي وخت"}, {"Europe_Central", "مرکزي اروپايياوړي وخت"}, {"Europe_Eastern", "ختيځ اروپايي اوړي وخت"}, {"Europe_Western", "د لودیځے اورپا د اوړي وخت"}, {"Falkland", "د فوکلنډ ټاپو اوړي وخت"}, {"Fiji", "فجي د اوړي وخت"}, {"Georgia", "د جورجيا د اوړي وخت"}, {"Greenland_Eastern", "د ختیځ ګرینلینډ اوړي وخت"}, {"Greenland_Western", "لویدیځ ګرینلینډ اوړي وخت"}, {"Hawaii", "هوایی الیوتین رڼا ورځے وخت"}, {"Hawaii_Aleutian", "هوایی الیوتین رڼا ورځے وخت"}, {"Hong_Kong", "هانګ کانګ اوړي وخت"}, {"Hovd", "هاوډ اوړي وخت"}, {"Iran", "د ایران د ورځے وخت"}, {"Irkutsk", "ارکوټسک اوړي وخت"}, {"Israel", "د اسراییلو د ورځے وخت"}, {"Japan", "جاپان د رڼا ورځے وخت"}, {"Korea", "د کوریا د ورځے د ورځے وخت"}, {"Krasnoyarsk", "کريسنويارسک اوړي وخت"}, {"Lord_Howe", "رب هاو د ورځے د رڼا وخت"}, {"Magadan", "ميګډان اوړي وخت"}, {"Mauritius", "ماريشيس د اوړي وخت"}, {"Mexico_Pacific", "مکسیکن پیسفک رڼا ورځے وخت"}, {"Mongolia", "اولان باټر اوړي وخت"}, {"Moscow", "ماسکو سمر وخت"}, {"New_Caledonia", "نيو کايډونيا اوړي وخت"}, {"New_Zealand", "د نیوزی لینڈ د ورځے د رڼا وخت"}, {"Newfoundland", "د نوي فیلډلینډ رڼا ورځے وخت"}, {"Norfolk", "د نورفکاس ټاپو اوړي وخت"}, {"Noronha", "فرنانڈو دي نورونھا اوړي وخت"}, {"Novosibirsk", "نووسيبرسک اوړي وخت"}, {"Omsk", "اومسک اوړي وخت"}, {"Pakistan", "پاکستان اوړي وخت"}, {"Paraguay", "پاراګوای اوړي وخت"}, {"Peru", "پیرو اوړي وخت"}, {"Philippines", "فلپاين اوړي وخت"}, {"Pierre_Miquelon", "سینټ پییرا و ميکلين رڼا ورځے وخت"}, {"Sakhalin", "سخلين اوړي وخت"}, {"Samoa", "د سموا د ورځے روښانه کول"}, {"Taipei", "تايپي د ورځې روښانه وخت"}, {"Tonga", "ټونګا اوړي وخت"}, {"Turkmenistan", "ترکمنستان اوړي وخت"}, {"Uruguay", "یوروګوای اوړي وخت"}, {"Uzbekistan", "ازبکستان اوړي وخت"}, {"Vanuatu", "ونواتو اوړي وخت"}, {"Vladivostok", "ولاديوستاک اوړي وخت"}, {"Volgograd", "والګوګراد اوړي وخت"}, {"Yakutsk", "ياکوټسک د اوړي وخت"}, {"Yekaterinburg", "د ياکټرنبرګ د اوړي وخت"}},
	},
	{
		{{"Acre", "Horário Padrão do Acre"}, {"Afghanistan", "Horário do Afeganistão"}, {"Africa_Central", "Horário da África Central"}, {"Africa_Eastern", "Horário da África Oriental"}, {"Africa_Southern", "Horário da África do Sul"}, {"Africa_Western", "Horário da África Ocidental"}, {"Alaska", "Horário Padrão do Alasca"}, {"Almaty", "Horário Padrão do Almaty"}, {"Amazon", "Horário Padrão do Amazonas"}, {"America_Central", "Horário Padrão Central"}, {"America_Eastern", "Horário Padrão do Leste"}, {"America_Mountain", "Horário Padrão das Montanhas"}, {"America_Pacific", "Horário Padrão do Pacífico"}, {"Anadyr", "Horário Padrão do Anadyr"}, {"Apia", "Horário Padrão de Apia"}, {"Aqtau", "Horário Padrão do Aqtau"}, {"Aqtobe", "Horário Padrão do Aqtobe"}, {"Arabian", "Horário Padrão da Arábia"}, {"Argentina", "Horário Padrão da Argentina"}, {"Argentina_Western", "Horário Padrão da Argentina Ocidental"}, {"Armenia", "Horário Padrão da Armênia"}, {"Atlantic", "Horário Padrão do Atlântico"}, {"Australia_Central", "Horário Padrão da Austrália Central"}, {"Australia_CentralWestern", "Horário Padrão da Austrália Centro-Ocidental"}, {"Australia_Eastern", "Horário Padrão da Austrália Oriental"}, {"Australia_Western", "Horário Padrão da Austrália Ocidental"}, {"Azerbaijan", "Horário Padrão do Arzeibaijão"}, {"Azores", "Horário Padrão dos Açores"}, {"Bangladesh", "Horário Padrão de Bangladesh"}, {"Bhutan", "Horário do Butão"}, {"Bolivia", "Horário da Bolívia"}, {"Brasilia", "Horário Padrão de Brasília"}, {"Brunei", "Horário de Brunei Darussalam"}, {"Cape_Verde", "Horário Padrão de Cabo Verde"}, {"Chamorro", "Horário de Chamorro"}, {"Chatham", "Horário Padrão de Chatham"}, {"Chile", "Horário Padrão do Chile"}, {"China", "Horário Padrão da China"}, {"Christmas", "Horário da Ilha Christmas"}, {"Cocos", "Horário das Ilhas Coco"}, {"Colombia", "Horário Padrão da Colômbia"}, {"Cook", "Horário Padrão das Ilhas Cook"}, {"Cuba", "Horário Padrão de Cuba"}, {"Davis", "Horário de Davis"}, {"DumontDUrville", "Horário de Dumont-d’Urville"}, {"East_Timor", "Horário do Timor-Leste"}, {"Easter", "Horário Padrão da Ilha de Páscoa"}, {"Ecuador", "Horário do Equador"}, {"Europe_Central", "Horário Padrão da Europa Central"}, {"Europe_Eastern", "Horário Padrão da Europa Oriental"}, {"Europe_Further_Eastern", "Horário do Extremo Leste Europeu"}, {"Europe_Western", "Horário Padrão da Europa Ocidental"}, {"Falkland", "Horário Padrão das Ilhas Malvinas"}, {"Fiji", "Horário Padrão de Fiji"}, {"French_Guiana", "Horário da Guiana Francesa"}, {"French_Southern", "Horário dos Territórios Franceses do Sul e Antártida"}, {"GMT", "Horário do Meridiano de Greenwich"}, {"Galapagos", "Horário de Galápagos"}, {"Gambier", "Horário de Gambier"}, {"Georgia", "Horário Padrão da Geórgia"}, {"Gilbert_Islands", "Horário das Ilhas Gilberto"}, {"Greenland_Eastern", "Horário Padrão da Groelândia Oriental"}, {"Greenland_Western", "Horário Padrão da Groenlândia Ocidental"}, {"Guam", "Horário Padrão de Guam"}, {"Gulf", "Horário do Golfo"}, {"Guyana", "Horário da Guiana"}, {"Hawaii", "Horário Padrão do Havaí e Ilhas Aleutas"}, {"Hawaii_Aleutian", "Horário Padrão do Havaí e Ilhas Aleutas"}, {"Hong_Kong", "Horário Padrão de Hong Kong"}, {"Hovd", "Horário Padrão de Hovd"}, {"India", "Horário Padrão da Índia"}, {"Indian_Ocean", "Horário do Oceano Índico"}, {"Indochina", "Horário da Indochina"}, {"Indonesia_Central", "Horário da Indonésia Central"}, {"Indonesia_Eastern", "Horário da Indonésia Oriental"}, {"Indonesia_Western", "Horário da Indonésia Ocidental"}, {"Iran", "Horário Padrão do Irã"}, {"Irkutsk", "Horário Padrão de Irkutsk"}, {"Israel", "Horário Padrão de Israel"}, {"Japan", "Horário Padrão do Japão"}, {"Kamchatka", "Horário Padrão de Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Horário do Cazaquistão"}, {"Kazakhstan_Eastern", "Horário do Cazaquistão Oriental"}, {"Kazakhstan_Western", "Horário do Cazaquistão Ocidental"}, {"Korea", "Horário Padrão da Coreia"}, {"Kosrae", "Horário de Kosrae"}, {"Krasnoyarsk", "Horário Padrão de Krasnoyarsk"}, {"Kyrgystan", "Horário do Quirguistão"}, {"Lanka", "Horário de Lanka"}, {"Line_Islands", "Horário das Ilhas da Linha"}, {"Lord_Howe", "Horário Padrão de Lord Howe"}, {"Macau", "Horário Padrão de Macau"}, {"Magadan", "Horário Padrão de Magadan"}, {"Malaysia", "Horário da Malásia"}, {"Maldives", "Horário das Ilhas Maldivas"}, {"Marquesas", "Horário das Marquesas"}, {"Marshall_Islands", "Horário das Ilhas Marshall"}, {"Mauritius", "Horário Padrão de Maurício"}, {"Mawson", "Horário de Mawson"}, {"Mexico_Pacific", "Horário Padrão do Pacífico Mexicano"}, {"Mongolia", "Horário Padrão de Ulan Bator"}, {"Moscow", "Horário Padrão de Moscou"}, {"Myanmar", "Horário de Mianmar"}, {"Nauru", "Horário de Nauru"}, {"Nepal", "Horário do Nepal"}, {"New_Caledonia", "Horário Padrão da Nova Caledônia"}, {"New_Zealand", "Horário Padrão da Nova Zelândia"}, {"Newfoundland", "Horário Padrão da Terra Nova"}, {"Niue", "Horário de Niue"}, {"Norfolk", "Horário Padrão da Ilha Norfolk"}, {"Noronha", "Horário Padrão de Fernando de Noronha"}, {"North_Mariana", "Horário das Ilhas Mariana do Norte"}, {"Novosibirsk", "Horário Padrão de Novosibirsk"}, {"Omsk", "Horário Padrão de Omsk"}, {"Pakistan", "Horário Padrão do Paquistão"}, {"Palau", "Horário de Palau"}, {"Papua_New_Guinea", "Horário de Papua-Nova Guiné"}, {"Paraguay", "Horário Padrão do Paraguai"}, {"Peru", "Horário Padrão do Peru"}, {"Philippines", "Horário Padrão das Filipinas"}, {"Phoenix_Islands", "Horário das Ilhas Fênix"}, {"Pierre_Miquelon", "Horário Padrão de São Pedro e Miquelão"}, {"Pitcairn", "Horário de Pitcairn"}, {"Ponape", "Horário de Ponape"}, {"Pyongyang", "Horário de Pyongyang"}, {"Qyzylorda", "Horário Padrão de Qyzylorda"}, {"Reunion", "Horário de Reunião"}, {"Rothera", "Horário de Rothera"}, {"Sakhalin", "Horário Padrão de Sacalina"}, {"Samara", "Horário Padrão de Samara"}, {"Samoa", "Horário Padrão de Samoa"}, {"Seychelles", "Horário de Seicheles"}, {"Singapore", "Horário Padrão de Singapura"}, {"Solomon", "Horário das Ilhas Salomão"}, {"South_Georgia", "Horário da Geórgia do Sul"}, {"Suriname", "Horário do Suriname"}, {"Syowa", "Horário de Syowa"}, {"Tahiti", "Horário do Taiti"}, {"Taipei", "Horário Padrão de Taipei"}, {"Tajikistan", "Horário do Tajiquistão"}, {"Tokelau", "Horário de Tokelau"}, {"Tonga", "Horário Padrão de Tonga"}, {"Truk", "Horário de Chuuk"}, {"Turkey", "Horário Padrão da Turquia"}, {"Turkmenistan", "Horário Padrão do Turcomenistão"}, {"Tuvalu", "Horário de Tuvalu"}, {"Uruguay", "Horário Padrão do Uruguai"}, {"Uzbekistan", "Horário Padrão do Uzbequistão"}, {"Vanuatu", "Horário Padrão de Vanuatu"}, {"Venezuela", "Horário da Venezuela"}, {"Vladivostok", "Horário Padrão de Vladivostok"}, {"Volgograd", "Horário Padrão de Volgogrado"}, {"Vostok", "Horário de Vostok"}, {"Wake", "Horário das Ilhas Wake"}, {"Wallis", "Horário de Wallis e Futuna"}, {"Yakutsk", "Horário Padrão de Yakutsk"}, {"Yekaterinburg", "Horário Padrão de Ecaterimburgo"}, {"Yukon", "Horário do Yukon"}},
		{{"Acre", "Horário de Verão do Acre"}, {"Alaska", "Horário de Verão do Alasca"}, {"Almaty", "Horário de Verão do Almaty"}, {"Amazon", "Horário de Verão do Amazonas"}, {"America_Central", "Horário de Verão Central"}, {"America_Eastern", "Horário de Verão do Leste"}, {"America_Mountain", "Horário de Verão das Montanhas"}, {"America_Pacific", "Horário de Verão do Pacífico"}, {"Anadyr", "Horário de Verão do Anadyr"}, {"Apia", "Horário de Verão de Apia"}, {"Aqtau", "Horário de Verão do Aqtau"}, {"Aqtobe", "Horário de Verão do Aqtobe"}, {"Arabian", "Horário de Verão da Arábia"}, {"Argentina", "Horário de Verão da Argentina"}, {"Argentina_Western", "Horário de Verão da Argentina Ocidental"}, {"Armenia", "Horário de Verão da Armênia"}, {"Atlantic", "Horário de Verão do Atlântico"}, {"Australia_Central", "Horário de Verão da Austrália Central"}, {"Australia_CentralWestern", "Horário de Verão da Austrália Centro-Ocidental"}, {"Australia_Eastern", "Horário de Verão da Austrália Oriental"}, {"Australia_Western", "Horário de Verão da Austrália Ocidental"}, {"Azerbaijan", "Horário de Verão do Arzeibaijão"}, {"Azores", "Horário de Verão dos Açores"}, {"Bangladesh", "Horário de Verão de Bangladesh"}, {"Brasilia", "Horário de Verão de Brasília"}, {"Cape_Verde", "Horário de Verão de Cabo Verde"}, {"Chatham", "Horário de Verão de Chatham"}, {"Chile", "Horário de Verão do Chile"}, {"China", "Horário de Verão da China"}, {"Colombia", "Horário de Verão da Colômbia"}, {"Cook", "Meio Horário de Verão das Ilhas Cook"}, {"Cuba", "Horário de Verão de Cuba"}, {"Easter", "Horário de Verão da Ilha de Páscoa"}, {"Europe_Central", "Horário de Verão da Europa Central"}, {"Europe_Eastern", "Horário de Verão da Europa Oriental"}, {"Europe_Western", "Horário de Verão da Europa Ocidental"}, {"Falkland", "Horário de Verão das Ilhas Malvinas"}, {"Fiji", "Horário de Verão de Fiji"}, {"Georgia", "Horário de Verão da Geórgia"}, {"Greenland_Eastern", "Horário de Verão da Groelândia Oriental"}, {"Greenland_Western", "Horário de Verão da Groenlândia Ocidental"}, {"Hawaii_Aleutian", "Horário de Verão do Havaí e Ilhas Aleutas"}, {"Hong_Kong", "Horário de Verão de Hong Kong"}, {"Hovd", "Horário de Verão de Hovd"}, {"Iran", "Horário de Verão do Irã"}, {"Irkutsk", "Horário de Verão de Irkutsk"}, {"Israel", "Horário de Verão de Israel"}, {"Japan", "Horário de Verão do Japão"}, {"Kamchatka", "Horário de Verão de Petropavlovsk-Kamchatski"}, {"Korea", "Horário de Verão da Coreia"}, {"Krasnoyarsk", "Horário de Verão de Krasnoyarsk"}, {"Lord_Howe", "Horário de Verão de Lord Howe"}, {"Macau", "Horário de Verão de Macau"}, {"Magadan", "Horário de Verão de Magadan"}, {"Mauritius", "Horário de Verão de Maurício"}, {"Mexico_Pacific", "Horário de Verão do Pacífico Mexicano"}, {"Mongolia", "Horário de Verão de Ulan Bator"}, {"Moscow", "Horário de Verão de Moscou"}, {"New_Caledonia", "Horário de Verão da Nova Caledônia"}, {"New_Zealand", "Horário de Verão da Nova Zelândia"}, {"Newfoundland", "Horário de Verão da Terra Nova"}, {"Norfolk", "Horário de Verão da Ilha Norfolk"}, {"Noronha", "Horário de Verão de Fernando de Noronha"}, {"Novosibirsk", "Horário de Verão de Novosibirsk"}, {"Omsk", "Horário de Verão de Omsk"}, {"Pakistan", "Horário de Verão do Paquistão"}, {"Paraguay", "Horário de Verão do Paraguai"}, {"Peru", "Horário de Verão do Peru"}, {"Philippines", "Horário de Verão das Filipinas"}, {"Pierre_Miquelon", "Horário Verão de São Pedro e Miquelão"}, {"Qyzylorda", "Horário de Verão de Qyzylorda"}, {"Sakhalin", "Horário de Verão de Sacalina"}, {"Samara", "Horário de Verão de Samara"}, {"Samoa", "Horário de Verão de Samoa"}, {"Taipei", "Horário de Verão de Taipei"}, {"Tonga", "Horário de Verão de Tonga"}, {"Turkey", "Horário de Verão da Turquia"}, {"Turkmenistan", "Horário de Verão do Turcomenistão"}, {"Uruguay", "Horário de Verão do Uruguai"}, {"Uzbekistan", "Horário de Verão do Uzbequistão"}, {"Vanuatu", "Horário de Verão de Vanuatu"}, {"Vladivostok", "Horário de Verão de Vladivostok"}, {"Volgograd", "Horário de Verão de Volgogrado"}, {"Yakutsk", "Horário de Verão de Yakutsk"}, {"Yekaterinburg", "Horário de Verão de Ecaterimburgo"}},
	},
	{
		{{"Acre", "Hora padrão do Acre"}, {"Afghanistan", "Hora do Afeganistão"}, {"Africa_Central", "Hora da África Central"}, {"Africa_Eastern", "Hora da África Oriental"}, {"Africa_Southern", "Hora da África do Sul"}, {"Africa_Western", "Hora da África Ocidental"}, {"Alaska", "Hora padrão do Alasca"}, {"Almaty", "Hora padrão de Almaty"}, {"Amazon", "Hora padrão do Amazonas"}, {"America_Central", "Hora padrão central norte-americana"}, {"America_Eastern", "Hora padrão oriental norte-americana"}, {"America_Mountain", "Hora padrão de montanha norte-americana"}, {"America_Pacific", "Hora padrão do Pacífico norte-americana"}, {"Anadyr", "Hora padrão de Anadyr"}, {"Apia", "Hora padrão de Samoa"}, {"Aqtau", "Hora padrão de Aqtau"}, {"Aqtobe", "Hora padrão de Aqtobe"}, {"Arabian", "Hora padrão da Arábia"}, {"Argentina", "Hora padrão da Argentina"}, {"Argentina_Western", "Hora padrão da Argentina Ocidental"}, {"Armenia", "Hora padrão da Arménia"}, {"Atlantic", "Hora padrão do Atlântico"}, {"Australia_Central", "Hora padrão da Austrália Central"}, {"Australia_CentralWestern", "Hora padrão da Austrália Central Ocidental"}, {"Australia_Eastern", "Hora padrão da Austrália Oriental"}, {"Australia_Western", "Hora padrão da Austrália Ocidental"}, {"Azerbaijan", "Hora padrão do Azerbaijão"}, {"Azores", "Hora padrão dos Açores"}, {"Bangladesh", "Hora padrão do Bangladeche"}, {"Bhutan", "Hora do Butão"}, {"Bolivia", "Hora da Bolívia"}, {"Brasilia", "Hora padrão de Brasília"}, {"Brunei", "Hora do Brunei"}, {"Cape_Verde", "Hora padrão de Cabo Verde"}, {"Chamorro", "Hora padrão de Chamorro"}, {"Chatham", "Hora padrão de Chatham"}, {"Chile", "Hora padrão do Chile"}, {"China", "Hora padrão da China"}, {"Christmas", "Hora da Ilha do Natal"}, {"Cocos", "Hora das Ilhas Cocos"}, {"Colombia", "Hora padrão da Colômbia"}, {"Cook", "Hora padrão das Ilhas Cook"}, {"Cuba", "Hora padrão de Cuba"}, {"Davis", "Hora de Davis"}, {"DumontDUrville", "Hora de Dumont-d’Urville"}, {"East_Timor", "Hora de Timor Leste"}, {"Easter", "Hora padrão da Ilha da Páscoa"}, {"Ecuador", "Hora do Equador"}, {"Europe_Central", "Hora padrão da Europa Central"}, {"Europe_Eastern", "Hora padrão da Europa Oriental"}, {"Europe_Further_Eastern", "Hora do Extremo Leste da Europa"}, {"Europe_Western", "Hora padrão da Europa Ocidental"}, {"Falkland", "Hora padrão das Ilhas Falkland"}, {"Fiji", "Hora padrão de Fiji"}, {"French_Guiana", "Hora da Guiana Francesa"}, {"French_Southern", "Hora das Terras Austrais e Antárcticas Francesas"}, {"GMT", "Hora de Greenwich"}, {"Galapagos", "Hora das Galápagos"}, {"Gambier", "Hora de Gambier"}, {"Georgia", "Hora padrão da Geórgia"}, {"Gilbert_Islands", "Hora das Ilhas Gilbert"}, {"Greenland_Eastern", "Hora padrão da Gronelândia Oriental"}, {"Greenland_Western", "Hora padrão da Gronelândia Ocidental"}, {"Guam", "Hora padrão de Guam"}, {"Gulf", "Hora padrão do Golfo"}, {"Guyana", "Hora da Guiana"}, {"Hawaii", "Hora padrão do Havai e Aleutas"}, {"Hawaii_Aleutian", "Hora padrão do Havai e Aleutas"}, {"Hong_Kong", "Hora padrão de Hong Kong"}, {"Hovd", "Hora padrão de Hovd"}, {"India", "Hora padrão da Índia"}, {"Indian_Ocean", "Hora do Oceano Índico"}, {"Indochina", "Hora da Indochina"}, {"Indonesia_Central", "Hora da Indonésia Central"}, {"Indonesia_Eastern", "Hora da Indonésia Oriental"}, {"Indonesia_Western", "Hora da Indonésia Ocidental"}, {"Iran", "Hora padrão do Irão"}, {"Irkutsk", "Hora padrão de Irkutsk"}, {"Israel", "Hora padrão de Israel"}, {"Japan", "Hora padrão do Japão"}, {"Kamchatka", "Hora padrão de Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Hora do Cazaquistão"}, {"Kazakhstan_Eastern", "Hora do Cazaquistão Oriental"}, {"Kazakhstan_Western", "Hora do Cazaquistão Ocidental"}, {"Korea", "Hora padrão da Coreia"}, {"Kosrae", "Hora de Kosrae"}, {"Krasnoyarsk", "Hora padrão de Krasnoyarsk"}, {"Kyrgystan", "Hora do Quirguistão"}, {"Lanka", "Hora do Sri Lanka"}, {"Line_Islands", "Hora das Ilhas Line"}, {"Lord_Howe", "Hora padrão de Lord Howe"}, {"Macau", "Hora padrão de Macau"}, {"Magadan", "Hora padrão de Magadan"}, {"Malaysia", "Hora da Malásia"}, {"Maldives", "Hora das Maldivas"}, {"Marquesas", "Hora das Ilhas Marquesas"}, {"Marshall_Islands", "Hora das Ilhas Marshall"}, {"Mauritius", "Hora padrão da Maurícia"}, {"Mawson", "Hora de Mawson"}, {"Mexico_Pacific", "Hora padrão do Pacífico Mexicano"}, {"Mongolia", "Hora padrão de Ulan Bator"}, {"Moscow", "Hora padrão de Moscovo"}, {"Myanmar", "Hora de Mianmar"}, {"Nauru", "Hora de Nauru"}, {"Nepal", "Hora do Nepal"}, {"New_Caledonia", "Hora padrão da Nova Caledónia"}, {"New_Zealand", "Hora padrão da Nova Zelândia"}, {"Newfoundland", "Hora padrão da Terra Nova"}, {"Niue", "Hora de Niuê"}, {"Norfolk", "Hora padrão da Ilha Norfolk"}, {"Noronha", "Hora padrão de Fernando de Noronha"}, {"North_Mariana", "Hora das Ilhas Mariana do Norte"}, {"Novosibirsk", "Hora padrão de Novosibirsk"}, {"Omsk", "Hora padrão de Omsk"}, {"Pakistan", "Hora padrão do Paquistão"}, {"Palau", "Hora de Palau"}, {"Papua_New_Guinea", "Hora de Papua Nova Guiné"}, {"Paraguay", "Hora padrão do Paraguai"}, {"Peru", "Hora padrão do Peru"}, {"Philippines", "Hora padrão das Filipinas"}, {"Phoenix_Islands", "Hora das Ilhas Fénix"}, {"Pierre_Miquelon", "Hora padrão de São Pedro e Miquelão"}, {"Pitcairn", "Hora de Pitcairn"}, {"Ponape", "Hora de Pohnpei"}, {"Pyongyang", "Hora da Coreia do Norte"}, {"Qyzylorda", "Hora padrão de Qyzylorda"}, {"Reunion", "Hora de Reunião"}, {"Rothera", "Hora de Rothera"}, {"Sakhalin", "Hora padrão de Sacalina"}, {"Samara", "Hora padrão de Samara"}, {"Samoa", "Hora padrão de Samoa Americana"}, {"Seychelles", "Hora das Seicheles"}, {"Singapore", "Hora padrão de Singapura"}, {"Solomon", "Hora das Ilhas Salomão"}, {"South_Georgia", "Hora da Geórgia do Sul"}, {"Suriname", "Hora do Suriname"}, {"Syowa", "Hora de Syowa"}, {"Tahiti", "Hora do Taiti"}, {"Taipei", "Hora padrão de Taiwan"}, {"Tajikistan", "Hora do Tajiquistão"}, {"Tokelau", "Hora de Tokelau"}, {"Tonga", "Hora padrão de Tonga"}, {"Truk", "Hora de Chuuk"}, {"Turkey", "Hora Padrão da Turquia"}, {"Turkmenistan", "Hora padrão do Turquemenistão"}, {"Tuvalu", "Hora de Tuvalu"}, {"Uruguay", "Hora padrão do Uruguai"}, {"Uzbekistan", "Hora padrão do Uzbequistão"}, {"Vanuatu", "Hora padrão do Vanuatu"}, {"Venezuela", "Hora da Venezuela"}, {"Vladivostok", "Hora padrão de Vladivostok"}, {"Volgograd", "Hora padrão de Volgogrado"}, {"Vostok", "Hora de Vostok"}, {"Wake", "Hora da Ilha Wake"}, {"Wallis", "Hora de Wallis e Futuna"}, {"Yakutsk", "Hora padrão de Yakutsk"}, {"Yekaterinburg", "Hora padrão de Ecaterimburgo"}, {"Yukon", "Hora do Yukon"}},
		{{"Acre", "Hora de verão do Acre"}, {"Alaska", "Hora de verão do Alasca"}, {"Almaty", "Hora de verão de Almaty"}, {"Amazon", "Hora de verão do Amazonas"}, {"America_Central", "Hora de verão central norte-americana"}, {"America_Eastern", "Hora de verão oriental norte-americana"}, {"America_Mountain", "Hora de verão de montanha norte-americana"}, {"America_Pacific", "Hora de verão do Pacífico norte-americana"}, {"Anadyr", "Hora de verão de Anadyr"}, {"Apia", "Hora de verão de Samoa"}, {"Aqtau", "Hora de verão de Aqtau"}, {"Aqtobe", "Hora de verão de Aqtobe"}, {"Arabian", "Hora de verão da Arábia"}, {"Argentina", "Hora de verão da Argentina"}, {"Argentina_Western", "Hora de verão da Argentina Ocidental"}, {"Armenia", "Hora de verão da Arménia"}, {"Atlantic", "Hora de verão do Atlântico"}, {"Australia_Central", "Hora de verão da Austrália Central"}, {"Australia_CentralWestern", "Hora de verão da Austrália Central Ocidental"}, {"Australia_Eastern", "Hora de verão da Austrália Oriental"}, {"Australia_Western", "Hora de verão da Austrália Ocidental"}, {"Azerbaijan", "Hora de verão do Azerbaijão"}, {"Azores", "Hora de verão dos Açores"}, {"Bangladesh", "Hora de verão do Bangladeche"}, {"Brasilia", "Hora de verão de Brasília"}, {"Cape_Verde", "Hora de verão de Cabo Verde"}, {"Chatham", "Hora de verão de Chatham"}, {"Chile", "Hora de verão do Chile"}, {"China", "Hora de verão da China"}, {"Colombia", "Hora de verão da Colômbia"}, {"Cook", "Hora de verão das Ilhas Cook"}, {"Cuba", "Hora de verão de Cuba"}, {"Easter", "Hora de verão da Ilha da Páscoa"}, {"Europe_Central", "Hora de verão da Europa Central"}, {"Europe_Eastern", "Hora de verão da Europa Oriental"}, {"Europe_Western", "Hora de verão da Europa Ocidental"}, {"Falkland", "Hora de verão das Ilhas Falkland"}, {"Fiji", "Hora de verão de Fiji"}, {"Georgia", "Hora de verão da Geórgia"}, {"Greenland_Eastern", "Hora de verão da Gronelândia Oriental"}, {"Greenland_Western", "Hora de verão da Gronelândia Ocidental"}, {"Hawaii_Aleutian", "Hora de verão do Havai e Aleutas"}, {"Hong_Kong", "Hora de verão de Hong Kong"}, {"Hovd", "Hora de verão de Hovd"}, {"Iran", "Hora de verão do Irão"}, {"Irkutsk", "Hora de verão de Irkutsk"}, {"Israel", "Hora de verão de Israel"}, {"Japan", "Hora de verão do Japão"}, {"Kamchatka", "Hora de verão de Petropavlovsk-Kamchatski"}, {"Korea", "Hora de verão da Coreia"}, {"Krasnoyarsk", "Hora de verão de Krasnoyarsk"}, {"Lord_Howe", "Hora de verão de Lord Howe"}, {"Macau", "Hora de verão de Macau"}, {"Magadan", "Hora de verão de Magadan"}, {"Mauritius", "Hora de verão da Maurícia"}, {"Mexico_Pacific", "Hora de verão do Pacífico Mexicano"}, {"Mongolia", "Hora de verão de Ulan Bator"}, {"Moscow", "Hora de verão de Moscovo"}, {"New_Caledonia", "Hora de verão da Nova Caledónia"}, {"New_Zealand", "Hora de verão da Nova Zelândia"}, {"Newfoundland", "Hora de verão da Terra Nova"}, {"Norfolk", "Hora de verão da Ilha Norfolk"}, {"Noronha", "Hora de verão de Fernando de Noronha"}, {"Novosibirsk", "Hora de verão de Novosibirsk"}, {"Omsk", "Hora de verão de Omsk"}, {"Pakistan", "Hora de verão do Paquistão"}, {"Paraguay", "Hora de verão do Paraguai"}, {"Peru", "Hora de verão do Peru"}, {"Philippines", "Hora de verão das Filipinas"}, {"Pierre_Miquelon", "Hora de verão de São Pedro e Miquelão"}, {"Qyzylorda", "Hora de verão de Qyzylorda"}, {"Sakhalin", "Hora de verão de Sacalina"}, {"Samara", "Hora de verão de Samara"}, {"Samoa", "Hora de verão de Samoa Americana"}, {"Taipei", "Hora de verão de Taiwan"}, {"Tonga", "Hora de verão de Tonga"}, {"Turkey", "Hora de Verão da Turquia"}, {"Turkmenistan", "Hora de verão do Turquemenistão"}, {"Uruguay", "Hora de verão do Uruguai"}, {"Uzbekistan", "Hora de verão do Uzbequistão"}, {"Vanuatu", "Hora de verão do Vanuatu"}, {"Vladivostok", "Hora de verão de Vladivostok"}, {"Volgograd", "Hora de verão de Volgogrado"}, {"Yakutsk", "Hora de verão de Yakutsk"}, {"Yekaterinburg", "Hora de verão de Ecaterimburgo"}},
	},
	{
		{{"Afghanistan", "Hora de Afganistán"}, {"Africa_Central", "Hora de Africa Central"}, {"Africa_Eastern", "Hora de Africa Oriental"}, {"Africa_Southern", "Hora de Sudafrica"}, {"Africa_Western", "Hora de Africa Occidental"}, {"Alaska", "Hora Estandar de Alaska"}, {"Amazon", "Hora Estandar de Amazonas"}, {"America_Central", "Estandard Hora Central"}, {"America_Eastern", "Hora Estandar del Este"}, {"America_Mountain", "Hora Estandar de la Montaña"}, {"America_Pacific", "Hora Estandar del Pacífico"}, {"Apia", "Hora Estandar de Apia"}, {"Arabian", "Hora Estandar de Arabia"}, {"Argentina", "Hora Estandar de Argentina"}, {"Argentina_Western", "Hora Estandar del Oeste de Argentina"}, {"Armenia", "Hora Estandar de Armenia"}, {"Atlantic", "Hora Estandar del Atlántico"}, {"Australia_Central", "Hora Estandar de Australia Central"}, {"Australia_CentralWestern", "Hora Estandar de Australia Central Occidental"}, {"Australia_Eastern", "Hora Estandar de Australia Oriental"}, {"Australia_Western", "Hora Estandar de Australia Occidental"}, {"Azerbaijan", "Hora Estandar de Azerbaiyán"}, {"Azores", "Hora Estandar de las Azores"}, {"Bangladesh", "Hora Estandar de Bangladesh"}, {"Bhutan", "Hora de Bután"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Hora Estandar de Brasilia"}, {"Brunei", "Hora de Brunei Darussalam"}, {"Cape_Verde", "Hora Estandar de Cabo Verde"}, {"Chamorro", "Hora Estandar de Chamorro"}, {"Chatham", "Hora Estandar de Chatham"}, {"Chile", "Hora Estandar de Chile"}, {"China", "Hora Estandar de China"}, {"Christmas", "Hora de Isla Christmas"}, {"Cocos", "Hora de Islas Cocos"}, {"Colombia", "Hora Estandar de Colombia"}, {"Cook", "Hora Estandar de Isla Cocos"}, {"Cuba", "Hora Estandar de Cuba"}, {"Davis", "Hora de Davis"}, {"DumontDUrville", "Hora de Dumont-d’Urville"}, {"East_Timor", "Hora de Timor Oriental"}, {"Easter", "Hora Estandar de Isla de Pascua"}, {"Ecuador", "Hora de Ecuador"}, {"Europe_Central", "Hora Estandar de Europa Central"}, {"Europe_Eastern", "Hora Estandar de Europa Oriental"}, {"Europe_Further_Eastern", "Hora de Verano más Oriental de Europa"}, {"Europe_Western", "Hora Estandar de Europa Occidental"}, {"Falkland", "Hora Estandar de Islas Malvinas"}, {"Fiji", "Hora Estandar de Fiji"}, {"French_Guiana", "Hora de Guayana Francesa"}, {"French_Southern", "Hora Francés meridional y antártico"}, {"GMT", "Hora del Meridiano de Greenwich"}, {"Galapagos", "Hora de Islas Galápagos"}, {"Gambier", "Hora de Gambier"}, {"Georgia", "Hora Estandar de Georgia"}, {"Gilbert_Islands", "Hora de Islas Gilbert"}, {"Greenland_Eastern", "Hora Estandar de Groenlandia"}, {"Greenland_Western", "Hora Estandar de Groenlandia Occidental"}, {"Gulf", "Hora Estandar del Golfo"}, {"Guyana", "Hora de Guyana"}, {"Hawaii", "Hora Estandar de Hawai-Aleutiano"}, {"Hawaii_Aleutian", "Hora Estandar de Hawai-Aleutiano"}, {"Hong_Kong", "Hora Estandar de Hong Kong"}, {"Hovd", "Hora Estandar de Hovd"}, {"India", "Hora Estandar de India"}, {"Indian_Ocean", "Hora del Oceano Índico"}, {"Indochina", "Hora de Indochina"}, {"Indonesia_Central", "Hora de Indonesia Central"}, {"Indonesia_Eastern", "Hora de Indonesia Oriental"}, {"Indonesia_Western", "Hora de Indonesia Occidental"}, {"Iran", "Hora Estandar de Irán"}, {"Irkutsk", "Hora Estandar de Irkutsk"}, {"Israel", "Hora Estandar de Israel"}, {"Japan", "Hora Estandar de Japón"}, {"Kazakhstan", "Hora de Kazajistán"}, {"Kazakhstan_Eastern", "Hora de Kazajistán Oriental"}, {"Kazakhstan_Western", "Hora de Kazajistán del Oeste"}, {"Korea", "Hora Estandar de Corea"}, {"Kosrae", "Hora de Kosrae"}, {"Krasnoyarsk", "Hora Estandar de Krasnoyarsk"}, {"Kyrgystan", "Hora de Kirguistán"}, {"Line_Islands", "Hora de Espóradas Ecuatoriales"}, {"Lord_Howe", "Hora Estandar de Lord Howe"}, {"Magadan", "Hora Estandar de Magadan"}, {"Malaysia", "Hora de Malasia"}, {"Maldives", "Hora de Maldivas"}, {"Marquesas", "Hora de Marquesas"}, {"Marshall_Islands", "Hora de Islas Marshall"}, {"Mauritius", "Hora Estandar de Mauricio"}, {"Mawson", "Hora de Mawson"}, {"Mexico_Pacific", "Hora Estandar del Pacífico Mexicano"}, {"Mongolia", "Hora Estandar de Ulán Bator"}, {"Moscow", "Hora Estandar de Moscú"}, {"Myanmar", "Hora de Myanmar"}, {"Nauru", "Hora de Nauru"}, {"Nepal", "Hora de Nepal"}, {"New_Caledonia", "Hora Estandar de Nueva Caledonia"}, {"New_Zealand", "Hora Estandar de Nueva Zelanda"}, {"Newfoundland", "Hora Estandar de Terranova"}, {"Niue", "Hora de Niue"}, {"Norfolk", "Hora Estandar de la Isla Norfolk"}, {"Noronha", "Hora Estandar de Fernando de Noronha"}, {"Novosibirsk", "Hora Estandar de Novosibirsk"}, {"Omsk", "Hora Estandar de Omsk"}, {"Pakistan", "Hora Estandar de Pakistán"}, {"Palau", "Hora de Palau"}, {"Papua_New_Guinea", "Hora de Papua Nueva Guinea"}, {"Paraguay", "Hora Estandar de Paraguay"}, {"Peru", "Hora Estandar de Perú"}, {"Philippines", "Hora Estandar de Filipinas"}, {"Phoenix_Islands", "Hora de Islas Phoenix"}, {"Pierre_Miquelon", "Hora Estandar de San Pedro y Miquelón"}, {"Pitcairn", "Hora de Pitcairn"}, {"Ponape", "Hora de Pohnpei"}, {"Pyongyang", "Hora de Pionyang"}, {"Reunion", "Hora de Réunion"}, {"Rothera", "Hora de Rothera"}, {"Sakhalin", "Hora Estandar de Sakhalin"}, {"Samoa", "Hora Estandar de Samoa"}, {"Seychelles", "Hora de Seychelles"}, {"Singapore", "Hora Estandar de Singapur"}, {"Solomon", "Hora de Islas Salomón"}, {"South_Georgia", "Hora de Georgia del Sur"}, {"Suriname", "Hora de Surinam"}, {"Syowa", "Hora de Syowa"}, {"Tahiti", "Hora de Tahiti"}, {"Taipei", "Hora Estandar de Taipéi"}, {"Tajikistan", "Hora de Tayikistán"}, {"Tokelau", "Hora de Tokelau"}, {"Tonga", "Hora Estandar de Tonga"}, {"Truk", "Hora de Chuuk"}, {"Turkmenistan", "Hora Estandar de Turkmenistán"}, {"Tuvalu", "Hora de Tuvalu"}, {"Uruguay", "Hora Estandar de Uruguay"}, {"Uzbekistan", "Hora Estandar de Uzbekistán"}, {"Vanuatu", "Hora Estandar de Vanuatu"}, {"Venezuela", "Hora de Venezuela"}, {"Vladivostok", "Hora Estandar de Vladivostok"}, {"Volgograd", "Hora Estandar de Volgogrado"}, {"Vostok", "Hora de Vostok"}, {"Wake", "Hora de Isla Wake"}, {"Wallis", "Hora de Wallis y Futuna"}, {"Yakutsk", "Hora Estandar de Yakutsk"}, {"Yekaterinburg", "Hora Estandar de Ekaterinburgo"}, {"Yukon", "Yukon Ura"}},
		{{"Alaska", "Hora de Verano de Alaska"}, {"Amazon", "Hora de Verano de Amazonas"}, {"America_Central", "Hora Central de Verano"}, {"America_Eastern", "Hora de Verano del Este"}, {"America_Mountain", "Hora de Verano de la Montaña"}, {"America_Pacific", "Hora de Verano del Pacífico"}, {"Apia", "Hora de Verano de Apia"}, {"Arabian", "Hora de Verano de Arabia"}, {"Argentina", "Hora de Verano de Argentina"}, {"Argentina_Western", "Hora de Verano del Oeste de Argentina"}, {"Armenia", "Hora de Verano de Armenia"}, {"Atlantic", "Hora De Verano del Atlántico"}, {"Australia_Central", "Hora de Verano de Australia Central"}, {"Australia_CentralWestern", "Hora de Verano de Australia Central Occidental"}, {"Australia_Eastern", "Hora de Verano de Australia Oriental"}, {"Australia_Western", "Hora de Verano de Australia Occidental"}, {"Azerbaijan", "Hora de Verano de Azerbaiyán"}, {"Azores", "Hora de Verano de las Azores"}, {"Bangladesh", "Hora de Verano de Bangladesh"}, {"Brasilia", "Hora de Verano de Brasilia"}, {"Cape_Verde", "Hora de Verano de Cabo Verde"}, {"Chatham", "Hora de Verano de Chatham"}, {"Chile", "Hora de Verano de Chile"}, {"China", "Hora de Verano de China"}, {"Colombia", "Hora de Verano de Colombia"}, {"Cook", "Hora de Verano de Isla Cocos"}, {"Cuba", "Hora de Verano de Cuba"}, {"Easter", "Hora de Verano de Isla de Pascua"}, {"Europe_Central", "Hora de Verano de Europa Central"}, {"Europe_Eastern", "Hora de Verano de Europa Oriental"}, {"Europe_Western", "Hora de Verano de Europa Occidental"}, {"Falkland", "Hora de Verano de Islas Malvinas"}, {"Fiji", "Hora de Verano de Fiji"}, {"Georgia", "Hora de Verano de Georgia"}, {"Greenland_Eastern", "Hora de Verano de Groenlandia"}, {"Greenland_Western", "Hora de Verano de Groenlandia Occidental"}, {"Hawaii_Aleutian", "Hora de Verano de Hawai-Aleutiano"}, {"Hong_Kong", "Hora de Verano de Hong Kong"}, {"Hovd", "Hora de Verano de Hovd"}, {"Iran", "Hora de Verano de Irán"}, {"Irkutsk", "Hora de Verano de Irkutsk"}, {"Israel", "Hora de Verano de Israel"}, {"Japan", "Hora de Verano de Japón"}, {"Korea", "Hora de Verano de Corea"}, {"Krasnoyarsk", "Hora de Verano de Krasnoyarsk"}, {"Lord_Howe", "Hora de Verano de Lord Howe"}, {"Magadan", "Hora de Verano de Magadan"}, {"Mauritius", "Hora de Verano de Mauricio"}, {"Mexico_Pacific", "Hora de Verano del Pacífico Mexicano"}, {"Mongolia", "Hora de Verano de Ulán Bator"}, {"Moscow", "Hora de Verano de Moscú"}, {"New_Caledonia", "Hora de Verano de Nueva Caledonia"}, {"New_Zealand", "Hora de Verano de Nueva Zelanda"}, {"Newfoundland", "Hora de Verano de Terranova"}, {"Norfolk", "Hora de Verano de la Isla Norfolk"}, {"Noronha", "Hora de Verano de Fernando de Noronha"}, {"Novosibirsk", "Hora de Verano de Novosibirsk"}, {"Omsk", "Hora de Verano de Omsk"}, {"Pakistan", "Hora de Verano de Pakistán"}, {"Paraguay", "Hora de Verano de Paraguay"}, {"Peru", "Hora de Verano de Perú"}, {"Philippines", "Hora de Verano de Filipinas"}, {"Pierre_Miquelon", "Hora de Verano de San Pedro y Miquelón"}, {"Sakhalin", "Hora de Verano de Sakhalin"}, {"Samoa", "Hora de Verano de Samoa"}, {"Taipei", "Hora de Verano de Taipéi"}, {"Tonga", "Hora de Verano de Tonga"}, {"Turkmenistan", "Hora de Verano de Turkmenistán"}, {"Uruguay", "Hora de Verano de Uruguay"}, {"Uzbekistan", "Hora de Verano de Uzbekistán"}, {"Vanuatu", "Hora de Verano de Vanuatu"}, {"Vladivostok", "Hora de Verano de Vladivostok"}, {"Volgograd", "Hora de Verano de Volgogrado"}, {"Yakutsk", "Hora de Verano de Yakutsk"}, {"Yekaterinburg", "Hora de Verano de Ekaterinburgo"}},
	},
	{
		{{"GMT", "ग्रीन विच मीन टाइम"}},
		{},
	},
	{
		{{"Afghanistan", "akud n afɣanistan"}, {"Africa_Central", "akud n tefriqt n lwesṭ"}, {"Africa_Eastern", "akud n tefriqt n ccerq"}, {"Africa_Southern", "akud n tefriqt n wadday"}, {"Africa_Western", "akud n tefriqt n lɣerb"}, {"Alaska", "akud anaway n alaska"}, {"Amazon", "akud anaway n amazun"}, {"America_Central", "akud anaway n lwesṭ"}, {"America_Eastern", "akud anaway n lɣerb"}, {"America_Mountain", "akud anaway n idurar"}, {"America_Pacific", "akud anaway n pasifik"}, {"Apia", "akud anaway n samwa"}, {"Arabian", "akud anaway n waɛrab"}, {"Argentina", "akud anaway n arjentina"}, {"Argentina_Western", "akud anaway n lɣerb n arjentina"}, {"Armenia", "akud anaway n arminya"}, {"Atlantic", "akud anaway n atlantik"}, {"Australia_Central", "akud suntral anaway n wustralya"}, {"Australia_CentralWestern", "akud suntral anaway n lɣerb n wustralya"}, {"Australia_Eastern", "akud anaway n ccerq n wustralya"}, {"Australia_Western", "akud anaway n lɣerb n wustralya"}, {"Azerbaijan", "akud anaway n azrabidjan"}, {"Azores", "akud anaway n azuresn azures"}, {"Bangladesh", "akud anaway n bangladic"}, {"Bhutan", "akud n butan"}, {"Bolivia", "akud n bulibya"}, {"Brasilia", "akud anaway n brazilya"}, {"Brunei", "akud n brunay"}, {"Cape_Verde", "akud anaway n qabubirdi"}, {"Chamorro", "akud anaway n tcamuru"}, {"Chatham", "akud anaway n tcatem"}, {"Chile", "akud anaway n cili"}, {"China", "akud anaway n tcina"}, {"Christmas", "akud n tayzirt n kristmas"}, {"Cocos", "akud n tiyzirin n kukus"}, {"Colombia", "akud anaway n kulumbya"}, {"Cook", "akud anaway n tigzirin n kuk"}, {"Cuba", "akud anaway n kuba"}, {"Davis", "akud n deybis"}, {"DumontDUrville", "akud n dimun durbil"}, {"East_Timor", "akud n ṭimur licti"}, {"Easter", "akud anaway n isterayland"}, {"Ecuador", "akud n ikwadur"}, {"Europe_Central", "akud anaway n wuruppa n lwesṭ"}, {"Europe_Eastern", "akud anaway n wuruppa n ccerq"}, {"Europe_Further_Eastern", "akud n wuruppa tacerqect qaɛ"}, {"Europe_Western", "akud anaway n wuruppa n lɣerb"}, {"Falkland", "akud anaway n falkland"}, {"Fiji", "akud anaway n fiji"}, {"French_Guiana", "akud n ɣana tafransist"}, {"French_Southern", "akud n tiwaddayin tifransisin"}, {"GMT", "GMT"}, {"Galapagos", "akud n galappagus"}, {"Gambier", "akud n gambyi"}, {"Georgia", "akud anaway n jyurjya"}, {"Gilbert_Islands", "akud n tigzirin n gilbert"}, {"Greenland_Eastern", "akud anaway n ccerq n grinland"}, {"Greenland_Western", "akud anaway n lwesṭ n grinland"}, {"Gulf", "akud n lxalij"}, {"Guyana", "akud n guyana"}, {"Hawaii", "akud anaway n haway-alucyan"}, {"Hawaii_Aleutian", "akud anaway n haway-alucyan"}, {"Hong_Kong", "akud anaway n hungkung"}, {"Hovd", "akud anaway n xubd"}, {"India", "akud anawy n lhend"}, {"Indian_Ocean", "akud n lebḥer ahindi"}, {"Indochina", "akud n indutcina"}, {"Indonesia_Central", "akud n yindunisya n lwesṭ"}, {"Indonesia_Eastern", "akud n ccerq n yindunisya"}, {"Indonesia_Western", "akud n lɣerb n yindunisya"}, {"Iran", "akud anaway n iran"}, {"Irkutsk", "akud anaway n irkutsek"}, {"Israel", "akud anaway n yisrayil"}, {"Japan", "akud anaway n jjapun"}, {"Kazakhstan", "akud n kazaxistan"}, {"Kazakhstan_Eastern", "akud n ccerq n kazaxistan"}, {"Kazakhstan_Western", "akud n lɣerb n kazaxistan"}, {"Korea", "akud anaway n kurya"}, {"Kosrae", "akud n kursay"}, {"Krasnoyarsk", "akud anaway n krasnuyarsek"}, {"Kyrgystan", "akud n krigistan"}, {"Line_Islands", "akud n tigzirin n layn"}, {"Lord_Howe", "akud anaway n lurdhaw"}, {"Magadan", "akud anaway n magadan"}, {"Malaysia", "akud n malizya"}, {"Maldives", "akud n lmaldib"}, {"Marquesas", "akud n markwisas"}, {"Marshall_Islands", "akud n tigzirin n marcal"}, {"Mauritius", "akud anaway n mawritus"}, {"Mawson", "akud n mawson"}, {"Mexico_Pacific", "akud anaway n pasifik amiksikan"}, {"Mongolia", "akud anaway n ulanbaṭar"}, {"Moscow", "akud anaway n musku"}, {"Myanmar", "akud n myanmar"}, {"Nauru", "akud n nawru"}, {"Nepal", "akud n nnipal"}, {"New_Caledonia", "akud anaway n nyewkalidunya"}, {"New_Zealand", "akud anaway n nyuziland"}, {"Newfoundland", "akud anaway n nyuw fawemd land"}, {"Niue", "akud n nyu"}, {"Norfolk", "akud anaway n tayzirt n nurfuk"}, {"Noronha", "akud anaway n firnardu dinurunha"}, {"Novosibirsk", "akud anaway n nubusibirsek"}, {"Omsk", "akud anaway n umsek"}, {"Pakistan", "akud anaway n pakistan"}, {"Palau", "akud n palaw"}, {"Papua_New_Guinea", "akud n papwa ɣinya tamaynut"}, {"Paraguay", "akud anaway n pparagway"}, {"Peru", "akud anaway n ppiru"}, {"Philippines", "akud anaway n filippin"}, {"Phoenix_Islands", "akud n tigzirin n finiks"}, {"Pierre_Miquelon", "akud anaway n sant-pyiɣ d mikilun"}, {"Pitcairn", "akud n pitkirn"}, {"Ponape", "akud n punpi"}, {"Pyongyang", "akud n kurya n sennej"}, {"Reunion", "akud n riyunyun"}, {"Rothera", "akud n rutira"}, {"Sakhalin", "akud anaway n saxalin"}, {"Samoa", "akud anaway n samwa tamarikant"}, {"Seychelles", "akud n saycal"}, {"Singapore", "akud anaway n singappur"}, {"Solomon", "akud n tigzirin n sulumun"}, {"South_Georgia", "akud n jyurjya n wadday"}, {"Suriname", "akud n surinam"}, {"Syowa", "akud n syuwa"}, {"Tahiti", "akud n tahiti"}, {"Taipei", "akud anaway n ṭṭaywan"}, {"Tajikistan", "akud n tajikistan"}, {"Tokelau", "akud n tuwkulaw"}, {"Tonga", "akud anaway n ṭunga"}, {"Truk", "akud n tcuk"}, {"Turkmenistan", "akud anaway n ṭurkmanistan"}, {"Tuvalu", "akud n ṭubalu"}, {"Uruguay", "akud anaway n urugway"}, {"Uzbekistan", "akud n anaway n uẓbakistan"}, {"Vanuatu", "akud anaway n banwatu"}, {"Venezuela", "akud n vinzwila"}, {"Vladivostok", "akud anaway n bladibustuk"}, {"Volgograd", "akud anaway n bulgugrad"}, {"Vostok", "akud n bustuk"}, {"Wake", "akud n tagzirt n wayk"}, {"Wallis", "akud n walis d futuna"}, {"Yakutsk", "akud anaway n yakutsek"}, {"Yekaterinburg", "akud anaway n yakatirinburg"}, {"Yukon", "akud n yukun"}},
		{{"Alaska", "akud n uzil n alaska"}, {"Amazon", "akud n uzil n amazun"}, {"America_Central", "akud n uzil n lwesṭ"}, {"America_Eastern", "akud n uzil n lɣerb"}, {"America_Mountain", "akud n uzil n idurar"}, {"America_Pacific", "akud n uzil n pasifik"}, {"Apia", "akud n uzil n samwa"}, {"Arabian", "akud n uzil n waɛrab"}, {"Argentina", "akud n uzil n arjentina"}, {"Argentina_Western", "akud n uzil n lɣerb n arjentina"}, {"Armenia", "akud n uzil n arminya"}, {"Atlantic", "akud n uzil n atlantik"}, {"Australia_Central", "akud suntral n uzil n wustralya"}, {"Australia_CentralWestern", "akud suntral n uzil n lɣerb n wustralya"}, {"Australia_Eastern", "akud n uzil n ccerq n wustralya"}, {"Australia_Western", "akud n uzil n lɣerb n wustralya"}, {"Azerbaijan", "akud n uzil n azrabidjan"}, {"Azores", "akud n uzil n azures"}, {"Bangladesh", "akud n uzil n bangladic"}, {"Brasilia", "akud n uzil n brazilya"}, {"Cape_Verde", "akud n uzil n qabubirdi"}, {"Chatham", "akud n uzil n tcatem"}, {"Chile", "akud n uzil n cili"}, {"China", "akud n uzil n tcina"}, {"Colombia", "akud n uzil n kulumbya"}, {"Cook", "akud n uzil n tigzirin n kuk"}, {"Cuba", "akud n uzil n kuba"}, {"Easter", "akud n uzil n isterayland"}, {"Europe_Central", "akud n uzil n wuruppa n lwesṭ"}, {"Europe_Eastern", "akud n uzil n wuruppa n ccerq"}, {"Europe_Western", "akud n uzil n wuruppa n lɣerb"}, {"Falkland", "akud n uzil n falkland"}, {"Fiji", "akud n uzil n fiji"}, {"Georgia", "akud n uzil n jyurjya"}, {"Greenland_Eastern", "akud n uzil n ccerq n grinland"}, {"Greenland_Western", "akud n uzil n lwesṭ n grinland"}, {"Hawaii_Aleutian", "akud n uzil n haway-alucyan"}, {"Hong_Kong", "akud n uzil n hungkung"}, {"Hovd", "akud n uzil n xubd"}, {"Iran", "akud n uzil n iran"}, {"Irkutsk", "akud n uzil n irkutsek"}, {"Israel", "akud n uzil n yisrayil"}, {"Japan", "akud n uzil n jjapun"}, {"Korea", "akud n uzil n kurya"}, {"Krasnoyarsk", "akud n uzil n krasnuyarsek"}, {"Lord_Howe", "akud n uzil n lurdhaw"}, {"Magadan", "akud n uzil n magadan"}, {"Mauritius", "akud n uzil n mawritus"}, {"Mexico_Pacific", "akud n uzil n pasifik amiksikan"}, {"Mongolia", "akud n uzil n ulanbaṭar"}, {"Moscow", "akud n uzil n musku"}, {"New_Caledonia", "akud n uzil n nyewkalidunya"}, {"New_Zealand", "akud n uzil n nyuziland"}, {"Newfoundland", "akud n uzil n nyuw fawemd land"}, {"Norfolk", "akud n uzil n tayzirt n nurfuk"}, {"Noronha", "akud n uzil n firnardu dinurunha"}, {"Novosibirsk", "akud n uzil n nubusibirsek"}, {"Omsk", "akud n uzil n umsek"}, {"Pakistan", "akud n uzil n pakistan"}, {"Paraguay", "akud n uzil n pparagway"}, {"Peru", "akud n uzil n ppiru"}, {"Philippines", "akud n uzil n filippin"}, {"Pierre_Miquelon", "akud n uzil n sant-pyiɣ d mikilun"}, {"Sakhalin", "akud n uzil n saxalin"}, {"Samoa", "akud n uzil n samwa tamarikant"}, {"Taipei", "akud n uzil n ṭṭaywan"}, {"Tonga", "akud n uzil n ṭunga"}, {"Turkmenistan", "akud n uzil n ṭurkmanistan"}, {"Uruguay", "akud n uzil n urugway"}, {"Uzbekistan", "akud n uzil n uẓbakistan"}, {"Vanuatu", "akud n uzil n banwatu"}, {"Vladivostok", "akud n uzil n bladibustuk"}, {"Volgograd", "akud n uzil n bulgugrad"}, {"Yakutsk", "akud n uzil n yakutsek"}, {"Yekaterinburg", "akud n uzil n yakatirinburg"}},
	},
	{
		{{"Afghanistan", "temp da l’Afganistan"}, {"Africa_Central", "temp da l’Africa Centrala"}, {"Africa_Eastern", "temp da l’Africa Orientala"}, {"Africa_Southern", "temp normal da l’Africa Meridiunala"}, {"Africa_Western", "temp da l’Africa Occidentala"}, {"Alaska", "temp normal da l’Alasca"}, {"Amazon", "temp normal da l’Amazonas"}, {"America_Central", "temp normal dal center USA"}, {"America_Eastern", "temp normal da l’ost USA"}, {"America_Mountain", "temp normal dals Rocky Mountains"}, {"America_Pacific", "temp normal dal Pacific USA"}, {"Apia", "temp normal da la Samoa"}, {"Arabian", "temp normal arab"}, {"Argentina", "temp normal da l’Argentina"}, {"Argentina_Western", "temp normal da l’Argentina occidentala"}, {"Armenia", "temp normal da l’Armenia"}, {"Atlantic", "temp normal da l’Atlantic USA"}, {"Australia_Central", "temp normal dal center da l’Australia"}, {"Australia_CentralWestern", "temp normal dal center-vest da l’Australia"}, {"Australia_Eastern", "temp normal da l’ost da l’Australia"}, {"Australia_Western", "temp normal dal vest da l’Australia"}, {"Azerbaijan", "temp normal da l’Aserbaidschan"}, {"Azores", "temp normal da las Azoras"}, {"Bangladesh", "temp normal dal Bangladesch"}, {"Bhutan", "temp dal Butan"}, {"Bolivia", "temp da la Bolivia"}, {"Brasilia", "temp normal da la Brasilia"}, {"Brunei", "temp dal Brunei"}, {"Cape_Verde", "temp normal dal Cap Verd"}, {"Chamorro", "temp dals Chamorro"}, {"Chatham", "temp normal da las Inslas Chatham"}, {"Chile", "temp normal dal Chile"}, {"China", "temp normal da la China"}, {"Christmas", "temp da l’Insla da Nadal"}, {"Cocos", "temp da las Inslas Cocos"}, {"Colombia", "temp normal da la Columbia"}, {"Cook", "temp normal da las Inslas Cook"}, {"Cuba", "temp normal da la Cuba"}, {"Davis", "temp da Davis"}, {"DumontDUrville", "temp da Dumont d’Urville"}, {"East_Timor", "temp dal Timor da l’Ost"}, {"Easter", "temp normal da l’Insla da Pasca"}, {"Ecuador", "temp da l’Ecuador"}, {"Europe_Central", "temp normal da l’Europa Centrala"}, {"Europe_Eastern", "temp normal da l’Europa Orientala"}, {"Europe_Further_Eastern", "temp da l’extrem orient da l’Europa"}, {"Europe_Western", "temp normal da l’Europa dal Vest"}, {"Falkland", "temp normal da las Inslas Falkland"}, {"Fiji", "temp normal dal Fidschi"}, {"French_Guiana", "temp da la Guyana Franzosa"}, {"French_Southern", "temp dals Territoris Franzos Meridiunals ed Antarctics"}, {"GMT", "temp dal meridian da Greenwich"}, {"Galapagos", "temp da las Inslas Galápagos"}, {"Gambier", "temp da las Inslas Gambier"}, {"Georgia", "temp normal da la Georgia"}, {"Gilbert_Islands", "temp da las Inslas Gilbert"}, {"Greenland_Eastern", "temp normal da la Grönlanda orientala"}, {"Greenland_Western", "temp normal da la Grönlanda occidentala"}, {"Gulf", "temp normal dal Golf"}, {"Guyana", "temp da la Guyana"}, {"Hawaii", "temp normal dal Hawai e las Aleutinas"}, {"Hawaii_Aleutian", "temp normal dal Hawai e las Aleutinas"}, {"Hong_Kong", "temp normal dal Hongkong"}, {"Hovd", "temp normal da Hovd"}, {"India", "temp normal da l’India"}, {"Indian_Ocean", "temp da l’Ocean Indic"}, {"Indochina", "temp da l’Indochina"}, {"Indonesia_Central", "temp dal center da l’Indonesia"}, {"Indonesia_Eastern", "temp da l’ost da l’Indonesia"}, {"Indonesia_Western", "temp dal vest da l’Indonesia"}, {"Iran", "temp normal da l’Iran"}, {"Irkutsk", "temp normal dad Irkutsk"}, {"Israel", "temp normal da l’Israel"}, {"Japan", "temp normal dal Giapun"}, {"Kazakhstan", "temp dal Kasachstan"}, {"Kazakhstan_Eastern", "temp dal Kasachstan oriental"}, {"Kazakhstan_Western", "temp dal Kasachstan occidental"}, {"Korea", "temp normal da la Corea"}, {"Kosrae", "temp da Kosrae"}, {"Krasnoyarsk", "temp normal da Krasnojarsk"}, {"Kyrgystan", "temp dal Kirghistan"}, {"Line_Islands", "temp da las Inslas da la Lingia"}, {"Lord_Howe", "temp normal da Lord Howe"}, {"Magadan", "temp normal da Magadan"}, {"Malaysia", "temp da la Malaisia"}, {"Maldives", "temp da las Maledivas"}, {"Marquesas", "temp da las Inslas Marquesas"}, {"Marshall_Islands", "temp da las Inslas da Marshall"}, {"Mauritius", "temp normal dal Mauritius"}, {"Mawson", "temp da Mawson"}, {"Mexico_Pacific", "temp normal mexican dal Pacific"}, {"Mongolia", "temp normal dad Ulaanbaatar"}, {"Moscow", "temp normal da Moscau"}, {"Myanmar", "temp dal Myanmar"}, {"Nauru", "temp da Nauru"}, {"Nepal", "temp dal Nepal"}, {"New_Caledonia", "temp normal da la Nova Caledonia"}, {"New_Zealand", "temp normal da la Nova Zelanda"}, {"Newfoundland", "temp normal da la Terranova"}, {"Niue", "temp da Niue"}, {"Norfolk", "temp normal da l’Insla Norfolk"}, {"Noronha", "temp normal da Fernando de Noronha"}, {"Novosibirsk", "temp normal da Novosibirsk"}, {"Omsk", "temp normal dad Omsk"}, {"Pakistan", "temp normal dal Pakistan"}, {"Palau", "temp da Palau"}, {"Papua_New_Guinea", "temp da la Papua Nova Guinea"}, {"Paraguay", "temp normal dal Paraguai"}, {"Peru", "temp normal dal Peru"}, {"Philippines", "temp normal da las Filippinas"}, {"Phoenix_Islands", "temp da las Inslas Fenix"}, {"Pierre_Miquelon", "temp normal da Saint Pierre e Miquelon"}, {"Pitcairn", "temp da las Inslas Pitcairn"}, {"Ponape", "temp da Pohnpei"}, {"Pyongyang", "temp da la Corea dal Nord"}, {"Reunion", "temp da la Réunion"}, {"Rothera", "temp da Rothera"}, {"Sakhalin", "temp normal da Sachalin"}, {"Samoa", "temp normal da la Samoa Americana"}, {"Seychelles", "temp da las Seychellas"}, {"Singapore", "temp normal dal Singapur"}, {"Solomon", "temp da las Inslas da Salomon"}, {"South_Georgia", "temp da la Georgia dal Sid"}, {"Suriname", "temp dal Surinam"}, {"Syowa", "temp da Shōwa"}, {"Tahiti", "temp da Tahiti"}, {"Taipei", "temp normal dal Taiwan"}, {"Tajikistan", "temp dal Tadschikistan"}, {"Tokelau", "temp da Tokelau"}, {"Tonga", "temp normal da Tonga"}, {"Truk", "temp da Chuuk"}, {"Turkmenistan", "temp normal dal Turkmenistan"}, {"Tuvalu", "temp da las Inslas da Tuvalu"}, {"Uruguay", "temp normal da l’Uruguai"}, {"Uzbekistan", "temp normal da l’Usbekistan"}, {"Vanuatu", "temp normal dal Vanuatu"}, {"Venezuela", "temp da la Venezuela"}, {"Vladivostok", "temp normal da Vladivostok"}, {"Volgograd", "temp normal da Volgograd"}, {"Vostok", "temp da Vostok"}, {"Wake", "temp da l’Insla Wake"}, {"Wallis", "temp da Wallis e Futuna"}, {"Yakutsk", "temp normal da Jakutsk"}, {"Yekaterinburg", "temp normal da Jekaterinburg"}, {"Yukon", "temp dal Yukon"}},
		{{"Alaska", "temp da stad da l’Alasca"}, {"Amazon", "temp da stad da l’Amazonas"}, {"America_Central", "temp da stad dal center USA"}, {"America_Eastern", "temp da stad da l’ost USA"}, {"America_Mountain", "temp da stad dals Rocky Mountains"}, {"America_Pacific", "temp da stad dal Pacific USA"}, {"Apia", "temp da stad da la Samoa"}, {"Arabian", "temp da stad arab"}, {"Argentina", "temp da stad da l’Argentina"}, {"Argentina_Western", "temp da stad da l’Argentina occidentala"}, {"Armenia", "temp da stad da l’Armenia"}, {"Atlantic", "temp da stad da l’Atlantic USA"}, {"Australia_Central", "temp da stad dal center da l’Australia"}, {"Australia_CentralWestern", "temp da stad dal center-vest da l’Australia"}, {"Australia_Eastern", "temp da stad da l’ost da l’Australia"}, {"Australia_Western", "temp da stad dal vest da l’Australia"}, {"Azerbaijan", "temp da stad da l’Aserbaidschan"}, {"Azores", "temp da stad da las Azoras"}, {"Bangladesh", "temp da stad dal Bangladesch"}, {"Brasilia", "temp da stad da la Brasilia"}, {"Cape_Verde", "temp da stad dal Cap Verd"}, {"Chatham", "temp da stad da las Inslas Chatham"}, {"Chile", "temp da stad dal Chile"}, {"China", "temp da stad da la China"}, {"Colombia", "temp da stad da la Columbia"}, {"Cook", "temp da stad da las Inslas Cook"}, {"Cuba", "temp da stad da la Cuba"}, {"Easter", "temp da stad da l’Insla da Pasca"}, {"Europe_Central", "temp da stad da l’Europa Centrala"}, {"Europe_Eastern", "temp da stad da l’Europa Orientala"}, {"Europe_Western", "temp da stad da l’Europa dal Vest"}, {"Falkland", "temp da stad da las Inslas Falkland"}, {"Fiji", "temp da stad dal Fidschi"}, {"Georgia", "temp da stad da la Georgia"}, {"Greenland_Eastern", "temp da stad da la Grönlanda orientala"}, {"Greenland_Western", "temp da stad da la Grönlanda occidentala"}, {"Hawaii_Aleutian", "temp da stad dal Hawai e da las Aleutinas"}, {"Hong_Kong", "temp da stad dal Hongkong"}, {"Hovd", "temp da stad da Hovd"}, {"Iran", "temp da stad da l’Iran"}, {"Irkutsk", "temp da stad dad Irkutsk"}, {"Israel", "temp da stad da l’Israel"}, {"Japan", "temp da stad dal Giapun"}, {"Korea", "temp da stad da la Corea"}, {"Krasnoyarsk", "temp da stad da Krasnojarsk"}, {"Lord_Howe", "temp da stad da Lord Howe"}, {"Magadan", "temp da stad da Magadan"}, {"Mauritius", "temp da stad dal Mauritius"}, {"Mexico_Pacific", "temp da stad mexican dal Pacific"}, {"Mongolia", "temp da stad dad Ulaanbaatar"}, {"Moscow", "temp da stad da Moscau"}, {"New_Caledonia", "temp da stad da la Nova Caledonia"}, {"New_Zealand", "temp da stad da la Nova Zelanda"}, {"Newfoundland", "temp da stad da la Terranova"}, {"Norfolk", "temp da stad da l’Insla Norfolk"}, {"Noronha", "temp da stad da Fernando de Noronha"}, {"Novosibirsk", "temp da stad da Novosibirsk"}, {"Omsk", "temp da stad dad Omsk"}, {"Pakistan", "temp da stad dal Pakistan"}, {"Paraguay", "temp da stad dal Paraguai"}, {"Peru", "temp da stad dal Peru"}, {"Philippines", "temp da stad da las Filippinas"}, {"Pierre_Miquelon", "temp da stad da Saint Pierre e Miquelon"}, {"Sakhalin", "temp da stad da Sachalin"}, {"Samoa", "temp da stad da la Samoa Americana"}, {"Taipei", "temp da stad dal Taiwan"}, {"Tonga", "temp da stad da Tonga"}, {"Turkmenistan", "temp da stad dal Turkmenistan"}, {"Uruguay", "temp da stad da l’Uruguai"}, {"Uzbekistan", "temp da stad da l’Usbekistan"}, {"Vanuatu", "temp da stad dal Vanuatu"}, {"Vladivostok", "temp da stad da Vladivostok"}, {"Volgograd", "temp da stad da Volgograd"}, {"Yakutsk", "temp da stad da Jakutsk"}, {"Yekaterinburg", "temp da stad da Jekaterinburg"}},
	},
	{
		{{"Acre", "Ora standard Acre"}, {"Afghanistan", "Ora Afganistanului"}, {"Africa_Central", "Ora Africii Centrale"}, {"Africa_Eastern", "Ora Africii Orientale"}, {"Africa_Southern", "Ora Africii Meridionale"}, {"Africa_Western", "Ora Africii Occidentale"}, {"Alaska", "Ora standard din Alaska"}, {"Almaty", "Ora standard Almaty"}, {"Amazon", "Ora standard a Amazonului"}, {"America_Central", "Ora standard centrală nord-americană"}, {"America_Eastern", "Ora standard orientală nord-americană"}, {"America_Mountain", "Ora standard în zona montană nord-americană"}, {"America_Pacific", "Ora standard în zona Pacific nord-americană"}, {"Anadyr", "Ora standard din Anadyr"}, {"Apia", "Ora standard din Samoa"}, {"Aqtau", "Ora standard Aqtau"}, {"Aqtobe", "Ora standard Aqtobe"}, {"Arabian", "Ora standard arabă"}, {"Argentina", "Ora standard a Argentinei"}, {"Argentina_Western", "Ora standard a Argentinei Occidentale"}, {"Armenia", "Ora standard a Armeniei"}, {"Atlantic", "Ora standard în zona Atlantic nord-americană"}, {"Australia_Central", "Ora standard a Australiei Centrale"}, {"Australia_CentralWestern", "Ora standard a Australiei Central Occidentale"}, {"Australia_Eastern", "Ora standard a Australiei Orientale"}, {"Australia_Western", "Ora standard a Australiei Occidentale"}, {"Azerbaijan", "Ora standard a Azerbaidjanului"}, {"Azores", "Ora standard din Azore"}, {"Bangladesh", "Ora standard din Bangladesh"}, {"Bhutan", "Ora Bhutanului"}, {"Bolivia", "Ora Boliviei"}, {"Brasilia", "Ora standard a Brasiliei"}, {"Brunei", "Ora din Brunei Darussalam"}, {"Cape_Verde", "Ora standard din Capul Verde"}, {"Chamorro", "Ora din Chamorro"}, {"Chatham", "Ora standard din Chatham"}, {"Chile", "Ora standard din Chile"}, {"China", "Ora standard a Chinei"}, {"Christmas", "Ora din Insula Christmas"}, {"Cocos", "Ora Insulelor Cocos"}, {"Colombia", "Ora standard a Columbiei"}, {"Cook", "Ora standard a Insulelor Cook"}, {"Cuba", "Ora standard a Cubei"}, {"Davis", "Ora din Davis"}, {"DumontDUrville", "Ora din Dumont-d’Urville"}, {"East_Timor", "Ora Timorului de Est"}, {"Easter", "Ora standard din Insula Paștelui"}, {"Ecuador", "Ora Ecuadorului"}, {"Europe_Central", "Ora standard a Europei Centrale"}, {"Europe_Eastern", "Ora standard a Europei de Est"}, {"Europe_Further_Eastern", "Ora Europei de Est îndepărtate"}, {"Europe_Western", "Ora standard a Europei de Vest"}, {"Falkland", "Ora standard din Insulele Falkland"}, {"Fiji", "Ora standard din Fiji"}, {"French_Guiana", "Ora din Guyana Franceză"}, {"French_Southern", "Ora din Teritoriile Australe și Antarctice Franceze"}, {"GMT", "Ora de Greenwich"}, {"Galapagos", "Ora din Galapagos"}, {"Gambier", "Ora din Gambier"}, {"Georgia", "Ora standard a Georgiei"}, {"Gilbert_Islands", "Ora Insulelor Gilbert"}, {"Greenland_Eastern", "Ora standard a Groenlandei orientale"}, {"Greenland_Western", "Ora standard a Groenlandei occidentale"}, {"Gulf", "Ora standard a Golfului"}, {"Guyana", "Ora din Guyana"}, {"Hawaii", "Ora standard din Hawaii-Aleutine"}, {"Hawaii_Aleutian", "Ora standard din Hawaii-Aleutine"}, {"Hong_Kong", "Ora standard din Hong Kong"}, {"Hovd", "Ora standard din Hovd"}, {"India", "Ora Indiei"}, {"Indian_Ocean", "Ora Oceanului Indian"}, {"Indochina", "Ora Indochinei"}, {"Indonesia_Central", "Ora Indoneziei Centrale"}, {"Indonesia_Eastern", "Ora Indoneziei de Est"}, {"Indonesia_Western", "Ora Indoneziei de Vest"}, {"Iran", "Ora standard a Iranului"}, {"Irkutsk", "Ora standard din Irkuțk"}, {"Israel", "Ora standard a Israelului"}, {"Japan", "Ora standard a Japoniei"}, {"Kamchatka", "Ora standard din Petropavlovsk-Kamciațki"}, {"Kazakhstan", "Ora din Kazahstan"}, {"Kazakhstan_Eastern", "Ora din Kazahstanul de Est"}, {"Kazakhstan_Western", "Ora din Kazahstanul de Vest"}, {"Korea", "Ora standard a Coreei"}, {"Kosrae", "Ora din Kosrae"}, {"Krasnoyarsk", "Ora standard din Krasnoiarsk"}, {"Kyrgystan", "Ora din Kârgâzstan"}, {"Line_Islands", "Ora din Insulele Line"}, {"Lord_Howe", "Ora standard din Lord Howe"}, {"Magadan", "Ora standard din Magadan"}, {"Malaysia", "Ora din Malaysia"}, {"Maldives", "Ora din Maldive"}, {"Marquesas", "Ora Insulelor Marchize"}, {"Marshall_Islands", "Ora Insulelor Marshall"}, {"Mauritius", "Ora standard din Mauritius"}, {"Mawson", "Ora din Mawson"}, {"Mexico_Pacific", "Ora standard a zonei Pacific mexicane"}, {"Mongolia", "Ora standard din Ulan Bator"}, {"Moscow", "Ora standard a Moscovei"}, {"Myanmar", "Ora Myanmarului"}, {"Nauru", "Ora din Nauru"}, {"Nepal", "Ora Nepalului"}, {"New_Caledonia", "Ora standard a Noii Caledonii"}, {"New_Zealand", "Ora standard a Noii Zeelande"}, {"Newfoundland", "Ora standard din Newfoundland"}, {"Niue", "Ora din Niue"}, {"Norfolk", "Ora standard a Insulei Norfolk"}, {"Noronha", "Ora standard din Fernando de Noronha"}, {"Novosibirsk", "Ora standard din Novosibirsk"}, {"Omsk", "Ora standard din Omsk"}, {"Pakistan", "Ora standard a Pakistanului"}, {"Palau", "Ora din Palau"}, {"Papua_New_Guinea", "Ora din Papua Noua Guinee"}, {"Paraguay", "Ora standard din Paraguay"}, {"Peru", "Ora standard din Peru"}, {"Philippines", "Ora standard din Filipine"}, {"Phoenix_Islands", "Ora Insulelor Phoenix"}, {"Pierre_Miquelon", "Ora standard din Saint-Pierre și Miquelon"}, {"Pitcairn", "Ora din Pitcairn"}, {"Ponape", "Ora din Ponape"}, {"Pyongyang", "Ora din Pyongyang"}, {"Reunion", "Ora din Reunion"}, {"Rothera", "Ora din Rothera"}, {"Sakhalin", "Ora standard din Sahalin"}, {"Samara", "Ora standard din Samara"}, {"Samoa", "Ora standard din Samoa"}, {"Seychelles", "Ora din Seychelles"}, {"Singapore", "Ora din Singapore"}, {"Solomon", "Ora Insulelor Solomon"}, {"South_Georgia", "Ora Georgiei de Sud"}, {"Suriname", "Ora Surinamului"}, {"Syowa", "Ora din Syowa"}, {"Tahiti", "Ora din Tahiti"}, {"Taipei", "Ora standard din Taipei"}, {"Tajikistan", "Ora din Tadjikistan"}, {"Tokelau", "Ora din Tokelau"}, {"Tonga", "Ora standard din Tonga"}, {"Truk", "Ora din Chuuk"}, {"Turkmenistan", "Ora standard din Turkmenistan"}, {"Tuvalu", "Ora din Tuvalu"}, {"Uruguay", "Ora standard a Uruguayului"}, {"Uzbekistan", "Ora standard din Uzbekistan"}, {"Vanuatu", "Ora standard din Vanuatu"}, {"Venezuela", "Ora Venezuelei"}, {"Vladivostok", "Ora standard din Vladivostok"}, {"Volgograd", "Ora standard din Volgograd"}, {"Vostok", "Ora din Vostok"}, {"Wake", "Ora Insulei Wake"}, {"Wallis", "Ora din Wallis și Futuna"}, {"Yakutsk", "Ora standard din Iakuțk"}, {"Yekaterinburg", "Ora standard din Ekaterinburg"}, {"Yukon", "Ora din Yukon"}},
		{{"Acre", "Ora de vară Acre"}, {"Alaska", "Ora de vară din Alaska"}, {"Almaty", "Ora de vară Almaty"}, {"Amazon", "Ora de vară a Amazonului"}, {"America_Central", "Ora de vară centrală nord-americană"}, {"America_Eastern", "Ora de vară orientală nord-americană"}, {"America_Mountain", "Ora de vară în zona montană nord-americană"}, {"America_Pacific", "Ora de vară în zona Pacific nord-americană"}, {"Anadyr", "Ora de vară din Anadyr"}, {"Apia", "Ora de vară din Samoa"}, {"Aqtau", "Ora de vară a zonei Aqtau"}, {"Aqtobe", "Ora de vară a zonei Aqtobe"}, {"Arabian", "Ora de vară arabă"}, {"Argentina", "Ora de vară a Argentinei"}, {"Argentina_Western", "Ora de vară a Argentinei Occidentale"}, {"Armenia", "Ora de vară a Armeniei"}, {"Atlantic", "Ora de vară în zona Atlantic nord-americană"}, {"Australia_Central", "Ora de vară a Australiei Centrale"}, {"Australia_CentralWestern", "Ora de vară a Australiei Central Occidentale"}, {"Australia_Eastern", "Ora de vară a Australiei Orientale"}, {"Australia_Western", "Ora de vară a Australiei Occidentale"}, {"Azerbaijan", "Ora de vară a Azerbaidjanului"}, {"Azores", "Ora de vară din Azore"}, {"Bangladesh", "Ora de vară din Bangladesh"}, {"Brasilia", "Ora de vară a Brasiliei"}, {"Cape_Verde", "Ora de vară din Capul Verde"}, {"Chatham", "Ora de vară din Chatham"}, {"Chile", "Ora de vară din Chile"}, {"China", "Ora de vară a Chinei"}, {"Colombia", "Ora de vară a Columbiei"}, {"Cook", "Ora de vară a Insulelor Cook"}, {"Cuba", "Ora de vară a Cubei"}, {"Easter", "Ora de vară din Insula Paștelui"}, {"Europe_Central", "Ora de vară a Europei Centrale"}, {"Europe_Eastern", "Ora de vară a Europei de Est"}, {"Europe_Western", "Ora de vară a Europei de Vest"}, {"Falkland", "Ora de vară din Insulele Falkland"}, {"Fiji", "Ora de vară din Fiji"}, {"Georgia", "Ora de vară a Georgiei"}, {"Greenland_Eastern", "Ora de vară a Groenlandei orientale"}, {"Greenland_Western", "Ora de vară a Groenlandei occidentale"}, {"Hawaii_Aleutian", "Ora de vară din Hawaii-Aleutine"}, {"Hong_Kong", "Ora de vară din Hong Kong"}, {"Hovd", "Ora de vară din Hovd"}, {"Iran", "Ora de vară a Iranului"}, {"Irkutsk", "Ora de vară din Irkuțk"}, {"Israel", "Ora de vară a Israelului"}, {"Japan", "Ora de vară a Japoniei"}, {"Kamchatka", "Ora de vară din Petropavlovsk-Kamciațki"}, {"Korea", "Ora de vară a Coreei"}, {"Krasnoyarsk", "Ora de vară din Krasnoiarsk"}, {"Lord_Howe", "Ora de vară din Lord Howe"}, {"Magadan", "Ora de vară din Magadan"}, {"Mauritius", "Ora de vară din Mauritius"}, {"Mexico_Pacific", "Ora de vară a zonei Pacific mexicane"}, {"Mongolia", "Ora de vară din Ulan Bator"}, {"Moscow", "Ora de vară a Moscovei"}, {"New_Caledonia", "Ora de vară a Noii Caledonii"}, {"New_Zealand", "Ora de vară a Noii Zeelande"}, {"Newfoundland", "Ora de vară din Newfoundland"}, {"Norfolk", "Ora de vară a Insulei Norfolk"}, {"Noronha", "Ora de vară din Fernando de Noronha"}, {"Novosibirsk", "Ora de vară din Novosibirsk"}, {"Omsk", "Ora de vară din Omsk"}, {"Pakistan", "Ora de vară a Pakistanului"}, {"Paraguay", "Ora de vară din Paraguay"}, {"Peru", "Ora de vară din Peru"}, {"Philippines", "Ora de vară din Filipine"}, {"Pierre_Miquelon", "Ora de vară din Saint-Pierre și Miquelon"}, {"Sakhalin", "Ora de vară din Sahalin"}, {"Samara", "Ora de vară din Samara"}, {"Samoa", "Ora de vară din Samoa"}, {"Taipei", "Ora de vară din Taipei"}, {"Tonga", "Ora de vară din Tonga"}, {"Turkmenistan", "Ora de vară din Turkmenistan"}, {"Uruguay", "Ora de vară a Uruguayului"}, {"Uzbekistan", "Ora de vară din Uzbekistan"}, {"Vanuatu", "Ora de vară din Vanuatu"}, {"Vladivostok", "Ora de vară din Vladivostok"}, {"Volgograd", "Ora de vară din Volgograd"}, {"Yakutsk", "Ora de vară din Iakuțk"}, {"Yekaterinburg", "Ora de vară din Ekaterinburg"}},
	},
	{
		{{"Acre", "Акри стандартное время"}, {"Afghanistan", "Афганистан"}, {"Africa_Central", "Центральная Африка"}, {"Africa_Eastern", "Восточная Африка"}, {"Africa_Southern", "Южная Африка"}, {"Africa_Western", "Западная Африка"}, {"Alaska", "Аляска, стандартное время"}, {"Almaty", "Алма-Ата стандартное время"}, {"Amazon", "Амазонка, стандартное время"}, {"America_Central", "Центральная Америка, стандартное время"}, {"America_Eastern", "Восточная Америка, стандартное время"}, {"America_Mountain", "Стандартное горное время (Северная Америка)"}, {"America_Pacific", "Тихоокеанское стандартное время"}, {"Anadyr", "Анадырь стандартное время"}, {"Apia", "Апиа, стандартное время"}, {"Aqtau", "Актау, стандартное время"}, {"Aqtobe", "Актобе стандартное время"}, {"Arabian", "Саудовская Аравия, стандартное время"}, {"Argentina", "Аргентина, стандартное время"}, {"Argentina_Western", "Западная Аргентина, стандартное время"}, {"Armenia", "Армения, стандартное время"}, {"Atlantic", "Атлантическое стандартное время"}, {"Australia_Central", "Центральная Австралия, стандартное время"}, {"Australia_CentralWestern", "Центральная Австралия, западное стандартное время"}, {"Australia_Eastern", "Восточная Австралия, стандартное время"}, {"Australia_Western", "Западная Австралия, стандартное время"}, {"Azerbaijan", "Азербайджан, стандартное время"}, {"Azores", "Азорские о-ва, стандартное время"}, {"Bangladesh", "Бангладеш, стандартное время"}, {"Bhutan", "Бутан"}, {"Bolivia", "Боливия"}, {"Brasilia", "Бразилия, стандартное время"}, {"Brunei", "Бруней-Даруссалам"}, {"Cape_Verde", "Кабо-Верде, стандартное время"}, {"Casey", "Кейси"}, {"Chamorro", "Чаморро"}, {"Chatham", "Чатем, стандартное время"}, {"Chile", "Чили, стандартное время"}, {"China", "Китай, стандартное время"}, {"Christmas", "о-в Рождества"}, {"Cocos", "Кокосовые о-ва"}, {"Colombia", "Колумбия, стандартное время"}, {"Cook", "Острова Кука, стандартное время"}, {"Cuba", "Куба, стандартное время"}, {"Davis", "Дейвис"}, {"DumontDUrville", "Дюмон-д’Юрвиль"}, {"East_Timor", "Восточный Тимор"}, {"Easter", "О-в Пасхи, стандартное время"}, {"Ecuador", "Эквадор"}, {"Europe_Central", "Центральная Европа, стандартное время"}, {"Europe_Eastern", "Восточная Европа, стандартное время"}, {"Europe_Further_Eastern", "Московское время"}, {"Europe_Western", "Западная Европа, стандартное время"}, {"Falkland", "Фолклендские о-ва, стандартное время"}, {"Fiji", "Фиджи, стандартное время"}, {"French_Guiana", "Французская Гвиана"}, {"French_Southern", "Французские Южные и Антарктические территории"}, {"GMT", "Среднее время по Гринвичу"}, {"Galapagos", "Галапагосские о-ва"}, {"Gambier", "Гамбье"}, {"Georgia", "Грузия, стандартное время"}, {"Gilbert_Islands", "о-ва Гилберта"}, {"Greenland_Eastern", "Восточная Гренландия, стандарное время"}, {"Greenland_Western", "Западная Гренландия, стандартное время"}, {"Guam", "Гуам"}, {"Gulf", "Персидский залив"}, {"Guyana", "Гайана"}, {"Hawaii", "Гавайско-алеутское стандартное время"}, {"Hawaii_Aleutian", "Гавайско-алеутское стандартное время"}, {"Hong_Kong", "Гонконг, стандартное время"}, {"Hovd", "Ховд, стандартное время"}, {"India", "Индия"}, {"Indian_Ocean", "Индийский океан"}, {"Indochina", "Индокитай"}, {"Indonesia_Central", "Центральная Индонезия"}, {"Indonesia_Eastern", "Восточная Индонезия"}, {"Indonesia_Western", "Западная Индонезия"}, {"Iran", "Иран, стандартное время"}, {"Irkutsk", "Иркутск, стандартное время"}, {"Israel", "Израиль, стандартное время"}, {"Japan", "Япония, стандартное время"}, {"Kamchatka", "Петропавловск-Камчатский, стандартное время"}, {"Kazakhstan", "Казахстан"}, {"Kazakhstan_Eastern", "Восточный Казахстан"}, {"Kazakhstan_Western", "Западный Казахстан"}, {"Korea", "Корея, стандартное время"}, {"Kosrae", "Косрае"}, {"Krasnoyarsk", "Красноярск, стандартное время"}, {"Kyrgystan", "Киргизия"}, {"Lanka", "Шри-Ланка"}, {"Line_Islands", "о-ва Лайн"}, {"Lord_Howe", "Лорд-Хау, стандартное время"}, {"Macau", "Макао, стандартное время"}, {"Magadan", "Магадан, стандартное время"}, {"Malaysia", "Малайзия"}, {"Maldives", "Мальдивы"}, {"Marquesas", "Маркизские о-ва"}, {"Marshall_Islands", "Маршалловы Острова"}, {"Mauritius", "Маврикий, стандартное время"}, {"Mawson", "Моусон"}, {"Mexico_Pacific", "Тихоокеанское мексиканское стандартное время"}, {"Mongolia", "Улан-Батор, стандартное время"}, {"Moscow", "Москва, стандартное время"}, {"Myanmar", "Мьянма"}, {"Nauru", "Науру"}, {"Nepal", "Непал"}, {"New_Caledonia", "Новая Каледония, стандартное время"}, {"New_Zealand", "Новая Зеландия, стандартное время"}, {"Newfoundland", "Ньюфаундленд, стандартное время"}, {"Niue", "Ниуэ"}, {"Norfolk", "Норфолк, стандартное время"}, {"Noronha", "Фернанду-ди-Норонья, стандартное время"}, {"North_Mariana", "Северные Марианские о-ва"}, {"Novosibirsk", "Новосибирск, стандартное время"}, {"Omsk", "Омск, стандартное время"}, {"Pakistan", "Пакистан, стандартное время"}, {"Palau", "Палау"}, {"Papua_New_Guinea", "Папуа – Новая Гвинея"}, {"Paraguay", "Парагвай, стандартное время"}, {"Peru", "Перу, стандартное время"}, {"Philippines", "Филиппины, стандартное время"}, {"Phoenix_Islands", "о-ва Феникс"}, {"Pierre_Miquelon", "Сен-Пьер и Микелон, стандартное время"}, {"Pitcairn", "Питкэрн"}, {"Ponape", "Понпеи"}, {"Pyongyang", "Пхеньян"}, {"Qyzylorda", "Кызылорда, стандартное время*"}, {"Reunion", "Реюньон"}, {"Rothera", "Ротера"}, {"Sakhalin", "Сахалин, стандартное время"}, {"Samara", "Самарское стандартное время"}, {"Samoa", "Самоа, стандартное время"}, {"Seychelles", "Сейшельские Острова"}, {"Singapore", "Сингапур"}, {"Solomon", "Соломоновы Острова"}, {"South_Georgia", "Южная Георгия"}, {"Suriname", "Суринам"}, {"Syowa", "Сёва"}, {"Tahiti", "Таити"}, {"Taipei", "Тайвань, стандартное время"}, {"Tajikistan", "Таджикистан"}, {"Tokelau", "Токелау"}, {"Tonga", "Тонга, стандартное время"}, {"Truk", "Трук"}, {"Turkey", "Турецкое стандартное время"}, {"Turkmenistan", "Туркменистан, стандартное время"}, {"Tuvalu", "Тувалу"}, {"Uruguay", "Уругвай, стандартное время"}, {"Uzbekistan", "Узбекистан, стандартное время"}, {"Vanuatu", "Вануату, стандартное время"}, {"Venezuela", "Венесуэла"}, {"Vladivostok", "Владивосток, стандартное время"}, {"Volgograd", "Волгоград, стандартное время"}, {"Vostok", "Восток"}, {"Wake", "Уэйк"}, {"Wallis", "Уоллис и Футуна"}, {"Yakutsk", "Якутск, стандартное время"}, {"Yekaterinburg", "Екатеринбург, стандартное время"}, {"Yukon", "Юкон"}},
		{{"Acre", "Акри летнее время"}, {"Alaska", "Аляска, летнее время"}, {"Almaty", "Алма-Ата летнее время"}, {"Amazon", "Амазонка, летнее время"}, {"America_Central", "Центральная Америка, летнее время"}, {"America_Eastern", "Восточная Америка, летнее время"}, {"America_Mountain", "Летнее горное время (Северная Америка)"}, {"America_Pacific", "Тихоокеанское летнее время"}, {"Anadyr", "Анадырь летнее время"}, {"Apia", "Апиа, летнее время"}, {"Aqtau", "Актау летнее время"}, {"Aqtobe", "Актобе летнее время"}, {"Arabian", "Саудовская Аравия, летнее время"}, {"Argentina", "Аргентина, летнее время"}, {"Argentina_Western", "Западная Аргентина, летнее время"}, {"Armenia", "Армения, летнее время"}, {"Atlantic", "Атлантическое летнее время"}, {"Australia_Central", "Центральная Австралия, летнее время"}, {"Australia_CentralWestern", "Центральная Австралия, западное летнее время"}, {"Australia_Eastern", "Восточная Австралия, летнее время"}, {"Australia_Western", "Западная Австралия, летнее время"}, {"Azerbaijan", "Азербайджан, летнее время"}, {"Azores", "Азорские о-ва, летнее время"}, {"Bangladesh", "Бангладеш, летнее время"}, {"Brasilia", "Бразилия, летнее время"}, {"Cape_Verde", "Кабо-Верде, летнее время"}, {"Chatham", "Чатем, летнее время"}, {"Chile", "Чили, летнее время"}, {"China", "Китай, летнее время"}, {"Colombia", "Колумбия, летнее время"}, {"Cook", "Острова Кука, полулетнее время"}, {"Cuba", "Куба, летнее время"}, {"Easter", "О-в Пасхи, летнее время"}, {"Europe_Central", "Центральная Европа, летнее время"}, {"Europe_Eastern", "Восточная Европа, летнее время"}, {"Europe_Western", "Западная Европа, летнее время"}, {"Falkland", "Фолклендские о-ва, летнее время"}, {"Fiji", "Фиджи, летнее время"}, {"Georgia", "Грузия, летнее время"}, {"Greenland_Eastern", "Восточная Гренландия, летнее время"}, {"Greenland_Western", "Западная Гренландия, летнее время"}, {"Hawaii_Aleutian", "Гавайско-алеутское летнее время"}, {"Hong_Kong", "Гонконг, летнее время"}, {"Hovd", "Ховд, летнее время"}, {"Iran", "Иран, летнее время"}, {"Irkutsk", "Иркутск, летнее время"}, {"Israel", "Израиль, летнее время"}, {"Japan", "Япония, летнее время"}, {"Kamchatka", "Петропавловск-Камчатский, летнее время"}, {"Korea", "Корея, летнее время"}, {"Krasnoyarsk", "Красноярск, летнее время"}, {"Lord_Howe", "Лорд-Хау, летнее время"}, {"Macau", "Макао, летнее время"}, {"Magadan", "Магадан, летнее время"}, {"Mauritius", "Маврикий, летнее время"}, {"Mexico_Pacific", "Тихоокеанское мексиканское летнее время"}, {"Mongolia", "Улан-Батор, летнее время"}, {"Moscow", "Москва, летнее время"}, {"New_Caledonia", "Новая Каледония, летнее время"}, {"New_Zealand", "Новая Зеландия, летнее время"}, {"Newfoundland", "Ньюфаундленд, летнее время"}, {"Norfolk", "Норфолк, летнее время"}, {"Noronha", "Фернанду-ди-Норонья, летнее время"}, {"Novosibirsk", "Новосибирск, летнее время"}, {"Omsk", "Омск, летнее время"}, {"Pakistan", "Пакистан, летнее время"}, {"Paraguay", "Парагвай, летнее время"}, {"Peru", "Перу, летнее время"}, {"Philippines", "Филиппины, летнее время"}, {"Pierre_Miquelon", "Сен-Пьер и Микелон, летнее время"}, {"Qyzylorda", "Кызылорда, летнее время*"}, {"Sakhalin", "Сахалин, летнее время"}, {"Samara", "Самарское летнее время"}, {"Samoa", "Самоа, летнее время"}, {"Taipei", "Тайвань, летнее время"}, {"Tonga", "Тонга, летнее время"}, {"Turkey", "Турецкое летнее время"}, {"Turkmenistan", "Туркменистан, летнее время"}, {"Uruguay", "Уругвай, летнее время"}, {"Uzbekistan", "Узбекистан, летнее время"}, {"Vanuatu", "Вануату, летнее время"}, {"Vladivostok", "Владивосток, летнее время"}, {"Volgograd", "Волгоград, летнее время"}, {"Yakutsk", "Якутск, летнее время"}, {"Yekaterinburg", "Екатеринбург, летнее время"}},
	},
	{
		{{"GMT", "Greenwich Mean Time"}},
		{},
	},
	{
		{{"America_Central", "उत्तर अमेरिका: मध्य आदर्श समयः"}, {"America_Eastern", "उत्तर अमेरिका: पौर्व आदर्श समयः"}, {"America_Mountain", "उत्तर अमेरिका: शैल आदर्श समयः"}, {"America_Pacific", "उत्तर अमेरिका: सन्धिप्रिय आदर्श समयः"}, {"Atlantic", "अटलाण्टिक आदर्श समयः"}, {"Europe_Central", "मध्य यूरोपीय आदर्श समयः"}, {"Europe_Eastern", "पौर्व यूरोपीय आदर्श समयः"}, {"Europe_Western", "पाश्चात्य यूरोपीय आदर्श समयः"}, {"GMT", "ग्रीनविच मीन समयः"}},
		{{"America_Central", "उत्तर अमेरिका: मध्य अयाम समयः"}, {"America_Eastern", "उत्तर अमेरिका: पौर्व अयाम समय:"}, {"America_Mountain", "उत्तर अमेरिका: शैल अयाम समयः"}, {"America_Pacific", "उत्तर अमेरिका: सन्धिप्रिय अयाम समयः"}, {"Atlantic", "अटलाण्टिक अयाम समयः"}, {"Europe_Central", "मध्य यूरोपीय ग्रीष्म समयः"}, {"Europe_Eastern", "पौर्व यूरोपीय ग्रीष्म समयः"}, {"Europe_Western", "पाश्चात्य यूरोपीय ग्रीष्म समयः"}},
	},
	{
		{{"Arabian", "Арааб сүрүн кэмэ"}, {"Armenia", "Эрмээн сүрүн кэмэ"}, {"Australia_Central", "Киин Австралия сүрүн кэмэ"}, {"Australia_Eastern", "Илин Австралия сүрүн кэмэ"}, {"Australia_Western", "Арҕаа Австралия сүрүн кэмэ"}, {"China", "Кытай сүрүн кэмэ"}, {"GMT", "GMT"}, {"Georgia", "Курусуун сүрүн кэмэ"}, {"India", "Ииндийэ сүрүн кэмэ"}, {"Iran", "Ираан сүрүн кэмэ"}, {"Japan", "Дьоппуон сүрүн кэмэ"}, {"Kazakhstan_Eastern", "Илин Казахстаан кэмэ"}, {"Kazakhstan_Western", "Арҕаа Казахстаан кэмэ"}, {"Korea", "Кэриэй сүрүн кэмэ"}, {"Krasnoyarsk", "Красноярскай сүрүн кэмэ"}, {"Kyrgystan", "Кыргыстаан кэмэ"}, {"Magadan", "Магадаан сүрүн кэмэ"}, {"Mongolia", "Улан Баатар сүрүн кэмэ"}, {"Moscow", "Москуба сүрүн кэмэ"}, {"New_Zealand", "Саҥа Сэйлэнд сүрүн кэмэ"}, {"Novosibirsk", "Новосибирскай сүрүн кэмэ"}, {"Omsk", "Омскай сүрүн кэмэ"}, {"Pakistan", "Пакистаан сүрүн кэмэ"}, {"Sakhalin", "Сахалыын сүрүн кэмэ"}, {"Vladivostok", "Быладьыбастыак сүрүн кэмэ"}, {"Volgograd", "Волгоград сүрүн кэмэ"}, {"Yakutsk", "Дьокуускай сүрүн кэмэ"}, {"Yekaterinburg", "Екатеринбуур сүрүн кэмэ"}},
		{{"Arabian", "Арааб сайыҥҥы кэмэ"}, {"Armenia", "Эрмээн сайыҥҥы кэмэ"}, {"Australia_Central", "Киин Австралия сайыҥҥы кэмэ"}, {"Australia_Eastern", "Илин Австралия сайыҥҥы кэмэ"}, {"Australia_Western", "Арҕаа Австралия сайыҥҥы кэмэ"}, {"China", "Кытай сайыҥҥы кэмэ"}, {"Georgia", "Курусуун сайыҥҥы кэмэ"}, {"Iran", "Ыраан сайыҥҥы кэмэ"}, {"Japan", "Дьоппуон сайыҥҥы кэмэ"}, {"Korea", "Кэриэй сайыҥҥы кэмэ"}, {"Krasnoyarsk", "Красноярскай сайыҥҥы кэмэ"}, {"Magadan", "Магадаан сайыҥҥы кэмэ"}, {"Mongolia", "Улан Баатар сайыҥҥы кэмэ"}, {"Moscow", "Москуба сайыҥҥы кэмэ"}, {"New_Zealand", "Саҥа Сэйлэнд сайыҥҥы кэмэ"}, {"Novosibirsk", "Новосибирскай сайыҥҥы кэмэ"}, {"Omsk", "Омскай сайыҥҥы кэмэ"}, {"Pakistan", "Пакистаан сайыҥҥы кэмэ"}, {"Sakhalin", "Сахалыын сайыҥҥы кэмэ"}, {"Vladivostok", "Быладьыбастыак сайыҥҥы кэмэ"}, {"Volgograd", "Волгоград сайыҥҥы кэмэ"}, {"Yakutsk", "Дьокуускай сайыҥҥы кэмэ"}, {"Yekaterinburg", "Екатеринбуур сайыҥҥы кэмэ"}},
	},
	{
		{{"Alaska", "ᱟᱹᱞᱟᱥᱠᱟ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"Amazon", "ᱟᱢᱟᱡᱚᱱ ᱮᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱴᱚ"}, {"America_Central", "ᱛᱟᱱᱟᱞᱟ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"America_Eastern", "ᱤᱥᱴᱟᱨᱱ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"America_Mountain", "ᱢᱟᱩᱱᱴᱮᱱ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"America_Pacific", "ᱯᱮᱥᱤᱯᱷᱤᱠ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"Argentina", "ᱟᱹᱨᱡᱮᱱᱴᱤᱱᱟ ᱮᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱴᱚ"}, {"Argentina_Western", "ᱯᱟᱪᱮ ᱟᱹᱨᱡᱮᱱᱴᱤᱱᱟ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱴᱚ"}, {"Atlantic", "ᱮᱴᱞᱟᱱᱴᱤᱠ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"Cuba", "ᱠᱭᱩᱵᱟ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Europe_Central", "ᱥᱮᱱᱴᱨᱟᱞ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"Europe_Eastern", "ᱤᱥᱴᱟᱨᱱ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"Europe_Western", "ᱣᱮᱥᱴᱟᱨᱱ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱮᱥᱴᱮᱱᱰᱟᱨᱰ ᱚᱠᱛᱚ"}, {"GMT", "ᱜᱨᱤᱱᱣᱤᱪ ᱢᱤᱱ ᱚᱠᱛᱚ"}, {"Greenland_Eastern", "ᱥᱟᱢᱟᱝ ᱜᱽᱨᱤᱱᱞᱮᱱᱰ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Greenland_Western", "ᱯᱟᱪᱮ ᱜᱽᱨᱤᱱᱞᱮᱱᱰ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Hawaii", "ᱦᱟᱣᱟᱭᱤᱼᱟᱞᱮᱣᱴᱤᱭᱟᱱ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Hawaii_Aleutian", "ᱦᱟᱣᱟᱭᱤᱼᱟᱞᱮᱣᱴᱤᱭᱟᱱ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Mexico_Pacific", "ᱢᱮᱠᱥᱤᱠᱟᱱ ᱯᱨᱚᱥᱟᱱᱛ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Newfoundland", "ᱱᱤᱭᱩᱯᱷᱟᱩᱱᱰᱞᱮᱸᱰ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Pierre_Miquelon", "ᱮᱥ ᱴᱤ ᱯᱤᱭᱮᱨᱮ ᱟᱨ ᱢᱤᱠᱮᱞᱚᱱ ᱢᱟᱱᱚᱠ ᱚᱠᱛᱚ"}, {"Yukon", "ᱭᱩᱠᱚᱱ ᱚᱠᱛᱚ"}},
		{{"Alaska", "ᱟᱹᱞᱟᱥᱠᱟ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Amazon", "ᱟᱢᱟᱡᱚᱱ ᱥᱤᱛᱩᱝ ᱚᱠᱴᱚ"}, {"America_Central", "ᱛᱟᱱᱟᱞᱟ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"America_Eastern", "ᱤᱥᱴᱟᱨᱱ ᱥᱤᱧᱟᱜ ᱵᱚᱠᱛᱚ"}, {"America_Mountain", "ᱢᱟᱩᱱᱴᱮᱱ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"America_Pacific", "ᱯᱮᱥᱤᱯᱷᱤᱠ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Argentina", "ᱟᱹᱨᱡᱮᱱᱴᱤᱱᱟ ᱥᱤᱛᱩᱝ ᱚᱠᱴᱚ"}, {"Argentina_Western", "ᱯᱟᱪᱮ ᱟᱹᱨᱡᱮᱱᱴᱤᱱᱟ ᱥᱤᱛᱩᱝ ᱚᱠᱴᱚ"}, {"Atlantic", "ᱮᱴᱞᱟᱱᱴᱤᱠ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Cuba", "ᱠᱭᱩᱵᱟ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Europe_Central", "ᱥᱮᱱᱴᱨᱟᱞ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱥᱟᱢᱟᱨ ᱚᱠᱛᱚ"}, {"Europe_Eastern", "ᱤᱥᱴᱟᱨᱱ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱥᱟᱢᱟᱨ ᱚᱠᱛᱚ"}, {"Europe_Western", "ᱣᱮᱥᱴᱟᱨᱱ ᱩᱨᱚᱯᱤᱭᱟᱱ ᱥᱟᱢᱟᱨ ᱚᱠᱛᱚ"}, {"Greenland_Eastern", "ᱥᱟᱢᱟᱝ ᱜᱽᱨᱤᱱᱞᱮᱱᱰ ᱥᱤᱛᱩᱝ ᱚᱠᱛᱚ"}, {"Greenland_Western", "ᱯᱟᱪᱮ ᱜᱽᱨᱤᱱᱞᱮᱱᱰ ᱥᱤᱛᱩᱝ ᱚᱠᱛᱚ"}, {"Hawaii_Aleutian", "ᱦᱟᱣᱟᱭᱤᱼᱟᱞᱮᱣᱴᱤᱭᱟᱱ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Mexico_Pacific", "ᱢᱮᱠᱥᱤᱠᱟᱱ ᱯᱨᱚᱥᱟᱱᱛ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Newfoundland", "ᱱᱤᱭᱩᱯᱷᱟᱩᱱᱰᱞᱮᱸᱰ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}, {"Pierre_Miquelon", "ᱮᱥ ᱴᱤ ᱯᱤᱭᱮᱨᱮ ᱟᱨ ᱢᱤᱠᱮᱞᱚᱱ ᱥᱤᱧᱟᱜ ᱚᱠᱛᱚ"}},
	},
	{
		{{"Acre", "Ora istandard de Acre"}, {"Afghanistan", "Ora de s’Afghànistan"}, {"Africa_Central", "Ora de s’Àfrica tzentrale"}, {"Africa_Eastern", "Ora de s’Àfrica orientale"}, {"Africa_Southern", "Ora istandard de s’Àfrica meridionale"}, {"Africa_Western", "Ora de s’Àfrica otzidentale"}, {"Alaska", "Ora istandard de s’Alaska"}, {"Almaty", "Ora istandard de Almaty"}, {"Amazon", "Ora istandard de s’Amatzònia"}, {"America_Central", "Ora istandard tzentrale USA"}, {"America_Eastern", "Ora istandard orientale USA"}, {"America_Mountain", "Ora istandard Montes Pedrosos USA"}, {"America_Pacific", "Ora istandard de su Patzìficu USA"}, {"Anadyr", "Ora istandard de Anadyr"}, {"Apia", "Ora istandard de Apia"}, {"Aqtau", "Ora istandard de Aktau"}, {"Aqtobe", "Ora istandard de Aktobe"}, {"Arabian", "Ora istandard àraba"}, {"Argentina", "Ora istandard de s’Argentina"}, {"Argentina_Western", "Ora istandard de s’Argentina otzidentale"}, {"Armenia", "Ora istandard de s’Armènia"}, {"Atlantic", "Ora istandard de s’Atlànticu"}, {"Australia_Central", "Ora istandard de s’Austràlia tzentrale"}, {"Australia_CentralWestern", "Ora istandard de s’Austràlia tzentru-otzidentale"}, {"Australia_Eastern", "Ora istandard de s’Austràlia orientale"}, {"Australia_Western", "Ora istandard de s’Austràlia otzidentale"}, {"Azerbaijan", "Ora istandard de s’Azerbaigiàn"}, {"Azores", "Ora istandard de sas Azorras"}, {"Bangladesh", "Ora istandard de su Bangladesh"}, {"Bhutan", "Ora de su Bhutàn"}, {"Bolivia", "Ora de sa Bolìvia"}, {"Brasilia", "Ora istandard de Brasìlia"}, {"Brunei", "Ora de su Brunei"}, {"Cape_Verde", "Ora istandard de su Cabu Birde"}, {"Casey", "Ora de Casey"}, {"Chamorro", "Ora istandard de Chamorro"}, {"Chatham", "Ora istandard de sas Chatham"}, {"Chile", "Ora istandard de su Tzile"}, {"China", "Ora istandard de sa Tzina"}, {"Christmas", "Ora de s’Ìsula de sa Natividade"}, {"Cocos", "Ora de sas Ìsulas Cocos"}, {"Colombia", "Ora istandard de sa Colòmbia"}, {"Cook", "Ora istandard de sas Ìsulas Cook"}, {"Cuba", "Ora istandard de Cuba"}, {"Davis", "Ora de Davis"}, {"DumontDUrville", "Ora de Dumont-d’Urville"}, {"East_Timor", "Ora de su Timor Est"}, {"Easter", "Ora istandard de s’Ìsula de Pasca"}, {"Ecuador", "Ora de s’Ecuador"}, {"Europe_Central", "Ora istandard de s’Europa tzentrale"}, {"Europe_Eastern", "Ora istandard de s’Europa orientale"}, {"Europe_Further_Eastern", "Ora de s’estremu oriente europeu (Kaliningrad)"}, {"Europe_Western", "Ora istandard de s’Europa otzidentale"}, {"Falkland", "Ora istandard de sas Ìsulas Falkland"}, {"Fiji", "Ora istandard de sas Fiji"}, {"French_Guiana", "Ora de sa Guiana Frantzesa"}, {"French_Southern", "Ora de sa Terras australes e antàrticas frantzesas"}, {"GMT", "Ora de su meridianu de Greenwich"}, {"Galapagos", "Ora de sas Galàpagos"}, {"Gambier", "Ora de Gambier"}, {"Georgia", "Ora istandard de sa Geòrgia"}, {"Gilbert_Islands", "Ora de sas Ìsulas Gilbert"}, {"Greenland", "Ora istandard de sa Groenlàndia"}, {"Greenland_Eastern", "Ora istandard de sa Groenlàndia orientale"}, {"Greenland_Western", "Ora istandard de sa Groenlàndia otzidentale"}, {"Guam", "Ora istandard de Guàm"}, {"Gulf", "Ora istandard de su Gulfu"}, {"Guyana", "Ora de sa Guyana"}, {"Hawaii", "Ora istandard de sas ìsulas Hawaii-Aleutinas"}, {"Hawaii_Aleutian", "Ora istandard de sas ìsulas Hawaii-Aleutinas"}, {"Hong_Kong", "Ora istandard de Hong Kong"}, {"Hovd", "Ora istandard de Hovd"}, {"India", "Ora istandard de s’Ìndia"}, {"Indian_Ocean", "Ora de s’Otzèanu Indianu"}, {"Indochina", "Ora de s’Indotzina"}, {"Indonesia_Central", "Ora de s’Indonèsia tzentrale"}, {"Indonesia_Eastern", "Ora de s’Indonèsia orientale"}, {"Indonesia_Western", "Ora de s’Indonèsia otzidentale"}, {"Iran", "Ora istandard de s’Iràn"}, {"Irkutsk", "Ora istandard de Irkutsk"}, {"Israel", "Ora istandard de Israele"}, {"Japan", "Ora istandard de su Giapone"}, {"Kamchatka", "Ora istandard de Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Ora de su Kazàkistan"}, {"Kazakhstan_Eastern", "Ora de su Kazàkistan orientale"}, {"Kazakhstan_Western", "Ora de su Kazàkistan otzidentale"}, {"Korea", "Ora istandard coreana"}, {"Kosrae", "Ora de Kosrae"}, {"Krasnoyarsk", "Ora istandard de Krasnoyarsk"}, {"Kyrgystan", "Ora de su Kirghìzistan"}, {"Lanka", "Ora de Lanka"}, {"Line_Islands", "Ora de sas Ìsulas de sa Lìnia"}, {"Lord_Howe", "Ora istandard de Lord Howe"}, {"Macau", "Ora istandard de Macao"}, {"Magadan", "Ora istandard de Magadan"}, {"Malaysia", "Ora de sa Malèsia"}, {"Maldives", "Ora de sas Maldivas"}, {"Marquesas", "Ora de sas Marchesas"}, {"Marshall_Islands", "Ora de sas Ìsulas Marshall"}, {"Mauritius", "Ora istandard de sas Maurìtzius"}, {"Mawson", "Ora de Mawson"}, {"Mexico_Pacific", "Ora istandard de su Patzìficu (Mèssicu)"}, {"Mongolia", "Ora istandard de Ulàn Bator"}, {"Moscow", "Ora istandard de Mosca"}, {"Myanmar", "Ora de su Myanmàr"}, {"Nauru", "Ora de Nauru"}, {"Nepal", "Ora de su Nepal"}, {"New_Caledonia", "Ora istandard de sa Caledònia Noa"}, {"New_Zealand", "Ora istandard de sa Zelanda Noa"}, {"Newfoundland", "Ora istandard de Terranova"}, {"Niue", "Ora de Niue"}, {"Norfolk", "Ora istandard de s’Ìsula Norfolk"}, {"Noronha", "Ora istandard de su Fernando de Noronha"}, {"North_Mariana", "Ora de sas Ìsulas Mariannas Setentrionales"}, {"Novosibirsk", "Ora istandard de Novosibirsk"}, {"Omsk", "Ora istandard de Omsk"}, {"Pakistan", "Ora istandard de su Pàkistan"}, {"Palau", "Ora de Palau"}, {"Papua_New_Guinea", "Ora de sa Pàpua Guinea Noa"}, {"Paraguay", "Ora istandard de su Paraguay"}, {"Peru", "Ora istandard de su Perù"}, {"Philippines", "Ora istandard de sas Filipinas"}, {"Phoenix_Islands", "Ora de sas Ìsulas de sa Fenìtzie"}, {"Pierre_Miquelon", "Ora istandard de Saint-Pierre e Miquelon"}, {"Pitcairn", "Ora de sas Pitcairn"}, {"Ponape", "Ora de Pohnpei"}, {"Pyongyang", "Ora de Pyongyang"}, {"Qyzylorda", "Ora istandard de Qyzylorda"}, {"Reunion", "Ora de sa Reunione"}, {"Rothera", "Ora de Rothera"}, {"Sakhalin", "Ora istandard de Sakhalin"}, {"Samara", "Ora istandard de Samara"}, {"Samoa", "Ora istandard de sas Samoa"}, {"Seychelles", "Ora de sas Seychelles"}, {"Singapore", "Ora de Singapore"}, {"Solomon", "Ora de sas Ìsulas Salomone"}, {"South_Georgia", "Ora de sa Geòrgia de su Sud"}, {"Suriname", "Ora de su Suriname"}, {"Syowa", "Ora de Syowa"}, {"Tahiti", "Ora de Tahiti"}, {"Taipei", "Ora istandard de Taipei"}, {"Tajikistan", "Ora de su Tagìkistan"}, {"Tokelau", "Ora de su Tokelau"}, {"Tonga", "Ora istandard de su Tonga"}, {"Truk", "Ora de su Chuuk"}, {"Turkmenistan", "Ora istandard de su Turkmènistan"}, {"Tuvalu", "Ora de su Tuvalu"}, {"Uruguay", "Ora istandard de s’Uruguay"}, {"Uzbekistan", "Ora istandard de s’Uzbèkistan"}, {"Vanuatu", "Ora istandard de su Vanuatu"}, {"Venezuela", "Ora de su Venetzuela"}, {"Vladivostok", "Ora istandard de Vladivostok"}, {"Volgograd", "Ora istandard de Volgograd"}, {"Vostok", "Ora de Vostok"}, {"Wake", "Ora de sas Ìsulas Wake"}, {"Wallis", "Ora de Wallis e Futuna"}, {"Yakutsk", "Ora istandard de Yakutsk"}, {"Yekaterinburg", "Ora istandard de Yekaterinburg"}, {"Yukon", "Ora de su Yukon"}},
		{{"Acre", "Ora legale de Acre"}, {"Alaska", "Ora legale de s’Alaska"}, {"Almaty", "Ora legale de Almaty"}, {"Amazon", "Ora legale de s’Amatzònia"}, {"America_Central", "Ora legale tzentrale USA"}, {"America_Eastern", "Ora legale orientale USA"}, {"America_Mountain", "Ora legale Montes Pedrosos USA"}, {"America_Pacific", "Ora legale de su Patzìficu USA"}, {"Anadyr", "Ora legale de Anadyr"}, {"Apia", "Ora legale de Apia"}, {"Aqtau", "Ora legale de Aktau"}, {"Aqtobe", "Ora legale de Aktobe"}, {"Arabian", "Ora legale àraba"}, {"Argentina", "Ora legale de s’Argentina"}, {"Argentina_Western", "Ora legale de s’Argentina otzidentale"}, {"Armenia", "Ora legale de s’Armènia"}, {"Atlantic", "Ora legale de s’Atlànticu"}, {"Australia_Central", "Ora legale de s’Austràlia tzentrale"}, {"Australia_CentralWestern", "Ora legale de s’Austràlia tzentru-otzidentale"}, {"Australia_Eastern", "Ora legale de s’Austràlia orientale"}, {"Australia_Western", "Ora legale de s’Austràlia otzidentale"}, {"Azerbaijan", "Ora legale de s’Azerbaigiàn"}, {"Azores", "Ora legale de sas Azorras"}, {"Bangladesh", "Ora legale de su Bangladesh"}, {"Brasilia", "Ora legale de Brasìlia"}, {"Cape_Verde", "Ora legale de su Cabu Birde"}, {"Chatham", "Ora legale de sas Chatham"}, {"Chile", "Ora legale de su Tzile"}, {"China", "Ora legale de sa Tzina"}, {"Colombia", "Ora legale de sa Colòmbia"}, {"Cook", "Ora legale de sas Ìsulas Cook"}, {"Cuba", "Ora legale de Cuba"}, {"Easter", "Ora legale de s’Ìsula de Pasca"}, {"Europe_Central", "Ora legale de s’Europa tzentrale"}, {"Europe_Eastern", "Ora legale de s’Europa orientale"}, {"Europe_Western", "Ora legale de s’Europa otzidentale"}, {"Falkland", "Ora legale de sas Ìsulas Falkland"}, {"Fiji", "Ora legale de sas Fiji"}, {"Georgia", "Ora legale de sa Geòrgia"}, {"Greenland", "Ora legale de sa Groenlàndia"}, {"Greenland_Eastern", "Ora legale de sa Groenlàndia orientale"}, {"Greenland_Western", "Ora legale de sa Groenlàndia otzidentale"}, {"Hawaii_Aleutian", "Ora legale de sas ìsulas Hawaii-Aleutinas"}, {"Hong_Kong", "Ora legale de Hong Kong"}, {"Hovd", "Ora legale de Hovd"}, {"Iran", "Ora legale de s’Iràn"}, {"Irkutsk", "Ora legale de Irkutsk"}, {"Israel", "Ora legale de Israele"}, {"Japan", "Ora legale de su Giapone"}, {"Kamchatka", "Ora legale de Petropavlovsk-Kamchatski"}, {"Korea", "Ora legale coreana"}, {"Krasnoyarsk", "Ora legale de Krasnoyarsk"}, {"Lord_Howe", "Ora legale de Lord Howe"}, {"Macau", "Ora legale de Macao"}, {"Magadan", "Ora legale de Magadan"}, {"Mauritius", "Ora legale de sas Maurìtzius"}, {"Mexico_Pacific", "Ora legale de su Patzìficu (Mèssicu)"}, {"Mongolia", "Ora legale de Ulàn Bator"}, {"Moscow", "Ora legale de Mosca"}, {"New_Caledonia", "Ora legale de sa Caledònia Noa"}, {"New_Zealand", "Ora legale de sa Zelanda Noa"}, {"Newfoundland", "Ora legale de Terranova"}, {"Norfolk", "Ora legale de s’Ìsula Norfolk"}, {"Noronha", "Ora legale de su Fernando de Noronha"}, {"Novosibirsk", "Ora legale de Novosibirsk"}, {"Omsk", "Ora legale de Omsk"}, {"Pakistan", "Ora legale de su Pàkistan"}, {"Paraguay", "Ora legale de su Paraguay"}, {"Peru", "Ora legale de su Perù"}, {"Philippines", "Ora legale de sas Filipinas"}, {"Pierre_Miquelon", "Ora legale de Saint-Pierre e Miquelon"}, {"Qyzylorda", "Ora legale de Qyzylorda"}, {"Sakhalin", "Ora legale de Sakhalin"}, {"Samara", "Ora legale de Samara"}, {"Samoa", "Ora legale de sas Samoa"}, {"Taipei", "Ora legale de Taipei"}, {"Tonga", "Ora legale de su Tonga"}, {"Turkmenistan", "Ora legale de su Turkmènistan"}, {"Uruguay", "Ora legale de s’Uruguay"}, {"Uzbekistan", "Ora legale de s’Uzbèkistan"}, {"Vanuatu", "Ora legale de su Vanuatu"}, {"Vladivostok", "Ora legale de Vladivostok"}, {"Volgograd", "Ora legale de Volgograd"}, {"Yakutsk", "Ora legale de Yakutsk"}, {"Yekaterinburg", "Ora legale de Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "Ura di l’Afghànistan"}, {"Africa_Central", "Ura di l’Àfrica Cintrali"}, {"Africa_Eastern", "Ura di l’Àfrica di Livanti"}, {"Africa_Southern", "Ura Nurmali di l’Àfrica di Sciroccu"}, {"Africa_Western", "Ura di l’Àfrica di Punenti"}, {"Alaska", "Ura sulari di l’Alaska"}, {"Amazon", "Ura sulari di l’Amazzonia"}, {"America_Central", "Ura sulari cintrali"}, {"America_Eastern", "Ura sulari livantina"}, {"America_Mountain", "Ura sulari dî muntagni"}, {"America_Pacific", "Ura sulari dû Pacìficu"}, {"Apia", "Ura sulari di Samoa"}, {"Arabian", "Ura sulari Àrabba"}, {"Argentina", "Ura sulari di l’Argintina"}, {"Argentina_Western", "Ura sulari di l’Argintina di punenti"}, {"Armenia", "Ura sulari di l’Armenia"}, {"Atlantic", "Ura sulari di l’Atlànticu"}, {"Australia_Central", "Ura sulari di l’Australia cintrali"}, {"Australia_CentralWestern", "Ura sulari di l’Australia cintrali di punenti"}, {"Australia_Eastern", "Ura sulari di l’Australia di livanti"}, {"Australia_Western", "Ura sulari di l’Australia di punenti"}, {"Azerbaijan", "Ura sulari di l’Azerbaijan"}, {"Azores", "Ura sulari di l’Azzorri"}, {"Bangladesh", "Ura sulari dû Bàngladesh"}, {"Bhutan", "Ura dû Bhutan"}, {"Bolivia", "Ura dâ Bulivia"}, {"Brasilia", "Ura sulari di Brasilia"}, {"Brunei", "Ura dû Brunei"}, {"Cape_Verde", "Ura sulari di Capu Virdi"}, {"Chamorro", "Ura di Chamorro"}, {"Chatham", "Ura sulari di Chatham"}, {"Chile", "Ura sulari dû Cili"}, {"China", "Ura sulari dâ Cina"}, {"Christmas", "Ura di l’Ìsula di Natali"}, {"Cocos", "Ura di l’Ìsuli Cocos"}, {"Colombia", "Ura sulari dâ Culommia"}, {"Cook", "Ura sulari di l’Ìsuli Cook"}, {"Cuba", "Ura sulari di Cubba"}, {"Davis", "Ura di Davis"}, {"DumontDUrville", "Ura di Dumont d’Urville"}, {"East_Timor", "Ura di Timor di Livanti"}, {"Easter", "Ura sulari di l’Ìsula di Pasca"}, {"Ecuador", "Ura di l’Ècuador"}, {"Europe_Central", "Ura sulari Cintrali Eurupea"}, {"Europe_Eastern", "Ura sulari di l’Europa di Livanti"}, {"Europe_Further_Eastern", "Ura di l’Europa cchiù a Livanti"}, {"Europe_Western", "Ura sulari di l’Europa di Punenti"}, {"Falkland", "Ura sulari di l’Ìsuli Falkland"}, {"Fiji", "Ura sulari dî Figi"}, {"French_Guiana", "Ura dâ Guiana Francisi"}, {"French_Southern", "Ura Francisi di Sciroccu e di l’Antàrtidi"}, {"GMT", "Ura Minzana di Greenwich"}, {"Galapagos", "Ura dî Galàpagos"}, {"Gambier", "Ura di Gambier"}, {"Georgia", "Ura sulari dâ Giorgia"}, {"Gilbert_Islands", "Ura di l’Ìsuli Gilbert"}, {"Greenland", "Ura sulari dâ Gruillannia"}, {"Greenland_Eastern", "Ura sulari dâ Gruillannia livantina"}, {"Greenland_Western", "Ura sulari dâ Gruillannia punintina"}, {"Gulf", "Ura Nurmali dû Gurfu"}, {"Guyana", "Ura dâ Guiana"}, {"Hawaii", "Ura sulari di l’Hawaai"}, {"Hawaii_Aleutian", "Ura sulari di l’Hawaai"}, {"Hong_Kong", "Ura sulari di Hong Kong"}, {"Hovd", "Ura sulari di Khovd"}, {"India", "Ura sulari di l’Ìnnia"}, {"Indian_Ocean", "Ura di l’Ucianu Innianu"}, {"Indochina", "Ura di l’Innucina"}, {"Indonesia_Central", "Ura di l’Innunesia cintrali"}, {"Indonesia_Eastern", "Ura di l’Innunesia di livanti"}, {"Indonesia_Western", "Ura di l’Innunesia di punenti"}, {"Iran", "Ura sulari di l’Iran"}, {"Irkutsk", "Ura sulari di Irtkutsk"}, {"Israel", "Ura sulari di Isdraeli"}, {"Japan", "Ura sulari dû Giappuni"}, {"Kazakhstan", "Ura dû Kazzàkistan"}, {"Kazakhstan_Eastern", "Ura dû Kazzàkistan di Livanti"}, {"Kazakhstan_Western", "Ura dû Kazzàkistan di Punenti"}, {"Korea", "Ura sulari dâ Curìa"}, {"Kosrae", "Ura di Kosrae"}, {"Krasnoyarsk", "Ura sulari di Krasnoyarsk"}, {"Kyrgystan", "Ura dû Kirghìzzistan"}, {"Line_Islands", "Ura di l’Ìsuli Line"}, {"Lord_Howe", "Ura sulari di Lord Howe"}, {"Magadan", "Ura sulari di Magdan"}, {"Malaysia", "Ura dâ Malisia"}, {"Maldives", "Ura dî Mardivi"}, {"Marquesas", "Ura dî Marchesi"}, {"Marshall_Islands", "Ura di l’Ìsuli Marshall"}, {"Mauritius", "Ura sulari di Mauritius"}, {"Mawson", "Ura di Mawson"}, {"Mexico_Pacific", "Ura sulari dû Mèssicu Pacìficu"}, {"Mongolia", "Ura sulari di Ulaanbaatar"}, {"Moscow", "Ura sulari di Mosca"}, {"Myanmar", "Ura di Myanmar"}, {"Nauru", "Ura di Nauru"}, {"Nepal", "Ura dû Nepal"}, {"New_Caledonia", "Ura sulari dâ Nova Calidonia"}, {"New_Zealand", "Ura sulari dâ Nova Zilannia"}, {"Newfoundland", "Ura sulari di Tirranova"}, {"Niue", "Ura di Niue"}, {"Norfolk", "Ura sulari di l’Ìsula Norfolk"}, {"Noronha", "Ura sulari di Fernando di Noronha"}, {"Novosibirsk", "Ura sulari di Novosibirsk"}, {"Omsk", "Ura sulari di Omsk"}, {"Pakistan", "Ura sulari dû Pàkistan"}, {"Palau", "Ura di Palau"}, {"Papua_New_Guinea", "Ura dâ Papua Nova Guinìa"}, {"Paraguay", "Ura sulari dû Paraguay"}, {"Peru", "Ura sulari dû Pirù"}, {"Philippines", "Ura sulari dî Filippini"}, {"Phoenix_Islands", "Ura di l’Ìsuli Phoenix"}, {"Pierre_Miquelon", "Ura sulari di S. Pierre e Miquelon"}, {"Pitcairn", "Ura di Pitcairn"}, {"Ponape", "Ura di Pohnpei"}, {"Pyongyang", "Ura dâ Curia di Tramuntana"}, {"Reunion", "Ura di Réunion"}, {"Rothera", "Ura di Rothera"}, {"Sakhalin", "Ura sulari di Sakhalin"}, {"Samoa", "Ura sulari dî Samoa miricani"}, {"Seychelles", "Ura dî Seychelles"}, {"Singapore", "Ura di Singapuri"}, {"Solomon", "Ura di l’Ìsuli Salumuni"}, {"South_Georgia", "Ura dâ Giorgia di sciroccu"}, {"Suriname", "Ura dû Surinami"}, {"Syowa", "Ura di Syowa"}, {"Tahiti", "Ura di Tahiti"}, {"Taipei", "Ura sulari di Taiwan"}, {"Tajikistan", "Ura dû Taggìkistan"}, {"Tokelau", "Ura di Tukilau"}, {"Tonga", "Ura sulari di Tonga"}, {"Truk", "Ura di Chuuk"}, {"Turkmenistan", "Ura sulari dû Turkmènistan"}, {"Tuvalu", "Ura di Tuvalu"}, {"Uruguay", "Ura sulari di l’Uruguay"}, {"Uzbekistan", "Ura sulari di l’Uzbèkistan"}, {"Vanuatu", "Ura sulari di Vanuatu"}, {"Venezuela", "Ura dû Vinizzuela"}, {"Vladivostok", "Ura sulari di Vladìvustok"}, {"Volgograd", "Ura sulari di Vòlgugrad"}, {"Vostok", "Ura di Vostok"}, {"Wake", "Ura di l’Ìsula Wake"}, {"Wallis", "Ura di Wallis e Futuna"}, {"Yakutsk", "Ura sulari di Yakutsk"}, {"Yekaterinburg", "Ura sulari di Yekatirimmurgu"}, {"Yukon", "Ura dû Yukon"}},
		{{"Alaska", "Ura ligali di l’Alaska"}, {"Amazon", "Ura ligali di l’Amazzonia"}, {"America_Central", "Ura ligali cintrali"}, {"America_Eastern", "Ura ligali livantina"}, {"America_Mountain", "Ura ligali dî muntagni"}, {"America_Pacific", "Ura ligali dû Pacìficu"}, {"Apia", "Ura ligali di Samoa"}, {"Arabian", "Ura ligali Àrabba"}, {"Argentina", "Ura ligali di l’Argintina"}, {"Argentina_Western", "Ura ligali di l’Argintina di punenti"}, {"Armenia", "Ura ligali di l’Armenia"}, {"Atlantic", "Ura ligali di l’Atlànticu"}, {"Australia_Central", "Ura ligali di l’Australia cintrali"}, {"Australia_CentralWestern", "Ura ligali di l’Australia cintrali di punenti"}, {"Australia_Eastern", "Ura ligali di l’Australia di livanti"}, {"Australia_Western", "Ura ligali di l’Australia di punenti"}, {"Azerbaijan", "Ura ligali di l’Azerbaijan"}, {"Azores", "Ura ligali di l’Azzorri"}, {"Bangladesh", "Ura ligali dû Bàngladesh"}, {"Brasilia", "Ura ligali di Brasilia"}, {"Cape_Verde", "Ura ligali di Capu Virdi"}, {"Chatham", "Ura ligali di Chatham"}, {"Chile", "Ura ligali dû Cili"}, {"China", "Ura ligali dâ Cina"}, {"Colombia", "Ura ligali dâ Culommia"}, {"Cook", "Ura ligali di l’Ìsuli Cook"}, {"Cuba", "Ura ligali di Cubba"}, {"Easter", "Ura ligali di l’Ìsula di Pasca"}, {"Europe_Central", "Ura ligali Cintrali Eurupea"}, {"Europe_Eastern", "Ura ligali di l’Europa di Livanti"}, {"Europe_Western", "Ura ligali di l’Europa di Punenti"}, {"Falkland", "Ura ligali di l’Ìsuli Falkland"}, {"Fiji", "Ura ligali dî Figi"}, {"Georgia", "Ura ligali dâ Giorgia"}, {"Greenland", "Ura ligali dâ Gruillannia"}, {"Greenland_Eastern", "Ura livantina dâ Gruillannia livantina"}, {"Greenland_Western", "Ura ligali dâ Gruillannia punintina"}, {"Hawaii_Aleutian", "Ura ligali di l’Hawaai"}, {"Hong_Kong", "Ura ligali di Hong Kong"}, {"Hovd", "Ura ligali di Khovd"}, {"Iran", "Ura ligali di l’Iran"}, {"Irkutsk", "Ura ligali di Irtkutsk"}, {"Israel", "Ura ligali di Isdraeli"}, {"Japan", "Ura ligali dû Giappuni"}, {"Korea", "Ura ligali dâ Curìa"}, {"Krasnoyarsk", "Ura ligali di Krasnoyarsk"}, {"Lord_Howe", "Ura ligali di Lord Howe"}, {"Magadan", "Ura ligali di Magdan"}, {"Mauritius", "Ura ligali di Mauritius"}, {"Mexico_Pacific", "Ura ligali dû Mèssicu Pacìficu"}, {"Mongolia", "Ura ligali di Ulaanbaatar"}, {"Moscow", "Ura ligali di Mosca"}, {"New_Caledonia", "Ura ligali dâ Nova Calidonia"}, {"New_Zealand", "Ura ligali dâ Nova Zilannia"}, {"Newfoundland", "Ura ligali di Tirranova"}, {"Norfolk", "Ura ligali di l’Ìsula Norfolk"}, {"Noronha", "Ura ligali di Fernando di Noronha"}, {"Novosibirsk", "Ura ligali di Novosibirsk"}, {"Omsk", "Ura ligali di Omsk"}, {"Pakistan", "Ura ligali dû Pàkistan"}, {"Paraguay", "Ura ligali dû Paraguay"}, {"Peru", "Ura ligali dû Pirù"}, {"Philippines", "Ura ligali dî Filippini"}, {"Pierre_Miquelon", "Ura ligali di S. Pierre e Miquelon"}, {"Sakhalin", "Ura ligali di Sakhalin"}, {"Samoa", "Ura ligali dî Samoa miricani"}, {"Taipei", "Ura ligali di Taiwan"}, {"Tonga", "Ura ligali di Tonga"}, {"Turkmenistan", "Ura ligali dû Turkmènistan"}, {"Uruguay", "Ura ligali di l’Uruguay"}, {"Uzbekistan", "Ura ligali di l’Uzbèkistan"}, {"Vanuatu", "Ura ligali di Vanuatu"}, {"Vladivostok", "Ura ligali di Vladìvustok"}, {"Volgograd", "Ura ligali di Vòlgugrad"}, {"Yakutsk", "Ura ligali di Yakutsk"}, {"Yekaterinburg", "Ura ligali di Yekatirimmurgu"}},
	},
	{
		{{"Afghanistan", "افغانستان جو وقت"}, {"Africa_Central", "مرڪزي آفريقا جو وقت"}, {"Africa_Eastern", "اوڀر آفريڪا جو وقت"}, {"Africa_Southern", "ڏکڻ آفريڪا جو معياري وقت"}, {"Africa_Western", "اولهه آفريقا جو وقت"}, {"Alaska", "الاسڪا جو معياري وقت"}, {"Amazon", "ايميزون جو معياري وقت"}, {"America_Central", "مرڪزي معياري وقت"}, {"America_Eastern", "مشرقي معياري وقت"}, {"America_Mountain", "پهاڙي معياري وقت"}, {"America_Pacific", "پيسيفڪ معياري وقت"}, {"Apia", "اپيا جو معياري وقت"}, {"Arabian", "عربين جو معياري وقت"}, {"Argentina", "ارجنٽائن معياري وقت"}, {"Argentina_Western", "اولهه ارجنٽينا جو معياري وقت"}, {"Armenia", "آرمينيا جو معياري وقت"}, {"Atlantic", "ايٽلانٽڪ جو معياري وقت"}, {"Australia_Central", "آسٽريليا جو مرڪزي معياري وقت"}, {"Australia_CentralWestern", "آسٽريليا جو مرڪزي مغربي معياري وقت"}, {"Australia_Eastern", "آسٽريليا جو مشرقي معياري وقت"}, {"Australia_Western", "آسٽريليا جو مغربي معياري وقت"}, {"Azerbaijan", "آذربائيجان جو معياري وقت"}, {"Azores", "ازورز جو معياري وقت"}, {"Bangladesh", "بنگلاديش جو معياري وقت"}, {"Bhutan", "ڀوٽان جو وقت"}, {"Bolivia", "بولويائي وقت"}, {"Brasilia", "براسيليا جو معياري وقت"}, {"Brunei", "برونائي دارالسلام جو وقت"}, {"Cape_Verde", "ڪيپ ورڊ جو معياري وقت"}, {"Chamorro", "چمورو جو معياري وقت"}, {"Chatham", "چئٿم جو معياري وقت"}, {"Chile", "چلي جو معياري وقت"}, {"China", "چائنا جو معياري وقت"}, {"Christmas", "ڪرسمس آئي لينڊ جو وقت"}, {"Cocos", "ڪوڪوس آئي لينڊ جو وقت"}, {"Colombia", "ڪولمبيا جو معياري وقت"}, {"Cook", "ڪوڪ آئي لينڊ جو معياري وقت"}, {"Cuba", "ڪيوبا جو معياري وقت"}, {"Davis", "ڊيوس جو وقت"}, {"DumontDUrville", "ڊومانٽ درويئل جو وقت"}, {"East_Timor", "اوڀر تيمور جو وقت"}, {"Easter", "ايسٽر آئي لينڊ جو معياري وقت"}, {"Ecuador", "ايڪواڊور جو وقت"}, {"Europe_Central", "مرڪزي يورپي معياري وقت"}, {"Europe_Eastern", "مشرقي يورپي معياري وقت"}, {"Europe_Further_Eastern", "وڌيڪ مشرقي يورپي وقت"}, {"Europe_Western", "اولهه يورپي معياري وقت"}, {"Falkland", "فاڪ لينڊ آئي لينڊ جو معياري وقت"}, {"Fiji", "فجي جو معياري وقت"}, {"French_Guiana", "فرانسيسي گيانا جو وقت"}, {"French_Southern", "فرانسيسي ڏاکڻيو ۽ انٽارڪٽڪ وقت"}, {"GMT", "گرين وچ مين ٽائيم"}, {"Galapagos", "گالاپاگوز جو وقت"}, {"Gambier", "گيمبيئر جو وقت"}, {"Georgia", "جارجيا جو معياري وقت"}, {"Gilbert_Islands", "گلبرٽ آئي لينڊ جو وقت"}, {"Greenland_Eastern", "مشرقي گرين لينڊ جو معياري وقت"}, {"Greenland_Western", "مغربي گرين لينڊ جو معياري وقت"}, {"Gulf", "خليجي معياري وقت"}, {"Guyana", "گيانائي وقت"}, {"Hawaii", "هوائي اليوٽين جو معياري وقت"}, {"Hawaii_Aleutian", "هوائي اليوٽين جو معياري وقت"}, {"Hong_Kong", "هانگ ڪانگ جو معياري وقت"}, {"Hovd", "هووڊ جو معياري وقت"}, {"India", "ڀارت جو معياري وقت"}, {"Indian_Ocean", "هند سمنڊ جو وقت"}, {"Indochina", "انڊو چائنا جو وقت"}, {"Indonesia_Central", "مرڪزي انڊونيشيا جو وقت"}, {"Indonesia_Eastern", "اوڀر انڊونيشيا جو وقت"}, {"Indonesia_Western", "اولهه انڊونيشيا جو وقت"}, {"Iran", "ايران جو معياري وقت"}, {"Irkutsk", "ارڪتسڪ جو معياري وقت"}, {"Israel", "اسرائيل جو معياري وقت"}, {"Japan", "جاپان جو معياري وقت"}, {"Kazakhstan", "قزاقستان وقت"}, {"Kazakhstan_Eastern", "اوڀر قزاقستان جو وقت"}, {"Kazakhstan_Western", "اولهه قازقستان جو وقت"}, {"Korea", "ڪوريا جو معياري وقت"}, {"Kosrae", "ڪوسرائي جو وقت"}, {"Krasnoyarsk", "ڪریسنویارسڪ جو معياري وقت"}, {"Kyrgystan", "ڪرغزستان جو وقت"}, {"Line_Islands", "لائن آئي لينڊ جو وقت"}, {"Lord_Howe", "لورڊ هووي جو معياري وقت"}, {"Magadan", "مگادان جو معياري وقت"}, {"Malaysia", "ملائيشيا جو وقت"}, {"Maldives", "مالديپ جو وقت"}, {"Marquesas", "مرڪيوسس جو وقت"}, {"Marshall_Islands", "مارشل آئي لينڊ جو وقت"}, {"Mauritius", "موريشيس جو معياري وقت"}, {"Mawson", "مائوسن جو وقت"}, {"Mexico_Pacific", "ميڪسيڪن پيسيفڪ جو معياري وقت"}, {"Mongolia", "اولان باتر جو معياري وقت"}, {"Moscow", "ماسڪو جو معياري وقت"}, {"Myanmar", "ميانمار جو وقت"}, {"Nauru", "نائورو جو وقت"}, {"Nepal", "نيپال جو وقت"}, {"New_Caledonia", "نيو ڪيليڊونيا جو معياري وقت"}, {"New_Zealand", "نيوزيلينڊ جو معياري وقت"}, {"Newfoundland", "نيو فائونڊ لينڊ جو معياري وقت"}, {"Niue", "نيووي جو وقت"}, {"Norfolk", "نار فوڪ آئي لينڊ جو معياري وقت"}, {"Noronha", "فرنانڊو دي نورونها جو معياري وقت"}, {"Novosibirsk", "نوواسبئيرسڪ جو معياري وقت"}, {"Omsk", "اومسڪ جو معياري وقت"}, {"Pakistan", "پاڪستان جو معياري وقت"}, {"Palau", "پلائو جو وقت"}, {"Papua_New_Guinea", "پاپوا نيو گني جو وقت"}, {"Paraguay", "پيراگوئي جو معياري وقت"}, {"Peru", "پيرو جو معياري وقت"}, {"Philippines", "فلپائن جو معياري وقت"}, {"Phoenix_Islands", "فونيڪس آئي لينڊ جو وقت"}, {"Pierre_Miquelon", "سينٽ پيئر ۽ ميڪلون جو معياري وقت"}, {"Pitcairn", "پٽڪيرن جو وقت"}, {"Ponape", "پوناپي جو وقت"}, {"Pyongyang", "پيانگ يانگ جو وقت"}, {"Reunion", "ري يونين جو وقت"}, {"Rothera", "روٿيرا جو وقت"}, {"Sakhalin", "سخالين جو معياري وقت"}, {"Samoa", "ساموا جو معياري وقت"}, {"Seychelles", "شي شلز جو وقت"}, {"Singapore", "سنگاپور جو معياري وقت"}, {"Solomon", "سولومن آئي لينڊ جو وقت"}, {"South_Georgia", "ڏکڻ جارجيا جو وقت"}, {"Suriname", "سوري نام جو وقت"}, {"Syowa", "سائيوا جو وقت"}, {"Tahiti", "تاهيٽي جو وقت"}, {"Taipei", "تائپي جو معياري وقت"}, {"Tajikistan", "تاجڪستان جو وقت"}, {"Tokelau", "ٽوڪيلائو جو وقت"}, {"Tonga", "ٽونگا جو معياري وقت"}, {"Truk", "چيوڪ جو وقت"}, {"Turkmenistan", "ترڪمانستان جو معياري وقت"}, {"Tuvalu", "تووالو جو وقت"}, {"Uruguay", "يوروگائي جو معياري وقت"}, {"Uzbekistan", "ازبڪستان جو معياري وقت"}, {"Vanuatu", "وانواتو جو معياري وقت"}, {"Venezuela", "وينزويلا جو وقت"}, {"Vladivostok", "ولادووستوڪ جو معياري وقت"}, {"Volgograd", "وولگوگراد جو معياري وقت"}, {"Vostok", "ووسٽوڪ جو وقت"}, {"Wake", "ويڪ آئي لينڊ جو وقت"}, {"Wallis", "ويلس ۽ فتونا جو وقت"}, {"Yakutsk", "ياڪتسڪ جو معياري وقت"}, {"Yekaterinburg", "يڪاٽيرنبرگ جو معياري وقت"}, {"Yukon", "يڪون جو وقت"}},
		{{"Alaska", "الاسڪا جي ڏينهن جو وقت"}, {"Amazon", "ايميزون جي اونهاري جو وقت"}, {"America_Central", "مرڪزي ڏينهن جو وقت"}, {"America_Eastern", "مشرقي ڏينهن جو وقت"}, {"America_Mountain", "پهاڙي ڏينهن جو وقت"}, {"America_Pacific", "پيسيفڪ ڏينهن جو وقت"}, {"Apia", "اپيا جي ڏينهن جو وقت"}, {"Arabian", "عربين جي ڏينهن جو وقت"}, {"Argentina", "ارجنٽائن جي اونهاري جو وقت"}, {"Argentina_Western", "اولهه ارجنٽينا جي اونهاري جو وقت"}, {"Armenia", "آرمينيا جي اونهاري جو وقت"}, {"Atlantic", "ايٽلانٽڪ جي ڏينهن جو وقت"}, {"Australia_Central", "آسٽريليا جو مرڪزي ڏينهن جو وقت"}, {"Australia_CentralWestern", "آسٽريليا جو مرڪزي مغربي ڏينهن جو وقت"}, {"Australia_Eastern", "آسٽريليا جو مشرقي ڏينهن جو وقت"}, {"Australia_Western", "آسٽريليا جو مغربي ڏينهن جو وقت"}, {"Azerbaijan", "آذربائيجان جي اونهاري جو وقت"}, {"Azores", "ازورز جي اونهاري جو وقت"}, {"Bangladesh", "بنگلاديش جي اونهاري جو وقت"}, {"Brasilia", "براسيليا جي اونهاري جو وقت"}, {"Cape_Verde", "ڪيپ ورڊ جي اونهاري جو وقت"}, {"Chatham", "چئٿم جي ڏينهن جو وقت"}, {"Chile", "چلي جي اونهاري جو وقت"}, {"China", "چائنا جي ڏينهن جو وقت"}, {"Colombia", "ڪولمبيا جي اونهاري جو وقت"}, {"Cook", "ڪوڪ آئي لينڊ جي اڌ اونهاري جو وقت"}, {"Cuba", "ڪيوبا جي ڏينهن جو وقت"}, {"Easter", "ايسٽر آئي لينڊ جي اونهاري جو وقت"}, {"Europe_Central", "مرڪزي يورپي اونهاري جو وقت"}, {"Europe_Eastern", "مشرقي يورپي اونهاري جو وقت"}, {"Europe_Western", "اولهه يورپي ڏينهن جو وقت"}, {"Falkland", "فاڪ لينڊ آئي لينڊ جي اونهاري جو وقت"}, {"Fiji", "فجي جي اونهاري جو وقت"}, {"Georgia", "جارجيا جي اونهاري جو وقت"}, {"Greenland_Eastern", "مشرقي گرين لينڊ جي اونهاري جو وقت"}, {"Greenland_Western", "مغربي گرين لينڊ جي اونهاري جو وقت"}, {"Hawaii_Aleutian", "هوائي اليوٽين جي ڏينهن جو وقت"}, {"Hong_Kong", "هانگ ڪانگ جي اونهاري جو وقت"}, {"Hovd", "هووڊ جي اونهاري جو وقت"}, {"Iran", "ايران جي ڏينهن جو وقت"}, {"Irkutsk", "ارڪتسڪ جي ڏينهن جو وقت"}, {"Israel", "اسرائيل جي ڏينهن جو وقت"}, {"Japan", "جاپان جي ڏينهن جو وقت"}, {"Korea", "ڪوريا جي ڏينهن جو وقت"}, {"Krasnoyarsk", "ڪریسنویارسڪ جي ڏينهن جو وقت"}, {"Lord_Howe", "لورڊ هووي جي ڏينهن جو وقت"}, {"Magadan", "مگادان جي ڏينهن جي وقت"}, {"Mauritius", "موريشيس جي اونهاري جو وقت"}, {"Mexico_Pacific", "ميڪسيڪن پيسيفڪ جي ڏينهن جو وقت"}, {"Mongolia", "اولان باتر جي اونهاري جو وقت"}, {"Moscow", "ماسڪو جي ڏينهن جي وقت"}, {"New_Caledonia", "نيو ڪيليڊونيا جي اونهاري جو وقت"}, {"New_Zealand", "نيوزيلينڊ جي ڏينهن جو وقت"}, {"Newfoundland", "نيو فائونڊ لينڊ جي ڏينهن جو وقت"}, {"Norfolk", "نار فوڪ آئي لينڊ جي ڏينهن جو وقت"}, {"Noronha", "فرنانڊو دي نورونها جي اونهاري وقت"}, {"Novosibirsk", "نوواسبئيرسڪ جي ڏينهن جو وقت"}, {"Omsk", "اومسڪ جي ڏينهن جو وقت"}, {"Pakistan", "پاڪستان جي اونهاري جو وقت"}, {"Paraguay", "پيراگوئي جي اونهاري جو وقت"}, {"Peru", "پيرو جي اونهاري جو وقت"}, {"Philippines", "فلپائن جي اونهاري جو وقت"}, {"Pierre_Miquelon", "سينٽ پيئر ۽ ميڪلون جي ڏينهن جو وقت"}, {"Sakhalin", "سخالين جي ڏينهن جو وقت"}, {"Samoa", "ساموا جي ڏينهن جو وقت"}, {"Taipei", "تائپي جي ڏينهن جو وقت"}, {"Tonga", "ٽونگا جي اونهاري جو وقت"}, {"Turkmenistan", "ترڪمانستان جي اونهاري جو وقت"}, {"Uruguay", "يوروگائي جي اونهاري جو وقت"}, {"Uzbekistan", "ازبڪستان جي اونهاري جو وقت"}, {"Vanuatu", "وانواتو جي ڏينهن جو وقت"}, {"Vladivostok", "اولادووستوڪ جي ڏينهن جو وقت"}, {"Volgograd", "وولگوگراد جي ڏينهن جو وقت"}, {"Yakutsk", "ياڪتسڪ جي ڏينهن جو وقت"}, {"Yekaterinburg", "يڪاٽيرنبرگ جي ڏينهن جو وقت"}},
	},
	{
		{{"America_Central", "मरकज़ी मअयारी वक्त"}, {"America_Eastern", "ओभरी मअयारी वक्त"}, {"America_Mountain", "पहाड़ी मअयारी वक्त"}, {"America_Pacific", "पेसिफिक मअयारी वक्त"}, {"Atlantic", "अटलांटिक मअयारी वक्त"}, {"Azores", "एज़ोर्स मइयारी वक़्तु"}, {"Europe_Central", "मरकज़ी यूरोपी मअयारी वक्तु"}, {"Europe_Eastern", "ओभरी यूरोपी मअयारी वक्तु"}, {"Europe_Western", "उलहंदो यूरोपी मअयारी वक्तु"}, {"GMT", "ग्रीनविच मीन वक़्तु"}},
		{{"America_Central", "मरकज़ी दीं॒ह जो वक्त"}, {"America_Eastern", "ओभरी दीं॒ह जो वक्त"}, {"America_Mountain", "पहाड़ी दीं॒ह जो वक्त"}, {"America_Pacific", "पेसिफिक दीं॒ह जो वक्त"}, {"Atlantic", "अटलांटिक दीं॒ह जो वक्त"}, {"Azores", "एज़ोर्स ऊनहारे जो वक़्तु"}, {"Europe_Central", "मरकज़ी यूरोपी उनहारे जो वक्तु"}, {"Europe_Eastern", "ओभरी यूरोपी उनहारे जो वक्तु"}, {"Europe_Western", "उलहंदो यूरोपी उनहारे जो वक्तु"}},
	},
	{
		{{"Europe_Central", "gaska-Eurohpá dábálašáigi"}, {"Europe_Eastern", "nuorti-Eurohpá dábálašáigi"}, {"Europe_Western", "oarje-Eurohpá dábálašáigi"}, {"GMT", "Greenwich gaskka áigi"}, {"Moscow", "Moskva-dábálašáigi"}},
		{{"Europe_Central", "gaska-Eurohpá geassiáigi"}, {"Europe_Eastern", "nuorti-Eurohpá geassiáigi"}, {"Europe_Western", "oarje-Eurohpá geassiáigi"}, {"Moscow", "Moskva-geassiáigi"}},
	},
	{
		{{"Afghanistan", "Afganisthana áigi"}, {"Africa_Central", "Gaska-Afrihká áigi"}, {"Africa_Eastern", "Nuorta-Afrihká áigi"}, {"Africa_Southern", "Lulli-Afrihká dálveáigi"}, {"Africa_Western", "Oarje-Afrihká áigi"}, {"Alaska", "Alaska dálveáigi"}, {"Amazon", "Amazona dálveáigi"}, {"America_Central", "dábálaš dálveáigi"}, {"America_Eastern", "dálveáigi nuortan"}, {"America_Mountain", "dálveduottaráigi"}, {"America_Pacific", "Jaskesábi dálveáigi"}, {"Apia", "Apia dálveáigi"}, {"Arabian", "Arábia dálveáigi"}, {"Argentina", "Argentina dálveáigi"}, {"Argentina_Western", "Oarje-Argentina dálveáigi"}, {"Armenia", "Armenia dálveáigi"}, {"Atlantic", "atlántalaš dálveáigi"}, {"Australia_Central", "Gaska-Austrália dálveáigi"}, {"Australia_CentralWestern", "Gaska-Austrália oarjjabeali dálveáigi"}, {"Australia_Eastern", "Nuorta-Austrália dálveáigi"}, {"Australia_Western", "Oarje-Austrália dálveáigi"}, {"Azerbaijan", "Aserbaižana dálveáigi"}, {"Azores", "Azoraid dálveáigi"}, {"Bangladesh", "Bangladesha dálveáigi"}, {"Bhutan", "Bhutana áigi"}, {"Bolivia", "Bolivia áigi"}, {"Brasilia", "Brasilia dálveáigi"}, {"Brunei", "Brunei Darussalama áigi"}, {"Cape_Verde", "Kap Verde dálveáigi"}, {"Chamorro", "Čamorro dálveáigi"}, {"Chatham", "Chathama dálveáigi"}, {"Chile", "Chile dálveáigi"}, {"China", "Kiinná dálveáigi"}, {"Christmas", "Juovlasullo áigi"}, {"Cocos", "Kokossulloid áigi"}, {"Colombia", "Colombia dálveáigi"}, {"Cook", "Cooksulloid dálveáigi"}, {"Cuba", "Cuba dálveáigi"}, {"Davis", "Davisa áigi"}, {"DumontDUrville", "Dumont-d’Urville áigi"}, {"East_Timor", "Nuorta-Timora áigi"}, {"Easter", "Beassášsullo dálveáigi"}, {"Ecuador", "Ecuadora áigi"}, {"Europe_Central", "Gaska-Eurohpá dálveáigi"}, {"Europe_Eastern", "Nuorta-Eurohpa dálveáigi"}, {"Europe_Further_Eastern", "Gáiddus-Nuortti eurohpalaš áigi"}, {"Europe_Western", "Oarje-Eurohpá dálveáigi"}, {"Falkland", "Falklandsulluid dálveáigi"}, {"Fiji", "Fiji dálveáigi"}, {"French_Guiana", "Frankriikka Guyana áigi"}, {"French_Southern", "Frankriikka lulli & antárktisa áigi"}, {"GMT", "Greenwicha áigi"}, {"Galapagos", "Galapagosa áigi"}, {"Gambier", "Gambiera áigi"}, {"Georgia", "Georgia dálveáigi"}, {"Gilbert_Islands", "Gilbertsulloid áigi"}, {"Greenland_Eastern", "Nuorta-Ruonáeatnama dálveáigi"}, {"Greenland_Western", "Oarje-Ruonáeatnama dálveáigi"}, {"Gulf", "Golfa dálveáigi"}, {"Guyana", "Guyana áigi"}, {"Hawaii", "Hawaii-aleuhtalaš dálveáigi"}, {"Hawaii_Aleutian", "Hawaii-aleuhtalaš dálveáigi"}, {"Hong_Kong", "Hong Konga dálveáigi"}, {"Hovd", "Hovda dálveáigi"}, {"India", "India dálveáigi"}, {"Indian_Ocean", "Indiaábi áigi"}, {"Indochina", "Indokiinná áigi"}, {"Indonesia_Central", "Gaska-Indonesia áigi"}, {"Indonesia_Eastern", "Nuorta-Indonesia áigi"}, {"Indonesia_Western", "Oarje-Indonesia áigi"}, {"Iran", "Irana dálveáigi"}, {"Irkutsk", "Irkucka dálveáigi"}, {"Israel", "Israela dálveáigi"}, {"Japan", "Japána dálveáigi"}, {"Kazakhstan_Eastern", "Nuorta-Kasakstana áigi"}, {"Kazakhstan_Western", "Oarje-Kasakstana áigi"}, {"Korea", "Korea dálveáigi"}, {"Kosrae", "Kosraea áigi"}, {"Krasnoyarsk", "Krasnojarska dálveáigi"}, {"Kyrgystan", "Kirgisia áigi"}, {"Line_Islands", "Linesulloid áigi"}, {"Lord_Howe", "Lord Howe dálveáigi"}, {"Magadan", "Magadana dálveáigi"}, {"Malaysia", "Malesia áigi"}, {"Maldives", "Malediivvaid áigi"}, {"Marquesas", "Marquesasiid áigi"}, {"Marshall_Islands", "Marshallsulloid áigi"}, {"Mauritius", "Mauritiusa dálveáigi"}, {"Mawson", "Mawsona áigi"}, {"Mexico_Pacific", "Meksiko Jáskesábi dálveáigi"}, {"Mongolia", "Ulan-Batora dálveáigi"}, {"Moscow", "Moskva dálveáigi"}, {"Myanmar", "Myanmara áigi"}, {"Nauru", "Nauru áigi"}, {"Nepal", "Nepala áigi"}, {"New_Caledonia", "Ođđa-Kaledonia dálveáigi"}, {"New_Zealand", "Ođđa-Selánda dálveáigi"}, {"Newfoundland", "Newfoundlanda dálveáigi"}, {"Niue", "Niuea áigi"}, {"Norfolk", "Norfolksullo áigi"}, {"Noronha", "Fernando de Noronha dálveáigi"}, {"Novosibirsk", "Novosibirska dálveáigi"}, {"Omsk", "Omska dálveáigi"}, {"Pakistan", "Pakistana dálveáigi"}, {"Palau", "Palaua áigi"}, {"Papua_New_Guinea", "Papua Ođđa-Guinea áigi"}, {"Paraguay", "Paraguaya dálveáigi"}, {"Peru", "Peru dálveáigi"}, {"Philippines", "Filippiinnaid dálveáigi"}, {"Phoenix_Islands", "Phoenixsulloid áigi"}, {"Pierre_Miquelon", "St. Pierre & Miquelo dálveáigi"}, {"Pitcairn", "Pitcairnsulloid áigi"}, {"Ponape", "Ponape áigi"}, {"Pyongyang", "Pyongyanga áigi"}, {"Reunion", "Reuniona áigi"}, {"Rothera", "Rothera áigi"}, {"Sakhalin", "Sahalina dálveáigi"}, {"Samoa", "Samoa dálveáigi"}, {"Seychelles", "Seychellaid áigi"}, {"Singapore", "Singapore dálveáigi"}, {"Solomon", "Salomonsulloid áigi"}, {"South_Georgia", "Lulli-Georgia áigi"}, {"Suriname", "Suriname áigi"}, {"Syowa", "Syowa áigi"}, {"Tahiti", "Tahiti áigi"}, {"Taipei", "Taipeia dálveáigi"}, {"Tajikistan", "Tažikistana áigi"}, {"Tokelau", "Tokelaua áigi"}, {"Tonga", "Tonga dálveáigi"}, {"Truk", "Chuuka áigi"}, {"Turkmenistan", "Turkmenistana dálveáigi"}, {"Tuvalu", "Tuvalu áigi"}, {"Uruguay", "Uruguaya dálveáigi"}, {"Uzbekistan", "Usbekistana dálveáigi"}, {"Vanuatu", "Vanuatu dálveáigi"}, {"Venezuela", "Venezuela áigi"}, {"Vladivostok", "Vladivostoka dálveáigi"}, {"Volgograd", "Volgograda dálveáigi"}, {"Vostok", "Vostoka áigi"}, {"Wake", "Wakesullo áigi"}, {"Wallis", "Wallis- ja Futuna áigi"}, {"Yakutsk", "Jakucka dálveáigi"}, {"Yekaterinburg", "Jekaterinburga dálveáigi"}},
		{{"Alaska", "Alaska geasseáigi"}, {"Amazon", "Amazona geasseáigi"}, {"America_Central", "dábálaš geasseáigi"}, {"America_Eastern", "geasseáigi nuortan"}, {"America_Mountain", "geasseduottaráigi"}, {"America_Pacific", "Jaskesábi geasseáigi"}, {"Apia", "Apia geasseáigi"}, {"Arabian", "Arábia geasseáigi"}, {"Argentina", "Argentina geasseáigi"}, {"Argentina_Western", "Oarje-Argentina geasseáigi"}, {"Armenia", "Armenia geasseáigi"}, {"Atlantic", "atlántalaš geasseáigi"}, {"Australia_Central", "Gaska-Austrália geasseáigi"}, {"Australia_CentralWestern", "Gaska-Austrália oarjjabeali geasseáigi"}, {"Australia_Eastern", "Nuorta-Austrália geasseáigi"}, {"Australia_Western", "Oarje-Austrália geasseáigi"}, {"Azerbaijan", "Aserbaižana geasseáigi"}, {"Azores", "Azoraid geasseáigi"}, {"Bangladesh", "Bangladesha geasseáigi"}, {"Brasilia", "Brasilia geasseáigi"}, {"Cape_Verde", "Kap Verde geasseáigi"}, {"Chatham", "Chathama geasseáigi"}, {"Chile", "Chile geasseáigi"}, {"China", "Kiinná geasseáigi"}, {"Colombia", "Colombia geasseáigi"}, {"Cook", "Cooksulloid geasi beallemuttu áigi"}, {"Cuba", "Cuba geasseáigi"}, {"Easter", "Beassášsullo geasseáigi"}, {"Europe_Central", "Gaska-Eurohpá geasseáigi"}, {"Europe_Eastern", "Nuorta-Eurohpa geasseáigi"}, {"Europe_Western", "Oarje-Eurohpá geasseáigi"}, {"Falkland", "Falklandsulluid geasseáigi"}, {"Fiji", "Fiji geasseáigi"}, {"Georgia", "Georgia geasseáigi"}, {"Greenland_Eastern", "Nuorta-Ruonáeatnama geasseáigi"}, {"Greenland_Western", "Oarje-Ruonáeatnama geasseáigi"}, {"Hawaii_Aleutian", "Hawaii-aleuhtalaš geasseáigi"}, {"Hong_Kong", "Hong Konga geasseáigi"}, {"Hovd", "Hovda geasseáigi"}, {"Iran", "Irana geasseáigi"}, {"Irkutsk", "Irkucka geasseáigi"}, {"Israel", "Israela geasseáigi"}, {"Japan", "Japána geasseáigi"}, {"Korea", "Korea geasseáigi"}, {"Krasnoyarsk", "Krasnojarska geasseáigi"}, {"Lord_Howe", "Lord Howe geasseáigi"}, {"Magadan", "Magadana geasseáigi"}, {"Mauritius", "Mauritiusa geasseáigi"}, {"Mexico_Pacific", "Meksiko Jáskesábi geasseáigi"}, {"Mongolia", "Ulan-Batora geasseáigi"}, {"Moscow", "Moskva geasseáigi"}, {"New_Caledonia", "Ođđa-Kaledonia geasseáigi"}, {"New_Zealand", "Ođđa-Selánda geasseáigi"}, {"Newfoundland", "Newfoundlanda geasseáigi"}, {"Noronha", "Fernando de Noronha geasseáigi"}, {"Novosibirsk", "Novosibirska geasseáigi"}, {"Omsk", "Omska geasseáigi"}, {"Pakistan", "Pakistana geasseáigi"}, {"Paraguay", "Paraguaya geasseáigi"}, {"Peru", "Peru geasseáigi"}, {"Philippines", "Filippiinnaid geasseáigi"}, {"Pierre_Miquelon", "St. Pierre & Miquelo geasseáigi"}, {"Sakhalin", "Sahalina geasseáigi"}, {"Samoa", "Samoa geasseáigi"}, {"Taipei", "Taipeia geasseáigi"}, {"Tonga", "Tonga geasseáigi"}, {"Turkmenistan", "Turkmenistana geasseáigi"}, {"Uruguay", "Uruguaya geasseáigi"}, {"Uzbekistan", "Usbekistana geasseáigi"}, {"Vanuatu", "Vanuatu geasseáigi"}, {"Vladivostok", "Vladivostoka geasseáigi"}, {"Volgograd", "Volgograda geasseáigi"}, {"Yakutsk", "Jakucka geasseáigi"}, {"Yekaterinburg", "Jekaterinburga geasseáigi"}},
	},
	{
		{{"Acre", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢေႇၵႃႇ"}, {"Afghanistan", "ၶၢဝ်းယၢမ်း ဢႃႇၾၵၢၼ်ႇၼီႇသတၢၼ်ႇ"}, {"Africa_Central", "ၶၢဝ်းယၢမ်း ဢႃႇၾရိၵ ပွတ်းၵၢင်"}, {"Africa_Eastern", "ၶၢဝ်းယၢမ်း ဢႃႇၾရိၵ ပွတ်းဢွၵ်ႇ"}, {"Africa_Southern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၾရိၵၸၢၼ်း"}, {"Africa_Western", "ၶၢဝ်းယၢမ်း ဢႃႇၾရိၵ ပွတ်းတူၵ်း"}, {"Alaska", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢလႅတ်ႉသၵႃႇ"}, {"Almaty", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႄးလ်မႃႇတီႇ"}, {"Amazon", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢမၸွၼ်ႇ"}, {"America_Central", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပွတ်းၵၢင်"}, {"America_Eastern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပွတ်းဢွၵ်ႇ"}, {"America_Mountain", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သၼ်လွႆ"}, {"America_Pacific", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပၸိၾိၵ်ႉ"}, {"Anadyr", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၼႃႇတီႇယႃႇ"}, {"Apia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သႃႇမူဝ်းဝႃႇ"}, {"Aqtau", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႅၵ်ႉတၢဝ်ႉ"}, {"Aqtobe", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႅၵ်ႉတူဝ်ႇပီႇ"}, {"Arabian", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇရၢပ်ႈ"}, {"Argentina", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၵျႅၼ်ႇတီးၼႃး"}, {"Argentina_Western", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၵျႅၼ်ႇတီးၼႃး ပွတ်းတူၵ်း"}, {"Armenia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇမေးၼီးယႃး"}, {"Atlantic", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႅတ်ႉလႅၼ်းတိၵ်ႉ"}, {"Australia_Central", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢေႃႉသထရေးလီးယႃး"}, {"Australia_CentralWestern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပွတ်းတူၵ်း တွၼ်ႈၵၢင် ဢေႃႉသထရေးလီးယႃး"}, {"Australia_Eastern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပွတ်းဢွၵ်ႇ ဢေႃႉသထရေးလီးယႃး"}, {"Australia_Western", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပွတ်းတူၵ်း ဢေႃႉသထရေးလီးယႃး"}, {"Azerbaijan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၸႃႇပၢႆႇၸၼ်ႇ"}, {"Azores", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢႃႇၸူဝ်ႇရႅတ်ႉ"}, {"Bangladesh", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပင်းၵလႃးတဵတ်ႈ"}, {"Bhutan", "ၶၢဝ်းယၢမ်း ၽူႇတၢၼ်ႇ"}, {"Bolivia", "ၶၢဝ်းယၢမ်း ပူဝ်ႇလီးပီးယႃး"}, {"Brasilia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပရႃႇၸီးလီးယႃး"}, {"Brunei", "ၶၢဝ်းယၢမ်း ပရူႇၼၢႆး"}, {"Cape_Verde", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶဵပ်ႉဝႄႇတီႇ"}, {"Casey", "ၶၢဝ်းယၢမ်း ၶေႇသီႇ"}, {"Chamorro", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶျမ်ႇမူဝ်ႇရူဝ်ႇ"}, {"Chatham", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶျႅတ်ႉထမ်ႇ"}, {"Chile", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶျီႇလီႇ"}, {"China", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶႄႇ"}, {"Christmas", "ၶၢဝ်းယၢမ်း ၵုၼ်ၶရိတ်ႉသမၢတ်ႉ"}, {"Cocos", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်ၵူဝ်ႇၵူဝ်းသ်"}, {"Colombia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵူဝ်ႇလမ်ႇပီႇယႃႇ"}, {"Cook", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မူႇၵုၼ်ၶုၵ်ႈ"}, {"Cuba", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵူးပႃး"}, {"Davis", "ၶၢဝ်းယၢမ်း တေးဝိတ်ႉ"}, {"DumontDUrville", "ၶၢဝ်းယၢမ်း တူႇမွၼ်ႉ တဝီႇလ်"}, {"East_Timor", "ၶၢဝ်းယၢမ်း တီႇမေႃးလႅတ်ႉသ်တႄး"}, {"Easter", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵုၼ်ဢီႇသတႃႇ"}, {"Ecuador", "ၶၢဝ်းယၢမ်း ဢေႇၵႂႃႇတေႃႇ"}, {"Europe_Central", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ယူးရူပ်ႉ ပွတ်းၵၢင်"}, {"Europe_Eastern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ယူးရူပ်ႉ ပွတ်းဢွၵ်ႇ"}, {"Europe_Further_Eastern", "ၶၢဝ်းယၢမ်း ယူးရူပ်ႉ ပွတ်းဢွၵ်ႇ ဢၼ်ၵႆတိူဝ်း"}, {"Europe_Western", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ယူးရူပ်ႉ ပွတ်းတူၵ်း"}, {"Falkland", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မူႇၵုၼ် ၾွၵ်ႉလႅၼ်ႇ"}, {"Fiji", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၾီႇၵျီႇ"}, {"French_Guiana", "ၶၢဝ်းယၢမ်း ၵုၺ်ႇယႃႇၼႃႇ ၶွင် ၾရၢင်ႇသဵတ်ႈ"}, {"French_Southern", "ၶၢဝ်းယၢမ်း ၾရၢင်ႇသဵတ်ႈ ပွတ်းၸၢၼ်း လႄႈ ဢႅၼ်ႇတၢၵ်ႈတီးၵႃႈ"}, {"GMT", "ၶၢဝ်းယၢမ်း ၽတ်ႉၽဵင်ႇ ၵရိၼ်းဝိတ်ႉ"}, {"Galapagos", "ၶၢဝ်းယၢမ်း ၵႃႇလႃႇပၵူဝ်ႉသ်"}, {"Gambier", "ၶၢဝ်းယၢမ်း ၵမ်ႇပီႇယႃႇ"}, {"Georgia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵျေႃႇၵျႃႇ"}, {"Gilbert_Islands", "ၶၢဝ်းယၢမ်း ၵိလ်ပႅတ်ႉ"}, {"Greenland", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵရိၼ်းလႅၼ်း"}, {"Greenland_Eastern", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵရိၼ်းလႅၼ်း ပွတ်းဢွၵ်ႇ"}, {"Greenland_Western", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵရိၼ်းလႅၼ်း ပွတ်းတူၵ်း"}, {"Guam", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵႂၢမ်ႇ"}, {"Gulf", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢၢဝ်ႇ"}, {"Guyana", "ၶၢဝ်းယၢမ်း ၵၢႆႇယႃးၼႃႇ"}, {"Hawaii", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ႁႃႇဝၢႆးဢီး-ဢလူးသျၼ်ႇ"}, {"Hawaii_Aleutian", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ႁႃႇဝၢႆးဢီး-ဢလူးသျၼ်ႇ"}, {"Hong_Kong", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ႁွင်းၵွင်း"}, {"Hovd", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶူဝ်ဝေတေး"}, {"India", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢိၼ်းတီးယႃး"}, {"Indian_Ocean", "ၶၢဝ်းယၢမ်း သမုၵ်ႉတရႃႇဢိၼ်းတီးယႃး"}, {"Indochina", "ၶၢဝ်းယၢမ်း ဢိၼ်ႇတူဝ်ႇၶႄႇ"}, {"Indonesia_Central", "ၶၢဝ်းယၢမ်း ဢိၼ်ႇတူဝ်ႇၼီးသျႃး ပွတ်းၵၢင်"}, {"Indonesia_Eastern", "ၶၢဝ်းယၢမ်း ဢိၼ်ႇတူဝ်ႇၼီးသျႃး ပွတ်းဢွၵ်ႇ"}, {"Indonesia_Western", "ၶၢဝ်းယၢမ်း ဢိၼ်ႇတူဝ်ႇၼီးသျႃး ပွတ်းတူၵ်း"}, {"Iran", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢီႇရၢၼ်း"}, {"Irkutsk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢိရၶုတ်ႉသ်ၶ်"}, {"Israel", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢိတ်ႇသရေး"}, {"Japan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵျႃႇပၢၼ်ႇ"}, {"Kamchatka", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵမ်ႇၶျတ်ႉၵႃႇ"}, {"Kazakhstan", "ၶၢဝ်းယၢမ်း ၵႃႇၸၢၵ်ႈသတၼ်ႇ"}, {"Kazakhstan_Eastern", "ၶၢဝ်းယၢမ်း ၵႃႇၸၢၵ်ႈသတၼ်ႇ ပွတ်းဢွၵ်ႇ"}, {"Kazakhstan_Western", "ၶၢဝ်းယၢမ်း ၵႃႇၸၢၵ်ႈသတၼ်ႇ ပွတ်းတူၵ်း"}, {"Korea", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵၢဝ်းလီ"}, {"Kosrae", "ၶၢဝ်းယၢမ်း ၵွတ်ႉသ်ရေး"}, {"Krasnoyarsk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶရတ်ႉၼူဝ်ႇယႃႇသ်ၶ်"}, {"Kyrgystan", "ၶၢဝ်းယၢမ်း ၵႃႇၵိတ်ႈသတၼ်ႇ"}, {"Lanka", "ၶၢဝ်းယၢမ်း လင်းၵႃ"}, {"Line_Islands", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်လၢႆး"}, {"Lord_Howe", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း လွတ်ႉႁၢဝ်း"}, {"Macau", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မႃႇၵၢဝ်ႈ"}, {"Magadan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မႃႇၵႃႇတၢၼ်ႇ"}, {"Malaysia", "ၶၢဝ်းယၢမ်း မလေးသျႃး"}, {"Maldives", "ၶၢဝ်းယၢမ်း မေႃႇတိပ်ႈ"}, {"Marquesas", "ၶၢဝ်းယၢမ်း မႃၶေးသႅတ်ႉသ်"}, {"Marshall_Islands", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်မႃးသျႄႇ"}, {"Mauritius", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မေႃးရီႇသႃႇ"}, {"Mawson", "ၶၢဝ်းယၢမ်း မေႃသၼ်ႇ"}, {"Mexico_Pacific", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မႅၵ်ႇသီႇၵူဝ်ႇ ပၸိၾိၵ်ႉ"}, {"Mongolia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢူလၼ်ပႃတႃ"}, {"Moscow", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း မွတ်ႉသ်ၵူဝ်ႇ"}, {"Myanmar", "ၶၢဝ်းယၢမ်း မျၢၼ်ႇမႃႇ"}, {"Nauru", "ၶၢဝ်းယၢမ်း ၼၢဝ်ရူး"}, {"Nepal", "ၶၢဝ်းယၢမ်း ၼေႇပေႃး"}, {"New_Caledonia", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၼိဝ်းၶႄႇလီႇတူဝ်းၼီးယႃး"}, {"New_Zealand", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၼိဝ်းၸီႇလႅၼ်ႇ"}, {"Newfoundland", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၼိဝ်းၾွမ်ႇလႅၼ်ႇ"}, {"Niue", "ၶၢဝ်းယၢမ်း ၼီးဝႄႇ"}, {"Norfolk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၵုၼ်ၼေႃႇၾုၵ်ႉ"}, {"Noronha", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၾႃႇၼႅၼ်ႇတူဝ်ႇ တႄႇ တီႇၼူဝ်ႇရွၼ်ႇႁႃႇ"}, {"North_Mariana", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်မႃႇရီႇယႃႇၼႃႇ ပွတ်းႁွင်ႇ"}, {"Novosibirsk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၼူဝ်ႇဝူဝ်ႇသီႇပၢတ်ႉသ်ၶ်"}, {"Omsk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢူမ်းသ်ၶ်"}, {"Pakistan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပႃႇၵိတ်ႈသတၼ်ႇ"}, {"Palau", "ၶၢဝ်းယၢမ်း ပႃႇလၢဝ်း"}, {"Papua_New_Guinea", "ၶၢဝ်းယၢမ်း ပႃးပႂႃႇၼိဝ်းၵီးၼီး"}, {"Paraguay", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပႃႇရႃႇၵူၺ်း"}, {"Peru", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ပေႇရူႉ"}, {"Philippines", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၾီလိပ်ႈပိၼ်း"}, {"Phoenix_Islands", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်ၽီးၼိၵ်ႉ"}, {"Pierre_Miquelon", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သဵင်ႉပီးယႃး လႄႈ မိၵ်ႈၵွႆႇလွၼ်ႇ"}, {"Pitcairn", "ၶၢဝ်းယၢမ်း ၽိတ်ႉၶႅၼ်ႇ"}, {"Ponape", "ၶၢဝ်းယၢမ်း ပူၼ်းပေႇ"}, {"Pyongyang", "ၶၢဝ်းယၢမ်း ၵၢဝ်းလီႁွင်ႇ"}, {"Qyzylorda", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ၶုၺ်ႇၸီးလူဝ်ႇတႃႇ"}, {"Reunion", "ၶၢဝ်းယၢမ်း ရေႇၼီႇယၼ်ႇ"}, {"Rothera", "ၶၢဝ်းယၢမ်း ရူဝ်ႇထႄႇရႃႇ"}, {"Sakhalin", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သႃႇၶႃႇလိၼ်ႇ"}, {"Samara", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သႃႇမႃႇရႃႇ"}, {"Samoa", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သႃႇမူဝ်းဝႃႇ ၶွ\u200bင် ဢမႄႇရိၵၢၼ်ႇ"}, {"Seychelles", "ၶၢဝ်းယၢမ်း သေးသျႄႇ"}, {"Singapore", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း သိင်ႇၵႃႇပူဝ်ႇ"}, {"Solomon", "ၶၢဝ်းယၢမ်း မူႇၵုၼ်သေႃႇလေႃႇမၼ်ႇ"}, {"South_Georgia", "ၶၢဝ်းယၢမ်း ၵျေႃႇၵျႃႇ ပွတ်းၸၢၼ်း"}, {"Suriname", "ၶၢဝ်းယၢမ်း သျူးရီးၼႃႇမႄႇ"}, {"Syowa", "ၶၢဝ်းယၢမ်း သၢႆးဢူဝ်ႇဝႃႇ"}, {"Tahiti", "ၶၢဝ်းယၢမ်း တႁီႇတီႇ"}, {"Taipei", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ထၢႆႇဝၢၼ်း"}, {"Tajikistan", "ၶၢဝ်းယၢမ်း တႃႇၵျီႇၵီႇသတၼ်ႇ"}, {"Tokelau", "ၶၢဝ်းယၢမ်း ထူဝ်းၵေႇလၢဝ်ႇ"}, {"Tonga", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ထွင်းၵႃႇ"}, {"Truk", "ၶၢဝ်းယၢမ်း ၶျူၵ်ႉ"}, {"Turkmenistan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း တၢၵ်ႈမႅၼ်ႇၼီႇသတၼ်ႇ"}, {"Tuvalu", "ၶၢဝ်းယၢမ်း ထူးဝႃႇလူႇ"}, {"Uruguay", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢုရုၵူၺ်း"}, {"Uzbekistan", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဢူႇၸပႄႉၵိတ်ႇသတၼ်ႇ"}, {"Vanuatu", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဝႅၼ်ႇၼူးဝႃႇထူႇ"}, {"Venezuela", "ၶၢဝ်းယၢမ်း ဝႄႇၼေႇၸွႆးလႃး"}, {"Vladivostok", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဝလႃႇတီႇဝူဝ်ႇသတွၵ်ႉ"}, {"Volgograd", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ဝူဝ်ႇၵူဝ်ႇၵရၢတ်ႉ"}, {"Vostok", "ၶၢဝ်းယၢမ်း ဝေႃႉသ်တွၵ်ႉ"}, {"Wake", "ၶၢဝ်းယၢမ်း ၵုၼ်ဝဵၵ်ႉ"}, {"Wallis", "ၶၢဝ်းယၢမ်း ဝႃးလိတ်ႈ လႄႈ ၾူႇတူးၼႃး"}, {"Yakutsk", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ယၵုတ်ႉသ်ၶ်"}, {"Yekaterinburg", "လၵ်းၸဵင် ၶၢဝ်းယၢမ်း ယေႇၵႃႇတႄႇရိၼ်ႇပၢၵ်ႉ"}, {"Yukon", "ၶၢဝ်းယၢမ်း ယူးၵွၼ်ႇ"}},
		{{"Acre", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢေႇၵႃႇ"}, {"Alaska", "ၶၢဝ်းယၢမ်းၵၢင်ဝၼ်း ဢလႅတ်ႉသၵႃႇ"}, {"Almaty", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႄးလ်မႃႇတီႇ"}, {"Amazon", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢမၸွၼ်ႇ"}, {"America_Central", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပွတ်းၵၢင်"}, {"America_Eastern", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပွတ်းဢွၵ်ႇ"}, {"America_Mountain", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း သၼ်လွႆ"}, {"America_Pacific", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပၸိၾိၵ်ႉ"}, {"Anadyr", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႃႇၼႃႇတီႇယႃႇ"}, {"Apia", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း သႃႇမူဝ်းဝႃႇ"}, {"Aqtau", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႅၵ်ႉတၢဝ်ႉ"}, {"Aqtobe", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႅၵ်ႉတူဝ်ႇပီႇ"}, {"Arabian", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢႃႇရၢပ်ႈ"}, {"Argentina", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢႃႇၵျႅၼ်ႇတီးၼႃး"}, {"Argentina_Western", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢႃႇၵျႅၼ်ႇတီးၼႃး ပွတ်းတူၵ်း"}, {"Armenia", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႃႇမေးၼီးယႃး"}, {"Atlantic", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢႅတ်ႉလႅၼ်းတိၵ်ႉ"}, {"Australia_Central", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢေႃႉသထရေးလီးယႃး"}, {"Australia_CentralWestern", "ၶၢဝ်းယၢမ်း ပွတ်းတူၵ်း တွၼ်ႈၵၢင် ၶၢဝ်းမႆႈ ဢေႃႉသထရေးလီးယႃး"}, {"Australia_Eastern", "ၶၢဝ်းယၢမ်း ပွတ်းဢွၵ်ႇ ၵၢင်ဝၼ်း ဢေႃႉသထရေးလီးယႃး"}, {"Australia_Western", "ၶၢဝ်းယၢမ်း ပွတ်းတူၵ်း ၵၢင်ဝၼ်း ဢေႃႉသထရေးလီးယႃး"}, {"Azerbaijan", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႃႇၸႃႇပၢႆႇၸၼ်ႇ"}, {"Azores", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢႃႇၸူဝ်ႇရႅတ်ႉ"}, {"Bangladesh", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ပင်းၵလႃးတဵတ်ႈ"}, {"Brasilia", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပရႃႇၸီးလီးယႃး"}, {"Cape_Verde", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၶဵပ်ႉဝႄႇတီႇ"}, {"Chatham", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၶျႅတ်ႉထမ်ႇ"}, {"Chile", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၶျီႇလီႇ"}, {"China", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၶႄႇ"}, {"Colombia", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵူဝ်ႇလမ်ႇပီႇယႃႇ"}, {"Cook", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ မူႇၵုၼ်ၶုၵ်ႈ"}, {"Cuba", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵူးပႃး"}, {"Easter", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵုၼ်ဢီႇသတႃႇ"}, {"Europe_Central", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ယူးရူပ်ႉ ပွတ်းၵၢင်"}, {"Europe_Eastern", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ယူးရူပ်ႉ ပွတ်းဢွၵ်ႇ"}, {"Europe_Western", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ယူးရူပ်ႉ ပွတ်းတူၵ်း"}, {"Falkland", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း မူႇၵုၼ် ၾွၵ်ႉလႅၼ်ႇ"}, {"Fiji", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၾီႇၵျီႇ"}, {"Georgia", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၵျေႃႇၵျႃႇ"}, {"Greenland", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၵရိၼ်းလႅၼ်း"}, {"Greenland_Eastern", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵရိၼ်းလႅၼ်း ပွတ်းဢွၵ်ႇ"}, {"Greenland_Western", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵရိၼ်းလႅၼ်း ပွတ်းတူၵ်း"}, {"Hawaii_Aleutian", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ႁႃႇဝၢႆးဢီး-ဢလူးသျၼ်ႇ"}, {"Hong_Kong", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ႁွင်းၵွင်း"}, {"Hovd", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၶူဝ်ဝေတေး"}, {"Iran", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢီႇရၢၼ်း"}, {"Irkutsk", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢိရၶုတ်ႉသ်ၶ်"}, {"Israel", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢိတ်ႇသရေး"}, {"Japan", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵျႃႇပၢၼ်ႇ"}, {"Kamchatka", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၵမ်ႇၶျတ်ႉၵႃႇ"}, {"Korea", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵၢဝ်းလီ"}, {"Krasnoyarsk", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၶရတ်ႉၼူဝ်ႇယႃႇသ်ၶ်"}, {"Lord_Howe", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း လွတ်ႉႁၢဝ်း"}, {"Macau", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ မႃႇၵၢဝ်ႈ"}, {"Magadan", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ မႃႇၵႃႇတၢၼ်ႇ"}, {"Mauritius", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ မေႃးရီႇသႃႇ"}, {"Mexico_Pacific", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း မႅၵ်ႇသီႇၵူဝ်ႇ ပၸိၾိၵ်ႉ"}, {"Mongolia", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢူလၼ်ပႃတႃ"}, {"Moscow", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ မွတ်ႉသ်ၵူဝ်ႇ"}, {"New_Caledonia", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၼိဝ်းၶႄႇလီႇတူဝ်းၼီးယႃး"}, {"New_Zealand", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၼိဝ်းၸီႇလႅၼ်ႇ"}, {"Newfoundland", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၼိဝ်းၾွမ်ႇလႅၼ်ႇ"}, {"Norfolk", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၵုၼ်ၼေႃႇၾုၵ်ႉ"}, {"Noronha", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ၾႃႇၼႅၼ်ႇတူဝ်ႇ တႄႇ တီႇၼူဝ်ႇရွၼ်ႇႁႃႇ"}, {"Novosibirsk", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၼူဝ်ႇဝူဝ်ႇသီႇပၢတ်ႉသ်ၶ်"}, {"Omsk", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢူမ်းသ်ၶ်"}, {"Pakistan", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ပႃႇၵိတ်ႈသတၼ်ႇ"}, {"Paraguay", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပႃႇရႃႇၵူၺ်း"}, {"Peru", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ပေႇရူႉ"}, {"Philippines", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၾီလိပ်ႈပိၼ်း"}, {"Pierre_Miquelon", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း သဵင်ႉပီးယႃး လႄႈ မိၵ်ႈၵွႆႇလွၼ်ႇ"}, {"Qyzylorda", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ၶုၺ်ႇၸီးလူဝ်ႇတႃႇ"}, {"Sakhalin", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ သႃႇၶႃႇလိၼ်ႇ"}, {"Samara", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ သႃႇမႃႇရႃႇ"}, {"Samoa", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း သႃႇမူဝ်းဝႃႇ ၶွ\u200bင် ဢမႄႇရိၵၢၼ်ႇ"}, {"Taipei", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ထၢႆႇဝၢၼ်း"}, {"Tonga", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ထွင်းၵႃႇ"}, {"Turkmenistan", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ တၢၵ်ႈမႅၼ်ႇၼီႇသတၼ်ႇ"}, {"Uruguay", "ၶၢဝ်းယၢမ်း ၵၢင်ဝၼ်း ဢုရုၵူၺ်း"}, {"Uzbekistan", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဢူႇၸပႄႉၵိတ်ႇသတၼ်ႇ"}, {"Vanuatu", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဝႅၼ်ႇၼူးဝႃႇထူႇ"}, {"Vladivostok", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဝလႃႇတီႇဝူဝ်ႇသတွၵ်ႉ"}, {"Volgograd", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ဝူဝ်ႇၵူဝ်ႇၵရၢတ်ႉ"}, {"Yakutsk", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ယၵုတ်ႉသ်ၶ်"}, {"Yekaterinburg", "ၶၢဝ်းယၢမ်း ၶၢဝ်းမႆႈ ယေႇၵႃႇတႄႇရိၼ်ႇပၢၵ်ႉ"}},
	},
	{
		{{"Afghanistan", "ඇෆ්ගනිස්ථාන වේලාව"}, {"Africa_Central", "මධ්\u200dයම අප්\u200dරිකානු වේලාව"}, {"Africa_Eastern", "නැගෙනහිර අප්\u200dරිකානු වේලාව"}, {"Africa_Southern", "දකුණු අප්\u200dරිකානු වේලාව"}, {"Africa_Western", "බටහිර අප්\u200dරිකානු වේලාව"}, {"Alaska", "ඇලස්කා සම්මත වේලාව"}, {"Amazon", "ඇමර්සන් සම්මත වේලාව"}, {"America_Central", "උතුරු ඇමරිකානු මධ්\u200dයම සම්මත වේලාව"}, {"America_Eastern", "උතුරු ඇමරිකානු නැගෙනහිර සම්මත වේලාව"}, {"America_Mountain", "උතුරු ඇමරිකානු කඳුකර සම්මත වේලාව"}, {"America_Pacific", "උතුරු ඇමරිකානු පැසිෆික් සම්මත වේලාව"}, {"Apia", "අපියා සම්මත වේලාව"}, {"Arabian", "අරාබි සම්මත වේලාව"}, {"Argentina", "ආර්ජන්ටිනා සම්මත වේලාව"}, {"Argentina_Western", "බටහිර ආර්ජන්ටිනා සම්මත වේලාව"}, {"Armenia", "ආමේනියානු සම්මත වේලාව"}, {"Atlantic", "අත්ලාන්තික් සම්මත වේලාව"}, {"Australia_Central", "ඕස්ට්\u200dරේලියානු සම්මත වේලාව"}, {"Australia_CentralWestern", "මධ්\u200dයම බටහිර ඔස්ට්\u200dරේලියානු සම්මත වේලාව"}, {"Australia_Eastern", "නැගෙනහිර ඕස්ට්\u200dරේලියානු සම්මත වේලාව"}, {"Australia_Western", "බටහිර ඕස්ට්\u200dරේලියානු සම්මත වේලාව"}, {"Azerbaijan", "අසර්බයිජාන් සම්මත වේලාව"}, {"Azores", "ඇසොර්ස් සම්මත වේලාව"}, {"Bangladesh", "බංගලාදේශ සම්මත වේලාව"}, {"Bhutan", "භුතාන වේලාව"}, {"Bolivia", "බොලිවියා වේලාව"}, {"Brasilia", "බ්\u200dරසීල සම්මත වේලාව"}, {"Brunei", "බෘනායි දරුස්සලාම් වේලාව"}, {"Cape_Verde", "කේප්වේඩ් සම්මත වේලාව"}, {"Chamorro", "චමොරෝ වේලාව"}, {"Chatham", "චැතම් සම්මත වේලාව"}, {"Chile", "චිලී සම්මත වේලාව"}, {"China", "චීන සම්මත වේලාව"}, {"Christmas", "ක්\u200dරිස්මස් දුපත් වේලාව"}, {"Cocos", "කොකෝස් දුපත් වේලාව"}, {"Colombia", "කොලොම්බියා සම්මත වේලාව"}, {"Cook", "කුක් දුපත් සම්මත වේලාව"}, {"Cuba", "කියුබානු සම්මත වේලාව"}, {"Davis", "ඩාවිස් වේලාව"}, {"DumontDUrville", "දුමොන්ත්-ඩ්උර්විල් වේලාව"}, {"East_Timor", "නැගෙනහිර ටිමෝර් වේලාව"}, {"Easter", "ඊස්ටර් දූපත් සම්මත වේලාව"}, {"Ecuador", "ඉක්වදෝර් වේලාව"}, {"Europe_Central", "මධ්\u200dයම යුරෝපීය සම්මත වේලාව"}, {"Europe_Eastern", "නැගෙනහිර යුරෝපීය සම්මත වේලාව"}, {"Europe_Further_Eastern", "තවත්-නැගෙනහිර යුරෝපීය වේලාව"}, {"Europe_Western", "බටහිර යුරෝපීය සම්මත වේලාව"}, {"Falkland", "ෆෝක්ලන්ඩ් දූපත් සම්මත වේලාව"}, {"Fiji", "ෆිජි සම්මත වේලාව"}, {"French_Guiana", "ප්\u200dරංශ ගයනා වේලාව"}, {"French_Southern", "ප්\u200dරංශ දකුණුදිග සහ ඇන්ටාර්ක්ටික් වේලාව"}, {"GMT", "ග්\u200dරිනිච් මධ්\u200dයම වේලාව"}, {"Galapagos", "ගලපගොස් වේලාව"}, {"Gambier", "ගැම්බියර් වේලාව"}, {"Georgia", "ජෝර්ජියානු සම්මත වේලාව"}, {"Gilbert_Islands", "ගිල්බර්ට් දුපත් වේලාව"}, {"Greenland_Eastern", "නැගෙනහිර ග්\u200dරීන්ලන්ත සම්මත වේලාව"}, {"Greenland_Western", "බටහිර ග්\u200dරීන්ලන්ත සම්මත වේලාව"}, {"Gulf", "ගල්ෆ් වේලාව"}, {"Guyana", "ගයනා වේලාව"}, {"Hawaii", "හවායි-අලෙයුතියාන් සම්මත වේලාව"}, {"Hawaii_Aleutian", "හවායි-අලෙයුතියාන් සම්මත වේලාව"}, {"Hong_Kong", "හොංකොං සම්මත වේලාව"}, {"Hovd", "හොව්ඩ් සම්මත වේලාව"}, {"India", "ඉන්දියානු වේලාව"}, {"Indian_Ocean", "ඉන්දියන් සාගර වේලාව"}, {"Indochina", "ඉන්දුචීන වේලාව"}, {"Indonesia_Central", "මධ්\u200dයම ඉන්දුනීසියානු වේලාව"}, {"Indonesia_Eastern", "නැගෙනහිර ඉන්දුනීසියානු වේලාව"}, {"Indonesia_Western", "බටහිර ඉන්දුනීසියානු වේලාව"}, {"Iran", "ඉරාන සම්මත වේලාව"}, {"Irkutsk", "ඉර්කුට්ස්ක් සම්මත වේලාව"}, {"Israel", "ඊශ්\u200dරායල සම්මත වේලාව"}, {"Japan", "ජපාන සම්මත වේලාව"}, {"Kazakhstan", "කසකස්තාන වේලාව"}, {"Kazakhstan_Eastern", "නැගෙනහිර කසකස්තාන වේලාව"}, {"Kazakhstan_Western", "බටහිර කසකස්තාන වේලාව"}, {"Korea", "කොරියානු සම්මත වේලාව"}, {"Kosrae", "කොස්රේ වේලාව"}, {"Krasnoyarsk", "ක්\u200dරස්නොයාර්ස්ක් සම්මත වේලාව"}, {"Kyrgystan", "කිර්ගිස්තාන වේලාව"}, {"Lanka", "ශ්\u200dරී ලංකා වේලාව"}, {"Line_Islands", "ලයින් දුපත් වේලාව"}, {"Lord_Howe", "ලෝර්ඩ් හෝව් සම්මත වේලාව"}, {"Magadan", "මෙගඩන් සම්මත වේලාව"}, {"Malaysia", "මැලේසියානු වේලාව"}, {"Maldives", "මාලදිවයින් වේලාව"}, {"Marquesas", "මාර්කුඑසාස් වේලාව"}, {"Marshall_Islands", "මාර්ෂල් දුපත් වේලාව"}, {"Mauritius", "මුරුසි සම්මත වේලාව"}, {"Mawson", "මොව්සන් වේලාව"}, {"Mexico_Pacific", "මෙක්සිකෝ පැසිෆික් සම්මත වේලාව"}, {"Mongolia", "උලාන් බාටර් සම්මත වේලාව"}, {"Moscow", "මොස්කව් සම්මත වේලාව"}, {"Myanmar", "මියන්මාර් වේලාව"}, {"Nauru", "නාවුරු වේලාව"}, {"Nepal", "නේපාල වේලාව"}, {"New_Caledonia", "නව සෙලඩොනියානු සම්මත වේලාව"}, {"New_Zealand", "නවසීලන්ත සම්මත වේලාව"}, {"Newfoundland", "නිව්ෆවුන්ලන්ත සම්මත වේලාව"}, {"Niue", "නියු වේලාව"}, {"Norfolk", "නොෆොල්ක් දුපත් සම්මත වේලාව"}, {"Noronha", "ෆර්නැන්ඩෝ ඩි නොරොන්හා සම්මත වේලාව"}, {"Novosibirsk", "නොවසිබිර්ස්ක් සම්මත වේලාව"}, {"Omsk", "ඔම්ස්ක් සම්මත වේලාව"}, {"Pakistan", "පාකිස්ථාන සම්මත වේලාව"}, {"Palau", "පලාවු වේලාව"}, {"Papua_New_Guinea", "පැපුවා නිව් ගිනීයා වේලාව"}, {"Paraguay", "පැරගුවේ සම්මත වේලාව"}, {"Peru", "පේරු සම්මත වේලාව"}, {"Philippines", "පිලිපීන සම්මත වේලාව"}, {"Phoenix_Islands", "ෆීනික්ස් දුපත් වේලාව"}, {"Pierre_Miquelon", "ශාන්ත පියරේ සහ මැකෝලන් සම්මත වේලාව"}, {"Pitcairn", "පිට්කෙයාන් වේලාව"}, {"Ponape", "පොනපේ වේලාව"}, {"Pyongyang", "ප්යොන්ග්යන් වේලාව"}, {"Reunion", "රියුනියන් වේලාව"}, {"Rothera", "රොතෙරා වේලාව"}, {"Sakhalin", "සඛලින් සම්මත වේලාව"}, {"Samoa", "සැමෝවා සම්මත වේලාව"}, {"Seychelles", "සීෂෙල්ස් වේලාව"}, {"Singapore", "සිංගප්පුරු වේලාව"}, {"Solomon", "සොලොමන් දූපත් වේලාව"}, {"South_Georgia", "දකුණු ජෝජියා වේලාව"}, {"Suriname", "සුරිනාම වේලාව"}, {"Syowa", "ස්යෝවා වේලාව"}, {"Tahiti", "ටාහිටි වේලාව"}, {"Taipei", "තායිපේ සම්මත වේලාව"}, {"Tajikistan", "ටජිකිස්තාන වේලාව"}, {"Tokelau", "ටොකෙලාවු වේලාව"}, {"Tonga", "ටොංගා සම්මත වේලාව"}, {"Truk", "චුක් වේලාව"}, {"Turkmenistan", "ටර්ක්මෙනිස්තාන සම්මත වේලාව"}, {"Tuvalu", "ටුවාලු වේලාව"}, {"Uruguay", "උරුගුවේ සම්මත වේලාව"}, {"Uzbekistan", "උස්බෙකිස්තාන සම්මත වේලාව"}, {"Vanuatu", "වනුආටු සම්මත වේලාව"}, {"Venezuela", "වෙනිසියුලා වේලාව"}, {"Vladivostok", "ව්ලදිවෝස්ටෝක් සම්මත වේලාව"}, {"Volgograd", "වොල්ගොග්\u200dරාඩ් සම්මත වේලාව"}, {"Vostok", "වොස්ටොක් වේලාව"}, {"Wake", "වේක් දූපත් වේලාව"}, {"Wallis", "වැලිස් සහ ෆුටුනා වේලාව"}, {"Yakutsk", "යකුට්ස්ක් සම්මත වේලාව"}, {"Yekaterinburg", "යෙකටෙරින්බර්ග් සම්මත වේලාව"}, {"Yukon", "යුකොන් වේලාව"}},
		{{"Alaska", "ඇලස්කා දිවාආලෝක වේලාව"}, {"Amazon", "ඇමර්සන් ග්\u200dරීෂ්ම කාලය"}, {"America_Central", "උතුරු ඇමරිකානු මධ්\u200dයම දිවාආලෝක වේලාව"}, {"America_Eastern", "උතුරු ඇමරිකානු නැගෙනහිර දිවාආලෝක වේලාව"}, {"America_Mountain", "උතුරු ඇමරිකානු කඳුකර දිවාආලෝක වේලාව"}, {"America_Pacific", "උතුරු ඇමරිකානු පැසිෆික් දිවාආලෝක වේලාව"}, {"Apia", "අපියා දිවා වේලාව"}, {"Arabian", "අරාබි දහවල් වේලාව"}, {"Argentina", "ආර්ජන්ටිනා ග්\u200dරීෂ්ම කාලය"}, {"Argentina_Western", "බටහිර ආර්ජන්ටිනා ග්\u200dරීෂ්ම කාලය"}, {"Armenia", "ආමේනියානු ග්\u200dරීෂ්ම වේලාව"}, {"Atlantic", "අත්ලාන්තික් දිවාආලෝක වේලාව"}, {"Australia_Central", "මධ්\u200dයම ඔස්ට්\u200dරේලියානු දහවල් වේලාව"}, {"Australia_CentralWestern", "මධ්\u200dයම බටහිර ඔස්ට්\u200dරේලියානු දහවල් වේලාව"}, {"Australia_Eastern", "නැඟෙනහිර ඕස්ට්\u200dරේලියානු දහවල් වේලාව"}, {"Australia_Western", "බටහිර ඔස්ට්\u200dරේලියානු දහවල් වේලාව"}, {"Azerbaijan", "අසර්බයිජාන් ග්\u200dරීෂ්ම වේලාව"}, {"Azores", "ඇසොර්ස් ග්\u200dරීෂ්ම වේලාව"}, {"Bangladesh", "බංගලාදේශ ග්\u200dරීෂ්ම කාලය"}, {"Brasilia", "බ්\u200dරසීල ග්\u200dරීෂ්ම කාලය"}, {"Cape_Verde", "කේප්වේඩ් ග්\u200dරීෂ්ම කාලය"}, {"Chatham", "චැතම් දිවා වේලාව"}, {"Chile", "චිලී ග්\u200dරීෂ්ම කාලය"}, {"China", "චීන දහවල් වේලාව"}, {"Colombia", "කොලොම්බියා ග්\u200dරීෂ්ම කාලය"}, {"Cook", "කුක් දුපත් භාග ග්\u200dරීෂ්ම වේලාව"}, {"Cuba", "කියුබානු දිවාආලෝක වේලාව"}, {"Easter", "ඊස්ටර් දූපත් ග්\u200dරීෂ්ම කාලය"}, {"Europe_Central", "මධ්\u200dයම යුරෝපීය ග්\u200dරීෂ්ම වේලාව"}, {"Europe_Eastern", "නැගෙනහිර යුරෝපීය ග්\u200dරීෂ්ම වේලාව"}, {"Europe_Western", "බටහිර යුරෝපීය ග්\u200dරීෂ්ම වේලාව"}, {"Falkland", "ෆෝක්ලන්ඩ් දූපත් ග්\u200dරීෂ්ම කාලය"}, {"Fiji", "ෆිජි ග්\u200dරීෂ්ම වේලාව"}, {"Georgia", "ජෝර්ජියානු ග්\u200dරීෂ්ම වේලාව"}, {"Greenland_Eastern", "නැගෙනහිර ග්\u200dරීන්ලන්ත ග්\u200dරීෂ්ම කාලය"}, {"Greenland_Western", "බටහිර ග්\u200dරීන්ලන්ත ග්\u200dරීෂ්ම කාලය"}, {"Hawaii_Aleutian", "හවායි-අලෙයුතියාන් දිවාආලෝක වේලාව"}, {"Hong_Kong", "හොංකොං ග්\u200dරීෂ්ම වේලාව"}, {"Hovd", "හොව්ඩ් ග්\u200dරීෂ්ම වේලාව"}, {"Iran", "ඉරාන දිවා කාලය"}, {"Irkutsk", "ඉර්කුට්ස්ක් ග්\u200dරීෂ්ම වේලාව"}, {"Israel", "ඊශ්\u200dරායල දහවල් වේලාව"}, {"Japan", "ජපාන දහවල් වේලාව"}, {"Korea", "කොරියානු දහවල් වේලාව"}, {"Krasnoyarsk", "ක්\u200dරස්නොයාර්ස්ක් ග්\u200dරීෂ්ම වේලාව"}, {"Lord_Howe", "ලෝර්ඩ් හෝව් දිවා වේලාව"}, {"Magadan", "මෙගඩන් ග්\u200dරීෂ්ම වේලාව"}, {"Mauritius", "මුරුසි ග්\u200dරීෂ්ම කාලය"}, {"Mexico_Pacific", "මෙක්සිකෝ පැසිෆික් දිවාආලෝක වේලාව"}, {"Mongolia", "උලාන් බාටර් ග්\u200dරීෂ්ම වේලාව"}, {"Moscow", "මොස්කව් ග්\u200dරීෂ්ම වේලාව"}, {"New_Caledonia", "නව සෙලඩොනියානු ග්\u200dරීෂ්ම වේලාව"}, {"New_Zealand", "නවසීලන්ත දිවා වේලාව"}, {"Newfoundland", "නිව්ෆවුන්ලන්ත දිවාආලෝක වේලාව"}, {"Norfolk", "නොෆොල්ක් දුපත් ග්\u200dරීෂ්ම වේලාව"}, {"Noronha", "ෆර්නැන්ඩෝ ඩි නොරොන්හා ග්\u200dරීෂ්ම කාලය"}, {"Novosibirsk", "නොවසිබිර්ස්ක් ග්\u200dරීෂ්ම වේලාව"}, {"Omsk", "ඔම්ස්ක් ග්\u200dරීෂ්ම වේලාව"}, {"Pakistan", "පාකිස්ථාන ග්\u200dරීෂ්ම කාලය"}, {"Paraguay", "පැරගුවේ ග්\u200dරීෂ්ම කාලය"}, {"Peru", "පේරු ග්\u200dරීෂ්ම කාලය"}, {"Philippines", "පිලිපීන ග්\u200dරීෂ්ම වේලාව"}, {"Pierre_Miquelon", "ශාන්ත පියරේ සහ මැකෝලන් දිවාආලෝක වේලාව"}, {"Sakhalin", "සඛලින් ග්\u200dරීෂ්ම වේලාව"}, {"Samoa", "සැමෝවා ග්\u200dරීෂ්ම වේලාව"}, {"Taipei", "තායිපේ දහවල් වේලාව"}, {"Tonga", "ටොංගා ග්\u200dරීෂ්ම වේලාව"}, {"Turkmenistan", "ටර්ක්මෙනිස්තාන ග්\u200dරීෂ්ම වේලාව"}, {"Uruguay", "උරුගුවේ ග්\u200dරීෂ්ම කාලය"}, {"Uzbekistan", "උස්බෙකිස්තාන ග්\u200dරීෂ්ම වේලාව"}, {"Vanuatu", "වනුආටු ගිම්හාන වේලාව"}, {"Vladivostok", "ව්ලදිවෝස්ටෝක් ග්\u200dරීෂ්ම වේලාව"}, {"Volgograd", "වොල්ගොග්\u200dරාඩ් ග්\u200dරීෂ්ම වේලාව"}, {"Yakutsk", "යකුට්ස්ක් ග්\u200dරීෂ්ම වේලාව"}, {"Yekaterinburg", "යෙකටෙරින්බර්ග් ග්\u200dරීෂ්ම වේලාව"}},
	},
	{
		{{"Acre", "acrejský štandardný čas"}, {"Afghanistan", "afganský čas"}, {"Africa_Central", "stredoafrický čas"}, {"Africa_Eastern", "východoafrický čas"}, {"Africa_Southern", "juhoafrický čas"}, {"Africa_Western", "západoafrický čas"}, {"Alaska", "aljašský štandardný čas"}, {"Almaty", "almaatský štandardný čas"}, {"Amazon", "amazonský štandardný čas"}, {"America_Central", "severoamerický centrálny štandardný čas"}, {"America_Eastern", "severoamerický východný štandardný čas"}, {"America_Mountain", "severoamerický horský štandardný čas"}, {"America_Pacific", "severoamerický tichomorský štandardný čas"}, {"Anadyr", "Anadyrský štandardný čas"}, {"Apia", "apijský štandardný čas"}, {"Aqtau", "aktauský štandardný čas"}, {"Aqtobe", "aktobský štandardný čas"}, {"Arabian", "arabský štandardný čas"}, {"Argentina", "argentínsky štandardný čas"}, {"Argentina_Western", "západoargentínsky štandardný čas"}, {"Armenia", "arménsky štandardný čas"}, {"Atlantic", "atlantický štandardný čas"}, {"Australia_Central", "stredoaustrálsky štandardný čas"}, {"Australia_CentralWestern", "stredozápadný austrálsky štandardný čas"}, {"Australia_Eastern", "východoaustrálsky štandardný čas"}, {"Australia_Western", "západoaustrálsky štandardný čas"}, {"Azerbaijan", "azerbajdžanský štandardný čas"}, {"Azores", "azorský štandardný čas"}, {"Bangladesh", "bangladéšsky štandardný čas"}, {"Bhutan", "bhutánsky čas"}, {"Bolivia", "bolívijský čas"}, {"Brasilia", "brazílsky štandardný čas"}, {"Brunei", "brunejský čas"}, {"Cape_Verde", "kapverdský štandardný čas"}, {"Casey", "čas Caseyho stanice"}, {"Chamorro", "chamorrský čas"}, {"Chatham", "chathamský štandardný čas"}, {"Chile", "čilský štandardný čas"}, {"China", "čínsky štandardný čas"}, {"Christmas", "čas Vianočného ostrova"}, {"Cocos", "čas Kokosových ostrovov"}, {"Colombia", "kolumbijský štandardný čas"}, {"Cook", "štandardný čas Cookových ostrovov"}, {"Cuba", "kubánsky štandardný čas"}, {"Davis", "čas Davisovej stanice"}, {"DumontDUrville", "čas stanice Dumonta d’Urvillea"}, {"East_Timor", "východotimorský čas"}, {"Easter", "štandardný čas Veľkonočného ostrova"}, {"Ecuador", "ekvádorský čas"}, {"Europe_Central", "stredoeurópsky štandardný čas"}, {"Europe_Eastern", "východoeurópsky štandardný čas"}, {"Europe_Further_Eastern", "minský čas"}, {"Europe_Western", "západoeurópsky štandardný čas"}, {"Falkland", "falklandský štandardný čas"}, {"Fiji", "fidžijský štandardný čas"}, {"French_Guiana", "francúzskoguyanský čas"}, {"French_Southern", "čas Francúzskych južných a antarktických území"}, {"GMT", "greenwichský čas"}, {"Galapagos", "galapágsky čas"}, {"Gambier", "gambierský čas"}, {"Georgia", "gruzínsky štandardný čas"}, {"Gilbert_Islands", "čas Gilbertových ostrovov"}, {"Greenland_Eastern", "východogrónsky štandardný čas"}, {"Greenland_Western", "západogrónsky štandardný čas"}, {"Guam", "guamský čas"}, {"Gulf", "štandardný čas Perzského zálivu"}, {"Guyana", "guyanský čas"}, {"Hawaii", "havajsko-aleutský štandardný čas"}, {"Hawaii_Aleutian", "havajsko-aleutský štandardný čas"}, {"Hong_Kong", "hongkonský štandardný čas"}, {"Hovd", "chovdský štandardný čas"}, {"India", "indický čas"}, {"Indian_Ocean", "indickooceánsky čas"}, {"Indochina", "indočínsky čas"}, {"Indonesia_Central", "stredoindonézsky čas"}, {"Indonesia_Eastern", "východoindonézsky čas"}, {"Indonesia_Western", "západoindonézsky čas"}, {"Iran", "iránsky štandardný čas"}, {"Irkutsk", "irkutský štandardný čas"}, {"Israel", "izraelský štandardný čas"}, {"Japan", "japonský štandardný čas"}, {"Kamchatka", "Petropavlovsk-Kamčatský štandardný čas"}, {"Kazakhstan", "kazachstanský čas"}, {"Kazakhstan_Eastern", "východokazachstanský čas"}, {"Kazakhstan_Western", "západokazachstanský čas"}, {"Korea", "kórejský štandardný čas"}, {"Kosrae", "kosrajský čas"}, {"Krasnoyarsk", "krasnojarský štandardný čas"}, {"Kyrgystan", "kirgizský čas"}, {"Lanka", "srílanský čas"}, {"Line_Islands", "čas Rovníkových ostrovov"}, {"Lord_Howe", "štandardný čas ostrova lorda Howa"}, {"Macau", "macajský štandardný čas"}, {"Magadan", "magadanský štandardný čas"}, {"Malaysia", "malajzijský čas"}, {"Maldives", "maldivský čas"}, {"Marquesas", "markézsky čas"}, {"Marshall_Islands", "čas Marshallových ostrovov"}, {"Mauritius", "maurícijský štandardný čas"}, {"Mawson", "čas Mawsonovej stanice"}, {"Mexico_Pacific", "mexický tichomorský štandardný čas"}, {"Mongolia", "ulanbátarský štandardný čas"}, {"Moscow", "moskovský štandardný čas"}, {"Myanmar", "mjanmarský čas"}, {"Nauru", "nauruský čas"}, {"Nepal", "nepálsky čas"}, {"New_Caledonia", "novokaledónsky štandardný čas"}, {"New_Zealand", "novozélandský štandardný čas"}, {"Newfoundland", "newfoundlandský štandardný čas"}, {"Niue", "niuejský čas"}, {"Norfolk", "norfolský štandardný čas"}, {"Noronha", "štandardný čas súostrovia Fernando de Noronha"}, {"North_Mariana", "severomariánsky čas"}, {"Novosibirsk", "novosibirský štandardný čas"}, {"Omsk", "omský štandardný čas"}, {"Pakistan", "pakistanský štandardný čas"}, {"Palau", "palauský čas"}, {"Papua_New_Guinea", "čas Papuy-Novej Guiney"}, {"Paraguay", "paraguajský štandardný čas"}, {"Peru", "peruánsky štandardný čas"}, {"Philippines", "filipínsky štandardný čas"}, {"Phoenix_Islands", "čas Fénixových ostrovov"}, {"Pierre_Miquelon", "pierre-miquelonský štandardný čas"}, {"Pitcairn", "čas Pitcairnových ostrovov"}, {"Ponape", "ponapský čas"}, {"Pyongyang", "pchjongjanský čas"}, {"Qyzylorda", "kyzylordský štandardný čas"}, {"Reunion", "réunionský čas"}, {"Rothera", "čas Rotherovej stanice"}, {"Sakhalin", "sachalinský štandardný čas"}, {"Samara", "Samarský štandardný čas"}, {"Samoa", "samojský štandardný čas"}, {"Seychelles", "seychelský čas"}, {"Singapore", "singapurský štandardný čas"}, {"Solomon", "čas Šalamúnových ostrovov"}, {"South_Georgia", "čas Južnej Georgie"}, {"Suriname", "surinamský čas"}, {"Syowa", "čas stanice Šówa"}, {"Tahiti", "tahitský čas"}, {"Taipei", "tchajpejský štandardný čas"}, {"Tajikistan", "tadžický čas"}, {"Tokelau", "tokelauský čas"}, {"Tonga", "tonžský štandardný čas"}, {"Truk", "chuukský čas"}, {"Turkmenistan", "turkménsky štandardný čas"}, {"Tuvalu", "tuvalský čas"}, {"Uruguay", "uruguajský štandardný čas"}, {"Uzbekistan", "uzbecký štandardný čas"}, {"Vanuatu", "vanuatský štandardný čas"}, {"Venezuela", "venezuelský čas"}, {"Vladivostok", "vladivostocký štandardný čas"}, {"Volgograd", "volgogradský štandardný čas"}, {"Vostok", "čas stanice Vostok"}, {"Wake", "čas ostrova Wake"}, {"Wallis", "čas ostrovov Wallis a Futuna"}, {"Yakutsk", "jakutský štandardný čas"}, {"Yekaterinburg", "jekaterinburský štandardný čas"}, {"Yukon", "yukonský čas"}},
		{{"Acre", "acrejský letný čas"}, {"Alaska", "aljašský letný čas"}, {"Almaty", "almaatský letný čas"}, {"Amazon", "amazonský letný čas"}, {"America_Central", "severoamerický centrálny letný čas"}, {"America_Eastern", "severoamerický východný letný čas"}, {"America_Mountain", "severoamerický horský letný čas"}, {"America_Pacific", "severoamerický tichomorský letný čas"}, {"Anadyr", "Anadyrský letný čas"}, {"Apia", "apijský letný čas"}, {"Aqtau", "aktauský letný čas"}, {"Aqtobe", "aktobský letný čas"}, {"Arabian", "arabský letný čas"}, {"Argentina", "argentínsky letný čas"}, {"Argentina_Western", "západoargentínsky letný čas"}, {"Armenia", "arménsky letný čas"}, {"Atlantic", "atlantický letný čas"}, {"Australia_Central", "stredoaustrálsky letný čas"}, {"Australia_CentralWestern", "stredozápadný austrálsky letný čas"}, {"Australia_Eastern", "východoaustrálsky letný čas"}, {"Australia_Western", "západoaustrálsky letný čas"}, {"Azerbaijan", "azerbajdžanský letný čas"}, {"Azores", "azorský letný čas"}, {"Bangladesh", "bangladéšsky letný čas"}, {"Brasilia", "brazílsky letný čas"}, {"Cape_Verde", "kapverdský letný čas"}, {"Chatham", "chathamský letný čas"}, {"Chile", "čilský letný čas"}, {"China", "čínsky letný čas"}, {"Colombia", "kolumbijský letný čas"}, {"Cook", "letný čas Cookových ostrovov"}, {"Cuba", "kubánsky letný čas"}, {"Easter", "letný čas Veľkonočného ostrova"}, {"Europe_Central", "stredoeurópsky letný čas"}, {"Europe_Eastern", "východoeurópsky letný čas"}, {"Europe_Western", "západoeurópsky letný čas"}, {"Falkland", "falklandský letný čas"}, {"Fiji", "fidžijský letný čas"}, {"Georgia", "gruzínsky letný čas"}, {"Greenland_Eastern", "východogrónsky letný čas"}, {"Greenland_Western", "západogrónsky letný čas"}, {"Hawaii_Aleutian", "havajsko-aleutský letný čas"}, {"Hong_Kong", "hongkonský letný čas"}, {"Hovd", "chovdský letný čas"}, {"Iran", "iránsky letný čas"}, {"Irkutsk", "irkutský letný čas"}, {"Israel", "izraelský letný čas"}, {"Japan", "japonský letný čas"}, {"Kamchatka", "Petropavlovsk-Kamčatskijský letný čas"}, {"Korea", "kórejský letný čas"}, {"Krasnoyarsk", "krasnojarský letný čas"}, {"Lord_Howe", "letný čas ostrova lorda Howa"}, {"Macau", "macajský letný čas"}, {"Magadan", "magadanský letný čas"}, {"Mauritius", "maurícijský letný čas"}, {"Mexico_Pacific", "mexický tichomorský letný čas"}, {"Mongolia", "ulanbátarský letný čas"}, {"Moscow", "moskovský letný čas"}, {"New_Caledonia", "novokaledónsky letný čas"}, {"New_Zealand", "novozélandský letný čas"}, {"Newfoundland", "newfoundlandský letný čas"}, {"Norfolk", "norfolský letný čas"}, {"Noronha", "letný čas súostrovia Fernando de Noronha"}, {"Novosibirsk", "novosibirský letný čas"}, {"Omsk", "omský letný čas"}, {"Pakistan", "pakistanský letný čas"}, {"Paraguay", "paraguajský letný čas"}, {"Peru", "peruánsky letný čas"}, {"Philippines", "filipínsky letný čas"}, {"Pierre_Miquelon", "pierre-miquelonský letný čas"}, {"Qyzylorda", "kyzylordský letný čas"}, {"Sakhalin", "sachalinský letný čas"}, {"Samara", "Samarský letný čas"}, {"Samoa", "samojský letný čas"}, {"Taipei", "tchajpejský letný čas"}, {"Tonga", "tonžský letný čas"}, {"Turkmenistan", "turkménsky letný čas"}, {"Uruguay", "uruguajský letný čas"}, {"Uzbekistan", "uzbecký letný čas"}, {"Vanuatu", "vanuatský letný čas"}, {"Vladivostok", "vladivostocký letný čas"}, {"Volgograd", "volgogradský letný čas"}, {"Yakutsk", "jakutský letný čas"}, {"Yekaterinburg", "jekaterinburský letný čas"}},
	},
	{
		{{"GMT", "جی ایم ٹی"}},
		{},
	},
	{
		{{"Afghanistan", "Afganistanski čas"}, {"Africa_Central", "Centralnoafriški čas"}, {"Africa_Eastern", "Vzhodnoafriški čas"}, {"Africa_Southern", "Južnoafriški čas"}, {"Africa_Western", "Zahodnoafriški čas"}, {"Alaska", "Aljaški standardni čas"}, {"Amazon", "Amazonski standardni čas"}, {"America_Central", "Centralni standardni čas"}, {"America_Eastern", "Vzhodni standardni čas"}, {"America_Mountain", "Gorski standardni čas"}, {"America_Pacific", "Pacifiški standardni čas"}, {"Anadyr", "Anadirski standardni čas"}, {"Apia", "Standardni čas: Apia"}, {"Arabian", "Arabski standardni čas"}, {"Argentina", "Argentinski standardni čas"}, {"Argentina_Western", "Argentinski zahodni standardni čas"}, {"Armenia", "Armenski standardni čas"}, {"Atlantic", "Atlantski standardni čas"}, {"Australia_Central", "Avstralski centralni standardni čas"}, {"Australia_CentralWestern", "Avstralski centralni zahodni standardni čas"}, {"Australia_Eastern", "Avstralski vzhodni standardni čas"}, {"Australia_Western", "Avstralski zahodni standardni čas"}, {"Azerbaijan", "Azerbajdžanski standardni čas"}, {"Azores", "Azorski standardni čas"}, {"Bangladesh", "Bangladeški standardni čas"}, {"Bhutan", "Butanski čas"}, {"Bolivia", "Bolivijski čas"}, {"Brasilia", "Brasilski standardni čas"}, {"Brunei", "Brunejski čas"}, {"Cape_Verde", "Kapverdski standardni čas"}, {"Chamorro", "Čamorski standardni čas"}, {"Chatham", "Čatamski standardni čas"}, {"Chile", "Čilski standardni čas"}, {"China", "Kitajski standardni čas"}, {"Christmas", "Božičnootoški čas"}, {"Cocos", "Čas: Kokosovi otoki"}, {"Colombia", "Kolumbijski standardni čas"}, {"Cook", "Cookovootoški standardni čas"}, {"Cuba", "Kubanski standardni čas"}, {"Davis", "Čas: Davis"}, {"DumontDUrville", "Čas: Dumont-d’Urville"}, {"East_Timor", "Vzhodnotimorski čas"}, {"Easter", "Standardni čas: Velikonočni otok"}, {"Ecuador", "Ekvadorski čas"}, {"Europe_Central", "Srednjeevropski standardni čas"}, {"Europe_Eastern", "Vzhodnoevropski standardni čas"}, {"Europe_Further_Eastern", "Dodatni vzhodnoevropski čas"}, {"Europe_Western", "Zahodnoevropski standardni čas"}, {"Falkland", "Standardni čas: Falklandsko otočje"}, {"Fiji", "Fidžijski standardni čas"}, {"French_Guiana", "Čas: Francoska Gvajana"}, {"French_Southern", "Francoski južni in antarktični čas"}, {"GMT", "Greenwiški srednji čas"}, {"Galapagos", "Galapaški čas"}, {"Gambier", "Gambierski čas"}, {"Georgia", "Gruzijski standardni čas"}, {"Gilbert_Islands", "Čas: Gilbertovi otoki"}, {"Greenland_Eastern", "Vzhodnogrenlandski standardni čas"}, {"Greenland_Western", "Zahodnogrenlandski standardni čas"}, {"Gulf", "Zalivski standardni čas"}, {"Guyana", "Gvajanski čas"}, {"Hawaii", "Havajski aleutski standardni čas"}, {"Hawaii_Aleutian", "Havajski aleutski standardni čas"}, {"Hong_Kong", "Hongkonški standardni čas"}, {"Hovd", "Hovdski standardni čas"}, {"India", "Indijski standardni čas"}, {"Indian_Ocean", "Indijskooceanski čas"}, {"Indochina", "Indokitajski čas"}, {"Indonesia_Central", "Indonezijski osrednji čas"}, {"Indonesia_Eastern", "Indonezijski vzhodni čas"}, {"Indonesia_Western", "Indonezijski zahodni čas"}, {"Iran", "Iranski standardni čas"}, {"Irkutsk", "Irkutski standardni čas"}, {"Israel", "Izraelski standardni čas"}, {"Japan", "Japonski standardni čas"}, {"Kamchatka", "Petropavlovsk-Kamčatski standardni čas"}, {"Kazakhstan", "Kazahstanski čas"}, {"Kazakhstan_Eastern", "Vzhodni kazahstanski čas"}, {"Kazakhstan_Western", "Zahodni kazahstanski čas"}, {"Korea", "Korejski standardni čas"}, {"Kosrae", "Kosrajški čas"}, {"Krasnoyarsk", "Krasnojarski standardni čas"}, {"Kyrgystan", "Kirgizistanski čas"}, {"Line_Islands", "Čas: Ekvatorski otoki"}, {"Lord_Howe", "Standardni čas otoka Lord Howe"}, {"Magadan", "Magadanski standardni čas"}, {"Malaysia", "Malezijski čas"}, {"Maldives", "Maldivski čas"}, {"Marquesas", "Čas: Markizni otoki"}, {"Marshall_Islands", "Čas: Marshallovi otoki"}, {"Mauritius", "Mauricijski standardni čas"}, {"Mawson", "Mawsonski čas"}, {"Mexico_Pacific", "Mehiški pacifiški standardni čas"}, {"Mongolia", "Ulanbatorski standardni čas"}, {"Moscow", "Moskovski standardni čas"}, {"Myanmar", "Mjanmarski čas"}, {"Nauru", "Naurujski čas"}, {"Nepal", "Nepalski čas"}, {"New_Caledonia", "Novokaledonijski standardni čas"}, {"New_Zealand", "Novozelandski standardni čas"}, {"Newfoundland", "Novofundlandski standardni čas"}, {"Niue", "Niuejski čas"}, {"Norfolk", "Standardni čas: Norfolški otoki"}, {"Noronha", "Fernando de Noronški standardni čas"}, {"Novosibirsk", "Novosibirski standardni čas"}, {"Omsk", "Omski standardni čas"}, {"Pakistan", "Pakistanski standardni čas"}, {"Palau", "Palavski čas"}, {"Papua_New_Guinea", "Papuanski čas"}, {"Paraguay", "Paragvajski standardni čas"}, {"Peru", "Perujski standardni čas"}, {"Philippines", "Filipinski standardni čas"}, {"Phoenix_Islands", "Čas: Otočje Feniks"}, {"Pierre_Miquelon", "Standardni čas: Saint Pierre in Miquelon"}, {"Pitcairn", "Pitcairnski čas"}, {"Ponape", "Ponapski čas"}, {"Pyongyang", "Pjongjanški čas"}, {"Reunion", "Reunionski čas"}, {"Rothera", "Rotherski čas"}, {"Sakhalin", "Sahalinski standardni čas"}, {"Samara", "Samarski standardni čas"}, {"Samoa", "Samoanski standardni čas"}, {"Seychelles", "Sejšelski čas"}, {"Singapore", "Singapurski standardni čas"}, {"Solomon", "Salomonovootoški čas"}, {"South_Georgia", "Južnogeorgijski čas"}, {"Suriname", "Surinamski čas"}, {"Syowa", "Čas: Syowa"}, {"Tahiti", "Tahitijski čas"}, {"Taipei", "Tajpejski standardni čas"}, {"Tajikistan", "Tadžikistanski čas"}, {"Tokelau", "Tokelavski čas"}, {"Tonga", "Tongovski standardni čas"}, {"Truk", "Čas: Otok Chuuk"}, {"Turkey", "Turški standardni čas"}, {"Turkmenistan", "Turkmenistanski standardni čas"}, {"Tuvalu", "Tuvalujski čas"}, {"Uruguay", "Urugvajski standardni čas"}, {"Uzbekistan", "Uzbekistanski standardni čas"}, {"Vanuatu", "Vanuatujski standardni čas"}, {"Venezuela", "Venezuelski čas"}, {"Vladivostok", "Vladivostoški standardni čas"}, {"Volgograd", "Volgograjski standardni čas"}, {"Vostok", "Vostoški čas"}, {"Wake", "Čas: Otok Wake"}, {"Wallis", "Čas: Wallis in Futuna"}, {"Yakutsk", "Jakutski standardni čas"}, {"Yekaterinburg", "Jekaterinburški standardni čas"}, {"Yukon", "Jukonski čas"}},
		{{"Alaska", "Aljaški poletni čas"}, {"Amazon", "Amazonski poletni čas"}, {"America_Central", "Centralni poletni čas"}, {"America_Eastern", "Vzhodni poletni čas"}, {"America_Mountain", "Gorski poletni čas"}, {"America_Pacific", "Pacifiški poletni čas"}, {"Anadyr", "Anadirski poletni čas"}, {"Apia", "Poletni čas: Apia"}, {"Arabian", "Arabski poletni čas"}, {"Argentina", "Argentinski poletni čas"}, {"Argentina_Western", "Argentinski zahodni poletni čas"}, {"Armenia", "Armenski poletni čas"}, {"Atlantic", "Atlantski poletni čas"}, {"Australia_Central", "Avstralski centralni poletni čas"}, {"Australia_CentralWestern", "Avstralski centralni zahodni poletni čas"}, {"Australia_Eastern", "Avstralski vzhodni poletni čas"}, {"Australia_Western", "Avstralski zahodni poletni čas"}, {"Azerbaijan", "Azerbajdžanski poletni čas"}, {"Azores", "Azorski poletni čas"}, {"Bangladesh", "Bangladeški poletni čas"}, {"Brasilia", "Brasilski poletni čas"}, {"Cape_Verde", "Kapverdski poletni čas"}, {"Chatham", "Čatamski poletni čas"}, {"Chile", "Čilski poletni čas"}, {"China", "Kitajski poletni čas"}, {"Colombia", "Kolumbijski poletni čas"}, {"Cook", "Cookovootoški srednjepoletni čas"}, {"Cuba", "Kubanski poletni čas"}, {"Easter", "Poletni čas: Velikonočni otok"}, {"Europe_Central", "Srednjeevropski poletni čas"}, {"Europe_Eastern", "Vzhodnoevropski poletni čas"}, {"Europe_Western", "Zahodnoevropski poletni čas"}, {"Falkland", "Poletni čas: Falklandsko otočje"}, {"Fiji", "Fidžijski poletni čas"}, {"Georgia", "Gruzijski poletni čas"}, {"Greenland_Eastern", "Vzhodnogrenlandski poletni čas"}, {"Greenland_Western", "Zahodnogrenlandski poletni čas"}, {"Hawaii_Aleutian", "Havajski aleutski poletni čas"}, {"Hong_Kong", "Hongkonški poletni čas"}, {"Hovd", "Hovdski poletni čas"}, {"Iran", "Iranski poletni čas"}, {"Irkutsk", "Irkutski poletni čas"}, {"Israel", "Izraelski poletni čas"}, {"Japan", "Japonski poletni čas"}, {"Kamchatka", "Petropavlovsk-Kamčatski poletni čas"}, {"Korea", "Korejski poletni čas"}, {"Krasnoyarsk", "Krasnojarski poletni čas"}, {"Lord_Howe", "Poletni čas otoka Lord Howe"}, {"Magadan", "Magadanski poletni čas"}, {"Mauritius", "Mauricijski poletni čas"}, {"Mexico_Pacific", "Mehiški pacifiški poletni čas"}, {"Mongolia", "Ulanbatorski poletni čas"}, {"Moscow", "Moskovski poletni čas"}, {"New_Caledonia", "Novokaledonijski poletni čas"}, {"New_Zealand", "Novozelandski poletni čas"}, {"Newfoundland", "Novofundlandski poletni čas"}, {"Norfolk", "Poletni čas: Norfolški otoki"}, {"Noronha", "Fernando de Noronški poletni čas"}, {"Novosibirsk", "Novosibirski poletni čas"}, {"Omsk", "Omski poletni čas"}, {"Pakistan", "Pakistanski poletni čas"}, {"Paraguay", "Paragvajski poletni čas"}, {"Peru", "Perujski poletni čas"}, {"Philippines", "Filipinski poletni čas"}, {"Pierre_Miquelon", "Poletni čas: Saint Pierre in Miquelon"}, {"Sakhalin", "Sahalinski poletni čas"}, {"Samara", "Samarski poletni čas"}, {"Samoa", "Samoanski poletni čas"}, {"Taipei", "Tajpejski poletni čas"}, {"Tonga", "Tongovski poletni čas"}, {"Turkey", "Turški poletni čas"}, {"Turkmenistan", "Turkmenistanski poletni čas"}, {"Uruguay", "Urugvajski poletni čas"}, {"Uzbekistan", "Uzbekistanski poletni čas"}, {"Vanuatu", "Vanuatujski poletni čas"}, {"Vladivostok", "Vladivostoški poletni čas"}, {"Volgograd", "Volgograjski poletni čas"}, {"Yakutsk", "Jakutski poletni čas"}, {"Yekaterinburg", "Jekaterinburški poletni čas"}},
	},
	{
		{{"Acre", "Wakhtiga Caadiga ah ee Acre"}, {"Afghanistan", "Waqtiga Afggaanistaan"}, {"Africa_Central", "Waqtiga Bartamaha Afrika"}, {"Africa_Eastern", "Waqtiga Bariga Afrika"}, {"Africa_Southern", "Waqtiyada Caadiga Ah ee Koonfur Afrika"}, {"Africa_Western", "Waqtiga Galbeedka Afrika"}, {"Alaska", "Waqtiga Caadiga Ah ee Alaska"}, {"Almaty", "Waqtiga Caadiga ah ee Almaty"}, {"Amazon", "Waqtiga Caadiga Ah ee Amason"}, {"America_Central", "Waqtiga Caadiga Ah ee Bartamaha Waqooyiga Ameerika"}, {"America_Eastern", "Waqtiga Caadiga Ah ee Bariga Waqooyiga Ameerika"}, {"America_Mountain", "Waqtiga Caadiga ah ee Buuraleyda Waqooyiga Ameerika"}, {"America_Pacific", "Waqtiga Caadiga ah ee Basifika Waqooyiga Ameerika"}, {"Anadyr", "Wakhtiga Caadiga ah ee Anadyr"}, {"Apia", "Waqtiga Caadiga Ah ee Samoa"}, {"Aqtau", "Waqtiga Caadiga ah ee Aqtau"}, {"Aqtobe", "Waqtiga Caadiga ah ee Aqtobe"}, {"Arabian", "Waqtiga Caadiga Ah ee Carabta"}, {"Argentina", "Waqtiga Caadiga Ah ee Arjentiina"}, {"Argentina_Western", "Waqtiga Caadiga Ah ee Galbeedka Arjentiina"}, {"Armenia", "Waqtiga Caadiga Ah ee Armeeniya"}, {"Atlantic", "Waqtiga Caadiga Ah ee Atlantika Waqooyiga Ameerika"}, {"Australia_Central", "Waqtiga Caadiga Ah ee Bartamaha Astaraaliya"}, {"Australia_CentralWestern", "Waqtiga Caadiga Ah ee Bartamaha Galbeedka Astaraaliya"}, {"Australia_Eastern", "Waqtiyada Caadiga ah ee Bariga Astaraaliya"}, {"Australia_Western", "Waqtiga Caadiga Ah ee Galbeedka Astaraaliya"}, {"Azerbaijan", "Waqtiga Caadiga Ah ee Asarbeyjan"}, {"Azores", "Waqtiga Caadiga Ah ee Asores"}, {"Bangladesh", "Waqtiga Caadiga Ah ee Bangledeesh"}, {"Bhutan", "Waqtiga Butaan"}, {"Bolivia", "Waqtiga Boliifiya"}, {"Brasilia", "Waqtiga Caadiga ah ee Baraasiliya"}, {"Brunei", "Wakhtiga Brunei"}, {"Cape_Verde", "Waqtiga Caadiga Ah ee Keyb Faarde"}, {"Chamorro", "Wakhtiga Caadiga ah ee Chamorro"}, {"Chatham", "Waqtiga Caadiga Ah ee Jaatam"}, {"Chile", "Waqtiga Caadiga Ah ee Jili"}, {"China", "Waqtiga Caadiga Ah ee Shiinaha"}, {"Christmas", "Waqtiga Kirismas Aylaan"}, {"Cocos", "Waqtiga Kokos Aylaan"}, {"Colombia", "Waqtiga Caadiga Ah ee Kolambiya"}, {"Cook", "Waqtiga Caadiga Ah ee Jasiiradaha Cook"}, {"Cuba", "Waqtiga Caadiga Ah ee Kuuba"}, {"Davis", "Wakhtiga Davis"}, {"DumontDUrville", "Waqtiga Dumont - d’urfille"}, {"East_Timor", "Wakhtiga Timor-Leste"}, {"Easter", "Waqtiga Caadiga Ah ee Iistar Aylaan"}, {"Ecuador", "Waqtiga Ekuwadoor"}, {"Europe_Central", "Waqtiga Caadiga Ah ee Bartamaha Yurub"}, {"Europe_Eastern", "Waqtiga Caadiga Ah ee Bariga Yurub"}, {"Europe_Further_Eastern", "Waqtiga Bariga Fog ee Yurub"}, {"Europe_Western", "Waqtiga Caadiga Ah ee Galbeedka Yurub"}, {"Falkland", "Waqtiga Caadiga Ah ee Faalklaan Aylaanis"}, {"Fiji", "Waqtiga Caadiga Ah ee Fiji"}, {"French_Guiana", "Waqtiga Ferenj Guyana"}, {"French_Southern", "Waqtiga Koonfurta Faransiiska & Antaarktik"}, {"GMT", "Wakhtiga Giriinwij"}, {"Galapagos", "Waqtiga Galabagos"}, {"Gambier", "Waqtiga Gambiyar"}, {"Georgia", "Waqtiga Caadiga Ah ee Joorjiya"}, {"Gilbert_Islands", "Waqtiga Jilbeert Aylaan"}, {"Greenland_Eastern", "Waqtiga Caadiga ah ee Bariga Giriinlaan"}, {"Greenland_Western", "Waqtiga Caadiga Ah ee Galbeedka Giriinlaan"}, {"Gulf", "Waqtiga Gacanka"}, {"Guyana", "Waqtiga Guyaana"}, {"Hawaii", "Waqtiga Caadiga Ah ee Hawaay-Alutiyaan"}, {"Hawaii_Aleutian", "Waqtiga Caadiga Ah ee Hawaay-Alutiyaan"}, {"Hong_Kong", "Waqtiga Caadiga Ah ee Hoong Koong"}, {"Hovd", "Waqtiga Caadiga Ah ee Hofud"}, {"India", "Waqtiga Caadiga Ah ee Hindiya"}, {"Indian_Ocean", "Waqtiga Badweynta Hindiya"}, {"Indochina", "Wakhtiga Indochina"}, {"Indonesia_Central", "Waqtiga Bartamaha Indoneeysiya"}, {"Indonesia_Eastern", "Waqtiga Indoneeysiya"}, {"Indonesia_Western", "Waqtiga Galbeedka Indoneeysiya"}, {"Iran", "Waqtiga Caadiga Ah ee Iiraan"}, {"Irkutsk", "Waqtiga Caadiga Ah ee Irkutsik"}, {"Israel", "Waqtiga Caadiga Ah ee Israaiil"}, {"Japan", "Waqtiga Caadiga Ah ee Jabaan"}, {"Kamchatka", "Wakhtiga Caadiga ah ee Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Wakhtiga Kazakhistan"}, {"Kazakhstan_Eastern", "Waqtiga Bariga Kasakhistaan"}, {"Kazakhstan_Western", "Waqtiga Koonfurta Kasakhistan"}, {"Korea", "Waqtiga Caadiga Ah ee Kuuriya"}, {"Kosrae", "Waqtiga Kosriy"}, {"Krasnoyarsk", "Waqtiga Caadiga Ah ee Karasnoyarsik"}, {"Kyrgystan", "Waqtiga Kiyrigistaan"}, {"Line_Islands", "Waqtiga Leyn Aylaan"}, {"Lord_Howe", "Waqtiga Caadiga Ah ee Lod How"}, {"Magadan", "Waqtiga Caadiga Ah ee Magedan"}, {"Malaysia", "Waqtiga Maleyshiya"}, {"Maldives", "Waqtiga Maldifis"}, {"Marquesas", "Waqtiga Marquwesas"}, {"Marshall_Islands", "Waqtiga Maarshaal Aylaanis"}, {"Mauritius", "Waqtiga Caadiga Ah ee Morishiyaas"}, {"Mawson", "Waqtiga Mawson"}, {"Mexico_Pacific", "Waqtiga Caadiga Ah ee Baasifikada Meksiko"}, {"Mongolia", "Waqtiga Caadiga Ah ee Ulaanbaataar"}, {"Moscow", "Waqtiga Caadiga Ah ee Moskow"}, {"Myanmar", "Waqtiga Mayanmaar"}, {"Nauru", "Waqtiga Nawroo"}, {"Nepal", "Waqtiga Neebaal"}, {"New_Caledonia", "Waqtiga Caadiga Ah ee Niyuu Kaledoniya"}, {"New_Zealand", "Waqtiga Caadiga Ah ee Niyuu Si’laan"}, {"Newfoundland", "Waqtiga Caadiga Ah ee Niyuufoonlaan"}, {"Niue", "Waqtiga Niyuu"}, {"Norfolk", "Waqtiga Caadiga ah ee Norfolk Island"}, {"Noronha", "Waqtiga Caadiga Ah ee Farnaando de Nooronha"}, {"Novosibirsk", "Waqtiga Caadiga Ah ee Nofosibirsik"}, {"Omsk", "Waqtiga Caadiga Ah ee Omsk"}, {"Pakistan", "Waqtiga Caadiga Ah ee Bakistaan"}, {"Palau", "Waqtiga Balaw"}, {"Papua_New_Guinea", "Waqtiga Babuw Niyuu Giniya"}, {"Paraguay", "Waqtiga Caadiga Ah ee Baragwaay"}, {"Peru", "Waqtiga Caadiga Ah ee Beeru"}, {"Philippines", "Waqtiga Caadiga Ah ee Filibiin"}, {"Phoenix_Islands", "Waqtiga Foonikis Aylaanis"}, {"Pierre_Miquelon", "Waqtiga Caadiga Ah St. Beere & Mikiwelon"}, {"Pitcairn", "Waqtiga Bitkeen"}, {"Ponape", "Wakhtiga Pohnpei"}, {"Pyongyang", "Waqtiga Boyongyang"}, {"Qyzylorda", "Waqtiga Caadiga ah ee Qyzylorda"}, {"Reunion", "Waqtiga Riyuuniyon"}, {"Rothera", "Waqtiga Rotera"}, {"Sakhalin", "Waqtiga Caadiga Ah ee Sakhalin"}, {"Samara", "Wakhtiga Caadiga ah ee Samara"}, {"Samoa", "Waqtiga Caadiga Ah ee American Samoa"}, {"Seychelles", "Waqtiga Siishalis"}, {"Singapore", "Waqtiga Singabuur"}, {"Solomon", "Waqtiga Solomon Aylaanis"}, {"South_Georgia", "Waqtiga Sowt Joorjiya"}, {"Suriname", "Waqtiga Surineym"}, {"Syowa", "Waqtiga Siyowa"}, {"Tahiti", "Waqtiga Tahiti"}, {"Taipei", "Waqtiga Caadiga Ah ee Taiwan"}, {"Tajikistan", "Waqtiga Tajikistan"}, {"Tokelau", "Waqtiga Tokeluu"}, {"Tonga", "Waqtiga Caadiga Ah ee Tonga"}, {"Truk", "Waqtiga Juuk"}, {"Turkmenistan", "Waqtiga Caadiga Ah ee Turkmenistan"}, {"Tuvalu", "Waqtiga Tufalu"}, {"Uruguay", "Waqtiga Caadiga Ah ee Urugwaay"}, {"Uzbekistan", "Waqtiga Caadiga Ah ee Usbekistan"}, {"Vanuatu", "Waqtiga Caadiga Ah ee Fanuutu"}, {"Venezuela", "Waqtiga Fenezuweela"}, {"Vladivostok", "Waqtiga Caadiga Ah ee Faladifostok"}, {"Volgograd", "Waqtiga Caadiga Ah ee Folgograd"}, {"Vostok", "Waqtiga Fostok"}, {"Wake", "Waqtiga Wayk Iylaanis"}, {"Wallis", "Waqtiga Walis & Futuna"}, {"Yakutsk", "Waqtiga Caadiga Ah ee Yakut"}, {"Yekaterinburg", "Waqtiga Caadiga Ah ee Yekaterinbaag"}, {"Yukon", "Waqtiga Yukon"}},
		{{"Acre", "Wakhtiga Kulka ee Acre"}, {"Alaska", "Waqtiga Dharaarta ee Alaska"}, {"Almaty", "Saacada Waqtiga Kulaylaha ee Almaty"}, {"Amazon", "Waqtiga Xagaaga ee Amason"}, {"America_Central", "Waqtiga Dharaarta ee Bartamaha Waqooyiga Ameerika"}, {"America_Eastern", "Waqtiga Dharaarta ee Bariga Waqooyiga Ameerika"}, {"America_Mountain", "Waqtiga Dharaarta ee Buurleyda Waqooyiga Ameerika"}, {"America_Pacific", "Waqtiga Dharaarta ee Basifika Waqooyiga Ameerika"}, {"Anadyr", "Wakhtiga Kulka ee Anadyr"}, {"Apia", "Waqtiga Dharaarta ee Samoa"}, {"Aqtau", "Saacada Waqtiga Kulaylaha Aqtau"}, {"Aqtobe", "Saacada Waqtiga kulaylaha Aqtobe"}, {"Arabian", "Waqtiga Dharaarta ee Carabta"}, {"Argentina", "Waqtiga Xagaaga ee Arjentiina"}, {"Argentina_Western", "Waqtiga Xagaaga ee Galbeedka Arjentiina"}, {"Armenia", "Waqtiga Xagaaga ee Armeeniya"}, {"Atlantic", "Waqtiga Dharaarta ee Atlantika Waqooyiga Ameerika"}, {"Australia_Central", "Waqtiga Dharaarta ee Bartamaha Astaraaliya"}, {"Australia_CentralWestern", "Waqtiga Dharaarta Bartamaha Galbeedka Australiya"}, {"Australia_Eastern", "Waqtiga Dharaarta ee Bariga Astaraaliya"}, {"Australia_Western", "Waqtiga Dharaarta ee Galbeedka Astaraaliya"}, {"Azerbaijan", "Waqtiga Xagaaga ee Asarbeyjan"}, {"Azores", "Waqtiga Xagaaga ee Asores"}, {"Bangladesh", "Waqtiga Xagaaga ee Bangledeesh"}, {"Brasilia", "Waqtiga Xagaaga ee Baraasiliya"}, {"Cape_Verde", "Waqtiga Xagaaga ee Keyb Faarde"}, {"Chatham", "Waqtiga Dharaarta ee Jaatam"}, {"Chile", "Waqtiga Xagaaga ee Jili"}, {"China", "Waqtiga Dharaarta ee Shiinaha"}, {"Colombia", "Waqtiga Xagaaga ee Kolambiya"}, {"Cook", "Waqtiga Caadiga Ah ee Jasiiradaha Cook"}, {"Cuba", "Waqtiga Dharaarta ee Kuuba"}, {"Easter", "Waqtiga Xagaaga ee Iistar Aylaan"}, {"Europe_Central", "Waqtiga Xagaaga ee Bartamaha Yurub"}, {"Europe_Eastern", "Waqtiga Xagaaga ee Bariga Yurub"}, {"Europe_Western", "Waqtiga Xagaaga ee Galbeedka Yurub"}, {"Falkland", "Waqtiga Xagaaga ee Faalklaan Aylaanis"}, {"Fiji", "Waqtiga Xagaaga ee Fiji"}, {"Georgia", "Waqtiga Xagaaga ee Joorjiya"}, {"Greenland_Eastern", "Waqtiga Xagaaga ee Bariga Giriinlaan"}, {"Greenland_Western", "Waqtiga Xagaaga ee Galbeedka Giriinlaan"}, {"Hawaii_Aleutian", "Waqtiga Dharaarta ee Hawaay-Alutiyaan"}, {"Hong_Kong", "Waqtiga Xagaaga ee Hoong Koong"}, {"Hovd", "Waqtiga Xagaaga ee Hofud"}, {"Iran", "Waqtiga Dharaarta ee Iiraan"}, {"Irkutsk", "Waqtiga Xagaaga ee Irkutsik"}, {"Israel", "Waqtiga Dharaarta ee Israaiil"}, {"Japan", "Waqtiga Dharaarta ee Jabaan"}, {"Kamchatka", "Wakhtiga Kulka ee Petropavlovsk-Kamchatski"}, {"Korea", "Waqtiga Dharaarta ee Kuuriya"}, {"Krasnoyarsk", "Waqtiga Xagaaga ee Karasnoyarsik"}, {"Lord_Howe", "Waqtiga Dharaarta ee Lod How"}, {"Magadan", "Waqtiga Xagaaga ee Magedan"}, {"Mauritius", "Waqtiga Xagaaga ee Morishiyaas"}, {"Mexico_Pacific", "Waqtiga Dharaarta ee Baasifikada Meksiko"}, {"Mongolia", "Waqtiga Xagaaga ee Ulaanbaataar"}, {"Moscow", "Waqtiga Xagaaga ee Moskow"}, {"New_Caledonia", "Waqtiga Xagaaga ee Niyuu Kaledoniya"}, {"New_Zealand", "Waqtiga Dharaarta ee Niyuu Si’laan"}, {"Newfoundland", "Waqtiga Dharaarta ee Niyuufoonlaan"}, {"Norfolk", "Waqtiga Maalinta ee Norfolk Island"}, {"Noronha", "Waqtiga Xagaaga ee Farnaando de Nooronha"}, {"Novosibirsk", "Waqtiga Xagaaga ee Nofosibirsik"}, {"Omsk", "Waqtiga Xagaaga ee Omsk"}, {"Pakistan", "Waqtiga Xagaaga ee Bakistaan"}, {"Paraguay", "Waqtiga Xagaaga ee Baragwaay"}, {"Peru", "Waqtiga Xagaaga ee Beeru"}, {"Philippines", "Waqtiga Xagaaga ee Filibiin"}, {"Pierre_Miquelon", "Waqtiga Dharaarta ee St. Beere & Mikiwelon"}, {"Qyzylorda", "Saacada Waqtiga Kulaylaha Qyzylorda"}, {"Sakhalin", "Waqtiga Xagaaga ee Sakhalin"}, {"Samara", "Wakhtiga Kulka ee Samara"}, {"Samoa", "Waqtiga Dharaarta ee American Samoa"}, {"Taipei", "Waqtiga Dharaarta ee Teybey"}, {"Tonga", "Waqtiga Xagaaga ee Tonga"}, {"Turkmenistan", "Waqtiga Xagaaga ee Turkmenistan"}, {"Uruguay", "Waqtiga Xagaaga ee Urugwaay"}, {"Uzbekistan", "Waqtiga Xagaaga ee Usbekistan"}, {"Vanuatu", "Waqtiga Xagaaga ee Fanuutu"}, {"Vladivostok", "Waqtiga Xagaaga ee Faladifostok"}, {"Volgograd", "Waqtiga Xagaaga ee Folgograd"}, {"Yakutsk", "Waqtiga Xagaaga ee Yakut"}, {"Yekaterinburg", "Waqtiga Xagaaga ee Yekaterinbaag"}},
	},
	{
		{{"Acre", "Ora standarde e Ejkrit [Ako]"}, {"Afghanistan", "Ora e Afganistanit"}, {"Africa_Central", "Ora e Afrikës Qendrore"}, {"Africa_Eastern", "Ora e Afrikës Lindore"}, {"Africa_Southern", "Ora standarde e Afrikës Jugore"}, {"Africa_Western", "Ora e Afrikës Perëndimore"}, {"Alaska", "Ora standarde e Alaskës"}, {"Almaty", "Ora standarde e Almatit"}, {"Amazon", "Ora standarde e Amazonës"}, {"America_Central", "Ora standarde e SHBA-së Qendrore"}, {"America_Eastern", "Ora standarde e SHBA-së Lindore"}, {"America_Mountain", "Ora standarde e Territoreve Amerikane të Brezit Malor"}, {"America_Pacific", "Ora standarde e Territoreve Amerikane të Bregut të Paqësorit"}, {"Anadyr", "Ora standarde e Anadirit"}, {"Apia", "Ora standarde e Apias"}, {"Aqtau", "Ora standarde e Aktaut"}, {"Aqtobe", "Ora standarde e Aktobit"}, {"Arabian", "Ora standarde arabe"}, {"Argentina", "Ora standarde e Argjentinës"}, {"Argentina_Western", "Ora standarde e Argjentinës Perëndimore"}, {"Armenia", "Ora standarde e Armenisë"}, {"Atlantic", "Ora standarde e Atlantikut"}, {"Australia_Central", "Ora standarde e Australisë Qendrore"}, {"Australia_CentralWestern", "Ora standarde e Australisë Qendroro-Perëndimore"}, {"Australia_Eastern", "Ora standarde e Australisë Lindore"}, {"Australia_Western", "Ora standarde e Australisë Perëndimore"}, {"Azerbaijan", "Ora standarde e Azerbajxhanit"}, {"Azores", "Ora standarde e Azoreve"}, {"Bangladesh", "Ora standarde e Bangladeshit"}, {"Bhutan", "Ora e Butanit"}, {"Bolivia", "Ora e Bolivisë"}, {"Brasilia", "Ora standarde e Brazilisë"}, {"Brunei", "Ora e Brunei-Durasalamit"}, {"Cape_Verde", "Ora standarde e Kepit të Gjelbër"}, {"Casey", "Ora e Kejsit"}, {"Chamorro", "Ora e Kamorros"}, {"Chatham", "Ora standarde e Katamit"}, {"Chile", "Ora standarde e Kilit"}, {"China", "Ora standarde e Kinës"}, {"Christmas", "Ora e Ishullit të Krishtlindjeve"}, {"Cocos", "Ora e Ishujve Kokos"}, {"Colombia", "Ora standarde e Kolumbisë"}, {"Cook", "Ora standarde e Ishujve Kuk"}, {"Cuba", "Ora standarde e Kubës"}, {"Davis", "Ora e Dejvisit"}, {"DumontDUrville", "Ora e Dumont-d’Urvilës"}, {"East_Timor", "Ora e Timorit Lindor"}, {"Easter", "Ora standarde e Ishullit të Pashkës"}, {"Ecuador", "Ora e Ekuadorit"}, {"Europe_Central", "Ora standarde e Evropës Qendrore"}, {"Europe_Eastern", "Ora standarde e Evropës Lindore"}, {"Europe_Further_Eastern", "Ora e Evropës së Largët Lindore"}, {"Europe_Western", "Ora standarde e Evropës Perëndimore"}, {"Falkland", "Ora standarde e Ishujve Falkland"}, {"Fiji", "Ora standarde e Fixhit"}, {"French_Guiana", "Ora e Guajanës Franceze"}, {"French_Southern", "Ora e Territoreve Jugore dhe Antarktike Franceze"}, {"GMT", "Ora e Grinuiçit"}, {"Galapagos", "Ora e Galapagosit"}, {"Gambier", "Ora e Gambierit"}, {"Georgia", "Ora standarde e Gjeorgjisë"}, {"Gilbert_Islands", "Ora e Ishujve Gilbert"}, {"Greenland_Eastern", "Ora standarde e Grenlandës Lindore"}, {"Greenland_Western", "Ora standarde e Grënlandës Perëndimore"}, {"Guam", "Ora e Guamit"}, {"Gulf", "Ora e Gjirit"}, {"Guyana", "Ora e Guajanës"}, {"Hawaii", "Ora standarde e Ishujve Hauai-Aleutian"}, {"Hawaii_Aleutian", "Ora standarde e Ishujve Hauai-Aleutian"}, {"Hong_Kong", "Ora standarde e Hong-Kongut"}, {"Hovd", "Ora standarde e Hovdit"}, {"India", "Ora standarde e Indisë"}, {"Indian_Ocean", "Ora e Oqeanit Indian"}, {"Indochina", "Ora e Indokinës"}, {"Indonesia_Central", "Ora e Indonezisë Qendrore"}, {"Indonesia_Eastern", "Ora e Indonezisë Lindore"}, {"Indonesia_Western", "Ora e Indonezisë Perëndimore"}, {"Iran", "Ora standarde e Iranit"}, {"Irkutsk", "Ora standarde e Irkutskut"}, {"Israel", "Ora standarde e Izraelit"}, {"Japan", "Ora standarde e Japonisë"}, {"Kamchatka", "Ora standarde e Petropavllovsk-Kamçatkës"}, {"Kazakhstan", "Ora e Kazakistanit"}, {"Kazakhstan_Eastern", "Ora e Kazakistanit Lindor"}, {"Kazakhstan_Western", "Ora e Kazakistanit Perëndimor"}, {"Korea", "Ora standarde koreane"}, {"Kosrae", "Ora e Kosrës"}, {"Krasnoyarsk", "Ora standarde e Krasnojarskut"}, {"Kyrgystan", "Ora e Kirgizisë"}, {"Lanka", "Ora e Lankasë"}, {"Line_Islands", "Ora e Ishujve Sporadikë Ekuatorialë"}, {"Lord_Howe", "Ora standarde e Lord-Houit"}, {"Macau", "Ora standarde e Makaos"}, {"Magadan", "Ora standarde e Magadanit"}, {"Malaysia", "Ora e Malajzisë"}, {"Maldives", "Ora e Maldiveve"}, {"Marquesas", "Ora e Ishujve Markezë"}, {"Marshall_Islands", "Ora e Ishujve Marshall"}, {"Mauritius", "Ora standarde e Mauritiusit"}, {"Mawson", "Ora e Mausonit"}, {"Mexico_Pacific", "Ora standarde e Territoreve Meksikane të Bregut të Paqësorit"}, {"Mongolia", "Ora standarde e Ulan-Batorit"}, {"Moscow", "Ora standarde e Moskës"}, {"Myanmar", "Ora e Mianmarit"}, {"Nauru", "Ora e Naurusë"}, {"Nepal", "Ora e Nepalit"}, {"New_Caledonia", "Ora standarde e Kaledonisë së Re"}, {"New_Zealand", "Ora standarde e Zelandës së Re"}, {"Newfoundland", "Ora standarde e Njufaundlendit [Tokës së Re]"}, {"Niue", "Ora e Niuesë"}, {"Norfolk", "Ora standarde e Ishullit Norfolk"}, {"Noronha", "Ora standarde e Fernando-de-Noronjës"}, {"North_Mariana", "Ora e Ishujve të Marianës së Veriut"}, {"Novosibirsk", "Ora standarde e Novosibirskut"}, {"Omsk", "Ora standarde e Omskut"}, {"Pakistan", "Ora standarde e Pakistanit"}, {"Palau", "Ora e Palaut"}, {"Papua_New_Guinea", "Ora e Guinesë së Re-Papua"}, {"Paraguay", "Ora standarde e Paraguait"}, {"Peru", "Ora standarde e Perusë"}, {"Philippines", "Ora standarde e Filipineve"}, {"Phoenix_Islands", "Ora e Ishujve Feniks"}, {"Pierre_Miquelon", "Ora standarde e Shën-Pier dhe Mikelon"}, {"Pitcairn", "Ora e Pitkernit"}, {"Ponape", "Ora e Ponapeit"}, {"Pyongyang", "Ora e Penianit"}, {"Qyzylorda", "Ora standarde e Kizilordit"}, {"Reunion", "Ora e Reunionit"}, {"Rothera", "Ora e Rodherës"}, {"Sakhalin", "Ora standarde e Sakalinit"}, {"Samara", "Ora standarde e Samarës"}, {"Samoa", "Ora standarde e Samoas"}, {"Seychelles", "Ora e Sejshelleve"}, {"Singapore", "Ora e Singaporit"}, {"Solomon", "Ora e Ishujve Solomon"}, {"South_Georgia", "Ora e Xhorxhas të Jugut"}, {"Suriname", "Ora e Surinamit"}, {"Syowa", "Ora e Sjouit"}, {"Tahiti", "Ora e Tahitit"}, {"Taipei", "Ora standarde e Tajpeit"}, {"Tajikistan", "Ora e Taxhikistanit"}, {"Tokelau", "Ora e Tokelaut"}, {"Tonga", "Ora standarde e Tongës"}, {"Truk", "Ora e Çukut"}, {"Turkmenistan", "Ora standarde e Turkmenistanit"}, {"Tuvalu", "Ora e Tuvalusë"}, {"Uruguay", "Ora standarde e Uruguait"}, {"Uzbekistan", "Ora standarde e Uzbekistanit"}, {"Vanuatu", "Ora standarde e Vanuatusë"}, {"Venezuela", "Ora e Venezuelës"}, {"Vladivostok", "Ora standarde e Vladivostokut"}, {"Volgograd", "Ora standarde e Volgogradit"}, {"Vostok", "Ora e Vostokut"}, {"Wake", "Ora e Ishullit Uejk"}, {"Wallis", "Ora e Uollisit dhe Futunës"}, {"Yakutsk", "Ora standarde e Jakutskut"}, {"Yekaterinburg", "Ora standarde e Ekaterinburgut"}, {"Yukon", "Ora e Jukonit"}},
		{{"Acre", "Ora verore e Ejkrit [Ako]"}, {"Alaska", "Ora verore e Alaskës"}, {"Almaty", "Ora verore e Almatit"}, {"Amazon", "Ora verore e Amazonës"}, {"America_Central", "Ora verore e SHBA-së Qendrore"}, {"America_Eastern", "Ora verore e SHBA-së Lindore"}, {"America_Mountain", "Ora verore e Territoreve Amerikane të Brezit Malor"}, {"America_Pacific", "Ora verore e Territoreve Amerikane të Bregut të Paqësorit"}, {"Anadyr", "Ora verore e Anadirit"}, {"Apia", "Ora verore e Apias"}, {"Aqtau", "Ora verore e Aktaut"}, {"Aqtobe", "Ora verore e Aktobit"}, {"Arabian", "Ora verore arabe"}, {"Argentina", "Ora verore e Argjentinës"}, {"Argentina_Western", "Ora verore e Argjentinës Perëndimore"}, {"Armenia", "Ora verore e Armenisë"}, {"Atlantic", "Ora verore e Atlantikut"}, {"Australia_Central", "Ora verore e Australisë Qendrore"}, {"Australia_CentralWestern", "Ora verore e Australisë Qendroro-Perëndimore"}, {"Australia_Eastern", "Ora verore e Australisë Lindore"}, {"Australia_Western", "Ora verore e Australisë Perëndimore"}, {"Azerbaijan", "Ora verore e Azerbajxhanit"}, {"Azores", "Ora verore e Azoreve"}, {"Bangladesh", "Ora verore e Bangladeshit"}, {"Brasilia", "Ora verore e Brazilisë"}, {"Cape_Verde", "Ora verore e Kepit të Gjelbër"}, {"Chatham", "Ora verore e Katamit"}, {"Chile", "Ora verore e Kilit"}, {"China", "Ora verore e Kinës"}, {"Colombia", "Ora verore e Kolumbisë"}, {"Cook", "Ora verore e Ishujve Kuk"}, {"Cuba", "Ora verore e Kubës"}, {"Easter", "Ora verore e Ishullit të Pashkës"}, {"Europe_Central", "Ora verore e Evropës Qendrore"}, {"Europe_Eastern", "Ora verore e Evropës Lindore"}, {"Europe_Western", "Ora verore e Evropës Perëndimore"}, {"Falkland", "Ora verore e Ishujve Falkland"}, {"Fiji", "Ora verore e Fixhit"}, {"Georgia", "Ora verore e Gjeorgjisë"}, {"Greenland_Eastern", "Ora verore e Grenlandës Lindore"}, {"Greenland_Western", "Ora verore e Grënlandës Perëndimore"}, {"Hawaii_Aleutian", "Ora verore e Ishujve Hauai-Aleutian"}, {"Hong_Kong", "Ora verore e Hong-Kongut"}, {"Hovd", "Ora verore e Hovdit"}, {"Iran", "Ora verore e Iranit"}, {"Irkutsk", "Ora verore e Irkutskut"}, {"Israel", "Ora verore e Izraelit"}, {"Japan", "Ora verore e Japonisë"}, {"Kamchatka", "Ora verore e Petropavllovsk-Kamçatkës"}, {"Korea", "Ora verore koreane"}, {"Krasnoyarsk", "Ora verore e Krasnojarskut"}, {"Lord_Howe", "Ora verore e Lord-Houit"}, {"Macau", "Ora verore e Makaos"}, {"Magadan", "Ora verore e Magadanit"}, {"Mauritius", "Ora verore e Mauritiusit"}, {"Mexico_Pacific", "Ora verore e Territoreve Meksikane të Bregut të Paqësorit"}, {"Mongolia", "Ora verore e Ulan-Batorit"}, {"Moscow", "Ora verore e Moskës"}, {"New_Caledonia", "Ora verore e Kaledonisë së Re"}, {"New_Zealand", "Ora verore e Zelandës së Re"}, {"Newfoundland", "Ora verore e Njufaundlendit [Tokës së Re]"}, {"Norfolk", "Ora verore e Ishullit Norfolk"}, {"Noronha", "Ora verore e Fernando-de-Noronjës"}, {"Novosibirsk", "Ora verore e Novosibirskut"}, {"Omsk", "Ora verore e Omskut"}, {"Pakistan", "Ora verore e Pakistanit"}, {"Paraguay", "Ora Verore e Paraguait"}, {"Peru", "Ora verore e Perusë"}, {"Philippines", "Ora verore e Filipineve"}, {"Pierre_Miquelon", "Ora verore e Shën-Pier dhe Mikelon"}, {"Qyzylorda", "Ora verore e Kizilordit"}, {"Sakhalin", "Ora verore e Sakalinit"}, {"Samara", "Ora verore e Samarës"}, {"Samoa", "Ora verore e Samoas"}, {"Taipei", "Ora verore e Tajpeit"}, {"Tonga", "Ora verore e Tongës"}, {"Turkmenistan", "Ora verore e Turkmenistanit"}, {"Uruguay", "Ora verore e Uruguait"}, {"Uzbekistan", "Ora verore e Uzbekistanit"}, {"Vanuatu", "Ora verore e Vanuatusë"}, {"Vladivostok", "Ora verore e Vladivostokut"}, {"Volgograd", "Ora verore e Volgogradit"}, {"Yakutsk", "Ora verore e Jakutskut"}, {"Yekaterinburg", "Ora verore e Ekaterinburgut"}},
	},
	{
		{{"Acre", "Акре стандардно време"}, {"Afghanistan", "Авганистан време"}, {"Africa_Central", "Централно-афричко време"}, {"Africa_Eastern", "Источно-афричко време"}, {"Africa_Southern", "Јужно-афричко време"}, {"Africa_Western", "Западно-афричко време"}, {"Alaska", "Аљаска, стандардно време"}, {"Almaty", "Алмати стандардно време"}, {"Amazon", "Амазон, стандардно време"}, {"America_Central", "Северноамеричко централно стандардно време"}, {"America_Eastern", "Северноамеричко источно стандардно време"}, {"America_Mountain", "Северноамеричко планинско стандардно време"}, {"America_Pacific", "Северноамеричко пацифичко стандардно време"}, {"Anadyr", "Анадир стандардно време"}, {"Apia", "Апија, стандардно време"}, {"Aqtau", "Акватау стандардно време"}, {"Aqtobe", "Акутобе стандардно време"}, {"Arabian", "Арабијско стандардно време"}, {"Argentina", "Аргентина, стандардно време"}, {"Argentina_Western", "Западна Аргентина, стандардно време"}, {"Armenia", "Јерменија, стандардно време"}, {"Atlantic", "Атлантско стандардно време"}, {"Australia_Central", "Аустралијско централно стандардно време"}, {"Australia_CentralWestern", "Аустралијско централно западно стандардно време"}, {"Australia_Eastern", "Аустралијско источно стандардно време"}, {"Australia_Western", "Аустралијско западно стандардно време"}, {"Azerbaijan", "Азербејџан, стандардно време"}, {"Azores", "Азори, стандардно време"}, {"Bangladesh", "Бангладеш, стандардно време"}, {"Bhutan", "Бутан време"}, {"Bolivia", "Боливија време"}, {"Brasilia", "Бразилија, стандардно време"}, {"Brunei", "Брунеј Дарусалум време"}, {"Cape_Verde", "Зеленортска Острва, стандардно време"}, {"Chamorro", "Чаморо време"}, {"Chatham", "Чатам, стандардно време"}, {"Chile", "Чиле, стандардно време"}, {"China", "Кинеско стандардно време"}, {"Christmas", "Божићно острво време"}, {"Cocos", "Кокос (Келинг) Острва време"}, {"Colombia", "Колумбија, стандардно време"}, {"Cook", "Кукова острва, стандардно време"}, {"Cuba", "Куба, стандардно време"}, {"Davis", "Дејвис време"}, {"DumontDUrville", "Димон д’Урвил време"}, {"East_Timor", "Источни тимор време"}, {"Easter", "Ускршња острва, стандардно време"}, {"Ecuador", "Еквадор време"}, {"Europe_Central", "Средњеевропско стандардно време"}, {"Europe_Eastern", "Источноевропско стандардно време"}, {"Europe_Further_Eastern", "Време даљег истока Европе"}, {"Europe_Western", "Западноевропско стандардно време"}, {"Falkland", "Фолкландска Острва, стандардно време"}, {"Fiji", "Фиџи, стандардно време"}, {"French_Guiana", "Француска Гвајана време"}, {"French_Southern", "Француско јужно и антарктичко време"}, {"GMT", "Средње време по Гриничу"}, {"Galapagos", "Галапагос време"}, {"Gambier", "Гамбије време"}, {"Georgia", "Грузија, стандардно време"}, {"Gilbert_Islands", "Гилберт острва време"}, {"Greenland_Eastern", "Источни Гренланд, стандардно време"}, {"Greenland_Western", "Западни Гренланд, стандардно време"}, {"Guam", "Гуам стандардно време"}, {"Gulf", "Заливско време"}, {"Guyana", "Гвајана време"}, {"Hawaii", "Хавајско-алеутско стандардно време"}, {"Hawaii_Aleutian", "Хавајско-алеутско стандардно време"}, {"Hong_Kong", "Хонг Конг, стандардно време"}, {"Hovd", "Ховд, стандардно време"}, {"India", "Индијско стандардно време"}, {"Indian_Ocean", "Индијско океанско време"}, {"Indochina", "Индокина време"}, {"Indonesia_Central", "Централно-индонезијско време"}, {"Indonesia_Eastern", "Источно-индонезијско време"}, {"Indonesia_Western", "Западно-индонезијско време"}, {"Iran", "Иран, стандардно време"}, {"Irkutsk", "Иркуцк, стандардно време"}, {"Israel", "Израелско стандардно време"}, {"Japan", "Јапанско стандардно време"}, {"Kamchatka", "Петропавловско-камчатско стандардно време"}, {"Kazakhstan", "Казахстанско време"}, {"Kazakhstan_Eastern", "Источно-казахстанско време"}, {"Kazakhstan_Western", "Западно-казахстанско време"}, {"Korea", "Корејско стандардно време"}, {"Kosrae", "Кошре време"}, {"Krasnoyarsk", "Краснојарск, стандардно време"}, {"Kyrgystan", "Киргистан време"}, {"Lanka", "Шри Ланка време"}, {"Line_Islands", "Острва Лајн време"}, {"Lord_Howe", "Лорд Хов, стандардно време"}, {"Macau", "Макао стандардно време"}, {"Magadan", "Магадан, стандардно време"}, {"Malaysia", "Малезија време"}, {"Maldives", "Малдиви време"}, {"Marquesas", "Маркиз време"}, {"Marshall_Islands", "Маршалска Острва време"}, {"Mauritius", "Маурицијус, стандардно време"}, {"Mawson", "Мосон време"}, {"Mexico_Pacific", "Мексички Пацифик, стандардно време"}, {"Mongolia", "Улан Батор, стандардно време"}, {"Moscow", "Москва, стандардно време"}, {"Myanmar", "Мијанмар време"}, {"Nauru", "Науру време"}, {"Nepal", "Непал време"}, {"New_Caledonia", "Нова Каледонија, стандардно време"}, {"New_Zealand", "Нови Зеланд, стандардно време"}, {"Newfoundland", "Њуфаундленд, стандардно време"}, {"Niue", "Ниуе време"}, {"Norfolk", "Норфолк Острво, стандардно време"}, {"Noronha", "Фернандо де Нороња, стандардно време"}, {"North_Mariana", "Северна Маријанска Острва време"}, {"Novosibirsk", "Новосибирск, стандардно време"}, {"Omsk", "Омск, стандардно време"}, {"Pakistan", "Пакистан, стандардно време"}, {"Palau", "Палау време"}, {"Papua_New_Guinea", "Папуа Нова Гвинеја време"}, {"Paraguay", "Парагвај, стандардно време"}, {"Peru", "Перу, стандардно време"}, {"Philippines", "Филипини, стандардно време"}, {"Phoenix_Islands", "Феникс острва време"}, {"Pierre_Miquelon", "Сен Пјер и Микелон, стандардно време"}, {"Pitcairn", "Питкерн време"}, {"Ponape", "Понпеј време"}, {"Pyongyang", "Пјонгјаншко време"}, {"Qyzylorda", "Кизилорда стандардно време"}, {"Reunion", "Реинион време"}, {"Rothera", "Ротера време"}, {"Sakhalin", "Сахалин, стандардно време"}, {"Samara", "Самара стандардно време"}, {"Samoa", "Самоа, стандардно време"}, {"Seychelles", "Сејшели време"}, {"Singapore", "Сингапур, стандардно време"}, {"Solomon", "Соломонска Острва време"}, {"South_Georgia", "Јужна Џорџија време"}, {"Suriname", "Суринам време"}, {"Syowa", "Шова време"}, {"Tahiti", "Тахити време"}, {"Taipei", "Тајпеј, стандардно време"}, {"Tajikistan", "Таџикистан време"}, {"Tokelau", "Токелау време"}, {"Tonga", "Тонга, стандардно време"}, {"Truk", "Чуук време"}, {"Turkey", "Турска стандардно време"}, {"Turkmenistan", "Туркменистан, стандардно време"}, {"Tuvalu", "Тувалу време"}, {"Uruguay", "Уругвај, стандардно време"}, {"Uzbekistan", "Узбекистан, стандардно време"}, {"Vanuatu", "Вануату, стандардно време"}, {"Venezuela", "Венецуела време"}, {"Vladivostok", "Владивосток, стандардно време"}, {"Volgograd", "Волгоград, стандардно време"}, {"Vostok", "Восток време"}, {"Wake", "Вејк острво време"}, {"Wallis", "Валис и Футуна Острва време"}, {"Yakutsk", "Јакутск, стандардно време"}, {"Yekaterinburg", "Јекатеринбург, стандардно време"}, {"Yukon", "Јукон"}},
		{{"Acre", "Акре летње рачунање времена"}, {"Alaska", "Аљаска, летње време"}, {"Almaty", "Алмати летње рачунање времена"}, {"Amazon", "Амазон, летње време"}, {"America_Central", "Северноамеричко централно летње време"}, {"America_Eastern", "Северноамеричко источно летње време"}, {"America_Mountain", "Северноамеричко планинско летње време"}, {"America_Pacific", "Северноамеричко пацифичко летње време"}, {"Anadyr", "Анадир летње рачунање времена"}, {"Apia", "Апија, летње време"}, {"Aqtau", "Акватау летње рачунање времена"}, {"Aqtobe", "Акутобе летње рачунање времена"}, {"Arabian", "Арабијско летње време"}, {"Argentina", "Аргентина, летње време"}, {"Argentina_Western", "Западна Аргентина, летње време"}, {"Armenia", "Јерменија, летње време"}, {"Atlantic", "Атлантско летње време"}, {"Australia_Central", "Аустралијско централно летње време"}, {"Australia_CentralWestern", "Аустралијско централно западно летње време"}, {"Australia_Eastern", "Аустралијско источно летње време"}, {"Australia_Western", "Аустралијско западно летње време"}, {"Azerbaijan", "Азербејџан, летње време"}, {"Azores", "Азори, летње време"}, {"Bangladesh", "Бангладеш, летње време"}, {"Brasilia", "Бразилија, летње време"}, {"Cape_Verde", "Зеленортска Острва, летње време"}, {"Chatham", "Чатам, летње време"}, {"Chile", "Чиле, летње време"}, {"China", "Кина, летње време"}, {"Colombia", "Колумбија, летње време"}, {"Cook", "Кукова острва, полу-летње време"}, {"Cuba", "Куба, летње време"}, {"Easter", "Ускршња острва, летње време"}, {"Europe_Central", "Средњеевропско летње време"}, {"Europe_Eastern", "Источноевропско летње време"}, {"Europe_Western", "Западноевропско летње време"}, {"Falkland", "Фолкландска Острва, летње време"}, {"Fiji", "Фиџи, летње време"}, {"Georgia", "Грузија, летње време"}, {"Greenland_Eastern", "Источни Гренланд, летње време"}, {"Greenland_Western", "Западни Гренланд, летње време"}, {"Hawaii_Aleutian", "Хавајско-алеутско летње време"}, {"Hong_Kong", "Хонг Конг, летње време"}, {"Hovd", "Ховд, летње време"}, {"Iran", "Иран, летње време"}, {"Irkutsk", "Иркуцк, летње време"}, {"Israel", "Израелско летње време"}, {"Japan", "Јапанско летње време"}, {"Kamchatka", "Петропавловско-камчатско летње рачунање времена"}, {"Korea", "Корејско летње време"}, {"Krasnoyarsk", "Краснојарск, летње време"}, {"Lord_Howe", "Лорд Хов, летње време"}, {"Macau", "Макао летње рачунање времена"}, {"Magadan", "Магадан, летње време"}, {"Mauritius", "Маурицијус, летње време"}, {"Mexico_Pacific", "Мексички Пацифик, летње време"}, {"Mongolia", "Улан Батор, летње време"}, {"Moscow", "Москва, летње време"}, {"New_Caledonia", "Нова Каледонија, летње време"}, {"New_Zealand", "Нови Зеланд, летње време"}, {"Newfoundland", "Њуфаундленд, летње време"}, {"Norfolk", "Норфолк Острво, летње време"}, {"Noronha", "Фернандо де Нороња, летње време"}, {"Novosibirsk", "Новосибирск, летње време"}, {"Omsk", "Омск, летње време"}, {"Pakistan", "Пакистан, летње време"}, {"Paraguay", "Парагвај, летње време"}, {"Peru", "Перу, летње време"}, {"Philippines", "Филипини, летње време"}, {"Pierre_Miquelon", "Сен Пјер и Микелон, летње време"}, {"Qyzylorda", "Кизилорда летње рачунање времена"}, {"Sakhalin", "Сахалин, летње време"}, {"Samara", "Самара летње рачунање времена"}, {"Samoa", "Самоа, летње време"}, {"Taipei", "Тајпеј, летње време"}, {"Tonga", "Тонга, летње време"}, {"Turkey", "Турска летње рачунање времена"}, {"Turkmenistan", "Туркменистан, летње време"}, {"Uruguay", "Уругвај, летње време"}, {"Uzbekistan", "Узбекистан, летње време"}, {"Vanuatu", "Вануату, летње време"}, {"Vladivostok", "Владивосток, летње време"}, {"Volgograd", "Волгоград, летње време"}, {"Yakutsk", "Јакутск, летње време"}, {"Yekaterinburg", "Јекатеринбург, летње време"}},
	},
	{
		{{"Acre", "Акре стандардно време"}, {"Afghanistan", "Авганистан вријеме"}, {"Africa_Central", "Централно-афричко вријеме"}, {"Africa_Eastern", "Источно-афричко вријеме"}, {"Africa_Southern", "Јужно-афричко вријеме"}, {"Africa_Western", "Западно-афричко вријеме"}, {"Alaska", "Аљаска, стандардно вријеме"}, {"Almaty", "Алмати стандардно време"}, {"Amazon", "Амазон, стандардно вријеме"}, {"America_Central", "Сјеверноамеричко централно стандардно вријеме"}, {"America_Eastern", "Сјеверноамеричко источно стандардно вријеме"}, {"America_Mountain", "Сјеверноамеричко планинско стандардно вријеме"}, {"America_Pacific", "Сјеверноамеричко пацифичко стандардно вријеме"}, {"Anadyr", "Анадир стандардно време"}, {"Apia", "Апија, стандардно вријеме"}, {"Aqtau", "Акватау стандардно време"}, {"Aqtobe", "Акутобе стандардно време"}, {"Arabian", "Арабијско стандардно вријеме"}, {"Argentina", "Аргентина, стандардно вријеме"}, {"Argentina_Western", "Западна Аргентина, стандардно вријеме"}, {"Armenia", "Јерменија, стандардно вријеме"}, {"Atlantic", "Атлантско стандардно вријеме"}, {"Australia_Central", "Аустралијско централно стандардно вријеме"}, {"Australia_CentralWestern", "Аустралијско централно западно стандардно вријеме"}, {"Australia_Eastern", "Аустралијско источно стандардно вријеме"}, {"Australia_Western", "Аустралијско западно стандардно вријеме"}, {"Azerbaijan", "Азербејџан, стандардно вријеме"}, {"Azores", "Азори, стандардно вријеме"}, {"Bangladesh", "Бангладеш, стандардно вријеме"}, {"Bhutan", "Бутан вријеме"}, {"Bolivia", "Боливија вријеме"}, {"Brasilia", "Бразилија, стандардно вријеме"}, {"Brunei", "Брунеј Дарусалум вријеме"}, {"Cape_Verde", "Зеленортска Острва, стандардно вријеме"}, {"Chamorro", "Чаморо вријеме"}, {"Chatham", "Чатам, стандардно вријеме"}, {"Chile", "Чиле, стандардно вријеме"}, {"China", "Кинеско стандардно вријеме"}, {"Christmas", "Божићно острво вријеме"}, {"Cocos", "Кокосова (Килинг) острва вријеме"}, {"Colombia", "Колумбија, стандардно вријеме"}, {"Cook", "Кукова Острва, стандардно вријеме"}, {"Cuba", "Куба, стандардно вријеме"}, {"Davis", "Дејвис вријеме"}, {"DumontDUrville", "Димон д’Ирвил вријеме"}, {"East_Timor", "Источни Тимор вријеме"}, {"Easter", "Ускршња острва, стандардно вријеме"}, {"Ecuador", "Еквадор вријеме"}, {"Europe_Central", "Средњоевропско стандардно вријеме"}, {"Europe_Eastern", "Источноевропско стандардно вријеме"}, {"Europe_Further_Eastern", "Време даљег истока Европе"}, {"Europe_Western", "Западноевропско стандардно вријеме"}, {"Falkland", "Фолкландска Острва, стандардно вријеме"}, {"Fiji", "Фиџи, стандардно вријеме"}, {"French_Guiana", "Француска Гвајана вријеме"}, {"French_Southern", "Француско јужно и антарктичко вријеме"}, {"GMT", "Средње вријеме по Гриничу"}, {"Galapagos", "Галапагос вријеме"}, {"Gambier", "Гамбије вријеме"}, {"Georgia", "Грузија, стандардно вријеме"}, {"Gilbert_Islands", "Гилбертова острва вријеме"}, {"Greenland_Eastern", "Источни Гренланд, стандардно вријеме"}, {"Greenland_Western", "Западни Гренланд, стандардно вријеме"}, {"Guam", "Гуам стандардно време"}, {"Gulf", "Заливско вријеме"}, {"Guyana", "Гвајана вријеме"}, {"Hawaii", "Хавајско-алеутско стандардно вријеме"}, {"Hawaii_Aleutian", "Хавајско-алеутско стандардно вријеме"}, {"Hong_Kong", "Хонг Конг, стандардно вријеме"}, {"Hovd", "Ховд, стандардно вријеме"}, {"India", "Индијско стандардно вријеме"}, {"Indian_Ocean", "Индијско океанско вријеме"}, {"Indochina", "Индокина вријеме"}, {"Indonesia_Central", "Централно-индонезијско вријеме"}, {"Indonesia_Eastern", "Источно-индонезијско вријеме"}, {"Indonesia_Western", "Западно-индонезијско вријеме"}, {"Iran", "Иран, стандардно вријеме"}, {"Irkutsk", "Иркуцк, стандардно вријеме"}, {"Israel", "Израелско стандардно вријеме"}, {"Japan", "Јапанско стандардно вријеме"}, {"Kamchatka", "Петропавловско-камчатско стандардно време"}, {"Kazakhstan", "Казахстанско вријеме"}, {"Kazakhstan_Eastern", "Источно-казахстанско вријеме"}, {"Kazakhstan_Western", "Западно-казахстанско вријеме"}, {"Korea", "Корејско стандардно вријеме"}, {"Kosrae", "Кошре вријеме"}, {"Krasnoyarsk", "Краснојарск, стандардно вријеме"}, {"Kyrgystan", "Киргистан вријеме"}, {"Lanka", "Шри Ланка време"}, {"Line_Islands", "Линијска острва вријеме"}, {"Lord_Howe", "Лорд Хов, стандардно вријеме"}, {"Macau", "Макао стандардно време"}, {"Magadan", "Магадан, стандардно вријеме"}, {"Malaysia", "Малезија вријеме"}, {"Maldives", "Малдиви вријеме"}, {"Marquesas", "Маркиз вријеме"}, {"Marshall_Islands", "Маршалска Острва вријеме"}, {"Mauritius", "Маурицијус, стандардно вријеме"}, {"Mawson", "Мосон вријеме"}, {"Mexico_Pacific", "Мексички Пацифик, стандардно вријеме"}, {"Mongolia", "Улан Батор, стандардно вријеме"}, {"Moscow", "Москва, стандардно вријеме"}, {"Myanmar", "Мјанмар вријеме"}, {"Nauru", "Науру вријеме"}, {"Nepal", "Непал вријеме"}, {"New_Caledonia", "Нова Каледонија, стандардно вријеме"}, {"New_Zealand", "Нови Зеланд, стандардно вријеме"}, {"Newfoundland", "Њуфаундленд, стандардно вријеме"}, {"Niue", "Нијуе вријеме"}, {"Norfolk", "острво Норфолк, стандардно вријеме"}, {"Noronha", "Фернандо де Нороња, стандардно вријеме"}, {"North_Mariana", "Северна Маријанска Острва време"}, {"Novosibirsk", "Новосибирск, стандардно вријеме"}, {"Omsk", "Омск, стандардно вријеме"}, {"Pakistan", "Пакистан, стандардно вријеме"}, {"Palau", "Палау вријеме"}, {"Papua_New_Guinea", "Папуа Нова Гвинеја вријеме"}, {"Paraguay", "Парагвај, стандардно вријеме"}, {"Peru", "Перу, стандардно вријеме"}, {"Philippines", "Филипини, стандардно вријеме"}, {"Phoenix_Islands", "Феникс острва вријеме"}, {"Pierre_Miquelon", "Сен Пјер и Микелон, стандардно вријеме"}, {"Pitcairn", "Питкерн вријеме"}, {"Ponape", "Понпеј вријеме"}, {"Pyongyang", "Пјонгјаншко вријеме"}, {"Qyzylorda", "Кизилорда стандардно време"}, {"Reunion", "Реунион вријеме"}, {"Rothera", "Ротера вријеме"}, {"Sakhalin", "Сахалин, стандардно вријеме"}, {"Samara", "Самара стандардно време"}, {"Samoa", "Самоа, стандардно вријеме"}, {"Seychelles", "Сејшели вријеме"}, {"Singapore", "Сингапур, стандардно вријеме"}, {"Solomon", "Соломонска Острва вријеме"}, {"South_Georgia", "Јужна Џорџија вријеме"}, {"Suriname", "Суринам вријеме"}, {"Syowa", "Шова вријеме"}, {"Tahiti", "Тахити вријеме"}, {"Taipei", "Тајпеј, стандардно вријеме"}, {"Tajikistan", "Таџикистан вријеме"}, {"Tokelau", "Токелау вријеме"}, {"Tonga", "Тонга, стандардно вријеме"}, {"Truk", "Чук вријеме"}, {"Turkey", "Турска стандардно време"}, {"Turkmenistan", "Туркменистан, стандардно вријеме"}, {"Tuvalu", "Тувалу вријеме"}, {"Uruguay", "Уругвај, стандардно вријеме"}, {"Uzbekistan", "Узбекистан, стандардно вријеме"}, {"Vanuatu", "Вануату, стандардно вријеме"}, {"Venezuela", "Венецуела вријеме"}, {"Vladivostok", "Владивосток, стандардно вријеме"}, {"Volgograd", "Волгоград, стандардно вријеме"}, {"Vostok", "Восток вријеме"}, {"Wake", "острво Вејк вријеме"}, {"Wallis", "острва Валис и Футуна вријеме"}, {"Yakutsk", "Јакутск, стандардно вријеме"}, {"Yekaterinburg", "Јекатеринбург, стандардно вријеме"}, {"Yukon", "Јукон"}},
		{{"Acre", "Акре летње рачунање времена"}, {"Alaska", "Аљаска, љетње вријеме"}, {"Almaty", "Алмати летње рачунање времена"}, {"Amazon", "Амазон, љетње вријеме"}, {"America_Central", "Сјеверноамеричко централно љетње вријеме"}, {"America_Eastern", "Сјеверноамеричко источно љетње вријеме"}, {"America_Mountain", "Сјеверноамеричко планинско љетње вријеме"}, {"America_Pacific", "Сјеверноамеричко пацифичко летње вријеме"}, {"Anadyr", "Анадир летње рачунање времена"}, {"Apia", "Апија, љетње вријеме"}, {"Aqtau", "Акватау летње рачунање времена"}, {"Aqtobe", "Акутобе летње рачунање времена"}, {"Arabian", "Арабијско љетње вријеме"}, {"Argentina", "Аргентина, љетње вријеме"}, {"Argentina_Western", "Западна Аргентина, љетње вријеме"}, {"Armenia", "Јерменија, љетње вријеме"}, {"Atlantic", "Атлантско љетње вријеме"}, {"Australia_Central", "Аустралијско централно љетње вријеме"}, {"Australia_CentralWestern", "Аустралијско централно западно љетње вријеме"}, {"Australia_Eastern", "Аустралијско источно љетње вријеме"}, {"Australia_Western", "Аустралијско западно љетње вријеме"}, {"Azerbaijan", "Азербејџан, љетње вријеме"}, {"Azores", "Азори, љетње вријеме"}, {"Bangladesh", "Бангладеш, љетње вријеме"}, {"Brasilia", "Бразилија, љетње вријеме"}, {"Cape_Verde", "Зеленортска Острва, љетње вријеме"}, {"Chatham", "Чатам, љетње вријеме"}, {"Chile", "Чиле, љетње вријеме"}, {"China", "Кина, љетње вријеме"}, {"Colombia", "Колумбија, љетње вријеме"}, {"Cook", "Кукова Острва, полуљетње вријеме"}, {"Cuba", "Куба, љетње вријеме"}, {"Easter", "Ускршња острва, љетње вријеме"}, {"Europe_Central", "Средњоевропско љетње вријеме"}, {"Europe_Eastern", "Источноевропско љетње вријеме"}, {"Europe_Western", "Западноевропско љетње вријеме"}, {"Falkland", "Фолкландска Острва, љетње вријеме"}, {"Fiji", "Фиџи, љетње вријеме"}, {"Georgia", "Грузија, љетње вријеме"}, {"Greenland_Eastern", "Источни Гренланд, љетње вријеме"}, {"Greenland_Western", "Западни Гренланд, љетње вријеме"}, {"Hawaii_Aleutian", "Хавајско-алеутско љетње вријеме"}, {"Hong_Kong", "Хонг Конг, љетње вријеме"}, {"Hovd", "Ховд, љетње вријеме"}, {"Iran", "Иран, љетње вријеме"}, {"Irkutsk", "Иркуцк, љетње вријеме"}, {"Israel", "Израелско љетње вријеме"}, {"Japan", "Јапанско љетње вријеме"}, {"Kamchatka", "Петропавловско-камчатско летње рачунање времена"}, {"Korea", "Корејско љетње вријеме"}, {"Krasnoyarsk", "Краснојарск, љетње вријеме"}, {"Lord_Howe", "Лорд Хов, љетње вријеме"}, {"Macau", "Макао летње рачунање времена"}, {"Magadan", "Магадан, љетње вријеме"}, {"Mauritius", "Маурицијус, љетње вријеме"}, {"Mexico_Pacific", "Мексички Пацифик, љетње вријеме"}, {"Mongolia", "Улан Батор, љетње вријееме"}, {"Moscow", "Москва, љетње вријеме"}, {"New_Caledonia", "Нова Каледонија, љетње вријеме"}, {"New_Zealand", "Нови Зеланд, љетње вријеме"}, {"Newfoundland", "Њуфаундленд, љетње вријеме"}, {"Norfolk", "острво Норфолк, љетње вријеме"}, {"Noronha", "Фернандо де Нороња, љетње вријеме"}, {"Novosibirsk", "Новосибирск, љетње вријеме"}, {"Omsk", "Омск, љетње вријеме"}, {"Pakistan", "Пакистан, љетње вријеме"}, {"Paraguay", "Парагвај, љетње вријеме"}, {"Peru", "Перу, љетње вријеме"}, {"Philippines", "Филипини, љетње вријеме"}, {"Pierre_Miquelon", "Сен Пјер и Микелон, љетње вријеме"}, {"Qyzylorda", "Кизилорда летње рачунање времена"}, {"Sakhalin", "Сахалин, љетње вријеме"}, {"Samara", "Самара летње рачунање времена"}, {"Samoa", "Самоа, љетње вријеме"}, {"Taipei", "Тајпеј, љетње вријеме"}, {"Tonga", "Тонга, љетње вријеме"}, {"Turkey", "Турска летње рачунање времена"}, {"Turkmenistan", "Туркменистан, љетње вријеме"}, {"Uruguay", "Уругвај, љетње вријеме"}, {"Uzbekistan", "Узбекистан, љетње вријеме"}, {"Vanuatu", "Вануату, љетње вријеме"}, {"Vladivostok", "Владивосток, љетње вријеме"}, {"Volgograd", "Волгоград, љетње вријеме"}, {"Yakutsk", "Јакутск, љетње вријеме"}, {"Yekaterinburg", "Јекатеринбург, љетње вријеме"}},
	},
	{
		{{"Acre", "Akre standardno vreme"}, {"Afghanistan", "Avganistan vreme"}, {"Africa_Central", "Centralno-afričko vreme"}, {"Africa_Eastern", "Istočno-afričko vreme"}, {"Africa_Southern", "Južno-afričko vreme"}, {"Africa_Western", "Zapadno-afričko vreme"}, {"Alaska", "Aljaska, standardno vreme"}, {"Almaty", "Almati standardno vreme"}, {"Amazon", "Amazon, standardno vreme"}, {"America_Central", "Severnoameričko centralno standardno vreme"}, {"America_Eastern", "Severnoameričko istočno standardno vreme"}, {"America_Mountain", "Severnoameričko planinsko standardno vreme"}, {"America_Pacific", "Severnoameričko pacifičko standardno vreme"}, {"Anadyr", "Anadir standardno vreme"}, {"Apia", "Apija, standardno vreme"}, {"Aqtau", "Akvatau standardno vreme"}, {"Aqtobe", "Akutobe standardno vreme"}, {"Arabian", "Arabijsko standardno vreme"}, {"Argentina", "Argentina, standardno vreme"}, {"Argentina_Western", "Zapadna Argentina, standardno vreme"}, {"Armenia", "Jermenija, standardno vreme"}, {"Atlantic", "Atlantsko standardno vreme"}, {"Australia_Central", "Australijsko centralno standardno vreme"}, {"Australia_CentralWestern", "Australijsko centralno zapadno standardno vreme"}, {"Australia_Eastern", "Australijsko istočno standardno vreme"}, {"Australia_Western", "Australijsko zapadno standardno vreme"}, {"Azerbaijan", "Azerbejdžan, standardno vreme"}, {"Azores", "Azori, standardno vreme"}, {"Bangladesh", "Bangladeš, standardno vreme"}, {"Bhutan", "Butan vreme"}, {"Bolivia", "Bolivija vreme"}, {"Brasilia", "Brazilija, standardno vreme"}, {"Brunei", "Brunej Darusalum vreme"}, {"Cape_Verde", "Zelenortska Ostrva, standardno vreme"}, {"Chamorro", "Čamoro vreme"}, {"Chatham", "Čatam, standardno vreme"}, {"Chile", "Čile, standardno vreme"}, {"China", "Kinesko standardno vreme"}, {"Christmas", "Božićno ostrvo vreme"}, {"Cocos", "Kokos (Keling) Ostrva vreme"}, {"Colombia", "Kolumbija, standardno vreme"}, {"Cook", "Kukova ostrva, standardno vreme"}, {"Cuba", "Kuba, standardno vreme"}, {"Davis", "Dejvis vreme"}, {"DumontDUrville", "Dimon d’Urvil vreme"}, {"East_Timor", "Istočni timor vreme"}, {"Easter", "Uskršnja ostrva, standardno vreme"}, {"Ecuador", "Ekvador vreme"}, {"Europe_Central", "Srednjeevropsko standardno vreme"}, {"Europe_Eastern", "Istočnoevropsko standardno vreme"}, {"Europe_Further_Eastern", "Vreme daljeg istoka Evrope"}, {"Europe_Western", "Zapadnoevropsko standardno vreme"}, {"Falkland", "Folklandska Ostrva, standardno vreme"}, {"Fiji", "Fidži, standardno vreme"}, {"French_Guiana", "Francuska Gvajana vreme"}, {"French_Southern", "Francusko južno i antarktičko vreme"}, {"GMT", "Srednje vreme po Griniču"}, {"Galapagos", "Galapagos vreme"}, {"Gambier", "Gambije vreme"}, {"Georgia", "Gruzija, standardno vreme"}, {"Gilbert_Islands", "Gilbert ostrva vreme"}, {"Greenland_Eastern", "Istočni Grenland, standardno vreme"}, {"Greenland_Western", "Zapadni Grenland, standardno vreme"}, {"Guam", "Guam standardno vreme"}, {"Gulf", "Zalivsko vreme"}, {"Guyana", "Gvajana vreme"}, {"Hawaii", "Havajsko-aleutsko standardno vreme"}, {"Hawaii_Aleutian", "Havajsko-aleutsko standardno vreme"}, {"Hong_Kong", "Hong Kong, standardno vreme"}, {"Hovd", "Hovd, standardno vreme"}, {"India", "Indijsko standardno vreme"}, {"Indian_Ocean", "Indijsko okeansko vreme"}, {"Indochina", "Indokina vreme"}, {"Indonesia_Central", "Centralno-indonezijsko vreme"}, {"Indonesia_Eastern", "Istočno-indonezijsko vreme"}, {"Indonesia_Western", "Zapadno-indonezijsko vreme"}, {"Iran", "Iran, standardno vreme"}, {"Irkutsk", "Irkuck, standardno vreme"}, {"Israel", "Izraelsko standardno vreme"}, {"Japan", "Japansko standardno vreme"}, {"Kamchatka", "Petropavlovsko-kamčatsko standardno vreme"}, {"Kazakhstan", "Kazahstansko vreme"}, {"Kazakhstan_Eastern", "Istočno-kazahstansko vreme"}, {"Kazakhstan_Western", "Zapadno-kazahstansko vreme"}, {"Korea", "Korejsko standardno vreme"}, {"Kosrae", "Košre vreme"}, {"Krasnoyarsk", "Krasnojarsk, standardno vreme"}, {"Kyrgystan", "Kirgistan vreme"}, {"Lanka", "Šri Lanka vreme"}, {"Line_Islands", "Ostrva Lajn vreme"}, {"Lord_Howe", "Lord Hov, standardno vreme"}, {"Macau", "Makao standardno vreme"}, {"Magadan", "Magadan, standardno vreme"}, {"Malaysia", "Malezija vreme"}, {"Maldives", "Maldivi vreme"}, {"Marquesas", "Markiz vreme"}, {"Marshall_Islands", "Maršalska Ostrva vreme"}, {"Mauritius", "Mauricijus, standardno vreme"}, {"Mawson", "Moson vreme"}, {"Mexico_Pacific", "Meksički Pacifik, standardno vreme"}, {"Mongolia", "Ulan Bator, standardno vreme"}, {"Moscow", "Moskva, standardno vreme"}, {"Myanmar", "Mijanmar vreme"}, {"Nauru", "Nauru vreme"}, {"Nepal", "Nepal vreme"}, {"New_Caledonia", "Nova Kaledonija, standardno vreme"}, {"New_Zealand", "Novi Zeland, standardno vreme"}, {"Newfoundland", "Njufaundlend, standardno vreme"}, {"Niue", "Niue vreme"}, {"Norfolk", "Norfolk Ostrvo, standardno vreme"}, {"Noronha", "Fernando de Noronja, standardno vreme"}, {"North_Mariana", "Severna Marijanska Ostrva vreme"}, {"Novosibirsk", "Novosibirsk, standardno vreme"}, {"Omsk", "Omsk, standardno vreme"}, {"Pakistan", "Pakistan, standardno vreme"}, {"Palau", "Palau vreme"}, {"Papua_New_Guinea", "Papua Nova Gvineja vreme"}, {"Paraguay", "Paragvaj, standardno vreme"}, {"Peru", "Peru, standardno vreme"}, {"Philippines", "Filipini, standardno vreme"}, {"Phoenix_Islands", "Feniks ostrva vreme"}, {"Pierre_Miquelon", "Sen Pjer i Mikelon, standardno vreme"}, {"Pitcairn", "Pitkern vreme"}, {"Ponape", "Ponpej vreme"}, {"Pyongyang", "Pjongjanško vreme"}, {"Qyzylorda", "Kizilorda standardno vreme"}, {"Reunion", "Reinion vreme"}, {"Rothera", "Rotera vreme"}, {"Sakhalin", "Sahalin, standardno vreme"}, {"Samara", "Samara standardno vreme"}, {"Samoa", "Samoa, standardno vreme"}, {"Seychelles", "Sejšeli vreme"}, {"Singapore", "Singapur, standardno vreme"}, {"Solomon", "Solomonska Ostrva vreme"}, {"South_Georgia", "Južna Džordžija vreme"}, {"Suriname", "Surinam vreme"}, {"Syowa", "Šova vreme"}, {"Tahiti", "Tahiti vreme"}, {"Taipei", "Tajpej, standardno vreme"}, {"Tajikistan", "Tadžikistan vreme"}, {"Tokelau", "Tokelau vreme"}, {"Tonga", "Tonga, standardno vreme"}, {"Truk", "Čuuk vreme"}, {"Turkey", "Turska standardno vreme"}, {"Turkmenistan", "Turkmenistan, standardno vreme"}, {"Tuvalu", "Tuvalu vreme"}, {"Uruguay", "Urugvaj, standardno vreme"}, {"Uzbekistan", "Uzbekistan, standardno vreme"}, {"Vanuatu", "Vanuatu, standardno vreme"}, {"Venezuela", "Venecuela vreme"}, {"Vladivostok", "Vladivostok, standardno vreme"}, {"Volgograd", "Volgograd, standardno vreme"}, {"Vostok", "Vostok vreme"}, {"Wake", "Vejk ostrvo vreme"}, {"Wallis", "Valis i Futuna Ostrva vreme"}, {"Yakutsk", "Jakutsk, standardno vreme"}, {"Yekaterinburg", "Jekaterinburg, standardno vreme"}, {"Yukon", "Jukon"}},
		{{"Acre", "Akre letnje računanje vremena"}, {"Alaska", "Aljaska, letnje vreme"}, {"Almaty", "Almati letnje računanje vremena"}, {"Amazon", "Amazon, letnje vreme"}, {"America_Central", "Severnoameričko centralno letnje vreme"}, {"America_Eastern", "Severnoameričko istočno letnje vreme"}, {"America_Mountain", "Severnoameričko planinsko letnje vreme"}, {"America_Pacific", "Severnoameričko pacifičko letnje vreme"}, {"Anadyr", "Anadir letnje računanje vremena"}, {"Apia", "Apija, letnje vreme"}, {"Aqtau", "Akvatau letnje računanje vremena"}, {"Aqtobe", "Akutobe letnje računanje vremena"}, {"Arabian", "Arabijsko letnje vreme"}, {"Argentina", "Argentina, letnje vreme"}, {"Argentina_Western", "Zapadna Argentina, letnje vreme"}, {"Armenia", "Jermenija, letnje vreme"}, {"Atlantic", "Atlantsko letnje vreme"}, {"Australia_Central", "Australijsko centralno letnje vreme"}, {"Australia_CentralWestern", "Australijsko centralno zapadno letnje vreme"}, {"Australia_Eastern", "Australijsko istočno letnje vreme"}, {"Australia_Western", "Australijsko zapadno letnje vreme"}, {"Azerbaijan", "Azerbejdžan, letnje vreme"}, {"Azores", "Azori, letnje vreme"}, {"Bangladesh", "Bangladeš, letnje vreme"}, {"Brasilia", "Brazilija, letnje vreme"}, {"Cape_Verde", "Zelenortska Ostrva, letnje vreme"}, {"Chatham", "Čatam, letnje vreme"}, {"Chile", "Čile, letnje vreme"}, {"China", "Kina, letnje vreme"}, {"Colombia", "Kolumbija, letnje vreme"}, {"Cook", "Kukova ostrva, polu-letnje vreme"}, {"Cuba", "Kuba, letnje vreme"}, {"Easter", "Uskršnja ostrva, letnje vreme"}, {"Europe_Central", "Srednjeevropsko letnje vreme"}, {"Europe_Eastern", "Istočnoevropsko letnje vreme"}, {"Europe_Western", "Zapadnoevropsko letnje vreme"}, {"Falkland", "Folklandska Ostrva, letnje vreme"}, {"Fiji", "Fidži, letnje vreme"}, {"Georgia", "Gruzija, letnje vreme"}, {"Greenland_Eastern", "Istočni Grenland, letnje vreme"}, {"Greenland_Western", "Zapadni Grenland, letnje vreme"}, {"Hawaii_Aleutian", "Havajsko-aleutsko letnje vreme"}, {"Hong_Kong", "Hong Kong, letnje vreme"}, {"Hovd", "Hovd, letnje vreme"}, {"Iran", "Iran, letnje vreme"}, {"Irkutsk", "Irkuck, letnje vreme"}, {"Israel", "Izraelsko letnje vreme"}, {"Japan", "Japansko letnje vreme"}, {"Kamchatka", "Petropavlovsko-kamčatsko letnje računanje vremena"}, {"Korea", "Korejsko letnje vreme"}, {"Krasnoyarsk", "Krasnojarsk, letnje vreme"}, {"Lord_Howe", "Lord Hov, letnje vreme"}, {"Macau", "Makao letnje računanje vremena"}, {"Magadan", "Magadan, letnje vreme"}, {"Mauritius", "Mauricijus, letnje vreme"}, {"Mexico_Pacific", "Meksički Pacifik, letnje vreme"}, {"Mongolia", "Ulan Bator, letnje vreme"}, {"Moscow", "Moskva, letnje vreme"}, {"New_Caledonia", "Nova Kaledonija, letnje vreme"}, {"New_Zealand", "Novi Zeland, letnje vreme"}, {"Newfoundland", "Njufaundlend, letnje vreme"}, {"Norfolk", "Norfolk Ostrvo, letnje vreme"}, {"Noronha", "Fernando de Noronja, letnje vreme"}, {"Novosibirsk", "Novosibirsk, letnje vreme"}, {"Omsk", "Omsk, letnje vreme"}, {"Pakistan", "Pakistan, letnje vreme"}, {"Paraguay", "Paragvaj, letnje vreme"}, {"Peru", "Peru, letnje vreme"}, {"Philippines", "Filipini, letnje vreme"}, {"Pierre_Miquelon", "Sen Pjer i Mikelon, letnje vreme"}, {"Qyzylorda", "Kizilorda letnje računanje vremena"}, {"Sakhalin", "Sahalin, letnje vreme"}, {"Samara", "Samara letnje računanje vremena"}, {"Samoa", "Samoa, letnje vreme"}, {"Taipei", "Tajpej, letnje vreme"}, {"Tonga", "Tonga, letnje vreme"}, {"Turkey", "Turska letnje računanje vremena"}, {"Turkmenistan", "Turkmenistan, letnje vreme"}, {"Uruguay", "Urugvaj, letnje vreme"}, {"Uzbekistan", "Uzbekistan, letnje vreme"}, {"Vanuatu", "Vanuatu, letnje vreme"}, {"Vladivostok", "Vladivostok, letnje vreme"}, {"Volgograd", "Volgograd, letnje vreme"}, {"Yakutsk", "Jakutsk, letnje vreme"}, {"Yekaterinburg", "Jekaterinburg, letnje vreme"}},
	},
	{
		{{"Acre", "Akre standardno vreme"}, {"Afghanistan", "Avganistan vrijeme"}, {"Africa_Central", "Centralno-afričko vrijeme"}, {"Africa_Eastern", "Istočno-afričko vrijeme"}, {"Africa_Southern", "Južno-afričko vrijeme"}, {"Africa_Western", "Zapadno-afričko vrijeme"}, {"Alaska", "Aljaska, standardno vrijeme"}, {"Almaty", "Almati standardno vreme"}, {"Amazon", "Amazon, standardno vrijeme"}, {"America_Central", "Sjevernoameričko centralno standardno vrijeme"}, {"America_Eastern", "Sjevernoameričko istočno standardno vrijeme"}, {"America_Mountain", "Sjevernoameričko planinsko standardno vrijeme"}, {"America_Pacific", "Sjevernoameričko pacifičko standardno vrijeme"}, {"Anadyr", "Anadir standardno vreme"}, {"Apia", "Apija, standardno vrijeme"}, {"Aqtau", "Akvatau standardno vreme"}, {"Aqtobe", "Akutobe standardno vreme"}, {"Arabian", "Arabijsko standardno vrijeme"}, {"Argentina", "Argentina, standardno vrijeme"}, {"Argentina_Western", "Zapadna Argentina, standardno vrijeme"}, {"Armenia", "Jermenija, standardno vrijeme"}, {"Atlantic", "Atlantsko standardno vrijeme"}, {"Australia_Central", "Australijsko centralno standardno vrijeme"}, {"Australia_CentralWestern", "Australijsko centralno zapadno standardno vrijeme"}, {"Australia_Eastern", "Australijsko istočno standardno vrijeme"}, {"Australia_Western", "Australijsko zapadno standardno vrijeme"}, {"Azerbaijan", "Azerbejdžan, standardno vrijeme"}, {"Azores", "Azori, standardno vrijeme"}, {"Bangladesh", "Bangladeš, standardno vrijeme"}, {"Bhutan", "Butan vrijeme"}, {"Bolivia", "Bolivija vrijeme"}, {"Brasilia", "Brazilija, standardno vrijeme"}, {"Brunei", "Brunej Darusalum vrijeme"}, {"Cape_Verde", "Zelenortska Ostrva, standardno vrijeme"}, {"Chamorro", "Čamoro vrijeme"}, {"Chatham", "Čatam, standardno vrijeme"}, {"Chile", "Čile, standardno vrijeme"}, {"China", "Kinesko standardno vrijeme"}, {"Christmas", "Božićno ostrvo vrijeme"}, {"Cocos", "Kokosova (Kiling) ostrva vrijeme"}, {"Colombia", "Kolumbija, standardno vrijeme"}, {"Cook", "Kukova Ostrva, standardno vrijeme"}, {"Cuba", "Kuba, standardno vrijeme"}, {"Davis", "Dejvis vrijeme"}, {"DumontDUrville", "Dimon d’Irvil vrijeme"}, {"East_Timor", "Istočni Timor vrijeme"}, {"Easter", "Uskršnja ostrva, standardno vrijeme"}, {"Ecuador", "Ekvador vrijeme"}, {"Europe_Central", "Srednjoevropsko standardno vrijeme"}, {"Europe_Eastern", "Istočnoevropsko standardno vrijeme"}, {"Europe_Further_Eastern", "Vreme daljeg istoka Evrope"}, {"Europe_Western", "Zapadnoevropsko standardno vrijeme"}, {"Falkland", "Folklandska Ostrva, standardno vrijeme"}, {"Fiji", "Fidži, standardno vrijeme"}, {"French_Guiana", "Francuska Gvajana vrijeme"}, {"French_Southern", "Francusko južno i antarktičko vrijeme"}, {"GMT", "Srednje vrijeme po Griniču"}, {"Galapagos", "Galapagos vrijeme"}, {"Gambier", "Gambije vrijeme"}, {"Georgia", "Gruzija, standardno vrijeme"}, {"Gilbert_Islands", "Gilbertova ostrva vrijeme"}, {"Greenland_Eastern", "Istočni Grenland, standardno vrijeme"}, {"Greenland_Western", "Zapadni Grenland, standardno vrijeme"}, {"Guam", "Guam standardno vreme"}, {"Gulf", "Zalivsko vrijeme"}, {"Guyana", "Gvajana vrijeme"}, {"Hawaii", "Havajsko-aleutsko standardno vrijeme"}, {"Hawaii_Aleutian", "Havajsko-aleutsko standardno vrijeme"}, {"Hong_Kong", "Hong Kong, standardno vrijeme"}, {"Hovd", "Hovd, standardno vrijeme"}, {"India", "Indijsko standardno vrijeme"}, {"Indian_Ocean", "Indijsko okeansko vrijeme"}, {"Indochina", "Indokina vrijeme"}, {"Indonesia_Central", "Centralno-indonezijsko vrijeme"}, {"Indonesia_Eastern", "Istočno-indonezijsko vrijeme"}, {"Indonesia_Western", "Zapadno-indonezijsko vrijeme"}, {"Iran", "Iran, standardno vrijeme"}, {"Irkutsk", "Irkuck, standardno vrijeme"}, {"Israel", "Izraelsko standardno vrijeme"}, {"Japan", "Japansko standardno vrijeme"}, {"Kamchatka", "Petropavlovsko-kamčatsko standardno vreme"}, {"Kazakhstan", "Kazahstansko vrijeme"}, {"Kazakhstan_Eastern", "Istočno-kazahstansko vrijeme"}, {"Kazakhstan_Western", "Zapadno-kazahstansko vrijeme"}, {"Korea", "Korejsko standardno vrijeme"}, {"Kosrae", "Košre vrijeme"}, {"Krasnoyarsk", "Krasnojarsk, standardno vrijeme"}, {"Kyrgystan", "Kirgistan vrijeme"}, {"Lanka", "Šri Lanka vreme"}, {"Line_Islands", "Linijska ostrva vrijeme"}, {"Lord_Howe", "Lord Hov, standardno vrijeme"}, {"Macau", "Makao standardno vreme"}, {"Magadan", "Magadan, standardno vrijeme"}, {"Malaysia", "Malezija vrijeme"}, {"Maldives", "Maldivi vrijeme"}, {"Marquesas", "Markiz vrijeme"}, {"Marshall_Islands", "Maršalska Ostrva vrijeme"}, {"Mauritius", "Mauricijus, standardno vrijeme"}, {"Mawson", "Moson vrijeme"}, {"Mexico_Pacific", "Meksički Pacifik, standardno vrijeme"}, {"Mongolia", "Ulan Bator, standardno vrijeme"}, {"Moscow", "Moskva, standardno vrijeme"}, {"Myanmar", "Mjanmar vrijeme"}, {"Nauru", "Nauru vrijeme"}, {"Nepal", "Nepal vrijeme"}, {"New_Caledonia", "Nova Kaledonija, standardno vrijeme"}, {"New_Zealand", "Novi Zeland, standardno vrijeme"}, {"Newfoundland", "Njufaundlend, standardno vrijeme"}, {"Niue", "Nijue vrijeme"}, {"Norfolk", "ostrvo Norfolk, standardno vrijeme"}, {"Noronha", "Fernando de Noronja, standardno vrijeme"}, {"North_Mariana", "Severna Marijanska Ostrva vreme"}, {"Novosibirsk", "Novosibirsk, standardno vrijeme"}, {"Omsk", "Omsk, standardno vrijeme"}, {"Pakistan", "Pakistan, standardno vrijeme"}, {"Palau", "Palau vrijeme"}, {"Papua_New_Guinea", "Papua Nova Gvineja vrijeme"}, {"Paraguay", "Paragvaj, standardno vrijeme"}, {"Peru", "Peru, standardno vrijeme"}, {"Philippines", "Filipini, standardno vrijeme"}, {"Phoenix_Islands", "Feniks ostrva vrijeme"}, {"Pierre_Miquelon", "Sen Pjer i Mikelon, standardno vrijeme"}, {"Pitcairn", "Pitkern vrijeme"}, {"Ponape", "Ponpej vrijeme"}, {"Pyongyang", "Pjongjanško vrijeme"}, {"Qyzylorda", "Kizilorda standardno vreme"}, {"Reunion", "Reunion vrijeme"}, {"Rothera", "Rotera vrijeme"}, {"Sakhalin", "Sahalin, standardno vrijeme"}, {"Samara", "Samara standardno vreme"}, {"Samoa", "Samoa, standardno vrijeme"}, {"Seychelles", "Sejšeli vrijeme"}, {"Singapore", "Singapur, standardno vrijeme"}, {"Solomon", "Solomonska Ostrva vrijeme"}, {"South_Georgia", "Južna Džordžija vrijeme"}, {"Suriname", "Surinam vrijeme"}, {"Syowa", "Šova vrijeme"}, {"Tahiti", "Tahiti vrijeme"}, {"Taipei", "Tajpej, standardno vrijeme"}, {"Tajikistan", "Tadžikistan vrijeme"}, {"Tokelau", "Tokelau vrijeme"}, {"Tonga", "Tonga, standardno vrijeme"}, {"Truk", "Čuk vrijeme"}, {"Turkey", "Turska standardno vreme"}, {"Turkmenistan", "Turkmenistan, standardno vrijeme"}, {"Tuvalu", "Tuvalu vrijeme"}, {"Uruguay", "Urugvaj, standardno vrijeme"}, {"Uzbekistan", "Uzbekistan, standardno vrijeme"}, {"Vanuatu", "Vanuatu, standardno vrijeme"}, {"Venezuela", "Venecuela vrijeme"}, {"Vladivostok", "Vladivostok, standardno vrijeme"}, {"Volgograd", "Volgograd, standardno vrijeme"}, {"Vostok", "Vostok vrijeme"}, {"Wake", "ostrvo Vejk vrijeme"}, {"Wallis", "ostrva Valis i Futuna vrijeme"}, {"Yakutsk", "Jakutsk, standardno vrijeme"}, {"Yekaterinburg", "Jekaterinburg, standardno vrijeme"}, {"Yukon", "Jukon"}},
		{{"Acre", "Akre letnje računanje vremena"}, {"Alaska", "Aljaska, ljetnje vrijeme"}, {"Almaty", "Almati letnje računanje vremena"}, {"Amazon", "Amazon, ljetnje vrijeme"}, {"America_Central", "Sjevernoameričko centralno ljetnje vrijeme"}, {"America_Eastern", "Sjevernoameričko istočno ljetnje vrijeme"}, {"America_Mountain", "Sjevernoameričko planinsko ljetnje vrijeme"}, {"America_Pacific", "Sjevernoameričko pacifičko letnje vrijeme"}, {"Anadyr", "Anadir letnje računanje vremena"}, {"Apia", "Apija, ljetnje vrijeme"}, {"Aqtau", "Akvatau letnje računanje vremena"}, {"Aqtobe", "Akutobe letnje računanje vremena"}, {"Arabian", "Arabijsko ljetnje vrijeme"}, {"Argentina", "Argentina, ljetnje vrijeme"}, {"Argentina_Western", "Zapadna Argentina, ljetnje vrijeme"}, {"Armenia", "Jermenija, ljetnje vrijeme"}, {"Atlantic", "Atlantsko ljetnje vrijeme"}, {"Australia_Central", "Australijsko centralno ljetnje vrijeme"}, {"Australia_CentralWestern", "Australijsko centralno zapadno ljetnje vrijeme"}, {"Australia_Eastern", "Australijsko istočno ljetnje vrijeme"}, {"Australia_Western", "Australijsko zapadno ljetnje vrijeme"}, {"Azerbaijan", "Azerbejdžan, ljetnje vrijeme"}, {"Azores", "Azori, ljetnje vrijeme"}, {"Bangladesh", "Bangladeš, ljetnje vrijeme"}, {"Brasilia", "Brazilija, ljetnje vrijeme"}, {"Cape_Verde", "Zelenortska Ostrva, ljetnje vrijeme"}, {"Chatham", "Čatam, ljetnje vrijeme"}, {"Chile", "Čile, ljetnje vrijeme"}, {"China", "Kina, ljetnje vrijeme"}, {"Colombia", "Kolumbija, ljetnje vrijeme"}, {"Cook", "Kukova Ostrva, poluljetnje vrijeme"}, {"Cuba", "Kuba, ljetnje vrijeme"}, {"Easter", "Uskršnja ostrva, ljetnje vrijeme"}, {"Europe_Central", "Srednjoevropsko ljetnje vrijeme"}, {"Europe_Eastern", "Istočnoevropsko ljetnje vrijeme"}, {"Europe_Western", "Zapadnoevropsko ljetnje vrijeme"}, {"Falkland", "Folklandska Ostrva, ljetnje vrijeme"}, {"Fiji", "Fidži, ljetnje vrijeme"}, {"Georgia", "Gruzija, ljetnje vrijeme"}, {"Greenland_Eastern", "Istočni Grenland, ljetnje vrijeme"}, {"Greenland_Western", "Zapadni Grenland, ljetnje vrijeme"}, {"Hawaii_Aleutian", "Havajsko-aleutsko ljetnje vrijeme"}, {"Hong_Kong", "Hong Kong, ljetnje vrijeme"}, {"Hovd", "Hovd, ljetnje vrijeme"}, {"Iran", "Iran, ljetnje vrijeme"}, {"Irkutsk", "Irkuck, ljetnje vrijeme"}, {"Israel", "Izraelsko ljetnje vrijeme"}, {"Japan", "Japansko ljetnje vrijeme"}, {"Kamchatka", "Petropavlovsko-kamčatsko letnje računanje vremena"}, {"Korea", "Korejsko ljetnje vrijeme"}, {"Krasnoyarsk", "Krasnojarsk, ljetnje vrijeme"}, {"Lord_Howe", "Lord Hov, ljetnje vrijeme"}, {"Macau", "Makao letnje računanje vremena"}, {"Magadan", "Magadan, ljetnje vrijeme"}, {"Mauritius", "Mauricijus, ljetnje vrijeme"}, {"Mexico_Pacific", "Meksički Pacifik, ljetnje vrijeme"}, {"Mongolia", "Ulan Bator, ljetnje vrijeeme"}, {"Moscow", "Moskva, ljetnje vrijeme"}, {"New_Caledonia", "Nova Kaledonija, ljetnje vrijeme"}, {"New_Zealand", "Novi Zeland, ljetnje vrijeme"}, {"Newfoundland", "Njufaundlend, ljetnje vrijeme"}, {"Norfolk", "ostrvo Norfolk, ljetnje vrijeme"}, {"Noronha", "Fernando de Noronja, ljetnje vrijeme"}, {"Novosibirsk", "Novosibirsk, ljetnje vrijeme"}, {"Omsk", "Omsk, ljetnje vrijeme"}, {"Pakistan", "Pakistan, ljetnje vrijeme"}, {"Paraguay", "Paragvaj, ljetnje vrijeme"}, {"Peru", "Peru, ljetnje vrijeme"}, {"Philippines", "Filipini, ljetnje vrijeme"}, {"Pierre_Miquelon", "Sen Pjer i Mikelon, ljetnje vrijeme"}, {"Qyzylorda", "Kizilorda letnje računanje vremena"}, {"Sakhalin", "Sahalin, ljetnje vrijeme"}, {"Samara", "Samara letnje računanje vremena"}, {"Samoa", "Samoa, ljetnje vrijeme"}, {"Taipei", "Tajpej, ljetnje vrijeme"}, {"Tonga", "Tonga, ljetnje vrijeme"}, {"Turkey", "Turska letnje računanje vremena"}, {"Turkmenistan", "Turkmenistan, ljetnje vrijeme"}, {"Uruguay", "Urugvaj, ljetnje vrijeme"}, {"Uzbekistan", "Uzbekistan, ljetnje vrijeme"}, {"Vanuatu", "Vanuatu, ljetnje vrijeme"}, {"Vladivostok", "Vladivostok, ljetnje vrijeme"}, {"Volgograd", "Volgograd, ljetnje vrijeme"}, {"Yakutsk", "Jakutsk, ljetnje vrijeme"}, {"Yekaterinburg", "Jekaterinburg, ljetnje vrijeme"}},
	},
	{
		{{"America_Central", "Waktu Standar Tengah"}, {"America_Eastern", "Waktu Standar Wétan"}, {"America_Mountain", "Waktu Standar Pagunungan"}, {"America_Pacific", "Waktu Standar Pasifik"}, {"Atlantic", "Waktu Standar Atlantik"}, {"Colombia", "Waktu Standar Kolombia"}, {"Europe_Central", "Waktu Standar Éropa Tengah"}, {"Europe_Eastern", "Waktu Standar Éropa Timur"}, {"Europe_Western", "Waktu Standar Éropa Barat"}, {"GMT", "Waktu Greenwich"}, {"Galapagos", "Waktu Galapagos"}},
		{{"America_Central", "Waktu Usum Panas Tengah"}, {"America_Eastern", "Waktu Usum Panas Wétan"}, {"America_Mountain", "Waktu Usum Panas Pagunungan"}, {"America_Pacific", "Waktu Usum Panas Pasifik"}, {"Atlantic", "Waktu Usum Panas Atlantik"}, {"Colombia", "Waktu Usum Panas Kolombia"}, {"Europe_Central", "Waktu Usum Panas Éropa Tengah"}, {"Europe_Eastern", "Waktu Usum Panas Éropa Timur"}, {"Europe_Western", "Waktu Usum Panas Éropa Barat"}},
	},
	{
		{{"Acre", "västbrasiliansk normaltid"}, {"Afghanistan", "afghansk tid"}, {"Africa_Central", "centralafrikansk tid"}, {"Africa_Eastern", "östafrikansk tid"}, {"Africa_Southern", "sydafrikansk tid"}, {"Africa_Western", "västafrikansk tid"}, {"Alaska", "Alaska, normaltid"}, {"Almaty", "Almatynormaltid"}, {"Amazon", "Amazonas, normaltid"}, {"America_Central", "centralnordamerikansk normaltid"}, {"America_Eastern", "östnordamerikansk normaltid"}, {"America_Mountain", "Klippiga bergen, normaltid"}, {"America_Pacific", "västnordamerikansk normaltid"}, {"Anadyr", "Anadyrnormaltid"}, {"Apia", "Apia, normaltid"}, {"Aqtau", "Aqtaunormaltid"}, {"Aqtobe", "Aqtöbenormaltid"}, {"Arabian", "saudiarabisk normaltid"}, {"Argentina", "östargentinsk normaltid"}, {"Argentina_Western", "västargentinsk normaltid"}, {"Armenia", "armenisk normaltid"}, {"Atlantic", "nordamerikansk atlantnormaltid"}, {"Australia_Central", "centralaustralisk normaltid"}, {"Australia_CentralWestern", "västcentralaustralisk normaltid"}, {"Australia_Eastern", "östaustralisk normaltid"}, {"Australia_Western", "västaustralisk normaltid"}, {"Azerbaijan", "azerbajdzjansk normaltid"}, {"Azores", "azorisk normaltid"}, {"Bangladesh", "bangladeshisk normaltid"}, {"Bhutan", "bhutansk tid"}, {"Bolivia", "boliviansk tid"}, {"Brasilia", "Brasilia, normaltid"}, {"Brunei", "Bruneitid"}, {"Cape_Verde", "Kap Verde, normaltid"}, {"Casey", "Caseytid"}, {"Chamorro", "Chamorrotid"}, {"Chatham", "Chatham, normaltid"}, {"Chile", "chilensk normaltid"}, {"China", "kinesisk normaltid"}, {"Christmas", "Julöns tid"}, {"Cocos", "Keelingöarnas tid"}, {"Colombia", "colombiansk normaltid"}, {"Cook", "Cooköarnas normaltid"}, {"Cuba", "kubansk normaltid"}, {"Davis", "Davistid"}, {"DumontDUrville", "Dumont d’Urville-tid"}, {"East_Timor", "östtimorisk tid"}, {"Easter", "Påskön, normaltid"}, {"Ecuador", "ecuadoriansk tid"}, {"Europe_Central", "centraleuropeisk normaltid"}, {"Europe_Eastern", "östeuropeisk normaltid"}, {"Europe_Further_Eastern", "Kaliningradtid"}, {"Europe_Western", "västeuropeisk normaltid"}, {"Falkland", "Falklandsöarna, normaltid"}, {"Fiji", "Fiji, normaltid"}, {"French_Guiana", "Franska Guyanatid"}, {"French_Southern", "Franska Sydterritoriernas tid"}, {"GMT", "Greenwichtid"}, {"Galapagos", "Galápagostid"}, {"Gambier", "Gambiertid"}, {"Georgia", "georgisk normaltid"}, {"Gilbert_Islands", "Kiribatitid"}, {"Greenland_Eastern", "östgrönländsk normaltid"}, {"Greenland_Western", "västgrönländsk normaltid"}, {"Guam", "Guamtid"}, {"Gulf", "Persiska vikentid"}, {"Guyana", "Guyanatid"}, {"Hawaii", "Honolulu, normaltid"}, {"Hawaii_Aleutian", "aleutisk normaltid"}, {"Hong_Kong", "Hongkong, normaltid"}, {"Hovd", "Chovd, normaltid"}, {"India", "indisk tid"}, {"Indian_Ocean", "Brittiska Indiska oceanöarnas tid"}, {"Indochina", "indokinesisk tid"}, {"Indonesia_Central", "centralindonesisk tid"}, {"Indonesia_Eastern", "östindonesisk tid"}, {"Indonesia_Western", "västindonesisk tid"}, {"Iran", "iransk normaltid"}, {"Irkutsk", "Irkutsk, normaltid"}, {"Israel", "israelisk normaltid"}, {"Japan", "japansk normaltid"}, {"Kamchatka", "Kamtjatkanormaltid"}, {"Kazakhstan", "kazakstansk tid"}, {"Kazakhstan_Eastern", "östkazakstansk tid"}, {"Kazakhstan_Western", "västkazakstansk tid"}, {"Korea", "koreansk normaltid"}, {"Kosrae", "Kosraetid"}, {"Krasnoyarsk", "Krasnojarsk, normaltid"}, {"Kyrgystan", "kirgizisk tid"}, {"Lanka", "Sri Lankatid"}, {"Line_Islands", "Lineöarnas tid"}, {"Lord_Howe", "Lord Howe, normaltid"}, {"Macau", "Macaonormaltid"}, {"Magadan", "Magadan, normaltid"}, {"Malaysia", "malaysisk tid"}, {"Maldives", "Maldivernatid"}, {"Marquesas", "Marquesastid"}, {"Marshall_Islands", "Marshallöarnas tid"}, {"Mauritius", "Mauritius, normaltid"}, {"Mawson", "Mawsontid"}, {"Mexico_Pacific", "mexikansk stillahavstid, normaltid"}, {"Mongolia", "Ulaanbaatar, normaltid"}, {"Moscow", "Moskva, normaltid"}, {"Myanmar", "burmesisk tid"}, {"Nauru", "Naurutid"}, {"Nepal", "nepalesisk tid"}, {"New_Caledonia", "Nya Kaledonien, normaltid"}, {"New_Zealand", "nyzeeländsk normaltid"}, {"Newfoundland", "Newfoundland, normaltid"}, {"Niue", "Niuetid"}, {"Norfolk", "Norfolköns normaltid"}, {"Noronha", "Fernando de Noronha, normaltid"}, {"North_Mariana", "Nordmarianernas tid"}, {"Novosibirsk", "Novosibirsk, normaltid"}, {"Omsk", "Omsk, normaltid"}, {"Pakistan", "pakistansk normaltid"}, {"Palau", "Palautid"}, {"Papua_New_Guinea", "Papua Nya Guineas tid"}, {"Paraguay", "paraguayansk normaltid"}, {"Peru", "peruansk normaltid"}, {"Philippines", "filippinsk normaltid"}, {"Phoenix_Islands", "Enderburytid"}, {"Pierre_Miquelon", "Saint-Pierre-et-Miquelon, normaltid"}, {"Pitcairn", "Pitcairntid"}, {"Ponape", "Ponapetid"}, {"Pyongyang", "Pyongyangtid"}, {"Qyzylorda", "Qyzylordanormaltid"}, {"Reunion", "Réuniontid"}, {"Rothera", "Rotheratid"}, {"Sakhalin", "Sachalin, normaltid"}, {"Samara", "Samaranormaltid"}, {"Samoa", "samoansk normaltid"}, {"Seychelles", "Seychellernatid"}, {"Singapore", "Singaporetid"}, {"Solomon", "Salomonöarnas tid"}, {"South_Georgia", "sydgeorgisk tid"}, {"Suriname", "Surinamtid"}, {"Syowa", "Syowatid"}, {"Tahiti", "Tahititid"}, {"Taipei", "Taipei, normaltid"}, {"Tajikistan", "Tadzjikistantid"}, {"Tokelau", "Tokelautid"}, {"Tonga", "Tonga, normaltid"}, {"Truk", "Chuuktid"}, {"Turkey", "turkisk normaltid"}, {"Turkmenistan", "turkmensk normaltid"}, {"Tuvalu", "Tuvalutid"}, {"Uruguay", "uruguayansk normaltid"}, {"Uzbekistan", "uzbekisk normaltid"}, {"Vanuatu", "Vanuatu, normaltid"}, {"Venezuela", "venezuelansk tid"}, {"Vladivostok", "Vladivostok, normaltid"}, {"Volgograd", "Volgograd, normaltid"}, {"Vostok", "Vostoktid"}, {"Wake", "Wakeöarnas tid"}, {"Wallis", "Wallis- och Futunaöarnas tid"}, {"Yakutsk", "Jakutsk, normaltid"}, {"Yekaterinburg", "Jekaterinburg, normaltid"}, {"Yukon", "Yukontid"}},
		{{"Acre", "västbrasiliansk sommartid"}, {"Alaska", "Alaska, sommartid"}, {"Almaty", "Almatysommartid"}, {"Amazon", "Amazonas, sommartid"}, {"America_Central", "centralnordamerikansk sommartid"}, {"America_Eastern", "östnordamerikansk sommartid"}, {"America_Mountain", "Klippiga bergen, sommartid"}, {"America_Pacific", "västnordamerikansk sommartid"}, {"Anadyr", "Anadyrsommartid"}, {"Apia", "Apia, sommartid"}, {"Aqtau", "Aqtausommartid"}, {"Aqtobe", "Aqtöbesommartid"}, {"Arabian", "saudiarabisk sommartid"}, {"Argentina", "östargentinsk sommartid"}, {"Argentina_Western", "västargentinsk sommartid"}, {"Armenia", "armenisk sommartid"}, {"Atlantic", "nordamerikansk atlantsommartid"}, {"Australia_Central", "centralaustralisk sommartid"}, {"Australia_CentralWestern", "västcentralaustralisk sommartid"}, {"Australia_Eastern", "östaustralisk sommartid"}, {"Australia_Western", "västaustralisk sommartid"}, {"Azerbaijan", "azerbajdzjansk sommartid"}, {"Azores", "azorisk sommartid"}, {"Bangladesh", "bangladeshisk sommartid"}, {"Brasilia", "Brasilia, sommartid"}, {"Cape_Verde", "Kap Verde, sommartid"}, {"Chatham", "Chatham, sommartid"}, {"Chile", "chilensk sommartid"}, {"China", "kinesisk sommartid"}, {"Colombia", "colombiansk sommartid"}, {"Cook", "Cooköarnas sommartid"}, {"Cuba", "kubansk sommartid"}, {"Easter", "Påskön, sommartid"}, {"Europe_Central", "centraleuropeisk sommartid"}, {"Europe_Eastern", "östeuropeisk sommartid"}, {"Europe_Western", "västeuropeisk sommartid"}, {"Falkland", "Falklandsöarna, sommartid"}, {"Fiji", "Fiji, sommartid"}, {"Georgia", "georgisk sommartid"}, {"Greenland_Eastern", "östgrönländsk sommartid"}, {"Greenland_Western", "västgrönländsk sommartid"}, {"Hawaii_Aleutian", "aleutisk sommartid"}, {"Hong_Kong", "Hongkong, sommartid"}, {"Hovd", "Chovd, sommartid"}, {"Iran", "iransk sommartid"}, {"Irkutsk", "Irkutsk, sommartid"}, {"Israel", "israelisk sommartid"}, {"Japan", "japansk sommartid"}, {"Kamchatka", "Kamtjatkasommartid"}, {"Korea", "koreansk sommartid"}, {"Krasnoyarsk", "Krasnojarsk, sommartid"}, {"Lord_Howe", "Lord Howe, sommartid"}, {"Macau", "Macaosommartid"}, {"Magadan", "Magadan, sommartid"}, {"Mauritius", "Mauritius, sommartid"}, {"Mexico_Pacific", "mexikansk stillahavstid, sommartid"}, {"Mongolia", "Ulaanbaatar, sommartid"}, {"Moscow", "Moskva, sommartid"}, {"New_Caledonia", "Nya Kaledonien, sommartid"}, {"New_Zealand", "nyzeeländsk sommartid"}, {"Newfoundland", "Newfoundland, sommartid"}, {"Norfolk", "Norfolköns sommartid"}, {"Noronha", "Fernando de Noronha, sommartid"}, {"Novosibirsk", "Novosibirsk, sommartid"}, {"Omsk", "Omsk, sommartid"}, {"Pakistan", "pakistansk sommartid"}, {"Paraguay", "paraguayansk sommartid"}, {"Peru", "peruansk sommartid"}, {"Philippines", "filippinsk sommartid"}, {"Pierre_Miquelon", "Saint-Pierre-et-Miquelon, sommartid"}, {"Qyzylorda", "Qyzylordasommartid"}, {"Sakhalin", "Sachalin, sommartid"}, {"Samara", "Samarasommartid"}, {"Samoa", "samoansk sommartid"}, {"Taipei", "Taipei, sommartid"}, {"Tonga", "Tonga, sommartid"}, {"Turkey", "turkisk sommartid"}, {"Turkmenistan", "turkmensk sommartid"}, {"Uruguay", "uruguayansk sommartid"}, {"Uzbekistan", "uzbekisk sommartid"}, {"Vanuatu", "Vanuatu, sommartid"}, {"Vladivostok", "Vladivostok, sommartid"}, {"Volgograd", "Volgograd, sommartid"}, {"Yakutsk", "Jakutsk, sommartid"}, {"Yekaterinburg", "Jekaterinburg, sommartid"}},
	},
	{
		{{"Afghanistan", "Saa za Afghanistan"}, {"Africa_Central", "Saa za Afrika ya Kati"}, {"Africa_Eastern", "Saa za Afrika Mashariki"}, {"Africa_Southern", "Saa za Wastani za Afrika Kusini"}, {"Africa_Western", "Saa za Afrika Magharibi"}, {"Alaska", "Saa za Wastani za Alaska"}, {"Amazon", "Saa za Wastani za Amazon"}, {"America_Central", "Saa za Wastani za Kati"}, {"America_Eastern", "Saa za Wastani za Mashariki"}, {"America_Mountain", "Saa za Wastani za Mountain"}, {"America_Pacific", "Saa za Wastani za Pasifiki"}, {"Anadyr", "Saa za Wastani za Anadyr"}, {"Apia", "Saa za Wastani za Apia"}, {"Arabian", "Saa za Wastani za Uarabuni"}, {"Argentina", "Saa za Wastani za Argentina"}, {"Argentina_Western", "Saa za Wastani za Magharibi mwa Argentina"}, {"Armenia", "Saa za Wastani za Armenia"}, {"Atlantic", "Saa za Wastani za Atlantiki"}, {"Australia_Central", "Saa za Wastani za Australia ya Kati"}, {"Australia_CentralWestern", "Saa za Wastani za Magharibi ya Kati ya Australia"}, {"Australia_Eastern", "Saa za Wastani za Mashariki mwa Australia"}, {"Australia_Western", "Saa za Wastani za Australia Magharibi"}, {"Azerbaijan", "Saa za Wastani za Azerbaijan"}, {"Azores", "Saa za Wastani za Azores"}, {"Bangladesh", "Saa za Wastani za Bangladesh"}, {"Bhutan", "Saa za Bhutan"}, {"Bolivia", "Saa za Bolivia"}, {"Brasilia", "Saa za Wastani za Brasilia"}, {"Brunei", "Saa za Brunei Darussalam"}, {"Cape_Verde", "Saa za Wastani za Cape Verde"}, {"Chamorro", "Saa za Wastani za Chamorro"}, {"Chatham", "Saa za Wastani za Chatham"}, {"Chile", "Saa za Wastani za Chile"}, {"China", "Saa za Wastani za Uchina"}, {"Christmas", "Saa za Kisiwa cha Krismasi"}, {"Cocos", "Saa za Visiwa vya Cocos"}, {"Colombia", "Saa za Wastani za Kolombia"}, {"Cook", "Saa za Wastani za Visiwa vya Cook"}, {"Cuba", "Saa za Wastani ya Kuba"}, {"Davis", "Saa za Davis"}, {"DumontDUrville", "Saa za Dumont-d’Urville"}, {"East_Timor", "Saa za Timor Mashariki"}, {"Easter", "Saa za Wastani za Kisiwa cha Easter"}, {"Ecuador", "Saa za Ekwado"}, {"Europe_Central", "Saa za Wastani za Ulaya ya Kati"}, {"Europe_Eastern", "Saa za Wastani za Mashariki mwa Ulaya"}, {"Europe_Further_Eastern", "Saa za Mashariki zaidi mwa Ulaya"}, {"Europe_Western", "Saa za Wastani za Magharibi mwa Ulaya"}, {"Falkland", "Saa za Wastani za Visiwa vya Falkland"}, {"Fiji", "Saa za Wastani za Fiji"}, {"French_Guiana", "Saa za Guiana ya Ufaransa"}, {"French_Southern", "Saa za Kusini mwa Ufaransa na Antaktiki"}, {"GMT", "Saa za Greenwich"}, {"Galapagos", "Saa za Galapagos"}, {"Gambier", "Saa za Gambier"}, {"Georgia", "Saa za Wastani za Jojia"}, {"Gilbert_Islands", "Saa za Visiwa vya Gilbert"}, {"Greenland_Eastern", "Saa za Wastani za Greenland Mashariki"}, {"Greenland_Western", "Saa za Wastani za Greenland Magharibi"}, {"Gulf", "Saa za Wastani za Gulf"}, {"Guyana", "Saa za Guyana"}, {"Hawaii", "Saa za Wastani za Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Saa za Wastani za Hawaii-Aleutian"}, {"Hong_Kong", "Saa za Wastani za Hong Kong"}, {"Hovd", "Saa za Wastani za Hovd"}, {"India", "Saa za Wastani za India"}, {"Indian_Ocean", "Saa za Bahari Hindi"}, {"Indochina", "Saa za Indochina"}, {"Indonesia_Central", "Saa za Indonesia ya Kati"}, {"Indonesia_Eastern", "Saa za Mashariki mwa Indonesia"}, {"Indonesia_Western", "Saa za Magharibi mwa Indonesia"}, {"Iran", "Saa za Wastani za Iran"}, {"Irkutsk", "Saa za Wastani za Irkutsk"}, {"Israel", "Saa za Wastani za Israeli"}, {"Japan", "Saa za Wastani za Japani"}, {"Kamchatka", "Saa za Wastani za Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Saa za Kazakhstan"}, {"Kazakhstan_Eastern", "Saa za Kazakhstan Mashariki"}, {"Kazakhstan_Western", "Saa za Kazakhstan Magharibi"}, {"Korea", "Saa za Wastani za Korea"}, {"Kosrae", "Saa za Kosrae"}, {"Krasnoyarsk", "Saa za Wastani za Krasnoyask"}, {"Kyrgystan", "Saa za Kyrgystan"}, {"Line_Islands", "Saa za Visiwa vya Line"}, {"Lord_Howe", "Saa za Wastani za Lord Howe"}, {"Magadan", "Saa za Wastani za Magadan"}, {"Malaysia", "Saa za Malaysia"}, {"Maldives", "Saa za Maldives"}, {"Marquesas", "Saa za Marquesas"}, {"Marshall_Islands", "Saa za Visiwa vya Marshall"}, {"Mauritius", "Saa za Wastani za Morisi"}, {"Mawson", "Saa za Mawson"}, {"Mexico_Pacific", "Saa za wastani za pasifiki za Meksiko"}, {"Mongolia", "Saa za Wastani za Ulan Bator"}, {"Moscow", "Saa za Wastani za Moscow"}, {"Myanmar", "Saa za Myanmar"}, {"Nauru", "Saa za Nauru"}, {"Nepal", "Saa za Nepal"}, {"New_Caledonia", "Saa za Wastani za New Caledonia"}, {"New_Zealand", "Saa za Wastani za New Zealand"}, {"Newfoundland", "Saa za Wastani za Newfoundland"}, {"Niue", "Saa za Niue"}, {"Norfolk", "Saa za Wastani za Kisiwa cha Norfolk"}, {"Noronha", "Saa za Wastani za Fernando de Noronha"}, {"Novosibirsk", "Saa za Wastani za Novosibirsk"}, {"Omsk", "Saa za Wastani za Omsk"}, {"Pakistan", "Saa za Wastani za Pakistan"}, {"Palau", "Saa za Palau"}, {"Papua_New_Guinea", "Saa za Papua New Guinea"}, {"Paraguay", "Saa za Wastani za Paragwai"}, {"Peru", "Saa za Wastani za Peru"}, {"Philippines", "Saa za Wastani za Ufilipino"}, {"Phoenix_Islands", "Saa za Visiwa vya Phoenix"}, {"Pierre_Miquelon", "Saa za Wastani ya Saint-Pierre na Miquelon"}, {"Pitcairn", "Saa za Pitcairn"}, {"Ponape", "Saa za Ponape"}, {"Pyongyang", "Saa za Pyongyang"}, {"Reunion", "Saa za Reunion"}, {"Rothera", "Saa za Rothera"}, {"Sakhalin", "Saa za Wastani za Sakhalin"}, {"Samara", "Saa za Wastani za Samara"}, {"Samoa", "Saa za Wastani za Samoa"}, {"Seychelles", "Saa za Ushelisheli"}, {"Singapore", "Saa za Wastani za Singapore"}, {"Solomon", "Saa za Visiwa vya Solomon"}, {"South_Georgia", "Saa za Georgia Kusini"}, {"Suriname", "Saa za Suriname"}, {"Syowa", "Saa za Syowa"}, {"Tahiti", "Saa za Tahiti"}, {"Taipei", "Saa za Wastani za Taipei"}, {"Tajikistan", "Saa za Tajikistan"}, {"Tokelau", "Saa za Tokelau"}, {"Tonga", "Saa za Wastani za Tonga"}, {"Truk", "Saa za Chuuk"}, {"Turkmenistan", "Saa za Wastani za Turkmenistan"}, {"Tuvalu", "Saa za Tuvalu"}, {"Uruguay", "Saa za Wastani za Urugwai"}, {"Uzbekistan", "Saa za Wastani za Uzbekistan"}, {"Vanuatu", "Saa za Wastani za Vanuatu"}, {"Venezuela", "Saa za Venezuela"}, {"Vladivostok", "Saa za Wastani za Vladivostok"}, {"Volgograd", "Saa za Wastani za Volgograd"}, {"Vostok", "Saa za Vostok"}, {"Wake", "Saa za Kisiwa cha Wake"}, {"Wallis", "Saa za Wallis na Futuna"}, {"Yakutsk", "Saa za Wastani za Yakutsk"}, {"Yekaterinburg", "Saa za Wastani za Yekaterinburg"}, {"Yukon", "Saa za Yukon"}},
		{{"Alaska", "Saa za Mchana za Alaska"}, {"Amazon", "Saa za Majira ya joto za Amazon"}, {"America_Central", "Saa za Mchana za Kati"}, {"America_Eastern", "Saa za Mchana za Mashariki"}, {"America_Mountain", "Saa za Mchana za Mountain"}, {"America_Pacific", "Saa za Mchana za Pasifiki"}, {"Anadyr", "Saa za Kiangazi za Anadyr"}, {"Apia", "Saa za Mchana za Apia"}, {"Arabian", "Saa za Mchana za Arabiani"}, {"Argentina", "Saa za Majira ya joto za Argentina"}, {"Argentina_Western", "Saa za Majira ya joto za Magharibi mwa Argentina"}, {"Armenia", "Saa za Majira ya joto za Armenia"}, {"Atlantic", "Saa za Mchana za Atlantiki"}, {"Australia_Central", "Saa za Mchana za Australia ya Kati"}, {"Australia_CentralWestern", "Saa za Mchana za Magharibi ya Kati ya Australia"}, {"Australia_Eastern", "Saa za Mchana za Mashariki mwa Australia"}, {"Australia_Western", "Saa za Mchana za Australia Magharibi"}, {"Azerbaijan", "Saa za Majira ya joto za Azerbaijan"}, {"Azores", "Saa za Majira ya joto za Azores"}, {"Bangladesh", "Saa za Majira ya joto za Bangladesh"}, {"Brasilia", "Saa za Majira ya joto za Brasilia"}, {"Cape_Verde", "Saa za Majira ya joto za Cape Verde"}, {"Chatham", "Saa za Mchana za Chatham"}, {"Chile", "Saa za Majira ya joto za Chile"}, {"China", "Saa za Mchana za Uchina"}, {"Colombia", "Saa za Majira ya joto za Kolombia"}, {"Cook", "Saa za Majira nusu ya joto za Visiwa Cook"}, {"Cuba", "Saa za Mchana za Kuba"}, {"Easter", "Saa za Majira ya joto za Kisiwa cha Easter"}, {"Europe_Central", "Saa za Majira ya joto za Ulaya ya Kati"}, {"Europe_Eastern", "Saa za Majira ya joto za Mashariki mwa Ulaya"}, {"Europe_Western", "Saa za Majira ya joto za Magharibi mwa Ulaya"}, {"Falkland", "Saa za Majira ya joto za Visiwa vya Falkland"}, {"Fiji", "Saa za Majira ya joto za Fiji"}, {"Georgia", "Saa za Majira ya joto za Jojia"}, {"Greenland_Eastern", "Saa za Majira ya joto za Greenland Mashariki"}, {"Greenland_Western", "Saa za Majira ya joto za Greenland Magharibi"}, {"Hawaii_Aleutian", "Saa za Mchana za Hawaii-Aleutian"}, {"Hong_Kong", "Saa za Majira ya joto za Hong Kong"}, {"Hovd", "Saa za Majira ya joto za Hovd"}, {"Iran", "Saa za Mchana za Iran"}, {"Irkutsk", "Saa za Majira ya joto za Irkutsk"}, {"Israel", "Saa za Mchana za Israeli"}, {"Japan", "Saa za Mchana za Japan"}, {"Kamchatka", "Saa za Kiangazi za Petropavlovsk-Kamchatski"}, {"Korea", "Saa za Mchana za Korea"}, {"Krasnoyarsk", "Saa za Majira ya joto za Krasnoyarsk"}, {"Lord_Howe", "Saa za Mchana za Lord Howe"}, {"Magadan", "Saa za Majira ya joto za Magadan"}, {"Mauritius", "Saa za Majira ya joto za Morisi"}, {"Mexico_Pacific", "Saa za mchana za pasifiki za Meksiko"}, {"Mongolia", "Saa za Majira ya joto za Ulan Bator"}, {"Moscow", "Saa za Majira ya joto za Moscow"}, {"New_Caledonia", "Saa za Majira ya joto za New Caledonia"}, {"New_Zealand", "Saa za Mchana za New Zealand"}, {"Newfoundland", "Saa za Mchana za Newfoundland"}, {"Norfolk", "Saa za Majira ya joto za Kisiwa cha Norfolk"}, {"Noronha", "Saa za Majira ya joto za Fernando de Noronha"}, {"Novosibirsk", "Saa za Majira ya joto za Novosibirsk"}, {"Omsk", "Saa za Majira ya joto za Omsk"}, {"Pakistan", "Saa za Majira ya joto za Pakistan"}, {"Paraguay", "Saa za Majira ya joto za Paragwai"}, {"Peru", "Saa za Majira ya joto za Peru"}, {"Philippines", "Saa za Majira ya joto za Ufilipino"}, {"Pierre_Miquelon", "Saa za Mchana za Saint-Pierre na Miquelon"}, {"Sakhalin", "Saa za Majira ya joto za Sakhalin"}, {"Samara", "Saa za Kiangazi za Samara"}, {"Samoa", "Saa za Majira ya joto za Samoa"}, {"Taipei", "Saa za Mchana za Taipei"}, {"Tonga", "Saa za Majira ya joto za Tonga"}, {"Turkmenistan", "Saa za Majira ya joto za Turkmenistan"}, {"Uruguay", "Saa za Majira ya joto za Urugwai"}, {"Uzbekistan", "Saa za Majira ya joto za Uzbekistan"}, {"Vanuatu", "Saa za Majira ya joto za Vanuatu"}, {"Vladivostok", "Saa za Majira ya joto za Vladivostok"}, {"Volgograd", "Saa za Majira ya joto za Volgograd"}, {"Yakutsk", "Saa za Majira ya joto za Yakutsk"}, {"Yekaterinburg", "Saa za Majira ya joto za Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "Saa za Afghanistani"}, {"Africa_Central", "Saa za Afrika ya Kati"}, {"Africa_Eastern", "Saa za Afrika Mashariki"}, {"Africa_Southern", "Saa za Wastani za Afrika Kusini"}, {"Africa_Western", "Saa za Afrika Magharibi"}, {"Alaska", "Saa za Wastani za Alaska"}, {"Amazon", "Saa za Wastani za Amazon"}, {"America_Central", "Saa za Wastani za Kati"}, {"America_Eastern", "Saa za Wastani za Mashariki"}, {"America_Mountain", "Saa za Wastani za Mountain"}, {"America_Pacific", "Saa za Wastani za Pasifiki"}, {"Anadyr", "Saa za Wastani za Anadyr"}, {"Apia", "Saa za Wastani za Apia"}, {"Arabian", "Saa za Wastani za Uarabuni"}, {"Argentina", "Saa za Wastani za Ajentina"}, {"Argentina_Western", "Saa za Wastani za Magharibi mwa Ajentina"}, {"Armenia", "Saa za Wastani za Armenia"}, {"Atlantic", "Saa za Wastani za Atlantiki"}, {"Australia_Central", "Saa za Wastani za Australia ya Kati"}, {"Australia_CentralWestern", "Saa za Wastani za Magharibi mwa Australia ya Kati"}, {"Australia_Eastern", "Saa za Wastani za Mashariki mwa Australia"}, {"Australia_Western", "Saa za Wastani za Australia Magharibi"}, {"Azerbaijan", "Saa za Wastani za Azabajani"}, {"Azores", "Saa za Wastani za Azores"}, {"Bangladesh", "Saa za Wastani za Bangladeshi"}, {"Bhutan", "Saa za Butani"}, {"Bolivia", "Saa za Bolivia"}, {"Brasilia", "Saa za Wastani za Brazili"}, {"Brunei", "Saa za Brunei Darussalam"}, {"Cape_Verde", "Saa za Wastani za Kepuvede"}, {"Chamorro", "Saa za Wastani za Chamorro"}, {"Chatham", "Saa za Wastani za Chatham"}, {"Chile", "Saa za Wastani za Chile"}, {"China", "Saa za Wastani za Uchina"}, {"Christmas", "Saa za Kisiwa cha Krismasi"}, {"Cocos", "Saa za Visiwa vya Cocos"}, {"Colombia", "Saa za Wastani za Kolombia"}, {"Cook", "Saa za Wastani za Visiwa vya Cook"}, {"Cuba", "Saa za Wastani za Kuba"}, {"Davis", "Saa za Davis"}, {"DumontDUrville", "Saa za Dumont-d’Urville"}, {"East_Timor", "Saa za Timor Mashariki"}, {"Easter", "Saa za Wastani za Kisiwa cha Easter"}, {"Ecuador", "Saa za Ekwado"}, {"Europe_Central", "Saa za Wastani za Ulaya ya Kati"}, {"Europe_Eastern", "Saa za Wastani za Mashariki mwa Ulaya"}, {"Europe_Further_Eastern", "Saa za Mashariki zaidi mwa Ulaya"}, {"Europe_Western", "Saa za Wastani za Magharibi mwa Ulaya"}, {"Falkland", "Saa za Wastani za Visiwa vya Falkland"}, {"Fiji", "Saa za Wastani za Fiji"}, {"French_Guiana", "Saa za Guiana"}, {"French_Southern", "Saa za Kusini mwa Ufaransa na Antaktiki"}, {"GMT", "Saa za Greenwich"}, {"Galapagos", "Saa za Galapagos"}, {"Gambier", "Saa za Gambier"}, {"Georgia", "Saa za Wastani za Jiojia"}, {"Gilbert_Islands", "Saa za Visiwa vya Gilbert"}, {"Greenland_Eastern", "Saa za Wastani za Greenland Mashariki"}, {"Greenland_Western", "Saa za Wastani za Greenland Magharibi"}, {"Gulf", "Saa za Wastani za Ghuba"}, {"Guyana", "Saa za Guyana"}, {"Hawaii", "Saa za Wastani za Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Saa za Wastani za Hawaii-Aleutian"}, {"Hong_Kong", "Saa za Wastani za Hong Kong"}, {"Hovd", "Saa za Wastani za Hovd"}, {"India", "Saa za Wastani za India"}, {"Indian_Ocean", "Saa za Bahari Hindi"}, {"Indochina", "Saa za Indochina"}, {"Indonesia_Central", "Saa za Indonesia ya Kati"}, {"Indonesia_Eastern", "Saa za Mashariki mwa Indonesia"}, {"Indonesia_Western", "Saa za Magharibi mwa Indonesia"}, {"Iran", "Saa za Wastani za Irani"}, {"Irkutsk", "Saa za Wastani za Irkutsk"}, {"Israel", "Saa za Wastani za Israeli"}, {"Japan", "Saa za Wastani za Japani"}, {"Kamchatka", "Saa za Wastani za Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Saa za Kazakhstan"}, {"Kazakhstan_Eastern", "Saa za Kazakistani Mashariki"}, {"Kazakhstan_Western", "Saa za Kazakistani Magharibi"}, {"Korea", "Saa za Wastani za Korea"}, {"Kosrae", "Saa za Kosrae"}, {"Krasnoyarsk", "Saa za Wastani za Krasnoyask"}, {"Kyrgystan", "Saa za Kyrgystan"}, {"Line_Islands", "Saa za Visiwa vya Line"}, {"Lord_Howe", "Saa za Wastani za Lord Howe"}, {"Magadan", "Saa za Wastani za Magadan"}, {"Malaysia", "Saa za Malesia"}, {"Maldives", "Saa za Maldivi"}, {"Marquesas", "Saa za Marquesas"}, {"Marshall_Islands", "Saa za Visiwa vya Marshall"}, {"Mauritius", "Saa za Wastani za Morisi"}, {"Mawson", "Saa za Mawson"}, {"Mexico_Pacific", "Saa za wastani za pasifiki za Meksiko"}, {"Mongolia", "Saa za Wastani za Ulaanbataar"}, {"Moscow", "Saa za Wastani za Moscow"}, {"Myanmar", "Saa za Myanma"}, {"Nauru", "Saa za Nauru"}, {"Nepal", "Saa za Nepali"}, {"New_Caledonia", "Saa za Wastani za Kaledonia Mpya"}, {"New_Zealand", "Saa za Wastani za Nyuzilandi"}, {"Newfoundland", "Saa za Wastani za Newfoundland"}, {"Niue", "Saa za Niue"}, {"Norfolk", "Saa za Wastani za Kisiwa cha Norfolk"}, {"Noronha", "Saa za Wastani za Fernando de Noronha"}, {"Novosibirsk", "Saa za Wastani za Novosibirsk"}, {"Omsk", "Saa za Wastani za Omsk"}, {"Pakistan", "Saa za Wastani za Pakistani"}, {"Palau", "Saa za Palau"}, {"Papua_New_Guinea", "Saa za Papua"}, {"Paraguay", "Saa za Wastani za Paragwai"}, {"Peru", "Saa za Wastani za Peru"}, {"Philippines", "Saa za Wastani za Ufilipino"}, {"Phoenix_Islands", "Saa za Visiwa vya Finiksi"}, {"Pierre_Miquelon", "Saa za Wastani ya Saint-Pierre na Miquelon"}, {"Pitcairn", "Saa za Pitcairn"}, {"Ponape", "Saa za Ponape"}, {"Pyongyang", "Saa za Pyongyang"}, {"Reunion", "Saa za Reunion"}, {"Rothera", "Saa za Rothera"}, {"Sakhalin", "Saa za Wastani za Sakhalin"}, {"Samara", "Saa za Wastani za Samara"}, {"Samoa", "Saa za Wastani za Samoa"}, {"Seychelles", "Saa za Ushelisheli"}, {"Singapore", "Saa za Wastani za Singapoo"}, {"Solomon", "Saa za Visiwa vya Solomon"}, {"South_Georgia", "Saa za Jojia Kusini"}, {"Suriname", "Saa za Suriname"}, {"Syowa", "Saa za Syowa"}, {"Tahiti", "Saa za Tahiti"}, {"Taipei", "Saa za Wastani za Taipei"}, {"Tajikistan", "Saaza Tajikistani"}, {"Tokelau", "Saa za Tokelau"}, {"Tonga", "Saa za Wastani za Tonga"}, {"Truk", "Saa za Chuuk"}, {"Turkmenistan", "Saa za Wastani za Turkmenistani"}, {"Tuvalu", "Saa za Tuvalu"}, {"Uruguay", "Saa za Wastani za Urugwai"}, {"Uzbekistan", "Saa za wastani za Uzbekistani"}, {"Vanuatu", "Saa za Wastani za Vanuatu"}, {"Venezuela", "Saa za Venezuela"}, {"Vladivostok", "Saa za Wastani za Vladivostok"}, {"Volgograd", "Saa za Wastani za Volgograd"}, {"Vostok", "Saa za Vostok"}, {"Wake", "Saa za Kisiwa cha Wake"}, {"Wallis", "Saa za Wallis na Futuna"}, {"Yakutsk", "Saa za Wastani za Yakutsk"}, {"Yekaterinburg", "Saa za Wastani za Yekaterinburg"}, {"Yukon", "Saa za Yukon"}},
		{{"Alaska", "Saa za Mchana za Alaska"}, {"Amazon", "Saa za Majira ya Joto za Amazon"}, {"America_Central", "Saa za Mchana za Kati"}, {"America_Eastern", "Saa za Mchana za Mashariki"}, {"America_Mountain", "Saa za Mchana za Mountain"}, {"America_Pacific", "Saa za Mchana za Pasifiki"}, {"Anadyr", "Saa za Kiangazi za Anadyr"}, {"Apia", "Saa za Mchana za Apia"}, {"Arabian", "Saa za Mchana za Arabiani"}, {"Argentina", "Saa za Majira Joto za Ajentina"}, {"Argentina_Western", "Saa za Majira ya Joto za Magharibi mwa Ajentina"}, {"Armenia", "Saa za Majira ya Joto za Armenia"}, {"Atlantic", "Saa za Mchana za Atlantiki"}, {"Australia_Central", "Saa za Mchana za Australia ya Kati"}, {"Australia_CentralWestern", "Saa za Mchana za Magharibi mwa Australia ya Kati"}, {"Australia_Eastern", "Saa za Mchana za Mashariki mwa Australia"}, {"Australia_Western", "Saa za Mchana za Australia Magharibi"}, {"Azerbaijan", "Saa za Majira ya Joto za Azabajani"}, {"Azores", "Saa za Majira ya Joto za Azores"}, {"Bangladesh", "Saa za Majira ya Joto za Bangladeshi"}, {"Brasilia", "Saa za Majira ya Joto za Brazili"}, {"Cape_Verde", "Saa za Majira ya Joto za Kepuvede"}, {"Chatham", "Saa za Mchana za Chatham"}, {"Chile", "Saa za Majira ya joto za Chile"}, {"China", "Saa za Mchana za Uchina"}, {"Colombia", "Saa za Majira ya Joto za Kolombia"}, {"Cook", "Saa za Majira Nusu ya Joto za Visiwa vya Cook"}, {"Cuba", "Saa za Mchana za Kuba"}, {"Easter", "Saa za Majira ya Joto za Kisiwa cha Easter"}, {"Europe_Central", "Saa za Majira ya Joto za Ulaya ya Kati"}, {"Europe_Eastern", "Saa za Majira ya Joto za Mashariki mwa Ulaya"}, {"Europe_Western", "Saa za Majira ya Joto za Magharibi mwa Ulaya"}, {"Falkland", "Saa za Majira ya joto za Visiwa vya Falkland"}, {"Fiji", "Saa za Majira ya joto za Fiji"}, {"Georgia", "Saa za Majira ya Joto za Jiojia"}, {"Greenland_Eastern", "Saa za Majira ya Joto za Greenland Mashariki"}, {"Greenland_Western", "Saa za Majira ya joto za Greenland Magharibi"}, {"Hawaii_Aleutian", "Saa za Mchana za Hawaii-Aleutian"}, {"Hong_Kong", "Saa za Majira ya Joto za Hong Kong"}, {"Hovd", "Saa za Majira ya Joto za Hovd"}, {"Iran", "Saa za Mchana za Irani"}, {"Irkutsk", "Saa za Majira ya Joto za Irkutsk"}, {"Israel", "Saa za Mchana za Israeli"}, {"Japan", "Saa za Mchana za Japani"}, {"Kamchatka", "Saa za Kiangazi za Petropavlovsk-Kamchatski"}, {"Korea", "Saa za Mchana za Korea"}, {"Krasnoyarsk", "Saa za Majira ya Joto za Krasnoyarsk"}, {"Lord_Howe", "Saa za Mchana za Lord Howe"}, {"Magadan", "Saa za Majira ya Joto za Magadan"}, {"Mauritius", "Saa za Majira ya Joto za Morisi"}, {"Mexico_Pacific", "Saa za mchana za pasifiki za Meksiko"}, {"Mongolia", "Saa za Majira ya Joto za Ulaanbaatar"}, {"Moscow", "Saa za Majira ya Joto za Moscow"}, {"New_Caledonia", "Saa za Majira ya Joto za Kaledonia Mpya"}, {"New_Zealand", "Saa za Mchana za Nyuzilandi"}, {"Newfoundland", "Saa za Mchana za Newfoundland"}, {"Norfolk", "Saa za Majira ya Joto za Kisiwa cha Norfolk"}, {"Noronha", "Saa za Majira ya joto za Fernando de Noronha"}, {"Novosibirsk", "Saa za Majira ya Joto za Novosibirsk"}, {"Omsk", "Saa za Majira ya Joto za Omsk"}, {"Pakistan", "Saa za Majira ya Joto za Pakistani"}, {"Paraguay", "Saa za Majira ya Joto za Paragwai"}, {"Peru", "Saa za Majira ya Joto za Peru"}, {"Philippines", "Saa za Majira ya Joto za Ufilipino"}, {"Pierre_Miquelon", "Saa za Mchana za Saint-Pierre na Miquelon"}, {"Sakhalin", "Saa za Majira ya Joto za Sakhalin"}, {"Samara", "Saa za Kiangazi za Samara"}, {"Samoa", "Saa za Mchana za Samoa"}, {"Taipei", "Saa za Mchana za Taipei"}, {"Tonga", "Saa za Majira ya Joto za Tonga"}, {"Turkmenistan", "Saa za Majira ya Joto za Turkmenistani"}, {"Uruguay", "Saa za Majira ya Joto za Urugwai"}, {"Uzbekistan", "Saa za Majira ya Joto za Uzbekistani"}, {"Vanuatu", "Saa za Majira ya Joto za Vanuatu"}, {"Vladivostok", "Saa za Majira ya Joto za Vladivostok"}, {"Volgograd", "Saa za Majira ya Joto za Volgograd"}, {"Yakutsk", "Saa za Majira ya Joto za Yakutsk"}, {"Yekaterinburg", "Saa za Majira ya Joto za Yekaterinburg"}},
	},
	{
		{{"Acre", "ܥܕܢܘܬܐ ܫܪܫܝܬܐ ܥܕܢܘܬܐ ܫܪܫܝܬܐ ܕܐܝܟܝܪ"}, {"Afghanistan", "ܥܕܢܐ ܕܐܦܓܐܢܣܬܐܢ"}, {"Africa_Central", "ܥܕܢܐ ܕܡܨܥܝܬܐ ܐܦܪܝܩܐ"}, {"Africa_Eastern", "ܥܕܢܐ ܕܡܕܢܚ ܐܦܪܝܩܐ"}, {"Africa_Southern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܝܡܢ ܐܦܪܝܩܐ"}, {"Africa_Western", "ܥܕܢܐ ܕܡܥܪܒ ܐܦܪܝܩܐ"}, {"Alaska", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܠܐܣܟܐ"}, {"Almaty", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܠܡܐܬܝ"}, {"Amazon", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܡܙܢ"}, {"America_Central", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܡܨܥܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Eastern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܡܕܢܚܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Mountain", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܛܘܪܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Pacific", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܫܝܢܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"Anadyr", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܢܐܕܝܪ"}, {"Apia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܦܝܐ"}, {"Aqtau", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܟܬܐܘ"}, {"Aqtobe", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܟܬܘܒ"}, {"Arabian", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܪܒܝܐ"}, {"Argentina", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܪܓܢܬܝܢܐ"}, {"Argentina_Western", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܪܓܢܬܝܢܐ ܡܥܪܒܝܬܐ"}, {"Armenia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܪܡܢܝܐ"}, {"Atlantic", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܐܛܠܢܛܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"Australia_Central", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܣܬܪܠܝܐ ܡܨܥܝܬܐ"}, {"Australia_CentralWestern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܡܥܪܒܝܬܐ ܡܨܥܝܬܐ ܕܐܘܣܬܪܠܝܐ"}, {"Australia_Eastern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܣܬܪܠܝܐ ܡܕܢܚܝܬܐ"}, {"Australia_Western", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܣܬܪܠܝܐ ܡܥܪܒܝܬܐ"}, {"Azerbaijan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܙܪܒܝܓܐܢ"}, {"Azores", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܙܘܪ"}, {"Bangladesh", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܒܢܓܠܐܕܝܫ"}, {"Bhutan", "ܥܕܢܐ ܕܒܘܬܐܢ"}, {"Bolivia", "ܥܕܢܐ ܕܒܘܠܝܒܝܐ"}, {"Brasilia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܒܪܐܣܝܠܝܐ"}, {"Brunei", "ܥܕܢܐ ܕܒܪܘܢܐܝ ܕܐܪܘܣܐܠܡ"}, {"Cape_Verde", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܟܐܦ ܒܝܪܕܝ"}, {"Casey", "ܥܕܢܐ ܕܟܐܝܣܝ"}, {"Chamorro", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܫܐܡܘܪܘ"}, {"Chatham", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܫܐܬܡ"}, {"Chile", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܫܝܠܝ"}, {"China", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܨܝܢ"}, {"Christmas", "ܥܕܢܐ ܕܓܙܪܬܐ ܕܟܪܝܣܬܡܣ"}, {"Cocos", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܟܘܟܘܣ"}, {"Colombia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܟܘܠܘܡܒܝܐ"}, {"Cook", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܓܙܝܪ̈ܐ ܕܟܘܟ"}, {"Cuba", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܩܘܒܐ"}, {"Davis", "ܥܕܢܐ ܕܕܒܝܣ"}, {"DumontDUrville", "ܥܕܢܐ ܕܕܘܡܘܢܬ ܕܐܘܪܒܝܠ"}, {"East_Timor", "ܥܕܢܐ ܕܡܕܢܚ ܬܝܡܘܪ"}, {"Easter", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܙܪܬܐ ܦܨܚܐ"}, {"Ecuador", "ܥܕܢܐ ܕܐܩܘܐܕܘܪ"}, {"Europe_Central", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܪܘܦܐ ܡܨܥܝܬܐ"}, {"Europe_Eastern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܪܘܦܐ ܡܕܢܚܝܬܐ"}, {"Europe_Further_Eastern", "ܥܕܢܐ ܕܐܘܪܘܦܐ (ܗܡ ܡܕܢܚܐ)"}, {"Europe_Western", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܪܘܦܐ ܡܥܪܒܝܬܐ"}, {"Falkland", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܙܝܪ̈ܐ ܕܦܠܟܠܢܕ"}, {"Fiji", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܝܓܝ"}, {"French_Guiana", "ܥܕܢܐ ܕܓܘܝܐܢܐ ܦܪܢܣܝܬܐ"}, {"French_Southern", "ܥܕܢܐ ܕܦܪܢܣܐ ܬܝܡܢܝܬܐ ܘܐܢܬܪܬܝܟܐ"}, {"GMT", "ܥܕܢܐ ܕܓܪܝܢܟ"}, {"Galapagos", "ܥܕܢܐ ܕܓܐܠܦܐܓܘܣ"}, {"Gambier", "ܥܕܢܐ ܕܓܡܒܝܪ"}, {"Georgia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܘܪܓܝܐ"}, {"Gilbert_Islands", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܓܝܠܒܪܬ"}, {"Greenland_Eastern", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܓܪܝܢܠܢܕ ܡܕܢܚܝܬܐ"}, {"Greenland_Western", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܪܝܢܠܢܕ ܕܡܥܪܒܝܬܐ"}, {"Guam", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܘܐܡ"}, {"Gulf", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܥܠܢܐ"}, {"Guyana", "ܥܕܢܐ ܕܓܘܝܐܢܐ"}, {"Hawaii", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܗܐܘܐܝܝ ܐܠܘܫܝܢ"}, {"Hawaii_Aleutian", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܗܐܘܐܝܝ ܐܠܘܫܝܢ"}, {"Hong_Kong", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܗܘܢܓ ܟܘܢܓ"}, {"Hovd", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܗܘܒܕ"}, {"India", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܗܢܕܘ"}, {"Indian_Ocean", "ܥܕܢܐ ܕܐܘܩܝܢܘܣ ܗܢܕܘܝܐ"}, {"Indochina", "ܥܕܢܐ ܕܗܢܕܘܨܝܢ"}, {"Indonesia_Central", "ܥܕܢܐ ܕܗܢܕܘܨܝܢ ܡܨܥܝܬܐ"}, {"Indonesia_Eastern", "ܥܕܢܐ ܕܗܢܕܘܨܝܢ ܡܕܢܚܝܬܐ"}, {"Indonesia_Western", "ܥܕܢܐ ܕܗܢܕܘܨܝܢ ܡܥܪܒܝܬܐ"}, {"Iran", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܝܪܐܢ"}, {"Irkutsk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܝܪܟܘܬܣܟ"}, {"Israel", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܝܣܪܐܝܠ"}, {"Japan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܝܦܢ"}, {"Kamchatka", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܝܬܪܘܦܒܠܒܣܟܝ-ܟܐܡܟܬܣܟܝ"}, {"Kazakhstan", "ܥܕܢܐ ܕܟܙܩܣܬܐܢ"}, {"Kazakhstan_Eastern", "ܥܕܢܐ ܕܡܕܢܚ ܟܙܩܣܬܐܢ"}, {"Kazakhstan_Western", "ܥܕܢܐ ܕܡܥܪܒ ܟܙܩܣܬܐܢ"}, {"Korea", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܟܘܪܝܝܐ"}, {"Kosrae", "ܥܕܢܐ ܟܘܣܪܐܝ"}, {"Krasnoyarsk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܟܪܐܣܢܘܝܪܣܟ"}, {"Kyrgystan", "ܥܕܢܐ ܕܩܝܪܓܝܙܣܬܐܢ"}, {"Lanka", "ܥܕܢܐ ܕܠܐܢܟܐ"}, {"Line_Islands", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܠܐܝܢ"}, {"Lord_Howe", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܠܘܪܕ ܗܐܘ"}, {"Macau", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܐܟܐܘ"}, {"Magadan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܐܓܕܐܢ"}, {"Malaysia", "ܥܕܢܐ ܕܡܠܝܙܝܐ"}, {"Maldives", "ܥܕܢܐ ܕܓܙܪܬܐ ܡܐܠܕܝܒܝܬܐ"}, {"Marquesas", "ܥܕܢܐ ܕܡܐܪܟܐܘܣܐܣ"}, {"Marshall_Islands", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܡܐܪܫܐܠ"}, {"Mauritius", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܘܪܝܛܝܘܣ"}, {"Mawson", "ܥܕܢܐ ܕܡܐܘܣܘܢ"}, {"Mexico_Pacific", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܟܣܝܩܘ ܫܝܢܝܬܐ"}, {"Mongolia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܠܐܢܒܐܬܘܪ"}, {"Moscow", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܡܘܣܟܘ"}, {"Myanmar", "ܥܕܢܐ ܕܡܝܐܢܡܐܪ"}, {"Nauru", "ܥܕܢܐ ܕܢܐܘܪܘ"}, {"Nepal", "ܥܕܢܐ ܕܢܝܦܐܠ"}, {"New_Caledonia", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܢܝܘ ܟܠܝܕܘܢܝܐ"}, {"New_Zealand", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܢܝܘ ܙܝܠܢܕ"}, {"Newfoundland", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܢܝܘܦܐܘܢܠܢܕ"}, {"Niue", "ܥܕܢܐ ܕܢܝܘܝ"}, {"Norfolk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܓܙܪܬܐ ܕܢܘܪܦܠܟ"}, {"Noronha", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܪܢܢܕܘ ܕܢܘܪܘܢܗܐ"}, {"North_Mariana", "ܥܕܢܐ ܕܓܙܝܖ̈ܐ ܕܡܪܝܐܢܐ ܓܪܒܝܝܬܐ"}, {"Novosibirsk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܢܘܒܘܣܝܒܪܣܟ"}, {"Omsk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܘܡܣܟ"}, {"Pakistan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܐܟܣܬܐܢ"}, {"Palau", "ܥܕܢܐ ܕܦܠܐܘ"}, {"Papua_New_Guinea", "ܥܕܢܐ ܕܦܐܦܘܐ ܓܝܢܝܐ ܚܕܬܐ"}, {"Paraguay", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܪܓܘܐܝ"}, {"Peru", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܝܪܘ"}, {"Philippines", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܦܝܠܝܦܝܢܝܐ"}, {"Phoenix_Islands", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܦܝܢܝܟܣ"}, {"Pierre_Miquelon", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܣܐܢܬ ܦܝܥܪ ܘܡܩܘܠܘܢ"}, {"Pitcairn", "ܥܕܢܐ ܕܦܝܬܟܐܝܪܢ"}, {"Ponape", "ܥܕܢܐ ܕܦܘܢܐܦܝ"}, {"Pyongyang", "ܥܕܢܐ ܕܦܝܘܢܓܝܢܓ"}, {"Qyzylorda", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܟܝܙܝܠܘܪܕܐ"}, {"Reunion", "ܥܕܢܐ ܕܪܝܘܢܝܘܢ"}, {"Rothera", "ܥܕܢܐ ܕܪܘܬܝܪܐ"}, {"Sakhalin", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܣܐܚܐܠܝܢ"}, {"Samara", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܣܡܐܪܐ"}, {"Samoa", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܣܡܘܐ"}, {"Seychelles", "ܥܕܢܐ ܕܣܐܝܫܝܠ"}, {"Singapore", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܣܝܢܓܐܦܘܪ"}, {"Solomon", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܫܠܝܡܘܢ"}, {"South_Georgia", "ܥܕܢܐ ܕܓܝܘܪܓܝܐ ܬܝܡܢܝܬܐ"}, {"Suriname", "ܥܕܢܐ ܕܣܘܪܝܢܐܡ"}, {"Syowa", "ܥܕܢܐ ܕܣܝܘܐ"}, {"Tahiti", "ܥܕܢܐ ܕܬܗܝܬܝ"}, {"Taipei", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܐܝܦܐܝ"}, {"Tajikistan", "ܥܕܢܐ ܕܬܐܓܝܟܣܬܐܢ"}, {"Tokelau", "ܥܕܢܐ ܕܬܘܟܝܠܐܘ"}, {"Tonga", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܬܘܢܓܐ"}, {"Truk", "ܥܕܢܐ ܕܬܫܘܟ"}, {"Turkmenistan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܬܘܪܟܡܢܣܬܐܢ"}, {"Tuvalu", "ܥܕܢܐ ܕܬܘܒܐܠܘ"}, {"Uruguay", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܪܘܓܘܐܝ"}, {"Uzbekistan", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܐܘܙܒܟܣܬܐܢ"}, {"Vanuatu", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܒܐܢܘܐܛܘ"}, {"Venezuela", "ܥܕܢܐ ܕܒܢܙܘܝܠܐ"}, {"Vladivostok", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܒܠܐܕܝܒܘܣܬܘܟ"}, {"Volgograd", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܒܘܠܓܘܓܪܐܕ"}, {"Vostok", "ܥܕܢܐ ܕܒܘܣܬܘܟ"}, {"Wake", "ܥܕܢܐ ܕܓܙܝܪ̈ܐ ܕܘܐܝܟ"}, {"Wallis", "ܥܕܢܐ ܕܘܝܠܝܣ ܘܦܘܬܘܢܐ"}, {"Yakutsk", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܝܐܟܘܬܣܟ"}, {"Yekaterinburg", "ܥܕܢܐ ܡܫܘܚܬܢܝܬܐ ܕܝܟܐܬܝܪܢܒܝܪܓ"}, {"Yukon", "ܥܕܢܐ ܕܝܘܩܘܢ"}},
		{{"Acre", "ܥܕܢܘܬܐ ܩܝܬܝܬܐ ܕܐܝܟܝܪ"}, {"Alaska", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܠܐܣܟܐ"}, {"Almaty", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܠܡܐܬܝ"}, {"Amazon", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܡܙܢ"}, {"America_Central", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܡܨܥܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Eastern", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܡܕܢܚܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Mountain", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܛܘܪܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"America_Pacific", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܫܝܢܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"Anadyr", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܢܐܕܝܪ"}, {"Apia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܦܝܐ"}, {"Aqtau", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܟܬܐܘ"}, {"Aqtobe", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܟܬܘܒ"}, {"Arabian", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܪܒܝܐ"}, {"Argentina", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܪܓܢܬܝܢܐ"}, {"Argentina_Western", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܪܓܢܬܝܢܐ ܡܥܪܒܝܬܐ"}, {"Armenia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܪܡܢܝܐ"}, {"Atlantic", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܐܛܠܢܛܝܬܐ ܕܐܡܪܝܟܐ ܓܪܒܝܝܬܐ"}, {"Australia_Central", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܣܬܪܠܝܐ ܡܨܥܝܬܐ"}, {"Australia_CentralWestern", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܡܥܪܒܝܬܐ ܡܨܥܝܬܐ ܕܐܘܣܬܪܠܝܐ"}, {"Australia_Eastern", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܣܬܪܠܝܐ ܡܕܢܚܝܬܐ"}, {"Australia_Western", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܣܬܪܠܝܐ ܡܥܪܒܝܬܐ"}, {"Azerbaijan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܙܪܒܝܓܐܢ"}, {"Azores", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܙܘܪ"}, {"Bangladesh", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܒܢܓܠܐܕܝܫ"}, {"Brasilia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܒܪܐܣܝܠܝܐ"}, {"Cape_Verde", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܟܐܦ ܒܝܪܕܝ"}, {"Chatham", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܬܫܐܬܡ"}, {"Chile", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܬܫܝܠܝ"}, {"China", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܨܝܢ"}, {"Colombia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܟܘܠܘܡܒܝܐ"}, {"Cook", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܓܙܝܪ̈ܐ ܕܟܘܟ"}, {"Cuba", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܩܘܒܐ"}, {"Easter", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܓܙܪܬܐ ܦܨܚܐ"}, {"Europe_Central", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܪܘܦܐ ܡܨܥܝܬܐ"}, {"Europe_Eastern", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܪܘܦܐ ܡܕܢܚܝܬܐ"}, {"Europe_Western", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܪܘܦܐ ܡܥܪܒܝܬܐ"}, {"Falkland", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܓܙܝܪ̈ܐ ܕܦܠܟܠܢܕ"}, {"Fiji", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܝܓܝ"}, {"Georgia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܓܘܪܓܝܐ"}, {"Greenland_Eastern", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܓܪܝܢܠܢܕ ܡܕܢܚܝܬܐ"}, {"Greenland_Western", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܓܪܝܢܠܢܕ ܕܡܥܪܒܝܬܐ"}, {"Hawaii_Aleutian", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܗܐܘܐܝܝ ܐܠܘܫܝܢ"}, {"Hong_Kong", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܗܘܢܓ ܟܘܢܓ"}, {"Hovd", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܗܘܒܕ"}, {"Iran", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܝܪܐܢ"}, {"Irkutsk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܝܪܟܘܬܣܟ"}, {"Israel", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܝܣܪܐܝܠ"}, {"Japan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܝܦܢ"}, {"Kamchatka", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܝܬܪܘܦܒܠܒܣܟܝ-ܟܐܡܟܬܣܟܝ"}, {"Korea", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܟܘܪܝܝܐ"}, {"Krasnoyarsk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܟܪܐܣܢܘܝܪܣܟ"}, {"Lord_Howe", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܠܘܪܕ ܗܐܘ"}, {"Macau", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܡܐܟܐܘ"}, {"Magadan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܡܐܓܕܐܢ"}, {"Mauritius", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܡܘܪܝܛܝܘܣ"}, {"Mexico_Pacific", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܡܟܣܝܩܘ ܫܝܢܝܬܐ"}, {"Mongolia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܠܐܢܒܐܬܘܪ"}, {"Moscow", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܡܘܣܟܘ"}, {"New_Caledonia", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܢܝܘ ܟܠܝܕܘܢܝܐ"}, {"New_Zealand", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܢܝܘ ܙܝܠܢܕ"}, {"Newfoundland", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܢܝܘܦܐܘܢܠܢܕ"}, {"Norfolk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܓܙܪܬܐ ܕܢܘܪܦܠܟ"}, {"Noronha", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܪܢܢܕܘ ܕܢܘܪܘܢܗܐ"}, {"Novosibirsk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܢܘܒܘܣܝܒܪܣܟ"}, {"Omsk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܘܡܣܟ"}, {"Pakistan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܐܟܣܬܐܢ"}, {"Paraguay", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܪܓܘܐܝ"}, {"Peru", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܝܪܘ"}, {"Philippines", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܦܝܠܝܦܝܢܝܐ"}, {"Pierre_Miquelon", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܣܐܢܬ ܦܝܥܪ ܘܡܩܘܠܘܢ"}, {"Qyzylorda", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܟܝܙܝܠܘܪܕܐ"}, {"Sakhalin", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܣܐܚܐܠܝܢ"}, {"Samara", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܣܡܐܪܐ"}, {"Samoa", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܣܡܘܐ"}, {"Taipei", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܬܐܝܦܐܝ"}, {"Tonga", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܬܘܢܓܐ"}, {"Turkmenistan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܬܘܪܟܡܢܣܬܐܢ"}, {"Uruguay", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܪܘܓܘܐܝ"}, {"Uzbekistan", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܐܘܙܒܟܣܬܐܢ"}, {"Vanuatu", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܒܐܢܘܐܛܘ"}, {"Vladivostok", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܒܠܐܕܝܒܘܣܬܘܟ"}, {"Volgograd", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܒܘܠܓܘܓܪܐܕ"}, {"Yakutsk", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܝܐܟܘܬܣܟ"}, {"Yekaterinburg", "ܥܕܢܐ ܕܒܗܪ ܝܘܡܐ ܕܝܟܐܬܝܪܢܒܝܪܓ"}},
	},
	{
		{{"Afghanistan", "Afganistan"}, {"Africa_Central", "postrzodkowoafrykański czas"}, {"Africa_Eastern", "wschodnioafrykański czas"}, {"Africa_Southern", "połedniowoafrykański czas"}, {"Africa_Western", "zachodnioafrykański czas"}, {"Alaska", "alaskański czas sztandardowy"}, {"Almaty", "czas ałmacki sztandardowy"}, {"Amazon", "amazōński sztandardowy czas"}, {"America_Central", "czas postrzodkowoamerykański sztandardowy"}, {"America_Eastern", "czas wschodnioamerykański sztandardowy"}, {"America_Mountain", "czas gōrski sztandardowy"}, {"America_Pacific", "czas pacyficzny sztandardowy"}, {"Anadyr", "sztandardowy czas Anadyr"}, {"Aqtau", "czas auktaucki sztandardowy"}, {"Aqtobe", "czas aktiubiński sztandardowy"}, {"Arabian", "Pōłwysep Arabski (sztandardowy czas)"}, {"Argentina", "Argyntyna (sztandardowy czas)"}, {"Argentina_Western", "Argyntyna Zachodnio (sztandardowy czas)"}, {"Armenia", "Armynijo (sztandardowy czas)"}, {"Atlantic", "czas atlantycki sztandardowy"}, {"Australia_Central", "postrzodkowoaustralijski sztandardowy czas"}, {"Australia_CentralWestern", "postrzodkowo-zachodnioaustralijski sztandardowy czas"}, {"Australia_Eastern", "wschodnioaustralijski sztandardowy czas"}, {"Australia_Western", "zachodnioaustralijski sztandardowy czas"}, {"Azerbaijan", "Azerbejdżan (sztandardowy czas)"}, {"Azores", "Azory (sztandardowy czas)"}, {"Bangladesh", "Bangladesz (sztandardowy czas)"}, {"Bolivia", "Boliwijo"}, {"Brasilia", "Brasília (sztandardowy czas)"}, {"Cape_Verde", "Wyspy Zielōnego Przilōndka (sztandardowy czas)"}, {"Chamorro", "Czamorro"}, {"Chile", "Chile (sztandardowy czas)"}, {"China", "Chiny (sztandardowy czas)"}, {"Christmas", "Godnio Wyspa"}, {"Cocos", "Wyspy Kokosowe"}, {"Colombia", "Kolumbijo (sztandardowy czas)"}, {"Cook", "Wyspy Cooka (sztandardowy czas)"}, {"Cuba", "Kuba (sztandardowy czas)"}, {"DumontDUrville", "Dumont-d’Urville"}, {"East_Timor", "Timor Wschodni"}, {"Easter", "Wyspa Wielkanocno (sztandardowy czas)"}, {"Ecuador", "Ekwador"}, {"Europe_Central", "postrzodkowoeuropejski sztandardowy czas"}, {"Europe_Eastern", "wschodnioeuropejski sztandardowy czas"}, {"Europe_Further_Eastern", "wschodnioeuropejski dalszy czas"}, {"Europe_Western", "zachodnioeuropejski sztandardowy czas"}, {"Falkland", "Falklandy (sztandardowy czas)"}, {"Fiji", "Fidżi (sztandardowy czas)"}, {"French_Guiana", "Gujana Francusko"}, {"French_Southern", "Francuske Terytoria Połedniowe i Antarktyczne"}, {"GMT", "uniwersalnego czasu"}, {"Galapagos", "czas Galapagos"}, {"Gambier", "Wyspy Gambiera"}, {"Georgia", "Gruzyjo (sztandardowy czas)"}, {"Gilbert_Islands", "Gilbertowe Wyspy"}, {"Greenland_Eastern", "Grynlandyjo Wschodnia (sztandardowy czas)"}, {"Greenland_Western", "Grynlandyjo Zachodnio (sztandardowy czas)"}, {"Gulf", "Zatoka Perska"}, {"Guyana", "Gujana"}, {"Hawaii", "Hawaje-Aleuty (sztandardowy czas)"}, {"Hawaii_Aleutian", "Hawaje-Aleuty (sztandardowy czas)"}, {"Hong_Kong", "Hōngkōng (sztandardowy czas)"}, {"Hovd", "Kobdo (sztandardowy czas)"}, {"India", "indyjski sztandardowy czas"}, {"Indian_Ocean", "Ôcean Indyjski"}, {"Indochina", "indochiński czas"}, {"Indonesia_Central", "Indōnezyjo Postrzodkowo"}, {"Indonesia_Eastern", "Indōnezyjo Wschodnio"}, {"Indonesia_Western", "Indōnezyjo Zachodnio"}, {"Irkutsk", "Irkuck (sztandardowy czas)"}, {"Israel", "Izrael (sztandardowy czas)"}, {"Japan", "Japōnijo (sztandardowy czas)"}, {"Kamchatka", "sztandardowy czas Pietropawłowsk Kamczacki"}, {"Kazakhstan_Eastern", "Kazachstan Wschodni"}, {"Kazakhstan_Western", "Kazachstan Zachodni"}, {"Krasnoyarsk", "Krasnojarsk (sztandardowy czas)"}, {"Kyrgystan", "Kirgistan"}, {"Line_Islands", "Line Islands"}, {"Lord_Howe", "Lord Howe (sztandardowy czas)"}, {"Malaysia", "Malezyjo"}, {"Maldives", "Malediwy"}, {"Marquesas", "Markizy"}, {"Marshall_Islands", "Wyspy Marshalla"}, {"Mexico_Pacific", "Meksyk (czas pacyficzny sztandardowy)"}, {"Mongolia", "Ułan Bator (sztandardowy czas)"}, {"Moscow", "Moskwa (sztandardowy)"}, {"Myanmar", "Mjanma"}, {"New_Caledonia", "Nowo Kaledōnijo (sztandardowy czas)"}, {"New_Zealand", "Nowo Zelandyjo (sztandardowy czas)"}, {"Newfoundland", "Nowo Fundlandyjo (sztandardowy czas)"}, {"Noronha", "Fernando de Noronha (sztandardowy czas)"}, {"Novosibirsk", "Nowosybirsk (sztandardowy czas)"}, {"Omsk", "Ômsk (sztandardowy czas)"}, {"Papua_New_Guinea", "Papua-Nowo Gwinea"}, {"Paraguay", "Paragwaj (sztandardowy czas)"}, {"Peru", "Peru (sztandardowy czas)"}, {"Philippines", "Filipiny (sztandardowy czas)"}, {"Phoenix_Islands", "Fyniks"}, {"Pierre_Miquelon", "Saint-Pierre i Miquelon (sztandardowy czas)"}, {"Ponape", "Pohnpei"}, {"Pyongyang", "Pjōngjang"}, {"Qyzylorda", "czas kyzyłordzki sztandardowy"}, {"Sakhalin", "Sachalin (sztandardowy czas)"}, {"Samara", "sztandardowy czas Samara"}, {"Seychelles", "Seszele"}, {"Singapore", "Singapur"}, {"Solomon", "Wyspy Salōmōna"}, {"South_Georgia", "Georgia Połedniowo"}, {"Suriname", "Surinam"}, {"Taipei", "Tajpej (sztandardowy czas)"}, {"Tajikistan", "Tadżykistan"}, {"Tonga", "Tōnga (sztandardowy czas)"}, {"Truk", "Chuuk"}, {"Turkmenistan", "Turkmynistan (sztandardowy czas)"}, {"Uruguay", "Urugwaj (sztandardowy czas)"}, {"Venezuela", "Wynezuela"}, {"Vladivostok", "Władywostok (sztandardowy czas)"}, {"Volgograd", "Wołgograd (sztandardowy czas)"}, {"Vostok", "Wostok"}, {"Wallis", "Wallis i Futuna"}, {"Yakutsk", "Jakuck (sztandardowy czas)"}, {"Yekaterinburg", "Jekaterynburg (sztandardowy czas)"}, {"Yukon", "czas jukōński"}},
		{{"Alaska", "alaskański czas latowy"}, {"Almaty", "czas ałmacki latowy"}, {"Amazon", "amazōński latowy czas"}, {"America_Central", "czas postrzodkowoamerykański latowy"}, {"America_Eastern", "czas wschodnioamerykański latowy"}, {"America_Mountain", "czas gōrski latowy"}, {"America_Pacific", "czas pacyficzny latowy"}, {"Anadyr", "latowy czas Anadyr"}, {"Aqtau", "czas auktaucki latowy"}, {"Aqtobe", "czas aktiubiński latowy"}, {"Arabian", "Pōłwysep Arabski (latowy czas)"}, {"Argentina", "Argyntyna (latowy czas)"}, {"Argentina_Western", "Argyntyna Zachodnio (latowy czas)"}, {"Armenia", "Armynijo (latowy czas)"}, {"Atlantic", "czas atlantycki latowy"}, {"Australia_Central", "postrzodkowoaustralijski latowy czas"}, {"Australia_CentralWestern", "postrzodkowo-zachodnioaustralijski latowy czas"}, {"Australia_Eastern", "wschodnioaustralijski latowy czas"}, {"Australia_Western", "zachodnioaustralijski latowy czas"}, {"Azerbaijan", "Azerbejdżan (latowy czas)"}, {"Azores", "Azory (latowy czas)"}, {"Bangladesh", "Bangladesz (latowy czas)"}, {"Brasilia", "Brasília (latowy czas)"}, {"Cape_Verde", "Wyspy Zielōnego Przilōndka (latowy czas)"}, {"Chile", "Chile (latowy czas)"}, {"China", "Chiny (latowy czas)"}, {"Colombia", "Kolumbijo (latowy czas)"}, {"Cook", "Wyspy Cooka (latowy czas)"}, {"Cuba", "Kuba (latowy czas)"}, {"Easter", "Wyspa Wielkanocno (latowy czas)"}, {"Europe_Central", "postrzodkowoeuropejski latowy czas"}, {"Europe_Eastern", "wschodnioeuropejski latowy czas"}, {"Europe_Western", "zachodnioeuropejski latowy czas"}, {"Falkland", "Falklandy (latowy czas)"}, {"Fiji", "Fidżi (latowy czas)"}, {"Georgia", "Gruzyjo (latowy czas)"}, {"Greenland_Eastern", "Grynlandyjo Wschodnia (latowy czas)"}, {"Greenland_Western", "Grynlandyjo Zachodnio (latowy czas)"}, {"Hawaii_Aleutian", "Hawaje-Aleuty (latowy czas)"}, {"Hong_Kong", "Hōngkōng (latowy czas)"}, {"Hovd", "Kobdo (latowy czas)"}, {"Irkutsk", "Irkuck (latowy czas)"}, {"Israel", "Izrael (latowy czas)"}, {"Japan", "Japōnijo (latowy czas)"}, {"Kamchatka", "czas Pietropawłowsk Kamczacki latowy"}, {"Krasnoyarsk", "Krasnojarsk (latowy czas)"}, {"Lord_Howe", "Lord Howe (latowy czas)"}, {"Mexico_Pacific", "Meksyk (czas pacyficzny latowy)"}, {"Mongolia", "Ułan Bator (latowy czas)"}, {"Moscow", "Moskwa (latowy)"}, {"New_Caledonia", "Nowo Kaledōnijo (latowy czas)"}, {"New_Zealand", "Nowo Zelandyjo (latowy czas)"}, {"Newfoundland", "Nowo Fundlandyjo (latowy czas)"}, {"Noronha", "Fernando de Noronha (latowy czas)"}, {"Novosibirsk", "Nowosybirsk (latowy czas)"}, {"Omsk", "Ômsk (latowy czas)"}, {"Paraguay", "Paragwaj (latowy czas)"}, {"Peru", "Peru (latowy czas)"}, {"Philippines", "Filipiny (latowy czas)"}, {"Pierre_Miquelon", "Saint-Pierre i Miquelon (latowy czas)"}, {"Qyzylorda", "czas kyzyłordzki latowy"}, {"Sakhalin", "Sachalin (latowy czas)"}, {"Samara", "czas Samara latowy"}, {"Taipei", "Tajpej (latowy czas)"}, {"Tonga", "Tōnga (latowy czas)"}, {"Turkmenistan", "Turkmynistan (latowy czas)"}, {"Uruguay", "Urugwaj (latowy czas)"}, {"Vladivostok", "Władywostok (latowy czas)"}, {"Volgograd", "Wołgograd (latowy czas)"}, {"Yakutsk", "Jakuck (latowy czas)"}, {"Yekaterinburg", "Jekaterynburg (latowy czas)"}},
	},
	{
		{{"Acre", "அக்ரே தர நேரம்"}, {"Afghanistan", "ஆஃப்கானிஸ்தான் நேரம்"}, {"Africa_Central", "மத்திய ஆப்பிரிக்க நேரம்"}, {"Africa_Eastern", "கிழக்கு ஆப்பிரிக்க நேரம்"}, {"Africa_Southern", "தென் ஆப்பிரிக்க நிலையான நேரம்"}, {"Africa_Western", "மேற்கு ஆப்பிரிக்க நேரம்"}, {"Alaska", "அலாஸ்கா நிலையான நேரம்"}, {"Almaty", "அல்மாடி தர நேரம்"}, {"Amazon", "அமேசான் நிலையான நேரம்"}, {"America_Central", "மத்திய நிலையான நேரம்"}, {"America_Eastern", "கிழக்கத்திய நிலையான நேரம்"}, {"America_Mountain", "மவுன்டைன் நிலையான நேரம்"}, {"America_Pacific", "பசிபிக் நிலையான நேரம்"}, {"Anadyr", "அனாடையர் தர நேரம்"}, {"Apia", "ஏபியா நிலையான நேரம்"}, {"Aqtau", "அட்டௌ தர நேரம்"}, {"Aqtobe", "அட்டோபே தர நேரம்"}, {"Arabian", "அரேபிய நிலையான நேரம்"}, {"Argentina", "அர்ஜென்டினா நிலையான நேரம்"}, {"Argentina_Western", "மேற்கத்திய அர்ஜென்டினா நிலையான நேரம்"}, {"Armenia", "ஆர்மேனிய நிலையான நேரம்"}, {"Atlantic", "அட்லாண்டிக் நிலையான நேரம்"}, {"Australia_Central", "ஆஸ்திரேலியன் மத்திய நிலையான நேரம்"}, {"Australia_CentralWestern", "ஆஸ்திரேலியன் மத்திய மேற்கத்திய நிலையான நேரம்"}, {"Australia_Eastern", "ஆஸ்திரேலியன் கிழக்கத்திய நிலையான நேரம்"}, {"Australia_Western", "ஆஸ்திரேலியன் மேற்கத்திய நிலையான நேரம்"}, {"Azerbaijan", "அசர்பைஜான் நிலையான நேரம்"}, {"Azores", "அசோரஸ் நிலையான நேரம்"}, {"Bangladesh", "வங்கதேச நிலையான நேரம்"}, {"Bhutan", "பூடான் நேரம்"}, {"Bolivia", "பொலிவியா நேரம்"}, {"Brasilia", "பிரேசிலியா நிலையான நேரம்"}, {"Brunei", "புருனே டருஸ்ஸலாம் நேரம்"}, {"Cape_Verde", "கேப் வெர்டே நிலையான நேரம்"}, {"Chamorro", "சாமோரோ நிலையான நேரம்"}, {"Chatham", "சத்தாம் நிலையான நேரம்"}, {"Chile", "சிலி நிலையான நேரம்"}, {"China", "சீன நிலையான நேரம்"}, {"Christmas", "கிறிஸ்துமஸ் தீவு நேரம்"}, {"Cocos", "கோகோஸ் தீவுகள் நேரம்"}, {"Colombia", "கொலம்பியா நிலையான நேரம்"}, {"Cook", "குக் தீவுகள் நிலையான நேரம்"}, {"Cuba", "கியூபா நிலையான நேரம்"}, {"Davis", "டேவிஸ் நேரம்"}, {"DumontDUrville", "டுமோண்ட்-டி உர்வில்லே நேரம்"}, {"East_Timor", "கிழக்கு திமோர் நேரம்"}, {"Easter", "ஈஸ்டர் தீவு நிலையான நேரம்"}, {"Ecuador", "ஈக்வடார் நேரம்"}, {"Europe_Central", "மத்திய ஐரோப்பிய நிலையான நேரம்"}, {"Europe_Eastern", "கிழக்கத்திய ஐரோப்பிய நிலையான நேரம்"}, {"Europe_Further_Eastern", "தூர-கிழக்கு ஐரோப்பிய நேரம்"}, {"Europe_Western", "மேற்கத்திய ஐரோப்பிய நிலையான நேரம்"}, {"Falkland", "ஃபாக்லாந்து தீவுகள் நிலையான நேரம்"}, {"Fiji", "ஃபிஜி நிலையான நேரம்"}, {"French_Guiana", "ஃபிரஞ்சு கயானா நேரம்"}, {"French_Southern", "பிரெஞ்சு தெற்கத்திய & அண்டார்டிக் நேரம்"}, {"GMT", "கிரீன்விச் சராசரி நேரம்"}, {"Galapagos", "கலபகோஸ் நேரம்"}, {"Gambier", "கேம்பியர் நேரம்"}, {"Georgia", "ஜார்ஜியா நிலையான நேரம்"}, {"Gilbert_Islands", "கில்பர்ட் தீவுகள் நேரம்"}, {"Greenland_Eastern", "கிழக்கு கிரீன்லாந்து நிலையான நேரம்"}, {"Greenland_Western", "மேற்கு கிரீன்லாந்து நிலையான நேரம்"}, {"Guam", "கம் தர நேரம்"}, {"Gulf", "வளைகுடா நிலையான நேரம்"}, {"Guyana", "கயானா நேரம்"}, {"Hawaii", "ஹவாய்-அலேஷியன் நிலையான நேரம்"}, {"Hawaii_Aleutian", "ஹவாய்-அலேஷியன் நிலையான நேரம்"}, {"Hong_Kong", "ஹாங்காங் நிலையான நேரம்"}, {"Hovd", "ஹோவ்த் நிலையான நேரம்"}, {"India", "இந்திய நிலையான நேரம்"}, {"Indian_Ocean", "இந்தியப் பெருங்கடல் நேரம்"}, {"Indochina", "இந்தோசீன நேரம்"}, {"Indonesia_Central", "மத்திய இந்தோனேசிய நேரம்"}, {"Indonesia_Eastern", "கிழக்கத்திய இந்தோனேசிய நேரம்"}, {"Indonesia_Western", "மேற்கத்திய இந்தோனேசிய நேரம்"}, {"Iran", "ஈரான் நிலையான நேரம்"}, {"Irkutsk", "இர்குட்ஸ்க் நிலையான நேரம்"}, {"Israel", "இஸ்ரேல் நிலையான நேரம்"}, {"Japan", "ஜப்பான் நிலையான நேரம்"}, {"Kamchatka", "பெட்ரோபவ்லோவ்ஸ்க் கம்சட்ஸ்கி தர நேரம்"}, {"Kazakhstan", "கஜகஸ்தான் நேரம்"}, {"Kazakhstan_Eastern", "கிழக்கு கஜகஸ்தான் நேரம்"}, {"Kazakhstan_Western", "மேற்கு கஜகஸ்தான் நேரம்"}, {"Korea", "கொரிய நிலையான நேரம்"}, {"Kosrae", "கோஸ்ரே நேரம்"}, {"Krasnoyarsk", "க்ரஸ்னோயார்ஸ்க் நிலையான நேரம்"}, {"Kyrgystan", "கிர்கிஸ்தான் நேரம்"}, {"Lanka", "லங்கா நேரம்"}, {"Line_Islands", "லைன் தீவுகள் நேரம்"}, {"Lord_Howe", "லார்ட் ஹோவ் நிலையான நேரம்"}, {"Macau", "மக்காவ் தர நேரம்"}, {"Magadan", "மகதன் நிலையான நேரம்"}, {"Malaysia", "மலேஷிய நேரம்"}, {"Maldives", "மாலத்தீவுகள் நேரம்"}, {"Marquesas", "மார்கியூசாஸ் நேரம்"}, {"Marshall_Islands", "மார்ஷல் தீவுகள் நேரம்"}, {"Mauritius", "மொரிஷியஸ் நிலையான நேரம்"}, {"Mawson", "மாசன் நேரம்"}, {"Mexico_Pacific", "மெக்ஸிகன் பசிபிக் நிலையான நேரம்"}, {"Mongolia", "உலன் பாடர் நிலையான நேரம்"}, {"Moscow", "மாஸ்கோ நிலையான நேரம்"}, {"Myanmar", "மியான்மர் நேரம்"}, {"Nauru", "நவ்ரூ நேரம்"}, {"Nepal", "நேபாள நேரம்"}, {"New_Caledonia", "நியூ கலிடோனியா நிலையான நேரம்"}, {"New_Zealand", "நியூசிலாந்து நிலையான நேரம்"}, {"Newfoundland", "நியூஃபவுண்ட்லாந்து நிலையான நேரம்"}, {"Niue", "நியு நேரம்"}, {"Norfolk", "நார்ஃபோக் தீவு நிலையான நேரம்"}, {"Noronha", "பெர்னான்டோ டி நோரோன்ஹா நிலையான நேரம்"}, {"North_Mariana", "வடக்கு மரினா தீவுகள் நேரம்"}, {"Novosibirsk", "நோவோசிபிரிஸ்க் நிலையான நேரம்"}, {"Omsk", "ஓம்ஸ்க் நிலையான நேரம்"}, {"Pakistan", "பாகிஸ்தான் நிலையான நேரம்"}, {"Palau", "பாலவ் நேரம்"}, {"Papua_New_Guinea", "பபுவா நியூ கினியா நேரம்"}, {"Paraguay", "பராகுவே நிலையான நேரம்"}, {"Peru", "பெரு நிலையான நேரம்"}, {"Philippines", "பிலிப்பைன் நிலையான நேரம்"}, {"Phoenix_Islands", "ஃபோனிக்ஸ் தீவுகள் நேரம்"}, {"Pierre_Miquelon", "செயின்ட் பியரி & மிக்குயிலான் நிலையான நேரம்"}, {"Pitcairn", "பிட்கெய்ர்ன் நேரம்"}, {"Ponape", "போனாபே நேரம்"}, {"Pyongyang", "பியாங்யாங் நேரம்"}, {"Qyzylorda", "கைஜைலோர்டா தர நேரம்"}, {"Reunion", "ரீயூனியன் நேரம்"}, {"Rothera", "ரோதேரா நேரம்"}, {"Sakhalin", "சகலின் நிலையான நேரம்"}, {"Samara", "சமரா தர நேரம்"}, {"Samoa", "சமோவா நிலையான நேரம்"}, {"Seychelles", "சீசெல்ஸ் நேரம்"}, {"Singapore", "சிங்கப்பூர் நிலையான நேரம்"}, {"Solomon", "சாலமன் தீவுகள் நேரம்"}, {"South_Georgia", "தெற்கு ஜார்ஜியா நேரம்"}, {"Suriname", "சுரினாம் நேரம்"}, {"Syowa", "ஸ்யோவா நேரம்"}, {"Tahiti", "தஹிதி நேரம்"}, {"Taipei", "தாய்பே நிலையான நேரம்"}, {"Tajikistan", "தஜிகிஸ்தான் நேரம்"}, {"Tokelau", "டோக்கெலாவ் நேரம்"}, {"Tonga", "டோங்கா நிலையான நேரம்"}, {"Truk", "சுக் நேரம்"}, {"Turkey", "துருக்கி தர நேரம்"}, {"Turkmenistan", "துர்க்மெனிஸ்தான் நிலையான நேரம்"}, {"Tuvalu", "துவாலு நேரம்"}, {"Uruguay", "உருகுவே நிலையான நேரம்"}, {"Uzbekistan", "உஸ்பெகிஸ்தான் நிலையான நேரம்"}, {"Vanuatu", "வனுவாட்டு நிலையான நேரம்"}, {"Venezuela", "வெனிசுலா நேரம்"}, {"Vladivostok", "விளாடிவோஸ்டோக் நிலையான நேரம்"}, {"Volgograd", "வோல்கோக்ராட் நிலையான நேரம்"}, {"Vostok", "வோஸ்டோக் நேரம்"}, {"Wake", "வேக் தீவு நேரம்"}, {"Wallis", "வாலிஸ் மற்றும் ஃப்யூடுனா நேரம்"}, {"Yakutsk", "யகுட்ஸ்க் நிலையான நேரம்"}, {"Yekaterinburg", "யேகாடெரின்பர்க் நிலையான நேரம்"}, {"Yukon", "யூகோன் நேரம்"}},
		{{"Acre", "அக்ரே கோடை நேரம்"}, {"Alaska", "அலாஸ்கா பகலொளி நேரம்"}, {"Almaty", "அல்மாடி கோடை நேரம்"}, {"Amazon", "அமேசான் கோடை நேரம்"}, {"America_Central", "மத்திய பகலொளி நேரம்"}, {"America_Eastern", "கிழக்கத்திய பகலொளி நேரம்"}, {"America_Mountain", "மவுன்டைன் பகலொளி நேரம்"}, {"America_Pacific", "பசிபிக் பகலொளி நேரம்"}, {"Anadyr", "அனாடையர் கோடை நேரம்"}, {"Apia", "ஏபியா பகலொளி நேரம்"}, {"Aqtau", "அட்டௌ கோடை நேரம்"}, {"Aqtobe", "அட்டோபே கோடை நேரம்"}, {"Arabian", "அரேபிய பகலொளி நேரம்"}, {"Argentina", "அர்ஜென்டினா கோடை நேரம்"}, {"Argentina_Western", "மேற்கத்திய அர்ஜென்டினா கோடை நேரம்"}, {"Armenia", "ஆர்மேனிய கோடை நேரம்"}, {"Atlantic", "அட்லாண்டிக் பகலொளி நேரம்"}, {"Australia_Central", "ஆஸ்திரேலியன் மத்திய பகலொளி நேரம்"}, {"Australia_CentralWestern", "ஆஸ்திரேலியன் மத்திய மேற்கத்திய பகலொளி நேரம்"}, {"Australia_Eastern", "ஆஸ்திரேலியன் கிழக்கத்திய பகலொளி நேரம்"}, {"Australia_Western", "ஆஸ்திரேலியன் மேற்கத்திய பகலொளி நேரம்"}, {"Azerbaijan", "அசர்பைஜான் கோடை நேரம்"}, {"Azores", "அசோர்ஸ் கோடை நேரம்"}, {"Bangladesh", "வங்கதேச கோடை நேரம்"}, {"Brasilia", "பிரேசிலியா கோடை நேரம்"}, {"Cape_Verde", "கேப் வெர்டே கோடை நேரம்"}, {"Chatham", "சத்தாம் பகலொளி நேரம்"}, {"Chile", "சிலி கோடை நேரம்"}, {"China", "சீன பகலொளி நேரம்"}, {"Colombia", "கொலம்பியா கோடை நேரம்"}, {"Cook", "குக் தீவுகள் அரை கோடை நேரம்"}, {"Cuba", "கியூபா பகலொளி நேரம்"}, {"Easter", "ஈஸ்டர் தீவு கோடை நேரம்"}, {"Europe_Central", "மத்திய ஐரோப்பிய கோடை நேரம்"}, {"Europe_Eastern", "கிழக்கத்திய ஐரோப்பிய கோடை நேரம்"}, {"Europe_Western", "மேற்கத்திய ஐரோப்பிய கோடை நேரம்"}, {"Falkland", "ஃபாக்லாந்து தீவுகள் கோடை நேரம்"}, {"Fiji", "ஃபிஜி கோடை நேரம்"}, {"Georgia", "ஜார்ஜியா கோடை நேரம்"}, {"Greenland_Eastern", "கிழக்கு கிரீன்லாந்து கோடை நேரம்"}, {"Greenland_Western", "மேற்கு கிரீன்லாந்து கோடை நேரம்"}, {"Hawaii_Aleutian", "ஹவாய்-அலேஷியன் பகலொளி நேரம்"}, {"Hong_Kong", "ஹாங்காங் கோடை நேரம்"}, {"Hovd", "ஹோவ்த் கோடை நேரம்"}, {"Iran", "ஈரான் பகலொளி நேரம்"}, {"Irkutsk", "இர்குட்ஸ்க் கோடை நேரம்"}, {"Israel", "இஸ்ரேல் பகலொளி நேரம்"}, {"Japan", "ஜப்பான் பகலொளி நேரம்"}, {"Kamchatka", "பெட்ரோபவ்லோவ்ஸ்க் கம்சட்ஸ்கி கோடை நேரம்"}, {"Korea", "கொரிய பகலொளி நேரம்"}, {"Krasnoyarsk", "க்ரஸ்னோயார்ஸ்க் கோடை நேரம்"}, {"Lord_Howe", "லார்ட் ஹோவ் பகலொளி நேரம்"}, {"Macau", "மக்காவ் கோடை நேரம்"}, {"Magadan", "மகதன் கோடை நேரம்"}, {"Mauritius", "மொரிஷியஸ் கோடை நேரம்"}, {"Mexico_Pacific", "மெக்ஸிகன் பசிபிக் பகலொளி நேரம்"}, {"Mongolia", "உலன் பாடர் கோடை நேரம்"}, {"Moscow", "மாஸ்கோ கோடை நேரம்"}, {"New_Caledonia", "நியூ கலிடோனியா கோடை நேரம்"}, {"New_Zealand", "நியூசிலாந்து பகலொளி நேரம்"}, {"Newfoundland", "நியூஃபவுண்ட்லாந்து பகலொளி நேரம்"}, {"Norfolk", "நார்ஃபோக் தீவு பகலொளி நேரம்"}, {"Noronha", "பெர்னான்டோ டி நோரோன்ஹா கோடை நேரம்"}, {"Novosibirsk", "நோவோசிபிரிஸ்க் கோடை நேரம்"}, {"Omsk", "ஓம்ஸ்க் கோடை நேரம்"}, {"Pakistan", "பாகிஸ்தான் கோடை நேரம்"}, {"Paraguay", "பராகுவே கோடை நேரம்"}, {"Peru", "பெரு கோடை நேரம்"}, {"Philippines", "பிலிப்பைன் கோடை நேரம்"}, {"Pierre_Miquelon", "செயின்ட் பியரி & மிக்குயிலான் பகலொளி நேரம்"}, {"Qyzylorda", "கைஜைலோர்டா கோடை நேரம்"}, {"Sakhalin", "சகலின் கோடை நேரம்"}, {"Samara", "சமரா கோடை நேரம்"}, {"Samoa", "சமோவா பகலொளி நேரம்"}, {"Taipei", "தாய்பே பகலொளி நேரம்"}, {"Tonga", "டோங்கா கோடை நேரம்"}, {"Turkey", "துருக்கி கோடை நேரம்"}, {"Turkmenistan", "துர்க்மெனிஸ்தான் கோடை நேரம்"}, {"Uruguay", "உருகுவே கோடை நேரம்"}, {"Uzbekistan", "உஸ்பெகிஸ்தான் கோடை நேரம்"}, {"Vanuatu", "வனுவாட்டு கோடை நேரம்"}, {"Vladivostok", "விளாடிவோஸ்டோக் கோடை நேரம்"}, {"Volgograd", "வோல்கோக்ராட் கோடை நேரம்"}, {"Yakutsk", "யகுட்ஸ்க் கோடை நேரம்"}, {"Yekaterinburg", "யேகாடெரின்பர்க் கோடை நேரம்"}},
	},
	{
		{{"Acre", "ఏకర్ ప్రామాణిక సమయం"}, {"Afghanistan", "ఆఫ్ఘనిస్తాన్ సమయం"}, {"Africa_Central", "సెంట్రల్ ఆఫ్రికా సమయం"}, {"Africa_Eastern", "తూర్పు ఆఫ్రికా సమయం"}, {"Africa_Southern", "దక్షిణ ఆఫ్రికా ప్రామాణిక సమయం"}, {"Africa_Western", "పశ్చిమ ఆఫ్రికా సమయం"}, {"Alaska", "అలాస్కా ప్రామాణిక సమయం"}, {"Almaty", "అల్మాటి ప్రామాణిక సమయం"}, {"Amazon", "అమెజాన్ ప్రామాణిక సమయం"}, {"America_Central", "మధ్యమ ప్రామాణిక సమయం"}, {"America_Eastern", "తూర్పు ప్రామాణిక సమయం"}, {"America_Mountain", "మౌంటెయిన్ ప్రామాణిక సమయం"}, {"America_Pacific", "పసిఫిక్ ప్రామాణిక సమయం"}, {"Anadyr", "అనాన్డ్రి ప్రామాణిక సమయం"}, {"Apia", "సమోవా ప్రామాణిక సమయం"}, {"Aqtau", "అక్వాటు ప్రామాణిక సమయం"}, {"Aqtobe", "అక్టోబె ప్రామాణిక సమయం"}, {"Arabian", "అరేబియన్ ప్రామాణిక సమయం"}, {"Argentina", "అర్జెంటీనా ప్రామాణిక సమయం"}, {"Argentina_Western", "పశ్చిమ అర్జెంటీనా ప్రామాణిక సమయం"}, {"Armenia", "ఆర్మేనియా ప్రామాణిక సమయం"}, {"Atlantic", "అట్లాంటిక్ ప్రామాణిక సమయం"}, {"Australia_Central", "ఆస్ట్రేలియా మధ్యమ ప్రామాణిక సమయం"}, {"Australia_CentralWestern", "మధ్యమ ఆస్ట్రేలియన్ పశ్చిమ ప్రామాణిక సమయం"}, {"Australia_Eastern", "ఆస్ట్రేలియన్ తూర్పు ప్రామాణిక సమయం"}, {"Australia_Western", "ఆస్ట్రేలియన్ పశ్చిమ ప్రామాణిక సమయం"}, {"Azerbaijan", "అజర్బైజాన్ ప్రామాణిక సమయం"}, {"Azores", "అజోర్స్ ప్రామాణిక సమయం"}, {"Bangladesh", "బంగ్లాదేశ్ ప్రామాణిక సమయం"}, {"Bhutan", "భూటాన్ సమయం"}, {"Bolivia", "బొలీవియా సమయం"}, {"Brasilia", "బ్రెజిలియా ప్రామాణిక సమయం"}, {"Brunei", "బ్రూనై దారుసలామ్ సమయం"}, {"Cape_Verde", "కేప్ వెర్డె ప్రామాణిక సమయం"}, {"Chamorro", "చామర్రో ప్రామాణిక సమయం"}, {"Chatham", "చాథమ్ ప్రామాణిక సమయం"}, {"Chile", "చిలీ ప్రామాణిక సమయం"}, {"China", "చైనా ప్రామాణిక సమయం"}, {"Christmas", "క్రిస్మస్ దీవి సమయం"}, {"Cocos", "కోకోస్ దీవుల సమయం"}, {"Colombia", "కొలంబియా ప్రామాణిక సమయం"}, {"Cook", "కుక్ దీవుల ప్రామాణిక సమయం"}, {"Cuba", "క్యూబా ప్రామాణిక సమయం"}, {"Davis", "డేవిస్ సమయం"}, {"DumontDUrville", "డ్యూమాంట్-డి’ఉర్విల్లే సమయం"}, {"East_Timor", "తూర్పు తైమూర్ సమయం"}, {"Easter", "ఈస్టర్ దీవి ప్రామాణిక సమయం"}, {"Ecuador", "ఈక్వడార్ సమయం"}, {"Europe_Central", "సెంట్రల్ యూరోపియన్ ప్రామాణిక సమయం"}, {"Europe_Eastern", "తూర్పు యూరోపియన్ ప్రామాణిక సమయం"}, {"Europe_Further_Eastern", "సుదూర-తూర్పు యూరోపియన్ సమయం"}, {"Europe_Western", "పశ్చిమ యూరోపియన్ ప్రామాణిక సమయం"}, {"Falkland", "ఫాక్\u200cల్యాండ్ దీవుల ప్రామాణిక సమయం"}, {"Fiji", "ఫిజీ ప్రామాణిక సమయం"}, {"French_Guiana", "ఫ్రెంచ్ గయానా సమయం"}, {"French_Southern", "ఫ్రెంచ్ దక్షిణ మరియు అంటార్కిటిక్ సమయం"}, {"GMT", "గ్రీన్\u200cవిచ్ సగటు సమయం"}, {"Galapagos", "గాలాపాగోస్ సమయం"}, {"Gambier", "గాంబియర్ సమయం"}, {"Georgia", "జార్జియా ప్రామాణిక సమయం"}, {"Gilbert_Islands", "గిల్బర్ట్ దీవుల సమయం"}, {"Greenland_Eastern", "తూర్పు గ్రీన్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Greenland_Western", "పశ్చిమ గ్రీన్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Guam", "గ్వామ్ ప్రామాణిక సమయం"}, {"Gulf", "గల్ఫ్ ప్రామాణిక సమయం"}, {"Guyana", "గయానా సమయం"}, {"Hawaii", "హవాయ్-అల్యూషియన్ ప్రామాణిక సమయం"}, {"Hawaii_Aleutian", "హవాయ్-అల్యూషియన్ ప్రామాణిక సమయం"}, {"Hong_Kong", "హాంకాంగ్ ప్రామాణిక సమయం"}, {"Hovd", "హోవ్డ్ ప్రామాణిక సమయం"}, {"India", "భారతదేశ ప్రామాణిక సమయం"}, {"Indian_Ocean", "హిందూ మహా సముద్ర సమయం"}, {"Indochina", "ఇండోచైనా సమయం"}, {"Indonesia_Central", "సెంట్రల్ ఇండోనేషియా సమయం"}, {"Indonesia_Eastern", "తూర్పు ఇండోనేషియా సమయం"}, {"Indonesia_Western", "పశ్చిమ ఇండోనేషియా సమయం"}, {"Iran", "ఇరాన్ ప్రామాణిక సమయం"}, {"Irkutsk", "ఇర్కుట్స్క్ ప్రామాణిక సమయం"}, {"Israel", "ఇజ్రాయిల్ ప్రామాణిక సమయం"}, {"Japan", "జపాన్ ప్రామాణిక సమయం"}, {"Kamchatka", "పెట్రోపావ్లోవ్స్క్-కామ్ఛాట్స్కి ప్రామాణిక సమయం"}, {"Kazakhstan", "కజకిస్తాన్ సమయం"}, {"Kazakhstan_Eastern", "తూర్పు కజకి\u200cస్తాన్ సమయం"}, {"Kazakhstan_Western", "పశ్చిమ కజకిస్తాన్ సమయం"}, {"Korea", "కొరియన్ ప్రామాణిక సమయం"}, {"Kosrae", "కోస్రాయి సమయం"}, {"Krasnoyarsk", "క్రాస్నోయార్స్క్ ప్రామాణిక సమయం"}, {"Kyrgystan", "కిర్గిస్తాన్ సమయం"}, {"Lanka", "లంకా సమయం"}, {"Line_Islands", "లైన్ దీవుల సమయం"}, {"Lord_Howe", "లార్డ్ హోవ్ ప్రామాణిక సమయం"}, {"Macau", "మకావ్ ప్రామాణిక సమయం"}, {"Magadan", "మగడాన్ ప్రామాణిక సమయం"}, {"Malaysia", "మలేషియా సమయం"}, {"Maldives", "మాల్దీవుల సమయం"}, {"Marquesas", "మార్క్వేసాస్ సమయం"}, {"Marshall_Islands", "మార్షల్ దీవుల సమయం"}, {"Mauritius", "మారిషస్ ప్రామాణిక సమయం"}, {"Mawson", "మాసన్ సమయం"}, {"Mexico_Pacific", "మెక్సికన్ పసిఫిక్ ప్రామాణిక సమయం"}, {"Mongolia", "ఉలన్ బతోర్ ప్రామాణిక సమయం"}, {"Moscow", "మాస్కో ప్రామాణిక సమయం"}, {"Myanmar", "మయన్మార్ సమయం"}, {"Nauru", "నౌరు సమయం"}, {"Nepal", "నేపాల్ సమయం"}, {"New_Caledonia", "న్యూ కాలెడోనియా ప్రామాణిక సమయం"}, {"New_Zealand", "న్యూజిల్యాండ్ ప్రామాణిక సమయం"}, {"Newfoundland", "న్యూఫౌండ్\u200cల్యాండ్ ప్రామాణిక సమయం"}, {"Niue", "నియూ సమయం"}, {"Norfolk", "నార్ఫోక్ దీవి ప్రామాణిక సమయం"}, {"Noronha", "ఫెర్నాండో డి నొరోన్హా ప్రామాణిక సమయం"}, {"North_Mariana", "ఉత్తర మారియానా దీవుల సమయం"}, {"Novosibirsk", "నోవోసిబిర్క్స్ ప్రామాణిక సమయం"}, {"Omsk", "ఓమ్స్క్ ప్రామాణిక సమయం"}, {"Pakistan", "పాకిస్తాన్ ప్రామాణిక సమయం"}, {"Palau", "పాలావ్ సమయం"}, {"Papua_New_Guinea", "పాపువా న్యూ గినియా సమయం"}, {"Paraguay", "పరాగ్వే ప్రామాణిక సమయం"}, {"Peru", "పెరూ ప్రామాణిక సమయం"}, {"Philippines", "ఫిలిప్పైన్ ప్రామాణిక సమయం"}, {"Phoenix_Islands", "ఫినిక్స్ దీవుల సమయం"}, {"Pierre_Miquelon", "సెయింట్ పియెర్ మరియు మిక్వెలాన్ ప్రామాణిక సమయం"}, {"Pitcairn", "పిట్\u200cకైర్న్ సమయం"}, {"Ponape", "పొనేప్ సమయం"}, {"Pyongyang", "ఉత్తర కొరియా సమయం"}, {"Qyzylorda", "కిజిలోర్డా ప్రామాణిక సమయం"}, {"Reunion", "రీయూనియన్ సమయం"}, {"Rothera", "రొతేరా సమయం"}, {"Sakhalin", "సఖాలిన్ ప్రామాణిక సమయం"}, {"Samara", "సమారా ప్రామాణిక సమయం"}, {"Samoa", "సమోవా ప్రామాణిక సమయం"}, {"Seychelles", "సీషెల్స్ సమయం"}, {"Singapore", "సింగపూర్ ప్రామాణిక సమయం"}, {"Solomon", "సోలమన్ దీవుల సమయం"}, {"South_Georgia", "దక్షిణ జార్జియా సమయం"}, {"Suriname", "సూరినామ్ సమయం"}, {"Syowa", "స్యోవా సమయం"}, {"Tahiti", "తహితి సమయం"}, {"Taipei", "తైవాన్ ప్రామాణిక సమయం"}, {"Tajikistan", "తజికిస్తాన్ సమయం"}, {"Tokelau", "టోకెలావ్ సమయం"}, {"Tonga", "టాంగా ప్రామాణిక సమయం"}, {"Truk", "చక్ సమయం"}, {"Turkey", "టర్కీ ప్రామాణిక సమయం"}, {"Turkmenistan", "తుర్క్\u200cమెనిస్తాన్ ప్రామాణిక సమయం"}, {"Tuvalu", "తువాలు సమయం"}, {"Uruguay", "ఉరుగ్వే ప్రామాణిక సమయం"}, {"Uzbekistan", "ఉజ్బెకిస్తాన్ ప్రామాణిక సమయం"}, {"Vanuatu", "వనౌటు ప్రామాణిక సమయం"}, {"Venezuela", "వెనిజులా సమయం"}, {"Vladivostok", "వ్లాడివోస్టోక్ ప్రామాణిక సమయం"}, {"Volgograd", "వోల్గోగ్రాడ్ ప్రామాణిక సమయం"}, {"Vostok", "వోస్టోక్ సమయం"}, {"Wake", "వేక్ దీవి సమయం"}, {"Wallis", "వాలీస్ మరియు ఫుటునా సమయం"}, {"Yakutsk", "యాకుట్స్క్ ప్రామాణిక సమయం"}, {"Yekaterinburg", "యెకటెరిన్\u200cబర్గ్ ప్రామాణిక సమయం"}, {"Yukon", "యుకోన్ సమయం"}},
		{{"Acre", "ఏకర్ వేసవి సమయం"}, {"Alaska", "అలాస్కా పగటి వెలుతురు సమయం"}, {"Almaty", "అల్మాటి వేసవి సమయం"}, {"Amazon", "అమెజాన్ వేసవి సమయం"}, {"America_Central", "మధ్యమ పగటి వెలుతురు సమయం"}, {"America_Eastern", "తూర్పు పగటి వెలుతురు సమయం"}, {"America_Mountain", "మౌంటెయిన్ పగటి వెలుతురు సమయం"}, {"America_Pacific", "పసిఫిక్ పగటి వెలుతురు సమయం"}, {"Anadyr", "అనాన్డ్రి వేసవి సమయం"}, {"Apia", "సమోవా పగటి వెలుతురు సమయం"}, {"Aqtau", "అక్వాటు వేసవి సమయం"}, {"Aqtobe", "అక్టోబె వేసవి సమయం"}, {"Arabian", "అరేబియన్ పగటి వెలుతురు సమయం"}, {"Argentina", "ఆర్జెంటీనా వేసవి సమయం"}, {"Argentina_Western", "పశ్చిమ అర్జెంటీనా వేసవి సమయం"}, {"Armenia", "ఆర్మేనియా వేసవి సమయం"}, {"Atlantic", "అట్లాంటిక్ పగటి వెలుతురు సమయం"}, {"Australia_Central", "ఆస్ట్రేలియా మధ్యమ పగటి వెలుతురు సమయం"}, {"Australia_CentralWestern", "ఆస్ట్రేలియా మధ్యమ పశ్చిమ పగటి వెలుతురు సమయం"}, {"Australia_Eastern", "ఆస్ట్రేలియన్ తూర్పు పగటి వెలుతురు సమయం"}, {"Australia_Western", "ఆస్ట్రేలియన్ పశ్చిమ పగటి వెలుతురు సమయం"}, {"Azerbaijan", "అజర్బైజాన్ వేసవి సమయం"}, {"Azores", "అజోర్స్ వేసవి సమయం"}, {"Bangladesh", "బంగ్లాదేశ్ వేసవి సమయం"}, {"Brasilia", "బ్రెజిలియా వేసవి సమయం"}, {"Cape_Verde", "కేప్ వెర్డె వేసవి సమయం"}, {"Chatham", "చాథమ్ పగటి వెలుతురు సమయం"}, {"Chile", "చిలీ వేసవి సమయం"}, {"China", "చైనా పగటి వెలుతురు సమయం"}, {"Colombia", "కొలంబియా వేసవి సమయం"}, {"Cook", "కుక్ దీవుల అర్ధ వేసవి సమయం"}, {"Cuba", "క్యూబా పగటి వెలుతురు సమయం"}, {"Easter", "ఈస్టర్ దీవి వేసవి సమయం"}, {"Europe_Central", "సెంట్రల్ యూరోపియన్ వేసవి సమయం"}, {"Europe_Eastern", "తూర్పు యూరోపియన్ వేసవి సమయం"}, {"Europe_Western", "పశ్చిమ యూరోపియన్ వేసవి సమయం"}, {"Falkland", "ఫాక్\u200cల్యాండ్ దీవుల వేసవి సమయం"}, {"Fiji", "ఫిజీ వేసవి సమయం"}, {"Georgia", "జార్జియా వేసవి సమయం"}, {"Greenland_Eastern", "తూర్పు గ్రీన్\u200cల్యాండ్ వేసవి సమయం"}, {"Greenland_Western", "పశ్చిమ గ్రీన్\u200cల్యాండ్ వేసవి సమయం"}, {"Hawaii_Aleutian", "హవాయ్-అల్యూషియన్ పగటి వెలుతురు సమయం"}, {"Hong_Kong", "హాంకాంగ్ వేసవి సమయం"}, {"Hovd", "హోవ్డ్ వేసవి సమయం"}, {"Iran", "ఇరాన్ పగటి వెలుతురు సమయం"}, {"Irkutsk", "ఇర్కుట్స్క్ వేసవి సమయం"}, {"Israel", "ఇజ్రాయిల్ పగటి వెలుతురు సమయం"}, {"Japan", "జపాన్ పగటి వెలుతురు సమయం"}, {"Kamchatka", "పెట్రోపావ్లోవ్స్క్-కామ్ఛాట్స్కి వేసవి సమయం"}, {"Korea", "కొరియన్ పగటి వెలుతురు సమయం"}, {"Krasnoyarsk", "క్రాస్నోయార్స్క్ వేసవి సమయం"}, {"Lord_Howe", "లార్డ్ హోవ్ పగటి వెలుతురు సమయం"}, {"Macau", "మకావ్ వేసవి సమయం"}, {"Magadan", "మగడాన్ వేసవి సమయం"}, {"Mauritius", "మారిషస్ వేసవి సమయం"}, {"Mexico_Pacific", "మెక్సికన్ పసిఫిక్ పగటి వెలుతురు సమయం"}, {"Mongolia", "ఉలన్ బతోర్ వేసవి సమయం"}, {"Moscow", "మాస్కో వేసవి సమయం"}, {"New_Caledonia", "న్యూ కాలెడోనియా వేసవి సమయం"}, {"New_Zealand", "న్యూజిల్యాండ్ పగటి వెలుతురు సమయం"}, {"Newfoundland", "న్యూఫౌండ్\u200cల్యాండ్ పగటి వెలుతురు సమయం"}, {"Norfolk", "నార్ఫోక్ దీవి పగటి వెలుతురు సమయం"}, {"Noronha", "ఫెర్నాండో డి నొరోన్హా వేసవి సమయం"}, {"Novosibirsk", "నోవోసిబిర్స్క్ వేసవి సమయం"}, {"Omsk", "ఓమ్స్క్ వేసవి సమయం"}, {"Pakistan", "పాకిస్తాన్ వేసవి సమయం"}, {"Paraguay", "పరాగ్వే వేసవి సమయం"}, {"Peru", "పెరూ వేసవి సమయం"}, {"Philippines", "ఫిలిప్పైన్ వేసవి సమయం"}, {"Pierre_Miquelon", "సెయింట్ పియర్ మరియు మిక్వెలాన్ పగటి వెలుతురు సమయం"}, {"Qyzylorda", "కిజిలోర్డా వేసవి సమయం"}, {"Sakhalin", "సఖాలిన్ వేసవి సమయం"}, {"Samara", "సమారా వేసవి సమయం"}, {"Samoa", "సమోవా పగటి వెలుతురు సమయం"}, {"Taipei", "తైవాన్ పగటి వెలుతురు సమయం"}, {"Tonga", "టాంగా వేసవి సమయం"}, {"Turkey", "టర్కీ వేసవి సమయం"}, {"Turkmenistan", "తుర్క్\u200cమెనిస్తాన్ వేసవి సమయం"}, {"Uruguay", "ఉరుగ్వే వేసవి సమయం"}, {"Uzbekistan", "ఉజ్బెకిస్తాన్ వేసవి సమయం"}, {"Vanuatu", "వనౌటు వేసవి సమయం"}, {"Vladivostok", "వ్లాడివోస్టోక్ వేసవి సమయం"}, {"Volgograd", "వోల్గోగ్రాడ్ వేసవి సమయం"}, {"Yakutsk", "యాకుట్స్క్ వేసవి సమయం"}, {"Yekaterinburg", "యెకటెరిన్\u200cబర్గ్ వేసవి సమయం"}},
	},
	{
		{{"Afghanistan", "Вақти Афғонистон"}, {"Africa_Central", "Вақти Африқои Марказӣ"}, {"Africa_Eastern", "Вақти Африқои Шарқӣ"}, {"Africa_Southern", "Вақти стандартии Африқои Ҷанубӣ"}, {"Africa_Western", "Вақти Африқои Ғарбӣ"}, {"Alaska", "Вақти стандартии Аляска"}, {"Amazon", "Вақти стандартии Амазон"}, {"America_Central", "Вақти стандартии марказӣ"}, {"America_Eastern", "Вақти стандартии шарқӣ"}, {"America_Mountain", "Вақти стандартии кӯҳӣ"}, {"America_Pacific", "Вақти стандартии Уқёнуси Ором"}, {"Apia", "Вақти стандартии Апиа"}, {"Arabian", "Вақти стандартии Арабистон"}, {"Argentina", "Вақти стандартии Аргентина"}, {"Argentina_Western", "Вақти стандартии Аргентинаи Ғарбӣ"}, {"Armenia", "Вақти стандартии Арманистон"}, {"Atlantic", "Вақти стандартии атлантикӣ"}, {"Australia_Central", "Вақти стандартии Австралияи Марказӣ"}, {"Australia_CentralWestern", "Вақти стандартии Ғарбии Австралия Марказӣ"}, {"Australia_Eastern", "Вақти стандартии шарқии Австралия"}, {"Australia_Western", "Вақти стандартии Австралияи Ғарбӣ"}, {"Azerbaijan", "Вақти стандартии Озарбойҷон"}, {"Azores", "Вақти стандартии Азор"}, {"Bangladesh", "Вақти стандартии Бангладеш"}, {"Bhutan", "Вақти Бутан"}, {"Bolivia", "Вақти Боливия"}, {"Brasilia", "Вақти стандартии Бразилия"}, {"Brunei", "Вақти Бруней Доруссалом"}, {"Cape_Verde", "Вақти стандартии Кабо-Верде"}, {"Chamorro", "Вақти стандартии Чаморро"}, {"Chatham", "Вақти стандартии Чатам"}, {"Chile", "Вақти стандартии Чили"}, {"China", "Вақти стандартии Чин"}, {"Christmas", "Вақти ҷазираи Мавлуди Исо"}, {"Cocos", "Вақти Ҷазираҳои Кокос"}, {"Colombia", "Вақти стандартии Колумбия"}, {"Cook", "Вақти стандартии Ҷазираҳои Кук"}, {"Cuba", "Вақти стандартии Куба"}, {"Davis", "Вақти Давис"}, {"DumontDUrville", "Вақти Дюмон-д’Урвил"}, {"East_Timor", "Вақти Тимори Шарқӣ"}, {"Easter", "Вақти стандартии ҷазираи Пасха"}, {"Ecuador", "Вақти Эквадор"}, {"Europe_Central", "Вақти стандартии аврупоии марказӣ"}, {"Europe_Eastern", "Вақти стандартии аврупоии шарқӣ"}, {"Europe_Further_Eastern", "Вақти Аврупои Шарқӣ"}, {"Europe_Western", "Вақти стандартии аврупоии ғарбӣ"}, {"Falkland", "Вақти стандартии Ҷазираҳои Фолкленд"}, {"Fiji", "Вақти стандартии Фиҷи"}, {"French_Guiana", "Вақти Гвианаи Фаронса"}, {"French_Southern", "Вақти ҷанубӣ ва Антарктидаи Фаронса"}, {"GMT", "Вақти миёнаи Гринвич"}, {"Galapagos", "Вақти Галапагос"}, {"Gambier", "Вақти Гамбир"}, {"Georgia", "Вақти стандартии Гурҷистон"}, {"Gilbert_Islands", "Вақти Ҷазираҳои Гилберт"}, {"Greenland_Eastern", "Вақти стандартии Гренландияи Шарқӣ"}, {"Greenland_Western", "Вақти стандартии Гренландияи Ғарбӣ"}, {"Gulf", "Вақти стандартии Халиҷи Форс"}, {"Guyana", "Вақти Гайана"}, {"Hawaii", "Вақти стандартии Ҳавайӣ-Алеутӣ"}, {"Hawaii_Aleutian", "Вақти стандартии Ҳавайӣ-Алеутӣ"}, {"Hong_Kong", "Вақти стандартии Ҳонконг"}, {"Hovd", "Вақти стандартии Ҳовд"}, {"India", "Вақти стандартии Ҳиндустон"}, {"Indian_Ocean", "Вақти уқёнуси Ҳинд"}, {"Indochina", "Вақти Ҳиндучин"}, {"Indonesia_Central", "Вақти Индонезияи Марказӣ"}, {"Indonesia_Eastern", "Вақти шарқии Индонезия"}, {"Indonesia_Western", "Вақти Индонезияи Ғарбӣ"}, {"Iran", "Вақти стандартии Эрон"}, {"Irkutsk", "Вақти стандартии Иркутск"}, {"Israel", "Вақти стандартии Исроил"}, {"Japan", "Вақти стандартии Ҷопон"}, {"Kazakhstan", "Вақти Қазоқистон"}, {"Kazakhstan_Eastern", "Вақти Қазоқистони Шарқӣ"}, {"Kazakhstan_Western", "Вақти Қазоқистони Ғарбӣ"}, {"Korea", "Вақти стандартии Корея"}, {"Kosrae", "Вақти Косрае"}, {"Krasnoyarsk", "Вақти стандартии Красноярск"}, {"Kyrgystan", "Вақти Қирғизистон"}, {"Line_Islands", "Вақти Ҷазираҳои Лин"}, {"Lord_Howe", "Вақти стандартии Лорд Хоу"}, {"Magadan", "Вақти стандартии Магадан"}, {"Malaysia", "Вақти Малайзия"}, {"Maldives", "Вақти Малдив"}, {"Marquesas", "Вақти Маркес"}, {"Marshall_Islands", "Вақти Ҷазираҳои Маршалл"}, {"Mauritius", "Вақти стандартии Маврикий"}, {"Mawson", "Вақти Мавсон"}, {"Mexico_Pacific", "Вақти стандартии Уқёнуси Ором Мексика"}, {"Mongolia", "Вақти стандартии Улан-Батор"}, {"Moscow", "Вақти стандартии Маскав"}, {"Myanmar", "Вақти Мянма"}, {"Nauru", "Вақти Науру"}, {"Nepal", "Вақти Непал"}, {"New_Caledonia", "Вақти стандартии Каледонияи Нав"}, {"New_Zealand", "Вақти стандартии Зеландияи Нав"}, {"Newfoundland", "Вақти стандартии Нюфаундленд"}, {"Niue", "Вақти Ниуэ"}, {"Norfolk", "Вақти стандартии ҷазираи Норфолк"}, {"Noronha", "Вақти стандартии Фернандо де Норонха"}, {"Novosibirsk", "Вақти стандартии Новосибирск"}, {"Omsk", "Омск вақти стандартӣ"}, {"Pakistan", "Вақти стандартии Покистон"}, {"Palau", "Вақти Палау"}, {"Papua_New_Guinea", "Вақти Папуа Гвинеяи Нав"}, {"Paraguay", "Вақти стандартии Парагвай"}, {"Peru", "Вақти стандартии Перу"}, {"Philippines", "Вақти стандартии Филиппин"}, {"Phoenix_Islands", "Вақти Ҷазираҳои Финикс"}, {"Pierre_Miquelon", "Вақти стандартии Сент-Пиер ва Микелон"}, {"Pitcairn", "Вақти Питкэрн"}, {"Ponape", "Ponape Time"}, {"Pyongyang", "Вақти Пхенян"}, {"Reunion", "Вақти Реюнион"}, {"Rothera", "Вақти Ротера"}, {"Sakhalin", "Вақти стандартии Сахалин"}, {"Samoa", "Вақти стандартии Самоа"}, {"Seychelles", "Вақти Сейшел"}, {"Singapore", "Вақти стандартии Сингапур"}, {"Solomon", "Вақти Ҷазираҳои Соломон"}, {"South_Georgia", "Вақти Ҷорҷияи Ҷанубӣ"}, {"Suriname", "Вақти Суринам"}, {"Syowa", "Вақти Сёва"}, {"Tahiti", "Вақти Таити"}, {"Taipei", "Вақти стандартии Тайбэй"}, {"Tajikistan", "Вақти Тоҷикистон"}, {"Tokelau", "Вақти Токелау"}, {"Tonga", "Вақти стандартии Тонга"}, {"Truk", "Вақти Чук"}, {"Turkmenistan", "Вақти стандартии Туркманистон"}, {"Tuvalu", "Вақти Тувалу"}, {"Uruguay", "Вақти стандартии Уругвай"}, {"Uzbekistan", "Вақти стандартии Ӯзбекистон"}, {"Vanuatu", "Вақти стандартии Вануату"}, {"Venezuela", "Вақти Венесуэла"}, {"Vladivostok", "Вақти стандартии Владивосток"}, {"Volgograd", "Вақти стандартии Волгоград"}, {"Vostok", "Вақти Восток"}, {"Wake", "Вақти бедории ҷазира"}, {"Wallis", "Вақти Уоллис ва Футуна"}, {"Yakutsk", "Якутск вақти стандартӣ"}, {"Yekaterinburg", "Вақти стандартии Екатеринбург"}, {"Yukon", "Вақти Юкон"}},
		{{"Alaska", "Вақти рӯзонаи Аляска"}, {"Amazon", "Вақти тобистонаи Амазон"}, {"America_Central", "Вақти рӯзонаи марказӣ"}, {"America_Eastern", "Вақти рӯзонаи шарқӣ"}, {"America_Mountain", "Вақти рӯзонаи кӯҳӣ"}, {"America_Pacific", "Вақти рӯзонаи Уқёнуси Ором"}, {"Apia", "Вақти рӯзонаи Апиа"}, {"Arabian", "Вақти рӯзонаи Арабистон"}, {"Argentina", "Вақти тобистонаи Аргентина"}, {"Argentina_Western", "Вақти тобистонаи Аргентинаи Ғарбӣ"}, {"Armenia", "Вақти рӯзонаи Арманистон"}, {"Atlantic", "Вақти рӯзонаи атлантикӣ"}, {"Australia_Central", "Вақти рӯонаи Австралияи Марказӣ"}, {"Australia_CentralWestern", "Вақти рӯзонаи Ғарбии Австралия Марказӣ"}, {"Australia_Eastern", "Вақти рӯзонаи шарқии Австралия"}, {"Australia_Western", "Вақти рӯзонаи Австралияи Ғарбӣ"}, {"Azerbaijan", "Вақти тобистонаи Озарбойҷон"}, {"Azores", "Вақти тобистонаи Азор"}, {"Bangladesh", "Вақти тобистонаи Бангладеш"}, {"Brasilia", "Вақти тобистонаи Бразилия"}, {"Cape_Verde", "Вақти тобистонаи Кабо-Верде"}, {"Chatham", "Вақти рӯзонаи Чатам"}, {"Chile", "Вақти тобистонаи Чили"}, {"China", "Вақти рӯзонаи Чин"}, {"Colombia", "Вақти тобистонаи Колумбия"}, {"Cook", "Вақти нимаи тобистонаи Ҷазираҳои Кук"}, {"Cuba", "Вақти рӯзонаи Куба"}, {"Easter", "Вақти тобистонаи ҷазираи Пасха"}, {"Europe_Central", "Вақти тобистонаи аврупоии марказӣ"}, {"Europe_Eastern", "Вақти тобистонаи аврупоии шарқӣ"}, {"Europe_Western", "Вақти тобистонаи аврупоии ғарбӣ"}, {"Falkland", "Вақти тобистонаи Ҷазираҳои Фолкленд"}, {"Fiji", "Вақти тобистонаи Фиҷи"}, {"Georgia", "Вақти тобистонаи Гурҷистон"}, {"Greenland_Eastern", "Вақти тобистонаи Гренландияи Шарқӣ"}, {"Greenland_Western", "Вақти тобистонаи Гренландияи Ғарбӣ"}, {"Hawaii_Aleutian", "Вақти рӯзонаи Ҳавайӣ-Алеутӣ"}, {"Hong_Kong", "Вақти тобистонаи Ҳонконг"}, {"Hovd", "Вақти тобистонаи Ховд"}, {"Iran", "Вақти рӯзонаи Эрон"}, {"Irkutsk", "Вақти тобистонаи Иркутск"}, {"Israel", "Вақти рӯзонаи Исроил"}, {"Japan", "Вақти равшании Ҷопон"}, {"Korea", "Вақти рӯзонаи Корея"}, {"Krasnoyarsk", "Вақти тобистонаи Красноярск"}, {"Lord_Howe", "Вақти рӯзонаи Лорд Хоу"}, {"Magadan", "Вақти тобистонаи Магадан"}, {"Mauritius", "Вақти тобистонаи Маврикий"}, {"Mexico_Pacific", "Вақти рӯзонаи Уқёнуси Ором Мексика"}, {"Mongolia", "Вақти тобистонаи Улан-Батор"}, {"Moscow", "Вақти тобистонаи Маскав"}, {"New_Caledonia", "Вақти тобистонаи Каледонияи Нав"}, {"New_Zealand", "Вақти рӯзонаи Зеландияи Нав"}, {"Newfoundland", "Вақти рӯзонаи Нюфаундленд"}, {"Norfolk", "Вақти рӯзонаи ҷазираи Норфолк"}, {"Noronha", "Вақти тобистонаи Фернандо де Норонха"}, {"Novosibirsk", "Вақти тобистонаи Новосибирск"}, {"Omsk", "Вақти тобистонаи Омск"}, {"Pakistan", "Вақти тобистонаи Покистон"}, {"Paraguay", "Вақти тобистонаи Парагвай"}, {"Peru", "Вақти тобистонаи Перу"}, {"Philippines", "Вақти тобистонаи Филиппин"}, {"Pierre_Miquelon", "Вақти рӯзонаи Сент-Пиер ва Микелон"}, {"Sakhalin", "Вақти тобистонаи Сахалин"}, {"Samoa", "Вақти рӯзонаи Самоа"}, {"Taipei", "Вақти рӯзонаи Тайбэй"}, {"Tonga", "Вақти тобистонаи Тонга"}, {"Turkmenistan", "Вақти тобистонаи Туркманистон"}, {"Uruguay", "Вақти тобистонаи Уругвай"}, {"Uzbekistan", "Вақти тобистонаи Ӯзбекистон"}, {"Vanuatu", "Вақти тобистонаи Вануату"}, {"Vladivostok", "Вақти тобистонаи Владивосток"}, {"Volgograd", "Вақти тобистонаи Волгоград"}, {"Yakutsk", "Якутск вақти тобистона"}, {"Yekaterinburg", "Вақти тобистонаи Екатеринбург"}},
	},
	{
		{{"Acre", "เวลามาตรฐานอาเกร"}, {"Afghanistan", "เวลาอัฟกานิสถาน"}, {"Africa_Central", "เวลาแอฟริกากลาง"}, {"Africa_Eastern", "เวลาแอฟริกาตะวันออก"}, {"Africa_Southern", "เวลาแอฟริกาใต้"}, {"Africa_Western", "เวลาแอฟริกาตะวันตก"}, {"Alaska", "เวลามาตรฐานอะแลสกา"}, {"Almaty", "เวลามาตรฐานอัลมาตี"}, {"Amazon", "เวลามาตรฐานแอมะซอน"}, {"America_Central", "เวลามาตรฐานตอนกลางในอเมริกาเหนือ"}, {"America_Eastern", "เวลามาตรฐานทางตะวันออกในอเมริกาเหนือ"}, {"America_Mountain", "เวลามาตรฐานแถบภูเขาในอเมริกาเหนือ"}, {"America_Pacific", "เวลามาตรฐานแปซิฟิกในอเมริกาเหนือ"}, {"Anadyr", "เวลามาตรฐานอะนาดีร์"}, {"Apia", "เวลามาตรฐานอาปีอา"}, {"Aqtau", "เวลามาตรฐานอัคตาอู"}, {"Aqtobe", "เวลามาตรฐานอัคโทเบ"}, {"Arabian", "เวลามาตรฐานอาหรับ"}, {"Argentina", "เวลามาตรฐานอาร์เจนตินา"}, {"Argentina_Western", "เวลามาตรฐานทางตะวันตกของอาร์เจนตินา"}, {"Armenia", "เวลามาตรฐานอาร์เมเนีย"}, {"Atlantic", "เวลามาตรฐานแอตแลนติก"}, {"Australia_Central", "เวลามาตรฐานทางตอนกลางของออสเตรเลีย"}, {"Australia_CentralWestern", "เวลามาตรฐานทางตะวันตกตอนกลางของออสเตรเลีย"}, {"Australia_Eastern", "เวลามาตรฐานทางตะวันออกของออสเตรเลีย"}, {"Australia_Western", "เวลามาตรฐานทางตะวันตกของออสเตรเลีย"}, {"Azerbaijan", "เวลามาตรฐานอาเซอร์ไบจาน"}, {"Azores", "เวลามาตรฐานอะโซร์ส"}, {"Bangladesh", "เวลามาตรฐานบังกลาเทศ"}, {"Bhutan", "เวลาภูฏาน"}, {"Bolivia", "เวลาโบลิเวีย"}, {"Brasilia", "เวลามาตรฐานบราซิเลีย"}, {"Brunei", "เวลาบรูไนดารุสซาลาม"}, {"Cape_Verde", "เวลามาตรฐานเคปเวิร์ด"}, {"Casey", "เวลาเคซีย์"}, {"Chamorro", "เวลาชามอร์โร"}, {"Chatham", "เวลามาตรฐานแชทัม"}, {"Chile", "เวลามาตรฐานชิลี"}, {"China", "เวลามาตรฐานจีน"}, {"Christmas", "เวลาเกาะคริสต์มาส"}, {"Cocos", "เวลาหมู่เกาะโคโคส"}, {"Colombia", "เวลามาตรฐานโคลอมเบีย"}, {"Cook", "เวลามาตรฐานหมู่เกาะคุก"}, {"Cuba", "เวลามาตรฐานคิวบา"}, {"Davis", "เวลาเดวิส"}, {"DumontDUrville", "เวลาดูมองต์ดูร์วิลล์"}, {"East_Timor", "เวลาติมอร์ตะวันออก"}, {"Easter", "เวลามาตรฐานเกาะอีสเตอร์"}, {"Ecuador", "เวลาเอกวาดอร์"}, {"Europe_Central", "เวลามาตรฐานยุโรปกลาง"}, {"Europe_Eastern", "เวลามาตรฐานยุโรปตะวันออก"}, {"Europe_Further_Eastern", "เวลายุโรปตะวันออกไกล"}, {"Europe_Western", "เวลามาตรฐานยุโรปตะวันตก"}, {"Falkland", "เวลามาตรฐานหมู่เกาะฟอล์กแลนด์"}, {"Fiji", "เวลามาตรฐานฟิจิ"}, {"French_Guiana", "เวลาเฟรนช์เกียนา"}, {"French_Southern", "เวลาเฟรนช์เซาเทิร์นและแอนตาร์กติก"}, {"GMT", "เวลามาตรฐานกรีนิช"}, {"Galapagos", "เวลากาลาปาโกส"}, {"Gambier", "เวลาแกมเบียร์"}, {"Georgia", "เวลามาตรฐานจอร์เจีย"}, {"Gilbert_Islands", "เวลาหมู่เกาะกิลเบิร์ต"}, {"Greenland_Eastern", "เวลามาตรฐานกรีนแลนด์ตะวันออก"}, {"Greenland_Western", "เวลามาตรฐานกรีนแลนด์ตะวันตก"}, {"Guam", "เวลากวม"}, {"Gulf", "เวลากัลฟ์"}, {"Guyana", "เวลากายอานา"}, {"Hawaii", "เวลามาตรฐานฮาวาย-อะลูเชียน"}, {"Hawaii_Aleutian", "เวลามาตรฐานฮาวาย-อะลูเชียน"}, {"Hong_Kong", "เวลามาตรฐานฮ่องกง"}, {"Hovd", "เวลามาตรฐานฮอฟด์"}, {"India", "เวลาอินเดีย"}, {"Indian_Ocean", "เวลามหาสมุทรอินเดีย"}, {"Indochina", "เวลาอินโดจีน"}, {"Indonesia_Central", "เวลาอินโดนีเซียตอนกลาง"}, {"Indonesia_Eastern", "เวลาอินโดนีเซียฝั่งตะวันออก"}, {"Indonesia_Western", "เวลาอินโดนีเซียฝั่งตะวันตก"}, {"Iran", "เวลามาตรฐานอิหร่าน"}, {"Irkutsk", "เวลามาตรฐานอีร์คุตสค์"}, {"Israel", "เวลามาตรฐานอิสราเอล"}, {"Japan", "เวลามาตรฐานญี่ปุ่น"}, {"Kamchatka", "เวลาเปโตรปัฟลอฟสค์-คัมชัตสกี"}, {"Kazakhstan", "เวลาคาซัคสถาน"}, {"Kazakhstan_Eastern", "เวลาคาซัคสถานตะวันออก"}, {"Kazakhstan_Western", "เวลาคาซัคสถานตะวันตก"}, {"Korea", "เวลามาตรฐานเกาหลี"}, {"Kosrae", "เวลาคอสไร"}, {"Krasnoyarsk", "เวลามาตรฐานครัสโนยาสค์"}, {"Kyrgystan", "เวลาคีร์กีซสถาน"}, {"Lanka", "เวลาลังกา"}, {"Line_Islands", "เวลาหมู่เกาะไลน์"}, {"Lord_Howe", "เวลามาตรฐานลอร์ดโฮว์"}, {"Macau", "เวลามาตรฐานมาเก๊า"}, {"Magadan", "เวลามาตรฐานมากาดาน"}, {"Malaysia", "เวลามาเลเซีย"}, {"Maldives", "เวลามัลดีฟส์"}, {"Marquesas", "เวลามาร์เคซัส"}, {"Marshall_Islands", "เวลาหมู่เกาะมาร์แชลล์"}, {"Mauritius", "เวลามาตรฐานมอริเชียส"}, {"Mawson", "เวลามอว์สัน"}, {"Mexico_Pacific", "เวลามาตรฐานแปซิฟิกเม็กซิโก"}, {"Mongolia", "เวลามาตรฐานอูลานบาตอร์"}, {"Moscow", "เวลามาตรฐานมอสโก"}, {"Myanmar", "เวลาพม่า"}, {"Nauru", "เวลานาอูรู"}, {"Nepal", "เวลาเนปาล"}, {"New_Caledonia", "เวลามาตรฐานนิวแคลิโดเนีย"}, {"New_Zealand", "เวลามาตรฐานนิวซีแลนด์"}, {"Newfoundland", "เวลามาตรฐานนิวฟันด์แลนด์"}, {"Niue", "เวลานีอูเอ"}, {"Norfolk", "เวลามาตรฐานเกาะนอร์ฟอล์ก"}, {"Noronha", "เวลามาตรฐานหมู่เกาะเฟอร์นันโด"}, {"North_Mariana", "เวลาหมู่เกาะมาเรียนาเหนือ"}, {"Novosibirsk", "เวลามาตรฐานโนโวซีบีสค์"}, {"Omsk", "เวลามาตรฐานออมสค์"}, {"Pakistan", "เวลามาตรฐานปากีสถาน"}, {"Palau", "เวลาปาเลา"}, {"Papua_New_Guinea", "เวลาปาปัวนิวกินี"}, {"Paraguay", "เวลามาตรฐานปารากวัย"}, {"Peru", "เวลามาตรฐานเปรู"}, {"Philippines", "เวลามาตรฐานฟิลิปปินส์"}, {"Phoenix_Islands", "เวลาหมู่เกาะฟินิกซ์"}, {"Pierre_Miquelon", "เวลามาตรฐานแซงปีแยร์และมีเกอลง"}, {"Pitcairn", "เวลาพิตแคร์น"}, {"Ponape", "เวลาโปนาเป"}, {"Pyongyang", "เวลาเปียงยาง"}, {"Qyzylorda", "เวลามาตรฐานคืยซิลออร์ดา"}, {"Reunion", "เวลาเรอูนียง"}, {"Rothera", "เวลาโรธีรา"}, {"Sakhalin", "เวลามาตรฐานซาคาลิน"}, {"Samara", "เวลามาตรฐานซามารา"}, {"Samoa", "เวลามาตรฐานซามัว"}, {"Seychelles", "เวลาเซเชลส์"}, {"Singapore", "เวลาสิงคโปร์"}, {"Solomon", "เวลาหมู่เกาะโซโลมอน"}, {"South_Georgia", "เวลาเซาท์จอร์เจีย"}, {"Suriname", "เวลาซูรินาเม"}, {"Syowa", "เวลาไซโยวา"}, {"Tahiti", "เวลาตาฮีตี"}, {"Taipei", "เวลามาตรฐานไทเป"}, {"Tajikistan", "เวลาทาจิกิสถาน"}, {"Tokelau", "เวลาโตเกเลา"}, {"Tonga", "เวลามาตรฐานตองกา"}, {"Truk", "เวลาชุก"}, {"Turkey", "เวลามาตรฐานตุรกี"}, {"Turkmenistan", "เวลามาตรฐานเติร์กเมนิสถาน"}, {"Tuvalu", "เวลาตูวาลู"}, {"Uruguay", "เวลามาตรฐานอุรุกวัย"}, {"Uzbekistan", "เวลามาตรฐานอุซเบกิสถาน"}, {"Vanuatu", "เวลามาตรฐานวานูอาตู"}, {"Venezuela", "เวลาเวเนซุเอลา"}, {"Vladivostok", "เวลามาตรฐานวลาดีวอสตอค"}, {"Volgograd", "เวลามาตรฐานวอลโกกราด"}, {"Vostok", "เวลาวอสตอค"}, {"Wake", "เวลาเกาะเวก"}, {"Wallis", "เวลาวาลลิสและฟุตูนา"}, {"Yakutsk", "เวลามาตรฐานยาคุตสค์"}, {"Yekaterinburg", "เวลามาตรฐานเยคาเตรินบูร์ก"}, {"Yukon", "เวลายูคอน"}},
		{{"Acre", "เวลาฤดูร้อนอาเกร"}, {"Alaska", "เวลาออมแสงของอะแลสกา"}, {"Almaty", "เวลาฤดูร้อนอัลมาตี"}, {"Amazon", "เวลาฤดูร้อนแอมะซอน"}, {"America_Central", "เวลาออมแสงตอนกลางในอเมริกาเหนือ"}, {"America_Eastern", "เวลาออมแสงทางตะวันออกในอเมริกาเหนือ"}, {"America_Mountain", "เวลาออมแสงแถบภูเขาในอเมริกาเหนือ"}, {"America_Pacific", "เวลาออมแสงแปซิฟิกในอเมริกาเหนือ"}, {"Anadyr", "เวลาฤดูร้อนอะนาดีร์"}, {"Apia", "เวลาออมแสงอาปีอา"}, {"Aqtau", "เวลาฤดูร้อนอัคตาอู"}, {"Aqtobe", "เวลาฤดูร้อนอัคโทเบ"}, {"Arabian", "เวลาออมแสงอาหรับ"}, {"Argentina", "เวลาฤดูร้อนอาร์เจนตินา"}, {"Argentina_Western", "เวลาฤดูร้อนทางตะวันตกของอาร์เจนตินา"}, {"Armenia", "เวลาฤดูร้อนอาร์เมเนีย"}, {"Atlantic", "เวลาออมแสงของแอตแลนติก"}, {"Australia_Central", "เวลาออมแสงทางตอนกลางของออสเตรเลีย"}, {"Australia_CentralWestern", "เวลาออมแสงทางตะวันตกตอนกลางของออสเตรเลีย"}, {"Australia_Eastern", "เวลาออมแสงทางตะวันออกของออสเตรเลีย"}, {"Australia_Western", "เวลาออมแสงทางตะวันตกของออสเตรเลีย"}, {"Azerbaijan", "เวลาฤดูร้อนอาเซอร์ไบจาน"}, {"Azores", "เวลาฤดูร้อนอะโซร์ส"}, {"Bangladesh", "เวลาฤดูร้อนบังกลาเทศ"}, {"Brasilia", "เวลาฤดูร้อนบราซิเลีย"}, {"Cape_Verde", "เวลาฤดูร้อนเคปเวิร์ด"}, {"Chatham", "เวลาออมแสงแชทัม"}, {"Chile", "เวลาฤดูร้อนชิลี"}, {"China", "เวลาออมแสงจีน"}, {"Colombia", "เวลาฤดูร้อนโคลอมเบีย"}, {"Cook", "เวลาครึ่งฤดูร้อนหมู่เกาะคุก"}, {"Cuba", "เวลาออมแสงของคิวบา"}, {"Easter", "เวลาฤดูร้อนเกาะอีสเตอร์"}, {"Europe_Central", "เวลาฤดูร้อนยุโรปกลาง"}, {"Europe_Eastern", "เวลาฤดูร้อนยุโรปตะวันออก"}, {"Europe_Western", "เวลาฤดูร้อนยุโรปตะวันตก"}, {"Falkland", "เวลาฤดูร้อนหมู่เกาะฟอล์กแลนด์"}, {"Fiji", "เวลาฤดูร้อนฟิจิ"}, {"Georgia", "เวลาฤดูร้อนจอร์เจีย"}, {"Greenland_Eastern", "เวลาฤดูร้อนกรีนแลนด์ตะวันออก"}, {"Greenland_Western", "เวลาฤดูร้อนกรีนแลนด์ตะวันตก"}, {"Hawaii_Aleutian", "เวลาออมแสงฮาวาย-อะลูเชียน"}, {"Hong_Kong", "เวลาฤดูร้อนฮ่องกง"}, {"Hovd", "เวลาฤดูร้อนฮอฟด์"}, {"Iran", "เวลาออมแสงอิหร่าน"}, {"Irkutsk", "เวลาฤดูร้อนอีร์คุตสค์"}, {"Israel", "เวลาออมแสงอิสราเอล"}, {"Japan", "เวลาออมแสงญี่ปุ่น"}, {"Kamchatka", "เวลาฤดูร้อนเปโตรปัฟลอฟสค์-คัมชัตสกี"}, {"Korea", "เวลาออมแสงเกาหลี"}, {"Krasnoyarsk", "เวลาฤดูร้อนครัสโนยาสค์"}, {"Lord_Howe", "เวลาออมแสงลอร์ดโฮว์"}, {"Macau", "เวลาฤดูร้อนมาเก๊า"}, {"Magadan", "เวลาฤดูร้อนมากาดาน"}, {"Mauritius", "เวลาฤดูร้อนของมอริเชียส"}, {"Mexico_Pacific", "เวลาออมแสงแปซิฟิกเม็กซิโก"}, {"Mongolia", "เวลาฤดูร้อนอูลานบาตอร์"}, {"Moscow", "เวลาฤดูร้อนมอสโก"}, {"New_Caledonia", "เวลาฤดูร้อนนิวแคลิโดเนีย"}, {"New_Zealand", "เวลาออมแสงนิวซีแลนด์"}, {"Newfoundland", "เวลาออมแสงนิวฟันด์แลนด์"}, {"Norfolk", "เวลาออมแสงเกาะนอร์ฟอล์ก"}, {"Noronha", "เวลาฤดูร้อนของหมู่เกาะเฟอร์นันโด"}, {"Novosibirsk", "เวลาฤดูร้อนโนโวซีบีสค์"}, {"Omsk", "เวลาฤดูร้อนออมสค์"}, {"Pakistan", "เวลาฤดูร้อนปากีสถาน"}, {"Paraguay", "เวลาฤดูร้อนปารากวัย"}, {"Peru", "เวลาฤดูร้อนเปรู"}, {"Philippines", "เวลาฤดูร้อนฟิลิปปินส์"}, {"Pierre_Miquelon", "เวลาออมแสงของแซงปีแยร์และมีเกอลง"}, {"Qyzylorda", "เวลาฤดูร้อนคืยซิลออร์ดา"}, {"Sakhalin", "เวลาฤดูร้อนซาคาลิน"}, {"Samara", "เวลาฤดูร้อนซามารา"}, {"Samoa", "เวลาฤดูร้อนซามัว"}, {"Taipei", "เวลาออมแสงไทเป"}, {"Tonga", "เวลาฤดูร้อนตองกา"}, {"Turkey", "เวลาฤดูร้อนตุรกี"}, {"Turkmenistan", "เวลาฤดูร้อนเติร์กเมนิสถาน"}, {"Uruguay", "เวลาฤดูร้อนอุรุกวัย"}, {"Uzbekistan", "เวลาฤดูร้อนอุซเบกิสถาน"}, {"Vanuatu", "เวลาฤดูร้อนวานูอาตู"}, {"Vladivostok", "เวลาฤดูร้อนวลาดีวอสตอค"}, {"Volgograd", "เวลาฤดูร้อนวอลโกกราด"}, {"Yakutsk", "เวลาฤดูร้อนยาคุตสค์"}, {"Yekaterinburg", "เวลาฤดูร้อนเยคาเตรินบูร์ก"}},
	},
	{
		{{"Acre", "ናይ መደበኛ ግዘ ኣክሪ"}, {"Afghanistan", "ናይ አፍጋኒስታን ግዘ"}, {"Africa_Central", "ግዜ ማእከላይ ኣፍሪቃ"}, {"Africa_Eastern", "ግዜ ምብራቕ ኣፍሪቃ"}, {"Africa_Southern", "ግዜ ደቡብ ኣፍሪቃ"}, {"Africa_Western", "ግዜ ምዕራብ ኣፍሪቃ"}, {"Alaska", "ናይ መደበኛ ግዘ አላስካ"}, {"Amazon", "ናይ መደበኛ ግዘ ኣማዞን"}, {"America_Central", "ናይ መደበኛ ግዘ ማእከላይ አመሪካ"}, {"America_Eastern", "ናይ መደበኛ ግዘ ምብራቓዊ ኣመሪካ"}, {"America_Mountain", "ናይ መደበኛ ግዘ ጎቦ ኣመሪካ"}, {"America_Pacific", "ናይ መደበኛ ግዘ ፓስፊክ"}, {"Apia", "ናይ መደበኛ አፒያ ግዘ"}, {"Arabian", "ናይ መደበኛ አረብ ግዘ"}, {"Argentina", "ናይ መደበኛ ግዘ ኣርጀንቲና"}, {"Argentina_Western", "ናይ መደበኛ ግዘ ምዕራባዊ ኣርጀንቲና"}, {"Armenia", "ናይ መደበኛ አርሜኒያ ግዘ"}, {"Atlantic", "ናይ መደበኛ ግዘ አትላንቲክ"}, {"Australia_Central", "ናይ ማዕከላይ መደበኛ አውስራሊያ ግዘ"}, {"Australia_CentralWestern", "ናይ ምዕራባዊ መደበኛ አውስራሊያ ግዘ"}, {"Australia_Eastern", "ናይ ምብራቓዊ መደበኛ ኣውስትራልያ ግዘ"}, {"Australia_Western", "ናይ ምዕራባዊ መደበኛ አውስትራሊያ ግዘ"}, {"Azerbaijan", "ናይ መደበኛ አዘርባዣን ግዘ"}, {"Azores", "ናይ መደበኛ ግዘ ኣዞረስ"}, {"Bangladesh", "ናይ መደበኛ ባንግላዲሽ ግዘ"}, {"Bhutan", "ናይ ቡህታን ግዘ"}, {"Bolivia", "ግዜ ቦሊቭያ"}, {"Brasilia", "ናይ መደበኛ ግዘ ብራዚልያ"}, {"Brunei", "ናይ ብሩኔ ዳሩሳሌም ግዘ"}, {"Cape_Verde", "ናይ መደበኛ ግዘ ኬፕ ቨርደ"}, {"Chamorro", "ናይ መደበኛ ቻሞሮ ግዘ"}, {"Chatham", "ናይ መደበኛ ቻትሃም ግዘ"}, {"Chile", "ናይ መደበኛ ግዘ ቺሌ"}, {"China", "ናይ መደበኛ ቻይና ግዘ"}, {"Christmas", "ናይ ልደት ደሴት ግዘ"}, {"Cocos", "ናይ ኮኮስ ደሴት ግዘ"}, {"Colombia", "ናይ መደበኛ ግዘ ኮሎምብያ"}, {"Cook", "ናይ መደበኛ ኩክ ደሴት ግዘ"}, {"Cuba", "ናይ መደበኛ ግዘ ኩባ"}, {"Davis", "ናይ ዴቪስ ግዘ"}, {"DumontDUrville", "ናይ ዱሞ-ዱርቪል ግዘ"}, {"East_Timor", "ናይ ምብራቅ ቲሞር ግዘ"}, {"Easter", "ናይ መደበኛ ግዘ ደሴት ፋሲካ"}, {"Ecuador", "ግዜ ኤኳዶር"}, {"Europe_Central", "ናይ መደበኛ ግዘ ማእከላይ ኤውሮጳ"}, {"Europe_Eastern", "ናይ መደበኛ ግዘ ምብራቕ ኤውሮጳ"}, {"Europe_Further_Eastern", "ናይ ርሑቕ-ምብራቕ ኤውሮጳዊ ግዘ"}, {"Europe_Western", "ናይ መደበኛ ምዕራባዊ ኤውሮጳዊ ግዘ"}, {"Falkland", "ናይ መደበኛ ግዘ ደሴታት ፎክላንድ"}, {"Fiji", "ናይ መደበኛ ፊጂ ግዘ"}, {"French_Guiana", "ግዜ ፈረንሳዊት ጊያና"}, {"French_Southern", "ናይ ደቡባዊን ኣንታርቲክ ግዘ"}, {"GMT", "GMT"}, {"Galapagos", "ግዜ ጋላፓጎስ"}, {"Gambier", "ናይ ጋምቢየር ግዘ"}, {"Georgia", "ናይ መደበኛ ጆርጂያ ግዘ"}, {"Gilbert_Islands", "ናይ ጊልበርት ደሴታት ግዘ"}, {"Greenland_Eastern", "ናይ መደበኛ ምብራቓዊ ግዘ ግሪንላንድ"}, {"Greenland_Western", "ናይ መደበኛ ምዕራብ ግሪንላንድ ግዘ"}, {"Gulf", "ናይ መደበኛ ገልፍ ግዘ"}, {"Guyana", "ግዜ ጉያና"}, {"Hawaii", "ናይ መደበኛ ሃዋይ-ኣሌውቲያን ግዘ"}, {"Hawaii_Aleutian", "ናይ መደበኛ ሃዋይ-ኣሌውቲያን ግዘ"}, {"Hong_Kong", "ናይ መደበኛ ሆንግ ኮንግ ግዘ"}, {"Hovd", "ናይ መደበኛ ሆቭድ ግዘ"}, {"India", "ናይ መደበኛ ህንድ ግዘ"}, {"Indian_Ocean", "ግዜ ህንዳዊ ውቅያኖስ"}, {"Indochina", "ናይ ኢንዶቻይና ግዘ"}, {"Indonesia_Central", "ናይ ማእከላይ ኢንዶነዥያ ግዘ"}, {"Indonesia_Eastern", "ናይ ምብራቓዊ ኢንዶነዥያ ግዘ"}, {"Indonesia_Western", "ናይ ምዕራባዊ ኢንዶነዥያ ግዘ"}, {"Iran", "ናይ መደበኛ ኢራን ግዘ"}, {"Irkutsk", "ናይ መደበኛ ኢርኩትስክ ግዘ"}, {"Israel", "ናይ መደበኛ እስራኤል ግዘ"}, {"Japan", "ናይ መደበኛ ጃፓን ግዘ"}, {"Kazakhstan", "ናይ ካዛኪስታን ግዘ"}, {"Kazakhstan_Eastern", "ናይ ምብራቅ ካዛኪስታን ግዘ"}, {"Kazakhstan_Western", "ናይ ምዕራብ ካዛኪስታን ግዘ"}, {"Korea", "ናይ መደበኛ ኮሪያን ግዘ"}, {"Kosrae", "ናይ ኮርሳይ ግዘ"}, {"Krasnoyarsk", "ናይ መደበኛ ክራንስኖያርክ ግዘ"}, {"Kyrgystan", "ናይ ክርጅስታን ግዘ"}, {"Line_Islands", "ናይ ላይን ደሴታት ግዘ"}, {"Lord_Howe", "ናይ መድበኛ ሎርድ ሆው ግዘ"}, {"Magadan", "ናይ መደበኛ ሜጋዳን ግዘ"}, {"Malaysia", "ናይ ማሌዢያ ግዘ"}, {"Maldives", "ናይ ሞልዲቭስ ግዘ"}, {"Marquesas", "ናይ ማርኩዌሳስ ግዘ"}, {"Marshall_Islands", "ናይ ማርሻል ደሴታት ግዘ"}, {"Mauritius", "ናይ መደበኛ ግዘ ማውሪሸስ"}, {"Mawson", "ናይ ማውሶን ግዘ"}, {"Mexico_Pacific", "ናይ መደበኛ ሜክሲኮ ፓስፊክ ግዘ"}, {"Mongolia", "ናይ መደበኛ ኡላንባታር ግዘ"}, {"Moscow", "ናይ መደበኛ ሞስኮው ግዘ"}, {"Myanmar", "ናይ ምያንማር ግዘ"}, {"Nauru", "ናይ ናውሩ ግዘ"}, {"Nepal", "ናይ ኔፓል ግዘ"}, {"New_Caledonia", "ናይ መደበኛ ኒው ካሌዶኒያ ግዘ"}, {"New_Zealand", "ናይ መደበኛ ኒው ዚላንድ ግዘ"}, {"Newfoundland", "ናይ መደበኛ ኒውፋውንድላንድ ግዘ"}, {"Niue", "ናይ ኒዌ ግዘ"}, {"Norfolk", "ናይ መደበኛ ኖርፎልክ ደሴት ግዘ"}, {"Noronha", "ናይ መደበኛ ግዘ ፈርናንዶ ደ ኖሮንያ"}, {"Novosibirsk", "ናይ መደበኛ ኖቮሲሪስክ ግዘ"}, {"Omsk", "ናይ መደበኛ ኦምስክ ግዘ"}, {"Pakistan", "ናይ መደበኛ ፓኪስታን ግዘ"}, {"Palau", "ናይ ፓላው ግዘ"}, {"Papua_New_Guinea", "ናይ ፓፗ ኒው ጊኒ ግዘ"}, {"Paraguay", "ናይ መደበኛ ግዘ ፓራጓይ"}, {"Peru", "ናይ መደበኛ ግዘ ፔሩ"}, {"Philippines", "ናይ መደበኛ ፊሊፒን ግዘ"}, {"Phoenix_Islands", "ናይ ፊኒክስ ደሴታት ግዘ"}, {"Pierre_Miquelon", "ናይ መደበኛ ቅዱስ ፒየርን ሚከሎን ግዘ"}, {"Pitcairn", "ናይ ፒትቻይርን ግዘ"}, {"Ponape", "ናይ ፖናፔ ግዘ"}, {"Pyongyang", "ናይ ፕዮንግያንግ ግዘ"}, {"Reunion", "ግዜ ርዩንየን"}, {"Rothera", "ናይ ሮቴራ ግዘ"}, {"Sakhalin", "ናይ መደበኛ ሳክሃሊን ግዘ"}, {"Samoa", "ናይ መደበኛ ሳሞዋ ግዘ"}, {"Seychelles", "ግዜ ሲሸልስ"}, {"Singapore", "ናይ መደበኛ ሲጋፖር ግዘ"}, {"Solomon", "ናይ ሶሎሞን ደሴታት ግዘ"}, {"South_Georgia", "ግዜ ደቡብ ጆርጅያ"}, {"Suriname", "ግዜ ሱሪናም"}, {"Syowa", "ናይ ስዮዋ ግዘ"}, {"Tahiti", "ናይ ቲሂቲ ግዘ"}, {"Taipei", "ናይ መደበኛ ቴፒ ግዘ"}, {"Tajikistan", "ናይ ታጃክስታን ግዘ"}, {"Tokelau", "ናይ ቶኬላው ግዘ"}, {"Tonga", "ናይ መደበኛ ቶንጋ ግዘ"}, {"Truk", "ናይ ቹክ ግዘ"}, {"Turkmenistan", "ናይ መደበኛ ቱርክሜኒስታን ግዘ"}, {"Tuvalu", "ናይ ቱቫሉ ግዘ"}, {"Uruguay", "ናይ መደበኛ ግዘ ኡራጓይ"}, {"Uzbekistan", "ናይ መደበኛ ኡዝቤኪስታን ግዘ"}, {"Vanuatu", "ናይ መደበኛ ቫኗታው ግዘ"}, {"Venezuela", "ግዜ ቬኔዝዌላ"}, {"Vladivostok", "ናይ መደበኛ ቭላዲቮስቶክ ግዘ"}, {"Volgograd", "ናይ መደበኛ ቮልጎግራድ ግዘ"}, {"Vostok", "ናይ ቮስቶክ ግዘ"}, {"Wake", "ናይ ዌክ ደሴት ግዘ"}, {"Wallis", "ናይ ዌልስን ፉቷ ግዘ"}, {"Yakutsk", "ናይ መደበኛ ያኩትስክ ግዘ"}, {"Yekaterinburg", "ናይ መደበኛ ያክተርኒበርግ ግዘ"}, {"Yukon", "ናይ ዩኮን ግዘ"}},
		{{"Acre", "ግዜ ክረምቲ ኣክሪ"}, {"Alaska", "ናይ መዓልቲ ግዘ አላስካ"}, {"Amazon", "ግዜ ክረምቲ ኣማዞን"}, {"America_Central", "ናይ መዓልቲ ግዘ ማእከላይ አመሪካ"}, {"America_Eastern", "ናይ መዓልቲ ግዘ ምብራቓዊ አመሪካ"}, {"America_Mountain", "ናይ መዓልቲ ግዘ ጎቦ አመሪካ"}, {"America_Pacific", "ናይ መዓልቲ ግዘ ፓስፊክ"}, {"Apia", "ናይ መዓልቲ አፒያ ግዘ"}, {"Arabian", "ናይ መዓልቲ አረብ ግዘ"}, {"Argentina", "ግዜ ክረምቲ ኣርጀንቲና"}, {"Argentina_Western", "ግዜ ክረምቲ ምዕራባዊ ኣርጀንቲና"}, {"Armenia", "ናይ ክረምቲ አርሜኒያ ግዘ"}, {"Atlantic", "ናይ መዓልቲ ግዘ አትላንቲክ"}, {"Australia_Central", "ናይ ማዕከላይ መዓልቲ አውስራሊያ ግዘ"}, {"Australia_CentralWestern", "ናይ ምዕራባዊ መዓልቲ አውስራሊያ ግዘ"}, {"Australia_Eastern", "ናይ ምብራቓዊ መዓልቲ ኣውስትራልያ ግዘ"}, {"Australia_Western", "ናይ ምዕራባዊ መዓልቲ አውስትራሊያ ግዘ"}, {"Azerbaijan", "ናይ ክረምቲ አዘርባዣን ግዘ"}, {"Azores", "ናይ ክረምቲ አዞረስ ግዘ"}, {"Bangladesh", "ናይ ክረምቲ ባንግላዲሽ ግዘ"}, {"Brasilia", "ግዜ ክረምቲ ብራዚልያ"}, {"Cape_Verde", "ግዜ ክረምቲ ኬፕ ቨርደ"}, {"Chatham", "ናይ መዓልቲ ቻትሃም ግዘ"}, {"Chile", "ግዜ ክረምቲ ቺሌ"}, {"China", "ናይ መዓልቲ ቻይና ግዘ"}, {"Colombia", "ግዜ ክረምቲ ኮሎምብያ"}, {"Cook", "ናይ ፍርቂ ክረምቲ ኩክ ደሴት ግዘ"}, {"Cuba", "ናይ መዓልቲ ግዘ ኩባ"}, {"Easter", "ግዜ ክረምቲ ደሴት ፋሲካ"}, {"Europe_Central", "ግዜ ክረምቲ ኤውሮጳ"}, {"Europe_Eastern", "ግዜ ክረምቲ ምብራቕ ኤውሮጳ"}, {"Europe_Western", "ናይ ክረምቲ ምዕራባዊ ኤውሮጳዊ ግዘ"}, {"Falkland", "ግዜ ከረምቲ ደሴታት ፎክላንድ"}, {"Fiji", "ናይ ክረምቲ ፊጂ ግዘ"}, {"Georgia", "ናይ ክረምቲ ጆርጂያ ግዘ"}, {"Greenland_Eastern", "ናይ መዓልቲ ምብራቓዊ ግዘ ግሪንላንድ"}, {"Greenland_Western", "ናይ መዓልቲ ምዕራብ ግሪንላንድ ግዘ"}, {"Hawaii_Aleutian", "ናይ መዓልቲ ሃዋይ-ኣሌውቲያን ግዘ"}, {"Hong_Kong", "ናይ ክረምቲ ሆንግ ኮንግ ግዘ"}, {"Hovd", "ናይ ክረምቲ ሆቭድ ግዘ"}, {"Iran", "ናይ መዓልቲ ኢራን ግዘ"}, {"Irkutsk", "ናይ ክረምቲ ኢርኩትስክ ግዘ"}, {"Israel", "ናይ መዓልቲ እስራኤል ግዘ"}, {"Japan", "ናይ መዓልቲ ጃፓን ግዘ"}, {"Korea", "ናይ መዓልቲ ኮሪያን ግዘ"}, {"Krasnoyarsk", "ናይ ክረምቲ ክራንስኖያርክ ግዘ"}, {"Lord_Howe", "ናይ መዓልቲ ሎርድ ሆው ግዘ"}, {"Magadan", "ናይ ክረምቲ ሜጋዳን ግዘ"}, {"Mauritius", "ግዜ ክረምቲ ማውሪሸስ"}, {"Mexico_Pacific", "ናይ መዓልቲ ሜክሲኮ ፓስፊክ ግዘ"}, {"Mongolia", "ናይ ክረምቲ ኡላንባታር ግዘ"}, {"Moscow", "ናይ ክረምቲ ሞስኮው ግዘ"}, {"New_Caledonia", "ናይ ክረምቲ ኒው ካሌዶኒያ ግዘ"}, {"New_Zealand", "ናይ መዓልቲ ኒው ዚላንድ ግዘ"}, {"Newfoundland", "ናይ መዓልቲ ኒውፋውንድላንድ ግዘ"}, {"Norfolk", "ናይ መዓልቲ ኖርፎልክ ደሴት ግዘ"}, {"Noronha", "ግዜ ክረምቲ ፈርናንዶ ደ ኖሮንያ"}, {"Novosibirsk", "ናይ ክረምቲ ኖቮሲሪስክ ግዘ"}, {"Omsk", "ናይ ክረምቲ ኦምስክ ግዘ"}, {"Pakistan", "ናይ ክረምቲ ፓኪስታን ግዘ"}, {"Paraguay", "ግዜ ክረምቲ ፓራጓይ"}, {"Peru", "ግዜ ክረምቲ ፔሩ"}, {"Philippines", "ናይ ክረምቲ ፊሊፒን ግዘ"}, {"Pierre_Miquelon", "ናይ መዓልቲ ቅዱስ ፒየርን ሚከሎን ግዘ"}, {"Sakhalin", "ናይ ክረምቲ ሳክሃሊን ግዘ"}, {"Samoa", "ናይ መዓልቲ ሳሞዋ ግዘ"}, {"Taipei", "ናይ መዓልቲ ቴፒ ግዘ"}, {"Tonga", "ናይ ክረምቲ ቶንጋ ግዘ"}, {"Turkmenistan", "ናይ ክረምቲ ቱርክሜኒስታን ግዘ"}, {"Uruguay", "ግዜ ክረምቲ ኡራጓይ"}, {"Uzbekistan", "ናይ ክረምቲ ኡዝቤኪስታን ግዘ"}, {"Vanuatu", "ናይ ክረምቲ ቫኗታው ግዘ"}, {"Vladivostok", "ናይ ክረምቲ ቭላዲቮስቶክ ግዘ"}, {"Volgograd", "ናይ ክረምቲ ቮልጎግራድ ግዘ"}, {"Yakutsk", "ናይ ክረምቲ ያኩትስክ ግዘ"}, {"Yekaterinburg", "ናይ ክረምቲ ያክተርኒበርግ ግዘ"}},
	},
	{
		{{"Acre", "Akri standart wagty"}, {"Afghanistan", "Owganystan wagty"}, {"Africa_Central", "Merkezi Afrika wagty"}, {"Africa_Eastern", "Gündogar Afrika wagty"}, {"Africa_Southern", "Günorta Afrika standart wagty"}, {"Africa_Western", "Günbatar Afrika wagty"}, {"Alaska", "Alýaska standart wagty"}, {"Amazon", "Amazon standart wagty"}, {"America_Central", "Merkezi Amerika standart wagty"}, {"America_Eastern", "Demirgazyk Amerika gündogar standart wagty"}, {"America_Mountain", "Demirgazyk Amerika dag standart wagty"}, {"America_Pacific", "Demirgazyk Amerika Ýuwaş umman standart wagty"}, {"Apia", "Apia standart wagty"}, {"Arabian", "Arap ýurtlary standart wagty"}, {"Argentina", "Argentina standart wagty"}, {"Argentina_Western", "Günbatar Argentina standart wagty"}, {"Armenia", "Ermenistan standart wagty"}, {"Atlantic", "Atlantik standart wagty"}, {"Australia_Central", "Merkezi Awstraliýa standart wagty"}, {"Australia_CentralWestern", "Merkezi Awstraliýa günbatar standart wagty"}, {"Australia_Eastern", "Gündogar Awstraliýa standart wagty"}, {"Australia_Western", "Günbatar Awstraliýa standart wagty"}, {"Azerbaijan", "Azerbaýjan standart wagty"}, {"Azores", "Azor adalary standart wagty"}, {"Bangladesh", "Bangladeş standart wagty"}, {"Bhutan", "Butan wagty"}, {"Bolivia", "Boliwiýa wagty"}, {"Brasilia", "Braziliýa standart wagty"}, {"Brunei", "Bruneý-Darussalam wagty"}, {"Cape_Verde", "Kabo-Werde standart wagty"}, {"Chamorro", "Çamorro wagty"}, {"Chatham", "Çatem standart wagty"}, {"Chile", "Çili standart wagty"}, {"China", "Hytaý standart wagty"}, {"Christmas", "Roždestwo adasy wagty"}, {"Cocos", "Kokos adalary wagty"}, {"Colombia", "Kolumbiýa standart wagty"}, {"Cook", "Kuk adalary standart wagty"}, {"Cuba", "Kuba standart wagty"}, {"Davis", "Deýwis wagty"}, {"DumontDUrville", "Dýumon-d-Ýurwil wagty"}, {"East_Timor", "Gündogar Timor wagty"}, {"Easter", "Pasha adasy standart wagty"}, {"Ecuador", "Ekwador wagty"}, {"Europe_Central", "Merkezi Ýewropa standart wagty"}, {"Europe_Eastern", "Gündogar Ýewropa standart wagty"}, {"Europe_Further_Eastern", "Uzak Gündogar Ýewropa wagty"}, {"Europe_Western", "Günbatar Ýewropa standart wagty"}, {"Falkland", "Folklend adalary standart wagty"}, {"Fiji", "Fiji standart wagty"}, {"French_Guiana", "Fransuz Gwianasy wagty"}, {"French_Southern", "Fransuz Günorta we Antarktika ýerleri wagty"}, {"GMT", "Grinwiç ortaça wagty"}, {"Galapagos", "Galapagos adalary wagty"}, {"Gambier", "Gambýe wagty"}, {"Georgia", "Gruziýa standart wagty"}, {"Gilbert_Islands", "Gilbert adalary wagty"}, {"Greenland_Eastern", "Gündogar Grenlandiýa standart wagty"}, {"Greenland_Western", "Günbatar Grenlandiýa standart wagty"}, {"Gulf", "Pars aýlagy standart wagty"}, {"Guyana", "Gaýana wagty"}, {"Hawaii", "Gawaý-Aleut standart wagty"}, {"Hawaii_Aleutian", "Gawaý-Aleut standart wagty"}, {"Hong_Kong", "Gonkong standart wagty"}, {"Hovd", "Howd standart wagty"}, {"India", "Hindistan standart wagty"}, {"Indian_Ocean", "Hindi ummany wagty"}, {"Indochina", "Hindihytaý wagty"}, {"Indonesia_Central", "Merkezi Indoneziýa wagty"}, {"Indonesia_Eastern", "Gündogar Indoneziýa wagty"}, {"Indonesia_Western", "Günbatar Indoneziýa wagty"}, {"Iran", "Eýran standart wagty"}, {"Irkutsk", "Irkutsk standart wagty"}, {"Israel", "Ysraýyl standart wagty"}, {"Japan", "Ýaponiýa standart wagty"}, {"Kazakhstan", "Gazagystan wagty"}, {"Kazakhstan_Eastern", "Gündogar Gazagystan wagty"}, {"Kazakhstan_Western", "Günbatar Gazagystan wagty"}, {"Korea", "Koreýa standart wagty"}, {"Kosrae", "Kosraýe wagty"}, {"Krasnoyarsk", "Krasnoýarsk standart wagty"}, {"Kyrgystan", "Gyrgyzystan wagty"}, {"Line_Islands", "Laýn adalary wagty"}, {"Lord_Howe", "Lord-Hau standart wagty"}, {"Magadan", "Magadan standart wagty"}, {"Malaysia", "Malaýziýa wagty"}, {"Maldives", "Maldiwler wagty"}, {"Marquesas", "Markiz adalary wagty"}, {"Marshall_Islands", "Marşall adalary wagty"}, {"Mauritius", "Mawrikiý standart wagty"}, {"Mawson", "Mouson wagty"}, {"Mexico_Pacific", "Meksikan Ýuwaş umman standart wagty"}, {"Mongolia", "Ulan-Bator standart wagty"}, {"Moscow", "Moskwa standart wagty"}, {"Myanmar", "Mýanma wagty"}, {"Nauru", "Nauru wagty"}, {"Nepal", "Nepal wagty"}, {"New_Caledonia", "Täze Kaledoniýa standart wagty"}, {"New_Zealand", "Täze Zelandiýa standart wagty"}, {"Newfoundland", "Nýufaundlend standart wagty"}, {"Niue", "Niue wagty"}, {"Norfolk", "Norfolk adasy standart wagty"}, {"Noronha", "Fernandu-di-Noronýa standart wagty"}, {"Novosibirsk", "Nowosibirsk standart wagty"}, {"Omsk", "Omsk standart wagty"}, {"Pakistan", "Pakistan standart wagty"}, {"Palau", "Palau wagty"}, {"Papua_New_Guinea", "Papua - Täze Gwineýa wagty"}, {"Paraguay", "Paragwaý standart wagty"}, {"Peru", "Peru standart wagty"}, {"Philippines", "Filippinler standart wagty"}, {"Phoenix_Islands", "Feniks adalary wagty"}, {"Pierre_Miquelon", "Sen-Pýer we Mikelon standart wagty"}, {"Pitcairn", "Pitkern wagty"}, {"Ponape", "Ponape wagty"}, {"Pyongyang", "Phenýan wagty"}, {"Reunion", "Reýunýon wagty"}, {"Rothera", "Rotera wagty"}, {"Sakhalin", "Sahalin standart wagty"}, {"Samoa", "Samoa standart wagty"}, {"Seychelles", "Seýşel adalary wagty"}, {"Singapore", "Singapur wagty"}, {"Solomon", "Solomon adalary wagty"}, {"South_Georgia", "Günorta Georgiýa wagty"}, {"Suriname", "Surinam wagty"}, {"Syowa", "Sýowa wagty"}, {"Tahiti", "Taiti wagty"}, {"Taipei", "Taýbeý standart wagty"}, {"Tajikistan", "Täjigistan wagty"}, {"Tokelau", "Tokelau wagty"}, {"Tonga", "Tonga standart wagty"}, {"Truk", "Çuuk wagty"}, {"Turkmenistan", "Türkmenistan standart wagty"}, {"Tuvalu", "Tuwalu wagty"}, {"Uruguay", "Urugwaý standart wagty"}, {"Uzbekistan", "Özbegistan standart wagty"}, {"Vanuatu", "Wanuatu standart wagty"}, {"Venezuela", "Wenesuela wagty"}, {"Vladivostok", "Wladiwostok standart wagty"}, {"Volgograd", "Wolgograd standart wagty"}, {"Vostok", "Wostok wagty"}, {"Wake", "Weýk adasy wagty"}, {"Wallis", "Uollis we Futuna wagty"}, {"Yakutsk", "Ýakutsk standart wagty"}, {"Yekaterinburg", "Ýekaterinburg standart wagty"}, {"Yukon", "Ýukon wagty"}},
		{{"Acre", "Akri tomusky wagty"}, {"Alaska", "Alýaska tomusky wagty"}, {"Amazon", "Amazon tomusky wagty"}, {"America_Central", "Merkezi Amerika tomusky wagty"}, {"America_Eastern", "Demirgazyk Amerika gündogar tomusky wagty"}, {"America_Mountain", "Demirgazyk Amerika dag tomusky wagty"}, {"America_Pacific", "Demirgazyk Amerika Ýuwaş umman tomusky wagty"}, {"Anadyr", "Anadyr tomusky wagty"}, {"Apia", "Apia tomusky wagty"}, {"Arabian", "Arap ýurtlary tomusky wagty"}, {"Argentina", "Argentina tomusky wagty"}, {"Argentina_Western", "Günbatar Argentina tomusky wagty"}, {"Armenia", "Ermenistan tomusky wagty"}, {"Atlantic", "Atlantik tomusky wagty"}, {"Australia_Central", "Merkezi Awstraliýa tomusky wagty"}, {"Australia_CentralWestern", "Merkezi Awstraliýa günbatar tomusky wagty"}, {"Australia_Eastern", "Gündogar Awstraliýa tomusky wagty"}, {"Australia_Western", "Günbatar Awstraliýa tomusky wagty"}, {"Azerbaijan", "Azerbaýjan tomusky wagty"}, {"Azores", "Azor adalary tomusky wagty"}, {"Bangladesh", "Bangladeş tomusky wagty"}, {"Brasilia", "Braziliýa tomusky wagty"}, {"Cape_Verde", "Kabo-Werde tomusky wagty"}, {"Chatham", "Çatem tomusky wagty"}, {"Chile", "Çili tomusky wagty"}, {"China", "Hytaý tomusky wagty"}, {"Colombia", "Kolumbiýa tomusky wagty"}, {"Cook", "Kuk adalary tomusky wagty"}, {"Cuba", "Kuba tomusky wagty"}, {"Easter", "Pasha adasy tomusky wagty"}, {"Europe_Central", "Merkezi Ýewropa tomusky wagty"}, {"Europe_Eastern", "Gündogar Ýewropa tomusky wagty"}, {"Europe_Western", "Günbatar Ýewropa tomusky wagty"}, {"Falkland", "Folklend adalary tomusky wagty"}, {"Fiji", "Fiji tomusky wagty"}, {"Georgia", "Gruziýa tomusky wagty"}, {"Greenland_Eastern", "Gündogar Grenlandiýa tomusky wagty"}, {"Greenland_Western", "Günbatar Grenlandiýa tomusky wagty"}, {"Hawaii_Aleutian", "Gawaý-Aleut tomusky wagty"}, {"Hong_Kong", "Gonkong tomusky wagty"}, {"Hovd", "Howd tomusky wagty"}, {"Iran", "Eýran tomusky wagty"}, {"Irkutsk", "Irkutsk tomusky wagty"}, {"Israel", "Ysraýyl tomusky wagty"}, {"Japan", "Ýaponiýa tomusky wagty"}, {"Kamchatka", "Kamçatka tomusky wagty"}, {"Korea", "Koreýa tomusky wagty"}, {"Krasnoyarsk", "Krasnoýarsk tomusky wagty"}, {"Lord_Howe", "Lord-Hau tomusky wagty"}, {"Magadan", "Magadan tomusky wagty"}, {"Mauritius", "Mawrikiý tomusky wagty"}, {"Mexico_Pacific", "Meksikan Ýuwaş umman tomusky wagty"}, {"Mongolia", "Ulan-Bator tomusky wagty"}, {"Moscow", "Moskwa tomusky wagty"}, {"New_Caledonia", "Täze Kaledoniýa tomusky wagty"}, {"New_Zealand", "Täze Zelandiýa tomusky wagty"}, {"Newfoundland", "Nýufaundlend tomusky wagty"}, {"Norfolk", "Norfolk adasy tomusky wagty"}, {"Noronha", "Fernandu-di-Noronýa tomusky wagty"}, {"Novosibirsk", "Nowosibisk tomusky wagty"}, {"Omsk", "Omsk tomusky wagty"}, {"Pakistan", "Pakistan tomusky wagty"}, {"Paraguay", "Paragwaý tomusky wagty"}, {"Peru", "Peru tomusky wagty"}, {"Philippines", "Filippinler tomusky wagty"}, {"Pierre_Miquelon", "Sen-Pýer we Mikelon tomusky wagty"}, {"Sakhalin", "Sahalin tomusky wagty"}, {"Samara", "Samara tomusky wagty"}, {"Samoa", "Samoa tomusky wagty"}, {"Taipei", "Taýbeý tomusky wagty"}, {"Tonga", "Tonga tomusky wagty"}, {"Turkmenistan", "Türkmenistan tomusky wagty"}, {"Uruguay", "Urugwaý tomusky wagty"}, {"Uzbekistan", "Özbegistan tomusky wagty"}, {"Vanuatu", "Wanuatu tomusky wagty"}, {"Vladivostok", "Wladiwostok tomusky wagty"}, {"Volgograd", "Wolgograd tomusky wagty"}, {"Yakutsk", "Ýakutsk tomusky wagty"}, {"Yekaterinburg", "Ýekaterinburg tomusky wagty"}},
	},
	{
		{{"GMT", "Palogare ya nako ya ngwaga le ngwaga ya Greenwich"}},
		{},
	},
	{
		{{"Acre", "houa fakaʻakelī taimi totonu"}, {"Afghanistan", "houa fakaʻafikānisitani"}, {"Africa_Central", "houa fakaʻafelika-loto"}, {"Africa_Eastern", "houa fakaʻafelika-hahake"}, {"Africa_Southern", "houa fakaʻafelika-tonga"}, {"Africa_Western", "houa fakaʻafelika-hihifo"}, {"Alaska", "houa fakaʻalasika taimi totonu"}, {"Almaty", "houa fakaʻalamati taimi totonu"}, {"Amazon", "houa fakaʻamasōne taimi totonu"}, {"America_Central", "houa fakaʻamelika-tokelau loto taimi totonu"}, {"America_Eastern", "houa fakaʻamelika-tokelau hahake taimi totonu"}, {"America_Mountain", "houa fakaʻamelika-tokelau moʻunga taimi totonu"}, {"America_Pacific", "houa fakaʻamelika-tokelau pasifika taimi totonu"}, {"Anadyr", "houa fakalūsia-ʻanatili taimi totonu"}, {"Apia", "houa fakaapia taimi totonu"}, {"Aqtau", "houa fakaʻakitau taimi totonu"}, {"Aqtobe", "houa fakaʻakitōpe taimi totonu"}, {"Arabian", "houa fakaʻalepea taimi totonu"}, {"Argentina", "houa fakaʻasenitina taimi totonu"}, {"Argentina_Western", "houa fakaʻasenitina-hihifo taimi totonu"}, {"Armenia", "houa fakaʻāmenia taimi totonu"}, {"Atlantic", "houa fakaʻamelika-tokelau ʻatalanitiki taimi totonu"}, {"Australia_Central", "houa fakaʻaositelēlia-loto taimi totonu"}, {"Australia_CentralWestern", "houa fakaʻaositelēlia-loto-hihifo taimi totonu"}, {"Australia_Eastern", "houa fakaʻaositelēlia-hahake taimi totonu"}, {"Australia_Western", "houa fakaʻaositelēlia-hihifo taimi totonu"}, {"Azerbaijan", "houa fakaʻasapaisani taimi totonu"}, {"Azores", "houa fakaʻāsolesi taimi totonu"}, {"Bangladesh", "houa fakapāngilātesi taimi totonu"}, {"Bhutan", "houa fakapūtani"}, {"Bolivia", "houa fakapolīvia"}, {"Brasilia", "houa fakapalāsila taimi totonu"}, {"Brunei", "houa fakapulunei"}, {"Cape_Verde", "houa fakamuiʻi-vēte taimi totonu"}, {"Casey", "houa fakakeesi"}, {"Chamorro", "houa fakakamolo"}, {"Chatham", "houa fakasatihami taimi totonu"}, {"Chile", "houa fakasili taimi totonu"}, {"China", "houa fakasiaina taimi totonu"}, {"Christmas", "houa fakamotukilisimasi"}, {"Cocos", "houa fakamotukokosi"}, {"Colombia", "houa fakakolomipia taimi totonu"}, {"Cook", "houa fakaʻotumotukuki taimi totonu"}, {"Cuba", "houa fakakiupa taimi totonu"}, {"Davis", "houa fakatavisi"}, {"DumontDUrville", "houa fakatūmoni-tūvile"}, {"East_Timor", "houa fakatimoa-hahake"}, {"Easter", "houa fakalapanui taimi totonu"}, {"Ecuador", "houa fakaʻekuetoa"}, {"Europe_Central", "houa fakaʻeulope-loto taimi totonu"}, {"Europe_Eastern", "houa fakaʻeulope-hahake taimi totonu"}, {"Europe_Further_Eastern", "houa fakaʻeulope-hahake-ange"}, {"Europe_Western", "houa fakaʻeulope-hihifo taimi totonu"}, {"Falkland", "houa fakaʻotumotu-fokulani taimi totonu"}, {"Fiji", "houa fakafisi taimi totonu"}, {"French_Guiana", "houa fakakuiana-fakafalanisē"}, {"French_Southern", "houa fakaʻanetātikafalanisē"}, {"GMT", "houa fakakiliniuisi mālie"}, {"Galapagos", "houa fakakalapakosi"}, {"Gambier", "houa fakakamipiē"}, {"Georgia", "houa fakaseōsia taimi totonu"}, {"Gilbert_Islands", "houa fakakilipasi"}, {"Greenland_Eastern", "houa fakafonuamata-hahake taimi totonu"}, {"Greenland_Western", "houa fakafonuamata-hihifo taimi totonu"}, {"Guam", "houa fakakuami"}, {"Gulf", "houa fakakūlifi"}, {"Guyana", "houa fakakuiana"}, {"Hawaii", "houa fakahauaiʻi-aleuti taimi totonu"}, {"Hawaii_Aleutian", "houa fakahauaiʻi-aleuti taimi totonu"}, {"Hong_Kong", "houa fakahongi-kongi taimi totonu"}, {"Hovd", "houa fakahovite taimi totonu"}, {"India", "houa fakaʻinitia"}, {"Indian_Ocean", "houa fakamoanaʻinitia"}, {"Indochina", "houa fakaʻinitosiaina"}, {"Indonesia_Central", "houa fakaʻinitonisia-loto"}, {"Indonesia_Eastern", "houa fakaʻinitonisia-hahake"}, {"Indonesia_Western", "houa fakaʻinitonisia-hihifo"}, {"Iran", "houa fakaʻilaani taimi totonu"}, {"Irkutsk", "houa fakalūsia-ʻīkutisiki taimi totonu"}, {"Israel", "houa fakaʻisileli taimi totonu"}, {"Japan", "houa fakasiapani taimi totonu"}, {"Kamchatka", "houa fakalūsia-petelopavilovisiki taimi totonu"}, {"Kazakhstan", "houa fakakasakitani"}, {"Kazakhstan_Eastern", "houa fakakasakitani-hahake"}, {"Kazakhstan_Western", "houa fakakasakitani-hihifo"}, {"Korea", "houa fakakōlea taimi totonu"}, {"Kosrae", "houa fakakosilae"}, {"Krasnoyarsk", "houa fakalūsia-kalasinoiāsiki taimi totonu"}, {"Kyrgystan", "houa fakakīkisitani"}, {"Lanka", "houa fakalangikā"}, {"Line_Islands", "houa fakaʻotumotulaine"}, {"Lord_Howe", "houa fakamotuʻeikihoue taimi totonu"}, {"Macau", "houa fakamakau taimi totonu"}, {"Magadan", "houa fakalūsia-makatani taimi totonu"}, {"Malaysia", "houa fakamaleisia"}, {"Maldives", "houa fakamalativisi"}, {"Marquesas", "houa fakamākesasi"}, {"Marshall_Islands", "houa fakaʻotumotumasolo"}, {"Mauritius", "houa fakamaulitiusi taimi totonu"}, {"Mawson", "houa fakamausoni"}, {"Mexico_Pacific", "houa fakamekisikou-pasifika taimi totonu"}, {"Mongolia", "houa fakaʻulānipātā taimi totonu"}, {"Moscow", "houa fakalūsia-mosikou taimi totonu"}, {"Myanmar", "houa fakapema"}, {"Nauru", "houa fakanaulu"}, {"Nepal", "houa fakanepali"}, {"New_Caledonia", "houa fakakaletōniafoʻou taimi totonu"}, {"New_Zealand", "houa fakanuʻusila taimi totonu"}, {"Newfoundland", "houa fakafonuaʻilofoʻou taimi totonu"}, {"Niue", "houa fakaniuē"}, {"Norfolk", "houa fakanoafōki taimi totonu"}, {"Noronha", "houa fakafēnanito-te-nolōnia taimi totonu"}, {"North_Mariana", "houa fakamalianatokelau"}, {"Novosibirsk", "houa fakalūsia-novosipīsiki taimi totonu"}, {"Omsk", "houa fakalūsia-ʻomisiki taimi totonu"}, {"Pakistan", "houa fakapākisitani taimi totonu"}, {"Palau", "houa fakapalau"}, {"Papua_New_Guinea", "houa fakapapuaniukini"}, {"Paraguay", "houa fakapalakuai taimi totonu"}, {"Peru", "houa fakapelū taimi totonu"}, {"Philippines", "houa fakafilipaine taimi totonu"}, {"Phoenix_Islands", "houa fakaʻotumotufoinikisi"}, {"Pierre_Miquelon", "houa fakasā-piea-mo-mikeloni taimi totonu"}, {"Pitcairn", "houa fakapitikani"}, {"Ponape", "houa fakapōnapē"}, {"Pyongyang", "houa fakapiongiangi"}, {"Qyzylorda", "houa fakakisilōta taimi totonu"}, {"Reunion", "houa fakalēunioni"}, {"Rothera", "houa fakalotela"}, {"Sakhalin", "houa fakalūsia-sakāline taimi totonu"}, {"Samara", "houa fakalūsia-samala taimi totonu"}, {"Samoa", "houa fakahaʻamoa taimi totonu"}, {"Seychelles", "houa fakaʻotumotu-seiseli"}, {"Singapore", "houa fakasingapoa"}, {"Solomon", "houa fakaʻotumotusolomone"}, {"South_Georgia", "houa fakasiosiatonga"}, {"Suriname", "houa fakasuliname"}, {"Syowa", "houa fakasioua"}, {"Tahiti", "houa fakatahisi"}, {"Taipei", "houa fakataipei taimi totonu"}, {"Tajikistan", "houa fakatasikitani"}, {"Tokelau", "houa fakatokelau"}, {"Tonga", "houa fakatonga taimi totonu"}, {"Truk", "houa fakatūke"}, {"Turkmenistan", "houa fakatūkimenisitani taimi totonu"}, {"Tuvalu", "houa fakatūvalu"}, {"Uruguay", "houa fakaʻulukuai taimi totonu"}, {"Uzbekistan", "houa fakaʻusipekitani taimi totonu"}, {"Vanuatu", "houa fakavanuatu taimi totonu"}, {"Venezuela", "houa fakavenesuela"}, {"Vladivostok", "houa fakalūsia-valativositoki taimi totonu"}, {"Volgograd", "houa fakalūsia-volikokalati taimi totonu"}, {"Vostok", "houa fakavositoki"}, {"Wake", "houa fakamotuueke"}, {"Wallis", "houa fakaʻuvea mo futuna"}, {"Yakutsk", "houa fakalūsia-ʻiākutisiki taimi totonu"}, {"Yekaterinburg", "houa fakalūsia-ʻiekatelinepūki taimi totonu"}, {"Yukon", "houa fakaiukoni"}},
		{{"Acre", "houa fakaʻakelī taimi liliu"}, {"Alaska", "houa fakaʻalasika taimi liliu"}, {"Almaty", "houa fakaʻalamati taimi liliu"}, {"Amazon", "houa fakaʻamasōne taimi liliu"}, {"America_Central", "houa fakaʻamelika-tokelau loto taimi liliu"}, {"America_Eastern", "houa fakaʻamelika-tokelau hahake taimi liliu"}, {"America_Mountain", "houa fakaʻamelika-tokelau moʻunga taimi liliu"}, {"America_Pacific", "houa fakaʻamelika-tokelau pasifika taimi liliu"}, {"Anadyr", "houa fakalūsia-ʻanatili taimi liliu"}, {"Apia", "houa fakaapia taimi liliu"}, {"Aqtau", "houa fakaʻakitau taimi liliu"}, {"Aqtobe", "houa fakaʻakitōpe taimi liliu"}, {"Arabian", "houa fakaʻalepea taimi liliu"}, {"Argentina", "houa fakaʻasenitina taimi liliu"}, {"Argentina_Western", "houa fakaʻasenitina-hihifo taimi liliu"}, {"Armenia", "houa fakaʻāmenia taimi liliu"}, {"Atlantic", "houa fakaʻamelika-tokelau ʻatalanitiki taimi liliu"}, {"Australia_Central", "houa fakaʻaositelēlia-loto taimi liliu"}, {"Australia_CentralWestern", "houa fakaʻaositelēlia-loto-hihifo taimi liliu"}, {"Australia_Eastern", "houa fakaʻaositelēlia-hahake taimi liliu"}, {"Australia_Western", "houa fakaʻaositelēlia-hihifo taimi liliu"}, {"Azerbaijan", "houa fakaʻasapaisani taimi liliu"}, {"Azores", "houa fakaʻāsolesi taimi liliu"}, {"Bangladesh", "houa fakapāngilātesi taimi liliu"}, {"Brasilia", "houa fakapalāsila taimi liliu"}, {"Cape_Verde", "houa fakamuiʻi-vēte taimi liliu"}, {"Chatham", "houa fakasatihami taimi liliu"}, {"Chile", "houa fakasili taimi liliu"}, {"China", "houa fakasiaina taimi liliu"}, {"Colombia", "houa fakakolomipia taimi liliu"}, {"Cook", "houa fakaʻotumotukuki taimi liliu"}, {"Cuba", "houa fakakiupa taimi liliu"}, {"Easter", "houa fakalapanui taimi liliu"}, {"Europe_Central", "houa fakaʻeulope-loto taimi liliu"}, {"Europe_Eastern", "houa fakaʻeulope-hahake taimi liliu"}, {"Europe_Western", "houa fakaʻeulope-hihifo taimi liliu"}, {"Falkland", "houa fakaʻotumotu-fokulani taimi liliu"}, {"Fiji", "houa fakafisi taimi liliu"}, {"Georgia", "houa fakaseōsia taimi liliu"}, {"Greenland_Eastern", "houa fakafonuamata-hahake taimi liliu"}, {"Greenland_Western", "houa fakafonuamata-hihifo taimi liliu"}, {"Hawaii_Aleutian", "houa fakahauaiʻi-aleuti taimi liliu"}, {"Hong_Kong", "houa fakahongi-kongi taimi liliu"}, {"Hovd", "houa fakahovite taimi liliu"}, {"Iran", "houa fakaʻilaani taimi liliu"}, {"Irkutsk", "houa fakalūsia-ʻīkutisiki taimi liliu"}, {"Israel", "houa fakaʻisileli taimi liliu"}, {"Japan", "houa fakasiapani taimi liliu"}, {"Kamchatka", "houa fakalūsia-petelopavilovisiki taimi liliu"}, {"Korea", "houa fakakōlea taimi liliu"}, {"Krasnoyarsk", "houa fakalūsia-kalasinoiāsiki taimi liliu"}, {"Lord_Howe", "houa fakamotuʻeikihoue taimi liliu"}, {"Macau", "houa fakamakau taimi liliu"}, {"Magadan", "houa fakalūsia-makatani taimi liliu"}, {"Mauritius", "houa fakamaulitiusi taimi liliu"}, {"Mexico_Pacific", "houa fakamekisikou-pasifika taimi liliu"}, {"Mongolia", "houa fakaʻulānipātā taimi liliu"}, {"Moscow", "houa fakalūsia-mosikou taimi liliu"}, {"New_Caledonia", "houa fakakaletōniafoʻou taimi liliu"}, {"New_Zealand", "houa fakanuʻusila taimi liliu"}, {"Newfoundland", "houa fakafonuaʻilofoʻou taimi liliu"}, {"Norfolk", "houa fakanoafōki taimi liliu"}, {"Noronha", "houa fakafēnanito-te-nolōnia taimi liliu"}, {"Novosibirsk", "houa fakalūsia-novosipīsiki taimi liliu"}, {"Omsk", "houa fakalūsia-ʻomisiki taimi liliu"}, {"Pakistan", "houa fakapākisitani taimi liliu"}, {"Paraguay", "houa fakapalakuai taimi liliu"}, {"Peru", "houa fakapelū taimi liliu"}, {"Philippines", "houa fakafilipaine taimi liliu"}, {"Pierre_Miquelon", "houa fakasā-piea-mo-mikeloni taimi liliu"}, {"Qyzylorda", "houa fakakisilōta taimi liliu"}, {"Sakhalin", "houa fakalūsia-sakāline taimi liliu"}, {"Samara", "houa fakalūsia-samala taimi liliu"}, {"Samoa", "houa fakahaʻamoa taimi liliu"}, {"Taipei", "houa fakataipei taimi liliu"}, {"Tonga", "houa fakatonga taimi liliu"}, {"Turkmenistan", "houa fakatūkimenisitani taimi liliu"}, {"Uruguay", "houa fakaʻulukuai taimi liliu"}, {"Uzbekistan", "houa fakaʻusipekitani taimi liliu"}, {"Vanuatu", "houa fakavanuatu taimi liliu"}, {"Vladivostok", "houa fakalūsia-valativositoki taimi liliu"}, {"Volgograd", "houa fakalūsia-volikokalati taimi liliu"}, {"Yakutsk", "houa fakalūsia-ʻiākutisiki taimi liliu"}, {"Yekaterinburg", "houa fakalūsia-ʻiekatelinepūki taimi liliu"}},
	},
	{
		{{"GMT", "tenpo pi ma Keni"}},
		{},
	},
	{
		{{"Acre", "Acre Standart Saati"}, {"Afghanistan", "Afganistan Saati"}, {"Africa_Central", "Orta Afrika Saati"}, {"Africa_Eastern", "Doğu Afrika Saati"}, {"Africa_Southern", "Güney Afrika Standart Saati"}, {"Africa_Western", "Batı Afrika Saati"}, {"Alaska", "Alaska Standart Saati"}, {"Almaty", "Almatı Standart Saati"}, {"Amazon", "Amazon Standart Saati"}, {"America_Central", "Kuzey Amerika Merkezi Standart Saati"}, {"America_Eastern", "Kuzey Amerika Doğu Standart Saati"}, {"America_Mountain", "Kuzey Amerika Dağ Standart Saati"}, {"America_Pacific", "Kuzey Amerika Pasifik Standart Saati"}, {"Anadyr", "Anadır Standart Saati"}, {"Apia", "Apia Standart Saati"}, {"Aqtau", "Aktav Standart Saati"}, {"Aqtobe", "Aktöbe Standart Saati"}, {"Arabian", "Arabistan Standart Saati"}, {"Argentina", "Arjantin Standart Saati"}, {"Argentina_Western", "Batı Arjantin Standart Saati"}, {"Armenia", "Ermenistan Standart Saati"}, {"Atlantic", "Atlantik Standart Saati"}, {"Australia_Central", "Orta Avustralya Standart Saati"}, {"Australia_CentralWestern", "İç Batı Avustralya Standart Saati"}, {"Australia_Eastern", "Doğu Avustralya Standart Saati"}, {"Australia_Western", "Batı Avustralya Standart Saati"}, {"Azerbaijan", "Azerbaycan Standart Saati"}, {"Azores", "Azorlar Standart Saati"}, {"Bangladesh", "Bangladeş Standart Saati"}, {"Bhutan", "Butan Saati"}, {"Bolivia", "Bolivya Saati"}, {"Brasilia", "Brasilia Standart Saati"}, {"Brunei", "Brunei Darü’s-Selam Saati"}, {"Cape_Verde", "Cape Verde Standart Saati"}, {"Casey", "Casey Saati"}, {"Chamorro", "Chamorro Saati"}, {"Chatham", "Chatham Standart Saati"}, {"Chile", "Şili Standart Saati"}, {"China", "Çin Standart Saati"}, {"Christmas", "Christmas Adası Saati"}, {"Cocos", "Cocos Adaları Saati"}, {"Colombia", "Kolombiya Standart Saati"}, {"Cook", "Cook Adaları Standart Saati"}, {"Cuba", "Küba Standart Saati"}, {"Davis", "Davis Saati"}, {"DumontDUrville", "Dumont-d’Urville Saati"}, {"East_Timor", "Doğu Timor Saati"}, {"Easter", "Paskalya Adası Standart Saati"}, {"Ecuador", "Ekvador Saati"}, {"Europe_Central", "Orta Avrupa Standart Saati"}, {"Europe_Eastern", "Doğu Avrupa Standart Saati"}, {"Europe_Further_Eastern", "İleri Doğu Avrupa Saati"}, {"Europe_Western", "Batı Avrupa Standart Saati"}, {"Falkland", "Falkland Adaları Standart Saati"}, {"Fiji", "Fiji Standart Saati"}, {"French_Guiana", "Fransız Guyanası Saati"}, {"French_Southern", "Fransız Güney ve Antarktika Saati"}, {"GMT", "Greenwich Ortalama Saati"}, {"Galapagos", "Galapagos Saati"}, {"Gambier", "Gambier Saati"}, {"Georgia", "Gürcistan Standart Saati"}, {"Gilbert_Islands", "Gilbert Adaları Saati"}, {"Greenland_Eastern", "Doğu Grönland Standart Saati"}, {"Greenland_Western", "Batı Grönland Standart Saati"}, {"Guam", "Guam Standart Saati"}, {"Gulf", "Körfez Saati"}, {"Guyana", "Guyana Saati"}, {"Hawaii", "Hawaii-Aleut Standart Saati"}, {"Hawaii_Aleutian", "Hawaii-Aleut Standart Saati"}, {"Hong_Kong", "Hong Kong Standart Saati"}, {"Hovd", "Hovd Standart Saati"}, {"India", "Hindistan Standart Saati"}, {"Indian_Ocean", "Hint Okyanusu Saati"}, {"Indochina", "Hindiçin Saati"}, {"Indonesia_Central", "Orta Endonezya Saati"}, {"Indonesia_Eastern", "Doğu Endonezya Saati"}, {"Indonesia_Western", "Batı Endonezya Saati"}, {"Iran", "İran Standart Saati"}, {"Irkutsk", "İrkutsk Standart Saati"}, {"Israel", "İsrail Standart Saati"}, {"Japan", "Japonya Standart Saati"}, {"Kamchatka", "Petropavlovsk-Kamçatski Standart Saati"}, {"Kazakhstan", "Kazakistan Saati"}, {"Kazakhstan_Eastern", "Doğu Kazakistan Saati"}, {"Kazakhstan_Western", "Batı Kazakistan Saati"}, {"Korea", "Kore Standart Saati"}, {"Kosrae", "Kosrae Saati"}, {"Krasnoyarsk", "Krasnoyarsk Standart Saati"}, {"Kyrgystan", "Kırgızistan Saati"}, {"Lanka", "Lanka Saati"}, {"Line_Islands", "Line Adaları Saati"}, {"Lord_Howe", "Lord Howe Standart Saati"}, {"Macau", "Makao Standart Saati"}, {"Magadan", "Magadan Standart Saati"}, {"Malaysia", "Malezya Saati"}, {"Maldives", "Maldivler Saati"}, {"Marquesas", "Markiz Adaları Saati"}, {"Marshall_Islands", "Marshall Adaları Saati"}, {"Mauritius", "Mauritius Standart Saati"}, {"Mawson", "Mawson Saati"}, {"Mexico_Pacific", "Meksika Pasifik Kıyısı Standart Saati"}, {"Mongolia", "Ulan Batur Standart Saati"}, {"Moscow", "Moskova Standart Saati"}, {"Myanmar", "Myanmar Saati"}, {"Nauru", "Nauru Saati"}, {"Nepal", "Nepal Saati"}, {"New_Caledonia", "Yeni Kaledonya Standart Saati"}, {"New_Zealand", "Yeni Zelanda Standart Saati"}, {"Newfoundland", "Newfoundland Standart Saati"}, {"Niue", "Niue Saati"}, {"Norfolk", "Norfolk Adası Standart Saati"}, {"Noronha", "Fernando de Noronha Standart Saati"}, {"North_Mariana", "Kuzey Mariana Adaları Saati"}, {"Novosibirsk", "Novosibirsk Standart Saati"}, {"Omsk", "Omsk Standart Saati"}, {"Pakistan", "Pakistan Standart Saati"}, {"Palau", "Palau Saati"}, {"Papua_New_Guinea", "Papua Yeni Gine Saati"}, {"Paraguay", "Paraguay Standart Saati"}, {"Peru", "Peru Standart Saati"}, {"Philippines", "Filipinler Standart Saati"}, {"Phoenix_Islands", "Phoenix Adaları Saati"}, {"Pierre_Miquelon", "Saint Pierre ve Miquelon Standart Saati"}, {"Pitcairn", "Pitcairn Saati"}, {"Ponape", "Ponape Saati"}, {"Pyongyang", "Pyongyang Saati"}, {"Qyzylorda", "Kızılorda Standart Saati"}, {"Reunion", "Reunion Saati"}, {"Rothera", "Rothera Saati"}, {"Sakhalin", "Sahalin Standart Saati"}, {"Samara", "Samara Standart Saati"}, {"Samoa", "Samoa Standart Saati"}, {"Seychelles", "Seyşeller Saati"}, {"Singapore", "Singapur Standart Saati"}, {"Solomon", "Solomon Adaları Saati"}, {"South_Georgia", "Güney Georgia Saati"}, {"Suriname", "Surinam Saati"}, {"Syowa", "Showa Saati"}, {"Tahiti", "Tahiti Saati"}, {"Taipei", "Taipei Standart Saati"}, {"Tajikistan", "Tacikistan Saati"}, {"Tokelau", "Tokelau Saati"}, {"Tonga", "Tonga Standart Saati"}, {"Truk", "Chuuk Saati"}, {"Turkey", "Türkiye Standart Saati"}, {"Turkmenistan", "Türkmenistan Standart Saati"}, {"Tuvalu", "Tuvalu Saati"}, {"Uruguay", "Uruguay Standart Saati"}, {"Uzbekistan", "Özbekistan Standart Saati"}, {"Vanuatu", "Vanuatu Standart Saati"}, {"Venezuela", "Venezuela Saati"}, {"Vladivostok", "Vladivostok Standart Saati"}, {"Volgograd", "Volgograd Standart Saati"}, {"Vostok", "Vostok Saati"}, {"Wake", "Wake Adası Saati"}, {"Wallis", "Wallis ve Futuna Saati"}, {"Yakutsk", "Yakutsk Standart Saati"}, {"Yekaterinburg", "Yekaterinburg Standart Saati"}, {"Yukon", "Yukon Saati"}},
		{{"Acre", "Acre Yaz Saati"}, {"Alaska", "Alaska Yaz Saati"}, {"Almaty", "Almatı Yaz Saati"}, {"Amazon", "Amazon Yaz Saati"}, {"America_Central", "Kuzey Amerika Merkezi Yaz Saati"}, {"America_Eastern", "Kuzey Amerika Doğu Yaz Saati"}, {"America_Mountain", "Kuzey Amerika Dağ Yaz Saati"}, {"America_Pacific", "Kuzey Amerika Pasifik Yaz Saati"}, {"Anadyr", "Anadır Yaz Saati"}, {"Apia", "Apia Yaz Saati"}, {"Aqtau", "Aktav Yaz Saati"}, {"Aqtobe", "Aktöbe Yaz Saati"}, {"Arabian", "Arabistan Yaz Saati"}, {"Argentina", "Arjantin Yaz Saati"}, {"Argentina_Western", "Batı Arjantin Yaz Saati"}, {"Armenia", "Ermenistan Yaz Saati"}, {"Atlantic", "Atlantik Yaz Saati"}, {"Australia_Central", "Orta Avustralya Yaz Saati"}, {"Australia_CentralWestern", "İç Batı Avustralya Yaz Saati"}, {"Australia_Eastern", "Doğu Avustralya Yaz Saati"}, {"Australia_Western", "Batı Avustralya Yaz Saati"}, {"Azerbaijan", "Azerbaycan Yaz Saati"}, {"Azores", "Azorlar Yaz Saati"}, {"Bangladesh", "Bangladeş Yaz Saati"}, {"Brasilia", "Brasilia Yaz Saati"}, {"Cape_Verde", "Cape Verde Yaz Saati"}, {"Chatham", "Chatham Yaz Saati"}, {"Chile", "Şili Yaz Saati"}, {"China", "Çin Yaz Saati"}, {"Colombia", "Kolombiya Yaz Saati"}, {"Cook", "Cook Adaları Yarı Yaz Saati"}, {"Cuba", "Küba Yaz Saati"}, {"Easter", "Paskalya Adası Yaz Saati"}, {"Europe_Central", "Orta Avrupa Yaz Saati"}, {"Europe_Eastern", "Doğu Avrupa Yaz Saati"}, {"Europe_Western", "Batı Avrupa Yaz Saati"}, {"Falkland", "Falkland Adaları Yaz Saati"}, {"Fiji", "Fiji Yaz Saati"}, {"Georgia", "Gürcistan Yaz Saati"}, {"Greenland_Eastern", "Doğu Grönland Yaz Saati"}, {"Greenland_Western", "Batı Grönland Yaz Saati"}, {"Hawaii_Aleutian", "Hawaii-Aleut Yaz Saati"}, {"Hong_Kong", "Hong Kong Yaz Saati"}, {"Hovd", "Hovd Yaz Saati"}, {"Iran", "İran Yaz Saati"}, {"Irkutsk", "İrkutsk Yaz Saati"}, {"Israel", "İsrail Yaz Saati"}, {"Japan", "Japonya Yaz Saati"}, {"Kamchatka", "Petropavlovsk-Kamçatski Yaz Saati"}, {"Korea", "Kore Yaz Saati"}, {"Krasnoyarsk", "Krasnoyarsk Yaz Saati"}, {"Lord_Howe", "Lord Howe Yaz Saati"}, {"Macau", "Makao Yaz Saati"}, {"Magadan", "Magadan Yaz Saati"}, {"Mauritius", "Mauritius Yaz Saati"}, {"Mexico_Pacific", "Meksika Pasifik Kıyısı Yaz Saati"}, {"Mongolia", "Ulan Batur Yaz Saati"}, {"Moscow", "Moskova Yaz Saati"}, {"New_Caledonia", "Yeni Kaledonya Yaz Saati"}, {"New_Zealand", "Yeni Zelanda Yaz Saati"}, {"Newfoundland", "Newfoundland Yaz Saati"}, {"Norfolk", "Norfolk Adası Yaz Saati"}, {"Noronha", "Fernando de Noronha Yaz Saati"}, {"Novosibirsk", "Novosibirsk Yaz Saati"}, {"Omsk", "Omsk Yaz Saati"}, {"Pakistan", "Pakistan Yaz Saati"}, {"Paraguay", "Paraguay Yaz Saati"}, {"Peru", "Peru Yaz Saati"}, {"Philippines", "Filipinler Yaz Saati"}, {"Pierre_Miquelon", "Saint Pierre ve Miquelon Yaz Saati"}, {"Qyzylorda", "Kızılorda Yaz Saati"}, {"Sakhalin", "Sahalin Yaz Saati"}, {"Samara", "Samara Yaz Saati"}, {"Samoa", "Samoa Yaz Saati"}, {"Taipei", "Taipei Yaz Saati"}, {"Tonga", "Tonga Yaz Saati"}, {"Turkey", "Türkiye Yaz Saati"}, {"Turkmenistan", "Türkmenistan Yaz Saati"}, {"Uruguay", "Uruguay Yaz Saati"}, {"Uzbekistan", "Özbekistan Yaz Saati"}, {"Vanuatu", "Vanuatu Yaz Saati"}, {"Vladivostok", "Vladivostok Yaz Saati"}, {"Volgograd", "Volgograd Yaz Saati"}, {"Yakutsk", "Yakutsk Yaz Saati"}, {"Yekaterinburg", "Yekaterinburg Yaz Saati"}},
	},
	{
		{{"America_Central", "Snegun Jikan Con-Amarika"}, {"America_Eastern", "Snegun Jikan Ton-Amarika"}, {"America_Mountain", "Snegun Jikan Yama-Amarika"}, {"America_Pacific", "Snegun Jikan Amarika-Daybinyan"}, {"Atlantic", "Snegun Jikan Yayun Tasiyan"}, {"Europe_Central", "Snegun Jikan Conow"}, {"Europe_Eastern", "Snegun Jikan Tonow"}, {"Europe_Western", "Snegun Jikan Siow"}, {"GMT", "Jikan Quri Grinweyji"}},
		{{"America_Central", "Jikan Con-Amarika o Karat Rbagan"}, {"America_Eastern", "Jikan Ton-Amarika o Karat Rbagan"}, {"America_Mountain", "Jikan Hidaw niq Yama-Amarika"}, {"America_Pacific", "Jikan Amarika-Daybinyan o Karat Rbagan"}, {"Atlantic", "Jikan Yayun Tasiyan o Karat Rbagan"}, {"Europe_Central", "Jikan Conow o Karat Rbagan"}, {"Europe_Eastern", "Jikan Tonow o Karat Rbagan"}, {"Europe_Western", "Jikan Siow o Karat Rbagan"}},
	},
	{
		{{"Afghanistan", "افغانستان سی وَخ"}, {"Africa_Central", "وسطی افریقہ ٹائم"}, {"Africa_Eastern", "مشرقی افریقہ ٹائم"}, {"Africa_Southern", "جنوبی افریقہ سٹینڈرڈ ٹائم"}, {"Africa_Western", "مغربی افریقہ ٹائم"}, {"Alaska", "الاسکا اسٹینڈرڈ ٹائم"}, {"Amazon", "ایمیزون سی معیاری وَخ"}, {"America_Central", "سنٹرل اسٹینڈرڈ ٹائم"}, {"America_Eastern", "ایسٹرن اسٹینڈرڈ ٹائم"}, {"America_Mountain", "ماؤنٹین اسٹینڈرڈ ٹائم"}, {"America_Pacific", "پیسفک اسٹینڈرڈ ٹائم"}, {"Apia", "ایپیا سٹینڈرڈ ٹائم"}, {"Arabian", "عرب سی معیاری وَخ"}, {"Argentina", "ارجنٹینا سی معیاری وَخ"}, {"Argentina_Western", "مغربی ارجنٹینا سی معیاری وَخ"}, {"Armenia", "آرمینیا سی معیاری وَخ"}, {"Atlantic", "اٹلانٹک اسٹینڈرڈ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل اسٹینڈرڈ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Western", "سٹریلیا ویسٹرن اسٹینڈرڈ ٹائم"}, {"Azerbaijan", "آذربائیجان سی معیاری وَخ"}, {"Azores", "ازوریس سی معیاری وَخ"}, {"Bangladesh", "بنگلہ دیش سی معیاری وَخ"}, {"Bhutan", "بھوٹان سی وَخ"}, {"Bolivia", "بولیویا سی وَخ"}, {"Brasilia", "برازیلیا اسٹینڈرڈ ٹائم"}, {"Brunei", "برونئی دارالسلام ٹائم"}, {"Cape_Verde", "کیپ ورڈی سٹینڈرڈ ٹائم"}, {"Chamorro", "چامورو سٹینڈرڈ ٹائم"}, {"Chatham", "چیتھم اسٹینڈرڈ ٹائم"}, {"Chile", "چلی سی معیاری وَخ"}, {"China", "چین سٹینڈرڈ ٹائم"}, {"Christmas", "کرسمس آئلینڈ ٹائم"}, {"Cocos", "کوکوس آئلینڈز ٹائم"}, {"Colombia", "کولمبیا سی معیاری وَخ"}, {"Cook", "کک آئلینڈز سٹینڈرڈ ٹائم"}, {"Cuba", "کیوبا اسٹینڈرڈ ٹائم"}, {"Davis", "ڈیوس ٹائم"}, {"DumontDUrville", "ڈومونٹ-ڈی’ارویلے ٹائم"}, {"East_Timor", "مشرقی تیمور ٹائم"}, {"Easter", "ایسٹر آئلینڈ سی معیاری وَخ"}, {"Ecuador", "ایکواڈور سی وَخ"}, {"Europe_Central", "وسطی یورپ سی معیاری وَخ"}, {"Europe_Eastern", "مشرقی یورپ سی معیاری وَخ"}, {"Europe_Further_Eastern", "بعید مشرقی یورپی وَخ"}, {"Europe_Western", "مغربی یورپ سی معیاری وَخ"}, {"Falkland", "فاک لینڈ آئلینڈز سی معیاری وَخ"}, {"Fiji", "فجی سٹینڈرڈ ٹائم"}, {"French_Guiana", "فرینچ گیانا سی وَخ"}, {"French_Southern", "فرینچ جنوبی آں انٹارکٹک ٹائم"}, {"GMT", "گرین وچ سی اصل وَخ"}, {"Galapagos", "گالاپاگوز سی وَخ"}, {"Gambier", "گیمبیئر ٹائم"}, {"Georgia", "جارجیا سی معیاری وَخ"}, {"Gilbert_Islands", "جلبرٹ آئلینڈز ٹائم"}, {"Greenland_Eastern", "مشرقی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Greenland_Western", "مغربی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Gulf", "خلیج سی معیاری وَخ"}, {"Guyana", "گیانا سی وَخ"}, {"Hawaii", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hong_Kong", "ہانگ سینگ سٹینڈرڈ ٹائم"}, {"Hovd", "ہووڈ سٹینڈرڈ ٹائم"}, {"India", "ہندوستان سی معیاری وَخ"}, {"Indian_Ocean", "بحر ہند ٹائم"}, {"Indochina", "ہند چین ٹائم"}, {"Indonesia_Central", "وسطی انڈونیشیا ٹائم"}, {"Indonesia_Eastern", "مشرقی انڈونیشیا ٹائم"}, {"Indonesia_Western", "مغربی انڈونیشیا ٹائم"}, {"Iran", "ایران سی معیاری وَخ"}, {"Irkutsk", "ارکتسک سٹینڈرڈ ٹائم"}, {"Israel", "اسرائیل سی معیاری وَخ"}, {"Japan", "جاپان سٹینڈرڈ ٹائم"}, {"Kazakhstan_Eastern", "مشرقی قزاخستان سی وَخ"}, {"Kazakhstan_Western", "مغربی قزاخستان سی وَخ"}, {"Korea", "کوریا سٹینڈرڈ ٹائم"}, {"Kosrae", "کوسرے ٹائم"}, {"Krasnoyarsk", "کرسنویارسک سٹینڈرڈ ٹائم"}, {"Kyrgystan", "کرغستان سی وَخ"}, {"Line_Islands", "لائن آئلینڈز ٹائم"}, {"Lord_Howe", "لارڈ ہووے اسٹینڈرڈ ٹائم"}, {"Magadan", "مگادان اسٹینڈرڈ ٹائم"}, {"Malaysia", "ملیشیا ٹائم"}, {"Maldives", "مالدیپ سی وَخ"}, {"Marquesas", "مارکیسس ٹائم"}, {"Marshall_Islands", "مارشل آئلینڈز ٹائم"}, {"Mauritius", "ماریشس سٹینڈرڈ ٹائم"}, {"Mawson", "ماؤسن ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک اسٹینڈرڈ ٹائم"}, {"Mongolia", "یولان بیتور سٹینڈرڈ ٹائم"}, {"Moscow", "ماسکو اسٹینڈرڈ ٹائم"}, {"Myanmar", "میانمار ٹائم"}, {"Nauru", "ناؤرو ٹائم"}, {"Nepal", "نیپال سی وَخ"}, {"New_Caledonia", "نیو کیلیڈونیا سٹینڈرڈ ٹائم"}, {"New_Zealand", "نیوزی لینڈ سی معیاری وَخ"}, {"Newfoundland", "نیو فاؤنڈ لینڈ اسٹینڈرڈ ٹائم"}, {"Niue", "نیئو ٹائم"}, {"Norfolk", "نارفوک آئلینڈ سی معیاری وَخ"}, {"Noronha", "فرنانڈو ڈی نورنہا سی معیاری وَخ"}, {"Novosibirsk", "نوووسیبرسک سٹینڈرڈ ٹائم"}, {"Omsk", "اومسک سٹینڈرڈ ٹائم"}, {"Pakistan", "پاکستان سی معیاری وَخ"}, {"Palau", "پلاؤ ٹائم"}, {"Papua_New_Guinea", "پاپوآ نیو گنی ٹائم"}, {"Paraguay", "پیراگوئے سی معیاری وَخ"}, {"Peru", "پیرو سی معیاری وَخ"}, {"Philippines", "فلپائن سٹینڈرڈ ٹائم"}, {"Phoenix_Islands", "فینکس آئلینڈز ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر آں مکلیئون اسٹینڈرڈ ٹائم"}, {"Pitcairn", "پٹکائرن ٹائم"}, {"Ponape", "پوناپے ٹائم"}, {"Pyongyang", "پیانگ یانگ وَخ"}, {"Reunion", "ری یونین ٹائم"}, {"Rothera", "روتھیرا سی وَخ"}, {"Sakhalin", "سخالین سٹینڈرڈ ٹائم"}, {"Samoa", "ساموآ سٹینڈرڈ ٹائم"}, {"Seychelles", "سیشلیز ٹائم"}, {"Singapore", "سنگاپور سٹینڈرڈ ٹائم"}, {"Solomon", "سولمن آئلینڈز ٹائم"}, {"South_Georgia", "جنوبی جارجیا ٹائم"}, {"Suriname", "سورینام سی وَخ"}, {"Syowa", "سیووا ٹائم"}, {"Tahiti", "تاہیتی ٹائم"}, {"Taipei", "تائی پیئی اسٹینڈرڈ ٹائم"}, {"Tajikistan", "تاجکستان سی وَخ"}, {"Tokelau", "ٹوکیلاؤ ٹائم"}, {"Tonga", "ٹونگا سٹینڈرڈ ٹائم"}, {"Truk", "چوک ٹائم"}, {"Turkmenistan", "ترکمانستان سی معیاری وَخ"}, {"Tuvalu", "ٹوالو ٹائم"}, {"Uruguay", "یوروگوئے سی معیاری وَخ"}, {"Uzbekistan", "ازبکستان سی معیاری وَخ"}, {"Vanuatu", "وانوآٹو سٹینڈرڈ ٹائم"}, {"Venezuela", "وینزوئیلا سی وَخ"}, {"Vladivostok", "ولادی ووستک سٹینڈرڈ ٹائم"}, {"Volgograd", "وولگوگراد اسٹینڈرڈ ٹائم"}, {"Vostok", "ووسٹاک سی وَخ"}, {"Wake", "ویک آئلینڈ ٹائم"}, {"Wallis", "والیز اور فٹونا ٹائم"}, {"Yakutsk", "یکوتسک اسٹینڈرڈ ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ اسٹینڈرڈ ٹائم"}, {"Yukon", "یوکون ٹائم"}},
		{{"Alaska", "الاسکا ڈے لائٹ ٹائم"}, {"Amazon", "امیزون سی موسم گرما سی وَخ"}, {"America_Central", "سنٹرل ڈے لائٹ ٹائم"}, {"America_Eastern", "ایسٹرن ڈے لائٹ ٹائم"}, {"America_Mountain", "ماؤنٹین ڈے لائٹ ٹائم"}, {"America_Pacific", "پیسفک ڈے لائٹ ٹائم"}, {"Apia", "ایپیا ڈے لائٹ ٹائم"}, {"Arabian", "عرب ڈے لائٹ ٹائم"}, {"Argentina", "ارجنٹینا سی موسم گرما سی وَخ"}, {"Argentina_Western", "مغربی ارجنٹینا سی موسم گرما سی وَخ"}, {"Armenia", "آرمینیا سی موسم گرما سی وَخ"}, {"Atlantic", "اٹلانٹک ڈے لائٹ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل ڈے لائٹ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن ڈے لائٹ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن ڈے لائٹ ٹائم"}, {"Australia_Western", "آسٹریلین ویسٹرن ڈے لائٹ ٹائم"}, {"Azerbaijan", "آذربائیجان سی موسم گرما سی وَخ"}, {"Azores", "ازوریس سی موسم گرما سی وَخ"}, {"Bangladesh", "بنگلہ دیش سی موسم گرما سی وَخ"}, {"Brasilia", "برازیلیا سمر ٹائم"}, {"Cape_Verde", "کیپ ورڈی سمر ٹائم"}, {"Chatham", "چیتھم ڈے لائٹ ٹائم"}, {"Chile", "چلی سی موسم گرما سی وَخ"}, {"China", "چینی ڈے لائٹ ٹائم"}, {"Colombia", "کولمبیا سی موسم گرما سی وَخ"}, {"Cook", "کک آئلینڈز نصف سمر ٹائم"}, {"Cuba", "کیوبا ڈے لائٹ ٹائم"}, {"Easter", "ایسٹر آئلینڈ سی موسم گرما سی وَخ"}, {"Europe_Central", "وسطی یورپ سی موسم گرما سی وَخ"}, {"Europe_Eastern", "مشرقی یورپ سی موسم گرما سی وَخ"}, {"Europe_Western", "مغربی یورپ سی موسم گرما سی وَخ"}, {"Falkland", "فاک لینڈ آئلینڈز سی موسم گرما سی وَخ"}, {"Fiji", "فجی سمر ٹائم"}, {"Georgia", "جارجیا سی موسم گرما سی وَخ"}, {"Greenland_Eastern", "مشرقی گرین لینڈ سی موسم گرما سی وَخ"}, {"Greenland_Western", "مغربی گرین لینڈ سی موسم گرما سی وَخ"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن ڈے لائٹ ٹائم"}, {"Hong_Kong", "ہانگ سینگ سمر ٹائم"}, {"Hovd", "ہووڈ سمر ٹائم"}, {"Iran", "ایران ڈے لائٹ ٹائم"}, {"Irkutsk", "ارکتسک سمر ٹائم"}, {"Israel", "اسرائیل ڈے لائٹ ٹائم"}, {"Japan", "جاپان ڈے لائٹ ٹائم"}, {"Korea", "کوریا ڈے لائٹ ٹائم"}, {"Krasnoyarsk", "کریسنویارسک سمر ٹائم"}, {"Lord_Howe", "لارڈ ہووے ڈے لائٹ ٹائم"}, {"Magadan", "میگیدن سمر ٹائم"}, {"Mauritius", "ماریشس سمر ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک ڈے لائٹ ٹائم"}, {"Mongolia", "یولان بیتور سمر ٹائم"}, {"Moscow", "ماسکو سمر ٹائم"}, {"New_Caledonia", "نیو کیلیڈونیا سمر ٹائم"}, {"New_Zealand", "نیوزی لینڈ ڈے لائٹ ٹائم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ ڈے لائٹ ٹائم"}, {"Norfolk", "نارفوک آئلینڈ سی موسم گرما سی وَخ"}, {"Noronha", "فرنانڈو ڈی نورونہا سمر ٹائم"}, {"Novosibirsk", "نوووسیبرسک سمر ٹائم"}, {"Omsk", "اومسک سمر ٹائم"}, {"Pakistan", "پاکستان سی موسم گرما سی وَخ"}, {"Paraguay", "پیراگوئے سی موسم گرما سی وَخ"}, {"Peru", "پیرو سی موسم گرما سی وَخ"}, {"Philippines", "فلپائن سمر ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر آں مکلیئون ڈے لائٹ ٹائم"}, {"Sakhalin", "سخالین سمر ٹائم"}, {"Samoa", "ساموآ ڈے لائٹ ٹائم"}, {"Taipei", "تئی پیئی ڈے لائٹ ٹائم"}, {"Tonga", "ٹونگا سمر ٹائم"}, {"Turkmenistan", "ترکمانستان سی موسم گرما سی وَخ"}, {"Uruguay", "یوروگوئے سی موسم گرما سی وَخ"}, {"Uzbekistan", "ازبکستان سی موسم گرما سی وَخ"}, {"Vanuatu", "وانوآٹو سمر ٹائم"}, {"Vladivostok", "ولادی ووستک سمر ٹائم"}, {"Volgograd", "وولگوگراد سمر ٹائم"}, {"Yakutsk", "یکوتسک سمر ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ سمر ٹائم"}},
	},
	{
		{{"Acre", "Акр стандарт вакыты"}, {"Afghanistan", "Әфганстан вакыты"}, {"Africa_Central", "Үзәк Африка вакыты"}, {"Africa_Eastern", "Көнчыгыш Африка вакыты"}, {"Africa_Southern", "Көньяк Африка вакыты"}, {"Africa_Western", "Көнбатыш Африка вакыты"}, {"Alaska", "Аляска стандарт вакыты"}, {"Amazon", "Амазонка стандарт вакыты"}, {"America_Central", "Төньяк Америка гадәти үзәк вакыты"}, {"America_Eastern", "Төньяк Америка гадәти көнчыгыш вакыты"}, {"America_Mountain", "Төньяк Америка гадәти тау вакыты"}, {"America_Pacific", "Төньяк Америка гадәти Тын океан вакыты"}, {"Anadyr", "Анадырь стандарт вакыты"}, {"Apia", "Апиа стандарт вакыты"}, {"Arabian", "Гарәп стандарт вакыты"}, {"Argentina", "Аргентина Стандарт вакыты"}, {"Argentina_Western", "Көнбатыш Аргентина стандарт вакыты"}, {"Armenia", "Армения стандарт вакыты"}, {"Atlantic", "Төньяк Америка гадәти атлантик вакыты"}, {"Australia_Central", "Австралия үзәк стандарт вакыты"}, {"Australia_CentralWestern", "Австралия үзәк көнбатыш стандарт вакыты"}, {"Australia_Eastern", "Көнчыгыш Австралия стандарт вакыты"}, {"Australia_Western", "Көнбатыш Австралия стандарт вакыты"}, {"Azerbaijan", "Әзербайҗан стандарт вакыты"}, {"Azores", "Азор утраулары стандарт вакыты"}, {"Bangladesh", "Бангладеш стандарт вакыты"}, {"Bhutan", "Бутан вакыты"}, {"Bolivia", "Боливия вакыты"}, {"Brasilia", "Бразилиа стандарт вакыты"}, {"Brunei", "Бруней-Даруссалам вакыты"}, {"Cape_Verde", "Кабо-Верде стандарт вакыты"}, {"Chamorro", "Чаморро стандарт вакыты"}, {"Chatham", "Чатем стандарт вакыты"}, {"Chile", "Чили стандарт вакыты"}, {"China", "Кытай стандарт вакыты"}, {"Christmas", "Раштуа утравы вакыты"}, {"Cocos", "Кокос утраулары вакыты"}, {"Colombia", "Колумбия стандарт вакыты"}, {"Cook", "Кук утраулары стандарт вакыты"}, {"Cuba", "Куба стандарт вакыты"}, {"Davis", "Дэвис вакыты"}, {"DumontDUrville", "Дюмон д’Юрвиль вакыты"}, {"East_Timor", "Көнчыгыш Тимор вакыты"}, {"Easter", "Пасха утравы стандарт вакыты"}, {"Ecuador", "Эквадор вакыты"}, {"Europe_Central", "гадәти Үзәк Европа вакыты"}, {"Europe_Eastern", "гадәти Көнчыгыш Европа вакыты"}, {"Europe_Further_Eastern", "Ерак Көнчыгыш Европа вакыты"}, {"Europe_Western", "гадәти Көнбатыш Европа вакыты"}, {"Falkland", "Фолкленд утраулары стандарт вакыты"}, {"Fiji", "Фиджи стандарт вакыты"}, {"French_Guiana", "Француз Гвиана вакыты"}, {"French_Southern", "Француз көньяк һәм Антарктика вакыты"}, {"GMT", "Гринвич уртача вакыты"}, {"Galapagos", "Галапагос утраулары вакыты"}, {"Gambier", "Гамбье вакыты"}, {"Georgia", "Грузия стандарт вакыты"}, {"Gilbert_Islands", "Гилберт утраулары вакыты"}, {"Greenland_Eastern", "Көнчыгыш Гренландия стандарт вакыты"}, {"Greenland_Western", "Көнбатыш Гренландия стандарт вакыты"}, {"Gulf", "Фарсы култыгының стандарт вакыты"}, {"Guyana", "Гайана вакыты"}, {"Hawaii", "Гавай-Алеут стандарт вакыты"}, {"Hawaii_Aleutian", "Гавай-Алеут стандарт вакыты"}, {"Hong_Kong", "Гонконг стандарт вакыты"}, {"Hovd", "Ховд стандарт вакыты"}, {"India", "Һинд стандарт вакыты"}, {"Indian_Ocean", "Һинд океаны вакыты"}, {"Indochina", "Һинд-кытай вакыты"}, {"Indonesia_Central", "Үзәк Индонезия вакыты"}, {"Indonesia_Eastern", "Көнчыгыш Индонезия вакыты"}, {"Indonesia_Western", "Көнбатыш Индонезия вакыты"}, {"Iran", "Иран стандарт вакыты"}, {"Irkutsk", "Иркутск стандарт вакыты"}, {"Israel", "Исраил стандарт вакыты"}, {"Japan", "Япон стандарт вакыты"}, {"Kamchatka", "Петропавловск-Камчатский стандарт вакыты"}, {"Kazakhstan", "Казахстан вакыты"}, {"Kazakhstan_Eastern", "Көнчыгыш Казахстан вакыты"}, {"Kazakhstan_Western", "Көнбатыш Казахстан вакыты"}, {"Korea", "Корея стандарт вакыты"}, {"Kosrae", "Косраэ вакыты"}, {"Krasnoyarsk", "Красноярск стандарт вакыты"}, {"Kyrgystan", "Кыргызстан вакыты"}, {"Line_Islands", "Лайн утраулары вакыты"}, {"Lord_Howe", "Лорд Хау стандарт вакыты"}, {"Magadan", "Магадан стандарт вакыты"}, {"Malaysia", "Малайзия вакыты"}, {"Maldives", "Мальдив утраулары вакыты"}, {"Marquesas", "Маркиз утраулары вакыты"}, {"Marshall_Islands", "Маршалл утраулары вакыты"}, {"Mauritius", "Маврикий стандарт вакыты"}, {"Mawson", "Моусон вакыты"}, {"Mexico_Pacific", "Мексика Тыныч океан стандартвакыты"}, {"Mongolia", "Улан-Батор стандарт вакыты"}, {"Moscow", "Мәскәү стандарт вакыты"}, {"Myanmar", "Мьянма вакыты"}, {"Nauru", "Науру вакыты"}, {"Nepal", "Непал вакыты"}, {"New_Caledonia", "Яңа Каледония стандарт вакыты"}, {"New_Zealand", "Яңа Зеландия стандарт вакыты"}, {"Newfoundland", "Ньюфаундленд стандарт вакыты"}, {"Niue", "Ниуэ вакыты"}, {"Norfolk", "Норфолк утравы стандарт вакыты"}, {"Noronha", "Фернанду-ди-Норонья стандарт вакыты"}, {"Novosibirsk", "Новосибирск стандарт вакыты"}, {"Omsk", "Омск стандарт вакыты"}, {"Pakistan", "Пакистан стандарт вакыты"}, {"Palau", "Палау вакыты"}, {"Papua_New_Guinea", "Папуа Яңа Гвинея вакыты"}, {"Paraguay", "Парагвай стандарт вакыты"}, {"Peru", "Перу стандарт вакыты"}, {"Philippines", "Филиппин стандарт вакыты"}, {"Phoenix_Islands", "Феникс утраулары вакыты"}, {"Pierre_Miquelon", "Сен-Пьер һәм Микелон стандарт вакыты"}, {"Pitcairn", "Питкэрн вакыты"}, {"Ponape", "Понапе вакыты"}, {"Pyongyang", "Пхеньян вакыты"}, {"Reunion", "Реюньон вакыты"}, {"Rothera", "Ротера вакыты"}, {"Sakhalin", "Сахалин стандарт вакыты"}, {"Samara", "Самара стандарт вакыты"}, {"Samoa", "Самоа стандарт вакыты"}, {"Seychelles", "Сейшел утраулары вакыты"}, {"Singapore", "Сингапур стандарт вакыты"}, {"Solomon", "Соломон утраулары вакыты"}, {"South_Georgia", "Көньяк Джорджия вакыты"}, {"Suriname", "Суринам вакыты"}, {"Syowa", "Сёва вакыты"}, {"Tahiti", "Таити вакыты"}, {"Taipei", "Тайпей стандарт вакыты"}, {"Tajikistan", "Таҗикстан вакыты"}, {"Tokelau", "Токелау вакыты"}, {"Tonga", "Тонга стандарт вакыты"}, {"Truk", "Чуук вакыты"}, {"Turkmenistan", "Төркмәнстан стандарт вакыты"}, {"Tuvalu", "Тувалу вакыты"}, {"Uruguay", "Уругвай стандарт вакыты"}, {"Uzbekistan", "Үзбәкстан стандарт вакыты"}, {"Vanuatu", "Вануату стандарт вакыты"}, {"Venezuela", "Венесуэла вакыты"}, {"Vladivostok", "Владивосток стандарт вакыты"}, {"Volgograd", "Волгоград стандарт вакыты"}, {"Vostok", "Восток вакыты"}, {"Wake", "Уэйк утравы вакыты"}, {"Wallis", "Уоллис һәм Футуна вакыты"}, {"Yakutsk", "Якутск стандарт вакыты"}, {"Yekaterinburg", "Екатеринбург стандарт вакыты"}, {"Yukon", "Юкон вакыты"}},
		{{"Acre", "Акр җәйге вакыты"}, {"Alaska", "Аляска җәйге вакыты"}, {"Amazon", "Амазонка җәйге вакыты"}, {"America_Central", "Төньяк Америка җәйге үзәк вакыты"}, {"America_Eastern", "Төньяк Америка җәйге көнчыгыш вакыты"}, {"America_Mountain", "Төньяк Америка җәйге тау вакыты"}, {"America_Pacific", "Төньяк Америка җәйге Тын океан вакыты"}, {"Anadyr", "Анадырь җәйге вакыты"}, {"Apia", "Апиа җәйге вакыты"}, {"Arabian", "Гарәп җәйге вакыты"}, {"Argentina", "Аргентина җәйге вакыты"}, {"Argentina_Western", "Көнбатыш Аргентина җәйге вакыты"}, {"Armenia", "Армения җәйге вакыты"}, {"Atlantic", "Төньяк Америка җәйге атлантик вакыты"}, {"Australia_Central", "Австралия үзәк җәйге вакыты"}, {"Australia_CentralWestern", "Австралия үзәк көнбатыш җәйге вакыты"}, {"Australia_Eastern", "Көнчыгыш Австралия җәйге вакыты"}, {"Australia_Western", "Көнбатыш Австралия җәйге вакыты"}, {"Azerbaijan", "Әзербайҗан җәйге вакыты"}, {"Azores", "Азоу утраулары җәйге вакыты"}, {"Bangladesh", "Бангладеш җәйге вакыты"}, {"Brasilia", "Бразилиа җәйге вакыты"}, {"Cape_Verde", "Кабо-Верде җәйге вакыты"}, {"Chatham", "Чатем җәйге вакыты"}, {"Chile", "Чили җәйге вакыты"}, {"China", "Кытай җәйге вакыты"}, {"Colombia", "Колумбия җәйге вакыты"}, {"Cook", "Кук утраулары уртача җәйге вакыты"}, {"Cuba", "Куба җәйге вакыты"}, {"Easter", "Пасха утравы җәйге вакыты"}, {"Europe_Central", "җәйге Үзәк Европа вакыты"}, {"Europe_Eastern", "җәйге Көнчыгыш Европа вакыты"}, {"Europe_Western", "җәйге Көнбатыш Европа вакыты"}, {"Falkland", "Фолкленд утраулары җәйге вакыты"}, {"Fiji", "Фиджи җәйге вакыты"}, {"Georgia", "Грузия җәйге вакыты"}, {"Greenland_Eastern", "Көнчыгыш Гренландия җәйге вакыты"}, {"Greenland_Western", "Көнбатыш Гренландия җәйге вакыты"}, {"Hawaii_Aleutian", "Гавай-Алеут җәйге вакыты"}, {"Hong_Kong", "Гонконг җәйге вакыты"}, {"Hovd", "Ховд җәйге вакыты"}, {"Iran", "Иран җәйге вакыты"}, {"Irkutsk", "Иркутск җәйге вакыты"}, {"Israel", "Исраил җәйге вакыты"}, {"Japan", "Япон җәйге вакыты"}, {"Kamchatka", "Петропавловск-Камчатский җәйге вакыты"}, {"Korea", "Корея җәйге вакыты"}, {"Krasnoyarsk", "Красноярск җәйге вакыты"}, {"Lord_Howe", "Лорд Хау җәйге вакыты"}, {"Magadan", "Магадан җәйге вакыты"}, {"Mauritius", "Маврикий җәйге вакыты"}, {"Mexico_Pacific", "Мексика Тыныч океан җәйге вакыты"}, {"Mongolia", "Улан-Батор җәйге вакыты"}, {"Moscow", "Мәскәү җәйге вакыты"}, {"New_Caledonia", "Яңа Каледония җәйге вакыты"}, {"New_Zealand", "Яңа Зеландия җәйге вакыты"}, {"Newfoundland", "Ньюфаундленд җәйге вакыты"}, {"Norfolk", "Норфолк утравы җәйге вакыты"}, {"Noronha", "Фернанду-ди-Норонья җәйге вакыты"}, {"Novosibirsk", "Новосибирск җәйге вакыты"}, {"Omsk", "Омск җәйге вакыты"}, {"Pakistan", "Пакистан җәйге вакыты"}, {"Paraguay", "Парагвай җәйге вакыты"}, {"Peru", "Перу җәйге вакыты"}, {"Philippines", "Филиппин җәйге вакыты"}, {"Pierre_Miquelon", "Сен-Пьер һәм Микелон җәйге вакыты"}, {"Sakhalin", "Сахалин җәйге вакыты"}, {"Samara", "Самара җәйге вакыты"}, {"Samoa", "Самоа җәйге вакыты"}, {"Taipei", "Тайпей җәйге вакыты"}, {"Tonga", "Тонга җәйге вакыты"}, {"Turkmenistan", "Төркмәнстан җәйге вакыты"}, {"Uruguay", "Уругвай җәйге вакыты"}, {"Uzbekistan", "Үзбәкстан җәйге вакыты"}, {"Vanuatu", "Вануату җәйге вакыты"}, {"Vladivostok", "Владивосток җәйге вакыты"}, {"Volgograd", "Волгоград җәйге вакыты"}, {"Yakutsk", "Якутск җәйге вакыты"}, {"Yekaterinburg", "Екатеринбург җәйге вакыты"}},
	},
	{
		{{"GMT", "Гринвичтиң ортаа үези"}},
		{},
	},
	{
		{{"Acre", "ئاكرې ئۆلچەملىك ۋاقتى"}, {"Afghanistan", "ئافغانىستان ۋاقتى"}, {"Africa_Central", "ئوتتۇرا ئافرىقا ۋاقتى"}, {"Africa_Eastern", "شەرقىي ئافرىقا ۋاقتى"}, {"Africa_Southern", "جەنۇبىي ئافرىقا ئۆلچەملىك ۋاقتى"}, {"Africa_Western", "غەربىي ئافرىقا ۋاقتى"}, {"Alaska", "ئالياسكا ئۆلچەملىك ۋاقتى"}, {"Almaty", "ئالمۇتا ئۆلچەملىك ۋاقتى"}, {"Amazon", "ئامازون ئۆلچەملىك ۋاقتى"}, {"America_Central", "ئوتتۇرا قىسىم ئۆلچەملىك ۋاقتى"}, {"America_Eastern", "شەرقىي قىسىم ئۆلچەملىك ۋاقتى"}, {"America_Mountain", "تاغ ئۆلچەملىك ۋاقتى"}, {"America_Pacific", "تىنچ ئوكيان ئۆلچەملىك ۋاقتى"}, {"Anadyr", "ئانادىر ئۆلچەملىك ۋاقتى"}, {"Aqtau", "ئاقتاي ئۆلچەملىك ۋاقتى"}, {"Aqtobe", "ئاقتۆبە ئۆلچەملىك ۋاقتى"}, {"Arabian", "ئەرەب ئۆلچەملىك ۋاقتى"}, {"Argentina", "ئارگېنتىنا ئۆلچەملىك ۋاقتى"}, {"Argentina_Western", "غەربىي ئارگېنتىنا ئۆلچەملىك ۋاقتى"}, {"Armenia", "ئەرمېنىيە ئۆلچەملىك ۋاقتى"}, {"Atlantic", "ئاتلانتىك ئوكيان ئۆلچەملىك ۋاقتى"}, {"Australia_Central", "ئاۋسترالىيە ئوتتۇرا قىسىم ئۆلچەملىك ۋاقتى"}, {"Australia_CentralWestern", "ئاۋستىرالىيە ئوتتۇرا غەربىي قىسىم ئۆلچەملىك ۋاقتى"}, {"Australia_Eastern", "ئاۋسترالىيە شەرقىي قىسىم ئۆلچەملىك ۋاقتى"}, {"Australia_Western", "ئاۋسترالىيە غەربىي قىسىم ئۆلچەملىك ۋاقتى"}, {"Azerbaijan", "ئەزەربەيجان ئۆلچەملىك ۋاقتى"}, {"Azores", "ئازور ئۆلچەملىك ۋاقتى"}, {"Bangladesh", "باڭلادىش ئۆلچەملىك ۋاقتى"}, {"Bhutan", "بۇتان ۋاقتى"}, {"Bolivia", "بولىۋىيە ۋاقتى"}, {"Brasilia", "بىرازىلىيە ئۆلچەملىك ۋاقتى"}, {"Brunei", "بىرۇنىي دارۇسسالام ۋاقتى"}, {"Cape_Verde", "يېشىل تۇمشۇق ئۆلچەملىك ۋاقتى"}, {"Casey", "كاسېي ۋاقتى"}, {"Chamorro", "چاموررو ئۆلچەملىك ۋاقتى"}, {"Chatham", "چاتام ئۆلچەملىك ۋاقتى"}, {"Chile", "چىلى ئۆلچەملىك ۋاقتى"}, {"China", "جۇڭگو ئۆلچەملىك ۋاقتى"}, {"Christmas", "روژدېستۋو ئارىلى ۋاقتى"}, {"Cocos", "كوكۇس ئارىلى ۋاقتى"}, {"Colombia", "كولومبىيە ئۆلچەملىك ۋاقتى"}, {"Cook", "كۇك ئاراللىرى ئۆلچەملىك ۋاقتى"}, {"Cuba", "كۇبا ئۆلچەملىك ۋاقتى"}, {"Davis", "داۋىس ۋاقتى"}, {"DumontDUrville", "دۇمونت-دۇرۋىل ۋاقتى"}, {"East_Timor", "شەرقىي تىمور ۋاقتى"}, {"Easter", "پاسكاليا ئارىلى ئۆلچەملىك ۋاقتى"}, {"Ecuador", "ئېكۋادور ۋاقتى"}, {"Europe_Central", "ئوتتۇرا ياۋروپا ئۆلچەملىك ۋاقتى"}, {"Europe_Eastern", "شەرقىي ياۋروپا ئۆلچەملىك ۋاقتى"}, {"Europe_Western", "غەربىي ياۋروپا ئۆلچەملىك ۋاقتى"}, {"Falkland", "فالكلاند ئاراللىرى ئۆلچەملىك ۋاقتى"}, {"Fiji", "فىجى ئۆلچەملىك ۋاقتى"}, {"French_Guiana", "فىرانسىيەگە قاراشلىق گىۋىيانا ۋاقتى"}, {"French_Southern", "فىرانسىيەگە قاراشلىق جەنۇبىي ۋە ئانتاركتىكا ۋاقتى"}, {"GMT", "گىرىنۋىچ ۋاقتى"}, {"Galapagos", "گالاپاگوس ۋاقتى"}, {"Gambier", "گامبىيېر ۋاقتى"}, {"Georgia", "گىرۇزىيە ئۆلچەملىك ۋاقتى"}, {"Gilbert_Islands", "گىلبېرت ئاراللىرى ۋاقتى"}, {"Greenland_Eastern", "شەرقىي گىرېنلاند ئۆلچەملىك ۋاقتى"}, {"Greenland_Western", "غەربىي گىرېنلاند ئۆلچەملىك ۋاقتى"}, {"Guam", "گۇئام ئۆلچەملىك ۋاقتى"}, {"Gulf", "گۇلف ئۆلچەملىك ۋاقتى"}, {"Guyana", "گىۋىيانا ۋاقتى"}, {"Hawaii", "ھاۋاي-ئالېيۇت ئۆلچەملىك ۋاقتى"}, {"Hawaii_Aleutian", "ھاۋاي-ئالېيۇت ئۆلچەملىك ۋاقتى"}, {"Hong_Kong", "شياڭگاڭ ئۆلچەملىك ۋاقتى"}, {"Hovd", "خوۋد ئۆلچەملىك ۋاقتى"}, {"India", "ھىندىستان ئۆلچەملىك ۋاقتى"}, {"Indian_Ocean", "ھىندى ئوكيان ۋاقتى"}, {"Indochina", "ھىندى چىنى ۋاقتى"}, {"Indonesia_Central", "ئوتتۇرا ھىندونېزىيە ۋاقتى"}, {"Indonesia_Eastern", "شەرقىي ھىندونېزىيە ۋاقتى"}, {"Indonesia_Western", "غەربىي ھىندونېزىيە ۋاقتى"}, {"Iran", "ئىران ئۆلچەملىك ۋاقتى"}, {"Irkutsk", "ئىركۇتسك ئۆلچەملىك ۋاقتى"}, {"Israel", "ئىسرائىلىيە ئۆلچەملىك ۋاقتى"}, {"Japan", "ياپونىيە ئۆلچەملىك ۋاقتى"}, {"Kamchatka", "پېتروپاۋلوۋسك-كامچاتكسكى ئۆلچەملىك ۋاقتى"}, {"Kazakhstan_Eastern", "شەرقىي قازاقىستان ۋاقتى"}, {"Kazakhstan_Western", "غەربىي قازاقىستان ۋاقتى"}, {"Korea", "كورىيە ئۆلچەملىك ۋاقتى"}, {"Kosrae", "كوسرائې ۋاقتى"}, {"Krasnoyarsk", "كىراسنويارسك ئۆلچەملىك ۋاقتى"}, {"Kyrgystan", "قىرغىزىستان ۋاقتى"}, {"Lanka", "سىرى لانكا ۋاقتى"}, {"Line_Islands", "لاين ئاراللىرى ۋاقتى"}, {"Lord_Howe", "لورد-خاي ئۆلچەملىك ۋاقتى"}, {"Macau", "ئاۋمېن ئۆلچەملىك ۋاقتى"}, {"Magadan", "ماگادان ئۆلچەملىك ۋاقتى"}, {"Malaysia", "مالايشىيا ۋاقتى"}, {"Maldives", "مالدىۋې ۋاقتى"}, {"Marquesas", "ماركىز ۋاقتى"}, {"Marshall_Islands", "مارشال ئاراللىرى ۋاقتى"}, {"Mauritius", "ماۋرىتىئۇس ئۆلچەملىك ۋاقتى"}, {"Mawson", "ماۋسون ۋاقتى"}, {"Mexico_Pacific", "مېكسىكا تىنچ ئوكيان ئۆلچەملىك ۋاقتى"}, {"Mongolia", "ئۇلانباتور ئۆلچەملىك ۋاقتى"}, {"Moscow", "موسكۋا ئۆلچەملىك ۋاقتى"}, {"Myanmar", "بىرما ۋاقتى"}, {"Nauru", "ناۋرۇ ۋاقتى"}, {"Nepal", "نېپال ۋاقتى"}, {"New_Caledonia", "يېڭى كالېدونىيە ئۆلچەملىك ۋاقتى"}, {"New_Zealand", "يېڭى زېلاندىيە ئۆلچەملىك ۋاقتى"}, {"Newfoundland", "نىۋفوئۇنلاند ئۆلچەملىك ۋاقتى"}, {"Niue", "نىيۇئې ۋاقتى"}, {"Norfolk", "نورفولك ئاراللىرى ئۆلچەملىك ۋاقتى"}, {"Noronha", "فېرناندو-نورونخا ئۆلچەملىك ۋاقتى"}, {"North_Mariana", "شىمالىي مارىيانا ئاراللىرى ۋاقتى"}, {"Novosibirsk", "نوۋوسىبىرسك ئۆلچەملىك ۋاقتى"}, {"Omsk", "ئومسك ئۆلچەملىك ۋاقتى"}, {"Pakistan", "پاكىستان ئۆلچەملىك ۋاقتى"}, {"Palau", "پالاۋ ۋاقتى"}, {"Papua_New_Guinea", "پاپۇئا يېڭى گىۋىنېيەسى ۋاقتى"}, {"Paraguay", "پاراگۋاي ئۆلچەملىك ۋاقتى"}, {"Peru", "پېرۇ ئۆلچەملىك ۋاقتى"}, {"Philippines", "فىلىپپىن ئۆلچەملىك ۋاقتى"}, {"Phoenix_Islands", "فېنىكس ئاراللىرى ۋاقتى"}, {"Pierre_Miquelon", "ساينىت پىئېر ۋە مىكېلون ئۆلچەملىك ۋاقتى"}, {"Pitcairn", "پىتكاير ۋاقتى"}, {"Ponape", "پونپېي ۋاقتى"}, {"Qyzylorda", "قىزىلئوردا ئۆلچەملىك ۋاقتى"}, {"Reunion", "رېئونىيون ۋاقتى"}, {"Rothera", "روتېرا ۋاقتى"}, {"Sakhalin", "ساخارىن ئۆلچەملىك ۋاقتى"}, {"Samara", "سامارا ئۆلچەملىك ۋاقتى"}, {"Samoa", "ساموئا ئۆلچەملىك ۋاقتى"}, {"Seychelles", "سېيشېل ۋاقتى"}, {"Singapore", "سىنگاپور ۋاقتى"}, {"Solomon", "سولومون ئاراللىرى ۋاقتى"}, {"South_Georgia", "جەنۇبىي جورجىيە ۋاقتى"}, {"Suriname", "سۇرىنام ۋاقتى"}, {"Syowa", "شوۋا ۋاقتى"}, {"Tahiti", "تايتى ۋاقتى"}, {"Taipei", "تەيبېي ئۆلچەملىك ۋاقتى"}, {"Tajikistan", "تاجىكىستان ۋاقتى"}, {"Tokelau", "توكېلاۋ ۋاقتى"}, {"Tonga", "تونگا ئۆلچەملىك ۋاقتى"}, {"Truk", "چۇك ۋاقتى"}, {"Turkmenistan", "تۈركمەنىستان ئۆلچەملىك ۋاقتى"}, {"Tuvalu", "تۇۋالۇ ۋاقتى"}, {"Uruguay", "ئۇرۇگۋاي ئۆلچەملىك ۋاقتى"}, {"Uzbekistan", "ئۆزبېكىستان ئۆلچەملىك ۋاقتى"}, {"Vanuatu", "ۋانۇئاتۇ ئۆلچەملىك ۋاقتى"}, {"Venezuela", "ۋېنېزۇئېلا ۋاقتى"}, {"Vladivostok", "ۋىلادىۋوستوك ئۆلچەملىك ۋاقتى"}, {"Volgograd", "ۋولگاگراد ئۆلچەملىك ۋاقتى"}, {"Vostok", "ۋوستوك ۋاقتى"}, {"Wake", "ۋېيك ئارىلى ۋاقتى"}, {"Wallis", "ۋاللىس ۋە فۇتۇنا ۋاقتى"}, {"Yakutsk", "ياكۇتسك ئۆلچەملىك ۋاقتى"}, {"Yekaterinburg", "يېكاتېرىنبۇرگ ئۆلچەملىك ۋاقتى"}},
		{{"Acre", "ئاكرى يازلىق ۋاقتى"}, {"Alaska", "ئالياسكا يازلىق ۋاقتى"}, {"Almaty", "ئالمۇتا يازلىق ۋاقتى"}, {"Amazon", "ئامازون يازلىق ۋاقتى"}, {"America_Central", "ئوتتۇرا قىسىم يازلىق ۋاقتى"}, {"America_Eastern", "شەرقىي قىسىم يازلىق ۋاقتى"}, {"America_Mountain", "تاغ يازلىق ۋاقتى"}, {"America_Pacific", "تىنچ ئوكيان يازلىق ۋاقتى"}, {"Anadyr", "ئانادىر يازلىق ۋاقتى"}, {"Aqtau", "ئاقتاي يازلىق ۋاقتى"}, {"Aqtobe", "ئاقتۆبە يازلىق ۋاقتى"}, {"Arabian", "ئەرەب يازلىق ۋاقتى"}, {"Argentina", "ئارگېنتىنا يازلىق ۋاقتى"}, {"Argentina_Western", "غەربىي ئارگېنتىنا يازلىق ۋاقتى"}, {"Armenia", "ئەرمېنىيە يازلىق ۋاقتى"}, {"Atlantic", "ئاتلانتىك ئوكيان يازلىق ۋاقتى"}, {"Australia_Central", "ئاۋسترالىيە ئوتتۇرا قىسىم يازلىق ۋاقتى"}, {"Australia_CentralWestern", "ئاۋسترالىيە ئوتتۇرا غەربىي قىسىم يازلىق ۋاقتى"}, {"Australia_Eastern", "ئاۋسترالىيە شەرقىي قىسىم يازلىق ۋاقتى"}, {"Australia_Western", "ئاۋسترالىيە غەربىي قىسىم يازلىق ۋاقتى"}, {"Azerbaijan", "ئەزەربەيجان يازلىق ۋاقتى"}, {"Azores", "ئازور يازلىق ۋاقتى"}, {"Bangladesh", "باڭلادىش يازلىق ۋاقتى"}, {"Brasilia", "بىرازىلىيە يازلىق ۋاقتى"}, {"Cape_Verde", "يېشىل تۇمشۇق يازلىق ۋاقتى"}, {"Chatham", "چاتام يازلىق ۋاقتى"}, {"Chile", "چىلى يازلىق ۋاقتى"}, {"China", "جۇڭگو يازلىق ۋاقتى"}, {"Colombia", "كولومبىيە يازلىق ۋاقتى"}, {"Cook", "كۇك ئاراللىرى يېرىم يازلىق ۋاقتى"}, {"Cuba", "كۇبا يازلىق ۋاقتى"}, {"Easter", "ئېستېر ئارىلى يازلىق ۋاقتى"}, {"Europe_Central", "ئوتتۇرا ياۋروپا يازلىق ۋاقتى"}, {"Europe_Eastern", "شەرقىي ياۋروپا يازلىق ۋاقتى"}, {"Europe_Western", "غەربىي ياۋروپا يازلىق ۋاقتى"}, {"Falkland", "فالكلاند ئاراللىرى يازلىق ۋاقتى"}, {"Fiji", "فىجى يازلىق ۋاقتى"}, {"Georgia", "گىرۇزىيە يازلىق ۋاقتى"}, {"Greenland_Eastern", "شەرقىي گىرېنلاند يازلىق ۋاقتى"}, {"Greenland_Western", "غەربىي گىرېنلاند يازلىق ۋاقتى"}, {"Hawaii_Aleutian", "ھاۋاي-ئالېيۇت يازلىق ۋاقتى"}, {"Hong_Kong", "شياڭگاڭ يازلىق ۋاقتى"}, {"Hovd", "خوۋد يازلىق ۋاقتى"}, {"Iran", "ئىران يازلىق ۋاقتى"}, {"Irkutsk", "ئىركۇتسك يازلىق ۋاقتى"}, {"Israel", "ئىسرائىلىيە يازلىق ۋاقتى"}, {"Japan", "ياپونىيە يازلىق ۋاقتى"}, {"Kamchatka", "پېتروپاۋلوۋسك-كامچاتكسكى يازلىق ۋاقتى"}, {"Korea", "كورىيە يازلىق ۋاقتى"}, {"Krasnoyarsk", "كىراسنويارسك يازلىق ۋاقتى"}, {"Lord_Howe", "لورد-خاي يازلىق ۋاقتى"}, {"Macau", "ئاۋمېن يازلىق ۋاقتى"}, {"Magadan", "ماگادان يازلىق ۋاقتى"}, {"Mauritius", "ماۋرىتىئۇس يازلىق ۋاقتى"}, {"Mexico_Pacific", "مېكسىكا تىنچ ئوكيان يازلىق ۋاقتى"}, {"Mongolia", "ئۇلانباتور يازلىق ۋاقتى"}, {"Moscow", "موسكۋا يازلىق ۋاقتى"}, {"New_Caledonia", "يېڭى كالېدونىيە يازلىق ۋاقتى"}, {"New_Zealand", "يېڭى زېلاندىيە يازلىق ۋاقتى"}, {"Newfoundland", "نىۋفوئۇنلاند يازلىق ۋاقتى"}, {"Norfolk", "نورفولك ئاراللىرى يازلىق ۋاقتى"}, {"Noronha", "فېرناندو-نورونخا يازلىق ۋاقتى"}, {"Novosibirsk", "نوۋوسىبىرسك يازلىق ۋاقتى"}, {"Omsk", "ئومسك يازلىق ۋاقتى"}, {"Pakistan", "پاكىستان يازلىق ۋاقتى"}, {"Paraguay", "پاراگۋاي يازلىق ۋاقتى"}, {"Peru", "پېرۇ يازلىق ۋاقتى"}, {"Philippines", "فىلىپپىن يازلىق ۋاقتى"}, {"Pierre_Miquelon", "ساينىت پىئېر ۋە مىكېلون يازلىق ۋاقتى"}, {"Qyzylorda", "قىزىلئوردا يازلىق ۋاقتى"}, {"Sakhalin", "ساخارىن يازلىق ۋاقتى"}, {"Samara", "سامارا يازلىق ۋاقتى"}, {"Samoa", "سەمەرقەنت يازلىق ۋاقتى"}, {"Taipei", "تەيبېي يازلىق ۋاقتى"}, {"Tonga", "تونگا يازلىق ۋاقتى"}, {"Turkmenistan", "تۈركمەنىستان يازلىق ۋاقتى"}, {"Uruguay", "ئۇرۇگۋاي يازلىق ۋاقتى"}, {"Uzbekistan", "ئۆزبېكىستان يازلىق ۋاقتى"}, {"Vanuatu", "ۋانۇئاتۇ يازلىق ۋاقتى"}, {"Vladivostok", "ۋىلادىۋوستوك يازلىق ۋاقتى"}, {"Volgograd", "ۋولگاگراد يازلىق ۋاقتى"}, {"Yakutsk", "ياكۇتسك يازلىق ۋاقتى"}, {"Yekaterinburg", "يېكاتېرىنبۇرگ يازلىق ۋاقتى"}},
	},
	{
		{{"Acre", "час: Акрі, стандартний"}, {"Afghanistan", "за часом в Афганістані"}, {"Africa_Central", "за центральноафриканським часом"}, {"Africa_Eastern", "за східноафриканським часом"}, {"Africa_Southern", "за південноафриканським часом"}, {"Africa_Western", "за західноафриканським часом"}, {"Alaska", "за стандартним часом на Алясці"}, {"Amazon", "за стандартним часом на Амазонці"}, {"America_Central", "за північноамериканським центральним стандартним часом"}, {"America_Eastern", "за північноамериканським східним стандартним часом"}, {"America_Mountain", "за північноамериканським гірським стандартним часом"}, {"America_Pacific", "за північноамериканським тихоокеанським стандартним часом"}, {"Anadyr", "час: Анадир, стандартний"}, {"Apia", "за стандартним часом в Апіа"}, {"Arabian", "за арабським стандартним часом"}, {"Argentina", "за стандартним аргентинським часом"}, {"Argentina_Western", "за стандартним західноаргентинським часом"}, {"Armenia", "за вірменським стандартним часом"}, {"Atlantic", "за атлантичним стандартним часом"}, {"Australia_Central", "за стандартним центральноавстралійським часом"}, {"Australia_CentralWestern", "за стандартним центральнозахідним австралійським часом"}, {"Australia_Eastern", "за стандартним східноавстралійським часом"}, {"Australia_Western", "за стандартним західноавстралійським часом"}, {"Azerbaijan", "за стандартним азербайджанським часом"}, {"Azores", "за стандартним часом на Азорських островах"}, {"Bangladesh", "за стандартним часом у Бангладеш"}, {"Bhutan", "за часом у Бутані"}, {"Bolivia", "за болівійським часом"}, {"Brasilia", "за стандартним бразильським часом"}, {"Brunei", "за часом у Брунеї"}, {"Cape_Verde", "за стандартним часом на островах Кабо-Верде"}, {"Chamorro", "за часом на Північних Маріанських островах"}, {"Chatham", "за стандартним часом на архіпелазі Чатем"}, {"Chile", "за стандартним чилійським часом"}, {"China", "за китайським стандартним часом"}, {"Christmas", "за часом на острові Різдва"}, {"Cocos", "за часом на Кокосових островах"}, {"Colombia", "за стандартним колумбійським часом"}, {"Cook", "за стандартним часом на Островах Кука"}, {"Cuba", "за стандартним часом на Кубі"}, {"Davis", "за часом на станції Девіс"}, {"DumontDUrville", "за часом на станції Дюмон дʼЮрвіль"}, {"East_Timor", "за часом у Східному Тиморі"}, {"Easter", "за стандартним часом на острові Пасхи"}, {"Ecuador", "за часом в Еквадорі"}, {"Europe_Central", "за центральноєвропейським стандартним часом"}, {"Europe_Eastern", "за східноєвропейським стандартним часом"}, {"Europe_Further_Eastern", "за далекосхідним європейським часом"}, {"Europe_Western", "за західноєвропейським стандартним часом"}, {"Falkland", "за стандартним часом на Фолклендських Островах"}, {"Fiji", "за стандартним часом у Фіджі"}, {"French_Guiana", "за часом Французької Ґвіани"}, {"French_Southern", "за часом на Французьких Південних і Антарктичних територіях"}, {"GMT", "за Гринвічем"}, {"Galapagos", "за часом Ґалапаґосу"}, {"Gambier", "за часом на острові Ґамбʼє"}, {"Georgia", "за стандартним грузинським часом"}, {"Gilbert_Islands", "за часом на островах Гілберта"}, {"Greenland_Eastern", "за стандартним східним часом у Ґренландії"}, {"Greenland_Western", "за стандартним західним часом у Ґренландії"}, {"Guam", "за часом на острові Гуам"}, {"Gulf", "за часом Перської затоки"}, {"Guyana", "за часом у Ґаяні"}, {"Hawaii", "за стандартним гавайсько-алеутським часом"}, {"Hawaii_Aleutian", "за стандартним гавайсько-алеутським часом"}, {"Hong_Kong", "за стандартним часом у Гонконзі"}, {"Hovd", "за стандартним часом у Ховді"}, {"India", "за індійським стандартним часом"}, {"Indian_Ocean", "за часом в Індійському Океані"}, {"Indochina", "за часом в Індокитаї"}, {"Indonesia_Central", "за центральноіндонезійським часом"}, {"Indonesia_Eastern", "за східноіндонезійським часом"}, {"Indonesia_Western", "за західноіндонезійським часом"}, {"Iran", "за іранським стандартним часом"}, {"Irkutsk", "за іркутським стандартним часом"}, {"Israel", "за ізраїльським стандартним часом"}, {"Japan", "за японським стандартним часом"}, {"Kamchatka", "за камчатським стандартним часом"}, {"Kazakhstan", "за часом у Казахстані"}, {"Kazakhstan_Eastern", "за східним часом у Казахстані"}, {"Kazakhstan_Western", "за західним часом у Казахстані"}, {"Korea", "за корейським стандартним часом"}, {"Kosrae", "за часом на острові Косрае"}, {"Krasnoyarsk", "за красноярським стандартним часом"}, {"Kyrgystan", "за часом у Киргизстані"}, {"Lanka", "час: Ланка"}, {"Line_Islands", "за часом на острові Лайн"}, {"Lord_Howe", "за стандартним часом на острові Лорд-Хау"}, {"Magadan", "за магаданським стандартним часом"}, {"Malaysia", "за часом у Малайзії"}, {"Maldives", "за часом на Мальдівах"}, {"Marquesas", "за часом на Маркізьких островах"}, {"Marshall_Islands", "за часом на Маршаллових Островах"}, {"Mauritius", "за стандартним часом на острові Маврикій"}, {"Mawson", "за часом на станції Моусон"}, {"Mexico_Pacific", "за стандартним тихоокеанським часом у Мексиці"}, {"Mongolia", "за стандартним часом в Улан-Баторі"}, {"Moscow", "за московським стандартним часом"}, {"Myanmar", "за часом у Мʼянмі"}, {"Nauru", "за часом на острові Науру"}, {"Nepal", "за часом у Непалі"}, {"New_Caledonia", "за стандартним часом на островах Нової Каледонії"}, {"New_Zealand", "за стандартним часом у Новій Зеландії"}, {"Newfoundland", "за стандартним часом на острові Ньюфаундленд"}, {"Niue", "за часом на острові Ніуе"}, {"Norfolk", "за стандартним часом на острові Норфолк"}, {"Noronha", "за стандартним часом на архіпелазі Фернанду-ді-Норонья"}, {"Novosibirsk", "за новосибірським стандартним часом"}, {"Omsk", "за омським стандартним часом"}, {"Pakistan", "за стандартним часом у Пакистані"}, {"Palau", "за часом на острові Палау"}, {"Papua_New_Guinea", "за часом на островах Папуа-Нова Ґвінея"}, {"Paraguay", "за стандартним параґвайським часом"}, {"Peru", "за стандартним часом у Перу"}, {"Philippines", "за стандартним часом на Філіппінах"}, {"Phoenix_Islands", "за часом на островах Фенікс"}, {"Pierre_Miquelon", "за стандартним часом на островах Сен-П’єр і Мікелон"}, {"Pitcairn", "за часом на островах Піткерн"}, {"Ponape", "за часом на острові Понапе"}, {"Pyongyang", "за часом у Пхеньяні"}, {"Qyzylorda", "час: Кизилорда, стандартний"}, {"Reunion", "за часом на острові Реюньйон"}, {"Rothera", "за часом на станції Ротера"}, {"Sakhalin", "за сахалінським стандартним часом"}, {"Samara", "за самарським стандартним часом"}, {"Samoa", "за стандартним часом на острові Самоа"}, {"Seychelles", "за часом на Сейшельських Островах"}, {"Singapore", "за часом у Сінгапурі"}, {"Solomon", "за часом на Соломонових Островах"}, {"South_Georgia", "за часом на острові Південна Джорджія"}, {"Suriname", "за часом у Суринамі"}, {"Syowa", "за часом на станції Сева"}, {"Tahiti", "за часом на острові Таїті"}, {"Taipei", "за стандартним часом у Тайбеї"}, {"Tajikistan", "за часом у Таджикистані"}, {"Tokelau", "за часом на островах Токелау"}, {"Tonga", "за стандартним часом на островах Тонга"}, {"Truk", "за часом на островах Чуук"}, {"Turkey", "за турецьким стандартним часом"}, {"Turkmenistan", "за стандартним часом у Туркменістані"}, {"Tuvalu", "за часом на островах Тувалу"}, {"Uruguay", "за стандартним часом в Уруґваї"}, {"Uzbekistan", "за стандартним часом в Узбекистані"}, {"Vanuatu", "за стандартним часом на островах Вануату"}, {"Venezuela", "за часом у Венесуелі"}, {"Vladivostok", "за владивостоцьким стандартним часом"}, {"Volgograd", "за волгоградським стандартним часом"}, {"Vostok", "за часом на станції Восток"}, {"Wake", "за часом на острові Вейк"}, {"Wallis", "за часом на островах Уолліс і Футуна"}, {"Yakutsk", "за якутським стандартним часом"}, {"Yekaterinburg", "за єкатеринбурзьким стандартним часом"}, {"Yukon", "за стандартним часом на Юконі"}},
		{{"Acre", "час: Акрі, літній"}, {"Alaska", "за літнім часом на Алясці"}, {"Amazon", "за літнім часом на Амазонці"}, {"America_Central", "за північноамериканським центральним літнім часом"}, {"America_Eastern", "за північноамериканським східним літнім часом"}, {"America_Mountain", "за північноамериканським гірським літнім часом"}, {"America_Pacific", "за північноамериканським тихоокеанським літнім часом"}, {"Anadyr", "час: Анадир, літній"}, {"Apia", "за літнім часом в Апіа"}, {"Arabian", "за арабським літнім часом"}, {"Argentina", "за літнім аргентинським часом"}, {"Argentina_Western", "за літнім за західноаргентинським часом"}, {"Armenia", "за вірменським літнім часом"}, {"Atlantic", "за атлантичним літнім часом"}, {"Australia_Central", "за літнім центральноавстралійським часом"}, {"Australia_CentralWestern", "за літнім центральнозахідним австралійським часом"}, {"Australia_Eastern", "за літнім східноавстралійським часом"}, {"Australia_Western", "за літнім західноавстралійським часом"}, {"Azerbaijan", "за літнім азербайджанським часом"}, {"Azores", "за літнім часом на Азорських островах"}, {"Bangladesh", "за літнім часом у Бангладеш"}, {"Brasilia", "за літнім бразильським часом"}, {"Cape_Verde", "за літнім часом на островах Кабо-Верде"}, {"Chatham", "за літнім часом на архіпелазі Чатем"}, {"Chile", "за літнім чилійським часом"}, {"China", "за китайським літнім часом"}, {"Colombia", "за літнім колумбійським часом"}, {"Cook", "за літнім часом на Островах Кука"}, {"Cuba", "за літнім часом на Кубі"}, {"Easter", "за літнім часом на острові Пасхи"}, {"Europe_Central", "за центральноєвропейським літнім часом"}, {"Europe_Eastern", "за східноєвропейським літнім часом"}, {"Europe_Western", "за західноєвропейським літнім часом"}, {"Falkland", "за літнім часом на Фолклендських Островах"}, {"Fiji", "за літнім часом у Фіджі"}, {"Georgia", "за літнім грузинським часом"}, {"Greenland_Eastern", "за літнім східним часом у Ґренландії"}, {"Greenland_Western", "за літнім західним часом у Ґренландії"}, {"Hawaii_Aleutian", "за літнім гавайсько-алеутським часом"}, {"Hong_Kong", "за літнім часом у Гонконзі"}, {"Hovd", "за літнім часом у Ховді"}, {"Iran", "за іранським літнім часом"}, {"Irkutsk", "за іркутським літнім часом"}, {"Israel", "за ізраїльським літнім часом"}, {"Japan", "за японським літнім часом"}, {"Kamchatka", "за камчатським літнім часом"}, {"Korea", "за корейським літнім часом"}, {"Krasnoyarsk", "за красноярським літнім часом"}, {"Lord_Howe", "за літнім часом на острові Лорд-Хау"}, {"Magadan", "за магаданським літнім часом"}, {"Mauritius", "за літнім часом на острові Маврикій"}, {"Mexico_Pacific", "за літнім тихоокеанським часом у Мексиці"}, {"Mongolia", "за літнім часом в Улан-Баторі"}, {"Moscow", "за московським літнім часом"}, {"New_Caledonia", "за літнім часом на островах Нової Каледонії"}, {"New_Zealand", "за літнім часом у Новій Зеландії"}, {"Newfoundland", "за літнім часом у Ньюфаундленд"}, {"Norfolk", "за літнім часом на острові Норфолк"}, {"Noronha", "за літнім часом на архіпелазі Фернанду-ді-Норонья"}, {"Novosibirsk", "за новосибірським літнім часом"}, {"Omsk", "за омським літнім часом"}, {"Pakistan", "за літнім часом у Пакистані"}, {"Paraguay", "за літнім параґвайським часом"}, {"Peru", "за літнім часом у Перу"}, {"Philippines", "за літнім часом на Філіппінах"}, {"Pierre_Miquelon", "за літнім часом на островах Сен-П’єр і Мікелон"}, {"Qyzylorda", "час: Кизилорда, літній"}, {"Sakhalin", "за сахалінським літнім часом"}, {"Samara", "за самарським літнім часом"}, {"Samoa", "за літнім часом на острові Самоа"}, {"Taipei", "за літнім часом у Тайбеї"}, {"Tonga", "за літнім часом на островах Тонга"}, {"Turkey", "за турецьким літнім часом"}, {"Turkmenistan", "за літнім часом у Туркменістані"}, {"Uruguay", "за літнім часом в Уруґваї"}, {"Uzbekistan", "за літнім часом в Узбекистані"}, {"Vanuatu", "за літнім часом на островах Вануату"}, {"Vladivostok", "за владивостоцьким літнім часом"}, {"Volgograd", "за волгоградським літнім часом"}, {"Yakutsk", "за якутським літнім часом"}, {"Yekaterinburg", "за єкатеринбурзьким літнім часом"}},
	},
	{
		{{"Afghanistan", "افغانستان کا وقت"}, {"Africa_Central", "وسطی افریقہ ٹائم"}, {"Africa_Eastern", "مشرقی افریقہ ٹائم"}, {"Africa_Southern", "جنوبی افریقہ سٹینڈرڈ ٹائم"}, {"Africa_Western", "مغربی افریقہ ٹائم"}, {"Alaska", "الاسکا اسٹینڈرڈ ٹائم"}, {"Amazon", "ایمیزون کا معیاری وقت"}, {"America_Central", "سنٹرل اسٹینڈرڈ ٹائم"}, {"America_Eastern", "ایسٹرن اسٹینڈرڈ ٹائم"}, {"America_Mountain", "ماؤنٹین اسٹینڈرڈ ٹائم"}, {"America_Pacific", "پیسفک اسٹینڈرڈ ٹائم"}, {"Anadyr", "انیدر اسٹینڈرڈ ٹائم"}, {"Apia", "ایپیا سٹینڈرڈ ٹائم"}, {"Arabian", "عرب کا معیاری وقت"}, {"Argentina", "ارجنٹینا کا معیاری وقت"}, {"Argentina_Western", "مغربی ارجنٹینا کا معیاری وقت"}, {"Armenia", "آرمینیا کا معیاری وقت"}, {"Atlantic", "اٹلانٹک اسٹینڈرڈ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل اسٹینڈرڈ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Western", "آسٹریلیا ویسٹرن اسٹینڈرڈ ٹائم"}, {"Azerbaijan", "آذربائیجان کا معیاری وقت"}, {"Azores", "ازوریس کا معیاری وقت"}, {"Bangladesh", "بنگلہ دیش کا معیاری وقت"}, {"Bhutan", "بھوٹان کا وقت"}, {"Bolivia", "بولیویا کا وقت"}, {"Brasilia", "برازیلیا اسٹینڈرڈ ٹائم"}, {"Brunei", "برونئی دارالسلام ٹائم"}, {"Cape_Verde", "کیپ ورڈی سٹینڈرڈ ٹائم"}, {"Chamorro", "چامورو سٹینڈرڈ ٹائم"}, {"Chatham", "چیتھم اسٹینڈرڈ ٹائم"}, {"Chile", "چلی کا معیاری وقت"}, {"China", "چین سٹینڈرڈ ٹائم"}, {"Christmas", "کرسمس آئلینڈ ٹائم"}, {"Cocos", "کوکوس آئلینڈز ٹائم"}, {"Colombia", "کولمبیا کا معیاری وقت"}, {"Cook", "کک آئلینڈز سٹینڈرڈ ٹائم"}, {"Cuba", "کیوبا اسٹینڈرڈ ٹائم"}, {"Davis", "ڈیوس ٹائم"}, {"DumontDUrville", "ڈومونٹ-ڈی’ارویلے ٹائم"}, {"East_Timor", "مشرقی تیمور ٹائم"}, {"Easter", "ایسٹر آئلینڈ کا معیاری وقت"}, {"Ecuador", "ایکواڈور کا وقت"}, {"Europe_Central", "وسطی یورپ کا معیاری وقت"}, {"Europe_Eastern", "مشرقی یورپ کا معیاری وقت"}, {"Europe_Further_Eastern", "بعید مشرقی یورپی وقت"}, {"Europe_Western", "مغربی یورپ کا معیاری وقت"}, {"Falkland", "فاک لینڈ آئلینڈز کا معیاری وقت"}, {"Fiji", "فجی سٹینڈرڈ ٹائم"}, {"French_Guiana", "فرینچ گیانا کا وقت"}, {"French_Southern", "فرینچ جنوبی اور انٹارکٹک ٹائم"}, {"GMT", "گرین وچ کا اصل وقت"}, {"Galapagos", "گالاپاگوز کا وقت"}, {"Gambier", "گیمبیئر ٹائم"}, {"Georgia", "جارجیا کا معیاری وقت"}, {"Gilbert_Islands", "جلبرٹ آئلینڈز ٹائم"}, {"Greenland_Eastern", "مشرقی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Greenland_Western", "مغربی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Gulf", "خلیج کا معیاری وقت"}, {"Guyana", "گیانا کا وقت"}, {"Hawaii", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hong_Kong", "ہانگ کانگ سٹینڈرڈ ٹائم"}, {"Hovd", "ہووڈ سٹینڈرڈ ٹائم"}, {"India", "ہندوستان کا معیاری وقت"}, {"Indian_Ocean", "بحر ہند ٹائم"}, {"Indochina", "ہند چین ٹائم"}, {"Indonesia_Central", "وسطی انڈونیشیا ٹائم"}, {"Indonesia_Eastern", "مشرقی انڈونیشیا ٹائم"}, {"Indonesia_Western", "مغربی انڈونیشیا ٹائم"}, {"Iran", "ایران کا معیاری وقت"}, {"Irkutsk", "ارکتسک سٹینڈرڈ ٹائم"}, {"Israel", "اسرائیل کا معیاری وقت"}, {"Japan", "جاپان سٹینڈرڈ ٹائم"}, {"Kamchatka", "پیٹروپاؤلووسک-کیمچسکی اسٹینڈرڈ ٹائم"}, {"Kazakhstan", "قازقستان کا وقت"}, {"Kazakhstan_Eastern", "مشرقی قزاخستان کا وقت"}, {"Kazakhstan_Western", "مغربی قزاخستان کا وقت"}, {"Korea", "کوریا سٹینڈرڈ ٹائم"}, {"Kosrae", "کوسرے ٹائم"}, {"Krasnoyarsk", "کرسنویارسک سٹینڈرڈ ٹائم"}, {"Kyrgystan", "کرغستان کا وقت"}, {"Line_Islands", "لائن آئلینڈز ٹائم"}, {"Lord_Howe", "لارڈ ہووے اسٹینڈرڈ ٹائم"}, {"Magadan", "مگادان اسٹینڈرڈ ٹائم"}, {"Malaysia", "ملیشیا ٹائم"}, {"Maldives", "مالدیپ کا وقت"}, {"Marquesas", "مارکیسس ٹائم"}, {"Marshall_Islands", "مارشل آئلینڈز ٹائم"}, {"Mauritius", "ماریشس سٹینڈرڈ ٹائم"}, {"Mawson", "ماؤسن ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک اسٹینڈرڈ ٹائم"}, {"Mongolia", "یولان بیتور سٹینڈرڈ ٹائم"}, {"Moscow", "ماسکو اسٹینڈرڈ ٹائم"}, {"Myanmar", "میانمار ٹائم"}, {"Nauru", "ناؤرو ٹائم"}, {"Nepal", "نیپال کا وقت"}, {"New_Caledonia", "نیو کیلیڈونیا سٹینڈرڈ ٹائم"}, {"New_Zealand", "نیوزی لینڈ کا معیاری وقت"}, {"Newfoundland", "نیو فاؤنڈ لینڈ اسٹینڈرڈ ٹائم"}, {"Niue", "نیئو ٹائم"}, {"Norfolk", "نارفوک آئلینڈ کا معیاری وقت"}, {"Noronha", "فرنانڈو ڈی نورنہا کا معیاری وقت"}, {"Novosibirsk", "نوووسیبرسک سٹینڈرڈ ٹائم"}, {"Omsk", "اومسک سٹینڈرڈ ٹائم"}, {"Pakistan", "پاکستان کا معیاری وقت"}, {"Palau", "پلاؤ ٹائم"}, {"Papua_New_Guinea", "پاپوآ نیو گنی ٹائم"}, {"Paraguay", "پیراگوئے کا معیاری وقت"}, {"Peru", "پیرو کا معیاری وقت"}, {"Philippines", "فلپائن سٹینڈرڈ ٹائم"}, {"Phoenix_Islands", "فینکس آئلینڈز ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر اور مکلیئون اسٹینڈرڈ ٹائم"}, {"Pitcairn", "پٹکائرن ٹائم"}, {"Ponape", "پوناپے ٹائم"}, {"Pyongyang", "پیانگ یانگ وقت"}, {"Reunion", "ری یونین ٹائم"}, {"Rothera", "روتھیرا کا وقت"}, {"Sakhalin", "سخالین سٹینڈرڈ ٹائم"}, {"Samara", "سمارا اسٹینڈرڈ ٹائم"}, {"Samoa", "ساموآ سٹینڈرڈ ٹائم"}, {"Seychelles", "سیشلیز ٹائم"}, {"Singapore", "سنگاپور سٹینڈرڈ ٹائم"}, {"Solomon", "سولمن آئلینڈز ٹائم"}, {"South_Georgia", "جنوبی جارجیا ٹائم"}, {"Suriname", "سورینام کا وقت"}, {"Syowa", "سیووا ٹائم"}, {"Tahiti", "تاہیتی ٹائم"}, {"Taipei", "تائی پیئی اسٹینڈرڈ ٹائم"}, {"Tajikistan", "تاجکستان کا وقت"}, {"Tokelau", "ٹوکیلاؤ ٹائم"}, {"Tonga", "ٹونگا سٹینڈرڈ ٹائم"}, {"Truk", "چوک ٹائم"}, {"Turkmenistan", "ترکمانستان کا معیاری وقت"}, {"Tuvalu", "ٹوالو ٹائم"}, {"Uruguay", "یوروگوئے کا معیاری وقت"}, {"Uzbekistan", "ازبکستان کا معیاری وقت"}, {"Vanuatu", "وانوآٹو سٹینڈرڈ ٹائم"}, {"Venezuela", "وینزوئیلا کا وقت"}, {"Vladivostok", "ولادی ووستک سٹینڈرڈ ٹائم"}, {"Volgograd", "وولگوگراد اسٹینڈرڈ ٹائم"}, {"Vostok", "ووسٹاک کا وقت"}, {"Wake", "ویک آئلینڈ ٹائم"}, {"Wallis", "والیز اور فٹونا ٹائم"}, {"Yakutsk", "یکوتسک اسٹینڈرڈ ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ اسٹینڈرڈ ٹائم"}, {"Yukon", "یوکون ٹائم"}},
		{{"Alaska", "الاسکا ڈے لائٹ ٹائم"}, {"Amazon", "امیزون کا موسم گرما کا وقت"}, {"America_Central", "سنٹرل ڈے لائٹ ٹائم"}, {"America_Eastern", "ایسٹرن ڈے لائٹ ٹائم"}, {"America_Mountain", "ماؤنٹین ڈے لائٹ ٹائم"}, {"America_Pacific", "پیسفک ڈے لائٹ ٹائم"}, {"Anadyr", "انیدر سمر ٹائم"}, {"Apia", "ایپیا ڈے لائٹ ٹائم"}, {"Arabian", "عرب ڈے لائٹ ٹائم"}, {"Argentina", "ارجنٹینا کا موسم گرما کا وقت"}, {"Argentina_Western", "مغربی ارجنٹینا کا موسم گرما کا وقت"}, {"Armenia", "آرمینیا کا موسم گرما کا وقت"}, {"Atlantic", "اٹلانٹک ڈے لائٹ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل ڈے لائٹ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن ڈے لائٹ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن ڈے لائٹ ٹائم"}, {"Australia_Western", "آسٹریلین ویسٹرن ڈے لائٹ ٹائم"}, {"Azerbaijan", "آذربائیجان کا موسم گرما کا وقت"}, {"Azores", "ازوریس کا موسم گرما کا وقت"}, {"Bangladesh", "بنگلہ دیش کا موسم گرما کا وقت"}, {"Brasilia", "برازیلیا سمر ٹائم"}, {"Cape_Verde", "کیپ ورڈی سمر ٹائم"}, {"Chatham", "چیتھم ڈے لائٹ ٹائم"}, {"Chile", "چلی کا موسم گرما کا وقت"}, {"China", "چینی ڈے لائٹ ٹائم"}, {"Colombia", "کولمبیا کا موسم گرما کا وقت"}, {"Cook", "کک آئلینڈز نصف سمر ٹائم"}, {"Cuba", "کیوبا ڈے لائٹ ٹائم"}, {"Easter", "ایسٹر آئلینڈ کا موسم گرما کا وقت"}, {"Europe_Central", "وسطی یورپ کا موسم گرما کا وقت"}, {"Europe_Eastern", "مشرقی یورپ کا موسم گرما کا وقت"}, {"Europe_Western", "مغربی یورپ کا موسم گرما کا وقت"}, {"Falkland", "فاک لینڈ آئلینڈز کا موسم گرما کا وقت"}, {"Fiji", "فجی سمر ٹائم"}, {"Georgia", "جارجیا کا موسم گرما کا وقت"}, {"Greenland_Eastern", "مشرقی گرین لینڈ کا موسم گرما کا وقت"}, {"Greenland_Western", "مغربی گرین لینڈ کا موسم گرما کا وقت"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن ڈے لائٹ ٹائم"}, {"Hong_Kong", "ہانگ کانگ سمر ٹائم"}, {"Hovd", "ہووڈ سمر ٹائم"}, {"Iran", "ایران ڈے لائٹ ٹائم"}, {"Irkutsk", "ارکتسک سمر ٹائم"}, {"Israel", "اسرائیل ڈے لائٹ ٹائم"}, {"Japan", "جاپان ڈے لائٹ ٹائم"}, {"Kamchatka", "پیٹروپاؤلووسک-کیمچسکی سمر ٹائم"}, {"Korea", "کوریا ڈے لائٹ ٹائم"}, {"Krasnoyarsk", "کریسنویارسک سمر ٹائم"}, {"Lord_Howe", "لارڈ ہووے ڈے لائٹ ٹائم"}, {"Magadan", "میگیدن سمر ٹائم"}, {"Mauritius", "ماریشس سمر ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک ڈے لائٹ ٹائم"}, {"Mongolia", "یولان بیتور سمر ٹائم"}, {"Moscow", "ماسکو سمر ٹائم"}, {"New_Caledonia", "نیو کیلیڈونیا سمر ٹائم"}, {"New_Zealand", "نیوزی لینڈ ڈے لائٹ ٹائم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ ڈے لائٹ ٹائم"}, {"Norfolk", "نارفوک آئلینڈ ڈے لائٹ وقت"}, {"Noronha", "فرنانڈو ڈی نورونہا سمر ٹائم"}, {"Novosibirsk", "نوووسیبرسک سمر ٹائم"}, {"Omsk", "اومسک سمر ٹائم"}, {"Pakistan", "پاکستان کا موسم گرما کا وقت"}, {"Paraguay", "پیراگوئے کا موسم گرما کا وقت"}, {"Peru", "پیرو کا موسم گرما کا وقت"}, {"Philippines", "فلپائن سمر ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر اور مکلیئون ڈے لائٹ ٹائم"}, {"Sakhalin", "سخالین سمر ٹائم"}, {"Samara", "سمارا سمر ٹائم"}, {"Samoa", "ساموآ ڈے لائٹ ٹائم"}, {"Taipei", "تئی پیئی ڈے لائٹ ٹائم"}, {"Tonga", "ٹونگا سمر ٹائم"}, {"Turkmenistan", "ترکمانستان کا موسم گرما کا وقت"}, {"Uruguay", "یوروگوئے کا موسم گرما کا وقت"}, {"Uzbekistan", "ازبکستان کا موسم گرما کا وقت"}, {"Vanuatu", "وانوآٹو سمر ٹائم"}, {"Vladivostok", "ولادی ووستک سمر ٹائم"}, {"Volgograd", "وولگوگراد سمر ٹائم"}, {"Yakutsk", "یکوتسک سمر ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ سمر ٹائم"}},
	},
	{
		{{"Afghanistan", "افغانستان ٹائم"}, {"Africa_Central", "وسطی افریقہ ٹائم"}, {"Africa_Eastern", "مشرقی افریقہ ٹائم"}, {"Africa_Southern", "جنوبی افریقہ سٹینڈرڈ ٹائم"}, {"Africa_Western", "مغربی افریقہ ٹائم"}, {"Alaska", "الاسکا اسٹینڈرڈ ٹائم"}, {"Amazon", "ایمیزون سٹینڈرڈ ٹائم"}, {"America_Central", "سنٹرل اسٹینڈرڈ ٹائم"}, {"America_Eastern", "ایسٹرن اسٹینڈرڈ ٹائم"}, {"America_Mountain", "ماؤنٹین اسٹینڈرڈ ٹائم"}, {"America_Pacific", "پیسفک اسٹینڈرڈ ٹائم"}, {"Anadyr", "انیدر اسٹینڈرڈ ٹائم"}, {"Apia", "ایپیا سٹینڈرڈ ٹائم"}, {"Arabian", "عرب سٹینڈرڈ ٹائم"}, {"Argentina", "ارجنٹینا کا معیاری وقت"}, {"Argentina_Western", "مغربی ارجنٹینا سٹینڈرڈ ٹائم"}, {"Armenia", "آرمینیا سٹینڈرڈ ٹائم"}, {"Atlantic", "اٹلانٹک اسٹینڈرڈ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل اسٹینڈرڈ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن اسٹینڈرڈ ٹائم"}, {"Australia_Western", "آسٹریلیا ویسٹرن اسٹینڈرڈ ٹائم"}, {"Azerbaijan", "آذربائیجان سٹینڈرڈ ٹائم"}, {"Azores", "ازوریس کا معیاری وقت"}, {"Bangladesh", "بنگلہ دیش سٹینڈرڈ ٹائم"}, {"Bhutan", "بھوٹان ٹائم"}, {"Bolivia", "بولیویا ٹائم"}, {"Brasilia", "برازیلیا سٹینڈرڈ ٹائم"}, {"Brunei", "برونئی دارالسلام ٹائم"}, {"Cape_Verde", "کیپ ورڈی سٹینڈرڈ ٹائم"}, {"Chamorro", "چامورو سٹینڈرڈ ٹائم"}, {"Chatham", "چیتھم اسٹینڈرڈ ٹائم"}, {"Chile", "چلی سٹینڈرڈ ٹائم"}, {"China", "چین سٹینڈرڈ ٹائم"}, {"Christmas", "کرسمس آئلینڈ ٹائم"}, {"Cocos", "کوکوس آئلینڈز ٹائم"}, {"Colombia", "کولمبیا سٹینڈرڈ ٹائم"}, {"Cook", "کک آئلینڈز سٹینڈرڈ ٹائم"}, {"Cuba", "کیوبا اسٹینڈرڈ ٹائم"}, {"Davis", "ڈیوس ٹائم"}, {"DumontDUrville", "ڈومونٹ-ڈی’ارویلے ٹائم"}, {"East_Timor", "مشرقی تیمور ٹائم"}, {"Easter", "ایسٹر آئلینڈ سٹینڈرڈ ٹائم"}, {"Ecuador", "ایکواڈور ٹائم"}, {"Europe_Central", "وسطی یورپ کا معیاری وقت"}, {"Europe_Eastern", "مشرقی یورپ کا معیاری وقت"}, {"Europe_Further_Eastern", "بعید مشرقی یورپی وقت"}, {"Europe_Western", "مغربی یورپ کا معیاری وقت"}, {"Falkland", "فاک لینڈ آئلینڈز سٹینڈرڈ ٹائم"}, {"Fiji", "فجی سٹینڈرڈ ٹائم"}, {"French_Guiana", "فرینچ گیانا ٹائم"}, {"French_Southern", "فرینچ جنوبی اور انٹارکٹک ٹائم"}, {"GMT", "گرین وچ مین ٹائم"}, {"Galapagos", "گالاپاگوز ٹائم"}, {"Gambier", "گیمبیئر ٹائم"}, {"Georgia", "جارجیا سٹینڈرڈ ٹائم"}, {"Gilbert_Islands", "جلبرٹ آئلینڈز ٹائم"}, {"Greenland_Eastern", "مشرقی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Greenland_Western", "مغربی گرین لینڈ اسٹینڈرڈ ٹائم"}, {"Gulf", "خلیج سٹینڈرڈ ٹائم"}, {"Guyana", "گیانا ٹائم"}, {"Hawaii", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن اسٹینڈرڈ ٹائم"}, {"Hong_Kong", "ہانگ کانگ سٹینڈرڈ ٹائم"}, {"Hovd", "ہووڈ سٹینڈرڈ ٹائم"}, {"India", "انڈیا سٹینڈرڈ ٹائم"}, {"Indian_Ocean", "بحر ہند ٹائم"}, {"Indochina", "ہند چین ٹائم"}, {"Indonesia_Central", "وسطی انڈونیشیا ٹائم"}, {"Indonesia_Eastern", "مشرقی انڈونیشیا ٹائم"}, {"Indonesia_Western", "مغربی انڈونیشیا ٹائم"}, {"Iran", "ایران سٹینڈرڈ ٹائم"}, {"Irkutsk", "ارکتسک سٹینڈرڈ ٹائم"}, {"Israel", "اسرائیل سٹینڈرڈ ٹائم"}, {"Japan", "جاپان سٹینڈرڈ ٹائم"}, {"Kamchatka", "پیٹروپاؤلووسک-کیمچسکی اسٹینڈرڈ ٹائم"}, {"Kazakhstan", "قازقستان کا وقت"}, {"Kazakhstan_Eastern", "مشرقی قزاخستان ٹائم"}, {"Kazakhstan_Western", "مغربی قزاخستان ٹائم"}, {"Korea", "کوریا سٹینڈرڈ ٹائم"}, {"Kosrae", "کوسرے ٹائم"}, {"Krasnoyarsk", "کرسنویارسک سٹینڈرڈ ٹائم"}, {"Kyrgystan", "کرغستان ٹائم"}, {"Line_Islands", "لائن آئلینڈز ٹائم"}, {"Lord_Howe", "لارڈ ہووے اسٹینڈرڈ ٹائم"}, {"Magadan", "مگادان اسٹینڈرڈ ٹائم"}, {"Malaysia", "ملیشیا ٹائم"}, {"Maldives", "مالدیپ ٹائم"}, {"Marquesas", "مارکیسس ٹائم"}, {"Marshall_Islands", "مارشل آئلینڈز ٹائم"}, {"Mauritius", "ماریشس سٹینڈرڈ ٹائم"}, {"Mawson", "ماؤسن ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک اسٹینڈرڈ ٹائم"}, {"Mongolia", "یولان بیتور سٹینڈرڈ ٹائم"}, {"Moscow", "ماسکو اسٹینڈرڈ ٹائم"}, {"Myanmar", "میانمار ٹائم"}, {"Nauru", "ناؤرو ٹائم"}, {"Nepal", "نیپال ٹائم"}, {"New_Caledonia", "نیو کیلیڈونیا سٹینڈرڈ ٹائم"}, {"New_Zealand", "نیوزی لینڈ سٹینڈرڈ ٹائم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ اسٹینڈرڈ ٹائم"}, {"Niue", "نیئو ٹائم"}, {"Norfolk", "نارفوک آئلینڈ کا معیاری وقت"}, {"Noronha", "فرنانڈو ڈی نورنہا سٹینڈرڈ ٹائم"}, {"Novosibirsk", "نوووسیبرسک سٹینڈرڈ ٹائم"}, {"Omsk", "اومسک سٹینڈرڈ ٹائم"}, {"Pakistan", "پاکستان سٹینڈرڈ ٹائم"}, {"Palau", "پلاؤ ٹائم"}, {"Papua_New_Guinea", "پاپوآ نیو گنی ٹائم"}, {"Paraguay", "پیراگوئے سٹینڈرڈ ٹائم"}, {"Peru", "پیرو سٹینڈرڈ ٹائم"}, {"Philippines", "فلپائن سٹینڈرڈ ٹائم"}, {"Phoenix_Islands", "فینکس آئلینڈز ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر اور مکلیئون اسٹینڈرڈ ٹائم"}, {"Pitcairn", "پٹکائرن ٹائم"}, {"Ponape", "پوناپے ٹائم"}, {"Pyongyang", "پیانگ یانگ وقت"}, {"Reunion", "ری یونین ٹائم"}, {"Rothera", "روتھیرا ٹائم"}, {"Sakhalin", "سخالین سٹینڈرڈ ٹائم"}, {"Samara", "سمارا اسٹینڈرڈ ٹائم"}, {"Samoa", "ساموآ سٹینڈرڈ ٹائم"}, {"Seychelles", "سیشلیز ٹائم"}, {"Singapore", "سنگاپور سٹینڈرڈ ٹائم"}, {"Solomon", "سولمن آئلینڈز ٹائم"}, {"South_Georgia", "جنوبی جارجیا ٹائم"}, {"Suriname", "سورینام ٹائم"}, {"Syowa", "سیووا ٹائم"}, {"Tahiti", "تاہیتی ٹائم"}, {"Taipei", "تائی پیئی اسٹینڈرڈ ٹائم"}, {"Tajikistan", "تاجکستان ٹائم"}, {"Tokelau", "ٹوکیلاؤ ٹائم"}, {"Tonga", "ٹونگا سٹینڈرڈ ٹائم"}, {"Truk", "چوک ٹائم"}, {"Turkmenistan", "ترکمانستان سٹینڈرڈ ٹائم"}, {"Tuvalu", "ٹوالو ٹائم"}, {"Uruguay", "یوروگوئے سٹینڈرڈ ٹائم"}, {"Uzbekistan", "ازبکستان سٹینڈرڈ ٹائم"}, {"Vanuatu", "وانوآٹو سٹینڈرڈ ٹائم"}, {"Venezuela", "وینزوئیلا ٹائم"}, {"Vladivostok", "ولادی ووستک سٹینڈرڈ ٹائم"}, {"Volgograd", "وولگوگراد اسٹینڈرڈ ٹائم"}, {"Vostok", "ووسٹاک ٹائم"}, {"Wake", "ویک آئلینڈ ٹائم"}, {"Wallis", "والیز اور فٹونا ٹائم"}, {"Yakutsk", "یکوتسک اسٹینڈرڈ ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ اسٹینڈرڈ ٹائم"}, {"Yukon", "یوکون ٹائم"}},
		{{"Alaska", "الاسکا ڈے لائٹ ٹائم"}, {"Amazon", "ایمیزون سمر ٹائم"}, {"America_Central", "سنٹرل ڈے لائٹ ٹائم"}, {"America_Eastern", "ایسٹرن ڈے لائٹ ٹائم"}, {"America_Mountain", "ماؤنٹین ڈے لائٹ ٹائم"}, {"America_Pacific", "پیسفک ڈے لائٹ ٹائم"}, {"Anadyr", "انیدر سمر ٹائم"}, {"Apia", "ایپیا ڈے لائٹ ٹائم"}, {"Arabian", "عرب ڈے لائٹ ٹائم"}, {"Argentina", "ارجنٹینا کا موسم گرما کا وقت"}, {"Argentina_Western", "مغربی ارجنٹینا سمر ٹائم"}, {"Armenia", "آرمینیا سمر ٹائم"}, {"Atlantic", "اٹلانٹک ڈے لائٹ ٹائم"}, {"Australia_Central", "آسٹریلین سنٹرل ڈے لائٹ ٹائم"}, {"Australia_CentralWestern", "آسٹریلین سنٹرل ویسٹرن ڈے لائٹ ٹائم"}, {"Australia_Eastern", "آسٹریلین ایسٹرن ڈے لائٹ ٹائم"}, {"Australia_Western", "آسٹریلین ویسٹرن ڈے لائٹ ٹائم"}, {"Azerbaijan", "آذربائیجان سمر ٹائم"}, {"Azores", "ازوریس کا موسم گرما کا وقت"}, {"Bangladesh", "بنگلہ دیش سمر ٹائم"}, {"Brasilia", "برازیلیا سمر ٹائم"}, {"Cape_Verde", "کیپ ورڈی سمر ٹائم"}, {"Chatham", "چیتھم ڈے لائٹ ٹائم"}, {"Chile", "چلی سمر ٹائم"}, {"China", "چینی ڈے لائٹ ٹائم"}, {"Colombia", "کولمبیا سمر ٹائم"}, {"Cook", "کک آئلینڈز نصف سمر ٹائم"}, {"Cuba", "کیوبا ڈے لائٹ ٹائم"}, {"Easter", "ایسٹر آئلینڈ سمر ٹائم"}, {"Europe_Central", "وسطی یورپ کا موسم گرما کا وقت"}, {"Europe_Eastern", "مشرقی یورپ کا موسم گرما کا وقت"}, {"Europe_Western", "مغربی یورپ کا موسم گرما کا وقت"}, {"Falkland", "فاک لینڈ آئلینڈز سمر ٹائم"}, {"Fiji", "فجی سمر ٹائم"}, {"Georgia", "جارجیا سمر ٹائم"}, {"Greenland_Eastern", "مشرقی گرین لینڈ کا موسم گرما کا وقت"}, {"Greenland_Western", "مغربی گرین لینڈ کا موسم گرما کا وقت"}, {"Hawaii_Aleutian", "ہوائی الیوٹیئن ڈے لائٹ ٹائم"}, {"Hong_Kong", "ہانگ کانگ سمر ٹائم"}, {"Hovd", "ہووڈ سمر ٹائم"}, {"Iran", "ایران ڈے لائٹ ٹائم"}, {"Irkutsk", "ارکتسک سمر ٹائم"}, {"Israel", "اسرائیل ڈے لائٹ ٹائم"}, {"Japan", "جاپان ڈے لائٹ ٹائم"}, {"Kamchatka", "پیٹروپاؤلووسک-کیمچسکی سمر ٹائم"}, {"Korea", "کوریا ڈے لائٹ ٹائم"}, {"Krasnoyarsk", "کریسنویارسک سمر ٹائم"}, {"Lord_Howe", "لارڈ ہووے ڈے لائٹ ٹائم"}, {"Magadan", "میگیدن سمر ٹائم"}, {"Mauritius", "ماریشس سمر ٹائم"}, {"Mexico_Pacific", "میکسیکن پیسفک ڈے لائٹ ٹائم"}, {"Mongolia", "یولان بیتور سمر ٹائم"}, {"Moscow", "ماسکو سمر ٹائم"}, {"New_Caledonia", "نیو کیلیڈونیا سمر ٹائم"}, {"New_Zealand", "نیوزی لینڈ ڈے لائٹ ٹائم"}, {"Newfoundland", "نیو فاؤنڈ لینڈ ڈے لائٹ ٹائم"}, {"Norfolk", "نارفوک آئلینڈ ڈے لائٹ وقت"}, {"Noronha", "فرنانڈو ڈی نورونہا سمر ٹائم"}, {"Novosibirsk", "نوووسیبرسک سمر ٹائم"}, {"Omsk", "اومسک سمر ٹائم"}, {"Pakistan", "پاکستان سمر ٹائم"}, {"Paraguay", "پیراگوئے سمر ٹائم"}, {"Peru", "پیرو سمر ٹائم"}, {"Philippines", "فلپائن سمر ٹائم"}, {"Pierre_Miquelon", "سینٹ پیئر اور مکلیئون ڈے لائٹ ٹائم"}, {"Sakhalin", "سخالین سمر ٹائم"}, {"Samara", "سمارا سمر ٹائم"}, {"Samoa", "ساموآ ڈے لائٹ ٹائم"}, {"Taipei", "تئی پیئی ڈے لائٹ ٹائم"}, {"Tonga", "ٹونگا سمر ٹائم"}, {"Turkmenistan", "ترکمانستان سمر ٹائم"}, {"Uruguay", "یوروگوئے سمر ٹائم"}, {"Uzbekistan", "ازبکستان سمر ٹائم"}, {"Vanuatu", "وانوآٹو سمر ٹائم"}, {"Vladivostok", "ولادی ووستک سمر ٹائم"}, {"Volgograd", "وولگوگراد سمر ٹائم"}, {"Yakutsk", "یکوتسک سمر ٹائم"}, {"Yekaterinburg", "یکاٹیرِنبرگ سمر ٹائم"}},
	},
	{
		{{"Afghanistan", "Afgʻoniston vaqti"}, {"Africa_Central", "Markaziy Afrika vaqti"}, {"Africa_Eastern", "Sharqiy Afrika vaqti"}, {"Africa_Southern", "Janubiy Afrika standart vaqti"}, {"Africa_Western", "Gʻarbiy Afrika vaqti"}, {"Alaska", "Alyaska standart vaqti"}, {"Amazon", "Amazonka standart vaqti"}, {"America_Central", "Markaziy Amerika standart vaqti"}, {"America_Eastern", "Sharqiy Amerika standart vaqti"}, {"America_Mountain", "Tog‘ standart vaqti (AQSH)"}, {"America_Pacific", "Tinch okeani standart vaqti"}, {"Apia", "Apia standart vaqti"}, {"Arabian", "Saudiya Arabistoni standart vaqti"}, {"Argentina", "Argentina standart vaqti"}, {"Argentina_Western", "Gʻarbiy Argentina standart vaqti"}, {"Armenia", "Armaniston standart vaqti"}, {"Atlantic", "Atlantika standart vaqti"}, {"Australia_Central", "Markaziy Avstraliya standart vaqti"}, {"Australia_CentralWestern", "Markaziy Avstraliya g‘arbiy standart vaqti"}, {"Australia_Eastern", "Sharqiy Avstraliya standart vaqti"}, {"Australia_Western", "G‘arbiy Avstraliya standart vaqti"}, {"Azerbaijan", "Ozarbayjon standart vaqti"}, {"Azores", "Azor orollari standart vaqti"}, {"Bangladesh", "Bangladesh standart vaqti"}, {"Bhutan", "Butan vaqti"}, {"Bolivia", "Boliviya vaqti"}, {"Brasilia", "Braziliya standart vaqti"}, {"Brunei", "Bruney-Dorussalom vaqti"}, {"Cape_Verde", "Kabo-Verde standart vaqti"}, {"Chamorro", "Chamorro standart vaqti"}, {"Chatham", "Chatem standart vaqti"}, {"Chile", "Chili standart vaqti"}, {"China", "Xitoy standart vaqti"}, {"Christmas", "Rojdestvo oroli vaqti"}, {"Cocos", "Kokos orollari vaqti"}, {"Colombia", "Kolumbiya standart vaqti"}, {"Cook", "Kuk orollari standart vaqti"}, {"Cuba", "Kuba standart vaqti"}, {"Davis", "Deyvis vaqti"}, {"DumontDUrville", "Dyumon-d’Yurvil vaqti"}, {"East_Timor", "Sharqiy Timor vaqti"}, {"Easter", "Pasxa oroli standart vaqti"}, {"Ecuador", "Ekvador vaqti"}, {"Europe_Central", "Markaziy Yevropa standart vaqti"}, {"Europe_Eastern", "Sharqiy Yevropa standart vaqti"}, {"Europe_Further_Eastern", "Kaliningrad va Minsk vaqti"}, {"Europe_Western", "G‘arbiy Yevropa standart vaqti"}, {"Falkland", "Folklend orollari standart vaqti"}, {"Fiji", "Fiji standart vaqti"}, {"French_Guiana", "Fransuz Gvianasi vaqti"}, {"French_Southern", "Fransuz Janubiy hududlari va Antarktika vaqti"}, {"GMT", "Grinvich o‘rtacha vaqti"}, {"Galapagos", "Galapagos vaqti"}, {"Gambier", "Gambye vaqti"}, {"Georgia", "Gruziya standart vaqti"}, {"Gilbert_Islands", "Gilbert orollari vaqti"}, {"Greenland_Eastern", "Sharqiy Grenlandiya standart vaqti"}, {"Greenland_Western", "G‘arbiy Grenlandiya standart vaqti"}, {"Gulf", "Fors ko‘rfazi standart vaqti"}, {"Guyana", "Gayana vaqti"}, {"Hawaii", "Gavayi-aleut standart vaqti"}, {"Hawaii_Aleutian", "Gavayi-aleut standart vaqti"}, {"Hong_Kong", "Gonkong standart vaqti"}, {"Hovd", "Xovd standart vaqti"}, {"India", "Hindiston standart vaqti"}, {"Indian_Ocean", "Hind okeani vaqti"}, {"Indochina", "Hindixitoy vaqti"}, {"Indonesia_Central", "Markaziy Indoneziya vaqti"}, {"Indonesia_Eastern", "Sharqiy Indoneziya vaqti"}, {"Indonesia_Western", "Gʻarbiy Indoneziya vaqti"}, {"Iran", "Eron standart vaqti"}, {"Irkutsk", "Irkutsk standart vaqti"}, {"Israel", "Isroil standart vaqti"}, {"Japan", "Yaponiya standart vaqti"}, {"Kazakhstan", "Qozogʻiston vaqti"}, {"Kazakhstan_Eastern", "Sharqiy Qozogʻiston vaqti"}, {"Kazakhstan_Western", "Gʻarbiy Qozogʻiston vaqti"}, {"Korea", "Koreya standart vaqti"}, {"Kosrae", "Kosrae vaqti"}, {"Krasnoyarsk", "Krasnoyarsk standart vaqti"}, {"Kyrgystan", "Qirgʻiziston vaqti"}, {"Line_Islands", "Layn orollari vaqti"}, {"Lord_Howe", "Lord-Xau standart vaqti"}, {"Magadan", "Magadan standart vaqti"}, {"Malaysia", "Malayziya vaqti"}, {"Maldives", "Maldiv orollari vaqti"}, {"Marquesas", "Markiz orollari vaqti"}, {"Marshall_Islands", "Marshall orollari vaqti"}, {"Mauritius", "Mavrikiy standart vaqti"}, {"Mawson", "Mouson vaqti"}, {"Mexico_Pacific", "Meksika Tinch okeani standart vaqti"}, {"Mongolia", "Ulan-Bator standart vaqti"}, {"Moscow", "Moskva standart vaqti"}, {"Myanmar", "Myanma vaqti"}, {"Nauru", "Nauru vaqti"}, {"Nepal", "Nepal vaqti"}, {"New_Caledonia", "Yangi Kaledoniya standart vaqti"}, {"New_Zealand", "Yangi Zelandiya standart vaqti"}, {"Newfoundland", "Nyufaundlend standart vaqti"}, {"Niue", "Niuye vaqti"}, {"Norfolk", "Norfolk oroli standart vaqti"}, {"Noronha", "Fernandu-di-Noronya standart vaqti"}, {"Novosibirsk", "Novosibirsk standart vaqti"}, {"Omsk", "Omsk standart vaqti"}, {"Pakistan", "Pokiston standart vaqti"}, {"Palau", "Palau vaqti"}, {"Papua_New_Guinea", "Papua-Yangi Gvineya vaqti"}, {"Paraguay", "Paragvay standart vaqti"}, {"Peru", "Peru standart vaqti"}, {"Philippines", "Filippin standart vaqti"}, {"Phoenix_Islands", "Feniks orollari vaqti"}, {"Pierre_Miquelon", "Sen-Pyer va Mikelon standart vaqti"}, {"Pitcairn", "Pitkern vaqti"}, {"Ponape", "Ponape vaqti"}, {"Pyongyang", "Pxenyan vaqti"}, {"Reunion", "Reyunion vaqti"}, {"Rothera", "Rotera vaqti"}, {"Sakhalin", "Saxalin standart vaqti"}, {"Samoa", "Samoa standart vaqti"}, {"Seychelles", "Seyshel orollari vaqti"}, {"Singapore", "Singapur vaqti"}, {"Solomon", "Solomon orollari vaqti"}, {"South_Georgia", "Janubiy Georgiya vaqti"}, {"Suriname", "Surinam vaqti"}, {"Syowa", "Syova vaqti"}, {"Tahiti", "Taiti vaqti"}, {"Taipei", "Tayvan standart vaqti"}, {"Tajikistan", "Tojikiston vaqti"}, {"Tokelau", "Tokelau vaqti"}, {"Tonga", "Tonga standart vaqti"}, {"Truk", "Chuuk vaqti"}, {"Turkmenistan", "Turkmaniston standart vaqti"}, {"Tuvalu", "Tuvalu vaqti"}, {"Uruguay", "Urugvay standart vaqti"}, {"Uzbekistan", "O‘zbekiston standart vaqti"}, {"Vanuatu", "Vanuatu standart vaqti"}, {"Venezuela", "Venesuela vaqti"}, {"Vladivostok", "Vladivostok standart vaqti"}, {"Volgograd", "Volgograd standart vaqti"}, {"Vostok", "Vostok vaqti"}, {"Wake", "Ueyk oroli vaqti"}, {"Wallis", "Uollis va Futuna vaqti"}, {"Yakutsk", "Yakutsk standart vaqti"}, {"Yekaterinburg", "Yekaterinburg standart vaqti"}, {"Yukon", "Yukon vaqti"}},
		{{"Alaska", "Alyaska yozgi vaqti"}, {"Amazon", "Amazonka yozgi vaqti"}, {"America_Central", "Markaziy Amerika yozgi vaqti"}, {"America_Eastern", "Sharqiy Amerika yozgi vaqti"}, {"America_Mountain", "Tog‘ yozgi vaqti (AQSH)"}, {"America_Pacific", "Tinch okeani yozgi vaqti"}, {"Apia", "Apia yozgi vaqti"}, {"Arabian", "Saudiya Arabistoni yozgi vaqti"}, {"Argentina", "Argentina yozgi vaqti"}, {"Argentina_Western", "Gʻarbiy Argentina yozgi vaqti"}, {"Armenia", "Armaniston yozgi vaqti"}, {"Atlantic", "Atlantika yozgi vaqti"}, {"Australia_Central", "Markaziy Avstraliya yozgi vaqti"}, {"Australia_CentralWestern", "Markaziy Avstraliya g‘arbiy yozgi vaqti"}, {"Australia_Eastern", "Sharqiy Avstraliya yozgi vaqti"}, {"Australia_Western", "G‘arbiy Avstraliya yozgi vaqti"}, {"Azerbaijan", "Ozarbayjon yozgi vaqti"}, {"Azores", "Azor orollari yozgi vaqti"}, {"Bangladesh", "Bangladesh yozgi vaqti"}, {"Brasilia", "Braziliya yozgi vaqti"}, {"Cape_Verde", "Kabo-Verde yozgi vaqti"}, {"Chatham", "Chatem yozgi vaqti"}, {"Chile", "Chili yozgi vaqti"}, {"China", "Xitoy yozgi vaqti"}, {"Colombia", "Kolumbiya yozgi vaqti"}, {"Cook", "Kuk orollari yarim yozgi vaqti"}, {"Cuba", "Kuba yozgi vaqti"}, {"Easter", "Pasxa oroli yozgi vaqti"}, {"Europe_Central", "Markaziy Yevropa yozgi vaqti"}, {"Europe_Eastern", "Sharqiy Yevropa yozgi vaqti"}, {"Europe_Western", "G‘arbiy Yevropa yozgi vaqti"}, {"Falkland", "Folklend orollari yozgi vaqti"}, {"Fiji", "Fiji yozgi vaqti"}, {"Georgia", "Gruziya yozgi vaqti"}, {"Greenland_Eastern", "Sharqiy Grenlandiya yozgi vaqti"}, {"Greenland_Western", "G‘arbiy Grenlandiya yozgi vaqti"}, {"Hawaii_Aleutian", "Gavayi-aleut yozgi vaqti"}, {"Hong_Kong", "Gonkong yozgi vaqti"}, {"Hovd", "Xovd yozgi vaqti"}, {"Iran", "Eron yozgi vaqti"}, {"Irkutsk", "Irkutsk yozgi vaqti"}, {"Israel", "Isroil yozgi vaqti"}, {"Japan", "Yaponiya yozgi vaqti"}, {"Korea", "Koreya yozgi vaqti"}, {"Krasnoyarsk", "Krasnoyarsk yozgi vaqti"}, {"Lord_Howe", "Lord-Xau yozgi vaqti"}, {"Magadan", "Magadan yozgi vaqti"}, {"Mauritius", "Mavrikiy yozgi vaqti"}, {"Mexico_Pacific", "Meksika Tinch okeani yozgi vaqti"}, {"Mongolia", "Ulan-Bator yozgi vaqti"}, {"Moscow", "Moskva yozgi vaqti"}, {"New_Caledonia", "Yangi Kaledoniya yozgi vaqti"}, {"New_Zealand", "Yangi Zelandiya yozgi vaqti"}, {"Newfoundland", "Nyufaundlend yozgi vaqti"}, {"Norfolk", "Norfolk oroli yozgi vaqti"}, {"Noronha", "Fernandu-di-Noronya yozgi vaqti"}, {"Novosibirsk", "Novosibirsk yozgi vaqti"}, {"Omsk", "Omsk yozgi vaqti"}, {"Pakistan", "Pokiston yozgi vaqti"}, {"Paraguay", "Paragvay yozgi vaqti"}, {"Peru", "Peru yozgi vaqti"}, {"Philippines", "Filippin yozgi vaqti"}, {"Pierre_Miquelon", "Sen-Pyer va Mikelon yozgi vaqti"}, {"Sakhalin", "Saxalin yozgi vaqti"}, {"Samoa", "Samoa yozgi vaqti"}, {"Taipei", "Tayvan yozgi vaqti"}, {"Tonga", "Tonga yozgi vaqti"}, {"Turkmenistan", "Turkmaniston yozgi vaqti"}, {"Uruguay", "Urugvay yozgi vaqti"}, {"Uzbekistan", "O‘zbekiston yozgi vaqti"}, {"Vanuatu", "Vanuatu yozgi vaqti"}, {"Vladivostok", "Vladivostok yozgi vaqti"}, {"Volgograd", "Volgograd yozgi vaqti"}, {"Yakutsk", "Yakutsk yozgi vaqti"}, {"Yekaterinburg", "Yekaterinburg yozgi vaqti"}},
	},
	{
		{{"Afghanistan", "افغانستان وقتی"}},
		{},
	},
	{
		{{"Afghanistan", "Афғонистон вақти"}, {"Africa_Central", "Марказий Африка вақти"}, {"Africa_Eastern", "Шарқий Африка вақти"}, {"Africa_Southern", "Жанубий Африка вақти"}, {"Africa_Western", "Ғарбий Африка вақти"}, {"Alaska", "Аляска стандарт вақти"}, {"Amazon", "Амазонка стандарт вақти"}, {"America_Central", "Шимолий Америка марказий стандарт вақти"}, {"America_Eastern", "Шимолий Америка шарқий стандарт вақти"}, {"America_Mountain", "Шимолий Америка тоғ стандарт вақти"}, {"America_Pacific", "Шимолий Америка тинч океани стандарт вақти"}, {"Arabian", "Арабистон стандарт вақти"}, {"Argentina", "Аргентина стандарт вақти"}, {"Argentina_Western", "Ғарбий Аргентина стандарт вақти"}, {"Armenia", "Арманистон стандарт вақти"}, {"Atlantic", "Атлантика стандарт вақти"}, {"Australia_Central", "Марказий Австралия стандарт вақти"}, {"Australia_CentralWestern", "Марказий Австралия Ғарбий стандарт вақти"}, {"Australia_Eastern", "Шарқий Австралия стандарт вақти"}, {"Australia_Western", "Ғарбий Австралия стандарт вақти"}, {"Azerbaijan", "Озарбайжон стандарт вақти"}, {"Azores", "Азор стандарт вақти"}, {"Bangladesh", "Бангладеш стандарт вақти"}, {"Bhutan", "Бутан вақти"}, {"Bolivia", "Боливия вақти"}, {"Brasilia", "Бразилия стандарт вақти"}, {"Brunei", "Бруней Даруссалом вақти"}, {"Cape_Verde", "Кабо-Верде стандарт вақти"}, {"Chamorro", "Каморро вақти"}, {"Chatham", "Чатхам стандарт вақти"}, {"Chile", "Чили стандарт вақти"}, {"China", "Хитой стандарт вақти"}, {"Christmas", "Рождество ороли вақти"}, {"Cocos", "Кокос ороллари вақти"}, {"Colombia", "Колумбия стандарт вақти"}, {"Cook", "Кук ороллари стандарт вақти"}, {"Cuba", "Куба стандарт вақти"}, {"Davis", "Дэвис вақти"}, {"DumontDUrville", "Думонт-д-Урвил вақти"}, {"East_Timor", "Шарқий Тимор вақти"}, {"Easter", "Пасхи ороли стандарт вақти"}, {"Ecuador", "Эквадор вақти"}, {"Europe_Central", "Марказий Европа стандарт вақти"}, {"Europe_Eastern", "Шарқий Европа стандарт вақти"}, {"Europe_Western", "Ғарбий Европа стандарт вақти"}, {"Falkland", "Фолькленд ороллари стандарт вақти"}, {"Fiji", "Фижи стандарт вақти"}, {"French_Guiana", "Француз Гвианаси вақти"}, {"French_Southern", "Француз жанубий ва Антарктика вақти"}, {"GMT", "Гринвич вақти"}, {"Galapagos", "Галапагос вақти"}, {"Gambier", "Гамбиер вақти"}, {"Georgia", "Грузия стандарт вақти"}, {"Gilbert_Islands", "Гилберт ороллари вақти"}, {"Greenland_Eastern", "Шарқий Гренландия стандарт вақти"}, {"Greenland_Western", "Ғарбий Гренландия стандарт вақти"}, {"Gulf", "Кўрфаз вақти"}, {"Guyana", "Гайана вақти"}, {"Hawaii", "Гавайи-алеут стандарт вақти"}, {"Hawaii_Aleutian", "Гавайи-алеут стандарт вақти"}, {"Hong_Kong", "Гонконг стандарт вақти"}, {"Hovd", "Ховд стандарт вақти"}, {"India", "Ҳиндистон вақти"}, {"Indian_Ocean", "Ҳинд океани вақти"}, {"Indochina", "Ҳинд-Хитой вақти"}, {"Indonesia_Central", "Марказий Индонезия вақти"}, {"Indonesia_Eastern", "Шарқий Индонезия вақти"}, {"Indonesia_Western", "Ғарбий Индонезия вақти"}, {"Iran", "Эрон стандарт вақти"}, {"Irkutsk", "Иркутск стандарт вақти"}, {"Israel", "Исроил стандарт вақти"}, {"Japan", "Япония стандарт вақти"}, {"Kazakhstan_Eastern", "Шарқий Қозоғистон вақти"}, {"Kazakhstan_Western", "Ғарбий Қозоғистон вақти"}, {"Korea", "Корея стандарт вақти"}, {"Kosrae", "Косрае вақти"}, {"Krasnoyarsk", "Красноярск стандарт вақти"}, {"Kyrgystan", "Қирғизистон вақти"}, {"Line_Islands", "Лайн ороллари вақти"}, {"Lord_Howe", "Лорд Хове стандарт вақти"}, {"Magadan", "Магадан стандарт вақти"}, {"Malaysia", "Малайзия вақти"}, {"Maldives", "Мальдив ороллар"}, {"Marquesas", "Маркезас вақти"}, {"Marshall_Islands", "Маршалл ороллари вақти"}, {"Mauritius", "Маврикий стандарт вақти"}, {"Mawson", "Моувсон вақти"}, {"Mongolia", "Улан-Батор стандарт вақти"}, {"Moscow", "Москва стандарт вақти"}, {"Myanmar", "Мьянма вақти"}, {"Nauru", "Науру вақти"}, {"Nepal", "Непал вақти"}, {"New_Caledonia", "Янги Каледония стандарт вақти"}, {"New_Zealand", "Янги Зеландия стандарт вақти"}, {"Newfoundland", "Ньюфаундленд стандарт вақти"}, {"Niue", "Ниуе вақти"}, {"Norfolk", "Норфолк ороли стандарт вақти"}, {"Noronha", "Фернандо де Норонья стандарт вақти"}, {"Novosibirsk", "Новосибирск стандарт вақти"}, {"Omsk", "Омск стандарт вақти"}, {"Pakistan", "Покистон стандарт вақти"}, {"Palau", "Палау вақти"}, {"Papua_New_Guinea", "Папуа-Янги Гвинея вақти"}, {"Paraguay", "Парагвай стандарт вақти"}, {"Peru", "Перу стандарт вақти"}, {"Philippines", "Филиппин стандарт вақти"}, {"Phoenix_Islands", "Феникс ороллари вақти"}, {"Pierre_Miquelon", "Сент-Пьер ва Микелон стандарт вақти"}, {"Pitcairn", "Питкерн вақти"}, {"Ponape", "Понапе вақти"}, {"Reunion", "Реюньон вақти"}, {"Rothera", "Ротера вақти"}, {"Sakhalin", "Сахалин стандарт вақти"}, {"Samoa", "Самоа стандарт вақти"}, {"Seychelles", "Сейшел ороллари вақти"}, {"Singapore", "Сингапур вақти"}, {"Solomon", "Соломон ороллари вақти"}, {"South_Georgia", "Жанубий Джорджия вақти"}, {"Suriname", "Суринам вақти"}, {"Syowa", "Сьова вақти"}, {"Tahiti", "Таити вақти"}, {"Taipei", "Тайпей стандарт вақти"}, {"Tajikistan", "Тожикистон вақти"}, {"Tokelau", "Токелау вақти"}, {"Tonga", "Тонга стандарт вақти"}, {"Truk", "Чуук вақти"}, {"Turkmenistan", "Туркманистон стандарт вақти"}, {"Tuvalu", "Тувалу вақти"}, {"Uruguay", "Уругвай стандарт вақти"}, {"Uzbekistan", "Ўзбекистон стандарт вақти"}, {"Vanuatu", "Вануату стандарт вақти"}, {"Venezuela", "Венесуэла вақти"}, {"Vladivostok", "Владивосток стандарт вақти"}, {"Volgograd", "Волгоград стандарт вақти"}, {"Vostok", "Восток вақти"}, {"Wake", "Уэйк ороли вақти"}, {"Wallis", "Уэллис ва Футуна вақти"}, {"Yakutsk", "Якутск стандарт вақти"}, {"Yekaterinburg", "Екатеринбург стандарт вақти"}},
		{{"Alaska", "Аляска кундузги вақти"}, {"Amazon", "Амазонка ёзги вақти"}, {"America_Central", "Шимолий Америка марказий кундузги вақти"}, {"America_Eastern", "Шимолий Америка шарқий кундузги вақти"}, {"America_Mountain", "Шимолий Америка тоғ кундузги вақти"}, {"America_Pacific", "Шимолий Америка тинч океани кундузги вақти"}, {"Arabian", "Арабистон кундузги вақти"}, {"Argentina", "Аргентина ёзги вақти"}, {"Argentina_Western", "Ғарбий Аргентина ёзги вақти"}, {"Armenia", "Арманистон ёзги вақти"}, {"Atlantic", "Атлантика кундузги вақти"}, {"Australia_Central", "Марказий Австралия кундузги вақти"}, {"Australia_CentralWestern", "Марказий Австралия Ғарбий кундузги вақти"}, {"Australia_Eastern", "Шарқий Австралия кундузги вақти"}, {"Australia_Western", "Ғарбий Австралия кундузги вақти"}, {"Azerbaijan", "Озарбайжон ёзги вақти"}, {"Azores", "Азор ёзги вақти"}, {"Bangladesh", "Бангладеш ёзги вақти"}, {"Brasilia", "Бразилия ёзги вақти"}, {"Cape_Verde", "Кабо-Верде ёзги вақти"}, {"Chatham", "Чатхам кундузги вақти"}, {"Chile", "Чили ёзги вақти"}, {"China", "Хитой кундузги вақти"}, {"Colombia", "Колумбия ёзги вақти"}, {"Cook", "Кук ороллари ярим ёзги вақти"}, {"Cuba", "Куба кундузги вақти"}, {"Easter", "Пасхи ороли ёзги вақти"}, {"Europe_Central", "Марказий Европа ёзги вақти"}, {"Europe_Eastern", "Шарқий Европа ёзги вақти"}, {"Europe_Western", "Ғарбий Европа ёзги вақти"}, {"Falkland", "Фолькленд ороллари ёзги вақти"}, {"Fiji", "Фижи ёзги вақти"}, {"Georgia", "Грузия ёзги вақти"}, {"Greenland_Eastern", "Шарқий Гренландия ёзги вақти"}, {"Greenland_Western", "Ғарбий Гренландия ёзги вақти"}, {"Hawaii_Aleutian", "Гавайи-алеут кундузги вақти"}, {"Hong_Kong", "Гонконг ёзги вақти"}, {"Hovd", "Ховд ёзги вақти"}, {"Iran", "Эрон кундузги вақти"}, {"Irkutsk", "Иркутск ёзги вақти"}, {"Israel", "Исроил кундузги вақти"}, {"Japan", "Япония кундузги вақти"}, {"Korea", "Корея кундузги вақти"}, {"Krasnoyarsk", "Красноярск ёзги вақти"}, {"Lord_Howe", "Лорд Хове кундузги вақти"}, {"Magadan", "Магадан ёзги вақти"}, {"Mauritius", "Маврикий ёзги вақти"}, {"Mongolia", "Улан-Батор ёзги вақти"}, {"Moscow", "Москва ёзги вақти"}, {"New_Caledonia", "Янги Каледония ёзги вақти"}, {"New_Zealand", "Янги Зеландия кундузги вақти"}, {"Newfoundland", "Ньюфаундленд кундузги вақти"}, {"Norfolk", "Норфолк ороли ёзги вақти"}, {"Noronha", "Фернандо де Норонья ёзги вақти"}, {"Novosibirsk", "Новосибирск ёзги вақти"}, {"Omsk", "Омск ёзги вақти"}, {"Pakistan", "Покистон ёзги вақти"}, {"Paraguay", "Парагвай ёзги вақти"}, {"Peru", "Перу ёзги вақти"}, {"Philippines", "Филиппин ёзги вақти"}, {"Pierre_Miquelon", "Сент-Пьер ва Микелон кундузги вақти"}, {"Sakhalin", "Сахалин ёзги вақти"}, {"Samoa", "Самоа кундузги вақти"}, {"Taipei", "Тайпей кундузги вақти"}, {"Tonga", "Тонга ёзги вақти"}, {"Turkmenistan", "Туркманистон ёзги вақти"}, {"Uruguay", "Уругвай ёзги вақти"}, {"Uzbekistan", "Ўзбекистон ёзги вақти"}, {"Vanuatu", "Вануату ёзги вақти"}, {"Vladivostok", "Владивосток ёзги вақти"}, {"Volgograd", "Волгоград ёзги вақти"}, {"Yakutsk", "Якутск ёзги вақти"}, {"Yekaterinburg", "Екатеринбург ёзги вақти"}},
	},
	{
		{{"Afghanistan", "Ora de l’Afghanistan"}, {"Africa_Central", "Ora de l’Àfrega sentrale"}, {"Africa_Eastern", "Ora de l’Àfrega orientale"}, {"Africa_Southern", "Ora de l’Àfrega meridionale"}, {"Africa_Western", "Ora de l’Àfrega osidentale"}, {"Alaska", "Ora normale de l’Alaska"}, {"Amazon", "Ora normale de l’Amasonia"}, {"America_Central", "Ora normale de’l nord Amèrega sentrale"}, {"America_Eastern", "Ora normale de’l nord Amèrega orientale"}, {"America_Mountain", "Ora normale de’l nord Amèrega de le montagne"}, {"America_Pacific", "Ora normale de’l Pasìfego"}, {"Apia", "Ora normale de Apia"}, {"Arabian", "Ora normale de l’Arabia"}, {"Argentina", "Ora normale de l’Arjentina"}, {"Argentina_Western", "Ora normale de l’Arjentina osidentale"}, {"Armenia", "Ora normale de l’Armenia"}, {"Atlantic", "Ora normale de l’Atlàntego"}, {"Australia_Central", "Ora normale de l’Australia sentrale"}, {"Australia_CentralWestern", "Ora normale de l’Australia sentro osidentale"}, {"Australia_Eastern", "Ora normale de l’Australia orientale"}, {"Australia_Western", "Ora normale de l’Australia osidentale"}, {"Azerbaijan", "Ora normale de l’Azerbaijan"}, {"Azores", "Ora normale de le Azore"}, {"Bangladesh", "Ora normale de’l Bangladesh"}, {"Bhutan", "Ora de’l Butan"}, {"Bolivia", "Ora de la Bolivia"}, {"Brasilia", "Ora normale de Brazilia"}, {"Brunei", "Ora de’l Brunéi"}, {"Cape_Verde", "Ora normale de Cao Verdo"}, {"Chamorro", "Ora de Chamoro"}, {"Chatham", "Ora normale de le Ìzole Ciatem"}, {"Chile", "Ora normale de’l Cile"}, {"China", "Ora normale de la Sina"}, {"Christmas", "Ora de l’Ìzola de Nadale"}, {"Cocos", "Ora de le Ìzole Cocos"}, {"Colombia", "Ora normale de la Colonbia"}, {"Cook", "Ora normale de le Ìzole Cook"}, {"Cuba", "Ora normale de Cuba"}, {"Davis", "Ora de Davis"}, {"DumontDUrville", "Ora de Dumont d’Urville"}, {"East_Timor", "Ora de’l Timor Est"}, {"Easter", "Ora normale de l’Ìzola de Pascua"}, {"Ecuador", "Ora de l’Ècuador"}, {"Europe_Central", "Ora normale de l’Europa sentrale"}, {"Europe_Eastern", "Ora normale de l’Europa orientale"}, {"Europe_Further_Eastern", "Ora de l’Europa stra orientale"}, {"Europe_Western", "Ora normale de l’Europa osidentale"}, {"Falkland", "Ora normale de le Ìzole Malvine"}, {"Fiji", "Ora normale de le Ìzole Fiji"}, {"French_Guiana", "Ora de la Guyana franseze"}, {"French_Southern", "Ora de le Tere fransezi de’l sud e de l’Antàrtego"}, {"GMT", "Ora de’l meridian de Greenwich"}, {"Galapagos", "Ora de le Galàpagos"}, {"Gambier", "Ora de le Ìzole Gambier"}, {"Georgia", "Ora normale de la Jeorja"}, {"Gilbert_Islands", "Ora de le Ìzole Gilbert"}, {"Greenland_Eastern", "Ora normale de la Groenlanda orientale"}, {"Greenland_Western", "Ora normale de la Groenlanda osidentale"}, {"Gulf", "Ora de’l Golfo"}, {"Guyana", "Ora de la Guyana"}, {"Hawaii", "Ora normale de Hawai e Aleutine"}, {"Hawaii_Aleutian", "Ora normale de Hawai e Aleutine"}, {"Hong_Kong", "Ora normale de Hong Kong"}, {"Hovd", "Ora normale de Hovd"}, {"India", "Ora de l’India"}, {"Indian_Ocean", "Ora de l’Osèano Indian"}, {"Indochina", "Ora de l’Indosina"}, {"Indonesia_Central", "Ora de l’Indonezia sentrale"}, {"Indonesia_Eastern", "Ora de l’Indonezia orientale"}, {"Indonesia_Western", "Ora de l’Indonezia osidentale"}, {"Iran", "Ora normale de l’Iran"}, {"Irkutsk", "Ora normale de Irkutsk"}, {"Israel", "Ora normale de Izraele"}, {"Japan", "Ora normale de’l Japon"}, {"Kazakhstan", "Ora de’l Kazakistan"}, {"Kazakhstan_Eastern", "Ora de’l Kazakistan orientale"}, {"Kazakhstan_Western", "Ora de’l Kazakistan osidentale"}, {"Korea", "Ora normale de la Corèa"}, {"Kosrae", "Ora de l’Ìzola Kosrae"}, {"Krasnoyarsk", "Ora normale de Krasnoyarsk"}, {"Kyrgystan", "Ora de’l Kirghizistan"}, {"Line_Islands", "Ora de le Ìzole Ecuatoriali"}, {"Lord_Howe", "Ora normale de l’Ìzola Lord Howe"}, {"Magadan", "Ora normale de Magadan"}, {"Malaysia", "Ora de la Malezia"}, {"Maldives", "Ora de le Ìzole Maldive"}, {"Marquesas", "Ora de le Ìzole Marchezi"}, {"Marshall_Islands", "Ora de le Ìzole Marshall"}, {"Mauritius", "Ora normale de le Ìzole Maurisio"}, {"Mawson", "Ora de Mawson"}, {"Mexico_Pacific", "Ora normale de’l Mèsego de’l Pasìfego"}, {"Mongolia", "Ora normale de Ulan Bàtor"}, {"Moscow", "Ora normale de Mosca"}, {"Myanmar", "Ora de Myanmar (Birmania)"}, {"Nauru", "Ora de l’Ìzola Nauru"}, {"Nepal", "Ora de’l Nepal"}, {"New_Caledonia", "Ora normale de la Nova Caledonia"}, {"New_Zealand", "Ora normale de la Nova Zelanda"}, {"Newfoundland", "Ora normale de Teranova"}, {"Niue", "Ora de l’Ìzola Niue"}, {"Norfolk", "Ora normale de l’Ìzola Norfolk"}, {"Noronha", "Ora normale de Fernando de Noronha"}, {"Novosibirsk", "Ora normale de Novosibirsk"}, {"Omsk", "Ora normale de Omsk"}, {"Pakistan", "Ora normale de’l Pakistan"}, {"Palau", "Ora de le Ìzole Palàu"}, {"Papua_New_Guinea", "Ora de la Papua Nova Guinèa"}, {"Paraguay", "Ora normale de’l Paraguài"}, {"Peru", "Ora normale de’l Perù"}, {"Philippines", "Ora normale de le Ìzole Filipine"}, {"Phoenix_Islands", "Ora de le Ìzole Fenize"}, {"Pierre_Miquelon", "Ora normale de S. Piero e Michelon"}, {"Pitcairn", "Ora de le Ìzole Pitcairn"}, {"Ponape", "Ora de l’Ìzola Ponpèi"}, {"Pyongyang", "Ora de Pyongyang"}, {"Reunion", "Ora de l’Ìzola Reunion"}, {"Rothera", "Ora de Rothera"}, {"Sakhalin", "Ora normale de Sakalin"}, {"Samoa", "Ora normale de le Ìzole Samòa"}, {"Seychelles", "Ora d’istà de le Ìzole Seisel"}, {"Singapore", "Ora de Singapore"}, {"Solomon", "Ora de le Ìzole Salomon"}, {"South_Georgia", "Ora de la Georgia de’l sud"}, {"Suriname", "Ora de’l Suriname"}, {"Syowa", "Ora de Syowa"}, {"Tahiti", "Ora de l’Ìzola Taiti"}, {"Taipei", "Ora normale de Taipéi"}, {"Tajikistan", "Ora de’l Tajikistan"}, {"Tokelau", "Ora de le Ìzole Tokelàu"}, {"Tonga", "Ora normale de le Ìzole Tonga"}, {"Truk", "Ora de’l Chuuk"}, {"Turkmenistan", "Ora normale de’l Turkmenistan"}, {"Tuvalu", "Ora de le Ìzole Tuvalu"}, {"Uruguay", "Ora normale de l’Uruguài"}, {"Uzbekistan", "Ora normale de l’Uzbekistan"}, {"Vanuatu", "Ora normale de le Ìzole Vanuatu"}, {"Venezuela", "Ora de’l Venesuela"}, {"Vladivostok", "Ora normale de Vladivostok"}, {"Volgograd", "Ora normale de Volgogrado"}, {"Vostok", "Ora de Vostok"}, {"Wake", "Ora de l’Atolo Wake"}, {"Wallis", "Ora de le Ìzole Wallis e Futuna"}, {"Yakutsk", "Ora normale de Yakutsk"}, {"Yekaterinburg", "Ora normale de Ekaterinburgo"}, {"Yukon", "Ora de’l Yukon"}},
		{{"Alaska", "Ora d’istà de l’Alaska"}, {"Amazon", "Ora d’istà de l’Amasonia"}, {"America_Central", "Ora d’istà de’l nord Amèrega sentrale"}, {"America_Eastern", "Ora d’istà de’l nord Amèrega orientale"}, {"America_Mountain", "Ora d’istà de’l nord Amèrega de le montagne"}, {"America_Pacific", "Ora d’istà de’l Pasìfego"}, {"Apia", "Ora d’istà de Apia"}, {"Arabian", "Ora d’istà de l’Arabia"}, {"Argentina", "Ora d’istà de l’Arjentina"}, {"Argentina_Western", "Ora d’istà de l’Arjentina osidentale"}, {"Armenia", "Ora d’istà de l’Armenia"}, {"Atlantic", "Ora d’istà de l’Atlàntego"}, {"Australia_Central", "Ora d’istà de l’Australia sentrale"}, {"Australia_CentralWestern", "Ora d’istà de l’Australia sentro osidentale"}, {"Australia_Eastern", "Ora d’istà de l’Australia orientale"}, {"Australia_Western", "Ora d’istà de l’Australia osidentale"}, {"Azerbaijan", "Ora d’istà de l’Azerbaijan"}, {"Azores", "Ora d’istà de le Azore"}, {"Bangladesh", "Ora d’istà de’l Bangladesh"}, {"Brasilia", "Ora d’istà de Brazilia"}, {"Cape_Verde", "Ora d’istà de Cao Verdo"}, {"Chatham", "Ora d’istà de le Ìzole Ciatem"}, {"Chile", "Ora d’istà de’l Cile"}, {"China", "Ora d’istà de la Sina"}, {"Colombia", "Ora d’istà de la Colonbia"}, {"Cook", "Ora d’istà de le Ìzole Cook"}, {"Cuba", "Ora d’istà de Cuba"}, {"Easter", "Ora d’istà de l’Ìzola de Pascua"}, {"Europe_Central", "Ora d’istà de l’Europa sentrale"}, {"Europe_Eastern", "Ora d’istà de l’Europa orientale"}, {"Europe_Western", "Ora d’istà de l’Europa osidentale"}, {"Falkland", "Ora d’istà de le Ìzole Malvine"}, {"Fiji", "Ora d’istà de le Ìzole Fiji"}, {"Georgia", "Ora d’istà de la Jeorja"}, {"Greenland_Eastern", "Ora d’istà de la Groenlanda orientale"}, {"Greenland_Western", "Ora d’istà de la Groenlanda osidentale"}, {"Hawaii_Aleutian", "Ora d’istà de Hawai e Aleutine"}, {"Hong_Kong", "Ora d’istà de Hong Kong"}, {"Hovd", "Ora d’istà de Hovd"}, {"Iran", "Ora d’istà de l’Iran"}, {"Irkutsk", "Ora d’istà de Irkutsk"}, {"Israel", "Ora d’istà de Izraele"}, {"Japan", "Ora d’istà de’l Japon"}, {"Korea", "Ora d’istà de la Corèa"}, {"Krasnoyarsk", "Ora d’istà de Krasnoyarsk"}, {"Lord_Howe", "Ora d’istà de l’Ìzola Lord Howe"}, {"Magadan", "Ora d’istà de Magadan"}, {"Mauritius", "Ora d’istà de le Ìzole Maurisio"}, {"Mexico_Pacific", "Ora d’istà de’l Mèsego de’l Pasìfego"}, {"Mongolia", "Ora d’istà de Ulan Bàtor"}, {"Moscow", "Ora d’istà de Mosca"}, {"New_Caledonia", "Ora d’istà de la Nova Caledonia"}, {"New_Zealand", "Ora d’istà de la Nova Zelanda"}, {"Newfoundland", "Ora d’istà de Teranova"}, {"Norfolk", "Ora d’istà de l’Ìzola Norfolk"}, {"Noronha", "Ora d’istà de Fernando de Noronha"}, {"Novosibirsk", "Ora d’istà de Novosibirsk"}, {"Omsk", "Ora d’istà de Omsk"}, {"Pakistan", "Ora d’istà de’l Pakistan"}, {"Paraguay", "Ora d’istà de’l Paraguài"}, {"Peru", "Ora d’istà de’l Perù"}, {"Philippines", "Ora d’istà de le Ìzole Filipine"}, {"Pierre_Miquelon", "Ora d’istà de S. Piero e Michelon"}, {"Sakhalin", "Ora d’istà de Sakalin"}, {"Samoa", "Ora d’istà de le Ìzole Samòa"}, {"Taipei", "Ora d’istà de Taipéi"}, {"Tonga", "Ora d’istà de le Ìzole Tonga"}, {"Turkmenistan", "Ora d’istà de’l Turkmenistan"}, {"Uruguay", "Ora d’istà de l’Uruguài"}, {"Uzbekistan", "Ora d’istà de l’Uzbekistan"}, {"Vanuatu", "Ora d’istà de le Ìzole Vanuatu"}, {"Vladivostok", "Ora d’istà de Vladivostok"}, {"Volgograd", "Ora d’istà de Volgogrado"}, {"Yakutsk", "Ora d’istà de Yakutsk"}, {"Yekaterinburg", "Ora d’istà de Ekaterinburgo"}},
	},
	{
		{{"Acre", "Giờ Chuẩn Acre"}, {"Afghanistan", "Giờ Afghanistan"}, {"Africa_Central", "Giờ Trung Phi"}, {"Africa_Eastern", "Giờ Đông Phi"}, {"Africa_Southern", "Giờ Chuẩn Nam Phi"}, {"Africa_Western", "Giờ Tây Phi"}, {"Alaska", "Giờ Chuẩn Alaska"}, {"Almaty", "Giờ Chuẩn Almaty"}, {"Amazon", "Giờ Chuẩn Amazon"}, {"America_Central", "Giờ chuẩn miền Trung"}, {"America_Eastern", "Giờ chuẩn miền Đông"}, {"America_Mountain", "Giờ chuẩn miền núi"}, {"America_Pacific", "Giờ chuẩn Thái Bình Dương"}, {"Anadyr", "Giờ Chuẩn Anadyr"}, {"Apia", "Giờ Chuẩn Apia"}, {"Aqtau", "Giờ Chuẩn Aqtau"}, {"Aqtobe", "Giờ Chuẩn Aqtobe"}, {"Arabian", "Giờ chuẩn Ả Rập"}, {"Argentina", "Giờ Chuẩn Argentina"}, {"Argentina_Western", "Giờ chuẩn miền tây Argentina"}, {"Armenia", "Giờ Chuẩn Armenia"}, {"Atlantic", "Giờ Chuẩn Đại Tây Dương"}, {"Australia_Central", "Giờ Chuẩn Miền Trung Australia"}, {"Australia_CentralWestern", "Giờ Chuẩn Miền Trung Tây Australia"}, {"Australia_Eastern", "Giờ Chuẩn Miền Đông Australia"}, {"Australia_Western", "Giờ Chuẩn Miền Tây Australia"}, {"Azerbaijan", "Giờ Chuẩn Azerbaijan"}, {"Azores", "Giờ chuẩn Azores"}, {"Bangladesh", "Giờ Chuẩn Bangladesh"}, {"Bhutan", "Giờ Bhutan"}, {"Bolivia", "Giờ Bolivia"}, {"Brasilia", "Giờ Chuẩn Brasilia"}, {"Brunei", "Giờ Brunei Darussalam"}, {"Cape_Verde", "Giờ Chuẩn Cape Verde"}, {"Casey", "Giờ Casey"}, {"Chamorro", "Giờ Chamorro"}, {"Chatham", "Giờ Chuẩn Chatham"}, {"Chile", "Giờ Chuẩn Chile"}, {"China", "Giờ Chuẩn Trung Quốc"}, {"Christmas", "Giờ Đảo Christmas"}, {"Cocos", "Giờ Quần Đảo Cocos"}, {"Colombia", "Giờ Chuẩn Colombia"}, {"Cook", "Giờ Chuẩn Quần Đảo Cook"}, {"Cuba", "Giờ Chuẩn Cuba"}, {"Davis", "Giờ Davis"}, {"DumontDUrville", "Giờ Dumont-d’Urville"}, {"East_Timor", "Giờ Đông Timor"}, {"Easter", "Giờ Chuẩn Đảo Phục Sinh"}, {"Ecuador", "Giờ Ecuador"}, {"Europe_Central", "Giờ chuẩn Trung Âu"}, {"Europe_Eastern", "Giờ chuẩn Đông Âu"}, {"Europe_Further_Eastern", "Giờ Viễn đông Châu Âu"}, {"Europe_Western", "Giờ Chuẩn Tây Âu"}, {"Falkland", "Giờ Chuẩn Quần Đảo Falkland"}, {"Fiji", "Giờ Chuẩn Fiji"}, {"French_Guiana", "Giờ Guiana thuộc Pháp"}, {"French_Southern", "Giờ Nam Cực và Nam Nước Pháp"}, {"GMT", "Giờ Trung bình Greenwich"}, {"Galapagos", "Giờ Galapagos"}, {"Gambier", "Giờ Gambier"}, {"Georgia", "Giờ Chuẩn Georgia"}, {"Gilbert_Islands", "Giờ Quần Đảo Gilbert"}, {"Greenland_Eastern", "Giờ Chuẩn Miền Đông Greenland"}, {"Greenland_Western", "Giờ Chuẩn Miền Tây Greenland"}, {"Guam", "Giờ Chuẩn Guam"}, {"Gulf", "Giờ Chuẩn Vùng Vịnh"}, {"Guyana", "Giờ Guyana"}, {"Hawaii", "Giờ Chuẩn Hawaii-Aleut"}, {"Hawaii_Aleutian", "Giờ Chuẩn Hawaii-Aleut"}, {"Hong_Kong", "Giờ Chuẩn Hồng Kông"}, {"Hovd", "Giờ Chuẩn Hovd"}, {"India", "Giờ Chuẩn Ấn Độ"}, {"Indian_Ocean", "Giờ Ấn Độ Dương"}, {"Indochina", "Giờ Đông Dương"}, {"Indonesia_Central", "Giờ Miền Trung Indonesia"}, {"Indonesia_Eastern", "Giờ Miền Đông Indonesia"}, {"Indonesia_Western", "Giờ Miền Tây Indonesia"}, {"Iran", "Giờ Chuẩn Iran"}, {"Irkutsk", "Giờ Chuẩn Irkutsk"}, {"Israel", "Giờ Chuẩn Israel"}, {"Japan", "Giờ Chuẩn Nhật Bản"}, {"Kamchatka", "Giờ chuẩn Petropavlovsk-Kamchatski"}, {"Kazakhstan", "Giờ Kazakhstan"}, {"Kazakhstan_Eastern", "Giờ Miền Đông Kazakhstan"}, {"Kazakhstan_Western", "Giờ Miền Tây Kazakhstan"}, {"Korea", "Giờ Chuẩn Hàn Quốc"}, {"Kosrae", "Giờ Kosrae"}, {"Krasnoyarsk", "Giờ Chuẩn Krasnoyarsk"}, {"Kyrgystan", "Giờ Kyrgystan"}, {"Lanka", "Giờ Lanka"}, {"Line_Islands", "Giờ Quần Đảo Line"}, {"Lord_Howe", "Giờ Chuẩn Lord Howe"}, {"Macau", "Giờ Chuẩn Ma Cao"}, {"Magadan", "Giờ Chuẩn Magadan"}, {"Malaysia", "Giờ Malaysia"}, {"Maldives", "Giờ Maldives"}, {"Marquesas", "Giờ Marquesas"}, {"Marshall_Islands", "Giờ Quần Đảo Marshall"}, {"Mauritius", "Giờ Chuẩn Mauritius"}, {"Mawson", "Giờ Mawson"}, {"Mexico_Pacific", "Giờ Chuẩn Thái Bình Dương Mexico"}, {"Mongolia", "Giờ chuẩn Ulan Bator"}, {"Moscow", "Giờ Chuẩn Matxcơva"}, {"Myanmar", "Giờ Myanmar"}, {"Nauru", "Giờ Nauru"}, {"Nepal", "Giờ Nepal"}, {"New_Caledonia", "Giờ Chuẩn New Caledonia"}, {"New_Zealand", "Giờ Chuẩn New Zealand"}, {"Newfoundland", "Giờ Chuẩn Newfoundland"}, {"Niue", "Giờ Niue"}, {"Norfolk", "Giờ Chuẩn Đảo Norfolk"}, {"Noronha", "Giờ Chuẩn Fernando de Noronha"}, {"North_Mariana", "Giờ Quần Đảo Bắc Mariana"}, {"Novosibirsk", "Giờ chuẩn Novosibirsk"}, {"Omsk", "Giờ chuẩn Omsk"}, {"Pakistan", "Giờ Chuẩn Pakistan"}, {"Palau", "Giờ Palau"}, {"Papua_New_Guinea", "Giờ Papua New Guinea"}, {"Paraguay", "Giờ Chuẩn Paraguay"}, {"Peru", "Giờ Chuẩn Peru"}, {"Philippines", "Giờ Chuẩn Philippin"}, {"Phoenix_Islands", "Giờ Quần Đảo Phoenix"}, {"Pierre_Miquelon", "Giờ Chuẩn St. Pierre và Miquelon"}, {"Pitcairn", "Giờ Pitcairn"}, {"Ponape", "Giờ Ponape"}, {"Pyongyang", "Giờ Bình Nhưỡng"}, {"Qyzylorda", "Giờ Chuẩn Qyzylorda"}, {"Reunion", "Giờ Reunion"}, {"Rothera", "Giờ Rothera"}, {"Sakhalin", "Giờ Chuẩn Sakhalin"}, {"Samara", "Giờ Chuẩn Samara"}, {"Samoa", "Giờ Chuẩn Samoa"}, {"Seychelles", "Giờ Seychelles"}, {"Singapore", "Giờ Singapore"}, {"Solomon", "Giờ Quần Đảo Solomon"}, {"South_Georgia", "Giờ Nam Georgia"}, {"Suriname", "Giờ Suriname"}, {"Syowa", "Giờ Syowa"}, {"Tahiti", "Giờ Tahiti"}, {"Taipei", "Giờ Chuẩn Đài Bắc"}, {"Tajikistan", "Giờ Tajikistan"}, {"Tokelau", "Giờ Tokelau"}, {"Tonga", "Giờ Chuẩn Tonga"}, {"Truk", "Giờ Chuuk"}, {"Turkey", "Giờ Chuẩn Thổ Nhĩ Kỳ"}, {"Turkmenistan", "Giờ Chuẩn Turkmenistan"}, {"Tuvalu", "Giờ Tuvalu"}, {"Uruguay", "Giờ Chuẩn Uruguay"}, {"Uzbekistan", "Giờ Chuẩn Uzbekistan"}, {"Vanuatu", "Giờ Chuẩn Vanuatu"}, {"Venezuela", "Giờ Venezuela"}, {"Vladivostok", "Giờ Chuẩn Vladivostok"}, {"Volgograd", "Giờ Chuẩn Volgograd"}, {"Vostok", "Giờ Vostok"}, {"Wake", "Giờ Đảo Wake"}, {"Wallis", "Giờ Wallis và Futuna"}, {"Yakutsk", "Giờ Chuẩn Yakutsk"}, {"Yekaterinburg", "Giờ Chuẩn Yekaterinburg"}, {"Yukon", "Giờ Yukon"}},
		{{"Acre", "Giờ Mùa Hè Acre"}, {"Alaska", "Giờ Mùa Hè Alaska"}, {"Almaty", "Giờ Mùa Hè Almaty"}, {"Amazon", "Giờ Mùa Hè Amazon"}, {"America_Central", "Giờ mùa hè miền Trung"}, {"America_Eastern", "Giờ mùa hè miền Đông"}, {"America_Mountain", "Giờ mùa hè miền núi"}, {"America_Pacific", "Giờ mùa hè Thái Bình Dương"}, {"Anadyr", "Giờ mùa hè Anadyr"}, {"Apia", "Giờ Mùa Hè Apia"}, {"Aqtau", "Giờ Mùa Hè Aqtau"}, {"Aqtobe", "Giờ Mùa Hè Aqtobe"}, {"Arabian", "Giờ Mùa Hè Ả Rập"}, {"Argentina", "Giờ Mùa Hè Argentina"}, {"Argentina_Western", "Giờ mùa hè miền tây Argentina"}, {"Armenia", "Giờ Mùa Hè Armenia"}, {"Atlantic", "Giờ mùa hè Đại Tây Dương"}, {"Australia_Central", "Giờ Mùa Hè Miền Trung Australia"}, {"Australia_CentralWestern", "Giờ Mùa Hè Miền Trung Tây Australia"}, {"Australia_Eastern", "Giờ Mùa Hè Miền Đông Australia"}, {"Australia_Western", "Giờ Mùa Hè Miền Tây Australia"}, {"Azerbaijan", "Giờ Mùa Hè Azerbaijan"}, {"Azores", "Giờ mùa hè Azores"}, {"Bangladesh", "Giờ Mùa Hè Bangladesh"}, {"Brasilia", "Giờ Mùa Hè Brasilia"}, {"Cape_Verde", "Giờ Mùa Hè Cape Verde"}, {"Chatham", "Giờ Mùa Hè Chatham"}, {"Chile", "Giờ Mùa Hè Chile"}, {"China", "Giờ Mùa Hè Trung Quốc"}, {"Colombia", "Giờ Mùa Hè Colombia"}, {"Cook", "Giờ Nửa Mùa Hè Quần Đảo Cook"}, {"Cuba", "Giờ Mùa Hè Cuba"}, {"Easter", "Giờ Mùa Hè Đảo Phục Sinh"}, {"Europe_Central", "Giờ mùa hè Trung Âu"}, {"Europe_Eastern", "Giờ mùa hè Đông Âu"}, {"Europe_Western", "Giờ mùa hè Tây Âu"}, {"Falkland", "Giờ Mùa Hè Quần Đảo Falkland"}, {"Fiji", "Giờ Mùa Hè Fiji"}, {"Georgia", "Giờ Mùa Hè Georgia"}, {"Greenland_Eastern", "Giờ Mùa Hè Miền Đông Greenland"}, {"Greenland_Western", "Giờ Mùa Hè Miền Tây Greenland"}, {"Hawaii_Aleutian", "Giờ Mùa Hè Hawaii-Aleut"}, {"Hong_Kong", "Giờ Mùa Hè Hồng Kông"}, {"Hovd", "Giờ Mùa Hè Hovd"}, {"Iran", "Giờ Mùa Hè Iran"}, {"Irkutsk", "Giờ Mùa Hè Irkutsk"}, {"Israel", "Giờ Mùa Hè Israel"}, {"Japan", "Giờ Mùa Hè Nhật Bản"}, {"Kamchatka", "Giờ mùa hè Petropavlovsk-Kamchatski"}, {"Korea", "Giờ Mùa Hè Hàn Quốc"}, {"Krasnoyarsk", "Giờ Mùa Hè Krasnoyarsk"}, {"Lord_Howe", "Giờ Mùa Hè Lord Howe"}, {"Macau", "Giờ Mùa Hè Ma Cao"}, {"Magadan", "Giờ mùa hè Magadan"}, {"Mauritius", "Giờ Mùa Hè Mauritius"}, {"Mexico_Pacific", "Giờ Mùa Hè Thái Bình Dương Mexico"}, {"Mongolia", "Giờ mùa hè Ulan Bator"}, {"Moscow", "Giờ Mùa Hè Matxcơva"}, {"New_Caledonia", "Giờ Mùa Hè New Caledonia"}, {"New_Zealand", "Giờ Mùa Hè New Zealand"}, {"Newfoundland", "Giờ Mùa Hè Newfoundland"}, {"Norfolk", "Giờ Mùa Hè Đảo Norfolk"}, {"Noronha", "Giờ Mùa Hè Fernando de Noronha"}, {"Novosibirsk", "Giờ mùa hè Novosibirsk"}, {"Omsk", "Giờ mùa hè Omsk"}, {"Pakistan", "Giờ Mùa Hè Pakistan"}, {"Paraguay", "Giờ Mùa Hè Paraguay"}, {"Peru", "Giờ Mùa Hè Peru"}, {"Philippines", "Giờ Mùa Hè Philippin"}, {"Pierre_Miquelon", "Giờ Mùa Hè Saint Pierre và Miquelon"}, {"Qyzylorda", "Giờ Mùa Hè Qyzylorda"}, {"Sakhalin", "Giờ mùa hè Sakhalin"}, {"Samara", "Giờ mùa hè Samara"}, {"Samoa", "Giờ ban ngày Samoa"}, {"Taipei", "Giờ Mùa Hè Đài Bắc"}, {"Tonga", "Giờ Mùa Hè Tonga"}, {"Turkey", "Giờ Mùa Hè Thổ Nhĩ Kỳ"}, {"Turkmenistan", "Giờ Mùa Hè Turkmenistan"}, {"Uruguay", "Giờ Mùa Hè Uruguay"}, {"Uzbekistan", "Giờ Mùa Hè Uzbekistan"}, {"Vanuatu", "Giờ Mùa Hè Vanuatu"}, {"Vladivostok", "Giờ mùa hè Vladivostok"}, {"Volgograd", "Giờ Mùa Hè Volgograd"}, {"Yakutsk", "Giờ mùa hè Yakutsk"}, {"Yekaterinburg", "Giờ mùa hè Yekaterinburg"}},
	},
	{
		{{"GMT", "okathi oolikana wa Greenwich"}},
		{},
	},
	{
		{{"Atlantic", "Atlantiši Standardzit"}, {"Europe_Central", "Mitteleuropäiši Standardzit"}, {"Europe_Eastern", "Ošteuropäiši Standardzit"}, {"Europe_Western", "Wešteuropäiši Standardzit"}},
		{{"Atlantic", "Atlantiši Summerzit"}, {"Europe_Central", "Mitteleuropäiši Summerzit"}, {"Europe_Eastern", "Ošteuropäiši Summerzit"}, {"Europe_Western", "Wešteuropäiši Summerzit"}},
	},
	{
		{{"Afghanistan", "waxtu Afganistan"}, {"Africa_Central", "Waxtu Afrique Centrale"}, {"Africa_Eastern", "Waxtu Afrique sowwu jant"}, {"Africa_Southern", "Afrique du Sud"}, {"Africa_Western", "Waxtu sowwu Afrique"}, {"Alaska", "Waxtu buñ miin ci Alaska"}, {"Amazon", "Waxtu buñ jagleel Amazon"}, {"America_Central", "CST (waxtu estàndaaru sàntaraal)"}, {"America_Eastern", "EST (waxtu estàndaaru penku)"}, {"America_Mountain", "MST (waxtu estàndaaru tundu)"}, {"America_Pacific", "PST (waxtu estàndaaru pasifik)"}, {"Apia", "Waxtu buñ miin ci Apia"}, {"Arabian", "Waxtu buñ miin ci Araab"}, {"Argentina", "Waxtu buñ miin ci Arsantiin"}, {"Argentina_Western", "Waxtu buñ miin ci sowwu Argentine"}, {"Armenia", "Waxtu buñ miin ci Armeni"}, {"Atlantic", "AST (waxtu estàndaaru penku)"}, {"Australia_Central", "Waxtu buñ miin ci Australie"}, {"Australia_CentralWestern", "Waxtu buñ miin ci diggu sowwu Australie"}, {"Australia_Eastern", "Waxtu penku bu Australie"}, {"Australia_Western", "Waxtu buñ miin ci sowwu Australie"}, {"Azerbaijan", "Waxtu Azerbaïdjan"}, {"Azores", "Waxtu buñ miin ci Azores"}, {"Bangladesh", "Waxtu buñ miin ci Bangladesh"}, {"Bhutan", "waxtu Bhoutan"}, {"Bolivia", "Waxtu Bolivie"}, {"Brasilia", "Brasilia time"}, {"Brunei", "Brunei Darussalam"}, {"Cape_Verde", "Cape Verde waxtu"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Waxtu buñ miin ci Sili"}, {"China", "Waxtu buñ miin ci Chine"}, {"Christmas", "waxtu ile bu noel"}, {"Cocos", "Waxtu ile Cocos"}, {"Colombia", "Waxtu buñ miin ci Kolombi"}, {"Cook", "Waxtu buñ miin ci Ile Cook"}, {"Cuba", "waxtu buñ miin ci Cuba"}, {"Davis", "Waxtu Davis"}, {"DumontDUrville", "Dumont-d’Urville"}, {"East_Timor", "Timor oriental"}, {"Easter", "Waxtu buñ miin ci Ile de Pâques"}, {"Ecuador", "waxtu Ecuador"}, {"Europe_Central", "CEST (waxtu estàndaaru ëroop sàntaraal)"}, {"Europe_Eastern", "EEST (waxtu estàndaaru ëroop u penku)"}, {"Europe_Further_Eastern", "waxtu penku Europe"}, {"Europe_Western", "WEST (waxtu estàndaaru ëroop u sowwu-jant)"}, {"Falkland", "Falkland waxtu buñ miin"}, {"Fiji", "Fidji"}, {"French_Guiana", "Guyane française"}, {"French_Southern", "Waxtu Sud ak Antarctique bu Français"}, {"GMT", "GMT (waxtu Greenwich)"}, {"Galapagos", "waxtu galapagos"}, {"Gambier", "Waxtu Gambier"}, {"Georgia", "Georgie waxtu"}, {"Gilbert_Islands", "waxtu ile Gilbert"}, {"Greenland_Eastern", "Waxtu buñ miin ci penku Greenland"}, {"Greenland_Western", "waxtu buñ miin ci sowwu Groenland"}, {"Gulf", "Waxtu Golf"}, {"Guyana", "Waxtu Guyana"}, {"Hawaii", "Waxtu buñ jagleel Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Waxtu buñ jagleel Hawaii-Aleutian"}, {"Hong_Kong", "waxtu buñ miin ci Hong Kong"}, {"Hovd", "Hovd waxtu standard"}, {"India", "Waxtu Inde"}, {"Indian_Ocean", "Waxtu géeju Inde"}, {"Indochina", "waxtu Indochine"}, {"Indonesia_Central", "Waxtu Enndonesi bu diggi bi"}, {"Indonesia_Eastern", "waxtu penku Enndonesi"}, {"Indonesia_Western", "waxtu sowwu Enndonesi"}, {"Iran", "Iran waxtu buñ miin"}, {"Irkutsk", "waxtu Irkutsk time"}, {"Israel", "Waxtu buñ miin ci Israel"}, {"Japan", "Waxtu japon"}, {"Kazakhstan", "Waxtu Kazakhstaan"}, {"Kazakhstan_Eastern", "Kazakhstan penku"}, {"Kazakhstan_Western", "Kazakhstan bu sowwu jant"}, {"Korea", "waxtu buñ miin ci Kore"}, {"Kosrae", "Waxtu Kosrae"}, {"Krasnoyarsk", "Krasnoyarsk waxtu"}, {"Kyrgystan", "Waxtu Kirgistan"}, {"Line_Islands", "Waxtu Ile Line"}, {"Lord_Howe", "Lord Howe waxtu buñ miin"}, {"Magadan", "Magadan, waxtu"}, {"Malaysia", "Malaysie"}, {"Maldives", "Waxtu Maldives"}, {"Marquesas", "Waxtu Marquesas"}, {"Marshall_Islands", "Waxtu Ile Marshall"}, {"Mauritius", "Waxtu buñ miin ci Maurice"}, {"Mawson", "waxtu Mawson"}, {"Mexico_Pacific", "Waxtu buñ miin ci pasifik bu Mexico"}, {"Mongolia", "Ulaanbatar"}, {"Moscow", "Moscow Waxtu"}, {"Myanmar", "waxtu Myanmar"}, {"Nauru", "waxtu Nauru"}, {"Nepal", "waxtu Nepal"}, {"New_Caledonia", "Waxtu buñ miin ci Caledonie bu bees"}, {"New_Zealand", "Waxtu buñ miin ci Nouvelle-Zélande"}, {"Newfoundland", "Terre Neuve"}, {"Niue", "Waxtu Niue"}, {"Norfolk", "Waxtu buñ miin ci Ile Norfolk"}, {"Noronha", "Vernando de Noronha"}, {"Novosibirsk", "Novosibirsk waxtu"}, {"Omsk", "Waxtu buñ miin ci Omsk"}, {"Pakistan", "Waxtu buñ miin ci Pakistan"}, {"Palau", "waxtu Palau"}, {"Papua_New_Guinea", "Papouasie-Nouvelle-Guiné"}, {"Paraguay", "paraguay waxtu"}, {"Peru", "Peru waxtu buñ miin"}, {"Philippines", "waxtu buñ miin ci filipiin"}, {"Phoenix_Islands", "waxtu ile Phoenix"}, {"Pierre_Miquelon", "Saint Pierre & Miquelon"}, {"Pitcairn", "Waxtu Pitcairn"}, {"Ponape", "Waxtu Ponape"}, {"Pyongyang", "waxtu Pyongyang"}, {"Reunion", "waxtu ndaje"}, {"Rothera", "Waxtu Rotera"}, {"Sakhalin", "Saxalin"}, {"Samoa", "Samoa Standard Time"}, {"Seychelles", "Waxtu Seychelles"}, {"Singapore", "waxtu buñ miin ci Singapuur"}, {"Solomon", "Waxtu Ile Solomon"}, {"South_Georgia", "Georgie du Sud"}, {"Suriname", "Waxtu Surinam"}, {"Syowa", "waxtu syowa"}, {"Tahiti", "waxtu Tahiti"}, {"Taipei", "Waxtu buñ miin ci Taipei"}, {"Tajikistan", "Waxtu Tajikistaan"}, {"Tokelau", "Tokelau time"}, {"Tonga", "Tonga waxtu buñ miin"}, {"Truk", "Waxtu Chuuk"}, {"Turkmenistan", "Waxtu buñ miin"}, {"Tuvalu", "Waxtu Tuvalu"}, {"Uruguay", "Uruguay waxtu buñ miin"}, {"Uzbekistan", "Waxtu buñ miin ci Ouzbékistan"}, {"Vanuatu", "Waxtu miin"}, {"Venezuela", "Waxtu Venezuela"}, {"Vladivostok", "Vladivostok ci waxtu"}, {"Volgograd", "Volgograd waxtu buñ miin"}, {"Vostok", "Waxtu Vostok"}, {"Wake", "Waxtu Ile Wake"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Waxtu Yakutsk"}, {"Yekaterinburg", "Yekatérinbourg"}, {"Yukon", "Waxtu Yukon"}},
		{{"Alaska", "Waxtu bëccëg ci Alaska"}, {"Amazon", "Waxtu ete bu Amazon"}, {"America_Central", "CDT (waxtu bëccëgu sàntaraal"}, {"America_Eastern", "EDT (waxtu bëccëgu penku)"}, {"America_Mountain", "MDT (waxtu bëccëgu tundu)"}, {"America_Pacific", "PDT (waxtu bëccëgu pasifik)"}, {"Apia", "Apia waxtu bëccëg"}, {"Arabian", "Waxtu bëccëg ci Araab"}, {"Argentina", "Waxtu ete bu Argentine"}, {"Argentina_Western", "waxtu ete bu sowwu Argentine"}, {"Armenia", "Waxtu ete bu Armeni"}, {"Atlantic", "ADT (waxtu bëccëgu atlàntik)"}, {"Australia_Central", "Waxtu bëccëg ci diggu Australie"}, {"Australia_CentralWestern", "Waxtu bëccëg ci diggu sowwu Australie"}, {"Australia_Eastern", "Waxtu buñ miin ci penku Australie"}, {"Australia_Western", "Waxtu bëccëg bu sowwu Australie"}, {"Azerbaijan", "Waxtu ete bu Azerbaïdjan"}, {"Azores", "Azores waxtu ete"}, {"Bangladesh", "Waxtu ete bu Bangladesh"}, {"Brasilia", "Brasilia summer time"}, {"Cape_Verde", "Cape Verde ci jamonoy ete"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Waxtu ete bu Sili"}, {"China", "Chine waxtu bëccëg"}, {"Colombia", "Jamonoy ete ci Kolombi"}, {"Cook", "Ile Cook xaaju ete"}, {"Cuba", "Cuba waxtu bëccëg"}, {"Easter", "Jamonoy ete ci Ile de Pâques"}, {"Europe_Central", "CEST (waxtu ete wu ëroop sàntaraal)"}, {"Europe_Eastern", "EEST (waxtu ete wu ëroop u penku)"}, {"Europe_Western", "WEST (waxtu ete wu ëroop u sowwu-jant)"}, {"Falkland", "Jamonoy ete ci ile Falkland"}, {"Fiji", "Jamonoy ete ci Fiji"}, {"Georgia", "Georgie waxtu ete"}, {"Greenland_Eastern", "Waxtu ete bu penku Greenland"}, {"Greenland_Western", "waxtu ete bu sowwu Groenland"}, {"Hawaii_Aleutian", "Waxtu bëccëg bu Hawaii-Aleutian"}, {"Hong_Kong", "Jamonoy ete ci Hong Kong"}, {"Hovd", "Hovd waxtu ete"}, {"Iran", "Waxtu bëccëg ci Iran"}, {"Irkutsk", "Waxtu ete bu Irkutsk"}, {"Israel", "Israel waxtu bëccëg"}, {"Japan", "Japon waxtu bëccëg"}, {"Korea", "waxtu bëccëg ci Kore"}, {"Krasnoyarsk", "Krasnoyarsk ci jamonoy ete"}, {"Lord_Howe", "ord Howe Daylight Time"}, {"Magadan", "Waxtu ete bu Magadan"}, {"Mauritius", "Waxtu ete bu Maurice"}, {"Mexico_Pacific", "Waxtu bëccëg ci Pacific Mexique"}, {"Mongolia", "Ulaan Baatar time"}, {"Moscow", "Waxtu ete bu Moscou"}, {"New_Caledonia", "Waxtu ete bu Nouvelle Caledonie"}, {"New_Zealand", "Nouvelle-Zélande"}, {"Newfoundland", "Terre-Neuve"}, {"Norfolk", "waxtu bëccëg ci ile Norfolk"}, {"Noronha", "Fernando de noronha temps d’été"}, {"Novosibirsk", "Novosibirsk ci jamonoy ete"}, {"Omsk", "Omsk waxtu ete"}, {"Pakistan", "Waxtu ete bu Pakistan"}, {"Paraguay", "Paraguay waxtu ete"}, {"Peru", "Peru waxtu ete"}, {"Philippines", "Jamonoy ete ci Philippines"}, {"Pierre_Miquelon", "Saint Pierre and Miquelon"}, {"Sakhalin", "Sakhalin Sakhalin"}, {"Samoa", "Samoa waxtu bëccëg"}, {"Taipei", "Taipei waxtu leeralu bis"}, {"Tonga", "Jamonoy ete ci Tonga"}, {"Turkmenistan", "Waxtu ete bu Turkmenistan"}, {"Uruguay", "Uruguay waxtu ete"}, {"Uzbekistan", "Waxtu ete bu Ouzbékistan"}, {"Vanuatu", "Waxtu ete bu Vanuatu"}, {"Vladivostok", "Vladivostok ci jamonoy ete"}, {"Volgograd", "Jamonoy ete bu Volgograd"}, {"Yakutsk", "Waxtu ete bu Yakutsk"}, {"Yekaterinburg", "Jamonoy ete"}},
	},
	{
		{{"Afghanistan", "Afghanistan Time"}, {"Africa_Central", "Central Africa Time"}, {"Africa_Eastern", "East Africa Time"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "West Africa Time"}, {"Alaska", "Alaska Standard Time"}, {"Amazon", "Amazon Standard Time"}, {"America_Central", "Central Standard Time"}, {"America_Eastern", "Eastern Standard Time"}, {"America_Mountain", "Mountain Standard Time"}, {"America_Pacific", "Pacific Standard Time"}, {"Apia", "Apia Standard Time"}, {"Arabian", "Arabian Standard Time"}, {"Argentina", "Argentina Standard Time"}, {"Argentina_Western", "Western Argentina Standard Time"}, {"Armenia", "Armenia Standard Time"}, {"Atlantic", "Atlantic Standard Time"}, {"Australia_Central", "Australian Central Standard Time"}, {"Australia_CentralWestern", "Australian Central Western Standard Time"}, {"Australia_Eastern", "Australian Eastern Standard Time"}, {"Australia_Western", "Australian Western Standard Time"}, {"Azerbaijan", "Azerbaijan Standard Time"}, {"Azores", "Azores Standard Time"}, {"Bangladesh", "Bangladesh Standard Time"}, {"Bhutan", "Bhutan Time"}, {"Bolivia", "Bolivia Time"}, {"Brasilia", "Brasilia Standard Time"}, {"Brunei", "Brunei Darussalam Time"}, {"Cape_Verde", "Cape Verde Standard Time"}, {"Chamorro", "Chamorro Standard Time"}, {"Chatham", "Chatham Standard Time"}, {"Chile", "Chile Standard Time"}, {"China", "China Standard Time"}, {"Christmas", "Christmas Island Time"}, {"Cocos", "Cocos Islands Time"}, {"Colombia", "Colombia Standard Time"}, {"Cook", "Cook Islands Standard Time"}, {"Cuba", "Cuba Standard Time"}, {"Davis", "Davis Time"}, {"DumontDUrville", "Dumont-d’Urville Time"}, {"East_Timor", "East Timor Time"}, {"Easter", "Easter Island Standard Time"}, {"Ecuador", "Ecuador Time"}, {"Europe_Central", "Central European Standard Time"}, {"Europe_Eastern", "Eastern European Standard Time"}, {"Europe_Further_Eastern", "Further-eastern European Time"}, {"Europe_Western", "Western European Standard Time"}, {"Falkland", "Falkland Islands Standard Time"}, {"Fiji", "Fiji Standard Time"}, {"French_Guiana", "French Guiana Time"}, {"French_Southern", "French Southern & Antarctic Time"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Galapagos Time"}, {"Gambier", "Gambier Time"}, {"Georgia", "Georgia Standard Time"}, {"Gilbert_Islands", "Gilbert Islands Time"}, {"Greenland_Eastern", "East Greenland Standard Time"}, {"Greenland_Western", "West Greenland Standard Time"}, {"Gulf", "Gulf Standard Time"}, {"Guyana", "Guyana Time"}, {"Hawaii", "Hawaii-Aleutian Standard Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Standard Time"}, {"Hong_Kong", "Hong Kong Standard Time"}, {"Hovd", "Hovd Standard Time"}, {"India", "India Standard Time"}, {"Indian_Ocean", "Indian Ocean Time"}, {"Indochina", "Indochina Time"}, {"Indonesia_Central", "Central Indonesia Time"}, {"Indonesia_Eastern", "Eastern Indonesia Time"}, {"Indonesia_Western", "Western Indonesia Time"}, {"Iran", "Iran Standard Time"}, {"Irkutsk", "Irkutsk Standard Time"}, {"Israel", "Israel Standard Time"}, {"Japan", "Japan Standard Time"}, {"Kazakhstan", "Kazakhstan Time"}, {"Kazakhstan_Eastern", "East Kazakhstan Time"}, {"Kazakhstan_Western", "West Kazakhstan Time"}, {"Korea", "Korean Standard Time"}, {"Kosrae", "Kosrae Time"}, {"Krasnoyarsk", "Krasnoyarsk Standard Time"}, {"Kyrgystan", "Kyrgyzstan Time"}, {"Line_Islands", "Line Islands Time"}, {"Lord_Howe", "Lord Howe Standard Time"}, {"Magadan", "Magadan Standard Time"}, {"Malaysia", "Malaysia Time"}, {"Maldives", "Maldives Time"}, {"Marquesas", "Marquesas Time"}, {"Marshall_Islands", "Marshall Islands Time"}, {"Mauritius", "Mauritius Standard Time"}, {"Mawson", "Mawson Time"}, {"Mexico_Pacific", "Mexican Pacific Standard Time"}, {"Mongolia", "Ulaanbaatar Standard Time"}, {"Moscow", "Moscow Standard Time"}, {"Myanmar", "Myanmar Time"}, {"Nauru", "Nauru Time"}, {"Nepal", "Nepal Time"}, {"New_Caledonia", "New Caledonia Standard Time"}, {"New_Zealand", "New Zealand Standard Time"}, {"Newfoundland", "Newfoundland Standard Time"}, {"Niue", "Niue Time"}, {"Norfolk", "Norfolk Island Standard Time"}, {"Noronha", "Fernando de Noronha Standard Time"}, {"Novosibirsk", "Novosibirsk Standard Time"}, {"Omsk", "Omsk Standard Time"}, {"Pakistan", "Pakistan Standard Time"}, {"Palau", "Palau Time"}, {"Papua_New_Guinea", "Papua New Guinea Time"}, {"Paraguay", "Paraguay Standard Time"}, {"Peru", "Peru Standard Time"}, {"Philippines", "Philippine Standard Time"}, {"Phoenix_Islands", "Phoenix Islands Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Standard Time"}, {"Pitcairn", "Pitcairn Time"}, {"Ponape", "Ponape Time"}, {"Pyongyang", "Pyongyang Time"}, {"Reunion", "Réunion Time"}, {"Rothera", "Rothera Time"}, {"Sakhalin", "Sakhalin Standard Time"}, {"Samoa", "Samoa Standard Time"}, {"Seychelles", "Seychelles Time"}, {"Singapore", "Singapore Standard Time"}, {"Solomon", "Solomon Islands Time"}, {"South_Georgia", "South Georgia Time"}, {"Suriname", "Suriname Time"}, {"Syowa", "Syowa Time"}, {"Tahiti", "Tahiti Time"}, {"Taipei", "Taipei Standard Time"}, {"Tajikistan", "Tajikistan Time"}, {"Tokelau", "Tokelau Time"}, {"Tonga", "Tonga Standard Time"}, {"Truk", "Chuuk Time"}, {"Turkmenistan", "Turkmenistan Standard Time"}, {"Tuvalu", "Tuvalu Time"}, {"Uruguay", "Uruguay Standard Time"}, {"Uzbekistan", "Uzbekistan Standard Time"}, {"Vanuatu", "Vanuatu Standard Time"}, {"Venezuela", "Venezuela Time"}, {"Vladivostok", "Vladivostok Standard Time"}, {"Volgograd", "Volgograd Standard Time"}, {"Vostok", "Vostok Time"}, {"Wake", "Wake Island Time"}, {"Wallis", "Wallis & Futuna Time"}, {"Yakutsk", "Yakutsk Standard Time"}, {"Yekaterinburg", "Yekaterinburg Standard Time"}, {"Yukon", "Yukon Time"}},
		{{"Alaska", "Alaska Daylight Time"}, {"Amazon", "Amazon Summer Time"}, {"America_Central", "Central Daylight Time"}, {"America_Eastern", "Eastern Daylight Time"}, {"America_Mountain", "Mountain Daylight Time"}, {"America_Pacific", "Pacific Daylight Time"}, {"Apia", "Apia Daylight Time"}, {"Arabian", "Arabian Daylight Time"}, {"Argentina", "Argentina Summer Time"}, {"Argentina_Western", "Western Argentina Summer Time"}, {"Armenia", "Armenia Summer Time"}, {"Atlantic", "Atlantic Daylight Time"}, {"Australia_Central", "Australian Central Daylight Time"}, {"Australia_CentralWestern", "Australian Central Western Daylight Time"}, {"Australia_Eastern", "Australian Eastern Daylight Time"}, {"Australia_Western", "Australian Western Daylight Time"}, {"Azerbaijan", "Azerbaijan Summer Time"}, {"Azores", "Azores Summer Time"}, {"Bangladesh", "Bangladesh Summer Time"}, {"Brasilia", "Brasilia Summer Time"}, {"Cape_Verde", "Cape Verde Summer Time"}, {"Chatham", "Chatham Daylight Time"}, {"Chile", "Chile Summer Time"}, {"China", "China Daylight Time"}, {"Colombia", "Colombia Summer Time"}, {"Cook", "Cook Islands Half Summer Time"}, {"Cuba", "Cuba Daylight Time"}, {"Easter", "Easter Island Summer Time"}, {"Europe_Central", "Central European Summer Time"}, {"Europe_Eastern", "Eastern European Summer Time"}, {"Europe_Western", "Western European Summer Time"}, {"Falkland", "Falkland Islands Summer Time"}, {"Fiji", "Fiji Summer Time"}, {"Georgia", "Georgia Summer Time"}, {"Greenland_Eastern", "East Greenland Summer Time"}, {"Greenland_Western", "West Greenland Summer Time"}, {"Hawaii_Aleutian", "Hawaii-Aleutian Daylight Time"}, {"Hong_Kong", "Hong Kong Summer Time"}, {"Hovd", "Hovd Summer Time"}, {"Iran", "Iran Daylight Time"}, {"Irkutsk", "Irkutsk Summer Time"}, {"Israel", "Israel Daylight Time"}, {"Japan", "Japan Daylight Time"}, {"Korea", "Korean Daylight Time"}, {"Krasnoyarsk", "Krasnoyarsk Summer Time"}, {"Lord_Howe", "Lord Howe Daylight Time"}, {"Magadan", "Magadan Summer Time"}, {"Mauritius", "Mauritius Summer Time"}, {"Mexico_Pacific", "Mexican Pacific Daylight Time"}, {"Mongolia", "Ulaanbaatar Summer Time"}, {"Moscow", "Moscow Summer Time"}, {"New_Caledonia", "New Caledonia Summer Time"}, {"New_Zealand", "New Zealand Daylight Time"}, {"Newfoundland", "Newfoundland Daylight Time"}, {"Norfolk", "Norfolk Island Daylight Time"}, {"Noronha", "Fernando de Noronha Summer Time"}, {"Novosibirsk", "Novosibirsk Summer Time"}, {"Omsk", "Omsk Summer Time"}, {"Pakistan", "Pakistan Summer Time"}, {"Paraguay", "Paraguay Summer Time"}, {"Peru", "Peru Summer Time"}, {"Philippines", "Philippine Summer Time"}, {"Pierre_Miquelon", "St. Pierre & Miquelon Daylight Time"}, {"Sakhalin", "Sakhalin Summer Time"}, {"Samoa", "Samoa Daylight Time"}, {"Taipei", "Taipei Daylight Time"}, {"Tonga", "Tonga Summer Time"}, {"Turkmenistan", "Turkmenistan Summer Time"}, {"Uruguay", "Uruguay Summer Time"}, {"Uzbekistan", "Uzbekistan Summer Time"}, {"Vanuatu", "Vanuatu Summer Time"}, {"Vladivostok", "Vladivostok Summer Time"}, {"Volgograd", "Volgograd Summer Time"}, {"Yakutsk", "Yakutsk Summer Time"}, {"Yekaterinburg", "Yekaterinburg Summer Time"}},
	},
	{
		{{"Afghanistan", "अफ़गानिस्ताने दा टैम"}, {"Africa_Central", "मध्य अफ़्रीका दा टैम"}, {"Africa_Eastern", "पैलेी अफ़्रीका दा टैम"}, {"Africa_Southern", "दखण अफ़्रीका दा मानक टैम"}, {"Africa_Western", "पश्चिम अफ़्रीका दा टैम"}, {"Alaska", "अलास्\u200dका दा मानक टैम"}, {"Amazon", "एमेज़ोन दा मानक टैम"}, {"America_Central", "उत्तरी अमरिकी केंद्रीय दा मानक टैम"}, {"America_Eastern", "उत्तरी अमरिकी पैलेी दा मानक टैम"}, {"America_Mountain", "उत्तरी अमेरिकी पहाड़े दा मानक टैम"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांते दा मानक टैम"}, {"Apia", "एपिआ दा मानक टैम"}, {"Arabian", "अरब दा मानक टैम"}, {"Argentina", "अर्जेंटीना दा मानक टैम"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना दा मानक टैम"}, {"Armenia", "आर्मेनिया दा मानक टैम"}, {"Atlantic", "अटलांटिक दा मानक टैम"}, {"Australia_Central", "अस्\u200dट्रेलिया दे केंद्रीय दा मानक टैम"}, {"Australia_CentralWestern", "अस्\u200dट्रेलिया केंद्रीय पश्चिमी दा मानक टैम"}, {"Australia_Eastern", "अस्\u200dट्रेलियाई पैलेी दा मानक टैम"}, {"Australia_Western", "अस्ट्रेलिया पश्चिमी दा मानक टैम"}, {"Azerbaijan", "अज़रबैजान दा मानक टैम"}, {"Azores", "अज़ोरेस दा मानक टैम"}, {"Bangladesh", "बांग्लादेशे दा मानक टैम"}, {"Bhutan", "भूटाने दा टैम"}, {"Bolivia", "बोलीविया दा टैम"}, {"Brasilia", "ब्राज़ीलिया दा मानक टैम"}, {"Brunei", "ब्रूनेई दारूस्सलम दा टैम"}, {"Cape_Verde", "केप वर्ड दा मानक टैम"}, {"Chamorro", "चामोरो दा मानक टैम"}, {"Chatham", "चैथम दा मानक टैम"}, {"Chile", "चिली दा मानक टैम"}, {"China", "चीन दा मानक टैम"}, {"Christmas", "क्रिसमस टापू दा टैम"}, {"Cocos", "कोकोस टापुआं दे झुंडे दा टैम"}, {"Colombia", "कोलंबिया दा मानक टैम"}, {"Cook", "कुक टापुआं दे झुंडे दा मानक टैम"}, {"Cuba", "क्यूबा दा मानक टैम"}, {"Davis", "डेविस दा टैम"}, {"DumontDUrville", "ड्यूमोंट डी अर्विले दा टैम"}, {"East_Timor", "पैलेी तिमोरे दा टैम"}, {"Easter", "ईस्टर टापू दा मानक टैम"}, {"Ecuador", "इक्वाडोर दा टैम"}, {"Europe_Central", "बिचले यूरोपे दा मानक टैम"}, {"Europe_Eastern", "पैलेी यूरोपे दा मानक टैम"}, {"Europe_Further_Eastern", "अग्र पैलेी यूरोपे दा टैम"}, {"Europe_Western", "पश्चिमी यूरोपे दा मानक टैम"}, {"Falkland", "फ़ॉकलैंड टापुआं दे झुंडे दा मानक टैम"}, {"Fiji", "फ़िजी दा मानक टैम"}, {"French_Guiana", "फ़्रेंच गुयाना दा टैम"}, {"French_Southern", "दखणे बखें फ़्रांस कने अंटार्कटिक दा टैम"}, {"GMT", "ग्रीनविच मीन टैम"}, {"Galapagos", "गैलापेगोस दा टैम"}, {"Gambier", "गैंबियर दा टैम"}, {"Georgia", "जॉर्जिया दा मानक टैम"}, {"Gilbert_Islands", "गिल्बर्ट टापुआं दे झुंडे दा टैम"}, {"Greenland_Eastern", "पैलेी ग्रीनलैंड दा मानक टैम"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड दा मानक टैम"}, {"Gulf", "खाड़ी दा मानक टैम"}, {"Guyana", "गुयाना दा टैम"}, {"Hawaii", "हवाई–आल्यूशन दा मानक टैम"}, {"Hawaii_Aleutian", "हवाई–आल्यूशन दा मानक टैम"}, {"Hong_Kong", "हाँग काँग दा मानक टैम"}, {"Hovd", "होव्ड दा मानक टैम"}, {"India", "भारते दा मानक टैम"}, {"Indian_Ocean", "हिंद महासागर दा टैम"}, {"Indochina", "इंडोचाइना दा टैम"}, {"Indonesia_Central", "बिचले इंडोनेशिया दा टैम"}, {"Indonesia_Eastern", "पैलेी इंडोनेशिया दा टैम"}, {"Indonesia_Western", "पश्चिमी इंडोनेशिया दा टैम"}, {"Iran", "ईरान दा मानक टैम"}, {"Irkutsk", "इर्कुत्स्क दा मानक टैम"}, {"Israel", "इज़राइल दा मानक टैम"}, {"Japan", "जापाने दा मानक टैम"}, {"Kazakhstan_Eastern", "पैले कज़ाखस्तान दा टैम"}, {"Kazakhstan_Western", "पश्चिम कज़ाखस्तान दा टैम"}, {"Korea", "कोरियाई दा मानक टैम"}, {"Kosrae", "कोसराए दा टैम"}, {"Krasnoyarsk", "क्रास्नोयार्स्क दा मानक टैम"}, {"Kyrgystan", "किर्गिस्\u200dतान दा टैम"}, {"Line_Islands", "लाइन टापुआं दे झुंडे दा टैम"}, {"Lord_Howe", "लॉर्ड होवे दा मानक टैम"}, {"Magadan", "मागादान दा मानक टैम"}, {"Malaysia", "मलेशिया दा टैम"}, {"Maldives", "मालदीव दा टैम"}, {"Marquesas", "मार्केसस दा टैम"}, {"Marshall_Islands", "मार्शल टापुआं दे झुंडे दा टैम"}, {"Mauritius", "मॉरीशस दा मानक टैम"}, {"Mawson", "माव्सन दा टैम"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत दा मानक टैम"}, {"Mongolia", "उलान बटोर दा मानक टैम"}, {"Moscow", "मॉस्को दा मानक टैम"}, {"Myanmar", "म्यांमार दा टैम"}, {"Nauru", "नौरू दा टैम"}, {"Nepal", "नेपाल दा टैम"}, {"New_Caledonia", "न्यू कैलेडोनिया दा मानक टैम"}, {"New_Zealand", "न्यूज़ीलैंड दा मानक टैम"}, {"Newfoundland", "न्यूफ़ाउंडलैंड दा मानक टैम"}, {"Niue", "नीयू दा टैम"}, {"Norfolk", "नॉरफ़ॉक टापू दा मानक टैम"}, {"Noronha", "फ़र्नांर्डो डे नोरोन्हा मानक दा टैम"}, {"Novosibirsk", "नोवोसिबिर्स्क दा मानक टैम"}, {"Omsk", "ओम्स्क दा मानक टैम"}, {"Pakistan", "पाकिस्ताने दा मानक टैम"}, {"Palau", "पलाउ दा टैम"}, {"Papua_New_Guinea", "पापुआ न्यू गिनी दा टैम"}, {"Paraguay", "पैराग्वे दा मानक टैम"}, {"Peru", "पेरू दा मानक टैम"}, {"Philippines", "फ़िलिपीन दा मानक टैम"}, {"Phoenix_Islands", "फ़ीनिक्स टापुआं दे झुंडे दा टैम"}, {"Pierre_Miquelon", "सेंट पिएरे कने मिक्वेलान दा मानक टैम"}, {"Pitcairn", "पिटकैर्न दा टैम"}, {"Ponape", "पोनापे दा टैम"}, {"Pyongyang", "प्योंगयांग दा टैम"}, {"Reunion", "रीयूनियन दा टैम"}, {"Rothera", "रोथेरा दा टैम"}, {"Sakhalin", "सखालिन दा मानक टैम"}, {"Samoa", "समोआ दा मानक टैम"}, {"Seychelles", "सेशेल्स दा टैम"}, {"Singapore", "सिंगापुरे दा टैम"}, {"Solomon", "सोलोमन टापुआं दे झुंडे दा टैम"}, {"South_Georgia", "दखण जॉर्जिया दा टैम"}, {"Suriname", "सूरीनामे दा टैम"}, {"Syowa", "स्योवा दा टैम"}, {"Tahiti", "ताहिती दा टैम"}, {"Taipei", "ताइपे दा मानक टैम"}, {"Tajikistan", "ताजिकिस्ताने दा टैम"}, {"Tokelau", "टोकेलाऊ दा टैम"}, {"Tonga", "टोंगा दा मानक टैम"}, {"Truk", "चुक दा टैम"}, {"Turkmenistan", "तुर्कमेनिस्ताने दा मानक टैम"}, {"Tuvalu", "तुवालू दा टैम"}, {"Uruguay", "उरुग्वे दा मानक टैम"}, {"Uzbekistan", "उज़्बेकिस्तान दा मानक टैम"}, {"Vanuatu", "वनुआतू दा मानक टैम"}, {"Venezuela", "वेनेज़ुएला दा टैम"}, {"Vladivostok", "व्लादिवोस्तोक दा मानक टैम"}, {"Volgograd", "वोल्गोग्राड दा मानक टैम"}, {"Vostok", "वोस्तोक दा टैम"}, {"Wake", "वेक टापू दा टैम"}, {"Wallis", "वालिस कने फ़्यूचूना दा टैम"}, {"Yakutsk", "याकुत्स्क दा मानक टैम"}, {"Yekaterinburg", "येकातेरिनबर्ग दा मानक टैम"}, {"Yukon", "युकॉन टैम"}},
		{{"Alaska", "अलास्\u200dका दे ध्याड़े दे उजाले दा टैम"}, {"Amazon", "एमेज़ोन दी तोंदिया दा टैम"}, {"America_Central", "उत्तरी अमरिकी केंद्रीय दे ध्याड़े दे उजाले दा टैम"}, {"America_Eastern", "उत्तरी अमरिकी पैलेी दे ध्याड़े दे उजाले दा टैम"}, {"America_Mountain", "उत्तरी अमेरिकी पहाड़े दे ध्याड़े दे उजाले दा टैम"}, {"America_Pacific", "उत्तरी अमेरिकी प्रशांते दे ध्याड़े दे उजाले दा टैम"}, {"Apia", "एपिआ दे ध्याड़े दे उजाले दा टैम"}, {"Arabian", "अरब दे ध्याड़े दे उजाले दा टैम"}, {"Argentina", "अर्जेंटीना दी तोंदिया दा टैम"}, {"Argentina_Western", "पश्चिमी अर्जेंटीना दी तोंदिया दा टैम"}, {"Armenia", "आर्मेनिया दी तोंदिया दा टैम"}, {"Atlantic", "अटलांटिक दे ध्याड़े दे उजाले दा टैम"}, {"Australia_Central", "अस्\u200dट्रेलिया दे केंद्रीय दे ध्याड़े दे उजाले दा टैम"}, {"Australia_CentralWestern", "अस्\u200dट्रेलियाई केंद्रीय पश्चिमी ध्याड़े दे उजाले दा टैम"}, {"Australia_Eastern", "अस्\u200dट्रेलियाई पैलेी दे ध्याड़े दे उजाले दा टैम"}, {"Australia_Western", "अस्ट्रेलिया पश्चिमी दे ध्याड़े दे उजाले दा टैम"}, {"Azerbaijan", "अज़रबैजान दी तोंदिया दा टैम"}, {"Azores", "अज़ोरेस दी तोंदिया दा टैम"}, {"Bangladesh", "बांग्लादेशे दी तोंदिया दा टैम"}, {"Brasilia", "ब्राज़ीलिया दी तोंदिया दा टैम"}, {"Cape_Verde", "केप वर्ड दी तोंदिया दा टैम"}, {"Chatham", "चैथम दे ध्याड़े दे उजाले दा टैम"}, {"Chile", "चिली दी तोंदिया दा टैम"}, {"China", "चीन दे ध्याड़े दे उजाले दा टैम"}, {"Colombia", "कोलंबिया दी तोंदिया दा टैम"}, {"Cook", "कुक टापुआं दे झुंडे दा अद्धी तोंदिया दा टैम"}, {"Cuba", "क्यूबा दे ध्याड़े दे उजाले दा टैम"}, {"Easter", "ईस्टर टापू दी तोंदिया दा टैम"}, {"Europe_Central", "बिचले यूरोपे दी तोंदिया दा टैम"}, {"Europe_Eastern", "पैलेी यूरोपे दी तोंदिया दा टैम"}, {"Europe_Western", "पश्चिमी यूरोपे दी तोंदिया दा टैम"}, {"Falkland", "फ़ॉकलैंड टापुआं दे झुंडे दी तोंदिया दा टैम"}, {"Fiji", "फ़िजी दी तोंदिया दा टैम"}, {"Georgia", "जॉर्जिया दी तोंदिया दा टैम"}, {"Greenland_Eastern", "पैलेी ग्रीनलैंड दी तोंदिया दा टैम"}, {"Greenland_Western", "पश्चिमी ग्रीनलैंड दी तोंदिया दा टैम"}, {"Hawaii_Aleutian", "हवाई–आल्यूशन दे ध्याड़े दे उजाले दा टैम"}, {"Hong_Kong", "हाँग काँग दी तोंदिया दा टैम"}, {"Hovd", "होव्ड दी तोंदिया दा टैम"}, {"Iran", "ईरान दे ध्याड़े दे उजाले दा टैम"}, {"Irkutsk", "इर्कुत्स्क दी तोंदिया दा टैम"}, {"Israel", "इज़राइल दे ध्याड़े दे उजाले दा टैम"}, {"Japan", "जापाने दे ध्याड़े दे उजाले दा टैम"}, {"Korea", "कोरियाई दे ध्याड़े दे उजाले दा टैम"}, {"Krasnoyarsk", "क्रास्नोयार्स्क दी तोंदिया दा टैम"}, {"Lord_Howe", "लॉर्ड होवे दे ध्याड़े दे उजाले दा टैम"}, {"Magadan", "मागादान दी तोंदिया दा टैम"}, {"Mauritius", "मॉरीशस दी तोंदिया दा टैम"}, {"Mexico_Pacific", "मेक्सिकन प्रशांत दे ध्याड़े दे उजाले दा टैम"}, {"Mongolia", "उलान बटोर दी तोंदिया दा टैम"}, {"Moscow", "मॉस्को दि तोंदिया दा टैम"}, {"New_Caledonia", "न्यू कैलेडोनिया दी तोंदिया दा टैम"}, {"New_Zealand", "न्यूज़ीलैंड दे ध्याड़े दे उजाले दा टैम"}, {"Newfoundland", "न्यूफ़ाउंडलैंड दे ध्याड़े दे उजाले दा टैम"}, {"Norfolk", "नॉरफ़ॉक टापू दे ध्याड़े दे उजाले दा टैम"}, {"Noronha", "फ़र्नांर्डो डे नोरोन्हा दे तोंदिया दा टैम"}, {"Novosibirsk", "नोवोसिबिर्स्क दी तोंदिया दा टैम"}, {"Omsk", "ओम्स्क दी तोंदिया दा टैम"}, {"Pakistan", "पाकिस्ताने दी तोंदिया दा टैम"}, {"Paraguay", "पैराग्वे दी तोंदिया दा टैम"}, {"Peru", "पेरू दी तोंदिया दा टैम"}, {"Philippines", "फ़िलिपीन दी तोंदिया दा टैम"}, {"Pierre_Miquelon", "सेंट पिएरे कने मिक्वेलान दे ध्याड़े दे उजाले दा टैम"}, {"Sakhalin", "सखालिन दी तोंदिया दा टैम"}, {"Samoa", "समोआ दे ध्याड़े दे उजाले दा टैम"}, {"Taipei", "ताइपे दे ध्याड़े दे उजाले दा टैम"}, {"Tonga", "टोंगा दी तोंदिया दा टैम"}, {"Turkmenistan", "तुर्कमेनिस्ताने दी तोंदिया दा टैम"}, {"Uruguay", "उरुग्वे दी तोंदिया दा टैम"}, {"Uzbekistan", "उज़्बेकिस्तान दी तोंदिया दा टैम"}, {"Vanuatu", "वनुआतू दी तोंदिया दा टैम"}, {"Vladivostok", "व्लादिवोस्तोक दी तोंदिया दा टैम"}, {"Volgograd", "वोल्गोग्राड दी तोंदिया दा टैम"}, {"Yakutsk", "याकुत्स्क दी तोंदिया दा टैम"}, {"Yekaterinburg", "येकातेरिनबर्ग दी तोंदिया दा टैम"}},
	},
	{
		{{"Afghanistan", "Àkókò Afghanistan"}, {"Africa_Central", "Àkókò Àárín Afírikà"}, {"Africa_Eastern", "Àkókò Ìlà-Oòrùn Afírikà"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "Àkókò Ìwọ̀-Oòrùn Afírikà"}, {"Alaska", "Àkókò Àfẹnukò Alásíkà"}, {"Amazon", "Àkókò Afẹnukò Amásọ́nì"}, {"America_Central", "àkókò asiko àárín gbùngbùn"}, {"America_Eastern", "Akókò Àsikò Ìha Ìla Oòrùn"}, {"America_Mountain", "Àkókò asiko òkè"}, {"America_Pacific", "Àkókò àsikò Pàsífíìkì"}, {"Apia", "Àkókò Àfẹnukò Apia"}, {"Arabian", "Àkókò Àfẹnukò Arabia"}, {"Argentina", "Aago àsìkò Argentina"}, {"Argentina_Western", "Àkókò Iwọ́-oòrùn Àfẹnukò Ajẹ́ntínà"}, {"Armenia", "Àkókò Àfẹnukò Armenia"}, {"Atlantic", "Àkókò àsikò Àtìláńtíìkì"}, {"Australia_Central", "Àkókò Àfẹnukò Ààrin Gùngùn Australia"}, {"Australia_CentralWestern", "Àkókò Àfẹnukò Ààrin Gùngùn Ìwọ̀-Oòrùn Australia"}, {"Australia_Eastern", "Àkókò Àfẹnukò Ìlà-Oòrùn Australia"}, {"Australia_Western", "Àkókò Àfẹnukò Ìwọ̀-Oòrùn Australia"}, {"Azerbaijan", "Àkókò Àfẹnukò Azerbaijan"}, {"Azores", "Àkókò Àfẹnukò Ásọ́sì"}, {"Bangladesh", "Àkókò Àfẹnukò Bangladesh"}, {"Bhutan", "Àkókò Bhutan"}, {"Bolivia", "Aago Bolivia"}, {"Brasilia", "Aago àsìkò Bùràsílíà"}, {"Brunei", "Brunei Darussalam Time"}, {"Cape_Verde", "Àkókò Àfẹnukò Képú Fáàdì"}, {"Chamorro", "Àkókò Àfẹnukò Chamorro"}, {"Chatham", "Àkókò Àfẹnukò Chatam"}, {"Chile", "Àkókò Àfẹnukò Ṣílè"}, {"China", "Àkókò Ìfẹnukòsí Ṣáínà"}, {"Christmas", "Àkókò Erékùsù Christmas"}, {"Cocos", "Àkókò Àwọn Erékùsù Cocos"}, {"Colombia", "Aago àsìkò Kolombia"}, {"Cook", "Àkókò Àfẹnukò Àwọn Erekusu Kuuku"}, {"Cuba", "Àkókò Àfẹnukò Kúbà"}, {"Davis", "Àkókò Davis"}, {"DumontDUrville", "Àkókò Dumont-d’Urville"}, {"East_Timor", "Àkókò Ìlà oorùn Timor"}, {"Easter", "Aago àsìkò Easter Island"}, {"Ecuador", "Aago Ecuador"}, {"Europe_Central", "Àkókò Àárin àsikò Europe"}, {"Europe_Eastern", "Àkókò àsikò Ìhà Ìlà Oòrùn Europe"}, {"Europe_Further_Eastern", "Àkókò Iwájú Ìlà Oòrùn Yúróòpù"}, {"Europe_Western", "Àkókò àsikò Ìwọ Oòrùn Europe"}, {"Falkland", "Àkókò Àfẹnukò Etíkun Fókílándì"}, {"Fiji", "Àkókò Àfẹnukò Fiji"}, {"French_Guiana", "Àkókò Gúyánà Fáránsè"}, {"French_Southern", "Àkókò Gúsù Fáransé àti Àntátíìkì"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Aago Galapago"}, {"Gambier", "Àkókò Gambia"}, {"Georgia", "Àkókò Àfẹnukò Georgia"}, {"Gilbert_Islands", "Àkókò Àwọn Erekusu Gilibati"}, {"Greenland_Eastern", "Àkókò Ìwọ̀ Ìfẹnukò oorùn Greenland"}, {"Greenland_Western", "Àkókò Àfẹnukò Ìwọ̀ Oòrùn Greenland"}, {"Gulf", "Àkókò Àfẹnukò Gulf"}, {"Guyana", "Àkókò Gúyànà"}, {"Hawaii", "Àkókò Àfẹnukò Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Àkókò Àfẹnukò Hawaii-Aleutian"}, {"Hong_Kong", "Àkókò Ìfẹnukòsí Hong Kong"}, {"Hovd", "Àkókò Ìfẹnukòsí Hofidi"}, {"India", "Àkókò Àfẹnukò India"}, {"Indian_Ocean", "Àkókò Etíkun Índíà"}, {"Indochina", "Àkókò Indochina"}, {"Indonesia_Central", "Àkókò Ààrin Gbùngbùn Indonesia"}, {"Indonesia_Eastern", "Àkókò Ìlà oorùn Indonesia"}, {"Indonesia_Western", "Àkókò Ìwọ̀ oorùn Indonesia"}, {"Iran", "Àkókò Àfẹnukò Irani"}, {"Irkutsk", "Àkókò Àfẹnukò Íkósíkì"}, {"Israel", "Àkókò Àfẹnukò Israel"}, {"Japan", "Àkókò Ìfẹnukòsí Japan"}, {"Kazakhstan", "Aago Kasasitáànì"}, {"Kazakhstan_Eastern", "Àkókò Ìlà-Oòrùn Kasasitáànì"}, {"Kazakhstan_Western", "Àkókò Ìwọ̀-Oòrùn Kasasitáànì"}, {"Korea", "Àkókò Ìfẹnukòsí Koria"}, {"Kosrae", "Àkókò Kosirai"}, {"Krasnoyarsk", "Àkókò Àfẹnukò Krasinoyasiki"}, {"Kyrgystan", "Àkókò Kirigisitaani"}, {"Line_Islands", "Àkókò Àwọn Erekusu Laini"}, {"Lord_Howe", "Àkókò Àfẹnukò Lord Howe"}, {"Magadan", "Àkókò Àfẹnukò Magadani"}, {"Malaysia", "Àkókò Malaysia"}, {"Maldives", "Àkókò Maldives"}, {"Marquesas", "Àkókò Makuesasi"}, {"Marshall_Islands", "Àkókò Àwọn Erekusu Masaali"}, {"Mauritius", "Àkókò Àfẹnukò Máríṣúṣì"}, {"Mawson", "Àkókò Mawson"}, {"Mexico_Pacific", "Àkókò Àfẹnukò Pásífíìkì Mẹ́síkò"}, {"Mongolia", "Àkókò Ìfẹnukòsí Ulaanbaatar"}, {"Moscow", "Àkókò Àfẹnukò Mosiko"}, {"Myanmar", "Àkókò Ìlà Myanmar"}, {"Nauru", "Àkókò Nauru"}, {"Nepal", "Àkókò Nepali"}, {"New_Caledonia", "Àkókò Àfẹnukò Kalidonia Tuntun"}, {"New_Zealand", "Àkókò Àfẹnukò New zealand"}, {"Newfoundland", "Àkókò Àfẹnukò Newfoundland"}, {"Niue", "Àkókò Niue"}, {"Norfolk", "Àkókò Àfẹnukò Erékùsù Norfolk"}, {"Noronha", "Aago àsìkò Fenando de Norona"}, {"Novosibirsk", "Àkókò Àfẹnukò Nofosibiriki"}, {"Omsk", "Àkókò Àfẹnukò Omisiki"}, {"Pakistan", "Àkókò Àfẹnukò Pakistani"}, {"Palau", "Àkókò Palau"}, {"Papua_New_Guinea", "Àkókò Papua New Guinea"}, {"Paraguay", "Àkókò Àfẹnukò Párágúwè"}, {"Peru", "Àkókò Àfẹnukò Pérù"}, {"Philippines", "Àkókò Àfẹnukò Filipininni"}, {"Phoenix_Islands", "Àkókò Àwọn Erékùsù Phoenix"}, {"Pierre_Miquelon", "Àkókò Àfẹnukò Pierre & Miquelon"}, {"Pitcairn", "Àkókò Pitcairn"}, {"Ponape", "Àkókò Ponape"}, {"Pyongyang", "Àkókò Pyongyangi"}, {"Reunion", "Àkókò Rẹ́yúníọ́nì"}, {"Rothera", "Àkókò Rothera"}, {"Sakhalin", "Àkókò Àfẹnukò Sakhalin"}, {"Samoa", "Àkókò Àfẹnukò Samoa"}, {"Seychelles", "Àkókò Sèṣẹ́ẹ̀lì"}, {"Singapore", "Àkókò Àfẹnukò Singapore"}, {"Solomon", "Àkókò Àwọn Erekusu Solomon"}, {"South_Georgia", "Àkókò Gúsù Jọ́jíà"}, {"Suriname", "Àkókò Súrínámù"}, {"Syowa", "Àkókò Syowa"}, {"Tahiti", "Àkókò Tahiti"}, {"Taipei", "Àkókò Ìfẹnukòsí Taipei"}, {"Tajikistan", "Àkókò Tajikisitaani"}, {"Tokelau", "Àkókò Tokelau"}, {"Tonga", "Àkókò Àfẹnukò Tonga"}, {"Truk", "Àkókò Chuuk"}, {"Turkmenistan", "Àkókò Àfẹnukò Turkimenistani"}, {"Tuvalu", "Àkókò Tufalu"}, {"Uruguay", "Àkókò Àfẹnukò Úrúgúwè"}, {"Uzbekistan", "Àkókò Àfẹnukò Usibekistani"}, {"Vanuatu", "Àkókò Àfẹnukò Fanuatu"}, {"Venezuela", "Aago Venezuela"}, {"Vladivostok", "Àkókò Àfẹnukò Filadifositoki"}, {"Volgograd", "Àkókò Àfẹnukò Foligogiradi"}, {"Vostok", "Àkókò Vostok"}, {"Wake", "Àkókò Erékùsù Wake"}, {"Wallis", "Àkókò Wallis & Futuina"}, {"Yakutsk", "Àkókò Àfẹnukò Yatutsk"}, {"Yekaterinburg", "Àkókò Àfẹnukò Yekaterinburg"}, {"Yukon", "Àkókò Yúkọ́nì"}},
		{{"Alaska", "Àkókò Ojúmọ́ Alásíkà"}, {"Amazon", "Àkókò Oru Amásọ́nì"}, {"America_Central", "Akókò àárín gbùngbùn ojúmọmọ"}, {"America_Eastern", "Àkókò ojúmọmọ Ìhà Ìlà Oòrun"}, {"America_Mountain", "Àkókò ojúmọmọ Ori-òkè"}, {"America_Pacific", "Àkókò Ìyálẹta Pàsífíìkì"}, {"Apia", "Àkókò Ojúmọmọ Apia"}, {"Arabian", "Àkókò Ojúmọmọ Arabia"}, {"Argentina", "Aago Soma Argentina"}, {"Argentina_Western", "Àkókò Oru Iwọ́-oòrùn Ajẹ́ntínà"}, {"Armenia", "Àkókò Sọmà Arabia"}, {"Atlantic", "Àkókò Ìyálẹta Àtìláńtíìkì"}, {"Australia_Central", "Àkókò Ojúmọmọ Ààrin Gùngùn Australia"}, {"Australia_CentralWestern", "Àkókò Ojúmọmọ Ààrin Gùngùn Ìwọ̀-Oòrùn Australia"}, {"Australia_Eastern", "Àkókò Ojúmọmọ Ìlà-Oòrùn Australia"}, {"Australia_Western", "Àkókò Ojúmọmọ Ìwọ̀-Oòrùn Australia"}, {"Azerbaijan", "Àkókò Sọmà Azerbaijan"}, {"Azores", "Àkókò Ooru Ásọ́sì"}, {"Bangladesh", "Àkókò Sọmà Bangladesh"}, {"Brasilia", "Aago Soma Brasilia"}, {"Cape_Verde", "Àkókò Ẹ̀rún Képú Fáàdì"}, {"Chatham", "Àkókò Ojúmọmọ Chatam"}, {"Chile", "Àkókò Oru Ṣílè"}, {"China", "Àkókò Ojúmọmọ Ṣáínà"}, {"Colombia", "Aago Soma Colombia"}, {"Cook", "Àkókò Ilaji Sọma Àwọn Erekusu Kuuku"}, {"Cuba", "Àkókò Ojúmọmọ Kúbà"}, {"Easter", "Aago Soma Easter Island"}, {"Europe_Central", "Àkókò Àárin Sọmà Europe"}, {"Europe_Eastern", "Àkókò Sọmà Ìha Ìlà Oòrùn Europe"}, {"Europe_Western", "Àkókò Sọmà Ìhà Ìwọ Oòrùn Europe"}, {"Falkland", "Àkókò Ooru Etíkun Fókílándì"}, {"Fiji", "Àkókò Sọma Fiji"}, {"Georgia", "Àkókò Sọmà Georgia"}, {"Greenland_Eastern", "Àkókò ìgbà Ooru Greenland"}, {"Greenland_Western", "Àkókò Àfẹnukò Ìgba Oòru Greenland"}, {"Hawaii_Aleutian", "Àkókò Ojúmọmọ Hawaii-Aleutian"}, {"Hong_Kong", "Àkókò Sọmà Hong Kong"}, {"Hovd", "Àkókò Sọmà Hofidi"}, {"Iran", "Àkókò Ojúmọmọ Irani"}, {"Irkutsk", "Àkókò Sọmà Íkúsíkì"}, {"Israel", "Àkókò Ojúmọmọ Israel"}, {"Japan", "Àkókò Sọmà Japan"}, {"Korea", "Àkókò Ojúmọmọ Koria"}, {"Krasnoyarsk", "Àkókò Sọmà Krasinoyasiki"}, {"Lord_Howe", "Àkókò Ojúmọmọ Lord Howe"}, {"Magadan", "Àkókò Sọmà Magadani"}, {"Mauritius", "Àkókò Ooru Máríṣúṣì"}, {"Mexico_Pacific", "Àkókò Ojúmọmọ Pásífíìkì Mẹ́síkò"}, {"Mongolia", "Àkókò Sọmà Ulaanbaatar"}, {"Moscow", "Àkókò Sọmà Mosiko"}, {"New_Caledonia", "Àkókò Sọma Kalidonia Tuntun"}, {"New_Zealand", "Àkókò Ojúmọmọ New Zealand"}, {"Newfoundland", "Àkókò Ojúmọmọ Newfoundland"}, {"Norfolk", "Àkókò Ojúmọmọ Erékùsù Norfolk"}, {"Noronha", "Aago Soma Fernando de Noronha"}, {"Novosibirsk", "Àkókò Sọmà Noforibisiki"}, {"Omsk", "Àkókò Sọmà Omisiki"}, {"Pakistan", "Àkókò Sọmà Pakistani"}, {"Paraguay", "Àkókò Ooru Párágúwè"}, {"Peru", "Àkókò Ooru Pérù"}, {"Philippines", "Àkókò Sọmà Filipininni"}, {"Pierre_Miquelon", "Àkókò Ojúmọmọ Pierre & Miquelon"}, {"Sakhalin", "Àkókò Sọmà Sakhalin"}, {"Samoa", "Àkókò Ojúmọmọ Samoa"}, {"Taipei", "Àkókò Ojúmọmọ Taipei"}, {"Tonga", "Àkókò Sọmà Tonga"}, {"Turkmenistan", "Àkókò Sọmà Turkmenistani"}, {"Uruguay", "Aago Soma Uruguay"}, {"Uzbekistan", "Àkókò Sọmà Usibekistani"}, {"Vanuatu", "Àkókò Sọmà Fanuatu"}, {"Vladivostok", "Àkókò Sọmà Filadifositoki"}, {"Volgograd", "Àkókò Sọmà Foligogiradi"}, {"Yakutsk", "Àkókò Sọmà Yatutsk"}, {"Yekaterinburg", "Àkókò Sọmà Yekaterinburg"}},
	},
	{
		{{"Afghanistan", "Àkókò Afghanistan"}, {"Africa_Central", "Àkókò Àárín Afírikà"}, {"Africa_Eastern", "Àkókò Ìlà-Oòrùn Afírikà"}, {"Africa_Southern", "South Africa Standard Time"}, {"Africa_Western", "Àkókò Ìwɔ̀-Oòrùn Afírikà"}, {"Alaska", "Àkókò Àfɛnukò Alásíkà"}, {"Amazon", "Àkókò Afɛnukò Amásɔ́nì"}, {"America_Central", "àkókò asiko àárín gbùngbùn"}, {"America_Eastern", "Akókò Àsikò Ìha Ìla Oòrùn"}, {"America_Mountain", "Àkókò asiko òkè"}, {"America_Pacific", "Àkókò àsikò Pàsífíìkì"}, {"Apia", "Àkókò Àfɛnukò Apia"}, {"Arabian", "Àkókò Àfɛnukò Arabia"}, {"Argentina", "Aago àsìkò Argentina"}, {"Argentina_Western", "Àkókò Iwɔ́-oòrùn Àfɛnukò Ajɛ́ntínà"}, {"Armenia", "Àkókò Àfɛnukò Armenia"}, {"Atlantic", "Àkókò àsikò Àtìláńtíìkì"}, {"Australia_Central", "Àkókò Àfɛnukò Ààrin Gùngùn Australia"}, {"Australia_CentralWestern", "Àkókò Àfɛnukò Ààrin Gùngùn Ìwɔ̀-Oòrùn Australia"}, {"Australia_Eastern", "Àkókò Àfɛnukò Ìlà-Oòrùn Australia"}, {"Australia_Western", "Àkókò Àfɛnukò Ìwɔ̀-Oòrùn Australia"}, {"Azerbaijan", "Àkókò Àfɛnukò Azerbaijan"}, {"Azores", "Àkókò Àfɛnukò Ásɔ́sì"}, {"Bangladesh", "Àkókò Àfɛnukò Bangladesh"}, {"Bhutan", "Àkókò Bhutan"}, {"Bolivia", "Aago Bolivia"}, {"Brasilia", "Aago àsìkò Bùràsílíà"}, {"Brunei", "Brunei Darussalam Time"}, {"Cape_Verde", "Àkókò Àfɛnukò Képú Fáàdì"}, {"Chamorro", "Àkókò Àfɛnukò Chamorro"}, {"Chatham", "Àkókò Àfɛnukò Chatam"}, {"Chile", "Àkókò Àfɛnukò Shílè"}, {"China", "Àkókò Ìfɛnukòsí Sháínà"}, {"Christmas", "Àkókò Erékùsù Christmas"}, {"Cocos", "Àkókò Àwɔn Erékùsù Cocos"}, {"Colombia", "Aago àsìkò Kolombia"}, {"Cook", "Àkókò Àfɛnukò Àwɔn Erekusu Kuuku"}, {"Cuba", "Àkókò Àfɛnukò Kúbà"}, {"Davis", "Àkókò Davis"}, {"DumontDUrville", "Àkókò Dumont-d’Urville"}, {"East_Timor", "Àkókò Ìlà oorùn Timor"}, {"Easter", "Aago àsìkò Easter Island"}, {"Ecuador", "Aago Ecuador"}, {"Europe_Central", "Àkókò Àárin àsikò Europe"}, {"Europe_Eastern", "Àkókò àsikò Ìhà Ìlà Oòrùn Europe"}, {"Europe_Further_Eastern", "Àkókò Iwájú Ìlà Oòrùn Yúróòpù"}, {"Europe_Western", "Àkókò àsikò Ìwɔ Oòrùn Europe"}, {"Falkland", "Àkókò Àfɛnukò Etíkun Fókílándì"}, {"Fiji", "Àkókò Àfɛnukò Fiji"}, {"French_Guiana", "Àkókò Gúyánà Fáránsè"}, {"French_Southern", "Àkókò Gúsù Fáransé àti Àntátíìkì"}, {"GMT", "Greenwich Mean Time"}, {"Galapagos", "Aago Galapago"}, {"Gambier", "Àkókò Gambia"}, {"Georgia", "Àkókò Àfɛnukò Georgia"}, {"Gilbert_Islands", "Àkókò Àwɔn Erekusu Gilibati"}, {"Greenland_Eastern", "Àkókò Ìwɔ̀ Ìfɛnukò oorùn Greenland"}, {"Greenland_Western", "Àkókò Àfɛnukò Ìwɔ̀ Oòrùn Greenland"}, {"Gulf", "Àkókò Àfɛnukò Gulf"}, {"Guyana", "Àkókò Gúyànà"}, {"Hawaii", "Àkókò Àfɛnukò Hawaii-Aleutian"}, {"Hawaii_Aleutian", "Àkókò Àfɛnukò Hawaii-Aleutian"}, {"Hong_Kong", "Àkókò Ìfɛnukòsí Hong Kong"}, {"Hovd", "Àkókò Ìfɛnukòsí Hofidi"}, {"India", "Àkókò Àfɛnukò India"}, {"Indian_Ocean", "Àkókò Etíkun Índíà"}, {"Indochina", "Àkókò Indochina"}, {"Indonesia_Central", "Àkókò Ààrin Gbùngbùn Indonesia"}, {"Indonesia_Eastern", "Àkókò Ìlà oorùn Indonesia"}, {"Indonesia_Western", "Àkókò Ìwɔ̀ oorùn Indonesia"}, {"Iran", "Àkókò Àfɛnukò Irani"}, {"Irkutsk", "Àkókò Àfɛnukò Íkósíkì"}, {"Israel", "Àkókò Àfɛnukò Israel"}, {"Japan", "Àkókò Ìfɛnukòsí Japan"}, {"Kazakhstan", "Aago Kasasitáànì"}, {"Kazakhstan_Eastern", "Àkókò Ìlà-Oòrùn Kasasitáànì"}, {"Kazakhstan_Western", "Àkókò Ìwɔ̀-Oòrùn Kasasitáànì"}, {"Korea", "Àkókò Ìfɛnukòsí Koria"}, {"Kosrae", "Àkókò Kosirai"}, {"Krasnoyarsk", "Àkókò Àfɛnukò Krasinoyasiki"}, {"Kyrgystan", "Àkókò Kirigisitaani"}, {"Line_Islands", "Àkókò Àwɔn Erekusu Laini"}, {"Lord_Howe", "Àkókò Àfɛnukò Lord Howe"}, {"Magadan", "Àkókò Àfɛnukò Magadani"}, {"Malaysia", "Àkókò Malaysia"}, {"Maldives", "Àkókò Maldives"}, {"Marquesas", "Àkókò Makuesasi"}, {"Marshall_Islands", "Àkókò Àwɔn Erekusu Masaali"}, {"Mauritius", "Àkókò Àfɛnukò Máríshúshì"}, {"Mawson", "Àkókò Mawson"}, {"Mexico_Pacific", "Àkókò Àfɛnukò Pásífíìkì Mɛ́síkò"}, {"Mongolia", "Àkókò Ìfɛnukòsí Ulaanbaatar"}, {"Moscow", "Àkókò Àfɛnukò Mosiko"}, {"Myanmar", "Àkókò Ìlà Myanmar"}, {"Nauru", "Àkókò Nauru"}, {"Nepal", "Àkókò Nepali"}, {"New_Caledonia", "Àkókò Àfɛnukò Kalidonia Tuntun"}, {"New_Zealand", "Àkókò Àfɛnukò New zealand"}, {"Newfoundland", "Àkókò Àfɛnukò Newfoundland"}, {"Niue", "Àkókò Niue"}, {"Norfolk", "Àkókò Àfɛnukò Erékùsù Norfolk"}, {"Noronha", "Aago àsìkò Fenando de Norona"}, {"Novosibirsk", "Àkókò Àfɛnukò Nofosibiriki"}, {"Omsk", "Àkókò Àfɛnukò Omisiki"}, {"Pakistan", "Àkókò Àfɛnukò Pakistani"}, {"Palau", "Àkókò Palau"}, {"Papua_New_Guinea", "Àkókò Papua New Guinea"}, {"Paraguay", "Àkókò Àfɛnukò Párágúwè"}, {"Peru", "Àkókò Àfɛnukò Pérù"}, {"Philippines", "Àkókò Àfɛnukò Filipininni"}, {"Phoenix_Islands", "Àkókò Àwɔn Erékùsù Phoenix"}, {"Pierre_Miquelon", "Àkókò Àfɛnukò Pierre & Miquelon"}, {"Pitcairn", "Àkókò Pitcairn"}, {"Ponape", "Àkókò Ponape"}, {"Pyongyang", "Àkókò Pyongyangi"}, {"Reunion", "Àkókò Rɛ́yúníɔ́nì"}, {"Rothera", "Àkókò Rothera"}, {"Sakhalin", "Àkókò Àfɛnukò Sakhalin"}, {"Samoa", "Àkókò Àfɛnukò Samoa"}, {"Seychelles", "Àkókò Sèshɛ́ɛ̀lì"}, {"Singapore", "Àkókò Àfɛnukò Singapore"}, {"Solomon", "Àkókò Àwɔn Erekusu Solomon"}, {"South_Georgia", "Àkókò Gúsù Jɔ́jíà"}, {"Suriname", "Àkókò Súrínámù"}, {"Syowa", "Àkókò Syowa"}, {"Tahiti", "Àkókò Tahiti"}, {"Taipei", "Àkókò Ìfɛnukòsí Taipei"}, {"Tajikistan", "Àkókò Tajikisitaani"}, {"Tokelau", "Àkókò Tokelau"}, {"Tonga", "Àkókò Àfɛnukò Tonga"}, {"Truk", "Àkókò Chuuk"}, {"Turkmenistan", "Àkókò Àfɛnukò Turkimenistani"}, {"Tuvalu", "Àkókò Tufalu"}, {"Uruguay", "Àkókò Àfɛnukò Úrúgúwè"}, {"Uzbekistan", "Àkókò Àfɛnukò Usibekistani"}, {"Vanuatu", "Àkókò Àfɛnukò Fanuatu"}, {"Venezuela", "Aago Venezuela"}, {"Vladivostok", "Àkókò Àfɛnukò Filadifositoki"}, {"Volgograd", "Àkókò Àfɛnukò Foligogiradi"}, {"Vostok", "Àkókò Vostok"}, {"Wake", "Àkókò Erékùsù Wake"}, {"Wallis", "Àkókò Wallis & Futuina"}, {"Yakutsk", "Àkókò Àfɛnukò Yatutsk"}, {"Yekaterinburg", "Àkókò Àfɛnukò Yekaterinburg"}, {"Yukon", "Àkókò Yúkɔ́nì"}},
		{{"Alaska", "Àkókò Ojúmɔ́ Alásíkà"}, {"Amazon", "Àkókò Oru Amásɔ́nì"}, {"America_Central", "Akókò àárín gbùngbùn ojúmɔmɔ"}, {"America_Eastern", "Àkókò ojúmɔmɔ Ìhà Ìlà Oòrun"}, {"America_Mountain", "Àkókò ojúmɔmɔ Ori-òkè"}, {"America_Pacific", "Àkókò Ìyálɛta Pàsífíìkì"}, {"Apia", "Àkókò Ojúmɔmɔ Apia"}, {"Arabian", "Àkókò Ojúmɔmɔ Arabia"}, {"Argentina", "Aago Soma Argentina"}, {"Argentina_Western", "Àkókò Oru Iwɔ́-oòrùn Ajɛ́ntínà"}, {"Armenia", "Àkókò Sɔmà Arabia"}, {"Atlantic", "Àkókò Ìyálɛta Àtìláńtíìkì"}, {"Australia_Central", "Àkókò Ojúmɔmɔ Ààrin Gùngùn Australia"}, {"Australia_CentralWestern", "Àkókò Ojúmɔmɔ Ààrin Gùngùn Ìwɔ̀-Oòrùn Australia"}, {"Australia_Eastern", "Àkókò Ojúmɔmɔ Ìlà-Oòrùn Australia"}, {"Australia_Western", "Àkókò Ojúmɔmɔ Ìwɔ̀-Oòrùn Australia"}, {"Azerbaijan", "Àkókò Sɔmà Azerbaijan"}, {"Azores", "Àkókò Ooru Ásɔ́sì"}, {"Bangladesh", "Àkókò Sɔmà Bangladesh"}, {"Brasilia", "Aago Soma Brasilia"}, {"Cape_Verde", "Àkókò Ɛ̀rún Képú Fáàdì"}, {"Chatham", "Àkókò Ojúmɔmɔ Chatam"}, {"Chile", "Àkókò Oru Shílè"}, {"China", "Àkókò Ojúmɔmɔ Sháínà"}, {"Colombia", "Aago Soma Colombia"}, {"Cook", "Àkókò Ilaji Sɔma Àwɔn Erekusu Kuuku"}, {"Cuba", "Àkókò Ojúmɔmɔ Kúbà"}, {"Easter", "Aago Soma Easter Island"}, {"Europe_Central", "Àkókò Àárin Sɔmà Europe"}, {"Europe_Eastern", "Àkókò Sɔmà Ìha Ìlà Oòrùn Europe"}, {"Europe_Western", "Àkókò Sɔmà Ìhà Ìwɔ Oòrùn Europe"}, {"Falkland", "Àkókò Ooru Etíkun Fókílándì"}, {"Fiji", "Àkókò Sɔma Fiji"}, {"Georgia", "Àkókò Sɔmà Georgia"}, {"Greenland_Eastern", "Àkókò ìgbà Ooru Greenland"}, {"Greenland_Western", "Àkókò Àfɛnukò Ìgba Oòru Greenland"}, {"Hawaii_Aleutian", "Àkókò Ojúmɔmɔ Hawaii-Aleutian"}, {"Hong_Kong", "Àkókò Sɔmà Hong Kong"}, {"Hovd", "Àkókò Sɔmà Hofidi"}, {"Iran", "Àkókò Ojúmɔmɔ Irani"}, {"Irkutsk", "Àkókò Sɔmà Íkúsíkì"}, {"Israel", "Àkókò Ojúmɔmɔ Israel"}, {"Japan", "Àkókò Sɔmà Japan"}, {"Korea", "Àkókò Ojúmɔmɔ Koria"}, {"Krasnoyarsk", "Àkókò Sɔmà Krasinoyasiki"}, {"Lord_Howe", "Àkókò Ojúmɔmɔ Lord Howe"}, {"Magadan", "Àkókò Sɔmà Magadani"}, {"Mauritius", "Àkókò Ooru Máríshúshì"}, {"Mexico_Pacific", "Àkókò Ojúmɔmɔ Pásífíìkì Mɛ́síkò"}, {"Mongolia", "Àkókò Sɔmà Ulaanbaatar"}, {"Moscow", "Àkókò Sɔmà Mosiko"}, {"New_Caledonia", "Àkókò Sɔma Kalidonia Tuntun"}, {"New_Zealand", "Àkókò Ojúmɔmɔ New Zealand"}, {"Newfoundland", "Àkókò Ojúmɔmɔ Newfoundland"}, {"Norfolk", "Àkókò Ojúmɔmɔ Erékùsù Norfolk"}, {"Noronha", "Aago Soma Fernando de Noronha"}, {"Novosibirsk", "Àkókò Sɔmà Noforibisiki"}, {"Omsk", "Àkókò Sɔmà Omisiki"}, {"Pakistan", "Àkókò Sɔmà Pakistani"}, {"Paraguay", "Àkókò Ooru Párágúwè"}, {"Peru", "Àkókò Ooru Pérù"}, {"Philippines", "Àkókò Sɔmà Filipininni"}, {"Pierre_Miquelon", "Àkókò Ojúmɔmɔ Pierre & Miquelon"}, {"Sakhalin", "Àkókò Sɔmà Sakhalin"}, {"Samoa", "Àkókò Ojúmɔmɔ Samoa"}, {"Taipei", "Àkókò Ojúmɔmɔ Taipei"}, {"Tonga", "Àkókò Sɔmà Tonga"}, {"Turkmenistan", "Àkókò Sɔmà Turkmenistani"}, {"Uruguay", "Aago Soma Uruguay"}, {"Uzbekistan", "Àkókò Sɔmà Usibekistani"}, {"Vanuatu", "Àkókò Sɔmà Fanuatu"}, {"Vladivostok", "Àkókò Sɔmà Filadifositoki"}, {"Volgograd", "Àkókò Sɔmà Foligogiradi"}, {"Yakutsk", "Àkókò Sɔmà Yatutsk"}, {"Yekaterinburg", "Àkókò Sɔmà Yekaterinburg"}},
	},
	{
		{{"Acre", "Hurariyu Retewa Acre yara"}, {"Afghanistan", "Afegãniretãma Hurariyu"}, {"Africa_Central", "Afirika Piterawara Hurariyu"}, {"Africa_Eastern", "Afirika Uriẽtawara Hurariyu"}, {"Africa_Southern", "Afirika Su suí Hurariyu"}, {"Africa_Western", "Afirikia Usidẽtawara Hurariyu"}, {"Alaska", "Alasika Hurariyu Eté"}, {"Almaty", "Aumati Hurariyu Eté"}, {"Amazon", "Amasuna Hurariyu Eté"}, {"America_Central", "Hurariyu Retewa Piterawara"}, {"America_Eastern", "Hurariyu Retewa Lesiti yara"}, {"America_Mountain", "Óra Pã Krĩ-ag tá"}, {"America_Pacific", "Óra Pã Pasifiku tá"}, {"Anadyr", "Anadí Hurariyu Retewa"}, {"Apia", "Apiya Uraruiyu Retewa"}, {"Aqtau", "Akitau Hurariyu Retewa"}, {"Aqtobe", "Akitubi Hurariyu Retewa"}, {"Arabian", "Arábiya Hurariyu Retewa"}, {"Argentina", "Argẽtina Hurariyu Retewa"}, {"Argentina_Western", "Argẽtina Usidẽtawara Hurariyu Retewa"}, {"Armenia", "Arimẽniya Hurariyu Retewa"}, {"Atlantic", "Atalãtiku Hurariyu Retewa"}, {"Australia_Central", "Ausitaraliya Piterawara Hurariyu Retewa"}, {"Australia_CentralWestern", "Ausitaraliya Piterawara-Usidẽtawara Hurariyu Retewa"}, {"Australia_Eastern", "Ausitaraliya Uriẽtawara Hurariyu Retewa"}, {"Australia_Western", "Ausitaraliya Usidẽtawara Hurariyu Retewa"}, {"Azerbaijan", "Aseriretãma Hurariyu Retewa"}, {"Azores", "Asori-ita Hurariyu Retewa"}, {"Bangladesh", "Bãkaradexi Horariyu Retewa"}, {"Bhutan", "Butãu Hurariyu"}, {"Bolivia", "Buríwia Hurariyu"}, {"Brasilia", "Óra Pã Brasília tá"}, {"Brunei", "Burunei Darusaram Hurariyu"}, {"Cape_Verde", "Kabu Suikiri Hurariyu Retewa"}, {"Chamorro", "Xamoro Hurariyu"}, {"Chatham", "Xatham Hurariyu Retewa"}, {"Chile", "Xiri Hurariyu Retewa"}, {"China", "Xina Hurariyu Retewa"}, {"Christmas", "KapuãmaKiritima Hurariyu"}, {"Cocos", "Kapuã-ita Kuku-ita Hurariyu"}, {"Colombia", "Kurũbia Hurariyu Retewa"}, {"Cook", "Kapuã-ita Kooki Hurariyu Retewa"}, {"Cuba", "Kuba Hurariyu Retewa"}, {"Davis", "Dawi Hurariyu"}, {"DumontDUrville", "Dumont-d’Urville Hurariyu"}, {"East_Timor", "Timu-Semusawa Hurariyu"}, {"Easter", "Pasikuwa Kapuãma Hurariyu Retewa"}, {"Ecuador", "Ekuadú Hurariyu"}, {"Europe_Central", "Eurupa Piterawara Hurariyu Retewa"}, {"Europe_Eastern", "Eurupa Uriẽtawara Hurariyu Retewa"}, {"Europe_Further_Eastern", "Eurupa Lesiti-eté Hurariyu"}, {"Europe_Western", "Eurupa Usidẽtawara Hurariyu Retewa"}, {"Falkland", "Kapuã-ita Mawina Hurariyu Retewa"}, {"Fiji", "Fiyi Hurariyu Retewa"}, {"French_Guiana", "Giyana Frãsa yara Hurariyu"}, {"French_Southern", "Tetãma Su suí Frãsa yara asuí Ãtartida Hurariyu"}, {"GMT", "Greenwich Miridiyanu yara Hurariyu"}, {"Galapagos", "Garapagu-ita Hurariyu"}, {"Gambier", "Gãbiere Hurariyu"}, {"Georgia", "Geugiya Hurariyu Retewa"}, {"Gilbert_Islands", "Kapuã-ita Yubetu Hurariyu"}, {"Greenland_Eastern", "Guruẽrãdiya Uriẽtawara Hurariyu Retewa"}, {"Greenland_Western", "Guruẽrãdiya Usidẽtawara Hurariyu Retewa"}, {"Guam", "Guwã Hurariyu Retewa"}, {"Gulf", "Golfo Hurariyu"}, {"Guyana", "Giyana Hurariyu"}, {"Hawaii", "Hawaí asuí Kapuã-ita Areuta-ita Hurariyu Retewa"}, {"Hawaii_Aleutian", "Hawaí asuí Kapuã-ita Areuta-ita Hurariyu Retewa"}, {"Hong_Kong", "Hũg Kũg Hurariyu Retewa"}, {"Hovd", "Howidi Hurariyu Retewa"}, {"India", "Ĩdia Hurariyu Retewa"}, {"Indian_Ocean", "Useyanu Ĩdiku Hurariyu"}, {"Indochina", "Ĩdoxina Hurariyu"}, {"Indonesia_Central", "Ĩdonesiya Piterawara Hurariyu"}, {"Indonesia_Eastern", "Ĩdonesiya Uriẽtawara Hurariyu"}, {"Indonesia_Western", "Ĩdonesiya Usidẽtawara Hurariyu"}, {"Iran", "Irã Hurariyu Retewa"}, {"Irkutsk", "Irkutisiki Hurariyu Retewa"}, {"Israel", "Isirayeu Hurariyu Retewa"}, {"Japan", "Nipõ Hurariyu Retewa"}, {"Kamchatka", "Petropavlovsk-Kamchatski Hurariyu Retewa"}, {"Kazakhstan_Eastern", "Kasakiretãma Uriẽtawara Hurariyu"}, {"Kazakhstan_Western", "Kasakiretãma Usidẽtawara Hurariyu"}, {"Korea", "Kureya Hurariyu Retewa"}, {"Kosrae", "Kusirai Hurariyu"}, {"Krasnoyarsk", "Karasinoyarisiki Hurariyu Retewa"}, {"Kyrgystan", "Kirigiretãma Hurariyu"}, {"Lanka", "Rãka Hurariyu"}, {"Line_Islands", "Kapuã-ita Inĩbu Hurariyu"}, {"Lord_Howe", "Lurudi Howe Hurariyu Retewa"}, {"Macau", "Makau Hurariyu Retewa"}, {"Magadan", "Magadã Hurariyu Retewa"}, {"Malaysia", "Malasiya Hurariyu"}, {"Maldives", "Kapuã-ita Maudiwa-ita Hurariyu"}, {"Marquesas", "Makesa-ita Hurariyu"}, {"Marshall_Islands", "Kapuã-ita Marshall Hurariyu"}, {"Mauritius", "Maurisiyu Hurariyu Retewa"}, {"Mawson", "Mausũ Hurariyu"}, {"Mexico_Pacific", "Pasifiku Mexikanu Hurariyu Retewa"}, {"Mongolia", "Urã Baturu Hurariyu Retewa"}, {"Moscow", "Moskou Hurariyu Retewa"}, {"Myanmar", "Miyamá Hurariyu"}, {"Nauru", "Nauru Hurariyu"}, {"Nepal", "Nepau Hurariyu"}, {"New_Caledonia", "Karedũniya Pisasú Hurariyu Retewa"}, {"New_Zealand", "Serãdiya Pisasú Hurariyu Retewa"}, {"Newfoundland", "Iwí Pisasú Hurariyu Retewa"}, {"Niue", "Niwe Hurariyu"}, {"Norfolk", "Kapuãma Norfolk Hurariyu Retewa"}, {"Noronha", "Fenãdu Nuruyã Hurariyu Retewa"}, {"North_Mariana", "Kapuã-ita Mariyãna Nuti suí Hurariyu"}, {"Novosibirsk", "Sibirisiki Pisasú Hurariyu Retewa"}, {"Omsk", "Omisiki Hurariyu Retewa"}, {"Pakistan", "Pakiretãma Hurariyu Retewa"}, {"Palau", "Parau Hurariyu"}, {"Papua_New_Guinea", "Papuwa-Giné Pisasú Hurariyu"}, {"Paraguay", "Paraguwai Hurariyu Retewa"}, {"Peru", "Peru Hurariyu Retewa"}, {"Philippines", "Firipina Hurariyu Retewa"}, {"Phoenix_Islands", "Kapuã-ita Fẽnix"}, {"Pierre_Miquelon", "Sã Peduru asuí Mikirãu Hurariyu Retewa"}, {"Pitcairn", "Pitcairn Hurariyu"}, {"Ponape", "Ponape Hurariyu"}, {"Pyongyang", "Piúgiãgi Hurariyu"}, {"Qyzylorda", "Kisiroda Hurariyu Retewa"}, {"Reunion", "Yumuatirisawa Hurariyu"}, {"Rothera", "Rotera Hurariyu"}, {"Sakhalin", "Sakarina Hurariyu Retewa"}, {"Samara", "Samara Hurariyu Retewa"}, {"Samoa", "Samowa Hurariyu Retewa"}, {"Seychelles", "Seixeri Hurariyu"}, {"Singapore", "Sĩgapura Hurariyu Retewa"}, {"Solomon", "Kapuãma-ita Sarumũ Hurariyu"}, {"South_Georgia", "Geugiya Su suí Hurariyu"}, {"Suriname", "Suriname Hurariyu"}, {"Syowa", "Siyowa Hurariyu"}, {"Tahiti", "Taiti Hurariyu"}, {"Taipei", "Taipei Hurariyu Retewa"}, {"Tajikistan", "Tayikiretãma Hurariyu"}, {"Tokelau", "Tokerau Hurariyu"}, {"Tonga", "Tõga Hurariyu Retewa"}, {"Truk", "Chuuk Hurariyu"}, {"Turkmenistan", "Turkuranaretãma Hurariyu Retewa"}, {"Tuvalu", "Tuvaru Hurariyu"}, {"Uruguay", "Uruguwai Hurariyu Retewa"}, {"Uzbekistan", "Yũbuesara-retãma Hurariyu Retewa"}, {"Vanuatu", "Wanuatu Hurariyu Retewa"}, {"Venezuela", "Wenẽsuera Hurariyu"}, {"Vladivostok", "Waradiwosituki Hurariyu Retewa"}, {"Volgograd", "Worwogaradu Hurariyu Retewa"}, {"Vostok", "Wosituki Hurariyu"}, {"Wake", "Kapuã-ita Wake"}, {"Wallis", "Wari asuí Futuna Hurariyu"}, {"Yakutsk", "Yakutisiki Hurariyu Retewa"}, {"Yekaterinburg", "Ekaterĩbugu Hurariyu Retewa"}},
		{{"Acre", "Hurariyu Kurasí Ara Acre yara"}, {"Alaska", "Alasika Kurasí Ara Hurariyu"}, {"Almaty", "Aumati Kurasí Ara Hurariyu"}, {"Amazon", "Amasuna Kurasí Ara Hurariyu"}, {"America_Central", "Hurariyu Kurasí Ara Piterawara"}, {"America_Eastern", "Hurariyu Kurasí Ara Lesiti"}, {"America_Mountain", "Rỹ kã Óra Krĩ-ag tá"}, {"America_Pacific", "Rỹ Kã Óra Pasifiku tá"}, {"Anadyr", "Anadí Kurasí Ara Hurariyu"}, {"Apia", "Apiya Kurasí Ara Hurariyu"}, {"Aqtau", "Akitau Kurasí Ara Hurariyu"}, {"Aqtobe", "Akitubi Kurasí Ara Hurariyu"}, {"Arabian", "Arabiya Kurasí Ara Hurariyu"}, {"Argentina", "Argẽtina Kurasí Ara Hurariyu"}, {"Argentina_Western", "Argẽtina Usidẽtawara Kurasí Ara Hurariyu"}, {"Armenia", "Arimẽniya Kurasí Ara Hurariyu"}, {"Atlantic", "Atalãtku Kurasí Ara Hurariyu"}, {"Australia_Central", "Ausitaraliya Piterawara Kurasí Ara Hurariyu"}, {"Australia_CentralWestern", "Ausitaraliya Piterawara-Usidẽtawara Kurasí Ara Hurariyu"}, {"Australia_Eastern", "Ausitaraliya Uriẽtawara Kurasí Ara Hurariyu"}, {"Australia_Western", "Ausitaraliya Usidẽtawara Kurasí Ara Hurariyu"}, {"Azerbaijan", "Aseriretãma Kurasí Ara Hurariyu"}, {"Azores", "Asori-ita Kurasí Ara Hurariyu"}, {"Bangladesh", "Bãkaradexi Kurasí Ara Horariyu"}, {"Brasilia", "Rỹ Kã Óra Brasília tá"}, {"Cape_Verde", "Kabu Suikiri Kurasí Ara Hurariyu"}, {"Chatham", "Xatham Kurasí Ara Hurariyu"}, {"Chile", "Xiri Kurasí Ara Hurariyu"}, {"China", "Xina Kurasí Ara Hurariyu"}, {"Colombia", "Kurũbia Kurasí Ara Hurariyu"}, {"Cook", "Kapuã-ita Kooki Kurasí Ara Pitera Hurariyu"}, {"Cuba", "Kuba Kurasí Ara Hurariyu"}, {"Easter", "Pasikuwa Kapuãma Kurasí Ara Hurariyu"}, {"Europe_Central", "Eurupa Piterawara Kurasí Ara Hurariyu"}, {"Europe_Eastern", "Eurupa Uriẽtawara Kurasí Ara Hurariyu"}, {"Europe_Western", "Eurupa Usidẽtawara Kurasí Ara Hurariyu"}, {"Falkland", "Kapuã-ita Mawina Kurasí Ara Hurariyu"}, {"Fiji", "Fiyi Kurasí Ara Hurariyu"}, {"Georgia", "Geugiya Kurasí Ara Hurariyu"}, {"Greenland_Eastern", "Guruẽrãdiya Uriẽtawara Kurasí Ara Hurariyu"}, {"Greenland_Western", "Guruẽrãdiya Usidẽtawara Kurasí Ara Hurariyu"}, {"Hawaii_Aleutian", "Hawaí asuí Kapuã-ita Areuta-ita Kurasí Ara Hurariyu"}, {"Hong_Kong", "Hũg Kũg Kurasí Ara Hurariyu"}, {"Hovd", "Howidi Kurasí Ara Hurariyu"}, {"Iran", "Irã Kurasí Ara Hurariyu"}, {"Irkutsk", "Irkutisiki Kurasí Ara Hurariyu"}, {"Israel", "Isirayeu Kurasí Ara Hurariyu"}, {"Japan", "Nipõ Kurasí Ara Hurariyu"}, {"Kamchatka", "Petropavlovsk-Kamchatski Kurasí Ara Hurariyu"}, {"Korea", "Kureya Kurasí Ara Hurariyu"}, {"Krasnoyarsk", "Karasinoyarisiki Kurasí Ara Hurariyu"}, {"Lord_Howe", "Lurudi Howe Kurasí Ara Hurariyu"}, {"Macau", "Makau Kurasí Ara Hurariyu"}, {"Magadan", "Magadã Kurasí Ara Hurariyu"}, {"Mauritius", "Maurisiyu Kurasí Ara Hurariyu"}, {"Mexico_Pacific", "Pasifiku Mexikanu Kurasí Ara Hurariyu"}, {"Mongolia", "Urã Baturu Kurasí Ara Hurariyu"}, {"Moscow", "Moskou Kurasí Ara Hurariyu"}, {"New_Caledonia", "Karedũniya Pisasú Kurasí Ara Hurariyu"}, {"New_Zealand", "Serãdiya Pisasú Kurasí Ara Hurariyu"}, {"Newfoundland", "Iwí Pisasú Kurasí Ara Hurariyu"}, {"Norfolk", "Kapuãma Norfolk Kurasí Ara Hurariyu"}, {"Noronha", "Fenãdu Nuruyã Kurasí Ara Hurariyu"}, {"Novosibirsk", "Sibirisiki Pisasú Kurasí Ara Hurariyu"}, {"Omsk", "Omisiki Kurasí Ara Hurariyu"}, {"Pakistan", "Pakiretãma Kurasí Ara Hurariyu"}, {"Paraguay", "ParaguwaiKurasí Ara Hurariyu"}, {"Peru", "Peru Kurasí Ara Hurariyu"}, {"Philippines", "Firipina Kurasí Ara Hurariyu"}, {"Pierre_Miquelon", "Sã Peduru asuí Mikirãu Kurasí Ara Hurariyu"}, {"Qyzylorda", "Kisiroda Kurasí Ara Hurariyu"}, {"Sakhalin", "Sakarina Kurasí Ara Hurariyu"}, {"Samara", "Samara Kurasí Ara Hurariyu"}, {"Samoa", "Samowa Kurasí Ara Hurariyu"}, {"Taipei", "Taipei Kurasí Ara Hurariyu"}, {"Tonga", "Tõga Kurasí Ara Hurariyu"}, {"Turkmenistan", "Turkuranaretãma Kurasí Ara Hurariyu"}, {"Uruguay", "Uruguwai Kurasí Ara Hurariyu"}, {"Uzbekistan", "Yũbuesara-retãma Kurasí Ara Hurariyu"}, {"Vanuatu", "Wanuatu Kurasí Ara Hurariyu"}, {"Vladivostok", "Waradiwosituki Kurasí Ara Hurariyu"}, {"Volgograd", "Worwogaradu Kurasí Ara Hurariyu"}, {"Yakutsk", "Yakutisiki Kurasí Ara Hurariyu"}, {"Yekaterinburg", "Ekaterĩbugu Kurasí Ara Hurariyu"}},
	},
	{
		{{"Acre", "Hurariyu Retewa Acre yara"}, {"Afghanistan", "Afegãniretãma Hurariyu"}, {"Africa_Central", "Afirika Piterawara Hurariyu"}, {"Africa_Eastern", "Afirika Uriẽtawara Hurariyu"}, {"Africa_Southern", "Afirika Su suí Hurariyu"}, {"Africa_Western", "Afirikia Usidẽtawara Hurariyu"}, {"Alaska", "Alasika Hurariyu Eté"}, {"Almaty", "Aumati Hurariyu Eté"}, {"Amazon", "Amasuna Hurariyu Eté"}, {"America_Central", "Hurariyu Retewa Piterawara"}, {"America_Eastern", "Hurariyu Retewa Lesiti yara"}, {"America_Mountain", "Óra Pã Krĩ-ag tá"}, {"America_Pacific", "Óra Pã Pasifiku tá"}, {"Anadyr", "Anadí Hurariyu Retewa"}, {"Apia", "Apiya Uraruiyu Retewa"}, {"Aqtau", "Akitau Hurariyu Retewa"}, {"Aqtobe", "Akitubi Hurariyu Retewa"}, {"Arabian", "Arábiya Hurariyu Retewa"}, {"Argentina", "Argẽtina Hurariyu Retewa"}, {"Argentina_Western", "Argẽtina Usidẽtawara Hurariyu Retewa"}, {"Armenia", "Arimẽniya Hurariyu Retewa"}, {"Atlantic", "Atalãtiku Hurariyu Retewa"}, {"Australia_Central", "Ausitaraliya Piterawara Hurariyu Retewa"}, {"Australia_CentralWestern", "Ausitaraliya Piterawara-Usidẽtawara Hurariyu Retewa"}, {"Australia_Eastern", "Ausitaraliya Uriẽtawara Hurariyu Retewa"}, {"Australia_Western", "Ausitaraliya Usidẽtawara Hurariyu Retewa"}, {"Azerbaijan", "Aseriretãma Hurariyu Retewa"}, {"Azores", "Asori-ita Hurariyu Retewa"}, {"Bangladesh", "Bãkaradexi Horariyu Retewa"}, {"Bhutan", "Butan Hurariyu"}, {"Bolivia", "Buríwia Hurariyu"}, {"Brasilia", "Óra Pã Brasília tá"}, {"Brunei", "Burunei Darusaram Hurariyu"}, {"Cape_Verde", "Kabu Suikiri Hurariyu Retewa"}, {"Chamorro", "Xamoro Hurariyu"}, {"Chatham", "Xatham Hurariyu Retewa"}, {"Chile", "Xiri Hurariyu Retewa"}, {"China", "Xina Hurariyu Retewa"}, {"Christmas", "KapuãmaKiritima Hurariyu"}, {"Cocos", "Kapuã-ita Kuku-ita Hurariyu"}, {"Colombia", "Kurũbia Hurariyu Retewa"}, {"Cook", "Kapuã-ita Kooki Hurariyu Retewa"}, {"Cuba", "Kuba Hurariyu Retewa"}, {"Davis", "Dawi Hurariyu"}, {"DumontDUrville", "Dumont-d’Urville Hurariyu"}, {"East_Timor", "Timu-Semusawa Hurariyu"}, {"Easter", "Pasikuwa Kapuãma Hurariyu Retewa"}, {"Ecuador", "Ekuadú Hurariyu"}, {"Europe_Central", "Eurupa Piterawara Hurariyu Retewa"}, {"Europe_Eastern", "Eurupa Uriẽtawara Hurariyu Retewa"}, {"Europe_Further_Eastern", "Eurupa Lesiti-eté Hurariyu"}, {"Europe_Western", "Eurupa Usidẽtawara Hurariyu Retewa"}, {"Falkland", "Kapuã-ita Mawina Hurariyu Retewa"}, {"Fiji", "Fiyi Hurariyu Retewa"}, {"French_Guiana", "Giyana Frãsa yara Hurariyu"}, {"French_Southern", "Tetãma Su suí Frãsa yara asuí Ãtartida Hurariyu"}, {"GMT", "Greenwich Miridiyanu yara Hurariyu"}, {"Galapagos", "Garapagu-ita Hurariyu"}, {"Gambier", "Gãbiere Hurariyu"}, {"Georgia", "Geugiya Hurariyu Retewa"}, {"Gilbert_Islands", "Kapuã-ita Yubetu Hurariyu"}, {"Greenland_Eastern", "Guruẽrãdiya Uriẽtawara Hurariyu Retewa"}, {"Greenland_Western", "Guruẽrãdiya Usidẽtawara Hurariyu Retewa"}, {"Guam", "Guwã Hurariyu Retewa"}, {"Gulf", "Golfo Hurariyu"}, {"Guyana", "Giyana Hurariyu"}, {"Hawaii", "Hawaí asuí Kapuã-ita Areuta-ita Hurariyu Retewa"}, {"Hawaii_Aleutian", "Hawaí asuí Kapuã-ita Areuta-ita Hurariyu Retewa"}, {"Hong_Kong", "Hũg Kũg Hurariyu Retewa"}, {"Hovd", "Howidi Hurariyu Retewa"}, {"India", "Ĩdia Hurariyu Retewa"}, {"Indian_Ocean", "Useyanu Ĩdiku Hurariyu"}, {"Indochina", "Ĩdoxina Hurariyu"}, {"Indonesia_Central", "Ĩdonesiya Piterawara Hurariyu"}, {"Indonesia_Eastern", "Ĩdonesiya Uriẽtawara Hurariyu"}, {"Indonesia_Western", "Ĩdonesiya Usidẽtawara Hurariyu"}, {"Iran", "Irã Hurariyu Retewa"}, {"Irkutsk", "Irkutisiki Hurariyu Retewa"}, {"Israel", "Isirayeu Hurariyu Retewa"}, {"Japan", "Nipõ Hurariyu Retewa"}, {"Kamchatka", "Petropavlovsk-Kamchatski Hurariyu Retewa"}, {"Kazakhstan_Eastern", "Kasakiretãma Uriẽtawara Hurariyu"}, {"Kazakhstan_Western", "Kasakiretãma Usidẽtawara Hurariyu"}, {"Korea", "Kureya Hurariyu Retewa"}, {"Kosrae", "Kusirai Hurariyu"}, {"Krasnoyarsk", "Karasinoyarisiki Hurariyu Retewa"}, {"Kyrgystan", "Kirigiretãma Hurariyu"}, {"Lanka", "Rãka Hurariyu"}, {"Line_Islands", "Kapuã-ita Inĩbu Hurariyu"}, {"Lord_Howe", "Lurudi Howe Hurariyu Retewa"}, {"Macau", "Makau Hurariyu Retewa"}, {"Magadan", "Magadã Hurariyu Retewa"}, {"Malaysia", "Malasiya Hurariyu"}, {"Maldives", "Kapuã-ita Maudiwa-ita Hurariyu"}, {"Marquesas", "Makesa-ita Hurariyu"}, {"Marshall_Islands", "Kapuã-ita Marshall Hurariyu"}, {"Mauritius", "Maurisiyu Hurariyu Retewa"}, {"Mawson", "Mausũ Hurariyu"}, {"Mexico_Pacific", "Pasifiku Mexikanu Hurariyu Retewa"}, {"Mongolia", "Urã Baturu Hurariyu Retewa"}, {"Moscow", "Moskou Hurariyu Retewa"}, {"Myanmar", "Miyamá Hurariyu"}, {"Nauru", "Nauru Hurariyu"}, {"Nepal", "Nepau Hurariyu"}, {"New_Caledonia", "Karedũniya Pisasú Hurariyu Retewa"}, {"New_Zealand", "Serãdiya Pisasú Hurariyu Retewa"}, {"Newfoundland", "Iwí Pisasú Hurariyu Retewa"}, {"Niue", "Niwe Hurariyu"}, {"Norfolk", "Kapuãma Norfolk Hurariyu Retewa"}, {"Noronha", "Fenãdu Nuruyã Hurariyu Retewa"}, {"North_Mariana", "Kapuã-ita Mariyãna Nuti suí Hurariyu"}, {"Novosibirsk", "Sibirisiki Pisasú Hurariyu Retewa"}, {"Omsk", "Omisiki Hurariyu Retewa"}, {"Pakistan", "Pakiretãma Hurariyu Retewa"}, {"Palau", "Parau Hurariyu"}, {"Papua_New_Guinea", "Papuwa-Giné Pisasú Hurariyu"}, {"Paraguay", "Paraguwai Hurariyu Retewa"}, {"Peru", "Peru Hurariyu Retewa"}, {"Philippines", "Firipina Hurariyu Retewa"}, {"Phoenix_Islands", "Kapuã-ita Fẽnix"}, {"Pierre_Miquelon", "Sã Peduru asuí Mikiran Hurariyu Retewa"}, {"Pitcairn", "Pitcairn Hurariyu"}, {"Ponape", "Ponape Hurariyu"}, {"Pyongyang", "Piúgiãgi Hurariyu"}, {"Qyzylorda", "Kisiroda Hurariyu Retewa"}, {"Reunion", "Yumuatirisawa Hurariyu"}, {"Rothera", "Rotera Hurariyu"}, {"Sakhalin", "Sakarina Hurariyu Retewa"}, {"Samara", "Samara Hurariyu Retewa"}, {"Samoa", "Samowa Hurariyu Retewa"}, {"Seychelles", "Seixeri Hurariyu"}, {"Singapore", "Sĩgapura Hurariyu Retewa"}, {"Solomon", "Kapuãma-ita Sarumũ Hurariyu"}, {"South_Georgia", "Geugiya Su suí Hurariyu"}, {"Suriname", "Suriname Hurariyu"}, {"Syowa", "Siyowa Hurariyu"}, {"Tahiti", "Taiti Hurariyu"}, {"Taipei", "Taipei Hurariyu Retewa"}, {"Tajikistan", "Tayikiretãma Hurariyu"}, {"Tokelau", "Tokerau Hurariyu"}, {"Tonga", "Tõga Hurariyu Retewa"}, {"Truk", "Chuuk Hurariyu"}, {"Turkmenistan", "Turkuranaretãma Hurariyu Retewa"}, {"Tuvalu", "Tuvaru Hurariyu"}, {"Uruguay", "Uruguwai Hurariyu Retewa"}, {"Uzbekistan", "Yũbuesara-retãma Hurariyu Retewa"}, {"Vanuatu", "Wanuatu Hurariyu Retewa"}, {"Venezuela", "Wenẽsuera Hurariyu"}, {"Vladivostok", "Waradiwosituki Hurariyu Retewa"}, {"Volgograd", "Worwogaradu Hurariyu Retewa"}, {"Vostok", "Wosituki Hurariyu"}, {"Wake", "Kapuã-ita Wake"}, {"Wallis", "Wari asuí Futuna Hurariyu"}, {"Yakutsk", "Yakutisiki Hurariyu Retewa"}, {"Yekaterinburg", "Ekaterĩbugu Hurariyu Retewa"}},
		{{"Acre", "Hurariyu Kurasí Ara Acre yara"}, {"Alaska", "Alasika Kurasí Ara Hurariyu"}, {"Almaty", "Aumati Kurasí Ara Hurariyu"}, {"Amazon", "Amasuna Kurasí Ara Hurariyu"}, {"America_Central", "Hurariyu Kurasí Ara Piterawara"}, {"America_Eastern", "Hurariyu Kurasí Ara Lesiti"}, {"America_Mountain", "Rỹ kã Óra Krĩ-ag tá"}, {"America_Pacific", "Rỹ Kã Óra Pasifiku tá"}, {"Anadyr", "Anadí Kurasí Ara Hurariyu"}, {"Apia", "Apiya Kurasí Ara Hurariyu"}, {"Aqtau", "Akitau Kurasí Ara Hurariyu"}, {"Aqtobe", "Akitubi Kurasí Ara Hurariyu"}, {"Arabian", "Arabiya Kurasí Ara Hurariyu"}, {"Argentina", "Argẽtina Kurasí Ara Hurariyu"}, {"Argentina_Western", "Argẽtina Usidẽtawara Kurasí Ara Hurariyu"}, {"Armenia", "Arimẽniya Kurasí Ara Hurariyu"}, {"Atlantic", "Atalãtku Kurasí Ara Hurariyu"}, {"Australia_Central", "Ausitaraliya Piterawara Kurasí Ara Hurariyu"}, {"Australia_CentralWestern", "Ausitaraliya Piterawara-Usidẽtawara Kurasí Ara Hurariyu"}, {"Australia_Eastern", "Ausitaraliya Uriẽtawara Kurasí Ara Hurariyu"}, {"Australia_Western", "Ausitaraliya Usidẽtawara Kurasí Ara Hurariyu"}, {"Azerbaijan", "Aseriretãma Kurasí Ara Hurariyu"}, {"Azores", "Asori-ita Kurasí Ara Hurariyu"}, {"Bangladesh", "Bãkaradexi Kurasí Ara Horariyu"}, {"Brasilia", "Rỹ Kã Óra Brasília tá"}, {"Cape_Verde", "Kabu Suikiri Kurasí Ara Hurariyu"}, {"Chatham", "Xatham Kurasí Ara Hurariyu"}, {"Chile", "Xiri Kurasí Ara Hurariyu"}, {"China", "Xina Kurasí Ara Hurariyu"}, {"Colombia", "Kurũbia Kurasí Ara Hurariyu"}, {"Cook", "Kapuã-ita Kooki Kurasí Ara Pitera Hurariyu"}, {"Cuba", "Kuba Kurasí Ara Hurariyu"}, {"Easter", "Pasikuwa Kapuãma Kurasí Ara Hurariyu"}, {"Europe_Central", "Eurupa Piterawara Kurasí Ara Hurariyu"}, {"Europe_Eastern", "Eurupa Uriẽtawara Kurasí Ara Hurariyu"}, {"Europe_Western", "Eurupa Usidẽtawara Kurasí Ara Hurariyu"}, {"Falkland", "Kapuã-ita Mawina Kurasí Ara Hurariyu"}, {"Fiji", "Fiyi Kurasí Ara Hurariyu"}, {"Georgia", "Geugiya Kurasí Ara Hurariyu"}, {"Greenland_Eastern", "Guruẽrãdiya Uriẽtawara Kurasí Ara Hurariyu"}, {"Greenland_Western", "Guruẽrãdiya Usidẽtawara Kurasí Ara Hurariyu"}, {"Hawaii_Aleutian", "Hawaí asuí Kapuã-ita Areuta-ita Kurasí Ara Hurariyu"}, {"Hong_Kong", "Hũg Kũg Kurasí Ara Hurariyu"}, {"Hovd", "Howidi Kurasí Ara Hurariyu"}, {"Iran", "Irã Kurasí Ara Hurariyu"}, {"Irkutsk", "Irkutisiki Kurasí Ara Hurariyu"}, {"Israel", "Isirayeu Kurasí Ara Hurariyu"}, {"Japan", "Nipõ Kurasí Ara Hurariyu"}, {"Kamchatka", "Petropavlovsk-Kamchatski Kurasí Ara Hurariyu"}, {"Korea", "Kureya Kurasí Ara Hurariyu"}, {"Krasnoyarsk", "Karasinoyarisiki Kurasí Ara Hurariyu"}, {"Lord_Howe", "Lurudi Howe Kurasí Ara Hurariyu"}, {"Macau", "Makau Kurasí Ara Hurariyu"}, {"Magadan", "Magadã Kurasí Ara Hurariyu"}, {"Mauritius", "Maurisiyu Kurasí Ara Hurariyu"}, {"Mexico_Pacific", "Pasifiku Mexikanu Kurasí Ara Hurariyu"}, {"Mongolia", "Urã Baturu Kurasí Ara Hurariyu"}, {"Moscow", "Moskou Kurasí Ara Hurariyu"}, {"New_Caledonia", "Karedũniya Pisasú Kurasí Ara Hurariyu"}, {"New_Zealand", "Serãdiya Pisasú Kurasí Ara Hurariyu"}, {"Newfoundland", "Iwí Pisasú Kurasí Ara Hurariyu"}, {"Norfolk", "Kapuãma Norfolk Kurasí Ara Hurariyu"}, {"Noronha", "Fenãdu Nuruyã Kurasí Ara Hurariyu"}, {"Novosibirsk", "Sibirisiki Pisasú Kurasí Ara Hurariyu"}, {"Omsk", "Omisiki Kurasí Ara Hurariyu"}, {"Pakistan", "Pakiretãma Kurasí Ara Hurariyu"}, {"Paraguay", "ParaguwaiKurasí Ara Hurariyu"}, {"Peru", "Peru Kurasí Ara Hurariyu"}, {"Philippines", "Firipina Kurasí Ara Hurariyu"}, {"Pierre_Miquelon", "Sã Peduru asuí Mikiran Kurasí Ara Hurariyu"}, {"Qyzylorda", "Kisiroda Kurasí Ara Hurariyu"}, {"Sakhalin", "Sakarina Kurasí Ara Hurariyu"}, {"Samara", "Samara Kurasí Ara Hurariyu"}, {"Samoa", "Samowa Kurasí Ara Hurariyu"}, {"Taipei", "Taipei Kurasí Ara Hurariyu"}, {"Tonga", "Tõga Kurasí Ara Hurariyu"}, {"Turkmenistan", "Turkuranaretãma Kurasí Ara Hurariyu"}, {"Uruguay", "Uruguwai Kurasí Ara Hurariyu"}, {"Uzbekistan", "Yũbuesara-retãma Kurasí Ara Hurariyu"}, {"Vanuatu", "Wanuatu Kurasí Ara Hurariyu"}, {"Vladivostok", "Waradiwosituki Kurasí Ara Hurariyu"}, {"Volgograd", "Worwogaradu Kurasí Ara Hurariyu"}, {"Yakutsk", "Yakutisiki Kurasí Ara Hurariyu"}, {"Yekaterinburg", "Ekaterĩbugu Kurasí Ara Hurariyu"}},
	},
	{
		{{"Acre", "艾克標準時間"}, {"Afghanistan", "阿富汗時間"}, {"Africa_Central", "中非時間"}, {"Africa_Eastern", "東非時間"}, {"Africa_Southern", "南非標準時間"}, {"Africa_Western", "西非時間"}, {"Alaska", "阿拉斯加標準時間"}, {"Almaty", "阿拉木圖標準時間"}, {"Amazon", "亞馬遜標準時間"}, {"America_Central", "中部標準時間"}, {"America_Eastern", "東部標準時間"}, {"America_Mountain", "山區標準時間"}, {"America_Pacific", "太平洋標準時間"}, {"Anadyr", "阿那底河標準時間"}, {"Apia", "阿皮亞標準時間"}, {"Aqtau", "阿克陶標準時間"}, {"Aqtobe", "阿克托比標準時間"}, {"Arabian", "阿拉伯標準時間"}, {"Argentina", "阿根廷標準時間"}, {"Argentina_Western", "阿根廷西部標準時間"}, {"Armenia", "亞美尼亞標準時間"}, {"Atlantic", "大西洋標準時間"}, {"Australia_Central", "澳洲中部標準時間"}, {"Australia_CentralWestern", "澳洲中西部標準時間"}, {"Australia_Eastern", "澳洲東部標準時間"}, {"Australia_Western", "澳洲西部標準時間"}, {"Azerbaijan", "亞塞拜然標準時間"}, {"Azores", "亞速爾群島標準時間"}, {"Bangladesh", "孟加拉標準時間"}, {"Bhutan", "不丹時間"}, {"Bolivia", "玻利維亞時間"}, {"Brasilia", "巴西利亞標準時間"}, {"Brunei", "汶萊時間"}, {"Cape_Verde", "維德角標準時間"}, {"Casey", "凱西站時間"}, {"Chamorro", "查莫洛時間"}, {"Chatham", "查坦群島標準時間"}, {"Chile", "智利標準時間"}, {"China", "中國標準時間"}, {"Christmas", "聖誕島時間"}, {"Cocos", "科科斯群島時間"}, {"Colombia", "哥倫比亞標準時間"}, {"Cook", "庫克群島標準時間"}, {"Cuba", "古巴標準時間"}, {"Davis", "戴維斯時間"}, {"DumontDUrville", "杜蒙杜比爾時間"}, {"East_Timor", "東帝汶時間"}, {"Easter", "復活節島標準時間"}, {"Ecuador", "厄瓜多時間"}, {"Europe_Central", "中歐標準時間"}, {"Europe_Eastern", "東歐標準時間"}, {"Europe_Further_Eastern", "歐洲遠東時間"}, {"Europe_Western", "西歐標準時間"}, {"Falkland", "福克蘭群島標準時間"}, {"Fiji", "斐濟標準時間"}, {"French_Guiana", "法屬圭亞那時間"}, {"French_Southern", "法國南方及南極時間"}, {"GMT", "格林威治標準時間"}, {"Galapagos", "加拉巴哥群島時間"}, {"Gambier", "甘比爾群島時間"}, {"Georgia", "喬治亞標準時間"}, {"Gilbert_Islands", "吉爾伯特群島時間"}, {"Greenland_Eastern", "格陵蘭東部標準時間"}, {"Greenland_Western", "格陵蘭西部標準時間"}, {"Guam", "關島標準時間"}, {"Gulf", "波斯灣海域標準時間"}, {"Guyana", "蓋亞那時間"}, {"Hawaii", "夏威夷-阿留申標準時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申標準時間"}, {"Hong_Kong", "香港標準時間"}, {"Hovd", "科布多標準時間"}, {"India", "印度標準時間"}, {"Indian_Ocean", "印度洋時間"}, {"Indochina", "印度支那時間"}, {"Indonesia_Central", "印尼中部時間"}, {"Indonesia_Eastern", "印尼東部時間"}, {"Indonesia_Western", "印尼西部時間"}, {"Iran", "伊朗標準時間"}, {"Irkutsk", "伊爾庫次克標準時間"}, {"Israel", "以色列標準時間"}, {"Japan", "日本標準時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克標準時間"}, {"Kazakhstan", "哈薩克時間"}, {"Kazakhstan_Eastern", "東哈薩克時間"}, {"Kazakhstan_Western", "西哈薩克時間"}, {"Korea", "韓國標準時間"}, {"Kosrae", "科斯瑞時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克標準時間"}, {"Kyrgystan", "吉爾吉斯時間"}, {"Lanka", "蘭卡時間"}, {"Line_Islands", "萊恩群島時間"}, {"Lord_Howe", "豪勳爵島標準時間"}, {"Macau", "澳門標準時間"}, {"Magadan", "馬加丹標準時間"}, {"Malaysia", "馬來西亞時間"}, {"Maldives", "馬爾地夫時間"}, {"Marquesas", "馬可薩斯時間"}, {"Marshall_Islands", "馬紹爾群島時間"}, {"Mauritius", "模里西斯標準時間"}, {"Mawson", "莫森時間"}, {"Mexico_Pacific", "墨西哥太平洋標準時間"}, {"Mongolia", "烏蘭巴托標準時間"}, {"Moscow", "莫斯科標準時間"}, {"Myanmar", "緬甸時間"}, {"Nauru", "諾魯時間"}, {"Nepal", "尼泊爾時間"}, {"New_Caledonia", "新喀里多尼亞標準時間"}, {"New_Zealand", "紐西蘭標準時間"}, {"Newfoundland", "紐芬蘭標準時間"}, {"Niue", "紐埃島時間"}, {"Norfolk", "諾福克島標準時間"}, {"Noronha", "費爾南多 - 迪諾羅尼亞標準時間"}, {"North_Mariana", "北馬里亞納群島時間"}, {"Novosibirsk", "新西伯利亞標準時間"}, {"Omsk", "鄂木斯克標準時間"}, {"Pakistan", "巴基斯坦標準時間"}, {"Palau", "帛琉時間"}, {"Papua_New_Guinea", "巴布亞紐幾內亞時間"}, {"Paraguay", "巴拉圭標準時間"}, {"Peru", "秘魯標準時間"}, {"Philippines", "菲律賓標準時間"}, {"Phoenix_Islands", "鳳凰群島時間"}, {"Pierre_Miquelon", "聖皮埃爾和密克隆群島標準時間"}, {"Pitcairn", "皮特肯時間"}, {"Ponape", "波納佩時間"}, {"Pyongyang", "平壤時間"}, {"Qyzylorda", "克孜勒奧爾達標準時間"}, {"Reunion", "留尼旺時間"}, {"Rothera", "羅瑟拉時間"}, {"Sakhalin", "庫頁島標準時間"}, {"Samara", "薩馬拉標準時間"}, {"Samoa", "薩摩亞標準時間"}, {"Seychelles", "塞席爾時間"}, {"Singapore", "新加坡標準時間"}, {"Solomon", "索羅門群島時間"}, {"South_Georgia", "南喬治亞時間"}, {"Suriname", "蘇利南時間"}, {"Syowa", "昭和基地時間"}, {"Tahiti", "大溪地時間"}, {"Taipei", "台北標準時間"}, {"Tajikistan", "塔吉克時間"}, {"Tokelau", "托克勞群島時間"}, {"Tonga", "東加標準時間"}, {"Truk", "楚克島時間"}, {"Turkey", "土耳其標準時間"}, {"Turkmenistan", "土庫曼標準時間"}, {"Tuvalu", "吐瓦魯時間"}, {"Uruguay", "烏拉圭標準時間"}, {"Uzbekistan", "烏茲別克標準時間"}, {"Vanuatu", "萬那杜標準時間"}, {"Venezuela", "委內瑞拉時間"}, {"Vladivostok", "海參崴標準時間"}, {"Volgograd", "伏爾加格勒標準時間"}, {"Vostok", "沃斯托克時間"}, {"Wake", "威克島時間"}, {"Wallis", "瓦利斯和富圖納群島時間"}, {"Yakutsk", "雅庫次克標準時間"}, {"Yekaterinburg", "葉卡捷琳堡標準時間"}, {"Yukon", "育空時間"}},
		{{"Acre", "艾克夏令時間"}, {"Alaska", "阿拉斯加夏令時間"}, {"Almaty", "阿拉木圖夏令時間"}, {"Amazon", "亞馬遜夏令時間"}, {"America_Central", "中部夏令時間"}, {"America_Eastern", "東部夏令時間"}, {"America_Mountain", "山區夏令時間"}, {"America_Pacific", "太平洋夏令時間"}, {"Anadyr", "阿那底河夏令時間"}, {"Apia", "阿皮亞夏令時間"}, {"Aqtau", "阿克陶夏令時間"}, {"Aqtobe", "阿克托比夏令時間"}, {"Arabian", "阿拉伯夏令時間"}, {"Argentina", "阿根廷夏令時間"}, {"Argentina_Western", "阿根廷西部夏令時間"}, {"Armenia", "亞美尼亞夏令時間"}, {"Atlantic", "大西洋夏令時間"}, {"Australia_Central", "澳洲中部夏令時間"}, {"Australia_CentralWestern", "澳洲中西部夏令時間"}, {"Australia_Eastern", "澳洲東部夏令時間"}, {"Australia_Western", "澳洲西部夏令時間"}, {"Azerbaijan", "亞塞拜然夏令時間"}, {"Azores", "亞速爾群島夏令時間"}, {"Bangladesh", "孟加拉夏令時間"}, {"Brasilia", "巴西利亞夏令時間"}, {"Cape_Verde", "維德角夏令時間"}, {"Chatham", "查坦群島夏令時間"}, {"Chile", "智利夏令時間"}, {"China", "中國夏令時間"}, {"Colombia", "哥倫比亞夏令時間"}, {"Cook", "庫克群島半夏令時間"}, {"Cuba", "古巴夏令時間"}, {"Easter", "復活節島夏令時間"}, {"Europe_Central", "中歐夏令時間"}, {"Europe_Eastern", "東歐夏令時間"}, {"Europe_Western", "西歐夏令時間"}, {"Falkland", "福克蘭群島夏令時間"}, {"Fiji", "斐濟夏令時間"}, {"Georgia", "喬治亞夏令時間"}, {"Greenland_Eastern", "格陵蘭東部夏令時間"}, {"Greenland_Western", "格陵蘭西部夏令時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申夏令時間"}, {"Hong_Kong", "香港夏令時間"}, {"Hovd", "科布多夏令時間"}, {"Iran", "伊朗夏令時間"}, {"Irkutsk", "伊爾庫次克夏令時間"}, {"Israel", "以色列夏令時間"}, {"Japan", "日本夏令時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克日光節約時間"}, {"Korea", "韓國夏令時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克夏令時間"}, {"Lord_Howe", "豪勳爵島夏令時間"}, {"Macau", "澳門夏令時間"}, {"Magadan", "馬加丹夏令時間"}, {"Mauritius", "模里西斯夏令時間"}, {"Mexico_Pacific", "墨西哥太平洋夏令時間"}, {"Mongolia", "烏蘭巴托夏令時間"}, {"Moscow", "莫斯科夏令時間"}, {"New_Caledonia", "新喀里多尼亞群島夏令時間"}, {"New_Zealand", "紐西蘭夏令時間"}, {"Newfoundland", "紐芬蘭夏令時間"}, {"Norfolk", "諾福克島夏令時間"}, {"Noronha", "費爾南多 - 迪諾羅尼亞夏令時間"}, {"Novosibirsk", "新西伯利亞夏令時間"}, {"Omsk", "鄂木斯克夏令時間"}, {"Pakistan", "巴基斯坦夏令時間"}, {"Paraguay", "巴拉圭夏令時間"}, {"Peru", "秘魯夏令時間"}, {"Philippines", "菲律賓夏令時間"}, {"Pierre_Miquelon", "聖皮埃爾和密克隆群島夏令時間"}, {"Qyzylorda", "克孜勒奧爾達夏令時間"}, {"Sakhalin", "庫頁島夏令時間"}, {"Samara", "薩馬拉夏令時間"}, {"Samoa", "薩摩亞夏令時間"}, {"Taipei", "台北夏令時間"}, {"Tonga", "東加夏令時間"}, {"Turkey", "土耳其夏令時間"}, {"Turkmenistan", "土庫曼夏令時間"}, {"Uruguay", "烏拉圭夏令時間"}, {"Uzbekistan", "烏茲別克夏令時間"}, {"Vanuatu", "萬那杜夏令時間"}, {"Vladivostok", "海參崴夏令時間"}, {"Volgograd", "伏爾加格勒夏令時間"}, {"Yakutsk", "雅庫次克夏令時間"}, {"Yekaterinburg", "葉卡捷琳堡夏令時間"}},
	},
	{
		{{"Acre", "艾克标准时间"}, {"Afghanistan", "阿富汗时间"}, {"Africa_Central", "中非时间"}, {"Africa_Eastern", "东非时间"}, {"Africa_Southern", "南非标准时间"}, {"Africa_Western", "西非时间"}, {"Alaska", "阿拉斯加标准时间"}, {"Almaty", "阿拉木图标准时间"}, {"Amazon", "亚马逊标准时间"}, {"America_Central", "中部标准时间"}, {"America_Eastern", "东部标准时间"}, {"America_Mountain", "山区标准时间"}, {"America_Pacific", "太平洋标准时间"}, {"Anadyr", "阿那底河标准时间"}, {"Apia", "阿皮亚标准时间"}, {"Aqtau", "阿克陶标准时间"}, {"Aqtobe", "阿克托比标准时间"}, {"Arabian", "阿拉伯标准时间"}, {"Argentina", "阿根廷标准时间"}, {"Argentina_Western", "阿根廷西部标准时间"}, {"Armenia", "亚美尼亚标准时间"}, {"Atlantic", "大西洋标准时间"}, {"Australia_Central", "澳洲中部标准时间"}, {"Australia_CentralWestern", "澳洲中西部标准时间"}, {"Australia_Eastern", "澳洲东部标准时间"}, {"Australia_Western", "澳洲西部标准时间"}, {"Azerbaijan", "亚塞拜然标准时间"}, {"Azores", "亚速尔群岛标准时间"}, {"Bangladesh", "孟加拉标准时间"}, {"Bhutan", "不丹时间"}, {"Bolivia", "玻利维亚时间"}, {"Brasilia", "巴西利亚标准时间"}, {"Brunei", "汶莱时间"}, {"Cape_Verde", "维德角标准时间"}, {"Casey", "凯西站时间"}, {"Chamorro", "查莫洛时间"}, {"Chatham", "查坦群岛标准时间"}, {"Chile", "智利标准时间"}, {"China", "中国标准时间"}, {"Christmas", "圣诞岛时间"}, {"Cocos", "科科斯群岛时间"}, {"Colombia", "哥伦比亚标准时间"}, {"Cook", "库克群岛标准时间"}, {"Cuba", "古巴标准时间"}, {"Davis", "戴维斯时间"}, {"DumontDUrville", "杜蒙杜比尔时间"}, {"East_Timor", "东帝汶时间"}, {"Easter", "复活节岛标准时间"}, {"Ecuador", "厄瓜多时间"}, {"Europe_Central", "中欧标准时间"}, {"Europe_Eastern", "东欧标准时间"}, {"Europe_Further_Eastern", "欧洲远东时间"}, {"Europe_Western", "西欧标准时间"}, {"Falkland", "福克兰群岛标准时间"}, {"Fiji", "斐济标准时间"}, {"French_Guiana", "法属圭亚那时间"}, {"French_Southern", "法国南方及南极时间"}, {"GMT", "格林威治标准时间"}, {"Galapagos", "加拉巴哥群岛时间"}, {"Gambier", "甘比尔群岛时间"}, {"Georgia", "乔治亚标准时间"}, {"Gilbert_Islands", "吉尔伯特群岛时间"}, {"Greenland_Eastern", "格陵兰东部标准时间"}, {"Greenland_Western", "格陵兰西部标准时间"}, {"Guam", "关岛标准时间"}, {"Gulf", "波斯湾海域标准时间"}, {"Guyana", "盖亚那时间"}, {"Hawaii", "夏威夷-阿留申标准时间"}, {"Hawaii_Aleutian", "夏威夷-阿留申标准时间"}, {"Hong_Kong", "香港标准时间"}, {"Hovd", "科布多标准时间"}, {"India", "印度标准时间"}, {"Indian_Ocean", "印度洋时间"}, {"Indochina", "印度支那时间"}, {"Indonesia_Central", "印尼中部时间"}, {"Indonesia_Eastern", "印尼东部时间"}, {"Indonesia_Western", "印尼西部时间"}, {"Iran", "伊朗标准时间"}, {"Irkutsk", "伊尔库次克标准时间"}, {"Israel", "以色列标准时间"}, {"Japan", "日本标准时间"}, {"Kamchatka", "彼得罗巴甫洛夫斯克标准时间"}, {"Kazakhstan", "哈萨克时间"}, {"Kazakhstan_Eastern", "东哈萨克时间"}, {"Kazakhstan_Western", "西哈萨克时间"}, {"Korea", "韩国标准时间"}, {"Kosrae", "科斯瑞时间"}, {"Krasnoyarsk", "克拉斯诺亚尔斯克标准时间"}, {"Kyrgystan", "吉尔吉斯时间"}, {"Lanka", "兰卡时间"}, {"Line_Islands", "莱恩群岛时间"}, {"Lord_Howe", "豪勋爵岛标准时间"}, {"Macau", "澳门标准时间"}, {"Magadan", "马加丹标准时间"}, {"Malaysia", "马来西亚时间"}, {"Maldives", "马尔地夫时间"}, {"Marquesas", "马可萨斯时间"}, {"Marshall_Islands", "马绍尔群岛时间"}, {"Mauritius", "模里西斯标准时间"}, {"Mawson", "莫森时间"}, {"Mexico_Pacific", "墨西哥太平洋标准时间"}, {"Mongolia", "乌兰巴托标准时间"}, {"Moscow", "莫斯科标准时间"}, {"Myanmar", "缅甸时间"}, {"Nauru", "诺鲁时间"}, {"Nepal", "尼泊尔时间"}, {"New_Caledonia", "新喀里多尼亚标准时间"}, {"New_Zealand", "纽西兰标准时间"}, {"Newfoundland", "纽芬兰标准时间"}, {"Niue", "纽埃岛时间"}, {"Norfolk", "诺福克岛标准时间"}, {"Noronha", "费尔南多 - 迪诺罗尼亚标准时间"}, {"North_Mariana", "北马里亚纳群岛时间"}, {"Novosibirsk", "新西伯利亚标准时间"}, {"Omsk", "鄂木斯克标准时间"}, {"Pakistan", "巴基斯坦标准时间"}, {"Palau", "帛琉时间"}, {"Papua_New_Guinea", "巴布亚纽几内亚时间"}, {"Paraguay", "巴拉圭标准时间"}, {"Peru", "秘鲁标准时间"}, {"Philippines", "菲律宾标准时间"}, {"Phoenix_Islands", "凤凰群岛时间"}, {"Pierre_Miquelon", "圣皮埃尔和密克隆群岛标准时间"}, {"Pitcairn", "皮特肯时间"}, {"Ponape", "波纳佩时间"}, {"Pyongyang", "平壤时间"}, {"Qyzylorda", "克孜勒奥尔达标准时间"}, {"Reunion", "留尼旺时间"}, {"Rothera", "罗瑟拉时间"}, {"Sakhalin", "库页岛标准时间"}, {"Samara", "萨马拉标准时间"}, {"Samoa", "萨摩亚标准时间"}, {"Seychelles", "塞席尔时间"}, {"Singapore", "新加坡标准时间"}, {"Solomon", "索罗门群岛时间"}, {"South_Georgia", "南乔治亚时间"}, {"Suriname", "苏利南时间"}, {"Syowa", "昭和基地时间"}, {"Tahiti", "大溪地时间"}, {"Taipei", "台北标准时间"}, {"Tajikistan", "塔吉克时间"}, {"Tokelau", "托克劳群岛时间"}, {"Tonga", "东加标准时间"}, {"Truk", "楚克岛时间"}, {"Turkey", "土耳其标准时间"}, {"Turkmenistan", "土库曼标准时间"}, {"Tuvalu", "吐瓦鲁时间"}, {"Uruguay", "乌拉圭标准时间"}, {"Uzbekistan", "乌兹别克标准时间"}, {"Vanuatu", "万那杜标准时间"}, {"Venezuela", "委内瑞拉时间"}, {"Vladivostok", "海参崴标准时间"}, {"Volgograd", "伏尔加格勒标准时间"}, {"Vostok", "沃斯托克时间"}, {"Wake", "威克岛时间"}, {"Wallis", "瓦利斯和富图纳群岛时间"}, {"Yakutsk", "雅库次克标准时间"}, {"Yekaterinburg", "叶卡捷琳堡标准时间"}, {"Yukon", "育空时间"}},
		{{"Acre", "艾克夏令时间"}, {"Alaska", "阿拉斯加夏令时间"}, {"Almaty", "阿拉木图夏令时间"}, {"Amazon", "亚马逊夏令时间"}, {"America_Central", "中部夏令时间"}, {"America_Eastern", "东部夏令时间"}, {"America_Mountain", "山区夏令时间"}, {"America_Pacific", "太平洋夏令时间"}, {"Anadyr", "阿那底河夏令时间"}, {"Apia", "阿皮亚夏令时间"}, {"Aqtau", "阿克陶夏令时间"}, {"Aqtobe", "阿克托比夏令时间"}, {"Arabian", "阿拉伯夏令时间"}, {"Argentina", "阿根廷夏令时间"}, {"Argentina_Western", "阿根廷西部夏令时间"}, {"Armenia", "亚美尼亚夏令时间"}, {"Atlantic", "大西洋夏令时间"}, {"Australia_Central", "澳洲中部夏令时间"}, {"Australia_CentralWestern", "澳洲中西部夏令时间"}, {"Australia_Eastern", "澳洲东部夏令时间"}, {"Australia_Western", "澳洲西部夏令时间"}, {"Azerbaijan", "亚塞拜然夏令时间"}, {"Azores", "亚速尔群岛夏令时间"}, {"Bangladesh", "孟加拉夏令时间"}, {"Brasilia", "巴西利亚夏令时间"}, {"Cape_Verde", "维德角夏令时间"}, {"Chatham", "查坦群岛夏令时间"}, {"Chile", "智利夏令时间"}, {"China", "中国夏令时间"}, {"Colombia", "哥伦比亚夏令时间"}, {"Cook", "库克群岛半夏令时间"}, {"Cuba", "古巴夏令时间"}, {"Easter", "复活节岛夏令时间"}, {"Europe_Central", "中欧夏令时间"}, {"Europe_Eastern", "东欧夏令时间"}, {"Europe_Western", "西欧夏令时间"}, {"Falkland", "福克兰群岛夏令时间"}, {"Fiji", "斐济夏令时间"}, {"Georgia", "乔治亚夏令时间"}, {"Greenland_Eastern", "格陵兰东部夏令时间"}, {"Greenland_Western", "格陵兰西部夏令时间"}, {"Hawaii_Aleutian", "夏威夷-阿留申夏令时间"}, {"Hong_Kong", "香港夏令时间"}, {"Hovd", "科布多夏令时间"}, {"Iran", "伊朗夏令时间"}, {"Irkutsk", "伊尔库次克夏令时间"}, {"Israel", "以色列夏令时间"}, {"Japan", "日本夏令时间"}, {"Kamchatka", "彼得罗巴甫洛夫斯克日光节约时间"}, {"Korea", "韩国夏令时间"}, {"Krasnoyarsk", "克拉斯诺亚尔斯克夏令时间"}, {"Lord_Howe", "豪勋爵岛夏令时间"}, {"Macau", "澳门夏令时间"}, {"Magadan", "马加丹夏令时间"}, {"Mauritius", "模里西斯夏令时间"}, {"Mexico_Pacific", "墨西哥太平洋夏令时间"}, {"Mongolia", "乌兰巴托夏令时间"}, {"Moscow", "莫斯科夏令时间"}, {"New_Caledonia", "新喀里多尼亚群岛夏令时间"}, {"New_Zealand", "纽西兰夏令时间"}, {"Newfoundland", "纽芬兰夏令时间"}, {"Norfolk", "诺福克岛夏令时间"}, {"Noronha", "费尔南多 - 迪诺罗尼亚夏令时间"}, {"Novosibirsk", "新西伯利亚夏令时间"}, {"Omsk", "鄂木斯克夏令时间"}, {"Pakistan", "巴基斯坦夏令时间"}, {"Paraguay", "巴拉圭夏令时间"}, {"Peru", "秘鲁夏令时间"}, {"Philippines", "菲律宾夏令时间"}, {"Pierre_Miquelon", "圣皮埃尔和密克隆群岛夏令时间"}, {"Qyzylorda", "克孜勒奥尔达夏令时间"}, {"Sakhalin", "库页岛夏令时间"}, {"Samara", "萨马拉夏令时间"}, {"Samoa", "萨摩亚夏令时间"}, {"Taipei", "台北夏令时间"}, {"Tonga", "东加夏令时间"}, {"Turkey", "土耳其夏令时间"}, {"Turkmenistan", "土库曼夏令时间"}, {"Uruguay", "乌拉圭夏令时间"}, {"Uzbekistan", "乌兹别克夏令时间"}, {"Vanuatu", "万那杜夏令时间"}, {"Vladivostok", "海参崴夏令时间"}, {"Volgograd", "伏尔加格勒夏令时间"}, {"Yakutsk", "雅库次克夏令时间"}, {"Yekaterinburg", "叶卡捷琳堡夏令时间"}},
	},
	{
		{{"Acre", "阿克里标准时间"}, {"Afghanistan", "阿富汗时间"}, {"Africa_Central", "中部非洲时间"}, {"Africa_Eastern", "东部非洲时间"}, {"Africa_Southern", "南非标准时间"}, {"Africa_Western", "西部非洲时间"}, {"Alaska", "阿拉斯加标准时间"}, {"Almaty", "阿拉木图标准时间"}, {"Amazon", "亚马逊标准时间"}, {"America_Central", "北美中部标准时间"}, {"America_Eastern", "北美东部标准时间"}, {"America_Mountain", "北美山区标准时间"}, {"America_Pacific", "北美太平洋标准时间"}, {"Anadyr", "阿纳德尔标准时间"}, {"Apia", "阿皮亚标准时间"}, {"Aqtau", "阿克套标准时间"}, {"Aqtobe", "阿克托别标准时间"}, {"Arabian", "阿拉伯标准时间"}, {"Argentina", "阿根廷标准时间"}, {"Argentina_Western", "阿根廷西部标准时间"}, {"Armenia", "亚美尼亚标准时间"}, {"Atlantic", "大西洋标准时间"}, {"Australia_Central", "澳大利亚中部标准时间"}, {"Australia_CentralWestern", "澳大利亚中西部标准时间"}, {"Australia_Eastern", "澳大利亚东部标准时间"}, {"Australia_Western", "澳大利亚西部标准时间"}, {"Azerbaijan", "阿塞拜疆标准时间"}, {"Azores", "亚速尔群岛标准时间"}, {"Bangladesh", "孟加拉标准时间"}, {"Bhutan", "不丹时间"}, {"Bolivia", "玻利维亚标准时间"}, {"Brasilia", "巴西利亚标准时间"}, {"Brunei", "文莱达鲁萨兰时间"}, {"Cape_Verde", "佛得角标准时间"}, {"Casey", "凯西时间"}, {"Chamorro", "查莫罗时间"}, {"Chatham", "查塔姆标准时间"}, {"Chile", "智利标准时间"}, {"China", "中国标准时间"}, {"Christmas", "圣诞岛时间"}, {"Cocos", "科科斯群岛时间"}, {"Colombia", "哥伦比亚标准时间"}, {"Cook", "库克群岛标准时间"}, {"Cuba", "古巴标准时间"}, {"Davis", "戴维斯时间"}, {"DumontDUrville", "迪蒙·迪维尔时间"}, {"East_Timor", "东帝汶时间"}, {"Easter", "复活节岛标准时间"}, {"Ecuador", "厄瓜多尔标准时间"}, {"Europe_Central", "中欧标准时间"}, {"Europe_Eastern", "东欧标准时间"}, {"Europe_Further_Eastern", "欧洲极东时间"}, {"Europe_Western", "西欧标准时间"}, {"Falkland", "福克兰群岛标准时间"}, {"Fiji", "斐济标准时间"}, {"French_Guiana", "法属圭亚那标准时间"}, {"French_Southern", "法属南方和南极领地时间"}, {"GMT", "格林尼治标准时间"}, {"Galapagos", "科隆群岛时间"}, {"Gambier", "甘比尔时间"}, {"Georgia", "格鲁吉亚标准时间"}, {"Gilbert_Islands", "吉尔伯特群岛时间"}, {"Greenland_Eastern", "格陵兰岛东部标准时间"}, {"Greenland_Western", "格陵兰岛西部标准时间"}, {"Guam", "关岛时间"}, {"Gulf", "海湾标准时间"}, {"Guyana", "圭亚那时间"}, {"Hawaii", "夏威夷-阿留申标准时间"}, {"Hawaii_Aleutian", "夏威夷-阿留申标准时间"}, {"Hong_Kong", "香港标准时间"}, {"Hovd", "科布多标准时间"}, {"India", "印度时间"}, {"Indian_Ocean", "印度洋时间"}, {"Indochina", "中南半岛时间"}, {"Indonesia_Central", "印度尼西亚中部时间"}, {"Indonesia_Eastern", "印度尼西亚东部时间"}, {"Indonesia_Western", "印度尼西亚西部时间"}, {"Iran", "伊朗标准时间"}, {"Irkutsk", "伊尔库茨克标准时间"}, {"Israel", "以色列标准时间"}, {"Japan", "日本标准时间"}, {"Kamchatka", "彼得罗巴甫洛夫斯克-堪察加标准时间"}, {"Kazakhstan", "哈萨克斯坦时间"}, {"Kazakhstan_Eastern", "哈萨克斯坦东部时间"}, {"Kazakhstan_Western", "哈萨克斯坦西部时间"}, {"Korea", "韩国标准时间"}, {"Kosrae", "科斯雷时间"}, {"Krasnoyarsk", "克拉斯诺亚尔斯克标准时间"}, {"Kyrgystan", "吉尔吉斯斯坦时间"}, {"Lanka", "兰卡时间"}, {"Line_Islands", "莱恩群岛时间"}, {"Lord_Howe", "豪勋爵岛标准时间"}, {"Macau", "澳门标准时间"}, {"Magadan", "马加丹标准时间"}, {"Malaysia", "马来西亚时间"}, {"Maldives", "马尔代夫时间"}, {"Marquesas", "马克萨斯群岛时间"}, {"Marshall_Islands", "马绍尔群岛时间"}, {"Mauritius", "毛里求斯标准时间"}, {"Mawson", "莫森时间"}, {"Mexico_Pacific", "墨西哥太平洋标准时间"}, {"Mongolia", "乌兰巴托标准时间"}, {"Moscow", "莫斯科标准时间"}, {"Myanmar", "缅甸时间"}, {"Nauru", "瑙鲁时间"}, {"Nepal", "尼泊尔时间"}, {"New_Caledonia", "新喀里多尼亚标准时间"}, {"New_Zealand", "新西兰标准时间"}, {"Newfoundland", "纽芬兰标准时间"}, {"Niue", "纽埃时间"}, {"Norfolk", "诺福克岛标准时间"}, {"Noronha", "费尔南多-迪诺罗尼亚岛标准时间"}, {"North_Mariana", "北马里亚纳群岛时间"}, {"Novosibirsk", "新西伯利亚标准时间"}, {"Omsk", "鄂木斯克标准时间"}, {"Pakistan", "巴基斯坦标准时间"}, {"Palau", "帕劳时间"}, {"Papua_New_Guinea", "巴布亚新几内亚时间"}, {"Paraguay", "巴拉圭标准时间"}, {"Peru", "秘鲁标准时间"}, {"Philippines", "菲律宾标准时间"}, {"Phoenix_Islands", "菲尼克斯群岛时间"}, {"Pierre_Miquelon", "圣皮埃尔和密克隆群岛标准时间"}, {"Pitcairn", "皮特凯恩时间"}, {"Ponape", "波纳佩时间"}, {"Pyongyang", "平壤时间"}, {"Qyzylorda", "克孜洛尔达标准时间"}, {"Reunion", "留尼汪时间"}, {"Rothera", "罗瑟拉时间"}, {"Sakhalin", "库页岛标准时间"}, {"Samara", "萨马拉标准时间"}, {"Samoa", "萨摩亚标准时间"}, {"Seychelles", "塞舌尔时间"}, {"Singapore", "新加坡标准时间"}, {"Solomon", "所罗门群岛时间"}, {"South_Georgia", "南乔治亚岛时间"}, {"Suriname", "苏里南时间"}, {"Syowa", "昭和时间"}, {"Tahiti", "塔希提岛时间"}, {"Taipei", "台北标准时间"}, {"Tajikistan", "塔吉克斯坦时间"}, {"Tokelau", "托克劳时间"}, {"Tonga", "汤加标准时间"}, {"Truk", "楚克时间"}, {"Turkey", "土耳其标准时间"}, {"Turkmenistan", "土库曼斯坦标准时间"}, {"Tuvalu", "图瓦卢时间"}, {"Uruguay", "乌拉圭标准时间"}, {"Uzbekistan", "乌兹别克斯坦标准时间"}, {"Vanuatu", "瓦努阿图标准时间"}, {"Venezuela", "委内瑞拉时间"}, {"Vladivostok", "海参崴标准时间"}, {"Volgograd", "伏尔加格勒标准时间"}, {"Vostok", "沃斯托克时间"}, {"Wake", "威克岛时间"}, {"Wallis", "瓦利斯和富图纳时间"}, {"Yakutsk", "雅库茨克标准时间"}, {"Yekaterinburg", "叶卡捷琳堡标准时间"}, {"Yukon", "育空时间"}},
		{{"Acre", "阿克里夏令时间"}, {"Alaska", "阿拉斯加夏令时间"}, {"Almaty", "阿拉木图夏令时间"}, {"Amazon", "亚马逊夏令时间"}, {"America_Central", "北美中部夏令时间"}, {"America_Eastern", "北美东部夏令时间"}, {"America_Mountain", "北美山区夏令时间"}, {"America_Pacific", "北美太平洋夏令时间"}, {"Anadyr", "阿纳德尔夏令时间"}, {"Apia", "阿皮亚夏令时间"}, {"Aqtau", "阿克套夏令时间"}, {"Aqtobe", "阿克托别夏令时间"}, {"Arabian", "阿拉伯夏令时间"}, {"Argentina", "阿根廷夏令时间"}, {"Argentina_Western", "阿根廷西部夏令时间"}, {"Armenia", "亚美尼亚夏令时间"}, {"Atlantic", "大西洋夏令时间"}, {"Australia_Central", "澳大利亚中部夏令时间"}, {"Australia_CentralWestern", "澳大利亚中西部夏令时间"}, {"Australia_Eastern", "澳大利亚东部夏令时间"}, {"Australia_Western", "澳大利亚西部夏令时间"}, {"Azerbaijan", "阿塞拜疆夏令时间"}, {"Azores", "亚速尔群岛夏令时间"}, {"Bangladesh", "孟加拉夏令时间"}, {"Brasilia", "巴西利亚夏令时间"}, {"Cape_Verde", "佛得角夏令时间"}, {"Chatham", "查塔姆夏令时间"}, {"Chile", "智利夏令时间"}, {"China", "中国夏令时间"}, {"Colombia", "哥伦比亚夏令时间"}, {"Cook", "库克群岛仲夏时间"}, {"Cuba", "古巴夏令时间"}, {"Easter", "复活节岛夏令时间"}, {"Europe_Central", "中欧夏令时间"}, {"Europe_Eastern", "东欧夏令时间"}, {"Europe_Western", "西欧夏令时间"}, {"Falkland", "福克兰群岛夏令时间"}, {"Fiji", "斐济夏令时间"}, {"Georgia", "格鲁吉亚夏令时间"}, {"Greenland_Eastern", "格陵兰岛东部夏令时间"}, {"Greenland_Western", "格陵兰岛西部夏令时间"}, {"Hawaii_Aleutian", "夏威夷-阿留申夏令时间"}, {"Hong_Kong", "香港夏令时间"}, {"Hovd", "科布多夏令时间"}, {"Iran", "伊朗夏令时间"}, {"Irkutsk", "伊尔库茨克夏令时间"}, {"Israel", "以色列夏令时间"}, {"Japan", "日本夏令时间"}, {"Kamchatka", "彼得罗巴甫洛夫斯克-堪察加夏令时间"}, {"Korea", "韩国夏令时间"}, {"Krasnoyarsk", "克拉斯诺亚尔斯克夏令时间"}, {"Lord_Howe", "豪勋爵岛夏令时间"}, {"Macau", "澳门夏令时间"}, {"Magadan", "马加丹夏令时间"}, {"Mauritius", "毛里求斯夏令时间"}, {"Mexico_Pacific", "墨西哥太平洋夏令时间"}, {"Mongolia", "乌兰巴托夏令时间"}, {"Moscow", "莫斯科夏令时间"}, {"New_Caledonia", "新喀里多尼亚夏令时间"}, {"New_Zealand", "新西兰夏令时间"}, {"Newfoundland", "纽芬兰夏令时间"}, {"Norfolk", "诺福克岛夏令时间"}, {"Noronha", "费尔南多-迪诺罗尼亚岛夏令时间"}, {"Novosibirsk", "新西伯利亚夏令时间"}, {"Omsk", "鄂木斯克夏令时间"}, {"Pakistan", "巴基斯坦夏令时间"}, {"Paraguay", "巴拉圭夏令时间"}, {"Peru", "秘鲁夏令时间"}, {"Philippines", "菲律宾夏令时间"}, {"Pierre_Miquelon", "圣皮埃尔和密克隆群岛夏令时间"}, {"Qyzylorda", "克孜洛尔达夏令时间"}, {"Sakhalin", "库页岛夏令时间"}, {"Samara", "萨马拉夏令时间"}, {"Samoa", "萨摩亚夏令时间"}, {"Taipei", "台北夏令时间"}, {"Tonga", "汤加夏令时间"}, {"Turkey", "土耳其夏令时间"}, {"Turkmenistan", "土库曼斯坦夏令时间"}, {"Uruguay", "乌拉圭夏令时间"}, {"Uzbekistan", "乌兹别克斯坦夏令时间"}, {"Vanuatu", "瓦努阿图夏令时间"}, {"Vladivostok", "海参崴夏令时间"}, {"Volgograd", "伏尔加格勒夏令时间"}, {"Yakutsk", "雅库茨克夏令时间"}, {"Yekaterinburg", "叶卡捷琳堡夏令时间"}},
	},
	{
		{{"Acre", "艾克標準時間"}, {"Afghanistan", "阿富汗時間"}, {"Africa_Central", "中非時間"}, {"Africa_Eastern", "東非時間"}, {"Africa_Southern", "南非標準時間"}, {"Africa_Western", "西非時間"}, {"Alaska", "阿拉斯加標準時間"}, {"Almaty", "阿拉木圖標準時間"}, {"Amazon", "亞馬遜標準時間"}, {"America_Central", "中部標準時間"}, {"America_Eastern", "東部標準時間"}, {"America_Mountain", "山區標準時間"}, {"America_Pacific", "太平洋標準時間"}, {"Anadyr", "阿那底河標準時間"}, {"Apia", "阿皮亞標準時間"}, {"Aqtau", "阿克陶標準時間"}, {"Aqtobe", "阿克托比標準時間"}, {"Arabian", "阿拉伯標準時間"}, {"Argentina", "阿根廷標準時間"}, {"Argentina_Western", "阿根廷西部標準時間"}, {"Armenia", "亞美尼亞標準時間"}, {"Atlantic", "大西洋標準時間"}, {"Australia_Central", "澳洲中部標準時間"}, {"Australia_CentralWestern", "澳洲中西部標準時間"}, {"Australia_Eastern", "澳洲東部標準時間"}, {"Australia_Western", "澳洲西部標準時間"}, {"Azerbaijan", "亞塞拜然標準時間"}, {"Azores", "亞速爾群島標準時間"}, {"Bangladesh", "孟加拉標準時間"}, {"Bhutan", "不丹時間"}, {"Bolivia", "玻利維亞時間"}, {"Brasilia", "巴西利亞標準時間"}, {"Brunei", "汶萊時間"}, {"Cape_Verde", "維德角標準時間"}, {"Casey", "凱西站時間"}, {"Chamorro", "查莫洛時間"}, {"Chatham", "查坦群島標準時間"}, {"Chile", "智利標準時間"}, {"China", "中國標準時間"}, {"Christmas", "聖誕島時間"}, {"Cocos", "科科斯群島時間"}, {"Colombia", "哥倫比亞標準時間"}, {"Cook", "庫克群島標準時間"}, {"Cuba", "古巴標準時間"}, {"Davis", "戴維斯時間"}, {"DumontDUrville", "杜蒙杜比爾時間"}, {"East_Timor", "東帝汶時間"}, {"Easter", "復活節島標準時間"}, {"Ecuador", "厄瓜多時間"}, {"Europe_Central", "中歐標準時間"}, {"Europe_Eastern", "東歐標準時間"}, {"Europe_Further_Eastern", "歐洲遠東時間"}, {"Europe_Western", "西歐標準時間"}, {"Falkland", "福克蘭群島標準時間"}, {"Fiji", "斐濟標準時間"}, {"French_Guiana", "法屬圭亞那時間"}, {"French_Southern", "法國南方及南極時間"}, {"GMT", "格林威治標準時間"}, {"Galapagos", "加拉巴哥群島時間"}, {"Gambier", "甘比爾群島時間"}, {"Georgia", "喬治亞標準時間"}, {"Gilbert_Islands", "吉爾伯特群島時間"}, {"Greenland_Eastern", "格陵蘭東部標準時間"}, {"Greenland_Western", "格陵蘭西部標準時間"}, {"Guam", "關島標準時間"}, {"Gulf", "波斯灣海域標準時間"}, {"Guyana", "蓋亞那時間"}, {"Hawaii", "夏威夷-阿留申標準時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申標準時間"}, {"Hong_Kong", "香港標準時間"}, {"Hovd", "科布多標準時間"}, {"India", "印度標準時間"}, {"Indian_Ocean", "印度洋時間"}, {"Indochina", "中南半島時間"}, {"Indonesia_Central", "印尼中部時間"}, {"Indonesia_Eastern", "印尼東部時間"}, {"Indonesia_Western", "印尼西部時間"}, {"Iran", "伊朗標準時間"}, {"Irkutsk", "伊爾庫次克標準時間"}, {"Israel", "以色列標準時間"}, {"Japan", "日本標準時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克標準時間"}, {"Kazakhstan", "哈薩克時間"}, {"Kazakhstan_Eastern", "東哈薩克時間"}, {"Kazakhstan_Western", "西哈薩克時間"}, {"Korea", "韓國標準時間"}, {"Kosrae", "科斯瑞時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克標準時間"}, {"Kyrgystan", "吉爾吉斯時間"}, {"Lanka", "蘭卡時間"}, {"Line_Islands", "萊恩群島時間"}, {"Lord_Howe", "豪勳爵島標準時間"}, {"Macau", "澳門標準時間"}, {"Magadan", "馬加丹標準時間"}, {"Malaysia", "馬來西亞時間"}, {"Maldives", "馬爾地夫時間"}, {"Marquesas", "馬可薩斯時間"}, {"Marshall_Islands", "馬紹爾群島時間"}, {"Mauritius", "模里西斯標準時間"}, {"Mawson", "莫森時間"}, {"Mexico_Pacific", "墨西哥太平洋標準時間"}, {"Mongolia", "烏蘭巴托標準時間"}, {"Moscow", "莫斯科標準時間"}, {"Myanmar", "緬甸時間"}, {"Nauru", "諾魯時間"}, {"Nepal", "尼泊爾時間"}, {"New_Caledonia", "新喀里多尼亞標準時間"}, {"New_Zealand", "紐西蘭標準時間"}, {"Newfoundland", "紐芬蘭標準時間"}, {"Niue", "紐埃島時間"}, {"Norfolk", "諾福克島標準時間"}, {"Noronha", "費爾南多 - 迪諾羅尼亞標準時間"}, {"North_Mariana", "北馬里亞納群島時間"}, {"Novosibirsk", "新西伯利亞標準時間"}, {"Omsk", "鄂木斯克標準時間"}, {"Pakistan", "巴基斯坦標準時間"}, {"Palau", "帛琉時間"}, {"Papua_New_Guinea", "巴布亞紐幾內亞時間"}, {"Paraguay", "巴拉圭標準時間"}, {"Peru", "秘魯標準時間"}, {"Philippines", "菲律賓標準時間"}, {"Phoenix_Islands", "鳳凰群島時間"}, {"Pierre_Miquelon", "聖皮埃與密克隆群島標準時間"}, {"Pitcairn", "皮特肯時間"}, {"Ponape", "波納佩時間"}, {"Pyongyang", "平壤時間"}, {"Qyzylorda", "克孜勒奧爾達標準時間"}, {"Reunion", "留尼旺時間"}, {"Rothera", "羅瑟拉時間"}, {"Sakhalin", "庫頁島標準時間"}, {"Samara", "薩馬拉標準時間"}, {"Samoa", "薩摩亞標準時間"}, {"Seychelles", "塞席爾時間"}, {"Singapore", "新加坡標準時間"}, {"Solomon", "索羅門群島時間"}, {"South_Georgia", "南喬治亞時間"}, {"Suriname", "蘇利南時間"}, {"Syowa", "昭和基地時間"}, {"Tahiti", "大溪地時間"}, {"Taipei", "台北標準時間"}, {"Tajikistan", "塔吉克時間"}, {"Tokelau", "托克勞群島時間"}, {"Tonga", "東加標準時間"}, {"Truk", "楚克島時間"}, {"Turkey", "土耳其標準時間"}, {"Turkmenistan", "土庫曼標準時間"}, {"Tuvalu", "吐瓦魯時間"}, {"Uruguay", "烏拉圭標準時間"}, {"Uzbekistan", "烏茲別克標準時間"}, {"Vanuatu", "萬那杜標準時間"}, {"Venezuela", "委內瑞拉時間"}, {"Vladivostok", "海參崴標準時間"}, {"Volgograd", "伏爾加格勒標準時間"}, {"Vostok", "沃斯托克時間"}, {"Wake", "威克島時間"}, {"Wallis", "瓦利斯和富圖納群島時間"}, {"Yakutsk", "雅庫次克標準時間"}, {"Yekaterinburg", "葉卡捷琳堡標準時間"}, {"Yukon", "育空地區時間"}},
		{{"Acre", "艾克夏令時間"}, {"Alaska", "阿拉斯加夏令時間"}, {"Almaty", "阿拉木圖夏令時間"}, {"Amazon", "亞馬遜夏令時間"}, {"America_Central", "中部夏令時間"}, {"America_Eastern", "東部夏令時間"}, {"America_Mountain", "山區夏令時間"}, {"America_Pacific", "太平洋夏令時間"}, {"Anadyr", "阿那底河夏令時間"}, {"Apia", "阿皮亞夏令時間"}, {"Aqtau", "阿克陶夏令時間"}, {"Aqtobe", "阿克托比夏令時間"}, {"Arabian", "阿拉伯夏令時間"}, {"Argentina", "阿根廷夏令時間"}, {"Argentina_Western", "阿根廷西部夏令時間"}, {"Armenia", "亞美尼亞夏令時間"}, {"Atlantic", "大西洋夏令時間"}, {"Australia_Central", "澳洲中部夏令時間"}, {"Australia_CentralWestern", "澳洲中西部夏令時間"}, {"Australia_Eastern", "澳洲東部夏令時間"}, {"Australia_Western", "澳洲西部夏令時間"}, {"Azerbaijan", "亞塞拜然夏令時間"}, {"Azores", "亞速爾群島夏令時間"}, {"Bangladesh", "孟加拉夏令時間"}, {"Brasilia", "巴西利亞夏令時間"}, {"Cape_Verde", "維德角夏令時間"}, {"Chatham", "查坦群島夏令時間"}, {"Chile", "智利夏令時間"}, {"China", "中國夏令時間"}, {"Colombia", "哥倫比亞夏令時間"}, {"Cook", "庫克群島半夏令時間"}, {"Cuba", "古巴夏令時間"}, {"Easter", "復活節島夏令時間"}, {"Europe_Central", "中歐夏令時間"}, {"Europe_Eastern", "東歐夏令時間"}, {"Europe_Western", "西歐夏令時間"}, {"Falkland", "福克蘭群島夏令時間"}, {"Fiji", "斐濟夏令時間"}, {"Georgia", "喬治亞夏令時間"}, {"Greenland_Eastern", "格陵蘭東部夏令時間"}, {"Greenland_Western", "格陵蘭西部夏令時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申夏令時間"}, {"Hong_Kong", "香港夏令時間"}, {"Hovd", "科布多夏令時間"}, {"Iran", "伊朗夏令時間"}, {"Irkutsk", "伊爾庫次克夏令時間"}, {"Israel", "以色列夏令時間"}, {"Japan", "日本夏令時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克日光節約時間"}, {"Korea", "韓國夏令時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克夏令時間"}, {"Lord_Howe", "豪勳爵島夏令時間"}, {"Macau", "澳門夏令時間"}, {"Magadan", "馬加丹夏令時間"}, {"Mauritius", "模里西斯夏令時間"}, {"Mexico_Pacific", "墨西哥太平洋夏令時間"}, {"Mongolia", "烏蘭巴托夏令時間"}, {"Moscow", "莫斯科夏令時間"}, {"New_Caledonia", "新喀里多尼亞群島夏令時間"}, {"New_Zealand", "紐西蘭夏令時間"}, {"Newfoundland", "紐芬蘭夏令時間"}, {"Norfolk", "諾福克島夏令時間"}, {"Noronha", "費爾南多 - 迪諾羅尼亞夏令時間"}, {"Novosibirsk", "新西伯利亞夏令時間"}, {"Omsk", "鄂木斯克夏令時間"}, {"Pakistan", "巴基斯坦夏令時間"}, {"Paraguay", "巴拉圭夏令時間"}, {"Peru", "秘魯夏令時間"}, {"Philippines", "菲律賓夏令時間"}, {"Pierre_Miquelon", "聖皮埃與密克隆群島夏令時間"}, {"Qyzylorda", "克孜勒奧爾達夏令時間"}, {"Sakhalin", "庫頁島夏令時間"}, {"Samara", "薩馬拉夏令時間"}, {"Samoa", "薩摩亞夏令時間"}, {"Taipei", "台北夏令時間"}, {"Tonga", "東加夏令時間"}, {"Turkey", "土耳其夏令時間"}, {"Turkmenistan", "土庫曼夏令時間"}, {"Uruguay", "烏拉圭夏令時間"}, {"Uzbekistan", "烏茲別克夏令時間"}, {"Vanuatu", "萬那杜夏令時間"}, {"Vladivostok", "海參崴夏令時間"}, {"Volgograd", "伏爾加格勒夏令時間"}, {"Yakutsk", "雅庫次克夏令時間"}, {"Yekaterinburg", "葉卡捷琳堡夏令時間"}},
	},
	{
		{{"GMT", "Gwzlinzveihci Byauhcunj Sizgenh"}},
		{},
	},
	{
		{{"Acre", "艾克標準時間"}, {"Afghanistan", "阿富汗時間"}, {"Africa_Central", "中非時間"}, {"Africa_Eastern", "東非時間"}, {"Africa_Southern", "南非時間"}, {"Africa_Western", "西非時間"}, {"Alaska", "阿拉斯加標準時間"}, {"Almaty", "阿拉木圖標準時間"}, {"Amazon", "亞馬遜標準時間"}, {"America_Central", "北美中部標準時間"}, {"America_Eastern", "北美東部標準時間"}, {"America_Mountain", "北美山區標準時間"}, {"America_Pacific", "北美太平洋標準時間"}, {"Anadyr", "阿那底河標準時間"}, {"Apia", "阿皮亞標準時間"}, {"Aqtau", "阿克陶標準時間"}, {"Aqtobe", "阿克托比標準時間"}, {"Arabian", "阿拉伯標準時間"}, {"Argentina", "阿根廷標準時間"}, {"Argentina_Western", "阿根廷西部標準時間"}, {"Armenia", "亞美尼亞標準時間"}, {"Atlantic", "大西洋標準時間"}, {"Australia_Central", "澳洲中部標準時間"}, {"Australia_CentralWestern", "澳洲中西部標準時間"}, {"Australia_Eastern", "澳洲東部標準時間"}, {"Australia_Western", "澳洲西部標準時間"}, {"Azerbaijan", "亞塞拜疆標準時間"}, {"Azores", "亞速爾群島標準時間"}, {"Bangladesh", "孟加拉標準時間"}, {"Bhutan", "不丹時間"}, {"Bolivia", "玻利維亞時間"}, {"Brasilia", "巴西利亞標準時間"}, {"Brunei", "汶萊時間"}, {"Cape_Verde", "佛得角標準時間"}, {"Casey", "凱西站時間"}, {"Chamorro", "查莫洛時間"}, {"Chatham", "查坦群島標準時間"}, {"Chile", "智利標準時間"}, {"China", "中國標準時間"}, {"Christmas", "聖誕島時間"}, {"Cocos", "可可斯群島時間"}, {"Colombia", "哥倫比亞標準時間"}, {"Cook", "庫克群島標準時間"}, {"Cuba", "古巴標準時間"}, {"Davis", "戴維斯時間"}, {"DumontDUrville", "迪蒙迪維爾時間"}, {"East_Timor", "東帝汶時間"}, {"Easter", "復活節島標準時間"}, {"Ecuador", "厄瓜多爾時間"}, {"Europe_Central", "中歐標準時間"}, {"Europe_Eastern", "東歐標準時間"}, {"Europe_Further_Eastern", "歐洲遠東時間"}, {"Europe_Western", "西歐標準時間"}, {"Falkland", "福克蘭群島標準時間"}, {"Fiji", "斐濟標準時間"}, {"French_Guiana", "法屬圭亞那時間"}, {"French_Southern", "法國南方及南極時間"}, {"GMT", "格林威治標準時間"}, {"Galapagos", "加拉帕戈群島時間"}, {"Gambier", "甘比爾群島時間"}, {"Georgia", "格魯吉亞標準時間"}, {"Gilbert_Islands", "吉爾伯特群島時間"}, {"Greenland_Eastern", "格陵蘭東部標準時間"}, {"Greenland_Western", "格陵蘭西部標準時間"}, {"Guam", "關島標準時間"}, {"Gulf", "波斯灣海域時間"}, {"Guyana", "圭亞那時間"}, {"Hawaii", "夏威夷-阿留申標準時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申標準時間"}, {"Hong_Kong", "香港標準時間"}, {"Hovd", "科布多標準時間"}, {"India", "印度時間"}, {"Indian_Ocean", "印度洋時間"}, {"Indochina", "中南半島時間"}, {"Indonesia_Central", "印尼中部時間"}, {"Indonesia_Eastern", "印尼東部時間"}, {"Indonesia_Western", "印尼西部時間"}, {"Iran", "伊朗標準時間"}, {"Irkutsk", "伊爾庫茨克標準時間"}, {"Israel", "以色列標準時間"}, {"Japan", "日本標準時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克標準時間"}, {"Kazakhstan", "哈薩克時間"}, {"Kazakhstan_Eastern", "東哈薩克時間"}, {"Kazakhstan_Western", "西哈薩克時間"}, {"Korea", "韓國標準時間"}, {"Kosrae", "科斯雷時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克標準時間"}, {"Kyrgystan", "吉爾吉斯時間"}, {"Lanka", "蘭卡時間"}, {"Line_Islands", "萊恩群島時間"}, {"Lord_Howe", "豪勳爵島標準時間"}, {"Macau", "澳門標準時間"}, {"Magadan", "馬加丹標準時間"}, {"Malaysia", "馬來西亞時間"}, {"Maldives", "馬爾代夫時間"}, {"Marquesas", "馬克薩斯時間"}, {"Marshall_Islands", "馬紹爾群島時間"}, {"Mauritius", "毛里裘斯標準時間"}, {"Mawson", "莫森時間"}, {"Mexico_Pacific", "墨西哥太平洋標準時間"}, {"Mongolia", "烏蘭巴托標準時間"}, {"Moscow", "莫斯科標準時間"}, {"Myanmar", "緬甸時間"}, {"Nauru", "瑙魯時間"}, {"Nepal", "尼泊爾時間"}, {"New_Caledonia", "新喀里多尼亞標準時間"}, {"New_Zealand", "紐西蘭標準時間"}, {"Newfoundland", "紐芬蘭標準時間"}, {"Niue", "紐埃島時間"}, {"Norfolk", "諾福克島標準時間"}, {"Noronha", "費爾南多迪諾羅尼亞標準時間"}, {"North_Mariana", "北馬里亞納群島時間"}, {"Novosibirsk", "新西伯利亞標準時間"}, {"Omsk", "鄂木斯克標準時間"}, {"Pakistan", "巴基斯坦標準時間"}, {"Palau", "帛琉時間"}, {"Papua_New_Guinea", "巴布亞新畿內亞時間"}, {"Paraguay", "巴拉圭標準時間"}, {"Peru", "秘魯標準時間"}, {"Philippines", "菲律賓標準時間"}, {"Phoenix_Islands", "鳳凰群島時間"}, {"Pierre_Miquelon", "聖皮埃與密克隆群島標準時間"}, {"Pitcairn", "皮特康時間"}, {"Ponape", "波納佩時間"}, {"Pyongyang", "平壤時間"}, {"Qyzylorda", "克孜勒奧爾達標準時間"}, {"Reunion", "留尼旺時間"}, {"Rothera", "羅瑟拉時間"}, {"Sakhalin", "庫頁島標準時間"}, {"Samara", "薩馬拉標準時間"}, {"Samoa", "薩摩亞標準時間"}, {"Seychelles", "塞舌爾時間"}, {"Singapore", "新加坡時間"}, {"Solomon", "所羅門群島時間"}, {"South_Georgia", "南喬治亞時間"}, {"Suriname", "蘇里南時間"}, {"Syowa", "昭和基地時間"}, {"Tahiti", "大溪地時間"}, {"Taipei", "台北標準時間"}, {"Tajikistan", "塔吉克時間"}, {"Tokelau", "托克勞群島時間"}, {"Tonga", "湯加標準時間"}, {"Truk", "楚克島時間"}, {"Turkey", "土耳其標準時間"}, {"Turkmenistan", "土庫曼標準時間"}, {"Tuvalu", "圖瓦盧時間"}, {"Uruguay", "烏拉圭標準時間"}, {"Uzbekistan", "烏茲別克標準時間"}, {"Vanuatu", "瓦努阿圖標準時間"}, {"Venezuela", "委內瑞拉時間"}, {"Vladivostok", "海參崴標準時間"}, {"Volgograd", "伏爾加格勒標準時間"}, {"Vostok", "沃斯托克時間"}, {"Wake", "威克島時間"}, {"Wallis", "瓦利斯和富圖納群島時間"}, {"Yakutsk", "雅庫茨克標準時間"}, {"Yekaterinburg", "葉卡捷琳堡標準時間"}, {"Yukon", "育空地區時間"}},
		{{"Acre", "艾克夏令時間"}, {"Alaska", "阿拉斯加夏令時間"}, {"Almaty", "阿拉木圖夏令時間"}, {"Amazon", "亞馬遜夏令時間"}, {"America_Central", "北美中部夏令時間"}, {"America_Eastern", "北美東部夏令時間"}, {"America_Mountain", "北美山區夏令時間"}, {"America_Pacific", "北美太平洋夏令時間"}, {"Anadyr", "阿那底河夏令時間"}, {"Apia", "阿皮亞夏令時間"}, {"Aqtau", "阿克陶夏令時間"}, {"Aqtobe", "阿克托比夏令時間"}, {"Arabian", "阿拉伯夏令時間"}, {"Argentina", "阿根廷夏令時間"}, {"Argentina_Western", "阿根廷西部夏令時間"}, {"Armenia", "亞美尼亞夏令時間"}, {"Atlantic", "大西洋夏令時間"}, {"Australia_Central", "澳洲中部夏令時間"}, {"Australia_CentralWestern", "澳洲中西部夏令時間"}, {"Australia_Eastern", "澳洲東部夏令時間"}, {"Australia_Western", "澳洲西部夏令時間"}, {"Azerbaijan", "亞塞拜疆夏令時間"}, {"Azores", "亞速爾群島夏令時間"}, {"Bangladesh", "孟加拉夏令時間"}, {"Brasilia", "巴西利亞夏令時間"}, {"Cape_Verde", "佛得角夏令時間"}, {"Chatham", "查坦群島夏令時間"}, {"Chile", "智利夏令時間"}, {"China", "中國夏令時間"}, {"Colombia", "哥倫比亞夏令時間"}, {"Cook", "庫克群島半夏令時間"}, {"Cuba", "古巴夏令時間"}, {"Easter", "復活節島夏令時間"}, {"Europe_Central", "中歐夏令時間"}, {"Europe_Eastern", "東歐夏令時間"}, {"Europe_Western", "西歐夏令時間"}, {"Falkland", "福克蘭群島夏令時間"}, {"Fiji", "斐濟夏令時間"}, {"Georgia", "格魯吉亞夏令時間"}, {"Greenland_Eastern", "格陵蘭東部夏令時間"}, {"Greenland_Western", "格陵蘭西部夏令時間"}, {"Hawaii_Aleutian", "夏威夷-阿留申夏令時間"}, {"Hong_Kong", "香港夏令時間"}, {"Hovd", "科布多夏令時間"}, {"Iran", "伊朗夏令時間"}, {"Irkutsk", "伊爾庫茨克夏令時間"}, {"Israel", "以色列夏令時間"}, {"Japan", "日本夏令時間"}, {"Kamchatka", "彼得羅巴甫洛夫斯克日光節約時間"}, {"Korea", "韓國夏令時間"}, {"Krasnoyarsk", "克拉斯諾亞爾斯克夏令時間"}, {"Lord_Howe", "豪勳爵島夏令時間"}, {"Macau", "澳門夏令時間"}, {"Magadan", "馬加丹夏令時間"}, {"Mauritius", "毛里裘斯夏令時間"}, {"Mexico_Pacific", "墨西哥太平洋夏令時間"}, {"Mongolia", "烏蘭巴托夏令時間"}, {"Moscow", "莫斯科夏令時間"}, {"New_Caledonia", "新喀里多尼亞夏令時間"}, {"New_Zealand", "紐西蘭夏令時間"}, {"Newfoundland", "紐芬蘭夏令時間"}, {"Norfolk", "諾福克島夏令時間"}, {"Noronha", "費爾南多迪諾羅尼亞夏令時間"}, {"Novosibirsk", "新西伯利亞夏令時間"}, {"Omsk", "鄂木斯克夏令時間"}, {"Pakistan", "巴基斯坦夏令時間"}, {"Paraguay", "巴拉圭夏令時間"}, {"Peru", "秘魯夏令時間"}, {"Philippines", "菲律賓夏令時間"}, {"Pierre_Miquelon", "聖皮埃與密克隆群島夏令時間"}, {"Qyzylorda", "克孜勒奧爾達夏令時間"}, {"Sakhalin", "庫頁島夏令時間"}, {"Samara", "薩馬拉夏令時間"}, {"Samoa", "薩摩亞夏令時間"}, {"Taipei", "台北夏令時間"}, {"Tonga", "湯加夏令時間"}, {"Turkey", "土耳其夏令時間"}, {"Turkmenistan", "土庫曼夏令時間"}, {"Uruguay", "烏拉圭夏令時間"}, {"Uzbekistan", "烏茲別克夏令時間"}, {"Vanuatu", "瓦努阿圖夏令時間"}, {"Vladivostok", "海參崴夏令時間"}, {"Volgograd", "伏爾加格勒夏令時間"}, {"Yakutsk", "雅庫茨克夏令時間"}, {"Yekaterinburg", "葉卡捷琳堡夏令時間"}},
	},
	{
		{{"Afghanistan", "Isikhathi sase-Afghanistan"}, {"Africa_Central", "Isikhathi sase-Central Africa"}, {"Africa_Eastern", "Isikhathi saseMpumalanga Afrika"}, {"Africa_Southern", "Isikhathi esivamile saseNingizimu Afrika"}, {"Africa_Western", "Isikhathi saseNtshonalanga Afrika"}, {"Alaska", "Isikhathi sase-Alaska esijwayelekile"}, {"Amazon", "Isikhathi sase-Amazon esijwayelekile"}, {"America_Central", "Isikhathi sase-North American Central esijwayelekile"}, {"America_Eastern", "Isikhathi sase-North American East esijwayelekile"}, {"America_Mountain", "Isikhathi sase-North American Mountain esijwayelekile"}, {"America_Pacific", "Isikhathi sase-North American Pacific esijwayelekile"}, {"Anadyr", "esase-Anadyr Standard Time"}, {"Apia", "Isikhathi sase-Apia esivamile"}, {"Arabian", "Isikhathi esivamile sase-Arabian"}, {"Argentina", "Isikhathi sase-Argentina esijwayelekile"}, {"Argentina_Western", "Isikhathi saseNyakatho ne-Argentina esijwayelekile"}, {"Armenia", "Isikhathi esivamile sase-Armenia"}, {"Atlantic", "Isikhathi sase-Atlantic esijwayelekile"}, {"Australia_Central", "Isikhathi sase-Australian Central esivamile"}, {"Australia_CentralWestern", "Isikhathi sase-Australian Central West esivamile"}, {"Australia_Eastern", "Isikhathi esivamile sase-Australian East"}, {"Australia_Western", "Isikhathi sase-Australian Western esivamile"}, {"Azerbaijan", "Isikhathi esivamile sase-Azerbaijan"}, {"Azores", "Isikhathi esijwayelekile sase-Azores"}, {"Bangladesh", "Isikhathi sase-Bangladesh esivamile"}, {"Bhutan", "Isikhathi sase-Bhutan"}, {"Bolivia", "Isikhathi sase-Bolivia"}, {"Brasilia", "Isikhathi sase-Brasilia esijwayelekile"}, {"Brunei", "Isikhathi sase-Brunei Darussalam"}, {"Cape_Verde", "Isikhathi esezingeni sase-Cape Verde"}, {"Chamorro", "Isikhathi esivamile sase-Chamorro"}, {"Chatham", "Isikhathi esivamile sase-Chatham"}, {"Chile", "Isikhathi sase-Chile esijwayelekile"}, {"China", "Isikhathi esivamile sase-China"}, {"Christmas", "Isikhathi sase-Christmas Island"}, {"Cocos", "Isikhathi sase-Cocos Islands"}, {"Colombia", "Isikhathi sase-Colombia esijwayelekile"}, {"Cook", "Isikhathi esivamile sase-Cook Islands"}, {"Cuba", "Isikhathi sase-Cuba esijwayelekile"}, {"Davis", "Isikhathi sase-Davis"}, {"DumontDUrville", "Isikhathi sase-Dumont-d’Urville"}, {"East_Timor", "Isikhathi sase-East Timor"}, {"Easter", "Isikhathi sase-Easter Island esijwayelekile"}, {"Ecuador", "Isikhathi sase-Ecuador"}, {"Europe_Central", "Isikhathi esijwayelekile sase-Central Europe"}, {"Europe_Eastern", "Isikhathi esijwayelekile sase-Eastern Europe"}, {"Europe_Further_Eastern", "Isikhathi sase-Further-eastern Europe"}, {"Europe_Western", "Isikhathi esijwayelekile sase-Western Europe"}, {"Falkland", "Isikhathi sase-Falkland Islands esijwayelekile"}, {"Fiji", "Isikhathi esivamile sase-Fiji"}, {"French_Guiana", "Isikhathi sase-French Guiana"}, {"French_Southern", "Isikhathi sase-French Southern nase-Antarctic"}, {"GMT", "Isikhathi sase-Greenwich Mean"}, {"Galapagos", "Isikhathi sase-Galapagos"}, {"Gambier", "Isikhathi sase-Gambier"}, {"Georgia", "Isikhathi esivamile sase-Georgia"}, {"Gilbert_Islands", "Isikhathi sase-Gilbert Islands"}, {"Greenland_Eastern", "Isikhathi sase-East Greenland esijwayelekile"}, {"Greenland_Western", "Isikhathi sase-West Greenland esijwayelekile"}, {"Gulf", "Isikhathi esivamile sase-Gulf"}, {"Guyana", "Isikhathi sase-Guyana"}, {"Hawaii", "Isikhathi sase-Hawaii-Aleutia esijwayelekile"}, {"Hawaii_Aleutian", "Isikhathi sase-Hawaii-Aleutia esijwayelekile"}, {"Hong_Kong", "Isikhathi esivamile sase-Hong Kong"}, {"Hovd", "Isikhathi Esimisiwe sase-Hovd"}, {"India", "Isikhathi sase-India esivamile"}, {"Indian_Ocean", "Isikhathi sase-Indian Ocean"}, {"Indochina", "Isikhathi sase-Indochina"}, {"Indonesia_Central", "Isikhathi sase-Central Indonesia"}, {"Indonesia_Eastern", "Isikhathi sase-Eastern Indonesia"}, {"Indonesia_Western", "Isikhathi sase-Western Indonesia"}, {"Iran", "Isikhathi sase-Iran esivamile"}, {"Irkutsk", "Isikhathi Esivamile sase-Irkutsk"}, {"Israel", "Isikhathi esivamile sase-Israel"}, {"Japan", "Isikhathi esivamile sase-Japan"}, {"Kamchatka", "esase-Petropavlovsk-Kamchatski Standard Time"}, {"Kazakhstan", "Isikhathi saseKazakhstan"}, {"Kazakhstan_Eastern", "Isikhathi sase-Mpumalanga ne-Kazakhstan"}, {"Kazakhstan_Western", "Isikhathi saseNtshonalanga ne-Kazakhstan"}, {"Korea", "Isikhathi Esivamile sase-Korea"}, {"Kosrae", "Isikhathi sase-Kosrae"}, {"Krasnoyarsk", "Isikhathi Esivamile sase-Krasnoyarsk"}, {"Kyrgystan", "Isikhathi sase-Kyrgystan"}, {"Line_Islands", "Isikhathi sase-Line Islands"}, {"Lord_Howe", "Isikhathi sase-Lord Howe esivamile"}, {"Magadan", "Isikhathi Esivamile sase-Magadan"}, {"Malaysia", "Isikhathi sase-Malaysia"}, {"Maldives", "Isikhathi sase-Maldives"}, {"Marquesas", "Isikhathi sase-Marquesas"}, {"Marshall_Islands", "Isikhathi sase-Marshall Islands"}, {"Mauritius", "Isikhathi esivamile sase-Mauritius"}, {"Mawson", "Isikhathi sase-Mawson"}, {"Mexico_Pacific", "Isikhathi sase-Mexican Pacific esijwayelekile"}, {"Mongolia", "Isikhathi Esimisiwe sase-Ulan Bator"}, {"Moscow", "Isikhathi sase-Moscow esijwayelekile"}, {"Myanmar", "Isikhathi sase-Myanmar"}, {"Nauru", "Isikhathi sase-Nauru"}, {"Nepal", "Isikhathi sase-Nepal"}, {"New_Caledonia", "Isikhathi sase-New Caledonia esivamile"}, {"New_Zealand", "Isikhathi esivamile sase-New Zealand"}, {"Newfoundland", "Isikhathi sase-Newfoundland esijwayelekile"}, {"Niue", "Isikhathi sase-Niue"}, {"Norfolk", "Isikhathi sase-Norfolk Islands esivamile"}, {"Noronha", "Isikhathi sase-Fernando de Noronha esivamile"}, {"Novosibirsk", "Isikhathi Esivamile sase-Novosibirsk"}, {"Omsk", "Isikhathi Esivamile sase-Omsk"}, {"Pakistan", "Isikhathi sase-Pakistan esivamile"}, {"Palau", "Isikhathi sase-Palau"}, {"Papua_New_Guinea", "Isikhathi sase-Papua New Guinea"}, {"Paraguay", "Isikhathi sase-Paraguay esivamile"}, {"Peru", "Isikhathi sase-Peru esivamile"}, {"Philippines", "Isikhathi esivamile sase-Philippine"}, {"Phoenix_Islands", "Isikhathi sase-Phoenix Islands"}, {"Pierre_Miquelon", "Iikhathi sase-Saint Pierre nase-Miquelon esijwayelekile"}, {"Pitcairn", "Isikhathi sase-Pitcairn"}, {"Ponape", "Isikhathi sase-Ponape"}, {"Pyongyang", "Isikhathi sase-Pyongyang"}, {"Reunion", "Isikhathi sase-Reunion"}, {"Rothera", "Isikhathi sase-Rothera"}, {"Sakhalin", "Isikhathi Esivamile sase-Sakhalin"}, {"Samara", "esase-Samara Standard Time"}, {"Samoa", "Isikhathi sase-Samoa esivamile"}, {"Seychelles", "Isikhathi sase-Seychelles"}, {"Singapore", "Isikhathi esivamile sase-Singapore"}, {"Solomon", "Isikhathi sase-Solomon Islands"}, {"South_Georgia", "Isikhathi sase-South Georgia"}, {"Suriname", "Isikhathi sase-Suriname"}, {"Syowa", "Isikhathi sase-Syowa"}, {"Tahiti", "Isikhathi sase-Tahiti"}, {"Taipei", "Isikhathi esivamile sase-Taipei"}, {"Tajikistan", "Isikhathi sase-Tajikistan"}, {"Tokelau", "Isikhathi sase-Tokelau"}, {"Tonga", "Isikhathi sase-Tonga esivamile"}, {"Truk", "Isikhathi sase-Chuuk"}, {"Turkmenistan", "Isikhathi esivamile sase-Turkmenistan"}, {"Tuvalu", "Isikhathi sase-Tuvalu"}, {"Uruguay", "Isikhathi sase-Uruguay esijwayelekile"}, {"Uzbekistan", "Isikhathi esivamile sase-Uzbekistan"}, {"Vanuatu", "Isikhathi sase-Vanuatu esijwayelekile"}, {"Venezuela", "Isikhathi sase-Venezuela"}, {"Vladivostok", "Isikhathi Esivamile sase-Vladivostok"}, {"Volgograd", "Isikhathi Esivamile sase-Volgograd"}, {"Vostok", "Isikhathi sase-Vostok"}, {"Wake", "Isikhathi sase-Wake Island"}, {"Wallis", "Isikhathi sase-Wallis nase-Futuna"}, {"Yakutsk", "Isikhathi Esivamile sase-Yakutsk"}, {"Yekaterinburg", "Isikhathi Esivamile sase-Yekaterinburg"}, {"Yukon", "Yukon Time"}},
		{{"Alaska", "Isikhathi sase-Alaska sasemini"}, {"Amazon", "Isikhathi sase-Amazon sasehlobo"}, {"America_Central", "Isikhathi sase-North American Central sasemini"}, {"America_Eastern", "Isikhathi sase-North American East sasemini"}, {"America_Mountain", "Isikhathi sase-North American Mountain sasemini"}, {"America_Pacific", "Isikhathi sase-North American Pacific sasemini"}, {"Anadyr", "esase-Anadyr Summer Time"}, {"Apia", "Isikhathi sase-Apia sasemini"}, {"Arabian", "Isikhathi semini sase-Arabian"}, {"Argentina", "Isikhathi sase-Argentina sasehlobo"}, {"Argentina_Western", "Isikhathi saseNyakatho ne-Argentina sasehlobo"}, {"Armenia", "Isikhathi sehlobo sase-Armenia"}, {"Atlantic", "Isikhathi sase-Atlantic sasemini"}, {"Australia_Central", "Isikhathi sase-Australian Central sasemini"}, {"Australia_CentralWestern", "Isikhathi sasemini sase-Australian Central West"}, {"Australia_Eastern", "Isikhathi sasemini sase-Australian East"}, {"Australia_Western", "Isikhathi sase-Australian Western sasemini"}, {"Azerbaijan", "Isikhathi sehlobo sase-Azerbaijan"}, {"Azores", "Isikhathi sasehlobo sase-Azores"}, {"Bangladesh", "Isikhathi sase-Bangladesh sasehlobo"}, {"Brasilia", "Isikhathi sase-Brasilia sasehlobo"}, {"Cape_Verde", "Isikhathi sehlobo sase-Cape Verde"}, {"Chatham", "Isikhathi sasemini sase-Chatham"}, {"Chile", "Isikhathi sase-Chile sasehlobo"}, {"China", "Isikhathi semini sase-China"}, {"Colombia", "Isikhathi sase-Colombia sasehlobo"}, {"Cook", "Isikhathi esiyingxenye yasehlobo sase-Cook Islands"}, {"Cuba", "Isikhathi sase-Cuba sasemini"}, {"Easter", "Isikhathi sase-Easter Island sasehlobo"}, {"Europe_Central", "Isikhathi sasehlobo sase-Central Europe"}, {"Europe_Eastern", "Isikhathi sasehlobo sase-Eastern Europe"}, {"Europe_Western", "Isikhathi sasehlobo sase-Western Europe"}, {"Falkland", "Isikhathi sase-Falkland Islands sasehlobo"}, {"Fiji", "Isikhathi sehlobo sase-Fiji"}, {"Georgia", "Isikhathi sehlobo sase-Georgia"}, {"Greenland_Eastern", "Isikhathi sase-East Greenland sasemini"}, {"Greenland_Western", "Isikhathi sase-West Greenland sasehlobo"}, {"Hawaii_Aleutian", "Isikhathi sase-Hawaii-Aleutia sasemini"}, {"Hong_Kong", "Isikhathi sehlobo sase-Hong Kong"}, {"Hovd", "Isikhathi sehlobo e-Hovd"}, {"Iran", "Isikhathi sase-Iran sasemini"}, {"Irkutsk", "Isikhathi sasehlobo e-Irkutsk"}, {"Israel", "Isikhathi sasemini sakwa-Israel"}, {"Japan", "Isikhathi semini sase-Japan"}, {"Kamchatka", "esase-Petropavlovsk-Kamchatski Summer Time"}, {"Korea", "Isikhathi semini sase-Korea"}, {"Krasnoyarsk", "Isikhathi sasehlobo e-Krasnoyarsk"}, {"Lord_Howe", "Isikhathi sase-Lord Howe sasemini"}, {"Magadan", "Isikhathi sasehlobo e-Magadan"}, {"Mauritius", "Isikhathi sehlobo sase-Mauritius"}, {"Mexico_Pacific", "Isikhathi sase-Mexican Pacific sasemini"}, {"Mongolia", "Isikhathi sehlobo e-Ulan Bator"}, {"Moscow", "Isikhathi sasehlobo e-Moscow"}, {"New_Caledonia", "Isikhathi sase-New Caledonia sasehlobo"}, {"New_Zealand", "Isikhathi sasemini sase-New Zealand"}, {"Newfoundland", "Isikhathi sase-Newfoundland sasemini"}, {"Norfolk", "Isikhathi sase-Norfolk Islands sasehlobo"}, {"Noronha", "Isikhathi sase-Fernando de Noronha sasehlobo"}, {"Novosibirsk", "Isikhathi sasehlobo sase-Novosibirsk"}, {"Omsk", "Isikhathi sasehlobo sase-Omsk"}, {"Pakistan", "Isikhathi sase-Pakistan sasehlobo"}, {"Paraguay", "Isikhathi sase-Paraguay sasehlobo"}, {"Peru", "Isikhathi sase-Peru sasehlobo"}, {"Philippines", "Isikhathi sehlobo sase-Philippine"}, {"Pierre_Miquelon", "Isikhathi sase-Saint Pierre nase-Miquelon sasemini"}, {"Sakhalin", "Isikhathi sasehlobo e-Sakhalin"}, {"Samara", "esase-Samara Summer Time"}, {"Samoa", "Isikhathi sase-Samoa sasemini"}, {"Taipei", "Isikhathi semini sase-Taipei"}, {"Tonga", "Isikhathi sase-Tonga sasehlobo"}, {"Turkmenistan", "Isikhathi sehlobo sase-Turkmenistan"}, {"Uruguay", "Isikhathi sase-Uruguay sasehlobo"}, {"Uzbekistan", "Isikhathi sehlobo sase-Uzbekistan"}, {"Vanuatu", "Isikhathi sase-Vanuatu sasehlobo"}, {"Vladivostok", "Isikhathi sasehlobo e-Vladivostok"}, {"Volgograd", "Isikhathi sase-Volgograd sasehlobo"}, {"Yakutsk", "Isikhathi sasehlobo e-Yakutsk"}, {"Yekaterinburg", "Isikhathi sasehlobo e-Yekaterinburg"}},
	},
}

var _locales = [...]localeEntry{
	{"aa", 0, 1, 2, 0, 1, 1, 2, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 1, 0},
	{"aa_dj", 0, 1, 3, 0, 1, 1, 2, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 2, 0},
	{"aa_er", 0, 1, 2, 0, 1, 1, 2, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 3, 0},
	{"aa_et", 0, 1, 2, 0, 1, 1, 2, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 1, 0},
	{"ab", 4, 5, 6, 3, 1, 1, 4, 2, 17, 3, 4, 5, 6, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 0, 4, 1},
	{"ab_ge", 4, 5, 6, 3, 1, 1, 4, 2, 17, 3, 4, 5, 6, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 0, 4, 1},
	{"af", 7, 1, 8, 5, 1, 1, 6, 2, 17, 3, 4, 5, 6, 7, 28, 9, 29, 30, 23, 24, 25, 26, 16, 31, 5, 2},
	{"af_na", 7, 1, 8, 5, 1, 1, 6, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 30, 12, 13, 14, 15, 16, 31, 5, 2},
	{"af_za", 7, 1, 8, 5, 1, 1, 6, 2, 17, 3, 4, 5, 6, 7, 28, 9, 29, 30, 23, 24, 25, 26, 16, 31, 5, 2},
	{"agq", 9, 1, 10, 7, 1, 1, 8, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"agq_cm", 9, 1, 10, 7, 1, 1, 8, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"ak", 1, 11, 12, 9, 1, 9, 10, 1, 2, 3, 4, 5, 6, 7, 38, 39, 40, 36, 12, 13, 14, 15, 16, 31, 6, 3},
	{"ak_gh", 1, 11, 12, 9, 1, 9, 10, 1, 2, 3, 4, 5, 6, 7, 38, 39, 40, 36, 12, 13, 14, 15, 16, 31, 6, 3},
	{"am", 13, 1, 14, 11, 1, 12, 13, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 11, 41, 42, 43, 44, 16, 31, 7, 4},
	{"am_et", 13, 1, 14, 11, 1, 12, 13, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 11, 41, 42, 43, 44, 16, 31, 7, 4},
	{"an", 15, 1, 16, 14, 1, 1, 15, 2, 1, 3, 4, 5, 6, 7, 45, 46, 34, 36, 47, 48, 49, 50, 27, 31, 8, 5},
	{"an_es", 15, 1, 16, 14, 1, 1, 15, 2, 1, 3, 4, 5, 6, 7, 45, 46, 34, 36, 47, 48, 49, 50, 27, 31, 8, 5},
	{"ann", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ann_ng", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"apc", 1, 1, 18, 1, 1, 1, 17, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 9, 0},
	{"apc_sy", 1, 1, 18, 1, 1, 1, 17, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 9, 0},
	{"ar", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_001", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_ae", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 11, 6},
	{"ar_bh", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_dj", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 12, 6},
	{"ar_dz", 1, 1, 20, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_eg", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_eh", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_er", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 13, 6},
	{"ar_il", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 47, 48, 49, 50, 61, 62, 10, 6},
	{"ar_iq", 21, 1, 22, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_jo", 1, 1, 22, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_km", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 23, 24, 25, 26, 61, 62, 10, 6},
	{"ar_kw", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_lb", 1, 1, 22, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 14, 6},
	{"ar_ly", 1, 1, 19, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_ma", 1, 1, 23, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 23, 24, 25, 26, 61, 62, 10, 6},
	{"ar_mr", 1, 1, 24, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_om", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_ps", 1, 1, 22, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_qa", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_sa", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 63, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_sd", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_so", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 63, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 15, 6},
	{"ar_ss", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 16, 6},
	{"ar_sy", 1, 1, 22, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_td", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_tn", 1, 1, 20, 1, 1, 18, 19, 2, 1, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"ar_ye", 1, 1, 19, 1, 1, 18, 19, 1, 2, 55, 56, 5, 57, 7, 58, 33, 59, 60, 41, 42, 43, 44, 61, 62, 10, 6},
	{"arn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"arn_cl", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"as", 25, 1, 26, 20, 1, 1, 21, 1, 2, 3, 4, 5, 6, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 31, 17, 7},
	{"as_in", 25, 1, 26, 20, 1, 1, 21, 1, 2, 3, 4, 5, 6, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 31, 17, 7},
	{"asa", 27, 1, 28, 22, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 18, 0},
	{"asa_tz", 27, 1, 28, 22, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 18, 0},
	{"ast", 29, 1, 30, 24, 1, 25, 26, 2, 1, 3, 4, 5, 6, 7, 45, 46, 34, 36, 23, 24, 25, 26, 27, 0, 19, 8},
	{"ast_es", 29, 1, 30, 24, 1, 25, 26, 2, 1, 3, 4, 5, 6, 7, 45, 46, 34, 36, 23, 24, 25, 26, 27, 0, 19, 8},
	{"az", 31, 1, 32, 27, 1, 28, 29, 2, 1, 3, 4, 5, 6, 7, 75, 33, 34, 22, 23, 24, 25, 26, 27, 0, 20, 9},
	{"az_arab", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"az_arab_iq", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"az_arab_ir", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"az_arab_tr", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"az_cyrl", 33, 1, 34, 30, 1, 1, 31, 2, 1, 3, 4, 5, 6, 7, 75, 33, 34, 22, 23, 24, 25, 26, 27, 0, 0, 0},
	{"az_cyrl_az", 33, 1, 34, 30, 1, 1, 31, 2, 1, 3, 4, 5, 6, 7, 75, 33, 34, 22, 23, 24, 25, 26, 27, 0, 0, 0},
	{"az_latn", 31, 1, 32, 27, 1, 28, 29, 2, 1, 3, 4, 5, 6, 7, 75, 33, 34, 22, 23, 24, 25, 26, 27, 0, 20, 9},
	{"az_latn_az", 31, 1, 32, 27, 1, 28, 29, 2, 1, 3, 4, 5, 6, 7, 75, 33, 34, 22, 23, 24, 25, 26, 27, 0, 20, 9},
	{"ba", 35, 36, 37, 32, 33, 34, 35, 1, 17, 3, 4, 5, 6, 7, 76, 77, 78, 22, 23, 24, 25, 26, 27, 27, 21, 10},
	{"ba_ru", 35, 36, 37, 32, 33, 34, 35, 1, 17, 3, 4, 5, 6, 7, 76, 77, 78, 22, 23, 24, 25, 26, 27, 27, 21, 10},
	{"bal", 38, 1, 39, 36, 1, 1, 37, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 79, 80, 81, 82, 54, 0, 22, 11},
	{"bal_arab", 38, 1, 39, 36, 1, 1, 37, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 79, 80, 81, 82, 54, 0, 22, 11},
	{"bal_arab_pk", 38, 1, 39, 36, 1, 1, 37, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 79, 80, 81, 82, 54, 0, 22, 11},
	{"bal_latn", 40, 1, 41, 38, 39, 1, 40, 1, 2, 3, 4, 5, 6, 7, 83, 66, 35, 36, 84, 85, 86, 87, 54, 0, 23, 12},
	{"bal_latn_pk", 40, 1, 41, 38, 39, 1, 40, 1, 2, 3, 4, 5, 6, 7, 83, 66, 35, 36, 84, 85, 86, 87, 54, 0, 23, 12},
	{"bas", 42, 1, 43, 41, 1, 1, 42, 2, 17, 3, 4, 5, 6, 18, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"bas_cm", 42, 1, 43, 41, 1, 1, 42, 2, 17, 3, 4, 5, 6, 18, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"be", 44, 1, 45, 43, 1, 1, 44, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 91, 92, 24, 25, 26, 27, 0, 24, 13},
	{"be_by", 44, 1, 45, 43, 1, 1, 44, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 91, 92, 24, 25, 26, 27, 0, 24, 13},
	{"be_tarask", 46, 1, 47, 45, 1, 1, 46, 2, 17, 3, 4, 5, 6, 18, 88, 89, 93, 91, 92, 24, 25, 26, 27, 0, 25, 14},
	{"bem", 48, 1, 49, 1, 1, 1, 47, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 26, 0},
	{"bem_zm", 48, 1, 49, 1, 1, 1, 47, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 26, 0},
	{"bew", 50, 1, 51, 48, 1, 1, 49, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 94, 95, 96, 97, 54, 16, 0, 15},
	{"bew_id", 50, 1, 51, 48, 1, 1, 49, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 94, 95, 96, 97, 54, 16, 0, 15},
	{"bez", 52, 1, 53, 50, 1, 1, 51, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"bez_tz", 52, 1, 53, 50, 1, 1, 51, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"bg", 54, 1, 55, 52, 1, 1, 53, 2, 17, 3, 4, 5, 6, 7, 88, 89, 98, 99, 100, 101, 49, 50, 27, 102, 27, 16},
	{"bg_bg", 54, 1, 55, 52, 1, 1, 53, 2, 17, 3, 4, 5, 6, 7, 88, 89, 98, 99, 100, 101, 49, 50, 27, 102, 27, 16},
	{"bgc", 1, 1, 56, 1, 1, 1, 54, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 0, 17},
	{"bgc_in", 1, 1, 56, 1, 1, 1, 54, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 0, 17},
	{"bgn", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bgn_ae", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bgn_af", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bgn_ir", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bgn_om", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bgn_pk", 1, 1, 17, 1, 1, 1, 16, 103, 104, 3, 63, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 28, 18},
	{"bho", 1, 1, 57, 1, 1, 1, 55, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 16, 0, 0, 17},
	{"bho_in", 1, 1, 57, 1, 1, 1, 55, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 16, 0, 0, 17},
	{"blo", 58, 1, 59, 56, 1, 57, 58, 2, 17, 3, 4, 5, 6, 105, 106, 52, 107, 108, 23, 24, 25, 26, 109, 0, 0, 19},
	{"blo_bj", 58, 1, 59, 56, 1, 57, 58, 2, 17, 3, 4, 5, 6, 105, 106, 52, 107, 108, 23, 24, 25, 26, 109, 0, 0, 19},
	{"blt", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"blt_vn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"bm", 60, 1, 61, 59, 1, 1, 60, 1, 2, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"bm_ml", 60, 1, 61, 59, 1, 1, 60, 1, 2, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"bm_nkoo", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"bm_nkoo_ml", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"bn", 62, 1, 63, 61, 1, 62, 63, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 110, 111, 17, 20},
	{"bn_bd", 62, 1, 63, 61, 1, 62, 63, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 110, 111, 17, 20},
	{"bn_in", 64, 1, 63, 61, 1, 64, 63, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 112, 113, 29, 20},
	{"bo", 65, 1, 66, 65, 1, 1, 66, 1, 2, 3, 4, 5, 6, 7, 114, 115, 116, 30, 23, 24, 25, 26, 54, 0, 30, 0},
	{"bo_cn", 65, 1, 66, 65, 1, 1, 66, 1, 2, 3, 4, 5, 6, 7, 114, 115, 116, 30, 23, 24, 25, 26, 54, 0, 30, 0},
	{"bo_in", 65, 1, 66, 65, 1, 1, 66, 1, 2, 3, 4, 5, 6, 7, 114, 115, 116, 30, 41, 42, 43, 44, 54, 0, 0, 0},
	{"bqi", 1, 1, 67, 1, 1, 1, 67, 1, 2, 117, 4, 5, 57, 7, 51, 52, 53, 30, 23, 24, 25, 26, 118, 119, 0, 21},
	{"bqi_ir", 1, 1, 67, 1, 1, 1, 67, 1, 2, 117, 4, 5, 57, 7, 51, 52, 53, 30, 23, 24, 25, 26, 118, 119, 0, 21},
	{"br", 68, 1, 69, 68, 1, 1, 69, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 31, 22},
	{"br_fr", 68, 1, 69, 68, 1, 1, 69, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 31, 22},
	{"brx", 70, 1, 71, 70, 1, 1, 71, 1, 2, 3, 4, 5, 6, 64, 75, 66, 35, 120, 121, 122, 123, 124, 73, 31, 32, 23},
	{"brx_in", 70, 1, 71, 70, 1, 1, 71, 1, 2, 3, 4, 5, 6, 64, 75, 66, 35, 120, 121, 122, 123, 124, 73, 31, 32, 23},
	{"bs", 72, 73, 74, 72, 73, 1, 74, 2, 1, 3, 4, 5, 6, 7, 125, 126, 127, 128, 23, 24, 25, 26, 27, 0, 33, 24},
	{"bs_cyrl", 75, 1, 76, 75, 1, 1, 76, 2, 1, 3, 4, 5, 6, 7, 129, 130, 131, 132, 23, 24, 25, 26, 27, 0, 34, 25},
	{"bs_cyrl_ba", 75, 1, 76, 75, 1, 1, 76, 2, 1, 3, 4, 5, 6, 7, 129, 130, 131, 132, 23, 24, 25, 26, 27, 0, 34, 25},
	{"bs_latn", 72, 73, 74, 72, 73, 1, 74, 2, 1, 3, 4, 5, 6, 7, 125, 126, 127, 128, 23, 24, 25, 26, 27, 0, 33, 24},
	{"bs_latn_ba", 72, 73, 74, 72, 73, 1, 74, 2, 1, 3, 4, 5, 6, 7, 125, 126, 127, 128, 23, 24, 25, 26, 27, 0, 33, 24},
	{"bss", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"bss_cm", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"bua", 1, 1, 77, 1, 77, 1, 78, 2, 17, 3, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 26},
	{"bua_ru", 1, 1, 77, 1, 77, 1, 78, 2, 17, 3, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 26},
	{"byn", 78, 1, 79, 79, 1, 1, 80, 1, 2, 3, 4, 5, 6, 7, 133, 9, 10, 11, 41, 42, 43, 44, 16, 0, 35, 0},
	{"byn_er", 78, 1, 79, 79, 1, 1, 80, 1, 2, 3, 4, 5, 6, 7, 133, 9, 10, 11, 41, 42, 43, 44, 16, 0, 35, 0},
	{"ca", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 36, 27},
	{"ca_ad", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 36, 27},
	{"ca_es", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 36, 27},
	{"ca_es_valencia", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 36, 27},
	{"ca_fr", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 37, 27},
	{"ca_it", 80, 1, 81, 81, 1, 1, 82, 2, 1, 3, 4, 5, 6, 18, 134, 135, 34, 36, 136, 48, 49, 50, 27, 102, 36, 27},
	{"cad", 1, 1, 82, 1, 1, 1, 83, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"cad_us", 1, 1, 82, 1, 1, 1, 83, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"cch", 83, 1, 84, 84, 1, 1, 85, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"cch_ng", 83, 1, 84, 84, 1, 1, 85, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ccp", 85, 1, 86, 86, 1, 1, 87, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 110, 111, 39, 28},
	{"ccp_bd", 85, 1, 86, 86, 1, 1, 87, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 110, 111, 39, 28},
	{"ccp_in", 85, 1, 86, 86, 1, 1, 87, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 110, 111, 39, 28},
	{"ce", 87, 1, 88, 88, 88, 1, 89, 1, 2, 3, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 40, 29},
	{"ce_ru", 87, 1, 88, 88, 88, 1, 89, 1, 2, 3, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 40, 29},
	{"ceb", 89, 1, 90, 90, 1, 1, 91, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 41, 30},
	{"ceb_ph", 89, 1, 90, 90, 1, 1, 91, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 41, 30},
	{"cgg", 91, 1, 92, 92, 1, 1, 93, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 42, 0},
	{"cgg_ug", 91, 1, 92, 92, 1, 1, 93, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 42, 0},
	{"cho", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"cho_us", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"chr", 93, 1, 94, 94, 1, 95, 96, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 31, 29, 31},
	{"chr_us", 93, 1, 94, 94, 1, 95, 96, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 31, 29, 31},
	{"cic", 1, 1, 95, 1, 1, 1, 97, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"cic_us", 1, 1, 95, 1, 1, 1, 97, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"ckb", 1, 1, 96, 1, 1, 98, 99, 1, 2, 3, 4, 5, 57, 7, 51, 142, 53, 30, 41, 42, 43, 44, 54, 0, 43, 0},
	{"ckb_iq", 1, 1, 96, 1, 1, 98, 99, 1, 2, 3, 4, 5, 57, 7, 51, 142, 53, 30, 41, 42, 43, 44, 54, 0, 43, 0},
	{"ckb_ir", 1, 1, 96, 1, 1, 98, 99, 1, 2, 3, 4, 5, 57, 7, 51, 142, 53, 30, 23, 24, 25, 26, 54, 0, 43, 0},
	{"co", 97, 1, 98, 100, 1, 101, 102, 2, 17, 3, 4, 5, 6, 18, 143, 144, 34, 11, 23, 24, 25, 26, 27, 145, 44, 32},
	{"co_fr", 97, 1, 98, 100, 1, 101, 102, 2, 17, 3, 4, 5, 6, 18, 143, 144, 34, 11, 23, 24, 25, 26, 27, 145, 44, 32},
	{"cop", 99, 100, 99, 103, 103, 103, 104, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 33},
	{"cop_eg", 99, 100, 99, 103, 103, 103, 104, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 33},
	{"cs", 101, 1, 102, 105, 1, 1, 106, 2, 17, 3, 4, 5, 6, 18, 146, 147, 148, 22, 149, 48, 49, 50, 27, 0, 45, 34},
	{"cs_cz", 101, 1, 102, 105, 1, 1, 106, 2, 17, 3, 4, 5, 6, 18, 146, 147, 148, 22, 149, 48, 49, 50, 27, 0, 45, 34},
	{"csw", 103, 1, 103, 107, 108, 107, 107, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 35},
	{"csw_ca", 103, 1, 103, 107, 108, 107, 107, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 35},
	{"cu", 104, 1, 105, 109, 1, 1, 110, 2, 17, 3, 4, 5, 6, 18, 150, 52, 53, 151, 23, 24, 25, 26, 27, 0, 46, 36},
	{"cu_ru", 104, 1, 105, 109, 1, 1, 110, 2, 17, 3, 4, 5, 6, 18, 150, 52, 53, 151, 23, 24, 25, 26, 27, 0, 46, 36},
	{"cv", 106, 1, 107, 111, 1, 111, 112, 2, 17, 3, 4, 5, 6, 18, 152, 153, 154, 151, 155, 156, 25, 26, 27, 102, 38, 37},
	{"cv_ru", 106, 1, 107, 111, 1, 111, 112, 2, 17, 3, 4, 5, 6, 18, 152, 153, 154, 151, 155, 156, 25, 26, 27, 102, 38, 37},
	{"cy", 108, 1, 109, 113, 1, 114, 115, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 47, 38},
	{"cy_gb", 108, 1, 109, 113, 1, 114, 115, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 47, 38},
	{"da", 110, 1, 111, 116, 1, 117, 118, 2, 1, 3, 4, 5, 6, 18, 157, 147, 158, 22, 94, 95, 96, 97, 27, 0, 48, 39},
	{"da_dk", 110, 1, 111, 116, 1, 117, 118, 2, 1, 3, 4, 5, 6, 18, 157, 147, 158, 22, 94, 95, 96, 97, 27, 0, 48, 39},
	{"da_gl", 110, 1, 111, 116, 1, 117, 118, 2, 1, 3, 4, 5, 6, 18, 157, 147, 158, 22, 94, 95, 96, 97, 27, 0, 48, 39},
	{"dav", 112, 1, 113, 119, 1, 1, 120, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"dav_ke", 112, 1, 113, 119, 1, 1, 120, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"de", 114, 1, 115, 121, 1, 1, 122, 2, 1, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 50, 40},
	{"de_at", 116, 1, 117, 121, 1, 1, 122, 2, 17, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 54, 0, 50, 40},
	{"de_be", 114, 1, 115, 121, 1, 1, 122, 2, 1, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 50, 40},
	{"de_ch", 114, 1, 115, 121, 1, 123, 122, 1, 161, 3, 4, 5, 6, 7, 159, 147, 160, 22, 23, 24, 25, 26, 162, 0, 51, 41},
	{"de_de", 114, 1, 115, 121, 1, 1, 122, 2, 1, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 50, 40},
	{"de_it", 116, 1, 117, 121, 1, 1, 122, 2, 1, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 50, 40},
	{"de_li", 114, 1, 115, 121, 1, 1, 122, 1, 161, 3, 4, 5, 6, 7, 159, 147, 160, 22, 23, 24, 25, 26, 54, 0, 51, 40},
	{"de_lu", 114, 1, 115, 121, 1, 1, 122, 2, 1, 3, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 52, 40},
	{"dje", 118, 1, 119, 124, 1, 1, 125, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"dje_ne", 118, 1, 119, 124, 1, 1, 125, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"doi", 120, 1, 121, 126, 127, 1, 128, 1, 2, 3, 4, 5, 6, 7, 163, 164, 165, 36, 41, 42, 43, 44, 16, 0, 53, 42},
	{"doi_in", 120, 1, 121, 126, 127, 1, 128, 1, 2, 3, 4, 5, 6, 7, 163, 164, 165, 36, 41, 42, 43, 44, 16, 0, 53, 42},
	{"dsb", 122, 1, 123, 129, 1, 130, 131, 2, 1, 3, 4, 5, 6, 18, 159, 147, 166, 167, 47, 48, 49, 50, 27, 0, 54, 43},
	{"dsb_de", 122, 1, 123, 129, 1, 130, 131, 2, 1, 3, 4, 5, 6, 18, 159, 147, 166, 167, 47, 48, 49, 50, 27, 0, 54, 43},
	{"dua", 124, 1, 125, 132, 1, 1, 133, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"dua_cm", 124, 1, 125, 132, 1, 1, 133, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"dv", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 64, 32, 33, 67, 68, 23, 24, 25, 26, 73, 0, 55, 0},
	{"dv_mv", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 64, 32, 33, 67, 68, 23, 24, 25, 26, 73, 0, 55, 0},
	{"dyo", 126, 1, 127, 134, 1, 1, 135, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"dyo_sn", 126, 1, 127, 134, 1, 1, 135, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"dz", 128, 129, 130, 136, 1, 1, 137, 1, 2, 3, 4, 5, 6, 168, 169, 170, 171, 30, 172, 173, 174, 175, 112, 0, 56, 44},
	{"dz_bt", 128, 129, 130, 136, 1, 1, 137, 1, 2, 3, 4, 5, 6, 168, 169, 170, 171, 30, 172, 173, 174, 175, 112, 0, 56, 44},
	{"ebu", 131, 1, 132, 138, 1, 1, 139, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"ebu_ke", 131, 1, 132, 138, 1, 1, 139, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"ee", 133, 1, 134, 140, 1, 1, 141, 1, 2, 3, 4, 5, 6, 7, 176, 177, 178, 108, 179, 180, 181, 182, 16, 31, 57, 45},
	{"ee_gh", 133, 1, 134, 140, 1, 1, 141, 1, 2, 3, 4, 5, 6, 7, 176, 177, 178, 108, 179, 180, 181, 182, 16, 31, 57, 45},
	{"ee_tg", 133, 1, 134, 140, 1, 1, 141, 1, 2, 3, 4, 5, 6, 7, 176, 177, 178, 108, 23, 24, 25, 26, 16, 31, 57, 45},
	{"el", 135, 1, 136, 142, 1, 143, 144, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 27, 0, 58, 46},
	{"el_cy", 135, 1, 136, 142, 1, 143, 144, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 27, 0, 58, 46},
	{"el_gr", 135, 1, 136, 142, 1, 143, 144, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 27, 0, 58, 46},
	{"el_polyton", 137, 1, 138, 142, 1, 143, 144, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 27, 0, 58, 46},
	{"en", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_001", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_150", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_ae", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_ag", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_ai", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 61, 48},
	{"en_as", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_at", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 11, 23, 24, 25, 26, 54, 54, 60, 48},
	{"en_au", 143, 140, 141, 16, 148, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 62, 49},
	{"en_bb", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 63, 48},
	{"en_be", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 29, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_bi", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 31, 64, 47},
	{"en_bm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 65, 48},
	{"en_bs", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 66, 48},
	{"en_bw", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 183, 9, 29, 11, 23, 24, 25, 26, 16, 31, 67, 48},
	{"en_bz", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 183, 9, 10, 11, 23, 24, 25, 26, 16, 31, 68, 48},
	{"en_ca", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 30, 12, 13, 14, 15, 16, 31, 69, 50},
	{"en_cc", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 70, 48},
	{"en_ch", 142, 140, 141, 16, 145, 146, 147, 1, 161, 3, 4, 5, 6, 7, 74, 33, 34, 22, 23, 24, 25, 26, 162, 162, 60, 48},
	{"en_ck", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 71, 48},
	{"en_cm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_cx", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 70, 48},
	{"en_cy", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_cz", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_de", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_dg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_dk", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 11, 94, 95, 96, 97, 27, 27, 72, 48},
	{"en_dm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_dsrt", 144, 1, 145, 149, 1, 1, 150, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 38, 51},
	{"en_dsrt_us", 144, 1, 145, 149, 1, 1, 150, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 38, 51},
	{"en_ee", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_er", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 73, 48},
	{"en_es", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_fi", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 18, 74, 33, 34, 11, 184, 185, 186, 187, 27, 27, 60, 48},
	{"en_fj", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 74, 48},
	{"en_fk", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 75, 48},
	{"en_fm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_fr", 142, 140, 141, 16, 145, 146, 147, 2, 188, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_gb", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_gd", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_ge", 142, 140, 141, 16, 145, 146, 147, 2, 188, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 76, 48},
	{"en_gg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_gh", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 77, 48},
	{"en_gi", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 78, 48},
	{"en_gm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 79, 48},
	{"en_gs", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_gu", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_gy", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 80, 48},
	{"en_hk", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_hu", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_id", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 94, 95, 96, 97, 16, 31, 81, 48},
	{"en_ie", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_il", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 47, 48, 49, 50, 16, 31, 60, 48},
	{"en_im", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_in", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 11, 12, 13, 14, 15, 112, 31, 82, 48},
	{"en_io", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_it", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_je", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 60, 48},
	{"en_jm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 83, 48},
	{"en_jp", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 141, 47, 48, 49, 50, 16, 31, 59, 47},
	{"en_ke", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 84, 48},
	{"en_ki", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 70, 48},
	{"en_kn", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_ky", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 85, 48},
	{"en_lc", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_lr", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 86, 48},
	{"en_ls", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 87, 48},
	{"en_lt", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_lv", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_mg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 88, 48},
	{"en_mh", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_mo", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 89, 48},
	{"en_mp", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_ms", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 61, 48},
	{"en_mt", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 9, 29, 11, 23, 24, 25, 26, 16, 31, 90, 48},
	{"en_mu", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 91, 48},
	{"en_mv", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 32, 33, 67, 68, 23, 24, 25, 26, 54, 54, 92, 48},
	{"en_mw", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 93, 48},
	{"en_my", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 94, 48},
	{"en_na", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 95, 48},
	{"en_nf", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 70, 48},
	{"en_ng", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 96, 48},
	{"en_nl", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 109, 189, 60, 48},
	{"en_no", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 18, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 97, 48},
	{"en_nr", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 70, 48},
	{"en_nu", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 71, 48},
	{"en_nz", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 71, 48},
	{"en_pg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 98, 48},
	{"en_ph", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_pk", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 10, 11, 12, 13, 14, 15, 16, 31, 99, 48},
	{"en_pl", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 102, 60, 48},
	{"en_pn", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 71, 48},
	{"en_pr", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_pt", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 102, 60, 48},
	{"en_pw", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_ro", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 102, 60, 48},
	{"en_rw", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 100, 48},
	{"en_sb", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 101, 48},
	{"en_sc", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 102, 48},
	{"en_sd", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_se", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 18, 74, 33, 34, 30, 23, 24, 25, 26, 27, 27, 103, 48},
	{"en_sg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 104, 48},
	{"en_sh", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 105, 48},
	{"en_shaw", 146, 147, 148, 151, 1, 1, 152, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 0, 52},
	{"en_shaw_gb", 146, 147, 148, 151, 1, 1, 152, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 0, 52},
	{"en_si", 142, 140, 141, 16, 145, 146, 147, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 102, 60, 48},
	{"en_sk", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 102, 60, 48},
	{"en_sl", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 106, 48},
	{"en_ss", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 107, 48},
	{"en_sx", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 108, 48},
	{"en_sz", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 109, 48},
	{"en_tc", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_tk", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 71, 48},
	{"en_to", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 110, 48},
	{"en_tt", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 111, 48},
	{"en_tv", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 70, 48},
	{"en_tz", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 112, 48},
	{"en_ua", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 27, 60, 48},
	{"en_ug", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 113, 48},
	{"en_um", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_us", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_us_posix", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 190, 6, 191, 137, 138, 139, 108, 12, 13, 14, 15, 192, 31, 59, 47},
	{"en_vc", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 61, 48},
	{"en_vg", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 60, 48},
	{"en_vi", 139, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 59, 47},
	{"en_vu", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 114, 48},
	{"en_ws", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 115, 48},
	{"en_za", 142, 140, 141, 16, 145, 146, 147, 2, 17, 3, 4, 5, 6, 7, 183, 9, 29, 141, 23, 24, 25, 26, 16, 31, 87, 48},
	{"en_zm", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 116, 48},
	{"en_zw", 142, 140, 141, 16, 145, 146, 147, 1, 2, 3, 4, 5, 6, 7, 183, 9, 193, 36, 23, 24, 25, 26, 16, 31, 60, 48},
	{"eo", 149, 1, 150, 153, 1, 1, 154, 2, 188, 3, 4, 5, 6, 7, 194, 195, 196, 30, 23, 24, 25, 26, 197, 198, 117, 53},
	{"eo_001", 149, 1, 150, 153, 1, 1, 154, 2, 188, 3, 4, 5, 6, 7, 194, 195, 196, 30, 23, 24, 25, 26, 197, 198, 117, 53},
	{"es", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 136, 48, 49, 50, 27, 0, 118, 54},
	{"es_419", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_ar", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 54, 189, 120, 55},
	{"es_bo", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 201, 36, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_br", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 23, 24, 25, 26, 16, 0, 121, 55},
	{"es_bz", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 23, 24, 25, 26, 16, 0, 119, 55},
	{"es_cl", 151, 1, 152, 155, 1, 158, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 67, 120, 12, 13, 14, 15, 202, 0, 120, 55},
	{"es_co", 151, 1, 152, 155, 159, 156, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 203, 204, 12, 13, 14, 15, 54, 0, 120, 55},
	{"es_cr", 151, 1, 152, 155, 1, 156, 157, 2, 17, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_cu", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 120, 55},
	{"es_do", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 31, 122, 55},
	{"es_ea", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 136, 48, 49, 50, 27, 0, 118, 54},
	{"es_ec", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 202, 0, 123, 55},
	{"es_es", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 136, 48, 49, 50, 27, 0, 118, 54},
	{"es_gq", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 136, 48, 49, 50, 16, 0, 124, 54},
	{"es_gt", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 203, 204, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_hn", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 205, 206, 34, 36, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_ic", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 136, 48, 49, 50, 27, 0, 118, 54},
	{"es_mx", 153, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 11, 12, 13, 14, 15, 16, 0, 125, 56},
	{"es_ni", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 119, 55},
	{"es_pa", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 207, 208, 12, 13, 14, 15, 16, 0, 126, 55},
	{"es_pe", 154, 1, 155, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 204, 12, 13, 14, 15, 54, 0, 127, 55},
	{"es_ph", 151, 1, 152, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 18, 199, 200, 34, 36, 12, 13, 14, 15, 27, 0, 128, 54},
	{"es_pr", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 207, 208, 12, 13, 14, 15, 16, 0, 123, 55},
	{"es_py", 156, 1, 152, 155, 1, 160, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 109, 0, 129, 55},
	{"es_sv", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 123, 55},
	{"es_us", 151, 1, 152, 155, 1, 156, 157, 1, 2, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 16, 0, 130, 57},
	{"es_uy", 154, 1, 155, 155, 1, 156, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 54, 189, 131, 55},
	{"es_ve", 156, 1, 152, 155, 1, 161, 157, 2, 1, 3, 4, 5, 6, 7, 199, 200, 34, 36, 12, 13, 14, 15, 202, 0, 132, 55},
	{"et", 157, 1, 158, 162, 1, 1, 163, 2, 17, 209, 4, 5, 6, 7, 159, 147, 158, 22, 23, 24, 25, 26, 27, 102, 133, 58},
	{"et_ee", 157, 1, 158, 162, 1, 1, 163, 2, 17, 209, 4, 5, 6, 7, 159, 147, 158, 22, 23, 24, 25, 26, 27, 102, 133, 58},
	{"eu", 159, 1, 160, 164, 1, 1, 165, 2, 1, 209, 4, 5, 6, 210, 211, 212, 213, 214, 155, 156, 25, 26, 27, 102, 17, 59},
	{"eu_es", 159, 1, 160, 164, 1, 1, 165, 2, 1, 209, 4, 5, 6, 210, 211, 212, 213, 214, 155, 156, 25, 26, 27, 102, 17, 59},
	{"ewo", 161, 1, 162, 166, 1, 1, 167, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ewo_cm", 161, 1, 162, 166, 1, 1, 167, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"fa", 163, 1, 164, 1, 1, 168, 169, 1, 2, 117, 4, 5, 57, 7, 32, 33, 34, 214, 136, 215, 49, 50, 118, 119, 134, 60},
	{"fa_af", 165, 1, 166, 1, 1, 168, 169, 1, 2, 117, 4, 5, 57, 7, 32, 33, 34, 214, 136, 215, 49, 50, 54, 216, 134, 60},
	{"fa_ir", 163, 1, 164, 1, 1, 168, 169, 1, 2, 117, 4, 5, 57, 7, 32, 33, 34, 214, 136, 215, 49, 50, 118, 119, 134, 60},
	{"ff", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_adlm", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 135, 61},
	{"ff_adlm_bf", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 136, 61},
	{"ff_adlm_cm", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 136, 61},
	{"ff_adlm_gh", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 41, 42, 43, 44, 54, 0, 136, 61},
	{"ff_adlm_gm", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 41, 42, 43, 44, 54, 0, 137, 61},
	{"ff_adlm_gn", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 135, 61},
	{"ff_adlm_gw", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 136, 61},
	{"ff_adlm_lr", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 41, 42, 43, 44, 54, 0, 136, 61},
	{"ff_adlm_mr", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 41, 42, 43, 44, 54, 0, 138, 61},
	{"ff_adlm_ne", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 136, 61},
	{"ff_adlm_ng", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 139, 61},
	{"ff_adlm_sl", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 41, 42, 43, 44, 54, 0, 140, 61},
	{"ff_adlm_sn", 1, 1, 169, 172, 1, 1, 173, 1, 217, 3, 4, 5, 6, 7, 218, 219, 220, 68, 23, 24, 25, 26, 54, 0, 136, 61},
	{"ff_latn", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_bf", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_cm", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_gh", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 27, 0, 0, 0},
	{"ff_latn_gm", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 27, 0, 141, 0},
	{"ff_latn_gn", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_gw", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_lr", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 27, 0, 0, 0},
	{"ff_latn_mr", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 27, 0, 142, 0},
	{"ff_latn_ne", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_ng", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ff_latn_sl", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 27, 0, 143, 0},
	{"ff_latn_sn", 167, 1, 168, 170, 1, 1, 171, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"fi", 170, 1, 171, 174, 1, 1, 175, 2, 17, 209, 4, 5, 6, 18, 221, 147, 166, 167, 184, 185, 186, 187, 27, 0, 144, 62},
	{"fi_fi", 170, 1, 171, 174, 1, 1, 175, 2, 17, 209, 4, 5, 6, 18, 221, 147, 166, 167, 184, 185, 186, 187, 27, 0, 144, 62},
	{"fil", 172, 172, 173, 176, 1, 1, 177, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 145, 63},
	{"fil_ph", 172, 172, 173, 176, 1, 1, 177, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 31, 145, 63},
	{"fo", 174, 1, 175, 178, 1, 179, 180, 2, 1, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 102, 17, 64},
	{"fo_dk", 174, 1, 175, 178, 1, 179, 180, 2, 1, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 102, 146, 64},
	{"fo_fo", 174, 1, 175, 178, 1, 179, 180, 2, 1, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 102, 17, 64},
	{"fr", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_be", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 204, 222, 24, 25, 26, 27, 102, 147, 65},
	{"fr_bf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_bi", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 148, 65},
	{"fr_bj", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_bl", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ca", 178, 1, 177, 181, 1, 182, 183, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 30, 223, 224, 225, 226, 27, 102, 149, 66},
	{"fr_cd", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 150, 65},
	{"fr_cf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_cg", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ch", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 7, 74, 33, 34, 22, 227, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ci", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_cm", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_dj", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 151, 65},
	{"fr_dz", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 152, 65},
	{"fr_fr", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ga", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_gf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_gn", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_gp", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_gq", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ht", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 153, 65},
	{"fr_km", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 154, 65},
	{"fr_lu", 176, 1, 177, 181, 1, 182, 183, 2, 1, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 155, 65},
	{"fr_ma", 179, 1, 177, 181, 1, 182, 183, 2, 1, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_mc", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_mf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_mg", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ml", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_mq", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_mr", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 156, 65},
	{"fr_mu", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_nc", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_ne", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_pf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_pm", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_re", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_rw", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 157, 65},
	{"fr_sc", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 158, 65},
	{"fr_sn", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_sy", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 159, 65},
	{"fr_td", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 147, 65},
	{"fr_tg", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_tn", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 160, 65},
	{"fr_vu", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 12, 13, 14, 15, 27, 102, 161, 65},
	{"fr_wf", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"fr_yt", 176, 1, 177, 181, 1, 182, 183, 2, 188, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 147, 65},
	{"frr", 180, 1, 181, 184, 1, 185, 186, 2, 1, 3, 4, 5, 6, 7, 159, 147, 158, 11, 23, 24, 25, 26, 54, 0, 162, 67},
	{"frr_de", 180, 1, 181, 184, 1, 185, 186, 2, 1, 3, 4, 5, 6, 7, 159, 147, 158, 11, 23, 24, 25, 26, 54, 0, 162, 67},
	{"fur", 182, 1, 183, 187, 1, 1, 188, 2, 1, 3, 4, 5, 6, 7, 228, 229, 230, 11, 23, 24, 25, 26, 54, 0, 0, 68},
	{"fur_it", 182, 1, 183, 187, 1, 1, 188, 2, 1, 3, 4, 5, 6, 7, 228, 229, 230, 11, 23, 24, 25, 26, 54, 0, 0, 68},
	{"fy", 184, 1, 185, 189, 1, 1, 190, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 231, 189, 163, 69},
	{"fy_nl", 184, 1, 185, 189, 1, 1, 190, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 231, 189, 163, 69},
	{"ga", 186, 1, 187, 191, 1, 192, 193, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 16, 31, 164, 70},
	{"ga_gb", 186, 1, 187, 191, 1, 192, 193, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 16, 31, 164, 70},
	{"ga_ie", 186, 1, 187, 191, 1, 192, 193, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 16, 31, 164, 70},
	{"gaa", 188, 1, 189, 194, 1, 1, 195, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 31, 165, 71},
	{"gaa_gh", 188, 1, 189, 194, 1, 1, 195, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 31, 165, 71},
	{"gd", 190, 1, 191, 196, 1, 197, 198, 1, 2, 3, 4, 5, 6, 7, 232, 233, 234, 11, 23, 24, 25, 26, 16, 31, 166, 72},
	{"gd_gb", 190, 1, 191, 196, 1, 197, 198, 1, 2, 3, 4, 5, 6, 7, 232, 233, 234, 11, 23, 24, 25, 26, 16, 31, 166, 72},
	{"gez", 1, 1, 192, 1, 1, 1, 199, 1, 235, 3, 4, 5, 6, 7, 236, 9, 10, 11, 41, 42, 43, 44, 16, 0, 0, 0},
	{"gez_er", 1, 1, 192, 1, 1, 1, 199, 1, 235, 3, 4, 5, 6, 7, 236, 9, 10, 11, 41, 42, 43, 44, 16, 0, 35, 0},
	{"gez_et", 1, 1, 192, 1, 1, 1, 199, 1, 235, 3, 4, 5, 6, 7, 236, 9, 10, 11, 41, 42, 43, 44, 16, 0, 0, 0},
	{"gl", 193, 194, 195, 200, 201, 202, 203, 2, 1, 3, 4, 5, 6, 18, 199, 200, 237, 11, 23, 24, 25, 26, 27, 102, 167, 73},
	{"gl_es", 193, 194, 195, 200, 201, 202, 203, 2, 1, 3, 4, 5, 6, 18, 199, 200, 237, 11, 23, 24, 25, 26, 27, 102, 167, 73},
	{"gn", 1, 1, 196, 1, 1, 1, 204, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 74},
	{"gn_py", 1, 1, 196, 1, 1, 1, 204, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 74},
	{"gsw", 197, 1, 198, 205, 1, 1, 206, 1, 161, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 168, 75},
	{"gsw_ch", 197, 1, 198, 205, 1, 1, 206, 1, 161, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 168, 75},
	{"gsw_fr", 197, 1, 198, 205, 1, 1, 206, 1, 161, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 168, 75},
	{"gsw_li", 197, 1, 198, 205, 1, 1, 206, 1, 161, 209, 4, 5, 6, 18, 159, 147, 160, 22, 23, 24, 25, 26, 27, 0, 168, 75},
	{"gu", 199, 1, 200, 207, 1, 208, 209, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 79, 238, 81, 82, 112, 113, 169, 76},
	{"gu_in", 199, 1, 200, 207, 1, 208, 209, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 79, 238, 81, 82, 112, 113, 169, 76},
	{"guz", 201, 1, 202, 210, 1, 1, 211, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"guz_ke", 201, 1, 202, 210, 1, 1, 211, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"gv", 203, 1, 204, 212, 1, 1, 213, 1, 2, 3, 4, 5, 6, 7, 28, 9, 239, 11, 23, 24, 25, 26, 16, 0, 0, 0},
	{"gv_im", 203, 1, 204, 212, 1, 1, 213, 1, 2, 3, 4, 5, 6, 7, 28, 9, 239, 11, 23, 24, 25, 26, 16, 0, 0, 0},
	{"ha", 205, 1, 206, 214, 1, 215, 216, 1, 2, 3, 4, 5, 6, 7, 240, 66, 35, 36, 23, 24, 25, 26, 54, 0, 53, 77},
	{"ha_arab", 207, 1, 208, 217, 1, 1, 218, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ha_arab_ng", 207, 1, 208, 217, 1, 1, 218, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ha_arab_sd", 207, 1, 208, 217, 1, 1, 218, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ha_gh", 205, 1, 206, 214, 1, 215, 216, 1, 2, 3, 4, 5, 6, 7, 240, 66, 35, 36, 12, 13, 14, 15, 54, 0, 53, 77},
	{"ha_ne", 205, 1, 206, 214, 1, 215, 216, 1, 2, 3, 4, 5, 6, 7, 240, 66, 35, 36, 23, 24, 25, 26, 54, 0, 53, 77},
	{"ha_ng", 205, 1, 206, 214, 1, 215, 216, 1, 2, 3, 4, 5, 6, 7, 240, 66, 35, 36, 23, 24, 25, 26, 54, 0, 53, 77},
	{"haw", 209, 1, 210, 219, 1, 1, 220, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 38, 0},
	{"haw_us", 209, 1, 210, 219, 1, 1, 220, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 16, 31, 38, 0},
	{"he", 211, 1, 212, 221, 1, 222, 223, 1, 2, 55, 4, 5, 57, 7, 241, 242, 243, 167, 47, 48, 49, 50, 244, 0, 170, 78},
	{"he_il", 211, 1, 212, 221, 1, 222, 223, 1, 2, 55, 4, 5, 57, 7, 241, 242, 243, 167, 47, 48, 49, 50, 244, 0, 170, 78},
	{"hi", 213, 1, 214, 224, 1, 225, 226, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 0, 171, 79},
	{"hi_in", 213, 1, 214, 224, 1, 225, 226, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 0, 171, 79},
	{"hi_latn", 139, 140, 141, 227, 228, 229, 230, 1, 2, 3, 4, 5, 6, 64, 74, 66, 245, 11, 12, 13, 14, 15, 112, 112, 82, 80},
	{"hi_latn_in", 139, 140, 141, 227, 228, 229, 230, 1, 2, 3, 4, 5, 6, 64, 74, 66, 245, 11, 12, 13, 14, 15, 112, 112, 82, 80},
	{"hnj", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"hnj_hmnp", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"hnj_hmnp_us", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"hr", 216, 1, 217, 72, 73, 1, 74, 2, 1, 209, 4, 5, 6, 18, 125, 126, 127, 246, 155, 24, 25, 26, 27, 0, 173, 81},
	{"hr_ba", 216, 1, 217, 72, 73, 1, 74, 2, 1, 209, 4, 5, 6, 18, 125, 126, 127, 128, 155, 24, 25, 26, 27, 0, 173, 81},
	{"hr_hr", 216, 1, 217, 72, 73, 1, 74, 2, 1, 209, 4, 5, 6, 18, 125, 126, 127, 246, 155, 24, 25, 26, 27, 0, 173, 81},
	{"hsb", 218, 1, 219, 231, 1, 232, 233, 2, 1, 3, 4, 5, 6, 18, 159, 147, 166, 167, 47, 48, 49, 247, 27, 0, 54, 82},
	{"hsb_de", 218, 1, 219, 231, 1, 232, 233, 2, 1, 3, 4, 5, 6, 18, 159, 147, 166, 167, 47, 48, 49, 247, 27, 0, 54, 82},
	{"ht", 176, 1, 177, 181, 1, 182, 183, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 153, 65},
	{"ht_ht", 176, 1, 177, 181, 1, 182, 183, 2, 17, 3, 4, 5, 6, 18, 32, 33, 34, 11, 23, 24, 25, 26, 27, 102, 153, 65},
	{"hu", 220, 1, 221, 234, 1, 1, 235, 2, 17, 3, 4, 5, 6, 7, 248, 249, 250, 251, 47, 48, 49, 50, 27, 0, 174, 83},
	{"hu_hu", 220, 1, 221, 234, 1, 1, 235, 2, 17, 3, 4, 5, 6, 7, 248, 249, 250, 251, 47, 48, 49, 50, 27, 0, 174, 83},
	{"hy", 222, 1, 223, 236, 1, 237, 238, 2, 17, 3, 4, 5, 6, 7, 252, 253, 254, 22, 23, 24, 25, 26, 27, 0, 29, 84},
	{"hy_am", 222, 1, 223, 236, 1, 237, 238, 2, 17, 3, 4, 5, 6, 7, 252, 253, 254, 22, 23, 24, 25, 26, 27, 0, 29, 84},
	{"ia", 224, 225, 226, 239, 1, 240, 241, 2, 1, 3, 4, 5, 6, 7, 255, 256, 34, 120, 23, 24, 25, 26, 54, 189, 175, 85},
	{"ia_001", 224, 225, 226, 239, 1, 240, 241, 2, 1, 3, 4, 5, 6, 7, 255, 256, 34, 120, 23, 24, 25, 26, 54, 189, 175, 85},
	{"id", 227, 1, 228, 242, 1, 1, 243, 2, 1, 3, 4, 5, 6, 7, 183, 33, 34, 11, 94, 95, 96, 97, 16, 0, 176, 86},
	{"id_id", 227, 1, 228, 242, 1, 1, 243, 2, 1, 3, 4, 5, 6, 7, 183, 33, 34, 11, 94, 95, 96, 97, 16, 0, 176, 86},
	{"ie", 229, 1, 230, 244, 1, 245, 246, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 167, 23, 24, 25, 26, 109, 0, 177, 87},
	{"ie_ee", 229, 1, 230, 244, 1, 245, 246, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 167, 23, 24, 25, 26, 109, 0, 177, 87},
	{"ig", 231, 1, 232, 247, 1, 1, 248, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 23, 24, 25, 26, 16, 31, 53, 88},
	{"ig_ng", 231, 1, 232, 247, 1, 1, 248, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 23, 24, 25, 26, 16, 31, 53, 88},
	{"ii", 1, 1, 233, 249, 1, 1, 250, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 30, 89},
	{"ii_cn", 1, 1, 233, 249, 1, 1, 250, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 30, 89},
	{"io", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"io_001", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"is", 234, 1, 235, 251, 1, 252, 253, 2, 1, 3, 4, 5, 6, 7, 159, 147, 158, 167, 23, 24, 25, 26, 27, 0, 178, 90},
	{"is_is", 234, 1, 235, 251, 1, 252, 253, 2, 1, 3, 4, 5, 6, 7, 159, 147, 158, 167, 23, 24, 25, 26, 27, 0, 178, 90},
	{"it", 236, 1, 237, 254, 1, 1, 255, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 27, 0, 179, 91},
	{"it_ch", 236, 1, 237, 254, 1, 1, 255, 1, 161, 3, 4, 5, 6, 7, 74, 33, 34, 22, 23, 24, 25, 26, 162, 0, 179, 91},
	{"it_it", 236, 1, 237, 254, 1, 1, 255, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 27, 0, 179, 91},
	{"it_sm", 236, 1, 237, 254, 1, 1, 255, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 27, 0, 179, 91},
	{"it_va", 236, 1, 237, 254, 1, 1, 255, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 27, 0, 179, 91},
	{"iu", 1, 1, 238, 1, 1, 1, 256, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 208, 41, 42, 43, 44, 54, 0, 0, 0},
	{"iu_ca", 1, 1, 238, 1, 1, 1, 256, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 208, 41, 42, 43, 44, 54, 0, 0, 0},
	{"iu_latn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"iu_latn_ca", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ja", 1, 1, 239, 257, 1, 1, 258, 1, 2, 3, 4, 5, 6, 7, 257, 258, 259, 141, 260, 48, 49, 50, 16, 31, 180, 92},
	{"ja_jp", 1, 1, 239, 257, 1, 1, 258, 1, 2, 3, 4, 5, 6, 7, 257, 258, 259, 141, 260, 48, 49, 50, 16, 31, 180, 92},
	{"jbo", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 93},
	{"jbo_001", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 93},
	{"jgo", 1, 1, 240, 1, 1, 1, 259, 2, 1, 3, 4, 5, 6, 7, 140, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"jgo_cm", 1, 1, 240, 1, 1, 1, 259, 2, 1, 3, 4, 5, 6, 7, 140, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"jmc", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"jmc_tz", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"jv", 243, 1, 228, 262, 1, 1, 263, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 120, 23, 24, 25, 26, 54, 0, 0, 94},
	{"jv_id", 243, 1, 228, 262, 1, 1, 263, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 120, 23, 24, 25, 26, 54, 0, 0, 94},
	{"ka", 244, 1, 245, 264, 1, 265, 266, 2, 17, 3, 4, 5, 6, 7, 261, 66, 262, 22, 23, 24, 25, 26, 27, 0, 181, 95},
	{"ka_ge", 244, 1, 245, 264, 1, 265, 266, 2, 17, 3, 4, 5, 6, 7, 261, 66, 262, 22, 23, 24, 25, 26, 27, 0, 181, 95},
	{"kaa", 87, 246, 88, 267, 268, 269, 270, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kaa_cyrl", 87, 246, 88, 267, 268, 269, 270, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kaa_cyrl_uz", 87, 246, 88, 267, 268, 269, 270, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kaa_latn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kaa_latn_uz", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kab", 247, 1, 248, 271, 272, 271, 271, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 37, 0, 182, 96},
	{"kab_dz", 247, 1, 248, 271, 272, 271, 271, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 12, 13, 14, 15, 37, 0, 182, 96},
	{"kaj", 249, 1, 250, 273, 1, 1, 274, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kaj_ng", 249, 1, 250, 273, 1, 1, 274, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kam", 251, 1, 252, 275, 1, 1, 276, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"kam_ke", 251, 1, 252, 275, 1, 1, 276, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"kcg", 253, 1, 254, 277, 1, 1, 278, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kcg_ng", 253, 1, 254, 277, 1, 1, 278, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 141, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kde", 241, 1, 255, 279, 1, 1, 280, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 18, 0},
	{"kde_tz", 241, 1, 255, 279, 1, 1, 280, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 18, 0},
	{"kea", 256, 1, 257, 281, 1, 282, 283, 2, 17, 3, 4, 5, 6, 7, 263, 264, 34, 11, 23, 24, 25, 26, 27, 102, 183, 97},
	{"kea_cv", 256, 1, 257, 281, 1, 282, 283, 2, 17, 3, 4, 5, 6, 7, 263, 264, 34, 11, 23, 24, 25, 26, 27, 102, 183, 97},
	{"kek", 1, 1, 258, 1, 1, 1, 284, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kek_gt", 1, 1, 258, 1, 1, 1, 284, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ken", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ken_cm", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kgp", 259, 1, 260, 285, 1, 286, 287, 2, 1, 3, 4, 5, 6, 7, 265, 266, 267, 11, 23, 24, 25, 26, 54, 0, 184, 98},
	{"kgp_br", 259, 1, 260, 285, 1, 286, 287, 2, 1, 3, 4, 5, 6, 7, 265, 266, 267, 11, 23, 24, 25, 26, 54, 0, 184, 98},
	{"khq", 118, 1, 119, 288, 1, 1, 289, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"khq_ml", 118, 1, 119, 288, 1, 1, 289, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"ki", 261, 1, 262, 290, 1, 1, 291, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"ki_ke", 261, 1, 262, 290, 1, 1, 291, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"kk", 263, 1, 264, 292, 1, 1, 293, 2, 17, 3, 4, 5, 6, 7, 268, 269, 270, 22, 23, 24, 25, 26, 27, 0, 185, 99},
	{"kk_arab", 1, 1, 265, 294, 1, 295, 296, 1, 2, 3, 4, 5, 6, 7, 271, 272, 273, 120, 23, 24, 25, 26, 54, 0, 0, 100},
	{"kk_arab_cn", 1, 1, 265, 294, 1, 295, 296, 1, 2, 3, 4, 5, 6, 7, 271, 272, 273, 120, 23, 24, 25, 26, 54, 0, 0, 100},
	{"kk_cyrl", 263, 1, 264, 292, 1, 1, 293, 2, 17, 3, 4, 5, 6, 7, 268, 269, 270, 22, 23, 24, 25, 26, 27, 0, 185, 99},
	{"kk_cyrl_kz", 263, 1, 264, 292, 1, 1, 293, 2, 17, 3, 4, 5, 6, 7, 268, 269, 270, 22, 23, 24, 25, 26, 27, 0, 185, 99},
	{"kk_kz", 263, 1, 264, 292, 1, 1, 293, 2, 17, 3, 4, 5, 6, 7, 268, 269, 270, 22, 23, 24, 25, 26, 27, 0, 185, 99},
	{"kkj", 1, 1, 266, 1, 1, 1, 297, 2, 1, 3, 4, 5, 6, 7, 28, 33, 34, 274, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kkj_cm", 1, 1, 266, 1, 1, 1, 297, 2, 1, 3, 4, 5, 6, 7, 28, 33, 34, 274, 23, 24, 25, 26, 54, 0, 0, 0},
	{"kl", 267, 1, 268, 298, 1, 1, 299, 2, 1, 209, 4, 5, 6, 18, 28, 9, 29, 30, 94, 95, 96, 97, 202, 0, 186, 0},
	{"kl_gl", 267, 1, 268, 298, 1, 1, 299, 2, 1, 209, 4, 5, 6, 18, 28, 9, 29, 30, 94, 95, 96, 97, 202, 0, 186, 0},
	{"kln", 269, 1, 270, 300, 1, 1, 301, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"kln_ke", 269, 1, 270, 300, 1, 1, 301, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"km", 1, 1, 271, 302, 1, 303, 304, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 41, 42, 43, 44, 37, 275, 187, 101},
	{"km_kh", 1, 1, 271, 302, 1, 303, 304, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 41, 42, 43, 44, 37, 275, 187, 101},
	{"kn", 272, 1, 273, 305, 1, 1, 306, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 36, 79, 238, 81, 82, 16, 31, 188, 102},
	{"kn_in", 272, 1, 273, 305, 1, 1, 306, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 36, 79, 238, 81, 82, 16, 31, 188, 102},
	{"ko", 1, 1, 274, 307, 1, 1, 308, 1, 2, 3, 4, 5, 6, 7, 276, 277, 278, 279, 280, 281, 123, 282, 16, 31, 189, 103},
	{"ko_cn", 1, 1, 274, 307, 1, 1, 308, 1, 2, 3, 4, 5, 6, 7, 276, 277, 278, 279, 283, 284, 25, 26, 16, 31, 189, 103},
	{"ko_kp", 1, 1, 274, 307, 1, 1, 308, 1, 2, 3, 4, 5, 6, 7, 276, 277, 278, 279, 280, 281, 123, 282, 16, 31, 189, 104},
	{"ko_kr", 1, 1, 274, 307, 1, 1, 308, 1, 2, 3, 4, 5, 6, 7, 276, 277, 278, 279, 280, 281, 123, 282, 16, 31, 189, 103},
	{"kok", 275, 276, 277, 1, 309, 310, 311, 1, 2, 3, 4, 5, 6, 7, 32, 33, 285, 68, 41, 42, 43, 44, 112, 112, 190, 105},
	{"kok_deva", 275, 276, 277, 1, 309, 310, 311, 1, 2, 3, 4, 5, 6, 7, 32, 33, 285, 68, 41, 42, 43, 44, 112, 112, 190, 105},
	{"kok_deva_in", 275, 276, 277, 1, 309, 310, 311, 1, 2, 3, 4, 5, 6, 7, 32, 33, 285, 68, 41, 42, 43, 44, 112, 112, 190, 105},
	{"kok_latn", 278, 1, 279, 312, 1, 313, 314, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 68, 286, 287, 288, 289, 112, 0, 0, 106},
	{"kok_latn_in", 278, 1, 279, 312, 1, 313, 314, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 68, 286, 287, 288, 289, 112, 0, 0, 106},
	{"kpe", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"kpe_gn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 191, 0},
	{"kpe_lr", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"ks", 280, 1, 281, 315, 1, 1, 316, 1, 104, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 0, 53, 107},
	{"ks_arab", 280, 1, 281, 315, 1, 1, 316, 1, 104, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 0, 53, 107},
	{"ks_arab_in", 280, 1, 281, 315, 1, 1, 316, 1, 104, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 0, 53, 107},
	{"ks_deva", 282, 1, 283, 317, 1, 1, 318, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 121, 122, 123, 282, 54, 0, 38, 108},
	{"ks_deva_in", 282, 1, 283, 317, 1, 1, 318, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 121, 122, 123, 282, 54, 0, 38, 108},
	{"ksb", 241, 1, 284, 319, 1, 1, 320, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"ksb_tz", 241, 1, 284, 319, 1, 1, 320, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"ksf", 285, 1, 286, 321, 1, 1, 322, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ksf_cm", 285, 1, 286, 321, 1, 1, 322, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ksh", 287, 1, 288, 323, 1, 324, 325, 2, 17, 209, 4, 5, 6, 18, 290, 147, 291, 292, 23, 24, 25, 26, 27, 0, 0, 109},
	{"ksh_de", 287, 1, 288, 323, 1, 324, 325, 2, 17, 209, 4, 5, 6, 18, 290, 147, 291, 292, 23, 24, 25, 26, 27, 0, 0, 109},
	{"ku", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"ku_arab", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ku_arab_iq", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ku_arab_ir", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ku_latn", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"ku_latn_iq", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"ku_latn_sy", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"ku_latn_tr", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"ku_tr", 289, 1, 290, 326, 1, 327, 328, 2, 1, 3, 4, 5, 6, 293, 294, 295, 35, 22, 23, 24, 25, 26, 27, 102, 38, 110},
	{"kw", 291, 1, 292, 329, 1, 1, 330, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 16, 0, 0, 0},
	{"kw_gb", 291, 1, 292, 329, 1, 1, 330, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 16, 0, 0, 0},
	{"kxv", 293, 1, 294, 331, 1, 332, 333, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 31, 0, 111},
	{"kxv_deva", 295, 1, 296, 334, 335, 335, 336, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 112},
	{"kxv_deva_in", 295, 1, 296, 334, 335, 335, 336, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 112},
	{"kxv_latn", 293, 1, 294, 331, 1, 332, 333, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 31, 0, 111},
	{"kxv_latn_in", 293, 1, 294, 331, 1, 332, 333, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 31, 0, 111},
	{"kxv_orya", 297, 1, 298, 337, 1, 338, 339, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 113},
	{"kxv_orya_in", 297, 1, 298, 337, 1, 338, 339, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 113},
	{"kxv_telu", 1, 1, 299, 1, 1, 1, 340, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 114},
	{"kxv_telu_in", 1, 1, 299, 1, 1, 1, 340, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 12, 13, 14, 15, 112, 0, 0, 114},
	{"ky", 300, 1, 88, 341, 1, 342, 343, 2, 17, 3, 4, 5, 6, 7, 296, 297, 298, 36, 23, 24, 25, 26, 27, 0, 192, 115},
	{"ky_kg", 300, 1, 88, 341, 1, 342, 343, 2, 17, 3, 4, 5, 6, 7, 296, 297, 298, 36, 23, 24, 25, 26, 27, 0, 192, 115},
	{"la", 301, 1, 302, 344, 1, 1, 345, 2, 17, 3, 4, 5, 6, 7, 299, 300, 301, 302, 23, 24, 25, 26, 54, 0, 0, 0},
	{"la_va", 301, 1, 302, 344, 1, 1, 345, 2, 17, 3, 4, 5, 6, 7, 299, 300, 301, 302, 23, 24, 25, 26, 54, 0, 0, 0},
	{"lag", 303, 1, 304, 346, 1, 1, 347, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 18, 0},
	{"lag_tz", 303, 1, 304, 346, 1, 1, 347, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 18, 0},
	{"lb", 305, 1, 306, 348, 1, 349, 350, 2, 1, 3, 4, 5, 6, 18, 159, 147, 158, 22, 23, 24, 25, 26, 27, 0, 193, 116},
	{"lb_lu", 305, 1, 306, 348, 1, 349, 350, 2, 1, 3, 4, 5, 6, 18, 159, 147, 158, 22, 23, 24, 25, 26, 27, 0, 193, 116},
	{"lg", 307, 1, 308, 351, 1, 1, 352, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 42, 0},
	{"lg_ug", 307, 1, 308, 351, 1, 1, 352, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 42, 0},
	{"lij", 309, 1, 310, 353, 1, 1, 354, 2, 1, 3, 4, 5, 6, 7, 303, 304, 305, 11, 23, 24, 25, 26, 27, 102, 194, 117},
	{"lij_it", 309, 1, 310, 353, 1, 1, 354, 2, 1, 3, 4, 5, 6, 7, 303, 304, 305, 11, 23, 24, 25, 26, 27, 102, 194, 117},
	{"lkt", 1, 1, 311, 1, 355, 1, 356, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"lkt_us", 1, 1, 311, 1, 355, 1, 356, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"lld", 1, 312, 313, 357, 1, 1, 358, 2, 1, 3, 4, 5, 6, 7, 306, 33, 34, 22, 23, 24, 25, 26, 27, 27, 0, 118},
	{"lld_it", 1, 312, 313, 357, 1, 1, 358, 2, 1, 3, 4, 5, 6, 7, 306, 33, 34, 22, 23, 24, 25, 26, 27, 27, 0, 118},
	{"lmo", 1, 1, 314, 1, 1, 1, 359, 2, 161, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 119},
	{"lmo_it", 1, 1, 314, 1, 1, 1, 359, 2, 161, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 119},
	{"ln", 315, 1, 316, 360, 1, 1, 361, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 195, 120},
	{"ln_ao", 315, 1, 316, 360, 1, 1, 361, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 195, 120},
	{"ln_cd", 315, 1, 316, 360, 1, 1, 361, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 195, 120},
	{"ln_cf", 315, 1, 316, 360, 1, 1, 361, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 195, 120},
	{"ln_cg", 315, 1, 316, 360, 1, 1, 361, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 195, 120},
	{"lo", 317, 1, 318, 362, 1, 363, 364, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 36, 307, 308, 49, 50, 202, 0, 196, 121},
	{"lo_la", 317, 1, 318, 362, 1, 363, 364, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 36, 307, 308, 49, 50, 202, 0, 196, 121},
	{"lrc", 1, 1, 319, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 43, 122},
	{"lrc_iq", 1, 1, 319, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 43, 122},
	{"lrc_ir", 1, 1, 319, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 43, 122},
	{"lt", 320, 1, 321, 365, 1, 366, 367, 2, 17, 209, 4, 5, 6, 18, 309, 310, 311, 30, 23, 24, 25, 26, 27, 0, 197, 123},
	{"lt_lt", 320, 1, 321, 365, 1, 366, 367, 2, 17, 209, 4, 5, 6, 18, 309, 310, 311, 30, 23, 24, 25, 26, 27, 0, 197, 123},
	{"ltg", 1, 1, 322, 1, 1, 1, 368, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 124},
	{"ltg_lv", 1, 1, 322, 1, 1, 1, 368, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 124},
	{"lu", 323, 1, 324, 369, 1, 1, 370, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 198, 0},
	{"lu_cd", 323, 1, 324, 369, 1, 1, 370, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 198, 0},
	{"luo", 325, 1, 326, 371, 1, 1, 372, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 49, 0},
	{"luo_ke", 325, 1, 326, 371, 1, 1, 372, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 49, 0},
	{"luy", 327, 1, 28, 373, 1, 1, 374, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 312, 0, 49, 0},
	{"luy_ke", 327, 1, 28, 373, 1, 1, 374, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 312, 0, 49, 0},
	{"lv", 328, 1, 329, 375, 1, 376, 377, 2, 17, 3, 4, 5, 6, 7, 313, 314, 315, 22, 23, 24, 25, 26, 27, 0, 199, 125},
	{"lv_lv", 328, 1, 329, 375, 1, 376, 377, 2, 17, 3, 4, 5, 6, 7, 313, 314, 315, 22, 23, 24, 25, 26, 27, 0, 199, 125},
	{"lzz", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"lzz_tr", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mai", 330, 1, 331, 224, 1, 1, 378, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 38, 126},
	{"mai_in", 330, 1, 331, 224, 1, 1, 378, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 38, 126},
	{"mas", 332, 1, 333, 260, 1, 1, 379, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"mas_ke", 332, 1, 333, 260, 1, 1, 379, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"mas_tz", 332, 1, 333, 260, 1, 1, 379, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 200, 0},
	{"mdf", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mdf_ru", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mer", 334, 1, 335, 380, 1, 1, 381, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"mer_ke", 334, 1, 335, 380, 1, 1, 381, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"mfe", 336, 1, 337, 382, 1, 1, 383, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mfe_mu", 336, 1, 337, 382, 1, 1, 383, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mg", 338, 1, 339, 384, 1, 1, 385, 1, 2, 3, 4, 5, 6, 7, 32, 33, 53, 30, 23, 24, 25, 26, 54, 16, 0, 0},
	{"mg_mg", 338, 1, 339, 384, 1, 1, 385, 1, 2, 3, 4, 5, 6, 7, 32, 33, 53, 30, 23, 24, 25, 26, 54, 16, 0, 0},
	{"mgh", 340, 1, 341, 386, 1, 1, 387, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 201, 0},
	{"mgh_mz", 340, 1, 341, 386, 1, 1, 387, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 201, 0},
	{"mgo", 342, 1, 343, 1, 1, 388, 389, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mgo_cm", 342, 1, 343, 1, 1, 388, 389, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mhn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mhn_it", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"mi", 344, 1, 345, 390, 1, 391, 392, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 120, 12, 13, 14, 15, 54, 0, 202, 127},
	{"mi_nz", 344, 1, 345, 390, 1, 391, 392, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 120, 12, 13, 14, 15, 54, 0, 202, 127},
	{"mic", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"mic_ca", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"mk", 346, 1, 347, 393, 1, 1, 394, 2, 1, 3, 4, 5, 6, 18, 88, 89, 90, 316, 23, 24, 25, 26, 27, 0, 203, 128},
	{"mk_mk", 346, 1, 347, 393, 1, 1, 394, 2, 1, 3, 4, 5, 6, 18, 88, 89, 90, 316, 23, 24, 25, 26, 27, 0, 203, 128},
	{"ml", 348, 1, 349, 395, 396, 397, 398, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 36, 317, 318, 43, 44, 16, 31, 204, 129},
	{"ml_in", 348, 1, 349, 395, 396, 397, 398, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 36, 317, 318, 43, 44, 16, 31, 204, 129},
	{"mn", 350, 1, 351, 399, 1, 1, 400, 1, 2, 3, 4, 5, 6, 7, 319, 320, 321, 151, 155, 156, 25, 26, 54, 0, 205, 130},
	{"mn_mn", 350, 1, 351, 399, 1, 1, 400, 1, 2, 3, 4, 5, 6, 7, 319, 320, 321, 151, 155, 156, 25, 26, 54, 0, 205, 130},
	{"mn_mong", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 38, 0},
	{"mn_mong_cn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 38, 0},
	{"mn_mong_mn", 352, 1, 353, 401, 402, 1, 403, 1, 2, 3, 4, 5, 6, 7, 322, 323, 324, 151, 155, 156, 25, 26, 16, 0, 38, 131},
	{"mni", 354, 355, 356, 1, 404, 1, 405, 1, 2, 3, 4, 5, 6, 7, 325, 138, 139, 36, 41, 42, 43, 44, 54, 0, 38, 132},
	{"mni_beng", 354, 355, 356, 1, 404, 1, 405, 1, 2, 3, 4, 5, 6, 7, 325, 138, 139, 36, 41, 42, 43, 44, 54, 0, 38, 132},
	{"mni_beng_in", 354, 355, 356, 1, 404, 1, 405, 1, 2, 3, 4, 5, 6, 7, 325, 138, 139, 36, 41, 42, 43, 44, 54, 0, 38, 132},
	{"mni_mtei", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 65, 66, 67, 68, 326, 327, 328, 329, 54, 0, 0, 0},
	{"mni_mtei_in", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 65, 66, 67, 68, 326, 327, 328, 329, 54, 0, 0, 0},
	{"moh", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"moh_ca", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"mr", 357, 1, 358, 406, 1, 225, 407, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 41, 42, 43, 44, 16, 31, 206, 133},
	{"mr_in", 357, 1, 358, 406, 1, 225, 407, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 41, 42, 43, 44, 16, 31, 206, 133},
	{"ms", 359, 1, 360, 408, 1, 409, 410, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 204, 12, 13, 14, 15, 16, 31, 207, 134},
	{"ms_arab", 1, 1, 361, 1, 1, 1, 411, 1, 2, 3, 4, 5, 6, 7, 58, 33, 230, 204, 41, 42, 43, 44, 16, 31, 0, 135},
	{"ms_arab_bn", 1, 1, 361, 1, 1, 1, 411, 2, 1, 3, 4, 5, 6, 7, 9, 33, 230, 204, 41, 42, 43, 44, 54, 31, 0, 135},
	{"ms_arab_my", 1, 1, 361, 1, 1, 1, 411, 1, 2, 3, 4, 5, 6, 7, 58, 33, 230, 204, 41, 42, 43, 44, 16, 31, 0, 135},
	{"ms_bn", 359, 1, 360, 408, 1, 409, 410, 2, 1, 3, 4, 5, 6, 7, 9, 33, 34, 204, 12, 13, 14, 15, 54, 31, 207, 134},
	{"ms_id", 359, 1, 360, 408, 1, 409, 410, 2, 1, 3, 4, 5, 6, 7, 183, 33, 34, 11, 94, 95, 96, 97, 16, 16, 207, 134},
	{"ms_my", 359, 1, 360, 408, 1, 409, 410, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 204, 12, 13, 14, 15, 16, 31, 207, 134},
	{"ms_sg", 359, 1, 360, 408, 1, 409, 410, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 204, 12, 13, 14, 15, 16, 31, 207, 134},
	{"mt", 362, 363, 364, 412, 413, 1, 414, 1, 2, 3, 4, 5, 6, 7, 330, 331, 29, 11, 23, 24, 25, 26, 16, 0, 17, 136},
	{"mt_mt", 362, 363, 364, 412, 413, 1, 414, 1, 2, 3, 4, 5, 6, 7, 330, 331, 29, 11, 23, 24, 25, 26, 16, 0, 17, 136},
	{"mua", 365, 1, 366, 415, 1, 1, 416, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"mua_cm", 365, 1, 366, 415, 1, 1, 416, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 16, 31, 0, 0},
	{"mus", 1, 1, 367, 1, 1, 1, 417, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"mus_us", 1, 1, 367, 1, 1, 1, 417, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 54, 0, 38, 0},
	{"mww", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"mww_hmnp", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"mww_hmnp_us", 1, 1, 215, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 172, 0},
	{"my", 368, 1, 369, 1, 1, 418, 419, 1, 2, 3, 4, 5, 6, 7, 332, 52, 53, 36, 283, 284, 49, 50, 27, 54, 208, 137},
	{"my_mm", 368, 1, 369, 1, 1, 418, 419, 1, 2, 3, 4, 5, 6, 7, 332, 52, 53, 36, 283, 284, 49, 50, 27, 54, 208, 137},
	{"myv", 1, 1, 370, 420, 1, 1, 421, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 138},
	{"myv_ru", 1, 1, 370, 420, 1, 1, 421, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 138},
	{"mzn", 1, 1, 163, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 29, 0},
	{"mzn_ir", 1, 1, 163, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 29, 0},
	{"naq", 139, 1, 371, 422, 1, 1, 423, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 0, 0, 0},
	{"naq_na", 139, 1, 371, 422, 1, 1, 423, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 0, 0, 0},
	{"nb", 372, 1, 373, 424, 1, 117, 118, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 209, 139},
	{"nb_no", 372, 1, 373, 424, 1, 117, 118, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 209, 139},
	{"nb_sj", 372, 1, 373, 424, 1, 117, 118, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 209, 139},
	{"nd", 374, 1, 375, 425, 1, 1, 426, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 0, 0},
	{"nd_zw", 374, 1, 375, 425, 1, 1, 426, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 0, 0},
	{"nds", 376, 1, 377, 427, 1, 1, 428, 2, 1, 3, 4, 5, 6, 18, 334, 147, 158, 91, 335, 336, 337, 338, 27, 0, 210, 140},
	{"nds_de", 376, 1, 377, 427, 1, 1, 428, 2, 1, 3, 4, 5, 6, 18, 334, 147, 158, 91, 335, 336, 337, 338, 27, 0, 210, 140},
	{"nds_nl", 376, 1, 377, 427, 1, 1, 428, 2, 1, 3, 4, 5, 6, 18, 334, 147, 158, 91, 335, 336, 337, 338, 27, 0, 210, 140},
	{"ne", 1, 378, 379, 429, 1, 1, 430, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 214, 23, 24, 25, 26, 73, 0, 211, 141},
	{"ne_in", 1, 378, 379, 429, 1, 1, 430, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 214, 41, 42, 43, 44, 73, 0, 211, 141},
	{"ne_np", 1, 378, 379, 429, 1, 1, 430, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 214, 23, 24, 25, 26, 73, 0, 211, 141},
	{"nl", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 212, 142},
	{"nl_aw", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 213, 142},
	{"nl_be", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 204, 23, 24, 25, 26, 109, 189, 212, 142},
	{"nl_bq", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 214, 142},
	{"nl_cw", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 215, 142},
	{"nl_nl", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 212, 142},
	{"nl_sr", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 212, 142},
	{"nl_sx", 380, 1, 381, 431, 1, 1, 432, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 120, 23, 24, 25, 26, 109, 189, 215, 142},
	{"nmg", 382, 1, 383, 433, 1, 1, 434, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"nmg_cm", 382, 1, 383, 433, 1, 1, 434, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 0, 0, 0},
	{"nn", 372, 1, 373, 435, 1, 435, 436, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 216, 143},
	{"nn_no", 372, 1, 373, 435, 1, 435, 436, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 216, 143},
	{"nnh", 1, 1, 384, 1, 1, 1, 437, 2, 1, 3, 4, 5, 6, 7, 339, 340, 35, 11, 23, 24, 25, 26, 54, 0, 0, 0},
	{"nnh_cm", 1, 1, 384, 1, 1, 1, 437, 2, 1, 3, 4, 5, 6, 7, 339, 340, 35, 11, 23, 24, 25, 26, 54, 0, 0, 0},
	{"no", 372, 1, 373, 424, 1, 117, 118, 2, 17, 209, 4, 5, 6, 18, 146, 147, 158, 22, 23, 24, 25, 26, 333, 189, 209, 139},
	{"nqo", 385, 1, 386, 438, 1, 439, 440, 1, 104, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 217, 144},
	{"nqo_gn", 385, 1, 386, 438, 1, 439, 440, 1, 104, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 217, 144},
	{"nr", 387, 1, 388, 441, 1, 1, 442, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 0},
	{"nr_za", 387, 1, 388, 441, 1, 1, 442, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 0},
	{"nso", 389, 1, 390, 443, 1, 1, 444, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 16, 0, 145},
	{"nso_za", 389, 1, 390, 443, 1, 1, 444, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 16, 0, 145},
	{"nus", 391, 1, 392, 445, 1, 1, 446, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 204, 341, 342, 14, 15, 16, 31, 218, 0},
	{"nus_ss", 391, 1, 392, 445, 1, 1, 446, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 204, 341, 342, 14, 15, 16, 31, 218, 0},
	{"nv", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"nv_us", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ny", 393, 1, 394, 447, 1, 1, 448, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"ny_mw", 393, 1, 394, 447, 1, 1, 448, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 0},
	{"nyn", 91, 1, 92, 92, 1, 1, 93, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 42, 0},
	{"nyn_ug", 91, 1, 92, 92, 1, 1, 93, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 42, 0},
	{"oc", 395, 1, 396, 1, 1, 449, 450, 2, 17, 3, 4, 5, 6, 343, 344, 46, 34, 11, 23, 24, 25, 345, 37, 275, 219, 146},
	{"oc_es", 397, 398, 399, 451, 1, 452, 453, 1, 2, 3, 4, 5, 6, 343, 199, 200, 34, 11, 136, 48, 49, 50, 37, 37, 220, 147},
	{"oc_fr", 395, 1, 396, 1, 1, 449, 450, 2, 17, 3, 4, 5, 6, 343, 344, 46, 34, 11, 23, 24, 25, 345, 37, 275, 219, 146},
	{"oka", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"oka_ca", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"oka_us", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"om", 400, 1, 401, 454, 1, 1, 455, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 0, 1, 148},
	{"om_et", 400, 1, 401, 454, 1, 1, 455, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 12, 13, 14, 15, 16, 0, 1, 148},
	{"om_ke", 400, 1, 401, 454, 1, 1, 455, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 0, 221, 148},
	{"or", 1, 1, 402, 456, 1, 1, 457, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 31, 204, 149},
	{"or_in", 1, 1, 402, 456, 1, 1, 457, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 16, 31, 204, 149},
	{"os", 403, 1, 404, 458, 1, 1, 459, 2, 17, 3, 4, 5, 6, 7, 346, 347, 348, 22, 23, 24, 25, 26, 54, 0, 38, 150},
	{"os_ge", 403, 1, 404, 458, 1, 1, 459, 2, 17, 3, 4, 5, 6, 7, 346, 347, 348, 22, 23, 24, 25, 26, 54, 0, 38, 150},
	{"os_ru", 403, 1, 404, 458, 1, 1, 459, 2, 17, 3, 4, 5, 6, 7, 346, 347, 348, 22, 23, 24, 25, 26, 54, 0, 222, 150},
	{"osa", 405, 1, 406, 1, 460, 1, 461, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 54, 0, 38, 0},
	{"osa_us", 405, 1, 406, 1, 460, 1, 461, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 54, 0, 38, 0},
	{"pa", 407, 1, 408, 462, 1, 463, 464, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 54, 223, 151},
	{"pa_arab", 1, 1, 409, 1, 1, 1, 465, 1, 2, 55, 4, 5, 57, 7, 183, 33, 34, 11, 41, 42, 43, 44, 54, 0, 224, 0},
	{"pa_arab_pk", 1, 1, 409, 1, 1, 1, 465, 1, 2, 55, 4, 5, 57, 7, 183, 33, 34, 11, 41, 42, 43, 44, 54, 0, 224, 0},
	{"pa_guru", 407, 1, 408, 462, 1, 463, 464, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 54, 223, 151},
	{"pa_guru_in", 407, 1, 408, 462, 1, 463, 464, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 54, 223, 151},
	{"pap", 410, 1, 411, 1, 1, 1, 466, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 120, 23, 24, 25, 26, 54, 0, 225, 152},
	{"pap_aw", 410, 1, 411, 1, 1, 1, 466, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 120, 23, 24, 25, 26, 54, 0, 225, 152},
	{"pap_cw", 410, 1, 411, 1, 1, 1, 466, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 120, 23, 24, 25, 26, 54, 0, 225, 152},
	{"pcm", 412, 1, 413, 467, 1, 1, 468, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 48, 25, 26, 16, 0, 226, 153},
	{"pcm_ng", 412, 1, 413, 467, 1, 1, 468, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 48, 25, 26, 16, 0, 226, 153},
	{"pi", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"pi_latn", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"pi_latn_gb", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"pis", 1, 1, 414, 1, 1, 1, 469, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 154},
	{"pis_sb", 1, 1, 414, 1, 1, 1, 469, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 154},
	{"pl", 415, 416, 417, 470, 471, 472, 473, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 91, 23, 24, 25, 26, 27, 102, 227, 155},
	{"pl_pl", 415, 416, 417, 470, 471, 472, 473, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 91, 23, 24, 25, 26, 27, 102, 227, 155},
	{"pms", 1, 1, 418, 1, 1, 1, 474, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 156},
	{"pms_it", 1, 1, 418, 1, 1, 1, 474, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 156},
	{"prg", 419, 1, 420, 475, 1, 1, 476, 2, 17, 3, 4, 5, 6, 7, 349, 350, 351, 22, 23, 24, 25, 26, 27, 0, 0, 157},
	{"prg_pl", 419, 1, 420, 475, 1, 1, 476, 2, 17, 3, 4, 5, 6, 7, 349, 350, 351, 22, 23, 24, 25, 26, 27, 0, 0, 157},
	{"ps", 1, 421, 422, 1, 1, 1, 477, 2, 1, 117, 4, 5, 57, 7, 352, 52, 53, 214, 136, 215, 49, 50, 54, 31, 46, 158},
	{"ps_af", 1, 421, 422, 1, 1, 1, 477, 2, 1, 117, 4, 5, 57, 7, 352, 52, 53, 214, 136, 215, 49, 50, 54, 31, 46, 158},
	{"ps_pk", 1, 421, 422, 1, 1, 1, 477, 2, 1, 117, 4, 5, 57, 7, 352, 52, 53, 214, 41, 42, 43, 44, 54, 31, 46, 159},
	{"pt", 423, 1, 424, 478, 1, 1, 479, 2, 1, 3, 4, 5, 6, 7, 199, 200, 237, 11, 23, 24, 25, 26, 54, 0, 228, 160},
	{"pt_ao", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_br", 423, 1, 424, 478, 1, 1, 479, 2, 1, 3, 4, 5, 6, 7, 199, 200, 237, 11, 23, 24, 25, 26, 54, 0, 228, 160},
	{"pt_ch", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_cv", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 230, 161},
	{"pt_gq", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_gw", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_lu", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 231, 161},
	{"pt_mo", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 12, 13, 14, 15, 27, 102, 232, 161},
	{"pt_mz", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 233, 161},
	{"pt_pt", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_st", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"pt_tl", 423, 1, 424, 480, 1, 478, 479, 2, 17, 3, 4, 5, 6, 7, 199, 200, 230, 11, 23, 24, 25, 26, 27, 102, 229, 161},
	{"qu", 425, 1, 426, 481, 1, 1, 482, 1, 2, 3, 4, 5, 6, 18, 65, 33, 34, 36, 12, 13, 14, 15, 54, 0, 234, 162},
	{"qu_bo", 425, 1, 426, 481, 1, 1, 482, 2, 1, 3, 4, 5, 6, 18, 65, 33, 34, 36, 12, 13, 14, 15, 54, 0, 235, 162},
	{"qu_ec", 425, 1, 426, 481, 1, 1, 482, 1, 2, 3, 4, 5, 6, 18, 65, 33, 34, 36, 12, 13, 14, 15, 54, 0, 236, 162},
	{"qu_pe", 425, 1, 426, 481, 1, 1, 482, 1, 2, 3, 4, 5, 6, 18, 65, 33, 34, 36, 12, 13, 14, 15, 54, 0, 234, 162},
	{"quc", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"quc_gt", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"raj", 1, 1, 57, 1, 1, 1, 226, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 0, 163},
	{"raj_in", 1, 1, 57, 1, 1, 1, 226, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 0, 163},
	{"rhg", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 50, 54, 0, 0, 0},
	{"rhg_rohg", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 50, 54, 0, 0, 0},
	{"rhg_rohg_bd", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 41, 42, 43, 44, 54, 0, 0, 0},
	{"rhg_rohg_mm", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 50, 54, 0, 0, 0},
	{"rif", 427, 1, 428, 483, 1, 484, 485, 2, 17, 3, 4, 5, 6, 191, 28, 9, 29, 11, 23, 24, 25, 26, 27, 0, 0, 164},
	{"rif_ma", 427, 1, 428, 483, 1, 484, 485, 2, 17, 3, 4, 5, 6, 191, 28, 9, 29, 11, 23, 24, 25, 26, 27, 0, 0, 164},
	{"rm", 429, 1, 430, 486, 1, 1, 487, 2, 188, 3, 4, 5, 6, 7, 353, 33, 67, 120, 23, 24, 25, 26, 16, 31, 38, 165},
	{"rm_ch", 429, 1, 430, 486, 1, 1, 487, 2, 188, 3, 4, 5, 6, 7, 353, 33, 67, 120, 23, 24, 25, 26, 16, 31, 38, 165},
	{"rn", 431, 1, 432, 488, 1, 1, 489, 2, 1, 3, 4, 5, 6, 18, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 237, 0},
	{"rn_bi", 431, 1, 432, 488, 1, 1, 489, 2, 1, 3, 4, 5, 6, 18, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 237, 0},
	{"ro", 433, 1, 434, 490, 1, 491, 492, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 22, 23, 24, 25, 26, 27, 102, 238, 166},
	{"ro_md", 433, 1, 434, 493, 1, 494, 492, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 22, 23, 24, 25, 26, 27, 102, 239, 166},
	{"ro_ro", 433, 1, 434, 490, 1, 491, 492, 2, 1, 3, 4, 5, 6, 18, 74, 33, 34, 22, 23, 24, 25, 26, 27, 102, 238, 166},
	{"rof", 435, 1, 436, 495, 1, 1, 496, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"rof_tz", 435, 1, 436, 495, 1, 1, 496, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"root", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ru", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 240, 167},
	{"ru_by", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 241, 167},
	{"ru_kg", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 242, 167},
	{"ru_kz", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 240, 167},
	{"ru_md", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 243, 167},
	{"ru_ru", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 240, 167},
	{"ru_ua", 437, 1, 438, 497, 1, 1, 498, 2, 17, 3, 4, 5, 6, 18, 88, 89, 90, 22, 23, 24, 25, 26, 27, 0, 240, 167},
	{"rw", 431, 1, 439, 499, 1, 1, 500, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 168},
	{"rw_rw", 431, 1, 439, 499, 1, 1, 500, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 168},
	{"rwk", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"rwk_tz", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"sa", 440, 441, 442, 224, 1, 1, 501, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 54, 0, 169},
	{"sa_in", 440, 441, 442, 224, 1, 1, 501, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 54, 0, 169},
	{"sah", 443, 1, 444, 502, 1, 1, 503, 2, 17, 3, 4, 5, 6, 7, 354, 355, 356, 214, 23, 24, 25, 26, 27, 0, 38, 170},
	{"sah_ru", 443, 1, 444, 502, 1, 1, 503, 2, 17, 3, 4, 5, 6, 7, 354, 355, 356, 214, 23, 24, 25, 26, 27, 0, 38, 170},
	{"saq", 445, 1, 446, 504, 1, 1, 505, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"saq_ke", 445, 1, 446, 504, 1, 1, 505, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 49, 0},
	{"sat", 447, 1, 448, 506, 1, 1, 507, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 0, 171},
	{"sat_deva", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 0, 0},
	{"sat_deva_in", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 0, 0},
	{"sat_olck", 447, 1, 448, 506, 1, 1, 507, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 0, 171},
	{"sat_olck_in", 447, 1, 448, 506, 1, 1, 507, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 36, 41, 42, 43, 44, 54, 0, 0, 171},
	{"sbp", 449, 1, 450, 508, 1, 1, 509, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"sbp_tz", 449, 1, 450, 508, 1, 1, 509, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 37, 0, 18, 0},
	{"sc", 451, 1, 452, 510, 1, 1, 511, 2, 1, 3, 4, 5, 6, 7, 357, 358, 359, 11, 23, 24, 25, 26, 27, 102, 244, 172},
	{"sc_it", 451, 1, 452, 510, 1, 1, 511, 2, 1, 3, 4, 5, 6, 7, 357, 358, 359, 11, 23, 24, 25, 26, 27, 102, 244, 172},
	{"scn", 453, 1, 454, 512, 1, 513, 514, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 54, 0, 0, 173},
	{"scn_it", 453, 1, 454, 512, 1, 513, 514, 2, 1, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 54, 0, 0, 173},
	{"sd", 1, 1, 455, 1, 515, 1, 516, 1, 2, 3, 4, 5, 6, 7, 137, 52, 53, 30, 41, 42, 43, 44, 54, 31, 17, 174},
	{"sd_arab", 1, 1, 455, 1, 515, 1, 516, 1, 2, 3, 4, 5, 6, 7, 137, 52, 53, 30, 41, 42, 43, 44, 54, 31, 17, 174},
	{"sd_arab_pk", 1, 1, 455, 1, 515, 1, 516, 1, 2, 3, 4, 5, 6, 7, 137, 52, 53, 30, 41, 42, 43, 44, 54, 31, 17, 174},
	{"sd_deva", 456, 457, 458, 517, 518, 1, 519, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 54, 0, 53, 175},
	{"sd_deva_in", 456, 457, 458, 517, 518, 1, 519, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 41, 42, 43, 44, 54, 0, 53, 175},
	{"sdh", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sdh_iq", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sdh_ir", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"se", 459, 1, 460, 520, 1, 1, 521, 2, 17, 209, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 245, 176},
	{"se_fi", 461, 1, 460, 522, 523, 522, 524, 2, 17, 209, 4, 5, 6, 18, 32, 33, 34, 22, 23, 24, 25, 26, 27, 0, 245, 177},
	{"se_no", 459, 1, 460, 520, 1, 1, 521, 2, 17, 209, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 245, 176},
	{"se_se", 459, 1, 460, 520, 1, 1, 521, 2, 17, 209, 4, 5, 6, 18, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 246, 176},
	{"seh", 462, 1, 463, 525, 1, 1, 526, 2, 1, 3, 4, 5, 6, 7, 199, 200, 237, 36, 23, 24, 25, 26, 37, 0, 201, 0},
	{"seh_mz", 462, 1, 463, 525, 1, 1, 526, 2, 1, 3, 4, 5, 6, 7, 199, 200, 237, 36, 23, 24, 25, 26, 37, 0, 201, 0},
	{"ses", 118, 1, 119, 124, 1, 1, 527, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"ses_ml", 118, 1, 119, 124, 1, 1, 527, 1, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"sg", 464, 1, 465, 528, 1, 1, 529, 2, 1, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 202, 0, 0, 0},
	{"sg_cf", 464, 1, 465, 528, 1, 1, 529, 2, 1, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 202, 0, 0, 0},
	{"sgs", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sgs_lt", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"shi", 466, 1, 467, 530, 1, 1, 531, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"shi_latn", 468, 1, 469, 532, 1, 1, 533, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"shi_latn_ma", 468, 1, 469, 532, 1, 1, 533, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"shi_tfng", 466, 1, 467, 530, 1, 1, 531, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"shi_tfng_ma", 466, 1, 467, 530, 1, 1, 531, 2, 17, 3, 4, 5, 6, 7, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"shn", 470, 471, 472, 534, 535, 534, 536, 1, 2, 3, 4, 5, 6, 7, 360, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 178},
	{"shn_mm", 470, 471, 472, 534, 535, 534, 536, 1, 2, 3, 4, 5, 6, 7, 360, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 178},
	{"shn_th", 470, 471, 472, 534, 535, 534, 536, 1, 2, 3, 4, 5, 6, 7, 360, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 178},
	{"si", 473, 474, 473, 537, 1, 538, 539, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 94, 95, 96, 97, 16, 31, 247, 179},
	{"si_lk", 473, 474, 473, 537, 1, 538, 539, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 94, 95, 96, 97, 16, 31, 247, 179},
	{"sid", 1, 1, 17, 540, 1, 1, 541, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 1, 0},
	{"sid_et", 1, 1, 17, 540, 1, 1, 541, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 1, 0},
	{"sk", 475, 1, 476, 542, 1, 1, 543, 2, 17, 3, 4, 5, 6, 18, 146, 147, 148, 292, 47, 48, 49, 50, 27, 102, 248, 180},
	{"sk_sk", 475, 1, 476, 542, 1, 1, 543, 2, 17, 3, 4, 5, 6, 18, 146, 147, 148, 292, 47, 48, 49, 50, 27, 102, 248, 180},
	{"skr", 1, 1, 477, 1, 1, 1, 544, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 181},
	{"skr_pk", 1, 1, 477, 1, 1, 1, 544, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 181},
	{"sl", 478, 1, 479, 545, 1, 1, 546, 2, 1, 209, 4, 5, 6, 18, 159, 147, 158, 292, 23, 24, 25, 26, 27, 102, 249, 182},
	{"sl_si", 478, 1, 479, 545, 1, 1, 546, 2, 1, 209, 4, 5, 6, 18, 159, 147, 158, 292, 23, 24, 25, 26, 27, 102, 249, 182},
	{"sma", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sma_no", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sma_se", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"smj", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"smj_no", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"smj_se", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"smn", 480, 1, 481, 547, 548, 549, 550, 2, 17, 3, 4, 5, 6, 18, 361, 362, 363, 167, 184, 185, 186, 187, 27, 0, 0, 0},
	{"smn_fi", 480, 1, 481, 547, 548, 549, 550, 2, 17, 3, 4, 5, 6, 18, 361, 362, 363, 167, 184, 185, 186, 187, 27, 0, 0, 0},
	{"sms", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sms_fi", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sn", 482, 1, 483, 551, 1, 552, 553, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 31, 0, 0},
	{"sn_zw", 482, 1, 483, 551, 1, 552, 553, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 31, 0, 0},
	{"so", 484, 1, 485, 554, 1, 1, 555, 1, 2, 3, 4, 5, 6, 7, 137, 138, 10, 11, 12, 13, 14, 15, 16, 31, 250, 183},
	{"so_dj", 484, 1, 485, 554, 1, 1, 555, 1, 2, 3, 4, 5, 6, 7, 137, 138, 10, 11, 12, 13, 14, 15, 16, 31, 251, 183},
	{"so_et", 484, 1, 485, 554, 1, 1, 555, 1, 2, 3, 4, 5, 6, 7, 137, 138, 10, 11, 12, 13, 14, 15, 16, 31, 252, 183},
	{"so_ke", 484, 1, 485, 554, 1, 1, 555, 1, 2, 3, 4, 5, 6, 7, 137, 138, 10, 11, 23, 24, 25, 26, 16, 31, 253, 183},
	{"so_so", 484, 1, 485, 554, 1, 1, 555, 1, 2, 3, 4, 5, 6, 7, 137, 138, 10, 11, 12, 13, 14, 15, 16, 31, 250, 183},
	{"sq", 486, 1, 487, 556, 1, 1, 557, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 167, 364, 365, 14, 15, 27, 102, 254, 184},
	{"sq_al", 486, 1, 487, 556, 1, 1, 557, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 167, 364, 365, 14, 15, 27, 102, 254, 184},
	{"sq_mk", 486, 1, 487, 556, 1, 1, 557, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 167, 23, 24, 25, 26, 27, 102, 255, 184},
	{"sq_xk", 486, 1, 487, 556, 1, 1, 557, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 167, 23, 24, 25, 26, 27, 102, 254, 184},
	{"sr", 488, 1, 489, 558, 1, 559, 560, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 185},
	{"sr_cyrl", 488, 1, 489, 558, 1, 559, 560, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 185},
	{"sr_cyrl_ba", 488, 1, 489, 75, 1, 559, 76, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 186},
	{"sr_cyrl_me", 490, 1, 489, 558, 1, 559, 561, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 185},
	{"sr_cyrl_rs", 488, 1, 489, 558, 1, 559, 560, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 185},
	{"sr_cyrl_xk", 490, 1, 489, 558, 1, 559, 560, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 256, 185},
	{"sr_latn", 491, 1, 492, 562, 1, 563, 564, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 257, 187},
	{"sr_latn_ba", 491, 1, 492, 72, 1, 563, 74, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 257, 188},
	{"sr_latn_me", 493, 1, 492, 562, 1, 563, 565, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 257, 187},
	{"sr_latn_rs", 491, 1, 492, 562, 1, 563, 564, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 257, 187},
	{"sr_latn_xk", 493, 1, 492, 562, 1, 563, 564, 2, 1, 3, 4, 5, 6, 7, 125, 126, 366, 128, 23, 24, 25, 26, 27, 102, 257, 187},
	{"ss", 494, 1, 495, 566, 1, 1, 567, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 258, 0},
	{"ss_sz", 494, 1, 495, 566, 1, 1, 567, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 16, 0, 258, 0},
	{"ss_za", 494, 1, 495, 566, 1, 1, 567, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 258, 0},
	{"ssy", 0, 1, 2, 568, 1, 1, 569, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 35, 0},
	{"ssy_er", 0, 1, 2, 568, 1, 1, 569, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 0, 35, 0},
	{"st", 496, 1, 497, 570, 1, 1, 571, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 168},
	{"st_ls", 496, 1, 497, 570, 1, 1, 571, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 12, 13, 14, 15, 16, 0, 259, 168},
	{"st_za", 496, 1, 497, 570, 1, 1, 571, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 168},
	{"su", 498, 1, 499, 572, 1, 1, 573, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 36, 184, 185, 186, 187, 16, 0, 53, 189},
	{"su_latn", 498, 1, 499, 572, 1, 1, 573, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 36, 184, 185, 186, 187, 16, 0, 53, 189},
	{"su_latn_id", 498, 1, 499, 572, 1, 1, 573, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 36, 184, 185, 186, 187, 16, 0, 53, 189},
	{"suz", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"suz_deva", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"suz_deva_np", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"suz_sunu", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"suz_sunu_np", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"sv", 500, 1, 501, 574, 1, 575, 576, 2, 17, 209, 4, 5, 6, 18, 32, 33, 34, 30, 23, 24, 25, 26, 27, 0, 260, 190},
	{"sv_ax", 500, 1, 501, 574, 1, 575, 576, 2, 17, 209, 4, 5, 6, 18, 32, 33, 166, 167, 23, 24, 25, 26, 27, 0, 260, 190},
	{"sv_fi", 500, 1, 501, 574, 1, 575, 576, 2, 17, 209, 4, 5, 6, 18, 32, 33, 34, 167, 94, 185, 186, 187, 27, 0, 260, 190},
	{"sv_se", 500, 1, 501, 574, 1, 575, 576, 2, 17, 209, 4, 5, 6, 18, 32, 33, 34, 30, 23, 24, 25, 26, 27, 0, 260, 190},
	{"sw", 241, 1, 28, 1, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 261, 191},
	{"sw_cd", 241, 1, 28, 1, 1, 1, 23, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 262, 191},
	{"sw_ke", 241, 1, 28, 1, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 263, 192},
	{"sw_tz", 241, 1, 28, 1, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 261, 191},
	{"sw_ug", 241, 1, 28, 1, 1, 1, 23, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 264, 191},
	{"syr", 502, 503, 504, 577, 1, 1, 578, 1, 2, 3, 4, 5, 6, 7, 367, 368, 369, 370, 41, 42, 43, 44, 54, 0, 265, 193},
	{"syr_iq", 502, 503, 504, 577, 1, 1, 578, 1, 2, 3, 4, 5, 6, 7, 367, 368, 369, 370, 41, 42, 43, 44, 54, 0, 265, 193},
	{"syr_sy", 502, 503, 504, 577, 1, 1, 578, 1, 2, 3, 4, 5, 6, 7, 367, 368, 369, 370, 41, 42, 43, 44, 54, 0, 265, 193},
	{"szl", 505, 1, 506, 579, 1, 580, 581, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 22, 23, 24, 25, 26, 27, 54, 0, 194},
	{"szl_pl", 505, 1, 506, 579, 1, 580, 581, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 22, 23, 24, 25, 26, 27, 54, 0, 194},
	{"ta", 507, 1, 508, 582, 1, 583, 584, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 112, 31, 204, 195},
	{"ta_in", 507, 1, 508, 582, 1, 583, 584, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 41, 42, 43, 44, 112, 31, 204, 195},
	{"ta_lk", 507, 1, 508, 582, 1, 583, 584, 1, 2, 3, 4, 5, 6, 64, 65, 66, 35, 36, 23, 24, 25, 26, 112, 31, 266, 195},
	{"ta_my", 507, 1, 508, 582, 1, 583, 584, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 41, 42, 43, 44, 54, 31, 267, 195},
	{"ta_sg", 507, 1, 508, 582, 1, 583, 584, 1, 2, 3, 4, 5, 6, 7, 65, 66, 35, 36, 41, 42, 43, 44, 54, 31, 268, 195},
	{"te", 509, 1, 510, 585, 1, 586, 587, 1, 2, 3, 4, 5, 6, 7, 371, 66, 35, 120, 41, 42, 43, 44, 112, 31, 29, 196},
	{"te_in", 509, 1, 510, 585, 1, 586, 587, 1, 2, 3, 4, 5, 6, 7, 371, 66, 35, 120, 41, 42, 43, 44, 112, 31, 29, 196},
	{"teo", 511, 1, 512, 588, 1, 1, 589, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 42, 0},
	{"teo_ke", 511, 1, 512, 588, 1, 1, 589, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 269, 0},
	{"teo_ug", 511, 1, 512, 588, 1, 1, 589, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 31, 42, 0},
	{"tg", 513, 1, 514, 590, 1, 1, 591, 2, 17, 3, 4, 5, 6, 7, 183, 9, 29, 11, 23, 24, 25, 26, 27, 0, 270, 197},
	{"tg_tj", 513, 1, 514, 590, 1, 1, 591, 2, 17, 3, 4, 5, 6, 7, 183, 9, 29, 11, 23, 24, 25, 26, 27, 0, 270, 197},
	{"th", 515, 1, 516, 592, 1, 593, 594, 1, 2, 3, 4, 5, 6, 7, 372, 373, 34, 36, 374, 375, 25, 26, 16, 31, 271, 198},
	{"th_th", 515, 1, 516, 592, 1, 593, 594, 1, 2, 3, 4, 5, 6, 7, 372, 373, 34, 36, 374, 375, 25, 26, 16, 31, 271, 198},
	{"ti", 517, 1, 518, 595, 1, 1, 596, 1, 2, 3, 4, 5, 6, 7, 376, 138, 139, 108, 41, 42, 43, 44, 16, 0, 272, 199},
	{"ti_er", 517, 1, 518, 595, 1, 1, 596, 1, 2, 3, 4, 5, 6, 7, 376, 138, 139, 108, 41, 42, 43, 44, 16, 0, 273, 199},
	{"ti_et", 517, 1, 518, 595, 1, 1, 596, 1, 2, 3, 4, 5, 6, 7, 376, 138, 139, 108, 41, 42, 43, 44, 16, 0, 272, 199},
	{"tig", 519, 1, 520, 597, 1, 1, 598, 1, 2, 3, 4, 5, 6, 7, 377, 9, 10, 11, 41, 42, 43, 44, 16, 0, 35, 0},
	{"tig_er", 519, 1, 520, 597, 1, 1, 598, 1, 2, 3, 4, 5, 6, 7, 377, 9, 10, 11, 41, 42, 43, 44, 16, 0, 35, 0},
	{"tk", 521, 1, 522, 599, 1, 600, 601, 2, 17, 3, 4, 5, 6, 18, 378, 33, 34, 22, 23, 24, 25, 26, 27, 0, 274, 200},
	{"tk_tm", 521, 1, 522, 599, 1, 600, 601, 2, 17, 3, 4, 5, 6, 18, 378, 33, 34, 22, 23, 24, 25, 26, 27, 0, 274, 200},
	{"tn", 523, 1, 524, 602, 1, 1, 603, 1, 161, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 201},
	{"tn_bw", 523, 1, 524, 602, 1, 1, 603, 1, 161, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 201},
	{"tn_za", 523, 1, 524, 602, 1, 1, 603, 1, 161, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 201},
	{"to", 525, 1, 526, 604, 1, 1, 605, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 54, 0, 275, 202},
	{"to_to", 525, 1, 526, 604, 1, 1, 605, 1, 2, 3, 4, 5, 6, 7, 32, 33, 34, 36, 12, 13, 14, 15, 54, 0, 275, 202},
	{"tok", 1, 1, 527, 1, 606, 1, 607, 2, 17, 3, 4, 5, 6, 7, 379, 380, 379, 30, 381, 382, 25, 26, 383, 0, 0, 203},
	{"tok_001", 1, 1, 527, 1, 606, 1, 607, 2, 17, 3, 4, 5, 6, 7, 379, 380, 379, 30, 381, 382, 25, 26, 383, 0, 0, 203},
	{"tpi", 528, 1, 529, 608, 1, 1, 609, 1, 2, 3, 4, 5, 6, 7, 384, 9, 29, 11, 84, 85, 86, 87, 27, 0, 0, 0},
	{"tpi_pg", 528, 1, 529, 608, 1, 1, 609, 1, 2, 3, 4, 5, 6, 7, 384, 9, 29, 11, 84, 85, 86, 87, 27, 0, 0, 0},
	{"tr", 530, 1, 531, 610, 1, 611, 612, 2, 1, 3, 4, 5, 6, 293, 378, 33, 34, 91, 23, 24, 25, 26, 16, 31, 276, 204},
	{"tr_cy", 530, 1, 531, 610, 1, 611, 612, 2, 1, 3, 4, 5, 6, 293, 378, 33, 34, 91, 12, 13, 14, 15, 16, 31, 276, 204},
	{"tr_tr", 530, 1, 531, 610, 1, 611, 612, 2, 1, 3, 4, 5, 6, 293, 378, 33, 34, 91, 23, 24, 25, 26, 16, 31, 276, 204},
	{"trv", 532, 1, 533, 613, 1, 1, 614, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 205},
	{"trv_tw", 532, 1, 533, 613, 1, 1, 614, 1, 2, 3, 4, 5, 6, 7, 140, 52, 53, 30, 12, 13, 14, 15, 54, 0, 0, 205},
	{"trw", 1, 1, 409, 1, 1, 1, 615, 1, 2, 3, 4, 5, 6, 7, 385, 386, 387, 36, 41, 42, 43, 44, 54, 0, 38, 206},
	{"trw_pk", 1, 1, 409, 1, 1, 1, 615, 1, 2, 3, 4, 5, 6, 7, 385, 386, 387, 36, 41, 42, 43, 44, 54, 0, 38, 206},
	{"ts", 534, 1, 535, 616, 1, 1, 617, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"ts_za", 534, 1, 535, 616, 1, 1, 617, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"tt", 536, 1, 537, 618, 1, 1, 619, 2, 17, 3, 4, 5, 6, 18, 388, 389, 390, 22, 47, 48, 49, 50, 27, 0, 277, 207},
	{"tt_ru", 536, 1, 537, 618, 1, 1, 619, 2, 17, 3, 4, 5, 6, 18, 388, 389, 390, 22, 47, 48, 49, 50, 27, 0, 277, 207},
	{"twq", 118, 1, 119, 124, 1, 1, 527, 1, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"twq_ne", 118, 1, 119, 124, 1, 1, 527, 1, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"tyv", 538, 246, 539, 620, 621, 620, 622, 1, 17, 3, 4, 5, 6, 7, 391, 392, 393, 30, 23, 24, 25, 26, 54, 0, 0, 208},
	{"tyv_ru", 538, 246, 539, 620, 621, 620, 622, 1, 17, 3, 4, 5, 6, 7, 391, 392, 393, 30, 23, 24, 25, 26, 54, 0, 0, 208},
	{"tzm", 540, 1, 541, 623, 1, 1, 624, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 0, 0},
	{"tzm_ma", 540, 1, 541, 623, 1, 1, 624, 2, 17, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 0, 0},
	{"ug", 1, 1, 542, 625, 1, 626, 627, 1, 2, 3, 4, 5, 6, 7, 394, 395, 396, 30, 23, 24, 25, 26, 16, 31, 278, 209},
	{"ug_cn", 1, 1, 542, 625, 1, 626, 627, 1, 2, 3, 4, 5, 6, 7, 394, 395, 396, 30, 23, 24, 25, 26, 16, 31, 278, 209},
	{"uk", 543, 544, 545, 52, 1, 1, 628, 2, 17, 3, 4, 5, 6, 7, 397, 398, 399, 22, 23, 24, 25, 26, 27, 0, 279, 210},
	{"uk_ua", 543, 544, 545, 52, 1, 1, 628, 2, 17, 3, 4, 5, 6, 7, 397, 398, 399, 22, 23, 24, 25, 26, 27, 0, 279, 210},
	{"ur", 1, 1, 477, 1, 1, 1, 629, 1, 2, 55, 4, 5, 57, 7, 385, 386, 387, 36, 41, 42, 43, 44, 16, 31, 29, 211},
	{"ur_in", 1, 1, 477, 1, 1, 1, 629, 1, 2, 55, 4, 5, 57, 7, 385, 386, 387, 36, 41, 42, 43, 44, 16, 31, 280, 212},
	{"ur_pk", 1, 1, 477, 1, 1, 1, 629, 1, 2, 55, 4, 5, 57, 7, 385, 386, 387, 36, 41, 42, 43, 44, 16, 31, 29, 211},
	{"uz", 546, 1, 547, 630, 1, 631, 632, 2, 17, 3, 4, 5, 6, 7, 400, 401, 402, 11, 136, 215, 25, 26, 27, 31, 281, 213},
	{"uz_arab", 548, 1, 166, 633, 1, 1, 169, 2, 1, 117, 4, 5, 57, 7, 403, 404, 34, 214, 136, 215, 49, 50, 27, 0, 0, 214},
	{"uz_arab_af", 548, 1, 166, 633, 1, 1, 169, 2, 1, 117, 4, 5, 57, 7, 403, 404, 34, 214, 136, 215, 49, 50, 27, 0, 0, 214},
	{"uz_cyrl", 87, 1, 549, 634, 1, 635, 636, 2, 17, 3, 4, 5, 6, 7, 261, 66, 35, 11, 155, 156, 25, 26, 27, 0, 282, 215},
	{"uz_cyrl_uz", 87, 1, 549, 634, 1, 635, 636, 2, 17, 3, 4, 5, 6, 7, 261, 66, 35, 11, 155, 156, 25, 26, 27, 0, 282, 215},
	{"uz_latn", 546, 1, 547, 630, 1, 631, 632, 2, 17, 3, 4, 5, 6, 7, 400, 401, 402, 11, 136, 215, 25, 26, 27, 31, 281, 213},
	{"uz_latn_uz", 546, 1, 547, 630, 1, 631, 632, 2, 17, 3, 4, 5, 6, 7, 400, 401, 402, 11, 136, 215, 25, 26, 27, 31, 281, 213},
	{"vai", 550, 1, 551, 1, 1, 1, 637, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 41, 42, 43, 44, 16, 31, 0, 0},
	{"vai_latn", 1, 1, 17, 1, 1, 1, 638, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 0, 0},
	{"vai_latn_lr", 1, 1, 17, 1, 1, 1, 638, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 16, 31, 0, 0},
	{"vai_vaii", 550, 1, 551, 1, 1, 1, 637, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 41, 42, 43, 44, 16, 31, 0, 0},
	{"vai_vaii_lr", 550, 1, 551, 1, 1, 1, 637, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 41, 42, 43, 44, 16, 31, 0, 0},
	{"ve", 552, 1, 553, 639, 1, 1, 640, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 0},
	{"ve_za", 552, 1, 553, 639, 1, 1, 640, 2, 17, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 16, 0, 0, 0},
	{"vec", 554, 1, 555, 641, 1, 1, 642, 2, 188, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 197, 0, 283, 216},
	{"vec_it", 554, 1, 555, 641, 1, 1, 642, 2, 188, 3, 4, 5, 6, 7, 32, 33, 34, 11, 23, 24, 25, 26, 197, 0, 283, 216},
	{"vi", 556, 1, 557, 643, 1, 644, 645, 2, 1, 3, 4, 5, 6, 7, 65, 66, 35, 36, 23, 24, 25, 26, 27, 0, 271, 217},
	{"vi_vn", 556, 1, 557, 643, 1, 644, 645, 2, 1, 3, 4, 5, 6, 7, 65, 66, 35, 36, 23, 24, 25, 26, 27, 0, 271, 217},
	{"vmw", 1, 1, 558, 1, 1, 1, 646, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 218},
	{"vmw_mz", 1, 1, 558, 1, 1, 1, 646, 2, 1, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 27, 0, 0, 218},
	{"vo", 559, 1, 560, 647, 1, 1, 648, 1, 2, 3, 4, 5, 6, 7, 405, 52, 406, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"vo_001", 559, 1, 560, 647, 1, 1, 648, 1, 2, 3, 4, 5, 6, 7, 405, 52, 406, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"vun", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"vun_tz", 241, 1, 242, 260, 1, 1, 261, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 16, 0, 18, 0},
	{"wa", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"wa_be", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"wae", 561, 1, 562, 649, 1, 1, 650, 2, 161, 3, 4, 5, 6, 7, 159, 147, 158, 30, 23, 24, 25, 26, 54, 0, 53, 219},
	{"wae_ch", 561, 1, 562, 649, 1, 1, 650, 2, 161, 3, 4, 5, 6, 7, 159, 147, 158, 30, 23, 24, 25, 26, 54, 0, 53, 219},
	{"wal", 519, 1, 520, 1, 1, 1, 651, 1, 161, 3, 4, 5, 6, 7, 407, 9, 10, 11, 41, 42, 43, 44, 16, 0, 1, 0},
	{"wal_et", 519, 1, 520, 1, 1, 1, 651, 1, 161, 3, 4, 5, 6, 7, 407, 9, 10, 11, 41, 42, 43, 44, 16, 0, 1, 0},
	{"wbp", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"wbp_au", 1, 1, 17, 1, 1, 1, 16, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 0, 0},
	{"wo", 563, 1, 564, 652, 1, 1, 653, 2, 1, 3, 4, 5, 6, 7, 408, 66, 35, 120, 23, 24, 25, 26, 54, 0, 284, 220},
	{"wo_sn", 563, 1, 564, 652, 1, 1, 653, 2, 1, 3, 4, 5, 6, 7, 408, 66, 35, 120, 23, 24, 25, 26, 54, 0, 284, 220},
	{"xh", 565, 140, 566, 654, 655, 1, 656, 1, 17, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 0, 285, 221},
	{"xh_za", 565, 140, 566, 654, 655, 1, 656, 1, 17, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 0, 285, 221},
	{"xnr", 330, 1, 567, 657, 1, 658, 659, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 0, 0, 222},
	{"xnr_in", 330, 1, 567, 657, 1, 658, 659, 1, 2, 3, 4, 5, 6, 64, 74, 33, 34, 36, 41, 42, 43, 44, 112, 0, 0, 222},
	{"xog", 307, 1, 308, 660, 1, 1, 661, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 42, 0},
	{"xog_ug", 307, 1, 308, 660, 1, 1, 661, 1, 2, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 27, 0, 42, 0},
	{"yav", 568, 1, 569, 662, 1, 1, 663, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 102, 0, 0},
	{"yav_cm", 568, 1, 569, 662, 1, 1, 663, 2, 17, 3, 4, 5, 6, 7, 32, 33, 34, 36, 23, 24, 25, 26, 27, 102, 0, 0},
	{"yi", 1, 1, 570, 1, 1, 1, 664, 1, 2, 3, 4, 5, 6, 7, 409, 410, 411, 11, 23, 24, 25, 26, 54, 0, 0, 0},
	{"yi_ua", 1, 1, 570, 1, 1, 1, 664, 1, 2, 3, 4, 5, 6, 7, 409, 410, 411, 11, 23, 24, 25, 26, 54, 0, 0, 0},
	{"yo", 1, 1, 571, 665, 1, 1, 666, 1, 2, 3, 4, 5, 6, 7, 412, 34, 413, 36, 23, 48, 414, 415, 16, 31, 38, 223},
	{"yo_bj", 1, 1, 572, 667, 1, 1, 668, 1, 2, 3, 4, 5, 6, 7, 412, 34, 413, 36, 23, 48, 414, 415, 16, 31, 38, 224},
	{"yo_ng", 1, 1, 571, 665, 1, 1, 666, 1, 2, 3, 4, 5, 6, 7, 412, 34, 413, 36, 23, 48, 414, 415, 16, 31, 38, 223},
	{"yrl", 573, 1, 574, 669, 1, 1, 670, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 286, 225},
	{"yrl_br", 573, 1, 574, 669, 1, 1, 670, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 23, 24, 25, 26, 54, 0, 286, 225},
	{"yrl_co", 573, 1, 574, 669, 1, 1, 670, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 54, 0, 286, 226},
	{"yrl_ve", 573, 1, 574, 669, 1, 1, 670, 2, 1, 3, 4, 5, 6, 7, 74, 33, 34, 11, 12, 13, 14, 15, 54, 0, 286, 226},
	{"yue", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 417, 418, 419, 420, 16, 31, 287, 227},
	{"yue_hans", 239, 1, 239, 674, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 421, 422, 25, 26, 16, 31, 288, 228},
	{"yue_hans_cn", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"yue_hant", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 423, 424, 425, 426, 16, 31, 287, 230},
	{"yue_hant_cn", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 421, 422, 25, 26, 16, 31, 287, 230},
	{"yue_hant_hk", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 423, 424, 425, 426, 16, 31, 287, 230},
	{"yue_hant_mo", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 423, 424, 425, 426, 16, 31, 287, 230},
	{"za", 1, 1, 576, 1, 1, 1, 675, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 290, 231},
	{"za_cn", 1, 1, 576, 1, 1, 1, 675, 1, 2, 3, 4, 5, 6, 7, 51, 52, 53, 30, 23, 24, 25, 26, 54, 0, 290, 231},
	{"zgh", 466, 1, 467, 530, 1, 1, 676, 2, 17, 3, 4, 5, 6, 18, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"zgh_ma", 466, 1, 467, 530, 1, 1, 676, 2, 17, 3, 4, 5, 6, 18, 32, 33, 35, 36, 23, 24, 25, 26, 37, 0, 0, 0},
	{"zh", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"zh_hans", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"zh_hans_cn", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"zh_hans_hk", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 36, 427, 428, 419, 420, 16, 31, 291, 229},
	{"zh_hans_mo", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 36, 427, 428, 419, 420, 16, 31, 292, 229},
	{"zh_hans_my", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 427, 428, 419, 420, 16, 31, 289, 229},
	{"zh_hans_sg", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 11, 427, 428, 419, 420, 16, 31, 291, 229},
	{"zh_hant", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 423, 424, 425, 426, 16, 31, 287, 230},
	{"zh_hant_hk", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 36, 417, 418, 419, 420, 16, 31, 293, 232},
	{"zh_hant_mo", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 36, 417, 418, 419, 420, 16, 31, 294, 232},
	{"zh_hant_my", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 427, 428, 419, 420, 16, 31, 287, 230},
	{"zh_hant_tw", 1, 1, 239, 671, 1, 672, 673, 1, 2, 3, 4, 5, 6, 7, 416, 258, 258, 214, 423, 424, 425, 426, 16, 31, 287, 230},
	{"zh_latn", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"zh_latn_cn", 239, 1, 575, 674, 1, 1, 673, 1, 2, 3, 4, 5, 6, 7, 257, 258, 258, 214, 283, 284, 25, 26, 16, 31, 289, 229},
	{"zu", 577, 578, 579, 677, 1, 1, 678, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 31, 295, 233},
	{"zu_za", 577, 578, 579, 677, 1, 1, 678, 1, 2, 3, 4, 5, 6, 7, 137, 138, 139, 108, 23, 24, 25, 26, 16, 31, 295, 233},
}
